/*

OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--------------------------
@Copyright (c) 1997-1999 The Regents of the University of California.
All rights reserved.

Permission is hereby granted, without written agreement and without
license or royalty fees, to use, copy, modify, and distribute this
software and its documentation for any purpose, provided that the
above copyright notice and the following two paragraphs appear in all
copies of this software.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

                                                PT_COPYRIGHT_VERSION_2
                                                COPYRIGHTENDKEY
@ProposedRating Yellow (cxh@eecs.berkeley.edu)
@AcceptedRating Yellow (cxh@eecs.berkeley.edu)
*/
package ptolemy;


import starlogo.*;
import java.awt.*;
import java.awt.Graphics2D;
import java.awt.event.*;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.io.*;
import java.io.IOException;
import java.util.*;
import java.net.*;
import java.text.*;
import yoyo.*;
import javax.swing.JFrame;
import java.util.*;


public class LineChart extends JFrame implements StarLogoFrame, MenuListener, SLPrintable {

    public PlotBox framePlot; /** framePlot */
    public PlotBox widgetPlot; /** widgetPlot */

    //protected Frame test;

    protected Application app;
    protected int cindex;
    protected int idCount;
    protected int type;
    protected LineWidget lw;
    protected Vector sets;
    protected PlotDialog pd = null;
    protected SLMenu slmenu = null;
    protected String preTitle;

    public boolean autoscaleX = true;
    public boolean autoscaleY = true;

    public LineChart copy(Application app, LineWidget lw) {
	LineChart lc = new LineChart(app, lw.id, lw.id, lw.sets, lw.type, lw);
	if (type != LineWidget.HISTOGRAM) {
	    lc.framePlot = ((LineLive)framePlot).copy(app, lw);
	    lc.widgetPlot = ((LineLive)widgetPlot).copy(app, lw);
	} else {
	    lc.framePlot = ((HistoLive)framePlot).copy(app, lw);
	    lc.widgetPlot = ((HistoLive)widgetPlot).copy(app, lw);
	}
	lc.autoscaleX = autoscaleX;
	lc.autoscaleY = autoscaleY;
	return lc;
    }

    public LineChart(Application theApp, int cindex, int idCount, Vector sets, int type, LineWidget lw)  {
	super("StarLogo Plot");
	this.app = theApp;
	this.type = type;
	this.cindex = cindex;
	this.idCount = idCount;
	this.lw=lw;
	this.sets = sets;
	this.preTitle = lw.id + ". ";
	//System.out.println("setting up new line chart... type: " + type);
	if (type == LineWidget.HISTOGRAM) {
	    //System.out.println("* * * histogram * * *");
	    framePlot = new HistoLive(app, lw, type);
	    widgetPlot = new HistoLive(app, lw, type);
	    //((HistoLive)framePlot).setIndex(cindex);
	}
	else {
	    //System.out.println("* * * line live * * *");
	    framePlot = new LineLive(app, lw, type);
	    widgetPlot = new LineLive(app, lw, type);
	    //((LineLive)framePlot).setIndex(cindex);
	}

	
	javax.swing.ImageIcon icon = new javax.swing.ImageIcon(LineWidget.class.getResource("gifs/icon.gif"));
	setIconImage(icon.getImage());

	//framePlot.setTitle("Plot " + (1 + (idCount / 10)));
	framePlot.setTitle(app.textBundle.getString("plot") + " " + 
			   idCount, preTitle);
	widgetPlot.setColor(true);

	widgetPlot.setGrid(false);
	widgetPlot.setAxes(false);
	
	widgetPlot.addYTick("", 0);

	if (type == LineWidget.LINE) {
	    widgetPlot.addXTick("", 0);
	    ((LineLive)widgetPlot).setMarksStyle("none");
	}

	    slmenu = new JSLMenu(this, app);

	//slmenu = new NSLMenu(this, app);
	slmenu.disableMoreEditMenu();
	slmenu.setMenuBar(this);
	
	app.addWindow(this);
	app.registerFrameTracker(this);
	addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent evt) {
		    app.removeWindow(LineChart.this);
		    setVisible(false);
		}
	    });
        getContentPane().add("Center", framePlot);
	getContentPane().setBackground(Color.white);
	getContentPane().setForeground(Color.white);
	// FIXME: This should not be hardwired in here.
        setSize(500, 300);
	addNotify();


    }

    public void initWidgetPlot() {
	// plot widget needs to be added to a frame before it will draw.
	Frame dummy = new Frame();
	dummy.setLayout(null);
	dummy.add(widgetPlot);
	dummy.addNotify();
	dummy.setSize(widgetPlot.getSize().width, widgetPlot.getSize().height);
    }

    public void _data() {
	if (StarLogo.DEBUG) {
	    System.out.println("Data...");
	}
	app.maybeCompileAndThenRun(true, new Runnable() {
	    public void run() {
		PtolemyBarDialog gsd = 
		    Application.createPtolemyBarDialog(LineChart.this, sets, app, type, idCount);
		gsd.show();
		if (gsd.pressedOK()) {
		    gsd.getValues(lw, LineChart.this);
		    app.rancompilerp = false;
		    app.project.markChanged();
		    repaintWithInterface();
		}
	    }
	    });
    }

    public void _properties() {
	lw.sl.stopAll();
	if (pd == null) {
	    pd = Application.createPlotDialog(this, type, framePlot, lw, app);
	    //System.out.println("(null)ymaxT = " + plot.getYRange()[1]);
	}
	//System.out.println("(before show)ymaxT = " + plot.getYRange()[1]);
	pd.show();
	//System.out.println("(after show)ymaxT = " + plot.getYRange()[1]);
	if (pd.pressedOK()) {
	    pd.getValues(lw, framePlot);
	    pd.getValues(lw, widgetPlot);
	    setTitle(app.textBundle.getString("plot") + " " + idCount + ". " + 
		     framePlot.getTitle());
	    app.rancompilerp = false;
	    app.project.markChanged();
	    //System.out.println("(if ok)ymaxT = " + plot.getYRange()[1]);
	    //		iface.slmenu.changeMe(oldTitle, plot.getTitle());
	    //System.out.println("(before repaint)ymaxT = " + plot.getYRange()[1]);
	    repaintWithInterface();
	    //System.out.println("(after repaint)ymaxT = " + plot.getYRange()[1]);
	}
    }


    public void markChanged() {}
    public void clearChanged() {}
    

    /** Set the title of the graph.
     *  @param title The title.
     */
    public void setPlotTitle(String title) {
	setTitle(title);
	framePlot.setTitle(title, preTitle);
	widgetPlot.setTitle(title, preTitle);
    }

    public String getPreTitle() {
	return preTitle;
    }

    /** Get the title of the graph.
     */
    public String getTitle() {
	return framePlot.getTitle();
    }

	public int getPlotId() {
		return cindex;
	}

    public String getPlotTitle() {
	return framePlot.getTitle();
    }

    /** Set the label for the X (horizontal) axis.
     *  @param label The label.
     */
    public void setXLabel(String label) { 
	framePlot.setXLabel(label);
	widgetPlot.setXLabel(label);
    }
    
    public void setXMin(double xmin) {
	framePlot.setXRange(xmin, framePlot.getXRange()[1], !autoscaleX);
	widgetPlot.setXRange(xmin, widgetPlot.getXRange()[1], !autoscaleX);
    }

    public void setYMin(double ymin) {
	framePlot.setYRange(ymin, framePlot.getYRange()[1], !autoscaleY);
	widgetPlot.setYRange(ymin, widgetPlot.getYRange()[1], !autoscaleY);
    }

    public void setXMax(double xmax) {
	framePlot.setXRange(framePlot.getXRange()[0], xmax, !autoscaleX);
	widgetPlot.setXRange(widgetPlot.getXRange()[0], xmax, !autoscaleX);
    }

    public void setYMax(double ymax) {
	framePlot.setYRange(framePlot.getYRange()[0], ymax, !autoscaleY);
	widgetPlot.setYRange(widgetPlot.getYRange()[0], ymax, !autoscaleY);
    }

    public void setXRange(double xmin, double xmax) {
	//System.out.println("set x range: " + xmin + ", " + xmax);
	framePlot.setXRange(xmin, xmax, !autoscaleX);
	widgetPlot.setXRange(xmin, xmax, !autoscaleX);
    }

    public void setYRange(double ymin, double ymax) {
	//System.out.println("set y range: " + ymin + ", " + ymax);
	framePlot.setYRange(ymin, ymax, !autoscaleY);
	widgetPlot.setYRange(ymin, ymax, !autoscaleY);
    }


    public double getXMin() {
	return framePlot.getXRange()[0];
    }

    public double getYMin() {
	return framePlot.getYRange()[0];
    }

    public double getXMax() {
	return framePlot.getXRange()[1];
    }

    public double getYMax() {
	return framePlot.getYRange()[1];
    }


    public String getFrameXLabel() {
	return framePlot.getXLabel();
    }

    /** Set the label for the Y (vertical) axis.
     *  @param label The label.
     */
    public void setYLabel(String label) {
	framePlot.setYLabel(label);
	widgetPlot.setYLabel(label);
    }

    public String getFrameYLabel() {
	return framePlot.getYLabel();
    }

    public double getPPXCor(int plotpen) {
	return ((LineLive)framePlot).ppxcor(plotpen);
    }

    public double getPPYCor(int plotpen) {
	return ((LineLive)framePlot).ppycor(plotpen);
    }
    
    public void clearLegend() {
	framePlot.clearLegend();
    }
    
    /** Control whether the grid is drawn.
     *  @param grid If true, a grid is drawn.
     */
    public void setGrid(boolean grid) {
	framePlot.setGrid(grid);
	widgetPlot.setGrid(grid);
    }

    public void setWidgetGrid(boolean grid) {
	widgetPlot.setGrid(grid);
    }

    public boolean getGrid() {
	return framePlot.getGrid();
    }

    /** If the argument is true, then the default is to connect
     *  subsequent points with a line.  If the argument is false, then
     *  points are not connected.  When points are by default
     *  connected, individual points can be not connected by giving the
     *  appropriate argument to addPoint().  Also, a different default
     *  can be set for each dataset, overriding this global default.
     *
     * Only use when widgetPlot is a Plot
     */
    public void setConnected(boolean on) {
	((Plot)widgetPlot).setConnected(on);
    }
    
    public boolean getConnected() {
	return ((Plot)widgetPlot).getConnected();
    }

    public void ppd() {
	ppd(framePlot);
	ppd(widgetPlot);
    }

    public void ppd(PlotBox box) {
	((Plot)box).setConnected(true);
	((Plot)box).setMarksStyle("none");
	_setConnected((Plot)box, true);
    }

    public void ppu() {
	ppu(framePlot);
	ppu(widgetPlot);
    }

    public void ppu(PlotBox box) {
	((Plot)box).setConnected(false);
	((Plot)box).setMarksStyle("points");
	_setConnected((Plot)box, false);
    }

    /** Set the marks style to "none", "points", "dots", or "various".
     *  In the last case, unique marks are used for the first ten data
     *  sets, then recycled.
     *  @param style A string specifying the style for points.
     */
    /*
    public void setMarksStyle(String style) {
	((Plot)widgetPlot).setMarksStyle(style);
    }
    */
    /** Set the size of the plot widget.
     *  @param width The width, in pixels.
     *  @param height The height, in pixels.
     */
    public void setWidgetSize(int width, int height) {
	widgetPlot.setSize(width, height);
    }

    public Dimension getWidgetSize() {
	return widgetPlot.getSize();
    }
    
    public void setSLColor(int loc, int color) {
	framePlot._slcolors[loc] = color;
	widgetPlot._slcolors[loc] = color;
    }

    public int getSLColor(int loc) {
	return framePlot._slcolors[loc];
    }

    public void setColor(int loc, Color color) {
	framePlot._colors[loc] = color;
	widgetPlot._colors[loc] = color;
    }
    
    /**
     * Only use when internal PlotBoxes are LineLive
     */
    public void setPointsPersistence(int count) {
	((LineLive)framePlot).setPointsPersistence(count);
	((LineLive)widgetPlot).setPointsPersistence(count);
    }

    public void setHistoNum(int count) {
	((HistoLive)framePlot).setNum(count);
	((HistoLive)widgetPlot).setNum(count);
    }
    
    public void setLineNum(int count) {
	((LineLive)framePlot).setNum(count);
	((LineLive)widgetPlot).setNum(count);
    }

    public int getLineNum() {
			//	return ((LineLive)framePlot).num;
			return ((LineLive)framePlot).getNum();
    }

    public int getHistoNum() {
	return ((HistoLive)framePlot).num;
    }
   
    public double[] getHistoXRange() {
	return ((HistoLive)framePlot).getXRange();
    }

    public double[] getLineXRange() {
	return ((LineLive)framePlot).getXRange();
    }

    public double[] getWidgetXRange() {
	return widgetPlot.getXRange();
    }

    public double[] getWidgetYRange() {
	return widgetPlot.getYRange();
    }

    public double[] getFrameXRange() {
	return framePlot.getXRange();
    }

    public double[] getFrameYRange() {
	return framePlot.getYRange();
    }

    public void disableDataInFormatMenu() {
	slmenu.disableDataInFormatMenu();
    }

    /** Add a legend (displayed at the upper right) for the specified
     *  data set with the specified string.  Short strings generally
     *  fit better than long strings.  If the string is empty, or the
     *  argument is null, then no legend is added.
     *  @param dataset The dataset index.
     *  @param legend The label for the dataset.
     */
    public void addLegend(int dataset, String legend) {
	framePlot.addLegend(dataset, legend);
	repaintFrame();
    }
    
    public void repaintWithInterface() {
	repaint();
	app.iface.repaint(lw.position);
    }

    public void repaint() {
	repaintFrame();
	repaintWidget();
    }

    public void repaintFrame() {
	framePlot.repaint();
    }

    public void repaintWidget() {
	//widgetPlot.repaint();
    }

    /*
    public void show() {
	try { throw new Exception(); } catch (Exception e) { e.printStackTrace(); }
	super.show();
    }

    public void setVisible(boolean b) {
	try { throw new Exception(); } catch (Exception e) { e.printStackTrace(); }
	super.setVisible(b);
    }
    */

    public void reset() {
	//System.out.println("reset line widget");
	double[] xRange;
	if (type == LineWidget.HISTOGRAM) {
	    xRange = getHistoXRange();
	} else {
	    xRange = getLineXRange();
	}
	framePlot.clear(false, autoscaleX, autoscaleY, framePlot.getGrid());
	widgetPlot.clear(false, autoscaleX, autoscaleY, widgetPlot.getGrid());
	if (type == LineWidget.HISTOGRAM) {
	    setHistoTime(0.0);
	} else {
	    setLiveTime(0.0);
	}
	if (type == LineWidget.BAR) {
	    explicitLineXRange(xRange[0], xRange[1]);
	}
	repaintWithInterface();
    }
    
    public void setHistoTime(double t) {
	((HistoLive)framePlot).time = t;
	((HistoLive)widgetPlot).time = t;
    }

    public void setLiveTime(double t) {
	((LineLive)framePlot).time = t;
	((LineLive)widgetPlot).time = t;
    }
    
    public void explicitLineXRange(double min, double max) {
	((LineLive)framePlot).explicitXRange(min, max);
	((LineLive)widgetPlot).explicitXRange(min, max);
    }

    public void explicitWidgetXRange(double min, double max) {
	widgetPlot.explicitXRange(min, max);
    }

    public void removeHistoHashPoint(int who) {
	((HistoLive)framePlot).removeHashPoint(who);
	((HistoLive)widgetPlot).removeHashPoint(who);
    }

    public void removeLiveHashPoint(int who) {
	((LineLive)framePlot).removeHashPoint(who);
	((LineLive)widgetPlot).removeHashPoint(who);
    }

    private void autoscale() {
	if (autoscaleX && autoscaleY) {
	    framePlot.fillPlot();
	    widgetPlot.fillPlot();
	} else if (autoscaleX) {
	    framePlot.fillPlotX();
	    widgetPlot.fillPlotX();
	} else if (autoscaleY) {
	    framePlot.fillPlotY();
	    widgetPlot.fillPlotY();
	}
    }	

    /**
     * internal Plotboxes must be LineLives
     */
    public void addDataPoint(int plotpen, double x) {
	((LineLive)framePlot).addDataPoint(plotpen, x);
	((LineLive)widgetPlot).addDataPoint(plotpen, x);
	autoscale();	
    }

    public void addDataPoint(int plotpen, double x, double y) {
	((LineLive)framePlot).addDataPoint(plotpen, x, y);
	((LineLive)widgetPlot).addDataPoint(plotpen, x, y);
	autoscale();
    }

    public void addDataPointToHash(int dataset, int key, double x, double y) {
	((LineLive)framePlot).addDataPointToHash(dataset, key, x, y);
	((LineLive)widgetPlot).addDataPointToHash(dataset, key, x, y);
	autoscale();
    }

    public void addHistoDataPointToHash(int dataset, int key, double y, int num) {
	((HistoLive)framePlot).addDataPointToHash(dataset, key, y, num);
	((HistoLive)widgetPlot).addDataPointToHash(dataset, key, y, num);
	autoscale();
    }

    public Image getWidgetImage() {
	return widgetPlot.getOffscreenBuffer();
    }

    /** Specify a tick mark for the X axis.  The label given is placed
     *  on the axis at the position given by <i>position</i>. If this
     *  is called once or more, automatic generation of tick marks is
     *  disabled.  The tick mark will appear only if it is within the X
     *  range.
     *  @param label The label for the tick mark.
     *  @param position The position on the X axis.
     */
    public void addXTick(String label, double position) {
	framePlot.addXTick(label, position);
    }
    
    public void paintWidget(Graphics g) {
	widgetPlot.paintNoBuffer(g);
    }

    /*    public void paint(Graphics g) {
	super.paint(g);
    }
    */

    public void setFrameTitle(String title) {
	framePlot.setTitle(title, preTitle);
    }

    public void setBinWidth(int plotpen, double width) {
	((HistoLive)framePlot).setBinWidth(plotpen, width);
	((HistoLive)widgetPlot).setBinWidth(plotpen, width);
    }


    public void _exportImage() throws Exception {
	//System.out.println("In PlotLiveFrame._exportImage()");
	Image image = createImage(framePlot.getSize().width, 
				  framePlot.getSize().height);
	Graphics g = image.getGraphics();
	framePlot.printAll(g);
	app.sl.PictInstance.savePicture(app.sl, image);
    }

    public void _close() {
	dispose();
    }


    public void print() {
	Print print = new Print(this, framePlot.getTitle());
	print.print();
    }


    public int print(Graphics graphics, PageFormat pageFormat, int pageIndex) {
	if (pageIndex > 0) return Printable.NO_SUCH_PAGE;

	Graphics2D g2d = (Graphics2D)graphics;
	g2d.translate (pageFormat.getImageableX (),
		       pageFormat.getImageableY ());
	g2d.setPaint (Color.black);

	Dimension plotSize = framePlot.getMinimumSize();
	
	int plotWidth = plotSize.width;
	int plotHeight = plotSize.height;
	double pageWidth = pageFormat.getImageableWidth();
	double pageHeight = pageFormat.getImageableHeight();
	
	Print.scaleGraphics(g2d, pageWidth, pageHeight,
			    plotWidth, plotHeight);
	framePlot.print(g2d);
	return Printable.PAGE_EXISTS;
    }


    ///////////////////////////////////////////////////////////////////
	////                         private methods                   ////

	/** Get the current connected state of all the point in the
	 * plot.  NOTE: This method reaches into the public members of
	 * the Plot class, taking advantage of the fact that this class is
	 * in the same package.
	 */
	private boolean[][] _getConnected(Plot plot) {
	    Vector points = plot._points;
	    boolean[][] result = new boolean[points.size()][];
	    for (int dataset = 0; dataset < points.size(); dataset++) {
		Vector pts = (Vector)points.elementAt(dataset);
		result[dataset] = new boolean[pts.size()];
		for (int i = 0; i < pts.size(); i++) {
		    PlotPoint pt = (PlotPoint)pts.elementAt(i);
		    result[dataset][i] = pt.connected;
		}
	    }
	    return result;
	}

    /** Set the current connected state of all the point in the
     * plot.  NOTE: This method reaches into the public members of
     * the Plot class, taking advantage of the fact that this class is
     * in the same package.
     */
    private void _setConnected(Plot plot, boolean value) {
        Vector points = plot._points;
        // Make sure the default matches.
        plot.setConnected(value);
        boolean[][] result = new boolean[points.size()][];
        for (int dataset = 0; dataset < points.size(); dataset++) {
            Vector pts = (Vector)points.elementAt(dataset);
            result[dataset] = new boolean[pts.size()];
            boolean first = true;
            for (int i = 0; i < pts.size(); i++) {
                PlotPoint pt = (PlotPoint)pts.elementAt(i);
                pt.connected = value && !first;
                first = false;
            }
        }
    }

    /** delegate events to responsible parties
     */
    public void itemStateChanged(ItemEvent evt) {
    }

    public void actionPerformed(ActionEvent evt) {
    }


    /** Set the current connected state of all the point in the
     * plot.  NOTE: This method reaches into the public members of
     * the plot class, taking advantage of the fact that this class is
     * in the same package.
     */
    private void _restoreConnected(boolean[][] original) {
        Vector points = ((Plot)framePlot)._points;
        boolean[][] result = new boolean[points.size()][];
        for (int dataset = 0; dataset < points.size(); dataset++) {
            Vector pts = (Vector)points.elementAt(dataset);
            result[dataset] = new boolean[pts.size()];
            for (int i = 0; i < pts.size(); i++) {
                PlotPoint pt = (PlotPoint)pts.elementAt(i);
                pt.connected = original[dataset][i];
            }
        }
    }

}
