/*

OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--------------------------
@Copyright (c) 1997-1999 The Regents of the University of California.
All rights reserved.

Permission is hereby granted, without written agreement and without
license or royalty fees, to use, copy, modify, and distribute this
software and its documentation for any purpose, provided that the
above copyright notice and the following two paragraphs appear in all
copies of this software.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

                                                PT_COPYRIGHT_VERSION_2
                                                COPYRIGHTENDKEY
@ProposedRating Yellow (cxh@eecs.berkeley.edu)
@AcceptedRating Yellow (cxh@eecs.berkeley.edu)
*/
package ptolemy;

import java.awt.*;
import java.io.*;
import java.util.*;
import starlogo.LineWidget;
import starlogo.StarLogo;
import starlogo.*;

public class LineLive extends Plot {

    int n;
    double val;
    LineWidget lw;
    //Vector data;
    public double time = 0;
    private Application app;

    public LineLive(Application app, LineWidget lw, int type) {
	super();
	this.lw = lw;
	this.app = app;
	this.type = type;
	this.num = 1;
	initme();
	//data = new Vector();
    }

    public PlotBox copy(Application app, LineWidget l) {
	LineLive ll = new LineLive(app, l, type);
	super.copy(l, ll);
	ll.n = n;
	ll.val = val;
	ll.time = time;
	return ll;
    }

    public void allOff() {
	setGrid(false);
	setTitle("", lw.tbc.getPreTitle());
	setXLabel("");
	setYLabel("");
    }

    public Object clone() {
	try {
	    return super.clone();
	}
	catch (CloneNotSupportedException c) {
	    c.printStackTrace(); return "False";
	}
    }

    public void initme() {
	setupLineWidget(lw);
	if (StarLogo.DEBUG) {System.out.println("initialized lw");}
	setGrid(false);
	setBars(false);
	setConnected(true);
	setButtons(false);
	setPointsPersistence(0);
	setMarksStyle("none"); //i just like these! =)
	if (type == LineWidget.BAR) {
	    setBars(true);
	    setConnected(false);
	    //	    setPointsPersistence(1);
	}
	if (type == LineWidget.PHASE ||
	    type == LineWidget.XYPLOT) {
	    setBars(false);
	    setMarksStyle("points");
	}
	if (type == LineWidget.XYPLOT) {
	    setConnected(false);
	}
    }

    public synchronized void addDataPoint(int n, double val) {
	if (type == LineWidget.LINE) {
		//	    System.out.println("this: " + this + " n: " + n + ", " + time + ", " + getNum() + "," + val);

		// MTB
		//System.out.println("is connected " + n + ": " +_isConnected(n));
		//addPoint(n, (int)time, val, _isConnected(n-1));
		// previously addPoint was always just true?

		addPoint(n, (int)time, val, true);
		time += 1.0 / getNum();
	}
	if (type == LineWidget.BAR)
	 	  addPoint(1, n, val, false);

	//repaint();
    }

    public synchronized void addPoints() {
	throw new RuntimeException ("not implemented; use addDataPoint and addDataPointToVector instead");
    }

    public synchronized void addDataPoint(int n, double x, double y) {
	if (StarLogo.DEBUG) {System.out.println("add data point: " + n + "," + x + "," + y);}
	addPoint(n, x, y, true);
	//repaint();
    }

    //used for scatterplot and bar chart
    public synchronized void addDataPointToHash(int dataset, int key, double x, double y) {
	this.addHashPoint(dataset, key, x, y);
	if (type == LineWidget.BAR) {
		//System.out.println("repaint...");
	    //repaint();
	}
    }

    /////////////////////////////////////////
    public Hashtable hashtable = new Hashtable();

    public synchronized void addHashPoint(int dataset, int key, double x, double y) {
	Integer datasetInt = new Integer(dataset);
	PlotPoint pp = new PlotPoint();
	pp.x = x;
	pp.y = y;
	Hashtable temp = null;
	if (hashtable.get(datasetInt) == null) {
	    temp = new Hashtable();
	    temp.put(new Integer(key), pp);
	    hashtable.put(datasetInt, temp);
	}
	else {
	    Integer keyInt = new Integer(key);
	    temp = (Hashtable)hashtable.get(datasetInt);
	    if (temp.get(keyInt) != null) {
		PlotPoint ppTemp = (PlotPoint)temp.get(keyInt);
		//System.out.println("erasing...: x: " + ppTemp.x + " y: " + ppTemp.y);
		this.erasePoint(dataset, ppTemp.x, ppTemp.y);
	    }
	    temp.put(keyInt, pp);
	}
	//System.out.println("dataset: " + dataset + " x: " + x + " y: " + y);
	this.addPoint(dataset, x, y, getConnected());
    }

    public synchronized void removeHashPoint(int key) {
		
	for(Enumeration keys = hashtable.keys(); keys.hasMoreElements();) {
		Integer nextInt = (Integer)keys.nextElement();
	    if (hashtable.get(nextInt) != null) {
		Hashtable temp = (Hashtable)hashtable.get(nextInt);
		Integer keyInt = new Integer(key);
		if (temp.get(keyInt) != null) {
		    PlotPoint pp = ((PlotPoint)temp.get(keyInt));
		    erasePoint(nextInt.intValue(), pp.x, pp.y);
		    temp.remove(keyInt);
		}
	    }
	}
    }

    public synchronized void erasePoint(int dataset, double x, double y) {
	PlotPoint pp = null;
    Vector pointsVector;
    try {
	    pointsVector = (Vector)_points.elementAt(dataset);
    } catch (ArrayIndexOutOfBoundsException aioobe) {
        return;
    }
	for (int i=0; i<pointsVector.size(); i++) {
	    pp = (PlotPoint)pointsVector.elementAt(i);
	    if (pp.x == x && pp.y == y) {
		pointsVector.removeElementAt(i);
		return;
	    }
	}
	//System.out.println("ERROR: bad arguments to erasePoint(int, double, double)");
    }

    /////////////////////////////////////////


    public synchronized void addDataPointToVector(int n, double x, double y, int count) {
	throw new RuntimeException("addDataPointToVector not implemented.  use addDataPointToHash() instead.");
	/**
	Vector v = new Vector();
	v.addElement(new Integer(n));
	v.addElement(new Double(x));
	v.addElement(new Double(y));
	data.addElement(v);
	if (data.size() >= (num * count)) {
	    clear(false); //clear datasets before repainting
	    for (int i=0; i<data.size(); i++) {
		v = (Vector)data.elementAt(i);
		addPoint(((Integer)v.elementAt(0)).intValue(),
			 ((Double)v.elementAt(1)).doubleValue(),
			 ((Double)v.elementAt(2)).doubleValue(),
			 false);
	    }
	    repaint();
	    data = new Vector();
	    lw.parent.repaint(lw.position);
	}
	**/
    }//end sync function


    /**
       public synchronized void addPoints() {
       //addPoint(1, n-1, val, false);
       if (type == LineWidget.LINE) {
       addPoint(n, time[n]++ / num, val, true);
       }
    }
    */

    public synchronized void ppreset(int dataset) {
	//	time[dataset] = 0;
	clear(dataset);
    }

    /**
       public synchronized void addDataPoint(int n, double val) {
       this.n = n;
       this.val = val;
       this.addPoints();
       }
	  public synchronized void addDataPoint(int n, double x, double y) {
	    addPoint(n, x, y, false);
	    repaint();
	    }
    */

	// MTB
//     public void setNum(int num) {
	//System.out.println("num: " + num);
// 	if (num > 0) {
// 	    this.num = num;
// 	}
//     }
//     public int getNum() {
// 	return this.num;
//     }

    public double ppxcor(int plotpen) {
	double xcor = 0.0;
	try {
	    Vector pts = (Vector)_points.elementAt(plotpen);
	    if (pts.size() != 0) {
		xcor = ((PlotPoint)pts.elementAt(pts.size() - 1)).x;
	    }
	}
	catch (Exception e) {
	    e.printStackTrace();
	}
	return xcor;

    }
    public double ppycor(int plotpen) {
	double ycor = 0.0;
	try {
	    Vector pts = (Vector)_points.elementAt(plotpen);
	    if (pts.size() != 0) {
		ycor = ((PlotPoint)pts.elementAt(pts.size() - 1)).y;
	    }
	}
	catch (Exception e) {
	    e.printStackTrace();
	}
	return ycor;
    }

	int getNum() { return num; }

	void setNum(int num) { this.num = num; }

    //	private int time[] = new int[15];
	private double d1 = 0;
	private double d2 = 0;
	private double d3 = 0;
	private boolean _first = true;
	private double _count = 0.0;


	public void samplePlot() {
        Random random = new Random();
	}


    public String toString() {
	return LineWidget.getType(app, type) + lw.id + ((this == lw.tbc.widgetPlot) ? " widget" : " frame");
    }
}
