/*
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/


package starlogo;

import java.util.Enumeration;
import java.util.Vector;
import java.awt.event.*;
import java.awt.*;
import java.awt.image.*;

public class AltPatchCanvas extends PatchCanvas implements ImageProducer {

    ImageConsumer ic;
    int[] pixels;
    //TurtleShapes ts = new TurtleShapes();
    //int[][] shapes = ts.shapes;
  
	
    int rowbytes, nextlineoffset, nextpatchoffset;

    public AltPatchCanvas(StarLogo sl) {
	super(sl);
    }

    public AltPatchCanvas(StarLogo sl, PatchCanvasInterface oldpc) {
	super(sl, oldpc);
    }


    public boolean init(JInterface parent) {
	this.parent = parent;
	//GraphicsConfiguration gc = ((Component)parent).getGraphicsConfiguration();
	//image = gc.createCompatibleVolatileImage(position.width, position.height);
	image = parent.createImage(this);
	xoffset = position.x;
	yoffset = position.y;
	rowbytes = position.width;
	pixels = new int[position.width * position.height];
	nextlineoffset = rowbytes * (sl.cellsize - 1);
	nextpatchoffset = - ((rowbytes - 1) * sl.cellsize);
	updateImage();
	return true;
    }

    public void updateImage() {
	if (sl.cellsize < 8) 
	    updateImageNoShapes();
	else updateImageShapes();
    }

    public void updateImageNoShapes() {
	if (pixels == null) return;
	int poffy = sl.widthp2 + 1; // first visible logical patch
	int xypix = 0; // first pixel
	for (int y = sl.height; y != 0; y--) {
	    for (int x = sl.width; x != 0; x--) {
		Patch patch = sl.patches[poffy];				
		int color = 0;
		if (patch.topturtle != null && patch.topturtle.shownp) {
		    //drawTurtle(patch.topturtle);
		    color = Colors.colorarrayints[(int)(patch.topturtle.color * 32.0)];
		} else {
		    color = Colors.colorarrayints[(int)(patch.pc * 32.0)];
		}
		for(int i = sl.cellsize; i != 0; i--) {
		    for(int j = sl.cellsize-1; j >= 0; j--) {
			pixels[xypix + j] = color;
		    }
		    xypix += rowbytes;
		}
		pcdraws++;
		poffy++; // next logical patch
		xypix += nextpatchoffset; // next onscreen patch
	    }
	    xypix += nextlineoffset;
	    poffy += 2; // skip two patches
	}
	startProduction(ic);
	uptodate = true;
    }

    public void updateImageShapes() {
	if (pixels == null) return;
	int poffy = sl.widthp2 + 1; // first visible logical patch
	int xypix = 0; // first pixel
	int cellsize = sl.cellsize;
	for (int y = sl.height; y != 0; y--) {
	    for (int x = sl.width; x != 0; x--) {
		Patch patch = sl.patches[poffy];				
		if (patch.topturtle != null && patch.topturtle.shownp) {
		    //drawTurtle(patch.topturtle);
		    int tcolmask = Colors.colorarrayints[(int)(patch.topturtle.color * 32.0)];
		    int pcolmask = Colors.colorarrayints[(int)(patch.pc * 32.0)];
		    TurtleShapes shape = TurtleShapes.getShape(patch.topturtle.shape);
		    int heading = ((int) ((patch.topturtle.heading+11.25) / 22.5)) % 16;
		    double[] colors = (shape.getColorsForSize(cellsize))[heading];
		    int[]    masks  = (shape.getMasksForSize(cellsize))[heading];
		    for (int i=0, z=0; i<cellsize; i++) {
			for (int j=0; j<cellsize; j++) {
			    pixels[xypix + j] = (masks[2*z] & tcolmask) |
				(masks[2*z+1] & pcolmask) | 
				Colors.colorarrayints[(int)(colors[z] * 32.0)];
			    z++;
			}
			xypix += rowbytes;
		    }
		} else {
		    //drawPatch(patch);
		    int color = Colors.colorarrayints[(int)(patch.pc * 32.0)];
		    for(int i = cellsize; i != 0; i--) {
			for(int j = cellsize-1; j >= 0; j--) {
			    pixels[xypix + j] = color;
			}
			xypix += rowbytes;
		    }
		}
		pcdraws++;
		poffy++; // next logical patch
		xypix += nextpatchoffset; // next onscreen patch
	    }
	    xypix += nextlineoffset;
	    poffy += 2; // skip two patches
	}
	startProduction(ic);
	uptodate = true;
    }

    public void drawPatch(double xcor, double ycor, double color) {
	drawPatch(xcor, ycor, color, sl.patches);
    } 

    public void drawPatch(double xcor, double ycor, double color, Patch[] patches) {
	drawCell((int)xcor, (int)ycor, Colors.colorarrayints[(int)(color * 32.0)]);
    }

    public void drawPatch(Patch p) {
	drawPatch(p, sl.patches); 
    }
    
    public void drawPatch(Patch p, Patch[] patches) {
	drawCell((int)p.xcor-1, (int)p.ycor-1, Colors.colorarrayints[(int)(p.pc * 32.0)]);
    }

    public void drawTurtle(Turtle t) {
	drawTurtle(t, sl.patches); 
    }
    
    public void drawTurtle(Turtle t, Patch[] patches) {
	if (!displayp) return;

	int cellsize = sl.cellsize;
			
	if (cellsize < 8) { 
	    drawCell((int)t.xcor, (int)t.ycor, Colors.colorarrayints[(int)(t.color * 32.0)]);
	    return;
	}
	pcdraws++;
	int startx = ((int)(t.xcor)) * cellsize;
	int starty = ((int)(t.ycor)) * cellsize;
			
	int xypix = startx + (starty * rowbytes);
	int oldxypix = xypix;
	int tcol = Colors.colorarrayints[(int)(t.color * 32.0)];
	
	int pcol = Colors.colorarrayints[(int)((patches[t.patch].pc) * 32.0)];
	int z=0;
	TurtleShapes ts = TurtleShapes.getShape(t.shape);
	int heading = (int) (((t.heading+11.25) / 22.5) % 16);
	double[] colors = (ts.getColorsForSize(cellsize))[heading];
	int[]    masks  = (ts.getMasksForSize(cellsize))[heading];
	int index = 0;
	try {
	    for (int i=0; i<cellsize; i++) {
		for (int j=0; j<cellsize; j++) {
		    //index = i*cellsize + j;
		    pixels[xypix + j] = (masks[2*index] & tcol) | 
			(masks[2*index+1] & pcol) |
			Colors.colorarrayints[(int)(colors[index] * 32.0)];
		    index++;
		}
		xypix += rowbytes;
	    }
	} catch (IndexOutOfBoundsException e) {
	    System.out.println("Exception trying to draw turtle at (" + t.xcor + 
			       ", " + t.ycor + ") = [" + startx + ", " + starty + "]");
	    e.printStackTrace();
	}
	if (ic != null) {
	    ic.setPixels(startx, starty, cellsize, cellsize, Colors.cm, pixels, oldxypix, rowbytes);
	    //ic.imageComplete(ImageConsumer.SINGLEFRAMEDONE);
	}
	sl.patchesneedalittleredrawingp = true;
	uptodate = true;
	// } closes else block
    }

    void drawCell(int x, int y, int color) {
	if (!displayp) return;
	pcdraws++;
	int cellsize = sl.cellsize;
	int startx = x * cellsize;
	int starty = y * cellsize;
	int xypix = startx + (starty * rowbytes); // go to the right patch
	int oldxypix = xypix;
	//System.out.println("print pixels at " + x + " " + y + " xypix = " + xypix + " color = " + Integer.toString(color, 16));
	try {
	    for(int i = 0; i < cellsize; i++) {
		for(int j = 0; j < cellsize; j++) {
		    pixels[xypix + j] = color;
		}
		xypix += rowbytes;
	    }
	} catch (IndexOutOfBoundsException e) {
	    System.out.println("Exception trying to draw cell at (" + x + 
			       ", " + y + ") = [" + startx + ", " + starty + "]");
	    e.printStackTrace();
	}
	//printPixels(x, y);
	if (ic != null) {
	    ic.setPixels(startx, starty, cellsize, cellsize, Colors.cm, pixels, oldxypix, rowbytes);
	    //ic.imageComplete(ImageConsumer.SINGLEFRAMEDONE);
	}
	sl.patchesneedalittleredrawingp = true;
	uptodate = true;
    }

    public void printPixels(int whichx, int whichy) {
	int poffy = sl.widthp2 + 1; // first visible logical patch
	int xypix = 0; // first pixel
	for (int y = sl.height; y != 0; y--) {
	    for (int x = sl.width; x != 0; x--) {
		Patch patch = sl.patches[poffy];
		int color = 
		    Colors.colorarrayints[(int)(((patch.topturtle != null && patch.topturtle.shownp) ?
						 patch.topturtle.color : patch.pc) * 32.0)];
		if ((whichx == (sl.width - x)) &&
		    (whichy == (sl.height - y))) {
		    //System.out.println("x = " + (sl.width - x) + " y = " + (sl.height - y));
		    //System.out.println("xypix = " + xypix);
		    //System.out.println("color is " + Integer.toString(pixels[xypix], 16));
		}
		for(int i = sl.cellsize; i != 0; i--) {
		    for(int j = sl.cellsize-1; j >= 0; j--) {
			pixels[xypix + j] = color;
		    }
		    xypix += rowbytes;
		}
		//draw patch
		poffy++; // next logical patch
		xypix += nextpatchoffset; // next onscreen patch
	    }
	    xypix += nextlineoffset;
	    poffy += 2; // skip two patches
	}
    }	


    public void setOffscreen(boolean os) {
	//synchronized(sl.runningp) {
	sl.offscreen = os;
	if (os) {
	    sl.locked = true;
	    //setGraphics(image.getGraphics());
	    updateImage();
	    xoffset = 0;
	    yoffset = 0;
	    //	System.out.println("offscreen is true");
	} else {
	    // keep the turtles from moving until the patchcanvas image is written to screen
	    //sl.locked = true;
	    //setGraphics(getDrawingSurface());
	    xoffset = position.x;
	    yoffset = position.y;
	    //System.out.println("offscreen is false");
	}
	//}
    }

    public void clearImage() {
	//System.out.println("clear image");
	int size = position.height * position.width;
	for(int i = 0; i<size; i++) {
	    pixels[i] = Colors.colorarrayints[0]; // black
	}
	startProduction(ic);
    }

    public void refresh() {
	if (ic != null) {
	    ic.imageComplete(ImageConsumer.SINGLEFRAMEDONE);
	    startProduction(ic);
	}
	parent.repaint(position);
	
	/*
	  Graphics g = parent.getGraphics();
	  g.drawImage(image, position.x, position.y, parent);
	  g.dispose();
	*/
    }

    public void paint(Graphics g) {
	//GraphicsConfiguration gc = ((Component)parent).getGraphicsConfiguration();
	//int valcode = ((VolatileImage)image).validate(gc);
	//if (valcode == VolatileImage.IMAGE_INCOMPATIBLE) {
	//    init(parent);
	//}
    	if (parent == null) return;
	if (parent.resizingp() && parent.widgetTouched() == this) {
	    //System.out.println("resizing graphics screen");
	    g.drawImage(image, originalPosition.x, originalPosition.y, this);
	    g.setColor(Color.gray);
	    g.drawRect(position.x, position.y, position.width, position.height);
	} else {
	    boolean newoffscreen = getOffscreen();
	    //System.out.println(newoffscreen);
	    boolean changedtofalse = false;
	    if (newoffscreen!=sl.offscreen) {
		//System.out.println("different");
		setOffscreen(newoffscreen);
		changedtofalse = !newoffscreen;
	    }
	    //synchronized(sl.runningp) {
	    if (!uptodate && displayp) updateImage();
	    //long time = System.currentTimeMillis();
	    //try {Thread.currentThread().sleep(50); } catch(Exception e) {}
	    g.drawImage(image, position.x, position.y, this);
	    //super.paint(g);
	    if (changedtofalse && sl.locked) {
		// unlocking turtles
		//synchronized(sl.runningp) {
		sl.locked = false;
		sl.runningp.notify();
		//}
	    }
	}
    }


    public void addConsumer(ImageConsumer ic) {
	startProduction(ic);
	//System.out.println("called add Consumer with " + ic);
    }

    public boolean isConsumer(ImageConsumer ic) {
	return (this.ic == ic); 
    }

    public void removeConsumer(ImageConsumer ic) {
	//System.out.println("called remove consumer wiht " + ic);
	this.ic = null; 
    }

    public void requestTopDownLeftRightResend(ImageConsumer ic) {
	ic.setHints(ImageConsumer.TOPDOWNLEFTRIGHT);
	ic.setPixels(0, 0, position.width, position.height, 
		     Colors.cm, pixels, 0, rowbytes);	// As many times as needed
	ic.imageComplete(ImageConsumer.SINGLEFRAMEDONE);
	ic.setHints(ImageConsumer.RANDOMPIXELORDER);
    }

    public void startProduction(ImageConsumer ic) {
	if (ic == null) return;
	//System.out.println("called start production with " + ic);
	if (this.ic == null) {
	    this.ic = ic; 
	    ic.setColorModel(Colors.cm);
	    //System.out.println("color model is " + Colors.cm);
	    ic.setDimensions(position.width, position.height);
	}
	ic.setHints(ImageConsumer.TOPDOWNLEFTRIGHT);
	ic.setPixels(0, 0, position.width, position.height,
		     Colors.cm, pixels, 0, rowbytes);	// As many times as needed
	ic.imageComplete(ImageConsumer.SINGLEFRAMEDONE);
	//ic.setHints(ImageConsumer.RANDOMPIXELORDER);
    }

  
    public void setDrawingSurface() { 
	/*
	  try
	  {
	  throw new Exception();
	  } catch (Exception e) {
	  e.printStackTrace();
	  }
	*/
	return;
    }
  
}

