/*
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/


package starlogo;

import yoyo.YoYo;
import yoyo.EnqueueContext;
import yoyo.Context;

import java.awt.*;
import java.awt.event.*;
import java.awt.datatransfer.*;
import java.io.*;
import java.net.*;


import java.util.*;
import yoyo.Tokenizer;
import java.lang.reflect.*;

import ptolemy.*;
//import ptolemy.plot.linechart;
import ptolemy.PlotBox;
//import ptolemy.plot.plotml.histogram;
// added by MIM
//import template.general.*;
// end added by MIM
import javax.swing.SwingUtilities;
//import logoblocks.*; //use reflection

public class Application {

    public volatile boolean initializedp = false;

    protected Vector windows = new Vector();
    public StarLogoFrame activeFrame;
    private Stack cursorStack = new Stack();


    // these fields especially are passed as state within the Application object
    public ControlCenter procWin;
    public InfoWindow infoWin;
    public OutputWindow outWin;
    //public RTGraphWindow plotWin;
    public JInterface iface;
	  public ShapeChooser shapeChooser;
  //added by MIM
  public template.Template theTemplate; //Template template;
  //end added by MIM

	public Vector barcharts;
	public Vector linecharts;
	public Vector histograms;


    public StarLogo sl;
    public Project project;
    public Clipboard clip;
    public EnqueueContext cc;
    public PrintWriter message;
    public PrintWriter err;
    public PrintWriter log;

    //public MacUtils macutils;
    public FilenameFilter projectFileFilter;
    public FilenameFilter textFileFilter;
    public FilenameFilter htmlFileFilter;
    public FilenameFilter pictureFileFilter;
    public FilenameFilter shapeFileFilter;

    public boolean doubleBufferp = true;

	public static Application starlogoapplication;

    // other public fields
    public volatile boolean rancompilerp = true, compiled = true;
    public volatile boolean rancompilerpForVariables = true, 
	compiledVariables = true;
    public volatile boolean disableMonitors = false;

    public  PropertyResourceBundle textBundle, commandsBundle; 
    // private
    private Closer closer;
    
    public boolean htmlp = false;
    public boolean kioskp = false;
	public boolean locked = false;

   // for network starlogo
   public Application(StarLogo sl, Socket client, Context c) {   
      this.htmlp = true;
      this.kioskp = false;
	  this.locked = false;
      err = new PrintWriter(System.out, true);
      starlogoapplication = this;
      this.sl = sl;
      sl.app = this;
      if (!YoYo.swingavailablep) System.exit(1);
      sl.init(client);
      // initialize event listeners before constructing windows so that they windows can add
      // them as they're constructed
      this.procWin = newControlCenter(this);
      cursorStack.push(new Cursor(Cursor.DEFAULT_CURSOR));
      addWindow(this.procWin);
      this.iface = createInterface(this);
      this.project = new Project(this);
      TurtleShapes.loadDefaultShapes(this);
      // procWin is initially active frame
      initializedp = true;
    }

    public Application(StarLogo sl, boolean htmlp, boolean kioskp, Context c) {
	this.htmlp = htmlp; 
	this.kioskp = kioskp;
	this.locked = false;
	setupPlatformSpecificClasses();
	err = new PrintWriter(System.out, true); 
	log = new PrintWriter(getLogFileStream(), true);
	if (!htmlp) cc = new EnqueueContext(c, err, err, log);
	sl.c.log = log;
	sl.c.err = err;
	starlogoapplication = this;
	this.sl = sl;
	sl.app = this;
	if (!YoYo.swingavailablep) System.exit(1);

	textBundle = ((PropertyResourceBundle)ResourceBundle.getBundle("starlogo.GUIBundle", YoYo.language));
	commandsBundle = ((PropertyResourceBundle)ResourceBundle.getBundle("starlogo.SLCommands", YoYo.language));
	sl.init();
	cursorStack.push(new Cursor(Cursor.DEFAULT_CURSOR));
	// initialize event listeners before constructing windows so that they windows can add
	// them as they're constructed
	if (!htmlp) {
	    this.closer = new Closer(this);
	    // initialize procwin, interface, and project
	    try {
		clip = Toolkit.getDefaultToolkit().getSystemClipboard();
	    }
	    catch (Exception e) {
		clip = new Clipboard("StarLogo Clipboard");
	    }
	}
	
	
	if (!htmlp) {
	    this.procWin = newControlCenter(this);
	    this.activeFrame = this.procWin;
	    addWindow(this.procWin);
	    
	    this.infoWin = new InfoWindow(this);
	    this.infoWin.init();
	    addWindow(this.infoWin);
	    this.outWin = new OutputWindow(this);
	    this.outWin.init();
	    addWindow(this.outWin);
	    //this.plotWin = new RTGraphWindow(this);
	    //addWindow(this.plotWin);
	    // !@!@
	    this.barcharts = new Vector();
	    this.linecharts = new Vector();
	    this.histograms = new Vector();      
	    // added by MIM
	    if (YoYo.swingavailablep && !YoYo.macintoshp) 
		this.theTemplate = template.general.Template.makeTemplate(this);
	    // end added by MIM
	}
	
      
      this.iface = createInterface(this);
      this.iface.init();
      this.project = new Project(this);

      TurtleShapes.loadDefaultShapes(this);
      if (!htmlp) this.shapeChooser = createShapeChooser(this);
      
      // procWin is initially active frame
      if (htmlp && YoYo.applet != null) {
	  YoYo.applet.add((Component)this.iface);
      } 

      if (sl.patchcanvas instanceof PatchCanvas)
	  ((PatchCanvas)sl.patchcanvas).init(this.iface);

      if (!htmlp) {
	  this.activeFrame = procWin;
		// moved to TopLevel
// 	  iface.frame().show();
// 	  procWin.show();
      }

      maybeCompileAndThenRun(false, new Runnable() {
	      public void run() {
		  initializedp = true;
	      }
	  });
    }

    public void setupPlatformSpecificClasses() {
	if (!kioskp && YoYo.applet == null) {
	    if (YoYo.macintoshp) {
		try {
		    Class machandlers = Class.forName("starlogo.MacHandlers");
		    Constructor cons = machandlers.getConstructor(new Class[]{Application.class});
		    cons.newInstance(new Object[]{this});
		    if (YoYo.macosxp) {
			//macutils = (MacUtils)Class.forName("starlogo.MacUtils10").newInstance();
			if (YoYo.swingavailablep) {
			    doubleBufferp = false;
			}

			projectFileFilter = (FilenameFilter)Class.forName("starlogo.FilterBySuffix").newInstance();
			textFileFilter = (FilenameFilter)Class.forName("starlogo.FilterBySuffix").newInstance();
			htmlFileFilter = (FilenameFilter)Class.forName("starlogo.FilterBySuffix").newInstance();
			pictureFileFilter = (FilenameFilter)Class.forName("starlogo.FilterBySuffix").newInstance();
			shapeFileFilter = (FilenameFilter)Class.forName("starlogo.FilterBySuffix").newInstance();
			Method filterMethod = Class.forName("starlogo.FilterBySuffix").getMethod("addSuffix", new Class[]{String.class});
			//filterMethod.invoke(projectFileFilter, new Object[]{"slogo"});
			//filterMethod.invoke(projectFileFilter, new Object[]{"slo"});
			//filterMethod.invoke(projectFileFilter, new Object[]{""});
			filterMethod.invoke(htmlFileFilter, new Object[]{"html"});
			filterMethod.invoke(htmlFileFilter, new Object[]{"htm"});
			filterMethod.invoke(pictureFileFilter, new Object[]{"gif"});
			filterMethod.invoke(pictureFileFilter, new Object[]{"png"});
			filterMethod.invoke(pictureFileFilter, new Object[]{"jif"});
			filterMethod.invoke(pictureFileFilter, new Object[]{"jpg"});
			filterMethod.invoke(pictureFileFilter, new Object[]{"jpeg"});
			filterMethod.invoke(shapeFileFilter, new Object[]{"sls"});
		    }
		} 
		catch (IllegalAccessException e) { e.printStackTrace(); }
		catch (InstantiationException f) { f.printStackTrace(); }
		catch (ClassNotFoundException g) { g.printStackTrace(); }
		catch (NoSuchMethodException h) { h.printStackTrace(); }
		catch (InvocationTargetException i) { i.printStackTrace(); }
	    } else {
		if (YoYo.swingavailablep) doubleBufferp = false;
		try {
		    projectFileFilter = (FilenameFilter)Class.forName("starlogo.FilterBySuffix").newInstance();
		    textFileFilter = (FilenameFilter)Class.forName("starlogo.FilterBySuffix").newInstance();
		    htmlFileFilter = (FilenameFilter)Class.forName("starlogo.FilterBySuffix").newInstance();
		    pictureFileFilter = (FilenameFilter)Class.forName("starlogo.FilterBySuffix").newInstance();
		    shapeFileFilter = (FilenameFilter)Class.forName("starlogo.FilterBySuffix").newInstance();
		    Method filterMethod = Class.forName("starlogo.FilterBySuffix").getMethod("addSuffix", new Class[]{String.class});
		    filterMethod.invoke(projectFileFilter, new Object[]{"slogo"});
		    filterMethod.invoke(projectFileFilter, new Object[]{"slo"});
		    filterMethod.invoke(htmlFileFilter, new Object[]{"html"});
		    filterMethod.invoke(htmlFileFilter, new Object[]{"htm"});
		    filterMethod.invoke(pictureFileFilter, new Object[]{"gif"});
		    filterMethod.invoke(pictureFileFilter, new Object[]{"png"});
		    filterMethod.invoke(pictureFileFilter, new Object[]{"jif"});
		    filterMethod.invoke(pictureFileFilter, new Object[]{"jpg"});
		    filterMethod.invoke(pictureFileFilter, new Object[]{"jpeg"});
		    filterMethod.invoke(shapeFileFilter, new Object[]{"sls"});
		} 
		catch (IllegalAccessException e) { e.printStackTrace(); }
		catch (InstantiationException f) { f.printStackTrace(); }
		catch (ClassNotFoundException g) { g.printStackTrace(); }
		catch (InvocationTargetException h) { h.printStackTrace(); }
		catch (NoSuchMethodException i) { i.printStackTrace(); }
	    }
	}
    }
    
    public OutputStream getLogFileStream() {
	// make a new file called starlogo<blah>.log
	// in a temp directory
	if (!YoYo.macintoshp && !kioskp && YoYo.applet == null) {
	    try {
		File f = File.createTempFile("starlogo", ".log");
		return new BufferedOutputStream(new FileOutputStream(f));
	    } catch (Exception e) { 
		return System.err;
	    }
	} return System.err;
    }

    private class CompileRunner implements Runnable {
	private Runnable toDoWhenDone;
	private Runnable toDoWhenFailed;
	private boolean justVariables;
	private ProgressBar pb;

	public CompileRunner(boolean justVariables, 
			     Runnable success, Runnable failure, 
			     ProgressBar pb) {
	    toDoWhenDone = success;
	    toDoWhenFailed = failure;
	    this.justVariables = justVariables;
	    this.pb = pb;
	}

	public boolean needToWait() {
	    if (justVariables) {
		return !rancompilerpForVariables;
	    } else {
		return !rancompilerp;
	    }
	}
	
	public boolean successfulCompile() {
	    if (justVariables) {
		return compiledVariables;
	    } else {
		return compiled;
	    }
	}

	public void run() {
	    rancompilerp = false;
	    rancompilerpForVariables = false;
	    project.compile(Application.this, justVariables, pb);
	    while(needToWait()) {
		try {
		    Thread.currentThread().sleep(100);
		} catch (InterruptedException ie) {}
	    }
	    //System.out.println("compile finished");
	    if (successfulCompile()) {
		toDoWhenDone.run();
	    } else {
		toDoWhenFailed.run();
	    }
	}
    }

    public void maybeCompileAndThenAlwaysRun(boolean justVariables, Runnable r) {
	maybeCompileAndThenRunElseRun(justVariables, r, r);
    }

    public void maybeCompileAndThenRun(boolean justVariables, Runnable r) {
	maybeCompileAndThenRunElseRun(justVariables, r, 
				      new Runnable() { public void run() {} });
    }

    public void maybeCompileAndThenRunElseRun(boolean justVariables, Runnable success, Runnable failure) {
	maybeCompileAndThenRunElseRunIfFailedAlotRun(justVariables, success, failure,
						     new Runnable() { public void run() {} });
    }

    public void maybeCompileAndThenRunElseRunIfFailedAlotRun(boolean justVariables, Runnable success, Runnable failure, Runnable multifailure) {
	if (kioskp || YoYo.applet != null) {
	    // don't compile in applets and kiosk
	    success.run(); 
	    return;
	}
	// Have you already tried running the compiler?
	if ( !justVariables && rancompilerp ) {
	    // Did it succeed?
	    if ( (!justVariables && compiled) || 
		 (justVariables && compiledVariables)) {
		//System.out.println("Already compiled.");
		// just run success and exit 
		success.run();
	    } else {
		//System.out.println("Already ran compiler, but compile failed.");
		multifailure.run();
	    }
	    return;
	} else {
	    //System.out.println("Need to run compiler.");
	    ProgressBar pb = new ProgressBar(this, textBundle.getString("compiling"));
	    CompileRunner cr = new CompileRunner(justVariables, success, failure, pb);
	    new Thread(cr).start();
	}
    }

    private MessageDialog md;

    public void registerCloser(Frame c) {
	c.addWindowListener(closer);
    }

    private class Closer extends WindowAdapter {
	Application app;

	public Closer(Application app) {
	    this.app = app;
	}

	public void windowClosing(WindowEvent evt) {
	    if (!YoYo.macintoshp) {
		project.quit();
	    }
	}
    }

    public void registerFrameTracker(Frame f) {
	f.addWindowListener(new FocusTracker(f));
    }

    private class FocusTracker extends WindowAdapter {
	Frame f;

	public FocusTracker(Frame f) {
	    this.f = f;
	}

	public void windowActivated(WindowEvent e) {
	    activeFrame = (StarLogoFrame)f;
	}
    }

    /* Compiling Widgets */

    void setButtonCommand(int num, Object[] ilist, Object[] olist) {
    //System.out.println("set button command for button " + num);
	Object o = iface.getButton(num);
	//System.out.println("button is " + o.toString());
	if (o instanceof ButtonWidget) {
	    ButtonWidget bw = (ButtonWidget)o;
	    bw.setCompiledCommand(ilist, olist);
	}
    }

    void setMonitorCommand(int num, Object[] ilist, Object[] olist,
			   Object[] ilist2, Object[] olist2) {
	Object o = iface.getMonitor(num);
	if (o instanceof MonitorWidget) {
	    MonitorWidget mw = (MonitorWidget)o;
	    mw.setCompiledCommand(ilist, olist, ilist2, olist2);
	}
    }

	void setLineCommand(int num, Object[] ilist, Object[] olist,
				   Object[] ilist2, Object[] olist2) {
		Object o = iface.getLineWidget(num);
		if (o instanceof LineWidget) {
		    LineWidget mw = (LineWidget)o;
		    mw.setCompiledCommand(ilist, olist, ilist2, olist2);
		}
    }



    /* Executing Widgets */

    void runTurtleCommand(Object[] ilist, Object[] olist, SLCommand slc) {
	slc.runCompiledCommand(ilist, olist);
    }

    void runTurtleCommand(Object[] ilist, Object[] olist, TurtleMonitor slc) {
	slc.runCompiledCommand(ilist, olist);
    }

    void runPatchCommand(Object[] ilist, Object[] olist, SLCommand slc) {
	slc.runCompiledCommand(ilist, olist);
    }

    void runPatchCommand(Object[] ilist, Object[] olist, PatchMonitor slc) {
	slc.runCompiledCommand(ilist, olist);
    }

    void runObserverCommand(Object[] ilist, Object[] olist, SLCommand slc) {
	slc.runCompiledCommand(ilist, olist);
    }

		void runColorChooserCommand(Object[] ilist, Object[] olist, ColorChooser cc) {
	cc.runCompiledCommand(ilist, olist);
		}

    int numMonitors() {
	return iface.numMonitorsOn() + iface.numLineWidgetsOn();
    }

    void runMonitors() {
	if (disableMonitors) return;

	sl.frameCount.reset();
	
	MonitorWidget[] mwarray = iface.getMonitors();
	synchronized (sl.runningp) {
	    for(int i = 0; i<mwarray.length; i++) {
		//System.out.println("starting monitor " + mwarray[i]);
		MonitorWidget mw = mwarray[i];
		if (mw.whichip >= 0)
		    mw.sp = StackProcessParent.runMonitor(sl, mw.whichip, true, mw);
	    }
	}
	LineWidget[] lwarray = iface.getLineWidgets();
	synchronized (sl.runningp) {
	    for(int i = 0; i<lwarray.length; i++) {
		//System.out.println("starting monitor " + mwarray[i]);
		LineWidget lw = lwarray[i];
		if (lw.whichip >= 0)
		    lw.sp = StackProcessParent.runLineWidget(sl, lw.whichip, true, lw);
	    }
	}

    }


    void runMonitorsOnce() {
	if (disableMonitors) return;
	
	//if patch canvas is covered, monitors won't run, 
	//or have error, they'll just be blank
	if (sl.locked == true) {return;}
	else{
	MonitorWidget[] mwarray = iface.getMonitors();
	synchronized (sl.runningp) {
	    for(int i = 0; i<mwarray.length; i++) {
		//System.out.println("starting monitor " + mwarray[i]);
		MonitorWidget mw = mwarray[i];
		if (mw.whichip >= 0)
		    mw.sp = StackProcessParent.runMonitor(sl, mw.onewhichip, false, mw);
	    }
	}
	LineWidget[] lwarray = iface.getLineWidgets();
	synchronized (sl.runningp) {
		for(int i = 0; i<lwarray.length; i++) {
		//System.out.println("starting monitor " + mwarray[i]);
		LineWidget lw = lwarray[i];
		if (lw.whichip >= 0 && 
		    lw.type != LineWidget.LINE &&
		    lw.type != LineWidget.PHASE)
			lw.sp = StackProcessParent.runLineWidget(sl, lw.onewhichip, false, lw);
		}
	}
    }
    }
    void stopMonitors() {
	if (disableMonitors) return;
	//System.out.println("stop monitors");
	MonitorWidget[] mwarray = iface.getMonitors();
	synchronized(sl.runningp) {
	    for (int i = 0; i<mwarray.length; i++) {
		//System.out.println("stopping monitor " + mwarray[i]);
		MonitorWidget mw = mwarray[i];
		if (mw.sp != null) {
		    sl.list.removeMonitor(mw.sp);
		    // Run the monitor one last time
		    //System.out.println("run monitor one last time " + mwarray[i]);
		    if (mw.onewhichip >= 0) StackProcessParent.runMonitor(sl, mw.onewhichip, false, mw);
		    mw.sp = null;
		}
	    }
	}
	LineWidget[] lwarray = iface.getLineWidgets();
	synchronized(sl.runningp) {
		for (int i = 0; i<lwarray.length; i++) {
		LineWidget lw = lwarray[i];
		if (lw.sp != null) {
			sl.list.removeLineWidget(lw.sp);
			// Run the monitor one last time
			//System.out.println("run monitor one last time " + mwarray[i]);
			if (lw.onewhichip >= 0) StackProcessParent.runLineWidget(sl, lw.onewhichip, false, lw);
			lw.sp = null;
		}
		}
	}
    }

    /* Cursor management */

    
  public void addWindow(StarLogoFrame slf) {
      windows.addElement(slf);
  }

    public void removeWindow(StarLogoFrame slf) {
	windows.removeElement(slf);
    }

  public void setCursor(Cursor c) {
      cursorStack.push(c);
      for (Enumeration e = windows.elements(); e.hasMoreElements(); ) {
	  ((StarLogoFrame)e.nextElement()).setCursor(c);    
      }
  }
    
    public Cursor getCursor() {
	return ((Cursor)(cursorStack.peek()));
    }
    
    public void restoreCursor() {  	
	cursorStack.pop();
	Cursor c = getCursor();
	for (Enumeration e = windows.elements(); e.hasMoreElements(); ) {
	    ((StarLogoFrame)e.nextElement()).setCursor(c);    
	}
    }


    public void markChanged() {
	for (Enumeration e = windows.elements(); e.hasMoreElements(); ) {
	    Object o = e.nextElement();
	    ((StarLogoFrame)o).markChanged();    
	}
    }

    public void clearChanged() {
	for (Enumeration e = windows.elements(); e.hasMoreElements(); ) {
	    ((StarLogoFrame)e.nextElement()).clearChanged();    
	}
    }

  // !@!@
    public int createNewMonitorWidget(int i) {
  	  return MonitorWidget.createNewFromPtolemy(this.iface, i);
    }

  // !@!@
    public static String startswithp(String s, String[] list) {
      for(int i = 0; i<list.length; i++) {
        if (s.startsWith(list[i])) return list[i];
      }
      return null;
    }
  // !@!@
    public static void removeQuotes(Vector v) {
      String str;
      for (int i=0; i<v.size(); i++) {
        if (v.elementAt(i) instanceof String) {
  	str = (String)v.elementAt(i);
  	if (str.startsWith("\"")&&str.endsWith("\""))
  	  v.setElementAt(str.substring(1, str.length()-1), i);
        }
      }
    }

    public void usedPlotCommands() {
	//System.out.println("i used a plot command!");
	boolean needRetrofit = true;
	for (int i=0; i<iface.linewidgets().size(); i++) {
	    if (!(((LineWidget)iface.linewidgets().elementAt(i)).wizardOn)) {
		//if there exists a LineWidget NOT using the wizard,
		//then no need to add new retrofit widget, else we
		//need to add (or "re-add") it to the canvas.
		needRetrofit = false;
	    }
	}//end for
	if (needRetrofit) {
	    //System.out.println("placing widget in corner");
	    LineWidget.createNew(LineWidget.LINE, iface);
	}
    }

    public String[] getShapeNames() {
	return TurtleShapes.getShapeNames();
    }

    public static void setTurtleMonitor(Turtle t) {
	    JTurtleMonitor.setTurtle(t);
    }

    public static void updateEntityMonitors() {
	    JTurtleMonitor.updateOpen();
	    JPatchMonitor.updateOpen();
    }


    public static TurtleMonitor newTurtleMonitor(Turtle tur, double delay, Application app) {
	    return new JTurtleMonitor(tur, delay, app);
    }

    public static PatchMonitor newPatchMonitor(Patch tur, double delay, Application app) {
	    return new JPatchMonitor(tur, delay, app);
    }

    public static ControlCenter newControlCenter(Application app) {
	    return new JControlCenter(app);
    }



  public static ShapeChooser createShapeChooser(Application app) {
      return JShapeChooser.create(app);
  }

    public static JInterface createInterface(Application app) {
	return new JInterface(app);
    }

    public static ButtonDialog createButtonDialog(Frame f, String title, Application app) {
	    return JButtonDialog.create(f, title, app);
    }

    public static ButtonDialog createButtonDialog(Frame f, String title, ButtonWidget bw, Application app) {
	    return JButtonDialog.create(f, title, bw, app);
    }

    public static SliderDialog createSliderDialog(Frame f, Application app, String title) {
	    return JSliderDialog.create(f, title, app);
    }

    public static SliderDialog createSliderDialog(Frame f, Application app, String title, SliderWidget sw) {
	    return JSliderDialog.create(f, title, sw, app);
    }

    public static LineDialog createLineDialog(Frame f, Application app, String title) {
	    return JLineDialog.create(f, title, app);
    }

    public static MonitorDialog createMonitorDialog(Frame f, Application app, String title) {
	    return JMonitorDialog.create(f, app, title);
    }

    public static MonitorDialog createMonitorDialog(Frame f, Application app, String title, MonitorWidget mw) {
	    return JMonitorDialog.create(f, title, app, mw);
    }

    public static TextWidgetDialog createTextWidgetDialog(Frame f, Application app, String title) {
	    return JTextWidgetDialog.create(f, app, title);
    }

    public static TextWidgetDialog createTextWidgetDialog(Frame f, Application app, String title, TextWidget bw) {
	    return JTextWidgetDialog.create(f, app, title, bw);
    }

    public static TextDialog createTextDialog(Frame f, String title, String message) {
	    return new JTextDialog(f, title, message);
    }

    public static ChoiceDialog createChoiceDialog(Frame f, String title,
						  String message,
						  String yes, String cancel, String not) {
	    return new JChoiceDialog(f, title, message, yes, cancel, not);
    }


    public static AboutDialog createAboutDialog(Frame f, Application app) {
	    return new JAboutDialog(f, app);
    }

    public static FAQDialog createFAQDialog(Application app, Frame f) {
	return new FAQDialog(app, f);
    }

    public static MessageStream createMessageStream(Application app, StarLogoFrame f) {
	return new JMessageStream(app, f);
    }

    public static MessageDialog createMessageDialog(Application app, StarLogoFrame f, String title) {
	if (f != null) {
	    return new JMessageDialog(app, f, title);
	} else {
	    return new JMessageDialog(app, title);
	}
    }

    public static void createMessageDialog(Application app, StarLogoFrame f, String title, String message, boolean modalp) {
	JMessageDialog.displayMessage(app, f, title, message, modalp);
    }

    public static MessageDialog 
	createMessageDialog(Application app, StarLogoFrame parent, String title, String contents, 
			    boolean showOK, boolean showCancel, boolean modal, 
			    boolean shortCutsOn, int focusPref) {
	if (parent != null) {
	    return new JMessageDialog(app, parent, title, contents, showOK, 
				      showCancel, modal, shortCutsOn, 
				      focusPref);
	} else {
	    return new JMessageDialog(app, title, contents, showOK, 
				      showCancel, modal, shortCutsOn, 
				      focusPref);
	}	    
    }


    public static void interruptWithMessage(Application app, StarLogoFrame parent, String message) {
	createMessageDialog(app, parent, "StarLogo Message", message, false);
    }

    public static ColorLegendDialog createColorLegendDialog(Frame f, Application app, String title, ColorLegendWidget clw) {
	    return JColorLegendDialog.create(f, app, title, clw);
    }

    public static ColorLegendDialog createColorLegendDialog(Frame f, Application app, String title) {
	    return JColorLegendDialog.create(f, app, title);
    }

    public static PlotDialog createPlotDialog(Frame f, int type, PlotBox plot, LineWidget lw, Application app) {
	return new JPlotDialog(f, type, plot, lw, app);
    }

    public static PtolemyBarDialog createPtolemyBarDialog(Frame f, Vector sets, 
							  Application app, 
							  int type, int id) {       
	return new JPtolemyBarDialog(f, sets, app, type, id);
    }

    public void flushTextAreas() {
	if (!kioskp && YoYo.applet == null) {
	    procWin.flush();
	    JTurtleMonitor.flushCommandCenters();
	    JPatchMonitor.flushCommandCenters();
	}
    }

}

