/*
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

package starlogo;

public class AverageCounter {

    int[] list;
    int[] sortedlist;
    int listptr;
    int size;
    boolean full;
    int elements;

    AverageCounter(int size) {
	list = new int[size];
	this.size = size;
	listptr = 0;
	elements = 0;
	full = false;
	sortedlist = null;
    }

    public void add(int value) {
	if (value == 0) return;
	sortedlist = null;
	list[listptr] = value;
	listptr = (listptr + 1) % size;
	if (!full) {
	    elements++;
	    if (elements == size) {
		full = true;
	    }
	}
    }

    public void reset() {
	elements = 0;
	listptr = 0;
	full = false;
	sortedlist = null;
    }

    public boolean ready() {
	return full;
    }

    public double getMedian() {
	if (full) {
	    if (sortedlist == null) sortList();
	    if ((size % 2) == 0) {
		int index = size / 2 - 1;
		return ((double)(sortedlist[index] + sortedlist[index + 1])) 
		    / 2.0;
	    } else return sortedlist[size / 2];
	} else return -1;
    }

    public void sortList() {
	sortedlist = mergesort(list, 0, size - 1);
    }

    public static int[] mergesort(int[] dlist, int begin, int end) {
	if (begin == end)
	    return new int[] {dlist[begin]};
	int middle = (begin + end) / 2;
	int[] lista = mergesort(dlist, begin, middle);
	int[] listb = mergesort(dlist, middle+1, end);
	return merge(lista, listb);
    }
	
    private static int[] merge(int[] lista, int[] listb) {
	int[] listc = new int[lista.length+listb.length];
	int a=0,b=0;
	for (int i=0; i<listc.length; i++) {
	    if (a >= lista.length) {
		System.arraycopy(listb,b,listc,i,listb.length-b);
		break;
	    }
	    if (b >= listb.length) {
		System.arraycopy(lista,a,listc,i,lista.length-a);
		break;
	    }
	    if (lista[a] <= listb[b]) {
		listc[i] = lista[a++];
	    }	else {
		listc[i] = listb[b++];
	    }
	}
	return listc;
    }

    public double getAverage() {
	if (full) {
	    //System.out.println("elements in avg:");
	    int sum = 0;
	    for(int i = 0; i < size; i++) {
		sum += list[i];
		//System.out.println("list[" + i + "] = " + list[i]);
	    }
	    return ((double)sum) / ((double)size);
	} else return -1;
    }

}
