/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.util.Enumeration;
import java.util.Vector;
import java.awt.event.*;
import java.awt.*;
import java.awt.image.*;

public class BufferedPatchCanvas extends PatchCanvas {

    DataBuffer pixelbuffer;
    //TurtleShapes ts = new TurtleShapes();
    //int[][] shapes = ts.shapes;
    BufferedImage image;
	
    int rowbytes, nextlineoffset, nextpatchoffset;

    public BufferedPatchCanvas(StarLogo sl) {
	super(sl);
    }

    public BufferedPatchCanvas(StarLogo sl, PatchCanvasInterface oldpc) {
	super(sl, oldpc);
    }

    public boolean init(JInterface parent) {
	this.parent = parent;
	GraphicsConfiguration gc = ((Component)parent).getGraphicsConfiguration();
	image = gc.createCompatibleImage(position.width, position.height);

	ColorModel cm = image.getColorModel();
	if (cm instanceof PackedColorModel) {
	    Colors.setupColorArrayBufPC((PackedColorModel)cm);
	} else return false;

	//image = parent.createImage(this);
	xoffset = position.x;
	yoffset = position.y;
	rowbytes = position.width;
	pixelbuffer = image.getRaster().getDataBuffer();
	//System.out.println("pixelbuffer is " + pixelbuffer);
	//System.out.println("sample model is " + image.getSampleModel());
	nextlineoffset = rowbytes * (sl.cellsize - 1);
	nextpatchoffset = - ((rowbytes - 1) * sl.cellsize);
	updateImage();
	return true;
    }

    public void updateImage() {
	uptodate = false;
    }

    public void updateImageInternal() {
	if (sl.cellsize < 8) 
	    updateImageNoShapes();
	else updateImageShapes();
    }

    public void updateImageNoShapes() {
	if (pixelbuffer == null) return;
	int poffy = sl.widthp2 + 1; // first visible logical patch
	int xypix = 0; // first pixel
	for (int y = sl.height; y != 0; y--) {
	    for (int x = sl.width; x != 0; x--) {
		Patch patch = sl.patches[poffy];				
		int color = 0;
		if (patch.topturtle != null && patch.topturtle.shownp) {
		    //drawTurtle(patch.topturtle);
		    color = Colors.colorarrayintsBufPC[(int)(patch.topturtle.color * 32.0)];
		} else {
		    if (patch.pc != 200.0) {
			color = Colors.colorarrayintsBufPC[(int)(patch.pc * 32.0)];
		    }
		}
		for(int i = sl.cellsize; i != 0; i--) {
		    for(int j = sl.cellsize-1; j >= 0; j--) {
			pixelbuffer.setElem(xypix + j, color);
		    }
		    xypix += rowbytes;
		}
		pcdraws++;
		poffy++; // next logical patch
		xypix += nextpatchoffset; // next onscreen patch
	    }
	    xypix += nextlineoffset;
	    poffy += 2; // skip two patches
	}
	//image.getRaster().setDataElements(0, 0, position.width, position.height, pixelbuffer);
	//startProduction(ic);
	uptodate = true;
    }

    public void updateImageShapes() {
	if (pixelbuffer == null) return;
	int poffy = sl.widthp2 + 1; // first visible logical patch
	int xypix = 0; // first pixel
	int cellsize = sl.cellsize;
	for (int y = sl.height; y != 0; y--) {
	    for (int x = sl.width; x != 0; x--) {
		Patch patch = sl.patches[poffy];				
		if (patch.topturtle != null && patch.topturtle.shownp) {
		    //drawTurtle(patch.topturtle);
		    int tcolmask = Colors.colorarrayintsBufPC[(int)(patch.topturtle.color * 32.0)];
		    int pcolmask = (patch.pc == 200.0) ? 0 : Colors.colorarrayintsBufPC[(int)(patch.pc * 32.0)];
		    TurtleShapes shape = TurtleShapes.getShape(patch.topturtle.shape);
		    int heading = ((int) ((patch.topturtle.heading+11.25) / 22.5)) % 16;
		    double[] colors = (shape.getColorsForSize(cellsize))[heading];
		    int[]    masks  = (shape.getMasksForSize(cellsize))[heading];
		    for (int i=0, z=0; i<cellsize; i++) {
			for (int j=0; j<cellsize; j++) {
			    int z2 = 2*z;
			    int tmask = masks[z2];
			    int pmask = masks[z2+1];
			    int colindex = (int)(colors[z] * 32.0);
			    int color = Colors.colorarrayintsBufPC[colindex];

 			    try {
				pixelbuffer.setElem(xypix + j, 
						    (tmask & tcolmask) |
						    (pmask & pcolmask) |
						    color);
			    } catch (ArrayIndexOutOfBoundsException e) {
				System.out.println("Tried to draw turtle @ (" + x + ", "  + y + " but it was out of bounds.");
				e.printStackTrace();
			    }
			    
			    z++;
			}
			xypix += rowbytes;
		    }
		} else {
		    //drawPatch(patch);
		    int color = 0;
		    if (patch.pc != 200.0) {
			color = Colors.colorarrayintsBufPC[(int)(patch.pc * 32.0)];
		    }
		    for(int i = cellsize; i != 0; i--) {
			for(int j = cellsize-1; j >= 0; j--) {
			    try {
				pixelbuffer.setElem(xypix + j, color);
			    } catch (ArrayIndexOutOfBoundsException e) {
				System.out.println("Tried to set patch @ (" + x + ", "  + y + " to color: " + color + " but it was out of bounds.");
				e.printStackTrace();
			    }
			}
			xypix += rowbytes;
		    }
		}
		pcdraws++;
		poffy++; // next logical patch
		xypix += nextpatchoffset; // next onscreen patch
	    }
	    xypix += nextlineoffset;
	    poffy += 2; // skip two patches
	}
	//startProduction(ic);
	//image.getRaster().setDataElements(0, 0, position.width, position.height, pixelbuffer);
	uptodate = true;
    }

    public void drawPatch(double xcor, double ycor, double color) {
	drawPatch(xcor, ycor, color, sl.patches);
    } 

    public void drawPatch(double xcor, double ycor, double color, Patch[] patches) {
	if (color == 200.0) return;
	drawCell((int)xcor, (int)ycor, Colors.colorarrayintsBufPC[(int)(color * 32.0)]);
    }

    public void drawPatch(Patch p) {
	drawPatch(p, sl.patches); 
    }
    
    public void drawPatch(Patch p, Patch[] patches) {
	if (p.pc == 200.0) return;
	drawCell((int)p.xcor-1, (int)p.ycor-1, Colors.colorarrayintsBufPC[(int)(p.pc * 32.0)]);
    }

    public void drawTurtle(Turtle t) {
	drawTurtle(t, sl.patches); 
    }
    
    public void drawTurtle(Turtle t, Patch[] patches) {
	if (true) {
	    if (displayp) {
		uptodate = false;
		sl.patchesneedalittleredrawingp = true;
	    }
	    return;
	}

	if (!displayp) return;

	int cellsize = sl.cellsize;
			
	if (cellsize < 8) {
	    drawCell((int)t.xcor, (int)t.ycor, Colors.colorarrayintsBufPC[(int)(t.color * 32.0)]);
	    return;
	}
	pcdraws++;		
	int startx = ((int)(t.xcor)) * cellsize;
	int starty = ((int)(t.ycor)) * cellsize;
			
	int xypix = startx + (starty * rowbytes);
	int oldxypix = xypix;
	int tcol = Colors.colorarrayintsBufPC[(int)(t.color * 32.0)];
	
	Patch patch = patches[t.patch];
	int pcol = (patch.pc == 200.0) ? 0 : Colors.colorarrayintsBufPC[(int)((patch.pc) * 32.0)];
	int z=0;
	TurtleShapes ts = TurtleShapes.getShape(t.shape);
	int heading = (int) (((t.heading+11.25) / 22.5) % 16);
	double[] colors = (ts.getColorsForSize(cellsize))[heading];
	int[]    masks  = (ts.getMasksForSize(cellsize))[heading];
	int index = 0;
	for (int i=0; i<cellsize; i++) {
	    for (int j=0; j<cellsize; j++) {
		index = i*cellsize + j;

		int i2 = 2*index;
		int tmask = masks[i2];
		int pmask = masks[i2+1];
		int colindex = (int)(colors[index] * 32.0);
		int color = Colors.colorarrayintsBufPC[colindex];
		
		try {
		    pixelbuffer.setElem(xypix + j, 
					(tmask & tcol) |
					(pmask & pcol) |
					color);
		} catch (IndexOutOfBoundsException e) {
		    System.out.println("Exception trying to draw turtle at (" + t.xcor + 
				       ", " + t.ycor + ") = [" + startx + ", " + starty + "]");
		    e.printStackTrace();
		}
	    }
	    xypix += rowbytes;
	}
	sl.patchesneedalittleredrawingp = true;
	uptodate = true;
    }

    void drawCell(int x, int y, int color) {

	if (true) {
	    if (displayp) {
		uptodate = false;
		sl.patchesneedalittleredrawingp = true;
	    }
	    return;
	}

	if (!displayp) return;
	pcdraws++;
	int cellsize = sl.cellsize;
	int startx = x * cellsize;
	int starty = y * cellsize;
	int xypix = startx + (starty * rowbytes); // go to the right patch
	int oldxypix = xypix;
	//System.out.println("print pixels at " + x + " " + y + " xypix = " + xypix + " color = " + Integer.toString(color, 16));
	for(int i = 0; i < cellsize; i++) {
	    for(int j = 0; j < cellsize; j++) {
		try {
		    pixelbuffer.setElem(xypix + j, color);
		} catch (IndexOutOfBoundsException e) {
		    System.out.println("Exception trying to draw cell at (" + x + 
				       ", " + y + ") = [" + startx + ", " + starty + "]");
		    e.printStackTrace();
		}

	    }
	    xypix += rowbytes;
	}
	//printPixels(x, y);
	//if (ic != null) {
	//    ic.setPixels(startx, starty, cellsize, cellsize, Colors.cm, pixels, oldxypix, rowbytes);
	    //ic.imageComplete(ImageConsumer.SINGLEFRAMEDONE);
	//}
	//image.getRaster().setDataElements(startx, starty, cellsize, cellsize, pixelbuffer);
	sl.patchesneedalittleredrawingp = true;
	uptodate = true;
    }

    public void printPixels(int whichx, int whichy) {
	int poffy = sl.widthp2 + 1; // first visible logical patch
	int xypix = 0; // first pixel
	for (int y = sl.height; y != 0; y--) {
	    for (int x = sl.width; x != 0; x--) {
		Patch patch = sl.patches[poffy];
		int color = 
		    Colors.colorarrayintsBufPC[(int)(((patch.topturtle != null && patch.topturtle.shownp) ?
						 patch.topturtle.color : patch.pc) * 32.0)];
		if ((whichx == (sl.width - x)) &&
		    (whichy == (sl.height - y))) {
		    //System.out.println("x = " + (sl.width - x) + " y = " + (sl.height - y));
		    //System.out.println("xypix = " + xypix);
		    //System.out.println("color is " + Integer.toString(pixels[xypix], 16));
		}
		for(int i = sl.cellsize; i != 0; i--) {
		    for(int j = sl.cellsize-1; j >= 0; j--) {
			pixelbuffer.setElem(xypix + j, color);
		    }
		    xypix += rowbytes;
		}
		//draw patch
		poffy++; // next logical patch
		xypix += nextpatchoffset; // next onscreen patch
	    }
	    xypix += nextlineoffset;
	    poffy += 2; // skip two patches
	}
    }	


    public void setOffscreen(boolean os) {
	//synchronized(sl.runningp) {
	sl.offscreen = os;
	if (os) {
	    sl.locked = true;
	    //setGraphics(image.getGraphics());
	    updateImage();
	    xoffset = 0;
	    yoffset = 0;
	    //	System.out.println("offscreen is true");
	} else {
	    // keep the turtles from moving until the patchcanvas image is written to screen
	    //sl.locked = true;
	    //setGraphics(getDrawingSurface());
	    xoffset = position.x;
	    yoffset = position.y;
	    //System.out.println("offscreen is false");
	}
	//}
    }

    public void clearImage() {
	//System.out.println("clear image");
	
	int size = position.height * position.width;
	for(int i = 0; i<size; i++) {
	    pixelbuffer.setElem(i, Colors.colorarrayintsBufPC[0]); // black
	}
	//image.getRaster().setDataElements(0, 0, position.width, position.height, pixelbuffer);
	//startProduction(ic);
    }

    public void refresh() {
	//if (ic != null) {
	//    ic.imageComplete(ImageConsumer.SINGLEFRAMEDONE);
	//    startProduction(ic);
	//}
	if (displayp) uptodate = false;
	parent.repaint(position);
	
	/*
	  Graphics g = parent.getGraphics();
	  g.drawImage(image, position.x, position.y, parent);
	  g.dispose();
	*/
    }

    public void paint(Graphics g) {
    	if (parent == null) return;
	if (parent.resizingp() && parent.widgetTouched() == this) {
	    //System.out.println("resizing graphics screen");
	    g.drawImage(image, originalPosition.x, originalPosition.y, this);
	    g.setColor(Color.gray);
	    g.drawRect(position.x, position.y, position.width, position.height);
	} else {
	    boolean newoffscreen = getOffscreen();
	    //System.out.println(newoffscreen);
	    boolean changedtofalse = false;
	    if (newoffscreen!=sl.offscreen) {
		//System.out.println("different");
		setOffscreen(newoffscreen);
		changedtofalse = !newoffscreen;
	    }
	    //synchronized(sl.runningp) {
	    if (!uptodate && displayp) updateImageInternal();
	    //long time = System.currentTimeMillis();
	    //try {Thread.currentThread().sleep(50); } catch(Exception e) {}
	    //System.out.println("draw image: " + image);
	    if (image != null) g.drawImage(image, position.x, position.y, this);
	    //super.paint(g);
	    if (changedtofalse && sl.locked) {
		// unlocking turtles
		//synchronized(sl.runningp) {
		sl.locked = false;
		sl.runningp.notify();
		//}
	    }
	}
    }
  
    public void setDrawingSurface() { 
	/*
	  try
	  {
	  throw new Exception();
	  } catch (Exception e) {
	  e.printStackTrace();
	  }
	*/
	return;
    }
  
}

