/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.awt.event.*;
import java.io.IOException;
import java.awt.*;
import yoyo.*;
import java.util.Vector;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Observable;
import java.net.MalformedURLException;
import java.net.URL;
import java.io.PrintWriter;

public class ButtonWidget extends WidgetDrag
    implements StarLogoCaller, java.util.Observer, DisplayMessage {

    public static final String prePrefix = "pre-compile-button ";

    public static final String turCPrefix = "compile-turtle-button ";
    public static final String obsCPrefix = "compile-observer-button ";

    public static final String turPrefix = "turtle-button-r ";
    public static final String obsPrefix = "observer-button-r ";

    public static final int FIRST_BUTTON_ID = 1;
    public static int idCount = FIRST_BUTTON_ID;

    static final int DOWN = 0;
    static final int UP = 1;

    public String name, command;
    public boolean forever, showName, turtlep;
    public Font font;
    public Object[] listtorun, listtocompile;
    public int id;
    public String keyboard;

    int whichip = -1;

    static Image[] butCorners = new Image[2];
    static Image[] butN = new Image[2];
    static Image[] butE = new Image[2];
    static Image[] butS = new Image[2];
    static Image[] butW = new Image[2];
    static Image looper;
    
    boolean moved = false;
    boolean buttonDown = false;
    StarLogo sl;
    Application app;

    int textWidth, textHeight, textDescent;

    FontMetrics fm = null;

    static Font BUTTON_FONT;

    static {
	try {
	    BUTTON_FONT = WIDGET_FONT;
	    if (YoYo.macintoshp) BUTTON_FONT = Font.getFont("starlogo.button.font", BUTTON_FONT);
	} catch (Exception e) { e.printStackTrace(); }
    }

    public ButtonWidget(String name, String listtorun, Rectangle position,
			boolean turorobsp, boolean foreverp, boolean shownamep,
			int bid, JInterface iface, StarLogo sl, String descript, String keyboard) {
	this.parent = iface;
	this.sl = sl;
	this.app = sl.app;
	this.name = name;
	this.command = listtorun;
	this.forever = foreverp;
	this.showName = shownamep;
	this.turtlep = turorobsp;
	this.position = new Rectangle(position);
	this.id = bid;
	this.font = BUTTON_FONT;
	this.keyboard = keyboard;
	fm = parent.getToolkit().getFontMetrics(font);
	textDescent = fm.getMaxDescent();
	textHeight =  fm.getMaxAscent()+textDescent;
	updateMinSize();

	//addObserver(parent);
	//app.procWin.addObserver(this);
	//sl.putCaller(id, this);
	if (!(ToolTipWidget.empty(descript))) setToolTip(new ToolTipWidget(parent, descript));
    }

    public ButtonWidget(String name, String listtorun, Rectangle position,
			boolean turorobsp, boolean foreverp, boolean shownamep,
			int bid, JInterface iface, int whichip, StarLogo sl, String descript, String keyboard) {
	this (name, listtorun, position, turorobsp, foreverp, shownamep, bid, iface, sl, descript, keyboard);
	this.whichip = whichip;
    }

    public ButtonWidget(JInterface parent, StarLogo sl) {
	this(parent, sl, "", "", false, false, true, "", "");
    }

    public ButtonWidget(JInterface iface, StarLogo sl, String command,
			String name, boolean forever,
			boolean showName, boolean turtlep, String descript, String keyboard) {

	// trim iface.outline to cover button but also fit on screen
	if (iface.outline().x + MIN_WIDTH > iface.getSize().width) {
	    iface.outline().width = iface.outline().width + iface.outline().x -
		iface.getSize().width + MIN_WIDTH;
	    iface.outline().x=iface.getSize().width-MIN_WIDTH;
	}
	if (iface.outline().width<MIN_WIDTH)
	    iface.outline().width = MIN_WIDTH;

	if (iface.outline().y+MIN_HEIGHT>iface.getSize().height) {
	    iface.outline().height=iface.outline().height+iface.outline().y-
		iface.getSize().height+MIN_HEIGHT;
	    iface.outline().y=iface.getSize().height-MIN_HEIGHT;
	}
	if (iface.outline().height<MIN_HEIGHT)
	    iface.outline().height = MIN_HEIGHT;

	this.parent = iface;
	this.sl = sl;
	this.app = sl.app;
	this.name = name;
	this.command = command;
	this.forever = forever;
	this.showName = showName;
	this.turtlep = turtlep;
	this.position = new Rectangle(iface.outline());
	this.keyboard = keyboard;

	this.id = iface.getHighestButtonID() + 1; 
	idCount++;

	this.font = WIDGET_FONT;
	fm = parent.getToolkit().getFontMetrics(font);
	textDescent = fm.getMaxDescent();
	textHeight =  fm.getMaxAscent()+textDescent;
	updateMinSize();

	//addObserver(parent);
	//app.procWin.addObserver(this);
	//sl.putCaller(id, this);
	if (!(ToolTipWidget.empty(descript))) setToolTip(new ToolTipWidget(parent, descript));
    }

    static public void createNew(JInterface iface) {
	ButtonDialog bd =
	    iface.app().createButtonDialog((Frame)iface.frame(), iface.app().textBundle.getString("new_button_label"), iface.app());
	bd.show();

	if (bd.pressedOK()) {
	    ButtonWidget bw = new ButtonWidget(iface, iface.sl());
	    bd.getValues(bw);
	    bw.updateMinSize();
	    iface.addWidget(bw);
	    iface.app().rancompilerp = false;
	    iface.app().rancompilerpForVariables = false;
	    //iface.app().project.compile(iface.app());
	}
	bd.dispose();
    }

    public void edit() {
	ButtonDialog bd = app.createButtonDialog((Frame)parent.frame(), app.textBundle.getString("edit_button_label"), this, app);
	bd.show();
	if (bd.pressedOK()) {
	    parent.addHandlesToClipRect(this);
	    bd.getValues(this);
	    int textWidthOld = textWidth;
	    textWidth = fm.stringWidth(getText());
	    if (forever) textWidth+=18;
	    if (textWidth != textWidthOld) updateMinSize();
	    update(position);
	    app.rancompilerp = false;
	    app.rancompilerpForVariables = false;
	    app.project.markChanged();
	    //app.project.compile(app);
	}
	bd.dispose();
	super.edit();
    }

    /**
     * Used to load buttons from a file for current star logo
     */
    public static ButtonWidget load(ArrayList items, JInterface iface, StarLogo sl,
				    boolean htmlp) {

	Application app = sl.app;
	boolean turorobsp = true, foreverp = false, shownamep = false;
	int top = 0, left = 0, width = 50, height = 20, bid = 1;
	String name = app.textBundle.getString("button") + "1", 
	    listtorun = app.textBundle.getString("nothing"), descript = "",
		keyboard = "";
	int whichip = -1;

	int i = 0;
	while (i<items.size()) {
	    String item = items.get(i).toString();
	    if (item.equals("turtle-or-observer?")) {
		String value = items.get(i+1).toString();
		turorobsp = (value.equals("turtle"));
		i += 2;
		continue;
	    }
	    if (item.equals("top-left")) {
		String value = items.get(i+1).toString();
		String value2 = items.get(i+2).toString();
		top = Integer.parseInt(value);
		left = Integer.parseInt(value2);
		i += 3;
		continue;
	    }
	    if (item.equals("width-height")) {
		String value = items.get(i+1).toString();
		String value2 = items.get(i+2).toString();
		width = Integer.parseInt(value);
		height = Integer.parseInt(value2);
		i += 3;
		continue;
	    }
	    if (item.equals("name")) {
		name = items.get(i+1).toString();
		i += 2;
		continue;
	    }
	    if (item.equals("line-to-run")) {
		listtorun = items.get(i+1).toString();
		i += 2;
		continue;
	    }
	    if (item.equals("forever?")) {
		String value = items.get(i+1).toString();
		foreverp = value.equals("true");
		i += 2;
		continue;
	    }
	    if (item.equals("show-name?")) {
		String value = items.get(i+1).toString();
		shownamep = value.equals("true");
		i += 2;
		continue;
	    }
	    if (item.equals("button-number")) {
		String value = items.get(i+1).toString();
		bid = Integer.parseInt(value);
		i += 2;
		continue;
	    }
		if (item.equals("keyboard-key")) {
		keyboard = items.get(i+1).toString();
		i += 2;
		//System.out.println("keyboard found" + keyboard);
		continue;
	    }
	    if (item.equals("toolTip-text")) {
		String value = items.get(i+1).toString();
		descript = value;
		i += 2;
		continue;
	    }
	    if (htmlp) {
		if (item.equals("whichip")) {
		    String value = items.get(i+1).toString();
		    whichip = Integer.parseInt(value);
		    i += 2;
		    continue;
		}
	    }
	    // skip the next if we haven't found anything
	    i+=2;
	}

	Rectangle bounds = iface.getBoundaries();
	Rectangle position = new Rectangle(left + bounds.x, top + bounds.y, width, height);
	if (bid >= idCount) idCount = bid + 1;

	if (htmlp) {
	    return new ButtonWidget(name, listtorun, position, turorobsp, foreverp, shownamep,
				    bid, iface, whichip, sl, descript, keyboard);
	} else {
	    return new ButtonWidget(name, listtorun, position, turorobsp, foreverp, shownamep,
				    bid, iface, sl, descript, keyboard);
	}
    }

    public String getSaveString(boolean htmlp) throws IOException {
	Rectangle bounds = parent.getBoundaries();
	return "SLButton\r\n" +
	    "turtle-or-observer? " + ((turtlep) ? "turtle\r\n" : "observer\r\n") +
	    "top-left " + (position.y - bounds.y) + " " + (position.x- bounds.x) + "\r\n" +
	    "width-height " + position.width + " " + position.height + "\r\n" +
	    "name \"" + name + "\"\r\n" +
	    "line-to-run \"" + Utilities.printable(command) + "\"\r\n" +
	    "forever? " + ((forever) ? "true\r\n" : "false\r\n") +
	    "button-number " + id + "\r\n" +
	    "show-name? " + ((showName) ? "true\r\n" : "false\r\n") +
	    ((htmlp && whichip >= 0) ? "whichip " + whichip + "\r\n" : "") +
	    getToolTipSaveString() +
		"keyboard-key \"" + Utilities.printable(keyboard) + "\"\r\n" + "\r\n" ;
    }

    /**
     * Used for loading files from Mac star logo
     */

    //used for loading from file
    public ButtonWidget(ArrayList args, int i, JInterface parent, StarLogo sl) {
	//System.out.println("new old button widget from args: " + YoYo.printToString(args));
	position = new Rectangle();
	position.x = new Integer(args.get(i++).toString()).intValue()+
	    parent.getBoundaries().x;
	position.y = new Integer(args.get(i++).toString()).intValue()+
	    parent.getBoundaries().y;
	position.width = new Integer(args.get(i++).toString()).intValue()+
	    parent.getBoundaries().x-position.x;
	position.height = new Integer(args.get(i++).toString()).intValue()+
	    parent.getBoundaries().y-position.y;
	name = args.get(i++).toString();
	command = args.get(i++).toString();
	forever = ((args.get(i++).toString().toUpperCase().trim().equals("T"))
		   ? true : false);
	font = WIDGET_FONT;
	id = new Integer(args.get(i++).toString()).intValue();
	if (id >= idCount)
	    idCount = id + 1;
	//System.out.println("id is " + id);
	showName = ((args.get(i++).toString().toUpperCase().trim().equals("T"))
		    ? true : false);
	//if last line of file in old starlogo format,
	//don't read past EOF looking for turtle/observer switch
	if (i<=args.size()-1) {
	    if (args.get(i++).toString().equals("observer")) {
		turtlep = false;
	    }
	    else turtlep = true;
	}
	else turtlep = true;

	this.parent = parent;
	this.sl = sl;
	this.app = sl.app;

	fm = parent.getToolkit().getFontMetrics(font);
	textDescent = fm.getMaxDescent();
	textHeight =  fm.getMaxAscent()+textDescent;
	updateMinSize();

	//addObserver(parent);
	//app.procWin.addObserver(this);
	//sl.putCaller(id, this);
    }
	
	void keyClicked() {
    if (parent.getToolbar().buttonDown == ToolbarWidget.POINTER ||
	    parent.getToolbar().paintActivep()) {
	    if (buttonDown) {
		if (whichip >= 0) {
		    synchronized (sl.runningp) {
			toggleButton();
		    }
		}
	    } else {
		Runnable r = new Runnable() {
			public void run() {
			    //			    System.out.println("running after compiling button: " + whichip + " " + this);
			    if (whichip >= 0) {
				synchronized (sl.runningp) {
				    toggleButton();
				}
			    } else {
				if (!app.kioskp && YoYo.applet == null) {
				    app.createMessageDialog(app, (StarLogoFrame)parent.frame(), 
							    app.textBundle.getString("warning_label"),
							    app.textBundle.getString("project_did_not_compile") + " " +
							    app.textBundle.getString("button_will_work_later"), 
							    false);
				}
			    }
			}
		    };
		app.maybeCompileAndThenRunElseRunIfFailedAlotRun(false, r, 
								 new Runnable() { public void run() {} }, r);
	    }
	}
    }


    void mouseClicked(MouseEvent e) {
	if (parent.getToolbar().buttonDown == ToolbarWidget.POINTER ||
	    parent.getToolbar().paintActivep()) {
	    if (buttonDown) {
		if (whichip >= 0) {
		    synchronized (sl.runningp) {
			toggleButton();
		    }
		}
	    } else {
		Runnable r = new Runnable() {
			public void run() {
			    //			    System.out.println("running after compiling button: " + whichip + " " + this);
			    if (whichip >= 0) {
				synchronized (sl.runningp) {
				    toggleButton();
				}
			    } else {
				if (!app.kioskp && YoYo.applet == null) {
				    app.createMessageDialog(app, (StarLogoFrame)parent.frame(), 
							    app.textBundle.getString("warning_label"),
							    app.textBundle.getString("project_did_not_compile") + " " +
							    app.textBundle.getString("button_will_work_later"), 
							    false);
				}
			    }
			}
		    };
		app.maybeCompileAndThenRunElseRunIfFailedAlotRun(false, r, 
								 new Runnable() { public void run() {} }, r);
	    }
	}
    }

    //long timePushed = 0;

    public void turnButtonOn() {
	if (!buttonDown) {
	    buttonDown = true;
	    //timePushed = System.currentTimeMillis();
	    if (turtlep) spp = StackProcessParent.runTurtles(sl, whichip, forever, this);
	    else spp = StackProcessParent.runObserver(sl, whichip, forever, this);
	    parent.repaint(position);
	}
    }

    public void turnButtonOff() {
	if (buttonDown) popButton();
    }


    public void toggleButton() {
	buttonDown = (!buttonDown);
	if (buttonDown) {
	    if (whichip >= 0) {
		if (sl.locked == true) {
		    displayOneString(app.textBundle.getString("cant_run_button") + " " + 
				     name + " " + 
				     app.textBundle.getString("cant_run_while_obscured"));
		    buttonDown = (!buttonDown);
	    	} else {
		    //timePushed = System.currentTimeMillis();
		    if (turtlep)
			spp = StackProcessParent.runTurtles(sl, whichip, forever, this);
		    else
			spp = StackProcessParent.runObserver(sl, whichip, forever, this);
		    parent.repaint(position);
		}
	    } else
		buttonDown = !buttonDown;
    	}
	else
	    popButton();
    }

    public void popButton() {
	if (!donep()) {
	    stopAll();
	}
    }

    public String getPreCommandToCompile() {
	return prePrefix + id + "\"" + name + "\"";
    }

    public String getCommandToCompile() {
	String foreverPrefix = app.commandsBundle.getString("loop") + " [";
	if (turtlep) {
	    if (forever) return turCPrefix + id + " \"" + name + "\" [" +
		foreverPrefix + command + "] ]";
	    else return turCPrefix + id + " \"" + name + "\" [" + command + "]";
	}
	else {
	    if (forever) return obsCPrefix + id + " \"" + name + "\" [" +
		foreverPrefix + command + "] ]";
	    else return obsCPrefix + id + " \"" + name + "\" [" + command + "]";
	}
    }

    public void preCompileCommand() {
	//System.out.println("pre compile command for button");
	whichip = -1;
	if (command != "" && command != null) {
	    synchronized(sl.runningp) {
		sl.stopAllStarLogo();
	    }
	    listtocompile = Reader.read(getPreCommandToCompile());
	    Object[] newlist = new Object[listtocompile.length + 1];
	    System.arraycopy(listtocompile, 0, newlist, 0, listtocompile.length);
	    newlist[newlist.length -1] = this;
	    app.cc.enqueue(newlist, app.message, app.err, null);
	}
    }

    public void compileCommand() {
	//System.out.println("compile command");
	whichip = -1;
	if (command != "" && command != null) {
	    synchronized(sl.runningp) {
		sl.stopAllStarLogo();
	    }
	    listtocompile = Reader.read(getCommandToCompile());
	    Object[] newlist = new Object[listtocompile.length + 1];
	    System.arraycopy(listtocompile, 0, newlist, 0, listtocompile.length);
	    newlist[newlist.length -1] = this;
	    if (StarLogo.DEBUG) {
		app.log.println("Trying to compile button: " + this);
	    }
	    app.cc.enqueue(newlist, app.message, app.err, null);
	}
    }

    public void setCompiledCommand(Object[] ilist, Object[] olist) {
	//System.out.println("button set compiled command");
	if (StarLogo.DEBUG) {
	    app.log.println("Finished compiling button " + name +
			    "  " + YoYo.printToString(ilist) + " " +
			    YoYo.printToString(olist));
	}
	int[] newarray = new int[ilist.length];
	for(int i = 0; i<ilist.length; i++) {
	    newarray[i] = ((Number)ilist[i]).intValue();
	}
	whichip = sl.internalSetInstructions(toString(), newarray, olist);
	if (StarLogo.DEBUG) {
	    app.log.println("Button's whichip = " + whichip);
	}
    }

    public StackProcessParent spp;
    private Vector spps = new Vector();
    private int sppcount = 0;

    public void done(StackProcessParent spp) {
	spps.removeElement(spp);
	sppcount--;
	//System.out.println("empty? " + spps.isEmpty() + " sppcount " + sppcount);
	if (sppcount == 0) {
	    //System.out.println("button is done. spp = null");
	    this.spp = null;
	    /*
	      while ((System.currentTimeMillis() - timePushed) < 20) { 
	      Thread.currentThread().yield(); 
	      }
	    */
	    buttonDown = false;
	    parent.repaint(position);
	}
    }

    public boolean donep() {
	return (sppcount == 0);
    }

    public StarLogoCaller caller() {
	return this;
    }

    //int count = 1;

    public void addStackProcessParent(StackProcessParent s) {
	//System.out.println("adding spp " + count++);
	spps.addElement(s);
	sppcount++;
    }

    public void stopAll() {
	int num = 0;
	for(Enumeration e = ((Vector)spps.clone()).elements(); e.hasMoreElements();) {
	    StackProcessParent spp = (StackProcessParent)e.nextElement();
	    //System.out.println("BW: stopping stack process parent " + num + " spp:" + spp);
	    spp.stopAll(); //sl.stopProcess(spp);
	    num++;
	}
	//spps.removeAllElements();
	//assert(spp.numdone == spp.count);
	//assert(spp.numdone == num);
	//System.out.println("stopped " + num + " spps");
	spp = null;
	buttonDown = false;
	parent.repaint(position);
    }

    public void callerPrintln(String s) {
	if (!app.kioskp && YoYo.applet == null) {
	    if (turtlep) {
		app.procWin.turtlePrintln(s);
	    } else app.procWin.observerPrintln(s);
	} else if (YoYo.applet != null) YoYo.applet.showStatus(s);
    }

    public void callerClearText() {
	if (!app.kioskp && YoYo.applet == null) {
	    if (turtlep) {
		app.procWin.turtleClearCC();
	    } else app.procWin.observerClearCC();
	} else if (YoYo.applet != null) YoYo.applet.showStatus("");
    }

    public void done(Context c) {
	//System.out.println("done starting yoyo button");
    }

    public void displayOneString(String s) {
	app.createMessageDialog(app, app.activeFrame, app.textBundle.getString("error_in") + " " + name, 
				s, true);
    }

    public void displayAString(String s) {
	app.createMessageDialog(app, app.activeFrame, app.textBundle.getString("error_in") + " " + 
				name, s, true);
    }

    public void update() {
	//compileCommand();
	setChanged();
	notifyObservers();
    }

    public void update(Object arg) {
	//compileCommand();
	setChanged();
	notifyObservers(arg);
    }

    public void update(Observable obs, Object arg) {
	if (arg instanceof SLProc)
	    update();
    }


    public String toString() {
	return("Button:  " + getText());
    }

    public String getText() {
	return(showName ? name : command);
    }

    public void delete() {
	//app.procWin.deleteObserver(this);
	parent.buttons().removeElement(this);
	super.delete();
    }

    public int getToolbarButton() {
	// what button on the toolbar corresponds to this widgetdrag-- overrridden
	return ToolbarWidget.BUTTON;
    }

    public void updateMinSize() {
	textWidth = fm.stringWidth(getText());
	if (forever) textWidth+=18;
	int minwidth = 5 + 5 + textWidth;
	if (minwidth < 40) minwidth = 40;
	if (position.width < minwidth) position.width = minwidth;

	if (parent.outline() != null) {
	    // trim iface.outline to cover button but also fit on screen
	    parent.outline().width = position.width;
	    if (parent.outline().x + parent.outline().width > parent.getSize().width) {
		parent.outline().width = parent.getSize().width - parent.outline().x;
		//parent.outline().x=parent.getSize().width-minwidth;
	    }
	    if (parent.outline().width<minwidth)
		parent.outline().width = minwidth;
	}
	/* Now for the real minwidth */
	MIN_WIDTH = 40;

    }

    public void paint(Graphics g) {
	int but = (buttonDown) ? DOWN : UP;

	g.translate(position.x,position.y);
	Shape oldclip = g.getClip();

	/* Topleft */
	g.clipRect(0, 0, 5, 5);
	g.drawImage(butCorners[but], 0, 0, null);
	/* Topright */
	g.setClip(oldclip);
	g.clipRect(position.width - 5, 0, 5, 5);
	g.drawImage(butCorners[but], position.width - 10, 0, null);
	/* Botleft */
	g.setClip(oldclip);
	g.clipRect(0, position.height - 5, 5, 5);
	g.drawImage(butCorners[but], 0, position.height - 10, null);
	/* botright */
	g.setClip(oldclip);
	g.clipRect(position.width - 5, position.height - 5, 5, 5);
	g.drawImage(butCorners[but], position.width - 10, position.height - 10, null);

	g.setClip(oldclip);
	g.drawImage(butN[but], 5, 0, position.width - 10, 5, null);
	g.drawImage(butE[but], position.width - 5, 5, 5, position.height - 10, null);
	g.drawImage(butS[but], 5, position.height - 5, position.width - 10, 5, null);
	g.drawImage(butW[but], 0, 5, 5, position.height - 10, null);

	if (but==UP)
	    g.setColor(new Color(0, 153, 255));
	else
	    g.setColor(new Color(24, 99, 189));
	g.clipRect(4, 4, position.width - 8, position.height - 8);
	g.fillRect(5, 5, position.width - 9, position.height - 9);

	paintText(g, but, oldclip);
	g.setClip(oldclip);
	g.translate(-position.x, -position.y);
    }

    protected void paintText(Graphics g, int but, Shape oldclip) {
	if (but==UP)
	    g.setColor(Color.black);
	else
	    g.setColor(Color.white);
	g.setFont(font);

	g.setClip(oldclip);
	g.clipRect(4, 4, position.width-9, position.height-9);
	
	if (forever) {
	    g.drawString(getText(), (position.width - textWidth) / 2,
			 (position.height / 2) + (textHeight / 2) - textDescent);
	    int centerwidth = (position.width + textWidth) / 2 - 16;
	    int centerheight = position.height / 2 - 8;
	    /* Make sure you don't draw off the button */
	    g.setClip(oldclip);
	    g.clipRect(centerwidth, centerheight, 16, 16);
	    if (but == DOWN) g.drawImage(looper, centerwidth, centerheight, null);
	    else g.drawImage(looper, centerwidth - 16, centerheight, null);
	} else {
	    g.drawString(getText(), (position.width - textWidth) / 2,
			 (position.height / 2) + (textHeight / 2) - textDescent);
	}
    }



    public static void loadImages(String gifdir, MediaTracker tracker, Toolkit kit)
	throws MalformedURLException {
	// requires: trackerID in Widget indexes start of empty ID #s in <tracker>
	butCorners[DOWN] = kit.createImage(ButtonWidget.class.getResource(gifdir + "button-corners-down.gif"));
	butN[DOWN] = kit.createImage(ButtonWidget.class.getResource(gifdir + "but-top-down.gif"));
	butE[DOWN] = kit.createImage(ButtonWidget.class.getResource(gifdir + "but-right-down.gif"));
	butS[DOWN] = kit.createImage(ButtonWidget.class.getResource(gifdir + "but-bot-down.gif"));
	butW[DOWN] = kit.createImage(ButtonWidget.class.getResource(gifdir + "but-left-down.gif"));
	looper = kit.createImage(ButtonWidget.class.getResource(gifdir + "looper.gif"));
	butCorners[UP] = kit.createImage(ButtonWidget.class.getResource(gifdir + "button-corners-up.gif"));
	butN[UP] = kit.createImage(ButtonWidget.class.getResource(gifdir + "but-top-up.gif"));
	butE[UP] = kit.createImage(ButtonWidget.class.getResource(gifdir + "but-right-up.gif"));
	butS[UP] = kit.createImage(ButtonWidget.class.getResource(gifdir + "but-bot-up.gif"));
	butW[UP] = kit.createImage(ButtonWidget.class.getResource(gifdir + "but-left-up.gif"));

	tracker.addImage(butN[DOWN], trackerID++);
	tracker.addImage(butCorners[DOWN], trackerID++);
	tracker.addImage(butE[DOWN], trackerID++);
	tracker.addImage(butS[DOWN], trackerID++);
	tracker.addImage(butW[DOWN], trackerID++);
	tracker.addImage(looper, trackerID++);
	tracker.addImage(butN[UP], trackerID++);
	tracker.addImage(butCorners[UP], trackerID++);
	tracker.addImage(butE[UP], trackerID++);
	tracker.addImage(butS[UP], trackerID++);
	tracker.addImage(butW[UP], trackerID++);
    }

    public WidgetDrag copy(int edition, Rectangle r) {
	String s = name + " " + app.textBundle.getString("copy") + WidgetDrag.editionString(edition);
	return new ButtonWidget(s, command, r, turtlep, forever, showName, ++ButtonWidget.idCount, parent, sl, getToolText(),keyboard);
    }
	
	public static boolean characterIsLegit(char ch)
  {
      if ((Character.getType(ch) == Character.LOWERCASE_LETTER) ||
	 (Character.getType(ch) == Character.DECIMAL_DIGIT_NUMBER) ||
	(Character.getType(ch) == Character.CURRENCY_SYMBOL) ||
	(Character.getType(ch) == Character.CONNECTOR_PUNCTUATION) ||
	(Character.getType(ch) == Character.DASH_PUNCTUATION) ||
	(Character.getType(ch) == Character.MATH_SYMBOL) ||
	(Character.getType(ch) == Character.OTHER_PUNCTUATION) ||
	(Character.getType(ch) == Character.UPPERCASE_LETTER) ||
	(Character.getType(ch) == Character.START_PUNCTUATION) ||
	(Character.getType(ch) == Character.SPACE_SEPARATOR) ||
     (Character.getType(ch) == Character.END_PUNCTUATION))	
	 return true;
	 else
	 return false;

  }

}
