/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.util.Enumeration;
import java.util.Vector;
import yoyo.*;

public class CellSizeSliderWidget extends SliderWidget {

    public CellSizeSliderWidget(JInterface iface, Rectangle position) {
	super(iface.app().textBundle.getString("patch_size") , 
	      iface.app().commandsBundle.getString("%patch_size"), position,
	      1, 32, iface.sl().cellsize, true, idCount++, iface, iface.sl(),
	      iface.app().textBundle.getString("patch_size_tooltip"));
	updateMinSize();
    }

    public boolean draggable(MouseEvent e) {
    	return false;
    }

    public void updateValueAtEnd() {
	//System.out.println("Top of updateValueAtEnd in CellSizeSliderWidget");
	super.updateValueAtEnd();
	if (cur > 8 && cur <= 12) {
	    setValue(8);
	} else if (cur > 12 && cur <=24 ) {
	    setValue(16);
	} else if (cur > 24) {
	    setValue(32);
	} else setValue(cur);
    }

    public void setValueNoResize(int newval) {
	super.setValue(newval);
    }

    public void setValue(int newval) {
	//System.out.println("top of setValue. newval = " + newval + " and cur = " + cur);
	super.setValue(newval);
	//sl.resizePatches(sl.halfwidth, sl.halfheight, cur);
	sl.resizePatches(sl.halfwidth, sl.halfheight, newval);
    }

    public void jumpThumb(boolean increasing) {
	if (increasing) {
	    if (cur == 8) cur = 16;
	    else if (cur == 16) cur = 32;
	    else if (cur == 32) cur = 32;
	    else cur++;
	} else {
	    if (cur == 32) cur = 16;
	    else if (cur == 16) cur = 8;
	    else cur--;
	}
	//System.out.println("value of slider is about to be set to " +
	//		   cur + " in jumpThumb");
	setValue(cur);
    }

    public void paintValue(Graphics g) {
	int mycur;
	if (cur > 8 && cur <= 12) {
	    mycur = 8;
	} else if (cur > 12 && cur <=24 ) {
	    mycur = 16;
	} else if (cur > 24) {
	    mycur = 32;
	} else mycur = cur;
	String curString = String.valueOf(mycur);
	paintValue(g, curString);
    }

    public int resizeWhichCorner(Point anchor) {
	return 0;
    }

    public void edit() {}

    public String getSaveString(boolean htmlp) {
	return "";
    }

    public void mouseDragged(MouseEvent e) {
	//System.out.println("top of mouseDragged in CellSizeSliderWidget");
	updateValue();
	if (sliding) {
	    thumbPos.x=e.getX()-handle-position.x;
	    updateValueAtEnd();
	}
    }
}



