/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.awt.*;
import java.awt.event.*;
import java.io.IOException;
import java.net.*;
import java.util.Vector;
import java.util.Observable;
import java.util.Random;
import java.util.Enumeration;
import java.util.ArrayList;
import yoyo.*;
import java.lang.*;
//for rounding
import java.lang.Math;
import java.awt.image.ImageProducer;
import java.awt.Image;
import java.awt.image.ImageConsumer;
import java.awt.Graphics;

public class ColorLegendWidget extends WidgetDrag
    implements java.util.Observer, DisplayMessage, ImageProducer {

    public int cindex = 0;
    public boolean isVis = false;
    public boolean selected = false;

    //shape drawing stuff
    int pcolmask = 0x00000000;
    int tcolmask = 0xFFFFFFFF;
    Image image;
    ImageConsumer ic;   
    int[] pixels;
    int width = 16;
    int height = 16; 
    int rowbytes = width;
    //shape end
    public String name;
    public int color;
    public boolean showBorder;
    public int shape = 0;
    public Font font;
   
    Object value = VM.zeroInt;

    public int whichip = -1;
    public int onewhichip = -1;
    public StackProcess sp;

    public FontMetrics fm;
    public int textHeight, textDescent;

    StarLogo sl;
    Application app;
    
    public int id;
    public static int idCount = 1;
    public static int LEGEND_WIDTH = 60;
    public static int LEGEND_EDGE_WIDTH = 14;
    public static int LEGEND_HEIGHT = 16;

    static Font MONITOR_FONT;
  
    static {
    	try {
			MONITOR_FONT = WIDGET_FONT;
			if (YoYo.macintoshp) MONITOR_FONT = Font.getFont("starlogo.monitor.font", MONITOR_FONT);
    	} catch (Exception e) { e.printStackTrace(); }
    }
    	
  
    public ColorLegendWidget(String name, int color, Rectangle position,
			     boolean showborderp, int shape,
			     int mid, JInterface iface, StarLogo sl, String descript) {
	this.name = name;
	this.color = color;
	this.showBorder = showborderp;
	this.shape = shape;
	this.position = new Rectangle(position);
	this.position.height = LEGEND_HEIGHT;
	this.shape = shape;
	this.id = mid;
	this.parent = iface;
	this.sl = sl;
	this.app = sl.app;
	this.font = MONITOR_FONT;

	fm = iface.getToolkit().getFontMetrics(font);
	textDescent = fm.getMaxDescent();
	textHeight =  fm.getMaxAscent()+textDescent;

	//addObserver(parent);
	//app.procWin.addObserver(this);

	if (!(ToolTipWidget.empty(descript))) setToolTip(new ToolTipWidget(parent, descript));
	image = parent.createImage(this);
    }
    // public ColorLegendWidget(String name, String listtorun, Rectangle position,
    // 		       int shape, double delay, boolean showborderp, boolean isVis,
    // 		       int mid, Interface iface, int whichip, int onewhichip,
    // 		       StarLogo sl, String descript) {
    //     this(name, listtorun, position, shape, delay, showborderp, isVis, mid, iface, sl, descript);
    //     this.whichip = whichip;
    //     this.onewhichip = onewhichip;
    //   }

    public ColorLegendWidget(String name, int color, Rectangle position, boolean showborderp, int shape,
			     int mid, JInterface iface, int whichip, int onewhichip,
			     StarLogo sl, String descript) {
	this(name, color, position, showborderp, shape, mid, iface, sl, descript);
	this.whichip = whichip;
	this.onewhichip = onewhichip;
	image = parent.createImage(this);
    }

    public ColorLegendWidget(JInterface iface, StarLogo sl) {
	Dimension size = iface.getSize();
	Rectangle outline = iface.outline();
	if (outline.width<LEGEND_WIDTH)  outline.width = LEGEND_WIDTH;
	if (outline.x+LEGEND_WIDTH>size.width)
	    outline.x=parent.getSize().width-LEGEND_WIDTH;
	if (outline.y+LEGEND_HEIGHT>size.height) {
	    outline.height = outline.height + outline.y - size.height + LEGEND_HEIGHT;
	    outline.y = size.height - LEGEND_HEIGHT;
	}
	if (outline.height<LEGEND_HEIGHT)  outline.height = LEGEND_HEIGHT;

	this.position = new Rectangle(outline);
	this.position.height = LEGEND_HEIGHT;
	this.id = idCount++;
	this.parent = iface;
	this.sl = sl;
	this.app = sl.app;
	this.font = WIDGET_FONT;
	this.showBorder = true;

	fm = iface.getToolkit().getFontMetrics(font);
	textDescent = fm.getMaxDescent();
	textHeight =  fm.getMaxAscent()+textDescent;

	//addObserver(parent);
	//app.procWin.addObserver(this);
	image = parent.createImage(this);
    }

    public ColorLegendWidget(JInterface iface, StarLogo sl, boolean dummy) {
		image = parent.createImage(this);
	this.id = idCount++;
	this.parent = iface;
	this.sl = sl;
	this.app = sl.app;
	this.font = WIDGET_FONT;
	this.showBorder = false;

	fm = iface.getToolkit().getFontMetrics(font);
	textDescent = fm.getMaxDescent();
	textHeight =  fm.getMaxAscent()+textDescent;

    }

    //need to also save and be able to load the appropriate shape
    public static ColorLegendWidget load(ArrayList items, JInterface iface, StarLogo sl, boolean htmlp) {
	boolean showborderp = false;
	int shapep =0; 
	int colorp =20;
	int top = 0, left = 0, width = 50, height = 20, mid = 1;
	String name = "legend1", descript = "";
	int whichip = -1, onewhichip = -1;
	

	int i = 0;
	while (i<items.size()) {
	    String item = items.get(i).toString();
	    if (item.equals("top-left")) {
		String value = items.get(i+1).toString();
		String value2 = items.get(i+2).toString();
		top = Integer.parseInt(value);
		left = Integer.parseInt(value2);
		i += 3;
		continue;
	    }
	    if (item.equals("width-height")) {
		String value = items.get(i+1).toString();
		String value2 = items.get(i+2).toString();
		width = Integer.parseInt(value);
		height = Integer.parseInt(value2);
		i += 3;
		continue;
	    }
	    if (item.equals("name")) {
		name = items.get(i+1).toString();
		i += 2;
		continue;
	    }
	    if (item.equals("shape")) {
		String value = items.get(i+1).toString();
		shapep = Integer.parseInt(value);
		i += 2;
		continue;
	    }
	    if (item.equals("color")) {
		String value = items.get(i+1).toString();
		colorp = Integer.parseInt(value);
		i+=2;
		continue;
	    }
	    if (item.equals("show-border?")) {
		String value = items.get(i+1).toString();
		showborderp = Boolean.valueOf(value).booleanValue();
		i+=2;
		continue;
	    }  
	    if (item.equals("legend-number")) {
		String value = items.get(i+1).toString();
		mid = Integer.parseInt(value);
		i += 2;
		continue;
	    }
     	    if (item.equals("toolTip-text")) {
		String value = items.get(i+1).toString();
		descript = value;
		i += 2; 
		continue;
	    }
	    if (htmlp) {
		if (item.equals("whichip")) {
		    String value = items.get(i+1).toString();
		    whichip = Integer.parseInt(value);
		    i += 2;
		    continue;
		}
		if (item.equals("onewhichip")) {
		    String value = items.get(i+1).toString();
		    onewhichip = Integer.parseInt(value);
		    i += 2;
		    continue;
		}
	    }
	    i+=2;
	}

	Rectangle bounds = iface.getBoundaries();
	Rectangle position = new Rectangle(left + bounds.x, top + bounds.y, width, height);
	if (mid >= idCount) idCount = mid + 1;

	if (htmlp) {
	    return new ColorLegendWidget(name, colorp, position,
					 showborderp, shapep, mid, iface, whichip, onewhichip, sl, descript);
	} else {
	    return new ColorLegendWidget(name, colorp, position,
					 showborderp, shapep, mid, iface, sl, descript);
	}
    }

    public int getId() {
	return this.id;
    }
    public String getSaveString(boolean htmlp) throws IOException {
	Rectangle bounds = parent.getBoundaries();
	//   	if (!(isVis))
	String returnme = "SLLegend\r\n" +
	    "top-left " + (position.y - bounds.y) + " " + (position.x- bounds.x) + "\r\n" +
	    "width-height " + position.width + " " + position.height + "\r\n" +
	    "name \"" + name + "\"\r\n" + 
	    "shape " + shape + "\r\n" + 
	    "color " + color + "\r\n"+ 
	    "show-border? " + this.showBorder + "\r\n" + 
	    "legend-number " + id + "\r\n" +
	    getToolTipSaveString() +
	    ((htmlp && whichip >= 0 && onewhichip >= 0) ?
	     "whichip " + whichip + "\r\n" +
	     "onewhichip " + onewhichip + "\r\n" : "") +
	    "\r\n";
	return returnme;
    }

    static public void createNew(JInterface iface) {
	ColorLegendDialog cld = Application.createColorLegendDialog((Frame)iface.frame(), iface.app(), iface.app().textBundle.getString("new_legend" ));
	cld.show();
	if (cld.pressedOK()) {
	    ColorLegendWidget mw = new ColorLegendWidget(iface, iface.sl());
	    cld.getValues(mw);
	    mw.updateMinSize();
	    iface.addWidget(mw);
	}
	cld.dispose();
    }

    public void edit() {
	ColorLegendDialog cld = Application.createColorLegendDialog((Frame)parent.frame(), app, 
								    app.textBundle.getString("edit_legend"), this);

	cld.show();
	if (cld.pressedOK()) {
	    boolean showBorderOld = showBorder;
	    String oldName = name;
	    parent.addHandlesToClipRect(this);
	    cld.getValues(this);
	    if ((showBorderOld != showBorder) || 
		!(oldName.equals(name))) {
		updateMinSize();
		update(position);
		app.project.markChanged();
	    }
	}
	cld.dispose();
	super.edit();
    }
  
    public void setValue(Object value) {
	this.value = value;
	parent.repaint(position);
    }

    public void delete() {
	parent.legends().removeElement(this);
	super.delete();
	//app.procWin.deleteObserver(this);
    }

    public void update() {

    }

    public void update(Observable obs, Object arg) {
	if (arg instanceof SLProc)
	    update();
    }

    public void update(Object arg) {
	setChanged();
	notifyObservers(arg);
    }
        
    public String toString() {
	return(app.textBundle.getString("legend_label") + " " + name);
    }
  
    public void addConsumer(ImageConsumer ic) { 
	startProduction(ic);
	//System.out.println("called add Consumer with " + ic);
    }

    public boolean isConsumer(ImageConsumer ic) {
	return (this.ic == ic); 
    }

    public void removeConsumer(ImageConsumer ic) {
	//System.out.println("called remove consumer wiht " + ic);
	this.ic = null; 
    }
    public void requestTopDownLeftRightResend(ImageConsumer ic) {
	ic.setHints(ImageConsumer.TOPDOWNLEFTRIGHT);
	ic.setPixels(0, 0, width, height,
		     Colors.tcm, drawShape(shape), 0, rowbytes);	// As many times as needed
	ic.imageComplete(ImageConsumer.SINGLEFRAMEDONE);
	ic.setHints(ImageConsumer.RANDOMPIXELORDER);
    }

    public void startProduction(ImageConsumer ic) {
	if (ic == null) return;
	//System.out.println("called start production with " + ic);
	if (this.ic == null) {
	    this.ic = ic; 
	    ic.setColorModel(Colors.tcm);
	    ic.setDimensions(width, height);
	}
	ic.setHints(ImageConsumer.TOPDOWNLEFTRIGHT);
	ic.setPixels(0, 0, width, height,
		     Colors.tcm, drawShape(shape), 0, rowbytes);	// As many times as needed
	ic.imageComplete(ImageConsumer.SINGLEFRAMEDONE);
	ic.setHints(ImageConsumer.RANDOMPIXELORDER);
    }
    public void refresh() {
	//	for (int i = 0; i < TurtleShapes.MAXSHAPES; i++) {
				drawShape(shape);
				//	}
			if (ic != null) startProduction(ic);
    }


    public int[] drawShape(int shape) {
			int[] legendImagePixels = new int[TurtleShapes.MEDIUM * TurtleShapes.MEDIUM];
			int y = 0;
			TurtleShapes ts = TurtleShapes.getShape(shape);
			double[] colors = (ts.getColorsForSize(TurtleShapes.MEDIUM))[0];
			int[]    masks  = (ts.getMasksForSize(TurtleShapes.MEDIUM))[0];
			for (int z = 0; z < legendImagePixels.length; z++) {
				if (masks[2*z] == TurtleShapes.ON) {
					legendImagePixels[y++] = color;
				} else if (masks[2*z+1] == TurtleShapes.ON) {
				    legendImagePixels[y++] = pcolmask;
				} else {
				    legendImagePixels[y++] = Colors.colorarrayints[(int)(colors[z] * 32.0)];
				}
			}
			return legendImagePixels;
    }


    public void paint(Graphics g) {
	this.MIN_HEIGHT= 16;
	g.translate(position.x, position.y);

	Shape oldclip = g.getClip();
	if(color == -1)
 	    color = Colors.colorarrayints[(int)(8.5*32.0)];
    	
	if(showBorder){
	    g.setColor(Color.lightGray);
	    g.drawRect(0,0,position.width -1 , position.height -1);
	}

	g.clipRect(1,1, position.width -2, position.height - 2);
	g.drawImage(image, 0, 0, this); 

	g.setClip(oldclip);
	paintText(g); 

	g.setClip(oldclip);
	g.translate(-position.x, -position.y);
    }

    public void paintText(Graphics g) {
	g.setColor(Color.darkGray);
	g.setFont(font);
	
	int textWidth = fm.stringWidth(name);

	g.clipRect(0, 0, position.width, position.height);
	g.drawString(name, 35, 6 + textHeight / 2 - textDescent);
    }

 public int resizeWhichCorner(Point anchor) {
	Rectangle test = new Rectangle();
	test.setBounds(position.x-CORNER_WIDTH, position.y-CORNER_WIDTH,
		       CORNER_WIDTH, CORNER_WIDTH);
	if (test.contains(anchor.x, anchor.y)) return JInterface.RESIZELEFT;

	test.setBounds(position.x+position.width, position.y-CORNER_WIDTH,
		       CORNER_WIDTH, CORNER_WIDTH);
	if (test.contains(anchor.x, anchor.y)) return JInterface.RESIZERIGHT;

	test.setBounds(position.x-CORNER_WIDTH, position.y+position.height,
		       CORNER_WIDTH, CORNER_WIDTH);
	if (test.contains(anchor.x, anchor.y)) return JInterface.RESIZELEFT;

	test.setBounds(position.x+position.width, position.y + position.height,
		       CORNER_WIDTH, CORNER_WIDTH);
	if (test.contains(anchor.x, anchor.y)) return JInterface.RESIZERIGHT;

	return 0;
    }



    public void updateMinSize() {
	int textWidth = fm.stringWidth(name);
	int minwidth = width + textWidth + 19;
	int maxheight = fm.getAscent()+6;
	if (position.height != maxheight) {position.height = maxheight;}
	if (position.width != minwidth) {position.width = minwidth;}
	if (parent.outline() != null) {parent.outline().width = position.width;}
    }

   

    public void displayOneString(String s) {
	Application.createMessageDialog(app, (StarLogoFrame)parent.frame(), 
					app.textBundle.getString("error_label"), 
					s, true);
    }

    public void displayAString(String s) {
	Application.createMessageDialog(app, (StarLogoFrame)parent.frame(), 
					app.textBundle.getString("error_label"), 
					s, true);
    }

              
    public void handleMouseClick(MouseEvent e) {
	if (parent.getToolbar().buttonDown == ToolbarWidget.LEGEND) 
	    if (!app.kioskp && YoYo.applet == null) edit(); 
    } 

    public int getToolbarButton() { 
	// what button on the toolbar corresponds to this widgetdrag-- overrridden
	return ToolbarWidget.LEGEND;
    }
  
   

    public static void loadImages(String gifdir, MediaTracker tracker, Toolkit kit)
	throws MalformedURLException {
	// requires: trackerID in Widget indexes start of empty ID #s in <tracker>
    }
  
    public WidgetDrag copy(int edition, Rectangle r) {
	String s = name + " copy" + WidgetDrag.editionString(edition);
	return new ColorLegendWidget(s,color, r, showBorder, shape, ++ColorLegendWidget.idCount, parent, whichip, onewhichip, sl, getToolText());
    }
}





