/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.awt.*;
import java.io.*;
import yoyo.Context;
import java.awt.image.*;
import java.net.*;
import yoyo.YoYo;
import java.util.*;
//import java.awt.color.*;

public class Colors {

    static ColorModel cm, tcm;
    static final int numcolors = 140;
    static final int numprimarycolors = 14;
    static short[] colormap;
    static final int innercolorsbits = 5; 
    static final int innercolors = 1 << innercolorsbits;
    static final double innercolorsdouble = innercolors + 0.0;
    static ResourceBundle tBundle;

    public static ResourceBundle getRBundle() {
	if (tBundle == null) {
	    tBundle = ResourceBundle.getBundle("starlogo.GUIBundle", YoYo.language);
	}
	return tBundle;
    }
    
    static String[] colorStrings = null;
    
    public static final int BLACK = 0;
    public static final int WHITE = 9;
    public static final int GRAY = 5;
    public static final int RED = 15;
    public static final int ORANGE = 25;
    public static final int BROWN = 35;
    public static final int YELLOW = 45;
    public static final int GREEN = 55;
    public static final int LIME = 65;
    public static final int TURQUOISE = 75;
    public static final int CYAN = 85;
    public static final int SKY = 95;
    public static final int BLUE = 105;
    public static final int VIOLET = 115;
    public static final int MAGENTA = 125;
    public static final int PINK = 135;
    
    static Hashtable colornamehash;
    static Hashtable colornumhash;

    static boolean colornamehashsetup = false;

    /*
     * trailingZeroTable[i] is the number of trailing zero bits in the binary
     * representaion of i.
     */
    final static byte trailingZeroTable[] = {
      -25, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0,
	4, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0,
	5, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0,
	4, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0,
	6, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0,
	4, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0,
	5, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0,
	4, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0,
	7, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0,
	4, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0,
	5, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0,
	4, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0,
	6, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0,
	4, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0,
	5, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0,
	4, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0};


    static synchronized void setupColorNameHash() {
	if (!colornamehashsetup) {
	    colorStrings = new String[numprimarycolors];
	    colorStrings[0] = getRBundle().getString("gray");
	    colorStrings[1] = getRBundle().getString("red");
	    colorStrings[2] = getRBundle().getString("orange");
	    colorStrings[3] = getRBundle().getString("brown");
	    colorStrings[4] = getRBundle().getString("yellow");
	    colorStrings[5] = getRBundle().getString("green");
	    colorStrings[6] = getRBundle().getString("lime");
	    colorStrings[7] = getRBundle().getString("turquoise");
	    colorStrings[8] = getRBundle().getString("cyan");
	    colorStrings[9] = getRBundle().getString("sky");
	    colorStrings[10] = getRBundle().getString("blue");
	    colorStrings[11] = getRBundle().getString("violet");
	    colorStrings[12] = getRBundle().getString("magenta");
	    colorStrings[13] = getRBundle().getString("pink");
	    
	    colornamehash = new Hashtable(numprimarycolors + 2);
	    colornumhash = new Hashtable(numprimarycolors + 2);
	    
	    colornamehash.put(getRBundle().getString("black"), new Double(0));
	    colornumhash.put(new Double(0), getRBundle().getString("black"));
	    
	    colornamehash.put(getRBundle().getString("white"), new Double(9));
	    colornumhash.put(new Double(9), getRBundle().getString("white"));
	    for(int i = 0; i < numprimarycolors; i++) {
		colornamehash.put(colorStrings[i], new Double((i * 10) + 5));
		colornumhash.put(new Double((i * 10) + 5), colorStrings[i]);
	    }
	    colornamehashsetup = true;
	}
    }

    public static String doubleToString (double num) {
	if ((num - (int)num) == 0)
	    return Integer.toString((int)num);
	else
	    return Double.toString((double)((int)(1000*num))/1000);
    }

    public static int colorNameToNumber(String name) {
	if (colornamehash == null) { setupColorNameHash(); }
	Object o = colornamehash.get(name);
	if (o == null) return 0;
	return ((Double)o).intValue();
    }

    public static String colorNumToName(double col) {
	if (colornumhash == null) { setupColorNameHash(); }
	Object o = colornumhash.get(new Double(col));
	if (o == null) return doubleToString(col);
	else return ((String)o);
    }

// 	GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
// 	GraphicsDevice[] gs = ge.getScreenDevices();
// 	for (int j = 0; j < gs.length; j++) { 
// 	    GraphicsDevice gd = gs[j];
// 	    GraphicsConfiguration[] gc = gd.getConfigurations();
// 	    for (int i=0; i < gc.length; i++) {
// 		System.out.println("Graphics config " + i + " is " + gc[i]);
// 		ColorModel model = gc[i].getColorModel();
// 		System.out.println("color model is " + model);
// 	    }
// 	}

  static int[] reds = { 0, 28, 56, 85, 113, 142, 170, 199, 227, 255,
		  76, 115, 153, 191, 230, 255, 255, 255, 255, 255,
		  76, 115, 153, 191, 230, 255, 255, 255, 255, 255,
		  48, 72, 96, 120, 144, 160, 179, 198, 217, 236,
		  76, 115, 153, 191, 230, 255, 255, 255, 255, 255,
		  24, 36, 48, 60, 73, 81, 116, 151, 186, 221,
		  0, 0, 0, 0, 0, 0, 51, 102, 153, 204,
		  0, 0, 0, 0, 0, 0, 51, 102, 153, 204,
		  0, 0, 0, 0, 0, 0, 51, 102, 153, 204,
		  0, 0, 0, 0, 0, 0, 51, 102, 153, 204,
		  0, 0, 0, 0, 0, 0, 51, 102, 153, 204,
		  38, 57, 76, 95, 115, 127, 153, 179, 204, 230,
		  76, 115, 153, 191, 230, 255, 255, 255, 255, 255,
		  76, 115, 153, 191, 230, 255, 255, 255, 255, 255 };

  static int[] greens = {0, 28, 56, 85, 113, 142, 170, 199, 227, 255,
		 0, 0, 0, 0, 0, 0, 51, 102, 153, 204,
		 25, 38, 51, 63, 76, 85, 119, 153, 187, 221,
		 33, 50, 67, 84, 100, 112, 140, 169, 198, 227,
		 76, 115, 153, 191, 230, 255, 255, 255, 255, 255,
		 59, 88, 118, 148, 177, 197, 209, 220, 232, 244,
		 76, 115, 153, 191, 230, 255, 255, 255, 255, 255,
		 76, 115, 153, 191, 230, 255, 255, 255, 255, 255,
		 76, 115, 153, 191, 230, 255, 255, 255, 255, 255,
		 38, 57, 76, 95, 115, 127, 153, 179, 204, 230,
		 0, 0, 0, 0, 0, 0, 51, 102, 153, 204,
		 0, 0, 0, 0, 0, 0, 51, 102, 153, 204,
		 0, 0, 0, 0, 0, 0, 51, 102, 153, 204,
		 0, 0, 0, 0, 0, 0, 51, 102, 153, 204 };

  static int[] blues = {0, 28, 56, 85, 113, 142, 170, 199, 227, 255,
		0, 0, 0, 0, 0, 0, 51, 102, 153, 204,
		0, 0, 0, 0, 0, 0, 51, 102, 153, 204,
		24, 36, 48, 60, 72, 80, 115, 150, 185, 220,
		0, 0, 0, 0, 0, 0, 51, 102, 153, 204,
		0, 0, 0, 0, 0, 0, 51, 102, 153, 204,
		0, 0, 0, 0, 0, 0, 51, 102, 153, 204,
		59, 89, 118, 148, 178, 198, 209, 221, 232, 244,
		76, 115, 153, 191, 230, 255, 255, 255, 255, 255,
		76, 115, 153, 191, 230, 255, 255, 255, 255, 255,
		76, 115, 153, 191, 230, 255, 255, 255, 255, 255,
		76, 115, 153, 191, 230, 255, 255, 255, 255, 255,
		76, 115, 153, 191, 230, 255, 255, 255, 255, 255,
		38, 57, 76, 95, 115, 127, 153, 179, 204, 230 };

    static byte[] redbytes, greenbytes, bluebytes;

  static public Color[] colorarray;
    static int[] colorarrayints, colorarrayintsBufPC;

  static {
      try {
	  setupColorArray();
	  colorarrayints = new int[numcolors << innercolorsbits];
	  setupColorArrayIntegers(colorarrayints, 
				  new int[]{8, 8, 8, 8},
				  new int[]{0xff0000, 0xff00, 0xff, 0xff000000},
				  true);
	  colormap = readMap();
	  setupColorNameHash();
	  tcm = new DirectColorModel(32, 0xFF0000, 0x00FF00, 0x0000FF, 0xFF000000);
	  cm =  (YoYo.macosxp) ? tcm : new DirectColorModel(32, 0xFF0000, 0x00FF00, 0x0000FF);
      } catch (Exception e) { e.printStackTrace(); }
      
  }

  static void setupColorArray() {
    colorarray = new Color[numcolors << innercolorsbits];

    redbytes = new byte[numcolors << innercolorsbits + 1];
    greenbytes = new byte[numcolors << innercolorsbits + 1];
    bluebytes = new byte[numcolors << innercolorsbits + 1];

    for (int color = 0; color < (numcolors / 10); color++) {
      for (int shade = 0; shade < 9; shade++) {
	int lowcolor = (color * 10) + shade;
	int highcolor = lowcolor + 1;

	int lowred = reds[lowcolor], lowgreen = greens[lowcolor],
	    lowblue = blues[lowcolor];
	int highred = reds[highcolor], highgreen = greens[highcolor],
	  highblue = blues[highcolor];
	int diffred = highred - lowred + 1, diffgreen = highgreen - lowgreen + 1,
	  diffblue = highblue - lowblue + 1;
	double incred = ((double)diffred) / innercolorsdouble;
	double incgreen = ((double)diffgreen) / innercolorsdouble;
	double incblue = ((double)diffblue) / innercolorsdouble;

	for(int hue = 0; hue < innercolors; hue++) {
	  int index = (lowcolor << innercolorsbits) + hue;
	  int red = (int)((double)lowred + (incred * (double)hue));
	  int green = (int)((double)lowgreen + (incgreen * (double)hue));
	  int blue = (int)((double)lowblue + (incblue * (double)hue));
	  colorarray[index] = new Color(red, green, blue);
	  redbytes[index] = (byte)(red & 0xff);
	  greenbytes[index] = (byte)(green & 0xff);
	  bluebytes[index] = (byte)(blue & 0xff);
	}
      }

      // the last color goes to white at 9.99999
      int shade = 9;
      int lowcolor = (color * 10) + shade;
      int highcolor = lowcolor + 1;
      int lowred = reds[lowcolor], lowgreen = greens[lowcolor],
	lowblue = blues[lowcolor];
      int highred = 255, highgreen = 255, highblue = 255;
      int diffred = highred - lowred + 1, diffgreen = highgreen - lowgreen + 1,
	diffblue = highblue - lowblue + 1;
      double incred = ((double)diffred) / innercolorsdouble;
      double incgreen = ((double)diffgreen) / innercolorsdouble;
      double incblue = ((double)diffblue) / innercolorsdouble;

      for(int hue = 0; hue < innercolors; hue++) {
	int index = (lowcolor << innercolorsbits) + hue;
	int red = (int)((double)lowred + (incred * (double)hue));
	int green = (int)((double)lowgreen + (incgreen * (double)hue));
	int blue = (int)((double)lowblue + (incblue * (double)hue));
	colorarray[index] = new Color(red, green, blue);
	redbytes[index] = (byte)(red & 0xff);
	greenbytes[index] = (byte)(green & 0xff);
	bluebytes[index] = (byte)(blue & 0xff);
      }
    }
    redbytes[redbytes.length-1] = 0;
    greenbytes[greenbytes.length-1] = 0;
    bluebytes[bluebytes.length-1] = 0;

  }

    public static void setupColorArrayBufPC(PackedColorModel cm) {
	if (colorarrayintsBufPC == null) {
	    colorarrayintsBufPC = new int[numcolors << innercolorsbits];
	    setupColorArrayIntegers(colorarrayintsBufPC,
				    cm.getComponentSize(),
				    cm.getMasks(),
				    (cm.getTransparency() == 
				     Transparency.TRANSLUCENT));
	}
    }

    public static void setupColorArrayIntegers(int[] colorArrayOfInts, 
					       int[] colorBits, 
					       int[] colorMasks,
					       boolean addAlpha) {
	//	System.out.println("setup color array integers");
	int redBits = colorBits[0];
	int greenBits = colorBits[1];
	int blueBits = colorBits[2];
	int alphaBits = (addAlpha) ? colorBits[3] : 0;
// 	System.out.println("colorbits: (r,g,b,a) = (" + redBits + ", " +
// 			   greenBits + ", " + blueBits + ", " + alphaBits + ")");

	int redMask = colorMasks[0];
	int greenMask = colorMasks[1];
	int blueMask = colorMasks[2];
	int alphaMask = (addAlpha) ? colorMasks[3] : 0;
// 	System.out.println("colormasks: (r,g,b,a) = (" + Integer.toString(redMask,16) + ", " +
// 			   Integer.toString(greenMask,16) + ", " + 
// 			   Integer.toString(blueMask,16) + ", " + 
// 			   Integer.toString(alphaMask,16) + ")");

	int redShift = trailingZeroCnt(redMask);
	int greenShift = trailingZeroCnt(greenMask);
	int blueShift = trailingZeroCnt(blueMask);
	int alphaShift = (addAlpha) ? trailingZeroCnt(alphaMask) : 0;
// 	System.out.println("colorshifts: (r,g,b,a) = (" + redShift + ", " +
// 			   greenShift + ", " + blueShift + ", " + alphaShift + ")");

	if (addAlpha) {
	    for(int i = 0; i < colorarray.length; i++) {
		Color col = colorarray[i];
		int red = col.getRed() >>> (8 - redBits);
		int green = col.getGreen() >>> (8 - greenBits);
		int blue = col.getBlue() >>> (8 - blueBits);
		colorArrayOfInts[i] = (0xff << alphaShift) | (red << redShift) 
		    | (green << greenShift) | (blue << blueShift);
	    }
	} else {
	    for(int i = 0; i < colorarray.length; i++) {
		Color col = colorarray[i];
		//System.out.println("orig color: " + col);
		int red = col.getRed() >>> (8 - redBits);
		int green = col.getGreen() >>> (8 - greenBits);
		int blue = col.getBlue() >>> (8 - blueBits);
		colorArrayOfInts[i] = (red << redShift) | (green << greenShift) |
		    (blue << blueShift);
		//System.out.println("new color: " + Integer.toString(colorArrayOfInts[i], 2));
	    }
	}
    }

  public static short[] readMap() {
    try {
      URL u = Colors.class.getResource("colormap.yuv.bin");
      InputStream is = null;
      try {
	is = u.openStream();
	DataInputStream dis = new DataInputStream(new BufferedInputStream(is, 4096));
	short[] colormap = new short[dis.readInt()];
	for(int i = 0; i < colormap.length; i++) {
	  colormap[i] = dis.readShort();
	}
	return colormap;
      }
      catch(IOException e) { e.printStackTrace(); }
    }
    catch(Exception f) { f.printStackTrace(); }
    return null;
  }


  public static void writeMap(Context c) {
    short[] colormap = createInverseColorMap();
    try {
      File f = new File("c:\\colormap.bin");
      DataOutputStream dos = new DataOutputStream(new BufferedOutputStream(new FileOutputStream(f)));
      dos.writeInt(colormap.length);
      for(int i = 0; i < colormap.length; i++) {
	dos.writeShort(colormap[i]);
      }
      dos.flush();
      dos.close();
    }
    catch(IOException e) {
	c.err.println("problem");
    }
  }

  public static short[] createInverseColorMap() {
    short[] colormap = new short[innercolors * 64 * 32];
    double[] stary, staru, starv;
    stary = new double[colorarrayints.length];
    staru = new double[colorarrayints.length];
    starv = new double[colorarrayints.length];

    for(int i = 0; i < colorarrayints.length; i++) {
      int color = colorarrayints[i];
      int sr = ((color >> 16) & 0xFF);
      int sg = ((color >> 8) & 0xFF);
      int sb = color & 0xFF;
      stary[i] = sr; //0.299 * sr + 0.587 * sg + 0.114 * sb;
      staru[i] = sg; //-0.147 * sr + 0.289 * sg + 0.436 * sb;
      starv[i] = sb; //0.615 * sr - 0.515 * sg - 0.1 * sb;
    }

    for(int r = 0; r < 255; r += 8) {
      for(int g = 0; g < 255; g += 4) {
	for(int b = 0; b < 255; b += 8) {
	  double y = r; //0.299 * r + 0.587 * g + 0.114 * b;
	  double u = g; //-0.147 * r + 0.289 * g + 0.436 * b;
	  double v = b; //0.615 * r - 0.515 * g - 0.1 * b;
	  double mindist = Double.MAX_VALUE;
	  int minelem = 0;
	  for(int i = 0; i < colorarrayints.length; i++) {
	    double sy = stary[i];
	    double su = staru[i];
	    double sv = starv[i];
	    double dy = sy - y, du = su - u, dv = sv - v;
	    double dist = dy * dy + du * du + dv * dv;
	    if (dist < mindist) {
	      mindist = dist;
	      minelem = i;
	    }
	  }
	  colormap[(((r >> 3) & 0x1F) << 11) |
		   (((g >> 2) & 0x3F) << 5) |
		  (((b >> 3) & 0x1F))] = (short)minelem;
	}
      }
    }
    return colormap;
  }

  public static double mapColorToStarLogoColor(int rgb) {
    int r = (rgb >> 16) & 0xFF;
    int g = (rgb >> 8) & 0xFF;
    int b = rgb & 0xFF;
    int index = (((r >> 3) & 0x1F) << 11) |
		   (((g >> 2) & 0x3F) << 5) |
		  (((b >> 3) & 0x1F));
    double ret = (double)colormap[index] / innercolorsdouble;
    assert ret >= 0.0 || ret < numcolors : "Color out of bounds."; 
    return ret;
  }

   public static int trailingZeroCnt(int val) {
        // Loop unrolled for performance
        int byteVal = val & 0xff;
        if (byteVal != 0)
            return trailingZeroTable[byteVal];

        byteVal = (val >>> 8) & 0xff;
        if (byteVal != 0)
            return trailingZeroTable[byteVal] + 8;

        byteVal = (val >>> 16) & 0xff;
        if (byteVal != 0)
            return trailingZeroTable[byteVal] + 16;

        byteVal = (val >>> 24) & 0xff;
        return trailingZeroTable[byteVal] + 24;
    }



}





