/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.lang.Math;
import java.awt.event.*;
import java.awt.*;
import java.net.*;
import yoyo.*;

public class Colortool extends Widget implements MouseListener {

    static int cellWidth1 = 15;
    static int cellWidth2 = 21;
    static int cellHeight = 24;

    static int cellHeight1 = 15;
    static int cellHeight2 = 21;
    static int cellWidth = 24;

    private JInterface iface;

  public FontMetrics fm;
  public Font font;

    public int colorDown = 0;
    public int shadeDown = 5;
    public int curcolor = 5;

    boolean horizontalp;

    public Colortool(JInterface iface) {
	this(iface, true);
    }

    public Colortool(JInterface iface, boolean horizontalp) {
	super(iface);
	this.horizontalp = horizontalp;
	position = new Rectangle(0, 0, (horizontalp) ? cellWidth2 * 10 : cellHeight * 2, 
				 (horizontalp) ? cellWidth * 2 : cellHeight2 * 10);
	this.iface = iface;
	this.font = WIDGET_FONT;
	fm = iface.getToolkit().getFontMetrics(font);
    }

    public Colortool() {
	this(true);
    }

    public Colortool(boolean horizontalp) {
	super();
	this.horizontalp = horizontalp;
	position = new Rectangle(0, 0, (horizontalp) ? cellWidth2 * 10 : cellHeight * 2, 
				 (horizontalp) ? cellWidth * 2 : cellHeight2 * 10);
    }

    public void pointToButton(int x, int y) {
	if (horizontalp) {
	    int row = (int)java.lang.Math.floor((y - position.y) / cellHeight);
	    if (row == 0) {
		int column = (int)java.lang.Math.floor((x - position.x) / cellWidth1);
		setColor(column, shadeDown);
	    } else {
		int column = (int)java.lang.Math.floor((x - position.x) / cellWidth2);
		setColor(colorDown, column);
	    }
	} else {
	    int column = (int)java.lang.Math.floor((x - position.x) / cellWidth);
	    if (column == 0) {
		int row = (int)java.lang.Math.floor((y - position.y) / cellHeight1);
		setColor(row, shadeDown);
	    } else {
		int row = (int)java.lang.Math.floor((y - position.y) / cellHeight2);
		setColor(colorDown, row);
	    }
	}
    }

    public void mouseClicked(MouseEvent e) {
	if (!position.contains(new Point(e.getX(), e.getY()))) return;
	pointToButton(e.getX(), e.getY());
    }

    public void mouseDragged(MouseEvent e) {
	mouseClicked(e);
    }
    
    public void mousePressed(MouseEvent e) {
	mouseClicked(e);
    }
    
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {}

    public void setColor(int wholeColor) {
    	if ((wholeColor < 0) || (wholeColor >= Colors.numcolors)) {
	    iface.colorchooser().restoreOld();
	    return;
    	}
    	setColor((wholeColor / 10), (wholeColor % 10));
    }
    
    public void setColor(int newColor, int newShade) {
	//System.out.println("old color down = " + this.colorDown + " old shadeDown = "  + this.shadeDown);
	//System.out.println("color down = " + colorDown + " shadeDown = "  + shadeDown);
	//System.out.println("old curcolor = " + curcolor);
	//System.out.println("setting new color");
	if (newColor == colorDown && newShade == shadeDown) return;
	this.colorDown = newColor;
	this.shadeDown = newShade;
	curcolor = colorDown*10+shadeDown;
	parent.colorchooser().setColor(curcolor);
	parent.paintselect().updateTurtleColor(Colors.colorarray[curcolor * 32]);
	parent.addToClipRect(position);
	parent.addToClipRect(parent.paintselect().position);
	iface.repaint(iface.clipRect());
    }


    public void paint(Graphics g) {
	g.translate(position.x, position.y);
	Color oldcolor = g.getColor();
	if (horizontalp) {
	    for (int i = 0; i < Colors.numprimarycolors; i++) {
		int color = i*10+shadeDown;
		g.setColor(Colors.colorarray[color * 32]);
		g.fillRect(i*cellWidth1, 0, cellWidth1, cellHeight);
	    }

	    Color selectedColor = Colors.colorarray[curcolor * 32];
	    int blueComponent = 255 - selectedColor.getBlue();
	    int greenComponent = 255 - selectedColor.getGreen();
	    int redComponent = 255 - selectedColor.getRed();
	    Color highlightColor = new Color(redComponent, greenComponent, blueComponent);
	    g.setColor(highlightColor);
	    g.drawRect(colorDown*cellWidth1, 0, cellWidth1-1, cellHeight-1);
	    for (int i = 0; i < 10; i++) {
		int color = colorDown*10+i;
		g.setColor(Colors.colorarray[color * 32]);
		g.fillRect(i*cellWidth2, cellHeight, cellWidth2, cellHeight);
	    }
	    g.setColor(highlightColor);
	    g.drawRect(shadeDown*cellWidth2, cellHeight, cellWidth2-1, cellHeight-1);
	} else {
	    for (int i = 0; i < Colors.numprimarycolors; i++) {
		int color = i*10+shadeDown;
		g.setColor(Colors.colorarray[color * 32]);
		g.fillRect(0, i*cellHeight1, cellWidth, cellHeight1);
	    }

	    Color selectedColor = Colors.colorarray[curcolor * 32];
	    int blueComponent = 255 - selectedColor.getBlue();
	    int greenComponent = 255 - selectedColor.getGreen();
	    int redComponent = 255 - selectedColor.getRed();
	    Color highlightColor = new Color(redComponent, greenComponent, blueComponent);
	    g.setColor(highlightColor);
	    g.drawRect(0, colorDown*cellHeight1, cellWidth-1, cellHeight1-1);
	    for (int i = 0; i < 10; i++) {
		int color = colorDown*10+i;
		g.setColor(Colors.colorarray[color * 32]);
		g.fillRect(cellWidth, i*cellHeight2, cellWidth, cellHeight2);
	    }
	    g.setColor(highlightColor);
	    g.drawRect(cellWidth, shadeDown*cellHeight2, cellWidth-1, cellHeight2-1);
	}
	g.setColor(oldcolor);
	g.translate(-position.x, -position.y);
    }

}



