/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import yoyo.CompoundSymbol;
import yoyo.Context;
import yoyo.Symbol;
import yoyo.YoYoCaller;
import java.awt.Font;
import java.awt.Point;
import java.awt.Dimension;
import java.awt.Cursor;
import java.awt.Frame;
import java.util.Observer;
import java.util.Observable;
import java.io.LineNumberReader;
import java.io.StringReader;
import java.io.IOException;

public interface ControlCenter extends MenuListener,
				       YoYoCaller, DisplayMessage, StarLogoFrame, SLPrintable {

  public static final int tur = 0;
  public static final int obs = 1;
	
  public static final int LABEL_VERTICAL_SPACE = 5;  
  public static final int DRAG_HEIGHT = 4;

  abstract public void setObsProcs(Object[] procnames, Integer[] linenumbers);
  abstract public void setTurProcs(Object[] procnames, Integer[] linenumbers);
    abstract public void clearProcs();
  abstract public boolean lookupUfun(String name, int where);
	
  abstract SLCommand getSLCommand(int whichone);
  abstract SLProc getSLProc(int whichone);

  public void turtlePrint(String s);
  public void observerPrint(String s);
  public void turtlePrintln(String s);
  public void observerPrintln(String s);
  public void observerClearText();
  public void observerClearCC();
  public void turtleClearText();
  public void turtleClearCC();

  public void clearText();
  public void reset();
    public void setTitle(String s);
	
  public void resetText(String contents);
    public void compileText(ProgressBar pb, boolean justVariables);
  public String getSaveString();
  abstract public int getCCHeight(int which);
  abstract public void setCCHeight(int which, int height);

  public void setFont(Font f);
  public void setCursor(Cursor c);
  public Cursor getCursor();
  public Dimension getSize();
  public void setBounds(int x, int y, int width, int height); 
  public void show();
  public void deleteObservers();
  public void deleteObserver(Observer o);
  public void addObserver(Observer o);
  public void requestFocus();
  public void setVisible(boolean shownp);
  public Point getLocation();
    public void flush();

}
