/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.util.Vector;
import java.util.ArrayList;
import yoyo.YoYo;

public class ExtraInfo {

    private Application app;
    
    public int id = 0;
    public int order = 0;

    public boolean onExtra = true;
    public String instString;
    public String breedString;
    public String kindString;
    public String varString = "";
    public boolean onWith = false;
    public String withText;
    public boolean onNumber = true;

    public boolean xyOn = false;
    public boolean xVal = false;

    public static String[] FUNCTIONS;
    public static boolean initialized = false;

    public static void initializeExtraInfo(Application app) {
	FUNCTIONS = new String[]{
	    app.textBundle.getString("number_of"),
	    app.textBundle.getString("average_of"),
	    app.textBundle.getString("min_of"),
	    app.textBundle.getString("max_of"),
	    app.textBundle.getString("median_of"),
	    app.textBundle.getString("mode_of"),
	    app.textBundle.getString("variance_of"),
	    app.textBundle.getString("standard_deviation_of"),
	    app.textBundle.getString("globals"),
	    app.textBundle.getString("sliders"),
	};
	initialized = true;
    }

    ExtraInfo(Application app, int id, int order) {
	this.app = app;
	this.id = id;
	this.order = order;
	if (!initialized) initializeExtraInfo(app);
	this.instString = FUNCTIONS[0];
	this.breedString = app.textBundle.getString("turtles");
	this.kindString = app.textBundle.getString("nothing"); 
	this.withText = app.commandsBundle.getString("true");
    }

    ExtraInfo(Application app, int id, int order, boolean b) {
	this.app = app;
	this.id = id;
	this.order = order;
	this.onExtra = b;
	if (!initialized) initializeExtraInfo(app);
	this.instString = FUNCTIONS[0];
	this.breedString = app.textBundle.getString("turtles");
	this.kindString = app.textBundle.getString("nothing");
	this.withText = app.commandsBundle.getString("true");
    }

    ExtraInfo(Application app, int id, int order, boolean b, String i, 
	      String br, String v, String k, boolean o, String w) {
	this.app = app;
	this.id = id;
	this.order = order;
	onExtra = b;
	instString = i;
	if (instString.equalsIgnoreCase(app.textBundle.getString("number_of"))) 
	    onNumber = true;
	else onNumber = false;
	breedString = br;
	varString = v;
	kindString = k;
	onWith = o;
	withText = w;
    }

    ExtraInfo(Application app, int id, int order, boolean b, String i, 
	      String br, String v, String k, boolean o, String w, 
	      boolean xyOn, boolean xVal) {
	this.app = app;
	this.id = id;
	this.order = order;
	onExtra = b;
	instString = i;
	if (instString.equalsIgnoreCase(app.textBundle.getString("number_of"))) 
	    onNumber = true;
	else onNumber = false;
	breedString = br;
	varString = v;
	kindString = k;
	onWith = o;
	withText = w;
	this.xyOn = xyOn;
	this.xVal = xVal;
    }


    public String getInstFromEI() {
	return getInstFromEI(instString);
    }

    public String getInstFromEI(String instString) {
	StringBuffer sb = new StringBuffer();
	//System.out.println("getInstFromEi " + instString);
	if (onExtra) {
	    if(instString.equalsIgnoreCase(app.textBundle.getString("number_of"))) {
		if (breedString.equalsIgnoreCase(app.textBundle.getString("patches"))) {
		    sb.append(" ( ");
		    sb.append(app.commandsBundle.getString("count_patches_with"));
		    sb.append("[");
		    if (onWith) sb.append(withText).append("] )");
		    else sb.append("1 = 1] )");
		    return sb.toString();
		} else {
		    sb.append(" ( ");
		    sb.append(app.commandsBundle.getString("count_"));
		    sb.append(breedString);
		    if (onWith) {
			sb.append(app.commandsBundle.getString("_with"));
			sb.append(" [ ").append(withText).append(" ] ");
		    } 
		    sb.append(" ) ");
		    return sb.toString();
		}
	    }

	    if (instString.equalsIgnoreCase(app.textBundle.getString("average_of")) ||
		instString.equalsIgnoreCase(app.textBundle.getString("min_of")) ||
		instString.equalsIgnoreCase(app.textBundle.getString("max_of")) ||
		instString.equalsIgnoreCase(app.textBundle.getString("median_of")) ||
		instString.equalsIgnoreCase(app.textBundle.getString("mode_of")) ||
		instString.equalsIgnoreCase(app.textBundle.getString("variance_of")) ||
		instString.equalsIgnoreCase(app.textBundle.getString("standard_deviation_of"))) {
		sb.append(" ( ");
		sb.append(instStringToCommandString(instString)); 
		sb.append(breedString);
		if (onWith) {
		    sb.append(app.commandsBundle.getString("_with"));
		    sb.append(" ( ");
		    sb.append(withText);
		    sb.append(" ) ");
		}
		sb.append(" [ ");
		sb.append(varString);
		sb.append(" ] ) ");
		return sb.toString();
	    }
	    
	    if (instString.equalsIgnoreCase(app.textBundle.getString("globals")) ||
		instString.equalsIgnoreCase(app.textBundle.getString("sliders"))) {
		sb.append(varString);
		return sb.toString();
	    }
	    
	    if (instString.equalsIgnoreCase(app.textBundle.getString("histogram"))) {
		if (!xyOn) {
		    sb.append(" ");
		    sb.append(app.commandsBundle.getString("ask_"));
		    sb.append(breedString).append(" [ ");
		    if (onWith) {
			sb.append(app.commandsBundle.getString("if"));
			sb.append(" ( ").append(withText).append(") [ ");
		    }
		    sb.append(app.commandsBundle.getString("%ptolemy_plot"));
		    sb.append(" ").append(order).append(" ");
		    sb.append(id).append(" ( ").append(varString).append(" ) ] ");
		    if (onWith) sb.append("]");
		} else if (xVal) {
		    sb.append("(").append(varString).append(")");
		} else {
		    sb.append(" ");
		    sb.append(app.commandsBundle.getString("ask_"));
		    sb.append(breedString).append(" [ ");
		    if (onWith) {
			sb.append(app.commandsBundle.getString("if"));
			sb.append(" (").append(withText).append(") [ ");
		    }
		    sb.append(app.commandsBundle.getString("%ptolemy_plotxy"));
		    sb.append(" ").append(order).append(" ");
		    sb.append(id).append(" ( ").append(varString).append(" ) ");
		    //if (onWith) sb.append("] ");
		}
		return sb.toString();
	    }//end if(instString.equals("histogram")
	}//if onExtra
	return "";
    }
    
    public String instStringToCommandString(String instString) {
	if (instString.equalsIgnoreCase(app.textBundle.getString("average_of"))) {
	    return app.commandsBundle.getString("average_of_");
	} 
	if (instString.equalsIgnoreCase(app.textBundle.getString("min_of"))) {
	    return app.commandsBundle.getString("min_of_");
	} 
	if (instString.equalsIgnoreCase(app.textBundle.getString("max_of"))) {
	    return app.commandsBundle.getString("max_of_");
	} 
	if (instString.equalsIgnoreCase(app.textBundle.getString("median_of"))) {
	    return app.commandsBundle.getString("median_of_");
	} 
	if (instString.equalsIgnoreCase(app.textBundle.getString("mode_of"))) {
	    return app.commandsBundle.getString("mode_of_");
	} 
	if (instString.equalsIgnoreCase(app.textBundle.getString("variance_of"))) {
	    return app.commandsBundle.getString("variance_of_");
	} 
	if (instString.equalsIgnoreCase(app.textBundle.getString("standard_deviation_of"))) {
	    return app.commandsBundle.getString("sdev_of_");
	} 
	return null;
    }

    public static String[] getBreeds() {
 	String[] b = new String[Turtle.getNumBreeds()];
	for (int i=0; i<b.length; i++) {
	    b[i] = Turtle.getBreedName(i);
	}
	return b;
    }

    public static String[] getTurtleVars(Application app) {
	String[] t = new String[Turtle.getNumVars()+5];
	//		System.out.println("num: " + t.length);
	t[0] = app.commandsBundle.getString("xcor");
	t[1] = app.commandsBundle.getString("ycor");
	t[2] = app.commandsBundle.getString("heading");
	t[3] = app.commandsBundle.getString("shape");
	t[4] = app.commandsBundle.getString("color");
	for (int i=0; i < Turtle.getNumVars(); i++) {
	    t[i+5] = Turtle.getVarName(i);
	}
	return t;
    }

    public static String[] getPatchVars(Application app) {
	String[] p = new String[Patch.getNumVars()+3];
	p[0] = app.commandsBundle.getString("xcor");
	p[1] = app.commandsBundle.getString("ycor");
	p[2] = app.commandsBundle.getString("patchcolor");
	for (int i=0; i < Patch.getNumVars(); i++) {
	    p[i+3] = Patch.getVarName(i);
	}
	return p;
    }

    public static String[] getObserverVars() {
	String[] o = new String[Observer.getNumVars()];
	for (int i=0; i<o.length; i++) {
	    o[i] = Observer.getVarName(i);
	}
	return o;
    }

    public static ExtraInfo load(Application app, int id, int order, 
				 ArrayList items, int start, int end) {
	return load(app, id, order, items, "", start, end);
    }

    public static ExtraInfo load(Application app, int id, int order, 
				 ArrayList items, String append, int start, int end) {
	int i = start;
	String extrainfo = "extrainfo" + (append.equals("") ? "" : ("-" + append));
	boolean onp = false, withp = false, xyp = false, xvalp = false;
	String instruction = "", breed = "", var = "", withText = "", 
	    kind = "";
	//System.out.println("extrainfo source: " + YoYo.printToString(items.subList(start, end)));
	while (i < end) {
	    String item = items.get(i).toString();
	    // System.out.println("reading item: " + item);
	    if (item.equalsIgnoreCase(extrainfo + "-extra?")) {
		onp = items.get(i+1).toString().equalsIgnoreCase("true");
		i+=2;
		continue;
	    }
	    if (item.equalsIgnoreCase(extrainfo + "-instruction")) {
		instruction = items.get(i+1).toString();
		i+=2;
		continue;
	    }
	    if (item.equalsIgnoreCase(extrainfo + "-breed")) {
		breed = items.get(i+1).toString();
		i+=2;
		continue;
	    }
	    if (item.equalsIgnoreCase(extrainfo + "-var")) {
		var = items.get(i+1).toString();
		i+=2;
		continue;
	    }
	    if (item.equalsIgnoreCase(extrainfo + "-kind")) {
		kind = items.get(i+1).toString();
		i+=2;
		continue;
	    }
	    if (item.equalsIgnoreCase(extrainfo + "-with?")) {
		withp = items.get(i+1).toString().equalsIgnoreCase("true");
		i+=2;
		continue;
	    }
	    if (item.equalsIgnoreCase(extrainfo + "-with-text")) {
		withText = items.get(i+1).toString();
		i+=2;
		continue;
	    }
	    if (item.equalsIgnoreCase(extrainfo + "-xy?")) {
		xyp = items.get(i+1).toString().equalsIgnoreCase("true");
		i+=2;
		continue;
	    }
	    if (item.equalsIgnoreCase(extrainfo + "-xval?")) {
		xvalp = items.get(i+1).toString().equalsIgnoreCase("true");
		i+=2;
		continue;
	    }
	    i += 2;
	}
// 	System.out.println("extra info onp: " + onp + " instruction: " + 
// 			   instruction + " breed " + breed + " var " + var + 
// 			   " kind " + kind + " withp " + withp + 
// 			   " withText " + " xyp " + xyp + " xvalp " +
// 			   xvalp);
	//System.out.println("kind is " + kind);

	if (onp) {
	    return new ExtraInfo(app, id, order, true, instruction, breed, 
				 var, kind, withp, withText, xyp, xvalp); 
	} else {
	    return new ExtraInfo(app, id, order, false);
	}
    }   

    public String getSaveString() {
	return getSaveString("");
    }

    public String getSaveString(String append) {
	String extrainfo = "extrainfo" + (append.equals("") ? "" : ("-" + append));
	String s;
	if (onExtra) {
	    s = extrainfo + "-extra? true\r\n";
	    s += extrainfo + "-instruction \"" + Utilities.printable(instString) + "\"\r\n";
	    s += extrainfo + "-breed \"" + Utilities.printable(breedString) + "\"\r\n";
	    s += extrainfo + "-var \"" + Utilities.printable(varString) + "\"\r\n";
	    s += extrainfo + "-kind \"" + Utilities.printable(kindString) + "\"\r\n";
	    if (onWith) {
		s += extrainfo + "-with? true\r\n" +
		    extrainfo + "-with-text \"" + Utilities.printable(withText) + "\"\r\n";
	    } else {
		s += extrainfo + "-with false\r\n";
	    }
	    s += extrainfo + "-xy? " + ((xyOn) ? "true" : "false") + "\r\n";
	    s += extrainfo + "-xval? " + ((xVal) ? "true" : "false") + "\r\n";
	}
	else {
	    s = extrainfo + "-extra false\r\n";
	}
	return s;
    }

    public String toString() {
	return getSaveString();
    }

    ExtraInfo copy(Application app, int id, int order) {
        ExtraInfo c = new ExtraInfo(app, id, order);
        c.onExtra=onExtra;
        c.instString = instString;
        c.breedString = breedString;
        c.varString = varString;
	c.kindString = kindString;
        c.onWith = onWith;
        c.withText = withText;
        c.onNumber = onNumber;
        c.xyOn = xyOn;
        c.xVal = xVal;
        return c;
    }

}
