/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.awt.*;
import java.awt.event.*;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;
import yoyo.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.text.html.*;
import java.net.URL;

public class FAQDialog extends JSLDialog {
    
    private Application app;
    private JButton bok;
    private JTextArea text;
    private boolean oked;
    //private HTMLDocument htmlDoc; 

    public int OK_BUTTON = 0;

    
    public FAQDialog(Application app, Frame parent) {
	this(app, parent, app.textBundle.getString("faq"), 
	     true, false, false, 0);
    }
	    
    public FAQDialog(Application app, Frame parent, String title) {
	this(app, parent, title, true, true, true, 0);
    }
    
    public FAQDialog(Application app, Frame parent, String title, 
		     boolean showOK, boolean modal, 
		     boolean shortCutsOn, int focusPref) {
	super(parent, title, modal);
	this.app = app;
		    
	String contents = loadFAQ(app);

	setupComponents(contents, showOK);
	setupGBC(showOK);
	setupPosition();
	initOKCancel(bok, null);

	initFocus(bok);
	if (shortCutsOn) initShortcuts();
    }
	
    public static String loadFAQ(Application app) {
	try {
	    String name = app.textBundle.getString("faq_file");
	    URL  u =  FAQDialog.class.getResource(name);
	    return (String)PrimFile.prim_urltostring(u, app.sl.c);
	} catch (Exception e) {
	    app.err.println("FAQ didn't load properly.");
	    e.printStackTrace(app.err);
	}
	return app.textBundle.getString("error_loading_faq");
    }

    
    private void setupComponents(String contents, boolean showOK) {
	if (showOK) {
	    bok = new JButton(app.textBundle.getString("ok_label"));
	    if (!YoYo.macosxp) formatLabel(bok);
	}
	text = new JTextArea(contents, 30, 20);
	text.setEditable(false);
	text.setBackground(Color.white);
	text.setForeground(Color.white);
	colorText(text);
	text.setLineWrap(true);
	text.setWrapStyleWord(true);
    }
	
    private void setupGBC(boolean showOK) {
	GridBagLayout gbl = new GridBagLayout();
	getContentPane().setLayout(gbl);
	
	GridBagConstraints gbc = new GridBagConstraints();
	gbc.weighty = 100;
	gbc.weightx = 100;
	
	gbc.fill = GridBagConstraints.BOTH;
	gbc.anchor = GridBagConstraints.WEST;
	gbc.insets = new Insets(13, 13, 0, 13);
	gbc.gridx=0;
	gbc.gridy=0;
	gbc.gridwidth=2;
	gbc.gridheight=1;
	JScrollPane jsp = new JScrollPane(text, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
					  JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
	jsp.setBackground(Color.white);
	jsp.setForeground(Color.white);
	getContentPane().add(jsp, gbc);
			
	Graphics g=parent.getGraphics();
	FontMetrics fm=g.getFontMetrics();
	gbc.ipadx = (fm.stringWidth(app.textBundle.getString("cancel_label")) -
		     fm.stringWidth(app.textBundle.getString("ok_label")))/2;
	g.dispose();
	gbc.fill = GridBagConstraints.NONE;
	gbc.anchor = GridBagConstraints.CENTER;
	gbc.insets = new Insets(0, 13, 0, 13);
	gbc.weighty=0;
	gbc.gridx=1;
	gbc.gridy=1;
	getContentPane().add(bok, gbc);
	getRootPane().setDefaultButton(bok);
	
    }
	
    private void setupPosition() {
	//	setBackground(Color.white);
	//setForeground(Color.white);
	setSize(600,340);
	initLocation();
    }
	
    public JTextArea getTextArea() {
	return text;
    }

    public void setTextAreaForStream(TEStream tes) {
    	tes.setTextArea(text);
    }

    public void ok() {
	oked = true;
	super.ok();
	//super.ok();
    }

    public void kill() {
	setModal(false);
	dispose();
    }

    public boolean oked() {
	return oked;
    }

    public void addNotify() {
	super.addNotify();
	if (YoYo.macosxp) {
	    setBackground(UIManager.getColor("window"));
	} else {
	    setBackground(Color.white);
	}
    }

    public static void colorControl(Component c) {
	if (YoYo.macosxp) {
	    c.setBackground(UIManager.getColor("window"));
	} else {
	    c.setBackground(Color.white);
	}
	c.setForeground(Color.white);
    }

    static MessageDialog errorDialog;

    public static void displayMessage(Application app, StarLogoFrame f, String title, 
				      String message, boolean modalp) {
	errorDialog = new JMessageDialog(app, f, title, message);
	errorDialog.setModal((YoYo.unixp) ? false : modalp);
	SwingUtilities.invokeLater(new Runnable() {
		public void run() {
		    errorDialog.show();
		}
	    });
    }
}
