/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import yoyo.Context;
import yoyo.LogoError;
import yoyo.YoYoCaller;
import yoyo.YoYoObject;
import yoyo.ThrowReturn;
import yoyo.Symbol;
import yoyo.CompoundSymbol;
import yoyo.YoYo; 

public class FastPatchStackProcess extends StackProcess {

  static Pool pool;
  
  static {
   	try {
   		pool = new Pool(FastPatchStackProcess.class);
    } catch (Exception e) { e.printStackTrace(); }
   }
   
  public FastPatchStackProcess(Application app) {
    super(app);
    ilists = new int[1];
    ilistptr = 0;
  }

    public static FastPatchStackProcess create(Application app, int whichip, int ip, StackProcessParent spp, 
			       boolean outputsp) {
    FastPatchStackProcess fpsp = 
	(FastPatchStackProcess)StackProcess.create(app, whichip, ip, spp, null, 
						 null, null, null, outputsp, pool);
    return fpsp;
  }

    public static FastPatchStackProcess create(Application app, int whichip, int ip, StackProcessParent spp) {
	return create(app, whichip, ip, spp, false);
  }

    public static FastPatchStackProcess create(Application app, StackProcess process, int whichip, int ip, 
			       StackProcessParent spp) {
	FastPatchStackProcess fpsp = create(app, whichip, ip, spp, false);
    fpsp.callerentity = process.entity;
    if (process.ifp > 0) {
    int i = process.fp;
    int nargs = process.istack[process.ifp - 5]; // nargs
    i -= nargs;
    while(i < process.sp) {
      fpsp.ostack[fpsp.sp++] = process.ostack[i++];
    }
      fpsp.fp = nargs;
    }
    return fpsp;
  }

    public static FastPatchStackProcess create(Application app, int whichip, StackProcessParent spp) {
	return create(app, whichip, 0, spp, false);
  }

    public static FastPatchStackProcess create(Application app, int whichip, StackProcessParent spp, int bytecode) {
    FastPatchStackProcess fpsp = create(app, whichip, 0, spp, false);
    fpsp.istack[fpsp.isp++] = bytecode;
    return fpsp;
  }

  public static FastPatchStackProcess create(Application app, int whichip, int ip, StackProcessParent spp, int bytecode) {
    FastPatchStackProcess fpsp = create(app, whichip, ip, spp, false);
    fpsp.istack[fpsp.isp++] = bytecode;
    return fpsp;
  }

  public static FastPatchStackProcess create(Application app, StackProcess process, int whichip, int ip, 
			       StackProcessParent spp, int bytecode) {
    FastPatchStackProcess fpsp = create(app, whichip, ip, spp, false);
    fpsp.istack[fpsp.isp++] = bytecode;
    fpsp.callerentity = process.entity;
    if (process.ifp > 0) {
    int i = process.fp;
    int nargs = process.istack[process.ifp - 5]; // nargs
    i -= nargs;
    while(i < process.sp) {
      fpsp.ostack[fpsp.sp++] = process.ostack[i++];
    }
      fpsp.fp = nargs;
    }
    return fpsp;
  }

  public static FastPatchStackProcess create(Application app, StackProcess process, int whichip, int ip, 
			       StackProcessParent spp, boolean outputsp) {
    FastPatchStackProcess fpsp = create(app, whichip, ip, spp, outputsp);
    fpsp.callerentity = process.entity;
    if (process.ifp > 0) {
    int i = process.fp;
    int nargs = process.istack[process.ifp - 5]; // nargs
    i -= nargs;
    while(i < process.sp) {
      fpsp.ostack[fpsp.sp++] = process.ostack[i++];
    }
    fpsp.fp = nargs;
    }
    return fpsp;
  }

  public static FastPatchStackProcess create(Application app, int whichip, int ip, StackProcessParent spp, int bytecode,
			       boolean outputsp) {
    FastPatchStackProcess fpsp = create(app, whichip, ip, spp, outputsp);
    fpsp.istack[fpsp.isp++] = bytecode;
    return fpsp;
  }

  public static FastPatchStackProcess create(Application app, StackProcess process, int whichip, int ip, 
			       StackProcessParent spp, int bytecode,
			       boolean outputsp) {
    FastPatchStackProcess fpsp = create(app, whichip, ip, spp, outputsp);
    fpsp.istack[fpsp.isp++] = bytecode;
    fpsp.callerentity = process.entity;
    if (process.ifp > 0) {
    int i = process.fp;
    int nargs = process.istack[process.ifp - 5]; // nargs
    i -= nargs;
    while(i < process.sp) {
      fpsp.ostack[fpsp.sp++] = process.ostack[i++];
    }
      fpsp.fp = nargs;
    }
    return fpsp;
  }



  public void switchme(Context c) {
    int oldhere = here, oldip = ip, oldsp = sp, oldfp = fp, 
      oldisp = isp, oldifp = ifp, oldwhichip = whichip,
      oldcurrentwhichip = currentwhichip;
    int[] oldistack = istack, oldiinst = iinst;
    Object[] oldostack = ostack, oldoinst = oinst;
    long oldtimer = timer;
    StarLogo sl = spp.sl;
    Application app = sl.app;
	
    for(int y = 1; y<sl.height+1; y++) {
      int col = y*sl.widthp2;
      for(int x = 1; x<sl.width+1; x++) {
	Patch p = sl.patches[col + x];
	p.here = here;
	p.ip = ip;
	p.sp = sp;
	p.isp = isp;
	p.istack = istack;
	p.ostack = ostack;
	p.iinst = iinst;
	p.process = this;
	    
	try {
	  while (!donep) {
	    p.dispatch(c, null, null, p);
	  }
	  fp = oldfp;
	  ifp = oldifp;
	  whichip = oldwhichip;
	  timer = oldtimer;
	  whichip = oldcurrentwhichip;
	}
	catch (ThrowReturn tr) {
	  Object[] ilist = { cserror, tr.errortype, tr.value, this };
	  app.cc.enqueue(ilist, app.message, app.err, this);
	  sl.stopAllStarLogo();
	}
	catch (LogoError e) {
	  YoYoObject yob = new YoYoObject();
	  yob.defineValue(Symbol.lookup("instruction"), new Integer(entity.here));
	  yob.defineValue(Symbol.lookup("message"), e.getMessage());
	  yob.defineValue(Symbol.lookup("turtle?"), YoYo.symfalse);
	  yob.defineValue(Symbol.lookup("observer?"), YoYo.symfalse);
	  yob.defineValue(Symbol.lookup("patch?"), YoYo.symtrue);
	  yob.defineValue(Symbol.lookup("xcor"), new Double(x - 1 - sl.halfwidth));
	  yob.defineValue(Symbol.lookup("ycor"), new Double(sl.halfheight - (y - 1)));
	  Object[] ilist = { cserror, "vm-logo-error", yob, this };
	  app.cc.enqueue(ilist, app.message, app.err, this);
	  sl.stopAllStarLogo();
	}
	finally { donep = false; }
      }
    }
    sl.done(this);
  }

  transient int[] ilists;
  transient int ilistptr;
  transient int count = 0;
  transient int removals = 0;
  
  public void addilist(int index) {
    if (ilistptr == ilists.length) {
      int[] newilists = new int[ilists.length * 2];
      System.arraycopy(ilists, 0, newilists, 0, ilists.length);
      ilists = newilists;
    }
    ilists[ilistptr++] = index;
    count++;
  }
	
  public int getindexandremove(StackProcess[] sparray, StackProcess sp) {
    for(int i = 0; i<ilistptr; i++) {
      int index = ilists[i];
      if (index >= 0) {
	if (sparray[index] == sp) {
	  ilists[i] = -1;
	  count--;
	  removals++;
	  return index;
	}
      }
    }
    return -1;
  }
	
  public void removeyourilist(int index) {
    ilists[index] = -1;
    count--;
    removals++;
  }    
	
  public void removeilist(int index) {
    for(int i = 0; i<ilistptr; i++) {
      if (ilists[i] == index) {
	ilists[i] = -1;
	count--;
	break;
      }
    }
  }
	
  public void changeindex(int src, int dst) {
    //System.out.println("found hole. swapping src: " + src + " with dst: " + dst); 
    for(int i = 0; i<ilistptr; i++) {
      if (ilists[i] == src) {
	ilists[i] = dst;
	break;
      }
    }
  }
	
  public int numilists() {
    return count;
  }
	
  public void compact() {
    int latesthole = 0;
    for(int i = ilistptr - 1; i>= 0; i--) {
    loop:
      if (ilists[i] >= 0) {
	for(int j = latesthole; j<i; j++) {
	  if (ilists[j] < 0) {
	    ilists[j] = ilists[i];
	    ilists[i] = -1;
	    latesthole = j;
	    break loop;
	  }
	}
	ilistptr = count;
	break;
      }
    }
    removals = 0;
  }
	
  public void done() {
    donep = true;
    spp.done(this);
    pool.dropStackProcess(this);
  }


}
