/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.awt.event.*;
import java.awt.*;
import java.text.*;
import java.awt.font.*;
import java.awt.geom.*;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import yoyo.*;
import java.util.Enumeration;
import java.util.Observable;
import java.util.Observer;
import java.util.ArrayList;
import java.awt.datatransfer.*;
import java.io.LineNumberReader;
import java.io.PrintWriter;
import java.io.StringReader;
import java.io.IOException;
import java.io.OutputStream;
import javax.swing.*;
import javax.swing.undo.*;
import javax.swing.event.*;

public class InfoWindow extends JFrame implements MenuListener, ClipboardOwner, StarLogoFrame, SLPrintable {
    
    public JTextArea ta;
    PrintWriter pw;
    
    Clipboard clip;
    SLMenu slmenu;
    Application app;
    StarLogo sl;
    
    private static Font INFO_FONT;
    
    static {
    	try {
	    INFO_FONT = new Font("Monospaced", Font.PLAIN, 12);
	    if (YoYo.macintoshp) INFO_FONT = Font.getFont("starlogo.info.font", INFO_FONT);
    	} catch (Exception e) { e.printStackTrace(); }
	}
    UndoManager undoManager = new UndoManager() {
	    public void undoableEditHappened(UndoableEditEvent e) {
		super.undoableEditHappened(e);
		if (canUndo()) {
		    slmenu.setEditMenuState(app.textBundle.getString("edit_menu_undo"), true);
		} else slmenu.setEditMenuState(app.textBundle.getString("edit_menu_undo"), false);
		if (canRedo()) {
		    slmenu.setEditMenuState(app.textBundle.getString("edit_menu_redo"), true);
		} else slmenu.setEditMenuState(app.textBundle.getString("edit_menu_redo"), false);
	    }
	    public void redo() {
		super.redo();
		if (canUndo()) {
		    slmenu.setEditMenuState(app.textBundle.getString("edit_menu_undo"), true);
		} else slmenu.setEditMenuState(app.textBundle.getString("edit_menu_undo"), false);
		if (canRedo()) {
		    slmenu.setEditMenuState(app.textBundle.getString("edit_menu_redo"), true);
		} else slmenu.setEditMenuState(app.textBundle.getString("edit_menu_redo"), false);
	    }
	    public void undo() {
		super.undo();
		if (canUndo()) {
		    slmenu.setEditMenuState(app.textBundle.getString("edit_menu_undo"), true);
	} else slmenu.setEditMenuState(app.textBundle.getString("edit_menu_undo"), false);
		if (canRedo()) {
		    slmenu.setEditMenuState(app.textBundle.getString("edit_menu_redo"), true);
		} else slmenu.setEditMenuState(app.textBundle.getString("edit_menu_redo"), false);
	    }
	};
    
    public InfoWindow(Application app) {
	this(app, app.textBundle.getString("info_window_title"));
	if (!YoYo.macintoshp) {
	    javax.swing.ImageIcon icon = new javax.swing.ImageIcon(Pool.class.getResource("gifs/icon.gif"));
	    setIconImage(icon.getImage());
	}
    }
    
    public InfoWindow(Application app, String title) {
	super(title);
	
	this.app = app;
	this.sl = app.sl;
	clip = app.clip;
	// setup listeners by registering with <app>
	app.registerFrameTracker(this);
	//app.registerAutoCompiler(this);
    }
    
    public void init() {
	
	// setup frame's appearance 
	setLocation(0, 0);
	setSize(500,400);
	
	// setup components
	setupText();
	
	// do layout
	setupLayout();
	
	slmenu = new JSLMenu(this, app);
	    
	slmenu.setMenuBar(this);
	addNotify();
	
	// setup listeners
	setupKeyTrackers();
    }
    
    public void setupText() {
	//ta = new TextArea("", 10, 10, (this instanceof OutputWindow) ? 
	//TextArea.SCROLLBARS_BOTH : TextArea.SCROLLBARS_VERTICAL_ONLY);
	ta = new JTextArea("", 10, 10);
	ta.setLineWrap(true);
	ta.setWrapStyleWord(true);
	ta.setCaretPosition(0);
	ta.getDocument().addUndoableEditListener(undoManager);
	JScrollPane scrollPane = new 
	    JScrollPane(ta, 
			(YoYo.macintoshp ? JScrollPane.VERTICAL_SCROLLBAR_ALWAYS
			 : JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED), 
			JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
	ta.setFont(INFO_FONT);
	JSLDialog.colorText(ta);
	pw = new PrintWriter((new JTEStream(ta)), true);
	getContentPane().add(scrollPane, BorderLayout.CENTER);
    }
    
    void setupKeyTrackers() {
	ControlKeyAdapter cka = new ControlKeyAdapter(this);
	ta.addKeyListener(cka);
	ProcMouseAdapter cma = new ProcMouseAdapter(/*ta*/);
	ta.addMouseListener(cma);
	addKeyListener(cka);
    }
    
    public void setupLayout() {
	//getContentPane().add("Center", ta);
    }
    
    public String getSaveString() {
	return "\r\n`information`\r\n" + 
	    toExternalLinefeeds(ta.getText());
    }
    
    
    public String getContents() {
	return toExternalLinefeeds(ta.getText());
    }

    public void undo() {
	if (undoManager.canUndo()) undoManager.undo();
    }
    
    public void redo() {
	if (undoManager.canRedo()) undoManager.redo();
    }
    
    class ControlKeyAdapter extends KeyAdapter {
	Frame parent;
	
	ControlKeyAdapter(Frame parent) {
	    this.parent = parent;
	}
	
	public void keyPressed(KeyEvent e) {
	    //System.out.println("key pressed " + new Integer(e.getKeyChar()));
	    char key = e.getKeyChar();
	    if (YoYo.macintoshp && e.isMetaDown() && key == '.') { 
		synchronized(sl.runningp) {
		    sl.stopAllStarLogo();
		}
	    }			
	}
	
	public void keyTyped(KeyEvent e) {
	    //System.out.println("key typed " + new Integer(e.getKeyChar()));
	    app.project.markChanged();
	    char c = e.getKeyChar();
	    //  if (c=='\t' && e.getModifiers() == 0) {
	    // 	pw.print("   ");
	    // 	pw.flush();
	    // 	e.consume();
	    //       }
	}
	
	public void keyReleased(KeyEvent e) {
	    //System.out.println("key released " + new Integer(e.getKeyChar()));
	    char c = e.getKeyChar();
	    //if (c == '\t') e.consume();
	    
	    /*Component c = e.getComponent();*/
	    /*if (c instanceof TextComponent)
	      updateLine((TextComponent)e.getComponent());
	    */
	}
    }
    
    
    public void itemStateChanged(ItemEvent evt) {
	Object o = evt.getSource();
	String arg = "";
	boolean selected = false;
	if (o instanceof JRadioButtonMenuItem) {
	    JRadioButtonMenuItem cmi = (JRadioButtonMenuItem)o;
	    arg = cmi.getLabel();
	    selected = cmi.isSelected();
	} else if (o instanceof CheckboxMenuItem) {
	    CheckboxMenuItem cmi = (CheckboxMenuItem)o;
	    arg = cmi.getLabel();
	    selected = cmi.getState();
	}
	
	try {
	    int value = Integer.parseInt(arg);
	    Font current = ta.getFont();
	    ta.setFont(new Font(current.getName(), current.getStyle(), value));
	    slmenu.setCurrentSize(o);
	    return;
	}
	catch (Exception e) {
	    Font current = ta.getFont();
	    ta.setFont(new Font(arg, current.getStyle(), current.getSize()));
	    slmenu.setCurrentFont(o);
	    return;
	}
    }
    
    public void actionPerformed(ActionEvent e) {
	Object o = e.getSource();
	String arg = "";
	if (o instanceof JMenuItem) {
	    JMenuItem mi = (JMenuItem)o;
	    arg = mi.getLabel();
	} else if (o instanceof MenuItem) {
	    MenuItem mi = (MenuItem)o;
	    arg = mi.getLabel();
	}
	if (arg.equals(app.textBundle.getString("edit_menu_cut"))) {
	    String foo = ta.getSelectedText();
	    if(foo.equals("")) return;
	    StringSelection bar = new StringSelection(foo);
	    clip.setContents(bar, this);
	    int start = ta.getSelectionStart();
	    ta.replaceRange("", start, ta.getSelectionEnd());    
	    ta.select(start, start);
	    app.project.markChanged();
	    return;
	}
	if (arg.equals(app.textBundle.getString("edit_menu_copy"))) {
	    String foo = ta.getSelectedText();
	    if(foo.equals("")) return;
	    StringSelection bar = new StringSelection(foo);
	    clip.setContents(bar, this);      
	    return;
	}
	if (arg.equals(app.textBundle.getString("edit_menu_paste"))) {
	    Transferable trans = clip.getContents(this);
	    if (trans != null) {
		try {
		    Object thing = trans.getTransferData(DataFlavor.stringFlavor);
		    if (thing instanceof String) {
			String s = (String)thing;
			int len = s.length();
			int cur = ta.getCaretPosition();
			if (len == 0) {
			    ta.insert(s, cur);
			    ta.select(cur + len, cur + len);
			}
			else { 
			    int start = ta.getSelectionStart();
			    ta.replaceRange(s, start, ta.getSelectionEnd());      
			    ta.select(start + len, start + len);
			}
			app.project.markChanged();
		    }
		}
		catch (IOException ioe) {}
		catch (UnsupportedFlavorException ufe) {}
	    }
	    return;
	}
	if (arg.equals(app.textBundle.getString("edit_menu_select_all"))) {
	    ta.selectAll();
	    return;
	}
	if (arg.equals(app.textBundle.getString("edit_menu_clear"))) {
	    ta.setText("");
	    app.project.markChanged();
	}
	if (arg.equals(app.textBundle.getString("edit_menu_undo"))){
	    undo();
	    return;
	}
	if (arg.equals(app.textBundle.getString("edit_menu_redo"))) {
	    redo();
	    return;
	}
    }
    
    public void lostOwnership(Clipboard clip, Transferable t) {}
    
    
    public String toInternalLinefeeds(String input) {
	StringBuffer out = new StringBuffer(input.length());
	for(int i = 0; i < input.length(); i++) {
	    char c = input.charAt(i);
	    if (c == '\r') continue;
	    out.append(c);
	}
	return out.toString();
    }

    public String toExternalLinefeeds(String input) {
	StringBuffer out = new StringBuffer(input.length());
	for(int i = 0; i < input.length(); i++) {
	    char c = input.charAt(i);
	    if (c == '\n') { 
		out.append('\r'); out.append('\n'); 
	    }
	    else out.append(c);
	}
	return out.toString();
    }
    
    public void reset() {
	SwingUtilities.invokeLater(new Runnable() {
		public void run() {
		    ta.setText("");
		    ta.setCaretPosition(0);
		}
	    });
    }
    
    public void resetText(String contents) {
	String title = "`information`";
	int start = contents.indexOf(title);
	if (start == -1) { ta.setText(""); return; }
	int end = contents.indexOf("`", start+title.length());
	if (end == -1) end = contents.length();
	ta.setText(toInternalLinefeeds(contents.substring(contents.indexOf(title)+title.length(), end).trim()));
	ta.setCaretPosition(0);
    }
    
    
    public void print() {
	Print print = new Print(this, getWindowName());
	print.print();
    }

    public int print(Graphics graphics, PageFormat pageFormat, int pageIndex) {
	//System.out.println("print page " + pageIndex);
	Graphics2D g2d = (Graphics2D) graphics;
	//--- Translate the origin to 0,0 for the top left corner
	g2d.translate (pageFormat.getImageableX (), pageFormat.getImageableY ());
	//--- Set the drawing color to black
	g2d.setPaint (Color.black);
	
	double maxHeight = pageFormat.getImageableHeight();
	double maxWidth = pageFormat.getImageableWidth();

	boolean printedTitle = false;

	Print.Printed printed = new Print.Printed(0, pageIndex);

	printed = Print.printText(g2d, getWindowName(),
			    printed.endPoints, printed.pagesToGo, INFO_FONT, 
			    maxWidth, maxHeight);
	printed = Print.printText(g2d, ta.getText(), printed.endPoints + 12, 
			    printed.pagesToGo, INFO_FONT, maxWidth, maxHeight);

	if (printed.pagesToGo > 0) return Printable.NO_SUCH_PAGE;
	if (printed.pagesToGo == 0 && 
	    printed.endPoints == 0) return Printable.NO_SUCH_PAGE;
	return Printable.PAGE_EXISTS;
    }

    
    public String getWindowName() {
	return app.textBundle.getString("info_window_title");
    }
    
    public void markChanged() {    }
    public void clearChanged() {    }
}
