/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.awt.*;
import java.awt.event.*;
import yoyo.YoYo;
import javax.swing.*;
import javax.swing.text.*;
import info.clearthought.layout.TableLayout;

final class JButtonDialog extends JWidgetDialog implements ButtonDialog, FocusListener {
	
	private static JButtonDialog dialog;
    
	private ButtonGroup commandGroup, nameGroup;
	private JRadioButton turCheck, obsCheck, instCheck, nameCheck;
	private JCheckBox foreverCheck;
	private JLabel nameLab, instLab;
	private JPanel commandPanel, namePanel, butPanel, namefieldPanel, instructionfieldPanel;
	private JTextField commandText, nameText;
	//	private JScrollPane descriptScroll;
	private JLabel descriptLab;
	private JTextField descriptText;
	private String descript;
	private JLabel keyLab;
	private JTextField keyText;

	private JButtonDialog(Frame parent, String title, Application app) {
		super(parent, app, title);
		init();
		initFocus(commandText);
		descriptText.addFocusListener(this);
		pack();
	}
	
	// used for new buttons, requires <wd> instanceof ButtonWidget
	public static JButtonDialog create(Frame parent, String title, Application app) {
		if (dialog==null) {
	    dialog = new JButtonDialog(parent, title, app);
		} else {
	    dialog.setTitle(title);
		}
		dialog.addNotify();
		dialog.setFields("", app.textBundle.getString("button") + ButtonWidget.idCount, 
				 true, false, false, "","");
		return dialog;
	}
	
	// used for editing buttons, requires <wd> instanceof ButtonWidget
	public static JButtonDialog create(Frame parent, String title, WidgetDrag wd, Application app) {
		ButtonWidget bw = (ButtonWidget)wd;
		if (dialog==null) {
	    dialog = new JButtonDialog(parent, title, app);
		} else {
	    dialog.setTitle(title);
		}
		dialog.addNotify();
		dialog.setFields(bw.command, bw.name, bw.turtlep, 
				 bw.forever, bw.showName, bw.getToolText(), bw.keyboard);
		return dialog;
	}
	
	private void setFields(String command, String name, boolean turtlep,
			       boolean forever, boolean showName, String tooltip, String keyboard) {
		commandText.setText(command);
		commandText.setSelectionStart(0);
		commandText.setSelectionEnd(command.length());
		nameText.setText(name);
		nameText.setCaretPosition(0);
		keyText.setText(keyboard);
		keyText.setSelectionStart(0);
		keyText.setSelectionEnd(command.length());
		descriptText.setText(tooltip);
		descriptText.setCaretPosition(0);
		if (turtlep) {
	    turCheck.setSelected(true);
		} else {
	    obsCheck.setSelected(true);
		}
		foreverCheck.setSelected(forever);
		if (showName) {
	    nameCheck.setSelected(true);
		}
		else {
	    instCheck.setSelected(true);
		}
	}
	
	protected void setupComponents() {
		nameLab = new JLabel(app.textBundle.getString("name_label"));
		instLab = new JLabel(app.textBundle.getString("instruction_label"));
		keyLab = new JLabel(app.textBundle.getString("keyboard_label"));
		keyText = new JTextField(1);
		AbstractDocument doc = (AbstractDocument)keyText.getDocument();
	    doc.setDocumentFilter(new FixedSizeFilter(1));
		nameText = new JTextField(15);
		commandText = new JTextField(30);
	    
		descriptLab = new JLabel(app.textBundle.getString("tooltip_label"));
		descriptText = new JTextField();	    
		//descriptText.setLineWrap(true);
		//descriptText.setWrapStyleWord(true);

		//descriptScroll = new JScrollPane(descriptText);
		//descriptScroll.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		//descriptScroll.setPreferredSize(new Dimension((YoYo.macosxp) ? 30 : 50,50));
		//namefieldPanel = new JPanel(new GridLayout(2, 1, 5, 0));
		//instructionfieldPanel = new JPanel(new GridLayout(2, 1, 5, 0));
		
		//namefieldPanel.add(nameLab);
		//namefieldPanel.add(nameText);
		
		//instructionfieldPanel.add(instLab);
		//instructionfieldPanel.add(commandText);
		
		foreverCheck = new JCheckBox(app.textBundle.getString("forever_check_label"));
		
		butPanel = new JPanel(new GridLayout(1,2,5,0));
		if (YoYo.pcp) {
			butPanel.add(bok);
			butPanel.add(bca);
		} else {
			butPanel.add(bca);
			butPanel.add(bok);
		}
		
		commandGroup = new ButtonGroup();
		commandPanel = new JPanel(new GridLayout(2, 1, 5, 0));
		turCheck = new JRadioButton(app.textBundle.getString("turtle_check_label"), true);
		obsCheck = new JRadioButton(app.textBundle.getString("observer_check_label"), false);
		commandGroup.add(turCheck);
		commandGroup.add(obsCheck);
		commandPanel.add(turCheck);
		commandPanel.add(obsCheck);
		
		nameGroup = new ButtonGroup();
		namePanel = new JPanel(new GridLayout(2, 1, 5, 0));
		instCheck = new JRadioButton(app.textBundle.getString("instruction_check_label"), true);
		nameCheck = new JRadioButton(app.textBundle.getString("name_check_label"), false);
		nameGroup.add(instCheck);
		nameGroup.add(nameCheck);
		namePanel.add(nameCheck);
		namePanel.add(instCheck);
	
		// abstract this!
		if (!YoYo.macosxp) {
		    formatText(commandText);
		    formatText(nameText);
		    formatText(keyText);
		    formatLabel(nameLab);
		    formatLabel(instLab);
		    formatLabel(keyLab);
		    //formatLabel(namefieldPanel);
		    //formatLabel(instructionfieldPanel);
		    formatLabel(commandPanel);
		    formatLabel(turCheck);
		    formatLabel(obsCheck);
		    formatLabel(namePanel);
		    formatLabel(foreverCheck);
		    formatLabel(instCheck);
		    formatLabel(nameCheck);
		    formatLabel(butPanel);
		    formatLabel(bca);
		    formatLabel(bok);
		    
		    formatText(descriptText);
		    formatLabel(descriptLab);
		}
	}
	
	protected void setupGBC() {

		GridBagLayout gbl = new GridBagLayout();
		getContentPane().setLayout(gbl);
		GridBagConstraints gbc = new GridBagConstraints();

		gbc.weightx = 100;
		gbc.weighty = 100;
		gbc.gridx=0;
		gbc.gridy=0;
		gbc.gridwidth=1;
		gbc.gridheight=1;
		gbc.weighty = 0;
		gbc.anchor = GridBagConstraints.NORTHWEST;
		gbc.fill = GridBagConstraints.NONE;
		gbc.insets = new Insets(6, 13, 0, 0);
		getContentPane().add(nameLab, gbc);
		
		gbc.gridy = 1;
		gbc.gridwidth = 2;
		gbc.weighty = 20;
		gbc.anchor = GridBagConstraints.WEST;
		gbc.fill = GridBagConstraints.NONE;
		gbc.ipady = 0;
		gbc.insets = new Insets(0, 13, 0, 0);
		getContentPane().add(nameText, gbc);
		
		gbc.gridy = 3;
		gbc.gridwidth = 1;
		gbc.weighty = 0; 
		gbc.ipady = 0;
		gbc.anchor = GridBagConstraints.WEST;
		gbc.fill = GridBagConstraints.NONE;
		gbc.insets = new Insets(6, 13, 0, 0);
		getContentPane().add(instLab, gbc);
		
		gbc.gridy = 4;
		gbc.gridwidth = 3;
		gbc.weighty = 20;
		gbc.ipady = 0;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.insets = new Insets(0, 13, 0, 13);
		getContentPane().add(commandText, gbc);
		
		gbc.ipady = 0;
		gbc.gridy  = 6;
		gbc.gridx = 0;
		gbc.weighty = 0;
		gbc.gridwidth = 1;
		gbc.anchor = GridBagConstraints.WEST;
		gbc.fill = GridBagConstraints.NONE;
		gbc.insets = new Insets(6, 13, 6, 0);
		getContentPane().add(descriptLab, gbc);

		gbc.gridx = 0;
		gbc.gridy = 7;
		gbc.gridwidth = 3;
		gbc.weighty = 20;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.anchor = GridBagConstraints.WEST;
		gbc.insets = new Insets(0, 13, 0, 13);
		getContentPane().add(descriptText, gbc);		

		gbc.weighty = 20;
		gbc.gridheight = 2;
		gbc.gridy = 0;
		gbc.gridx = 3;
		gbc.gridwidth = 1; 
		gbc.ipady = 0;
		gbc.fill = GridBagConstraints.VERTICAL;
		gbc.anchor = GridBagConstraints.SOUTHWEST;
		gbc.insets = new Insets((YoYo.macintoshp) ? 13 : 6, 0, 0, 13);
		getContentPane().add(namePanel, gbc);
		
		gbc.gridy = 3;
		gbc.gridx = 3;
		gbc.insets = new Insets((YoYo.macintoshp) ? 13 : 6, 0, 0, 13);
		getContentPane().add(commandPanel, gbc);

		gbc.gridx = 0;
		gbc.gridy = 8;
		gbc.gridwidth = 1;
		gbc.gridheight = 1;
		gbc.anchor = GridBagConstraints.SOUTHWEST;
		gbc.insets = new Insets(6, 13, 0, 13);
		getContentPane().add(foreverCheck, gbc);

		gbc.gridx = 1;
		gbc.gridy = 8;
		gbc.weighty = 0;
		gbc.gridwidth = 3;
		gbc.anchor = GridBagConstraints.SOUTHWEST;
		gbc.fill = GridBagConstraints.NONE;
		gbc.insets = new Insets(13, 13, 13, 13);
		getContentPane().add(butPanel, gbc);
		
		gbc.ipady = 0;
		gbc.gridy  = 6;
		gbc.gridx = 3;
		gbc.weighty = 20;
		gbc.gridwidth = 1;
		gbc.anchor = GridBagConstraints.WEST;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.insets = new Insets(6, 6, 6, 0);
		getContentPane().add(keyLab, gbc);

		gbc.gridx = 3;
		gbc.gridy = 7;
		gbc.gridwidth = 1;
		gbc.weighty = 0;
		gbc.fill = GridBagConstraints.NONE;
		gbc.anchor = GridBagConstraints.WEST;
		gbc.insets = new Insets(0, 6, 0, 13);
		getContentPane().add(keyText, gbc);	
	}


	protected void setupPosition() {
		//setSize(400, (YoYo.linuxp) ? 190 : (YoYo.pcp) ? 180 : 170);
		//setSize(400, (YoYo.linuxp) ? 270 : (YoYo.pcp) ? 260 : 250);
		pack();
		initLocation();
	}
	
	protected boolean valid_inputs() {
		String name = nameText.getText().trim();
		String com = commandText.getText().trim();
		if (name.equals("")) {
		    JMessageDialog m = new JMessageDialog(app, (StarLogoFrame)parent, app.textBundle.getString("warning_label"),
							  app.textBundle.getString("must_specify_name"));
	    m.show();
	    return false;
		} else if (com.equals("")) {
		    JMessageDialog m = new JMessageDialog(app, (StarLogoFrame)parent, app.textBundle.getString("warning_label"),
							  app.textBundle.getString("sorry_specify_instruction"));
	    m.show();
	    return false;
		}
		return true;
	}
    
	/**
	 *  @requires <wd> instanceof ButtonWidget
	 */
	public void getValues(WidgetDrag wd) {
		ButtonWidget bw = (ButtonWidget)wd;
		bw.command = commandText.getText();
		bw.name = nameText.getText();
		bw.forever = foreverCheck.isSelected();
		bw.showName = nameCheck.isSelected();
		bw.font = Widget.WIDGET_FONT;
		bw.turtlep = !getObserverp();
		bw.keyboard = keyText.getText();
	
		if (!(ToolTipWidget.empty(descriptText.getText())))
	    bw.setToolTip(new ToolTipWidget(bw.parent, descriptText.getText()));
		else bw.removeToolTip();
	}
    
	private boolean getObserverp() {
		return obsCheck.isSelected();
	}
    
	public boolean shortcuts = true;
    
	public void focusGained(FocusEvent e) {
		shortcuts = false;
	}
    
	public void focusLost(FocusEvent e) {
		shortcuts = true;
	}
    
	public boolean shortcutsOn() { return shortcuts; }
}
