/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.lang.Math;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import info.clearthought.layout.TableLayout;
import yoyo.YoYo;

public class JChoiceDialog extends JSLDialog implements ActionListener, StarLogoFrame, ChoiceDialog {
	
	private JButton bok, bca, bnot, bfocus;
	private int numChoices;
	private final String chRET, chESC;
	private JLabel promptLab;
	private JPanel butPanel;
	private String result = null;

	// puts focus on default return, choiceRET
	public JChoiceDialog(Frame parent, String title, String prompt, String ok, String cancel, String not) {
	    super(parent, title, true);
	    this.chRET=ok;
	    this.chESC=cancel;
	    setupComponents(prompt, ok, cancel, not);
	    setupGBC();
	    setupPosition();
	    
	    initOKCancel(bok, bca);
	    initFocus(bok);
	    initShortcuts();
	}
	
	private void setupComponents(String prompt, String ok, String cancel, String not) {
	    bok = new JButton(ok);
	    if (!YoYo.macosxp) formatLabel(bok);
	    bok.addActionListener(this);
	    bca = new JButton(cancel);
	    if (!YoYo.macosxp) formatLabel(bca);
	    bca.addActionListener(this);
	    bnot = new JButton(not);
	    if (!YoYo.macosxp) formatLabel(bnot);
	    bnot.addActionListener(this);

	    promptLab = new JLabel("<html>" + prompt + "</html>");
	    if (!YoYo.macosxp) formatLabel(promptLab);
	}
	
	private void setupGBC() {

	    int borderWidth = (YoYo.macosxp) ? 20 : dluToPixelsHorizontal(7);
	    int borderHeight = (YoYo.macosxp) ? 20 : dluToPixelsVertical(7);
	    int buttonWidth = (YoYo.macosxp) ? 75 : dluToPixelsHorizontal(50);
	    int buttonHeight = (YoYo.macosxp) ? 22 : dluToPixelsVertical(14);
	    int textHeight = (YoYo.macosxp) ? 20 : dluToPixelsVertical(8);
	    int lineSepHeight = dluToPixelsVertical(2);
	    
	    double[][] size = {{ borderWidth, (YoYo.macosxp) ? buttonWidth + 30 : -2.0, 40.0, (YoYo.macosxp) ? buttonWidth: -2.0, (YoYo.macosxp) ? 12.0 : 4.0, (YoYo.macosxp) ? buttonWidth : -2.0, borderWidth }, 
			       { borderHeight, -1.0, (YoYo.macosxp) ? 10 : borderHeight, -2.0, borderHeight }};
	    LayoutManager layout = new TableLayout(size);
	    getContentPane().setLayout(layout);
	    
	    promptLab.setAlignmentY(promptLab.TOP_ALIGNMENT);
	    if (YoYo.macosxp) promptLab.setFont(UIManager.getFont("Panel.font"));
	    if (!YoYo.macosxp) promptLab.setPreferredSize(new Dimension(getPreferredSize().width, textHeight * 2 + lineSepHeight ));
	    getContentPane().add(promptLab, "1, 1, 5, 1");
	    
	    bok.setMinimumSize(new Dimension(buttonWidth, buttonHeight));
	    //if (YoYo.macosxp) bok.setSize(new Dimension(buttonWidth, buttonHeight));
	    if (YoYo.macosxp) bok.setFont(UIManager.getFont("Button.font"));
	    getContentPane().add((YoYo.pcp) ? bca : bok, "5, 3, f, f");
	    
	    bca.setMinimumSize(new Dimension(buttonWidth, buttonHeight));
	    //if (YoYo.macosxp) bca.setSize(new Dimension(buttonWidth, buttonHeight));
	    if (YoYo.macosxp) bca.setFont(UIManager.getFont("Button.font"));
	    getContentPane().add((YoYo.pcp) ? bok: bca, "3, 3, f, f");
	    
	    bnot.setMinimumSize(new Dimension(buttonWidth + 40, buttonHeight));
	    //if (YoYo.macosxp) bnot.setSize(new Dimension(buttonWidth + 40, buttonHeight));
	    if (YoYo.macosxp) bnot.setFont(UIManager.getFont("Button.font"));
	    getContentPane().add(bnot, "1, 3, f, f");
	    
	    /*
		GridBagLayout gbl = new GridBagLayout();
		getContentPane().setLayout(gbl);
		
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.anchor = GridBagConstraints.CENTER;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.weightx = 100;
		gbc.weighty = 0;
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.gridwidth = 1;
		gbc.gridheight =1;
		gbc.insets = new Insets(13, 13, 13, 13);
		getContentPane().add(promptLab, gbc);
		
		gbc.weighty = 100;
		gbc.weightx = 50;
		gbc.fill = GridBagConstraints.NONE;
		gbc.gridy = 1;
		gbc.insets = new Insets(0, 13, 13, 13);
		getContentPane().add(butPanel, gbc);
	    */
	}	
	
	private void setupPosition() {
	addNotify();
	
	if (YoYo.macosxp) {
	setSize(340, 130);
	} else if (YoYo.linuxp){
	setSize(300,130);
	//System.out.println("linux from setupPosition");
	} else if (YoYo.macintoshp) {
	setSize(300, 100);
	} else if (YoYo.solarisp) {
	setSize(300, 150);
	} else {
 	pack();
 	}
	initLocation();
	
	}
	
	protected void ok() {
		result = chRET;
		setVisible(false);
	}
	
	protected void cancel() {
		result = chESC;
		setVisible(false);
	}
	
	public void actionPerformed(ActionEvent e) {
		Object source = e.getSource();
		if (source instanceof JButton) {
			result = ((JButton)source).getLabel();
			setVisible(false);
		}
	}
	
	public String getResult() {
		show();
		return result;
	}

    public void markChanged() {    }
    public void clearChanged() {    }
}
