/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.awt.*; 
import java.awt.event.*;
import yoyo.YoYo;
import javax.swing.*;
import javax.swing.border.EmptyBorder;
import javax.swing.event.MouseInputAdapter.*;

public class JColorLegendDialog extends JWidgetDialog implements FocusListener, ColorLegendDialog{
    private static JColorLegendDialog dialog;
    private JLabel  shapeLab;
    private JPanel butJPanel,colorJPanel;
    private String name;
    public  static SColorChooser colorchooser;
    public SColortool colortool;
    JShapeChooser shapechoose;
    JMiniShapeChooser shapechooserpanel;
    JShapeChooserStatusPanel shapestatuspanel;
    private JCheckBox borderCheck;
    private JTextField nameText;
    private JPanel nameJPanel, shapeJPanel, statusPanel;
    private JLabel nameLab, meansLab, colorLab;
    public JInterface iface;
    private JLabel descriptLab;
    private JTextField descriptText;
    private String descript;

    private JColorLegendDialog(Frame parent, Application app, String title) {
	super(parent, app, title);
	init();
    }

    public static JColorLegendDialog create(Frame parent, Application app, String title, WidgetDrag wd) {
	ColorLegendWidget legend = (ColorLegendWidget)wd;
	if (dialog==null) {
	    dialog = new JColorLegendDialog(parent, app, title);
	} else {
	    dialog.setTitle(title);
	}
	dialog.addNotify();
	dialog.setFields(legend.name, legend.shape, 5.0, legend.showBorder, legend.getToolText(), legend.color);

	return dialog;
    }


    // for new legends
    public static JColorLegendDialog create(Frame parent, Application app, String title) {
	String name = app.textBundle.getString("legend_label") + (ColorLegendWidget.idCount);

	if (dialog==null) {
	    dialog = new JColorLegendDialog(parent, app, title);
	} else {
	    dialog.setTitle(title);
	}
	dialog.addNotify();
	dialog.setFields(name, 0, 0.5, false, "" ,54);
	return dialog;
    }

    
    private void setFields(String name, int shape, double delay, boolean showBorder, String descript, int color) {
	if(color !=54)
	    {
		int bob = new Double(Math.ceil(Colors.mapColorToStarLogoColor(color))).intValue();
		colortool.setColor(bob);
		shapechooserpanel.tcolmask = Colors.colorarrayints[bob * 32];
	    }
	else{
	    colortool.setColor(color);
	    shapechooserpanel.tcolmask = Colors.colorarrayints[color * 32];
	}
	shapechooserpanel.select(shape); 
	shapechooserpanel.refresh(); 
	borderCheck.setSelected(showBorder);
	repaint();
	this.name = name;
	nameText.setText(name);
	nameText.setCaretPosition(0);
	descriptText.setText(descript);
	descriptText.setCaretPosition(0);
    }


    public void setupComponents() {
	//this.app = app;
	borderCheck = new JCheckBox(app.textBundle.getString("legend_border_check"));
	shapechoose = JShapeChooser.create(app);
	shapechooserpanel = new JMiniShapeChooser(shapechoose, app);
	shapestatuspanel = shapechoose.statusPanel; 
	colortool = new SColortool(this);
	colorchooser = new SColorChooser(colortool, app.iface);
	nameLab = new JLabel(app.textBundle.getString("legend_shape_label"));
	meansLab = new JLabel(app.textBundle.getString("legend_definition_label"));
	colorLab = new JLabel(app.textBundle.getString("legend_color_label"));
	nameText = new JTextField(15);
	nameJPanel = new JPanel(new GridLayout(1,2,20,0));
	nameJPanel.add(nameText);

	colorJPanel = new JPanel(new GridLayout(1,2));
	colorJPanel.add(colortool); 
	colorJPanel.add(colorchooser);
	colortool.addMouseListener(new MouseAdapter() {
		public void mousePressed(MouseEvent e){ 
		    shapechooserpanel.tcolmask = Colors.colorarrayints[colortool.curcolor * 32];
		    shapechooserpanel.refresh(); 
		}  
		public void mouseReleased(MouseEvent e){ 
		    shapechooserpanel.tcolmask = Colors.colorarrayints[colortool.curcolor * 32];
		    shapechooserpanel.refresh(); 
		}
	    });
	colortool.addMouseMotionListener(new MouseMotionAdapter() {
		public void mouseDragged(MouseEvent e) {
		    shapechooserpanel.tcolmask = Colors.colorarrayints[colortool.curcolor * 32];
		    shapechooserpanel.refresh(); 
		}
	    });

	shapeJPanel = new JPanel(new GridLayout(1,1));	    
	shapeJPanel.add(shapechooserpanel);
	statusPanel = new JPanel(new GridLayout(1,1));
	statusPanel.add(shapestatuspanel);
	butJPanel = new JPanel();
	if (YoYo.macosxp) butJPanel.setBorder(new EmptyBorder(0, 0, 3, 0));
	//butJPanel.setLayout(new GridLayout(1, 2, (YoYo.macintoshp) ? 12 : 7, 0));
	butJPanel.setLayout(new FlowLayout((YoYo.pcp) ? FlowLayout.CENTER : FlowLayout.RIGHT,
					   (YoYo.macintoshp) ? 12 : 7, 0));
	// these extra micropanels are added so the buttons draw correctly on mac
	//JPanel okPanel = new JPanel(new FlowLayout());
	//JPanel caPanel = new JPanel(new FlowLayout());
	//okPanel.add(bok);
	//caPanel.add(bca);
	if (YoYo.pcp) {
	    butJPanel.add(bok);
	    butJPanel.add(bca);
	    //butJPanel.add(okPanel);
	    //butJPanel.add(caPanel);
	} else {
	    butJPanel.add(bca);
	    butJPanel.add(bok);
	    //butJPanel.add(caPanel);
	    //butJPanel.add(okPanel);
	}
	
	if (!YoYo.macosxp) {
	    formatLabel(bca);
	    formatLabel(bok);
	}
	    
	descriptLab = new JLabel(app.textBundle.getString("tooltip_label"));
	
	descriptText = new JTextField(15); 
    }

    protected void setupPosition() {
	//setSize(450, 500);
 	//setSize(350, (YoYo.macintoshp) ? 190 : (YoYo.linuxp) ? 220 : 208);
	setSize(355, (YoYo.macosxp) ? 420 : (YoYo.macintoshp && !YoYo.macosxp) ? 390 : (YoYo.linuxp) ? 420 : 410);
 	initLocation();
    }
    
    public void getValues(WidgetDrag wd) {
	ColorLegendWidget legend = (ColorLegendWidget)wd;
	legend.shape = shapechooserpanel.selectedShape; 
	legend.name = nameText.getText();
	legend.showBorder = borderCheck.isSelected();
	if(colortool.curcolor != 9)
	    legend.color = Colors.colorarrayints[colortool.curcolor * 32];
	else{
	    legend.color = Colors.colorarrayints[(int)(8.5*32)];
	}
	legend.refresh();
	legend.font = Widget.WIDGET_FONT;
	legend.update();
    	if (!(ToolTipWidget.empty(descriptText.getText())))
	    legend.setToolTip(new ToolTipWidget(legend.parent, descriptText.getText()));
    	else legend.removeToolTip();
    }
 
    public boolean shortcuts = true;

    public void focusGained(FocusEvent e) {
	shortcuts = false;
    }
    protected boolean valid_inputs() {
	
	return true;
    }
    public void focusLost(FocusEvent e) {
	shortcuts = true;
    }
    
    protected void setupGBC() {
	Container contentPane = getContentPane();
	getContentPane().setLayout(null);
	contentPane.add(nameLab);
	contentPane.add(meansLab);
	contentPane.add(shapeJPanel);
	contentPane.add(statusPanel);
	contentPane.add(nameJPanel);
	contentPane.add(colorLab);
	contentPane.add(colorJPanel);
	contentPane.add(descriptLab);
	contentPane.add(descriptText);
	contentPane.add(butJPanel);
	contentPane.add(borderCheck);
	Insets insets = contentPane.getInsets();
	nameLab.setBounds(5 + insets.left, 10 + insets.top, 50, 20); 
	shapeJPanel.setBounds(35 + insets.left, nameLab.getY()+nameLab.getHeight()+2, 290, 74);
	statusPanel.setBounds(35 + insets.left, shapeJPanel.getY() + shapeJPanel.getHeight(), 160, 25); 
	colorLab.setBounds(5+insets.left, statusPanel.getY() + statusPanel.getHeight()+10, 50,20);
	colorJPanel.setBounds(35 + insets.left, colorLab.getY() + colorLab.getHeight() + 2, 450, 70); 
	meansLab.setBounds(5 + insets.left, colorJPanel.getY() + colorJPanel.getHeight(), 70, 20);
	nameJPanel.setBounds(35+insets.left, meansLab.getY() + meansLab.getHeight(), 280, 20);
	borderCheck.setBounds(50+ insets.left, nameJPanel.getY() + nameJPanel.getHeight()+2, (YoYo.macintoshp || YoYo.linuxp) ? 130 : 100, 20);	
	descriptLab.setBounds(5 + insets.left, borderCheck.getY() + borderCheck.getHeight()+10 , 90, 20);
	descriptText.setBounds(35 + insets.left, descriptLab.getY() + descriptLab.getHeight() + 2, 195, 20);
	butJPanel.setBounds(insets.left + ((YoYo.macosxp) ? 144 : 80), descriptText.getY() + descriptText.getHeight() + ((YoYo.macosxp) ? 8 : 2), 190, (YoYo.macosxp) ? 30 : 30);

	validate();
	pack();
	repaint();
	/*
	GridBagLayout gbl = new GridBagLayout();
	getContentPane().setLayout(gbl);
	Container contentPane = getContentPane();
	GridBagConstraints gbc = new GridBagConstraints();
	*/ 

    }
 
    public void initShortcuts(){}
    public boolean shortcutsOn() { return shortcuts; }
    public void setChooserColor(){
    }
    public void setChooserColor(int color){
    }
    public SColorChooser colorchooser() { return colorchooser; }
  
}
