/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

//import yoyo.YoYo;

/*
 * a dialog to select graph colors and styles and edit dataset labels
 */

import java.awt.*;
import java.util.*;
import java.awt.event.*;
import javax.swing.*;
import yoyo.YoYo;

public class JColortoolDialog extends JSLDialog implements ColortoolDialog {
    
    private Application app;
    public ColorSquare ct;
    public JInterface iface;
    public JButton bok,bca;
    public LineSet ls;
    public JPanel p1, p2,p3;
    public JPanel p;
    public int row=-1, col=-1;
    
    public ShapeChooser chooser;
    public boolean turtleColor;
    
    public SColortool sct;
    
    
    public JColortoolDialog(Application app, ShapeChooser chooser, boolean turtleColor) {
	this(app, (Frame)chooser, (turtleColor) 
	     ? app.textBundle.getString("turtle_color_chooser") 
	     : app.textBundle.getString("patch_color_chooser"));
	this.chooser = chooser;
	this.app = app;
	this.turtleColor = turtleColor;
    }

    public JColortoolDialog(Application app, Frame f, String title) {
	this(app, f, title, null, null);
    }
    
    public JColortoolDialog(Application app, Frame f, String title, JPanel p, LineSet ls) {
	super(f, title, true);
	sct = new SColortool(this);
	this.app = app;
	this.p = p;
	this.ls = ls;

	setupComponents();
	setupGBC();
	setupPosition();

	initOKCancel(bok, bca);
	initFocus(bok);
	initShortcuts();
    }

    public void setupComponents() {
	bok = new JButton(app.textBundle.getString("ok_label"));
	bca = new JButton(app.textBundle.getString("cancel_label"));
	if (!YoYo.macosxp) {
	    formatLabel(bok);
	    formatLabel(bca);
	}
    }

    public void setupGBC() {
	getContentPane().setLayout(new BorderLayout());
	p2 = new JPanel();
	p2.setLayout(new GridLayout(1, 2, 4, 4));
	p3 = new JPanel();
	if (YoYo.macintoshp) {
	    p2.add(bca);
	    p2.add(bok);
	} else {
	    p2.add(bok);
	    p2.add(bca);
	}
	p3.setLayout(new BorderLayout());
	p3.add(p2, BorderLayout.EAST);
	getContentPane().add(sct, BorderLayout.NORTH);
	getContentPane().add(p3, BorderLayout.SOUTH);

    }

    public void setupPosition() {
	sct.setSize(210,48);
	pack();
    }

    public void ok() {
	if (ls != null) {
	    ls.color = sct.passColor;
	    p.setBackground(sct.passColor);
	} else {
	    if (turtleColor) {
		chooser.setTurtleColor(sct.passColor);
	    } else {
		chooser.setPatchColor(sct.passColor);
	    }
	}
	super.ok();
    }
    
    
} //end class
