/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.awt.event.*;
import java.awt.*;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import javax.swing.*;
import yoyo.*;
import java.util.Enumeration;
import java.util.Observable;
import java.util.Observer;
import java.util.Hashtable;
import java.awt.datatransfer.*;
import java.io.LineNumberReader;
import java.io.PrintWriter;
import java.io.StringReader;
import java.io.IOException;
import javax.swing.text.JTextComponent;
import javax.swing.text.BadLocationException;
import javax.swing.UIManager;
import javax.swing.text.StyleConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.CaretListener;
import javax.swing.event.CaretEvent;
import javax.swing.event.UndoableEditListener;
import javax.swing.undo.UndoableEdit;
import javax.swing.event.UndoableEditEvent;
import javax.swing.undo.UndoManager;
import javax.swing.JCheckBoxMenuItem;

public class JControlCenter extends JFrame implements ControlCenter, ChangeListener, CaretListener, SLPrintable {

    public static Font STATUS_FONT;

    //Box[] topcard = new Box[2];
    JPanel[] topcard = new JPanel[2];
    JPanel[] bottomcard = new JPanel[2];
    public JSLCommand[] command = new JSLCommand[2];
    public JSLProc[] proc = new JSLProc[2];
    JLabel[] commandLab = new JLabel[2], procLab = new JLabel[2];
    JSplitPane[] drag = new JSplitPane[2];

    JPanel tools;
    JLabel status;
    // JButton cgBut, stopBut;

    JTabbedPane tabs;

    int tabHeight;
    int cur;

    JControlCenterTextArea textFocus;

    Insets insets;
    int width;
    static final int minTextHeight = 60;

    public static SLMenu slmenu;

    static {
  	try {
	    STATUS_FONT = JSLDialog.LABEL_FONT;
	    if (YoYo.macintoshp) STATUS_FONT = Font.getFont("starlogo.status-line.font", STATUS_FONT);
	} catch (Exception e) { e.printStackTrace(); }
    }

    protected Dimension myPreferredSize;

    public void setMyPreferredSize(Dimension size) {
	myPreferredSize = size;
    }

    public Dimension getMyPreferredSize() {
	return myPreferredSize;
    }

    public JControlCenter(Application app){
	// initializations
	super(app.textBundle.getString("windows_menu_control_center"));
	this.app = app;
	this.sl = app.sl;

	// setup listeners by registering with <app>
	app.registerCloser(this);
	setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
	app.registerFrameTracker(this);

	setLocation(0, 85);

	if(YoYo.language.getLanguage().equals("en")){
	    setMyPreferredSize(new Dimension(285,500));
	} else if (YoYo.language.getLanguage().equals("es")){
	    setMyPreferredSize(new Dimension(320,500));
	}

	if (!YoYo.macintoshp) {
	    javax.swing.ImageIcon icon = 
		new javax.swing.ImageIcon(Pool.class.getResource("gifs/icon.gif"));
	    setIconImage(icon.getImage());
	}

	// setup components
	setupText();
	setupLabels();
	setupTopBottoms();
	setupDrag();
	setupTabs();

	setupTools();

	// do layout
	setupLayout();

	init();
    }


    void setupText() {
	proc[tur] = new JSLProc(true, sl, this);
	proc[obs] = new JSLProc(false, sl, this);
	command[tur] = new JSLCommand(SLCommand.TURTLE, sl, this);
	command[obs] = new JSLCommand(SLCommand.OBSERVER, sl, this);
	setFont(JSLProc.PROC_FONT);
    }



    void setupLabels() {
	status = new JLabel() {
		public Dimension getPreferredSize() {
		    return new Dimension(super.getPreferredSize().width,
					 getFontMetrics(getFont()).getHeight() + LABEL_VERTICAL_SPACE);
		}
	    };
	JSLDialog.colorControl(status);
	status.setFont(STATUS_FONT);
    }

    void setupTopBottoms() {
	for(int i = 0; i < 2; i++) {
	    //topcard[i] = Box.createVerticalBox();
	    topcard[i] = new JPanel(new BorderLayout());
	    //JPanel topbox = new JPanel(new GridLayout(1, 2));
	    JPanel topbox = new JPanel(new GridLayout(1, 1));
	    topbox.add(command[i].label);
	    topbox.setBackground(proc[i].label.getBackground());
	    topbox.setOpaque(true);
	    topcard[i].add(topbox, "North");
	    topcard[i].add(new JScrollPane(command[i],
					   JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER), BorderLayout.CENTER);
	    bottomcard[i] = new JPanel(new BorderLayout()); //Box.createVerticalBox();
	    JPanel bottombox = new JPanel(new GridLayout(1, 2));

	    bottombox.add(proc[i].label);
	    bottombox.add(proc[i].procMenuList());
	    bottombox.setBackground(proc[i].label.getBackground());
	    bottombox.setOpaque(true);
	    bottomcard[i].add(bottombox, "North");
	    bottomcard[i].add(new JScrollPane(proc[i],
					      JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED), "Center");
	}
    }

    void setupDrag() {
	for (int i=0; i<2; i++) {
	    drag[i] = new JSplitPane(JSplitPane.VERTICAL_SPLIT, true, topcard[i], bottomcard[i]);
	    if (YoYo.macintoshp) drag[i].setOneTouchExpandable(true);
	    drag[i].setDividerLocation(150);
	    //drag[i].setOpaque(true);
	    drag[i].setBackground(SystemColor.control); // command[i].label.getBackground());
	    drag[i].setPreferredSize(new Dimension(getMyPreferredSize().width, getMyPreferredSize().height / 2));
	}
    }

    void setupTabs() {
	tabs = new JTabbedPane();
	tabs.addTab(app.textBundle.getString("turtle_check_label"), drag[tur]);
	tabs.addTab(app.textBundle.getString("observer_check_label"), drag[obs]);
    }


    void setupTools() {
	tools = new JPanel(new GridLayout(1, 3));
	JSLDialog.colorControl(tools);

	setLine(1);
	tools.add(status);
    }


    void setupLayout() {
	//getContentPane().setLayout(null);
	getContentPane().add(tabs, "Center");
	tabs.setDoubleBuffered(true);
	tabs.setOpaque(true);
	getContentPane().add(tools, "South");
	tools.setDoubleBuffered(true);
	tools.setOpaque(true);
    }

    public void setObsProcs(Object[] procnames, Integer[] linenumbers) {
	proc[obs].setProcs(procnames, linenumbers);
    }

    public void setTurProcs(Object[] procnames, Integer[] linenumbers) {
	proc[tur].setProcs(procnames, linenumbers);
    }

    public void clearProcs() {
	proc[obs].clearUfuns();
	proc[tur].clearUfuns();
    }


    public boolean lookupUfun(String name, int where) {
	switch (where) {
	case obs: return proc[obs].lookupUfun(name);
	case tur: return proc[tur].lookupUfun(name);
	}
	return false;
    }

    public void setBounds(int x, int y, int width, int height) {
	super.setBounds(x, y, width, height);
	validate();
    }

    void setupKeyTrackers() {
	ControlKeyAdapter cka = new ControlKeyAdapter(this);
	//cgBut.addKeyListener(cka);
	//stopBut.addKeyListener(cka);
	addKeyListener(cka);
    }

    class ControlKeyAdapter extends KeyAdapter {

	JControlCenter parent;

	ControlKeyAdapter(JControlCenter parent) {
	    this.parent = parent;
	}

	public void keyPressed(KeyEvent e) {
	    char key = e.getKeyChar();
	    if (key == '\t' && e.isControlDown()) {
		// toggle between observer and turtle procs upon ctrl-tab
		parent.toggleTabs();
		e.consume();
		return;
	    }
	    if (YoYo.macintoshp && e.isMetaDown() && e.getKeyChar() == '.') {
		synchronized(parent.sl.runningp) {
		    parent.sl.stopAllStarLogo();
		}
		return;
	    }
	    if (!YoYo.macintoshp && e.isControlDown() && e.getKeyChar() == 'c') {
		synchronized(parent.sl.runningp) {
		    parent.sl.stopAllStarLogo();
		}
		return;
	    }
	}

    }


    public void caretUpdate(CaretEvent e) {
	updateLine(textFocus);
    }

    /**
     * Calculates the line number in the text area of the control center.
     */
    public void updateLine(JControlCenterTextArea tca) {
	// print the line number of the caret if nothing is selected, but the
	// beginning of the selection if something is selected (like MS Word)

 	if (tca instanceof JTextArea) {
	    try {
		JTextArea ta = (JTextArea)tca;
		int line = ta.getLineOfOffset(ta.getSelectionStart());
		//System.out.println("line = " + line);
		setLine(line + 1);
	    } catch (BadLocationException e) { 
		app.err.println("badlocationexception in updateLine");
	    }
	} else { setLine(-1); }
    }

    private void setLine(int line) {
	if (line >= 0) {
	    status.setText(app.textBundle.getString("line_label") +  line);
	} else { status.setText(""); }
    }



    void setupFocusTrackers() {
	tabs.addChangeListener(this);
	textFocus = command[tur];
	cur = tur;
    }

    public void toggleTabs() {
	tabs.setSelectedIndex(1 - cur);
    }

    public void stateChanged(ChangeEvent ce) {
  	//tabs.invalidate();
	int which = tabs.getSelectedIndex();
	if (textFocus instanceof JSLCommand) {
	    textFocus = command[which];
	} else textFocus = proc[which];
	updateLine(textFocus);
	cur = which;
    }

    void init() {
	slmenu = new JSLMenu(this, app);

	slmenu.setMenuBar(this);
	addNotify();
	setSize(getMyPreferredSize());
	// setup listeners
	setupKeyTrackers();
	setupFocusTrackers();
    }

    public JControlCenterTextArea getTextFocus() {
	return(textFocus);
    }

    public SLCommand getSLCommand(int whichone) {
	return command[whichone];
    }

    public SLProc getSLProc(int whichone) {
	return proc[whichone];
    }

    public int getCCHeight(int which) {
	return drag[which].getDividerLocation();
    }

    public void setCCHeight(int which, int height) {
	drag[which].setDividerLocation(height);
    }

    public void itemStateChanged(ItemEvent evt) {
	if(YoYo.macintoshp && !YoYo.macosxp) {
	    CheckboxMenuItem cmi = (CheckboxMenuItem)evt.getSource();
	    String arg = cmi.getLabel();

	    if (arg.equals(app.textBundle.getString("show_spaces_label"))) {
		char c = 0x25CF; //get a black dot from unicode
		SLProc active = getSLProc(cur);
		active.setShowSpaces(!active.showSpacesp());
		return;
	    }
        	
	    try {
        	int value = Integer.parseInt(arg);
        	Font current = 	getFont();
        	setFont(new Font(current.getName(), current.getStyle(), value));
        	slmenu.setCurrentSize(cmi);
        	return;
	    }
	    catch (Exception e) {
        	Font current = 	getFont();
        	setFont(new Font(arg, current.getStyle(), current.getSize()));
        	slmenu.setCurrentFont(cmi);
        	return;
	    }
	}
	else {
	    if (evt.getSource() instanceof JCheckBoxMenuItem) {
		JCheckBoxMenuItem jcmi = (JCheckBoxMenuItem)evt.getSource();
		String arg = jcmi.getLabel();

		if (arg.equals(app.textBundle.getString("show_spaces_label"))) {
		    char c = 0x25CF; //get a black dot from unicode
		    SLProc active = getSLProc(tabs.getSelectedIndex());
		    active.setShowSpaces(!active.showSpacesp());
		    return;
		}
	    }

	    if (evt.getSource() instanceof JRadioButtonMenuItem) {
		JRadioButtonMenuItem jcmi = (JRadioButtonMenuItem)evt.getSource();
		String arg = jcmi.getLabel();

		// on macosx, show spaces is a radio button menu item to get around a bug
		if (arg.equals(app.textBundle.getString("show_spaces_label"))) {
		    char c = 0x25CF; //get a black dot from unicode
		    SLProc active = getSLProc(tabs.getSelectedIndex());
		    active.setShowSpaces(!active.showSpacesp());
		    return;
		}

		try {
		    int value = Integer.parseInt(arg);
		    Font current = getFont();
		    setFont(new Font(current.getName(), current.getStyle(), value));
		    slmenu.setCurrentSize(jcmi);
		    return;
		}
		catch (Exception e) {
		    Font current = getFont();
		    setFont(new Font(arg, current.getStyle(), current.getSize()));
		    slmenu.setCurrentFont(jcmi);
		    return;
		}
	    }
	}
    }


    public void actionPerformed(ActionEvent e) {
	Object o = e.getSource();
	String arg = "";
	if (o instanceof JMenuItem) {
	    JMenuItem mi = (JMenuItem)o;
	    arg = mi.getLabel();
	} else if (o instanceof MenuItem) {
	    MenuItem mi = (MenuItem)o;
	    arg = mi.getLabel();
	}
	JControlCenterTextArea focus = getTextFocus();
	if (arg.equals(app.textBundle.getString("edit_menu_undo"))) {
	    focus.undo();
	    return;
	}
	if (arg.equals(app.textBundle.getString("edit_menu_redo"))) {
	    focus.redo();
	    return;
	}
	if (arg.equals(app.textBundle.getString("edit_menu_cut"))) {
	    ((JTextComponent)focus).cut();
	    /*
	      String foo = focus.getSelectedText();
	      if(foo == null || foo.equals("")) return;
	      StringSelection bar = new StringSelection(foo);
	      app.clip.setContents(bar, this);
	      int start = focus.getSelectionStart();
	      focus.replaceRange("", start, focus.getSelectionEnd());
	      focus.select(start, start);
	    */
	    if (focus instanceof JSLProc) {
		app.rancompilerp=false;
		app.rancompilerpForVariables = false;
		app.project.markChanged();
	    }
	    return;
	}
	if (arg.equals(app.textBundle.getString("edit_menu_copy"))) {
	    ((JTextComponent)focus).copy();
	    /*
	      String foo = focus.getSelectedText();
	      if(foo == null || foo.equals("")) return;
	      StringSelection bar = new StringSelection(foo);
	      app.clip.setContents(bar, this);
	    */
	    return;
	}
	if (arg.equals(app.textBundle.getString("edit_menu_paste"))) {
	    ((JTextComponent)focus).paste();
	    /*
	      Transferable trans = app.clip.getContents(this);
	      if (trans != null) {
	      try {
	      Object thing = trans.getTransferData(DataFlavor.stringFlavor);
	      if (thing instanceof String) {
	      String s = (String)thing;
	      int len = s.length();
	      int cur = focus.getCaretPosition();
	      if (len == 0) {
	      focus.insert(s, cur);
	      focus.select(cur + len, cur + len);
	      }
	      else {
	      int start = focus.getSelectionStart();
	      focus.replaceRange(s, start, focus.getSelectionEnd());
	      focus.select(start + len, start + len);
	      }
	      }
	    */
	    if (focus instanceof JSLProc) {
		app.rancompilerp=false;
		app.rancompilerpForVariables = false;
		app.project.markChanged();
	    }
	    //}
	    //catch (IOException e) {}
	    //catch (UnsupportedFlavorException e) {}
	    //}
	    return;
	}
	if (arg.equals(app.textBundle.getString("edit_menu_select_all"))) {
	    ((JTextComponent)focus).selectAll();
	    return;
	}
	if (arg.equals(app.textBundle.getString("edit_menu_clear"))) {
	    ((JTextComponent)focus).setText("");
	    if (focus instanceof JSLProc) {
		app.rancompilerp=false;
		app.rancompilerpForVariables = false;
		app.project.markChanged();
	    }
	}
    }


    public Application app;
    public StarLogo sl;
    ObservableInstance observed = new ObservableInstance();
    protected class ObservableInstance extends Observable {
	public void doChanged() {
	    setChanged();
	}
    }


    public void turtlePrint(String s) {
	getSLCommand(tur).print(s);
    }

    public void observerPrint(String s) {
	getSLCommand(obs).print(s);
    }

    public void turtlePrintln(String s) {
	getSLCommand(tur).println(s);
    }

    public void observerPrintln(String s) {
	getSLCommand(obs).println(s);
    }

    public void observerClearText() {
	getSLProc(obs).setText("");
    }

    public void turtleClearText() {
	getSLProc(tur).setText("");
    }

    public void observerClearCC() {
	getSLCommand(obs).callerClearText();
    }

    public void turtleClearCC() {
	getSLCommand(tur).callerClearText();
    }

    public void clearText() {
	observerClearText();
	turtleClearText();
    }

    public void reset() {
	deleteObservers();
	clearText();
	//compileText("");
    }


    public void resetText(String contents) {
	String turTitle = "`turtle`";
	String obsTitle = "`observer`";
	int turStart = contents.indexOf(turTitle) + turTitle.length();
	int turEnd = contents.indexOf("`", turStart);
	if (turEnd == -1)
	    turEnd = contents.length();

	int obsStart = contents.indexOf(obsTitle) + obsTitle.length();
	int obsEnd = contents.indexOf("`", obsStart);
	if (obsEnd == -1)
	    obsEnd = contents.length();

	String turtle = contents.substring(turStart, turEnd).trim();
	String observer = contents.substring(obsStart, obsEnd).trim();
	//System.out.println("turtle code is " + turtle);
	//System.out.println("observer code is " + observer);
	turt = processString(turtle, "\n");
	//System.out.println("setting text process string returned: "+ turt);
	//SwingUtilities.invokeLater(new Runnable() {
	//	    public void run() {
	getSLProc(tur).setText(turt);
	//	    }
	//	});

	obsv = processString(observer, "\n");
	//System.out.println("finished calling turtle settext");
	//SwingUtilities.invokeLater(new Runnable() {
	//	    public void run() {
	getSLProc(obs).setText(obsv);
	//	    }
	//	});
	//System.out.println("done setting text");

    }

    String turt, obsv;

    public String processString (String input, String lineending) {
	LineNumberReader lnr = new LineNumberReader(new StringReader(input));
	StringBuffer out = new StringBuffer();
	try {
	    do {
		String s = lnr.readLine();
		//System.out.println("s = " + s);
		if (s == null) {
		    //out.append('\n');
		    //System.out.println("s is null. returning " + out.toString());
		    return out.toString();
		}
		out.append(s);
		out.append(lineending);
	    } while (true);
	} catch (IOException e) { app.err.println("uh oh"); }
	return "Error reading file\n";
    }

    public void compileText(ProgressBar pb, boolean justVariables) {
	synchronized(sl.runningp) {
	    sl.stopAllStarLogo();
	}
	if (justVariables) {
	    Object[] compile = { Symbol.lookup("compiler"), Symbol.lookup("read-variables-code") };
	    Object[] ilist = { new CompoundSymbol(compile), getSLProc(obs).getText(),
			       getSLProc(tur).getText(), pb, this };
	    //System.out.println("enqueuing " + YoYo.printToString(ilist) + " to compile");
	    app.cc.enqueue(ilist, app.message, app.err, this);
	} else {
	    getSLProc(obs).clearUfuns();
	    getSLProc(tur).clearUfuns();
	    Object[] compile = { Symbol.lookup("compiler"), Symbol.lookup("read-code") };
	    Object[] ilist = { new CompoundSymbol(compile), getSLProc(obs).getText(),
			       getSLProc(tur).getText(), pb, this };
	    //System.out.println("enqueuing " + YoYo.printToString(ilist) + " to compile");
	    app.cc.enqueue(ilist, app.message, app.err, this);
	}
    }

    public String getSaveString() {
	return "`turtle`\r\n" +
	    processString(getSLProc(tur).getText().trim(), "\r\n") +
	    "\r\n\r\n`observer`\r\n" +
	    processString(getSLProc(obs).getText().trim(), "\r\n");
    }

    public void setFont(Font f) {
	super.setFont(f);
	if (getSLProc(obs)!=null) {
	    for (int i=0; i<2; i++) {
		getSLProc(i).setFont(f);
		getSLCommand(i).setFont(f);
	    }
	}
    }

    public void addObserver(Observer o) {
	observed.addObserver(o);
    }

    public void deleteObserver(Observer o) {
	observed.deleteObserver(o);
    }

    public void deleteObservers() {
	observed.deleteObservers();
    }

    public void done(Context c) {
	observed.doChanged();
    }

    public void displayOneString(String s) {
	app.createMessageDialog(app, app.activeFrame, app.textBundle.getString("error_label"), s, true);
    }

    public void displayAString(String s) {
	app.createMessageDialog(app, app.activeFrame, app.textBundle.getString("error_label"), s, true);
    }

    public void print() {
	Print print = new Print(this, app.textBundle.getString("windows_menu_control_center"));
	print.print();
    }

    public int print(Graphics graphics, PageFormat pageFormat, int pageIndex) {
	//System.out.println("print page " + pageIndex);
	Graphics2D g2d = (Graphics2D) graphics;
	//--- Translate the origin to 0,0 for the top left corner
	g2d.translate (pageFormat.getImageableX (), pageFormat.getImageableY ());
	//--- Set the drawing color to black
	g2d.setPaint (Color.black);
	
	double maxHeight = pageFormat.getImageableHeight();
	double maxWidth = pageFormat.getImageableWidth();

	boolean printedTitle = false;

	Print.Printed printed = new Print.Printed(0, pageIndex);

	printed = Print.printText(g2d, app.textBundle.getString("observer_procedures_label"),
			    printed.endPoints, printed.pagesToGo, JSLProc.PROC_FONT, 
			    maxWidth, maxHeight);
	printed = Print.printText(g2d, proc[obs].getText(), printed.endPoints + 12, 
			    printed.pagesToGo, JSLProc.PROC_FONT, maxWidth, maxHeight);

	printed = Print.printText(g2d, app.textBundle.getString("turtle_procedures_label"),
			    printed.endPoints + 24, printed.pagesToGo, JSLProc.PROC_FONT, 
			    maxWidth, maxHeight);
	printed = Print.printText(g2d, proc[tur].getText(), printed.endPoints + 12, 
			    printed.pagesToGo, JSLProc.PROC_FONT, maxWidth, maxHeight);

	if (printed.pagesToGo > 0) return Printable.NO_SUCH_PAGE;
	if (printed.pagesToGo == 0 && 
	    printed.endPoints == 0) return Printable.NO_SUCH_PAGE;
	return Printable.PAGE_EXISTS;
    }

    public void markChanged() {
	getRootPane().putClientProperty("windowModified", Boolean.TRUE);
    }

    public void clearChanged() {
	getRootPane().putClientProperty("windowModified", Boolean.FALSE);
    }

    public void flush() {
	getSLCommand(tur).flush();
	getSLCommand(obs).flush();
    }
}
