/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.util.Vector;
import java.util.Observable;
import java.util.Random;
import java.util.Enumeration;
import yoyo.*;
import java.util.Observer;
import java.awt.datatransfer.*;
import java.io.LineNumberReader;
import java.io.PrintWriter;
import java.io.StringReader;
import java.io.IOException;
import java.util.Hashtable;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.undo.*;
import javax.swing.border.LineBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.BevelBorder;
import java.awt.CardLayout;

abstract public class JEntityMonitor extends JFrame implements EntityMonitor, MenuListener {

    JPanel wholePanel, middleBox, bottomPanel;
    Box wholeBox;
    JScrollPane middlePane;
    CardLayout cl;

    protected Object lock = new Object();
  
    public static Font EM_FONT;
    public static Font LABEL_FONT;
    JSLCommand com;
    StarLogo sl;
    Application app;
    
    // static final String[] COLOR_STRING;
  
    public volatile boolean closep = true;
    double delay = 0.5;
    protected boolean runningp = false;
  
    SLMenu slmenu;
  
    Clipboard clip;
    Thread thisthr;
  
    int ccHeight = 54;
    int textHeight;
  
  
    boolean lastOwn = false;
    int lastField = -1;
    boolean fieldChanged = false;

    JLabel[] own_lbl;
    JTextField[] own_txt;
    Object[] ownval;

    static {
	try {
	    EM_FONT = Widget.WIDGET_FONT;
	    EM_FONT = Font.getFont("starlogo.entity-monitor.font", EM_FONT);
	    LABEL_FONT = Widget.WIDGET_FONT;
	    LABEL_FONT = Font.getFont("starlogo.entity-label.font", LABEL_FONT);
	} catch (Exception e) { e.printStackTrace(); }
    }


    public JEntityMonitor(String title, double delay, Application app)  {
	super(title);
	setResizable(true);
	this.delay = delay;
	this.app = app;
	this.sl = app.sl;
	this.clip = app.clip;
	registerCloser(this, app);
	app.registerFrameTracker(this);
	app.addWindow(this);
    }
  
    void setFields() {
	FontMetrics fm = app.iface.getToolkit().getFontMetrics(EM_FONT);
	textHeight = fm.getHeight() + fm.getAscent();
	Dimension d= new Dimension(67, textHeight);
	int tfheight = 0;

	setStateFields(d);
	setVariableFields(d);
    }

    abstract void setStateFields(Dimension d);
  
    void setVariableFields(Dimension d) {
 
	if (getNumVars() != 0) {
	    own_txt = new JTextField[getNumVars()];
	    own_lbl = new JLabel[getNumVars()];
	    ownval = new Object[getNumVars()];
      
	    for (int i=0; i < getNumVars(); i++) {
		own_lbl[i] = new JLabel(" " + getVarName(i));
		own_txt[i] = new JTextField(" ");
		own_txt[i].setName(getVarName(i));
		own_txt[i].setPreferredSize(d);
		JSLDialog.colorControl(own_lbl[i]);
		own_lbl[i].setFont(LABEL_FONT);
		formatText(own_txt[i]);
	    }
	}
    }

    public void formatText(JTextComponent tc) {
	tc.setFont(EM_FONT);
	tc.setBackground(Color.white);
	tc.setForeground(Color.black);
	if (app.kioskp || YoYo.applet != null) tc.setEditable(false); 
    }

    abstract int getVarPos(int which);
    abstract boolean getVarEnump(int which);
    abstract Object getVarValue(int varpos);

    void setOwnInit(int which) {
	int varpos = getVarPos(which);
	ownval[which] = getVarValue(varpos);
	if (ownval[which] instanceof String)
	    own_txt[which].setText("\"" + YoYo.printToString(ownval[which]) + "\"");
	else if (getVarEnump(which)) {
	    if (ownval[which] instanceof Number) {
		int index = ((Number)ownval[which]).intValue();
		if (index >= 1 && index <= VM.enumnames.length) {
		    own_txt[which].setText(VM.enumnames[index-1]);
		} else {
		    own_txt[which].setText(clipDigits(YoYo.printToString(ownval[which])));
		}
	    } else {
		own_txt[which].setText(clipDigits(YoYo.printToString(ownval[which])));
	    }
	} else {
	    own_txt[which].setText(clipDigits(YoYo.printToString(ownval[which])));
	}
	own_txt[which].setCaretPosition(own_txt[which].getText().length());
    }
  
    Vector whichToupdate = new Vector();
    
    void setOwn(int whicha) {
	int varpos = getVarPos(whicha);
	ownval[whicha] = getVarValue(varpos);
	whichToupdate.addElement(new Integer(whicha));
	SwingUtilities.invokeLater(new Runnable() {
		public void run() {
		    // copy whichToupdate into a local array so we don't 
		    // have lock it across the swing update.
		    int[] localupdates;
		    synchronized(whichToupdate) {
			int i = 0;
			localupdates = new int[whichToupdate.size()];
			for(Enumeration e = whichToupdate.elements(); e.hasMoreElements();) {
			    localupdates[i++] = ((Integer)e.nextElement()).intValue();
			}
			whichToupdate.removeAllElements();
		    }
		    if (!closep) {
			for(int i = 0; i < localupdates.length; i++) {
			    int which = localupdates[i];
			    if (ownval[which] instanceof String)
				own_txt[which].setText("\"" + YoYo.printToString(ownval[which]) + "\"");
			    else if (getVarEnump(which)) {
				if (ownval[which] instanceof Number) {
				    int index = ((Number)ownval[which]).intValue();
				    if (index >= 1 && index <= VM.enumnames.length) {
					own_txt[which].setText(VM.enumnames[index-1]);
				    } else {
					own_txt[which].setText(clipDigits(YoYo.printToString(ownval[which])));
					own_txt[which].selectAll();
				    }
				} else {
				    own_txt[which].setText(clipDigits(YoYo.printToString(ownval[which])));
				    own_txt[which].selectAll();
				}
			    } else {
				own_txt[which].setText(clipDigits(YoYo.printToString(ownval[which])));
				own_txt[which].selectAll();
			    }
			    // own_txt[which].setCaretPosition(own_txt[which].getText().length());
			}
		    }
		}
	    });
    }

    void startThread() {
	Thread thisthr = new Thread(this);
	thisthr.start();
    }
  
    String clipDigits(String str) {
	int digits = 3;
	int i = str.indexOf('.');
	int e = str.indexOf('E');
	//if the exponent is positive, cut to 3 decimals and display
	if ((e!=-1) && (str.charAt(e+1) != '-'))
	    {return (str.substring(0, i+digits+1) + str.substring(e, str.length()));}
	//if the exponent is negative, convert to normal string
	if ((e!=-1) && (str.charAt(e+1) == '-'))
	    {String zeros = "";
	    int myexp = Integer.parseInt(str.substring(e+2, str.length()));
	    for (int j=0; j<(myexp-1); j++) {zeros = zeros + "0";}
	    str = "0." + zeros + str.substring(0,i);
	    i = str.indexOf('.');
	    }
	//if no decimal, return the whole string
	if (i==-1)
	    return(str);
	//else figure out how to round
	else 
	    { 
		//if we already have <= the correct number of digits, just display them
		if (str.substring(i+1, str.length()).length() <= digits)
		    {str = str + "000";
		    return (str.substring(0, i+digits+1));}
		//otherwise, we have too many digits, so we need to round.
		else    	
		    {
			str = str + "000";	//make sure we have enough digits
			double tmp = Double.valueOf(str).doubleValue();
			int myfactor = 1;
			for (int j= 1; j<=digits; j++)
			    {myfactor = 10 * myfactor;} 
			tmp = tmp * myfactor;		//multiply by 10^(digits needed) so that we can use round
			int tmp_int = (int)Math.round(tmp);
			String zeros = "";		
			String dec = Integer.toString(tmp_int % myfactor);
			//if the result of the mod is not as many digits as the decimal is supposed to be, pad with zeros
			//so that 1.0008, digits = 3, gives 1.001, and 2.111, digits = 2, gives 2.11
			for (int j = dec.length(); j<digits; j++) {zeros = zeros + "0";}
			//System.out.println("tmp_int = " + tmp_int + " myfactor = " + myfactor + " zeros = " + zeros + " //tmp_int % myfactor = " + tmp_int % myfactor);
			str = (tmp_int / myfactor) + "." + zeros + dec;
			return(str); }
	    }
    }
	
    abstract public void setValues(int which);

    public String doubString (double num) {
	if ((num - (int)num) == 0)
	    return Integer.toString((int)num);
	else
	    return Double.toString((double)((int)(1000*num))/1000);
    }

    abstract public void initFrame();
  
    abstract public int getNumVars();

    abstract public JLabel[] getStateLabels();
    abstract public JComponent[] getStateTexts();
    abstract public int getNumStateVariables();

    JPanel topBox;

    void addMyComponents(int tfheight) {

	addTopBox(getNumStateVariables(), getStateLabels(), getStateTexts());
	addMiddleBox();
	addBottomBox();
	makeWholeBox(topBox);
    }    

    void addTopBox(int num, JLabel[] labels, JComponent[] texts) {
    
	topBox = new JPanel(new GridLayout(num, 2));
	for(int i = 0; i < num; i++) {
	    topBox.add(labels[i]);
	    topBox.add(texts[i]);
	}
	JSLDialog.colorControl(topBox);
    }
  
    void addMiddleBox() {
	middlePane = null;
    
	if (getNumVars() > 0) {
	    middleBox = new JPanel(new GridLayout(getNumVars(), 2));
      
	    for (int i=0; i < getNumVars(); i++) {
		middleBox.add(own_lbl[i]);
		middleBox.add(own_txt[i]);
	    }
	
	    middlePane = new JScrollPane(middleBox, 
					 JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, 
					 JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
	    JSLDialog.colorControl(middleBox);
	    JSLDialog.colorControl(middlePane);
	}
    }
  
    void addBottomBox() {
	Box bottomBox = Box.createVerticalBox();
	bottomBox.add(com.label);
	com.label.setForeground(Color.white);
	bottomBox.add(new JScrollPane(com,
				      JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, 
				      JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED)
	    {
		public Dimension getPreferredSize() {
		    return new Dimension(21, 80);
		}
	    });

	bottomPanel = new JPanel();
	bottomPanel.setLayout(new BorderLayout()); //Border(new EmptyBorder(-4,-4,-4,-4));
	bottomPanel.add(bottomBox);
	bottomPanel.setBackground(com.label.getBackground());
	bottomPanel.setOpaque(true);

    }
  
    void makeWholeBox(JPanel topBox) {
	wholeBox = Box.createVerticalBox();
	wholeBox.add(topBox);
	if (getNumVars() > 0) {
	    wholeBox.add(middlePane);
	}
	if (!app.kioskp && YoYo.applet == null) wholeBox.add(bottomPanel);
    
	wholePanel = new JPanel() {
		public Insets getInsets() {
		    return new Insets(4,4,4,4);
		}};
    
	//wholePanel.setLayout(new BorderLayout());
	//wholePanel.add(wholeBox);

	wholePanel.setBackground(SystemColor.control); //Colors.colorarray[(int)(tur.color * 32.0)]);
	wholePanel.setBorder(new LineBorder(Colors.colorarray[(int)(getEntityColor() * 32.0)], 4));
	wholePanel.setOpaque(true);
	wholePanel.setDoubleBuffered(true);
    }

    abstract public double getEntityColor();
    
    public  void updateOwn(boolean midboxp) {
	/* we shouldn't have a midbox now */
	if (getNumVars() == 0) {
	    if (!midboxp) return;
	    wholeBox.remove(middlePane);
	    //pack(); // apparently not appreciated by users
	    validate();
	    return;
	}
	/* we should have a mid box */
	/* if there wasn't one before, create it */
	if (!midboxp) {
	    middleBox = new JPanel(new GridLayout(getNumVars(), 2));
	    middlePane = new JScrollPane(middleBox, 
					 JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, 
					 JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
	    JSLDialog.colorControl(middleBox);
	    JSLDialog.colorControl(middlePane);
	} else {
	    /* get rid of all the old variables and resetup the layout manager */
	    middleBox.removeAll();
	    middleBox.setLayout(new GridLayout(getNumVars(), 2));
	}   
       
	FontMetrics fm = app.iface.getToolkit().getFontMetrics(EM_FONT);
	textHeight = fm.getHeight() + fm.getAscent();
	Dimension d= new Dimension(67, textHeight);
	int tfheight = 0;
	own_txt = new JTextField[getNumVars()];
	own_lbl = new JLabel[getNumVars()];
	ownval = new Object[getNumVars()];

	for (int i=0; i < getNumVars(); i++) {
	    own_lbl[i] = new JLabel(" " + getVarName(i));
	    own_txt[i] = new JTextField(" ");
	    own_txt[i].setName(getVarName(i));
	    own_txt[i].setPreferredSize(d);
	    own_txt[i].addKeyListener(tka);
	    own_txt[i].addFocusListener(tff);
	    formatText(own_txt[i]);
	    JSLDialog.colorControl(own_lbl[i]);
	    own_lbl[i].setFont(LABEL_FONT);
	    middleBox.add(own_lbl[i]);
	    middleBox.add(own_txt[i]);
	}
    
	if (!midboxp) {
	    /* if it wasn't there before, it needs to go before the bottomPanel */
	    if (!app.kioskp && YoYo.applet == null) wholeBox.remove(bottomPanel);
	    wholeBox.add(middlePane);
	    if (!app.kioskp && YoYo.applet == null) wholeBox.add(bottomPanel);
	}
	//pack();  // people seemed not to like the windows re-sizing on a compile
	validate();
    }

    abstract String getVarName(int which);

    TextKeyAdapter tka;
    tfFocus tff;

    void addListeners() {
	tka = new TextKeyAdapter();
	tff = new tfFocus();
	for (int i=0; i<getNumVars(); i++){
	    own_txt[i].addKeyListener(tka);
	    own_txt[i].addFocusListener(tff);
	    //own_txt[i].addWindowListener(tff);
	}
    
	com.addFocusListener(tff);
    }

    public void show() {
	super.show();
	closep = false;
    }

    public void registerCloser(Frame c, Application app) {
	c.addWindowListener(new Closer(app));
    }

    protected class Closer extends WindowAdapter {
	Application app;

	public Closer(Application app) {
	    this.app = app;
	}

	public void windowClosing(WindowEvent evt) {
	    synchronized (lock) {
		closep = true;
		evt.getWindow().setVisible(false);
		app.removeWindow((StarLogoFrame)evt.getWindow());
		closeMonitor();
	    }
	}
    }
  
    abstract void closeMonitor();
 
    public class tfFocus extends WindowAdapter implements FocusListener {
	//Component focus;
		
	public tfFocus() {
	    //this.focus = focus;
	}
		
	// SUN needs this one
	public void focusGained(FocusEvent e) {
	    if (e.getComponent() instanceof JTextComponent) {
		focus = (JTextComponent)e.getComponent();
	    } else focus = null;
	    //focus.requestFocus();
	    //lastFocus = withFocus;
	    //if (fieldChanged && withFocus instanceof JTextField) 
	    //	handlecr((JTextField) withFocus);
	    //withFocus = e.getComponent();
	}
		
	public void focusLost(FocusEvent e) {}
		
	// JVIEW needs this one (doesn't send focus event when dialog made active)
	/*public void windowActivated(WindowEvent evt) {
	//focus.requestFocus();
	withFocus = e.getComponent();
	}*/
    }

    abstract public void run();
  
    protected class TextKeyAdapter extends KeyAdapter {

	public void keyPressed(KeyEvent e) {
	    JTextField tf = (JTextField)e.getComponent();
	    char key = e.getKeyChar();
			
	    if (key == '\r' || key == '\n') {
		handlecr(tf);
		e.consume();
		//who_txt.requestFocus();
		return;
	    }
	    super.keyPressed(e);
	}
		
	public void keyTyped(KeyEvent e) {
	    JTextField tf = (JTextField)e.getComponent();
	    char key = e.getKeyChar();
	    //System.out.println("keytyped : " + key);
			
	    if (key == '\r' || key == '\n') {
		//handlecr(tf); 
		e.consume();
		return;
	    } else if (true)
		fieldChanged = true;
	    super.keyTyped(e);
	}
    }
  
    void handlecr(JTextField tf) {
	String lbl = tf.getName();
	String val = tf.getText();
	runLine(getCommand(lbl, val));
    }

    abstract String getCommand(String lbl, String val);
  
    public void done(Context c) {
	fieldChanged = false;
    }
	
    abstract String getCompileCommand(String s);

    private class JEntityRunner implements Runnable {
	private String s;
	public JEntityRunner(String s) {
	    this.s = s;
	}
	public void run() {
	    runningp = true;
	    if (s != null && s != "") {
		Object[] listtocompile = 
		    Reader.read(getCompileCommand(s));
		Object[] newlist = new Object[listtocompile.length + 1];
		System.arraycopy(listtocompile, 0, newlist, 0, listtocompile.length);
		newlist[newlist.length -1] = JEntityMonitor.this;
		sl.app.cc.enqueue(newlist, sl.app.message, sl.app.err, null);
	    }
	}
    }
  
    public void runLine(String s) {
	sl.app.maybeCompileAndThenRun(false, new JEntityRunner(s));
    }
  
    public int[] castToIntArray(Object[] ilist) {
	int[] newarray = new int[ilist.length];
	for(int i = 0; i<ilist.length; i++) {
	    newarray[i] = ((Number)ilist[i]).intValue();
	}
	return newarray;
    }
  
    abstract public void runCompiledCommand(Object[] ilist, Object[] olist);
      
    private Vector spps = new Vector();
    private int sppcount = 0;
	
    public void done(StackProcessParent spp) {
	spps.removeElement(spp);
	sppcount--;
	//		System.out.println("slcommand empty? " + spps.isEmpty() + " runningp = " + runningp);
	if (sppcount == 0 && runningp) {
	    //		    System.out.println("hi");
	    //			super.done((Context)null);
	    runningp = false;
	}
	if (lastField != -1) {
	    if (lastOwn)
		setOwn(lastField);
	    else 
		setValues(lastField);
	};
	lastField = -1;
	lastOwn = false;
	fieldChanged = false;
    }
	
    public boolean donep() {
	//System.out.println("slcommand done? " + (sppcount == 0));
	return (sppcount == 0);
    }
	
    public StarLogoCaller caller() {
	return this;
    }
	
    public void callerPrintln(String s) {}
    public void callerClearText() {}

    public void addStackProcessParent(StackProcessParent s) {
	spps.addElement(s);
	sppcount++;
    }
	
    public void stopAll() {
	//System.out.println("slcommand stop all");
	for(Enumeration e = ((Vector)spps.clone()).elements(); e.hasMoreElements();) {
	    StackProcessParent spp = (StackProcessParent)e.nextElement(); 
	    //System.out.println("trying to stop " + spp);
	    spp.stopAll();
	}
	//spps.removeAllElements();
    }
	
    // Only called when we are reporting an error.
    public void displayOneString(String s) {
	if (!com.runningp) {
	    com.displayOneString(s);
	} else StarLogoError.error(app, s, false); 
    }
	
    public void displayAString(String s) {
	if (!com.runningp) {
	    com.displayOneString(s);
	} else StarLogoError.error(app, s, false); 
    }

    public void itemStateChanged(ItemEvent evt) {
	Object o = evt.getSource();
	String arg = "";
	boolean selected = false;
	if (o instanceof JRadioButtonMenuItem) {
	    JRadioButtonMenuItem cmi = (JRadioButtonMenuItem)o;
	    arg = cmi.getLabel();
	    selected = cmi.isSelected();
	} else if (o instanceof CheckboxMenuItem) {
	    CheckboxMenuItem cmi = (CheckboxMenuItem)o;
	    arg = cmi.getLabel();
	    selected = cmi.getState();
	}
	try {
	    int value = Integer.parseInt(arg);
	    Font current = com.getFont();
	    com.setFont(new Font(current.getName(), current.getStyle(), value));
	    slmenu.setCurrentSize(o);
	    return;
	}
	catch (Exception e) {
	    Font current = com.getFont();
	    setFont(new Font(arg, current.getStyle(), current.getSize()));
	    slmenu.setCurrentFont(o);
	    return;
	}
    }

    JTextComponent focus;
  
    public JTextComponent getTextFocus() {
	return focus;
    }

    public void actionPerformed(ActionEvent e) {
	Object o = e.getSource();
	String arg = "";
	if (o instanceof JMenuItem) {
	    JMenuItem mi = (JMenuItem)o;
	    arg = mi.getLabel();
	} else if (o instanceof MenuItem) {
	    MenuItem mi = (MenuItem)o;
	    arg = mi.getLabel();
	}
	JTextComponent focus = getTextFocus();
	
	if (arg.equals(sl.app.textBundle.getString("edit_menu_cut"))) {
	    focus.cut();
	    
	    return;
	}
	if (arg.equals(sl.app.textBundle.getString("edit_menu_copy"))) {
	    focus.copy();
	    
	    return;
	}
	if (arg.equals(sl.app.textBundle.getString("edit_menu_paste"))) {
	    focus.paste();
	    
	    return;
	}
	if (arg.equals(sl.app.textBundle.getString("edit_menu_select_all"))) {
	    focus.selectAll();
	    return;
	}
	if (arg.equals(sl.app.textBundle.getString("edit_menu_clear"))) {
	    focus.setText("");
	}
    }

    public void markChanged() {    }

    public void clearChanged() {    }
    
}

