/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.awt.FontMetrics.*;
import java.net.URL;
import java.net.MalformedURLException;
import java.awt.*;
import java.awt.geom.Rectangle2D;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.event.*;
import yoyo.YoYo;
import java.applet.Applet;
import java.util.*;
import yoyo.Tokenizer;
import yoyo.Reader;
import yoyo.Symbol;
import java.io.LineNumberReader;
import java.io.StringReader;
import java.io.IOException;
import javax.swing.UIManager;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.JFrame;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JRadioButtonMenuItem;
import yoyo.YoYo;
import javax.swing.JMenuItem;
import javax.swing.JComponent;
import javax.swing.RepaintManager;

import java.awt.image.ImageObserver;
import java.awt.image.BufferedImage;


public class JInterface extends JComponent implements ImageObserver, MenuListener, SLPrintable, WidgetOwner {

    public static final int RESIZETOPLEFT = 1;
    public static final int RESIZETOPRIGHT = 2;
    public static final int RESIZEBOTLEFT = 3;
    public static final int RESIZEBOTRIGHT = 4;
    public static final int RESIZETOP = 5;
    public static final int RESIZEBOT = 6;
    public static final int RESIZELEFT = 7;
    public static final int RESIZERIGHT = 8;

    BufferedImage image;

    public ToolbarWidget toolbar;
    public StatusToolbar statusbar;
    public StopWidget stopwidget;
    public TrashWidget trashwidget;

    public Container parent;
    public InterfaceFrame frame;

    public WidgetHolder wh;
    boolean overtrashp = false;
    public Vector buttons;
    public Vector sliders;
    public Vector monitors;
    public Vector linewidgets;
    public Vector textwidgets;
    public Vector legends;
    //    public LineWidget retrofit = null;
    // changed  by MN - to different retrofit model
    public WidgetClipboard wclipboard;

    //PatchDims patchdims;
    Painttool painttool;
    Colortool colortool;
    PaintSelect paintselect;
    SpeedSliderWidget speedsliderwidget;
    CellSizeSliderWidget cellsizesliderwidget;
    ColorChooser colorchooser;
    BlueGrid bluegrid;
    Point anchor;
    boolean snapToGrid = false;
    int BUT_MASK;
    Rectangle paintRect;

    Image bgImage = null;
    String bgImageName = "";

    Widget keyboardFocus = null;
    WidgetLite tooltipShown = null;

    IKeyListener ikeylistener;

    public Application app;
    public StarLogo sl;

    static int toolbarheight = 36;
    static int toolbarwidth = 50;
    static int statusbarheight = 24;

    boolean pressedShiftControlp = false;
    boolean doubleClickedp = false;
    // iface is set and unset on mousePressed and mouseReleased
    Widget widgetTouched = null;
    boolean resizingp = false;
    boolean draggingp = false;
    boolean backgroundp = false;

    boolean firstdragp = false;
    boolean firstresizep = false;

    Rectangle clipRect = null;

    private boolean dwellToggle = false;
    private boolean toolTipStart = false;
    private Point dwellPoint = new Point(0, 0);
    private Point dwellPointSave = new Point(1, 1);
    private DwellTimer dwellTimer;
    private Widget dispWidget;
    private Object toolTipLock = new Object();
    private boolean dwellEnable = true;

    Point oldMousePosition = null, originalMousePosition = null;
    Point resizeAnchor = null;
    int resizeCorner = 0;

    SmartRectangle outline = null;
    Point outlineAnchor = null;


    public JInterface(Application app) {
	// initialize fields, and other fields depending on this
	this.app = app;
	this.sl = app.sl;
	this.wh = new WidgetHolder(app,this);
	this.buttons = new Vector(10, 5);
	this.sliders = new Vector(10, 5);
	this.monitors = new Vector(10, 5);
	this.legends = new Vector(10, 5);
	this.linewidgets = new Vector(10, 5);
	this.textwidgets = new Vector(10, 5);
	this.wclipboard = new WidgetClipboard(this);
	this.dwellTimer = new DwellTimer(750, this);
	//RepaintManager.setCurrentManager(new MultirectRepaintManager());

    }
    
    public void init() {
	// setup frame
	if (YoYo.applet != null) {
	    parent = YoYo.applet;
	    YoYo.applet.setLayout(null);
	    YoYo.applet.setBackground(Color.white);
	    YoYo.applet.setForeground(Color.black);
	}
	// load images for all widgets
	loadImages();
	// setup platform-specific stuff and the menu
	setupPlatform();
	// setup frame
	if (YoYo.applet == null) {
	    frame = new JInterfaceFrame(app, this);
	    parent = (JFrame)frame;
	    setDoubleBuffered(true);
	    setOpaque(true);
	    if (app.kioskp) ((JFrame)frame).setUndecorated(true);
	    parent.addNotify();
	} 
	else
	{
	   addKeyListener( new IKeyListener(this, sl()) );
	}
	// setup listeners
	setupListeners();

	// setup layout
	createComponents();
	setupLayout();

    }

    private void setupListeners() {
	// enable events in this
	enableEvents(AWTEvent.KEY_EVENT_MASK);
	enableEvents(AWTEvent.MOUSE_EVENT_MASK);
	enableEvents(AWTEvent.MOUSE_MOTION_EVENT_MASK);
	if (!app.kioskp && YoYo.applet == null) {
	    addComponentListener(new ComponentAdapter() {
		    public void componentResized(ComponentEvent e) {
			resizeWidgetsOnInterfaceResize();
		    }
		});
	}
    }

    public void resizeWidgetsOnInterfaceResize() {
	if (bluegrid != null) bluegrid.updatePosition();
	if (stopwidget != null) stopwidget.updatePosition();
	if (trashwidget != null) trashwidget.updatePosition();
	//System.out.println("height now is=" + getSize().height);
	if (statusbar != null) statusbar.updatePosition();
	repaint();
    }

    public String getBackgroundImageName() {
	return bgImageName;
    }
    
    public void setBackgroundImage(Image image, String name) {
	bgImage = image;
	if (name == null) 
	    bgImageName = "";
	else bgImageName = name;
	repaint();
    }

    public void maybeStopPolygonTool() {
	if (!app.kioskp && YoYo.applet == null) {
	    if (painttool().buttonDown == Painttool.POLY && 
		sl.patchcanvas instanceof PatchCanvas)
		((PatchCanvas)sl.patchcanvas).polygonCancelPoly();
	}
    }

    public void setTitle(String s) {
	if (YoYo.applet == null) frame.setTitle(s);
    }

    public String getTitle() {
	if (YoYo.applet == null) return frame.getTitle();
	else return app.textBundle.getString("starlogo_project");
    }

    // platform-specific stuff
    private void setupPlatform() {
  	// initialize selection button mask
  	if (YoYo.macintoshp)
  	    BUT_MASK = InputEvent.SHIFT_MASK;
  	else
  	    BUT_MASK = InputEvent.CTRL_MASK;
    }

    public void createComponents() {
	toolbar = new ToolbarWidget(this, true);
	statusbar = new StatusToolbar(this);

	// arrange statusbar
	stopwidget = new StopWidget(this);
	trashwidget = new TrashWidget(this);
	statusbar.stopwidget = stopwidget;
	statusbar.trashwidget = trashwidget;

	Insets insets = parent.getInsets();

	if (!app.kioskp && YoYo.applet == null) {
	    bluegrid = new BlueGrid(this);
	    wh.addGrid(bluegrid);

	    wh.addToolbarWidget(toolbar);
	    toolbar.setLocation(0, 0);

	    // add painttool
	    painttool = new Painttool(this, false);
	    painttool.setLocation(0, 40);
	    wh.addToolbarWidget(painttool);

	    // add colortool
	    colortool = new Colortool(this, false);
	    colortool.setLocation(0, 195);
	    wh.addToolbarWidget(colortool);

	    // add paintselect
	    paintselect = new PaintSelect(this, false);
	    paintselect.setLocation(0, 170);
	    wh.addToolbarWidget(paintselect);

	    // colorchooser
	    colorchooser = new ColorChooser(this);
	    colorchooser.setLocation(1, 412);
	    wh.addToolbarWidget(colorchooser);

	    // add statusbar
	    wh.addToolbarWidget(statusbar);
	    wh.addToolbarWidget(stopwidget);
	    wh.addToolbarWidget(trashwidget);
	    statusbar.setLocation(0,getSize().height-statusbarheight);

	    // speed slider widget
	    speedsliderwidget = new SpeedSliderWidget(this, new Rectangle(250, 6, 120, 20));
	    wh.addToolbarWidget(speedsliderwidget);

	    cellsizesliderwidget = new CellSizeSliderWidget(this, new Rectangle(390, 6, 100, 20));
	    wh.addToolbarWidget(cellsizesliderwidget);
	}

	if (sl.patchcanvas instanceof PatchCanvas) 
	    wh.setPatchCanvas((PatchCanvas)sl.patchcanvas);
    }	
    
    private void setupLayout() {
	// set layout and color
	setLayout(null);
	setBackground(Color.white);
	setForeground(Color.black);
	// set position
	Insets insets = getInsets();
	//System.out.println(insets.toString());

	setLocation(insets.left, insets.top);
	setSize(parent.getSize().width - insets.left - insets.right,
		parent.getSize().height - insets.top - insets.bottom);

	Rectangle bounds = getBoundaries();
	sl.patchcanvas.setLocation(bounds.x +
				   bounds.width -
				   //on pcs, the canvas was always starting out obscured
				   //by the right edge of the interface
				   sl.patchcanvas.getPosition().width - ((YoYo.macintoshp) ? 5 : 15),
				   bounds.y + 5);
	
	// create offscreen image for drawing
	if (app.doubleBufferp) {
	    image = parent.getGraphicsConfiguration().createCompatibleImage(getSize().width, getSize().height);
	    if (image == null) {
		//System.out.println("setupLayout new image null image. shit");
	    }
	}
    }

    // added by MIM
    public void setOutline(int x, int y, int width, int height){
	outline = new SmartRectangle(x, y, width, height);
    }

    public void resetOutline(){
	outline = null;
    }



    public void processTimerEvent() {
	if (dwellToggle && toolTipStart && dwellEnable) dwellPointSave.setLocation(dwellPoint);
	else {
	    synchronized(toolTipLock) {
		if ((dwellPointSave.x == dwellPoint.x) && (dwellPointSave.y == dwellPoint.y) && (dispWidget == null)) {
		    dispWidget = getWidgetAt(dwellPointSave.x, dwellPointSave.y);
		    if (dispWidget != null) {
			dispWidget.displayToolTip(dwellPointSave);
		    }
		}
	    }
	}
	dwellToggle = !dwellToggle;
    }

    public void processMouseEvent(MouseEvent e) {
	super.processMouseEvent(e);
	try {
	    //System.out.println("Top of processMouseEvent in JInterface: " + e);
	    //      if (!leftMB(e))
	    //          return;
	    // if you're adding to an existing selection, or if a widget can't handle it...
	    switch(e.getID()) {
	    case java.awt.event.MouseEvent.MOUSE_PRESSED:
		//System.out.println("Mouse_pressed");
		if (YoYo.applet == null) {app.iface.frame().requestFocusInWindow() ;}
		  else {this.requestFocusInWindow();}  
		VM.setMouseDown(true);
		break;
	    case java.awt.event.MouseEvent.MOUSE_RELEASED:
		//System.out.println("Mouse_released");
		VM.setMouseDown(false);
		break;
	    case java.awt.event.MouseEvent.MOUSE_ENTERED:
		//System.out.println("Mouse_entered");
		dwellEnable = true;
		break;
	    case java.awt.event.MouseEvent.MOUSE_EXITED:
		//System.out.println("Mouse_exited");
		dwellEnable = false;
		synchronized(toolTipLock) {
		    if (dispWidget != null) {
			dispWidget.dropToolTip();
			dispWidget = null;
		    }
	    }
	    break;
	}
	pressedShiftControlp = (((e.getModifiers() & BUT_MASK) != 0) && 
				(!app.kioskp && !app.locked && YoYo.applet == null));
	doubleClickedp = (e.getClickCount() == 2);
	clipRect = new Rectangle();
	//System.out.println("processMouseEvent: " + e);
	//System.out.println("clip rect = " + clipRect);
	if (widgetTouched == null) {
	    //System.out.println("mouse event " + e);
	    Widget obj = getWidgetAt(e.getX(), e.getY());
	    if (obj != null && !backgroundp) {
		//System.out.println("mouse event " + e + " on obj: " + obj);
		processWidgetMouseEvent(e, obj);
	    } else {
		// if we didn't click on the widget, did we get its corners?
		obj = getCornerAt(e.getX(), e.getY());
		if (obj != null && !backgroundp) {
		    processResizeEvent(e, obj);
		} else {
		    if (app.kioskp || app.locked || YoYo.applet != null) return;
		    //System.out.println("pressed or released on background: " + e);
		    switch(e.getID()) {
		    case java.awt.event.MouseEvent.MOUSE_PRESSED:
			backgroundp = true;
			mousePressed(e);
			break;
		    case java.awt.event.MouseEvent.MOUSE_RELEASED:
			mouseReleased(e);
			backgroundp = false;
			break;
		    default: return;
		    }
		    repaint(clipRect);
		    //clipRect = null;
		}
	    }
	} else {
	    Widget obj = widgetTouched;
	    if (resizingp) {
		processResizeEvent(e, obj);
	    } else {
		processWidgetMouseEvent(e, obj);
	    }
	}
	} catch (Exception exc) {
	    exc.printStackTrace(app.log);
	}
    }

    public void processMouseMotionEvent(MouseEvent e) {
	try {
	VM.setMousePos(e.getX(), e.getY());
        toolTipStart = true;
	dwellPoint.setLocation(e.getX(), e.getY());
        synchronized(toolTipLock) {
	    if (dispWidget != null) {
		dispWidget.dropToolTip();
		dispWidget = null;
	    }
        }
	switch(e.getID()) {
	case java.awt.event.MouseEvent.MOUSE_DRAGGED:
	    VM.setMouseDown(true);
	    break;
	case java.awt.event.MouseEvent.MOUSE_MOVED:
	    VM.setMouseDown(false);
	    break;
	}

	// Andy is clever. Without shift, you can't shift-double-click to edit.
	pressedShiftControlp = (((e.getModifiers() & BUT_MASK) != 0) && 
				(!app.kioskp && !app.locked && YoYo.applet == null));
	doubleClickedp = (e.getClickCount() == 2);
	clipRect = new Rectangle();

	Widget obj = widgetTouched;
	if (obj != null) {
	    if (resizingp) {
		processResizeEvent(e, obj);
	    } else {
		if (e.getID() == java.awt.event.MouseEvent.MOUSE_DRAGGED) {
		    processWidgetMouseEvent(e, obj);
		}
	    }
	} else {
	    //System.out.println("not touching any widget: " + e);
	    switch(e.getID()) {
	    case java.awt.event.MouseEvent.MOUSE_DRAGGED:
		mouseDragged(e);
		break;
	    case java.awt.event.MouseEvent.MOUSE_MOVED:
		mouseMoved(e);
		break;
	    default: return;
	    }
	    repaint(clipRect);
	    //clipRect = null;
	}

	if (!sl.runningnowp) {
	    try {
		Thread.currentThread().sleep(10);
	    }
	    catch (InterruptedException f) {}
	}
	} catch (Exception exc) {
	    exc.printStackTrace(app.log);
	}
    }

    public void processWidgetMouseEvent(MouseEvent e, Widget obj) {
	switch(e.getID()) {
	case java.awt.event.MouseEvent.MOUSE_PRESSED:
	    widgetTouched = obj;
	    //System.out.println("mouse pressed on " + obj);
	    if (obj.draggable(e)) {
		//System.out.println(obj + " is draggable");
		originalMousePosition = new Point(e.getX(), e.getY());
		oldMousePosition = new Point(e.getX(), e.getY());
		firstdragp = true;
	    } else {
		//System.out.println(obj + " is not draggable");
		if (!(obj instanceof PatchCanvas)) {
		    maybeStopPolygonTool();
		}
		obj.mousePressed(e);
	    }
            //System.out.println(clipRect);
	    repaint(clipRect);
	    break;
	case java.awt.event.MouseEvent.MOUSE_RELEASED:
	    if (draggingp || (obj instanceof Pushable)) {
		obj.mouseReleased(e);
	    } else {
		if (doubleClickedp) {
		    obj.mouseDoubleClicked(e);
		} else {
		    if (toolbar.buttonDown != ToolbarWidget.POINTER) {
			if (maybeEditWidget(obj)) {
			    widgetTouched = null;
			    draggingp = false;
			    resizingp = false;
			    firstdragp = false;
			    wh.setGrid(false);
			    oldMousePosition = null;
			    originalMousePosition = null;
			    //app.restoreCursor();
			    repaint(clipRect);
			    return;
			}
		    }
		    if (pressedShiftControlp) {
			if (obj.isSelected()) {
			    deselect(obj);
			    addHandlesToClipRect(obj);
			} else {
			    select(obj);
			    addHandlesToClipRect(obj);
			}
		    } else if (obj.isSelected()) {
			deselectAll(false);
			select(obj);
			addHandlesToClipRect(obj);
		    } else {
			if (getBoundaries().contains(e.getX(), e.getY())) {
			    deselectAll(false);
			    moveToFront(obj);
			}
			//  if ( getWidgetAt(e.getX(), e.getY()) == obj) {
			// 		obj.execute();
			// 	}
			if (getWidgetAt(e.getX(), e.getY()) == obj ||  obj instanceof PatchCanvas)
			    obj.mouseClicked(e);
		    }
		}
	    }
	    widgetTouched = null;
	    if (draggingp) app.restoreCursor();
	    draggingp = false;
	    firstdragp = false;
	    resizingp = false;
	    wh.setGrid(false);
	    oldMousePosition = null;
	    originalMousePosition = null;
            //System.out.println(clipRect);
	    repaint(clipRect);
	    break;
	case java.awt.event.MouseEvent.MOUSE_DRAGGED:
	    if (trashwidget.onPressable(e) && obj != trashwidget && draggingp)
		trashwidget.press();
	    else
		trashwidget.reset();
	    if (firstdragp) {
		if (distanceSquared(originalMousePosition, e.getX(), e.getY()) > 25) {
		    draggingp = true;
		    app.setCursor(Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR));
		    if (pressedShiftControlp) {
			moveToFront(obj);
			if (obj instanceof WidgetDrag) {
			    WidgetDrag[] todrag = { (WidgetDrag)obj };
			    moveWidgets(e, todrag);
			}
		    } else {
			if (obj.isSelected()) {
			    WidgetDrag[] todrag = wh.getSelectedWidgets(true);
			    for(int i = 0; i < todrag.length; i++) {
				moveToFront(todrag[i]);
			    }
			    moveWidgets(e, todrag);
			} else {
			    if (getBoundaries().contains(e.getX(), e.getY()))
				deselectAll(false);
			    moveToFront(obj);
			    if (obj instanceof WidgetDrag) {
				WidgetDrag[] todrag = { (WidgetDrag)obj };
				moveWidgets(e, todrag);
			    }
			}
		    }
		    firstdragp = false;
		}
	    } else if (draggingp) {
		// this isn't the first drag
		if (pressedShiftControlp) {
		    if (obj instanceof WidgetDrag) {
			WidgetDrag[] todrag = { (WidgetDrag)obj };
			moveWidgets(e, todrag);
		    }
		} else {
		    if (obj.isSelected()) {
			WidgetDrag[] todrag = wh.getSelectedWidgets(true);
			moveWidgets(e, todrag);
		    } else {
			if (obj instanceof WidgetDrag) {
			    WidgetDrag[] todrag = { (WidgetDrag)obj };
			    moveWidgets(e, todrag);
			}
		    }
		}
	    } else obj.mouseDragged(e);
	    repaint(clipRect);
	    break;
	case java.awt.event.MouseEvent.MOUSE_MOVED:
	    // nothing
	    break;
	default: return;
	}
    }

    protected void moveWidgets(MouseEvent e, WidgetDrag[] widgets) {
	trashwidget.mouseDragged(e);
	Point newMousePosition = limitPointToView(new Point(e.getX(), e.getY()));
	if (snapToGrid && firstdragp) {
	    wh.setGrid(true);
	    for (int i=0; i<widgets.length; i++)
		widgets[i].snapToGrid();
	}
	if (!oldMousePosition.equals(newMousePosition)) {
	    Point displacement = pointSubtract(newMousePosition, oldMousePosition);
	    if (snapToGrid) {
	    	displacement.x = BlueGrid.gridunit * (displacement.x / BlueGrid.gridunit);
	    	displacement.y = BlueGrid.gridunit * (displacement.y / BlueGrid.gridunit);
	    }
	    if (snapToGrid) {
		if (displacement.x != 0)
		    oldMousePosition.x = oldMousePosition.x + displacement.x;
		if (displacement.y != 0)
		    oldMousePosition.y = oldMousePosition.y + displacement.y;
	    } else {
	    	oldMousePosition = newMousePosition;
	    }
	    for (int i = 0; i < widgets.length; i++) {
		if (widgets[i].isSelected())
		    addHandlesToClipRect(widgets[i]);
		else
		    addToClipRect(widgets[i].position);
		widgets[i].moveBy(displacement);
		if (widgets[i].isSelected())
		    addHandlesToClipRect(widgets[i]);
		else
		    addToClipRect(widgets[i].position);
	    }
	}
    }

    public int distanceSquared(Point a, int x, int y) {
	return ((a.x - x) * (a.x - x)) + ((a.y - y) * (a.y - y));
    }

    public Point pointSubtract(Point a, Point b) {	
	return new Point(a.x - b.x, a.y - b.y);
    }

    public Point limitPointToView(Point p) {
	Rectangle boundaries = getBoundaries();
	if (p.x < boundaries.x) p.x = boundaries.x;
	if (p.y < boundaries.y) p.y = boundaries.y;
	if (p.x > (boundaries.x + boundaries.width))
	    p.x = (boundaries.x + boundaries.width);
	if (p.y > (boundaries.y + boundaries.height))
	    p.y = (boundaries.y + boundaries.height);
	return p;
    }

    protected void processResizeEvent(MouseEvent e, Widget obj) {
	//System.out.println("resize event " + e + " object: " +obj);
	switch(e.getID()) {
	case java.awt.event.MouseEvent.MOUSE_PRESSED:
	    maybeStopPolygonTool();
	    widgetTouched = obj;
	    moveToFront(obj);
	    if (obj.resizable()) {
		resizingp = true;
		firstresizep = true;
		resizeAnchor = new Point(e.getX(), e.getY());
		resizeCorner = obj.resizeWhichCorner(resizeAnchor);
		app.setCursor(resizeCursor(resizeCorner));
		oldMousePosition = resizeAnchor;
	    }
	    break;
	case java.awt.event.MouseEvent.MOUSE_RELEASED:
	    obj.endResize(e, resizeCorner);
	    resizingp = false;
	    wh.setGrid(false);
	    widgetTouched = null;
	    resizeCorner = 0;
	    resizeAnchor = null;
	    oldMousePosition = null;
	    app.restoreCursor();
	    resetStatus();
	    repaint(clipRect);
	    break;
	case java.awt.event.MouseEvent.MOUSE_DRAGGED:
	    Point newMousePosition = limitPointToView(new Point(e.getX(), e.getY()));
	    if (snapToGrid) {
	    	if (firstresizep) wh.setGrid(true);
	    	Point displacement = pointSubtract(newMousePosition, resizeAnchor);
	    	newMousePosition.x = BlueGrid.gridunit * ((resizeAnchor.x+displacement.x) / BlueGrid.gridunit);
	    	newMousePosition.y = BlueGrid.gridunit * ((resizeAnchor.y+displacement.y) / BlueGrid.gridunit);
	    }
	    resizeWidget(newMousePosition, obj, resizeCorner);
	    if (obj instanceof PatchCanvas) {
		PatchCanvas pc = (PatchCanvas)obj;
		setStatus("PatchCanvas width: " + (pc.position.width / sl.cellsize) +
			  " PatchCanvas height: " + (pc.position.height / sl.cellsize));
	    } else {
		setStatus("Widget width: " + obj.position.width + " Widget height: " + obj.position.height);
	    }
	    repaint(clipRect);
	    if (firstresizep) firstresizep = false;
	    break;
	default:
	    break;
	}
    }

    protected void resizeWidget(Point newMousePosition, Widget widget, int resizeCorner) {
	//System.out.println("resize Widget");
	if (!oldMousePosition.equals(newMousePosition)) {
	    widget.resize(newMousePosition, resizeCorner);
	    oldMousePosition = newMousePosition;
	}
    }

    public Cursor resizeCursor(int resizeCorner) {
	switch(resizeCorner) {
	case JInterface.RESIZETOPLEFT:
	    return Cursor.getPredefinedCursor(Cursor.NW_RESIZE_CURSOR);
	case JInterface.RESIZETOPRIGHT:
	    return Cursor.getPredefinedCursor(Cursor.NE_RESIZE_CURSOR);
	case JInterface.RESIZEBOTLEFT:
	    return Cursor.getPredefinedCursor(Cursor.SW_RESIZE_CURSOR);
	case JInterface.RESIZEBOTRIGHT:
	    return Cursor.getPredefinedCursor(Cursor.SE_RESIZE_CURSOR);
	case JInterface.RESIZETOP:
	    return Cursor.getPredefinedCursor(Cursor.N_RESIZE_CURSOR);
	case JInterface.RESIZEBOT:
	    return Cursor.getPredefinedCursor(Cursor.S_RESIZE_CURSOR);
	case JInterface.RESIZELEFT:
	    return Cursor.getPredefinedCursor(Cursor.W_RESIZE_CURSOR);
	case JInterface.RESIZERIGHT:
	    return Cursor.getPredefinedCursor(Cursor.E_RESIZE_CURSOR);
	}
	return null;
    }


    protected boolean maybeEditWidget(Widget obj) {
	if (app.kioskp || app.locked || YoYo.applet != null) return false;
	if ((obj instanceof ButtonWidget &&
	     toolbar.buttonDown == ToolbarWidget.BUTTON) ||
	    (obj instanceof TextWidget &&
	     toolbar.buttonDown == ToolbarWidget.TEXT) ||
	    (obj instanceof SliderWidget &&
	     toolbar.buttonDown == ToolbarWidget.SLIDER) ||
	    (obj instanceof MonitorWidget &&
	     toolbar.buttonDown == ToolbarWidget.MONITOR) ||
	    (obj instanceof ColorLegendWidget &&
	     toolbar.buttonDown == ToolbarWidget.LEGEND) ||
	    (obj instanceof LineWidget &&
	     toolbar.buttonDown == ToolbarWidget.LINE)) {
	    obj.edit();
	    toolbar.setButtonDown(ToolbarWidget.POINTER);
	    return true;
	}
	return false;
    }

    public void maybeReleasePatchCanvas() {
	//System.out.println("maybe release patch canvas");
	if (!(sl.patchcanvas instanceof PatchCanvas) ||
	    !isObscured((PatchCanvas)sl.patchcanvas)) {
	    //System.out.println("letting turtles go");
	    synchronized(sl.runningp) {
		//System.out.println("Setting sl.locked to false.");
		sl.locked = false;
		//System.out.println("sl.locked = false");
		//new Exception().printStackTrace();
		sl.runningp.notify();
	    }
	} 
	else {
	    //System.out.println("still obscured");
	}
    }

    protected void mousePressed(MouseEvent e) {
	maybeStopPolygonTool();
	if (getBoundaries().contains(e.getX(), e.getY())) {
	    outline = new SmartRectangle(e.getX(), e.getY(), 0, 0);
	    outlineAnchor = new Point(e.getX(), e.getY());
	    if (!pressedShiftControlp &&
		getBoundaries().contains(e.getX(), e.getY())) {
		//System.out.println("clicked on background without shift");
		deselectAll(false);
	    }
	}
    }

    protected void mouseDragged(MouseEvent e) {
	/* we didn't start the drag on the interface background */
	if (outline == null) return;
	addToClipRect(outline);
	Point p = limitPointToView(new Point(e.getX(), e.getY()));
	outline.setWidth(p.x - outlineAnchor.x);
	outline.setHeight(p.y - outlineAnchor.y);
	addToClipRect(outline);
	growClipRect(1,1);
    }

    protected void mouseReleased(MouseEvent e) {
	trashwidget.mouseReleased(e);

	if (outline == null) {
	    return;
	}
	if (toolbar.buttonDown == ToolbarWidget.POINTER ||
	    toolbar.paintActivep()) {
	    selectWidgets(outline);
	} else {
	    if (!app.kioskp && !app.locked && YoYo.applet == null) {
		switch (toolbar.buttonDown) {
		case ToolbarWidget.BUTTON:
		    ButtonWidget.createNew(this);
		    break;
		case ToolbarWidget.SLIDER:
		    SliderWidget.createNew(this);
		    break;
		case ToolbarWidget.MONITOR:
		    MonitorWidget.createNew(this);
		    break;
		case ToolbarWidget.TEXT:
		    TextWidget.createNew(this);
		    break;
		case ToolbarWidget.LEGEND:
		    ColorLegendWidget.createNew(this);
		    break;

 		case ToolbarWidget.LINE:
		    LineWidget.createNew(this);
		    break;
		default:
		    return;
		}
	    }
	    toolbar.setButtonDown(ToolbarWidget.POINTER);
	}
	addToClipRect(outline);
	growClipRect(1,1);
	outline = null;
	outlineAnchor = null;
	//System.out.println("cleared out cliprect");
    }

    protected void selectWidgets(Rectangle boundsRect) {
	//System.out.println("select widgets: " + boundsRect);
	if (YoYo.applet != null) return;
	for (Enumeration en = wh.listDragWidgets(); en.hasMoreElements(); ) {
	    Widget w = (WidgetDrag)en.nextElement();
	    if (w.intersects(boundsRect)) {
		if (!(toolbar.paintActivep() && w instanceof PatchCanvas)) {
		    select(w);
		    //System.out.println("selecting "+ w);
		}
	    }
	}
	if (selecting()) {
	    frame.getMenu().enableCutCopyClearMenu();
	    addToClipRect(wh.getSelectRect());
	    growClipRect(WidgetDrag.CORNER_WIDTH, WidgetDrag.CORNER_WIDTH);
	    //System.out.println("clip rect is " + clipRect);
	}
    }

    public void repaint(Rectangle clip) {
	trim(clip);
	if (!clip.isEmpty()) {
	    //System.out.println("repaint clip: " + clip);
	    repaint(clip.x, clip.y, clip.width, clip.height);
	}
    }

    protected void trim(Rectangle r) {
	// trims r so left boundary is non-negative (otherwise repaint causes bug in jdk1.2)
	if (r.getLocation().x<0)
	    r.setLocation(0, r.getLocation().y);
	if (r.getLocation().y<0) r.setLocation(r.getLocation().x, 0);
    }

    public void paintComponent(Graphics g) {
	super.paintComponent(g);
	
	//System.out.println("interface paint w/ size " + getSize());
	//try {throw new Exception();} catch(Exception e) {e.printStackTrace();}
	
	Graphics gi = null;

	Dimension iface_size = getSize();
	//System.out.println("iface size in paint: " + iface_size);
	Rectangle clip = g.getClipBounds();
	// sometimes getClipBounds returns null, so we have to set clip to size of iface
	// this assumes that <image> will be adjusted to be size of iface below
	if (clip==null) {
	    clip = new Rectangle(iface_size.width, iface_size.height);
	    //repaintAll = false;
	}

	Rectangle bounds = getBoundaries();

	//System.out.println("clip rect is " + clip);
	if (app.doubleBufferp) {
	    if (image == null ||
		((image.getWidth(this) != iface_size.width) ||
		 (image.getHeight(this) != iface_size.height))) {
		if (iface_size.width == 0 || iface_size.height == 0) return;
		//System.out.println("resizing offscreen");
		image = parent.getGraphicsConfiguration().createCompatibleImage(iface_size.width, iface_size.height);
		if (image == null) {
		    //System.out.println("paint new image null image. uh oh");
		    return;
		}
		gi = image.getGraphics();
	    }
	    else {
		gi = image.getGraphics();
		gi.setColor(Color.white);
	    }
	    if (bgImage != null) {
		gi.drawImage(bgImage, bounds.x, bounds.y, null);
	    } else {
		gi.fillRect(clip.x, clip.y, clip.width, clip.height);
	    }
	    gi.setClip(clip);
	} else {
	    g.setClip(clip);
	    g.setColor(Color.white);
	    //System.out.println("macosx setting clip region to " + clip);
	    if (bgImage != null) {
		g.drawImage(bgImage, bounds.x, bounds.y, null);
	    } else {
		g.fillRect(clip.x, clip.y, clip.width, clip.height);
	    }
	}

	g.setColor(Color.black);
	// draw components
        PatchCanvasInterface pci = getPatchCanvas();
	//	System.out.println("patchcanvas: " + pci);
	if (pci != null && 
	    pci instanceof PatchCanvas && 
	    clip.intersects(((PatchCanvas)pci).position)) {
	    //System.out.println("we have to redraw the patch canvas");
	    synchronized(sl.runningp) {
		paintComponents((app.doubleBufferp) ? gi : g, clip, true);
		// draw rest of toolbar
		if (!app.kioskp && YoYo.applet == null) paintToolbar((app.doubleBufferp) ? gi : g, clip);
		//System.out.println("Clip: ("+clip.x+","+clip.y+","+clip.width+","+clip.height+")");
		if ((outline!=null)&&(clip.intersects(outline)))
		    drawOutline((app.doubleBufferp) ? gi : g);
		// draw the tooltip if there is one
		if (tooltipShown != null && clip.intersects(tooltipShown.position)) {
		    tooltipShown.paint((app.doubleBufferp) ? gi : g);
		}
		if (app.doubleBufferp) {
		    g.drawImage(image, clip.x, clip.y, clip.x+clip.width, clip.y+clip.height,
				clip.x, clip.y, clip.x+clip.width, clip.y+clip.height,
				null, null);
		}
	    }
	} else {
	    // We don't have to repaint the patch canvas
	    //System.out.println("no painting the patch canvas");
	    paintComponents((app.doubleBufferp) ? gi : g, clip, true);
	    // draw rest of toolbar
	    if (!app.kioskp && YoYo.applet == null) paintToolbar((app.doubleBufferp) ? gi : g, clip);
	    //System.out.println("Clip: ("+clip.x+","+clip.y+","+clip.width+","+clip.height+")");
	    if ((outline!=null)&&(clip.intersects(outline)))
		drawOutline((app.doubleBufferp) ? gi : g);
	    if (tooltipShown != null && clip.intersects(tooltipShown.position)) {
		tooltipShown.paint((app.doubleBufferp) ? gi : g);
	    }
	    if (app.doubleBufferp) {
		g.drawImage(image, clip.x, clip.y, clip.x+clip.width, clip.y+clip.height,
			    clip.x, clip.y, clip.x+clip.width, clip.y+clip.height,
			    null, null);
	    }
	}
	//getToolkit().sync();
	if (app.doubleBufferp) gi.dispose();
    }

    public void printComponent(Graphics g) {
	g.setColor(Color.black);
	Shape clipshape = g.getClip();
	Rectangle clip = null;
	if (clipshape instanceof Rectangle) {
	    clip = (Rectangle)clipshape;
	} else if (clipshape instanceof Rectangle2D) {
	    clip = clipshape.getBounds();
	}
	assert (clip != null);
	// draw components
        PatchCanvasInterface pci = getPatchCanvas();
	//	System.out.println("patchcanvas: " + pci);
	if (pci != null && 
	    pci instanceof PatchCanvas && 
	    clip.intersects(((PatchCanvas)pci).position)) {
	    //System.out.println("we have to redraw the patch canvas");
	    synchronized(sl.runningp) {
		paintComponents(g, clip, false);
		//System.out.println("Clip: ("+clip.x+","+clip.y+","+clip.width+","+clip.height+")");
	    }
	} else {
	    paintComponents(g, clip, false);
	    //System.out.println("Clip: ("+clip.x+","+clip.y+","+clip.width+","+clip.height+")");
	}
    }

    protected void paintToolbar(Graphics g, Rectangle clip) {
	//System.out.println("entering paintToolbar");
	//	    if (!colortool.update) {
	//	g.setClip(colortool.position.x, colortool.position.y, colortool.position.width, colortool.position.height);
	//}
	if (toolbar == null) return;

	Rectangle r = new Rectangle();
	Dimension size = getSize();
	g.setColor(SystemColor.control);
	r.setBounds(0, 0, toolbarwidth, size.height);
	if (clip.intersects(r)) {
	    g.fillRect(0, 0, toolbarwidth, size.height);
	    g.draw3DRect(toolbar.position.x, 0, toolbarwidth - 1, size.height-1, true);
	}
	r.setBounds(0, 0, size.width, toolbarheight);
	if (clip.intersects(r)) {
	    g.fillRect(0, 0, size.width, toolbarheight);
	    g.draw3DRect(0, 0, size.width - 1, toolbarheight -1, true);
	}
	for (Enumeration e = wh.listToolbars(); e.hasMoreElements(); ) {
	    Widget w = (Widget)e.nextElement();
	    if (w.intersects(clip)) {
		w.paint(g);
	    }
	}
    }

    public PatchCanvasInterface getPatchCanvas() {
  	return wh.getPatchCanvas();
    }

    protected void paintComponents(Graphics g, Rectangle clip, boolean drawToolbars) {
	Widget w;
	//System.out.println("paint components");
	for (Enumeration e = (drawToolbars) 
		 ? wh.listAllWidgets() 
		 : wh.listDragWidgets();
	     e.hasMoreElements(); ) {
	    w = (Widget)e.nextElement();
	    //System.out.println("maybe painting widget: " + w);
	    if (w != null && w.intersects(clip)) {
		// draw widgets intersecting clip
		//System.out.println("painting widget: " + w);
		w.paint(g);
		if (w instanceof WidgetDrag) {
		    WidgetDrag wd = (WidgetDrag)w;
		    if (wd.isSelected()) {
			// draw corners of selected widget drags intersecting clip
			wd.drawCorners(g);
		    }
		}
	    }
	}
    }

    protected void drawOutline(Graphics g) {
	g.setColor(Color.gray);
	g.drawRect(outline.x, outline.y, outline.width, outline.height);
    }

    public Rectangle getBoundaries() {
	Dimension size = getSize();
	Insets insets = getInsets();
	//System.out.println("insets are: " + insets);
	if (!app.kioskp && YoYo.applet == null) {
		// System.out.println ("returning height= " + (size.height- toolbarheight-statusbarheight-insets.top));
		//  System.out.println ("returning height= " + (size.height));
	    return new Rectangle(toolbarwidth, toolbarheight+insets.top,
				 size.width - toolbarwidth,
				 size.height- toolbarheight-statusbarheight-insets.top);
	} else return new Rectangle(0, 0, size.width, size.height);
    }

    public Dimension getVisibleSize() {
	Dimension size = getSize();
	//System.out.println("interface size: " +size);
	return new Dimension(size.width - toolbarwidth,
	             Math.max(0, size.height-toolbarheight));
			    // Math.max(0, size.height-toolbarheight-statusbarheight));
    }
   
    public void setVisibleSize(int width, int height) {
	//System.out.println("set visible size width " + width + " height " + height);
	if (!app.kioskp && YoYo.applet == null) {
	    int newwidth = width + toolbarwidth;
	    int newheight;
            if (YoYo.macintoshp) 
               newheight = height + toolbarheight;
            else
               newheight = height + toolbarheight + statusbarheight;
	    //System.out.println("set frame size");
	    frame.setVisibleSize(newwidth, newheight);
       	//System.out.println("new vissize=" + newheight);
	    //System.out.println("set interface size");
            //setSize(newwidth, newheight);
	    //System.out.println("new framesize=" + newheight);
	    //repaintAll = false;
	    //System.out.println("set visible size finish");
	}
    }


    public Widget getWidgetAt(int x, int y) {
	Widget w;
	for (Enumeration e = wh.listAllWidgetsTopDown(); e.hasMoreElements(); ) {
	    w = (Widget)e.nextElement();
	    if (w.contains(x,y))
		return(w);
	}
	return(null);
    }

    public WidgetDrag getCornerAt(int x, int y) {
	Point p = new Point(x, y);
	for (Enumeration e = wh.listDragWidgets(); e.hasMoreElements(); ) {
	    WidgetDrag w = (WidgetDrag)e.nextElement();
	    if ( w.isSelected() && w.cornerContains(p))	return w;
	}
	return(null);
    }

    public void moveToFront(Widget w) {
	if (!(w instanceof WidgetDrag)) return;
	wh.moveToFront((WidgetDrag)w);
	if (w.isSelected()) {
	    addHandlesToClipRect(w);
	} else {
	    addToClipRect(w.position);
	}
    }

    // mark project in guitop as changed given a change in the interface
    public void markChanged() {
	app.project.markChanged();
    }

    public void clearSelections() {
	if (selecting()==true) {
	    //((WidgetGroup)widgets.elementAt(2)).clearSelections();
	    Rectangle clipRect = wh.getSelectRect();
	    clipRect.grow(WidgetDrag.CORNER_WIDTH, WidgetDrag.CORNER_WIDTH);
	    //widgets.removeElementAt(2);
	    wh.clearSelections();
	    //System.out.println("clearing selections: cliprect = " + clipRect);
	    repaint(clipRect);
	    if (YoYo.macintoshp) {
		frame.getMenu().disableClearMenu();
	    } else {
		frame.getMenu().disableCutCopyClearMenu();
	    }
	}
    }

    public boolean replacePatchCanvas(PatchCanvasInterface oldpc, PatchCanvasInterface pc) {
	return replacePatchCanvas(oldpc, pc, oldpc.getLocation().x, oldpc.getLocation().y);
    }

    public boolean replacePatchCanvas(PatchCanvasInterface oldpc, PatchCanvasInterface pc, int x, int y) {
	//System.out.println("replace patchcanvas " + oldpc + " with " + pc);
	pc.setLocation(x, y);
	// make sure to change the widgetdrag if there is one and preserve
	// drag bits
	if (pc instanceof PatchCanvas) {
	    PatchCanvas pcanvas = (PatchCanvas)pc;
	    wh.setPatchCanvas(pcanvas);
	} else {
	    wh.removePatchCanvas();
	}
	if (oldpc instanceof PatchCanvas) {
	    repaint(((PatchCanvas)oldpc).position);
	} else {
	    repaint();
	}
	if (pc.init(this)) {
	    if (pc instanceof PatchCanvas) {
		repaint(((PatchCanvas)pc).position);
		PatchCanvas pcanvas = (PatchCanvas)pc;
		pcanvas.setOffscreen(sl.offscreen);
	    } else {
		repaint();
	    }
	    return true;
	} else return false;
    }

    public void deleteSelections(boolean menup) {
	if (menup) { clipRect = new Rectangle(); }
	addToClipRect(wh.getSelectRect());
	clipRect.grow(WidgetDrag.CORNER_WIDTH, WidgetDrag.CORNER_WIDTH);
	WidgetDrag[] wda = wh.getSelectedWidgets(false);
	if (wda != null) {
	    for (int i=0; i<wda.length; i++)
		removeWidget(wda[i]);
	}
	wh.clearSelections();
	frame.getMenu().disableCutCopyClearMenu();
	if (menup) { repaint(clipRect); } //clipRect = null; }
	if (wda != null) {
	    //app.project.compile(app);
	    app.rancompilerp = false;
	    app.rancompilerpForVariables = false;
	    app.project.markChanged();
	}
    }

    public void select(Widget wd) {
	if (wd instanceof WidgetDrag) {
	    wh.setSelected((WidgetDrag)wd,true);
	    frame.getMenu().enableCutCopyClearMenu();
	}
    }

    public void select(Widget wd, boolean b) {
	if (wd instanceof WidgetDrag) {
	    wh.setSelected((WidgetDrag)wd,b);
	    if (b) frame.getMenu().enableCutCopyClearMenu();
	    else if (!wh.hasSelections()) {
		frame.getMenu().disableCutCopyClearMenu();
	    }
	}
    }

    public void deselect(Widget wd) {
	if (wd instanceof WidgetDrag) {
	    wh.setSelected((WidgetDrag)wd, false);
	    if (!wh.hasSelections()) {
		frame.getMenu().disableCutCopyClearMenu();
	    }
	}
    }


    public void deselectAll(boolean menup) {
    	if (!app.kioskp && !app.locked && YoYo.applet == null) colorchooser.restoreOld();
	if (selecting()) {
	    if (menup) { clipRect = new Rectangle(); }
	    Rectangle r = wh.getSelectRect();
	    wh.clearSelections();
	    frame.getMenu().disableCutCopyClearMenu();
	    addToClipRect(r);
	    growClipRect(WidgetDrag.CORNER_WIDTH, WidgetDrag.CORNER_WIDTH);
	    if (menup) { repaint(clipRect); } //clipRect = null; }
	}
    }

    public void selectAll(boolean menup) {
	if (menup) { clipRect = new Rectangle(); }
	for (Enumeration en = wh.listDragWidgets(); en.hasMoreElements(); ) {
	    WidgetDrag w = (WidgetDrag)en.nextElement();
	    if (w instanceof PatchCanvas) {
		if (!toolbar.paintActivep()) {
		    select(w);
	  	}
	    } else {
		select(w);
	    }
	    addHandlesToClipRect(w);
	}
	if (menup) {
	    repaint(clipRect);
	    //clipRect = null;
	}
    }


    public void undo(boolean menup) {
	if (menup) { clipRect = new Rectangle(); }
	PatchCanvasInterface pci = getPatchCanvas();
	if (pci instanceof PatchCanvas) {
	    ((PatchCanvas)pci).undo();
	}
	if (menup) { repaint(clipRect); } 
    }	

    public void redo(boolean menup) {
	if (menup) { clipRect = new Rectangle(); }
	PatchCanvasInterface pci = getPatchCanvas();
	if (pci instanceof PatchCanvas) {
	    ((PatchCanvas)pci).redo();
	}
	if (menup) { repaint(clipRect); } 
    }	

    protected void mouseMoved(MouseEvent e) {
	if (!app.kioskp && YoYo.applet == null)
	    updatePointerCoordinates(new Point(e.getX(), e.getY()));
	//System.out.println("interface mouseMoved: " + e); //System.out.println("&&&& sl.locked is equal to: " + sl.locked);
	if (toolbar.paintActivep()) {
	    //;System.out.println("paint active");
	    if (painttool.buttonDown == Painttool.POLY) {
		//System.out.println("in the poly tool");
		if (sl.patchcanvas instanceof PatchCanvas)
		    ((PatchCanvas)sl.patchcanvas).mouseMoved(e);
	    }
	}
    }


    // this updates the pointer coordinates as displayed in the status bar
    protected void updatePointerCoordinates(Point p) {
	Widget w = getWidgetAt(p.x, p.y);
	if (w instanceof PatchCanvas) {
	    PatchCanvas pc = (PatchCanvas)w;
	    statusbar.xcor = new Integer( (int) (1+(p.x-pc.position.x)/sl.cellsize-sl.widthp2/2));
	    statusbar.ycor = new Integer( (int) ((pc.position.y-p.y)/sl.cellsize+sl.heightp2/2-1));
	    addToClipRect(statusbar.position);
	} else {
	    if (statusbar.xcor != null) {
		statusbar.xcor = null;
		statusbar.ycor = null;
		addToClipRect(statusbar.position);
	    }
	}
    }

    public void addToClipRect(Rectangle r) {
	//        System.out.println(r);
	//        if (r.y == 10) {       try { throw (new Exception());} catch(Exception e) {e.printStackTrace();} }
  	if (clipRect.isEmpty()) {
	    clipRect.setBounds(r);
	} else clipRect = clipRect.union(r);
	//System.out.println("Add to clip rect: " + clipRect);
	//new Exception().printStackTrace();
	
    }

    public void growClipRect(int x, int y) {
	clipRect.grow(x, y);
    }

    public void addHandlesToClipRect(Rectangle r) {
	r.grow(WidgetDrag.CORNER_WIDTH, WidgetDrag.CORNER_WIDTH);
	addToClipRect(r);
    }

    public void addHandlesToClipRect(Widget w) {
	Rectangle r = new Rectangle(w.position);
	r.grow(WidgetDrag.CORNER_WIDTH, WidgetDrag.CORNER_WIDTH);
	addToClipRect(r);
    }

    public void addWidget(Widget w) {
	//System.out.println("adding widget: " + w + ", cliprect = " + clipRect);
	if (!app.kioskp && !app.locked && YoYo.applet == null) {
	    wh.addWidget(w);

	    // how implement new data structure with applet?
	} else wh.addWidget(w);

	addToClipRect(w.position);
	//System.out.println("adding widget: " + w);
	if (w instanceof ButtonWidget) { buttons.addElement(w); return; }
	if (w instanceof SliderWidget) { sliders.addElement(w); return; }
	if (w instanceof MonitorWidget) { monitors.addElement(w); return; }
	if (w instanceof LineWidget) { linewidgets.addElement(w); return; }
	if (w instanceof TextWidget) { textwidgets.addElement(w); return; }
	//	if (w instanceof ColorLegendWidget) { legends.addElement(w); return; }
	//    w.update();
	//System.out.println("adding widget: " + w);
	//System.out.println("clip rect = " + clipRect);
	app.project.markChanged();
    }

    public void removeWidget(Widget wd) {
	if (wd.isSelected()) {
	    addHandlesToClipRect(wd);
	} else addToClipRect(wd.position);
	wd.delete();
  	wh.removeWidget(wd);
	app.project.markChanged();
    }

    /**
     * returns true iff <under> is covered by another widget
     */
    public boolean isObscured(Widget under) {
	//System.out.print("is Obscured? " + under);
	//System.out.println("top of isObscured()");
	for (Enumeration e = wh.listAllWidgetsTopDown(); e.hasMoreElements(); ) {
	    Widget w = (Widget)e.nextElement();
	    // we're going top down. Once we've hit the under widget, nothing else can obscure it.
	    if (w == under) break;
	    Rectangle r;
	    if (w.isSelected()) {
		r = new Rectangle(w.position);
		r.grow(WidgetDrag.CORNER_WIDTH, WidgetDrag.CORNER_WIDTH);
	    } else {
		r = w.position;
	    }
	    if (under.position.intersects(r)) {
		//System.out.println("\nreturning true from isObscured. under = " + under);
		return true;
	    }
	}
	//System.out.println(" false");
	return false;
    }

    public Object[] getSliders() {
	Object[] output;

	Vector sliderinfo = new Vector();
	for(Enumeration e = sliders.elements(); e.hasMoreElements(); ) {
	    SliderWidget sw = (SliderWidget)e.nextElement();
	    sliderinfo.addElement(sw.var);
	    sliderinfo.addElement(new Integer(sw.id));
	}
	output = new Object[sliderinfo.size()];
	sliderinfo.copyInto(output);
	return output;
    }

    public Object getSlider(int num) {
	for(Enumeration e = sliders.elements(); e.hasMoreElements();) {
	    SliderWidget sw = (SliderWidget)e.nextElement();
	    if (sw.id == num) return sw;
	}
	return YoYo.symfalse;
    }

    public Object getButton(int num) {
	for(Enumeration e = buttons.elements(); e.hasMoreElements();) {
	    ButtonWidget bw = (ButtonWidget)e.nextElement();
	    if (bw.id == num) return bw;
	}
	return YoYo.symfalse;
    }
	
	 public Object getButtonByLetter(char letter) {
	for(Enumeration e = buttons.elements(); e.hasMoreElements();) {
	    ButtonWidget bw = (ButtonWidget)e.nextElement();
		if (bw.keyboard.length() > 0)
		{
		if (bw.keyboard.charAt (0) == letter) return bw;
		}
		/*if (Character.getType(bw.name.charAt (0)) == Character.START_PUNCTUATION)
			{if (bw.name.charAt (1) == letter) return bw;}
		else
			{if (bw.name.charAt (0) == letter) return bw;}*/
	}
	return YoYo.symfalse;
    }

    public Object getMonitor(int num) {
	for(Enumeration e = monitors.elements(); e.hasMoreElements();) {
	    MonitorWidget mw = (MonitorWidget)e.nextElement();
	    if (mw.id == num) return mw;
	}
	return YoYo.symfalse;
    }

    public Object getLineWidget(int num) {
	for(Enumeration e = linewidgets.elements(); e.hasMoreElements();) {
	    LineWidget lw = (LineWidget)e.nextElement();
	    //System.out.println("~~NUM: " + num + "_" + lw.id);
	    if (lw.id == num) return lw;
	}
	return YoYo.symfalse;
    }

    public Object getLineWidgetFromTitle(String title) {
	for(Enumeration e = linewidgets.elements(); e.hasMoreElements();) {
	    LineWidget lw = (LineWidget)e.nextElement();
	    //System.out.println("~~~title: " + title + " lw.title: " + lw.tbc.l.getTitle());
	    // if (lw.tbc.getTitle().equals(title)) return lw;
			if (JSLMenu.getMenuName(lw.tbc).equals(title)) return lw;
	}
	return YoYo.symfalse;
    }

    public LineWidget getOldestLineWidget() {
	LineWidget tmp = null;
	for (int i=0; i<linewidgets.size(); i++) {
	    tmp = (LineWidget)linewidgets.elementAt(i);
	    if (!tmp.wizardOn) {
		return tmp;
	    }
	}
	return null;
    }

    public Object getTextWidget(int num) {
	for(Enumeration e = textwidgets.elements(); e.hasMoreElements();) {
	    TextWidget tw = (TextWidget)e.nextElement();
	    if (tw.id == num) return tw;
	}
	return YoYo.symfalse;
    }

    public MonitorWidget[] getMonitors() {
	MonitorWidget[] output = new MonitorWidget[monitors.size()];
	monitors.copyInto(output);
	return output;
    }

    public ButtonWidget[] getButtons() {
	//System.out.println("buttons: " + YoYo.printToString(buttons));
	ButtonWidget[] output = new ButtonWidget[buttons.size()];
	buttons.copyInto(output);
	return output;
    }

    public LineWidget[] getLineWidgets() {
	LineWidget[] output = new LineWidget[linewidgets.size()];
	linewidgets.copyInto(output);
	return output;
    }

    public TextWidget[] getTextWidgets() {
	TextWidget[] output = new TextWidget[textwidgets.size()];
	textwidgets.copyInto(output);
	return output;
    }

    public Object[] getPreButtons() {
	Object[] output;

	Vector buttoninfo = new Vector();
	for(Enumeration e = buttons.elements(); e.hasMoreElements(); ) {
	    ButtonWidget bw = (ButtonWidget)e.nextElement();
	    buttoninfo.addElement(bw.name);
	    buttoninfo.addElement(new Integer(bw.id));
	}
	output = new Object[buttoninfo.size()];
	buttoninfo.copyInto(output);
	return output;
    }

    public int getHighestButtonID() {
	int highest = 0;
	for (Enumeration e = buttons.elements(); e.hasMoreElements();) {
	    ButtonWidget bw = (ButtonWidget)e.nextElement();
	    if (bw.id > highest) highest = bw.id;
	}
	return highest;
    }

    private class CompileWidgetSetupRunner implements Runnable {
	private ProgressBar pb;
	public CompileWidgetSetupRunner(ProgressBar pb) {
	    pb = pb;
	}
	public void run() {
	    int count = numButtons() + numMonitors() + numLineWidgets();
	    pb.addToLimit(count);
	}
    }

    private class CompileWidgetUpdateRunner implements Runnable {
	private ProgressBar pb;
	private String s;
	public CompileWidgetUpdateRunner(ProgressBar pb, String s) {
	    pb = pb;
	    s = s;
	}
	public void run() {
	    pb.update(s);
	}
    }

    public void compileWidgets(ProgressBar pb) {
	//System.out.println("compile widgets");
	//System.out.println("compile buttons " + YoYo.printToString(buttons));
	//SwingUtilities.invokeLater(new CompileWidgetSetupRunner(pb));
	for(Enumeration e = buttons.elements(); e.hasMoreElements(); ) {
	    ButtonWidget bw = (ButtonWidget)e.nextElement();
	    //  SwingUtilities.invokeLater(new CompileWidgetUpdateRunner(pb, "Compiling Button " + bw.name));
	    bw.compileCommand();
	}
	for(Enumeration e = monitors.elements(); e.hasMoreElements(); ) {
	    MonitorWidget mw = (MonitorWidget)e.nextElement();
	    //SwingUtilities.invokeLater(new CompileWidgetUpdateRunner(pb, "Compiling Monitor " + mw.name));
	    mw.compileCommand();
	}
	for(Enumeration e = linewidgets.elements(); e.hasMoreElements(); ) {
	    LineWidget lw = (LineWidget)e.nextElement();
	    //SwingUtilities.invokeLater(new CompileWidgetUpdateRunner(pb, "Compiling Plot " + lw.id));
	    lw.compileCommand();
	}
    }

    public int numButtons() {
	return buttons.size();
    }

    public int numMonitors() {
	return monitors.size();
    }

    public int numMonitorsOn() {
	int count = 0;
	for(Enumeration e = monitors.elements(); e.hasMoreElements(); ) {
	    if ( ((MonitorWidget)e.nextElement()).sp != null) {
		count++;
	    }
	}
	return count;
    }

    public void clearLineWidgets() {
	for (Enumeration e = linewidgets.elements(); e.hasMoreElements(); ) {
	    LineWidget lw = (LineWidget)e.nextElement();
	    lw.reset();
	}
    }

    public void closeWidgetWindows() {
	for (Enumeration e = linewidgets.elements(); e.hasMoreElements(); ) {
	    LineWidget lw = (LineWidget)e.nextElement();
	    lw.tbc.setVisible(false);
	}
    }

    public int numLineWidgets() {
	return linewidgets.size();
    }

    public int numLineWidgetsOn() {
	int count = 0;
	for(Enumeration e = linewidgets.elements(); e.hasMoreElements(); ) {
	    if ( ((LineWidget)e.nextElement()).sp != null) {
		count++;
	    }
	}
	return count;
    }

    public void erasePointsOnAllWidgets(int who) {
	LineWidget lw = null;
	for (Enumeration e = linewidgets.elements(); e.hasMoreElements(); ) {
	    lw = (LineWidget)e.nextElement();
	    lw.erasePoint(who);
	}
    }

    public String getSaveString(boolean htmlp) throws IOException {
	StringBuffer result = new StringBuffer("\r\n\r\n`interface`\r\n");
	Widget w;
	for (Enumeration e = wh.listAllWidgets(); e.hasMoreElements(); ) {
	    result.append(((Widget)e.nextElement()).getSaveString(htmlp));
	}
	return(result.toString());
    }

    public void reset() {
	/**
	   if (retrofit != null)
	   retrofit.tbc.pma.setVisible(false);
	   retrofit = null;
	**/
	closeWidgetWindows();
	//	clearLineWidgets();
	clearSelections();
	Widget tb = null;
	//  if (YoYo.applet == null) tb = wh.getToolbarWidget();
	//  Widget pc = wh.getPatchCanvas();
	wh.removeAllWidgets();
	buttons.removeAllElements();
	sliders.removeAllElements();
	monitors.removeAllElements();
	legends.removeAllElements();
	linewidgets.removeAllElements();
	textwidgets.removeAllElements();
	if (!app.kioskp && YoYo.applet == null) 
	    frame.getMenu().removeAllPlotWindows();

	//   if (YoYo.applet == null) setToolbarWidget(tb);
	//  addWidget(pc);
	//((PatchCanvas)pc).sl.resetCallers();
	ButtonWidget.idCount = ButtonWidget.FIRST_BUTTON_ID;
	SliderWidget.idCount = 0;
	MonitorWidget.idCount = 1;
	ColorLegendWidget.idCount =1;
	LineWidget.idCount = 2;
	TextWidget.idCount = 1;
	
    }


    public void loadFromString(String str, boolean htmlp) {
	//System.out.println("load from string");
	String title = "`interface`";
	int start = str.indexOf(title);
	if (start < 0) return;
	int end = str.indexOf("`", start+title.length());
	if (end < 0) end = str.length();
	str = str.substring(start + title.length(), end);

	reset();

	//System.out.println("found string: " +str);
	//System.out.println("yoyo applet? " + YoYo.applet);
	clipRect = new Rectangle();
	//System.out.println("laodFromString: clipRect = " + clipRect);
	readWidgets(str, htmlp);
	repaint(clipRect);
    }

    protected void readWidgets(String str, boolean htmlp) {
	//System.out.println("readWidgets: clipRect = " + clipRect);
	String[] newwidgets = { "SLButton", "SLSlider", "SLMonitor", "SLCanvas", "SLTextWidget","SLLineWidget","SLLegend" };
	String[] oldwidgets = { "BUTTON", "SLIDER", "MONITOR" };

	LineNumberReader lnr = new LineNumberReader(new StringReader(str));

	try {
	    String thing = lnr.readLine();
	    while (thing != null) {
		//System.out.println("read line: " + thing);
		String thingtrim = thing.trim();
		//System.out.println("trimmed line: " + thingtrim);
		String which = startswithp(thingtrim, newwidgets);
		if (which == null) {
		    //System.out.println("not a new widget");
		    which = startswithp(thingtrim, oldwidgets);
		    if (which == null) {
			//System.out.println("not an old widget");
			thing = lnr.readLine();
			continue;
		    }
		    //System.out.println("which: " + which);
		    //// old widgets
		    thing = lnr.readLine();
		    //System.out.println("thing: " + thing);
		    if (thing == null) return;
		    thingtrim = thing.trim();
		    StringBuffer buf = new StringBuffer();
		    while(startswithp(thingtrim, oldwidgets) == null) {
			buf.append(thingtrim + "\r\n");
			thing = lnr.readLine();
			if (thing == null) break;
			thingtrim = thing.trim();
		    }
		    //System.out.println("read buf: " + buf);
		    Tokenizer tokenizer = new Tokenizer(buf.toString());
		    ArrayList items = tokenizer.tokenize();
		    removeQuotes(items);
		    if (which.equals("BUTTON")) {
			//System.out.println(buf);
			addWidget(new ButtonWidget(items, 0, this, sl));
		    } else if (which.equals("SLIDER")) {
			//System.out.println(buf);
			addWidget(new SliderWidget(items, 0, this));
		    } else if (which.equals("MONITOR")) {
			//System.out.println(buf);
			addWidget(new MonitorWidget(items, 0, this, sl));
		    }
		    //thing = lnr.readLine();
		    continue;
		}

		thing = lnr.readLine();
		if (thing == null) return;
		thingtrim = thing.trim();
		StringBuffer buf = new StringBuffer();
		while(startswithp(thingtrim, newwidgets) == null) {
		    buf.append(thingtrim + "\r\n");
		    thing = lnr.readLine();
		    if (thing == null) break;
		    thingtrim = thing.trim();
		}
		Tokenizer tokenizer = new Tokenizer(buf.toString());
		ArrayList items = tokenizer.tokenize();
		removeQuotes(items);

		if (which.equals("SLButton")) {
		    addWidget(ButtonWidget.load(items, this, sl, htmlp));
		} else if (which.equals("SLSlider")) {
		    addWidget(SliderWidget.load(items, this, sl));
		} else if (which.equals("SLMonitor")) {
		    addWidget(MonitorWidget.load(items, this, sl, htmlp));
		}
		else if (which.equals("SLLegend")) {
		    addWidget(ColorLegendWidget.load(items, this, sl, htmlp));
  		}
		else if (which.equals("SLTextWidget")) {
		    addWidget(TextWidget.load(items, this, sl));
		} else if (which.equals("SLCanvas")) {
		    sl.patchcanvas.load(items, this, sl);
		} else if (which.equals("SLLineWidget")) {
		    addWidget(LineWidget.load(items, this, sl, htmlp));
		}
		continue;
	    }
	}
	catch (IOException e) { e.printStackTrace(); }
	catch (Exception f) {f.printStackTrace(); }
	//repaint();
    }

    public static String startswithp(String s, String[] list) {
	for(int i = 0; i<list.length; i++) {
	    if (s.startsWith(list[i])) return list[i];
	}
	return null;
    }

    public static void removeQuotes(ArrayList v) {
	String str;
	for (int i=0; i<v.size(); i++) {
	    if (v.get(i) instanceof String) {
		str = (String)v.get(i);
		if (str.startsWith("\"")&&str.endsWith("\""))
		    v.set(i, str.substring(1, str.length()-1));
	    }
	}
    }

    public static String removeQuotes(String str) {
	if (str.startsWith("\"")&&str.endsWith("\"")) {
	    return str.substring(1, str.length()-1);
	} else return str;
    }

    public void setCellScroll(int size) {
    	if (!app.kioskp && YoYo.applet == null) cellsizesliderwidget.setValueNoResize(size);
    }

    private void loadImages() {
	String gifdir = "gifs/";
	try {
	    Toolkit kit = Toolkit.getDefaultToolkit();
	    MediaTracker tracker = new MediaTracker((Component)this);
	    if (!app.kioskp && YoYo.applet == null) {
		ToolbarWidget.loadImages(gifdir, tracker, kit);
		Painttool.loadImages(gifdir, tracker, kit);
		PaintSelect.loadImages(gifdir, tracker, kit);
		JAboutDialog.loadImages(gifdir, tracker, kit);
		JPlotDialog.loadImages(gifdir, tracker, kit);
		StopWidget.loadImages(gifdir, tracker, kit);
		TrashWidget.loadImages(gifdir, tracker, kit);
		JLineDialogPanel.loadImages(gifdir, tracker, kit);
		JShapeEditorSizeChooserPanel.loadImages(gifdir, tracker, kit);
		JShapePaintSelect.loadImages(gifdir, tracker, kit);
		JShapeRotateMenu.loadImages(gifdir, tracker, kit);
		JShapeEditorPanel.loadImages(gifdir, tracker, kit);
		JShapeEditorRotatePanel.loadImages(gifdir, tracker, kit);
	    }
	    ButtonWidget.loadImages(gifdir, tracker, kit);
	    SliderWidget.loadImages(gifdir, tracker, kit);
	    MonitorWidget.loadImages(gifdir, tracker, kit);
	    LineWidget.loadImages(gifdir, tracker, kit);
	    ColorLegendWidget.loadImages(gifdir, tracker, kit);
	    try {
		tracker.waitForAll();
	    } catch (InterruptedException ie) {}
	} catch (Exception e) {
	    app.err.println("Error loading images.");
	    e.printStackTrace(app.err);
	    if (app.err != app.log) {
		app.log.println("Error loading images.");
		e.printStackTrace(app.log);
	    }
	}
    }


    public void itemStateChanged(ItemEvent evt) {
	if(YoYo.macintoshp && !YoYo.macosxp) {
	    // Get around a MacOS 9 bug
	    CheckboxMenuItem cmi = (CheckboxMenuItem)evt.getSource();
	    String arg = cmi.getLabel();
	    
	    if (arg.equals(app.textBundle.getString("edit_menu_snap_to_grid"))) {
		    snapToGrid = !snapToGrid; // cmi.getState();
		    return;
	    }
	      if (arg.equals(app.textBundle.getString("edit_menu_lock_interface"))) {
		  Runnable r = new Runnable() {
			public void run() {
			}
		 };
		       app.maybeCompileAndThenRunElseRunIfFailedAlotRun(false, r, r, r);
		       //System.out.println("locking");
		     app.locked = !app.locked;
			 repaint();
			 
		    return;
	    }
	    try {
		int value = Integer.parseInt(arg);
		Font current = getFont();
		setFont(new Font(current.getName(), current.getStyle(), value));
		frame.getMenu().setCurrentSize(cmi);
		return;
	    }
	    catch (Exception e) {
		Font current = getFont();
		setFont(new Font(arg, current.getStyle(), current.getSize()));
		frame.getMenu().setCurrentFont(cmi);
		return;
	    }
	} else {
	    if (evt.getSource() instanceof JCheckBoxMenuItem) {
		JCheckBoxMenuItem cmi = (JCheckBoxMenuItem)evt.getSource();
		String arg = cmi.getLabel();
		//System.out.println("item state changed: " + arg);
		if (arg.equals(app.textBundle.getString("edit_menu_snap_to_grid"))) {
		    snapToGrid = !snapToGrid; // cmi.getState();
		    return;
		}
		if (arg.equals(app.textBundle.getString("edit_menu_lock_interface"))) {
		Runnable r = new Runnable() {
			public void run() {
			}
		 };
		       app.maybeCompileAndThenRunElseRunIfFailedAlotRun(false, r, r, r);
		       //System.out.println("locking");
			    try {
		         Thread.currentThread().sleep(100);
		          } catch (InterruptedException ie) {}

				app.locked = !app.locked; // cmi.getState();
				repaint();
				return;
		}
	    }
	    
	    if (evt.getSource() instanceof JRadioButtonMenuItem) {
		JRadioButtonMenuItem cmi = (JRadioButtonMenuItem)evt.getSource();
		String arg = cmi.getLabel();
		
		// on MacOS X these menu items are radiobutton menu items, to get around a bug
		if (arg.equals(app.textBundle.getString("edit_menu_snap_to_grid"))) {
		    snapToGrid = !snapToGrid; 
		    return;
		}
			if (arg.equals(app.textBundle.getString("edit_menu_lock_interface"))) {
			
			Runnable r = new Runnable() {
			public void run() {
			}
		 };
		       app.maybeCompileAndThenRunElseRunIfFailedAlotRun(false, r, r, r);
		       //System.out.println("locking");
			    try {
		         Thread.currentThread().sleep(100);
		          } catch (InterruptedException ie) {}
		   app.locked = !app.locked; 
		   repaint();
		    return;
		}
		try {
		    int value = Integer.parseInt(arg);
		    Font current = getFont();
		    setFont(new Font(current.getName(), current.getStyle(), value));
		    frame.getMenu().setCurrentSize(cmi);
		    return;
		}
		catch (Exception e) {
		    Font current = getFont();
		    setFont(new Font(arg, current.getStyle(), current.getSize()));
		    frame.getMenu().setCurrentFont(cmi);
		    return;
		}
	    }
	}
    }
    

    public void actionPerformed(ActionEvent e) {
	Object o = e.getSource();
	String arg = "";
	if (o instanceof JMenuItem) {
	    JMenuItem mi = (JMenuItem)o;
	    arg = mi.getLabel();
	} else if (o instanceof MenuItem) {
	    MenuItem mi = (MenuItem)o;
	    arg = mi.getLabel();
	}
	if (arg.equals(app.textBundle.getString("edit_menu_undo"))) {
	    undo(true);
	    return;
	}

	if (arg.equals(app.textBundle.getString("edit_menu_redo"))) {
	    redo(true);
	    return;
	}

	if (arg.equals(app.textBundle.getString("edit_menu_cut"))) {
	    WidgetDrag[] wda = wh.getSelectedWidgets(false);
	    if (wda == null) {
		deselectAll(true);
		return;
	    }
	    wclipboard.clear();
	    wclipboard.setWidgets(wda);
	    deleteSelections(true);
	    wclipboard.setState(WidgetClipboard.CUT);
	    frame.getMenu().enablePasteMenu();
	    return;
	}

	if (arg.equals(app.textBundle.getString("edit_menu_copy"))) {
	    WidgetDrag[] wda = wh.getSelectedWidgets(false);
	    if (wda == null) {
		deselectAll(true);
		return;
	    }
	    wclipboard.clear();
	    wclipboard.setWidgets(wda);
	    wclipboard.setState(WidgetClipboard.COPY);
	    frame.getMenu().enablePasteMenu();
	    return;
	}

	if (arg.equals(app.textBundle.getString("edit_menu_paste"))) {
	    pasteWidgets(true);
	    return;
	}

	if (arg.equals(app.textBundle.getString("edit_menu_clear"))) {
	    if (selecting()==true) {
		deleteSelections(true);
	    }
	    return;
	}
	if (arg.equals(app.textBundle.getString("edit_menu_select_all"))) {
	    selectAll(true);
	    return;
	}
    }

    public void pasteWidgets(boolean menup) {
	// return if there is nothing to paste
	if (wclipboard.empty()) return;
	if (menup) { clipRect = new Rectangle(); }
	// deselect any selected widgets
	clearSelections();

	WidgetDrag[] clipb = wclipboard.getWidgets();
	int edition = wclipboard.getCopyNumber();

	int inCorner = 0;
	for (int i=0; i<clipb.length; i++) {
	    WidgetDrag w = clipb[i];
	    WidgetDrag wi = null;

	    // set up where it should be placed
	    Rectangle r;
	    if (wclipboard.getState() == WidgetClipboard.CUT) {
		r = w.position;
	    } else {
		r = new Rectangle(w.position.x+(edition-1)*20, w.position.y+(edition-1)*20, w.position.width, w.position.height);
	    }
	    if (!((getBoundaries().contains(r.x,r.y)) && (getBoundaries().contains(r.x+r.width,r.y+r.height)))) {
		// if it doesn't fit, throw it in the upper left
		r.x = getBoundaries().x + 20*inCorner;
		r.y = getBoundaries().y + 20*inCorner;
		inCorner++;
	    }
	    if (snapToGrid) w.snapToGrid();

	    // make the WidgetDrag to be pasted
	    if (wclipboard.getState() == WidgetClipboard.CUT) {
		wi = w;
	    } else {
		wi = w.copy(edition,r);
	    }

	    addWidget(wi);
	    select(wi);
	    addHandlesToClipRect(wi);
	}
	// put in correct state
	wclipboard.setState(WidgetClipboard.PASTE);
	frame.getMenu().enableCutCopyClearMenu();
	//repaint();
	if (menup) { repaint(clipRect); } //clipRect = null; }
	app.rancompilerp = false;
	app.rancompilerpForVariables = false;
	//app.project.compile(app);
	app.project.markChanged();
    }

    public boolean selecting() {
	return wh.hasSelections();
    }

    public void print() {
	Print print = new Print(this, "StarLogo");
	print.print();
    }

    public int print(Graphics graphics, PageFormat pageFormat, int pageIndex) {
	if (pageIndex > 0) return Printable.NO_SUCH_PAGE;

	Graphics2D g2d = (Graphics2D)graphics;
	g2d.translate (pageFormat.getImageableX (),
		       pageFormat.getImageableY ());
	g2d.translate(-toolbarwidth, - (toolbarheight + getInsets().top));
	
	g2d.setPaint (Color.black);

	Dimension ifaceSize = getVisibleSize();

	int ifaceWidth = ifaceSize.width;
	int ifaceHeight = ifaceSize.height;

	boolean tall = (ifaceWidth < ifaceHeight);

	double pageWidth = pageFormat.getImageableWidth();
	double pageHeight = pageFormat.getImageableHeight();
	
	Print.scaleGraphics(g2d, pageWidth, pageHeight,
			    ifaceWidth, ifaceHeight);
	
	print(g2d);

	return Printable.PAGE_EXISTS;
    }


    public void setStatus(String s) {
	statusbar.setStatusMessage(s);
	repaint(clipRect);
    }

    public void resetStatus() {
    	statusbar.setStatusMessage("");
    }

    public ToolbarWidget getToolbar() {
	return toolbar;
    }

    public boolean snapToGrid() {
	return snapToGrid;
    }

    public boolean resizingp() {
	return resizingp;
    }

    public boolean draggingp() {
	return draggingp;
    }

    public Widget widgetTouched() {
	return widgetTouched;
    }

    public WidgetHolder wh() {
	return wh;
    }

    public StarLogo sl() {
	return sl;
    }

    public Application app() { return app; }

    public boolean pressedShiftControlp() {
	return pressedShiftControlp;
    }
    public boolean firstresizep() { return firstresizep; }

    public int statusbarheight() {
	return statusbarheight;
    }

    public int toolbarheight() {
	return toolbarheight;
    }

    public int toolbarwidth() {
	return toolbarwidth;
    }

    public Frame frame() {
	return (Frame)frame;
    }

    public Rectangle clipRect() {
	return clipRect;
    }
    public void setClipRect(Rectangle r) { clipRect = r; }

    public Widget keyboardFocus() {
	return keyboardFocus;
    }
    public void setKeyboardFocus(Widget w) {
	keyboardFocus = w;
    }

    public PaintSelect paintselect() { return paintselect; }
    public Painttool painttool() { return painttool; }
    public Colortool colortool() { return colortool; }
    public ColorChooser colorchooser() { return colorchooser; }

    public Vector sliders() { return sliders; }
    public Vector legends() { return legends; }
    public Vector linewidgets() { return linewidgets; }
    public Vector buttons() { return buttons; }
    public Vector monitors() { return monitors; }
    public Vector textwidgets() { return textwidgets; }

    public SmartRectangle outline() { return outline; }

    public StatusToolbar statusbar() { return statusbar; }
    public TrashWidget trashwidget() { return trashwidget; }
    public SpeedSliderWidget speedsliderwidget() { return speedsliderwidget; }
    public ToolbarWidget toolbar() { return toolbar; }

	public void addPlotWindow(ptolemy.LineChart lc) {
	if (!app.kioskp && YoYo.applet == null) {
	    frame.getMenu().addPlotWindow(lc);
	}
    }

    public void removePlotWindow(ptolemy.LineChart lc) {
	if (!app.kioskp && YoYo.applet == null) {
	    frame.getMenu().removePlotWindow(lc);
	}
    }

    public void removeAllPlotWindows() {
	if (!app.kioskp && YoYo.applet == null) {
	    frame.getMenu().removeAllPlotWindows();
	}
    }


    public void changePlotWindow(String s1, String s2) {
	if (!app.kioskp && YoYo.applet == null) {
	    frame.getMenu().changePlotWindow(s1, s2);
	}
    }

    public void setTooltipWidget(WidgetLite w) {
	tooltipShown = w;
    }
    public void setBackgroundp(boolean val) {
	backgroundp = val;
    }

}

