/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.awt.Frame;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Insets;
import java.awt.Graphics;
import java.awt.MenuItem;
import java.awt.event.KeyEvent;
import java.awt.event.KeyAdapter;
import java.awt.Component;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.awt.print.PageFormat;
import java.util.Vector;
import java.util.Enumeration;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentAdapter;
import javax.swing.JFrame;
import javax.swing.WindowConstants;
import yoyo.YoYo;

public class JInterfaceFrame extends JFrame implements InterfaceFrame, StarLogoFrame, MenuListener, SLPrintable {

    public JInterface iface;
    public Application app;
  
    public SLMenu slmenu;

  public JInterfaceFrame(Application app, JInterface intface) {
    super("StarLogo");
    this.iface = intface;
    this.app = app;
    
    slmenu = new JSLMenu(this, intface, app);    
    if (!app.kioskp) {
	slmenu.setMenuBar(this);
	slmenu.disableCutCopyClearMenu();
	slmenu.disablePasteMenu();
    }

    setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
    javax.swing.ImageIcon icon = new javax.swing.ImageIcon(Pool.class.getResource("gifs/icon.gif"));
    setIconImage(icon.getImage());
    
    //setLayout(null);
    setBackground(Color.white);
    setForeground(Color.black);
    // XXX MAXG WHY? setOpaque(true);
    getContentPane().setBackground(Color.white);
    getContentPane().setForeground(Color.black);
    // XXX MAXG WHY? getContentPane().setOpaque(true);
    getContentPane().add((Component)iface);

    // set position
    setLocation(25,0);
    setSize(750, 575);

    app.registerFrameTracker(this);
    app.registerCloser(this);
    app.addWindow(this);   

    addKeyListener( new IKeyListener(iface, iface.sl()) );
/*
    addKeyListener(new KeyAdapter() {
	public void keyReleased(KeyEvent e) {
	  char key = e.getKeyChar();
	  if (((key==KeyEvent.VK_DELETE)||(key==KeyEvent.VK_BACK_SPACE))&&(iface.selecting()==true)) {
	    iface.deleteSelections(true);
	    e.consume();
	  }
	}
      });
*/

    //System.out.println("adding FocusListener");
    addFocusListener(new java.awt.event.FocusAdapter() {
	    public void focusLost(java.awt.event.FocusEvent e) {
		//System.out.println("focusLost - JIF");
		iface.setBackgroundp(false);
	    }
	});
  }

  public void setVisibleSize(int width, int height) {
    Insets in = getInsets();
    setSize(width + in.left + in.right, height + in.top + in.bottom);
    validate();
  }

	public void actionPerformed(ActionEvent e) { }
	
	public void itemStateChanged(ItemEvent e) { }

    public void print() {
	iface.print();
    }

    public int print(Graphics graphics, PageFormat pageFormat, int pageIndex) {
	return iface.print(graphics, pageFormat, pageIndex);
    }

    
  /*
  public Insets getInsets() {
    return new Insets(23,4,0,0); 
  }
  */
  
  public SLMenu getMenu() {
    return slmenu;
  }
  
  public Application getApp() {
    return app;
  }


    public void markChanged() {
	getRootPane().putClientProperty("windowModified", Boolean.TRUE);
    }

    public void clearChanged() {
	getRootPane().putClientProperty("windowModified", Boolean.FALSE);
    }
}
