/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.awt.*;
import java.awt.event.*;
import yoyo.YoYo;
import javax.swing.*;
import java.util.*;

public class JLineDialog extends JWidgetDialog implements LineDialog  {

    public static JLineDialog dialog;

    public JRadioButton line, bar, histogram, scatterplot, xyplot, pie;
    public JCheckBox wizard;
    public JLabel select;
    public ButtonGroup cbg;

    public JLineDialog(Frame parent, Application app, String title) {
	super(parent, app, title);
	init();
	initFocus(bok);
    }

    // for new LineWidgets
    public static JLineDialog create(Frame parent, String title, Application app) {
	String name = app.textBundle.getString("new_plot_label") + (LineWidget.idCount);
	if (dialog==null) {
	    //System.out.println("creating new JLineDialog...");
	    dialog = new JLineDialog(parent, app, title);
	} else {
	    dialog.setTitle(title);
	}
	dialog.addNotify();
	return dialog;
    }

    public void setupComponents() {
	select = new JLabel(app.textBundle.getString("select_type"));
		
	cbg = new ButtonGroup();
	line = new JRadioButton(app.textBundle.getString("line_chart"));
	bar = new  JRadioButton(app.textBundle.getString("bar_chart"));
	histogram = new JRadioButton(app.textBundle.getString("histogram"));
	scatterplot = new JRadioButton(app.textBundle.getString("scatter_plot"));
	xyplot = new JRadioButton(app.textBundle.getString("xy_plot"));
	pie = new JRadioButton(app.textBundle.getString("pie_chart"));
	wizard = new JCheckBox(app.textBundle.getString("plot_wizard"));
        
		
	line.setSelected(true);
	wizard.setSelected(true);

	cbg.add(line);
	cbg.add(bar);
	cbg.add(histogram);
	cbg.add(scatterplot);
	cbg.add(xyplot);
	cbg.add(pie);

	if (!YoYo.macosxp) {
	    formatLabel(line);
	    formatLabel(bar);
	    formatLabel(histogram);
	    formatLabel(scatterplot);
	    formatLabel(xyplot);
	    //formatLabel(pie);
	    formatLabel(select);
	    formatLabel(wizard);
	}
    }

    private JPanel wrapPanel(JPanel arg) {
	JPanel p = new JPanel();
	p.add(arg);
	return p;
    }

    protected void setupGBC() {
	JPanel p = new JPanel();
	p.setLayout(new GridLayout(2,3));

	JLineDialogPanel ldpLine = new JLineDialogPanel(LineWidget.LINE);
	JLineDialogPanel ldpBar = new JLineDialogPanel(LineWidget.BAR);
	JLineDialogPanel ldpHistogram = new JLineDialogPanel(LineWidget.HISTOGRAM);
	JLineDialogPanel ldpScatterplot = new JLineDialogPanel(LineWidget.XYPLOT);
	JLineDialogPanel ldpXyplot = new JLineDialogPanel(LineWidget.PHASE);

	JGridPanel lineP = new JGridPanel(true);
	lineP.setLayout(new BorderLayout());
	lineP.add(wrapPanel(ldpLine), BorderLayout.NORTH);
	lineP.add(line, BorderLayout.CENTER);
	JGridPanel barP = new JGridPanel(true);
	barP.setLayout(new BorderLayout());
	barP.add(wrapPanel(ldpBar), BorderLayout.NORTH);
	barP.add(bar, BorderLayout.CENTER);
	JGridPanel histogramP = new JGridPanel(true);
	histogramP.setLayout(new BorderLayout());
	histogramP.add(wrapPanel(ldpHistogram), BorderLayout.NORTH);
	histogramP.add(histogram, BorderLayout.CENTER);
	JGridPanel scatterplotP = new JGridPanel(true);
	scatterplotP.setLayout(new BorderLayout());
	scatterplotP.add(wrapPanel(ldpScatterplot), BorderLayout.NORTH);
	scatterplotP.add(scatterplot, BorderLayout.CENTER);
	JGridPanel xyplotP = new JGridPanel(true);
	xyplotP.setLayout(new BorderLayout());
	xyplotP.add(wrapPanel(ldpXyplot), BorderLayout.NORTH);
	xyplotP.add(xyplot, BorderLayout.CENTER);

	p.add(lineP);
	p.add(barP);
	p.add(histogramP);
	p.add(scatterplotP);
	p.add(xyplotP);
	p.add(new JGridPanel(true));
	JGridPanel p2 = new JGridPanel(false);
	JPanel buts = new JPanel();
	p2.setLayout(new BorderLayout());
	buts.setLayout(new GridLayout(1, 2, 4, 5));
	if (YoYo.pcp) {
	    buts.add(bok);
	    buts.add(bca);
	} else {
	    buts.add(bca);
	    buts.add(bok);
	}
	p2.add(buts, BorderLayout.EAST);
	p2.add(wizard, BorderLayout.WEST);

	getContentPane().setLayout(new BorderLayout());
	getContentPane().add(select, BorderLayout.NORTH);
	getContentPane().add(p, BorderLayout.CENTER);
	getContentPane().add(p2, BorderLayout.SOUTH);
	getContentPane().repaint();
    }

    protected void setupPosition() {
	setSize(360,
		(YoYo.macintoshp) ? 196 : (YoYo.linuxp) ? 196 : 196);
	initLocation();
    }

    protected boolean valid_inputs() {
	return true;
    }

    private JRadioButton getSelectedRadioButton(ButtonGroup cbg) {
	JRadioButton temp = null;
	Enumeration e = cbg.getElements();
	while (e.hasMoreElements()) {
	    temp = (JRadioButton)e.nextElement();
	    if (temp.isSelected()) {
		return temp;
	    }
	}
	return null;
    }

    // requires <wd> instanceof WidgetDrag
    public void getValues(WidgetDrag wd) {
	LineWidget lw = (LineWidget)wd;
	    JRadioButton selectedButton = getSelectedRadioButton(cbg);
	    String name = selectedButton.getLabel();
	    //System.out.println(name);
	    if (name != null) {
		if (name.equals( app.textBundle.getString("line_chart") ))
		    lw.type = LineWidget.LINE;
		else if (name.equals( app.textBundle.getString("bar_chart") ))
		    lw.type = LineWidget.BAR;
		else if (name.equals( app.textBundle.getString("histogram") )) {
		    lw.type = LineWidget.HISTOGRAM;
		}
		else if (name.equals( app.textBundle.getString("scatter_plot") ))
		    lw.type = LineWidget.XYPLOT;
		else if (name.equals( app.textBundle.getString("xy_plot") ))
		    lw.type = LineWidget.PHASE;
		//else if (name.equals( app.textBundle.getString("pie_chart") ))
		//			lw.type = LineWidget.PIE;
	    }
	    if (wizard.isSelected()) {
		lw.wizardOn = true;
	    } else {
		lw.wizardOn = false;
	    }

    }


    private class JGridPanel extends JPanel {

	boolean outline = false;

	public JGridPanel(boolean outline) {
	    super();
	    this.outline = outline;
	}

	public void paint(Graphics g) {
	    super.paint(g);
	    if (outline) {
		Dimension mySize = getSize();
		g.setColor(Color.lightGray);
		g.drawRect(1, 1, mySize.width - 1, mySize.height - 1);
	    }
	}

	public Insets getInsets() {
	    return new Insets(2, 2, 2, 2);
	}

    }//end private class

}
