/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.awt.*;
import java.awt.event.*;
import yoyo.YoYo;
import java.net.URL;
import java.net.MalformedURLException;
import java.awt.Dimension;
import javax.swing.*;

public class JLineDialogPanel extends JPanel {

    public static Image 
	linechartImage, barchartImage, histogramImage,
	scatterplotImage, xyplotImage;

    public static int WIDTH  = 30;
    public static int HEIGHT = 30;

    private int type;
    
    public JLineDialogPanel(int type) {
	this.type = type;
	setSize(WIDTH, HEIGHT);
    }

    public Dimension getPreferredSize() {
	return new Dimension(WIDTH, HEIGHT);
    }

    public void paint(Graphics g) {
	//g.setColor(Color.blue); 
	//g.fillRect(0,0, WIDTH, HEIGHT);
	//	System.out.println("type: " + type);
	g.setColor(Color.blue);
	switch(type) {
	case LineWidget.LINE:
	    if (linechartImage != null)
		g.drawImage(linechartImage, 0, 0, null);
	    break;
	case LineWidget.BAR:
	    if (barchartImage != null) 
		g.drawImage(barchartImage, 0, 0, null);
	    break;
	case LineWidget.HISTOGRAM:
	    if (histogramImage != null) 
		g.drawImage(histogramImage, 0, 0, null);
	    break;
	case LineWidget.XYPLOT:
	    if (scatterplotImage != null) 
		g.drawImage(scatterplotImage, 0, 0, null);
	    break;
	case LineWidget.PHASE:
	    if (xyplotImage != null) 
		g.drawImage(xyplotImage, 0, 0, null);
	    break;
	}//end switch
    }
  public static void loadImages(String gifdir, MediaTracker tracker, Toolkit kit)
    throws MalformedURLException {
    // requires: trackerID in Widget indexes start of empty ID #s in <tracker>

    URL u1 = ToolbarWidget.class.getResource(gifdir + "linecharticon.gif");
    if (u1 == null) throw new MalformedURLException();
    linechartImage = kit.createImage(u1);
    URL u2 = ToolbarWidget.class.getResource(gifdir + "barcharticon.gif");
    if (u2 == null) throw new MalformedURLException();
    barchartImage = kit.createImage(u2);

    URL u3 = ToolbarWidget.class.getResource(gifdir + "histogramicon.gif");
    if (u3 == null) throw new MalformedURLException();
    histogramImage = kit.createImage(u3);

    URL u4 = ToolbarWidget.class.getResource(gifdir + "scatterploticon.gif");
    if (u4 == null) throw new MalformedURLException();
    scatterplotImage = kit.createImage(u4);

    URL u5 = ToolbarWidget.class.getResource(gifdir + "xyploticon.gif");
    if (u5 == null) throw new MalformedURLException();
    xyplotImage = kit.createImage(u5);

    tracker.addImage(barchartImage, Widget.trackerID++);
    tracker.addImage(linechartImage, Widget.trackerID++);
    tracker.addImage(histogramImage, Widget.trackerID++);
    tracker.addImage(scatterplotImage, Widget.trackerID++);
    tracker.addImage(xyplotImage, Widget.trackerID++);
  }

}//end class




