/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.awt.*;
import java.awt.event.*;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;
import yoyo.*;
import javax.swing.*;

public class JMessageDialog extends JSLDialog implements MessageDialog {
	
    private JButton bok, bca;
    private JTextArea text;
    private boolean oked, canceled;
    private Application app;

    public JMessageDialog(Application app, StarLogoFrame parent, String title, String contents, 
			  boolean showOK, boolean showCancel, boolean modal, 
			  boolean shortCutsOn, int focusPref) {
	super((Frame)parent, title, modal);
	this.app = app;
	setupComponents(contents, showOK, showCancel);
	setupGBC(showOK, showCancel);
	setupPosition();
		
	if (showOK && showCancel) initOKCancel(bok, bca);
	if (showOK && (!showCancel)) initOKCancel(bok, null);
	if ((!showOK) && showCancel) initOKCancel(null, bca);
		
	if (showOK && (focusPref == OK_BUTTON)) initFocus(bok);
	if (showCancel && (focusPref == CANCEL_BUTTON)) initFocus(bca);
		
	if (shortCutsOn) initShortcuts();
    }
	
    public JMessageDialog(Application app, StarLogoFrame parent, String title) {
	this(app, parent, title, "");
    }
	
    public JMessageDialog(Application app, StarLogoFrame parent, String title, String contents) {
	this(app, parent, title, contents, true, false, true, true, OK_BUTTON);
    }


    public JMessageDialog(Application app, String title, String contents, 
			  boolean showOK, boolean showCancel, boolean modal, 
			  boolean shortCutsOn, int focusPref) {
	super(title, modal);
	this.app = app;
	setupComponents(contents, showOK, showCancel);
	setupGBC(showOK, showCancel);
	setupPosition();
		
	if (showOK && showCancel) initOKCancel(bok, bca);
	if (showOK && (!showCancel)) initOKCancel(bok, null);
	if ((!showOK) && showCancel) initOKCancel(null, bca);
		
	if (showOK && (focusPref == OK_BUTTON)) initFocus(bok);
	if (showCancel && (focusPref == CANCEL_BUTTON)) initFocus(bca);
		
	if (shortCutsOn) initShortcuts();
    }
	
    public JMessageDialog(Application app, String title) {
	this(app, title, "");
    }
	
    public JMessageDialog(Application app, String title, String contents) {
	this(app, title, contents, true, false, true, true, OK_BUTTON);
    }

    
    private void setupComponents(String contents, boolean showOK, boolean showCancel) {
	if (showOK) {
	    bok = new JButton(app.textBundle.getString("ok_label"));
	    if (!YoYo.macosxp) formatLabel(bok);
	}
	
	if (showCancel) {
	    bca = new JButton(app.textBundle.getString("cancel_label"));
	    if (!YoYo.macosxp) formatLabel(bca);
	}

	text = new JTextArea(contents, 6, 30); 
	text.setEditable(false);
	text.setBackground(Color.white);
	text.setForeground(Color.black);
	
	if (!YoYo.macosxp) formatLabel(text);
	text.setLineWrap(true);
	text.setWrapStyleWord(true);
    }
	
    private void setupGBC(boolean showOK, boolean showCancel) {
	GridBagLayout gbl = new GridBagLayout();
	getContentPane().setLayout(gbl);
	
	GridBagConstraints gbc = new GridBagConstraints();
	gbc.weighty = 100;
	gbc.weightx = 100;
	
	gbc.fill = GridBagConstraints.BOTH;
	gbc.anchor = GridBagConstraints.WEST;
	gbc.insets = new Insets(13, 13, 5, 13);
	gbc.gridx=0;
	gbc.gridy=0;
	gbc.gridwidth=2;
	gbc.gridheight=1;
	getContentPane().add(new JScrollPane(text, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, 
					     JScrollPane.HORIZONTAL_SCROLLBAR_NEVER), gbc);
		
	if (showOK && (!showCancel)) {
	    Graphics g= getGraphics();
	    assert (g != null);
	    FontMetrics fm=g.getFontMetrics();
	    gbc.ipadx = (fm.stringWidth(app.textBundle.getString("cancel_label")) -
			 fm.stringWidth(app.textBundle.getString("ok_label")))/2;
	    g.dispose();
	    gbc.fill = GridBagConstraints.NONE;
	    gbc.anchor = GridBagConstraints.EAST;
	    gbc.insets = new Insets(5, 13, 13, 13);
	    gbc.weighty=0;
	    gbc.gridx=1;
	    gbc.gridy=1;
	    getContentPane().add(bok, gbc);
	    getRootPane().setDefaultButton(bok);
	}

	if ((!showOK) && showCancel) {
	    gbc.fill = GridBagConstraints.NONE;
	    gbc.anchor = GridBagConstraints.EAST;
	    gbc.insets = new Insets(5, 13, 13, 13);
	    gbc.weighty=0;
	    gbc.gridx=1;
	    gbc.gridy=1;
	    getContentPane().add(bca, gbc);
	    getRootPane().setDefaultButton(bca);
	}

	if (showOK && showCancel) {
	    gbc.fill = GridBagConstraints.NONE;
	    gbc.anchor = GridBagConstraints.EAST;
	    gbc.insets = new Insets(5, 13, 13, 13);
	    gbc.gridwidth=1;
	    gbc.weighty=0;
	    gbc.gridx=1;
	    gbc.gridy=1;
	    Panel okcapanel = new Panel(new GridLayout(1, 2, 4, 4));
	    okcapanel.add(bok);
	    okcapanel.add(bca);
	    getContentPane().add(okcapanel, gbc);
	    getRootPane().setDefaultButton(bok);
	}
    }
	
    private void setupPosition() {
	setBackground(UIManager.getColor("window"));
	setSize(500,300);
	initLocation();
    }
	
    public JTextArea getTextArea() {
	return text;
    }

    public void setTextAreaForStream(TEStream tes) {
	tes.setTextArea(text);
    }

    public void ok() {
	oked = true;
	super.ok();
    }

    public void cancel() {
	canceled = true;
	super.cancel();
    }

    public void kill() {
	setModal(false);
	dispose();
    }

    public boolean oked() {
	return oked;
    }

    public boolean canceled() {
	return canceled;
    }

    static MessageDialog errorDialog;

    public static void displayMessage(Application app, StarLogoFrame f, String title, String message, boolean modalp) {
	errorDialog = (f != null) ? new JMessageDialog(app, f, title, message) : 
	    new JMessageDialog(app, title, message);
	errorDialog.setModal((YoYo.unixp) ? false : modalp);
	SwingUtilities.invokeLater(new Runnable() {
		public void run() {
		    errorDialog.show();
		}
	    });
    }
}
