/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/


package starlogo;

import java.awt.*;
import java.awt.event.*;
import yoyo.YoYo;
import javax.swing.*;

public class JMonitorDialog extends JWidgetDialog implements MonitorDialog, FocusListener{
	
    private static JMonitorDialog dialog;
    
    private JComboBox digChoice, delChoice;
    private JLabel instLab, digLab, delLab;
    private JPanel butPanel;
    private JTextField instText;
    private String name;
    
    private ButtonGroup nameGroup;
    private JRadioButton instCheck, nameCheck;
    private JTextField nameText;
    private JPanel namePanel, namefieldPanel, instructionfieldPanel;
    private JLabel nameLab;
    
    //private JScrollPane descriptScroll;
    private JLabel descriptLab;
    private JTextField descriptText;
    private String descript;
    
    private JMonitorDialog(Frame parent, Application app, String title) {
	super(parent, app, title);
	init();
	initFocus(instText);
	descriptText.addFocusListener(this);
    }
    
    // for editing monitors, requires <wd> instanceof WidgetDrag
    public static JMonitorDialog create(Frame parent, String title, Application app, WidgetDrag wd) {
	MonitorWidget monitor = (MonitorWidget)wd;
	if (dialog==null) {
	    dialog = new JMonitorDialog(parent, app, title);
	} else {
	    dialog.setTitle(title);
	}
	dialog.addNotify();
	dialog.setFields(monitor.name, monitor.inst, 
			 monitor.digits, monitor.delay, monitor.showName, monitor.getToolText());
	return dialog;
    }
    
    // for new monitors
    public static JMonitorDialog create(Frame parent, Application app, String title) {
	String name = app.textBundle.getString("monitor_label") + (MonitorWidget.idCount);
	
	if (dialog==null) {
	    dialog = new JMonitorDialog(parent, app, title);
	} else {
	    dialog.setTitle(title);
	}
	dialog.addNotify();
	dialog.setFields(name, "", 0, 0.5, false, "");
	return dialog;
    }
    
    private void setFields(String name, String inst, int digits, double delay, boolean showName, String descript) {
	this.name = name;
	instText.setText(inst);
	instText.setCaretPosition(0);
	nameText.setText(name);
	nameText.setCaretPosition(0);
	digChoice.setSelectedIndex(digits);
	delChoice.setSelectedIndex((int)(10*delay)-1);
	descriptText.setText(descript);
	descriptText.setCaretPosition(0);
	if (showName) {
	    nameCheck.setSelected(true);
	}
	else {
	    instCheck.setSelected(true);
	}
    }
    
    public void setupComponents() {
	nameLab = new JLabel(app.textBundle.getString("name_label"));
	nameText = new JTextField(15);
	
	instLab = new JLabel(app.textBundle.getString("instruction_label"));
	instText = new JTextField(30);
	
	nameGroup = new ButtonGroup();
	namePanel = new JPanel(new GridLayout(2,1,5,0));
	instCheck = new JRadioButton(app.textBundle.getString("instruction_check_label"), true);
	nameCheck = new JRadioButton(app.textBundle.getString("name_check_label"), false);
	nameGroup.add(instCheck);
	nameGroup.add(nameCheck);
	namePanel.add(nameCheck);
	namePanel.add(instCheck);
	
	digLab = new JLabel(app.textBundle.getString("decimal_places"), SwingConstants.RIGHT);
	digChoice = new JComboBox(new Object[]{"0", "1", "2", "3"});
	
	delLab = new JLabel(app.textBundle.getString("update_delay_sec"), SwingConstants.RIGHT);
	//later on might consider using numberformat objects
	//if we want to get really serious about the internationalization
	delChoice = new JComboBox(new Object[]{"0.1", "0.2", "0.3", "0.4", "0.5",
					       "0.6", "0.7", "0.8", "0.9", "1.0"});
	butPanel = new JPanel();
	butPanel.setLayout(new GridLayout(2, 1, 0, 5));
	if (YoYo.pcp) {
	    butPanel.add(bok);
	    butPanel.add(bca);
	} else {
	    butPanel.add(bca);
	    butPanel.add(bok);
	}
	
	descriptLab = new JLabel(app.textBundle.getString("tooltip_label"));
	descriptText = new JTextField(30);
	//descriptText.setLineWrap(true);
	//descriptText.setWrapStyleWord(true);
	
	//descriptScroll = new JScrollPane(descriptText);
	//descriptScroll.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
	//descriptScroll.setPreferredSize(new Dimension(50,70));
	// abstract this!

	if (!YoYo.macosxp) {
	    formatText(instText);
	    formatText(nameText);
	    //		formatText(descriptText);
	    formatLabel(nameLab);
	    formatLabel(instLab);
	    formatLabel(nameCheck);
	    formatLabel(instCheck);
	    //		formatLabel(namefieldPanel);
	    //		formatLabel(instructionfieldPanel);
	    formatLabel(namePanel);
	    formatLabel(digLab);
	    formatLabel(delLab);
	    formatLabel(digChoice);
	    formatLabel(delChoice);
	    formatLabel(descriptLab);
	    formatLabel(butPanel);
	    formatLabel(bok);
	    formatLabel(bca);
	}
    }

    
    protected void setupGBC() {
	GridBagLayout gbl = new GridBagLayout();
	getContentPane().setLayout(gbl);
	GridBagConstraints gbc = new GridBagConstraints();
	
	gbc.weightx = 100;
	gbc.weighty = 100;
	
	gbc.gridx=0;
	gbc.gridy=0;
	gbc.gridwidth=2;
	gbc.gridheight=1;
	gbc.weighty = 0;
	gbc.anchor = GridBagConstraints.NORTHWEST;
	gbc.fill = GridBagConstraints.NONE;
	gbc.insets = new Insets(6, 13, 0, 0);
	getContentPane().add(nameLab, gbc);
	
	gbc.gridy = 1;
	//gbc.weighty = 50;
	gbc.anchor = GridBagConstraints.WEST;
	gbc.fill = GridBagConstraints.HORIZONTAL;
	//gbc.ipady = 14;
	gbc.insets = new Insets(0, 13, 0, 0);
	getContentPane().add(nameText, gbc);
	
	//gbc.gridy = 1;
	//gbc.insets = new Insets(0, 13, 0, 6);
	//add(nameText, gbc);        
	gbc.gridy = 2;
	gbc.gridwidth = 2;
	//gbc.weighty = 0; 
	//gbc.ipady = 0;
	gbc.anchor = GridBagConstraints.WEST;
	gbc.fill = GridBagConstraints.NONE;
	gbc.insets = new Insets(6, 13, 0, 0);
	getContentPane().add(instLab, gbc);
	
	gbc.gridy = 3;
	gbc.gridwidth = 4;
	//gbc.weighty = 50;
	//gbc.ipady = 14;
	gbc.fill = GridBagConstraints.HORIZONTAL;
	gbc.insets = new Insets(0, 13, 0, 13);
	getContentPane().add(instText, gbc);
	
	gbc.gridy = 4;
	gbc.gridwidth = 2;
	//gbc.weighty = 0; 
	//gbc.ipady = 0;
	gbc.anchor = GridBagConstraints.WEST;
	gbc.fill = GridBagConstraints.NONE;
	gbc.insets = new Insets(6, 13, 0, 0);
	getContentPane().add(descriptLab, gbc);
	
	gbc.gridy = 5;
	gbc.gridwidth = 4;
	//gbc.weighty = 50;
	//gbc.ipady = 14;
	gbc.fill = GridBagConstraints.HORIZONTAL;
	gbc.insets = new Insets(0, 13, 0, 13);
	getContentPane().add(descriptText, gbc);
	
	//gbc.ipady = 0;
	gbc.gridx = 0;
	gbc.gridy = 6;
	gbc.gridwidth=1;
	gbc.gridheight = 1;
	//gbc.weighty = 0;
	gbc.anchor=GridBagConstraints.WEST;
	gbc.fill = GridBagConstraints.NONE;
	gbc.insets = new Insets(13, 13, 0, 0);
	getContentPane().add(digLab, gbc);
	gbc.gridx = 1;
	gbc.insets = new Insets(13, 0, 0, 0);
	gbc.anchor = GridBagConstraints.WEST;
	getContentPane().add(digChoice, gbc);
	
        gbc.gridx = 0;
	gbc.gridy = 7;
	//gbc.weightx=0;
	gbc.anchor=GridBagConstraints.WEST;
	gbc.insets = new Insets(0, 13, 13, 0);
	//gbc.insets = new Insets(0, 13, 13, 6);
	getContentPane().add(delLab, gbc);
	gbc.gridx = 1;
	gbc.insets = new Insets(0, 0, 13, 0);
	gbc.anchor = GridBagConstraints.WEST;
	getContentPane().add(delChoice, gbc);
	
	
	gbc.gridx=3;
	gbc.gridy=6;
	gbc.gridheight=2;
	gbc.ipadx=0;
	gbc.anchor=GridBagConstraints.EAST;
	gbc.insets = new Insets(13, 0, 13, 13);
	getContentPane().add(butPanel, gbc);
	
	//gbc.weighty = 50;
	gbc.gridheight = 3;
	gbc.gridy = 0;
	gbc.gridx = 2;
	gbc.gridwidth = 2; 
	//gbc.ipady = 14;
	gbc.fill = GridBagConstraints.BOTH;
	gbc.anchor = GridBagConstraints.SOUTHWEST;
	gbc.insets = new Insets((YoYo.macintoshp) ? 13 : 6, 13, 0, 13);
	getContentPane().add(namePanel, gbc);
	pack();
	
    }
    
    protected void setupPosition() {
	setSize((YoYo.macintoshp) ? 380 : 350, (YoYo.macosxp) ? 230 : (YoYo.macintoshp) ? 230 : (YoYo.linuxp) ? 250 : 270);
	initLocation();
    }
    
    protected boolean valid_inputs() {
	String inst = instText.getText().trim();
	if (inst.equals("")) {
	    JMessageDialog m = new JMessageDialog(app, (StarLogoFrame)parent, app.textBundle.getString("warning_label"),
						  app.textBundle.getString("sorry_specify_instruction"));
	    m.show();
	    return false;
	}
	return true;
    }
    
    // requires <wd> instanceof WidgetDrag
    public void getValues(WidgetDrag wd) {
	MonitorWidget monitor = (MonitorWidget)wd;
	monitor.inst = instText.getText().trim();
	monitor.name = nameText.getText();
	monitor.showName = nameCheck.isSelected();
	monitor.digits = digChoice.getSelectedIndex();
	monitor.font = Widget.WIDGET_FONT;
	
	monitor.delay = (delChoice.getSelectedIndex()+1)/10.0;
	//		monitor.sizeToText();
	monitor.update();
    	if (!(ToolTipWidget.empty(descriptText.getText())))
	    monitor.setToolTip(new ToolTipWidget(monitor.parent, descriptText.getText()));
    	else monitor.removeToolTip();
    }
    
    public boolean shortcuts = true;
    
    public void focusGained(FocusEvent e) {
	shortcuts = false;
    }
    
    public void focusLost(FocusEvent e) {
	shortcuts = true;
    }
    
    public boolean shortcutsOn() { return shortcuts; }
}
