/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.util.Vector;
import java.util.Observable;
import java.util.Random;
import java.util.Enumeration;
import yoyo.*;
import java.util.Observer;
import java.awt.datatransfer.*;
import java.io.LineNumberReader;
import java.io.PrintWriter;
import java.io.StringReader;
import java.io.IOException;
import java.util.Hashtable;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.undo.*;
import javax.swing.border.LineBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.BevelBorder;
import java.awt.CardLayout;

public class JPatchMonitor extends JEntityMonitor implements PatchMonitor {

  static Hashtable pmhash = new Hashtable();
  
  public static final int XCOR = 1, YCOR = 2, PC = 3;
  
  Patch pat;
  
  JLabel color_lbl = new JLabel("patch_color_label", JLabel.LEFT);
  JTextField color_txt = new JTextField();
  double colorval;

  JLabel xcor_lbl = new JLabel("xcor_label", JLabel.LEFT);
  JLabel xcor_txt;
  double xcorval;

  JLabel ycor_lbl = new JLabel("ycor_label", JLabel.LEFT);
  JLabel ycor_txt;
  double ycorval;

    static double delayTime = 0.5;
  static int numVars;
  public int getNumVars() {
    return numVars;
  }

  public JPatchMonitor(Patch pat, double delay, Application app)  {
    super(app.textBundle.getString("patch_label") + " " + pat.toString(), delay, app);
    this.pat = pat;

    this.com = new JSLCommand(SLCommand.PATCH, this.sl, pat, app); 
    numVars = Patch.numVars;
    pmhash.put(pat, this);
  }

  void setStateFields(Dimension d) {
    xcor_txt = new JLabel(Integer.toString((int)(pat.xcor - 1 - sl.halfwidth)), JLabel.LEFT);
    ycor_txt = new JLabel(Integer.toString((int)(sl.halfheight - (pat.ycor - 1))), JLabel.LEFT);
    color_txt.setName(app.textBundle.getString("patch_color_label"));
    
    color_txt.setPreferredSize(d);
    xcor_txt.setPreferredSize(d);
    ycor_txt.setPreferredSize(d);
    
    color_lbl.setPreferredSize(d);
    xcor_lbl.setPreferredSize(d);
    ycor_lbl.setPreferredSize(d);

    color_lbl.setText(app.textBundle.getString("patch_color_label"));
    xcor_lbl.setText(app.textBundle.getString("xcor_label"));
    ycor_lbl.setText(app.textBundle.getString("ycor_label"));

    color_lbl.setFont(LABEL_FONT);
    xcor_lbl.setFont(LABEL_FONT);
    ycor_lbl.setFont(LABEL_FONT);
    
    formatText(color_txt);
  }

  public String getVarName(int which) {
    return Patch.getVarName(which);
  }
  public int getVarPos(int which) {
    return Patch.getVarPos(which);
  }
  public Object getVarValue(int varpos) {
    return pat.turtlesown[varpos];
  }
  public boolean getVarEnump(int which) {
    return Patch.getVarEnump(which);
  }
	
  public void setValues(int which) {
    switch (which) {
    case PC:
      colorval = pat.pc;
      SwingUtilities.invokeLater(new Runnable() {
	      public void run() {
		  if (!closep) {
		      color_txt.setText(Colors.colorNumToName(colorval));
		      //color_txt.selectAll();
		      color_txt.setCaretPosition(color_txt.getText().length());
		      wholePanel.setBorder(new LineBorder(Colors.colorarray[(int)(colorval * 32.0)], 4));
		      //repaint();
		  }
	      }
	  });
      break;
    }
  }


  public void initFrame() {
    // setup frame's appearance

    //int totheight = 8 + numValues*textHeight + ccHeight;
    //System.out.println("num" + numValues + " th"+textHeight+" tot "+totheight);
    if (pat == null) return;

    if (YoYo.macintoshp) {
	slmenu = new JSLMenu(this, app);
	slmenu.setMenuBar(this);
    } 

    numVars = Patch.numVars;
    //setFont(EM_FONT);
    setFields();
    int tfheight = (own_txt == null ? 0 : own_txt[0].getPreferredSize().height);
    tfheight = 22;
    addMyComponents(tfheight);
    com.callerClearText();
    if (!app.kioskp && YoYo.applet == null) addListeners();
    synchronized(lock) {
	setValues(PC);
	for (int i=0; i< numVars; i++) setOwn(i);
    }
    //startThread();
  }

  public JLabel[] getStateLabels() {
    JLabel[] labels = { xcor_lbl, ycor_lbl, color_lbl };
    return labels;
  }

  public JComponent[] getStateTexts() {
    JComponent[] texts = { xcor_txt, ycor_txt, color_txt };
    return texts;
  }
    
  public int getNumStateVariables() {
    return 3;
  }

  void addMyComponents(int tfheight) {
    super.addMyComponents(tfheight);

    cl = new CardLayout();
    wholePanel.setLayout(cl);
    wholePanel.add(wholeBox, "alive");
    cl.show(wholePanel, "alive");
      
    getContentPane().add(wholePanel);
    addNotify();
    pack();
    xcor_txt.requestFocus();
  }    

  public double getEntityColor() {
    return pat.pc;
  }

  public static void updateOpen() {
    boolean midboxp = (numVars > 0);
    numVars = Patch.numVars;
    synchronized(pmhash) {
	for (Enumeration e = pmhash.elements() ; e.hasMoreElements() ;) {
	    JPatchMonitor pm = (JPatchMonitor)e.nextElement();
	    synchronized(pm.lock) {
		pm.updateOwn(midboxp);
	    }
	}
    }
  }

  void addListeners() {
    super.addListeners();

    xcor_txt.addFocusListener(tff);
    ycor_txt.addFocusListener(tff);

    color_txt.addKeyListener(tka);
    color_txt.addFocusListener(tff);
    
  }

    public static void flushCommandCenters() {
	synchronized(pmhash) {
	    for(Enumeration e = pmhash.elements(); e.hasMoreElements();) {
		JPatchMonitor jpm = (JPatchMonitor)e.nextElement();
		if (!jpm.closep) {
		    synchronized(jpm.lock) {
			jpm.com.flush();
		    }
		}
	    }
	}
    }

  
  public static void setPatch(Patch p) {
    JPatchMonitor pm = (JPatchMonitor) pmhash.get(p);
    if (pm != null) {
	synchronized(pm.lock) {
	    pm.pat = p;
	    p.hasPMp = true;
	    p.pm = pm;
	    pm.com.callerClearText();
	    //if (YoYo.applet == null) addListeners(); 
	    pm.setValues(PC);
	    for (int i=0; i< numVars; i++)
		pm.setOwn(i);
	    //pm.startThread();
	    pm.show();
	}
    }
  }

    public void registerCloser(Frame c, Application app) {
	c.addWindowListener(new PMCloser(app));
    }

    protected class PMCloser extends Closer {

	public PMCloser(Application app) {
	    super(app);
	}

	public void windowClosing(WindowEvent evt) {
	    synchronized(pmhash) {
		super.windowClosing(evt);
	    }
	}
    }



  public void closeMonitor() {
    if (pat != null) {
      pmhash.remove(pat);
      pat.hasPMp = false;
      pat.pm = null;
      pat = null;
      WindowPosition.closed(this);
      //tmvec.remove(this);
    }
  }

    public void run() {}

    static {
      new Thread(new Runnable() {
	      public void run() {
		  while(true) {
		      if (pmhash != null) {
			  synchronized(pmhash) {
			      for(Enumeration e = pmhash.elements(); e.hasMoreElements();) {
				  JPatchMonitor ptm = (JPatchMonitor)e.nextElement();
				  synchronized(ptm.lock) {	
				      Patch pat = ptm.pat;
				      if (pat != null && !ptm.closep) {
					  if ((ptm.colorval != pat.pc) && 
					      ((ptm.color_txt != ptm.focus) || (ptm.app.activeFrame != ptm)))
					      ptm.setValues(PC);
					  try {
					      for (int i=0; i< numVars; i++) {
						  if (pat.turtlesown.length > i &&
						      ptm.ownval.length > i &&
						      !(pat.turtlesown[i].equals(ptm.ownval[i]) || 
						    ((ptm.own_txt[i] == ptm.focus) && (ptm.app.activeFrame == ptm))))
						      ptm.setOwn(i);
					      }
					  } catch(NullPointerException ne) {}
					  catch(ArrayIndexOutOfBoundsException ae) {}
				      }
				  }
			      }
			  }
		      }
		      try {
			  Thread.sleep((long)(delayTime * 1000));
		      } catch (InterruptedException e) { }
		  }
	      }
	  }).start();
    }
  
  String getCommand(String lbl, String val) {
    //System.out.println("lbl: " + lbl);
    //System.out.println("color_txt: " + color_txt);
    if (lbl.equals(color_txt.getName())) {
      lastField = PC;
      return(app.commandsBundle.getString("setpc") + " " + val);
    } else {
      for (int i=0; i < numVars; i++) {
	if (lbl.equals(own_txt[i].getName())) {
	  lastField = i;
	  lastOwn = true;
	  return(app.commandsBundle.getString("set") + own_lbl[i].getText() + " " + val);
	}
      }
    }
    return "internal jpatchmonitor error";
  }
  
  public String getCompileCommand(String s) {
    return "compile-patch-monitor [" + s + "]";
  }
  
  public void runCompiledCommand(Object[] ilist, Object[] olist) {
    //System.out.println("pm run compiled command");
    int[] newarray = castToIntArray(ilist);
    
    if (sl.locked == true) {
	displayOneString(app.textBundle.getString("cant_run_patch_monitor") + " " +
			 app.textBundle.getString("cant_run_while_obscured"));
    } else {
	synchronized(sl.runningp) {
	    int whichip = sl.internalSetInstructions(("patch-monitor: " + xcorval + ", " + ycorval), newarray, olist);
	    StackProcessParent.runPatch(sl, whichip, pat, false, this);
	}
    }
  }
 	
  // Only called when we are reporting an error.
  public void displayOneString(String s) {
    super.displayOneString(s);
    synchronized(lock) {
	if (!(color_txt.getText().equals(Colors.colorNumToName(colorval))))
	    {setValues(PC); color_txt.selectAll();}
	for (int i=0; i< numVars; i++)
	    setOwn(i);
    }
  }
	
  public void displayAString(String s) {
    displayOneString(s);
  }
}

