/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import ptolemy.*;
import java.awt.*;
import java.util.*;
import java.net.*;
import javax.swing.*;
import yoyo.YoYo;

public class JPlotDialog extends JWidgetDialog implements PlotDialog {

    //String title="title";
    //String xlabel="time";
    //String ylabel="";
//     double xmin=0.0,xmax=100.0;
//     double ymin=0.0,ymax=100.0;
//     boolean gridOn=true;
//     boolean connected=true;

    // addition
    boolean manual=false;

    int type;
    int pts = 0; //0=pts, 1=line, 2=dots, 3 =x's

    //String ixmin, ixmax, iymin, iymax;

    public static Image linechartImage = null;
    public static Image barchartImage = null;
    public static Image histogramImage = null;
    public static Image scatterplotImage = null;
    public static Image xyplotImage = null;
    public static Image grid = null;

    PlotBox plot;
    JPanel main, top, bottom, west;
    JTextField xminT, xmaxT, yminT, ymaxT, xlabelT, ylabelT, titleT;
    JLabel xminL, xmaxL, yminL, ymaxL, xlabelL, ylabelL, titleL, delChoiceLabel;
    JCheckBox gridBox, connectedBox;//, xminAutoBox, yminAutoBox, xmaxAutoBox, ymaxAutoBox;

    // addition
    JCheckBox autoScaleBox;

    JRadioButton ptsR, noneR, dotsR, xsR;
    ButtonGroup cbg;
    JComboBox delChoice;
    LineWidget lw = null;

    public JPanel createGridPanel() {
	JPanel p = new JPanel();
	p.setLayout(new GridLayout(2,1));
	return p;
    }

    private String round(double d) {
	return ((((int)(10000 * d) / 10000.0)) + "");
    }


	
    public JPlotDialog(Frame parent, int type, PlotBox plot, LineWidget lw, Application app) {
	super(parent, app, app.textBundle.getString("plot_properties"));
	this.plot = plot;
	this.type = type;
	this.lw = lw;
	init();

	initFocus(bok);

    }

    public void setupComponents() {
	//setModal(true);

	cbg = new ButtonGroup();

	main = new JPanel();
	top = new JPanel();
	bottom = new JPanel();
	west = new JPanel();

	// setting initial plot range values

	//System.out.println("(CON1) plot.getX: " + round(plot.getXRange()[0]) + ", " + round(plot.getXRange()[1]));
	//System.out.println("(CON1) plot.getY: " + round(plot.getYRange()[0]) + ", " + round(plot.getYRange()[1]));

	xminT = new JTextField(2);
	xmaxT = new JTextField(2);
	yminT = new JTextField(2);
	ymaxT = new JTextField(2);

	xlabelT = new JTextField(6);
	ylabelT  = new JTextField(6);
	titleT = new JTextField(10);

	xminL = new JLabel(app.textBundle.getString("x_axis_min"), JLabel.CENTER);
	xmaxL = new JLabel(app.textBundle.getString("x_axis_max"), JLabel.CENTER);
	yminL = new JLabel(app.textBundle.getString("y_axis_min"), JLabel.CENTER);
	ymaxL = new JLabel(app.textBundle.getString("y_axis_max"), JLabel.CENTER);
	xlabelL = new JLabel(app.textBundle.getString("x_axis_label"), JLabel.CENTER);
	ylabelL = new JLabel(app.textBundle.getString("y_axis_label"), JLabel.CENTER);
	titleL = new JLabel(app.textBundle.getString("plot_title"), JLabel.CENTER);

	gridBox = new JCheckBox(app.textBundle.getString("show_grid"));
	connectedBox = new JCheckBox(app.textBundle.getString("connect_the_points"));

	//addition
	autoScaleBox = new JCheckBox(app.textBundle.getString("auto_scale_axes"));

	/*xminAutoBox = new JCheckBox("Calculated");
	  yminAutoBox = new JCheckBox("Calculated");
	  xmaxnAutoBox = new JCheckBox("Calculated");
	  ymaxAutoBox = new JCheckBox("Calculated");*/

	delChoice = new JComboBox(new Object[]{"0.1", "0.2", "0.3", "0.4",
					       "0.5", "1.0", "1.5", "2.0",
					       "2.5", "5.0"});
	delChoiceLabel = new JLabel(app.textBundle.getString("update_delay"));

	initializeFields();

	if (!YoYo.macosxp) {
	    formatText(xminT);
	    formatText(xmaxT);
	    formatText(yminT);
	    formatText(ymaxT);
	    formatText(xlabelT);
	    formatText(ylabelT);
	    formatText(titleT);
	    
	    formatLabel(xminL);
	    formatLabel(xmaxL);
	    formatLabel(yminL);
	    formatLabel(ymaxL);
	    formatLabel(xlabelL);
	    formatLabel(ylabelL);
	    formatLabel(titleL);
	    formatLabel(delChoice);
	    formatLabel(delChoiceLabel);
	    //gridBox.setFont(JSLDialog.LABEL_FONT);
	    formatLabel(gridBox);
	    formatLabel(connectedBox);
	    
	    //addition
	    formatLabel(autoScaleBox);
	    
	    /*formatLabel(xminAutoBox);
	      formatLabel(yminAutoBox);
	      formatLabel(xmaxAutoBox);*
	      formatLabel(ymaxAutoBox);*/
	}
    }

    public void setupGBC() {
	main.setLayout(new BorderLayout());
	west.setLayout(new BorderLayout());
	bottom.setLayout(new BorderLayout());

	JPanel titleP = new JPanel();
	titleP.setLayout( new GridLayout(2,1));
	titleP.add(titleL);
	titleP.add(titleT);
	top.add("Center", titleP);

	JPanel ymaxP = createGridPanel();
	ymaxP.add(ymaxT);
	ymaxP.add(ymaxL);
	JPanel yminP = createGridPanel();
	yminP.add(yminL);
	yminP.add(yminT);
	JPanel ymaxp = new JPanel();
	JPanel yminp = new JPanel();
	ymaxp.add(ymaxP);
	yminp.add(yminP);

	west.add("North", ymaxp);
	west.add("South", yminp);

	JPanel ylabelP = createGridPanel();
	ylabelP.add(ylabelL);
	ylabelP.add(ylabelT);
	JPanel westGrid = new JPanel();

	westGrid.add(ylabelP);

	west.add("Center", westGrid);

	JPanel westW = new JPanel();
	westW.setLayout(new GridLayout(2, 2));
	westW.add(new JPanel());
	if (type != LineWidget.BAR) westW.add(xminT);
	else westW.add(new JPanel());
	westW.add(new JPanel());
	if (type != LineWidget.BAR) westW.add(xminL);
	else westW.add(new JPanel());

	JPanel xmp = new JPanel();
	xmp.add(westW);

	bottom.add("West", xmp);

	JPanel eastE = new JPanel();
	eastE.setLayout(new GridLayout(2,1));
	if (type != LineWidget.BAR) {
	    eastE.add(xmaxT);
	    eastE.add(xmaxL);
	}
	else { eastE.add(new JPanel()); eastE.add(new JPanel()); }

	JPanel xmaxp = new JPanel();

	xmaxp.add(eastE);

	bottom.add("East", xmaxp);

	JPanel xlabelP = createGridPanel();
	xlabelP.add(xlabelT);
	xlabelP.add(xlabelL);
	JPanel bt = new JPanel();
	bt.add(xlabelP);
	bottom.add("Center", bt);
	main.add("North", top);
	getContentPane().setBackground(UIManager.getColor("window"));
	getContentPane().setLayout(new BorderLayout());
	setResizable(false);
	JPanel westP = new JPanel();	

	//System.out.println("created choice box");
	
	JPanel sth = new JPanel();
	GridBagLayout gbl = new GridBagLayout();
	GridBagConstraints gbc = new GridBagConstraints();
	sth.setLayout(gbl);

	gbc.fill = gbc.HORIZONTAL;
	gbc.weightx = 0.0;
	gbc.insets = new Insets(4,4,4,4);

	gbc.anchor = GridBagConstraints.WEST;
	gbl.setConstraints(gridBox, gbc);
	sth.add(gridBox);

	gbc.gridx = 2;
	gbc.weightx = 0.0;
	JPanel constraintsFieldPanel = new JPanel();
	constraintsFieldPanel.setLayout(new BorderLayout());
	constraintsFieldPanel.add(delChoiceLabel, BorderLayout.WEST);
	constraintsFieldPanel.add(delChoice, BorderLayout.CENTER);
	gbc.anchor = GridBagConstraints.CENTER;
	gbl.setConstraints(constraintsFieldPanel, gbc);
	sth.add(constraintsFieldPanel);

	gbc.weightx = 0.0;
	gbc.gridx = 3;
	gbc.anchor = GridBagConstraints.EAST;
	gbl.setConstraints((YoYo.pcp) ? bok : bca, gbc);
	sth.add((YoYo.pcp) ? bok : bca);

	gbc.gridy = 1;

	if ((type != LineWidget.BAR) && (type != LineWidget.HISTOGRAM))
	    {
		gbc.anchor = GridBagConstraints.WEST;
		gbc.gridx = 0;
		gbl.setConstraints(connectedBox, gbc);
		sth.add(connectedBox);
       
		gbc.gridx = 2; // next widget at position 2
		gbc.anchor = GridBagConstraints.CENTER;
	    } else {
		gbc.gridx = 0; // next widget at position 0
		gbc.anchor = GridBagConstraints.WEST;
	    }

	//addition
	gbl.setConstraints(autoScaleBox, gbc);
	sth.add(autoScaleBox);

	gbc.gridx = 3;

	gbc.anchor = GridBagConstraints.EAST;
	gbl.setConstraints((YoYo.pcp) ? bca : bok, gbc);
	sth.add((YoYo.pcp) ? bca : bok);

	main.add("South", bottom);
	main.add("West", west);

	//System.out.println("preparing draw panel...");
	//	main.add("East", new JDrawPanel(lw));

	if (type == LineWidget.LINE) {
	    main.add("East", new DrawPanel(linechartImage));
	}
	if (type == LineWidget.BAR) {
	    main.add("East", new DrawPanel(barchartImage));
	}
	if (type == LineWidget.HISTOGRAM) {
	    main.add("East", new DrawPanel(histogramImage));
	}
	if (type == LineWidget.XYPLOT) {
	    main.add("East", new DrawPanel(scatterplotImage));
	}
	if (type == LineWidget.PHASE) {
	    main.add("East", new DrawPanel(xyplotImage));
	}

	getContentPane().add("Center", new JPanel());
	getContentPane().add("West", main);
	getContentPane().add("South", sth);
	//System.out.println("one more image...");

    }

    public void setupPosition() {
	setSize(getPreferredSize());
	initLocation();
    }

    private void initializeFields() {
	xminT.setText(round(plot.getXRange()[0]));
	xmaxT.setText(round(plot.getXRange()[1]));
	yminT.setText(round(plot.getYRange()[0]));
	ymaxT.setText(round(plot.getYRange()[1]));

	xlabelT.setText(plot.getXLabel());
	ylabelT.setText(plot.getYLabel());
	titleT.setText(plot.getTitle());

	if (plot.getGrid()) gridBox.setSelected(true);

	if ((type != LineWidget.HISTOGRAM) &&
	    (type != LineWidget.BAR)) {
		  connectedBox.setSelected(((Plot)plot).getConnected());
	    autoScaleBox.setSelected(lw.tbc.autoscaleX || lw.tbc.autoscaleY);
	}
	delChoice.setSelectedItem(lw.delayStr);
    }

    public void show() {
	initializeFields();
	super.show();
    }


    public double doubleParser(String s) {
	return (new Double(s)).doubleValue();
    }

    public void getValues(LineWidget lw, PlotBox plot) {
	//sets vector contains (LineSet) objects
	plot.setTitle(titleT.getText(), lw.tbc.getPreTitle());
	plot.setXLabel(xlabelT.getText());
	plot.setYLabel(ylabelT.getText());
        lw.delayStr = (String)delChoice.getSelectedItem();
	
	manual = !(autoScaleBox.isSelected());
	lw.tbc.autoscaleX = !manual;
	lw.tbc.autoscaleY = !manual;

	plot.setXRange(doubleParser(xminT.getText()),
		       doubleParser(xmaxT.getText()), !lw.tbc.autoscaleX);
	//if ( (cymin != doubleParser(yminT.getText())) ||
	//(cymax != doubleParser(ymaxT.getText()))  )
	plot.setYRange(doubleParser(yminT.getText()),
		       doubleParser(ymaxT.getText()), !lw.tbc.autoscaleY);
	//System.out.println("setYRange to ymaxT: " + doubleParser(ymaxT.getText()));
	
	plot.setGrid(gridBox.isSelected());

	if (connectedBox.isSelected()) {
	    if (type != LineWidget.HISTOGRAM && type != LineWidget.BAR)
		lw.tbc.ppd(plot);
	} else {
	    if (type != LineWidget.HISTOGRAM && type != LineWidget.BAR)
		lw.tbc.ppu(plot);
	}
    }


    public static void loadImages(String gifdir, MediaTracker tracker, Toolkit kit)
	throws MalformedURLException {
	// requires: trackerID in Widget indexes start of empty ID #s in <tracker>
	//monEdges = kit.createImage(MonitorWidget.class.getResource(gifdir + "monitor.gif"));
	linechartImage = kit.createImage(PlotDialog.class.getResource(gifdir + "linegraphlarge.gif"));
	barchartImage = kit.createImage(PlotDialog.class.getResource(gifdir + "bargraphlarge.gif"));
	histogramImage = kit.createImage(PlotDialog.class.getResource(gifdir + "histogramlarge.gif"));
	scatterplotImage = kit.createImage(PlotDialog.class.getResource(gifdir + "scatterplotlarge.gif"));
	xyplotImage = kit.createImage(PlotDialog.class.getResource(gifdir + "xyplotlarge.gif"));
	grid = kit.createImage(PlotDialog.class.getResource(gifdir + "linegrid.gif"));
    }

    public void getValues(WidgetDrag wd) {}

    protected boolean valid_inputs() {return true;}


    class DrawPanel extends JPanel {
    Image im = null;
    Image imgrid = null;
    LineWidget lw = null;

    public DrawPanel(Image im) {

	this.im = im;
	setSize(340,185);
    }

    public Dimension getPreferredSize() {
	return new Dimension(340, 185);
    }
    public void paintComponent(Graphics g) {
	if (im != null) g.drawImage(im,0,0,this);
    }





}

}

