/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import yoyo.YoYo;
import java.awt.*;
import java.util.*;
import java.awt.event.*;
import ptolemy.*;
import javax.swing.*;

/*
 * a dialog to select graph colors and styles and edit dataset labels
 */
public class JPtolemyBarDialog extends JWidgetDialog implements PtolemyBarDialog {

    public JPanel pBig;
    public JPanel p2;
    public JPanel myPanel;
    public int idCount;
    public int type;

    public static Vector dataPanels;
    public static Vector sets;

    public static String[] turtleVars;
    public static String[] patchVars;
    public static String[] obsVars;
    public static String[] sliderVars;

    public GridBagLayout gridbag;
    public GridBagConstraints cbc;

    PSimpleCenteredPanelScrollPane sp1;
    //JScrollPane sp1;
    JInterface iface;

    //type = 0, 1, 2 => bar, line, histogram
    public JPtolemyBarDialog(Frame parent,
			     Vector sets,
			     Application app,
			     int type,
			     int idCount) {
	super(parent, app, app.textBundle.getString("plot_data"), true);
	
	this.sets = sets;
	this.app = app;
	this.idCount = idCount;
	this.type = type;

	turtleVars = ExtraInfo.getTurtleVars(app);
	patchVars = ExtraInfo.getPatchVars(app);
	obsVars = ExtraInfo.getObserverVars();

	Object[] slidersNamesAndIDs = app.iface.getSliders();
	sliderVars = new String[slidersNamesAndIDs.length/2];
	for(int i = 0; i < slidersNamesAndIDs.length; i+=2) {
	    String name = (String)slidersNamesAndIDs[i];
	    sliderVars[i/2] = name;
	}



	init();
	initFocus(bok);

    }


    public void setupComponents() {}

    public void setupPosition() {
	setSize(sp1.getPreferredSize());
	initLocation();
    }

    public void setupGBC() {
	//	setResizable(true);
	gridbag = new GridBagLayout();
	//cbc = new GridBagConstraints();
	GridBagConstraints c= new GridBagConstraints();

	pBig = new SpacePanel();
	pBig.setLayout(new GridLayout(10,1,4,0));

	dataPanels= new Vector();


	c.weightx=0;
	c.weighty=0;
	c.gridheight=1;
	c.gridwidth=c.REMAINDER;
	c.fill=c.NONE;
	c.insets= new Insets(0, 4, 0, 4);

	getContentPane().setLayout(gridbag);


	int i;
	ExtraInfo ei;
	LineSet ls;
	for (i=0; i<sets.size(); i++) {
	    ls = (LineSet)sets.elementAt(i);
	    ei = ls.ei;
	    //add one panel for each data set
	    pBig.add(makePDataSetStylePanel(parent,
					    this,
					    app,
					    idCount + i,
					    i,
					    ei,
					    ls.displayed,
					    Color.blue,
					    (LineSet)sets.elementAt(i)));
	}

	//	pBig.setBackground(SystemColor.control);

	PSingleCenteredPanel scp = new PSingleCenteredPanel(pBig);
	sp1 = new PSimpleCenteredPanelScrollPane(scp);

	c.weighty=1;
	c.weightx=1;
	c.fill=c.BOTH;
	gridbag.setConstraints(sp1,c);
	getContentPane().add(sp1);
	//gridbag.setConstraints(noto, c);
	//getContentPane().add(noto);

	p2= new JPanel();
	p2.setLayout(new BorderLayout());
	JPanel butP = new JPanel();
	butP.setLayout(new GridLayout(1, 2, 4, 4));
	if (YoYo.macintoshp) {
	    butP.add(bca);
	    butP.add(bok);
	} else {
	    butP.add(bok);
	    butP.add(bca);
	}
	p2.add(butP, BorderLayout.EAST);

	c.weighty=0;
	c.gridheight=c.REMAINDER;
	gridbag.setConstraints(p2, c);
	//	p2.setBackground(UIManager.getColor(app.textBundle.getString("window")));
	getContentPane().add(p2);

	pack();
    }


    public void getValues(LineWidget lw, LineChart tbc) {
	int i;
	DataPanel tempDataPanel;
	int count=0;
	LineSet ls;
	ExtraInfo ei;
	int max=0;

	tbc.clearLegend();
	//lwme.wizardOn = true;
	//lwme.tbc.pma._formatMenu.getItem(0).setEnabled(false);
	for (i=0; i<sets.size(); i++) {
	    tempDataPanel= (lw.type == LineWidget.XYPLOT || lw.type == LineWidget.PHASE) ?
		(DataPanel)(dataPanels.elementAt(2*i)) :
		(DataPanel)(dataPanels.elementAt(i));
	    ls = (LineSet)sets.elementAt(i);
	    if (lw.type == LineWidget.XYPLOT || lw.type == LineWidget.PHASE) {
		ls.xyOn = true;
		ls.ei.xyOn = true;
		ls.eix.xyOn = true;
	    }
	    ls.name = tempDataPanel.nameField.getText();
	    ls.displayed = tempDataPanel.cbDisplayed.isSelected();
	    if (tempDataPanel.field.isSelected()) {
		ls.inst = tempDataPanel.instT.getText();
		ls.ei.onExtra = false;
	    }
	    else {
		ls.ei.onExtra = true;
		ls.ei.instString = (String)tempDataPanel.instBox.getSelectedItem();
		if (ls.ei.instString.equalsIgnoreCase(app.textBundle.getString("number_of"))) 
		    ls.ei.onNumber = true;
		else ls.ei.onNumber = false;
		ls.ei.breedString = (String)tempDataPanel.breedBox.getSelectedItem();
		ls.ei.varString = (String)tempDataPanel.varBox.getSelectedItem();
		if (ls.ei.varString == null) ls.ei.varString = "";
		ls.ei.onWith = tempDataPanel.cbWith.isSelected();
		ls.ei.withText = tempDataPanel.t.getText();
		ls.inst = ls.ei.getInstFromEI();
	    }
	    if (ls.displayed) {
		count++; max=i;
		lw.addLegend(i+1, ls.name);
	    }

	    if (lw.type == LineWidget.XYPLOT || lw.type == LineWidget.PHASE) {
		tempDataPanel = (DataPanel)(dataPanels.elementAt(2*i + 1));
		if (tempDataPanel.field.isSelected()) {
		    ls.instx = tempDataPanel.instT.getText();
		    ls.eix.onExtra = false;
		    ls.eix.xVal = true;
		}
		else {
		    ls.eix.onExtra = true;
		    ls.eix.xVal = true;
		    ls.eix.instString = (String)tempDataPanel.instBox.getSelectedItem();
		    if (ls.eix.instString.equalsIgnoreCase(app.textBundle.getString("number_of"))) ls.eix.onNumber = true;
		    else ls.eix.onNumber = false;
		    ls.eix.breedString = (String)tempDataPanel.breedBox.getSelectedItem();
		    ls.eix.varString = (String)tempDataPanel.varBox.getSelectedItem();
		    if (ls.eix.varString == null) ls.eix.varString = "";
		    ls.eix.onWith = tempDataPanel.cbWith.isSelected();
		    ls.eix.withText = tempDataPanel.t.getText();
		    ls.instx = ls.eix.getInstFromEI();
		}
	    }

	    // this was (ls.id) % 10 - removed by JFK/MG
	    tbc.setColor(ls.id, ls.color);
	} //end for

	if (lw.type == LineWidget.BAR) {
	    tbc.explicitLineXRange(.5, 1.5 + max);
	    int counter = 1;
	    for (Enumeration e = sets.elements(); e.hasMoreElements();) {
		ls = (LineSet)e.nextElement();
		if (ls.displayed) {
		    char c1 = ' ', c2 = '-';
		    tbc.addXTick(ls.name.replace(c1,c2), counter++);
		}
	    }
	} else if (lw.type == LineWidget.HISTOGRAM) {
	    tbc.setHistoNum(count);
	} else {
	    tbc.setLineNum(count);
	}
    }

    public void paint(Graphics g) {
	super.paint(g);
	g.setColor(Color.black);
	g.drawRect(0,0,getBounds().width,getBounds().height);
    }

    public void getValues(WidgetDrag wd) {}

    public boolean valid_inputs() { return true; }

    protected JLabel newLabel(int vecNum, String title) {
	JLabel jp = new JLabel(title);
	jp.setBackground((vecNum % 2 == 0) ? Color.lightGray : Color.white);
	return jp;
    }

    public void maybeUpdateKind(ExtraInfo ei) {
	if (ei.kindString.equals("")) {
	    // try to figure it out
	    // is the instruction globals?
	    if (ei.instString.equalsIgnoreCase(app.textBundle.getString("globals"))) {
		ei.kindString = app.textBundle.getString("globals");
	    } else if (ei.breedString.equalsIgnoreCase(app.textBundle.getString("patches"))) {
		ei.kindString = app.textBundle.getString("patches");
		ei.breedString = app.textBundle.getString("patches");
	    } else if (ei.breedString.equalsIgnoreCase(app.textBundle.getString("turtles"))) {
		ei.kindString = app.textBundle.getString("turtles");
		ei.breedString = app.textBundle.getString("turtles");
	    } else {
		// could be an actual breed. Check all breeds;
		boolean foundBreed = false;
		int numBreeds = Turtle.getNumBreeds();
		for (int j=0; j< numBreeds; j++) {
		    String name = Turtle.getBreedName(j);
		    if (ei.breedString.equalsIgnoreCase(name)) {
			ei.kindString = app.textBundle.getString("turtles");
			foundBreed = true;
			break;
		    }
		}
		assert (foundBreed);
	    }
	} else if (ei.instString.equalsIgnoreCase(app.textBundle.getString("number_of"))) {
	    // just make sure.
	    ei.kindString = app.textBundle.getString("nothing");
	} else if (type == LineWidget.HISTOGRAM) {
	    if (ei.kindString.equalsIgnoreCase("globals") || 
		ei.kindString.equalsIgnoreCase("sliders")) {
		if (ei.breedString.equalsIgnoreCase(app.textBundle.getString("patches"))) {
		    ei.kindString = app.textBundle.getString("patches");
		    ei.breedString = app.textBundle.getString("patches");
		} else {
		    ei.kindString = app.textBundle.getString("turtles");
		}
		
	    }
	    
	}
    }

    public JPanel makeTextOrWizard(DataPanel d,
				   LineSet ls,
				   int vecNum,
				   ExtraInfo ei,
				   JPanel ip) {
	//System.out.println("make text or wizard: " + ei);
	JPanel mtow = new JPanel();
	mtow.setLayout(new BorderLayout());

	JPanel cardPanel = new JPanel();
	JPanel instPanel = new JPanel();
	JPanel textPanel = new JPanel();
	instPanel.setLayout(new BorderLayout());
	GridBagLayout gbl = new GridBagLayout();
	GridBagConstraints gbc = new GridBagConstraints();
	gbc.fill = GridBagConstraints.HORIZONTAL;
	gbc.weightx = 0.0;
	gbc.insets = new Insets(3,3,3,3);

	textPanel.setLayout(gbl);
	JLabel plotLabel = newLabel(vecNum, app.textBundle.getString("plot"));
	gbl.setConstraints(plotLabel, gbc);
	textPanel.add(plotLabel);
	gbc.weightx = 1.0;
	gbc.gridwidth = GridBagConstraints.REMAINDER;
	gbl.setConstraints(d.instT, gbc);
	textPanel.add(d.instT);

	CardLayout cl = new CardLayout();
	cardPanel.setLayout(cl);
	cardPanel.add("inst", instPanel);
	cardPanel.add("text", textPanel);

	//text and inst radio row
	JPanel radioP = new JPanel();
	radioP.setLayout(new GridLayout(1,2));
	radioP.add(d.field);
	radioP.add(d.box);
	JPanel radioPB = new JPanel();
	JPanel fillerP = new JPanel();
	radioPB.setLayout(new BorderLayout());
	radioPB.add(radioP, BorderLayout.WEST);
	radioPB.add(fillerP, BorderLayout.CENTER);

	JPanel cbp = new JPanel();
	cbp.setLayout(new BorderLayout());
	cbp.add(radioPB, BorderLayout.NORTH);
	cbp.add(cardPanel, BorderLayout.CENTER);

	JPanel cp = makePanel(d, vecNum, ei, ip, cardPanel, cl);
	instPanel.add(cp, BorderLayout.WEST);
	mtow.add(cbp, BorderLayout.SOUTH);
	if (ei.onExtra) { // show the wizard
	    //System.out.println("show the wizard");
	    cl.show(cardPanel, "inst");
	} else { // don't show the wizard
	    //System.out.println("don't show the wizard");
	    if (ls.ei == ei) {
		d.instT.setText(ls.inst);
	    } else {
		d.instT.setText(ls.instx);
	    }		
	    cl.show(cardPanel, "text");
	}

	Color tempColor = (vecNum % 2 == 0) ? Color.lightGray : Color.white;
	
	cardPanel.setBackground(tempColor);
	instPanel.setBackground(tempColor);
	textPanel.setBackground(tempColor);
	cp.setBackground(tempColor);
	radioP.setBackground(tempColor);
	radioPB.setBackground(tempColor);
	fillerP.setBackground(tempColor);
	d.field.setBackground(tempColor);
	d.box.setBackground(tempColor); 

	//d.breedBox.setBackground(tempColor);
	//d.instBox.setBackground(tempColor);
	//d.varBox.setBackground(tempColor);

	mtow.setBackground(tempColor);
	d.cbWith.setBackground(tempColor);
	return mtow;
    }

    public JPanel makePanel(DataPanel d,
			    int vecNum,
			    ExtraInfo ei,
			    JPanel ip,
			    JPanel cardPanel,
			    CardLayout cl) {

	//System.out.println("make wizard panel for " + ei);
	JLabel headLabel = null;

	JPanel makeP = new JPanel();
	makeP.setLayout(new BorderLayout());

	JPanel cp = new JPanel();
	cp.setLayout(new FlowLayout(FlowLayout.LEFT));

	if (type == LineWidget.HISTOGRAM ||
	    type == LineWidget.XYPLOT) {
	    headLabel = new JLabel(app.textBundle.getString("plot_value_of"));
	    d.instBox.addItem(app.textBundle.getString("histogram"));
	    d.instBox.setSelectedItem(app.textBundle.getString("histogram"));
	}
	else {
	    headLabel = new JLabel(app.textBundle.getString("plot"));
	    if (!ExtraInfo.initialized) { ExtraInfo.initializeExtraInfo(app); }
	    for (int i=0; i<ExtraInfo.FUNCTIONS.length; i++)
		d.instBox.addItem(ExtraInfo.FUNCTIONS[i]);
	    if (ei.instString != null)
		d.instBox.setSelectedItem(ei.instString);
	}
	if (ei.onExtra)
	    d.box.setSelected(true);
	else d.field.setSelected(true);

	maybeUpdateKind(ei);
	//d.setShowing(app, ei.kindString);

	d.breedBox.addItem(app.textBundle.getString("turtles"));
	String[] br = ExtraInfo.getBreeds();
	for (int i=0; i<br.length; i++) {
	    d.breedBox.addItem(br[i]);
	}
	d.breedBox.addItem(app.textBundle.getString("patches"));
	
	if (ei.breedString != null)
	    d.breedBox.setSelectedItem(ei.breedString);

	d.varBox = new JComboBox();
	d.varBox.setRenderer(new MyListCellRenderer(d.varBox, d.varBox.getRenderer(),
							d.varBox.getBackground()));
	setupVarBox(d, d.varBox, ei.varString, ei.kindString);

	d.breedBox.addItemListener(new BreedListener(vecNum,
						     d.varBox,
						     d.breedBox));
	
	d.cbWith = new JCheckBox(app.textBundle.getString("with"));
	d.cbWith.setSelected(ei.onWith);
	d.t = new JTextField("", 15);
	d.t.setText(ei.withText);

	JPanel wp = new JPanel();
	wp.setLayout(new FlowLayout(FlowLayout.LEFT));
	wp.add(d.cbWith); //, BorderLayout.WEST);
	wp.add(d.t); //, BorderLayout.EAST);
	d.t.setEnabled(ei.onWith);

	d.cbWith.addItemListener(new WithListener(vecNum,
						  d.cbWith,
						  d.t));
	cp.setLayout(new FlowLayout(FlowLayout.LEFT));
	cp.add(headLabel);

	if (type == LineWidget.HISTOGRAM ||
	    type == LineWidget.XYPLOT) {
	    //	    cp.add(valueOfLabel);
	    cp.add(d.varBox);
	    cp.add(d.forAll);
	    cp.add(d.breedBox);
	    //	    cp.add(wp);
	}
	else {
	    cp.add(d.instBox);

	    if (ei.onNumber) {
		cp.add(d.breedBox);
		//		cp.add(new SpacePanel());
		//		cp.add(wp);
	    }
	    else {
		cp.add(d.varBox);
		if (!(d.instBox.getSelectedItem().equals(app.textBundle.getString("globals")) ||
		      d.instBox.getSelectedItem().equals(app.textBundle.getString("sliders")))) {
		    cp.add(d.forAll);
		    cp.add(d.breedBox);
		    //		cp.add(wp);
		}
	    }
	}
	d.instBox.addItemListener(new CalcListener(vecNum,
						   ei,
						   headLabel,
						   d,
						   d.instBox,
						   d.varBox,
						   d.forAll,
						   d.breedBox,
						   wp,
						   cp));
	d.box.addItemListener(new InstructionListener(cl,
						      cardPanel,
						      d.field,
						      d.box));
	/**
	   vecNum,
	   ei,
	   cp,
	   d.instT,
	   ip,
	   d.field,
	   d.box)); **/
	d.field.addItemListener(new InstructionListener(cl, cardPanel,
							d.field, d.box));

 	dataPanels.addElement(d);
	JPanel top = new JPanel();
	top.setLayout(new BorderLayout());
	JPanel bottom = new JPanel();
	bottom.setLayout(new BorderLayout());

	Color tempColor = (vecNum % 2 == 0) ? Color.lightGray : Color.white;

	// 	//MTB
	// 	if (type == LineWidget.LINE) {
	// 	    d.connectBox = new ConnectBox(vecNum);
	// 	    d.connectBox.setBackground(tempColor);
	// 	    bottom.add(d.connectBox, BorderLayout.SOUTH);
	// 	}

	top.add(cp, BorderLayout.WEST);
	bottom.add(wp, BorderLayout.WEST);
	makeP.add(top, BorderLayout.NORTH);
	makeP.add(bottom, BorderLayout.SOUTH);

	//	if (vecNum % 2 == 0) {
	makeP.setBackground(tempColor);
	top.setBackground(tempColor);
	bottom.setBackground(tempColor);
	cp.setBackground(tempColor);
	wp.setBackground(tempColor);
	d.setBackground(tempColor);
	headLabel.setBackground(tempColor);
	//	}

	return makeP;
    }

    public void setupVarBox(DataPanel d, JComboBox varBox, 
			    String varString, String kindString) {

	d.setShowing(app, kindString);
	
	varBox.removeAllItems();

	switch (d.showingWhat) {
	case DataPanel.TURTLES:
	    for (int a=0; a<turtleVars.length; a++)
		varBox.addItem(turtleVars[a]);
	    break;
	case DataPanel.PATCHES:
	    for (int a=0; a<patchVars.length; a++)
		varBox.addItem(patchVars[a]);
	    break;
	case DataPanel.GLOBALS:
	    for (int a=0; a<obsVars.length; a++)
		varBox.addItem(obsVars[a]);
	    break;
	case DataPanel.SLIDERS:
	    for (int a=0; a<obsVars.length; a++)
		varBox.addItem(sliderVars[a]);
	    break;
	case DataPanel.NOTHING:
	    if (type == LineWidget.HISTOGRAM ||
		type == LineWidget.XYPLOT) {
		
		if (d.ei.breedString.equalsIgnoreCase(app.textBundle.getString("patches"))) {
		    for (int a=0; a<patchVars.length; a++)
			varBox.addItem(patchVars[a]);
		} else {
		    for (int a=0; a<turtleVars.length; a++)
			varBox.addItem(turtleVars[a]);
		}
	    }
	    break;
	}
	if (varString != null && !varString.equals(""))
	    varBox.setSelectedItem(varString);
    }


    public JPanel makePDataSetStylePanel(Frame parentFrame,
				     JPtolemyBarDialog sld,
				     Application app,
				     int mid, int vecNum,
				     ExtraInfo ei,
				     boolean displayed,
				     Color color,
				     LineSet lset) {
	//System.out.println("drawing data set style panel: " + mid);
	GridBagLayout gridbag = new GridBagLayout();
	GridBagConstraints cbc = new GridBagConstraints();

	JPanel p1 = new JPanel();
	p1.setLayout(gridbag);

	//	setLayout(gridbag);

	cbc.fill = GridBagConstraints.NONE; //GridBagConstraints.HORIZONTAL;
	cbc.weightx = 0.0;
	cbc.gridheight = 1;
	cbc.insets = new Insets(3,3,3,3);

	if (mid == -1) {
	} else {
	    
	    LineSet ls = (LineSet)JPtolemyBarDialog.sets.elementAt(vecNum);
	    DataPanel d = new DataPanel(ls, ls.ei);
	    //System.out.println("showing " + mid + " ei.kind " + ei.kindString);
	    //d.setShowing(app, ei.kindString);

	    JPanel ip = new JPanel();
	    ip.setLayout(new FlowLayout(FlowLayout.LEFT));

	    //start on top row (always on)
	    JLabel l = new JLabel((vecNum+1) + "", JLabel.CENTER);
	    l.setFont(new Font("Helvetica", Font.BOLD, 16));
	    d.colorButton.addActionListener(new ButtonListener(parentFrame,
							       d.buttonPanel,
							       ls,
							       d.colorButton));
	    JPanel on = new JPanel();
	    on.setLayout(new BorderLayout());
	    on.add(d.cbDisplayed, BorderLayout.CENTER);
	    gridbag.setConstraints(l, cbc);
	    p1.add(l);
	    gridbag.setConstraints(d.cbDisplayed, cbc);
	    p1.add(d.cbDisplayed);
	    gridbag.setConstraints(d.colorButton, cbc);
	    p1.add(d.colorButton);
	    gridbag.setConstraints(d.buttonPanel, cbc);
	    p1.add(d.buttonPanel);
	    JLabel ppn = newLabel(vecNum, app.textBundle.getString("plot_pen_name"));
	    gridbag.setConstraints(ppn, cbc);
	    p1.add(ppn);
	    cbc.weightx = 1.0;
	    cbc.gridwidth = GridBagConstraints.REMAINDER;
	    cbc.fill = cbc.HORIZONTAL;
	    gridbag.setConstraints(d.nameField, cbc);
	    p1.add(d.nameField);
	    cbc.gridwidth = 1;
	    cbc.fill = cbc.NONE;
	    cbc.weightx = 0.0;
	    //end top row

	    if (type == LineWidget.XYPLOT ||
		type == LineWidget.PHASE) {
		JLabel xvalue = new JLabel(app.textBundle.getString("x_value"));
		cbc.anchor = cbc.NORTHEAST;
		cbc.gridx = 0;
		cbc.gridwidth = 2;
		gridbag.setConstraints(xvalue, cbc);
		p1.add(xvalue);
		cbc.gridwidth = 1;
		cbc.anchor = cbc.CENTER;
	    }
	    JPanel mtow = makeTextOrWizard(d, ls, vecNum, ei, ip);
	    cbc.gridwidth = GridBagConstraints.REMAINDER;
	    cbc.fill = cbc.HORIZONTAL;
	    cbc.gridx = 2;
	    gridbag.setConstraints(mtow, cbc);
	    p1.add(mtow);
	    cbc.gridwidth = 1;
	    cbc.gridx = 0;
	    cbc.fill = cbc.NONE;

	    boolean two = false;
	    DataPanel d2 = null;
	    JPanel cp2 = null, ip2 = null, pp2 = null;

	    if (type == LineWidget.XYPLOT ||
		type == LineWidget.PHASE) {

		JLabel yvalue = new JLabel(app.textBundle.getString("y_value"));
		cbc.gridx = 0;
		cbc.gridwidth = 2;
		cbc.anchor = cbc.NORTHEAST;
		gridbag.setConstraints(yvalue, cbc);
		p1.add(yvalue);
		cbc.gridwidth = 1;
		cbc.anchor = cbc.CENTER;

		d2 = new DataPanel(ls, ls.eix);
		//d2.setShowing(app, ls.eix.kindString);
		cbc.gridwidth = GridBagConstraints.REMAINDER;
		cbc.gridx = 2;
		cbc.fill = cbc.HORIZONTAL;
		JPanel y_mtow = makeTextOrWizard(d2, ls, vecNum, ls.eix, ip);
		gridbag.setConstraints(y_mtow, cbc);
		p1.add(y_mtow);
		cbc.gridx = 0;
		cbc.gridwidth = 1;
		cbc.fill = cbc.NONE;
	    }

	    Color tempColor = (vecNum % 2 == 0) ? Color.lightGray : Color.white;
	    p1.setBackground(tempColor);
	    l.setBackground(tempColor);
	    d.cbDisplayed.setBackground(tempColor);
	    mtow.setBackground(tempColor);
	    d.cbWith.setBackground(tempColor);
	    if (d2 != null) d2.setBackground(tempColor);

	    p1.setOpaque(true);
	    l.setOpaque(true);
	    d.cbDisplayed.setOpaque(true);
	    mtow.setOpaque(true);
	    d.cbWith.setOpaque(true);
	    
	    cbc.gridwidth = 1;
	    pack();
	    repaint();
	}
	return p1;
    }


    class WithListener implements ItemListener {
	public int z;
	public JCheckBox cbWith;
	public JTextField t;
	public JPanel wp;
	public JPanel ip;

	public WithListener(int z,
			    JCheckBox cbWith,
			    JTextField t) {
	    this.z = z;
	    this.cbWith = cbWith;
	    this.t = t;
	    //	    this.wp = wp;
	    //this.ip = ip;
	}

	public void itemStateChanged(ItemEvent ie) {
	    t.setEnabled(cbWith.isSelected());
	}

    }

    class CalcListener implements ItemListener {

	public int z;
	public ExtraInfo ei;
	public JLabel headLabel;
	public DataPanel d;
	public JComboBox instBox;
	public JComboBox varBox;
	public JLabel forAll;
	public JPanel cp;
	public JPanel wp;
	public JComboBox breedBox;

	public CalcListener(int z,
			    ExtraInfo ei,
			    JLabel headLabel,
			    DataPanel d,
			    JComboBox instBox,
			    JComboBox varBox,
			    JLabel forAll,
			    JComboBox breedBox,
			    JPanel wp,
			    JPanel cp) {
	    this.z = z;
	    this.ei = ei;
	    this.d = d;
	    this.headLabel = headLabel;
	    this.instBox = instBox;
	    this.varBox = varBox;
	    this.forAll = forAll;
	    this.breedBox = breedBox;
	    this.wp = wp;
	    this.cp = cp;
	}

	public void itemStateChanged(ItemEvent ie) {
	    String selectedItem = (String)((JComboBox)ie.getSource()).getSelectedItem();
	    if (selectedItem.equalsIgnoreCase(app.textBundle.getString("number_of"))) {
		d.showNothing();
		cp.removeAll();
		cp.add(headLabel);
		cp.add(instBox);
		cp.add(breedBox);
		//		cp.add(new SpacePanel());
		//		cp.add(wp);
	    }
	    else if (selectedItem.equalsIgnoreCase(app.textBundle.getString("globals"))) {
		d.showGlobals();
		cp.removeAll();
		cp.add(headLabel);
		cp.add(instBox); //, BorderLayout.WEST);
		varBox.removeAllItems();
		for (int i=0; i<obsVars.length; i++)
		    varBox.addItem(obsVars[i]);
		cp.add(varBox);
		//no for all and breed box
	    }
	    else if (selectedItem.equalsIgnoreCase(app.textBundle.getString("sliders"))) {
		d.showSliders();
		cp.removeAll();
		cp.add(headLabel);
		cp.add(instBox); //, BorderLayout.WEST);
		varBox.removeAllItems();
		for (int i=0; i<sliderVars.length; i++)
		    varBox.addItem(sliderVars[i]);
		cp.add(varBox);
		//no for all and breed box
	    } else {
		varBox.removeAllItems();
		if (breedBox.getSelectedItem().equals(app.textBundle.getString("patches"))) {
		    d.showPatches();
		    for (int j=0; j<patchVars.length; j++) {
			varBox.addItem(patchVars[j]);
		    }
		} else {
		    //breedBox equals "turtles"
		    d.showTurtles();
		    for (int k=0; k<turtleVars.length; k++) {
			varBox.addItem(turtleVars[k]);
		    }
		}
		cp.removeAll();
		cp.add(headLabel);
		cp.add(instBox);
		cp.add(varBox);
		cp.add(forAll);
		cp.add(breedBox);
		//		cp.add(wp);
	    }
	    cp.validate();
	    cp.repaint();
	    //pack();
	    validate();
	    repaint();
	}
    }


    class InstructionListener implements ItemListener {
	public int z;
	public ExtraInfo ei;
	public JTextField instT;
	public JPanel ip, cp, instP, p;
	public GridBagLayout gbl;
	public GridBagConstraints gbc;

	public JRadioButton field, box;
	public CardLayout cl;
	public JPanel cardPanel;

	InstructionListener(CardLayout cl,
			    JPanel cardPanel,
			    JRadioButton field,
			    JRadioButton box) {
	    this.cl = cl;
	    this.cardPanel = cardPanel;
	    this.field = field;
	    this.box = box;
	}

	public void itemStateChanged(ItemEvent ae) {
	    if (field.isSelected()) {
		cl.last(cardPanel);
	    }
	    if (box.isSelected()) {
		cl.first(cardPanel);
	    }
	}
    }


    class PSingleCenteredPanel extends JPanel {
	Component child;

	PSingleCenteredPanel (Component child) {
	    super();
	    this.child=child;
	    GridBagLayout gridbag=new GridBagLayout();
	    GridBagConstraints c = new GridBagConstraints();
	    this.setLayout(gridbag);

	    c.gridwidth=c.REMAINDER;
	    c.gridheight=c.REMAINDER;
	    c.anchor=c.CENTER;

	    gridbag.setConstraints(child, c);
	    this.add(child);
	}

	public Dimension getPreferredSize() {
	    Dimension d = child.getPreferredSize();
	    d.width +=  0;//15; //15
	    d.height += 0;//20; //20
	    return d;
	}

    }

    class PSimpleCenteredPanelScrollPane extends JScrollPane {
	Component child;

	PSimpleCenteredPanelScrollPane(Component child) {
	    super(child,
		  JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
		  JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
	    //	    super(ScrollPane.SCROLLBARS_AS_NEEDED);
	    this.child = child;
	    //	    this.add(this.child);
	}

	public Dimension getPreferredSize() {
	    Dimension d= child.getPreferredSize();
	    Dimension result= new Dimension();
	    result.width=d.width+40;
	    result.height=d.height;
	    Toolkit toolkit=Toolkit.getDefaultToolkit();
	    if (toolkit.getScreenSize().height<(result.height+200)) {
		result.height=toolkit.getScreenSize().height-200;
	    }
	    /*	    if (toolkit.getScreenSize().width<(result.width+100)) {
	      result.width=toolkit.getScreenSize().width-100;
	      }
	    **/
	    return result;
	}

    }

    class BreedListener implements ItemListener {
	public int z;
	public JComboBox varBox;
	public JComboBox breedBox;

	public BreedListener(int z,
			     JComboBox varBox,
			     JComboBox breedBox) {
	    this.z = z;
	    this.varBox = varBox;
	    this.breedBox = breedBox;
	}

	public void itemStateChanged(ItemEvent ie) {
	    String selectedItem = (String)((JComboBox)ie.getSource()).getSelectedItem();
	    if (selectedItem.equalsIgnoreCase(app.textBundle.getString("patches"))) {
		varBox.removeAllItems();
		for (int i=0; i<patchVars.length; i++)
		    varBox.addItem(patchVars[i]);
	    }
	    else {
		varBox.removeAllItems();
		for (int i=0; i<turtleVars.length; i++)
		    varBox.addItem(turtleVars[i]);
	    }
	    validate();
	    repaint();

	}
	//cp.validate();
	//cp.repaint();
    }//end breed listener

    class ButtonListener implements ActionListener {

	Frame f;
	JPanel p;
	JInterface iface;
	LineSet ls;
	JButton b;

	ButtonListener(Frame f, JPanel p, LineSet ls, JButton b) {
	    this.f = f;
	    this.p = p;
	    this.ls = ls;
	    this.b=b;
	}

	public void actionPerformed(ActionEvent ae) {
	    String title = app.textBundle.getString("color_chooser");
	    JColortoolDialog ctd = new JColortoolDialog(app, f, title, p, ls);
	    ctd.show();
	    //System.out.println("finished...");
	}
    }

    class SpacePanel extends JPanel {

	SpacePanel() {
	    super();
	}

	public Dimension getPreferredSize() {
	    return new Dimension(500, super.getPreferredSize().height);
	}
	/**
	   public Dimension getMinimumSize() {
	   return new Dimension(400, 1600);
	   }
	**/
    }

    class ColorBoxPanel extends JPanel {

	public Dimension getPreferredSize() {
	    return new Dimension(16, 16);
	}

	public Dimension getMaximumSize() {
	    return new Dimension(16, 16);
	}

	public Dimension getMinimumSize() {
	    return new Dimension(16, 16);
	}

	public void paint(Graphics g) {
	    super.paint(g);
	    g.setColor(Color.black);
	    g.drawRect(0,0,15,15);
	}

    }

    class DisplayBox extends JCheckBox implements ItemListener {
	
	private DataPanel dp;
	
	DisplayBox(DataPanel panel) {
	    super(app.textBundle.getString("on"));
	    dp = panel;
	    addItemListener(this);
	}

	public void itemStateChanged(ItemEvent ie) {
	    boolean selected = ((JCheckBox)ie.getSource()).isSelected();
	    setPanelEnabled(selected);
	}

	void setPanelEnabled(boolean enabled) {
	    setSelected(enabled);
	    dp.colorButton.setEnabled(enabled);
	    if (dp.editName != null) dp.editName.setEnabled(enabled);
	    //if (dp.connected != null) dp.connected.setEnabled(enabled);
	    dp.breedBox.setEnabled(enabled);
	    if (dp.varBox != null) dp.varBox.setEnabled(enabled);
	    if (dp.cbWith != null) dp.cbWith.setEnabled(enabled);
	    if (dp.t != null) dp.t.setEnabled(enabled);
	    dp.nameField.setEnabled(enabled);
	    dp.instBox.setEnabled(enabled);
	    dp.instT.setEnabled(enabled);
	    dp.field.setEnabled(enabled);
	    dp.box.setEnabled(enabled);
	    dp.forAll.setEnabled(enabled);
	    dp.buttonPanel.setEnabled(enabled);
	    //if (dp.connectBox != null) dp.connectBox.setEnabled(enabled);
	}
    }

    class DataPanel {

	public JButton colorButton;
	public DisplayBox cbDisplayed;
	public JTextField editName;
	public JComboBox breedBox;
	public JComboBox varBox;
	public JCheckBox cbWith;
	public JTextField t;
	public JTextField nameField;
	public JComboBox instBox;
	public JTextField instT;
	public JRadioButton field;
	public JRadioButton box;
	public ButtonGroup cbg;
	public JLabel forAll;
	public JPanel buttonPanel;
	public LineSet ls;
	public ExtraInfo ei;

	public static final int NOTHING = 0;
	public static final int TURTLES = 1;
	public static final int PATCHES = 2;
	public static final int GLOBALS = 3;
	public static final int SLIDERS = 4;

	public int showingWhat = NOTHING;
	
	DataPanel(LineSet ls, ExtraInfo ei) {
	    this.ei = ei;
	    colorButton = new JButton(app.textBundle.getString("change_color"));
	    forAll = new JLabel(app.textBundle.getString("for_all"));
	    //	colorButton = new JButton();
	    // 	cbDisplayed = new JCheckBox(app.textBundle.getString("on"));
	    cbDisplayed = new DisplayBox(this);
	    nameField = new JTextField(app.textBundle.getString("blank_name") + ls.order, (YoYo.macosxp) ? 5 : 10);
	    instBox = new JComboBox();
	    instBox.setRenderer(new MyListCellRenderer(instBox, instBox.getRenderer(),
						       instBox.getBackground()));
	    cbg = new ButtonGroup();
	    field = new JRadioButton(app.textBundle.getString("text"));
	    box = new JRadioButton(app.textBundle.getString("wizard"));
	    cbg.add(field);
	    cbg.add(box);
	    breedBox = new JComboBox();
	    breedBox.setRenderer(new MyListCellRenderer(breedBox, breedBox.getRenderer(),
							breedBox.getBackground()));
	    instT= new JTextField(YoYo.macosxp ? 25 : 30);
	    buttonPanel = new ColorBoxPanel();

	    nameField.setText(ls.name);
	    buttonPanel.setBackground(ls.color);
	    cbDisplayed.setPanelEnabled(ls.displayed);
	    showingWhat = TURTLES;
	}

	public void showNothing() {
	    showingWhat = NOTHING;
	    String nothing = app.textBundle.getString("nothing");
	    ei.kindString = nothing;
	}

	public void showTurtles() {
	    showingWhat = TURTLES;
	    String turtles = app.textBundle.getString("turtles");
	    ei.kindString = turtles;
	}

	public void showPatches() {
	    showingWhat = PATCHES;
	    String patches = app.textBundle.getString("patches");
	    ei.kindString = patches;
	}

	public void showGlobals() {
	    showingWhat = GLOBALS;
	    String globals = app.textBundle.getString("globals");
	    ei.kindString = globals;
	}

	public void showSliders() {
	    showingWhat = SLIDERS;
	    String sliders = app.textBundle.getString("sliders");
	    ei.kindString = sliders;
	}

	public void setShowing(Application app, String kind) {
	    if (kind.equalsIgnoreCase(app.textBundle.getString("nothing"))) {
		showingWhat = NOTHING;
	    } else if (kind.equalsIgnoreCase(app.textBundle.getString("globals"))) {
		showingWhat = GLOBALS;
	    } else if (kind.equalsIgnoreCase(app.textBundle.getString("turtles"))) {
		showingWhat = TURTLES;
	    } else if (kind.equalsIgnoreCase(app.textBundle.getString("patches"))) {
		showingWhat = PATCHES;
	    } else if (kind.equalsIgnoreCase(app.textBundle.getString("sliders"))) {
		showingWhat = SLIDERS;
	    }
	    //System.out.println("showing what = " + showingWhat);
	}

	public void setBackground(Color c) {
	    if (YoYo.macosxp) {
		colorButton.setBackground(c);
		breedBox.setBackground(c);
		instBox.setBackground(c);
		if (varBox != null) varBox.setBackground(c);
	    } 
	    forAll.setBackground(c);
	}

	

    }

    class MyListCellRenderer implements ListCellRenderer {
	private JComboBox comboBox;
	private ListCellRenderer renderer;
	private Color c;
	
	public MyListCellRenderer(JComboBox jcb, ListCellRenderer lcr, Color c) {
	    this.comboBox = jcb;
	    this.renderer = lcr;
	    this.c = c;
	}
	public Component getListCellRendererComponent(JList list,
						      Object value,
						      int index,
						      boolean isSelected,
						      boolean cellHasFocus) {
	    Component ret = renderer.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
	    if (!isSelected) {
		ret.setBackground(c);
	    }
	    return ret;
	}
    }
}


