/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.awt.event.*;
import java.awt.*;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;
import java.net.URL;
import java.net.MalformedURLException;
import yoyo.*;
import javax.swing.event.DocumentListener;
import javax.swing.event.DocumentEvent;
import javax.swing.JLabel;
import javax.swing.undo.UndoManager;
import javax.swing.event.UndoableEditEvent;
import javax.swing.KeyStroke;
import java.awt.event.*;
import javax.swing.Action;
import javax.swing.text.*;
import javax.swing.SwingUtilities;

public class JSLCommand extends JCommandCenter implements SLCommand, JControlCenterTextArea {

    public static final String obsPrefix = "compile-observer-cc ";
    public static final String turPrefix = "compile-turtle-cc ";
    public static final String patPrefix = "compile-patch-cc ";
 
    public static Font PROC_FONT;

    static {
  	try {
	    PROC_FONT = new Font("Monospaced", Font.PLAIN, 12);
	    if (YoYo.macintoshp) PROC_FONT = Font.getFont("starlogo.procedures.font", PROC_FONT);
 	} catch (Exception e) { e.printStackTrace(); }
    } 
  
    String who;
    StarLogo sl;
    Application app;

    public boolean runningp = false;
    int which;
    boolean whop = false;
    Patch patch = null;
    JLabel label;
    JControlCenter jcc;

    private JLabel getLabel(String title, Color color) {
	JLabel out = new JLabel(title, JLabel.LEFT) {
		public Dimension getPreferredSize() {
		    return new Dimension(super.getPreferredSize().width,
					 getFontMetrics(getFont()).getHeight() + jcc.LABEL_VERTICAL_SPACE);
		}
	    };
	out.setBackground(color);
	return out;
    }

    public JSLCommand(int which, StarLogo sl, JControlCenter jcc) {
	super();
	//setLineWrap(true);

	this.sl = sl;
	this.app = sl.app;
	this.jcc = jcc;
	this.c = jcc.app.cc;
	this.which = which;
	//setBackground(Color.white);
	//setForeground(Color.black);//SLDialog.colorText(this);
	setFont(PROC_FONT);
	switch (which) {
	case TURTLE:
	    label = getLabel(sl.app.textBundle.getString("turtle_command_center_label"), new Color(23, 118, 51));
	    break;
	case OBSERVER:
	    label = getLabel(sl.app.textBundle.getString("observer_command_center_label"), new Color(92, 61, 36));
	    break;
	case PATCH:
	    label = getLabel(sl.app.textBundle.getString("patch_command_center_label"), new Color(23, 118, 51));
	    break;
	}


	label.setFont(JSLDialog.LABEL_FONT);
	label.setForeground(Color.white);
	label.setBackground(new Color(23, 118, 51));
    
	//disable the undo in the command center (but not the procedure window) since it doesn't work right yet
	//  getDocument().addUndoableEditListener(undoManager);
	//addKeyListener(new CommandKeyAdapter(this));
	setupKeys();
	addCaretListener(jcc);

	addMouseListener(new ControlMouseAdapter(this));
	getDocument().addDocumentListener(new SLCommandTextChanged());
	//jcc.app.registerAutoCompiler(this);
    }
    
    //format text areas
    public void formatText(JTextComponent tc) {
	tc.setFont(PROC_FONT);
	tc.setBackground(Color.white);
	tc.setForeground(Color.black);
    }
    
    
    public JSLCommand(int which, StarLogo sl, int who, Application app) {
	super(); //, java.awt.TextArea.SCROLLBARS_VERTICAL_ONLY);
	
	this.sl = sl;
	this.app = sl.app;
	this.which = which;
	this.c = app.cc;
	//setBackground(Color.white);
	//setForeground(Color.black);
	setFont(PROC_FONT);
	whop = true;
	this.who = new String(Integer.toString(who));

	label = getLabel((sl.app.textBundle.getString("turtle_number_label") + this.who) + " " +sl.app.textBundle.getString("command_center_label"), new Color(23, 118, 51));

	label.setFont(new Font("Dialog", Font.PLAIN, 12));
	label.setForeground(Color.white);
	//addKeyListener(new CommandKeyAdapter(this));
	setupKeys();
	
	//getDocument().addUndoableEditListener(undoManager);
	getDocument().addDocumentListener(new SLCommandTextChanged());
    }
    
    public JSLCommand(int which, StarLogo sl, Patch p, Application app) {
	super(); //""); //, java.awt.TextArea.SCROLLBARS_VERTICAL_ONLY);


	this.sl = sl;
	this.app = sl.app;
	this.which = which;
	this.c = app.cc;
	this.patch = p;
	//setBackground(Color.white);
	//setForeground(Color.black);
	setFont(PROC_FONT);
	label = getLabel(sl.app.textBundle.getString("patch_label") + p.toString() + sl.app.textBundle.getString("command_center_label"), new Color(23, 118, 51));

	label.setFont(new Font("Dialog", Font.PLAIN, 12));
	label.setForeground(Color.white);
	label.setBackground(new Color(23, 118, 51));
	//addKeyListener(new CommandKeyAdapter(this));
	setupKeys();
	
	//getDocument().addUndoableEditListener(undoManager);
	getDocument().addDocumentListener(new SLCommandTextChanged());
    }
    

    public void setupKeys() {
	KeyStroke ks;
	if (YoYo.macintoshp) {
	    ks = KeyStroke.getKeyStroke(KeyEvent.VK_PERIOD, Event.META_MASK); 
	    Action act = new TextAction("Break") {
		    public void actionPerformed(ActionEvent e) {
			synchronized(sl.runningp) {
			    sl.stopAllStarLogo();
			}
		    }
		};
	    km.addActionForKeyStroke(ks, act);
	} else {
	    ks = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0);
	    Action act = new TextAction("Break") {
		    public void actionPerformed(ActionEvent e) {
			synchronized(sl.runningp) {
			    sl.stopAllStarLogo();
			}
		    }
		};
	    km.addActionForKeyStroke(ks, act);
	}
    }


    private class CommandRunner implements Runnable {
	private String s;
	public CommandRunner(String s) {
	    this.s = s;
	}

	public void run() {
	    runningp = true;
	    //need to clear both command centers 
	    //(even if there are no turtles to clear the command center)
	    if (s != "" && s != null) {
		Object[] listtocompile = null;
		switch(which) {
		case TURTLE:
		    String whoPrefix = "[ " + app.commandsBundle.getString("if") + " " +
			app.commandsBundle.getString("who") + " = ";
		    listtocompile = 
			Reader.read(turPrefix + ((whop) ? whoPrefix + who : "") + "[" + s + "]");
		    break;
		case OBSERVER:
		    listtocompile =
			Reader.read(obsPrefix + "[" + s + "]");
		    break;
		case PATCH:
		    listtocompile =
			Reader.read(patPrefix + "[" + s + "]");
		    break;
		}
		Object[] newlist = new Object[listtocompile.length + 1];
		System.arraycopy(listtocompile, 0, newlist, 0, listtocompile.length);
		newlist[newlist.length -1] = JSLCommand.this;
		sl.app.cc.enqueue(newlist, sl.app.message, sl.app.err, JSLCommand.this);
	    }
	}
    }

    public void runLine(String s) {
	/* Make sure the command center isn't running something else */
	if (runningp) {
	    c.stop(); /* stop any yoyo compiling */
	    stopAll(); /* stop any starlogo processes spawned from this command center */
	}      
	//System.out.println("run line: " + s);
	if (s.equalsIgnoreCase(app.commandsBundle.getString("cc")) || 
	    s.equalsIgnoreCase(app.commandsBundle.getString("clear_command_center"))) { 
	    super.done(c); 
	    callerClearText(); 
	    return;
	}
	if (s.equalsIgnoreCase(app.commandsBundle.getString("%yoyo"))) { 
	    SwingUtilities.invokeLater(new Runnable() {
		    public void run() {
			new yoyo.JToplevel("YoYo", TopApplication.ub);
			JSLCommand.super.done(c);
		    }
		});
	    return;
	}
      
	sl.app.maybeCompileAndThenRunElseRunIfFailedAlotRun(false, 
							    new CommandRunner(s), 
							    new Runnable() {
								public void run() {
								    JSLCommand.super.done(c);
								}
							    },
							    new CommandRunner(s));
    }
    
    public void runCompiledCommand(Object[] ilist, Object[] olist) {
	int[] newarray = new int[ilist.length];
	for(int i = 0; i<ilist.length; i++) {
	    newarray[i] = ((Number)ilist[i]).intValue();
	}
	if (sl.locked == true) {
	    displayOneString(app.textBundle.getString("cant_run_cc") + " " +
			     app.textBundle.getString("cant_run_while_obscured"));
	}
	else {
	    synchronized(sl.runningp) {
		int whichip = sl.internalSetInstructions(toString(), newarray, olist);
		switch (which) {
		case TURTLE: StackProcessParent.runTurtles(sl, whichip, false, this);
		    break;
		case OBSERVER: StackProcessParent.runObserver(sl, whichip, false, this);
		    break;
		case PATCH: StackProcessParent.runPatch(sl, whichip, patch, false, this);
		    break;
		}
	    }}
    }
    
    
    /* YoYo done method */
    public void done(Context c) {
	//super.done(c);
    }
    
    private Vector spps = new Vector();
    private int sppcount = 0;

    public void done(StackProcessParent spp) {
	//		System.out.println("sl command done " + spp);
	spps.removeElement(spp);
	sppcount--;
	//		System.out.println("slcommand empty? " + spps.isEmpty() + " runningp = " + runningp);
	if (sppcount == 0 && runningp) {
	    //		    System.out.println("hi");
	    SwingUtilities.invokeLater(new Runnable() {
		    public void run() {
			JSLCommand.super.done(c);
		    }
		});
	    runningp = false;
	}
    }

    public boolean donep() {
	//System.out.println("slcommand done? " + (sppcount == 0));
	return (sppcount == 0);
    }

    public StarLogoCaller caller() {
	return this;
    }

    public void addStackProcessParent(StackProcessParent s) {
	spps.addElement(s);
	sppcount++;
    }

    public void stopAll() {
	//System.out.println("slcommand stop all");
	for(Enumeration e = ((Vector)spps.clone()).elements(); e.hasMoreElements();) {
	    StackProcessParent spp = (StackProcessParent)e.nextElement();
	    //System.out.println("trying to stop " + spp);
	    spp.stopAll();
	}
	//spps.removeAllElements();
    }

    // Only called when we are reporting an error.
    public void displayOneString(String s) {
	if (app.kioskp || YoYo.applet != null) sl.app.err.println(s);
	s += "\n";
	print(s);
	getOut().flush();
	SwingUtilities.invokeLater(new Runnable() {
		public void run() {
		    JSLCommand.super.done(c);
		}
	    });
	runningp = false;
    }

    public void displayAString(String s) {
	if (app.kioskp || YoYo.applet != null) sl.app.err.println(s);
	print(s);
	getOut().flush();
    }

    public void callerPrintln(String s) {
	println(s);
    }

    public void callerClearText() {
	if (!getText().equals("")) {
	    setText("");
	    //linemark = -1;
	}
    }

    public void println(String s) {
	print(s + "\n");
    }

    public void print(String s) {
	PrintWriter out = getOut();
	out.print(s);
	//out.flush();
    }


    class SLCommandTextChanged implements DocumentListener {
	public void changedUpdate(DocumentEvent e) {}
	public void checkToStop() {
	    if (c.runningnowp) {
		c.stop();
		synchronized(sl.runningp) {
		    stopAll();
		}
	    }
	    if (!donep()) {
		synchronized(sl.runningp) {
		    stopAll();
		}
	    }
	}
	public void insertUpdate(DocumentEvent e) {
	    checkToStop();
	}
	public void removeUpdate(DocumentEvent e) {
	    checkToStop();
	}
    }


    public class ControlMouseAdapter extends JProcMouseAdapter {

	JSLCommand parent;

	public ControlMouseAdapter(JSLCommand parent) {
	    super(parent);
	    this.parent = parent;
	}

	public void mousePressed(MouseEvent e) {
	    super.mousePressed(e);
	    parent.requestFocus();
	    parent.jcc.textFocus = parent;
	    parent.jcc.updateLine(parent);
	}

	public void mouseReleased(MouseEvent e) {
	    super.mouseReleased(e);
	}
    }

    public void undo() {
	//if (undoManager.canUndo()) undoManager.undo();
    }

    public void redo() {
	//if (undoManager.canRedo()) undoManager.redo();
    }




}
