/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.awt.*;
import java.awt.event.*;
import yoyo.YoYo;
import javax.swing.*;
import javax.swing.text.*;
import java.util.Enumeration;

abstract class JSLDialog extends JDialog implements SLDialog {

    protected Frame parent;
    
    public static Font TEXT_FONT;
    public static Font LABEL_FONT;
    public OKCancel okcancel;
    private Component focus;

	static {
		try {
			TEXT_FONT = new Font("Monospaced", Font.PLAIN, 12);
   		    if (YoYo.macintoshp) TEXT_FONT = Font.getFont("starlogo.text.font", TEXT_FONT);
			LABEL_FONT = new Font("Dialog", Font.PLAIN, 12);
    	    if (YoYo.macintoshp) LABEL_FONT = Font.getFont("starlogo.label.font", LABEL_FONT);
		} catch (Exception e) { e.printStackTrace(); }
	}

    // automaticall adds window closer
    protected JSLDialog(Frame parent, String title, boolean modal) {
	this(parent, title, modal, false);
	
    }

    protected JSLDialog(Frame parent, String title, boolean modal, boolean resizable) {
	super(parent, title, modal);
	this.parent = parent;
	assert parent instanceof StarLogoFrame;
	addWindowListener(new Closer());
	//addWindowListener(new MenuEnabler());
	if (!(resizable || YoYo.unixp)) setResizable(false);
	addNotify();
    }


    protected JSLDialog(String title, boolean modal) {
	this(title, modal, false);
	
    }

    protected JSLDialog(String title, boolean modal, boolean resizable) {
	super();
	setTitle(title);
	setModal(modal);
	addWindowListener(new Closer());
	//addWindowListener(new MenuEnabler());
	if (!(resizable || YoYo.unixp)) setResizable(false);
	addNotify();
    }

    public void addNotify() {
	super.addNotify();
	if (YoYo.macosxp) {
	    setBackground(UIManager.getColor("window"));
	} else {
	    setBackground(SystemColor.control);
	}
    }	

    /**
     * Initializes button listener for ok/cancel.
     */
    protected void initOKCancel(JButton bok, JButton bcancel) {
	okcancel = new OKCancel(bok, bcancel);
    }

    /**
     * Sets default focus to <focus>.
     */
    protected void initFocus(Component focus) {
	// todo: differentiate between focus-grabbing techniques based on VM?
	this.focus = focus;
	FocusGrabber fg = new FocusGrabber(focus);
	addFocusListener(fg);
	addWindowListener(fg);
    }

    /**
     * Initializes key shortcuts.
     */
    protected void initShortcuts() {
	addKeyListeners(this, new Shortcuts(this));
    }

     
    //Size must be set before this function is called.
    protected void initLocation() {
	Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
	int screenwidth = screen.width;
	Dimension dialog = getSize();
	int dialogwidth = dialog.width;
	if (!YoYo.solarisp) setLocation((screenwidth - dialogwidth) / 2, 60);
    }

    /** 
     * Format text areas.
     */
    public static void formatText(JTextComponent tc) {
	tc.setFont(TEXT_FONT);
	colorText(tc);
    }

    /** 
     * Format labels.
     */
    public static void formatLabel(Component c) {
	c.setFont(LABEL_FONT);
	colorControl(c);
    }

    public static void colorControl(Component c) {
	/*
	  if (YoYo.macosxp) {
	  c.setBackground(UIManager.getColor("window"));
	  } else {
	  if (!(c instanceof JComboBox)) 
	  c.setBackground(SystemColor.control);
	  }
	  c.setForeground(SystemColor.controlText);
	*/
    }		

    public static void colorText(JTextComponent c) {
	c.setBackground(Color.white);
	c.setForeground(Color.black);
    }		


    // function to call when canceling out of dialog
    protected void cancel() {
	setVisible(false);
    }

    // function to call when accepting dialog
    protected void ok() {
	setVisible(false);
    }

    // adds <a> to <container> and all components of <container>
    protected static void addKeyListeners(Container container, KeyListener key) {
	// add to this
	container.addKeyListener(key);
	// add to components in this
	Component[] components = container.getComponents();
	for (int i=0; i<components.length; i++) {
	    if (components[i] instanceof Container) {
		addKeyListeners((Container)components[i], key);
	    } else {
		components[i].addKeyListener(key);
	    }
	}
    }

    public boolean shortcutsOn() { return true; }

    /*** classes ***/

    // invokes ok/cancel functions upon pressing ok/cancel buttons
    protected class OKCancel implements ActionListener {
	public final JButton bok;
	public final JButton bcancel;

	public OKCancel(JButton bok, JButton bcancel) {
	    this.bok = bok;
	    this.bcancel = bcancel;
	    if (bok!=null) {
		bok.addActionListener(this);
	    }
	    if (bcancel!=null) {
		bcancel.addActionListener(this);
	    }
	}

	public void actionPerformed(ActionEvent evt) {
	    JButton b = (JButton)evt.getSource();
	    if (b==bok) {
		ok();
	    } else if (b==bcancel) {
		cancel();
	    }
	}
    }

    // maps return to ok, escape to cancel
    private class Shortcuts extends KeyAdapter {

	private JSLDialog sld;

	public Shortcuts(JSLDialog sl) { this.sld = sl;}

	public void keyPressed(KeyEvent e) {
	    char ch = e.getKeyChar();
	    if ((ch == '\n') || (ch == '\r')) {
		if (sld.shortcutsOn()) {
		   e.consume();
		   ok();
		}
	    }
	    else if (ch == '\u001B') {
		if (sld.shortcutsOn()) {
		   e.consume();
		   cancel();
		}
	    }
	}
    }

    // calls cancel upon closing window
    private class Closer extends WindowAdapter {
	public void windowActivated(WindowEvent evt) {
	    if (JSLDialog.this.okcancel != null) {
		JButton defaultp = (JSLDialog.this.focus instanceof JButton) 
		    ? ((JButton)JSLDialog.this.focus) : JSLDialog.this.okcancel.bok;
		JSLDialog.this.getRootPane().setDefaultButton(defaultp);
	    }
	}

	public void windowClosing(WindowEvent evt) {
	    cancel();
	}
    }

    // grabs focus for given component when dialog is activated
    private class FocusGrabber extends WindowAdapter implements FocusListener  {
	Component focus;

	public FocusGrabber(Component focus) {
	    this.focus = focus;
	}

	// SUN needs this one
	public void focusGained(FocusEvent e) {
	    focus.requestFocus();
	}

	public void focusLost(FocusEvent e) {}

	// JVIEW needs this one (doesn't send focus event when dialog made active)
	public void windowActivated(WindowEvent evt) {
	    focus.requestFocus();
	}
    }

    static int avgFontHeight;
    static int avgFontWidth;

    public void initAverages() {
	/*
	UIDefaults  defaults = UIManager.getDefaults();
	Enumeration enum     = defaults.keys();
	while (enum.hasMoreElements()) {
	    Object key = enum.nextElement();
	    
	    System.out.println(key + ": " + defaults.get(key));
	}
	*/
	FontMetrics fm = Toolkit.getDefaultToolkit().getFontMetrics(UIManager.getFont("Button.font"));
	avgFontHeight = fm.getHeight();
	int[] charwidths = fm.getWidths();
	int totalFontWidth = 0;
	for (int i = 0; i < charwidths.length; i++) {
	    totalFontWidth += charwidths[i];
	}
	avgFontWidth = totalFontWidth / charwidths.length;
    }

    public int dluToPixelsHorizontal(int dlu) {
	if (avgFontHeight == 0 || avgFontWidth == 0) { initAverages(); }
	if (YoYo.pcp) {
	    return dlu * avgFontWidth / 4; 
	    
	} else return dlu;

    }

    public int dluToPixelsVertical(int dlu) {
	if (avgFontHeight == 0 || avgFontWidth == 0) { initAverages(); }
	if (YoYo.pcp) {
	    return dlu * avgFontHeight / 8; 
	    
	} else return dlu;
    }

    public void markChanged() {    }
    public void clearChanged() {    }
}









