/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.awt.*;
import java.awt.event.*;
import yoyo.*;
import java.util.*;

import ptolemy.*;
//import template.*;

import javax.swing.*;


public class JSLMenu extends JMenuBar implements MenuListener, SLMenu {

    JRadioButtonMenuItem currentfont;
    JRadioButtonMenuItem currentsize;
    
   
    JMenu filemenu, windmenu, editmenu, fontmenu, sizemenu, 
	formatmenu, ptolemymenu, helpmenu, colormenu, soundmenu;

    //added by MIM
    static JMenuItem[] templateMI;
    static JSLMenu[] slmenus;
    //end MIM changes
    
    JMenuBar mb = new JMenuBar();
    Frame parent;
    MenuListener ml;
    Application app;
    JMenu clearmenu;
    public int commandMask = Toolkit.getDefaultToolkit().getMenuShortcutKeyMask();

    public JSLMenu() {}

    public JSLMenu(Frame parent, MenuListener ml, Application app) { 
	setBorderPainted(false);
	
	filemenu = new JMenu(app.textBundle.getString("file_menu_title"));
	setMnemonic(filemenu, KeyEvent.VK_F);
	windmenu = new JMenu(app.textBundle.getString("windows_menu_title"));
	setMnemonic(windmenu, KeyEvent.VK_W);
	editmenu = new JMenu(app.textBundle.getString("edit_menu_title"));
	setMnemonic(editmenu, KeyEvent.VK_E);
	fontmenu = new JMenu(app.textBundle.getString("font_menu_title"));
	setMnemonic(fontmenu, KeyEvent.VK_O);
	sizemenu = new JMenu(app.textBundle.getString("size_menu_title"));
	setMnemonic(sizemenu, KeyEvent.VK_S);
	formatmenu = new JMenu(app.textBundle.getString("format_menu_title"));
	setMnemonic(formatmenu, KeyEvent.VK_M);
	ptolemymenu = new JMenu(app.textBundle.getString("windows_menu_plot_windows"));
	setMnemonic(ptolemymenu, KeyEvent.VK_P);
	colormenu = new JMenu(app.textBundle.getString("color_menu_title"));
	setMnemonic(colormenu, KeyEvent.VK_C);
	
	this.parent=parent;
	this.ml = ml;
	this.app = app;

	setupMenus();
	mb.add(filemenu);
	mb.add(editmenu);
	if (parent instanceof LineChart) {  // || (parent instanceof RTGraphWindow))) {
	    mb.add(formatmenu);
	} else if (parent instanceof ShapeChooser) {
	    mb.add(colormenu);
 	} else {
	    mb.add(fontmenu);
	    mb.add(sizemenu);
	}
	mb.add(windmenu);

	if (helpmenu == null) {
	    helpmenu = new JMenu(app.textBundle.getString("help_menu_title"));
	    setMnemonic(helpmenu, KeyEvent.VK_H);
	    mb.add(helpmenu);
	    helpmenu.add(newMenuItem((app.textBundle.getString("help_menu_faq")), 0));
	    if (!YoYo.macintoshp) {
		helpmenu.addSeparator();
		helpmenu.add(newMenuItem((app.textBundle.getString("help_menu_about")), 0, KeyEvent.VK_A));
	    }
	}
	// added by MIM
	if (YoYo.swingavailablep) addTemplateToSLMenus();
	// end MIM
    }

    public JSLMenu(Frame parent, Application app) {
	this(parent, (MenuListener)parent, app);
    }
    
    public void setupMenus() {
	setupFileMenu();
	setupEditMenu();
	setupWindowsMenu();
	setupFontMenu();
	setupSizeMenu();
	setupColorMenu();
	setupFormatMenu();
    }

    public void setupFileMenu() {
	// file menu
	JMenuItem mi;
	if (YoYo.macintoshp ||
	    parent instanceof ControlCenter ||
	    parent instanceof InterfaceFrame ||
	    parent instanceof LineChart) {
	    filemenu.add(newMenuItem(app.textBundle.getString("file_menu_new_project"), KeyEvent.VK_N, KeyEvent.VK_N));
	    filemenu.add(newMenuItem(app.textBundle.getString("file_menu_open_project"), KeyEvent.VK_O, KeyEvent.VK_O));
	    filemenu.add(newMenuItem(app.textBundle.getString("file_menu_open_project_url"), KeyEvent.VK_L, KeyEvent.VK_L));
	}
	if (parent instanceof ControlCenter ||
	    parent instanceof InterfaceFrame) {
	    filemenu.add(newMenuItem(app.textBundle.getString("file_menu_close_project"), KeyEvent.VK_W, KeyEvent.VK_C));
	} 
	else if (parent instanceof InfoWindow ||
		   parent instanceof LineChart ||
		   parent instanceof EntityMonitor ||
		   parent instanceof ShapeChooser ) {

	    filemenu.add(newMenuItem(app.textBundle.getString("close_menu_item"), KeyEvent.VK_W, KeyEvent.VK_C));
	}
	
	if (YoYo.macintoshp ||
	    parent instanceof ControlCenter ||
	    parent instanceof InterfaceFrame ||
	    parent instanceof LineChart) {
	    filemenu.addSeparator();

		filemenu.add(newMenuItem(app.textBundle.getString("file_menu_save_project"), KeyEvent.VK_S, KeyEvent.VK_S));
		filemenu.add(newMenuItem(app.textBundle.getString("file_menu_save_project_as"), 0, KeyEvent.VK_A));
		filemenu.add(newMenuItem(app.textBundle.getString("file_menu_save_project_as_webpage"), 0, KeyEvent.VK_W));
		filemenu.add(newMenuItem(app.textBundle.getString("file_menu_save_next_version"), 0, KeyEvent.VK_V));
	}
	
	if (YoYo.macintoshp ||
	    parent == app.outWin ||
	    parent instanceof ControlCenter ||
	    parent instanceof InterfaceFrame) {
	    filemenu.addSeparator();
	    	    
	    filemenu.add(newMenuItem(app.textBundle.getString("file_menu_save_output"), 0));
	    filemenu.add(newMenuItem(app.textBundle.getString("file_menu_save_output_as"), 0));
	}
	
	if (YoYo.macintoshp ||
	    parent == app.infoWin ||
	    parent instanceof ControlCenter ||
	    parent instanceof InterfaceFrame) {
	    filemenu.addSeparator();
	    
	    filemenu.add(newMenuItem(app.textBundle.getString("file_menu_save_info"), 0));
	    filemenu.add(newMenuItem(app.textBundle.getString("file_menu_save_info_as"), 0));
	}
	
	if (YoYo.macintoshp ||
	    parent instanceof ControlCenter ||
	    parent instanceof InterfaceFrame ||
	    parent instanceof LineChart) {
	    filemenu.addSeparator();
	    if (parent instanceof LineChart) {
		filemenu.add(newMenuItem(app.textBundle.getString("file_menu_save_plot_as_picture"), 0));
	    } 
	    else {
		filemenu.add(newMenuItem(app.textBundle.getString("file_menu_import_picture"), 0, KeyEvent.VK_I));
		JMenu exportMenu = new JMenu(app.textBundle.getString("file_menu_export_picture"));
		exportMenu.add(newMenuItem(app.textBundle.getString("export_menu_patches"), 0, KeyEvent.VK_P));
		exportMenu.add(newMenuItem(app.textBundle.getString("export_menu_turtles_and_patches"), 0, KeyEvent.VK_A));
		exportMenu.add(newMenuItem(app.textBundle.getString("export_menu_interface"), 0, KeyEvent.VK_I));
		filemenu.add(exportMenu);
	    }
	}

	if (parent instanceof ShapeChooser) {
	    filemenu.addSeparator();
	    filemenu.add(newMenuItem(app.textBundle.getString("file_menu_import_shape"), 0));
	    filemenu.add(newMenuItem(app.textBundle.getString("file_menu_export_shape"), 0));
	    filemenu.addSeparator();
	    filemenu.add(newMenuItem(app.textBundle.getString("file_menu_restore_default_shapes"), 0));
	}

	if (YoYo.macintoshp ||
	    parent instanceof SLPrintable) {
	    filemenu.addSeparator();
	    JMenuItem jmi = newMenuItemWithMask(app.textBundle.getString("file_menu_page_setup"), 
						KeyEvent.VK_P, KeyEvent.VK_P, 
						commandMask | InputEvent.SHIFT_MASK);
	    if (!(parent instanceof SLPrintable)) {
		jmi.setEnabled(false);
	    }
	    filemenu.add(jmi);
	 
	    if (parent instanceof ControlCenter) {
		jmi = newMenuItem(app.textBundle.getString("file_menu_print_procs"), 
				  KeyEvent.VK_P, KeyEvent.VK_P);
	    } else {
		jmi = newMenuItem(app.textBundle.getString("file_menu_print"), 
				  KeyEvent.VK_P, KeyEvent.VK_P);
	    }
	    if (!(parent instanceof SLPrintable)) {
		jmi.setEnabled(false);
	    }
	    filemenu.add(jmi);
	}

	if (YoYo.macintoshp ||
	    parent instanceof ControlCenter ||
	    parent instanceof InterfaceFrame) {
	    if (!YoYo.macosxp) {
		filemenu.addSeparator();
		filemenu.add(newMenuItem(app.textBundle.getString("file_menu_quit"), KeyEvent.VK_Q, KeyEvent.VK_Q));
	    }
	} 
    }

    public void setupFormatMenu() {
	if (parent instanceof LineChart) {

		formatmenu.add(newMenuItem(app.textBundle.getString("format_menu_data"), 0, KeyEvent.VK_D));
		formatmenu.add(newMenuItem(app.textBundle.getString("format_menu_plot_properties"), 0, KeyEvent.VK_T));
	}
    }

    public void setupWindowsMenu() {
	// window menu
	if (YoYo.macintoshp || !(parent instanceof ControlCenter)) {
	    windmenu.add(newMenuItem(app.textBundle.getString("windows_menu_control_center"), KeyEvent.VK_R, KeyEvent.VK_R));
	}
	
	//if (YoYo.macintoshp || !(parent instanceof ControlCenter)) {
	//    windmenu.add(newMenuItem(app.textBundle.getString("windows_menu_music_center")));
	//}

	if (YoYo.macintoshp) {
	    windmenu.add(newMenuItem(app.textBundle.getString("windows_menu_info_window"), KeyEvent.VK_I));
	} 
	
	else {
	    if (!(parent == app.infoWin)) {
		windmenu.add(newMenuItem(app.textBundle.getString("windows_menu_info_window"), KeyEvent.VK_I, KeyEvent.VK_I));
	    }
	}
	
	if (YoYo.macintoshp || !(parent == app.outWin)) {
	    windmenu.add(newMenuItem(app.textBundle.getString("windows_menu_output_window"), 0, KeyEvent.VK_O));
	}

	if (YoYo.macintoshp || (parent instanceof InterfaceFrame)) { //JFK
	    //|| !(parent instanceof RTGraphWindow)) {
	    JMenuItem mi = newMenuItem(app.textBundle.getString("ptolemy_menu_create_new_plot"), 0);
	    mi.setEnabled(true);
	    ptolemymenu.add(mi);
	    windmenu.add(ptolemymenu);
	}
	
	if (YoYo.macintoshp || !(parent instanceof InterfaceFrame)) {
	    windmenu.add(newMenuItem(app.textBundle.getString("windows_menu_interface"), 0, KeyEvent.VK_S));
	}
	
	if (YoYo.macintoshp || !(parent instanceof ShapeChooser)) {
	    windmenu.add(newMenuItem(app.textBundle.getString("windows_menu_shape_chooser"), 
				     KeyEvent.VK_H, KeyEvent.VK_H));
	}
	if (!(YoYo.macintoshp)) {
	    windmenu.add(newMenuItem(app.textBundle.getString("windows_menu_template"), 
				     0, KeyEvent.VK_T));
	}

    }

    public void setupEditMenu() {
	if ((((parent instanceof ControlCenter || parent instanceof InfoWindow) 
	     && !(parent instanceof OutputWindow)) && YoYo.swingavailablep) ||
	    parent instanceof InterfaceFrame) {
	    editmenu.add(newMenuItem(app.textBundle.getString("edit_menu_undo"), KeyEvent.VK_Z, KeyEvent.VK_U));
	    JMenuItem jmi = newMenuItem(app.textBundle.getString("edit_menu_redo"), 0, KeyEvent.VK_R);
	    jmi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z, commandMask | ActionEvent.SHIFT_MASK));
	    editmenu.add(jmi);
	    editmenu.addSeparator();
	}
	
	    editmenu.add(newMenuItem(app.textBundle.getString("edit_menu_cut"), KeyEvent.VK_X, KeyEvent.VK_T));
	    editmenu.add(newMenuItem(app.textBundle.getString("edit_menu_copy"), KeyEvent.VK_C, KeyEvent.VK_C));
	    editmenu.add(newMenuItem(app.textBundle.getString("edit_menu_paste"), KeyEvent.VK_V, KeyEvent.VK_P));

	    if (!(parent instanceof ShapeChooser)) {
		editmenu.addSeparator();
		
		if (parent instanceof LineChart) {
		    editmenu.add(newMenuItem(app.textBundle.getString("edit_menu_clear_plot"), 0, 0));
		    //clearmenu = new JMenu("Clear Plot");
		    //clearmenu.addActionListener(this);
		    //editmenu.add(clearmenu);
		}  else {
		    editmenu.add(newMenuItem(app.textBundle.getString("edit_menu_clear"), KeyEvent.VK_DELETE, KeyEvent.VK_A));
		}
		
		editmenu.add(newMenuItem(app.textBundle.getString("edit_menu_select_all"), KeyEvent.VK_A, KeyEvent.VK_L));
	    }

	
	if(parent instanceof ControlCenter){
	    JCheckBoxMenuItem ssp = new JCheckBoxMenuItem(app.textBundle.getString("show_spaces_label"),false);
	    ssp.addItemListener(this);
	    // using ctrl-b instead- doesn't seem to be taken
	    ssp.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_B, commandMask));
	    setMnemonic(ssp, KeyEvent.VK_S);
	    editmenu.addSeparator();
	    editmenu.add(ssp);
	}
	
	if (parent instanceof InterfaceFrame) {
	    editmenu.addSeparator();
	    JCheckBoxMenuItem mi = null;
	    mi = new JCheckBoxMenuItem(app.textBundle.getString("edit_menu_snap_to_grid"),false);
	    mi.addItemListener(this);
	    mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_G, commandMask));
	    setMnemonic(mi, KeyEvent.VK_G);
	    editmenu.add(mi);
	    
	}
	
	if (parent instanceof InterfaceFrame) {
	    JCheckBoxMenuItem mi = null;
	    mi = new JCheckBoxMenuItem(app.textBundle.getString("edit_menu_lock_interface"),false);
	    mi.addItemListener(this);
	    editmenu.add(mi);
	}

	editmenu.addSeparator();
	editmenu.add(newMenuItem(app.textBundle.getString("edit_menu_calibrate_graphics")));


    }


    public void setupFontMenu() {
	JRadioButtonMenuItem jrbmi;
	// fontmenu
	//long now = System.currentTimeMillis();
	//	String[] fonts = Toolkit.getDefaultToolkit().getFontList();
	String[] fonts = GraphicsEnvironment.getLocalGraphicsEnvironment().getAvailableFontFamilyNames();
	//System.out.println("time to iterate fonts: " + (System.currentTimeMillis() - now));
	ButtonGroup fontgroup = new ButtonGroup();
	for (int i = 0; i< fonts.length; i++) {
	    jrbmi = new JRadioButtonMenuItem(fonts[i]);

	    //jrbmi.setFont(new Font(fonts[i], Font.PLAIN, 11));
	    if (fonts[i].equals("Monospaced")) {
		jrbmi.setSelected(true);
		currentfont = jrbmi;
	    }
	    if (!fonts[i].equals("ZapfDingbats")) {
		fontgroup.add(jrbmi);
		fontmenu.add(jrbmi);
	    }
	    jrbmi.addItemListener(this);
	}
    }

    public void setupSizeMenu() {
	JRadioButtonMenuItem jrbmi;
	ButtonGroup sizegroup = new ButtonGroup();
	for (int i = 0; i< sizes.length; i++) {
	    jrbmi = new JRadioButtonMenuItem(sizes[i]);
	    if (sizes[i].equals("12")) {
		jrbmi.setSelected(true);
		currentsize = jrbmi;
	    }
	    sizegroup.add(jrbmi);
	    sizemenu.add(jrbmi);
	    jrbmi.addItemListener(this);
	}

    }

    public void setupColorMenu() {
	colormenu.add(newMenuItem(app.textBundle.getString("color_menu_turtle_color"), 0, KeyEvent.VK_T));
	colormenu.add(newMenuItem(app.textBundle.getString("color_menu_patch_color"), 0, KeyEvent.VK_P));
	//colormenu.add(newMenuItem("Write Shapes to File", 0, KeyEvent.VK_W));
    }

    public void enableClearMenu() {
	if (clearmenu != null) clearmenu.setEnabled(true);
    }

    public void disableClearMenu() {
	if (clearmenu != null) clearmenu.setEnabled(false);
    } 

    public void disableMoreEditMenu() {
	for(int i = 0; i<editmenu.getItemCount(); i++) {
	    JMenuItem mi = editmenu.getItem(i);
	    if (mi != null) {
		if (!(mi.getLabel().equals(app.textBundle.getString("edit_menu_clear"))) &&
		    !(mi.getLabel().equals(app.textBundle.getString("edit_menu_clear_plot"))))
		    mi.setEnabled(false);
	    }
	}
    }

    public void disableEditMenu() {
	for(int i = 0; i<editmenu.getItemCount(); i++) {
	    JMenuItem mi = editmenu.getItem(i);
	    if (mi.getLabel() != app.textBundle.getString("edit_menu_select_all"))
		mi.setEnabled(false);
	}
    }

    public void disablePasteMenu() {
  	for(int i=0; i<editmenu.getItemCount(); i++) {
  		JMenuItem mi = editmenu.getItem(i);
  		if ((mi != null) && (mi.getLabel() == app.textBundle.getString("edit_menu_paste")))
  			mi.setEnabled(false);
  	}
    }

	public void enablePasteMenu() {
	    for(int i=0; i<editmenu.getItemCount(); i++) {
  		JMenuItem mi = editmenu.getItem(i);
  		if ((mi != null) && (mi.getLabel() == app.textBundle.getString("edit_menu_paste")))
		    mi.setEnabled(true);
	    }
	}

	public void disableDataInFormatMenu() {
		formatmenu.getItem(0).setEnabled(false);
	}

	public void disableCutCopyClearMenu() {
	    for(int i=0; i<editmenu.getItemCount(); i++) {
  		JMenuItem mi = editmenu.getItem(i);
		if (mi != null &&
		    ((mi.getLabel() == app.textBundle.getString("edit_menu_cut")) || 
		     (mi.getLabel() == app.textBundle.getString("edit_menu_copy")) || 
		     (mi.getLabel() == app.textBundle.getString("edit_menu_clear")) ||
		     (mi.getLabel() == app.textBundle.getString("edit_menu_clear_plot"))))
		    mi.setEnabled(false);
	    }
	}

    public void enableCutCopyClearMenu() {
	for(int i=0; i<editmenu.getItemCount(); i++) {
	    JMenuItem mi = editmenu.getItem(i);
	    if ((mi != null) && 
		((mi.getLabel() == app.textBundle.getString("edit_menu_cut")) || 
		 (mi.getLabel() == app.textBundle.getString("edit_menu_copy")) || 
		 (mi.getLabel() == app.textBundle.getString("edit_menu_clear")) || 
		 (mi.getLabel() == app.textBundle.getString("edit_menu_clear_plot"))))
		mi.setEnabled(true);
  	}
    }
    
    public void setEditMenuState(String name, boolean state) {
	for(int i = 0; i<editmenu.getItemCount(); i++) {
	    JMenuItem mi = editmenu.getItem(i);
	    if (mi != null && mi.getText().equals(name)) // menu item could be a separator, i.e. null
		mi.setEnabled(state);
	}
    }

    public void setLockState(boolean state) {
	int j=-1;
  	for(int i=0; i<editmenu.getItemCount(); i++) {
  		JMenuItem mi =  editmenu.getItem(i);
		//System.out.println (mi);
    		if ((mi != null) && (mi.getLabel() == app.textBundle.getString("edit_menu_lock_interface")))
			{ j = i;
			// System.out.println ("j=" + j);
			}
		}
	       JCheckBoxMenuItem mj = (JCheckBoxMenuItem) editmenu.getItem(j);
  			mj.setState(state);
  	
    }

    public boolean isATitle(String title) {
	Object lw = app.iface.getLineWidgetFromTitle(title);
	if (lw instanceof LineWidget) {
	    ((LineWidget)lw).show();
	    return true;
	}
	return false;
    }

	static String getMenuName(LineChart lc) {
		return lc.getPlotId() + ". " + lc.getTitle();
	}

//     public void addPlotWindow(String s) {
	public void addPlotWindow(LineChart lc) {
		String menuItemText = JSLMenu.getMenuName(lc);
	for(int i=0; i< slmenus.length; i++) {
	    if (slmenus[i].ptolemymenu.getItemCount() == 1 &&
		(slmenus[i].ptolemymenu.getItem(0).getLabel().equals(app.textBundle.getString("ptolemy_menu_create_new_plot")))){
		slmenus[i].ptolemymenu.removeAll();
	    }
	    JMenuItem m = new JMenuItem(menuItemText);
	    m.addActionListener(slmenus[i]);
	    slmenus[i].ptolemymenu.add(m);
	    //System.out.println("in addMe(String)...");
	}
    }

    public void changePlotWindow(String s, String sNew) {
	if (s.equals(sNew)) return;
	//System.out.println("Change plot window from " + s + " to " + sNew);
	for(int i=0; i< slmenus.length; i++) {
	    for (int j=0; j < slmenus[i].ptolemymenu.getItemCount(); j++) {
		if (slmenus[i].ptolemymenu.getItem(j).getLabel().equals(s)) {
		    slmenus[i].ptolemymenu.getItem(j).setLabel(sNew);
		}
	    }
	    //System.out.println("in changeMe(String)...");
	}
    }


    public void removePlotWindow(LineChart lc) {
			String s = JSLMenu.getMenuName(lc);
	for(int i=0; i< slmenus.length; i++) {
	    if (slmenus[i].ptolemymenu.getItemCount() == 1) {
		slmenus[i].ptolemymenu.removeAll();
		JMenuItem mi;
		
		mi = newMenuItem(app.textBundle.getString("ptolemy_menu_create_new_plot"), 0);

		slmenus[i].ptolemymenu.add(mi);
	    } else {
		int count = slmenus[i].ptolemymenu.getItemCount();
		for (int j=0; j<count; j++) {
		    JMenuItem m = slmenus[i].ptolemymenu.getItem(j);
		    if (m.getLabel().equals(s)) {
			slmenus[i].ptolemymenu.remove(j);
			break;
		    }
		}
	    }
	    //	System.out.println("in removeMe(String)...");
	}
    }

    public void removeAllPlotWindows() {
	for(int i=0; i< slmenus.length; i++) {
	    slmenus[i].ptolemymenu.removeAll();
	    JMenuItem mi;

	    mi = newMenuItem(app.textBundle.getString("ptolemy_menu_create_new_plot"), 0);
		
	    mi.setEnabled(false);
	    slmenus[i].ptolemymenu.add(mi);
	}
	//    System.out.println("in removeMe(String)...");
    }


    public void addTemplateToSLMenus(){
	if(slmenus == null){
	    slmenus = new JSLMenu[1];
	    templateMI = new JMenuItem[1];
	    slmenus[0] = this;
	    templateMI[0] = new JMenuItem(app.textBundle.getString("windows_menu_template"));
	    templateMI[0].addActionListener(this);
	}
	//need to clean up this template issue
	else {
	    JSLMenu[] temp =  new JSLMenu[slmenus.length + 1];
	    System.arraycopy(slmenus, 0, temp, 0, slmenus.length);
	    temp[slmenus.length] = this;
	    slmenus = temp;

	    JMenuItem[] tempMI = new JMenuItem[templateMI.length + 1];
	    System.arraycopy(templateMI, 0, tempMI, 0, templateMI.length);
	}
    }

    // end added by MIM

    //long prevItemEvent = 0;
    // delegate events to responsible parties
    public void itemStateChanged(ItemEvent evt) {
	/*
	  if (YoYo.macosxp) {
	    //System.out.println("item State Changed: " + evt);
	    long now = System.currentTimeMillis();
	    //System.out.println("time is " + now);
	    if (now < prevItemEvent + 750) {
		return;
	    }
	    prevItemEvent = now;
	}
	*/
	ml.itemStateChanged(evt);
    }


    public void actionPerformed(ActionEvent evt) {
	synchronized(app.sl.runningp) {
	    synchronized (app.sl.patchcanvas.getGraphicsLock()) {
		try {
		    if (app.sl.patchcanvas instanceof PatchCanvas)
			((PatchCanvas)app.sl.patchcanvas).setDrawingSurface();
		    if (evt.getSource() instanceof JMenuItem) {
			JMenuItem mi = (JMenuItem)evt.getSource();
			String arg = mi.getText();
			boolean handled = handleMenuItemAction(mi, arg);
			if (!handled) ml.actionPerformed(evt);
		    }
		}
		finally {
		    if (app.sl.patchcanvas instanceof PatchCanvas)
			((PatchCanvas)app.sl.patchcanvas).deleteDrawingSurface();
		}
	    }
	}
    }   

    public boolean handleMenuItemAction(JMenuItem mi, String arg) {
	if (arg.equals(app.textBundle.getString("file_menu_quit"))){
	    app.project.quit();
	    return true;
	}
	else if (arg.equals(app.textBundle.getString("file_menu_new_project")) || 
		 arg.equals(app.textBundle.getString("file_menu_close_project"))) {
	    Project.newProject(app);
	    return true;
	}
	else if (arg.equals(app.textBundle.getString("file_menu_open_project"))) {
	    Project.openProject(app);
	    return true;
	}
	else if (arg.equals(app.textBundle.getString("file_menu_open_project_url"))) {
	    Project.openURLProject(app);
	    return true;
	}
	else if (arg.equals(app.textBundle.getString("close_menu_item"))) {
	    parent.setVisible(false);
	    return true;
	}
	else if (arg.equals(app.textBundle.getString("file_menu_save_project"))) {
	    app.project.saveProject(app, false);
	    return true;
	}
	else if (arg.equals(app.textBundle.getString("file_menu_save_project_as"))) {
	    app.project.saveProjectAs(app, false);
	    return true;
	}
	else if (arg.equals(app.textBundle.getString("file_menu_save_next_version"))) {
	    app.project.saveVersion(app);
	    return true;
	}
	else if (arg.equals(app.textBundle.getString("file_menu_save_project_as_webpage"))) {
	    app.project.saveProject(app, true);
	    return true;
	}
	else if (arg.equals(app.textBundle.getString("file_menu_page_setup"))) {
	    Print.setupPage();
	    return true;
	}
	else if (arg.equals(app.textBundle.getString("file_menu_print")) ||
		 arg.equals(app.textBundle.getString("file_menu_print_procs"))) {
	    if (parent instanceof SLPrintable) 
		((SLPrintable)parent).print();
	    return true;
	}
	else if (arg.equals(app.textBundle.getString("file_menu_import_picture"))) {
	    
	    app.sl.PictInstance.loadPicture(app.sl);
	    synchronized(app.sl.runningp) {
		app.sl.redraw();
	    }
	    return true;
	}
	else if (arg.equals(app.textBundle.getString("file_menu_save_plot_as_picture"))) {
	    try {
		((LineChart)parent)._exportImage();
	    } catch (Exception e) {
		e.printStackTrace(System.err);
		Application.createMessageDialog(app, app.activeFrame, 
						app.textBundle.getString("starlogo_dialog_title"), 
						app.textBundle.getString("error_saving_plot_image_to_file") + 
						": " + e.getMessage(), 
						false);
	    }
	    //System.out.println("Saving plot...");
	    return true;
	}
	else if (arg.equals(app.textBundle.getString("edit_menu_calibrate_graphics"))) {
	    new Thread(new Runnable() {
		    public void run() {
			app.sl.calibrateGraphics();
		    }
		}).start();
	    return true;
	}
	else if (arg.equals(app.textBundle.getString("format_menu_data"))) {
	    ((LineChart)parent)._data();
	    return true;
	    //app.project.markChanged();
	}
	else if (arg.equals(app.textBundle.getString("format_menu_plot_properties"))) {
	    //System.out.println("plot properties");
	    String oldTitle = getMenuName((LineChart)parent);
	    ((LineChart)parent)._properties();
	    changePlotWindow(oldTitle, getMenuName((LineChart)parent));
	    return true;
	    //app.project.markChanged();
	}
	else if (arg.equals(app.textBundle.getString("edit_menu_clear_plot"))) {
	    ((LineChart)parent).reset();
	    return true;
	}
	else if (arg.equals(app.textBundle.getString("export_menu_interface"))) {
	    app.sl.PictInstance.savePicture(app.sl, Pictures.INTERFACE);
	    return true;
	}
	else if (arg.equals(app.textBundle.getString("export_menu_patches"))) {
	    app.sl.PictInstance.savePicture(app.sl, Pictures.PATCHES);
	    return true;
	}
	else if (arg.equals(app.textBundle.getString("export_menu_turtles_and_patches"))) {
	    app.sl.PictInstance.savePicture(app.sl, Pictures.TURTLES_AND_PATCHES);
	    return true;
	}
	else if (arg.equals(app.textBundle.getString("file_menu_save_output"))) {
	    app.project.saveText(app, Project.SAVE_OUTPUT);
	    return true;
	}
	else if (arg.equals(app.textBundle.getString("file_menu_save_output_as"))) {
	    app.project.saveTextAs(app, Project.SAVE_OUTPUT);
	    return true;
	}
	else if (arg.equals(app.textBundle.getString("file_menu_save_info"))) {
	    app.project.saveText(app, Project.SAVE_INFO);
	    return true;
	}
	else if (arg.equals(app.textBundle.getString("file_menu_save_info_as"))) {
	    app.project.saveTextAs(app, Project.SAVE_INFO);
	    return true;
	}
	else if (arg.equals(app.textBundle.getString("windows_menu_interface"))) {
	    app.iface.frame().setVisible(true);
	    app.iface.frame().requestFocus();
	    return true;
	}
	// begin code added by MIM
	else if (arg.equals(app.textBundle.getString("windows_menu_template"))) {
	    app.theTemplate.show();
	    app.theTemplate.requestFocus();
	    return true;
	}
	// end MIM addition
	else if (arg.equals(app.textBundle.getString("windows_menu_control_center"))) {
	    app.procWin.setVisible(true);
	    app.procWin.requestFocus();
	    return true;
	}
	//else if (arg.equals(app.textBundle.getString("windows_menu_music_center"))) {
	//    app.procWin.setVisible(true);
	//    app.procWin.requestFocus();
	//    return true;
	//}
	else if (arg.equals(app.textBundle.getString("windows_menu_info_window"))) {
	    app.infoWin.setVisible(true);
	    app.infoWin.requestFocus();
	    return true;
	}
	else if (arg.equals(app.textBundle.getString("windows_menu_output_window"))) {
	    app.outWin.setVisible(true);
	    app.outWin.requestFocus();
	    return true;
	}
	
	// tim
	else if (arg.equals(app.textBundle.getString("windows_menu_shape_chooser"))) {
	    app.shapeChooser.setVisible(true);
	    app.shapeChooser.requestFocus();
	    return true;
	}
	
	else if (arg.equals(app.textBundle.getString("ptolemy_menu_create_new_plot"))) {		
	    LineWidget.createNew(app.iface, this); 
	    return true; 
	} 
	else if (arg.equals(app.textBundle.getString("color_menu_turtle_color"))) {
	    JColortoolDialog dialog = new JColortoolDialog(app, app.shapeChooser, true);
	    dialog.setVisible(true);
	    return true;
	}
	
	else if (arg.equals(app.textBundle.getString("color_menu_patch_color"))) {
	    JColortoolDialog dialog = new JColortoolDialog(app, app.shapeChooser, false);
	    dialog.setVisible(true);
	    return true;
	}
	else if (arg.equals(app.textBundle.getString("help_menu_about"))){
	    app.createAboutDialog((Frame)app.activeFrame,app).show();
	    return true;
	}
	else if (arg.equals(app.textBundle.getString("help_menu_faq"))) {
	    app.createFAQDialog(app, (Frame)app.activeFrame).show();
	    return true;
	}
	else if (isATitle(arg)) {
	    return true;
	}
	return false;
    } 
			
		
    public void setCurrentSize(Object o) {
	JRadioButtonMenuItem jrbmi = (JRadioButtonMenuItem)o;
	currentsize = jrbmi;
    }

    public void setCurrentFont(Object o) {
	JRadioButtonMenuItem jrbmi = (JRadioButtonMenuItem)o;
	currentfont = jrbmi;
    }

    public void setMenuBar(Frame f) {
	JFrame frame = (JFrame)f;
	frame.setJMenuBar(mb);
    }

    public JMenuItem newMenuItem(String text) {
	JMenuItem mi = new JMenuItem(text);
	mi.addActionListener(this);
	return mi;
    }
    
    public JMenuItem newMenuItemWithMask(String text, int accel, int mnemonic, int mask) {
	JMenuItem mi = newMenuItemWithMask(text, accel, mask);
	setMnemonic(mi, mnemonic);
	return mi;
    }

    public JMenuItem newMenuItemWithMask(String text, int accel, int mask) {
	JMenuItem mi = new JMenuItem(text);
	if (accel != 0)
	    mi.setAccelerator(KeyStroke.getKeyStroke(accel, mask));
	mi.addActionListener(this);
	return mi;
    }

    public JMenuItem newMenuItem(String text, int accel) {
	return newMenuItemWithMask(text, accel, commandMask);
    }

    public JMenuItem newMenuItem(String text, int accel, int mnemonic) {
	JMenuItem mi = newMenuItem(text, accel);
	setMnemonic(mi, mnemonic);
	return mi;
    }

    public void setMnemonic(AbstractButton ab, int mnemonic) {
	if (!YoYo.macintoshp)
	    ab.setMnemonic(mnemonic);
    }
}
