/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.awt.Dimension;
import javax.swing.undo.UndoManager;
import javax.swing.event.UndoableEditEvent;
import java.io.PrintWriter;
import javax.swing.JLabel;
import javax.swing.JTextArea;
import javax.swing.DefaultComboBoxModel;
import java.util.Hashtable;
import java.awt.Font;
import java.awt.Color;
import yoyo.JTEStream;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import javax.swing.text.JTextComponent;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;
import java.awt.ItemSelectable;
import javax.swing.JComboBox;
import javax.swing.text.BadLocationException;
import java.util.Enumeration;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.swing.ComboBoxModel;
import javax.swing.event.ListDataEvent;

import yoyo.YoYo;

public class JSLProc extends JTextArea implements SLProc, JControlCenterTextArea {

  boolean turtlep;
  PrintWriter pw;
  JControlCenter jcc;
  JLabel label;
  JComboBox jcb;
  ItemListener itemlistener;
  StarLogo sl;
    Application app;

  boolean showspaces = false;

  public static Font PROC_FONT;

  static {
  	try {
 	  PROC_FONT = new Font("Monospaced", Font.PLAIN, 12);
 	  if (YoYo.macintoshp) PROC_FONT = Font.getFont("starlogo.procedures.font", PROC_FONT);
    } catch (Exception e) { e.printStackTrace(); }
  }

    UndoManager undoManager;

    public void setupUndoManager() {
	undoManager = new UndoManager() {
		public void undoableEditHappened(UndoableEditEvent e) {
		    super.undoableEditHappened(e);
		    if (canUndo()) {
			jcc.slmenu.setEditMenuState(app.textBundle.getString("edit_menu_undo"), true);
		    } else jcc.slmenu.setEditMenuState(app.textBundle.getString("edit_menu_undo"), false);
		    if (canRedo()) {
			jcc.slmenu.setEditMenuState(app.textBundle.getString("edit_menu_redo"), true);
		    } else jcc.slmenu.setEditMenuState(app.textBundle.getString("edit_menu_redo"), false);
		}
		public void redo() {
		    super.redo();
		    if (canUndo()) {
			jcc.slmenu.setEditMenuState(app.textBundle.getString("edit_menu_undo"), true);
		    } else jcc.slmenu.setEditMenuState(app.textBundle.getString("edit_menu_undo"), false);
		    if (canRedo()) {
			jcc.slmenu.setEditMenuState(app.textBundle.getString("edit_menu_redo"), true);
		    } else jcc.slmenu.setEditMenuState(app.textBundle.getString("edit_menu_redo"), false);
		}
		public void undo() {
		    super.undo();
		    if (canUndo()) {
			jcc.slmenu.setEditMenuState(app.textBundle.getString("edit_menu_undo"), true);
		    } else jcc.slmenu.setEditMenuState(app.textBundle.getString("edit_menu_undo"), false);
		    if (canRedo()) {
			jcc.slmenu.setEditMenuState(app.textBundle.getString("edit_menu_redo"), true);
		    } else jcc.slmenu.setEditMenuState(app.textBundle.getString("edit_menu_redo"), false);
		}
	    };
    }

  public JSLProc(boolean turtlep, StarLogo sl, JControlCenter jcc) {
    super("");
    setLineWrap(false);
    pw = new PrintWriter(new JTEStream(this), true);

    this.sl = sl;
    this.app = sl.app;
    this.jcc = jcc;
    this.turtlep = turtlep;
    setupUndoManager();
    setBackground(Color.white);
    setForeground(Color.black);
    setFont(PROC_FONT);
    label = new JLabel(((turtlep) ? sl.app.textBundle.getString("turtle_procedures_label") 
			: sl.app.textBundle.getString("observer_procedures_label")), JLabel.LEFT) {
	/*
	public Dimension getPreferredSize() {
	  return new Dimension(super.getPreferredSize().width,
			       getFontMetrics(getFont()).getHeight() + ControlCenter.LABEL_VERTICAL_SPACE);
			       }*/
      };
    label.setFont(new Font("Dialog", Font.PLAIN, 12));
    //    label.setOpaque(true);
    label.setForeground(Color.white);
    label.setBackground((turtlep) ? new Color(23, 118, 51) : new Color(92, 61, 36));
      getDocument().addUndoableEditListener(undoManager);
    addKeyListener(new ProcKeyAdapter(this));
    addCaretListener(jcc);
    addMouseListener(new ControlMouseAdapter(this));


    //temp -- later want this initialized to false
    //    this.showspaces = false;
  }

  Hashtable ufuns = new Hashtable();

  public void setProcs(Object[] procnames, Integer[] linenumbers) {
  	 jcb.removeItemListener(itemlistener);
  	 jcb.removeAllItems();
  	 //sort(procnames, linenumbers);
	 if (procnames.length == 0) {
	     jcb.addItem(sl.app.textBundle.getString("procedures_label"));
	 } else {
	     for(int i = 0; i < procnames.length; i++) {
		 ufuns.put(procnames[i], linenumbers[i]);
		 jcb.addItem(procnames[i]);
	     }
	 }
	 if (itemlistener == null) {gotoLine(0);}
	 itemlistener = new ItemListener() {
		 public void itemStateChanged(ItemEvent e) {
		     ItemSelectable is = e.getItemSelectable();
		     Object o = is.getSelectedObjects()[0];
		     Object p = ufuns.get(o);
		     requestFocus();
		     if (p != null) gotoLine(((Integer)p).intValue() - 1);
		 }
	     };
	 jcb.addItemListener(itemlistener);
  }

  public boolean lookupUfun(String name) {
    return ufuns.contains(name);
  }

  public void clearUfuns() {
    ufuns.clear();
    jcb.removeItemListener(itemlistener);
    jcb.removeAllItems();
    jcb.addItem(sl.app.textBundle.getString("procedures_label"));
    if (itemlistener == null) {gotoLine(0);}
    itemlistener = new ItemListener() {
	    public void itemStateChanged(ItemEvent e) {
		ItemSelectable is = e.getItemSelectable();
		Object o = is.getSelectedObjects()[0];
		Object p = ufuns.get(o);
		requestFocus();
		if (p != null) gotoLine(((Integer)p).intValue() - 1);
	    }
	};
    jcb.addItemListener(itemlistener);
  }

  public JComboBox procMenuList() {
  	if (jcb == null) jcb = new JComboBox(new DefaultComboBoxModel()) {
	    public void contentsChanged(ListDataEvent e) {
	      ComboBoxModel mod = getModel();
	      Object newSelectedItem = mod.getSelectedItem();

	      if ( selectedItemReminder == null ) {
		if ( newSelectedItem != null )
		  selectedItemChanged();
	      }
	      else {
		  selectedItemChanged();
	      }

	      if ( !this.isEditable() && newSelectedItem != null ) {
		int i,c;
		boolean shouldResetSelectedItem = true;
		Object o;
		Object selectedItem = mod.getSelectedItem();

		for ( i=0,c=mod.getSize();i<c;i++ ) {
		  o = mod.getElementAt(i);
		  if ( o.equals(selectedItem) ) {
                    shouldResetSelectedItem = false;
                    break;
		  }
		}

		if ( shouldResetSelectedItem ) {
		  if ( mod.getSize() > 0 )
                    setSelectedIndex(0);
		  else
                    setSelectedItem(null);
		}
	      }
	    }
	  };

	jcb.addItem(sl.app.textBundle.getString("procedures_label"));
	
  	return jcb;
  }

    public static void sort(String[] names) {
    }

    
  public void gotoLine(int lineno) {
//System.out.println("gotoline " + lineno);
     try {
       setCaretPosition(getLineStartOffset(lineno));
     } catch (BadLocationException ble) { } //System.out.println("bad line number " + lineno); }
  }

  class ProcKeyAdapter extends KeyAdapter {

    JSLProc parent;

    ProcKeyAdapter(JSLProc parent) {
      this.parent = parent;
    }
           
    public void keyPressed(KeyEvent e) {
      char key = e.getKeyChar();
      if (key == '\t' && e.isControlDown()) {
	// toggle between observer and turtle procs upon ctrl-tab
	parent.jcc.toggleTabs();
	//e.consume();
	return;
      } 
      if (YoYo.macintoshp && e.isMetaDown() && e.getKeyChar() == '.') {
	  synchronized(parent.sl.runningp) {
	      parent.sl.stopAllStarLogo();
	  }
	  //e.consume();
	  return;
      }
      if (!YoYo.macintoshp && e.getKeyCode() == KeyEvent.VK_ESCAPE) {
	  synchronized(parent.sl.runningp) {
	      parent.sl.stopAllStarLogo();
	  }
	  //e.consume();
	  return;
      }
      // if on the pc, eat an ALT-E in case the user is trying to access edit
      // menu.  He'll delete his selection if the e goes through.
      if (!YoYo.macintoshp && e.getKeyChar()=='e' && e.isAltDown()) {
	  //e.consume();
	  return;
      }
      // on the pc, delete doesn't register a keytype, so catch it manually in
      // a keypressed and delegate so that we can detect changes in the text.
      // same with shift-ins, incase they're inserting text.
      if (!YoYo.macintoshp && (e.getKeyCode()==KeyEvent.VK_DELETE ||
			       (e.getKeyCode()==KeyEvent.VK_INSERT && e.isShiftDown())))	{
	  keyTyped(e);
	  return;
      }
    }


      public void keyTyped(KeyEvent e) {
	  //    //System.out.println("key typed " + new Integer(e.getKeyChar()));
	  char c = e.getKeyChar();
	  //	char c;
	  //       // if (c=='\t' && e.getModifiers() == 0) {
	  // // 	parent.pw.print("   ");
	  // // 	parent.pw.flush();
	  // // 	e.consume();
	  // //       }
	  
	  // 	  //trying to add "show spaces"
 	  if (c == KeyEvent.VK_SPACE && this.parent.showspaces == true) {
	      c = 0x25CF; //get a black dot from unicode
	      setText(getText().replace(' ', c));
	      //test
	      //e.consume();
	  }
	  
	  // 	  if (this.parent.showspaces ==false){
	  // 	      c = 0x25CF; //get a black dot from unicode
	  // 	      setText(getText().replace(c,' '));
	  // 	  }
	  
	  parent.jcc.app.rancompilerp=false;
	  parent.jcc.app.rancompilerpForVariables = false;
	  parent.jcc.app.project.markChanged();
      }
      
  }

  public class ControlMouseAdapter extends JProcMouseAdapter {

    JSLProc parent;

    public ControlMouseAdapter(JSLProc parent) {
      super(parent);
      this.parent = parent;
    }

    public void mousePressed(MouseEvent e) {
      super.mousePressed(e);
      parent.requestFocus();
      parent.jcc.textFocus = parent;
      parent.jcc.updateLine(parent);
    }

    public void mouseReleased(MouseEvent e) {
      super.mouseReleased(e);
    }
  }

  public void undo() {
    if (undoManager.canUndo()) undoManager.undo();
  }

  public void redo() {
    if (undoManager.canRedo()) undoManager.redo();
  }

    public void setShowSpaces(boolean showspaces) {
	this.showspaces = showspaces;
	char c = 0x25CF;
	if (showspaces) {
	    setText(getText().replace(' ', c));
	} else {
	    setText(getText().replace(c, ' '));
	}
    }
			
    public boolean showSpacesp() {
	return showspaces;
    }

}
