/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.BorderFactory;
import yoyo.*;
import java.awt.event.*;
import java.io.*;
import java.awt.*;

public class JShapeChooser extends JFrame
    implements ShapeChooser, StarLogoFrame, MenuListener {
    
    Application app;
    
    JShapeChooserPanel mainPanel;
    JShapeChooserStatusPanel statusPanel;
    
    static JShapeChooser chooser;
    
    SLMenu slmenu;
    
    public JShapeChooser(Application app) {
	super(app.textBundle.getString("windows_menu_shape_chooser")) ;
	this.app = app;
	setTitle(app.textBundle.getString("windows_menu_shape_chooser"));
	addNotify();
	
	Insets insets = getInsets();
	setSize(insets.left+insets.right+JShapeChooserPanel.COLUMNS*
		(TurtleShapes.MEDIUM+JShapeChooserPanel.LINEWIDTH)
		+JShapeChooserPanel.LINEWIDTH, 
		insets.top+JShapeChooserPanel.ROWS*(TurtleShapes.MEDIUM+
						    JShapeChooserPanel.LINEWIDTH)
		+1+25+insets.bottom+ ((YoYo.macintoshp) ? 0 : 23));
	
	if (!YoYo.macintoshp) {
	    javax.swing.ImageIcon icon = new javax.swing.ImageIcon(Pool.class.getResource("gifs/icon.gif"));
	    setIconImage(icon.getImage());
	}

	//getContentPane().setLayout(null);
	JPanel jpane = new JPanel();
	jpane.setLayout(null);
	getContentPane().add(jpane);
	
	//jpane.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
	statusPanel = new JShapeChooserStatusPanel(this);
	mainPanel = new JShapeChooserPanel(this, app);
	
	jpane.add(mainPanel);
	mainPanel.setLocation(0, 0);
	
	jpane.add(statusPanel);
	statusPanel.setLocation(0, 4*(16+1)+5);
		
	    slmenu = new JSLMenu(this, app);
	slmenu.disablePasteMenu();
	
	slmenu.setMenuBar(this);
	app.registerFrameTracker(this);
	addKeyListener(mainPanel);
	setResizable(false);
	setLocation(100, 100);
    }
    
    public static JShapeChooser create(Application app) {
	if (chooser != null) return chooser;
	chooser = new JShapeChooser(app);
	return chooser;
    }
    
    public void updateName(String name, int number) {
	statusPanel.updateString(name, number);
    }
    
    public void actionPerformed(ActionEvent e) { 
	String command = e.getActionCommand();
	if (command.equals("edit_menu_cut")) {
	    TurtleShapes.cutShape(mainPanel.selectedShape, app);
	    app.rancompilerp = false;
	    app.project.markChanged();
	    slmenu.enablePasteMenu();
	    mainPanel.refresh();
	} else if (command.equals(app.textBundle.getString("edit_menu_copy"))) {
	    TurtleShapes.copyShape(mainPanel.selectedShape, app);
	    slmenu.enablePasteMenu();
	} else if (command.equals(app.textBundle.getString("edit_menu_paste"))) {
	    TurtleShapes.pasteShape(mainPanel.selectedShape, app);
	    updateName(TurtleShapes.getShapeNames()[mainPanel.selectedShape], 
		       mainPanel.selectedShape);
	    app.rancompilerp = false;
	    app.rancompilerpForVariables = false;
	    app.project.markChanged();
	    mainPanel.refresh();
	} else if (command.equals(app.textBundle.getString("file_menu_restore_default_shapes"))){

	    ChoiceDialog cd;
	   
	    cd = new JRestoreShapesDialog(this, app.textBundle.getString("starlogo_dialog_title"), 
					  app.textBundle.getString("restore_default_shapes"),
					  app.textBundle.getString("yes_label"), app.textBundle.getString("cancel_label"));
	    


	    String choice = cd.getResult();
	    if (choice.equals(app.textBundle.getString("yes_label"))){
		TurtleShapes.loadDefaultShapes(app);
		//for(int i = 0; i < TurtleShapes.MAXSHAPES; i++) {
		//    updateName(TurtleShapes.getShapeNames()[i], i);
		//}
		updateName(TurtleShapes.getShapeNames()[mainPanel.selectedShape], 
			   mainPanel.selectedShape);
		app.rancompilerp = false;
		app.rancompilerpForVariables = false;
		app.project.markChanged();
		mainPanel.refresh();
	    }
     	} else if (command.equals(app.textBundle.getString("file_menu_import_shape"))){ 
	    importShape();
	    updateName(TurtleShapes.getShapeNames()[mainPanel.selectedShape], 
		       mainPanel.selectedShape);
	    app.rancompilerp = false;
	    app.rancompilerpForVariables = false;
	    app.project.markChanged();
	    mainPanel.refresh();
	} else if (command.equals(app.textBundle.getString("file_menu_export_shape"))){
	    exportShape();
	}
    }
    
    public void refresh() {
	mainPanel.refresh();
    }
    
    public void refreshShape(int num) {
	mainPanel.refreshShape(num);
    }
    
    public void itemStateChanged(ItemEvent e) { }
    
    public void setTurtleColor(Color c) {
	mainPanel.tcolmask = c.getRGB();
	mainPanel.refresh();
    }
    
    public void setPatchColor(Color c) {
	mainPanel.pcolmask = c.getRGB();
	mainPanel.refresh();
    }
    
    public int selectedShape() {
	return mainPanel.selectedShape;
    }
    
    public void markChanged() {    }
    public void clearChanged() {    }

    public boolean exportShape() {
	FileDialog d = new FileDialog((Frame)app.activeFrame, 
				      app.textBundle.getString("file_menu_export_shape"), FileDialog.SAVE);
	
	if (app.project.curDir!=null) {
	    d.setDirectory(app.project.curDir.getPath());
	}
	d.setFilenameFilter(app.shapeFileFilter);
	d.show();
	SLURL curDir = new SLURL(SLURL.FILE, d.getDirectory());
	if (d.getFile()!=null) {
	    SLURL filename = new SLURL(SLURL.FILE, curDir.getPath(), d.getFile());
	    try {
		app.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		if (!(YoYo.macintoshp && YoYo.macosxp) && 
		    !filename.getFileExtension().toLowerCase().equals("sls")) {
		    filename.addFileExtension("sls");
		}

		String shapechars = TurtleShapes.saveShapes(selectedShape());

		File file = new File(filename.getFullPath());
		BufferedWriter out = new BufferedWriter(new FileWriter(file));
		out.write(shapechars);
		out.close();
	    } catch (FileNotFoundException fnfe) {
		Application.createMessageDialog(app, app.activeFrame, app.textBundle.getString("starlogo_dialog_title"),
						app.textBundle.getString("error_saving_shape") + ": " + 
						fnfe.getMessage(), false);
		return false;
	    } catch (IOException ioe) {
		Application.createMessageDialog(app, app.activeFrame, app.textBundle.getString("starlogo_dialog_title"),
						app.textBundle.getString("error_saving_shape") + ": " +
						ioe.getMessage(), false);
		return false;
	    } finally {
		app.restoreCursor();
	    }
	    return true;
	} else {
	    return false;
	}
	
    }
    
    public boolean importShape() {
	FileDialog d = new FileDialog((Frame)app.activeFrame, 
				      app.textBundle.getString("file_menu_import_shape"), FileDialog.LOAD);
	
	if (app.project.curDir != null) {
	    d.setDirectory(app.project.curDir.getPath());
	}
	d.setFilenameFilter(app.shapeFileFilter);
	d.show();
	SLURL curDir = new SLURL(SLURL.FILE, d.getDirectory());
	if (d.getFile()!=null) {
	    SLURL filename = new SLURL(SLURL.FILE, curDir.getPath(), d.getFile());
	    try {
		app.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		String shapestring = (String)PrimFile.prim_filetostring(filename.toString(), app.sl.c);
		TurtleShapes.loadShapes(shapestring, selectedShape(), false, app);
	    } catch (LogoError fnfe) {
		Application.createMessageDialog(app, app.activeFrame, app.textBundle.getString("starlogo_dialog_title"),
						app.textBundle.getString("error_loading_shape") + ": " + 
						fnfe.getMessage(), false);
		return false;
	    } finally {
		app.restoreCursor();
	    }
	    return true;
	} else {
	    return false;
	}
    }
}


