/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import yoyo.*;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.SystemColor;
import javax.swing.JPanel;
import java.awt.image.ImageProducer;
import java.awt.Image;
import java.awt.image.ImageConsumer;
import java.awt.Graphics;
import javax.swing.JComponent;
import java.awt.event.*;

public class JShapeChooserPanel extends JComponent implements ImageProducer, KeyListener {
    JShapeChooser chooser;
    int selectedShape;
    String selectedname;

    final static int LINEWIDTH = 2;
    final static int ROWS = 4;
    final static int COLUMNS = 16;

    int tcolmask = Colors.colorarrayints[55 * 32];
    int pcolmask = Colors.colorarrayints[ 0 * 32];

	boolean editorOpen = false;

    Image image;
    ImageConsumer ic;
    int[] pixels;
    int width = (TurtleShapes.MEDIUM + LINEWIDTH) * COLUMNS  + LINEWIDTH;
    int height = (TurtleShapes.MEDIUM + LINEWIDTH) * ROWS + LINEWIDTH;
    int rowbytes = width;

    Application app;

    JShapeChooserPanel(JShapeChooser chooser, Application app) {
	this.chooser = chooser;
	this.app = app;
	setSize(width, height);

	setOpaque(true);
	pixels = new int[width * height];
	image = chooser.createImage(this);

	refresh();

	int gray = Colors.colorarrayints[5 * 32];
	// draw 5 gray rows
	for(int i = 0; i < pixels.length; i += (width * (TurtleShapes.MEDIUM + LINEWIDTH))) {
	    for (int j = 0, l = 0; j < LINEWIDTH; j++, l += width) {
		for (int k = 0; k < width; k++) {
		    pixels[i + l + k] = gray;
		}
	    }
	}
	// draw 17 gray columns
	for(int i = 0; i < width; i+= (TurtleShapes.MEDIUM + LINEWIDTH)) {
	    for (int j = 0; j < pixels.length; j+= width) {
		for (int k = 0; k < LINEWIDTH; k++) {
		    pixels[i + j + k] = gray;
		}
	    }
	}

	selectedShape = 0;
	select(selectedShape);



	addMouseListener(new MouseAdapter() {
		public void mousePressed(MouseEvent e) {
		    int selection = eventToShape(e);
		    if (selection < 0 || selection >= 64) return; // mouse event out of bounds
		    select(selection);
		    if (e.getClickCount() > 1) openEditor(selection);
		}
		public void mouseReleased(MouseEvent e) {
		    mousePressed(e);
		}
	    });
	addMouseMotionListener(new MouseMotionAdapter() {
		public void mouseDragged(MouseEvent e) {
		    int selection = eventToShape(e);
		    if (selection < 0 || selection >= 64) return; // mouse event out of bounds
		    select(selection);
		    if (e.getClickCount() > 1) openEditor(selection);
		}
	    });

	addKeyListener(this);
    }

    public void refresh() {
	for (int i = 0; i < TurtleShapes.MAXSHAPES; i++) {
	    drawShape(i);
	}
	if (ic != null) startProduction(ic);
    }

    public void refreshShape(int shape) {
	drawShape(shape);
	if (ic != null) startProduction(ic);
    }

    public int[] drawShape(int shape) {
	int col = shape % COLUMNS;
	int row = shape / COLUMNS;
	int startx = col * (TurtleShapes.MEDIUM + LINEWIDTH) + LINEWIDTH;
	int starty = row * (TurtleShapes.MEDIUM + LINEWIDTH) + LINEWIDTH;
	int xypix = startx + (starty * rowbytes);

	int[] imagePixels = new int[TurtleShapes.MEDIUM * TurtleShapes.MEDIUM];
	int y = 0;

	//System.out.println("startx: " + startx + " starty: " + starty + " xypix: " + xypix + " end: " + (xypix + 256)); 
	TurtleShapes ts = TurtleShapes.getShape(shape);

	double[] colors = (ts.getColorsForSize(TurtleShapes.MEDIUM))[0];
	int[]    masks  = (ts.getMasksForSize(TurtleShapes.MEDIUM))[0];
	int      color  = 0;
	for (int i=0, z=0; i<TurtleShapes.MEDIUM; i++) {
	    for (int j=0; j<TurtleShapes.MEDIUM; j++) {
		if (masks[2*z] == TurtleShapes.ON) {
		    color = tcolmask;
		} else if (masks[2*z+1] == TurtleShapes.ON) {
		    color = pcolmask;
		} else {
		    color = Colors.colorarrayints[(int)(colors[z] * 32.0)];
		}
		pixels[xypix + j] = color;
		imagePixels[y++] = color;
		z++;
	    }
	    xypix += rowbytes;
	}

	return imagePixels;
    }

    public void drawSquare(int shape, boolean selectedp) {
    	int col = shape % COLUMNS;
    	int row = shape / COLUMNS;
    	int startx = col * (TurtleShapes.MEDIUM + LINEWIDTH);
    	int starty = row * (TurtleShapes.MEDIUM + LINEWIDTH);
    	int xypix = startx + (starty * rowbytes);
    	int color = (selectedp) ? Colors.colorarrayints[105 * 32] : Colors.colorarrayints[5 * 32];
    	// horizontal lines
    	for(int i = 0; i < (TurtleShapes.MEDIUM + LINEWIDTH + 1); i++) {
    	    for (int k = 0; k < LINEWIDTH; k++) {
		pixels[xypix + i + (rowbytes * k)] = color;
		pixels[xypix + (rowbytes * (TurtleShapes.MEDIUM + LINEWIDTH + k)) + i] = color;
    	    }
    	}
    			
    	// vertical lines
    	for(int i = 0; i < (TurtleShapes.MEDIUM + LINEWIDTH + 2); i++) {
    	    for (int k = 0; k < LINEWIDTH; k++) {
		pixels[xypix + i * rowbytes + k] = color;
		pixels[xypix + (i * rowbytes) + (TurtleShapes.MEDIUM + LINEWIDTH) + k] = color;
    	    }
    	}
    	if (ic != null) startProduction(ic);
    }
	
    public void paintComponent(Graphics g) {
	g.drawImage(image, 0, 0, this);
    }
  
    void select(int newSelection) {
	drawSquare(selectedShape, false);
	selectedShape = newSelection;
	drawSquare(selectedShape, true);
	selectedname = TurtleShapes.getShape(newSelection).name;
	if (selectedname == null) selectedname =  (app.textBundle.getString("shape_label")  + newSelection);
	chooser.updateName(selectedname, newSelection);
	repaint();
    }

    
    public void openEditor(int shape) {
    	if (editorOpen) return;
    	editorOpen = true;
	int size = app.sl.cellsize;
	if (size < TurtleShapes.SMALL) size = TurtleShapes.SMALL;
	JShapeEditor jse = new JShapeEditor(TurtleShapes.getShape(shape), shape, selectedname, size, chooser, app);
	// pause starlogo
	synchronized (app.sl.runningp) {
	    app.sl.locked = true;
	}
	jse.init();
	// unpause starlogo
	app.iface.maybeReleasePatchCanvas();
    }
    
    int eventToShape(MouseEvent e) {
	int x = Math.max(e.getX() - LINEWIDTH, 0) /
	    (TurtleShapes.MEDIUM + LINEWIDTH);
	int y = Math.max(e.getY() - LINEWIDTH, 0) /
	    (TurtleShapes.MEDIUM + LINEWIDTH);
	return y*16+x;
    }

	

    public void addConsumer(ImageConsumer ic) {
	startProduction(ic);
	//System.out.println("called add Consumer with " + ic);
    }

    public boolean isConsumer(ImageConsumer ic) {
	return (this.ic == ic); 
    }

    public void removeConsumer(ImageConsumer ic) {
	//System.out.println("called remove consumer wiht " + ic);
	this.ic = null; 
    }

    public void requestTopDownLeftRightResend(ImageConsumer ic) {
	ic.setHints(ImageConsumer.TOPDOWNLEFTRIGHT);
	ic.setPixels(0, 0, width, height,
		     Colors.cm, pixels, 0, rowbytes);	// As many times as needed
	ic.imageComplete(ImageConsumer.SINGLEFRAMEDONE);
	ic.setHints(ImageConsumer.RANDOMPIXELORDER);
    }

    public void startProduction(ImageConsumer ic) {
	if (ic == null) return;
	//System.out.println("called start production with " + ic);
	if (this.ic == null) {
	    this.ic = ic; 
	    ic.setColorModel(Colors.cm);
	    ic.setDimensions(width, height);
	}
	ic.setHints(ImageConsumer.TOPDOWNLEFTRIGHT);
	ic.setPixels(0, 0, width, height,
		     Colors.cm, pixels, 0, rowbytes);	// As many times as needed
	ic.imageComplete(ImageConsumer.SINGLEFRAMEDONE);
	ic.setHints(ImageConsumer.RANDOMPIXELORDER);
    }
	

		public void keyPressed(KeyEvent e) {
		    if (e.getKeyCode() == KeyEvent.VK_DOWN) {
			select((selectedShape+COLUMNS)%(ROWS * COLUMNS));
		    } else if (e.getKeyCode() == KeyEvent.VK_UP) {
			int num = (selectedShape+(ROWS - 1) * COLUMNS)%(ROWS * COLUMNS);
			select(num);
		    } else if (e.getKeyCode() == KeyEvent.VK_RIGHT) {
			int num = (selectedShape + 1) % (ROWS * COLUMNS);
			select(num);
		    } else if (e.getKeyCode() == KeyEvent.VK_LEFT) {
			int num = (selectedShape + (ROWS * COLUMNS - 1)) % (ROWS * COLUMNS);
			select(num);
		    }
		}
    
		public void keyReleased(KeyEvent e) {
		    if (e.getKeyCode() == KeyEvent.VK_ENTER) {
			openEditor(selectedShape);
		    }
		}

    public void keyTyped(KeyEvent e) {}
}
