/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.awt.*;
import java.awt.event.*;
import javax.swing.JPanel;

public class JShapeColorTool extends JPanel {

    JShapePaintSelect paintselect;

    static int cellHeight1 = 15;
    static int cellHeight2 = 21;
    static int cellWidth = 24;

    public int colorDown = 0;
    public int shadeDown = 5;
    public int curcolor = 5;

    JShapeEditor parent;

    public JShapeColorTool(JShapeEditor parent) {
	this.parent = parent;
	addMouseListener(new MouseAdapter() {
		public void mouseClicked(MouseEvent e) {
		    pointToButton(e.getX(), e.getY());
		}
		public void mousePressed(MouseEvent e) {
		    pointToButton(e.getX(), e.getY());
		}
	    });
	addMouseMotionListener(new MouseMotionAdapter() {
		public void mouseDragged(MouseEvent e) {
		    pointToButton(e.getX(), e.getY());
		}
	    });
    }

    public void paintComponent(Graphics g) {
	g.setColor(SystemColor.control);
	g.fillRect(0, 0, getSize().width, getSize().height);
	Color oldcolor = g.getColor();
	int rows = Colors.numcolors / 10;
	for (int i = 0; i < rows; i++) {
	    int color = i*10+shadeDown;
	    g.setColor(Colors.colorarray[color * 32]);
	    g.fillRect(0, i*cellHeight1, cellWidth, cellHeight1);
	}

	Color selectedColor = Colors.colorarray[curcolor * 32];
	int blueComponent = 255 - selectedColor.getBlue();
	int greenComponent = 255 - selectedColor.getGreen();
	int redComponent = 255 - selectedColor.getRed();
	Color highlightColor = new Color(redComponent, greenComponent, blueComponent);
	g.setColor(highlightColor);
	g.drawRect(0, colorDown*cellHeight1, cellWidth-1, cellHeight1-1);
	for (int i = 0; i < 10; i++) {
	    int color = colorDown*10+i;
	    g.setColor(Colors.colorarray[color * 32]);
	    g.fillRect(cellWidth, i*cellHeight2, cellWidth, cellHeight2);
	}
	g.setColor(highlightColor);
	g.drawRect(cellWidth, shadeDown*cellHeight2, cellWidth-1, cellHeight2-1);
	g.setColor(oldcolor);
    }

    public void pointToButton(int x, int y) {
	int column = (int)java.lang.Math.floor(x / cellWidth);
	if (column < 0) {
	    column = 0;
	} else if (column > 1) {
	    column = 1;
	}
	if (column == 0) {
	    int row = (int)java.lang.Math.floor(y / cellHeight1);
	    if (row < 0) {
		row = 0;
	    } else if (row > 13) {
		row = 13;
	    }
	    setColor(row, shadeDown);
	} else {
	    int row = (int)java.lang.Math.floor(y / cellHeight2);
	    if (row < 0) {
		row = 0;
	    } else if (row > 9) {
		row = 9;
	    }
	    setColor(colorDown, row);
	}
    }

    public void setColor(int wholeColor) {
	if ((wholeColor < 0) || (wholeColor >= Colors.numcolors)) {
	    return;
	}
	setColor((wholeColor / 10), (wholeColor % 10));
    }

    public void setColor(int newColor, int newShade) {
	paintselect.deselect();
	if (newColor == colorDown && newShade == shadeDown) return;
	this.colorDown = newColor;
	this.shadeDown = newShade;
	curcolor = colorDown*10+shadeDown;
	assert curcolor >= 0 && curcolor < Colors.numcolors : "Color out of range";
	repaint();
    }

    public Color getColor() {
	return Colors.colorarray[curcolor*32];
    }

    public double getColorDouble() {
	return curcolor;
    }

    public int getColorInt() {
	return Colors.colorarrayints[curcolor*32];
    }

}
