/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import yoyo.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import info.clearthought.layout.TableLayout;

public class JShapeEditor extends JSLDialog {
    
    int shapeNum;

    TurtleShapes editedShape;

    JTextField nameField;
    JShapeEditorPanel editor;
    JShapeEditorRotatePanel rotateToolbar;
    JShapeEditorSizeChooserPanel sizechooser;
    //paint panels
    JShapePaintTool painttool;
    JShapePaintSelect paintselect;
    JShapeColorTool colortool;

    JButton bok, bca, revertButton;
    JCheckBox rotatable;
    JShapeRotationPreviewButton shapeRotationPreview;

    //need to pass actual heading into TurtleShapes.getShape
    //following the TurtleShapes.getShape convention,
    // the integer 0-15 is the heading index
    //although i don't think we actually need it.
    //int heading = 0;
    int headingindex = 0;
    double heading = 0.0;
    int size;
    JShapeChooser jsc;
    Application app;

    public JShapeEditor(TurtleShapes t, int id, String name, int size, JShapeChooser jsch, Application app) {
	super(jsch, app.textBundle.getString("shape_number_label") + 
	      id + ": " + name, true);
	this.jsc = jsch;
	this.app = app;
	this.size = size;
	shapeNum = id;
	
	Container cp = getContentPane();
	double[][] sizes = {
	    { 50.0, 4.0, 40.0, 40.0, 25.0, 20.0, 80.0, 4.0, 46.0, 34.0, 7.0 },
	    { 32.0, 2.0, 4.0, 121.0, 35.0, 133.0, 4.0, 20.0, 4.0, 20.0, 4.0, (YoYo.macosxp) ? TableLayout.PREFERRED : 25.0, 7.0 }};
	LayoutManager layout = new TableLayout(sizes);
	cp.setLayout(layout);
	
	nameField = new JTextField(name);
	cp.add(nameField, "5, 7, 9, 7");
	initFocus(nameField);
	
// 	sizechooser = new JShapeEditorSizeChooserPanel(this);
// 	cp.add(sizechooser, "3, 0, 8, 2");
	
	
	revertButton = new JButton(app.textBundle.getString("revert"));
	//revertButton.addActionListener(this);
	revertButton.setVisible(false);
	cp.add(revertButton, "2, 9, 4, 9");
	
	rotatable = new JCheckBox(app.textBundle.getString("rotatable"));
	rotatable.setSelected(t.rotatep);
	if (!YoYo.macintoshp) rotatable.setMnemonic('r');
	cp.add(rotatable, "2, 11, 4, 11");
	
	
	JLabel label3 = new JLabel(app.textBundle.getString("shape_name"));
	cp.add(label3, "2, 7, 4, 7");
	
	rotateToolbar = new JShapeEditorRotatePanel(app, this);
	cp.add(rotateToolbar, "0, 0, 2, 0");
	
	painttool = new JShapePaintTool();
	cp.add(painttool, "0, 3, f, f");
	
	colortool = new JShapeColorTool(this);
	cp.add(colortool, "0, 5, 0, 11");
	
	paintselect = new JShapePaintSelect(colortool);
	colortool.paintselect = paintselect;
	cp.add(paintselect, "0, 4, f, f");
	
	editedShape = t.copy();
	sizechooser = new JShapeEditorSizeChooserPanel(this);
	cp.add(sizechooser, "3, 0, 8, 2");

	JPanel iconPanel = new JPanel();
	iconPanel.setLayout(null);
	shapeRotationPreview = new JShapeRotationPreviewButton(shapeNum, size, this);
	shapeRotationPreview.setToolTipText(app.textBundle.getString("preview_rotations"));
	iconPanel.add(shapeRotationPreview);
	cp.add(iconPanel, "9, 0, 9, 1");
	
	editor = new JShapeEditorPanel(this);
	editor.paintselect = paintselect;
	editor.painttool = painttool;
	cp.add(editor, "2, 3, 9, 5");

	bca = new JButton(app.textBundle.getString("cancel_label"));
	//bca.addActionListener(this);
	
	bok = new JButton(app.textBundle.getString("ok_label"));	
	//bok.addActionListener(this);
	initOKCancel(bok, bca);

	
	if (YoYo.macintoshp) {
	    cp.add(bok, "8, 11, 9, 11");
	    cp.add(bca, "6, 11, f, f");
	} else {
	    cp.add(bca, "8, 11, 9, 11");
	    cp.add(bok, "6, 11, f, f");
	}
	
	addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent e) {
		    jsc.mainPanel.refreshShape(shapeNum);
		    cancel();
		}
		});
	
	switchToSize(size);

	initShortcuts();
	pack();
	Dimension d = getSize();
	setSize(d.width + 7, d.height + 7);
	initLocation();
    }


    public void init() {
	setVisible(true);
    }

    public void switchToSize(int size) {
  	this.size = size;
  	editor.changeSize(size);
    }
    
	public void ok() {
	    String shapeName = nameField.getText().replace(' ', '-');
	    editedShape.rotatep = rotatable.isSelected();
	    editedShape.name = shapeName;
	    editedShape.id = shapeNum;
	    editedShape.reRotateShapes();
	    TurtleShapes.shapes[shapeNum] = editedShape;
	    jsc.updateName(shapeName, shapeNum);
	    jsc.refreshShape(shapeNum);
	    jsc.app.rancompilerp = false;
	    jsc.app.rancompilerpForVariables = false;
	    jsc.app.project.markChanged();
	    jsc.app.sl.redraw();
	    jsc.mainPanel.editorOpen = false;
		super.ok();
	}
	
	public void cancel() {
		jsc.mainPanel.editorOpen = false;
		super.cancel();
	}
	

}

class JShapeRotationPreviewButton extends JButton {
    
    ShapeIcon rotator;
    
    JShapeRotationPreviewButton(int shapeNum, int size, JShapeEditor editor) {
	this.rotator = new ShapeIcon(shapeNum, size, editor);
	rotator.setRotatable(true);
	setSize(size, size);
	setIcon(rotator);
	setLocation(8, 8);
	addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    rotator.toggleRotation();
		}
	    });
    }

    public void updateDisplay() {
	rotator.refresh();
    }

    public void updateSize(int newSize) {
	setSize(newSize, newSize);
	switch (newSize) {
	case TurtleShapes.SMALL:
	    setLocation(12, 15);
	    break;
	case TurtleShapes.MEDIUM:
	    setLocation(6, 11);
	    break;
	case TurtleShapes.LARGE:
	    setLocation(0, 3);
	}
	rotator.updateSize(newSize);
    }
    
}



