/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.awt.*;
import java.awt.image.ImageConsumer;
import java.awt.image.ImageProducer;
import java.awt.image.PixelGrabber;
import java.awt.image.MemoryImageSource;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.net.MalformedURLException;
import java.util.*;
import javax.swing.JPanel;

public class JShapeEditorPanel extends JPanel 
    implements ImageProducer, MouseListener, MouseMotionListener {
    
    public JShapePaintTool painttool;
    public JShapePaintSelect paintselect;

    static int GRAY = 0;
    static int WHITE = 0;
    static final int[] patchTrans8x8   = new int[8 * 8];
    static final int[] patchTrans16x16 = new int[17 * 17];
    static final int[] patchTrans32x32 = new int[35 * 35];
    static Image patchTrans16x16AsImage = null;

    static { 
	try {
	    GRAY =  Colors.colorarrayints[5 * 32];
	    WHITE = Colors.colorarrayints[9 * 32];
	    int color;
	    color = WHITE;
	    for (int i=0; i<8; i++) {
		for (int j=0; j<8; j++) {
		    patchTrans8x8[i*8+j] = color;
		    color = (color == WHITE) ? GRAY : WHITE;				
		}
		color = (color == WHITE) ? GRAY : WHITE;
	    }
	    color = WHITE;
	    for (int i=0; i<17; i++) {
		for (int j=0; j<17; j++) {
		    patchTrans16x16[i*17+j] = color;
		    color = (color == WHITE) ? GRAY : WHITE;
		}
	    }
	    color = WHITE;
	    for (int i=0; i<35; i++) {
		for (int j=0; j<35; j++) {
		    patchTrans32x32[i*35+j] = color;
		    color = (color == WHITE) ? GRAY : WHITE;
		}
	    }
	    MemoryImageSource source = new MemoryImageSource(17, 17, patchTrans16x16, 0, 17);
	    patchTrans16x16AsImage = Toolkit.getDefaultToolkit().createImage(source);
 
	} catch (Exception e) { e.printStackTrace(); }
    }
       
    static Image e8x8Image, e17x17Image, e35x35Image;
    static int[] turtleTrans8x8   = new int[8 * 8];
    static int[] turtleTrans16x16 = new int[17 * 17];
    static int[] turtleTrans32x32 = new int[35 * 35];
    
    double[] shapecolors;
    int[]    shapemasks;
    double[] shapecolors_backup;
    int[]    shapemasks_backup;
    
    JShapeEditor parent;
    int[] panelpixels;
    int[] panelpixels_backup;
    int gridlength;
    int width;
    int height;
    int size = 0;
    Image image;
    ImageConsumer ic;
    
    boolean startOfPoly = true;
    boolean drawingPoly = false;
    int polyHeadx;
    int polyHeady;
    
    private int mousedownx;
    private int mousedowny;
    private int mousedragx;
    private int mousedragy;
    
    public JShapeEditorPanel(JShapeEditor parent) {
	this.parent = parent;
	size = parent.size;
	setOpaque(true);
	init(true);
	image = parent.createImage(this);
	drawGrid();
	refresh();
	addMouseListener(this);
	addMouseMotionListener(this);
	setPreferredSize(new Dimension(9*32+1, 9*32+1));
    }
    
    public void init(boolean saveBackup) {
	this.shapecolors = parent.editedShape.getColorsForSize(size)[0];
	this.shapemasks = parent.editedShape.getMasksForSize(size)[0];

	if (saveBackup) {
	    this.shapecolors_backup = new double[shapecolors.length];
	    System.arraycopy(shapecolors, 0, shapecolors_backup, 0, shapecolors.length);
	    this.shapemasks_backup = new int[shapemasks.length];
	    System.arraycopy(shapemasks, 0, shapemasks_backup, 0, shapemasks.length);
	}

	this.size = size;
	
	switch(size) {
	case TurtleShapes.SMALL:
	    gridlength = 36;
	    break;
	case TurtleShapes.MEDIUM:
	    gridlength = 18;
	    break;
	case TurtleShapes.LARGE:
	    gridlength = 9;
	    break;
	}
	width = 289;
	height = 289;
	panelpixels = new int[width * height];
	if (saveBackup) {
	    panelpixels_backup = new int[width * height];
	}
	setSize(width, height);
    }
        
    public void updateDisplay() {
	parent.sizechooser.updateAll();
	parent.shapeRotationPreview.updateDisplay();
	startProduction(ic);
	parent.repaint();
    }

    public void lightweightUpdateDisplay() {
	refresh();
	startProduction(ic);
    }

    public void changeSize(int updatedSize) {
	parent.sizechooser.updateAll();
	boolean sizeChange = false;
	if (size != updatedSize) sizeChange = true;
	this.size = updatedSize;
	init(sizeChange);
	if (sizeChange) {
	    boolean startOfPoly = true;
	    boolean drawingPoly = false;
	}
	removeConsumer(null);
	image = parent.createImage(this);
	drawGrid();
	refresh();
	parent.repaint();
    }
    
    public void refresh() {
	for (int i=0; i<size; i++) {
	    for (int j=0; j<size; j++) {
		refreshSquare(i, j);
	    }
	}		
    }
    
    public void drawGrid() {
	for (int x=0; x<width; x+=gridlength) {
	    for (int y=0; y<height; y++) {
		panelpixels[y*width+x] = GRAY; // vertical lines
	    }
	}
	for (int y=0; y<height; y+=gridlength) {
	    for (int x=0; x<width; x++) {
		panelpixels[y*width+x] = GRAY; // horizontal lines
	    }
	}
    }
    
    private boolean isColoredSame(Point a, Point b) {
	int a_offset = a.y*size + a.x;
	int b_offset = b.y*size + b.x;
	return ((shapecolors[a_offset]    == shapecolors[b_offset]) &&
		(shapemasks[2*a_offset]   == shapemasks[2*b_offset]) &&
		(shapemasks[2*a_offset+1] == shapemasks[2*b_offset+1]));
    }
    
    public void colorSquare(int x, int y, int color) {
	int startx = x * gridlength + 1;
	int starty = y * gridlength + 1;
	int xypix = starty * width + startx;
	
	for (int i=0; i<gridlength-1; i++) {
	    for (int j=0; j<gridlength-1; j++) {
		panelpixels[xypix+j*width+i] = color;
	    }
	}
    }
    
    public void colorSquareTurtleTransparent(int x, int y) {
	int startx = x * gridlength + 1;
	int starty = y * gridlength + 1;
	int xypix = starty * width + startx;
	
	int[] turtleZeroPixels = null;
	switch(size) {
	case TurtleShapes.SMALL:
	    turtleZeroPixels = turtleTrans32x32;
	    break;
	case TurtleShapes.MEDIUM:
	    turtleZeroPixels = turtleTrans16x16;
	    break;
	case TurtleShapes.LARGE:
	    turtleZeroPixels = turtleTrans8x8;
	    break;
	}
	for (int i=0, z=0; i<gridlength-1; i++) {
	    for (int j=0; j<gridlength-1; j++) {
		panelpixels[xypix+i*width+j] = turtleZeroPixels[z++];
	    }
	}
    }
    
    public void colorSquarePatchTransparent(int x, int y) {
	int startx = x * gridlength + 1;
	int starty = y * gridlength + 1;
	int xypix = starty * width + startx;
	int z = 0;
	int[] colors = null;
	switch(size) {
	case TurtleShapes.SMALL:
	    colors = patchTrans32x32;
	    break;
	case TurtleShapes.MEDIUM:
	    colors = patchTrans16x16;
	    break;
	case TurtleShapes.LARGE:
	    colors = patchTrans8x8;
	    break;
	}
	for (int i=0; i<gridlength-1; i++) {
	    for (int j=0; j<gridlength-1; j++) {
		panelpixels[xypix+j*width+i] = colors[z++];
	    }
	}
    }
    
    private void refreshSquare(int x, int y) {
	int index = (y*size+x);
	if (shapemasks[2*index] == TurtleShapes.ON) {
	    colorSquareTurtleTransparent(x, y);
	    return;
	}
	
	if (shapemasks[2*index+1] == TurtleShapes.ON) {
	    colorSquarePatchTransparent(x, y);
	    return;
	}
	colorSquare(x, y, Colors.colorarrayints[(int)(32.0*shapecolors[index])]);
    }
    
    public void paintComponent(Graphics g) {
	super.paintComponent(g);
	g.drawImage(image, 0, 0, this);
    }
    
    public void addConsumer(ImageConsumer ic) {
	startProduction(ic);
    }
    
    public boolean isConsumer(ImageConsumer ic) {
	return (this.ic == ic);
    }
    
    public void removeConsumer(ImageConsumer ic) {
	this.ic = null;
    }
    
    public void requestTopDownLeftRightResend(ImageConsumer ic) {
	ic.setHints(ImageConsumer.TOPDOWNLEFTRIGHT);
	ic.setPixels(0, 0, width, height,
		     Colors.cm, panelpixels, 0, width);	// As many times as needed
	ic.imageComplete(ImageConsumer.SINGLEFRAMEDONE);
	ic.setHints(ImageConsumer.RANDOMPIXELORDER);
    }
    
    private int eventToPixelX(MouseEvent e) {
	return Math.min(size-1, e.getX()/gridlength);
    }

    private int eventToPixelY(MouseEvent e) {
	return Math.min(size-1, e.getY()/gridlength);
    }
    
    private void paintPixel(int x, int y) {
	int offset = y*size + x;
	if (offset >= size * size)
	    {
		//System.out.println("(" + pix.x + ", " + pix.y + ") is out of bounds");
	    }
	else if (paintselect.patch) {
	    shapecolors[offset]    = TurtleShapes.OFF;
	    shapemasks[2*offset]   = TurtleShapes.OFF;
	    shapemasks[2*offset+1] = TurtleShapes.ON;
	    colorSquarePatchTransparent(x, y);
	} else if (paintselect.turtle) {
	    shapecolors[offset]    = TurtleShapes.OFF;
	    shapemasks[2*offset]   = TurtleShapes.ON;
	    shapemasks[2*offset+1] = TurtleShapes.OFF;
	    colorSquareTurtleTransparent(x, y);
	} else {
	    shapecolors[offset]    = parent.colortool.getColorDouble();
	    shapemasks[2*offset]   = TurtleShapes.OFF;
	    shapemasks[2*offset+1] = TurtleShapes.OFF;
	    colorSquare(x, y, parent.colortool.getColorInt());
	}
    }
    
    private void clearPixel(int x, int y) {
	int offset = y*size+ x;
	shapecolors[offset]    = TurtleShapes.OFF;
	shapemasks[2*offset]   = TurtleShapes.OFF;
	shapemasks[2*offset+1] = TurtleShapes.ON;
	colorSquarePatchTransparent(x, y);
    }
    
    private void yoinkPixel(int x, int y) {
	int offset = y*size + x;
	if (shapemasks[2*offset] == TurtleShapes.ON) {
	    paintselect.mouseClicked(new MouseEvent(paintselect, -1, 0, 0, 28, 15, 0, false));
	} else if (shapemasks[2*offset+1] == TurtleShapes.ON) {
	    paintselect.mouseClicked(new MouseEvent(paintselect, -1, 0, 0, 5, 15, 0, false));
	} else {
	    paintselect.deselect();
	    parent.colortool.setColor((int)shapecolors[offset]);
	    //parent.colortool.setColor(Colors.colorarrayints[(int) (32.0 * shapecolors[offset])]);
	}
    }
    
    public void startProduction(ImageConsumer ic) {
	if (ic == null) return;
	if (this.ic == null) {
	    this.ic = ic;
	    ic.setColorModel(Colors.cm);
	    ic.setDimensions(width, height);
	}
	ic.setHints(ImageConsumer.TOPDOWNLEFTRIGHT);
	ic.setPixels(0, 0, width, height,
		     Colors.cm, panelpixels, 0, width);	// As many times as needed
	ic.imageComplete(ImageConsumer.SINGLEFRAMEDONE);
	ic.setHints(ImageConsumer.RANDOMPIXELORDER);
    }
    
    public void mouseClicked(MouseEvent e) {
    }
    
    public void mouseDragged(MouseEvent e) {
	switch (painttool.buttonDown) {
	case JShapePaintTool.NONE: 
	    return;
	case JShapePaintTool.PEN:
	    paintPixel(eventToPixelX(e), eventToPixelY(e));
	    break;
	case JShapePaintTool.ERASER:
	    clearPixel(eventToPixelX(e), eventToPixelY(e));
	    break;
	case JShapePaintTool.EYEDROPPER:
	    yoinkPixel(eventToPixelX(e), eventToPixelY(e));
	    return;
	case JShapePaintTool.LINE:
	    lineMouseDragged(eventToPixelX(e), eventToPixelY(e));
	    break;
	case JShapePaintTool.REC:
	    rectangleMouseDragged(eventToPixelX(e), eventToPixelY(e));
	    break;
	case JShapePaintTool.FILLREC:
	    fillRectangleMouseDragged(eventToPixelX(e), eventToPixelY(e));
	    break;
	case JShapePaintTool.OVAL:
	    ovalMouseDragged(eventToPixelX(e), eventToPixelY(e));
	    break;
	case JShapePaintTool.FILLOVAL:
	    fillOvalMouseDragged(eventToPixelX(e), eventToPixelY(e));
	    break;
	default:
	    break;
	}
	updateDisplay();
    }
    
    public void mouseMoved(MouseEvent e) {
	if (painttool.buttonDown == JShapePaintTool.POLY) {
	    polygonMouseMoved(eventToPixelX(e), eventToPixelY(e));
	}
    }
    
    public void mousePressed(MouseEvent e) {
	switch (painttool.buttonDown) {
	case JShapePaintTool.NONE:
	    return;
	case JShapePaintTool.PEN:
	    paintPixel(eventToPixelX(e), eventToPixelY(e));
	    break;
	case JShapePaintTool.ERASER:
	    clearPixel(eventToPixelX(e), eventToPixelY(e));
	    break;
	case JShapePaintTool.EYEDROPPER:
	    yoinkPixel(eventToPixelX(e), eventToPixelY(e));
	    return;
	case JShapePaintTool.REC:
	case JShapePaintTool.LINE:
	case JShapePaintTool.FILLREC:
	case JShapePaintTool.OVAL:
	case JShapePaintTool.FILLOVAL:
	    shapeMousePressed(eventToPixelX(e), eventToPixelY(e));
	    break;
	case JShapePaintTool.POLY:
	    polygonMousePressed(eventToPixelX(e), eventToPixelY(e));
	    break;
	case JShapePaintTool.FILLSHAPE:
	    fillArea(new Point(eventToPixelX(e), eventToPixelY(e)));
	    break;
	}
	updateDisplay();
    }
    
    public void mouseReleased(MouseEvent e) {
	switch (painttool.buttonDown) {
	case JShapePaintTool.LINE:
	    lineMouseReleased(eventToPixelX(e), eventToPixelY(e));
	    break;
	case JShapePaintTool.REC:
	    rectangleMouseReleased(eventToPixelX(e), eventToPixelY(e));
	    break;
	case JShapePaintTool.FILLREC:
	    fillRectangleMouseReleased(eventToPixelX(e), eventToPixelY(e));
	    break;
	case JShapePaintTool.OVAL:
	    ovalMouseReleased(eventToPixelX(e), eventToPixelY(e));
	    break;
	case JShapePaintTool.FILLOVAL:
	    fillOvalMouseReleased(eventToPixelX(e), eventToPixelY(e));
	    break;
	case JShapePaintTool.POLY:
	    polygonMouseReleased(eventToPixelX(e), eventToPixelY(e));
	    break;
	}
	updateDisplay();
    }
    
    void shapeMousePressed(int x, int y) {
	mousedownx = x;
	mousedowny = y;
	// remember the shapepixels so you can draw correctly during the drag
	System.arraycopy(shapecolors, 0, shapecolors_backup, 0, shapecolors.length);
	System.arraycopy(shapemasks, 0, shapemasks_backup, 0, shapemasks.length);
	System.arraycopy(panelpixels, 0, panelpixels_backup, 0, panelpixels.length);
    }
    
    /* LINE DRAWING */
    
    void lineMouseDragged(int x, int y) {
	System.arraycopy(shapecolors_backup, 0, shapecolors, 0, shapecolors.length);
	System.arraycopy(shapemasks_backup, 0, shapemasks, 0, shapemasks.length);
	System.arraycopy(panelpixels_backup, 0, panelpixels, 0, panelpixels.length);
	lineMouseReleased(x, y);
    }
    
    void lineMouseReleased(int x, int y) {
	if (x < 0) x = 0;
	else if (x >= size) x = size - 1;
	if (y < 0) y = 0;
	else if (y >= size) y = size - 1;
	Point[] pixels = patchesInLine(mousedownx, mousedowny, x, y);
	for (int i=0; i<pixels.length; i++) {
	    if (pixels[i] == null) return;
	    paintPixel(pixels[i].x, pixels[i].y);
	}
    }
    
    private Point[] patchesInLine(int x1, int y1, int x2, int y2) {
	int dx = (x1 < x2) ? x2 - x1 : x1 - x2;
	int dy = (y1 < y2) ? y2 - y1 : y1 - y2;
	Point[] v = new Point[2*(dx + dy)+1];
	int i=0;
	if (dx > dy) {
	    if (x1 < x2) {
		for (int x =  x1; x <= x2; x++) {
		    int y = (y2-y1)*(x-x1)/dx+y1;
		    v[i++] = new Point(x,y);
		}
	    }
	    else if (x1 > x2) {
		for (int x = x2; x <= x1; x++) {
		    int y = (y2-y1)*(x-x1)/(x2-x1)+y1;
		    v[i++] = new Point(x,y);
		}
	    }
	} else {
	    if (y1 < y2) {
		for (int y = y1; y <= y2; y++) {
		    int x = (x2-x1)*(y-y1)/dy+x1;
		    v[i++] = new Point(x,y);
		}
	    }
	    else if (y1 > y2) {
		for (int y = y2; y <= y1; y++) {
		    int x = (x2-x1)*(y-y1)/(y2-y1)+x1;
		    v[i++] = new Point(x,y);
		}
	    }
	    else {
		v[i++] = new Point(mousedownx, mousedowny);
	    }
	}
	return v;
    }
    
    /* RECTANGLE DRAWING */
    
    void rectangleMouseDragged(int x, int y) {
	System.arraycopy(shapecolors_backup, 0, shapecolors, 0, shapecolors.length);
	System.arraycopy(shapemasks_backup, 0, shapemasks, 0, shapemasks.length);
	System.arraycopy(panelpixels_backup, 0, panelpixels, 0, panelpixels.length);
	rectangleMouseReleased(x, y);
    }
    
    void rectangleMouseReleased(int x, int y) {
	if (x < 0) x = 0;
	else if (x >= size) x = size - 1;
	if (y < 0) y = 0;
	else if (y >= size) y = size - 1;
	int anchorx = Math.min(mousedownx, x);
	int anchory = Math.min(mousedowny, y);
	int cornerx = Math.max(mousedownx, x);
	int cornery = Math.max(mousedowny, y);
	for (int i=anchorx; i<=cornerx; i++) paintPixel(i, anchory);
	for (int i=anchorx; i<=cornerx; i++) paintPixel(i, cornery);
	for (int i=anchory; i<=cornery; i++) paintPixel(anchorx, i);
	for (int i=anchory; i<=cornery; i++) paintPixel(cornerx, i);
    }
    
    void fillRectangleMouseDragged(int x, int y) {
	System.arraycopy(shapecolors_backup, 0, shapecolors, 0, shapecolors.length);
	System.arraycopy(shapemasks_backup, 0, shapemasks, 0, shapemasks.length);
	System.arraycopy(panelpixels_backup, 0, panelpixels, 0, panelpixels.length);
	fillRectangleMouseReleased(x, y);
    }
    
    void fillRectangleMouseReleased(int x, int y) {
	if (x < 0) x = 0;
	else if (x >= size) x = size - 1;
	if (y < 0) y = 0;
	else if (y >= size) y = size - 1;
	int anchorx = Math.min(mousedownx, x);
	int anchory = Math.min(mousedowny, y);
	int cornerx = Math.max(mousedownx, x);
	int cornery = Math.max(mousedowny, y);
	for (int i=anchorx; i<=cornerx; i++) {
	    for (int j=anchory; j<=cornery; j++) {
		paintPixel(i, j);
	    }
	}
    }
    
    /* OVAL DRAWING */
    
    void ovalMouseDragged(int x, int y) {
	System.arraycopy(shapecolors_backup, 0, shapecolors, 0, shapecolors.length);
	System.arraycopy(shapemasks_backup, 0, shapemasks, 0, shapemasks.length);
	System.arraycopy(panelpixels_backup, 0, panelpixels, 0, panelpixels.length);
	ovalMouseReleased(x, y);
    }
    
    void ovalMouseReleased(int x, int y) {
        if (x < 0) x = 0;
        else if (x >= size) x = size - 1;
        if (y < 0) y = 0;
        else if (y >= size) y = size - 1;
    	paintPatchesOval(mousedownx, mousedowny, x, y, 0);
    }
    
    
    void fillOvalMouseDragged(int x, int y) {
    	System.arraycopy(shapecolors_backup, 0, shapecolors, 0, shapecolors.length);
    	System.arraycopy(shapemasks_backup, 0, shapemasks, 0, shapemasks.length);
	System.arraycopy(panelpixels_backup, 0, panelpixels, 0, panelpixels.length);
    	fillOvalMouseReleased(x, y);
    }

    void fillOvalMouseReleased(int x, int y) {
    	if (x < 0) x = 0;
    	else if (x >= size) x = size - 1;
    	if (y < 0) y = 0;
    	else if (y >= size) y = size - 1;
    	paintPatchesOval(mousedownx, mousedowny, x, y, 1);
    }

    private void paintPatchesOval(int x1, int y1, int x2, int y2, int fill)
    {
        int rx, ry;
        if(x1 > x2) {
            rx = x1;
            x1 = x2;
            x2 = rx;
        }
        if(y1 > y2) {
            ry = y1;
            y1 = y2;
            y2 = ry;
        }
        rx = x2 - x1;
        ry = y2 - y1;
        if (rx < 2) {
            paintBlock(x1, y1, x2, y2);
            return;
        }
        else if (ry < 2) {
            paintBlock(x1, y1, x2, y2);
            return;
        }
        int xc = x1 + x2 + 1;
        int yc = y1 + y2 + 1;
        int ry2 = ry * ry, rx2 = rx * rx;
        int x = ((xc % 2 == 0) ? 1 : 0);
        int y = (int)Math.sqrt((double)ry2 * (1. - (double)(x * x) / (double)rx2));
        while((double)(ry * x) / ((double)rx2 * Math.sqrt(1. - (double)(x * x) / (double)rx2)) < 1 && y >= 0)
	    {
		paintPointInOval(xc, yc, x, y, fill);
		x += 2;
		y = (int)Math.sqrt((double)ry2 * (1. - (double)(x * x) / (double)rx2));
	    }
        paintPointInOval(xc, yc, x, y, fill);
        y = ((yc % 2 == 0) ? 1 : 0);
        x = (int)Math.sqrt((double)rx2 * (1. - (double)(y * y) / (double)ry2));
        while((double)(rx * y) / ((double)ry2 * Math.sqrt(1. - (double)(y * y) / (double)ry2)) < 1 && x >= 0)
	    {
		paintPointInOval(xc, yc, x, y, fill);
		y += 2;
		x = (int)Math.sqrt((double)rx2 * (1. - (double)(y * y) / (double)ry2));
	    }
    }    
    
    private void paintBlock(int x1, int y1, int x2, int y2) {
        int x, y;
        for(x = x1; x <= x2; x++) {
	    for(y = y1; y <= y2; y++) {
		paintPixel(x, y);
	    }
	}
    }

    void paintPointInOval(int xc, int yc, int x, int y, int fill) {
        int x2 = (xc + x) / 2, y2= (yc + y) / 2, x1 = (xc - x - 1) / 2, y1 = (yc - y - 1) / 2;
        if(fill == 1) {
	    for(int i = x1; i <= x2; i++) {
		paintPixel(i, y1);
		paintPixel(i, y2);
	    }
	} else {
	    paintPixel(x1, y1);
	    paintPixel(x2, y1);
	    paintPixel(x1, y2);
	    paintPixel(x2, y2);
	}
    }

    /* POLYGON DRAWING */
    
    void polygonMouseMoved(int x, int y) {
	if (!drawingPoly) return;
	System.arraycopy(shapecolors_backup, 0, shapecolors, 0, shapecolors.length);
	System.arraycopy(shapemasks_backup, 0, shapemasks, 0, shapemasks.length);
	System.arraycopy(panelpixels_backup, 0, panelpixels, 0, panelpixels.length);
	Point[] v1 = patchesInLine(mousedownx, mousedowny, x, y);
	for (int i = 0; i < v1.length; i++) {
	    if (v1[i] == null) break;
	    paintPixel(v1[i].x, v1[i].y);
	}
	updateDisplay();		
    }
    
    void polygonMousePressed(int x, int y) {
	if (startOfPoly) {
	    System.arraycopy(shapecolors, 0, shapecolors_backup, 0, shapecolors.length);
	    System.arraycopy(shapemasks, 0, shapemasks_backup, 0, shapemasks.length);
	    System.arraycopy(panelpixels, 0, panelpixels_backup, 0, panelpixels.length);
	    drawingPoly = true;
	    mousedownx = x;
	    mousedowny = y;
	    mousedragx = mousedownx;
	    mousedragy = mousedowny;
	    paintPixel(x, y);
	    startOfPoly = false;
	    polyHeadx = x;
	    polyHeady = y;
	} else {
	    mousedragx = x;
	    mousedragy = y;
	    Point[] v1 = patchesInLine(mousedownx, mousedowny, mousedragx, mousedragy);
	    for (int i = 0; i < v1.length; i++) {
		if (v1[i] == null) break;
		paintPixel(v1[i].x, v1[i].y);
	    }
	    mousedownx = mousedragx;
	    mousedowny = mousedragy;
	    if (x == polyHeadx && y == polyHeady) {
		startOfPoly = true;
		drawingPoly = false;
	    }
	}
    }
    
    void polygonMouseReleased(int x, int y) {
	Point[] v1 = patchesInLine(mousedownx, mousedowny, mousedragx, mousedragy);
	for (int i = 0; i < v1.length; i++) {
	    if (v1[i] == null) break;
	    paintPixel(v1[i].x, v1[i].y);
	}
	System.arraycopy(shapecolors, 0, shapecolors_backup, 0, shapecolors.length);
	System.arraycopy(shapemasks, 0, shapemasks_backup, 0, shapemasks.length);
	System.arraycopy(panelpixels, 0, panelpixels_backup, 0, panelpixels.length);
    }
    
    /* SHAPE FILL DRAWING */
    
    void fillArea(Point p) {
	Stack   toCheck = new Stack();
	Hashtable checked = new Hashtable();
	Hashtable toColor = new Hashtable();
	Object o = new Object();
	
	Point a, b;
	toCheck.push(p);
	int[] dx = {0, 0, -1, 1};
	int[] dy = {1, -1, 0, 0};
	checked.put(p, o);
	while (!toCheck.empty()) {
	    a = (Point) toCheck.pop();
	    toColor.put(a,o);
	    for (int i=0; i<dx.length; i++) {
		b = new Point(a.x + dx[i], a.y + dy[i]);
		if (!checked.containsKey(b) &&
		    b.x > -1 && b.x < size && b.y > -1 && b.y < size &&
		    isColoredSame(a, b)) {
		    checked.put(b,o);
		    toCheck.push(new Point(b)); 
		}
	    }
	}
	Enumeration fillPts = toColor.keys();
	while (fillPts.hasMoreElements()) {
	    a = (Point)fillPts.nextElement();
	    paintPixel(a.x, a.y);
	}
    }
    
    public void mouseEntered(MouseEvent e) {}
    public void mouseExited(MouseEvent e) {}
    
    public static void loadImages(String gifdir, MediaTracker tracker, Toolkit kit) throws MalformedURLException {
	e8x8Image = kit.createImage(JShapeEditorPanel.class.getResource(gifdir + "turtle-transparent-8.gif"));
	e17x17Image = kit.createImage(JShapeEditorPanel.class.getResource(gifdir + "turtle-transparent-17.gif"));
	e35x35Image = kit.createImage(JShapeEditorPanel.class.getResource(gifdir + "turtle-transparent-35.gif"));
	tracker.addImage(e8x8Image, Widget.trackerID++);
	tracker.addImage(e17x17Image, Widget.trackerID++);
	tracker.addImage(e35x35Image, Widget.trackerID++);
	
	PixelGrabber pg;
	
	try {//*****JFK*****
	    pg = new PixelGrabber(e8x8Image, 0, 0, 8, 8, turtleTrans8x8, 0, 8);
	    pg.grabPixels();
	    pg = new PixelGrabber(e17x17Image, 0, 0, 17, 17, turtleTrans16x16, 0, 17);
	    pg.grabPixels();
	    pg = new PixelGrabber(e35x35Image, 0, 0, 35, 35, turtleTrans32x32, 0, 35);
	    pg.grabPixels();
	} catch (InterruptedException e) {}
    }
    
}
