/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class JShapeEditorRotatePanel extends JPanel {

    JShapeEditor parent;
    Application app; 

    static Image toolbarUp;
    static Image toolbarDown;

    int buttonDown = NONE;
    private static int NONE   = 0;
    private static final int ROTATE = 1;
    private static final int FLIP_Y = 2;
    private static final int FLIP_X = 3;

    public JShapeEditorRotatePanel(Application app, JShapeEditor parent) {
	setToolTipText("dummy text");
	this.parent = parent;		
	this.app = app;
	addMouseListener(new MouseAdapter() {
		public void mousePressed(MouseEvent e) {
		    buttonDown = getButton(e);
		    repaint();
		}
		
		public void mouseReleased(MouseEvent e) {
		    int releaseButton = getButton(e);
		    if (buttonDown != NONE && releaseButton == buttonDown) {
			switch(buttonDown) {
			case ROTATE:
			    rotate();
			    break;
			case FLIP_X:
			    flipx();
			    break;
			case FLIP_Y:
			    flipy();
			    break;
			}
		    }
		    buttonDown = NONE;
		    repaint();
		}
	    });
    }

    public void paintComponent(Graphics g) {
	g.drawImage(toolbarUp, 0, 0, 94, 32, this);
	if (buttonDown != NONE) {
	    Shape oldClip = g.getClip();
	    g.clipRect(31 * (buttonDown - 1), 0, 31, 31);
	    g.drawImage(toolbarDown, 0, 0, this);
	    g.setClip(oldClip);
	}
    }


    private int getButton(MouseEvent e) {
	int index = e.getX() / 31;
	return (index > 3 || e.getY() > 32) ? NONE : (index + 1);
    }

    static void loadImages(String gifdir, MediaTracker tracker, Toolkit kit) 
	throws java.net.MalformedURLException {
	toolbarUp = kit.createImage(JShapeEditorRotatePanel.class.getResource(gifdir  + "shape-toolbar.gif"));
	toolbarDown = kit.createImage(JShapeEditorRotatePanel.class.getResource(gifdir + "shape-toolbar-down.gif"));

	tracker.addImage(toolbarUp, Widget.trackerID++);
	tracker.addImage(toolbarDown, Widget.trackerID++);
    }	

    public void rotate() {
	TurtleShapes shape = parent.editedShape;
	int size = parent.size;
	double[] rotcolors = new double[shape.getColorsForSize(size)[0].length];
	int[] rotmasks     = new int[shape.getMasksForSize(size)[0].length];
	TurtleShapes.rotate(4, shape.getColorsForSize(size)[0], shape.getMasksForSize(size)[0], 
			    rotcolors, rotmasks);
	shape.setColorsForSize(rotcolors, size);
	shape.setMasksForSize(rotmasks, size);
	shape.reRotateShapes();
	parent.editor.changeSize(size);
    }
    
    public void flipx() {
	TurtleShapes shape = parent.editedShape;
	int size = parent.size;
	double[] flipcolors = new double[shape.getColorsForSize(size)[0].length];
	int[] flipmasks     = new int[shape.getMasksForSize(size)[0].length];
	TurtleShapes.flipX(shape.getColorsForSize(size)[0], shape.getMasksForSize(size)[0], 
			   flipcolors, flipmasks);
	shape.setColorsForSize(flipcolors, size);
	shape.setMasksForSize(flipmasks, size);
	shape.reRotateShapes();
	parent.editor.changeSize(size);
    }

    public void flipy() {
	TurtleShapes shape = parent.editedShape;
	int size = parent.size;
	double[] flipcolors = new double[shape.getColorsForSize(size)[0].length];
	int[] flipmasks     = new int[shape.getMasksForSize(size)[0].length];
	TurtleShapes.flipY(shape.getColorsForSize(size)[0], shape.getMasksForSize(size)[0], 
			   flipcolors, flipmasks);
	shape.setColorsForSize(flipcolors, size);
	shape.setMasksForSize(flipmasks, size);
	shape.reRotateShapes();
	parent.editor.changeSize(size);
    }
   
    public String getToolTipText(MouseEvent e) {
	int button = e.getX() / 31 + 1;
	switch(button) {
	case ROTATE:
	    return app.textBundle.getString("rotate_clockwise");
	case FLIP_X:
	    return app.textBundle.getString("flip_x");
	case FLIP_Y: // deliberate fallthru, button math may not be perfect
	default:
	    return app.textBundle.getString("flip_y");
	}

    }
	
}
