/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.GridLayout;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.SystemColor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseAdapter;
import javax.swing.ButtonGroup;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JButton;
import javax.swing.ImageIcon;
import java.awt.*;
import java.net.MalformedURLException;

public class JShapeEditorSizeChooserPanel extends JPanel {
    //JShapeEditorRotatePanel rot;
    JShapeEditor parent;
    RadioButton size8, size16, size32; 
    SizeChangerButtons eighttosixteen, sixteentothirtytwo;
    ButtonGroup group = new ButtonGroup();
    int size;

    static Image arrowsUp;
    static Image arrowsDown;

    
    public JShapeEditorSizeChooserPanel(JShapeEditor parent) {
	this.parent = parent;
	setLayout(new BorderLayout());

	size8 = new RadioButton(new ShapeIcon(parent.shapeNum, TurtleShapes.SMALL, parent), false);
	size16 = new RadioButton(new ShapeIcon(parent.shapeNum, TurtleShapes.MEDIUM, parent), false);
	size32 = new RadioButton(new ShapeIcon(parent.shapeNum, TurtleShapes.LARGE, parent), false);
	
	size8.setToolTipText("Edit the 8x8 version of this shape.");
	size16.setToolTipText("Edit the 16x16 version of this shape.");
	size32.setToolTipText("Edit the 32x32 version of this shape.");


	eighttosixteen = new SizeChangerButtons(this, parent.app, 
						SizeChangerButtons.EIGHTSIXTEEN);
	sixteentothirtytwo = new SizeChangerButtons(this, parent.app, 
						    SizeChangerButtons.SIXTEENTHIRTYTWO);
	eighttosixteen.setPreferredSize(new Dimension(26,28));
	sixteentothirtytwo.setPreferredSize(new Dimension(26,28));

	ActionListener sizeListener = new RadioSizeListener();
	size8.setBorderPainted(true);
	size16.setBorderPainted(true);
	size32.setBorderPainted(true);
	size8.addActionListener(sizeListener);
	size16.addActionListener(sizeListener);
	size32.addActionListener(sizeListener);
			
	group.add(size8);
	group.add(size16);
	group.add(size32);

	int gap = 5;
	setLayout(null);
	size8.setSize(9,9);
	size8.setLocation(20,15);
	add(size8);
	eighttosixteen.setSize(26,28);
	eighttosixteen.setLocation(size8.getSize().width + size8.getLocation().x + gap, 4);
	add(eighttosixteen);
	size16.setSize(17,17);
	size16.setLocation(eighttosixteen.getSize().width + eighttosixteen.getLocation().x + gap , 11);
	add(size16);
	sixteentothirtytwo.setSize(26,28);
	sixteentothirtytwo.setLocation(size16.getSize().width + size16.getLocation().x + gap, 4);
	add(sixteentothirtytwo);
	size32.setSize(33,33);
	size32.setLocation(sixteentothirtytwo.getSize().width + sixteentothirtytwo.getLocation().x + gap, 3);
	add(size32);
	//	add(buttonPanel, BorderLayout.EAST);
	size = parent.size;
	switch (size) {
	case 8: size8.setIconSelected(true);
	    break;
	case 16: size16.setIconSelected(true);
	    break;
	case 32: size32.setIconSelected(true);
	    break;
	}
	switchToSize(size);
    }

    public void updateAll(){
	if (size8.isIconSelected()) {
	    size8.updateDisplay();
	}
	if (size16.isIconSelected()) {
	    size16.updateDisplay();
	}
	if (size32.isIconSelected()) {
	    size32.updateDisplay();
	}
    }
   

    public void switchToSize(int size) {
	switch(size) {
	case TurtleShapes.SMALL:
	    size8.setSelected(true);
	    break;
	case TurtleShapes.MEDIUM:
	    size16.setSelected(true);
	    break;
	case TurtleShapes.LARGE:
	    size32.setSelected(true);
	    break;
	}

    }
    

    
    class RadioSizeListener implements ActionListener {
	public void actionPerformed(ActionEvent e) {
	    int size=8;
	    if(e.getSource() == size8)
		size = TurtleShapes.SMALL;
	    if(e.getSource() == size16)
		size = TurtleShapes.MEDIUM;
	    if(e.getSource() == size32)
		size = TurtleShapes.LARGE;
	    parent.switchToSize(size);
	    parent.shapeRotationPreview.updateSize(size);
	    switch(size){
	    case TurtleShapes.SMALL:
		size8.setIconSelected(true);
		size16.setIconSelected(false);
		size32.setIconSelected(false);
		break;

	    case TurtleShapes.MEDIUM:
		size8.setIconSelected(false);
		size16.setIconSelected(true);
		size32.setIconSelected(false);
		break;
		
	    case TurtleShapes.LARGE:
		size8.setIconSelected(false);
		size16.setIconSelected(false);
		size32.setIconSelected(true);
		break;
	    }
	}
    }

    class RadioButton extends JRadioButton {
	
	ShapeIcon icon;
	
	RadioButton(ShapeIcon icon, boolean selected) {
	    super(icon, selected);
	    this.icon = icon;
	}
	
	public void setIconSelected(boolean selected) {
	    icon.setSelected(selected);
	}	    

	public boolean isIconSelected() {
	    return icon.isSelected();
	}

	public void updateDisplay() {
	    //System.out.println("refreshing icon");
	    icon.refresh();
	}
    }

    class SizeChangerButtons extends JPanel {

	JShapeEditorSizeChooserPanel parent;
	
	int type = 1;
	int buttonDown = NONE;
	private static final int NONE = 0;
    private static final int RIGHT =1;
    private static final int LEFT =2;
    private static final int EIGHTSIXTEEN = 1;
    private static final int SIXTEENTHIRTYTWO =2;
    Application app;

    public SizeChangerButtons(JShapeEditorSizeChooserPanel theparent, Application theapp, int thetype) {
			setToolTipText("dummy text");	
	this.parent = theparent;
	this.app = theapp;
	this.type = thetype;
	addMouseListener(new MouseAdapter() {
		public void mousePressed(MouseEvent e){
		    buttonDown = getButton(e);
		    repaint();
		}

		public void mouseReleased(MouseEvent e){
		    int releaseButton = getButton(e);
		    if (buttonDown != NONE && releaseButton == buttonDown) {
			switch(buttonDown) {
			case LEFT:
			    if(type ==EIGHTSIXTEEN){
			
				//make shape = tempshape from jshapeeditor instead
				//using a copy rather than pulling from array.
				TurtleShapes shape = parent.parent.editedShape; 
				TurtleShapes.resize(shape.colors16x16[0], shape.masks16x16[0], TurtleShapes.SMALL,
						    shape.colors8x8[0], shape.masks8x8[0], app);
				parent.size8.updateDisplay();
				if (parent.parent.size == 8) {
				    parent.parent.shapeRotationPreview.updateDisplay();
				    parent.parent.editor.lightweightUpdateDisplay();
				}
				parent.parent.repaint();
			    }
			    if(type == SIXTEENTHIRTYTWO){
				TurtleShapes shape = parent.parent.editedShape; 
				TurtleShapes.resize(shape.colors32x32[0], shape.masks32x32[0], TurtleShapes.MEDIUM,
						    shape.colors16x16[0], shape.masks16x16[0], app);
				parent.size16.updateDisplay();
				if (parent.parent.size == 16) {
				    parent.parent.shapeRotationPreview.updateDisplay();
				    parent.parent.editor.lightweightUpdateDisplay();
				}
				parent.parent.repaint();
			    }
			    break;
			case RIGHT:
			    if(type == EIGHTSIXTEEN){
			
				TurtleShapes shape = parent.parent.editedShape; 
				TurtleShapes.resize(shape.colors8x8[0], shape.masks8x8[0], TurtleShapes.MEDIUM,
						    shape.colors16x16[0], shape.masks16x16[0], app);
				parent.size16.updateDisplay();
				if (parent.parent.size == 16) {
				    parent.parent.shapeRotationPreview.updateDisplay();
				    parent.parent.editor.lightweightUpdateDisplay();
				}
				parent.parent.repaint();
			    }
			    if(type == SIXTEENTHIRTYTWO){
				TurtleShapes shape = parent.parent.editedShape; 
				TurtleShapes.resize(shape.colors16x16[0], shape.masks16x16[0], TurtleShapes.LARGE,
						    shape.colors32x32[0], shape.masks32x32[0], app);
				parent.size32.updateDisplay();
				if (parent.parent.size == 32) {
				    parent.parent.shapeRotationPreview.updateDisplay();
				    parent.parent.editor.lightweightUpdateDisplay();
				}
				parent.parent.repaint();
			    }
			    break;

			}
		    }
		    buttonDown = NONE;
		    repaint();
		}

	    });
    }

    public void paintComponent(Graphics g) {
	g.drawImage(parent.arrowsUp, 0, 0, 26, 28 , this);
		
	if(buttonDown !=NONE){
	    Shape oldClip = g.getClip();
	    g.clipRect(0, 14 * (buttonDown -1), 26,14);
	    g.drawImage(parent.arrowsDown, 0, 0, this);
	    g.setClip(oldClip);
	}
    }

    private int getButton(MouseEvent e) {
	int index = e.getY() / 14;
	return (((index > 2 || e.getX() > 26) || (e.getY() <= 15 && e.getY() >= 13 )) ? NONE : (index + 1));
    }

	public String getToolTipText(MouseEvent event) {
		int button = getButton(event);
		switch(button) {
			case LEFT:
				if (type == EIGHTSIXTEEN) {
					return "Use the 16x16 shape as the 8x8 shape.";
				} else if (type == SIXTEENTHIRTYTWO) {
					return "Use the 32x32 shape as the 16x16 shape.";
				}
			case RIGHT:
				if (type == EIGHTSIXTEEN) {
					return "Use the 8x8 shape as the 16x16 shape.";
				} else if (type == SIXTEENTHIRTYTWO) {
					return "Use the 16x16 shape as the 32x32 shape.";
				}
		}
		return "";
	}
    
}

    static void loadImages(String gifdir, MediaTracker tracker, Toolkit kit) 
	throws java.net.MalformedURLException {
	arrowsUp = kit.createImage(JShapeEditorSizeChooserPanel.class.getResource(gifdir  + "leftandrightup.gif"));
	arrowsDown = kit.createImage(JShapeEditorSizeChooserPanel.class.getResource(gifdir + "leftandrightdown.gif"));
	
	tracker.addImage(arrowsUp, Widget.trackerID++);
	tracker.addImage(arrowsDown, Widget.trackerID++);
    }

}
