/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.net.*;
import javax.swing.JPanel;

public class JShapePaintSelect extends JPanel {

    boolean patch = false;
    boolean turtle = false;
    boolean paint = true;
    
    static int turtlewidth = 17;
    static int turtleheight = 17;
    static Image turtleImage;
    static Image newTurtleImage;
    static int colorBlack = -16777216;
    static int colorYellow = -256;
    
    Rectangle patchRect  = new Rectangle(4, 14, 16, 16);
    Rectangle turtleRect = new Rectangle(27, 14, 15, 15);
    
    int[] turtlepixels = new int[turtlewidth*turtleheight];
    MemoryImageSource source;
    int[] imagepixels = new int[turtlewidth*turtleheight];
    
    JShapeColorTool colortool;
    
    public JShapePaintSelect(JShapeColorTool colortool) {
	this.colortool = colortool;
	makeTurtlePixels();
	addMouseListener(new MouseAdapter() {
		public void mouseClicked(MouseEvent e) {
		    JShapePaintSelect.this.mouseClicked(e);
		}
	    });
    }

    public void mouseClicked(MouseEvent e) {
	if (patchRect.contains(e.getX(), e.getY())) {
	    //System.out.println("patchRect");
	    if (patch) return;
	    patch = true;
	    turtle = false;
	    paint = false;
	    updateTurtleColor(Colors.colorarray[0]);
	    repaint();
	} else if (turtleRect.contains(e.getX(), e.getY())) {
	    //System.out.println("turtleRect");
	    if (turtle) return;
	    patch = false;
	    turtle = true;
	    paint = false;
	    updateTurtleColor(Color.gray.darker());
	    repaint();
	}
    }


    public void makeTurtlePixels() {
	PixelGrabber pg = new PixelGrabber(turtleImage, 0, turtleheight,
					   turtlewidth, turtleheight, turtlepixels,
					   0, turtlewidth);
	try {
	    pg.grabPixels();
	} catch (InterruptedException e) {
	    //System.err.println("interrupted waiting for pixels!");
	}
	if ((pg.getStatus() & ImageObserver.ABORT) != 0) {
	    //System.err.println("image fetch aborted or errored");
	    return;
	}
	
	for (int i = 0; i < turtlewidth*turtleheight; i++) {
	    if (turtlepixels[i] != colorBlack) {
		imagepixels[i] = turtlepixels[i];
	    }
	}
	source = new MemoryImageSource(turtlewidth, turtleheight, imagepixels,
				       0, turtlewidth);
	source.setAnimated(true);
	newTurtleImage = Toolkit.getDefaultToolkit().createImage(source);
    }
    
    public void paintComponent(Graphics g) {
	g.setColor(SystemColor.control);
	g.fillRect(0, 0, getSize().width, getSize().height);		
	if (patch) {
	    g.drawImage(JShapeEditorPanel.patchTrans16x16AsImage, 4, 14, this);
	    g.setColor(Colors.colorarray[0]);
	    g.drawRect(3, 13, 17, 17);
	    Shape oldclip = g.getClip();
	    g.clipRect(27, 14, 16, 17);
	    g.drawImage(turtleImage, 27, 14, this);
	    g.setClip(oldclip);
	} else if (turtle) {
	    g.setColor(Colors.colorarray[0]);
	    g.fillRect(4, 14, 16, 16);
	    g.drawImage(newTurtleImage, 27, 14, this);
	} else {
	    g.setColor(Colors.colorarray[0]);
	    g.fillRect(4, 14, 16, 16);
	    Shape oldclip = g.getClip();
	    g.setClip(27, 14, 16, 17);
	    g.drawImage(turtleImage, 27, 14, this);
	    g.setClip(oldclip);
	}
    }
    
    void updateTurtleColor(Color color) {
	//System.out.println("Top of updateTurtleColor");
	int currentColor = color.getRGB();
	for (int i = 0; i < turtlewidth*turtleheight; i++) {
	    if (turtlepixels[i] == colorBlack) {
		imagepixels[i] = JShapeEditorPanel.patchTrans16x16[i];
	    } else if (turtlepixels[i] == colorYellow) {
		imagepixels[i] = 0xFF000000;
	    }
	}
	source.newPixels(0, 0, turtlewidth, turtleheight);
    }
    
    
    public void deselect() {
	if (!patch && !turtle) return;
	patch = false;
	turtle = false;
	paint = true;
	updateTurtleColor(Colors.colorarray[0]);
	repaint();
    }
    
    public static void loadImages(String gifdir, MediaTracker tracker,
				  Toolkit kit) 
	throws MalformedURLException {
	URL u1 = JShapePaintSelect.class.getResource(gifdir + "turtle.gif");
	if (u1 == null) throw new MalformedURLException();
	turtleImage = kit.createImage(u1);
	tracker.addImage(turtleImage, Widget.trackerID++);
    }
    
}
