/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.awt.*;
import java.awt.event.*;
import javax.swing.JPanel;

public class JShapePaintTool extends JPanel {
    
    Image unpressed = Painttool.paintVertImage;
    Image pressed   = Painttool.paintVertImagePush;
    
    public static final int NONE = 0;
    public static final int PEN = 1;
    public static final int ERASER = 2;
    public static final int LINE = 3;
    public static final int EYEDROPPER = 4;
    public static final int REC = 5;
    public static final int FILLREC = 6;
    public static final int OVAL = 7;
    public static final int FILLOVAL = 8;
    public static final int POLY = 9;
    public static final int FILLSHAPE = 10;
    
    public static int buttonDown = PEN;
    public int rowDown;
    public int columnDown;
    
    public int button;
    public int oldButtonDown = NONE;
    private int offset = 0;
    
    static final int buttonWidth = 24;
    static final int buttonHeight = 24;
    
    public JShapePaintTool() {
	addMouseListener(new MouseAdapter() {
		public void mouseClicked(MouseEvent e) {
		    oldButtonDown = buttonDown;
		    buttonDown = pointToButton(e.getX(), e.getY());
		    if (oldButtonDown == buttonDown) return;
		    repaint();
		}
		
		public void mousePressed(MouseEvent e) {
		    mouseClicked(e);
		}
	    });
	addMouseMotionListener(new MouseMotionAdapter() {
		public void mouseDragged(MouseEvent e) {
		    oldButtonDown = buttonDown;
		    buttonDown = pointToButton(e.getX(), e.getY());
		    if (oldButtonDown == buttonDown) return;
		    repaint();
		}
	    });

	buttonDown = PEN;
    }
    
    public void paintComponent(Graphics g) {
	g.setColor(SystemColor.control);
	g.drawLine(0, 49, 0, 121);
	g.drawImage(unpressed, 0, 0, this);
	if (buttonDown != NONE) {
	    Shape oldclip = g.getClip();
	    g.clipRect(buttonWidth* columnDown, buttonHeight * rowDown, buttonWidth, buttonHeight);
	    g.drawImage(pressed, 0, 0, null);
	    g.setClip(oldclip);
	}
    }
    
    public int pointToButton(int x, int y) {
	columnDown =  (int)java.lang.Math.ceil(2 * x / 49);
	rowDown = (int)java.lang.Math.ceil(5 * y / 121);
	button = rowDown * 2 + columnDown + 1;
	return button;
    }
}
