/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class JShapeRotateMenu extends JPanel implements ActionListener {

    JShapeEditor parent;

    static Image xFlipImage;
    static Image yFlipImage;
    static Image rotateImage;

    JButton flipx  = new JButton(new ImageIcon(xFlipImage));
    JButton flipy  = new JButton(new ImageIcon(yFlipImage));
    JButton rotate = new JButton(new ImageIcon(rotateImage));

    public JShapeRotateMenu(JShapeEditor parent) {
	this.parent = parent;
	setLayout(null);
		
	flipx.addActionListener(this);
	flipx.setSize(new Dimension(32, 32));
	flipx.setLocation(0, 0);
	add(flipx);

	flipy.addActionListener(this);
	flipy.setSize(new Dimension(32, 32));
	flipy.setLocation(32, 0);
	add(flipy);

	rotate.addActionListener(this);
	rotate.setSize(new Dimension(32, 32));
	rotate.setLocation(64, 0);
	add(rotate);
		
	setPreferredSize(new Dimension(96, 32));
    }

    public void actionPerformed(ActionEvent e) {
	Object src = e.getSource();
	if (src == flipx) {
	    flipx();
	} else if (src == flipy) {
	    flipy();
	} else if (src == rotate) {
	    rotate();
	}
    }

    public static void loadImages(String gifdir, MediaTracker tracker, Toolkit kit) {
	xFlipImage = kit.createImage(JShapeRotateMenu.class.getResource(gifdir + "flipxaxis.gif"));
	yFlipImage = kit.createImage(JShapeRotateMenu.class.getResource(gifdir + "flipyaxis.gif"));
	rotateImage = kit.createImage(JShapeRotateMenu.class.getResource(gifdir + "rotate.gif"));

	tracker.addImage(xFlipImage, Widget.trackerID++);
	tracker.addImage(yFlipImage, Widget.trackerID++);
	tracker.addImage(rotateImage, Widget.trackerID++);
    }


    public void rotate() {
	TurtleShapes shape = parent.editedShape;
	int size = parent.size;
	double[] rotcolors = new double[shape.getColorsForSize(size)[0].length];
	int[] rotmasks     = new int[shape.getMasksForSize(size)[0].length];
	TurtleShapes.rotate(4, shape.getColorsForSize(size)[0], shape.getMasksForSize(size)[0], 
			    rotcolors, rotmasks);
	shape.setColorsForSize(rotcolors, size);
	shape.setMasksForSize(rotmasks, size);
	shape.reRotateShapes();
	parent.editor.changeSize(size);
    }
    
    public void flipx() {
	TurtleShapes shape = parent.editedShape;
	int size = parent.size;
	double[] flipcolors = new double[shape.getColorsForSize(size)[0].length];
	int[] flipmasks     = new int[shape.getMasksForSize(size)[0].length];
	TurtleShapes.flipX(shape.getColorsForSize(size)[0], shape.getMasksForSize(size)[0], 
			   flipcolors, flipmasks);
	shape.setColorsForSize(flipcolors, size);
	shape.setMasksForSize(flipmasks, size);
	shape.reRotateShapes();
	parent.editor.changeSize(size);
    }

    public void flipy() {
	TurtleShapes shape = parent.editedShape;
	int size = parent.size;
	double[] flipcolors = new double[shape.getColorsForSize(size)[0].length];
	int[] flipmasks     = new int[shape.getMasksForSize(size)[0].length];
	TurtleShapes.flipY(shape.getColorsForSize(size)[0], shape.getMasksForSize(size)[0], 
			   flipcolors, flipmasks);
	shape.setColorsForSize(flipcolors, size);
	shape.setMasksForSize(flipmasks, size);
	shape.reRotateShapes();
	parent.editor.changeSize(size);
    }
     
}
