/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.awt.*;
import java.awt.event.*;
import yoyo.YoYo;
import javax.swing.*;
import javax.swing.border.*;

public class JSliderDialog extends JWidgetDialog implements SliderDialog, FocusListener {

  private static JSliderDialog dialog;

  public static final int MIN_SLIDER_VALUE = -999999999;
  public static final int MAX_SLIDER_VALUE =  999999999;
	
  private	int min, max, cur;
  private JLabel varLab, minLab, maxLab, curLab, nameLab;
  private JPanel butPanel, namePanel, namefieldPanel, varPanel, minPanel, maxPanel, curPanel;
  private String name, var;
  private JTextField varText, minText, maxText, curText, nameText;
  private ButtonGroup nameGroup;
  private JRadioButton instCheck, nameCheck;

    //private JScrollPane descriptScroll;
    private JLabel descriptLab;
    private JTextField descriptText;
    private String descript;

  private JSliderDialog(Frame parent, String title, Application app) {
    super(parent, app, title);
    init();
    initFocus(varText);
    descriptText.addFocusListener(this);
  }
	
  // for editing sliders, requires <wd> instanceof SliderWidget
  public static JSliderDialog create(Frame parent, String title, WidgetDrag wd, Application app) {
    SliderWidget sw = (SliderWidget)wd;
    if (dialog==null) {
      dialog = new JSliderDialog(parent, title, app);
    } else {
      dialog.setTitle(title);
    }
    dialog.addNotify();
    dialog.setFields(sw.name, sw.var, sw.min, sw.max, sw.cur, sw.showName, sw.getToolText());
    return dialog;
  }
	
  // for creating new sliders
  public static JSliderDialog create(Frame parent, String title, Application app) {
      String name = app.textBundle.getString("slider_label") + (SliderWidget.idCount+1);
    if (dialog==null) {
      dialog = new JSliderDialog(parent, title, app);
    } else {
      dialog.setTitle(title);
    }
    dialog.addNotify();
    dialog.setFields(name, "", 0, 99, 50, false, "");
    return dialog;
  }
	
  private void setFields(String name, String var, int min, int max, int cur, boolean showName, String descript) {
    this.name = name;
    this.var  = var;
		
    nameText.setText(name);
    nameText.setCaretPosition(0);
    varText.setText(var);
    varText.setCaretPosition(0);
    minText.setText(String.valueOf(min));
    minText.setCaretPosition(0);
    maxText.setText(String.valueOf(max));
    maxText.setCaretPosition(0);
    curText.setText(String.valueOf(cur));
    curText.setCaretPosition(0);
    descriptText.setText(descript);
    descriptText.setCaretPosition(0);
    if (showName) {
      nameCheck.setSelected(true);
    }
    else {
      instCheck.setSelected(true);
    }
  }
	
  protected void setupComponents() {

      nameLab = new JLabel(((InterfaceFrame)parent).getApp().textBundle.getString("name_label"));
    varLab = new JLabel(((InterfaceFrame)parent).getApp().textBundle.getString("variable_label"));
    minLab = new JLabel(((InterfaceFrame)parent).getApp().textBundle.getString("minimum_label"), SwingConstants.RIGHT);
    maxLab = new JLabel(((InterfaceFrame)parent).getApp().textBundle.getString("maximum_label"), SwingConstants.RIGHT);
    curLab = new JLabel(((InterfaceFrame)parent).getApp().textBundle.getString("current_label"), SwingConstants.RIGHT);
		
    Border oldBorder = minLab.getBorder();
    Insets oldBorderInsets = (oldBorder == null) 
	? new Insets(0, 0, 0, 0) 
	: oldBorder.getBorderInsets(minLab);
    minLab.setBorder(new EmptyBorder(oldBorderInsets.top, 
				     oldBorderInsets.left,
				     oldBorderInsets.bottom,
				     oldBorderInsets.right + 3));

    maxLab.setBorder(new EmptyBorder(oldBorderInsets.top, 
				     oldBorderInsets.left,
				     oldBorderInsets.bottom,
				     oldBorderInsets.right + 3));

    curLab.setBorder(new EmptyBorder(oldBorderInsets.top, 
				     oldBorderInsets.left,
				     oldBorderInsets.bottom,
				     oldBorderInsets.right + 3));


    nameText = new JTextField(9);
    varText = new JTextField(9);
    minText = new JTextField(5);
    maxText = new JTextField(5);
    curText = new JTextField(5);

    descriptLab = new JLabel(((InterfaceFrame)parent).getApp().textBundle.getString("tooltip_label"));
    descriptText = new JTextField();
    //descriptText.setLineWrap(true);
    //descriptText.setWrapStyleWord(true);
    //descriptScroll = new JScrollPane(descriptText);
    //descriptScroll.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
    //descriptScroll.setPreferredSize(new Dimension(80, 50));
 		
    //namefieldPanel = new Panel(new GridLayout(2, 1, 5, 0));
    //varPanel = new Panel(new GridLayout(2, 1, 5, 0));
    //minPanel = new Panel(new GridLayout(1, 2, 5, 0));
    //maxPanel = new Panel(new GridLayout(1, 2, 5, 0));
    //curPanel = new Panel(new GridLayout(1, 2, 5, 0));
		
    //minPanel.add(minLab);
    //minPanel.add(minText);
    //maxPanel.add(maxLab);
    //maxPanel.add(maxText);
    //curPanel.add(curLab);
    //curPanel.add(curText);

    nameGroup = new ButtonGroup();
    namePanel = new JPanel(new GridLayout(2, 1, 5, 0));
    instCheck = new JRadioButton(((InterfaceFrame)parent).getApp().textBundle.getString("variable_check_label"), true);
    nameCheck = new JRadioButton(((InterfaceFrame)parent).getApp().textBundle.getString("name_check_label"), false);
    nameGroup.add(instCheck);
    nameGroup.add(nameCheck);
    namePanel.add(nameCheck);
    namePanel.add(instCheck);

    //butPanel = new Panel();
    //butPanel.setLayout(new GridLayout(2, 1, 0, 5));
    //butPanel.add(bca);
    //butPanel.add(bok);
		
    // abstract this!
    if (!YoYo.macosxp) {
	formatText(varText);
	formatText(minText);
	formatText(maxText);
	formatText(curText);
	formatText(nameText);
	formatLabel(nameLab);
	formatLabel(varLab);
	formatLabel(minLab);
	formatLabel(maxLab);
	formatLabel(curLab);
	//formatLabel(butPanel);
	//formatLabel(namefieldPanel);
	//formatLabel(varPanel);
	//formatLabel(minPanel);
	//formatLabel(maxPanel);
	//formatLabel(curPanel);
	formatLabel(namePanel);
	formatLabel(nameCheck);
	formatLabel(instCheck);
	formatLabel(bok);
	formatLabel(bca);
	
	formatText(descriptText);
	formatLabel(descriptLab);
    }
  }
	
  protected void setupGBC() {
    GridBagLayout gbl = new GridBagLayout();
    getContentPane().setLayout(gbl);
    GridBagConstraints gbc = new GridBagConstraints();
		
    gbc.weightx = 100;
    gbc.weighty = 100;
    
    
    gbc.anchor=GridBagConstraints.WEST;
    gbc.fill=GridBagConstraints.HORIZONTAL;

    gbc.gridx=0;
    gbc.gridy=0;
    gbc.gridwidth=2;
    gbc.gridheight=1;
    gbc.weighty = 0;
    gbc.anchor=GridBagConstraints.NORTHWEST;
    gbc.fill=GridBagConstraints.NONE;
    gbc.insets = new Insets(6, 13, 0, 0);
    getContentPane().add(nameLab, gbc);
    
    gbc.gridy = 1;
    gbc.gridx = 0;
    gbc.weighty = 50;
    gbc.gridwidth = 2;
    gbc.anchor = GridBagConstraints.WEST;
    gbc.fill = GridBagConstraints.HORIZONTAL;
    gbc.ipady = 0;
    gbc.insets = new Insets(0, 13, 0, 0);
    getContentPane().add(nameText, gbc);
    
    gbc.gridy = 3;
	gbc.gridwidth = 1;
	gbc.weighty = 0; 
	gbc.ipady = 0;
	gbc.anchor = GridBagConstraints.WEST;
	gbc.fill = GridBagConstraints.NONE;
	gbc.insets = new Insets(6, 13, 0, 0);
	getContentPane().add(varLab, gbc);
	
	gbc.gridy = 4;
	gbc.gridwidth = 2;
	gbc.weighty = 50;
	gbc.ipady = 0;
	gbc.fill = GridBagConstraints.HORIZONTAL;
	gbc.insets = new Insets(0, 13, 0, 0);
	getContentPane().add(varText, gbc);
	
	gbc.weighty = 50;
	gbc.gridheight = 3;
	gbc.gridwidth = 2; 
	gbc.gridy = 0;
	gbc.gridx = 2;
	gbc.ipady = 0;
	gbc.fill = GridBagConstraints.BOTH;
	gbc.anchor = GridBagConstraints.SOUTHWEST;
	gbc.insets = new Insets((YoYo.macintoshp) ? 13 : 6, 13, 0, 13);
	getContentPane().add(namePanel, gbc);

	gbc.ipady = 0;
	gbc.gridx = 2;
	gbc.gridy = 3;
	gbc.gridwidth = 1;
	gbc.gridheight = 1;
	gbc.weighty = 0;
	gbc.anchor = GridBagConstraints.WEST;
	gbc.fill = GridBagConstraints.HORIZONTAL;
	gbc.insets = new Insets(13, 13, 0, 0);
	getContentPane().add(minLab, gbc);
	
	gbc.gridx = 3; 
	gbc.weighty = 10;
	gbc.ipady = 0;
	gbc.insets = new Insets(13, 0, 0, 13);
	getContentPane().add(minText, gbc);
	
	gbc.gridy = 4;
	gbc.gridx = 2;
	gbc.weighty = 0;
	gbc.ipady = 0;
	gbc.anchor = GridBagConstraints.SOUTHWEST;
	gbc.insets = new Insets(6, 13, 0, 0);
	getContentPane().add(maxLab, gbc);
	
	gbc.gridx = 3;
	gbc.gridwidth = 1; 
	gbc.weighty = 10;
	gbc.ipady = 0;
	gbc.insets = new Insets(6, 0, 0, 13);
	getContentPane().add(maxText, gbc);
	
	gbc.weighty = 0;
	gbc.gridx = 0;
	gbc.gridy = 5;
	gbc.ipady = 0;
        gbc.gridwidth = 2;
	gbc.anchor = GridBagConstraints.SOUTHWEST;
	gbc.fill = GridBagConstraints.HORIZONTAL;
	gbc.insets = new Insets(13, 13, 0, 0);
	getContentPane().add(descriptLab, gbc);
	
        gbc.ipady = 0;
        gbc.gridx=2;
        gbc.gridy=5;
        gbc.gridwidth = 1;
        gbc.weighty = 0;
        gbc.anchor = GridBagConstraints.NORTHWEST;
        gbc.fill=GridBagConstraints.HORIZONTAL;
        gbc.insets = new Insets(6, 13, 0, 0);
        getContentPane().add(curLab, gbc);
    
        gbc.gridy = 5;
        gbc.gridx = 3;
        gbc.gridwidth = 1;
        gbc.weighty = 10;
        gbc.insets = new Insets(6, 0, 0, 13);
        getContentPane().add(curText, gbc);

        gbc.gridy = 6;
        gbc.gridx = 0;
        gbc.gridwidth = 4;
        gbc.weighty = 10;
        gbc.insets = new Insets(6, 13, 6, 13);
        gbc.anchor = GridBagConstraints.NORTH;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        getContentPane().add(descriptText, gbc);

     gbc.gridy = 7;
     gbc.gridx = 2;
     gbc.gridwidth = 1;
     gbc.weighty = 0;
     gbc.anchor = GridBagConstraints.CENTER;
     gbc.insets = new Insets(13,13,13,13);
     getContentPane().add(((YoYo.pcp) ? bok : bca),gbc);
     gbc.gridx = 3;
     getContentPane().add(((YoYo.pcp) ? bca : bok),gbc);

    pack();

  }
	
  protected boolean valid_inputs() {
    if (nameText.getText().trim().equals("")) {
	JMessageDialog m = new JMessageDialog(app, (StarLogoFrame)parent, app.textBundle.getString("warning_label"), 
					      app.textBundle.getString("must_specify_name"));
      m.show();
      return false;
    }
    try {
      min = new Integer(minText.getText()).intValue();
      max = new Integer(maxText.getText()).intValue();
      if (min >= max)
	  {JMessageDialog m = new JMessageDialog(app, (StarLogoFrame)parent, app.textBundle.getString("warning_label"),
						 app.textBundle.getString("max_must_be_greater_than_min"));
	 m.show();
	 return false;}
      cur = new Integer(curText.getText()).intValue();
    } catch(NumberFormatException e) {
	JMessageDialog m = new JMessageDialog(app, (StarLogoFrame)parent, app.textBundle.getString("warning_label"),
					      app.textBundle.getString("min_max_current_must_be_integers"));
      m.show();
      return false;
    }
    if (cur < min) cur = min;
    if (cur > max) cur = max;

    if (min<MIN_SLIDER_VALUE) {
	JMessageDialog m = new JMessageDialog(app, (StarLogoFrame)parent, app.textBundle.getString("warning_label"),
					      app.textBundle.getString("limit_cannot_be_less_than") +
					      " " + MIN_SLIDER_VALUE);
      m.show();
      return false;
    }
    if (max>MAX_SLIDER_VALUE) {
	JMessageDialog m = new JMessageDialog(app, (StarLogoFrame)parent, app.textBundle.getString("warning_label"),
					      app.textBundle.getString("limit_cannot_be_more_than") +
					      " " + MAX_SLIDER_VALUE);
      m.show();
      return false;
    }
    String varname = varText.getText().trim();
    if (varname.equals("")) {
	JMessageDialog m = new JMessageDialog(app, (StarLogoFrame)parent,  app.textBundle.getString("warning_label"),
					      app.textBundle.getString("must_specify_variable")); 
      m.show();
      return false;
    } else if (varname.indexOf(" ")!=-1) {
	JMessageDialog m = new JMessageDialog(app, (StarLogoFrame)parent, app.textBundle.getString("warning_label"),
					      app.textBundle.getString("cannot_contain_spaces"));
      m.show();
      return false;
    }			
    return true;
  }
	
  protected void setupPosition() {
      //    setSize(360, (YoYo.solarisp) ? 200 : (YoYo.linuxp) ? 290 : (YoYo.pcp) ? 200: 170);
    setSize(360, (YoYo.solarisp) ? 300 : (YoYo.linuxp) ? 360 : (YoYo.pcp) ? 300 : 255);
    initLocation();
  }
	
  // requires <wd> instanceof SliderWidget, <parent> instancof GUITop
  public void getValues(WidgetDrag wd) {
    SliderWidget sd = (SliderWidget)wd;
    if (!var.equals(varText.getText().trim())) {
      var = varText.getText().trim();
      ((InterfaceFrame)parent).getApp().rancompilerp=false;
      ((InterfaceFrame)parent).getApp().rancompilerpForVariables = false;
    }
    sd.name = nameText.getText();
    sd.var = var;
    sd.min = min;
    sd.max = max;
    sd.cur = cur;
    sd.curInt = new Integer(cur);
    sd.showName = nameCheck.isSelected();
    sd.updateThumbPos();
    sd.update();

    if (!(ToolTipWidget.empty(descriptText.getText())))
	sd.setToolTip(new ToolTipWidget(sd.parent, descriptText.getText()));
    else sd.removeToolTip();
  }

  public boolean shortcuts = true;

  public void focusGained(FocusEvent e) {
	shortcuts = false;
  }

  public void focusLost(FocusEvent e) {
	shortcuts = true;
  }

  public boolean shortcutsOn() { return shortcuts; }

}











