/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.awt.*;
import java.awt.event.*;
import yoyo.YoYo;
import javax.swing.*;

public class JTextWidgetDialog extends JWidgetDialog implements TextWidgetDialog {

    private static JTextWidgetDialog dialog;

    private String words;
    private static JTextArea tester;
    
    static {
	try {
	    tester = new JTextArea(" ");
	} catch (Exception e) { e.printStackTrace(); }
    }
    
    private JScrollPane wordPanel;  
    private JLabel wordsLab;
    private JPanel butPanel;
    //try adding a third panel to get the "Text" label/direction thing not too big
    private JPanel dirPanel;
    JCheckBox borderCheck;


    private JTextWidgetDialog(Frame parent, Application app, String title) {
	super(parent, app, title);
	init();
	initFocus(tester);
    }

    //for editing textwidgets, requires <wd> instance of TextWidget
    public static JTextWidgetDialog create(Frame parent, Application app, String title, WidgetDrag wd) {
	TextWidget tw = (TextWidget)wd;
	if (dialog==null) {
	    dialog = new JTextWidgetDialog(parent, app, title);
	} else {
	    dialog.setTitle(title);
	}
	dialog.addNotify();
	dialog.setFields(tw.words, tw.drawBorder);
	return dialog;
    }

    //dialog for creating new textwidgets
    public static JTextWidgetDialog create(Frame parent, Application app, String title) {
	String words = " ";
	if (dialog == null) {
	    dialog = new JTextWidgetDialog(parent, app, title);
	} else {
	    dialog.setTitle(title);
	}
	dialog.addNotify();
	dialog.setFields(words, false);
	return dialog;
    }

    private void setFields(String words, boolean border) {
	this.words = words;
	borderCheck.setSelected(border);
	tester.setText(words);
	tester.setCaretPosition(0);
	//	tester.selectAll();
 
    }

    protected void setupComponents() {
	//try adding a third panel to get the "Text" label/direction thing not too big
	wordsLab = new JLabel(app.textBundle.getString("enter_text_label"));
	tester.setLineWrap(true);
	tester.setWrapStyleWord(true);
	wordPanel = new JScrollPane(tester);
	wordPanel.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
	wordPanel.setPreferredSize(new Dimension(50, 50));
			
	borderCheck = new JCheckBox(app.textBundle.getString("legend_border_check"));
	butPanel = new JPanel(); 
	butPanel.setLayout(new GridLayout(1, 2, 5,0));
	butPanel.add(borderCheck);
	if (YoYo.pcp) {
	    butPanel.add(bok);
	    butPanel.add(bca);
	} else {
	    butPanel.add(bca);
	    butPanel.add(bok);
	}

	if (!YoYo.macosxp) {
	    formatLabel(wordsLab);
	    formatLabel(borderCheck);
	    formatLabel(bok);
	    formatLabel(bca);
	}
    }
    
    protected void setupGBC() {

	GridBagLayout gbl = new GridBagLayout();
	getContentPane().setLayout(gbl);
	GridBagConstraints gbc = new GridBagConstraints();
    
	gbc.anchor = GridBagConstraints.WEST;
	gbc.fill = GridBagConstraints.HORIZONTAL;
	gbc.weightx = 100;
	gbc.weighty = 100;

	gbc.gridx = 0;
	gbc.gridy = 0;
	gbc.gridwidth = 2;
	gbc.gridheight = 1;
	gbc.weighty = 0;
	gbc.fill = GridBagConstraints.BOTH;
	gbc.insets = new Insets(6, 13, 0, 5);
	getContentPane().add(wordsLab, gbc);
	
	gbc.gridx = 0;
	gbc.gridy = 1;
	gbc.gridwidth = 5;
	gbc.gridheight =1;
	gbc.weighty = 100;
	gbc.insets = new Insets(6, 13, 0, 13);
	gbc.fill = GridBagConstraints.BOTH;
	gbc.anchor = GridBagConstraints.EAST;
	getContentPane().add(wordPanel, gbc);	    
	
	gbc.gridx = 0;
	gbc.gridy = 2;
	gbc.gridwidth = 1;
	gbc.insets = new Insets(13, 13, 13, 0);
	gbc.fill = GridBagConstraints.NONE;
	gbc.anchor = GridBagConstraints.WEST;
	getContentPane().add(borderCheck,gbc);

	gbc.gridx = 0; 
	gbc.gridy = 2;
	gbc.gridwidth = 2;
	gbc.gridheight = 1;
	gbc.weighty = 0;
	gbc.insets = new Insets(13,0,13,13);
	gbc.fill = GridBagConstraints.NONE;
	gbc.anchor = GridBagConstraints.SOUTHEAST;
	//   gbc.insets = new Insets(0, 5, 13, 13);
	getContentPane().add(butPanel, gbc);
    }
	
    protected boolean valid_inputs() {
	if (tester.getText().trim().equals("")) {
	    String warning = app.textBundle.getString("warning_label");
	    String noTextEntered = app.textBundle.getString("no_text_entered");
	    JMessageDialog m = new JMessageDialog(app, (StarLogoFrame)parent, warning, noTextEntered);
	    m.show();
	    return false;
	}
	//error checking goes here
	return true;
    }

    protected void setupPosition() {
	//use 300 for textfield
	//    setSize(300, 180);
	//use 400 for no dirPanel
	//    setSize(400, 250);
	setSize(350, 170);
	initLocation();
    }

    //overriding this from parent class -- disable key short cuts
    //(that way you can use <return> in your textarea and it won't be interpreted as hitting the ok button)
    protected void initShortcuts() { }


    //requires <wd> instanceof TextWidget, <parent> instanceof GUITop
    public void getValues(WidgetDrag wd) {
	TextWidget tw = (TextWidget)wd;
	tw.words = tester.getText();
	tw.drawBorder = borderCheck.isSelected();
	//for textareas, use the above line. for textfields (older version) use the below line.
	//    tw.words = Utilities.printable(wordsText.getText());
	tw.update();
    }

}
