/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.util.Vector;
import java.util.Observable;
import java.util.Random;
import java.util.Enumeration;
import yoyo.*;
import java.util.Observer;
import java.awt.datatransfer.*;
import java.io.LineNumberReader;
import java.io.PrintWriter;
import java.io.StringReader;
import java.io.IOException;
import java.util.Hashtable;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.undo.*;
import javax.swing.border.LineBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.BevelBorder;
import java.awt.CardLayout;

public class JTurtleMonitor extends JEntityMonitor implements TurtleMonitor {

    static Hashtable tmhash = new Hashtable();
  
    public static final int XCOR = 1, YCOR = 2, HEADING = 3, BREED = 4, SHAPE = 5, COLOR = 6, 
	GRABBED = 7, OWN = 8;
  
    Turtle tur;
    int who;
    int shape;
    String brname;
  
    JLabel who_lbl = new JLabel(app.textBundle.getString("who_label"), JLabel.LEFT);
    JLabel who_txt;
    
    JLabel grabbed_lbl = new JLabel(app.textBundle.getString("grabbed_label"), JLabel.LEFT);
    JLabel grabbed_txt = new JLabel(app.textBundle.getString("false_label"));
    VM grabbedval = null;
    JLabel breed_lbl = new JLabel(app.textBundle.getString("breed_label"), JLabel.LEFT);
    
    JTextField breed_txt = new JTextField();
    double breedval;
    
    JLabel shape_lbl = new JLabel(app.textBundle.getString("turtle_monitor_shape_label"));
    JTextField shape_txt = new JTextField();
    String shapeval;

    JLabel color_lbl = new JLabel(app.textBundle.getString("turtle_color_label"), JLabel.LEFT);
    JTextField color_txt = new JTextField();
    double colorval;

    JLabel xcor_lbl = new JLabel(app.textBundle.getString("xcor_label"), JLabel.LEFT);
    JTextField xcor_txt = new JTextField();
    double xcorval;

    JLabel ycor_lbl = new JLabel(app.textBundle.getString("ycor_label"), JLabel.LEFT);
    JTextField ycor_txt = new JTextField();
    double ycorval;

    JLabel heading_lbl = new JLabel(app.textBundle.getString("turtle_heading_label"), JLabel.LEFT);
    JTextField heading_txt = new JTextField();
    double headingval;
    protected boolean tlocked;
    static double delayTime = 0.5;
    static int numVars;
    public int getNumVars() {
	return numVars;
    }
 
    public JTurtleMonitor(Turtle tur, double delay, Application app)  {
	super(app.textBundle.getString("turtle_number_label") + tur.who, delay, app);
	this.tur = tur;
	//this.app = app;
	this.com = new JSLCommand(SLCommand.TURTLE, this.sl, tur.who, app);
	who = tur.who;
	shape = tur.shape;
	numVars = Turtle.numVars;
	tmhash.put(tur, this);
    }
		
    void setStateFields(Dimension d) {
	who_txt = new JLabel(Integer.toString(tur.who), JLabel.LEFT);
	who_txt.setRequestFocusEnabled(true);
	grabbedval = tur.grabber;
	grabbed_txt = new JLabel((grabbedval == null) ? app.textBundle.getString("false_label") : grabbedval.niceToString(), JLabel.LEFT);
	grabbed_lbl.setText((grabbedval == null) ? app.textBundle.getString("grabbed_label") : app.textBundle.getString("grabbed_by_label"));

	breed_txt.setName(app.textBundle.getString("breed_label"));
	//    breed_txt.setHorizontalAlignment(JTextField.LEFT);
	xcor_txt.setName(app.textBundle.getString("xcor_label"));
	//xcor_txt.setHorizontalAlignment(JTextField.LEFT);
	ycor_txt.setName(app.textBundle.getString("ycor_label"));
	// ycor_txt.setHorizontalAlignment(JTextField.LEFT);
	color_txt.setName(app.textBundle.getString("turtle_color_label"));
	//color_txt.setHorizontalAlignment(JTextField.LEFT);
	heading_txt.setName(app.textBundle.getString("turtle_heading_label"));
	//heading_txt.setHorizontalAlignment(JTextField.LEFT);
	shape_txt.setName(app.textBundle.getString("turtle_monitor_shape_label"));

	who_txt.setPreferredSize(d);
	grabbed_txt.setPreferredSize(d);
	breed_txt.setPreferredSize(d);
	shape_txt.setPreferredSize(d);
	color_txt.setPreferredSize(d);
	xcor_txt.setPreferredSize(d);
	ycor_txt.setPreferredSize(d);
	heading_txt.setPreferredSize(d);
    
	who_lbl.setPreferredSize(d);
	grabbed_lbl.setPreferredSize(d);
	breed_lbl.setPreferredSize(d);
	shape_lbl.setPreferredSize(d);
	color_lbl.setPreferredSize(d);
	xcor_lbl.setPreferredSize(d);
	ycor_lbl.setPreferredSize(d);
	heading_lbl.setPreferredSize(d);
    
	formatText(xcor_txt);
	formatText(ycor_txt);
	formatText(heading_txt);
	formatText(breed_txt);
	formatText(color_txt);
	formatText(shape_txt);
    
	who_lbl.setFont(LABEL_FONT);
	grabbed_lbl.setFont(LABEL_FONT);
	xcor_lbl.setFont(LABEL_FONT);
	ycor_lbl.setFont(LABEL_FONT);
	color_lbl.setFont(LABEL_FONT);
	heading_lbl.setFont(LABEL_FONT);
	breed_lbl.setFont(LABEL_FONT);
	shape_lbl.setFont(LABEL_FONT);

	//shape_txt.setBackground(SystemColor.text);
	//shape_txt.setForeground(SystemColor.textText);
    }

    public String getVarName(int which) {
	return Turtle.getVarName(which);
    }
    public int getVarPos(int which) {
	return Turtle.getVarPos(which);
    }
    public Object getVarValue(int varpos) {
	return tur.turtlesown[varpos];
    }
    public boolean getVarEnump(int which) {
	return Turtle.getVarEnump(which);
    }
	
    // before calling this, synchronize on this turtle monitor
    public void setValues(int which) {
	switch (which) {
	case XCOR :
	    xcorval = tur.xcor;
	    SwingUtilities.invokeLater(new Runnable() {
		    public void run() {
			if (!closep) {
			    xcor_txt.setText(doubString(xcorval - sl.turhalfwidth));
			    //xcor_txt.selectAll();
			    xcor_txt.setCaretPosition(xcor_txt.getText().length());
			}
		    }
		});
	    break;

	case YCOR :
	    ycorval = tur.ycor;
	    SwingUtilities.invokeLater(new Runnable() {
		    public void run() {
			if (!closep) {
			    ycor_txt.setText(doubString(sl.turhalfheight - ycorval));
			    //ycor_txt.selectAll();
			    ycor_txt.setCaretPosition(ycor_txt.getText().length());
			}
		    } 
		});
	    break;

	case HEADING :
	    headingval = tur.heading;
	    SwingUtilities.invokeLater(new Runnable() {
		    public void run() {
			if (!closep) {
			    heading_txt.setText(doubString(headingval));
			    //heading_txt.selectAll();
			    heading_txt.setCaretPosition(heading_txt.getText().length());
			}
		    }
		});
	    break;

	case BREED :
	    breedval = tur.breed;
	    SwingUtilities.invokeLater(new Runnable() {
		    public void run() {
			if (!closep) {
			    breed_txt.setText(breedString(breedval));
			    //breed_txt.selectAll();
			    breed_txt.setCaretPosition(breed_txt.getText().length());
			}
		    }
		});
	    break;

	case SHAPE :
	    //The "+1" adjusts for human counting
	    //Change here into the turtle name
	    //shapeval = (tur.shape + 1);
	    //shapeval = (tur.name);
	    //shapeval = shapes16x16[(tur.shape)].name;
	    shapeval = TurtleShapes.getShape(tur.shape).name;
	    SwingUtilities.invokeLater(new Runnable() {
		    public void run() {
			if (!closep) {
			    shape_txt.setText(shapeval);
			    shape_txt.setCaretPosition(shape_txt.getText().length());
			}
		    }
		});
	    break;

	case COLOR :
	    colorval = getEntityColor();
	    SwingUtilities.invokeLater(new Runnable() {
		    public void run() {
			if (!closep) {
			    color_txt.setText(Colors.colorNumToName(colorval));
			    //color_txt.selectAll();
			    color_txt.setCaretPosition(color_txt.getText().length());
			    wholePanel.setBorder(new LineBorder(Colors.colorarray[(int)(colorval * 32.0)], 4));
			}
		    }
		});
	    //repaint();
	    break;
	case GRABBED :
	    grabbedval = tur.grabber;
	    SwingUtilities.invokeLater(new Runnable() {
		    public void run() {
			if (!closep) {
			    grabbed_txt.setText((grabbedval == null) ? app.textBundle.getString("false_label") : grabbedval.niceToString());
			    grabbed_lbl.setText((grabbedval == null) ? app.textBundle.getString("grabbed_label") : app.textBundle.getString("grabbed_by_label"));

			}
		    }
		});
	    break;
	}
    }

    String breedString(double br) {
	String brname;
	if ((br - (int)br) != 0)
	    return doubString(br);
	for (int i = 0; i < Turtle.breednames.length; i++) {
	    if ((int)br == Turtle.breednums[i])
		return Turtle.breednames[i];
	};
	return doubString(br);
    }


    public void initFrame() {
	// setup frame's appearance

	//int totheight = 8 + numValues*textHeight + ccHeight;
	//System.out.println("num" + numValues + " th"+textHeight+" tot "+totheight);
	if (tur == null || !(tur.alive))
	    return;

	if (YoYo.macintoshp) {
		slmenu = new JSLMenu(this, app);
	    slmenu.setMenuBar(this);
	} 

	numVars = Turtle.numVars;
	//setFont(EM_FONT);
	setFields();
	int tfheight = (own_txt == null ? 0 : own_txt[0].getPreferredSize().height);
	tfheight = 22;
	addMyComponents(tfheight);
	com.callerClearText();
	if (!app.kioskp && YoYo.applet == null) addListeners();
	synchronized(lock) {
	    for (int i=XCOR; i<= GRABBED; i++)
		setValues(i);
	    for (int i=0; i< numVars; i++)
		setOwnInit(i);
	}
	//startThread();
    }

    public JLabel[] getStateLabels() {
	JLabel[] labels = { who_lbl, color_lbl, shape_lbl, xcor_lbl, ycor_lbl, 
			    heading_lbl, breed_lbl, grabbed_lbl };
	return labels;
    }

    public JComponent[] getStateTexts() {
	JComponent[] texts = { who_txt, color_txt, shape_txt, xcor_txt, ycor_txt,
			       heading_txt, breed_txt, grabbed_txt };
	return texts;
    }
    
    public int getNumStateVariables() {
	return 8;
    }

    void addMyComponents(int tfheight) {
	super.addMyComponents(tfheight);

	// dead turtle monitor side
	JLabel lbl = new JLabel(app.textBundle.getString("dead_turtle"));
	lbl.setBackground(Color.black);
	lbl.setForeground(Color.white);
	lbl.setFont(LABEL_FONT);
	lbl.setOpaque(true);
    
	cl = new CardLayout();
	wholePanel.setLayout(cl);
	wholePanel.add(lbl, app.textBundle.getString("dead"));
	wholePanel.add(wholeBox, app.textBundle.getString("alive"));
	cl.show(wholePanel, app.textBundle.getString("alive"));
      
	getContentPane().add(wholePanel);
	addNotify();
	pack();
	who_txt.requestFocus();
    }    

    public double getEntityColor() {
	return tur.color;
    }

    // executed from starlogo compilation thread
    public static void updateOpen() {
	boolean midboxp = (numVars > 0);
	numVars = Turtle.numVars;
	synchronized(tmhash) {
	    for (Enumeration e = tmhash.elements() ; e.hasMoreElements() ;) {
		JTurtleMonitor tm = (JTurtleMonitor)e.nextElement();
		synchronized(tm.lock) {
		    tm.updateOwn(midboxp);
		}
	    }
	}
    }

    void addListeners() {
	super.addListeners();

	who_txt.addFocusListener(tff);
	grabbed_txt.addFocusListener(tff);
    
	xcor_txt.addKeyListener(tka);
	xcor_txt.addFocusListener(tff);
	//xcor_txt.addWindowListener(tff);
    
	ycor_txt.addKeyListener(tka);
	ycor_txt.addFocusListener(tff);
	//ycor_txt.addWindowListener(tff);
    
	heading_txt.addKeyListener(tka);
	heading_txt.addFocusListener(tff);
	//heading_txt.addWindowListener(tff);
    
	breed_txt.addKeyListener(tka);
	breed_txt.addFocusListener(tff);
	//breed_txt.addWindowListener(tff);
    
	shape_txt.addKeyListener(tka);
	shape_txt.addFocusListener(tff);

	color_txt.addKeyListener(tka);
	color_txt.addFocusListener(tff);
	//color_txt.addWindowListener(tff);
    
    }

    public void die() {
	//	thisthr.stop();
	//removeAll();
	closep = true;
	SwingUtilities.invokeLater(new Runnable() {
	    public void run() {
		synchronized(lock) {
		    wholePanel.setBorder(new LineBorder(Color.black, 4));
		    cl.show(wholePanel, app.textBundle.getString("dead"));
		    //pack()
		}
	    }
	    });
    }
  
    public static void flushCommandCenters() {
	synchronized(tmhash) {
	    for(Enumeration e = tmhash.elements(); e.hasMoreElements();) {
		JTurtleMonitor jtm = (JTurtleMonitor)e.nextElement();
		if (!jtm.closep) {
		    synchronized(jtm.lock) {
			jtm.com.flush();
		    }
		}
	    }
	}
    }

    public static void setTurtle(Turtle t) {
	JTurtleMonitor tm = (JTurtleMonitor) tmhash.get(t);
	if (tm != null) {
	    synchronized(tm.lock) {
		tm.tur = t;
		t.hasTMp = true;
		t.tm = tm;
		tm.cl.show(tm.wholePanel, tm.app.textBundle.getString("alive"));
		tm.com.callerClearText();
		//if (YoYo.applet == null) addListeners(); 
		for (int i=XCOR; i<= GRABBED; i++)
		    tm.setValues(i);
		for (int i=0; i< numVars; i++)
		    tm.setOwnInit(i);
		//tm.startThread();
		tm.show();
	    }
	}
    }

    public void registerCloser(Frame c, Application app) {
	c.addWindowListener(new TMCloser(app));
    }

    protected class TMCloser extends Closer {

	public TMCloser(Application app) {
	    super(app);
	}

	public void windowClosing(WindowEvent evt) {
	    synchronized(tmhash) {
		super.windowClosing(evt);
	    }
	}
    }

    public void closeMonitor() {
	if (tur != null) {
	    tmhash.remove(tur);
	    tur.hasTMp = false;
	    tur.tm = null;
	    tur = null;
	    WindowPosition.closed(this);
	    //tmvec.remove(this);
	}
    }

    static {
	new Thread(new Runnable() {
		public void run() {
		    while(true) {
			if (tmhash != null) {
			    synchronized(tmhash) {
				for(Enumeration e = tmhash.elements(); e.hasMoreElements();) {
				    JTurtleMonitor jtm = (JTurtleMonitor)e.nextElement();
				    synchronized(jtm.lock) {
					Turtle tur = jtm.tur;
					if (tur != null && tur.alive && !jtm.closep) {
					    if ( (jtm.breedval != tur.breed) && 
						 ((jtm.breed_txt != jtm.focus) || (jtm.app.activeFrame != jtm)))
						jtm.setValues(BREED);
					    if ( (jtm.colorval != tur.color) && 
						 ((jtm.color_txt != jtm.focus) || (jtm.app.activeFrame != jtm)))
						jtm.setValues(COLOR); 
					    //if (jtm == null) System.out.println("jtm should not be null");
					    //if (jtm.shapeval == null) System.out.println("jtm shapeval should not be null");
					    //if (tur == null) System.out.println("tur should not be null");
					    //if (TurtleShapes.getShape(tur.shape) == null) System.out.println("tur.shape should not be null " + tur.shape);
					    if ( (jtm.shapeval != 
						  TurtleShapes.getShape(tur.shape).name) &&
						 ((jtm.shape_txt != jtm.focus) 
						  || (jtm.app.activeFrame != jtm)))
						jtm.setValues(SHAPE);
					    if ( (jtm.xcorval != tur.xcor) && 
						 ((jtm.xcor_txt != jtm.focus) || (jtm.app.activeFrame != jtm)))
						jtm.setValues(XCOR);
					    if ( (jtm.ycorval != tur.ycor) && 
						 ((jtm.ycor_txt != jtm.focus) || (jtm.app.activeFrame != jtm)))
						jtm.setValues(YCOR);
					    if ( (jtm.headingval != tur.heading) && 
						 ((jtm.heading_txt != jtm.focus) || (jtm.app.activeFrame != jtm)))
						jtm.setValues(HEADING);
					    if ( (jtm.grabbedval != tur.grabber))
						jtm.setValues(GRABBED);
					    try {
						for (int i=0; i< numVars; i++) {
						    if (!(tur.turtlesown[i].equals(jtm.ownval[i]) || ((jtm.own_txt[i] == jtm.focus) && (jtm.app.activeFrame == jtm))))
							jtm.setOwn(i);
						}
					    } catch(NullPointerException ne) {}
					    catch(ArrayIndexOutOfBoundsException ae){}
					}
				    }
				}
			    }
			}
			try {
			    Thread.sleep((long)(delayTime * 1000));
			} catch (InterruptedException e) {}
		    }
		}
	    }).start();
    }
  
    String getCommand(String lbl, String val) {
	if (lbl.equals(xcor_txt.getName())) {
	    lastField = XCOR;
	    return(app.commandsBundle.getString("setx") + " " + val);
	} else if (lbl.equals(ycor_txt.getName())) {
	    lastField = YCOR;
	    return(app.commandsBundle.getString("sety") + " " + val);
	} else if (lbl.equals(heading_txt.getName())) {
	    lastField = HEADING;
	    return(app.commandsBundle.getString("setheading") + " " + val);
	} else if (lbl.equals(shape_txt.getName())) {
	    lastField = SHAPE;
	    return(app.commandsBundle.getString("setshape") + " " + val);
	} else if (lbl.equals(color_txt.getName())) {
	    lastField = COLOR;
	    return(app.commandsBundle.getString("setcolor") + " " + val);
	} else if (lbl.equals(breed_txt.getName())) {
	    lastField = BREED;
	    return(app.commandsBundle.getString("setbreed") + " " + val);
	} else {
	    for (int i=0; i < getNumVars(); i++) {
		if (lbl.equals(own_txt[i].getName())) {
		    lastField = i;
		    lastOwn = true;
		    return(app.commandsBundle.getString("set") + own_lbl[i].getText() + " " + val);
		}
	    }
	    return "internal jturtlemonitor error";
	}
    }

    public String getCompileCommand(String s) {
	return "compile-turtle-monitor [" + s + "]";
    }
  
    public void runCompiledCommand(Object[] ilist, Object[] olist) {
	int[] newarray = castToIntArray(ilist);
	if (sl.locked == true) {
	    displayOneString(app.textBundle.getString("cant_run_turtle_monitor") + " "
			     + app.textBundle.getString("cant_run_while_obscured"));
	}
	else {
	    synchronized(sl.runningp) {
		int whichip = sl.internalSetInstructions("turtle-monitor: " + who_txt.getText(), 
							 newarray, olist);
		StackProcessParent.runTurtle(sl, whichip, tur, false, this);
	    }
	}
    }
  
    // Only called when we are reporting an error.
    public void displayOneString(String s) {
	super.displayOneString(s);
	synchronized(lock) {
	    if (!(breed_txt.getText().equals(breedString(breedval))))
		{setValues(BREED); breed_txt.selectAll();}
	    if (!(color_txt.getText().equals(Colors.colorNumToName(colorval))))
		{setValues(COLOR); color_txt.selectAll();}
	    if (!(shape_txt.getText().equals(shapeval)))
		{setValues(SHAPE); shape_txt.selectAll();}
	    if (!(xcor_txt.getText().equals(doubString(xcorval - sl.turhalfwidth))))
		{setValues(XCOR); xcor_txt.selectAll();}
	    if (!(ycor_txt.getText().equals(doubString(sl.turhalfheight - ycorval))))
		{setValues(YCOR); ycor_txt.selectAll();}
	    if (!(heading_txt.getText().equals(doubString(headingval))))
		{setValues(HEADING); heading_txt.selectAll();}
	    for (int i=0; i< numVars; i++)
		setOwn(i);
	}
    }
	
    public void displayAString(String s) {
	super.displayAString(s);
	synchronized(lock) {
	    if (!(breed_txt.getText().equals(breedString(breedval))))
		setValues(BREED);
	    if (!(color_txt.getText().equals(Colors.colorNumToName(colorval))))
		setValues(COLOR);
	    if (!(shape_txt.getText().equals(shapeval)))
		setValues(SHAPE);
	    if (!(xcor_txt.getText().equals(doubString(xcorval - sl.turhalfwidth))))
		setValues(XCOR);
	    if (!(ycor_txt.getText().equals(doubString(sl.turhalfheight - ycorval))))
		setValues(YCOR);
	    if (!(heading_txt.getText().equals(doubString(headingval))))
		setValues(HEADING);
	    for (int i=0; i< numVars; i++)
		setOwn(i);
	    //setValues(OWN);
	}
    }

    public void run() {}

}

