/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.util.*;
import java.awt.*;

public class LineSet {
    public int id; //LineSet ls = new LineSet(name, inst, id, ei);
    public String name;
    public String inst;
    public ExtraInfo ei;
    public boolean displayed;
    public double starlogocolor;
    public Color color;
    public int row, col; //for sl

    public static final int NUM = 10;
    public static double DELAY = .5;

    public boolean xyOn = false;
    public String instx = null;
    public ExtraInfo eix = null;
    public int type = -1;

    public int order;
    private Application app;


    LineSet(Application app, int id, int order) {
	this.id = id;
	this.order = order;
	this.app = app;
	name = app.textBundle.getString("blank_name") + id;
	inst = "";
	ei = new ExtraInfo(app, id, order);
	displayed = (id == 1) ? true : false;  // in a new line widget, pp1 is turned on
	starlogocolor = (id % 10) * 10 + 5;
	color = Colors.colorarray[(int)(starlogocolor*32.0)];
	xyOn = false;
    }

    LineSet(Application app, int id, int type, int order) {
	this(app, id, order);
	this.type = type;
	if (type == LineWidget.XYPLOT ||
	    type == LineWidget.PHASE) {
	    this.xyOn = true;
	    this.instx = "";
	    this.eix = new ExtraInfo(app,id, order);
	}
    }

    
    LineSet(Application app, 
	    String name,
	    int type,
	    String inst,
	    int id,
	    int order,
	    ExtraInfo ei,
	    boolean displayed) {
	this.app = app;
	this.name = name;
	this.type = type;
	this.inst = inst;
	this.id = id;
	this.order = order;
	this.ei = ei;
	this.displayed = displayed;
	this.xyOn = false;
    }

    LineSet(Application app, 
	    String name,
	    int type,
	    String inst,
	    int id,
	    int order,
	    ExtraInfo ei,
	    int color,
	    boolean displayed) {
	this(app, name, type, inst, id, order, ei, displayed);
	this.color = new Color(color);
    }

    LineSet(Application app, 
	    String name,
	    int type,
	    String inst,
	    int id,
	    int order,
	    ExtraInfo ei,
	    boolean displayed,
	    String instx,
	    ExtraInfo eix)
    {
	this.app = app;
	this.name = name;
	this.type = type;
	this.inst = inst;
	this.id = id;
	this.order = order;
	this.ei = ei;
	this.displayed = displayed;
	this.instx = instx;
	this.eix = eix;
	this.xyOn = true;
    }


    LineSet(Application app, String name,
	    int type,
	    String inst,
	    int id,
	    int order,
	    ExtraInfo ei,
	    int color,
	    boolean displayed,
	    String instx,
	    ExtraInfo eix) {
	this(app, name, type, inst, id, order, ei, displayed, instx, eix);
	this.color = new Color(color);
    }


    public String getCommandToCompile() {
	if (displayed && !inst.equals("")) {
	    String s = "";
	    if (!xyOn) {
		//if (StarLogo.DEBUG) { sl.app.log.println("<> <> <> type: " + type); }
		if (type == LineWidget.HISTOGRAM) {
		    //			    s = "%print-to-line3 " + id + " (" + inst + ") ";
		    s = " " + inst + " ";
		}
		else {
		    //			    s = "%print-to-line " + id + " (" + inst + ") ";
		    //			    s = " setplot " + order + " pp " + id + " plot (" + inst + ") ";
		    s = app.commandsBundle.getString("%ptolemy_plot") + " " + 
			order + " " + id + " (" + inst + ") ";
		}
	    }
	    else {
		if (type == LineWidget.XYPLOT) {
		    s = inst + instx + " ] ";
		    if (ei.onWith) {
			s += " ] ";
		    }
		} else {
		    s = app.commandsBundle.getString("%ptolemy_plotxy") + " " + 
			order + " " + id + " (" + inst + ") (" + instx + ") ";
		}
	    }
	    return s;
	}
	else { return ""; }
    }

    public static Vector loadLineSets(Application app, ArrayList items) {
	Vector sets = new Vector();
	int i = 0;
	int start = 0, end = 0;
	while (i < items.size()) {
	    String item = items.get(i).toString();
	    if (item.equals("lineset-id")) {
		start = i;
		int id = Integer.parseInt(items.get(i+1).toString());
		i++;
		// search for the end
		while(i < items.size()) {
		    String item2 = items.get(i).toString();
		    if (item2.equals("lineset-end") &&
			Integer.parseInt(items.get(i+1).toString()) == id) {
			end = i+2;
			sets.addElement(LineSet.load(app, items, start, end));
			i = end;
			break;
		    } else {
			i++;
		    }
		}
	    } else {
		i++;
	    }
	}
	return sets;
    }

    public static LineSet load(Application app, ArrayList items, int start, int end) {
	int i = start;
	String name = app.textBundle.getString("blank_name"), 
	    instruction = "", xyInstruction = "";
	int id = -1, order = -1, type = -1, color = -1;
	boolean displayedp = false, xyOn = false;
	ExtraInfo ei = null, eix = null;

	while (i < end) {
	    String item = items.get(i).toString();
	    if (item.equals("lineset-id")) {
		String value = items.get(i+1).toString();
		id = Integer.parseInt(value);
		i+=2;
		continue;
	    } 
	    if (item.equals("lineset-name")) {
		name = items.get(i+1).toString();
		i += 2;
		continue;
	    }
	    if (item.equals("lineset-order")) {
		String value = items.get(i+1).toString();
		order = Integer.parseInt(value);
		i+=2;
		continue;
	    } 
	    if (item.equals("lineset-type")) {
		String value = items.get(i+1).toString();
		type = Integer.parseInt(value);
		i+=2;
		continue;
	    } 
	    if (item.equals("lineset-color")) {
		String value = items.get(i+1).toString();
		color = Integer.parseInt(value);
		i+=2;
		continue;
	    } 
	    if (item.equals("lineset-displayed?")) {
		displayedp = items.get(i+1).toString().equals("true");
		i+=2;
		continue;
	    }
	    if (item.equals("lineset-instruction")) {
		instruction = items.get(i+1).toString();
		i+=2;
		continue;
	    }
	    if (item.equals("lineset-xy?")) {
		xyOn = items.get(i+1).toString().equals("true");
		i+=2;
		continue;
	    }
	    if (item.equals("lineset-xy-instruction")) {
		xyInstruction = items.get(i+1).toString();
		i+=2;
		continue;
	    }
	    // no match skip it
	    i+=2;
	}
	ei = ExtraInfo.load(app, id, order, items, start, end);
	if (xyOn) {
	    //System.out.println("xy plot");
	    eix = ExtraInfo.load(app, id, order, items, "xy", start, end);
	    //System.out.println("y extrainfo: " + eix);
	    return new LineSet(app, name, type, instruction, id, order, ei, color, displayedp, xyInstruction, eix);
	} else {
	    return new LineSet(app, name, type, instruction, id, order, ei, color, displayedp);
	}
    }

    public String getSaveString() {
	String s;
	s = "lineset-id " + id + "\r\n" +
	    "lineset-name \"" + Utilities.printable(name) + "\"\r\n" +
	    "lineset-order " + order + "\r\n" +
	    "lineset-type " + type + "\r\n" +
	    "lineset-color " + color.getRGB() + "\r\n" +
	    "lineset-displayed? " + ((displayed) ? "true" : "false") + "\r\n" +
	    "lineset-instruction \"" + Utilities.printable(inst) + "\"\r\n" +
	    ei.getSaveString();
	if (xyOn) {
	    s += "lineset-xy? true\r\n" +
		"lineset-xy-instruction \"" + Utilities.printable(instx) + "\"\r\n" +
		eix.getSaveString("xy");
	}
	s += "lineset-end " + id + "\r\n";
	return s;
    }

    LineSet copy(int id, int order) {
        LineSet c = new LineSet(app, id, order);
        c.name=name;
        c.inst=inst;
        c.ei=(ei == null ? null : ei.copy(app, id, order));
        c.displayed=displayed;
        c.starlogocolor=starlogocolor;
        c.color=color; // Color is immutable
        c.row=row;
        c.col=col;
        c.xyOn=xyOn;
        c.instx=instx;
        c.eix=(eix == null ? null : eix.copy(app, id, order));
        c.type=type;
        return c;
    }

}
