/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.awt.*;
import java.awt.event.*;
import java.io.IOException;
import java.net.*;
import java.text.*;
import java.util.Vector;
import java.util.ArrayList;
import java.util.Observable;
import java.util.Random;
import java.util.Enumeration;
import yoyo.*;
import ptolemy.*;
import starlogo.StarLogo;
import java.awt.image.BufferedImage;

public class LineWidget extends WidgetDrag implements StarLogoCaller, DisplayMessage {

    //  public double delay = 1.0;
    public String delayStr = "1.0";

    public LineChart tbc;
    public Vector sets;

    //Object value = VM.zeroInt;
    //String valuestring = "";

    public int whichip = -1;
    public int onewhichip = -1;
    public StackProcess sp;

    public static Font LINEWIDGET_FONT;
    public Font font;
    public FontMetrics fm;
    NumberFormat nf;
    String insttorun;
    public StarLogo sl;
    public Application app;

    public int id;
    public int order;
    public int type;
    public static int idCount = 2;
    public boolean wizardOn = true; //plots are wizard plots by default
    public boolean deleted = false; //for VM.java

    public static final int MIN_HEIGHT = 70;
    public static final int MIN_WIDTH = 70;

    public static final int BAR = 0;
    public static final int LINE = 1;
    public static final int HISTOGRAM = 2;
    public static final int XYPLOT = 3; //scatterplot
    public static final int PHASE = 4; //xyplot
    public static final int PIE = 5;

    public int plotpen = 1;

    static Image graph;

    static {
    	try {
			LINEWIDGET_FONT = WIDGET_FONT;
			if (YoYo.macintoshp) LINEWIDGET_FONT = Font.getFont("starlogo.linewidget.font", LINEWIDGET_FONT);
    	} catch (Exception e) { e.printStackTrace(); }
    }


    public LineWidget(int id, int order, int type, Vector sets, boolean wizardOn,
		      String title, String xlabel, String ylabel, String delayStr,
		      double xmin, double xmax, double ymin, double ymax, 
		      boolean grid, boolean connected, boolean autoscaleX, 
		      boolean autoscaleY, Rectangle position, 
		      Point pt, JInterface iface, StarLogo sl) {

	this.position = new Rectangle(position);
	this.delayStr = delayStr;
	this.id = id;
	this.parent = iface;
	this.sl = sl;
	this.app = sl.app;
	this.sets = sets;
	this.type = type;
	this.wizardOn = wizardOn;

	this.font = LINEWIDGET_FONT;
	fm = parent.getToolkit().getFontMetrics(font);
	nf = NumberFormat.getInstance();
	nf.setMaximumFractionDigits(0);
	//System.out.println("setting up line widget...type: " + type + " id: " + id);

	this.tbc = new LineChart(iface.app(), this.id, this.id, this.sets, this.type, this);

	if (! (title == null || title.equals("null"))) tbc.setPlotTitle(title);
	if (! (xlabel == null || xlabel.equals("null"))) tbc.setXLabel(xlabel);
	if (! (ylabel == null || ylabel.equals("null"))) tbc.setYLabel(ylabel);
	tbc.setGrid(grid);

	// adding in connected boolean
	//System.out.println("connected = " + connected);
	//if (tbc.l2 instanceof Plot) {
	if (type != HISTOGRAM) {
	    //((Plot)tbc.l2).setConnected(connected);
	    tbc.setConnected(connected);
	}
	tbc.autoscaleX = autoscaleX;
	tbc.autoscaleY = autoscaleY;

	tbc.setXRange(xmin, xmax);
	tbc.setYRange(ymin, ymax);

	tbc.setLocation(pt);

	this.tbc.setWidgetSize(30 +
			       this.position.width,
			       40 + this.position.height);
	iface.repaint(position);

	int count = 0, max = 0;

	for (int i =0; i<sets.size(); i++) {
	    LineSet ls = (LineSet)this.sets.elementAt(i);
	    tbc.setColor(ls.id % 10, ls.color);
	    if (ls.displayed) {count++; max=i;}
	}
	
	if (this.type == LineWidget.BAR) {
	    //System.out.println("setting persistence...");
	    tbc.setPointsPersistence(count);
	    tbc.setPointsPersistence(count);
	    tbc.explicitLineXRange(.5, 1.5 + max);
	    tbc.explicitWidgetXRange(.5, 1.5 + max);

	    int counter = 1;
	    for (Enumeration e = sets.elements(); e.hasMoreElements();) {
		LineSet ls = (LineSet)e.nextElement();
		if (ls.displayed) {
		    char c1 = ' ', c2 = '-';
		    this.tbc.addXTick(ls.name.replace(c1,c2), counter++);
		}
	    }
	} else if (this.type == LineWidget.HISTOGRAM) {
	    tbc.setHistoNum(count);
	} else {
	    tbc.setLineNum(count);
	}

	//try {
	//System.out.println("~~~" + this.getSaveString(false));
	//}
	//catch (Exception e) { }

	if (!this.wizardOn) {
	    tbc.disableDataInFormatMenu();
	}
	parent.addPlotWindow(tbc);
	//addObserver(parent);
	//app.procWin.addObserver(this);
    }

    public LineWidget(int id, int order, int type, Vector sets, boolean wizardOn,
		      String title, String xlabel, String ylabel, String delayStr,
		      double xmin, double xmax, double ymin, double ymax, boolean grid, 
		      boolean connected, boolean autoscaleX, boolean autoscaleY,
		      Rectangle position, Point pt, JInterface iface, StarLogo sl,
		      int whichip, int onewhichip) {
	this(id, order, type, sets, wizardOn,
	     title, xlabel, ylabel, delayStr,
	     xmin, xmax, ymin, ymax,
	     grid, connected, autoscaleX, autoscaleY, 
	     position, pt, iface, sl);
	this.whichip = whichip;
	this.onewhichip = onewhichip;
    }

    public LineWidget(JInterface iface, StarLogo sl) {

	Dimension size = iface.getSize();
	Rectangle outline = iface.outline();

	if (iface.outline().x + MIN_WIDTH > iface.getSize().width) {
	    iface.outline().width = iface.outline().width + iface.outline().x -
		iface.getSize().width + MIN_WIDTH;
	    iface.outline().x=iface.getSize().width-MIN_WIDTH;
	}
	if (iface.outline().width<MIN_WIDTH)
	    iface.outline().width = MIN_WIDTH;

	if (iface.outline().y+MIN_HEIGHT>iface.getSize().height) {
	    iface.outline().height=iface.outline().height+iface.outline().y-
		iface.getSize().height+MIN_HEIGHT;
	    iface.outline().y=iface.getSize().height-MIN_HEIGHT;
	}
	if (iface.outline().height<MIN_HEIGHT)
	    iface.outline().height = MIN_HEIGHT;

	this.position = new Rectangle(outline);
	this.parent = iface;
	this.sl = sl;
	this.app = sl.app;
	//this.wizardOn = true;
	this.font = LINEWIDGET_FONT;
	fm = parent.getToolkit().getFontMetrics(font);
	nf = NumberFormat.getInstance();
	nf.setMaximumFractionDigits(0);
	//addObserver(parent);
	//app.procWin.addObserver(this);
    }

    public LineWidget(Rectangle outline, JInterface iface, StarLogo sl) {
	this.position = new Rectangle(outline);
	this.parent = iface;
	this.sl = sl;
	this.app = sl.app;
	this.font = LINEWIDGET_FONT;
	fm = parent.getToolkit().getFontMetrics(font);
	nf = NumberFormat.getInstance();
	nf.setMaximumFractionDigits(0);

    }

    public LineWidget() {
	//default for retrofit
    }

    public LineWidget(JInterface iface, StarLogo sl, boolean dummy) {
	this.parent = iface;
	this.sl = sl;
	this.app = sl.app;
	this.font = LINEWIDGET_FONT;
	fm = parent.getToolkit().getFontMetrics(font);
	nf = NumberFormat.getInstance();
	nf.setMaximumFractionDigits(0);
	//addObserver(parent);
	//app.procWin.addObserver(this);
    }


    public static LineWidget load(ArrayList items, JInterface iface, StarLogo sl, boolean htmlp) {
	Application app = sl.app;

	int top = 0, left = 0, width = 50, height = 50, mid = 1;
	int type = LineWidget.LINE;
	int whichip = -1, onewhichip = -1;
	int order = 0;
	int i = 0;

	int xloc=0,yloc=0;
	double xmin=0.0,ymin=0.0,xmax=0.0,ymax=0.0;
	boolean grid = false, connected = true, wizardOn = true, 
	    autoscaleX = true, autoscaleY = true;
	String title= app.textBundle.getString("untitled"), 
	    xlabel=app.textBundle.getString("x_axis"), 
	    ylabel=app.textBundle.getString("y_axis"), 
	    inst="", delayStr="1.0";

	while (i<items.size()) {
	    String item = items.get(i).toString();
	    if (item.equals("top-left")) {
		String value = items.get(i+1).toString();
		String value2 = items.get(i+2).toString();
		top = Integer.parseInt(value);
		left = Integer.parseInt(value2);
		i += 3;
		//System.out.println("top: " + top + " left: " + left);
		continue;
	    }
	    if (item.equals("width-height")) {
		String value = items.get(i+1).toString();
		String value2 = items.get(i+2).toString();
		width = Integer.parseInt(value);
		height = Integer.parseInt(value2);
		//System.out.println("width: " + width + " height: " + height);
		i += 3;
		continue;
	    }
	    if (item.equals("id")) {
		String value = items.get(i+1).toString();
		mid = Integer.parseInt(value);
		//System.out.println("read in id number: " + mid);
		i += 2;
		continue;
	    }
	    if (item.equals("order")) {
		order = Integer.parseInt(items.get(i+1).toString());
		i+=2;
		continue;
	    }
	    if (item.equals("type")) {
		type = Integer.parseInt(items.get(i+1).toString());
		i+=2;
		continue;
	    }
	    if (item.equals("title")) {
		title = items.get(i+1).toString();
		i+=2;
		continue;
	    }
	    if (item.equals("delay-string")) {
		delayStr = items.get(i+1).toString();
		i+=2;
		continue;
	    }
	    if (item.equals("ymin"))  { 
		ymin = Double.valueOf(items.get(i+1).toString()).doubleValue();
		i+=2;
		continue;
	    }
	    if (item.equals("ymax"))  { 
		ymax = Double.valueOf(items.get(i+1).toString()).doubleValue();
		i+=2;
		continue;
	    }
	    if (item.equals("xmin"))  { 
		xmin = Double.valueOf(items.get(i+1).toString()).doubleValue();
		i+=2;
		continue;
	    }
	    if (item.equals("xmax"))  {
		xmax = Double.valueOf(items.get(i+1).toString()).doubleValue();
		i+=2;
		continue;
	    }
	    if (item.equals("xlabel")) {
		xlabel = items.get(i+1).toString();
		i+=2;
		continue;
	    }
	    if (item.equals("ylabel")) {
		ylabel = items.get(i+1).toString();
		i+=2;
		continue;
	    }
	    if (item.equals("grid")) {
		grid = items.get(i+1).toString().equals("true");
		i+=2;
		continue;
	    }
	    // adding in connected boolean
	    if (item.equals("connected")) {
	        connected = items.get(i+1).toString().equals("true");
	        i+=2;
	        continue;
	    }
	    if (item.equals("autoscale")) {
	        autoscaleX = items.get(i+1).toString().equals("true");
		autoscaleY = autoscaleX;
	        i+=2;
	        continue;
	    }
	    if (item.equals("autoscale-x")) {
	        autoscaleX = items.get(i+1).toString().equals("true");
	        i+=2;
	        continue;
	    }
	    if (item.equals("autoscale-y")) {
	        autoscaleY = items.get(i+1).toString().equals("true");
	        i+=2;
	        continue;
	    }
	    if (item.equals("location")) {
		xloc = Integer.parseInt(items.get(i+1).toString());
		yloc = Integer.parseInt(items.get(i+2).toString());
		i+=3;
		continue;
	    }
	    if (htmlp) {
		if (item.equals("whichip")) {
		    String value = items.get(i+1).toString();
		    whichip = Integer.parseInt(value);
		    i += 2;
		    continue;
		}
		if (item.equals("onewhichip")) {
		    String value = items.get(i+1).toString();
		    onewhichip = Integer.parseInt(value);
		    i += 2;
		    continue;
		}
	    }

	    if (item.equals("wizard-on")) {
		String value = items.get(i+1).toString();
		wizardOn = value.equals("true");
		i+=2;
		continue;
	    }
	    i+=2;
	}
	Vector sets = null;
	if (wizardOn) {
	    sets = LineSet.loadLineSets(sl.app, items);
	} else {
	    sets = new Vector();
	}

	Rectangle bounds = iface.getBoundaries();
	Rectangle position = new Rectangle(left + bounds.x, top + bounds.y, width, height);
	Point pt = new Point(xloc, yloc);

	if (mid >= idCount) idCount = mid + 1; //LineSet.NUM;

	if (htmlp) {
	    return new LineWidget(mid, order, type, sets, wizardOn,
				  title, xlabel, ylabel, delayStr, xmin, xmax, 
				  ymin, ymax, grid, connected, autoscaleX,
				  autoscaleY,
				  position, pt, iface, sl, whichip, onewhichip);
	}
	else {
	    return new LineWidget(mid, order, type, sets, wizardOn,
				  title, xlabel, ylabel, delayStr, xmin, xmax, 
				  ymin, ymax, grid, connected, autoscaleX,
				  autoscaleY,
				  position, pt, iface, sl);
	}
    }

    public void addLegend(int dataset, String name) {
	tbc.addLegend(dataset, name);
    }

    public int getId() {
	return this.id;
    }
    
    public String getLineSetString() {
	StringBuffer sb = new StringBuffer();
	if (wizardOn) {
	    sb.append("wizard-on true\r\n");
	    for (int i=0; i<sets.size(); i++) {
		sb.append(((LineSet)sets.elementAt(i)).getSaveString());
	    }
	}
	else {
	    sb.append("wizard-on false\r\n");
	}
	return sb.toString();
    }
    
    /** Saves a line widget to disk
     */
    public String getSaveString(boolean htmlp) throws IOException {
	Rectangle bounds = parent.getBoundaries();
	StringBuffer sb = new StringBuffer();
	sb.append("SLLineWidget\r\n");
	sb.append("top-left " + (position.y - bounds.y) + " " + (position.x - bounds.x) + "\r\n");
	sb.append("width-height " + position.width + " " + position.height + "\r\n");
	sb.append("id " + id + "\r\n");
	sb.append("order " + id + "\r\n");
	sb.append("type " + type + "\r\n");
	sb.append("title \"" + Utilities.printable(tbc.getTitle()) + "\"\r\n");
	sb.append("delay-string \"" + Utilities.printable(delayStr) + "\"\r\n");
	sb.append("ymin " + tbc.getYMin() + "\r\n");
	sb.append("ymax " + tbc.getYMax() + "\r\n");
	sb.append("xmin " + tbc.getXMin() + "\r\n");
	sb.append("xmax " + tbc.getXMax() + "\r\n");
	sb.append("xlabel \"" + ((tbc.getFrameXLabel() == null) ? 
				 "" : Utilities.printable(tbc.getFrameXLabel())) + "\"\r\n");
	sb.append("ylabel \"" + ((tbc.getFrameYLabel() == null) ? 
				 "" : Utilities.printable(tbc.getFrameYLabel())) + "\"\r\n");
	sb.append("grid " + ((tbc.getGrid()) ? "true" : "false") + "\r\n");
	sb.append((type != HISTOGRAM
		   ? ("connected " + (tbc.getConnected() ? "true" : "false") + "\r\n")
		   : ""));
	sb.append("autoscale " + ((tbc.autoscaleX || tbc.autoscaleY) ? "true" : "false") + "\r\n");
	sb.append("autoscale-x " + ((tbc.autoscaleX) ? "true" : "false") + "\r\n");
	sb.append("autoscale-y " + ((tbc.autoscaleY) ? "true" : "false") + "\r\n");
	sb.append("location " + (tbc.getLocation().x) + " " + (tbc.getLocation().y) + "\r\n");
	sb.append(((htmlp && whichip >= 0 && onewhichip >= 0) ?
		   "whichip " + whichip + "\r\n" +
		   "onewhichip " + onewhichip + "\r\n" : ""));
	sb.append(getLineSetString() + "\r\n");
	return sb.toString();
    }


    static public void createNew(JInterface iface) {
	LineDialog ld = Application.createLineDialog((Frame)iface.frame(), iface.app(), iface.app().textBundle.getString("new_plot_label"));
	ld.show();
	if (ld.pressedOK()) {
	    LineWidget lw = new LineWidget(iface, iface.sl());
	    ld.getValues(lw);
	    iface.addWidget(lw);
	    lw.id = LineWidget.idCount;
	    lw.order = LineWidget.idCount; // / LineSet.NUM; //iface.app.linecharts.size();
	    LineWidget.idCount = LineWidget.idCount + 1; // + LineSet.NUM;
	    lw.setupLineWidget(iface);
	}//end if (od)
    }

    // from menu item
    static public void createNew(JInterface iface, SLMenu menu) {
	LineDialog ld = Application.createLineDialog((Frame)iface.frame(), iface.app(), iface.app().textBundle.getString("new_plot_label"));

	ld.show();
	if (ld.pressedOK()) {
	    LineWidget lw = new LineWidget(iface, iface.sl(), true);
	    int j = 3;
	    lw.position = new Rectangle(52 + j, iface.getSize().height - lw.MIN_HEIGHT - j - 24,
					lw.MIN_WIDTH, lw.MIN_HEIGHT);
	    ld.getValues(lw);
	    iface.addWidget(lw);
	    lw.id = LineWidget.idCount;
	    lw.order = LineWidget.idCount; // / LineSet.NUM; //iface.app.linecharts.size();
	    LineWidget.idCount = LineWidget.idCount + 1; // + LineSet.NUM;
	    lw.setupLineWidget(iface);
	    iface.repaint(lw.position);
	}//end if (od)
    }
    
    //retrofit method for OLD visualization
    static public LineWidget createNew(int itype, JInterface iface) {
	LineWidget lw = new LineWidget(iface, iface.sl(), true);
	int j = 3;
	lw.position = new Rectangle(52 + j, iface.getSize().height - lw.MIN_HEIGHT - j - 24,
				    lw.MIN_WIDTH, lw.MIN_HEIGHT);
	if (iface.clipRect() == null) iface.setClipRect(new Rectangle());
	iface.addWidget(lw);
	lw.id = 1; //LineWidget.idCount;
	lw.order = 1; //LineWidget.idCount; // / LineSet.NUM;
	lw.wizardOn = false;
	//System.out.println("old create new");
	//LineWidget.idCount = LineWidget.idCount + 1; //LineSet.NUM;
	lw.type = LineWidget.LINE;
	lw.setupLineWidget(iface);
	iface.repaint(lw.position);
	return lw;
    }

    public void setupLineWidget(JInterface iface) {
	sets = new Vector(LineSet.NUM);
	for (int i=1; i<=LineSet.NUM; i++) {
	    sets.addElement(new LineSet(app, i, type, id));
	}
	tbc = new LineChart(app, id, id, sets, type, this);
	tbc.autoscaleX = true;
	tbc.autoscaleY = true;
	if (!wizardOn) {
	    tbc.disableDataInFormatMenu();
	}
	show();

	tbc.setWidgetSize(30 + position.width, 
			  40 + position.height); //30 + MIN_WIDTH, 40 + MIN_HEIGHT);
	tbc.validate();
	tbc.repaintWidget();
	//System.out.println("adding to slmenu...");
	iface.app().rancompilerp = false;
	iface.addPlotWindow(tbc);
    }



    public void resize(Point where, int resizeCorner) {
	super.resize(where, resizeCorner);
	tbc.setWidgetSize(30 + position.width,
			  40 + position.height);
      
    }

    public void mouseDoubleClicked(MouseEvent e) {
	if (!app.kioskp && YoYo.applet == null) tbc.setVisible(true);
    }

    public void edit() {
	parent.addHandlesToClipRect(this);
	show();
	super.edit();
    }

    public void show() {
	if (!app.kioskp && YoYo.applet == null) tbc.setVisible(true);
    }

    public void reset() {
	tbc.reset();
	parent.repaint(position);
    }

    public void erasePoint(int who) {
	if (type == LineWidget.HISTOGRAM) {
	    tbc.removeHistoHashPoint(who);
	}
	if (type == LineWidget.XYPLOT) {
	    tbc.removeLiveHashPoint(who);
	}
	tbc.repaintWithInterface();
    }

    public void delete() {
	tbc.setVisible(false);
	parent.linewidgets().removeElement(this);
	parent.removePlotWindow(tbc);
	super.delete();
	deleted = true;
	//app.procWin.deleteObserver(this);
    }

    public String getCommandToCompile(String s) {
	String cPrefix = "compile-line-command " + id + " ";
	String prefix = "[ "  + app.commandsBundle.getString("loop") + " [";
	String middle = " " + app.commandsBundle.getString("wait") + " ";
	String suffix = " ] ] ";

	String oneprefix = " [";
	String onesuffix = " ]";

	String str;
	if (s.trim().equals("")) {
	    return s.trim();
	}
	else {
	    if (type == LineWidget.BAR) {
		str = cPrefix + prefix + s +
		    middle + delayStr + suffix +
		    oneprefix + s + onesuffix;
	    }
	    else if ((type == LineWidget.XYPLOT) || (type == LineWidget.HISTOGRAM))  {
		str = cPrefix + prefix + s + middle + delayStr + suffix + oneprefix + s + onesuffix;
	    }
	    else {
		str = cPrefix + prefix +
		    s + middle + delayStr + suffix + oneprefix + s + onesuffix;
	    }
	    return str;
	}
    }

    private String stringBufferDelete(String s, int start, int finish) {
	// simulates that java 1.2 stringbuffer function delete
	// inputs and outputs a string instead of stringbuffer though
	StringBuffer sb = new StringBuffer();
	for (int n = 0; n < s.length(); n++)
	    if (n < start || n >= finish) sb.append(s.charAt(n));
	return sb.toString();
    }

    public String removeLoop(String s) {
	StringBuffer sb;
	String word = app.commandsBundle.getString("loop") + " [";
	for (int i=0; i<s.length(); i++) {
	    if (s.indexOf(word, i) != -1) {
		int j = s.indexOf(word, i);
		//System.out.println("i: " + i);
		//sb = new StringBuffer(s);
		//sb.delete(j, j+word.length());
		//s = sb.toString();
		/* java 1.1 string buffers don't have a delete function or anything similar */
		/* simulate the behavior */
		s = stringBufferDelete(s, j, j+word.length());
		/**/
		//System.out.println("s: " + s);
		j = s.indexOf("]", i);
		//sb = new StringBuffer(s);
		//sb.delete(j, j+1);
		//s = sb.toString();
		/* java 1.1 string buffers don't have a delete function or anything similar */
		/* simulate the behavior */
		s = stringBufferDelete(s, j, j+word.length());
		/**/
		//System.out.println("s: " + s);
		i = j + 1;
	    }
	}
	return s;
    }

    public static String getType(Application app, int type) {
 	switch (type) {
 	case BAR: return app.textBundle.getString("bar_chart"); 
 	case LINE: return app.textBundle.getString("line_chart"); 
 	case HISTOGRAM: return app.textBundle.getString("histogram"); 
 	case XYPLOT: return app.textBundle.getString("scatter_plot");
 	case PHASE: return app.textBundle.getString("xy_plot");
 	}
 	return null;
     }


    public void compileCommand() {
	//System.out.println("compile command: " + wizardOn);
	if (wizardOn) {
	    whichip = -1; onewhichip = -1;
	    synchronized(sl.runningp) {
		sl.stopAllStarLogo();
	    }
	    StringBuffer sb = new StringBuffer();
	    for (int i=0; i<sets.size(); i++) {
		sb.append( ((LineSet)sets.elementAt(i)).getCommandToCompile() );
	    }
	    String bufferToCompile = sb.toString().trim(); 
	    
	    //System.out.println(">>>" + ctc + "<<<");
	    if (!bufferToCompile.equals("")) {
		String ctc = getCommandToCompile(bufferToCompile);
		Object[] listtocompile = Reader.read(ctc);
		Object[] newlist = new Object[listtocompile.length + 1];
		System.arraycopy(listtocompile, 0, newlist, 0, listtocompile.length);
		newlist[newlist.length -1] = this;
		if (StarLogo.DEBUG) {
		    System.out.println("Trying to compile lineWidget: " + this + " Line: " + ctc);
		}
		app.cc.enqueue(newlist, app.message, app.err, this);
	    }
	}
    }
    public void setCompiledCommand(Object[] ilist, Object[] olist,
				   Object[] ilist2, Object[] olist2) {
	if (StarLogo.DEBUG) {
	    System.out.println("Finished compiling line widget: " + this);
	}
	int[] newarray = new int[ilist.length];
	int[] newarray2 = new int[ilist2.length];
	for(int i = 0; i<ilist.length; i++) {
	    newarray[i] = ((Number)ilist[i]).intValue();
	}
	for(int i = 0; i<ilist2.length; i++) {
	    newarray2[i] = ((Number)ilist2[i]).intValue();
	}
	if (StarLogo.DEBUG) {
	    System.out.println("line widget ilist: " + YoYo.printToString(ilist));
	    System.out.println("line widget olist: " + YoYo.printToString(olist));
	    System.out.println("line widget ilist2: " + YoYo.printToString(ilist2));
	    System.out.println("line widget olist2: " + YoYo.printToString(olist2));
	}
	whichip = sl.internalSetInstructions(toString(), newarray, olist);
	onewhichip = sl.internalSetInstructions(toString(), newarray2, olist2);
	if (StarLogo.DEBUG) {
	    app.log.println("line widget whichip = " + whichip + " onewhichip " + onewhichip);
	}
	synchronized(sl.runningp) {
	    if (onewhichip >= 0) StackProcessParent.runMonitor(sl, onewhichip, false, this);
	}
	sp = null;
    }

    public String toString() {
	return ("Line Widget: " + id);
    }

    static Color wizardBlue = new Color(0, 153, 255);

    Image widgetImage;

    public void paint(Graphics g) {
	//if (StarLogo.DEBUG) { app.log.println("repainting... ID: " + this.id);}

	g.translate(position.x, position.y);

	/* Draw background of widget */
	//g.setColor(SystemColor.control);
	//g.fill3DRect(0, 0, position.width, position.height, true);

	/* Draw outline: blue for wizard, gray for non-wizard */
	if (wizardOn) {
	    g.setColor(wizardBlue);
	} else {
	    g.setColor(SystemColor.control);
	}

	/* Give it a nice beveled edge */
	g.fill3DRect(0, 0, position.width, position.height, true);

	// why do we need to set its size over and over again?
	//this.tbc.setWidgetSize(30 + position.width, 32 + position.height);
	
	Dimension widgetSize = tbc.getWidgetSize();
	if (widgetImage == null ||
	    widgetImage.getWidth(parent) != widgetSize.width ||
	    widgetImage.getHeight(parent) != widgetSize.height) {
	    widgetImage = parent.createImage(widgetSize.width, widgetSize.height);
	    //parent.getGraphicsConfiguration().createCompatibleImage(widgetSize.width, widgetSize.height);
	}

	Graphics gr = widgetImage.getGraphics();
	this.tbc.paintWidget(gr);
	gr.dispose();

	g.drawImage(widgetImage, 5, 10, position.width - 5, position.height - 5,
		    10, 35, -1 + position.width, 8 + position.height, parent);
	
	/* Now draw the title */
	g.setColor(Color.black);
	g.setFont(font);
	
	if (fm.stringWidth(tbc.getPreTitle() + tbc.getTitle()) > (position.width - 6)) {
	    // draw a short title, cuz it's not going to fit
	    g.drawString(tbc.getPreTitle() + app.textBundle.getString("plot"), 3, 9);
	}
	else g.drawString(tbc.getPreTitle() + tbc.getTitle(), 3, 9);

	/* And the axis labels */
	g.drawString(nf.format(tbc.getWidgetYRange()[1]), 8, 20);
	g.drawString(nf.format(tbc.getWidgetYRange()[0]), 8, position.height - 18);
	if (this.type == LineWidget.LINE ||
	    this.type == LineWidget.PHASE ||
	    this.type == LineWidget.XYPLOT) {
	    double dmin = tbc.getWidgetXRange()[1];
	    int offset = (int)( Math.log(dmin) / Math.log(Math.E) );
	    g.drawString(nf.format(dmin), position.width - 12 - (3 * offset), position.height - 8);
	    dmin = tbc.getWidgetXRange()[0];
	    offset = (int)( Math.log(dmin) / Math.log(Math.E) );
	    g.drawString(nf.format(dmin), 12, position.height - 8);
	}

	g.translate(-position.x, -position.y);
    }

    public void done(Context c) {}

    private Vector spps = new Vector();
    private int sppcount = 0;

    public void done(StackProcessParent spp) {
	//System.out.println("button done");
	spps.removeElement(spp);
	sppcount--;
	if (sppcount == 0) {
	    sp = null;
	    parent.repaint(position);
	}
    }

    public void callerPrintln(String s) {}
    public void callerClearText() {}

    public StarLogoCaller caller() {
	return this;
    }

    public void addStackProcessParent(StackProcessParent s) {
	spps.addElement(s);
	sppcount++;
    }

    public boolean donep() {
	return (sppcount == 0);
    }

    public void stopAll() {
	for(Enumeration e = ((Vector)spps.clone()).elements(); e.hasMoreElements();) {
	    StackProcessParent spp = (StackProcessParent)e.nextElement();
	    spp.stopAll();
	}
	//spps.removeAllElements();
    }



    public void displayOneString(String s) {
	app.createMessageDialog(sl.app, app.activeFrame, app.textBundle.getString("error_in") + " " + 
				this.toString(), s, true);
    }

    public void displayAString(String s) {
	app.createMessageDialog(sl.app, app.activeFrame, app.textBundle.getString("error_in") + " " + 
				this.toString(), s, true);
    }

    public boolean wizardOn() {
	if (wizardOn) {
	    displayAString(app.textBundle.getString("cannot_use_wizard")); 
	    return true;
	}
	else {
	    //System.out.println("disabling data in format menu...");
	    //tbc.disableDataInFormatMenu();
	    return false;
	}
    }
    
    public int getToolbarButton() {
	return ToolbarWidget.LINE;
    }

    public static void loadImages(String gifdir, MediaTracker tracker, Toolkit kit)
	throws MalformedURLException {
	graph = kit.createImage(Pool.class.getResource(gifdir + "line-obj.gif"));
	tracker.addImage(graph, trackerID++);
    }

    public WidgetDrag copy(int edition, Rectangle r){
	LineWidget lw = new LineWidget(r, parent, sl); // r --> position
	lw.type = type;
	lw.id = LineWidget.idCount;
	lw.order = LineWidget.idCount; // / LineSet.NUM; //iface.app.linecharts.size();
	LineWidget.idCount++;
	lw.wizardOn = wizardOn;
	lw.plotpen = plotpen;
	
	int numSets = sets.size();
	lw.sets = new Vector(numSets);
	for (int i = 1; i <= numSets; i++) {
	    lw.sets.addElement(((LineSet)sets.elementAt(i-1)).copy(i, i));
	}

	lw.tbc = tbc.copy(app, lw);

	if (!lw.wizardOn) {
	    lw.tbc.disableDataInFormatMenu();
	}

	lw.tbc.setPlotTitle(tbc.getTitle() + " " + app.textBundle.getString("copy") + 
			    editionString(edition));
      
	lw.tbc.setWidgetSize(this.tbc.getWidgetSize().width, 
			     this.tbc.getWidgetSize().height);
	lw.tbc.validate();
	lw.tbc.repaint();

	lw.parent.addPlotWindow(lw.tbc);

	lw.app.rancompilerp = false;
	app.project.markChanged();
	
	return lw;

    }

    // / / / VM methods  / / //

    // * * *  primPlot methods  * * *
    public void plot(int plotpen, double x) {
	//cannot call plot on xyplot or scatterplot

	if (type == LineWidget.HISTOGRAM) {
	    //histogram requires plot-turtle-distribution or plot-patch-distribution
	} else if (type == LineWidget.LINE) {
	    tbc.addDataPoint(plotpen, x);
	    tbc.repaintWithInterface();
	} else if (type == LineWidget.BAR) {
	    tbc.addDataPointToHash(1, plotpen, plotpen, x);
	    tbc.repaintWithInterface();
	}
    }

    public void setplotppc(int plotpen, int color) {
	tbc.setColor(plotpen, Colors.colorarray[color * 32]);
	tbc.setSLColor(plotpen, color);
	tbc.repaintWithInterface();
    }

    public void setplotxmin(double xmin) {
	tbc.setXMin(xmin);
	tbc.repaintWithInterface();
    }

    public void setplotxmax(double xmax) {
	tbc.setXMax(xmax);
	tbc.repaintWithInterface();
    }

    public void setplotymin(double ymin) {
	tbc.setYMin(ymin);
	tbc.repaintWithInterface();
    }

    public void setplotymax(double ymax) {
	tbc.setYMax(ymax);
	tbc.repaintWithInterface();
    }

    public void setplotxrange(double xmin, double xmax) {
	tbc.setXRange(xmin, xmax);
	tbc.repaintWithInterface();
    }

    public void setplotyrange(double ymin, double ymax) {
	tbc.setYRange(ymin, ymax);
	tbc.repaintWithInterface();
    }

    public void ppreset(int plotpen) {
	//not implemented yet
	// must add clear(int dataset) to Histogram.java
    }

    public void setGrid(boolean b) {
	tbc.setGrid(b);
	tbc.repaintWithInterface();
    }

    public void ppd(int plotpen) {
	if (type != HISTOGRAM && type != BAR) {
	    tbc.ppd();
	    tbc.repaintWithInterface();
	}
    }

    public void ppu(int plotpen) {
	if (type != HISTOGRAM && type != BAR) {
	    tbc.ppu();
	    tbc.repaintWithInterface();
	}
    }

    public void setplottitle(String title) {
	//System.out.println("tbc.l.getTitle(): " + tbc.l.getTitle() + " title: " + title);
	String oldTitle = JSLMenu.getMenuName(tbc);
	tbc.setPlotTitle(title);
	tbc.setTitle(app.textBundle.getString("plot") + " " + 
		     id + ". " + title);
	parent.changePlotWindow(oldTitle, JSLMenu.getMenuName(tbc));
	tbc.repaintWithInterface();
    }

    public void setplotxlabel(String xlabel) {
	tbc.setXLabel(xlabel);
	tbc.repaintWithInterface();
    }

    public void setplotylabel(String ylabel) {
	tbc.setYLabel(ylabel);
	tbc.repaintWithInterface();
    }

    public void setnumplotpens(int num) {
	if (type == LineWidget.HISTOGRAM) {
	    tbc.setHistoNum(num);
	}
	else {
	    tbc.setLineNum(num);
	}
    }

    public void plotdistribution(int plotpen, int key, double value) {
	if (type == LineWidget.HISTOGRAM) {
	    tbc.addHistoDataPointToHash(plotpen, key, value, sl.numturtles);
	    tbc.repaintWithInterface();
	}
    }

    public void plotscatter(int plotpen, int key, double x, double y) {
	if (type == LineWidget.XYPLOT) {
	    //System.out.println("plotpen: " + plotpen);
	    tbc.addDataPointToHash(plotpen, key, x, y);
	    if (tbc.getLineNum() > 50) {
		tbc.setLineNum(0);
	    }
	  
	    tbc.setLineNum(1 + tbc.getLineNum());
	    //((LineLive)tbc.l).num++; ((LineLive)tbc.l2).num++;
	    // XXX num of l and l2 set to different numbers - is this one purpose?
	    tbc.repaintWithInterface();
	}
	else {
	    displayOneString(app.textBundle.getString("cannot_use_scatterplot")); 
	}
    }
    
    
    public void plotxy(int plotpen, double x, double y) {
	tbc.addDataPoint(plotpen, x, y);
	tbc.repaintWithInterface();
    }

    public void setbarwidth(int plotpen, double width) {
	tbc.setBinWidth(plotpen, width);
	tbc.repaintWithInterface();
    }


    // * * * primPlotReporter methods  * * *

    public void plotpen(int plotpen) {
	this.plotpen = plotpen;
    }

    public Integer plotppc(int plotpen) {
	return new Integer(tbc.getSLColor(plotpen));
    }

    public Double ppxcor(int plotpen) {
	if (type == LineWidget.HISTOGRAM) {
	    return new Double(0.0);
	}
	else {
	    return new Double(tbc.getPPXCor(plotpen));
	}
    }

    public Double ppycor(int plotpen) {
	if (type == LineWidget.HISTOGRAM) {
	    return new Double(0.0);
	}
	else {
	    return new Double(tbc.getPPYCor(plotpen));
	}
    }

    public Double plotxmin() {
	return new Double(tbc.getFrameXRange()[0]);
    }

    public Double plotxmax() {
	return new Double(tbc.getFrameXRange()[1]);
    }

    public Double plotymin() {
	return new Double(tbc.getFrameYRange()[0]);
    }

    public Double plotymax() {
	return new Double(tbc.getFrameYRange()[1]);
    }

    public String plottitle() {
	return tbc.getTitle();
    }

    public String plotxlabel() {
	return tbc.getFrameXLabel();
    }

    public String plotylabel() {
	return tbc.getFrameYLabel();
    }

    public Integer countplotpens() {
	if (type == LineWidget.HISTOGRAM) {
	    return new Integer(tbc.getHistoNum());
	}
	else {
	    return new Integer(tbc.getLineNum());
	}
    }

    // / / / / / / / / / / / //

    //hidden ptolemy commands %ptolemy-plot and %ptolemy-plotxy
    public void ptolemyplot(int plotpen, double x, Turtle tur, Patch pat, int width) {
	if (type == LineWidget.LINE || type == LineWidget.BAR) {
	    //System.out.println("ptolemy plot (plotpen, x): " + plotpen + ", " + x);
	    plot(plotpen, x);
	}
	else if (type == LineWidget.HISTOGRAM) {
	    if (tur != null) {
		plotdistribution(plotpen, tur.who, x);
	    }
	    else {
		plotdistribution(plotpen, (int)(pat.xcor * sl.width + pat.ycor), x);
	    }
	}
	else {
	    //System.out.println("error case; bad type");
	    /***/
	}
    }

    public void ptolemyplotxy(int plotpen, double argX, double argY, Turtle tur, Patch pat, int width) {
	if (type == LineWidget.PHASE) {
	    plotxy(plotpen, argX, argY);
	}
	else if (type == LineWidget.XYPLOT) {
	    if (tur != null) {
		plotscatter(plotpen, tur.who, argX, argY);
	    }
	    else {
		plotscatter(plotpen, (int)(pat.xcor * sl.width + pat.ycor), argX, argY);
	    }
	}
	else {
	    //System.out.println("error case; bad type");
	    /****error case*/
	}
    }

    public boolean plotgrid() {
	return tbc.getGrid();
    }

    public void setPlotGrid(boolean on) {
	tbc.setGrid(on);
	tbc.repaintWithInterface();
    }

}//end class, whew ;-)
