/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.awt.*;
import java.awt.event.*;
import java.io.IOException;
import java.net.*;
import java.util.Vector;
import java.util.ArrayList;
import java.util.Observable;
import java.util.Random;
import java.util.Enumeration;
import yoyo.*;
//for rounding
import java.lang.Math;

public class MonitorWidget extends WidgetDrag
  implements StarLogoCaller, java.util.Observer, DisplayMessage {

		public int cindex = 0;
		public boolean isVis = false;
  		public boolean selected = false;


  public String inst;
  public String name;
    public boolean showName;
  public int digits;
  public Font font;
  public double delay = 0.5;

  Object value = VM.zeroInt;
  String valuestring = "0";

  public int whichip = -1;
  public int onewhichip = -1;
  public StackProcess sp;

  public FontMetrics fm;
  public int textHeight, textDescent;

  String insttorun;
  StarLogo sl;
  Application app;

  public int id;
  public static int idCount = 1;
  public static int MONITOR_WIDTH = 30;
  public static int MONITOR_EDGE_WIDTH = 14;
  public static int MONITOR_HEIGHT = 36;

  static Image monEdges;
  static Image monMiddle;

  static Font MONITOR_FONT;

  static {
  	try {
     MONITOR_FONT = WIDGET_FONT;
     if (YoYo.macintoshp) MONITOR_FONT = Font.getFont("starlogo.monitor.font", MONITOR_FONT);
    } catch (Exception e) { e.printStackTrace(); }
  }

  public MonitorWidget(String name, String listtorun, Rectangle position,
		       int digits, double delay, boolean shownamep, boolean isVis,
		       int mid, JInterface iface, StarLogo sl, String descript) {
    this.name = name;
    this.showName = shownamep;
    this.isVis = isVis;
    this.inst = listtorun;
    this.position = new Rectangle(position);
    this.position.height = MONITOR_HEIGHT;
    this.digits = digits;
    this.delay = delay;
    this.id = mid;
    this.parent = iface;
    this.sl = sl;
    this.app = sl.app;
    this.font = MONITOR_FONT;

    fm = iface.getToolkit().getFontMetrics(font);
    textDescent = fm.getMaxDescent();
    textHeight =  fm.getMaxAscent()+textDescent;

    //addObserver(parent);
    //app.procWin.addObserver(this);

    if (!(ToolTipWidget.empty(descript))) setToolTip(new ToolTipWidget(parent, descript));

  }

  public MonitorWidget(String name, String listtorun, Rectangle position,
		       int digits, double delay, boolean shownamep, boolean isVis,
		       int mid, JInterface iface, int whichip, int onewhichip,
		       StarLogo sl, String descript) {
    this(name, listtorun, position, digits, delay, shownamep, isVis, mid, iface, sl, descript);
    this.whichip = whichip;
    this.onewhichip = onewhichip;
  }

  public MonitorWidget(JInterface iface, StarLogo sl) {

    Dimension size = iface.getSize();
    Rectangle outline = iface.outline();

    if (outline.width<MONITOR_WIDTH)  outline.width = MONITOR_WIDTH;
    if (outline.x+MONITOR_WIDTH>size.width)
      outline.x=parent.getSize().width-MONITOR_WIDTH;
    if (outline.y+MONITOR_HEIGHT>size.height) {
      outline.height = outline.height + outline.y - size.height + MONITOR_HEIGHT;
      outline.y = size.height - MONITOR_HEIGHT;
    }
    if (outline.height<MONITOR_HEIGHT)  outline.height = MONITOR_HEIGHT;

    this.position = new Rectangle(outline);
    this.position.height = MONITOR_HEIGHT;
    this.id = idCount++;
    this.parent = iface;
    this.sl = sl;
    this.app = sl.app;
    this.font = WIDGET_FONT;
    this.showName = false;

    fm = iface.getToolkit().getFontMetrics(font);
    textDescent = fm.getMaxDescent();
    textHeight =  fm.getMaxAscent()+textDescent;

    //addObserver(parent);
    //app.procWin.addObserver(this);
  }

  //used for loading from file
  public MonitorWidget(ArrayList args, int i, JInterface parent, StarLogo sl) {
    this.parent = parent;
    this.sl = sl;
    this.app = sl.app;
    float version = app.project.version;
    position = new Rectangle();
    position.x = new Integer(args.get(i++).toString()).intValue()+parent.getBoundaries().x;
    position.y = new Integer(args.get(i++).toString()).intValue()+parent.getBoundaries().y;
    position.width = new Integer(args.get(i++).toString()).intValue()+parent.getBoundaries().x-position.x;
    //  position.height = new Integer(args.get(i++).toString()).intValue()+parent.getBoundaries().y-position.y;
    i++;
    position.height = MONITOR_HEIGHT;
    name = args.get(i++).toString();
    inst = args.get(i++).toString();
    digits = new Integer(args.get(i++).toString()).intValue();
    if (version == Project.MAC_VERSION) {
      delay = 0.5;
    } else {
      delay = Double.valueOf(args.get(i++).toString()).doubleValue();
    }
    font = WIDGET_FONT;
    id = new Integer(args.get(i++).toString()).intValue();
    if (id>=idCount)
      idCount = id + 1;
    fm = parent.getToolkit().getFontMetrics(font);
    textDescent = fm.getMaxDescent();
    textHeight =  fm.getMaxAscent()+textDescent;

    updateMinSize();
    //app.procWin.addObserver(this);
  }

  public MonitorWidget(JInterface iface, StarLogo sl, boolean dummy) {
/**
    Dimension size = iface.getSize();
    Rectangle outline = iface.outline();

    if (outline.width<MONITOR_WIDTH)  outline.width = MONITOR_WIDTH;
    if (outline.x+MONITOR_WIDTH>size.width)
      outline.x=parent.getSize().width-MONITOR_WIDTH;
    if (outline.y+MONITOR_HEIGHT>size.height) {
      outline.height = outline.height + outline.y - size.height + MONITOR_HEIGHT;
      outline.y = size.height - MONITOR_HEIGHT;
    }
    if (outline.height<MONITOR_HEIGHT)  outline.height = MONITOR_HEIGHT;

    this.position = new Rectangle(outline);
    this.position.height = MONITOR_HEIGHT; **/
    this.id = idCount++;
    this.parent = iface;
    this.sl = sl;
    this.app = sl.app;
    this.font = WIDGET_FONT;
    this.showName = false;

    fm = iface.getToolkit().getFontMetrics(font);
    textDescent = fm.getMaxDescent();
    textHeight =  fm.getMaxAscent()+textDescent;

//    addObserver(parent);
//    app.procWin.addObserver(this);
  }


  public static MonitorWidget load(ArrayList items, JInterface iface, StarLogo sl, boolean htmlp) {
      boolean shownamep = false;
    int top = 0, left = 0, width = 50, height = 20, mid = 1;
    int digits = 0; double delay = .5;
    String name = "monitor1", listtorun = "nothing", descript = "";
    int whichip = -1, onewhichip = -1;
      boolean isVis = false;

    int i = 0;
    while (i<items.size()) {
      String item = items.get(i).toString();
      if (item.equals("top-left")) {
	String value = items.get(i+1).toString();
	String value2 = items.get(i+2).toString();
	top = Integer.parseInt(value);
	left = Integer.parseInt(value2);
	i += 3;
	continue;
      }
      if (item.equals("width-height")) {
	String value = items.get(i+1).toString();
	String value2 = items.get(i+2).toString();
	width = Integer.parseInt(value);
	height = Integer.parseInt(value2);
	i += 3;
	continue;
      }
      if (item.equals("name")) {
	name = items.get(i+1).toString();
	i += 2;
	continue;
      }
      if (item.equals("show-name?")) {
	  String value = items.get(i+1).toString();
	  shownamep = value.equals("true");
	  i+=2;
	  continue;
      }
      if (item.equals("isVis")) {
		  String value = items.get(i+1).toString();
		  isVis = value.equals("true");
		  i+=2;
		  continue;
	  }
      if (item.equals("list-to-run")) {
	listtorun = items.get(i+1).toString();
	i += 2;
	continue;
      }
      if (item.equals("monitor-number")) {
	String value = items.get(i+1).toString();
	mid = Integer.parseInt(value);
	i += 2;
	continue;
      }
      if (item.equals("digits")) {
	String value = items.get(i+1).toString();
	digits = Integer.parseInt(value);
	i += 2;
	continue;
      }
      if (item.equals("delay")) {
	String value = items.get(i+1).toString();
	delay = Double.valueOf(value).doubleValue();
	i += 2;
	continue;
      }
      if (item.equals("toolTip-text")) {
        String value = items.get(i+1).toString();
        descript = value;
        i += 2;
        continue;
	}
      if (htmlp) {
	if (item.equals("whichip")) {
	  String value = items.get(i+1).toString();
	  whichip = Integer.parseInt(value);
	  i += 2;
	  continue;
	}
	if (item.equals("onewhichip")) {
	  String value = items.get(i+1).toString();
	  onewhichip = Integer.parseInt(value);
	  i += 2;
	  continue;
	}
      }
      i+=2;
    }

    Rectangle bounds = iface.getBoundaries();
    Rectangle position = new Rectangle(left + bounds.x, top + bounds.y, width, height);
    if (mid >= idCount) idCount = mid + 1;

    if (htmlp) {
      return new MonitorWidget(name, listtorun, position, digits, delay,
			       shownamep, isVis, mid, iface, whichip, onewhichip, sl, descript);
    } else {
      return new MonitorWidget(name, listtorun, position, digits, delay,
			       shownamep, isVis, mid, iface, sl, descript);
    }
  }

  public int getId() {
	  return this.id;
  }
  public String getSaveString(boolean htmlp) throws IOException {
    Rectangle bounds = parent.getBoundaries();
//   	if (!(isVis))
    return "SLMonitor\r\n" +
      "top-left " + (position.y - bounds.y) + " " + (position.x- bounds.x) + "\r\n" +
      "width-height " + position.width + " " + position.height + "\r\n" +
      "name \"" + name + "\"\r\n" +


      "list-to-run \"" + Utilities.printable(inst) + "\"\r\n" +
      "digits " + digits + "\r\n" +
      "delay " + delay + "\r\n" +
      "monitor-number " + id + "\r\n" +
      "show-name? " + ((showName) ? "true\r\n" : "false\r\n") +
       getToolTipSaveString() +
      ((htmlp && whichip >= 0 && onewhichip >= 0) ?
       "whichip " + whichip + "\r\n" +
       "onewhichip " + onewhichip + "\r\n" : "") +
     "\r\n";
  }

  static public void createNew(JInterface iface) {
    MonitorDialog md = Application.createMonitorDialog((Frame)iface.frame(), iface.app(), iface.app().textBundle.getString("new_monitor_label"));

    md.show();
    if (md.pressedOK()) {
      MonitorWidget mw = new MonitorWidget(iface, iface.sl());
      md.getValues(mw);
      mw.updateMinSize();
      iface.addWidget(mw);
      iface.app().rancompilerp = false; //project.compile(iface.app());
    }
    md.dispose();
  }
// !@!@
	public void setIndex(int i) {
		this.cindex = i;
	}

// !@!@
  static public int createNewFromPtolemy(JInterface iface, int i) {
    MonitorDialog md = Application.createMonitorDialog((Frame)iface.frame(), iface.app(), iface.app().textBundle.getString("new_monitor_label"));
    md.show();
    if (md.pressedOK()) {
	//if (StarLogo.DEBUG) {System.out.println("OK clicked");}
      MonitorWidget mw = new MonitorWidget(iface, iface.sl(), true);
      //      System.out.println("<1>");
      md.getValues(mw);
//      System.out.println("2");
      iface.addWidget(mw);
//      System.out.println("3");
      iface.app().rancompilerp = false;
      //iface.app().project.compile(iface.app());
      mw.setIndex(i);
      mw.setVis(true);
    }
    md.dispose();
    return (idCount - 1);
  }
// !@!@
  public void setVis(boolean b) {
	  this.isVis = true;
  }
  public void edit() {
    MonitorDialog md = Application.createMonitorDialog((Frame)parent.frame(), app, 
						       app.textBundle.getString("edit_monitor_label"), this);
    md.show();
    if (md.pressedOK()) {
      boolean showNameOld = showName;
      String oldName = name;
      String oldInst = inst;
	parent.addHandlesToClipRect(this);
      md.getValues(this);
      if ((showNameOld != showName) ||
          (showName && !(oldName.equals(name))) ||
          (!showName && !(oldInst.equals(inst)))) updateMinSize();
      update(position);
      app.rancompilerp = false;
      app.project.markChanged();
      //app.project.compile(app);
    }
    md.dispose();
    super.edit();
  }

  public void setValue(Object value) {
      this.value = value;
      valuestring = formatValue();
      parent.repaint(position);
  }

  public void delete() {
    parent.monitors().removeElement(this);
    super.delete();
    //app.procWin.deleteObserver(this);
  }

  public String getCommandToCompile() {
    String cPrefix = "compile-monitor-command " + id + " ";
    String prefix = "[ " + app.commandsBundle.getString("loop") + " [ " +
	app.commandsBundle.getString("%print_to_monitor") + " " + id + " ";
    String middle = " " + app.commandsBundle.getString("wait") + " ";
    String suffix = " ] ] ";

    String oneprefix = " [ " + app.commandsBundle.getString("%print_to_monitor") + 
	" " + id + " ";
    String onesuffix = " ]";

    return cPrefix + prefix + inst + middle + delay + suffix +
      oneprefix + inst + onesuffix;
  }

  public void update() {
//    compileCommand();
  }

  public void update(Observable obs, Object arg) {
    if (arg instanceof SLProc)
      update();
  }

  public void update(Object arg) {
    setChanged();
    notifyObservers(arg);
  }

  public void compileCommand() {
    whichip = -1; onewhichip = -1;
    synchronized(sl.runningp) {
      sl.stopAllStarLogo();
    }
    Object[] listtocompile = Reader.read(getCommandToCompile());
    Object[] newlist = new Object[listtocompile.length + 1];
    System.arraycopy(listtocompile, 0, newlist, 0, listtocompile.length);
    newlist[newlist.length -1] = this;
    if (StarLogo.DEBUG) {
	app.log.println("Trying to compile monitor: " + this);
    }
    app.cc.enqueue(newlist, app.message, app.err, this);
  }

  public void setCompiledCommand(Object[] ilist, Object[] olist,
				 Object[] ilist2, Object[] olist2) {
      if (StarLogo.DEBUG) {
	  app.log.println("Finished compiling monitor: " + this);
      }
    int[] newarray = new int[ilist.length];
    int[] newarray2 = new int[ilist2.length];
    for(int i = 0; i<ilist.length; i++) {
      newarray[i] = ((Number)ilist[i]).intValue();
    }
    for(int i = 0; i<ilist2.length; i++) {
      newarray2[i] = ((Number)ilist2[i]).intValue();
    }
    if (StarLogo.DEBUG) {
	app.log.println("monitor ilist: " + YoYo.printToString(ilist));
	app.log.println("monitor olist: " + YoYo.printToString(olist));
	app.log.println("monitor ilist2: " + YoYo.printToString(ilist2));
	app.log.println("monitor olist2: " + YoYo.printToString(olist2));
    }
    whichip = sl.internalSetInstructions(toString(), newarray, olist);
    onewhichip = sl.internalSetInstructions(toString(), newarray2, olist2);
    if (StarLogo.DEBUG) {
	app.log.println("monitor whichip = " + whichip + " onewhichip " + onewhichip);
    }
//these next 3 lines are supposed to be commented....
//    synchronized(sl.runningp) {
//      if (onewhichip >= 0) StackProcessParent.runMonitor(sl, onewhichip, false, this);
//    }
    sp = null;
  }

  public String toString() {
    return("Monitor: " + inst);
  }

  public void paint(Graphics g) {
    g.translate(position.x, position.y);
    Shape oldclip = g.getClip();
    g.clipRect(0, 0, MONITOR_EDGE_WIDTH, MONITOR_HEIGHT);
    g.drawImage(monEdges, 0, 0, null);

    g.setClip(oldclip);
    g.clipRect(position.width - MONITOR_EDGE_WIDTH, 0, MONITOR_EDGE_WIDTH, MONITOR_HEIGHT);
    g.drawImage(monEdges, position.width - (2 * MONITOR_EDGE_WIDTH), 0, null);

    g.setClip(oldclip);
    g.clipRect(0, 0, position.width, position.height);
    g.drawImage(monMiddle, MONITOR_EDGE_WIDTH, 0,
		position.width-(2 * MONITOR_EDGE_WIDTH), MONITOR_HEIGHT, null);

    paintText(g, oldclip);

    g.setClip(oldclip);
    g.translate(-position.x, -position.y);
  }
    
    protected void paintText(Graphics g, Shape oldclip) {
	g.setColor(Color.black);
	g.setFont(font);
	int textWidth = fm.stringWidth((showName ? name : inst));
	// need to translate here so that clip doesn't have negative location (for ms java)
	g.setClip(oldclip);
	g.clipRect(5, 23, position.width-10, 10);
	g.drawString((showName ? name : inst), 5, 
		     ((YoYo.macosxp) ? 29 : 28) + textHeight / 2 - textDescent);
	
	g.setClip(oldclip);
	g.clipRect(10, 7, position.width-20, 12);
	g.drawString(valuestring, 10, 
		     ((YoYo.macosxp) ? 14 : 13) + textHeight / 2 - textDescent);
  }

  public void updateMinSize() {
    int textWidth = fm.stringWidth((showName ? name : inst));
    int minwidth = 10 + textWidth;
    if (position.width < minwidth) {position.width = minwidth;}
    if (parent.outline() != null) {parent.outline().width = position.width;}
  }


  public void done(Context c) {
  }

  private Vector spps = new Vector();
  private int sppcount = 0;

  public void done(StackProcessParent spp) {
    //System.out.println("button done");
    spps.removeElement(spp);
    sppcount--;
    if (sppcount == 0) {
      sp = null;
      parent.repaint(position);
    }
  }

  public void callerPrintln(String s) {}
  public void callerClearText() {}

  public StarLogoCaller caller() {
    return this;
  }

  public void addStackProcessParent(StackProcessParent s) {
    spps.addElement(s);
    sppcount++;
  }

  public boolean donep() {
    return (sppcount == 0);
  }

  public void stopAll() {
    for(Enumeration e = ((Vector)spps.clone()).elements(); e.hasMoreElements();) {
      StackProcessParent spp = (StackProcessParent)e.nextElement();
      spp.stopAll();
    }
    //spps.removeAllElements();
  }



  public void displayOneString(String s) {
      Application.createMessageDialog(sl.app, app.activeFrame, app.textBundle.getString("error_in") + " " + name, s, true);
  }

  public void displayAString(String s) {
      Application.createMessageDialog(sl.app, app.activeFrame, app.textBundle.getString("error_in") + " " + name, s, true);
  }

  String formatValue() {
    if (value instanceof Number) {
      return clipDigits(value.toString());
    }
    else return YoYo.printToString(value);
  }


  String clipDigits(String str) {
//System.out.println("top of clipdigits. str = " + str);
    int e = str.indexOf('E');
    int i = str.indexOf('.');

//this is to catch strings of the form -x.xxxxxxxxxE-x, which we were crashing on before
    int dash = str.indexOf('-');

//if the first dash is negative, it means we have a negative number, so pick again and check
    if (dash == 0)
	{String tmp = str.substring(1, str.length());
	 dash = tmp.indexOf('-');}
//if there's a dash but it's not the negative sign from scientific notation
    if ((e == -1) && (dash != -1))
//just cut the string off. this should work because the only context that this seems to occur in is
//when the number is of the form -0.000-x, where x is any number
//also, recalculate the dash, so that, below, we still catch if there's a negative number or not
	{str = str.substring(0, dash);
	 dash = str.indexOf('-');}

//if we're somehow using scientific notation, return the whole string regardless of the digits value
    if (e != -1)
	{//if the exponent is postive, just display it (since it doesn't kick in until E10)
	 if (str.charAt(e+1) != '-') {return(str);}
	 //else the exponent is negative, so convert to a normal number format and let it fall through
	//(it will be rounded in the final else clause below)
	 else
	   {String zeros = "";
	    int myexp = Integer.parseInt(str.substring(e+2, str.length()));
	    for (int j=0; j<(myexp-1); j++) {zeros = zeros + "0";}
//restore negatives, if we need to
	    if (dash == 0) {str = "-0." + zeros + str.substring(0,i);}
	    else {str = "0." + zeros + str.substring(0,i);}
//check if there's not another dash; there shouldn't be, at this point
	    dash = str.substring(1, str.length()).indexOf('-');
	    if (dash != -1) {str = str.substring(0,dash);}
	    i = str.indexOf('.');}
         }

//if there currently isn't a decimal and there shouldn't be one displayed, return the string
    if ((i==-1) && (digits==0))
      {return(str);}
//else if there currently isn't a decimal but we're asking to display one, pad the string
    else if ((i==-1) && (digits != 0))
	{str = str + ".000";
         i = str.indexOf('.');
	 return(str.substring(0, i+digits+1));}
//else if we're not supposed to display the decimals, throw away the decimals, but round first
    else if (digits==0)
//have to use floats to be able to round, but don't want to show the decimal point at the end
    	{float tmp = Float.valueOf(str).floatValue();
	 str = Float.toString(Math.round(tmp));
	 i = str.indexOf('.');
     	 return str.substring(0,i);
    }
//else if we have decimals and we're supposed to display them
    else
    {
	//if we already have <= the correct number of digits, just display them
	if (str.substring(i+1, str.length()).length() <= digits)
 		{str = str + "000";
		 return (str.substring(0, i+digits+1));}
	//otherwise, we have too many digits, so we need to round.
	else
 	{dash = str.indexOf('-');
	 str = str + "000";	//make sure we have enough digits
      	 double tmp = Double.valueOf(str).doubleValue();
     	 int myfactor = 1;
     	 for (int j= 1; j<=digits; j++)
		{myfactor = 10 * myfactor;}
      	 tmp = tmp * myfactor;		//multiply by 10^(digits needed) so that we can use round
      	 int tmp_int = (int)Math.round(tmp);
	 String zeros = "";
	 String dec = Integer.toString(tmp_int % myfactor);
//woops! i didn't know mod could return a negative number
	 if (dec.charAt(0) == '-') {dec = dec.substring(1, dec.length());}
//if the result of the mod is not as many digits as the decimal is supposed to be, pad with zeros
//so that 1.0008, digits = 3, gives 1.001, and 2.111, digits = 2, gives 2.11
 	 for (int j = dec.length(); j<digits; j++) {zeros = zeros + "0";}
//restore negatives if we need to
	 if (dash == -1) {str = (tmp_int / myfactor) + "." + zeros + dec;}
	 else {str = "-" + (tmp_int / myfactor) + "." + zeros + dec;}
         return(str); }
    }
  }

  public void handleMouseClick(MouseEvent e) {
    if (parent.getToolbar().buttonDown == ToolbarWidget.MONITOR)
      if (!app.kioskp && YoYo.applet == null) edit();
  }

  public int getToolbarButton() {
    // what button on the toolbar corresponds to this widgetdrag-- overrridden
    return ToolbarWidget.MONITOR;
  }

    /*
  public int calcCursor(int x, int y) {
    int i = super.calcCursor(x,y);
    if ((i==Cursor.NE_RESIZE_CURSOR)||(i==Cursor.SE_RESIZE_CURSOR))
      return(Cursor.E_RESIZE_CURSOR);
    else if ((i==Cursor.NW_RESIZE_CURSOR)||(i==Cursor.SW_RESIZE_CURSOR))
      return(Cursor.W_RESIZE_CURSOR);
    else
      return(i);
  }
    */

    public int resizeWhichCorner(Point anchor) {
	Rectangle test = new Rectangle();
	test.setBounds(position.x-CORNER_WIDTH, position.y-CORNER_WIDTH,
		       CORNER_WIDTH, CORNER_WIDTH);
	if (test.contains(anchor.x, anchor.y)) return JInterface.RESIZELEFT;

	test.setBounds(position.x+position.width, position.y-CORNER_WIDTH,
		       CORNER_WIDTH, CORNER_WIDTH);
	if (test.contains(anchor.x, anchor.y)) return JInterface.RESIZERIGHT;

	test.setBounds(position.x-CORNER_WIDTH, position.y+position.height,
		       CORNER_WIDTH, CORNER_WIDTH);
	if (test.contains(anchor.x, anchor.y)) return JInterface.RESIZELEFT;

	test.setBounds(position.x+position.width, position.y + position.height,
		       CORNER_WIDTH, CORNER_WIDTH);
	if (test.contains(anchor.x, anchor.y)) return JInterface.RESIZERIGHT;

	return 0;
    }

  public static void loadImages(String gifdir, MediaTracker tracker, Toolkit kit)
    throws MalformedURLException {
    // requires: trackerID in Widget indexes start of empty ID #s in <tracker>
    monEdges = kit.createImage(MonitorWidget.class.getResource(gifdir + "monitor.gif"));
    monMiddle = kit.createImage(MonitorWidget.class.getResource(gifdir + "mon-mid.gif"));

    tracker.addImage(monEdges, trackerID++);
    tracker.addImage(monMiddle, trackerID++);
  }

  public WidgetDrag copy(int edition, Rectangle r) {
    String s = name + " copy" + WidgetDrag.editionString(edition);
		return new MonitorWidget(s, inst, r, digits, delay, showName, isVis, ++MonitorWidget.idCount, parent, whichip, onewhichip, sl, getToolText());
	}
}




