/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.util.Enumeration;
import java.util.Vector;
import java.util.ArrayList;
import java.awt.event.*;
import java.awt.*;
import java.awt.image.*;
import java.util.Hashtable;
import yoyo.YoYo;
import java.net.*;
import java.io.*;
import yoyo.PrimTCP;

public class NetworkPatchCanvas implements PatchCanvasInterface {

    StarLogo sl;
    Socket socket;
    OutputStream out;

  public NetworkPatchCanvas(StarLogo sl, Socket client) {
      this.sl = sl;
      setClientSocket(client);
      //System.out.println("calling correct constructor");
  }

  public NetworkPatchCanvas(StarLogo sl, PatchCanvasInterface oldpc) {
      this.sl = sl;
      if (oldpc instanceof NetworkPatchCanvas) {
	  NetworkPatchCanvas npc = (NetworkPatchCanvas)oldpc;
	  if (npc.socket != null) {
	      socket = npc.socket;
	      out = npc.out;
	  }
      }
  }


    protected Object graphicslock = new Object();

    public Object getGraphicsLock() {
	return graphicslock;
    }

     public int pcdraws = 0;
 
     public void resetNumPatchesDrawn() {
 	pcdraws = 0;
     }
 
     public int getNumPatchesDrawn() {
 	return pcdraws;
     }


    public boolean init(JInterface iface) { return true; }
    public void dispose() {}

  public void setClientSocket(Socket client) {
	  socket = client;
	  try {
	      out = new BufferedOutputStream(client.getOutputStream(), 4096);
	  } catch (IOException e) {
	      e.printStackTrace();
	  }
  }


  public void updateGraphics(Graphics g) {
    int x, y, yStep, xStep, poffy;
    int cellsize = sl.cellsize;
    for (y = 0, yStep = 0, poffy=sl.widthp2; y < sl.height; y++, yStep+=cellsize, poffy+=sl.widthp2) {
      int poffset = poffy + 1;
      for (x = 0, xStep = 0; x<sl.width; x++, xStep+=cellsize) {
	Patch patch = sl.patches[poffset++];
	if (patch.topturtle != null) {
	    //g.setColor(Colors.colorarray[(int)(patch.topturtle.color * 32.0)]);
	  //g.fillRect(xStep, yStep, cellsize, cellsize);
	  addToPacket(x, y, (int)patch.topturtle.color);
	}
	else {
	    //g.setColor(Colors.colorarray[(int)(patch.pc * 32.0)]);
	    //g.fillRect(xStep, yStep, cellsize, cellsize);
	  addToPacket(x, y, (int)patch.pc);
	}
      }
    }
  }

    public void drawPatch(double xcor, double ycor, double color) {
      drawPatch(xcor, ycor, color, sl.patches);
  }

    public void drawPatch(double xcor, double ycor, double color, Patch[] patches) {
    if (!displayp) return;
    int cellsize = sl.cellsize;
    int x = (int)xcor;
    int y = (int)ycor;
    //sl.theg.setColor(Colors.colorarray[(int)(color * 32.0)]);
    //if (!sl.offscreen) uptodate = false;
    //sl.theg.fillRect(x*cellsize + xoffset, y*cellsize + yoffset, cellsize, cellsize);
    addToPacket(x, y, (int)color);
  }

    public void drawPatch(Patch p) {
      drawPatch(p, sl.patches);
  }

    public void drawPatch(Patch p, Patch[] patches) {
    if (!displayp) return;
    int cellsize = sl.cellsize;
    int x = (int)p.xcor - 1;
    int y = (int)p.ycor - 1;
    //sl.theg.setColor(Colors.colorarray[(int)(p.pc * 32.0)]);
    //if (!sl.offscreen) uptodate = false;
    //sl.theg.fillRect(x*cellsize + xoffset, y*cellsize + yoffset, cellsize, cellsize);
    addToPacket(x, y, (int)p.pc);
  }

    public void drawTurtle(Turtle t) {
      drawTurtle(t, sl.patches);
  }

    public void drawTurtle(Turtle t, Patch[] patches) {
    if (!displayp) return;
    //System.out.println("draw turtle " + t);
    int cellsize = sl.cellsize;
    int x = (int)t.xcor;
    int y = (int)t.ycor;
    //System.out.println("at x " + x + " y " + y);
    //System.out.println("color: " + t.color);
    //sl.theg.setColor(Colors.colorarray[(int)(t.color * 32.0)]);
    //if (!sl.offscreen) uptodate = false;
    //sl.theg.fillRect(x*cellsize + xoffset, y*cellsize + yoffset, cellsize, cellsize);
    addToPacket(x, y, (int)t.color);
  }

  public void clearImage() {
      //super.clearImage();
      // FIXME. Send black patches to client
  }

    byte[] bytes = new byte[600];
    int ptr = 0;

    synchronized void addToPacket(int x, int y, int color) {
	pcdraws++;
	bytes[ptr] = (byte)x;
	bytes[ptr+1] = (byte)y;
	bytes[ptr+2] = (byte)color;
	ptr += 3;
	if (ptr < 600) return;
	
	sendpacket(bytes);
	ptr = 0;
    }

    static int numsent = 0;

  public void sendpacket(byte[] bytes) {
      if (socket == null) return;
      try {
      if (numsent > 2) {
	  waitForAck();
      }
	  writeInt(bytes.length, out);
	  out.write(bytes);
	  out.flush();
	  numsent++;
	  //Thread.currentThread().sleep(10);
      } catch (IOException e) {
	  socket =null;
	  out = null;
	  e.printStackTrace();
      } //catch (InterruptedException f) {}
  }
    
    public void waitForAck() throws IOException {
	InputStream in = socket.getInputStream();
	in.read();
    }

  public static void writeInt(int i, OutputStream out) throws IOException {
    out.write((byte)((i >> 24) & 0xFF));
    out.write((byte)((i >> 16) & 0xff));
    out.write((byte)((i >> 8) & 0xff));
    out.write((byte)(i & 0xff));
  }

    public void setLocation(int x, int y) {}
    public Point getLocation() { return null; }
    public Rectangle getPosition() { return null; }

    public boolean displayp = true;
    public void display() { displayp = true; }
    public void noDisplay() { displayp = false; }

    public void loadImage(ImageProducer ip, Pictures pictures, int width, int height) {
	// FIXME
    }

    public void readImage(StarLogo sl, ImageProducer ip, int width, int height, 
    			   Pictures pictures) {
	// FIXME
    }


    /* Refresh backing store */    
    public void updateImage() {}

    /* Redisplay from backing store */
    public void refresh() { 
	// FIXME
    }

    public void load(ArrayList items, JInterface iface, StarLogo sl) {}

}

