/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.io.*;
import yoyo.YoYo;
import yoyo.Context;
import yoyo.LogoError;
import yoyo.PrimList;
import yoyo.PrimMath;
import yoyo.Primitives;
import yoyo.Toplevel;
import yoyo.Symbol;
import yoyo.TopApplication;
import java.io.Serializable;
import java.util.*;

public class Observer extends VM implements Serializable {

    Observer(StarLogo sl) {  // this is the observer
      this(1, 1, new Object[numVars], sl);
      for(int i = 0; i<numVars; i++) turtlesown[i] = zeroInt;
    }

    Observer(int plotpen, int plotwidget, Object[] town, StarLogo sl) {  // this is the observer
	//  System.out.println("create observer turtle");
	this.sl = sl;
	//System.out.println("observer plotpen: " + plotpen);
	this.plotpen = plotpen;
	this.plotwidget = plotwidget;
	turtlesown = town;
    }

  public static Observer restore(StarLogo sl, byte[] out, int offset) {
    // int who, shape, plotpen;
    // double color, xcor, ycor, heading, breed
    // boolean pendownp, shownp
    // save order:
    // who, xcor, ycor, heading, color, breed, shape, plotpen, pendownp, shownp
    int plotpen = 1, plotwidget = 1;

    int off = getOffset("plotpen");
    if (off >= 0)     plotpen = Utilities.readIntFromByteArray(out, offset + off);
    off = getOffset("plotwidget");
    if (off >= 0)     plotwidget = Utilities.readIntFromByteArray(out, offset + off);
    //System.out.println("make observer: plotpen " + plotpen);
    Object[] turtlesown = restoreVariables(out, offset + getStateLength());
    return new Observer(plotpen, plotwidget, turtlesown, sl);
  }


  public void save(byte[] out, int offset) {
    // int plotpen;
    // save order:
    // plotpen
    int off = getOffset("plotpen");
    if (off >= 0) Utilities.writeIntoByteArray(plotpen, out, offset + off);
    off = getOffset("plotwidget");
    if (off >= 0) Utilities.writeIntoByteArray(plotwidget, out, offset + off);
    saveVariables(out, offset + getStateLength());
  }

  public void saveVariables(byte[] out, int offset) {
    int off = 0;
    for(int i = 0; i < numVars; i++) {
      int pos = getVarPos(i);
      Object val = turtlesown[pos];
      int index = Utilities.addIntoTable(val, sl);
      Utilities.writeIntoByteArray(index, out, offset + off);
      off += 4;
    }
  }

  public static Object[] restoreVariables(byte[] out, int offset) {
    Object[] town = new Object[numVars];
    int off = 0;
    for(int i = 0; i < town.length; i++) {
      int intval = Utilities.readIntFromByteArray(out, offset + off);
      off += 4;
      Object val = Utilities.lookupInTable(intval);
      town[getVarPos(i)] = val;
    }
    return town;
  }


  public static Hashtable stateOffsetTable = initStateOffsets();

  public static Hashtable initStateOffsets() {
    Hashtable table = new Hashtable();
    table.put("plotpen", new Integer(0));
    table.put("plotwidget", new Integer(4));
    table.put("statelength", new Integer(8));
    table.put("length", new Integer(8));
    return table;
  }

  public static int getLength() {
    return getOffset("length");
  }

  public static void setLength(Integer val) {
    stateOffsetTable.put("length", val);
  }

  public static int getStateLength() {
    return getOffset("statelength");
  }

  public static int getOffset(String whichone) {
    if (stateOffsetTable.containsKey(whichone)) {
      return ((Integer)stateOffsetTable.get(whichone)).intValue();
    }
    return -1000;
  }

  public static byte[] getStateDesc() throws IOException {
    ByteArrayOutputStream baos;
    DataOutputStream dos = new DataOutputStream(baos = new ByteArrayOutputStream());
    for(Enumeration e = stateOffsetTable.keys(); e.hasMoreElements();) {
      String key = (String)e.nextElement();
      int val = ((Integer)stateOffsetTable.get(key)).intValue();
      dos.writeUTF(key);
      dos.writeInt(val);
    }
    dos.flush();
    dos.close();
    return baos.toByteArray();
  }

  public static void setStateDesc(byte[] bytes) throws IOException {
    stateOffsetTable.clear();
    DataInputStream dis =
      new DataInputStream(new ByteArrayInputStream(bytes));
    while(dis.available() > 0) {
      String key = dis.readUTF();
      int val = dis.readInt();
      stateOffsetTable.put(key, new Integer(val));
    }
    dis.close();
  }

    /*
    void olddiffuse(int pvar, double fraction) {
	double fract = fraction / 8.0;
	int col = sl.widthp2;
	for(int y = 1; y < sl.height+1; y++) {
	    for(int x = 1; x<sl.width+1; x++) {
		Object[] pown = sl.patches[col + x].patchesown;
		double pval = aDouble(pown[pvar]);
		pown[Patch.numVars] = new Double(pval * fract);
		pown[pvar] = new Double(pval - (pval*fraction));
	    }
	    col += sl.widthp2;
	}
	nsum(Patch.numVars, pvar, false);
    }
    */

    void diffuse(int pvar, double fraction) {
	double fract = fraction / 8.0;
	int col = sl.widthp2;
	for(int y = 1; y < sl.height+1; y++) {
	    for(int x = 1; x<sl.width+1; x++) {
		double[] pown = sl.patches[col + x].patchesowndoubles;
		double pval = pown[pvar]; //aDouble(pown[pvar]);
		pown[Patch.numVars] = pval * fract; //new Double(pval * fract);
		pown[pvar] = pval - (pval * fraction); //new Double(pval - (pval*fraction));
	    }
	    col += sl.widthp2;
	}
	nsum(Patch.numVars, pvar, false);
    }

    void diffusePC(double fraction) {
	double fract = fraction / 8.0;
	int col = sl.widthp2;
	for(int y = 1; y < sl.height+1; y++) {
	    for(int x = 1; x<sl.width+1; x++) {
	      Patch p = sl.patches[col + x];
	      double[] pown = p.patchesowndoubles;
	      double pval = p.pc;
	      pown[Patch.numVars] = pval * fract; //new Double(pval * fract);
	      p.pc = pval - (pval * fraction); //new Double(pval - (pval*fraction));
	    }
	    col += sl.widthp2;
	}
	nsumPC(Patch.numVars, false);
    }

    void diffuse4PC(double fraction) {
	double fract = fraction / 4.0;
	int col = sl.widthp2;
	for(int y = 1; y < sl.height+1; y++) {
	    for(int x = 1; x<sl.width+1; x++) {
	      Patch p = sl.patches[col + x];
	      double[] pown = p.patchesowndoubles;
	      double pval = p.pc;
	      pown[Patch.numVars] = pval * fract; //new Double(pval * fract);
	      p.pc = pval - (pval * fraction); //new Double(pval - (pval*fraction));
	    }
	    col += sl.widthp2;
	}
	nsum4PC(Patch.numVars, false);
    }

    void diffuse4(int pvar, double fraction) {
	double fract = fraction / 4.0;
	int col = sl.widthp2;
	for(int y = 1; y < sl.height+1; y++) {
	    for(int x = 1; x<sl.width+1; x++) {
		double[] pown = sl.patches[col + x].patchesowndoubles;
		double pval = pown[pvar];
		pown[Patch.numVars] = pval * fract; 
		pown[pvar] = pval - (pval * fraction);
	    }
	    col += sl.widthp2;
	}
	nsum4(Patch.numVars, pvar, false);
    }

   void setupNsum(int src, int dst, boolean erasedestp) {
       int poffset1 = 0;
       int poffset2 = sl.width + 1;
       for (int y = 0; y < sl.heightp2; y++) {
	   sl.patches[poffset1].patchesowndoubles[dst] = 0;
	   sl.patches[poffset2].patchesowndoubles[dst] = 0;
	   poffset1 += sl.widthp2;
	   poffset2 += sl.widthp2;
       }

       poffset1 = (sl.widthp2 * (sl.height + 1)) + 1;
       for (int y = 1; y < sl.width+1; y++) {
	   sl.patches[y].patchesowndoubles[dst] = 0;
	   sl.patches[poffset1].patchesowndoubles[dst] = 0;
	   poffset1++;
       }

       int col = sl.widthp2;
       if (erasedestp) {
	   for (int y = 1; y < sl.height+1; y++) {
	       for (int x = 1; x < sl.width+1; x++) {
		   Patch p = sl.patches[col + x];
		   //p.source = p.patchesowndoubles[src];
		   p.patchesowndoubles[dst] = 0;
	       }
	       col += sl.widthp2;
	   }
       } else {
	   /*
	   for (int y = 1; y < sl.height+1; y++) {
	   for (int x = 1; x < sl.width+1; x++) {
	   Patch p = sl.patches[col + x];
	   //p.source = aDouble(p.patchesown[src]);
	   //p.patchespwndoubles[dst] = = aDouble(p.patchesown[dst]);
	   }
	   col += sl.widthp2;
	   }
	   */
       }
   }

   void setupNsumPC(int src, boolean erasedestp) {
       int poffset1 = 0;
       int poffset2 = sl.width + 1;
       for (int y = 0; y < sl.heightp2; y++) {
	   sl.patches[poffset1].pc = 0;
	   sl.patches[poffset2].pc = 0;
	   poffset1 += sl.widthp2;
	   poffset2 += sl.widthp2;
       }

       poffset1 = (sl.widthp2 * (sl.height + 1)) + 1;
       for (int y = 1; y < sl.width+1; y++) {
	   sl.patches[y].pc = 0;
	   sl.patches[poffset1].pc = 0;
	   poffset1++;
       }

       int col = sl.widthp2;
       if (erasedestp) {
	   for (int y = 1; y < sl.height+1; y++) {
	       for (int x = 1; x < sl.width+1; x++) {
		   Patch p = sl.patches[col + x];
		   //p.source = p.patchesowndoubles[src];
		   p.pc = 0;
	       }
	       col += sl.widthp2;
	   }
       } else {
	   /*
	   for (int y = 1; y < sl.height+1; y++) {
	   for (int x = 1; x < sl.width+1; x++) {
	   Patch p = sl.patches[col + x];
	   //p.source = aDouble(p.patchesown[src]);
	   //p.patchespwndoubles[dst] = = aDouble(p.patchesown[dst]);
	   }
	   col += sl.widthp2;
	   }
	   */
       }
   }


  void copyBack(int dst) {
    int col = sl.widthp2;
    for (int y = 1; y < sl.height+1; y++) {
      for (int x = 1; x < sl.width+1; x++) {
	Patch p = sl.patches[col + x];
	p.turtlesown[dst] = new Double(p.patchesowndoubles[dst]);
      }
      col += sl.widthp2;
    }
  }


    void nsum(int src, int dst) {
	nsum(src, dst, true);
    }

    void nsumPC(int src) {
	nsumPC(src, true);
    }

    void nsum(int src, int dst, boolean erasedestp) {
	setupNsum(src, dst, erasedestp);
	int col = sl.widthp2;
	for(int y = 1; y < sl.height+1; y++) {
	    for(int x = 1; x< sl.width+1; x++) {
		int poffset = col + x;
		double val = sl.patches[poffset].patchesowndoubles[src];
		if (val == 0) continue;
		int wpp = poffset + sl.widthp2;
		int wmp = poffset - sl.widthp2;
		sl.patches[poffset+1].patchesowndoubles[dst] += val;
		sl.patches[poffset-1].patchesowndoubles[dst] += val;
		sl.patches[wpp+1].patchesowndoubles[dst] += val;
		sl.patches[wpp].patchesowndoubles[dst] += val;
		sl.patches[wpp-1].patchesowndoubles[dst] += val;
		sl.patches[wmp+1].patchesowndoubles[dst] += val;
		sl.patches[wmp].patchesowndoubles[dst] += val;
		sl.patches[wmp-1].patchesowndoubles[dst] += val;
	    }
	    col += sl.widthp2;
	}
 	fixVertical(dst);
	fixHorizontal(dst);
	fixCorners(dst);
	copyBack(dst);
    }

    void nsumPC(int src, boolean erasedestp) {
	setupNsumPC(src, erasedestp);
	int col = sl.widthp2;
	for(int y = 1; y < sl.height+1; y++) {
	    for(int x = 1; x< sl.width+1; x++) {
		int poffset = col + x;
		double val = sl.patches[poffset].patchesowndoubles[src];
		if (val == 0) continue;
		int wpp = poffset + sl.widthp2;
		int wmp = poffset - sl.widthp2;
		sl.patches[poffset+1].pc += val;
		sl.patches[poffset-1].pc += val;
		sl.patches[wpp+1].pc += val;
		sl.patches[wpp].pc += val;
		sl.patches[wpp-1].pc += val;
		sl.patches[wmp+1].pc += val;
		sl.patches[wmp].pc += val;
		sl.patches[wmp-1].pc += val;
	    }
	    col += sl.widthp2;
	}
 	fixVerticalPC();
	fixHorizontalPC();
	fixCornersPC();
	//copyBack(dst);
    }

    void nsum4(int src, int dst) {
	nsum4(src, dst, true);
    }

    void nsum4PC(int src) {
	nsum4PC(src, true);
    }

    void nsum4(int src, int dst, boolean erasedestp) {
	setupNsum(src, dst, erasedestp);
	int col = sl.widthp2;
	for(int y = 1; y < sl.height+1; y++) {
	    for(int x = 1; x< sl.width+1; x++) {
		int poffset = col + x;
		double val = sl.patches[poffset].patchesowndoubles[src];
		if (val == 0) continue;
		int wpp = poffset + sl.widthp2;
		int wmp = poffset - sl.widthp2;
		sl.patches[poffset+1].patchesowndoubles[dst] += val;
		sl.patches[poffset-1].patchesowndoubles[dst] += val;
		sl.patches[wpp].patchesowndoubles[dst] += val;
		sl.patches[wmp].patchesowndoubles[dst] += val;
	    }
	    col += sl.widthp2;
	}
 	fixVertical(dst);
	fixHorizontal(dst);
	copyBack(dst);
    }

    void nsum4PC(int src, boolean erasedestp) {
	setupNsumPC(src, erasedestp);
	int col = sl.widthp2;
	for(int y = 1; y < sl.height+1; y++) {
	    for(int x = 1; x< sl.width+1; x++) {
		int poffset = col + x;
		double val = sl.patches[poffset].patchesowndoubles[src];
		if (val == 0) continue;
		int wpp = poffset + sl.widthp2;
		int wmp = poffset - sl.widthp2;
		sl.patches[poffset+1].pc += val;
		sl.patches[poffset-1].pc += val;
		sl.patches[wpp].pc += val;
		sl.patches[wmp].pc += val;
	    }
	    col += sl.widthp2;
	}
 	fixVerticalPC();
	fixHorizontalPC();
	//copyBackPC();
    }

    void fixVertical(int dst) {
	int poffset1 = sl.widthp2;
	int poffset2 = sl.widthp2 + sl.width + 1;
	for (int y = 1; y < sl.height+1; y++) {
	    sl.patches[poffset1 + sl.width].patchesowndoubles[dst] +=
		sl.patches[poffset1].patchesowndoubles[dst];
	    sl.patches[poffset2 - sl.width].patchesowndoubles[dst] +=
		sl.patches[poffset2].patchesowndoubles[dst];
	    poffset1 += sl.widthp2;
	    poffset2 += sl.widthp2;
	}
    }

    void fixHorizontal(int dst) {
	int poffset = (sl.widthp2 * sl.height) + 1; // 2nd to last row
	for (int x = 1; x < sl.width+1; x++) {
	    sl.patches[poffset].patchesowndoubles[dst] +=
		sl.patches[x].patchesowndoubles[dst];
	    sl.patches[x + sl.widthp2].patchesowndoubles[dst] +=
		sl.patches[poffset + sl.widthp2].patchesowndoubles[dst];
	    poffset++;
	}
    }

    void fixCorners(int dst) {
	double val = sl.patches[0].patchesowndoubles[dst];
	sl.patches[sl.widthp2*(sl.heightp2-1)-2].patchesowndoubles[dst] += val;
	val = sl.patches[sl.widthp2*sl.heightp2-1].patchesowndoubles[dst];
	sl.patches[sl.widthp2+1].patchesowndoubles[dst] += val;
	val = sl.patches[sl.widthp2-1].patchesowndoubles[dst];
	sl.patches[sl.widthp2*(sl.heightp2-2) + 1].patchesowndoubles[dst] += val;
	val = sl.patches[sl.widthp2*(sl.heightp2-1)].patchesowndoubles[dst];
	sl.patches[2*(sl.widthp2-1)].patchesowndoubles[dst] += val;
    }

    void fixVerticalPC() {
	int poffset1 = sl.widthp2;
	int poffset2 = sl.widthp2 + sl.width + 1;
	for (int y = 1; y < sl.height+1; y++) {
	    sl.patches[poffset1 + sl.width].pc +=
		sl.patches[poffset1].pc;
	    sl.patches[poffset2 - sl.width].pc +=
		sl.patches[poffset2].pc;
	    poffset1 += sl.widthp2;
	    poffset2 += sl.widthp2;
	}
    }

    void fixHorizontalPC() {
	int poffset = (sl.widthp2 * sl.height) + 1; // 2nd to last row
	for (int x = 1; x < sl.width+1; x++) {
	    sl.patches[poffset].pc +=
		sl.patches[x].pc;
	    sl.patches[x + sl.widthp2].pc +=
		sl.patches[poffset + sl.widthp2].pc;
	    poffset++;
	}
    }

    void fixCornersPC() {
	double val = sl.patches[0].pc;
	sl.patches[sl.widthp2*(sl.heightp2-1)-2].pc += val;
	val = sl.patches[sl.widthp2*sl.heightp2-1].pc;
	sl.patches[sl.widthp2+1].pc += val;
	val = sl.patches[sl.widthp2-1].pc;
	sl.patches[sl.widthp2*(sl.heightp2-2) + 1].pc += val;
	val = sl.patches[sl.widthp2*(sl.heightp2-1)].pc;
	sl.patches[2*(sl.widthp2-1)].pc += val;
    }


  public final boolean grab(Object val) {
	Turtle st; int trueip, tempint; Object[] list;
	if (val instanceof Number) {
	    st = sl.getWho(((Number)val).intValue());
	    if (st != null && (st.grabber == null) && st.partner == null) {
		ostack[sp++] = st;
		ostack[sp++] = VM.grabbedMarker;
		partner = st;
		st.grabber = this;
		trueip = istack[--isp];
		istack[isp++] = 1;
		istack[isp++] = ip;
		ip = trueip;
		istack[isp++] = (VM.GRAB2 | (PRIMGRAB << 8) | (OTHER << 16));
		return true;
	    }
	    else return false;
	}
	else if (val instanceof Object[]) {
	    list = (Object[])val;
	    int count = 0;
	    for(tempint = list.length-1; tempint >= 0; tempint--) {
		st = sl.getWho(aInteger(list[tempint]));
		if (st != null && (st.grabber == null) && st.partner == null) {
		    ostack[sp++] = st;
		    ostack[sp++] = VM.grabbedMarker;
		    partner = st;
		    st.grabber = this;
		    count++;
		}
	    }
	    if (count > 0) {
		trueip = istack[--isp];
		istack[isp++] = count;
		istack[isp++] = ip;
		ip = trueip;
		istack[isp++] = (VM.GRAB2 | (PRIMGRAB << 8) | (OTHER << 16));
		return true;
	    }
	    else return false;
	}
	else return false;
    }

  public final boolean grabPatch(Patch p) {
    int trueip, tempint; Object[] list;
    if (!p.grabbedp && p.patchpartner == null) {
	ostack[sp++] = p;
	ostack[sp++] = VM.grabbedPatchMarker;
	patchpartner = p;
	p.grabbedp = true;
	trueip = istack[--isp];
	istack[isp++] = 1;
	istack[isp++] = ip;
	ip = trueip;
	istack[isp++] = (VM.GRABPATCH2 | (PRIMGRAB << 8) | (OTHER << 16));
	return true;
    }
    else return false;
  }


    static class VariableDesc {
	String name;
	int position;
	boolean enump;
	
	VariableDesc(String name, int position, boolean enump) {
	    this.name = name;
	    this.position = position;
	    this.enump = enump;
	}
    }

    static int numVars = 0;
    static VariableDesc[] vars = null;
    
    static public void setVars(Object[] town, StarLogo sl, Context c) {

	//System.out.println("observer setVars: " + YoYo.printToString(town));

	int newNumVars = town.length;
	setLength(new Integer(getStateLength() + (4 * newNumVars)));
	
	VariableDesc[] newvars = new VariableDesc[newNumVars];

	for (int i = 0; i < newNumVars; i++) {
	    Object[] pair = YoYo.aList(c, town[i]);
	    newvars[i] = new VariableDesc(YoYo.aString(c, pair[0]), 
					  YoYo.aInteger(c, pair[1]),
					  YoYo.aBoolean(c, pair[2]));
	}

	boolean[] donep = new boolean[newNumVars]; // they're all initialized false
	HashMap varMap = new HashMap(newNumVars);
	if (vars != null) {
	    // loop over old vars and map old pos to new pos
	    for(int i = 0; i < numVars; i++) {
		Integer oldPos = new Integer(vars[i].position);
		for(int j = 0; j < newNumVars; j++) {
		    if (newvars[j].name.equals(vars[i].name)) {
			Integer newPos = new Integer(newvars[j].position);
			varMap.put(oldPos, newPos);
			donep[newvars[j].position] = true;
			break;
		    }
		}
	    }
	} 
	
	Object[] newtown = new Object[newNumVars];
	for(Iterator i = varMap.entrySet().iterator(); i.hasNext(); ) {
	    Map.Entry entry = (Map.Entry)i.next();
	    int oldPos = ((Integer)entry.getKey()).intValue();
	    int newPos = ((Integer)entry.getValue()).intValue();
	    newtown[newPos] = sl.observer.turtlesown[oldPos];
	}
	for(int i = 0; i < newNumVars; i++) {
	    if (!donep[i]) newtown[i] = zeroInt;
	}
	sl.observer.turtlesown = newtown;

	vars = newvars;
	numVars = newNumVars;
    }

    public static int getNumVars() {
	return numVars;
    }
    public static String getVarName(int num) {
	return vars[num].name;
    }
    public static int getVarPos(int num) {
	return vars[num].position;
    }
    public static boolean getVarEnump(int num) {
	return vars[num].enump;
    }




    public String toString() {
	return "Observer";
    }

    public String niceToString() {
	return toString();
    }
}
