/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.awt.event.*;
import java.awt.*;
import yoyo.*;
import java.util.Enumeration;
import java.util.Observable;
import java.util.Observer;
import java.awt.datatransfer.*;
import java.io.LineNumberReader;
import java.io.PrintWriter;
import java.io.StringReader;
import java.io.IOException;
import java.io.OutputStream;
import javax.swing.*;

public class OutputWindow extends InfoWindow  {
    
    private static Font OUT_FONT;
    
    static {
    	try {
    		OUT_FONT = new Font("Monospaced", Font.PLAIN, 12);
    	} catch (Exception e)  { e.printStackTrace(); }
    }
    
    public OutputWindow(Application app) {
	super(app, app.textBundle.getString("windows_menu_output_window"));

	OUT_FONT = Font.getFont("starlogo.procedures.font", OUT_FONT);
    }
    
    public void init() {
	
	// setup frame's appearance 
	setLocation(0, 0);
	setSize(300,200);
	setFont(OUT_FONT);
	
	// setup components
	setupText();
	
	// do layout
	setupLayout();
	    slmenu = new JSLMenu(this, app);
	slmenu.setMenuBar(this);
	addNotify();
	
	// setup listeners
	setupKeyTrackers();
    }

    public void setupText() {
	//ta = new TextArea("", 10, 10, (this instanceof OutputWindow) ? 
	//TextArea.SCROLLBARS_BOTH : TextArea.SCROLLBARS_VERTICAL_ONLY);
	ta = new JTextArea("", 10, 10);
	JScrollPane scrollPane = new 
	    JScrollPane(ta, 
			(YoYo.macintoshp ? JScrollPane.VERTICAL_SCROLLBAR_ALWAYS
			 : JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED), 
			JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
	ta.setFont(OUT_FONT);
	JSLDialog.colorText(ta);
	pw = new PrintWriter((new JTEStream(ta)), true);
	getContentPane().add(scrollPane, BorderLayout.CENTER);
    }
    
    class ControlKeyAdapter extends KeyAdapter {
	JFrame parent;
	
	ControlKeyAdapter(JFrame parent) {
	    this.parent = parent;
	}
	
	public void keyPressed(KeyEvent e) {
	    //System.out.println("key pressed " + new Integer(e.getKeyChar()));
	    char key = e.getKeyChar();
	    if (YoYo.macintoshp && e.isMetaDown() && key == '.') { 
		synchronized(sl.runningp) {
		    sl.stopAllStarLogo();
		}
	    }			
	}
	
	public void keyTyped(KeyEvent e) {
	    //System.out.println("key typed " + new Integer(e.getKeyChar()));
	    char c = e.getKeyChar();
	    //  if (c=='\t' && e.getModifiers() == 0) {
	    // 	pw.print("   ");
	    // 	pw.flush();
	    // 	e.consume();
	    //       }
	}
	
	public void keyReleased(KeyEvent e) {
	    //System.out.println("key released " + new Integer(e.getKeyChar()));
	    char c = e.getKeyChar();
	    //if (c == '\t') e.consume();
	    
	    /*Component c = e.getComponent();*/
	    /*if (c instanceof TextComponent)
	      updateLine((TextComponent)e.getComponent());
	    */
	}
    }
    
    public void actionPerformed(ActionEvent e) {
	Object o = e.getSource();
	String arg = "";
	if (o instanceof JMenuItem) {
	    JMenuItem mi = (JMenuItem)o;
	    arg = mi.getLabel();
	} else if (o instanceof MenuItem) {
	    MenuItem mi = (MenuItem)o;
	    arg = mi.getLabel();
	}
	if (arg.equals(app.textBundle.getString("edit_menu_cut"))) {
	    String foo = ta.getSelectedText();
	    if(foo.equals("")) return;
	    StringSelection bar = new StringSelection(foo);
	    clip.setContents(bar, this);
	    int start = ta.getSelectionStart();
	    ta.replaceRange("", start, ta.getSelectionEnd());    
	    ta.select(start, start);
	    return;
	}
	if (arg.equals(app.textBundle.getString("edit_menu_copy"))) {
	    String foo = ta.getSelectedText();
	    if(foo.equals("")) return;
	    StringSelection bar = new StringSelection(foo);
	    clip.setContents(bar, this);      
	    return;
	}
	if (arg.equals(app.textBundle.getString("edit_menu_paste"))) {
	    Transferable trans = clip.getContents(this);
	    if (trans != null) {
		try {
		    Object thing = trans.getTransferData(DataFlavor.stringFlavor);
		    if (thing instanceof String) {
			String s = (String)thing;
			int len = s.length();
			int cur = ta.getCaretPosition();
			if (len == 0) {
			    ta.insert(s, cur);
			    ta.select(cur + len, cur + len);
			}
			else { 
			    int start = ta.getSelectionStart();
			    ta.replaceRange(s, start, ta.getSelectionEnd());      
			    ta.select(start + len, start + len);
			}
		    }
		}
		catch (IOException ioe) {}
		catch (UnsupportedFlavorException ufe) {}
	    }
	    return;
	}
	if (arg.equals(app.textBundle.getString("edit_menu_select_all"))) {
	    ta.selectAll();
	    return;
	}
	if (arg.equals(app.textBundle.getString("edit_menu_clear"))) {
	    ta.setText("");
	}
    }
    
    public void lostOwnership(Clipboard clip, Transferable t) {}
    
    public String getContents() {
	return ta.getText();
    }
    
    public void println(String s) {
	print(s + "\n");
    }
    
    StringBuffer tempsb = new StringBuffer();

    public void print(final String s) {
	tempsb.append(s);
	SwingUtilities.invokeLater(new Runnable() {
		public void run() {
		    synchronized(tempsb) {
			if (tempsb.length() > 0) {
			    int i = ta.getCaretPosition();
			    ta.insert(tempsb.toString(), i);
			    ta.setCaretPosition(i + tempsb.length());
			    tempsb.setLength(0);
			}
		    }
		}
	    });
				   
    }

    public String getWindowName() {
	return app.textBundle.getString("windows_menu_output_window");
    }
}
