/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.lang.Math;
import java.awt.event.*;
import java.awt.*;
import java.net.*;
import java.awt.image.*;
import yoyo.*;

// this class represents the two buttons used to select between
// painting patches or turtles in the paint mode

public class PaintSelect extends Widget {
    
    public static final int PATCH = 0;
    public static final int TURTLE = 1;

    static final int buttonWidth = 24;
    static final int buttonHeight = 24;

    static int turtlewidth = 16;
    static int turtleheight = 17;

    private JInterface iface;
    private StarLogo sl;

    static Image turtleImage;
    static Image newTurtleImage;

    static int colorTransparent = 12632256;
    static int colorBlack = -16777216;

    public int buttonDown = 0;
    public int oldButtonDown = 0;
    int[] turtlepixels = new int[turtlewidth*turtleheight];
    MemoryImageSource source;
    int[] imagepixels = new int[turtlewidth*turtleheight];

    boolean horizontalp;

    public PaintSelect(JInterface iface) {
	this(iface, true);
    }

    public PaintSelect(JInterface iface, boolean horizontalp) {
	super(iface);
	this.horizontalp = horizontalp;
	this.iface = iface;
	int width = (horizontalp) ? buttonWidth : buttonWidth * 2;
	int height = (horizontalp) ? buttonHeight * 2 : buttonHeight;
	position = new Rectangle(0, 0, width, height);
	this.sl = iface.sl();
	makeTurtlePixels();
	updateTurtleColor(Colors.colorarray[iface.colortool().curcolor * 32]);
    }

    public void makeTurtlePixels() {
	PixelGrabber pg = new PixelGrabber(turtleImage, 0, turtleheight, turtlewidth, turtleheight, turtlepixels, 0, turtlewidth);
	try {
	    pg.grabPixels();
	} catch (InterruptedException e) {
	    System.err.println("interrupted waiting for pixels!");
	}
	if ((pg.getStatus() & ImageObserver.ABORT) != 0) {
	    System.err.println("image fetch aborted or errored");
	    return;
	}
			
	for (int i = 0; i < turtlewidth*turtleheight; i++) {
	    if (turtlepixels[i] != colorBlack) {
		imagepixels[i] = turtlepixels[i];
	    }
	}
	source = new MemoryImageSource(turtlewidth, turtleheight, imagepixels, 0, turtlewidth);
	source.setAnimated(true);
	newTurtleImage = iface.getToolkit().createImage(source);
    }

    void updateTurtleColor(Color color) {
	int currentColor = color.getRGB();
	for (int i = 0; i < turtlewidth*turtleheight; i++) {
	    if (turtlepixels[i] == colorBlack) {
		imagepixels[i] = currentColor;
	    }
	}
	source.newPixels(0, 0, turtlewidth, turtleheight);
    }

    public void mouseClicked(MouseEvent e) {
	if (!position.contains(new Point(e.getX(), e.getY()))) return;
	oldButtonDown = buttonDown;
	double num = (horizontalp) ? ((e.getY() - position.y) / buttonHeight) :
	    ((e.getX() - position.x) / buttonWidth);
	buttonDown = (int)java.lang.Math.floor(num);
	if (oldButtonDown == buttonDown) return;
	parent.addToClipRect(position);
    }

    public void mouseDragged(MouseEvent e) {
	mouseClicked(e);
    }

    public void mousePressed(MouseEvent e) {
	mouseClicked(e);
    }

    public void paint(Graphics g) {
	g.translate(position.x, position.y);
	Color oldcolor = g.getColor();
	if (horizontalp) {
	    if (buttonDown == PATCH) {
		//System.out.println("printing patch selected");
		// patch button is selected
		// draw patch w/ current selected color
		int color = iface.colortool().curcolor;
		g.setColor(Colors.colorarray[color * 32]);
		g.fillRect(4, 4, 16, 16);
		// draw yellow rec around patch
		g.setColor(Colors.colorarray[45 * 32]);
		g.drawRect(3, 3, 17, 17);
		// draw black turtle
		Shape oldclip = g.getClip();
		g.clipRect(4, 27, 16, 17);
		g.drawImage(turtleImage, 4, 27, iface);
		g.setClip(oldclip);
	    }
	    else {
		// draw black patch and a colored turtle
		g.setColor(Colors.colorarray[0]);
		g.fillRect(4, 4, 16, 16);
		g.drawImage(newTurtleImage, 4, 27, iface);
	    }
	} else {
	    if (buttonDown == PATCH) {
		//System.out.println("printing patch selected");
		// patch button is selected
		// draw patch w/ current selected color
		int color = iface.colortool().curcolor;
		g.setColor(Colors.colorarray[color * 32]);
		g.fillRect(4, 4, 16, 16);
		// draw yellow rec around patch
		g.setColor(Colors.colorarray[45 * 32]);
		g.drawRect(3, 3, 17, 17);
		// draw black turtle
		Shape oldclip = g.getClip();
		g.clipRect(27, 4, 16, 17);
		g.drawImage(turtleImage, 27, 4, iface);
		g.setClip(oldclip);
	    }
	    else {
		// draw black patch and colored turtle
		g.setColor(Colors.colorarray[0]);
		g.fillRect(4, 4, 16, 16);
		g.drawImage(newTurtleImage, 27, 4, iface);
	    }
	}
	g.setColor(oldcolor);
	g.translate(-position.x, -position.y);
    }


    public static void loadImages(String gifdir, MediaTracker tracker, Toolkit kit) 
	throws MalformedURLException {
	URL u1 = ToolbarWidget.class.getResource(gifdir + "turtle.gif");
	if (u1 == null) throw new MalformedURLException();
	turtleImage = kit.createImage(u1);
	tracker.addImage(turtleImage, trackerID++);
    }
}




