/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.lang.Math;
import java.awt.event.*;
import java.awt.*;
import java.net.*;
import yoyo.*;

public class Painttool extends Widget {

    public static final int NONE = 0;
    public static final int PEN = 1;
    public static final int ERASER = 2;
    public static final int LINE = 3;
    public static final int EYEDROPPER = 4;
    public static final int REC = 5;
    public static final int FILLREC = 6;
    public static final int OVAL = 7;
    public static final int FILLOVAL = 8;
    public static final int POLY = 9;
    public static final int FILLSHAPE = 10;

    static final int buttonWidth = 24;
    static final int buttonHeight = 24;

    private StarLogo sl;
    
    private static final Font patchfont = new Font("Dialog", Font.PLAIN, 12);
    private FontMetrics patchfm;

    public static Image paintHorizImage, paintHorizImagePush,
	paintVertImage, paintVertImagePush;

    /* if buttonDown = NONE, rowDown = -1 or columnDown = -1 */
    public int buttonDown;
    public int rowDown;
    public int columnDown;

    public int oldButtonDown = NONE;
    private int offset = 0;

    public boolean horizontalp;

    public Painttool(JInterface parent) {
	this(parent, true);
    }

    public Painttool(JInterface parent, boolean horizontalp) {
	super(parent, (horizontalp) ? paintHorizImage : paintVertImage);
	this.sl = parent.sl();
	this.patchfm = parent.getToolkit().getFontMetrics(patchfont);
	this.horizontalp = horizontalp;
	buttonDown = NONE;
	rowDown = 0;
	columnDown = 0;
    }

    public void setButtonDown(int button) {
	drawButtonDown(button);
	if (oldButtonDown == NONE &&
	    buttonDown != NONE) {
	    parent.getToolbar().drawButtonDown(ToolbarWidget.NONE);
	}
    }

    public void drawButtonDown(int button) {
	oldButtonDown = buttonDown;
	buttonDown = button;
	buttonToRowColumn(button);
	if (oldButtonDown == buttonDown) return;
	parent.addToClipRect(position);
    }

    public void buttonToRowColumn(int button) {
	if (horizontalp) {
	    button = button - 1;
	    rowDown = button % 2;
	    columnDown = button % 5;
	} else {
	    button = button - 1;
	    rowDown = button % 5;
	    columnDown = button % 2;
	}	    
    }

    public int pointToButton(int x, int y) {
	int button = 0;
	if (horizontalp) {
	    columnDown =  (int)java.lang.Math.ceil(5 * (x - position.x) / image.getWidth(parent));
	    rowDown = (int)java.lang.Math.ceil(2 * (y - position.y) / image.getHeight(parent));
	    button = columnDown * 2 + rowDown + 1;
	} else {
	    columnDown =  (int)java.lang.Math.ceil(2 * (x - position.x) / image.getWidth(parent));
	    rowDown = (int)java.lang.Math.ceil(5 * (y - position.y) / image.getHeight(parent));
	    button = rowDown * 2 + columnDown + 1;
	}
	return button;	    
    }


    public void mouseClicked(MouseEvent e) {
	if (!position.contains(new Point(e.getX(), e.getY()))) return;
	oldButtonDown = buttonDown;
	buttonDown = pointToButton(e.getX(), e.getY());

	if (oldButtonDown == buttonDown) return;
	parent.addToClipRect(position);

	if (oldButtonDown == NONE) {
	    parent.getToolbar().drawButtonDown(ToolbarWidget.NONE);
	}

//         if (!parent.getToolbar().paintActivep() || 
//             parent.painttool().buttonDown != Painttool.POLY) {
// 	    if (sl.patchcanvas instanceof PatchCanvas)
// 		((PatchCanvas)sl.patchcanvas).polygonCancelPoly(e.getX(), e.getY());
// 	}
	
	if (sl.patchcanvas instanceof PatchCanvas)
	    ((PatchCanvas)sl.patchcanvas).saveUndo();
    }

    public void mouseDragged(MouseEvent e) {
	mouseClicked(e);
    }

    public void mousePressed(MouseEvent e) {
	mouseClicked(e);
    }



    public void paint(Graphics g) {
	g.translate(position.x,position.y);
	g.drawImage(image, 0, 0, null);
	if (buttonDown != NONE) {
	    Shape oldclip = g.getClip();
	    g.setClip(buttonWidth* columnDown, buttonHeight * rowDown, buttonWidth, buttonHeight);
	    g.drawImage((horizontalp) ? paintHorizImagePush : paintVertImagePush, 0, 0, null);
	    g.setClip(oldclip);
	}
	g.translate(-position.x,-position.y);
    }

    public static void loadImages(String gifdir, MediaTracker tracker, Toolkit kit) throws MalformedURLException {
	paintHorizImagePush = kit.createImage(Painttool.class.getResource(gifdir + "paint-toolbar-press.gif"));
	paintHorizImage = kit.createImage(Painttool.class.getResource(gifdir + "paint-toolbar2.gif"));
	paintVertImagePush = kit.createImage(Painttool.class.getResource(gifdir + "paint-toolbar-vertical-down.gif"));
	paintVertImage = kit.createImage(Painttool.class.getResource(gifdir + "paint-toolbar-vertical-up.gif"));
	tracker.addImage(paintHorizImagePush, trackerID++);
	tracker.addImage(paintHorizImage, trackerID++);
	tracker.addImage(paintVertImagePush, trackerID++);
	tracker.addImage(paintVertImage, trackerID++);
    }
}


