/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.io.*;
import yoyo.Context;
import yoyo.YoYo;
import yoyo.LogoError;
import yoyo.PrimList;
import yoyo.PrimMath;
import yoyo.Primitives;
import yoyo.Toplevel;
import yoyo.Symbol;
import yoyo.TopApplication;
import java.util.*;

class Patch extends VM implements Serializable {

    double pc = 0.0;
    double undoPC = 0.0;
    
    
    //boolean changed = true;
    transient double[] patchesowndoubles;
    transient Turtle topturtle = null, tophiddenturtle = null;
    transient boolean grabbedp = false;
    double xcor, ycor;
    // transient double source, destination;
    transient boolean hasPMp = false;
    transient PatchMonitor pm;
    
    public boolean checked;  //used for painting (PatchCanvas.fillArea)
    public boolean visible;

  Patch(double xcor, double ycor, StarLogo sl) {
    this(xcor, ycor, 0.0, 1, 1, new Object[numVars], sl);
    for(int i = 0; i<numVars; i++) {
      turtlesown[i] = zeroInt;
      patchesowndoubles[i] = 0.0;
    }
    patchesowndoubles[numVars] = 0.0;
  }

  Patch(double xcor, double ycor, double pc, int plotpen, int plotwidget, Object[] town, StarLogo sl) {
    this.xcor = xcor; this.ycor = ycor;
    this.pc = pc; this.plotpen = plotpen;
    this.plotwidget = plotwidget;
    this.sl = sl;
    if (xcor == 0 || xcor == sl.widthp2-1 ||
	ycor == 0 || ycor == sl.heightp2-1) {
	visible = false;
    } else {
	visible = true;
    }
    // Add one so that diffuse and nsum and stuff don't blow away
    // user variables.
    turtlesown = town;
    patchesowndoubles = new double[numVars + 1];
    for(int i = 0; i<numVars; i++) {
      if (turtlesown[i] instanceof Double) {
	patchesowndoubles[i] = ((Double)turtlesown[i]).doubleValue();
      } else patchesowndoubles[i] = 0.0;
    }
    patchesowndoubles[numVars] = 0.0;
    sl.list.create(this);
  }

  public static Patch restore(StarLogo sl, byte[] out, int offset) {
    // int who, shape, plotpen;
    // double color, xcor, ycor, heading, breed
    // boolean pendownp, shownp
    // save order:
    // who, xcor, ycor, heading, color, breed, shape, plotpen, pendownp, shownp
    double pc = 0;
    int plotpen = 1, plotwidget = 1;
    double xcor = sl.halfwidth, ycor = sl.halfheight;

    int off = getOffset("xcor");
    if (off >= 0)     xcor = Utilities.readDoubleFromByteArray(out, offset + off);
    off = getOffset("ycor");
    if (off >= 0)     ycor = Utilities.readDoubleFromByteArray(out, offset + off);
    off = getOffset("pc");
    if (off >= 0)     pc = Utilities.readDoubleFromByteArray(out, offset + off);
    off = getOffset("plotpen");
    if (off >= 0)     plotpen = Utilities.readIntFromByteArray(out, offset + off);
    off = getOffset("plotwidget");
    if (off >= 0)     plotwidget = Utilities.readIntFromByteArray(out, offset + off);
    //System.out.println("make patch: xcor " + xcor + " ycor " + ycor + " pc " + pc + "plotpen " + plotpen);
    Object[] patchesown = restoreVariables(out, offset + getStateLength());
    return new Patch(xcor, ycor, pc, plotpen, plotwidget, patchesown, sl);
  }

  public void save(byte[] out, int offset) {
    // int plotpen;
    // double pc, xcor, ycor
    // save order:
    // xcor, ycor, pc, plotpen
    int off = getOffset("xcor");
    if (off >= 0)     Utilities.writeIntoByteArray(xcor, out, offset + off);
    off = getOffset("ycor");
    if (off >= 0)     Utilities.writeIntoByteArray(ycor, out, offset + off);
    off = getOffset("pc");
    if (off >= 0)     Utilities.writeIntoByteArray(pc, out, offset + off);
    off = getOffset("plotpen");
    if (off >= 0)     Utilities.writeIntoByteArray(plotpen, out, offset + off);
    off = getOffset("plotwidget");
    if (off >= 0)     Utilities.writeIntoByteArray(plotwidget, out, offset + off);
    saveVariables(out, offset + getStateLength());
  }

  public void saveVariables(byte[] out, int offset) {
    int off = 0;
    for(int i = 0; i < numVars; i++) {
      int pos = getVarPos(i);
      Object val = turtlesown[pos];
      int index = Utilities.addIntoTable(val, sl);
      Utilities.writeIntoByteArray(index, out, offset + off);
      off += 4;
    }
  }

  public static Object[] restoreVariables(byte[] out, int offset) {
    //System.out.println("restore patch variables: " + numVars);
    Object[] town = new Object[numVars];
    int off = 0;
    for(int i = 0; i < town.length; i++) {
      int intval = Utilities.readIntFromByteArray(out, offset + off);
      //System.out.print("var: 0x" + Integer.toString(intval, 16));
      off += 4;
      Object val = Utilities.lookupInTable(intval);
      //System.out.println(" val = " + val);
      town[getVarPos(i)] = val;
    }
    return town;
  }

  public static Hashtable stateOffsetTable = initStateOffsets();


  public static Hashtable initStateOffsets() {
    Hashtable table = new Hashtable();
    table.put("xcor", new Integer(0));
    table.put("ycor", new Integer(8));
    table.put("pc", new Integer(16));
    table.put("plotpen", new Integer(24));
    table.put("plotwidget", new Integer(28));
    table.put("statelength", new Integer(32));
    table.put("length", new Integer(32));
    return table;
  }

  public static int getLength() {
    return getOffset("length");
  }

  public static void setLength(Integer val) {
    stateOffsetTable.put("length", val);
  }

  public static int getStateLength() {
    return getOffset("statelength");
  }

  public static int getOffset(String whichone) {
    if (stateOffsetTable.containsKey(whichone)) {
      return ((Integer)stateOffsetTable.get(whichone)).intValue();
    }
    return -1000;
  }

  public static byte[] getStateDesc() throws IOException {
    ByteArrayOutputStream baos;
    DataOutputStream dos = new DataOutputStream(baos = new ByteArrayOutputStream());
    for(Enumeration e = stateOffsetTable.keys(); e.hasMoreElements();) {
      String key = (String)e.nextElement();
      int val = ((Integer)stateOffsetTable.get(key)).intValue();
      dos.writeUTF(key);
      dos.writeInt(val);
    }
    dos.flush();
    dos.close();
    return baos.toByteArray();
  }

  public static void setStateDesc(byte[] bytes) throws IOException {
    stateOffsetTable.clear();
    DataInputStream dis =
      new DataInputStream(new ByteArrayInputStream(bytes));
    while(dis.available() > 0) {
      String key = dis.readUTF();
      int val = dis.readInt();
      stateOffsetTable.put(key, new Integer(val));
    }
    dis.close();
  }


  public final boolean grab(Object val) {
    Turtle st; int trueip, tempint; Object[] list;
    if (val instanceof Number) {
      st = sl.getWho(((Number)val).intValue());
      if (st != null && (st.grabber == null) && st.partner == null) {
	ostack[sp++] = st;
	ostack[sp++] = VM.grabbedMarker;
	partner = st;
	st.grabber = this;
	trueip = istack[--isp];
	istack[isp++] = 1;
	istack[isp++] = ip;
	ip = trueip;
	istack[isp++] = (VM.GRAB2 | (PRIMGRAB << 8) | (OTHER << 16));
	return true;
      }
      else return false;
    }
    else if (val instanceof Object[]) {
      list = (Object[])val;
      int count = 0;
      for(tempint = list.length-1; tempint >= 0; tempint--) {
	st = sl.getWho(aInteger(list[tempint]));
	if (st != null && (st.grabber == null) && st.partner == null) {
	  ostack[sp++] = st;
	  ostack[sp++] = VM.grabbedMarker;
	  partner = st;
	  st.grabber = this;
	  count++;
	}
      }
      if (count > 0) {
	trueip = istack[--isp];
	istack[isp++] = count;
	istack[isp++] = ip;
	ip = trueip;
	istack[isp++] = (VM.GRAB2 | (PRIMGRAB << 8) | (OTHER << 16));
	return true;
      }
      else return false;
    }
    else return false;
  }

  public final boolean grabPatch(Patch p) {
    if (grabbedp) return false;
    int trueip, tempint; Object[] list;
    if (p != this && !p.grabbedp && p.patchpartner == null) {
	ostack[sp++] = p;
	ostack[sp++] = VM.grabbedPatchMarker;
	patchpartner = p;
	p.grabbedp = true;
	trueip = istack[--isp];
	istack[isp++] = 1;
	istack[isp++] = ip;
	ip = trueip;
	istack[isp++] = (VM.GRABPATCH2 | (PRIMGRAB << 8) | (OTHER << 16));
	return true;
    }
    else return false;
  }


    static class VariableDesc {
	String name;
	int position;
	boolean enump;
	
	VariableDesc(String name, int position, boolean enump) {
	    this.name = name;
	    this.position = position;
	    this.enump = enump;
	}
    }

    static int numVars = 0;
    static VariableDesc[] vars = null;
    
    static public void setVars(Object[] pown, StarLogo sl, Context c) {

	//System.out.println("patch setVars: " + YoYo.printToString(pown));

	int newNumVars = pown.length;
	setLength(new Integer(getStateLength() + (4 * newNumVars)));
	
	VariableDesc[] newvars = new VariableDesc[newNumVars];

	for (int i = 0; i < newNumVars; i++) {
	    Object[] pair = YoYo.aList(c, pown[i]);
	    newvars[i] = new VariableDesc(YoYo.aString(c, pair[0]), 
					  YoYo.aInteger(c, pair[1]),
					  YoYo.aBoolean(c, pair[2]));
	}

	boolean[] donep = new boolean[newNumVars]; // they're all initialized false
	HashMap varMap = new HashMap(newNumVars);
	if (vars != null) {
	    // loop over old vars and map old pos to new pos
	    for(int i = 0; i < numVars; i++) {
		Integer oldPos = new Integer(vars[i].position);
		for(int j = 0; j < newNumVars; j++) {
		    if (newvars[j].name.equals(vars[i].name)) {
			Integer newPos = new Integer(newvars[j].position);
			varMap.put(oldPos, newPos);
			donep[newvars[j].position] = true;
			break;
		    }
		}
	    }
	} 
	
	if (sl.patches != null) {
	    for(int j = 0; j < sl.patches.length; j++) {
		Patch p = sl.patches[j];
		assert p != null : "uh oh, patch " + j + " doesn't exist";
		Object[] newpown = new Object[newNumVars];
		double[] newpownd = new double[newNumVars + 1];
		for(Iterator i = varMap.entrySet().iterator(); i.hasNext(); ) {
		    Map.Entry entry = (Map.Entry)i.next();
		    int oldPos = ((Integer)entry.getKey()).intValue();
		    int newPos = ((Integer)entry.getValue()).intValue();
		    newpown[newPos] = p.turtlesown[oldPos];
		    newpownd[newPos] = p.patchesowndoubles[oldPos];
		}
		for(int i = 0; i < newNumVars; i++) {
		    if (!donep[i]) {
			newpown[i] = zeroInt;
			newpownd[i] = 0.0;
		    }		    
		}
		newpownd[newNumVars] = 0.0;

		p.turtlesown = newpown;
		p.patchesowndoubles = newpownd;
	    }
	}

	vars = newvars;
	numVars = newNumVars;
    }

    public static int getNumVars() {
	return numVars;
    }
    public static String getVarName(int num) {
	return vars[num].name;
    }
    public static int getVarPos(int num) {
	return vars[num].position;
    }
    public static boolean getVarEnump(int num) {
	return vars[num].enump;
    }

  public String toString() {
    return "(" + ((int)(xcor - 1 - sl.halfwidth)) + ", " +
      ((int)(sl.halfheight - (ycor - 1))) + ")";
  }

  public String niceToString() {
    return "Patch " + toString();
  }

}
