/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.util.Enumeration;
import java.util.Vector;
import java.util.ArrayList;
import java.awt.event.*;
import java.awt.*;
import java.awt.image.*;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.LinkedList;
import yoyo.YoYo;

public class PatchCanvas extends WidgetDrag implements PatchCanvasInterface, ImageConsumer {

    public static int TRANSPARENT_COLOR = 0;

    public StarLogo sl;
    boolean obscured = false;
    boolean uptodate = false;
    boolean displayp = true;
    Rectangle originalPosition;

    public int xoffset, yoffset;

    public int mousedownx, mousedowny;
    public int mousedragx, mousedragy;
    boolean startOfPoly = false;
    int polyHeadx, polyHeady;

    boolean mousePressed;
    int ovalMaxX, ovalMaxY;
    boolean patchOverride;
    //Cursor customCursor;

    protected Object graphicslock = new Object();
    protected Graphics g;

    public Object getGraphicsLock() {
	return graphicslock;
    }

    public PatchCanvas(StarLogo sl) {
	this.sl = sl;
	setSize(sl.width*sl.cellsize, sl.height*sl.cellsize);
	MIN_WIDTH = 3 * sl.cellsize;
	MIN_HEIGHT = 3 * sl.cellsize;
    }

    public PatchCanvas(StarLogo sl, PatchCanvasInterface oldpc) {
	super((oldpc instanceof WidgetDrag) ? (WidgetDrag)oldpc : null);
	this.sl = sl;
	setSize(sl.width*sl.cellsize, sl.height*sl.cellsize);

	if (oldpc instanceof PatchCanvas) {
	    PatchCanvas thepc = (PatchCanvas)oldpc;
	    obscured = thepc.obscured;
	    MIN_WIDTH = thepc.MIN_WIDTH;
	    MIN_HEIGHT = thepc.MIN_HEIGHT;
	} else {
	    obscured = false;
	    MIN_WIDTH = 3 * sl.cellsize;
	    MIN_HEIGHT = 3 * sl.cellsize;
	}
    }

    public boolean init(JInterface parent) {
	this.parent = parent;
	//System.out.println("parent is " + parent);
	//wh = parent.wh;
	image = parent.createImage(position.width, position.height);
	xoffset = position.x;
	yoffset = position.y;
	//Image arrowImage = Toolkit.getDefaultToolkit().createImage(PatchCanvas.class.getResource("gifs/arrow.gif"));
	//customCursor = Toolkit.getDefaultToolkit().createCustomCursor(arrowImage, new Point(0,0), "hidden");
	//sl.app.setCursor(customCursor);
	setDrawingSurface();
	updateImage();
	return true;
    }

    public void dispose() {}

    public void setLocation(int x, int y) {
	super.setLocation(x, y);
	if (!sl.offscreen) {
	    xoffset = x;
	    yoffset = y;
	}
    }

    public void display() {
	if (!displayp) {
	    displayp = true;
	    uptodate = false;
	    refresh();
	}
    }
    public void noDisplay() {
	displayp = false;
    }

    public void updateGraphics(Graphics g) {
			
	int x, y, yStep, xStep, poffy;
	int cellsize = sl.cellsize;
	Shape oldClip = g.getClip();

	if (cellsize < 8) {
	    for (y = 0, yStep = 0, poffy=sl.widthp2; y < sl.height; y++, yStep+=cellsize, poffy+=sl.widthp2) {
		int poffset = poffy + 1;
		for (x = 0, xStep = 0; x<sl.width; x++, xStep+=cellsize) {
		    Patch patch = sl.patches[poffset++];
		    Turtle t = patch.topturtle;
		    if (t != null) {
			g.setColor(Colors.colorarray[(int)(t.color * 32.0)]);
			g.fillRect(xStep, yStep, cellsize, cellsize);
		    } else {
			if (patch.pc != 200.0) {
			    g.setColor(Colors.colorarray[(int)(patch.pc * 32.0)]);
			    g.fillRect(xStep, yStep, cellsize, cellsize);
			}
		    }
		    pcdraws++;
		}
	    }
	} else {
	    for (y = 0, yStep = 0, poffy=sl.widthp2; y < sl.height; y++, yStep+=cellsize, poffy+=sl.widthp2) {
		int poffset = poffy + 1;
		for (x = 0, xStep = 0; x<sl.width; x++, xStep+=cellsize) {
		    Patch patch = sl.patches[poffset++];

		    if (patch.pc != 200.0) {
			g.setColor(Colors.colorarray[(int)(patch.pc * 32.0)]);
			g.fillRect(xStep, yStep, cellsize, cellsize);
		    }

		    Turtle t = patch.topturtle;
		    if (t != null) {
			int heading = (int) (((t.heading+11.25) / 22.5) % 16);
			int srcx = heading * cellsize, srcy = 0;
			g.setClip(xStep, yStep, cellsize, cellsize);
			g.drawImage(TurtleShapes.getShapeAsImage(t, cellsize, sl.app),
				    xStep, yStep, xStep + cellsize, yStep + cellsize,
				    srcx, srcy, srcx + cellsize, srcy + cellsize, this);
			g.setClip(oldClip);
		    }
		    pcdraws++;
		}
	    }
	}
    }


    public void drawPatch(double xcor, double ycor, double color) {
	drawPatch(xcor, ycor, color, sl.patches);
    }

    public void drawPatch(double xcor, double ycor, double color, Patch[] patches) {
	if (!displayp) return;
	pcdraws++;
	int cellsize = sl.cellsize;
	int x = (int)xcor;
	int y = (int)ycor;
	if (color == 200.0) return;
	assert g != null;
	g.setColor(Colors.colorarray[(int)(color * 32.0)]);
	if (!sl.offscreen) uptodate = false;
	g.fillRect(x*cellsize + xoffset, y*cellsize + yoffset, cellsize, cellsize);
    }

    public void drawPatch(Patch p) {
	drawPatch(p, sl.patches);
    }

    public void drawPatch(Patch p, Patch[] patches) {
	// patches appears not to be used?
	if (!displayp) return;
	pcdraws++;
	int cellsize = sl.cellsize;
	int x = (int)p.xcor - 1;
	int y = (int)p.ycor - 1;
	if (p.pc == 200.0) return;
	if (g != null) {
	    g.setColor(Colors.colorarray[(int)(p.pc * 32.0)]);
	    if (!sl.offscreen) uptodate = false;
	    g.fillRect(x*cellsize + xoffset, y*cellsize + yoffset, cellsize, cellsize);
	}
    }

    public void drawTurtle(Turtle t) {
	drawTurtle(t, sl.patches);
    }

    public void drawTurtle(Turtle t, Patch[] patches) {
	if (!displayp) return;
	//System.out.println("draw turtle " + t);
	int cellsize = sl.cellsize;
	int x = (int)t.xcor;
	int y = (int)t.ycor;
	if ( x < sl.width && y <= sl.height ) {
	    if (cellsize < 8) {
		drawPatch(x, y, t.color, patches);
		return;
	    }
	    pcdraws++;
	    int dstx = x*cellsize + xoffset, dsty = y*cellsize + yoffset;
	    //System.out.println("at x " + x + " y " + y);
	    //System.out.println("color: " + t.color);

	    if (g == null) return;

	    if (patches[t.patch].pc != 200.0) {
		g.setColor(Colors.colorarray[(int)(patches[t.patch].pc * 32.0)]);
		g.fillRect(dstx, dsty, cellsize, cellsize);
	    }

	    //g.setColor(Colors.colorarray[(int)(t.color * 32.0)]);
	    if (!sl.offscreen) uptodate = false;
	    //g.fillRect(x*cellsize + xoffset, y*cellsize + yoffset, cellsize, cellsize);
	    int heading = (int) (((t.heading+11.25) / 22.5) % 16);
	    Shape oldClip = g.getClip();
	    int srcx = heading * cellsize, srcy = 0;
	    //System.out.println("srcx = " + srcx + " heading = " + heading);
	    g.setClip(dstx, dsty, cellsize, cellsize);
	    g.drawImage(TurtleShapes.getShapeAsImage(t, cellsize, sl.app),
			dstx, dsty, dstx + cellsize, dsty + cellsize,
			srcx, srcy, srcx + cellsize, srcy + cellsize, this);
	    g.setClip(oldClip);
	}
    }
    
    protected Graphics getDrawingSurface() {
	if (sl.offscreen) {
	    return image.getGraphics();
	} else {
	    return parent.getGraphics();
	}
    }

    public void setDrawingSurface() {
	//try { throw new Exception(); } catch (Exception e) { e.printStackTrace(); }
	//System.out.println("set drawing surface");
	if (g != null) { g.dispose(); }
	g = getDrawingSurface();
	//System.out.println("to " + g);
	//sl.app.setCursor(customCursor);
	assert g != null;
    }

    public void deleteDrawingSurface() {
	//System.out.println("clear drawing surface");
	//try { throw new Exception(); } catch (Exception e) { e.printStackTrace(); }
	//g.dispose();
	if (g != null) {  
	    Toolkit.getDefaultToolkit().sync();  
	    g.dispose(); 
	    g = null;
	}
	//sl.app.restoreCursor();
    }

    //void setGraphics(Graphics g) {
    //sl.theg = g;
    // }

    public void setOffscreen(boolean os) {
	synchronized(sl.runningp) {
	    sl.offscreen = os;
	    if (os) {
		//System.out.println("offscreen is true");
		sl.locked = true;
		//setGraphics(image.getGraphics());
		updateImage();
		xoffset = 0;
		yoffset = 0;
		//System.out.println("offscreen is true");
	    } else {
		// keep the turtles from moving until the patchcanvas image is written to screen
		//sl.locked = true;
		//setGraphics(getDrawingSurface());
		xoffset = position.x;
		yoffset = position.y;
		//System.out.println("offscreen is false");
	    }
	}
    }


    public void clearImage() {
	Graphics g = image.getGraphics();
	g.setColor(Color.black);
	g.fillRect(0, 0, image.getWidth(parent), image.getHeight(parent));
	g.dispose();
    }

    public void updateImage() {
	Graphics g = image.getGraphics();
	updateGraphics(g);
	uptodate = true;
	g.dispose();
    }

    public void refresh() {
	if (!displayp) return;
	parent.repaint(position);
    }

    boolean getOffscreen() {
	//System.out.println("get offscreen ");
	// if painttool is selected, offscreen = false
	/*
	  System.out.println("starlogo " + sl);
	  System.out.println("starlogo.app " + sl.app);
	  System.out.println("starlogo.app.iface " + parent);
	  System.out.println("starlogo.app.iface.toolbar " + parent.toolbar);
	*/
	if (parent.getToolbar().paintActivep()) {
	    return false;
	}
	// System.out.println("not painttool");
	// if outline is intersecting patchcanvas, offscreen = true
	if (parent.outline()!=null)
	    if (position.intersects(parent.outline()))
		return true;
	//System.out.println("parent outline does not intersect position");

	// if any part of the patch canvas is out of the bounds of the white part
	// of the interface, you must draw offscreen.
	Rectangle bounds = parent.getBoundaries();
	//System.out.println("interface boundaries are: " + bounds);
	//System.out.println("patch canvas position is: " + position);
	if (!(bounds.contains(position.x, position.y) &&
	      bounds.contains(position.x + position.width, position.y + position.height)))
	    return true;
	//System.out.println("interface boundary does not intersect position");

	// if patchcanvas being dragged, offscreen = true
	if (parent.draggingp() &&
	    ((!parent.widgetTouched().isSelected() && parent.widgetTouched() == this) ||
	     (parent.widgetTouched().isSelected() && isSelected())))
	    return true;
	//System.out.println("patch canvas is not being dragged");
	// if widget dragging over patchcanvas, offscreen = true
	// ** this clause is covered by isObscured() below
	// *** well, it probably goes faster than that clause
	if (parent.draggingp() &&
	    ((!parent.widgetTouched().isSelected() && position.intersects(parent.widgetTouched().position)) ||
	     (parent.widgetTouched().isSelected() && position.intersects(parent.wh().getSelectRectWithHandles()))))
	    return true;
	//System.out.println("nothing else is being dragged over the patch canvas");
	//if ((WidgetDrag.widgetDragging!=null && (turtledrag == null))&&	(position.intersects( parent.wh.getSelectRect() )))
	//if (turtleTouched == null) return true;
	//	if ((WidgetDrag.widgetDragging!=null && (turtleTouched == null)) &&
	//	    (position.intersects(WidgetDrag.widgetDraggingPosition)))
	//	    return true;
	//	System.out.println("widgetdragging not a turtle");
	// otherwise offscreen = true only if obscured by a widget
	//System.out.println("maybe the patch canvas is obscured?");
	return parent.isObscured(this);

    }

    public void paint(Graphics g) {
	if (parent == null) return;
	if (parent.resizingp() && parent.widgetTouched() == this && originalPosition != null) {
	    g.drawImage(image, originalPosition.x, originalPosition.y, this);
	    g.setColor(Color.gray);
	    g.drawRect(position.x, position.y, position.width, position.height);
	} else {
	    /*
	      synchronized (sl.runningp) {
	      sl.locked = true;
	      }
	    */
	    boolean newoffscreen = getOffscreen();
	    //System.out.println(newoffscreen);
	    boolean changedtofalse = false;
	    if (newoffscreen!=sl.offscreen) {
		//System.out.println("different");
		setOffscreen(newoffscreen);
		changedtofalse = !newoffscreen;
	    }
	    //synchronized(sl.runningp) {
	    if (!uptodate && displayp) updateImage();
	    //long time = System.currentTimeMillis();
	    //try {Thread.currentThread().sleep(50); } catch(Exception e) {}
	    super.paint(g);
	    //parent.getToolkit().sync();
	    //parent.parent.getToolkit().sync();
	    //System.out.println("drew patch canvas " + count++);
	    //try { throw (new Exception());} catch(Exception e) {e.printStackTrace();}
	    //}
	    //System.out.println("paint took " + (System.currentTimeMillis() - time) + " ms.");
	    if (changedtofalse && sl.locked) {
		// unlocking turtles
		//synchronized(sl.runningp) {
		sl.locked = false;
		sl.runningp.notify();
		//}
	    }
	} 
    }

    public int count = 0;

    public void load(ArrayList items, JInterface iface, StarLogo sl) {
	// get starting topleft position
	Rectangle bounds = iface.getBoundaries();
	int left = position.x - bounds.x;
	int top  = position.y - bounds.y;

	int i=0;
	while (i<items.size()) {
	    String item = items.get(i).toString();
	    if (item.equals("top-left")) {
		String value = items.get(i+1).toString();
		String value2 = items.get(i+2).toString();
		top = Integer.parseInt(value);
		left = Integer.parseInt(value2);
		i += 3;
		continue;
	    }
	    // skip the next if we haven't found anything
	    i+=2;
	}

	setLocation(left + bounds.x, top + bounds.y);
    }


    public String getSaveString(boolean htmlp) {
	return getSaveString();
    }

    public String getSaveString() {
	Rectangle bounds = parent.getBoundaries();
	return "SLCanvas\r\n" +
	    "top-left " + (position.y - bounds.y) + " " + (position.x- bounds.x) + "\r\n\r\n";
    }

    void moveToFront() {
	// patchcanvas is always in back
    }


    /*
      public void delete() {
      setSelected(false);
      }
    */

    Turtle turtleTouched = null;
    Patch patchTouched = null;

    /*
      public int calcCursor(int x, int y) {
      if (turtleTouched != null) return Cursor.HAND_CURSOR;
      return super.calcCursor(x, y);
      }
    */

    public void setSelected(boolean b) {
	if (sl.app.kioskp || sl.app.locked || YoYo.applet != null) return;
	super.setSelected(b);
	if (b) {
	    // start label of patch sizes
	    //drawPCSize();
	    //parent.drawCellSize();
	} else {
	    // clear labels of patch sizes fr/ interface
	    //parent.clearPatchDims();
	}
    }

    public void drawPCSize() {
	//parent.drawPCSize(getPatchDim(position.width),
	//			    getPatchDim(position.height));
    }

    void moveTurtleTo(Turtle t, int x, int y) {
	synchronized (getGraphicsLock()) {
	    synchronized (sl.runningp) {
		setDrawingSurface();
		double patchx = x / sl.cellsize;
		double patchy = y / sl.cellsize;
		sl.unlink(t);
		t.ycor = patchy + 0.5;
		t.xcor = patchx + 0.5;
		sl.wrapandlink(t);
		parent.repaint(position);
		deleteDrawingSurface();
	    }
	}
    }

    /*
      public void beginResize() {
      //System.out.println("begin resize");
      super.beginResize();
      oldposition = new Rectangle(position);
      }
    */

    // rounds dimension sizes from pixels to patches (same function for x and y)
    private int getPatchDim(int dim) {
	int result = (int)Math.round(dim / (double)sl.cellsize);
	result -= (1-(result & 1));
	return result;
    }

    // rounds dimension sizes from pixels to patches, with switch to only return a square
    private Dimension getPatchDim(int x, int y, boolean square) {
	x = getPatchDim(x);
	y = getPatchDim(y);
	// if square, shorten the longer dimension to the other
	if (square) {
	    if (x > y) {
		x = y;
	    } else {
		y = x;
	    }
	}
	Dimension result = new Dimension(x, y);
	return result;
    }

    public void endResize(MouseEvent e, int anchorCorner) {
	//System.out.println("end resize");

	// compute the new values of halfheight and halfwidth
	int patchwidth = position.width/sl.cellsize;
	int patchheight = position.height/sl.cellsize;

	boolean changed = sl.resizePatches((patchwidth-1)/2,
					   (patchheight-1)/2, sl.cellsize,
					   position.x, position.y, anchorCorner);
	if (changed) {
	    parent.addHandlesToClipRect(originalPosition);
	}

    }

    public void resize(Point where, int anchorCorner) {
	if (parent.firstresizep()) {
	    originalPosition = new Rectangle(position);
	}
	super.resize(where, anchorCorner);
	Dimension patchdim =
	    getPatchDim(position.width, position.height, parent.pressedShiftControlp());
	//parent.drawPCSize(patchdim.width, patchdim.height);

	// snap to an even patch boundary
	int newwidth = patchdim.width * sl.cellsize;
	int newheight = patchdim.height * sl.cellsize;
	int dx = position.width - newwidth;
	int dy = position.height - newheight;
	position.width = newwidth;
	position.height = newheight;
	switch(anchorCorner) {
	case JInterface.RESIZETOPLEFT:
	    position.x += dx;
	    position.y += dy;
	    break;
	case JInterface.RESIZEBOTLEFT:
	    position.x += dx;
	    break;
	case JInterface.RESIZETOPRIGHT:
	    position.y += dy;
	    break;
	}
	parent.addHandlesToClipRect(this);
    }

    public void checkBounds(Rectangle boundaries) {
	if (position.x < boundaries.x)
	    position.x = boundaries.x;
	if (position.y < boundaries.y)
	    position.y = boundaries.y;
    }

    public boolean draggable(MouseEvent e) {
	if (sl.app.kioskp || sl.app.locked || YoYo.applet != null) return false;
	if (!displayp) return true;
	if (parent.getToolbar().paintActivep()) return false;
	int x = e.getX() - position.x;
	int y = e.getY() - position.y;
	turtleTouched = touchingTurtle(x,y);
	if (turtleTouched == null) {
	    patchTouched = touchingPatch(x,y);
	}
	if (turtleTouched != null) return false;
	return true;
    }

    public void mousePressed(MouseEvent e) {
	//System.out.println("mosue pressed in patch canvas");
	if (!displayp) return;
	if (parent.getToolbar().paintActivep()) {
	    paintMousePressed(e);
	} 
	//should we make it do something annoying like pop up
	//a dialog to warn the user that they are about to 
	//change shape?
	//else if (sl.app.shapeChooser.isVisible()) {
	//		castShape(e);
	//}
    }

    public void castShape(MouseEvent e) {
	int x = e.getX(); int y = e.getY();
	if (x < position.x) {
	    x = position.x;
	} else if (x > position.x+position.width) {
	    x = position.x + position.width - 1;
	}
	if (y < position.y) {
	    y = position.y;
	} else if (y > position.y + position.height) {
	    y = position.y + position.height - 1;
	}
	Patch p = getPatchAt(x, y);
	Turtle t = p.topturtle;
	if (t == null) return;
	t.shape = sl.app.shapeChooser.selectedShape();
	drawTurtle(t);
    }


    public void mouseDragged(MouseEvent e) {
	if (!displayp) return;
	if (parent.getToolbar().paintActivep()) {
	    paintMouseDragged(e);
	} else {
	    if (turtleTouched != null) {
		int x = e.getX() - position.x;
		int y = e.getY() - position.y;
		if (x<0) {
		    x = 0;
		} else if (x >= position.width) {
		    x = position.width-1;
		}
		if (y<0) {
		    y = 0;
		} else if (y >= position.height) {
		    y = position.height-1;
		}
		moveTurtleTo(turtleTouched, x, y);
	    }
	}
    }

    /* only called when the polygon tool is active */
    public void mouseMoved(MouseEvent e) {
	if (!displayp) return;
	int x = e.getX();
	int y = e.getY();
	if (x < position.x) {
	    x = position.x;
	}
	else if (x > position.x+position.width) {
	    x = position.x + position.width - 1;
	}
	if (y < position.y) {
	    y = position.y;
	}
	else if (y > position.y+position.height) {
	    y = position.y + position.height -1;
	}
	synchronized(getGraphicsLock()) {
	    setDrawingSurface();
	    if (polygonMouseMoved(x, y)) {
		parent.repaint(position);
	    }
	    deleteDrawingSurface();
	}
    }


    void mouseDoubleClicked(MouseEvent e) {
	if (!displayp) return;
	if (parent.getToolbar().paintActivep()) {
	    if (parent.painttool().buttonDown == Painttool.POLY) {
		polygonMouseDoubleClicked(e.getX(), e.getY());
		parent.maybeReleasePatchCanvas();
	    }
	} else {
	    if (turtleTouched != null) {
		if (!turtleTouched.hasTMp) {
		    turtleTouched.tm = Application.newTurtleMonitor(turtleTouched, 0.5, sl.app);
		    turtleTouched.tm.initFrame();
		    turtleTouched.hasTMp = true;
                    turtleTouched.tm.setLocation(WindowPosition.nextPoint(turtleTouched.tm));
		}
		turtleTouched.tm.show();
		turtleTouched = null;
	    } else if (patchTouched != null) {
		if (!patchTouched.hasPMp) {
		    patchTouched.pm = Application.newPatchMonitor(patchTouched, 0.5, sl.app);
		    patchTouched.pm.initFrame();
		    patchTouched.hasPMp = true;
                    patchTouched.pm.setLocation(WindowPosition.nextPoint(patchTouched.pm));
		}
		patchTouched.pm.show();
		patchTouched = null;
	    }
	}
    }

    public void mouseClicked(MouseEvent e) {
	synchronized (getGraphicsLock()) {
	    setDrawingSurface();
	    if (parent.getToolbar().paintActivep() && displayp) {
		paintMouseReleased(e);
		
		if (startOfPoly) return;
	    } else {
		turtleTouched = null;
		patchTouched = null;
		deleteDrawingSurface();
	    }
	}
	parent.maybeReleasePatchCanvas();

	//System.out.println("ispc obscured");
	/*
	  if (!parent.isObscured(this)) {
	  //System.out.println("letting turtles go");
	  super.mouseReleased(e);
	  parent.addToClipRect(position);
	  synchronized(sl.runningp) {
	  sl.locked = false;
	  sl.runningp.notify();
	  }
	  return;
	*/
    }

    public void mouseReleased(MouseEvent e) {
	turtleTouched = null;
	patchTouched = null;
	parent.addToClipRect(position);
	super.mouseReleased(e);
    }

    public void moveBy(Point displacement) {
	if (!displayp) return;
	//parent.updatePointerCoordinates(displacement);
	super.moveBy(displacement);
	if (!sl.offscreen) {
	    xoffset = position.x;
	    yoffset = position.y;
	}
    }

    public int calcX(int x) {
	return (int)((x - position.x) / sl.cellsize);
    }
    
    public int calcY(int y) {
	return (int)((y - position.y) / sl.cellsize);
    }

    public int calcX1(int x) {
	int x1 = calcX(x);
	int max = (x1 > -1) ? x1 : -1;
	int min = (sl.width < max) ? sl.width : max;
	return min;
    }

    public int calcY1(int y) {
	int y1 = calcY(y);
	int max = (y1 > -1) ? y1 : -1;
	int min = (sl.height < max) ? sl.height : max;
	return min;
    }


    public int pcdraws = 0;

    public void resetNumPatchesDrawn() {
	pcdraws = 0;
    }
    
    public int getNumPatchesDrawn() {
	return pcdraws;
    }
 

    ArrayList undoTurtleArray = new ArrayList();

    public void saveUndo() {
	synchronized(sl.runningp) {
	    undoTurtleArray.clear();
	    for (int i=0; i<sl.patches.length; i++) {
		Patch p = sl.patches[i];
		p.undoPC = p.pc;
	    }
	}
    }
    
    public void redo() {} // not implemented

    public void undo() {
	synchronized (getGraphicsLock()) {
	    try {
		setDrawingSurface();
		if (parent.paintselect().buttonDown == PaintSelect.PATCH) {
		    patchesUndo();
		} else {
		    turtlesUndo();
		}
		uptodate = false;
	    } finally {
		deleteDrawingSurface();
	    }
	}
	parent.addToClipRect(position);
    }

    public void patchesUndo() {
	synchronized(sl.runningp) {
	    for(int i = 0; i < sl.patches.length; i++) {
		Patch p = sl.patches[i];
		double temp = p.undoPC;
		p.undoPC = p.pc;
		p.pc = temp;
	    }
	}
    }

    public void turtlesUndo() {
	synchronized(sl.runningp) {
	    for(Iterator i = undoTurtleArray.listIterator(); i.hasNext(); ) {
		Turtle turtle = (Turtle)(i.next());
		sl.unlink(turtle);
		sl.die(turtle);
	    }
	}
    }

    void paintMousePressed(MouseEvent e) {
	synchronized (sl.runningp) {
	    sl.locked = true;
	}
	int x = e.getX(); int y = e.getY();
	if (x < position.x) {
	    x = position.x;
	}
	else if (x > position.x+position.width) {
	    x = position.x + position.width - 1;
	}
	if (y < position.y) {
	    y = position.y;
	}
	else if (y > position.y+position.height) {
	    y = position.y + position.height -1;
	}
	patchOverride = true;
	synchronized(getGraphicsLock()) {
	try {
	    setDrawingSurface();
	    switch (parent.painttool().buttonDown) {
	    case Painttool.PEN:
		penMousePressed(x, y);
		break;
	    case Painttool.ERASER:
		eraserMousePressed(x, y);
		break;
	    case Painttool.EYEDROPPER:
		eyeDropperMousePressed(x, y);
		break;
	    case Painttool.REC:
		rectangleMousePressed(x, y);
		break;
	    case Painttool.LINE:
		lineMousePressed(x, y);
		break;
	    case Painttool.FILLREC:
		fillRectangleMousePressed(x, y);
		break;
	    case Painttool.POLY:
		polygonMousePressed(x, y);
		break;
	    case Painttool.FILLSHAPE:
		fillArea(x, y);
		break;
	    case Painttool.OVAL:
	    case Painttool.FILLOVAL:
		ovalMousePressed(x, y);
		break;
	    }
	    sl.app.project.markChanged();
	    parent.addToClipRect(position);
	    //	    parent.repaint(position);
	} finally {
	    deleteDrawingSurface();
	}
	}
    }

    void paintMouseDragged(MouseEvent e) {

	int x = e.getX(); int y = e.getY();

	if (x < position.x) {
	    x = position.x;
	}
	else if (x+1 >= position.x+position.width) {
	    x = position.x + position.width - 2;
	}
	if (y < position.y) {
	    y = position.y;
	}
	else if (y+1 >= position.y+position.height) {
	    y = position.y + position.height -2;
	}
	patchOverride = true;

	synchronized(getGraphicsLock()) {
	try {
	    setDrawingSurface();
	    switch (parent.painttool().buttonDown) {
	    case Painttool.PEN :
		penMouseDragged(x, y);
		break;
	    case Painttool.ERASER:
		eraserMouseDragged(x, y);
		break;
	    case Painttool.EYEDROPPER:
		eyeDropperMouseDragged(x, y);
		break;
	    case Painttool.REC:
		rectangleMouseDragged(x, y, parent.pressedShiftControlp());
		break;
	    case Painttool.FILLREC:
		fillRectangleMouseDragged(x, y, parent.pressedShiftControlp());
		break;
	    case Painttool.LINE:
		// XXX
		lineMouseDragged(x, y, parent.pressedShiftControlp());
		break;
	    case Painttool.POLY:
		polygonMouseDragged(x, y);
		break;
	    case Painttool.OVAL:
		ovalMouseDragged(x, y, parent.pressedShiftControlp());
		break;
	    case Painttool.FILLOVAL:
		fillOvalMouseDragged(x, y, parent.pressedShiftControlp());
		break;
	    }
	    sl.app.project.markChanged();
	    parent.addToClipRect(position);
	    //parent.repaint(position);
	} finally {
	    deleteDrawingSurface();
	}
	}
    }


    void paintMouseReleased(MouseEvent e) {
	int x = e.getX(); int y = e.getY();
	//if (!mousePressed) return;
	//mousePressed = false;
	if (x < position.x) {
	    x = position.x;
	}
	else if (x > position.x+position.width) {
	    x = position.x + position.width - 1;
	}
	if (y < position.y) {
	    y = position.y;
	}
	else if (y > position.y+position.height) {
	    y = position.y + position.height -1;
	}
	patchOverride = true;
	synchronized(getGraphicsLock()) {
	    try {
		setDrawingSurface();
	    switch (parent.painttool().buttonDown) {
	    case Painttool.REC:
		rectangleMouseReleased(x, y);
		break;
	    case Painttool.FILLREC:
		fillRectangleMouseReleased(x, y);
		break;
	    case Painttool.LINE:
		lineMouseReleased(x, y);
		break;
	    case Painttool.POLY:
		polygonMouseReleased(x, y);
		break;
	    case Painttool.OVAL:
		ovalMouseReleased(x, y);
		break;
	    case Painttool.FILLOVAL:
		fillOvalMouseReleased(x, y);
		break;
	    }
	    sl.app.project.markChanged();
	} finally {
	    deleteDrawingSurface();
	}
	}
    }


    /** Paint Tool: PEN **/
	
    void penMousePressed(int x, int y) {
	mousedownx = calcX1(x);
	mousedowny = calcY1(y);
	patchOverride = false;
	patchSetOne(mousedownx, mousedowny, parent.colortool().curcolor, false);
    }
	
    void penMouseDragged(int x, int y) {
	mousedragx = calcX1(x);
	mousedragy = calcY1(y);
	patchOverride = false;
	if (! (mousedownx == mousedragx && mousedowny == mousedragy)) {
	    double color = parent.colortool().curcolor;
	    patchLine(mousedownx, mousedowny,
		      mousedragx, mousedragy,
		      color, false);
	    mousedownx = mousedragx;
	    mousedowny = mousedragy;
	}
    }

    /** Paint Tool: ERASER **/

    void eraserMousePressed(int x, int y) {
	if (parent.paintselect().buttonDown == PaintSelect.PATCH) {
	    mousedownx = calcX1(x);
	    mousedowny = calcY1(y);
	    patchSetOne(mousedownx, mousedowny, 0, false);
	} else {
	    // kill all turtles
	    Patch p = getPatchAt(x, y);
	    killTurtlesHere(p);
	}
    }

    void killTurtlesHere(Patch p) {
	Turtle turtle = p.topturtle;
	Turtle hiddenturtle = p.tophiddenturtle;
	while (turtle != null) {
	    Turtle t = turtle;
	    turtle = turtle.underme;
	    sl.unlink(t);
	    sl.die(t);
	}
	while (hiddenturtle != null) {
	    Turtle t = hiddenturtle;
	    hiddenturtle = hiddenturtle.underme;
	    sl.unlink(t);
	    sl.die(t);
	}
    }

    void eraserMouseDragged(int x, int y) {
	mousedragx = calcX1(x);
	mousedragy = calcY1(y);
	
	if (! (mousedownx == mousedragx && mousedowny == mousedragy)) {
	    if (parent.paintselect().buttonDown == PaintSelect.PATCH) {
		patchSetOne(mousedragx, mousedragy, 0, false);
	    } else {
		Patch p = getPatchAt(x, y);
		killTurtlesHere(p);
	    }
	    mousedownx = mousedragx;
	    mousedowny = mousedragy;
	}
    }

    /* Paint Tool: Eye Dropper */

    void eyeDropperMousePressed(int x, int y) {
	Patch p = getPatchAt(x, y);
	if (parent.paintselect().buttonDown == PaintSelect.PATCH) {
	    if (parent.colortool().curcolor != (int)p.pc) {
	    	parent.colortool().setColor((int)p.pc);
	    }
	}
	else {
	    Turtle turtle = p.topturtle;
	    if (turtle != null) {
		if (turtle.color != parent.colortool().curcolor) {
		    parent.colortool().setColor((int)turtle.color);
		}
	    }
	}
    }


    void eyeDropperMouseDragged(int x, int y) {
	Patch p = getPatchAt(x, y);
	if (parent.paintselect().buttonDown == PaintSelect.PATCH) {
	    parent.colortool().setColor((int)p.pc);
	}
	else {
	    Turtle turtle = p.topturtle;
	    if (turtle != null) {
		parent.colortool().setColor((int)turtle.color);
	    }
	}
    }

    /* Paint Tool: Rectangle */

    void rectangleMousePressed(int x, int y) {
	mousedownx = calcX1(x);
	mousedowny = calcY1(y);
	mousedragx = mousedownx;
	mousedragy = mousedowny;
	patchSetOne(mousedownx, mousedowny, parent.colortool().curcolor, true);
    }


    void patchBox(int x1, int y1, int x2, int y2, double color, boolean xorp) {
	if (x1 > x2) {
	    int xtemp = x1;
	    x1 = x2;
	    x2 = xtemp;
	}
	if (y1 > y2) {
	    int ytemp = y1;
	    y1 = y2;
	    y2 = ytemp;
	}
	int patchOffset = adjustOffset(x1 - 1, y1);
	int yIncrease = sl.widthp2 + x1 - x2;
	int xIncrease = x2 - x1;
	if (x1 == x2) {
	    patchLine(x1, y1, x2, y2, color, xorp);
	} else {
	    for(int x = x1; x <= x2; x++) {
		patchOffset++;
		setDot(patchOffset, color, xorp);
	    }
	    if (y2 > (y1 + 1)) {
		for(int y = y1; y <= (y2 - 2); y++) {
		    patchOffset += yIncrease;
		    setDot(patchOffset, color, xorp);
		    patchOffset += xIncrease;
		    setDot(patchOffset, color, xorp);
		}
	    }
	    if (y2 > y1) {
		patchOffset += yIncrease;
		for(int x = x1; x <= x2; x++) {
		    setDot(patchOffset, color, xorp);
		    patchOffset++;
		}
	    }
	}
    }

    int setSquare(int center, int old, int centerDisp, int offDisp) {
	if (center == old)
	    return center;
	else {
	    double diff = old - center;
	    int sign = (diff > 0) ? 1 : -1;
	    return center + Math.abs(centerDisp - offDisp) * sign;
	}
    }

    void rectangleMouseDragged(int x, int y, boolean shiftp) {
	int x3 = calcX1(x);
	int y3 = calcY1(y);
	if (shiftp) {
	    if (Math.abs(x3 - mousedownx) > Math.abs(y3 - mousedowny)) {
		x3 = setSquare(mousedownx, x3, mousedowny, y3);
		y3 = setSquare(mousedowny, y3, mousedownx, x3);
	    }
	}
	
	if (! (mousedragx == x3 && mousedragy == y3)) {
	    double color = parent.colortool().curcolor;
	    patchBox(mousedownx, mousedowny, 
		     mousedragx, mousedragy, 
		     0 - color, true);
	    patchBox(mousedownx, mousedowny,
		     x3, y3,
		     color, true);
	    mousedragx = x3;
	    mousedragy = y3;
	}
    }

    void rectangleMouseReleased(int x, int y) {
	double color = parent.colortool().curcolor;
	patchBox(mousedownx, mousedowny, 
		  mousedragx, mousedragy, 
		  0 - color, true);
	patchOverride = false;
	patchBox(mousedownx, mousedowny,
		  mousedragx, mousedragy,
		  color, false);
    }

    /* Paint Tool: Fill Rectangle */

    void fillRectangleMousePressed(int x, int y) {
	mousedownx = calcX1(x);
	mousedowny = calcY1(y);
	mousedragx = mousedownx;
	mousedragy = mousedowny;
	patchSetOne(mousedownx, mousedowny, parent.colortool().curcolor, true);
    }

    void patchBoxFilled(int x1, int y1, int x2, int y2, double color, boolean xorp) {
	if (x1 > x2) {
	    int xtemp = x1;
	    x1 = x2;
	    x2 = xtemp;
	}
	if (y1 > y2) {
	    int ytemp = y1;
	    y1 = y2;
	    y2 = ytemp;
	}
	int patchOffset = adjustOffset(x1, y1);
	int yIncrease = sl.width + 1 + x1 - x2;
	for(int y = y1; y <= y2; y++) {
	    for(int x = x1; x <= x2; x++) {
		setDot(patchOffset, color, xorp);
		patchOffset++;
	    }
	    patchOffset += yIncrease;
	}
    }


    void fillRectangleMouseDragged(int x, int y, boolean shiftp) {
	int x3 = calcX1(x);
	int y3 = calcY1(y);
	if (shiftp) {
	    if (Math.abs(x3 - mousedownx) > Math.abs(y3 - mousedowny)) {
		x3 = setSquare(mousedownx, x3, mousedowny, y3);
		y3 = setSquare(mousedowny, y3, mousedownx, x3);
	    }
	}
	
	if (! (mousedragx == x3 && mousedragy == y3)) {
	    double color = parent.colortool().curcolor;
	    patchBoxFilled(mousedownx, mousedowny, 
			   mousedragx, mousedragy, 
			   0 - color, true);
	    patchBoxFilled(mousedownx, mousedowny,
			   x3, y3,
			   color, true);
	    mousedragx = x3;
	    mousedragy = y3;
	}
    }
    
    void fillRectangleMouseReleased(int x, int y) {
	double color = parent.colortool().curcolor;
	patchBoxFilled(mousedownx, mousedowny, 
		       mousedragx, mousedragy, 
		       0 - color, true);
	patchOverride = false;
	patchBoxFilled(mousedownx, mousedowny,
		       mousedragx, mousedragy,
		       color, false);
    }


    /* Paint Tool: Oval */

    void ovalMousePressed(int x, int y) {
	mousedownx = calcX1(x);
	mousedowny = calcY1(y);
	mousedragx = mousedownx;
	mousedragy = mousedowny;
	patchSetOne(mousedownx, mousedowny, parent.colortool().curcolor, true);
    }

    boolean checkCoorX(double coor) {
	return (coor >= 0.0) && (coor < (sl.width + 0.0));
    }

    boolean checkCoorY(double coor) {
	return (coor >= 0.0) && (coor < (sl.height + 0.0));
    }


    void patchPaint4(double x, double y, double xx, double yy, double color, boolean xorp) {
	double xplus = x + xx;
	double xminus = x - xx;
	double yplus = y + yy;
	double yminus = y - yy;
	if (checkCoorX(xplus) && checkCoorY(yplus)) {
	    setDot(adjustOffset((int)xplus, (int)yplus), color, xorp);
	}
	if (checkCoorX(xminus) && checkCoorY(yplus)) {
	    setDot(adjustOffset((int)xminus, (int)yplus), color, xorp);
	}
	if (checkCoorX(xplus) && checkCoorY(yminus)) {
	    setDot(adjustOffset((int)xplus, (int)yminus), color, xorp);
	}
	if (checkCoorX(xminus) && checkCoorY(yminus)) {
	    setDot(adjustOffset((int)xminus, (int)yminus), color, xorp);
	}

    }

    double ellipseFunction(double y, double a, double b) {
	return Math.round(a * Math.sqrt(1 - ((y * y) / (b * b))));
    }

    void patchCircle(int x1, int y1, int x2, int y2, double color, boolean xorp) {
	double xc = (double)(x1 + x2) / 2.0;
	double yc = (double)(y1 + y2) / 2.0;
	double a = Math.abs( (double)(x1 - x2) / 2.0);
	double b = Math.abs( (double)(y1 - y2) / 2.0);
	double xCorrection = (double)((x1 + x2) % 2) / 2.0;
	if (b == 0.0) {
	    patchLine(x1, y1, x2, y2, color, xorp);
	} else {
	    for(double y = (0.0 - b); y <= 0.0; y++) {
		for(double x = ellipseFunction(Math.max(0.0 - b, y - 0.5), a, b);
		    x <= Math.min(a, ellipseFunction(y + 0.5, a, b));
		    x++) {
		    patchPaint4(xc, yc, x + xCorrection, y, color, xorp);
		}
	    }
	}
    }

    void ovalMouseDragged(int x, int y, boolean shiftp) {
	int x3 = calcX1(x);
	int y3 = calcY1(y);
	if (shiftp) {
	    if (Math.abs(x3 - mousedownx) > Math.abs(y3 - mousedowny)) {
		x3 = setSquare(mousedownx, x3, mousedowny, y3);
		y3 = setSquare(mousedowny, y3, mousedownx, x3);
	    }
	}
	
	if (! (mousedragx == x3 && mousedragy == y3)) {
	    double color = parent.colortool().curcolor;
	    patchCircle(mousedownx, mousedowny, 
			mousedragx, mousedragy, 
			0 - color, true);
	    patchCircle(mousedownx, mousedowny,
			   x3, y3,
			   color, true);
	    mousedragx = x3;
	    mousedragy = y3;
	}
    }

    void ovalMouseReleased(int x, int y) {
	double color = parent.colortool().curcolor;
	patchCircle(mousedownx, mousedowny, 
		    mousedragx, mousedragy, 
		    0 - color, true);
	patchOverride = false;
	patchCircle(mousedownx, mousedowny,
		    mousedragx, mousedragy,
		    color, false);
    }


    /* Paint Tool: Fill Oval */

    void setDotCheck(int patchOffset, double color, boolean xorp) {
	if (patchOffset > 0 &&
	    patchOffset < (sl.widthp2 * (sl.height + 1))) {
	    setDot(patchOffset, color, xorp);
	}
    }

    void patchQuick2Lines(double x1, double y1, double xc, double yc,
			  double color, boolean xorp) {
	int patchOffset1 = adjustOffset((int)Math.max(0.0, xc - x1), (int)(yc + y1));
	int patchOffset2 = adjustOffset((int)Math.max(0.0, xc - x1), (int)(yc - y1));
	double ending = Math.min(sl.width + 0.0, xc + x1);

	if (y1 == 0.0) {
	    for(double x = Math.max(0, xc - x1); x <= ending; x++) {
		setDotCheck(patchOffset1, color, xorp);
		patchOffset1++;
	    }
	} else {
	    for(double x = Math.max(0, xc - x1); x <= ending; x++) {
		setDotCheck(patchOffset1, color, xorp);
		setDotCheck(patchOffset2, color, xorp);
		patchOffset1++;
		patchOffset2++;
	    }
	}
    }

    void patchCircleFilled(int x1, int y1, int x2, int y2, 
			   double color, boolean xorp) {
	double xc = (double)(x1 + x2) / 2.0;
	double yc = (double)(y1 + y2) / 2.0;
	double a = Math.abs( (double)(x1 - x2) / 2.0);
	double b = Math.abs( (double)(y1 - y2) / 2.0);
	double xCorrection = (double)((x1 + x2) % 2) / 2.0;
	if (b == 0.0) {
	    patchLine(x1, y1, x2, y2, color, xorp);
	} else {
	    for(double y = (0.0 - b); y <= 0.0; y++) {
		patchQuick2Lines(Math.min(a, xCorrection + ellipseFunction(y + 0.5, a, b)),
				 y, xc, yc, color, xorp);
	    }
	}
    }

    void fillOvalMouseDragged(int x, int y, boolean shiftp) {
	int x3 = calcX1(x);
	int y3 = calcY1(y);
	if (shiftp) {
	    if (Math.abs(x3 - mousedownx) > Math.abs(y3 - mousedowny)) {
		x3 = setSquare(mousedownx, x3, mousedowny, y3);
		y3 = setSquare(mousedowny, y3, mousedownx, x3);
	    }
	}
	
	if (! (mousedragx == x3 && mousedragy == y3)) {
	    double color = parent.colortool().curcolor;
	    patchCircleFilled(mousedownx, mousedowny, 
			      mousedragx, mousedragy, 
			      0 - color, true);
	    patchCircleFilled(mousedownx, mousedowny,
			      x3, y3,
			      color, true);
	    mousedragx = x3;
	    mousedragy = y3;
	}
    }

    void fillOvalMouseReleased(int x, int y) {
	double color = parent.colortool().curcolor;
	patchCircleFilled(mousedownx, mousedowny, 
			  mousedragx, mousedragy, 
			  0 - color, true);
	patchOverride = false;
	patchCircleFilled(mousedownx, mousedowny,
			  mousedragx, mousedragy,
			  color, false);
    }


    /* Paint Tool: Line */

    double carlosXOR(double c1, double c2) {
	double col = (c1 + c2) % Colors.numcolors;
	if (col < 0) col += Colors.numcolors;
	assert col >= 0.0 && col < Colors.numcolors : "Color3 is illegal: color1 " + c1 + " + color2 " + c2 + " = color3 " + col;
	return col;
    }

    double getDot(int patchOffset) {
	Patch p = sl.patches[patchOffset];
	if (parent.paintselect().buttonDown == PaintSelect.PATCH) {
	    return p.pc;
	} else {
	    Turtle turtle = p.topturtle;
	    if (turtle != null) {
		return turtle.color;
	    } else {
		return p.pc;
	    }
	}
    }

    void setDot(int patchOffset, double color, boolean xorp) {
	Patch p = sl.patches[patchOffset];
	color = (xorp) ? carlosXOR(p.pc, color) : color;
	if (parent.paintselect().buttonDown == PaintSelect.PATCH ||
	    color > Colors.numcolors ||
	    patchOverride) {
	    p.pc = color;
	    if (p.topturtle != null) {
		drawTurtle(p.topturtle);
	    } else {
		drawPatch(p);
	    }
	} else {
	    Turtle t = sl.cct(p.xcor - 1 - sl.halfwidth, sl.halfheight - (p.ycor - 1),
			      0, color, 0, 0);
	    undoTurtleArray.add(t);
	}

    }

    int adjustOffset(int x, int y) {
	return sl.widthp2 * (y + 1) + (x + 1);
    }

    void patchSetOne(int x, int y, double color, boolean xorp) {
	int patchOffset = adjustOffset(x, y);
	setDot(patchOffset, color, xorp);
    }

    int dirOfCH(int y1, int y2) {
	return (y1 > y2) ? 1 : -1;
    } 

    void patchLine(int x1, int y1, int x2, int y2, double color, boolean xorp) {
	//System.out.println("patch line: from: (" + x1 + ", " + y1 + ") to (" + 
	//x2 + ", " + y2 + ")");
	// Make sure x2 is bigger than x1
	if (x1 > x2) {
	    int xtemp = x1;
	    x1 = x2;
	    x2 = xtemp;
	    int ytemp = y1;
	    y1 = y2;
	    y2 = ytemp;
	}
	int patchOffset = adjustOffset(x1, y1);
	double xAdvanced = x1;
	double xIncrement = Math.min(x2 + 1 - x1, 
				     (double)(1 + (x2 - x1)) / 
				     (double)(1 + Math.abs(y2 - y1)));
	int yIncrease = dirOfCH(y2, y1) * sl.widthp2;
	int yDist = Math.abs(y2 - y1);
    //System.out.println("xInc " + xIncrement + " yInc " + yIncrease + 
    //		   " yDist " + yDist);
	for(int y = 0; y <= yDist; y++) {
	    //    System.out.println("patch offset = " + patchOffset);
	    double nextAdvance = xAdvanced + xIncrement;
	    //System.out.println("y = " + y + " xAdvanced = " + xAdvanced 
	    //		       + " nextAdvance = " + nextAdvance);
	    if ((int)xAdvanced == (int)nextAdvance) {
		//System.out.println("place dot");
		setDot(patchOffset, color, xorp);
	    } else {
		for(int x = (int)xAdvanced; x <= ((int)nextAdvance - 1); x++) {
		    //  System.out.println("move dot over by 1");
		    setDot(patchOffset, color, xorp);
		    patchOffset++;
		}
		//System.out.println("done moving dots over");
	    }
	    xAdvanced = nextAdvance;
	    patchOffset += yIncrease;
	}
    }

    void lineMousePressed(int x, int y) {
	mousedownx = calcX1(x);
	mousedowny = calcY1(y);
	mousedragx = mousedownx;
	mousedragy = mousedowny;
	patchSetOne(mousedownx, mousedowny, parent.colortool().curcolor, true);
    }

    void lineMouseDragged(int x, int y, boolean shiftp) {
	int x3 = calcX1(x);
	int y3 = calcY1(y);
	if (shiftp) {
	    if (Math.abs(x3 - mousedownx) > Math.abs(y3 - mousedowny)) {
		x3 = mousedownx;
		y3 = mousedowny;
	    }
	}
	if (! (mousedragx == x3 && mousedragy == y3)) {
	    double color = parent.colortool().curcolor;
	    patchLine(mousedownx, mousedowny,
		      mousedragx, mousedragy, 
		      0 - color, true);
	    patchLine(mousedownx, mousedowny,
		      x3, y3,
		      color, true);
	    mousedragx = x3;
	    mousedragy = y3;
	}
    }
	    
    void lineMouseReleased(int x, int y) {
	int color = parent.colortool().curcolor;
	patchLine(mousedownx, mousedowny, 
		  mousedragx, mousedragy, 
		  0 - color, true);
	patchOverride = false;
	patchLine(mousedownx, mousedowny,
		  mousedragx, mousedragy,
		  color, false);
    }


    /* Paint Tool: Polygon */

    int polyStartX = 0;
    int polyStartY = 0;
    int polyLineStartX = 0;
    int polyLineStartY = 0;
    int polyLineEndX = 0;
    int polyLineEndY = 0;
    
    void polygonMousePressed(int x, int y) {
	//System.out.println("poly mouse pressed");
	double color = parent.colortool().curcolor;
	int x1 = calcX1(x);
	int y1 = calcY1(y);
	if (startOfPoly) {
	    patchLine(polyLineStartX, polyLineStartY,
		      polyLineEndX, polyLineEndY,
		      0 - color, true);
	    patchOverride = false;
	    patchLine(polyLineStartX, polyLineStartY,
		      x1, y1,
		      color, false);
	    polyLineStartX = x1;
	    polyLineStartY = y1;
	    polyLineEndX = polyLineStartX;
	    polyLineEndY = polyLineStartY;
	    patchOverride = true;
	    patchSetOne(polyLineStartX, polyLineStartY, color, true);
	} else {
	    startOfPoly = true;
	    polyStartX = x1;
	    polyStartY = y1;
	    polyLineStartX = polyStartX;
	    polyLineStartY = polyStartY;
	    polyLineEndX = polyLineStartX;
	    polyLineEndY = polyLineStartY;
	    patchSetOne(polyLineStartX, polyLineStartY, color, true);
	}
    }


    // If they click out of the window, mark it completed
    boolean polygonMouseMoved(int x, int y) {
	//System.out.println("polygon mouse moved");
	if (!startOfPoly) return false;
	double color = parent.colortool().curcolor;
	int x1 = calcX1(x);
	int y1 = calcY1(y);
	if (!(polyLineEndX == x1 && polyLineEndY == y1)) {
	    patchLine(polyLineStartX, polyLineStartY,
		      polyLineEndX, polyLineEndY,
		      0.0 - color, true);
	    polyLineEndX = x1;
	    polyLineEndY = y1;
	    patchLine(polyLineStartX, polyLineStartY,
		      polyLineEndX, polyLineEndY,
		      color, true);
	}
	return true;
    }

    void polygonMouseDragged(int x, int y) {
	return;
    }

    void polygonMouseReleased(int x, int y) {
	return;
    }

    void polygonMouseDoubleClicked(int x, int y) {
	//System.out.println("mouse double-clicked");
	if (startOfPoly) {
	    double color = parent.colortool().curcolor;
	    int x1 = calcX1(x); 
	    int y1 = calcY1(y);
	    patchOverride = true;
	    patchLine(polyLineStartX, polyLineStartY,
		      polyLineEndX, polyLineEndY,
		      0.0 - color, true);
	    patchOverride = false;
	    patchLine(polyStartX, polyStartY, x1, y1, color, false);
	    patchLine(polyLineStartX, polyLineStartY, x1, y1, color, false);
	    startOfPoly = false;
	}
    }
		      
    void polygonCancelPoly() {
	if (startOfPoly) {
	    //System.out.println("cancel polygon tool");
	    // XXX make this like undo
	    startOfPoly = false;
	    parent.maybeReleasePatchCanvas();
	}
    }
	    
    LinkedList patchFillerList = new LinkedList();
    double patchFillerOldColor;
    double patchFillerNewColor;

    void fillArea(int x, int y) {
	mousedownx = calcX1(x);
	mousedowny = calcY1(y);
	patchOverride = false;
	int patchOffset = adjustOffset(mousedownx, mousedowny);
	patchFillerOldColor = getDot(patchOffset);
	patchFillerNewColor = parent.colortool().curcolor;
	patchFillerList.clear();
	patchFillerList.addFirst(new Integer(patchOffset));
	
	if (patchFillerOldColor != patchFillerNewColor) {
	    patchBox(-1, -1, sl.width, sl.height, 200.0, false);
	    setDot( ((Integer)patchFillerList.getFirst()).intValue(), patchFillerNewColor,
		    false);
	    patchFillIter();
	}
    }
    
    void patchFillIter() {
	if (patchFillerList.isEmpty()) return;
	int currentDot = ((Integer)patchFillerList.getFirst()).intValue();
	patchFillerList.removeFirst();
	patchFillIterHelper(currentDot + 1);
	patchFillIterHelper(currentDot - 1);
	patchFillIterHelper(currentDot + sl.widthp2);
	patchFillIterHelper(currentDot - sl.widthp2);
	patchFillIter();
    }

    void patchFillIterHelper(int patchOffset) {
	if (patchFillerOldColor == getDot(patchOffset)) {
	    patchFillerList.addFirst(new Integer(patchOffset));
	    setDot(patchOffset, patchFillerNewColor, false);
	}
    }

    Turtle touchingTurtle(int x, int y) {
	double patchx = x / sl.cellsize;
	double patchy = y / sl.cellsize;
	int patch = sl.patchat(patchx, patchy);
	return sl.patches[patch].topturtle;
    }

    Patch touchingPatch(int x, int y) {
	double patchx = x / sl.cellsize;
	double patchy = y / sl.cellsize;
	int patch = sl.patchat(patchx, patchy);
	return sl.patches[patch];
    }

    Patch getPatchAt(int x, int y) {
	double patchx = (x-position.x)/sl.cellsize;
	double patchy = (y-position.y)/sl.cellsize;
	int patch = sl.patchat(patchx, patchy);
	return sl.patches[patch];
    }

    public void readImage(StarLogo sl, ImageProducer ip, int width, int height, Pictures pictures) {
	Thread.currentThread().yield();
	if ((width != sl.width || height != sl.height) && !loadCanceled) {
	    sl.resizePatches(width / 2, height / 2, sl.cellsize);
	}
	Thread.currentThread().yield();
	sl.patchcanvas.loadImage(ip, pictures, width, height);
    }

    public void loadImage(ImageProducer ip, Pictures pictures, int width, int height) {
	if (!PatchCanvas.loadCanceled) {
	    this.ip = ip;
	    ip.addConsumer(this);
	    this.pictures = pictures;
	    setDimensions(width, height);
	    ip.startProduction(this);
	}
    }

    public void cancelImageLoad() {
	PatchCanvas.loadCanceled = true;
	if (ip != null) ip.removeConsumer(this);
	ip = null;
	rerror = null;
	gerror = null;
	berror = null;
	uptodate = false;
	parent.repaint(position);
    }

    ImageProducer ip;
    Pictures pictures;
    public static boolean loadCanceled;

    double[] rerror, gerror, berror;

    int widthIP, heightIP;

    public void imageComplete(int status) {
	ip.removeConsumer(this);
	ip = null;
	rerror = null;
	gerror = null;
	berror = null;
	uptodate = false;
	parent.repaint(position);
	//if (pictures != null) pictures.clearLoadingMessage();
    }

    public void setColorModel(ColorModel cm) {
    }

    public void setDimensions(int width, int height) {
	rerror = new double[width * height];
	gerror = new double[width * height];
	berror = new double[width * height];
	widthIP = width;
	heightIP = width;
    }

    public void setHints(int hints) {
    }

    public void setPixels(int xcor, int ycor, int width, int height, ColorModel cm,
			  byte[] pixels, int offset, int scansize) {
	try {
	    int poffy = (sl.widthp2 * (ycor + 1)) + xcor + 1;
	    int eoffy = widthIP * ycor + xcor;
	    //for use in the loop
	    byte pixel;
	    int rgb, r, g, b, newr, newg, newb, rgbpluserror, rgbmapped, mapr, mapg, mapb, dr, dg, db;
	    double starcolormapped;
	    for(int y = ycor; y < ycor + height; y++) {
		for(int x = xcor; x < xcor + width; x++) {
		    //System.out.println("pixel pos: " + (((y - ycor) * scansize) + x - xcor + offset));
		    pixel = pixels[((y - ycor) * scansize) + x - xcor + offset]; // this is right, java docs are extrememly misleading about this
		    rgb = cm.getRGB((int)pixel & 0xFF); //sign extending can cause problem, bug in java sometimes sends the bytes unsigned?
		    r = ((rgb >> 16) & 0xFF);
		    g = ((rgb >> 8) & 0xFF);
		    b = (rgb & 0xFF);
		    newr = (int)(r + rerror[eoffy + x]); if (newr > 255) newr = 255; if (newr < 0) newr = 0;
		    newg = (int)(g + gerror[eoffy + x]); if (newg > 255) newg = 255; if (newg < 0) newg = 0;
		    newb = (int)(b + berror[eoffy + x]); if (newb > 255) newb = 255; if (newb < 0) newb = 0;
		    rgbpluserror = (newr << 16) | (newg << 8) | newb;
		    starcolormapped = Colors.mapColorToStarLogoColor(rgbpluserror);

		    sl.patches[poffy + x].pc = starcolormapped;

		    rgbmapped = Colors.colorarrayints[(int)(starcolormapped * 32.0)];
		    mapr = ((rgbmapped >> 16) & 0xFF);
		    mapg = ((rgbmapped >> 8) & 0xFF);
		    mapb = (rgbmapped & 0xFF);
		    dr = newr - mapr;
		    dg = newg - mapg;
		    db = newb - mapb;

		    // Propogate error
		    if (x < width-1) {
			rerror[eoffy + x + 1] = (double)dr * (7.0 / 16.0);
			gerror[eoffy + x + 1] = (double)dg * (7.0 / 16.0);
			berror[eoffy + x + 1] = (double)db * (7.0 / 16.0);
			if (y < height - 1) {
			    rerror[eoffy + x + widthIP + 1] = (double)dr * (1.0 / 16.0);
			    gerror[eoffy + x + widthIP + 1] = (double)dg * (1.0 / 16.0);
			    berror[eoffy + x + widthIP + 1] = (double)db * (1.0 / 16.0);
			}
		    }
		    if (y < height - 1) {
			rerror[eoffy + x + widthIP] = (double)dr * (5.0 / 16.0);
			gerror[eoffy + x + widthIP] = (double)dg * (5.0 / 16.0);
			berror[eoffy + x + widthIP] = (double)db * (5.0 / 16.0);
			if (x > 1) {
			    rerror[eoffy + x + widthIP - 1] = (double)dr * (3.0 / 16.0);
			    gerror[eoffy + x + widthIP - 1] = (double)dg * (3.0 / 16.0);
			    berror[eoffy + x + widthIP - 1] = (double)db * (3.0 / 16.0);
			}
		    }
		}
		poffy += sl.widthp2;
		eoffy += widthIP;
	    }
	} catch(Exception e) { e.printStackTrace(); }
    }

    public void setPixels(int xcor, int ycor, int width, int height, ColorModel cm,
			  int[] pixels, int offset, int scansize) {
	try {
	    int poffy = (sl.widthp2 * (ycor + 1)) + xcor + 1;
	    int eoffy = widthIP * ycor + xcor;
	    //for use in the loop
	    int pixel, rgb, r, g, b, newr, newg, newb, rgbpluserror, rgbmapped, mapr, mapg, mapb, dr, dg, db;
	    double starcolormapped;
	    for(int y = ycor; y < ycor + height; y++) {
		for(int x = xcor; x < xcor + width; x++) {
		    pixel = pixels[((y - ycor) * scansize) + x - xcor + offset]; // this is right, java docs are extremely misleading about this
		    rgb = cm.getRGB(pixel);
		    r = ((rgb >> 16) & 0xFF);
		    g = ((rgb >> 8) & 0xFF);
		    b = (rgb & 0xFF);
		    newr = (int)(r + rerror[eoffy + x]); if (newr > 255) newr = 255; if (newr < 0) newr = 0;
		    newg = (int)(g + gerror[eoffy + x]); if (newg > 255) newg = 255; if (newg < 0) newg = 0;
		    newb = (int)(b + berror[eoffy + x]); if (newb > 255) newb = 255; if (newb < 0) newb = 0;
		    rgbpluserror = (newr << 16) | (newg << 8) | newb;
		    starcolormapped = Colors.mapColorToStarLogoColor(rgbpluserror);

		    sl.patches[poffy + x].pc = starcolormapped;

		    rgbmapped = Colors.colorarrayints[(int)(starcolormapped * 32.0)];
		    mapr = ((rgbmapped >> 16) & 0xFF);
		    mapg = ((rgbmapped >> 8) & 0xFF);
		    mapb = (rgbmapped & 0xFF);
		    dr = newr - mapr;
		    dg = newg - mapg;
		    db = newb - mapb;

		    // Propogate error
		    if (x < width-1) {
			rerror[eoffy + x + 1] = (double)dr * (7.0 / 16.0);
			gerror[eoffy + x + 1] = (double)dg * (7.0 / 16.0);
			berror[eoffy + x + 1] = (double)db * (7.0 / 16.0);
			if (y < height - 1) {
			    rerror[eoffy + x + widthIP + 1] = (double)dr * (1.0 / 16.0);
			    gerror[eoffy + x + widthIP + 1] = (double)dg * (1.0 / 16.0);
			    berror[eoffy + x + widthIP + 1] = (double)db * (1.0 / 16.0);
			}
		    }
		    if (y < height - 1) {
			rerror[eoffy + x + widthIP] = (double)dr * (5.0 / 16.0);
			gerror[eoffy + x + widthIP] = (double)dg * (5.0 / 16.0);
			berror[eoffy + x + widthIP] = (double)db * (5.0 / 16.0);
			if (x > 1) {
			    rerror[eoffy + x + widthIP - 1] = (double)dr * (3.0 / 16.0);
			    gerror[eoffy + x + widthIP - 1] = (double)dg * (3.0 / 16.0);
			    berror[eoffy + x + widthIP - 1] = (double)db * (3.0 / 16.0);
			}
		    }
		}
		poffy += sl.widthp2;
		eoffy += widthIP;
	    }
	}catch (Exception e)  { e.printStackTrace(); }
    }

    public void setProperties(Hashtable hash) {}
}



