/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.awt.*;
import java.awt.image.*;
import yoyo.LogoError;
import java.io.*;
import yoyo.YoYo;

public class Pictures {

    public final static int INTERFACE = 0;
    public final static int PATCHES = 1;
    public final static int TURTLES_AND_PATCHES = 2;

    public void loadPicture(StarLogo sl) {
	try {loadPictureInternal(sl);}
	catch(Exception e) {
	    e.printStackTrace(System.err);
	    Application.createMessageDialog(sl.app, sl.app.activeFrame, 
					    sl.app.textBundle.getString("starlogo_dialog_title"), 
					    sl.app.textBundle.getString("error_loading_image") + e.getMessage(), false);
	}
    }

    public void loadPicture(StarLogo sl, String filename) {
	SLURL u = new SLURL(sl.app.project.curDir); 
	u.setFile(filename);
	try {loadPictureInternal(sl, u);}
	catch(Exception e) {
	    e.printStackTrace(System.err);
	    Application.createMessageDialog(sl.app, sl.app.activeFrame, 
					    sl.app.textBundle.getString("starlogo_dialog_title"), 
					    sl.app.textBundle.getString("error_loading_image") + e.getMessage(), false);
	}
    }
    
    public void VMloadPicture(StarLogo sl) {
	try {loadPictureInternal(sl);}
	catch(Exception e) {
	    e.printStackTrace(System.err);
	    throw new LogoError(sl.app.textBundle.getString("error_loading_image") + e.getMessage());
	}
    }



    public void VMloadPicture(StarLogo sl, String filename) {
	SLURL u = new SLURL(sl.app.project.curDir); 
	u.setFile(filename);
	try {loadPictureInternal(sl, u);}
	catch(Exception e) {
	    e.printStackTrace(System.err);
	    throw new LogoError(sl.app.textBundle.getString("error_loading_image") + e.getMessage());
	}
    }

    public void loadPictureInternal(StarLogo sl) {
	SLURL filename = getLoadFileName(sl);
	if (filename != null) loadPictureInternal(sl, filename);
    }

    public void loadPictureInternal(StarLogo sl, SLURL filename) {
	Image picture = getImage(sl, filename);	
	int width = picture.getWidth(null);
	int height = picture.getHeight(null);
	if ((width < 1) || (height < 1)) 
	    StarLogoError.error(sl.app, sl.app.textBundle.getString("error_loading_image") + filename, 
				true);
	loadPictureInternal(sl, picture, width, height);
    }

    protected void loadPictureInternal(StarLogo sl, Image image, int width, int height) { 
	sl.patchcanvas.readImage(sl, image.getSource(), width, height, this);
    }
    

    protected SLURL getLoadFileName(StarLogo sl) {
	FileDialog d = new FileDialog((Frame)sl.app.activeFrame, 
				      sl.app.textBundle.getString("load_picture"), 
				      FileDialog.LOAD);
	if (sl.app.project.curDir!=null) {
	    d.setDirectory(sl.app.project.curDir.getPath());
	}
	d.setFilenameFilter(sl.app.pictureFileFilter);
	d.show();
	String curDir = d.getDirectory();
    	sl.app.project.curDir = new SLURL(SLURL.FILE, curDir);
    	if (d.getFile() != null) {
	    SLURL newname = new SLURL(SLURL.FILE, sl.app.project.curDir.getPath(),
				      d.getFile());
	    return newname;
	} else return null;
    }
    

    protected Image getImage(StarLogo sl, SLURL filename) {
	byte[] filebytes = filename.loadBinary(sl.c);
	Toolkit kit = Toolkit.getDefaultToolkit();
	MediaTracker tracker = new MediaTracker((Component)sl.app.iface);
	Image im = kit.createImage(filebytes);
	tracker.addImage(im, 0);
	try {
	    tracker.waitForAll();
	} catch (InterruptedException ie) {}
	return im;
    }
    


    /* --------- Saving Pictures ---------- */

    public void savePicture(StarLogo sl, int which) {
	try {savePicture(sl, getImage(sl, which));}
	catch(Exception e) {
	    e.printStackTrace(System.err);
	    Application.createMessageDialog(sl.app, sl.app.activeFrame, 
					    sl.app.textBundle.getString("starlogo_dialog_title"), 
					    sl.app.textBundle.getString("error_saving_image_to_file") + 
					    e.getMessage(), 
					    false);
	}
    }

    public void savePicture(StarLogo sl, String filename, int which) {
	SLURL u = new SLURL(sl.app.project.curDir); 
	u.setFile(filename);
	try {savePicture(sl, u, getImage(sl, which));}
	catch(Exception e) {
	    e.printStackTrace(System.err);
	    Application.createMessageDialog(sl.app, sl.app.activeFrame, 
					    sl.app.textBundle.getString("starlogo_dialog_title"), 
					    sl.app.textBundle.getString("error_saving_image_to_file") + 
					    e.getMessage(), 
					    false);
	}
    }

    public void VMsavePicture(StarLogo sl, int which) {
	try { savePicture(sl, getImage(sl, which));}
	catch(Exception e) {
	    e.printStackTrace(System.err);
	    throw new LogoError(sl.app.textBundle.getString("error_saving_image_to_file") + e.getMessage());
	}
    }

    public void VMsavePicture(StarLogo sl, String filename, int which) {
	SLURL u = new SLURL(sl.app.project.curDir); 
	u.setFile(filename);
	try { savePicture(sl, u, getImage(sl, which));}
	catch(Exception e) {
	    e.printStackTrace(System.err);
	    throw new LogoError(sl.app.textBundle.getString("error_saving_image_to_file") + e.getMessage());
	}
    }
    

    public void savePicture(StarLogo sl, Image image) throws Exception {
	if (image == null) return;
	SLURL filename = getSaveFileName(sl);
	if (filename != null) savePicture(sl, filename, image);
    }

    protected void savePicture(StarLogo sl, SLURL filename, Image image) throws Exception {
	if (!filename.getFileExtension().toLowerCase().endsWith("png")) {
	    filename.setFileExtension("png");
	    if (filename.exists()) {
		MessageDialog df = Application.createMessageDialog(sl.app, sl.app.activeFrame, 
								   sl.app.textBundle.getString("warning_already_exists"), 
								   sl.app.textBundle.getString("warning_the_file") + 
								   filename + '\n' +
								   sl.app.textBundle.getString("continuing_will_overwrite") + ".", 
								   true, true, true, false, 
								   MessageDialog.CANCEL_BUTTON); 
		df.show();
		if (df.canceled()) {
		    df.dispose();
		    return;
		}
		df.dispose();
	    }
	}
	File file = new File(filename.getFullPath());
	BufferedImage bi = toBufferedImage(image);
	javax.imageio.ImageIO.write(bi, "png", file);
    }

    protected Image getImage(StarLogo sl, int which) {
	Image image = null;
	Graphics g;
	switch(which) {
	case PATCHES: 
	    int cellsize = sl.cellsize;
	    image = sl.app.iface.getGraphicsConfiguration().createCompatibleImage(sl.width*cellsize, 
										  sl.height*cellsize);
	    g = image.getGraphics();
	    int x, y, poffy, xStep, yStep;
	    for (y = 0, yStep = 0, poffy=sl.widthp2; 
		 y < sl.height; 
		 y++, yStep+=cellsize, poffy+=sl.widthp2) {
		int poffset = poffy + 1;
		for (x = 0, xStep = 0; x<sl.width; x++, xStep+=cellsize) {
		    Patch patch = sl.patches[poffset++];
		    if (patch.pc != 200.0) {
			g.setColor(Colors.colorarray[(int)(patch.pc * 32.0)]);
			g.fillRect(xStep, yStep, cellsize, cellsize);
		    }
		}
	    }
	    Toolkit.getDefaultToolkit().sync();
	    g.dispose();
	    break;
	case TURTLES_AND_PATCHES:
	    if (sl.patchcanvas instanceof PatchCanvas) {
		PatchCanvas pc = (PatchCanvas)sl.patchcanvas;
		image = pc.parent.getGraphicsConfiguration().createCompatibleImage(pc.getPosition().width, 
										   pc.getPosition().height);
		g = image.getGraphics();
		pc.updateGraphics(g);
		g.dispose();
	    } 
	    break;
	case INTERFACE:
	    image = sl.app.iface.getGraphicsConfiguration().createCompatibleImage(sl.app.iface.getSize().width, 
										  sl.app.iface.getSize().height);
	    g = image.getGraphics();
	    sl.app.iface.paint(g);
	    g.dispose();
	    break;
	default: 
	    break;
	}
	return image;
    }


    // taken from http://javaalmanac.com/egs/java.awt.image/Image2Buf.html
    // This method returns a buffered image with the contents of an image
    protected static BufferedImage toBufferedImage(Image image) {
        if (image instanceof BufferedImage) {
            return (BufferedImage)image;
        }
    
        // This code ensures that all the pixels in the image are loaded
        image = new javax.swing.ImageIcon(image).getImage();
    
        // Determine if the image has transparent pixels; for this method's
        // implementation, see e665 Determining If an Image Has Transparent Pixels
        boolean hasAlpha = hasAlpha(image);
    
        // Create a buffered image with a format that's compatible with the screen
        BufferedImage bimage = null;
        GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
        try {
            // Determine the type of transparency of the new buffered image
            int transparency = Transparency.OPAQUE;
            if (hasAlpha) {
                transparency = Transparency.BITMASK;
            }
    
            // Create the buffered image
            GraphicsDevice gs = ge.getDefaultScreenDevice();
            GraphicsConfiguration gc = gs.getDefaultConfiguration();
            bimage = gc.createCompatibleImage(image.getWidth(null), 
					      image.getHeight(null), transparency);
        } catch (HeadlessException e) {
            // The system does not have a screen
        }
	
        if (bimage == null) {
            // Create a buffered image using the default color model
            int type = BufferedImage.TYPE_INT_RGB;
            if (hasAlpha) {
                type = BufferedImage.TYPE_INT_ARGB;
            }
            bimage = new BufferedImage(image.getWidth(null), image.getHeight(null), type);
        }
    
        // Copy image to buffered image
        Graphics g = bimage.createGraphics();
    
        // Paint the image onto the buffered image
        g.drawImage(image, 0, 0, null);
        g.dispose();
    
        return bimage;
    }

    // This is taken from http://javaalmanac.com/egs/java.awt.image/HasAlpha.html
    // This method returns true if the specified image has transparent pixels
    protected static boolean hasAlpha(Image image) {
        // If buffered image, the color model is readily available
        if (image instanceof BufferedImage) {
            BufferedImage bimage = (BufferedImage)image;
            return bimage.getColorModel().hasAlpha();
        }
    
        // Use a pixel grabber to retrieve the image's color model;
        // grabbing a single pixel is usually sufficient
	PixelGrabber pg = new PixelGrabber(image, 0, 0, 1, 1, false);
        try {
            pg.grabPixels();
        } catch (InterruptedException e) {
        }
    
        // Get the image's color model
        ColorModel cm = pg.getColorModel();
        return cm.hasAlpha();
    }


    protected SLURL getSaveFileName(StarLogo sl) {
	FileDialog d = new FileDialog((Frame)sl.app.activeFrame, sl.app.textBundle.getString("save_picture"), FileDialog.SAVE);

	if (sl.app.project.curDir!=null) {
	    d.setDirectory(sl.app.project.curDir.getPath());
	}
	d.setFilenameFilter(sl.app.pictureFileFilter); 
	d.show();
	sl.app.project.curDir = new SLURL(SLURL.FILE, d.getDirectory());
	if (d.getFile() == null) {
	    d.dispose();
	    return null;
	} else {
	    SLURL newname = new SLURL(SLURL.FILE, sl.app.project.curDir.getPath(),
				      d.getFile());
	    d.dispose();
	    return newname;
	}
    }

}


