/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

class Pool implements Runnable {
  
  final static Object undefined = new Object();
  
  StackProcess[] pool;
  int ptr;
  Class kind;
    Constructor cons;

  Pool(Class kind) {
    pool = new StackProcess[100];
    ptr = 0;
    this.kind = kind;
    try {
	cons = kind.getConstructor(new Class[]{Application.class});
    } catch (NoSuchMethodException nsme) {
	nsme.printStackTrace();
    }
    new Thread(this).start();
  }
  
  public void clearPool() {
      synchronized(this) {
	  ptr = 0;
	  pool = new StackProcess[100];
      }
  }

  public StackProcess getStackProcess(Application app) {
      synchronized(this) {
	  if (ptr == 0) {
	      StackProcess sp = null;
	      try {
		  sp = (StackProcess)cons.newInstance(new Object[]{app});
	      } 
	      catch (InvocationTargetException d) { d.printStackTrace(); }
	      catch (InstantiationException e) { e.printStackTrace(); }
	      catch (IllegalAccessException f) { f.printStackTrace(); }
	      return sp;
	  }
	  else {
	      pool[ptr-1].deadp = false;
	      return pool[--ptr];
	  }
      }
  }
  
  public void dropStackProcess(StackProcess array) {
      //System.out.println("drop stack process " + array);
    /*
      for(int j = 0; j < StackProcess.stacksize; j++) {
      array.ostack[j] = undefined;
      }
    */
      array.callerentity = null;
      array.originalentity = null;
      array.entity = null;
      array.nextentity = null;
      array.tur = null;
      array.pat = null;
      array.obs = null;
      array.spp = null;
      array.iinst = null;
      array.oinst = null;
      array.prevInList = null;
      //array.nextInList = null; // keep this so SPEnumerator works right
      array.prevInVM = null;
      array.nextInVM = null;
      array.deadp = true;

      synchronized(this) {
	  if (ptr >= pool.length) {
	      StackProcess[] newarray = new StackProcess[pool.length * 2];
	      System.arraycopy(pool, 0, newarray, 0, pool.length);
	      pool = newarray;
	  }
	  pool[ptr++] = array;
      }
    }

  // Cleanup thread. Runs every 5 seconds or so to clean out the pools.
  public void run() {
    Thread thread = Thread.currentThread();
    while (true) {
      try {
	thread.sleep(5000);
      }
      catch (InterruptedException e) {}
      clearPool();
    }
  }



}
