/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Insets;
import java.io.PrintWriter;
import java.io.OutputStream;
import java.awt.Frame;
import yoyo.YoYoMethods;
import yoyo.YoYo;
import yoyo.Primitive;
import yoyo.Context;
import yoyo.LogoError;
import yoyo.YoYoCaller;
import yoyo.Ufun;
import javax.swing.SwingUtilities;


public class PrimStarLogo implements YoYoMethods {

    public String[] methodList() {
	String[] output = {
	    "new-starlogo", "prim_newstarlogo",
	    "set-graphics", "prim_setgraphics",
	    "crt", "prim_newturtle",
	    "ca", "prim_ca",
	    "cg", "prim_cg",
	    "set-instructions", "prim_setinsts",
	    "reset-instructions", "prim_resetinsts",
	    "ask-observer", "prim_askobs",
	    "ask-colorchooser", "prim_askcolorchooser",
	    "ask-turtles", "prim_asktur",
	    "ask-patches", "prim_askpat",
	    "monitor-ask-turtles", "prim_monasktur",
	    "monitor-ask-patches", "prim_monaskpat",
	    "stop-turtles", "prim_stoptur",
	    "stop-observer", "prim_stopobs",
	    "stop-process", "prim_stopproc",
	    "get-sliders", "prim_getsliders",
	    "get-buttons", "prim_getbuttons",
	    "get-shapenames", "prim_getshapenames",
	    "error-dialog", "prim_errordialog",
	    "get-button", "prim_getbutton",
	    "get-monitor", "prim_getmonitor",
	    "get-linewidget", "prim_linewidget",
	    "set-button-command", "prim_setbuttoncommand",
	    "set-monitor-command", "prim_setmonitorcommand",
	    "set-line-command", "prim_setlinecommand",
	    "set-global-vars", "prim_setglobalvars",
	    "set-patch-vars", "prim_setpatchvars",
	    "set-turtle-vars", "prim_setturtlevars",
	    "set-turtle-breeds", "prim_setturtlebreeds",
	    "set-enum-names", "prim_setenumnames",
	    "set-progress-bar-limit-and-run", "prim_setpb",
	    "update-progress-bar", "prim_progress",
	    "get-iface", "prim_getiface",
	    "new-application", "prim_newapp",
	    "set-observer-proc-names", "prim_setobsprocs",
	    "set-turtle-proc-names", "prim_setturprocs",
	    "compile-done", "prim_compile_done",
	    "compile-variables-done", "prim_compile_variables_done",
	    "compile-failed", "prim_compile_failed",
	    "ptolemy-used-plot-commands", "prim_ptolemy_used_plot_commands",
	    "num-loops", "prim_num_loops",
	    "num-draws", "prim_num_draws",
	    "send-app-toplevel", "prim_send_app_toplevel",
	    "debug?", "prim_debugp",
	    "debug2?", "prim_debug2p",
	};
	return output;
    }

	public static void prim_send_app_toplevel(Object v0, Context c) {
	    Application app = (Application)v0;
	    if (app.kioskp) {
		TopKiosk.toplevel.app = app;
	    } else {
		Toplevel.toplevel.app = app;
	    }
	}

    public String[] infixList() {
	return null;
    }

    public void prePrimCall(Primitive p, Object[] arglist, Context c) {}
    public void postPrimCall(Primitive p, Context c) {}

    public static Object prim_newstarlogo(Object v0, Context c)  {
	return new StarLogo(c, YoYo.aBoolean(c, v0));
    }

    public static Object prim_setgraphics(Object v0, Context c) {
	Application app = aApp(c, v0);
	StarLogo sl = app.sl;
	//sl.patchcanvas.setGraphics(sl.patchcanvas.getDrawingSurface());
	if (YoYo.applet == null) {
	  sl.myframe = (Frame) app.iface.frame();
	  app.message = new PrintWriter((OutputStream)Application.createMessageStream(app, (StarLogoFrame)sl.myframe), true);
	} else {
	  app.message = new PrintWriter(System.out, true);
	}
	new Thread(sl).start();
	return null;
    }

    public static Object prim_newturtle(Object v0, Object v1, Context c)  {
	int num = YoYo.aInteger(c, v1);
	StarLogo sl = aStarLogo(c, v0);
	synchronized(sl.patchcanvas.getGraphicsLock()) {
	synchronized(sl.runningp) {
	    if (sl.patchcanvas instanceof PatchCanvas)
		((PatchCanvas)sl.patchcanvas).setDrawingSurface();
	    sl.crt(num);
	    if (sl.patchcanvas instanceof PatchCanvas)
		((PatchCanvas)sl.patchcanvas).deleteDrawingSurface();
	}
	}
	sl.redraw();
	return null;
    }

    public static Object prim_ca(Object v0, Context c)  {
	StarLogo sl = aStarLogo(c, v0);
	synchronized(sl.patchcanvas.getGraphicsLock()) {
	    synchronized(sl.runningp) {
	    if (sl.patchcanvas instanceof PatchCanvas)
		((PatchCanvas)sl.patchcanvas).setDrawingSurface();
	    sl.CA();
	    if (sl.patchcanvas instanceof PatchCanvas)
		((PatchCanvas)sl.patchcanvas).deleteDrawingSurface();
	}
	}
	sl.redraw();
	return null;
    }

    public static Object prim_cg(Object v0, Context c)  {
	StarLogo sl = aStarLogo(c, v0);
	synchronized (sl.patchcanvas.getGraphicsLock()) {
	    synchronized(sl.runningp) {
	    if (sl.patchcanvas instanceof PatchCanvas)
		((PatchCanvas)sl.patchcanvas).setDrawingSurface();
	    sl.CG();
	    if (sl.patchcanvas instanceof PatchCanvas)
		((PatchCanvas)sl.patchcanvas).deleteDrawingSurface();
	}
	}
	sl.redraw();
	return null;
    }

    public static Object prim_getshapenames(Object v0, Context c) {
		return aApp(c, v0).getShapeNames();
    }

    public static Object prim_getsliders(Object v0, Context c) {
	return aApp(c, v0).iface.getSliders();
    }

    public static Object prim_getbuttons(Object v0, Context c) {
	return aApp(c, v0).iface.getPreButtons();
    }

    public static Object prim_getbutton(Object v0, Object v1, Context c) {
	return aApp(c, v0).iface.getButton(YoYo.aInteger(c, v1));
    }

    public static Object prim_getmonitor(Object v0, Object v1, Context c) {
	return aApp(c, v0).iface.getMonitor(YoYo.aInteger(c, v1));
    }

    public static Object prim_getlinewidget(Object v0, Object v1, Context c) {
	return aApp(c, v0).iface.getLineWidget(YoYo.aInteger(c, v1));
    }

    public static void prim_setbuttoncommand(Object v0, Object v1, Object v2,
					     Object v3, Context c) {
	aApp(c, v0).setButtonCommand(YoYo.aInteger(c, v1), YoYo.aList(c, v2),
				     YoYo.aList(c, v3));
    }

    public static void prim_setmonitorcommand(Object v0, Object v1, Object v2,
					      Object v3, Object v4, Object v5,
					      Context c) {
	aApp(c, v0).setMonitorCommand(YoYo.aInteger(c, v1), YoYo.aList(c, v2),
				      YoYo.aList(c, v3), YoYo.aList(c, v4),
				      YoYo.aList(c, v5));
    }

    public static void prim_setlinecommand(Object v0, Object v1, Object v2,
					      Object v3, Object v4, Object v5,
					      Context c) {
	aApp(c, v0).setLineCommand(YoYo.aInteger(c, v1), YoYo.aList(c, v2),
				      YoYo.aList(c, v3), YoYo.aList(c, v4),
				      YoYo.aList(c, v5));
    }


    public static void prim_asktur(Object v0, Object v1, Object v2, Object v3, Context c) {
	aApp(c, v0).runTurtleCommand(YoYo.aList(c, v1), YoYo.aList(c, v2), (SLCommand)v3);
    }

    public static void prim_askpat(Object v0, Object v1, Object v2, Object v3, Context c) {
	aApp(c, v0).runPatchCommand(YoYo.aList(c, v1), YoYo.aList(c, v2), (SLCommand)v3);
    }

    public static void prim_monasktur(Object v0, Object v1, Object v2, Object v3, Context c) {
	aApp(c, v0).runTurtleCommand(YoYo.aList(c, v1), YoYo.aList(c, v2), (TurtleMonitor)v3);
    }

    public static void prim_monaskpat(Object v0, Object v1, Object v2, Object v3, Context c) {
	aApp(c, v0).runPatchCommand(YoYo.aList(c, v1), YoYo.aList(c, v2), (PatchMonitor)v3);
    }

    public static void prim_askobs(Object v0, Object v1, Object v2, Object v3, Context c) {
	aApp(c, v0).runObserverCommand(YoYo.aList(c, v1), YoYo.aList(c, v2), (SLCommand)v3);
    }

    public static void prim_askcolorchooser(Object v0, Object v1, Object v2, Object v3, Context c) {
	aApp(c, v0).runColorChooserCommand(YoYo.aList(c, v1), YoYo.aList(c, v2), (ColorChooser)v3);
    }

    public static void prim_resetinsts(Object v0, Context c) {
	StarLogo sl = aStarLogo(c, v0);
	synchronized(sl.runningp) {
	    //sl.stopAll();
	    sl.nextinst = 0;
	}
    }

    public static Object prim_setinsts(Object v0, Object v1, Object v2,
				       Object v3, Context c)  {
	StarLogo sl = aStarLogo(c, v0);
	String name = YoYo.aString(c, v1);
	Object[] ilist = YoYo.aList(c, v2);
	Object[] olist = YoYo.aList(c, v3);
	synchronized(sl.runningp) {
	    return new Integer(sl.setInstructions(name, ilist, olist));
	}
    }


    public static Object prim_stoptur(Object v0, Context c) {
	StarLogo sl = aStarLogo(c, v0);
	synchronized(sl.runningp) {
	    sl.stopTurtles();
	}
	return null;
    }

    public static Object prim_stopobs(Object v0, Context c) {
	StarLogo sl = aStarLogo(c, v0);
	synchronized(sl.runningp) {
	    sl.stopObserver();
	}
	return null;
    }

    public static Object prim_stopproc(Object v0, Object v1, Context c) {
	StarLogo sl = aStarLogo(c, v0);
	StackProcessParent spp = aStackProcessParent(c, v1);
	synchronized(sl.runningp) {
	    sl.stopProcess(spp);
	}
	return null;
    }

    public static void prim_errordialog(Object v1, Object v2,
					Context c) {
	DisplayMessage dm = (DisplayMessage)v1;
	String contents = YoYo.aString(c, v2);
	dm.displayOneString(contents);
    }

    public static void prim_setglobalvars(Object v0, Object v1, Context c) {
	StarLogo sl = aStarLogo(c, v0);
	Observer.setVars(YoYo.aList(c, v1), sl, c);
    }

    public static void prim_setturtlevars(Object v0, Object v1, Context c) {
	StarLogo sl = aStarLogo(c, v0);
	Turtle.setVars(YoYo.aList(c, v1), sl, c);
    }

    public static void prim_setenumnames(Object v0, Object v1, Context c) {
	StarLogo sl = aStarLogo(c, v0);
	VM.setEnumNames(YoYo.aList(c, v1), sl, c);
    }

    public static void prim_setpatchvars(Object v0, Object v1, Context c) {
	StarLogo sl = aStarLogo(c, v0);
	Patch.setVars(YoYo.aList(c, v1), sl, c);
    }

    public static void prim_setturtlebreeds(Object v0, Object v1, Context c) {
	StarLogo sl = aStarLogo(c, v0);
	Turtle.setBreeds(YoYo.aList(c, v1), c);
    }

  public static void prim_setobsprocs(Object v0, Object v1, Object v2, Context c) {
    aApp(c, v0).procWin.setObsProcs(YoYo.aList(c, v1), aIntegerList(c, v2));
  }

  public static void prim_setturprocs(Object v0, Object v1, Object v2, Context c) {
    aApp(c, v0).procWin.setTurProcs(YoYo.aList(c, v1), aIntegerList(c, v2));
  }
    public static Object prim_newapp(Object v0, Object v1, Object v2, Context c) {
	return new Application(aStarLogo(c, v0), YoYo.aBoolean(c, v1), YoYo.aBoolean(c, v2), c);
    }

    public static Object prim_getiface(Object v0, Context c) {
	return aApp(c, v0).iface;
    }

    static class SetPBRunner implements Runnable {
	private ProgressBar pb;
	private int limit;
	public SetPBRunner(ProgressBar pb, int limit) {
	    this.pb = pb;
	    this.limit = limit;
	}
	public void run() {
	    pb.setLimit(limit);
	    pb.start();
	}
    }

    public static void prim_setpb(Object v0, Object v1, Context c) {
	SwingUtilities.invokeLater(new SetPBRunner(aProgressBar(c, v0), YoYo.aInteger(c, v1)));
    }

    static class UpdatePBRunner implements Runnable {
	private ProgressBar pb;
	private int n;
	private String s;
	public UpdatePBRunner(ProgressBar pb, int n, String s) {
	    this.pb = pb;
	    this.n = n;
	    this.s = s;
	}
	public void run() {
	    pb.update(n, s);
	}
    }


    public static void prim_progress(Object v0, Object v1, Object v2, Context c) {
	while (!aProgressBar(c, v0).isVisible()) { Thread.currentThread().yield(); }
	SwingUtilities.invokeLater(new UpdatePBRunner(aProgressBar(c, v0), YoYo.aInteger(c, v1), YoYo.aString(c, v2)));
    }

    public static void prim_compile_done(Object v0, Object v1, Context c) {
	Application app = aApp(c, v0);
	app.project.compileDone(app, aProgressBar(c, v1));
    }

    public static void prim_compile_variables_done(Object v0, Object v1, Context c) {
	Application app = aApp(c, v0);
	app.project.compileVariablesDone(app, aProgressBar(c, v1));
    }


  public static void prim_compile_failed(Object v0, Object v1, Context c) {
    Application app = aApp(c, v0);
    app.project.compileFailed(app, aProgressBar(c, v1));
  }

    public static void prim_ptolemy_used_plot_commands(Object v0, Context c) {
	Application app = aApp(c, v0);
	app.usedPlotCommands();
    }

    public static Object prim_num_loops(Object v0, Context c) {
	Application app = aApp(c, v0);
	return new Integer(app.sl.loops);
    }

    public static Object prim_num_draws(Object v0, Context c) {
	Application app = aApp(c, v0);
	if (app.sl.patchcanvas instanceof PatchCanvas) {
	    return new Integer(((PatchCanvas)app.sl.patchcanvas).pcdraws);
	} else return new Integer(0);
    }

    public static Object prim_debugp(Object v0, Context c) {
	Application app = aApp(c, v0);
	return YoYo.truefalse(app.sl.DEBUG);
    }

    public static Object prim_debug2p(Object v0, Context c) {
	Application app = aApp(c, v0);
	return YoYo.truefalse(app.sl.DEBUG2);
    }


      public static StackProcessParent aStackProcessParent(Context c, Object v) {
	try { return (StackProcessParent)v; }
	catch (Exception e) {
	    throw new LogoError(YoYo.printToString(v) +
				" is not a stack process parent");
	}
    }

    public static StarLogo aStarLogo(Context c, Object v) {
	try { return (StarLogo)v; }
	catch (Exception e) {
	    throw new LogoError(YoYo.printToString(v) +
				" is not a StarLogo object");
	}
    }

    public static ProgressBar aProgressBar(Context c, Object v) {
	try { return (ProgressBar)v; }
	catch (Exception e) {
	    throw new LogoError(YoYo.printToString(v) +
				" is not a ProgressBar object");
	}
    }


    public static Application aApp(Context c, Object v) {
	try { return (Application)v; }
	catch (Exception e) {
	    throw new LogoError(YoYo.printToString(v) +
				" is not an Application object");
	}
    }

    public static Integer[] aIntegerList(Context c, Object v) {
    	if (v instanceof Object[]) {
    		Object[] o = (Object[])v;
    		Integer[] result = new Integer[o.length];
    		for(int i = 0; i < o.length; i++) {
    			if (o[i] instanceof Integer) {
    				result[i] = (Integer)o[i];
    			} else {
	    			throw new LogoError(YoYo.printToString(v) +
				           " is not a list of integers");
			    }
    		}
    		return result;
    	} else {
			throw new LogoError(YoYo.printToString(v) +
           " is not a list of integers");
		}
    }


}
