/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.awt.*;
import javax.swing.*;
import yoyo.*;

public class ProgressBar extends JDialog implements Runnable, StarLogoFrame {

    Application app;
    JProgressBar bar;
    JLabel label;
    int total;
    boolean done;

    public ProgressBar(Application app, String title) {
	super((Frame)app.activeFrame, title, true); 
	this.app = app;
	this.total = total;
	done = false;
	Box box = Box.createVerticalBox();
	bar = new JProgressBar(0, 100);
	bar.setValue(0);
	//bar.setBorderPainted(true);
	label = new JLabel(title, SwingConstants.LEFT);
	//bar.setStringPainted(true);
	box.add(Box.createVerticalStrut(5));
	box.add(label);
	box.add(Box.createVerticalStrut(5));
	box.add(Box.createGlue());
	box.add(bar);
	getContentPane().add(box);
	setSize(300, ((YoYo.pcp) ? 60 : 45) + 10 + label.getPreferredSize().height);
	setResizable(false);
	
	Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
	Dimension size = getSize();
	int screenwidth = screen.width;
	int dialogwidth = size.width;
	setLocation((screenwidth - dialogwidth) / 2, 60);
    }

    public void setLimit(int limit) {
	this.total = limit;
	bar.setMaximum(limit);
    }

    public void addToLimit(int limit) {
	this.total += limit;
	bar.setMaximum(total);
    }

    public void start() {
	if (!done) new Thread(this).start();
    }
  
    public void run() {
	if (!done) setVisible(true);
    }
    
    public void update(int position, String str) {
	//System.out.println("update " + position + " string " + str);
	if (done) return;
	//System.out.println("pos: " + position + " max = " + bar.getMaximum());
	if (position >= bar.getMaximum()) {
	    //done();
	    return;
	} else {
	    bar.setValue(position);
	    label.setText(str);
	}
    }

    public void update(String str) {
	if (done) return;
	if (bar.getValue() >= bar.getMaximum()) 
	    return;
	bar.setValue(bar.getValue() + 1);
	label.setText(str);
    }

    public void done() {
	//System.out.println("hide pb");
	hide();
	done = true;
    }

    public void markChanged() {    }
    public void clearChanged() {    }
}
