/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import yoyo.YoYo;
import yoyo.YoYoCaller;
import yoyo.Context;
import yoyo.PrimFile;
import yoyo.LogoError;
import yoyo.Tokenizer;
import yoyo.Reader;
import yoyo.Symbol;
import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.util.*;
import java.io.*;
import java.util.regex.*;
import javax.swing.SwingUtilities;
import java.nio.charset.UnsupportedCharsetException;

public class Project implements YoYoCaller, TimerNotify {

    public static String HEADER = "Java StarLogo";
    public static float CURRENT_VERSION = 2.1f;
    public static float MAC_VERSION = -1.0f;
    float version;

    private boolean contentsChanged;
    static Timer timer;
    private static int BACKUP_DELAY = 60 * 5; // five minutes
    SLURL filename, outputFilename, infoFilename;
    StarLogo sl;
    ControlCenter procWin;
    InfoWindow infoWin;
    OutputWindow outWin;
    //RTGraphWindow plotWin;
    JInterface iface;
    // added by MIM
    template.Template theTemplate;
    // end MIM changes
    static SLURL curDir;
    SLURL backupPendingDeletion;
    Application app;
    static String nativeEncoding = "";
    
    static {
	OutputStreamWriter out = new OutputStreamWriter(new ByteArrayOutputStream());
	nativeEncoding = out.getEncoding();
    }

    public Project(Application app) {
	this.app = app;
	this.sl = app.sl;
	if (!app.kioskp && YoYo.applet == null) {
	    this.procWin = app.procWin;
	    this.infoWin = app.infoWin;
	    this.outWin = app.outWin;
	    this.theTemplate = app.theTemplate;
	}
	this.iface = app.iface;
	// end MIM changes
	if ((!app.kioskp && YoYo.applet == null) && (curDir == null)) {
	    curDir = new SLURL(app.cc.base);
	    if (YoYo.pcp) { 
		//System.out.println("curDir = " + curDir);
		curDir.setPath(curDir.getFullPath() + "\\" + 
			       app.textBundle.getString("windows_dir_my_documents") + 
			       "\\");
		//System.out.println("curDir = " + curDir);
	    }
	}
	filename = null;
	clearChanged();
    }

    public static void newProject(final Application app) {
	Runnable r = new Runnable() { 
		public void run() {
		    StarLogo sl = app.sl;
		    ControlCenter procWin = app.procWin;
		    InfoWindow infoWin = app.infoWin;
		    OutputWindow outWin = app.outWin;
		    //RTGraphWindow plotWin = app.plotWin;
		    JInterface iface = app.iface;

		    // need close turtle monitors, reset shapes eventually
		    synchronized(sl.runningp) {
			app.sl.stopAllStarLogo();
		    }
		    app.iface.setTitle(app.textBundle.getString("starlogo_title"));
		    app.procWin.setTitle(app.textBundle.getString("windows_menu_control_center"));
		    
		    procWin.reset();
		    procWin.clearProcs();
		    iface.reset();
		    sl.resetPatches();
		    infoWin.reset();
		    outWin.reset();
		    iface.repaint();
		    StackProcess.pool.clearPool();
		    FastPatchStackProcess.pool.clearPool();
		    Project p = new Project(app);
		    app.project = p;
		    app.compiled = false;
		    app.rancompilerp = false;
		    app.compiledVariables = false;
		    app.rancompilerpForVariables = false;
		    app.maybeCompileAndThenRun(false, new NewProjectRunner(sl, p));
		}
	    };

	// if necessary, ask user to save changes, and return if answer is "cancel"
	if (app.project.saveChanges(app, r) == CANCEL) {
	    return;
	}
    }
    
    static class NewProjectRunner implements Runnable {
	private StarLogo sl;
	private Project p;
	public NewProjectRunner(StarLogo sl, Project p) {
	    this.sl = sl;
	    this.p = p;
	}
	public void run() {
	    sl.CA();
	    sl.crt(100);
	    sl.redraw();
	    p.clearChanged();
	}
    }

    public static void openProject(final Application app) {
	Runnable r = new Runnable() { 
		public void run() {
		    SLURL newname = null;
		    if (!app.kioskp && YoYo.applet == null) {
			FileDialog d = new FileDialog((Frame)app.activeFrame, 
						      app.textBundle.getString("file_menu_open_project"),
						      FileDialog.LOAD);
			
			
			if (curDir!=null && curDir.getProtocol() == SLURL.FILE) {
			    d.setDirectory(curDir.getFullPath());
			}
			d.setFilenameFilter(app.projectFileFilter);
			d.show();
			curDir = new SLURL(SLURL.FILE, d.getDirectory());
			if (d.getFile() != null) {
			    newname = new SLURL(SLURL.FILE, curDir.getPath(), d.getFile());
			} else newname = null;
			d.dispose();
		    } else {
			if (YoYo.applet != null) {
			    TextDialog t = Application.createTextDialog((Frame)app.activeFrame, app.textBundle.getString("file_menu_open_project"),
									app.textBundle.getString("file_path"));
			    
			    
			    newname = new SLURL(YoYo.applet.getCodeBase());
			    newname.setFile(t.getResult());
			    t.dispose();
			}
		    }
		    if (newname != null) {
			app.project = Project.loadFile(newname, app, false);
			runStartup(app);
		    }
		}
	    };
	
	// if necessary, ask user to save changes, and return if answer is "cancel"
	if (app.project.saveChanges(app, r) == CANCEL) {
	    return;
	}

    }

    public static void runStartup(Application app) {
	if (app.kioskp || YoYo.applet != null) {
	    System.out.println("running startup");
	    int whichip = app.sl.internalGetWhichip("startup");
	    System.out.println("startup is " + whichip);
	    if (whichip == -1) return;
	    int[] newiinst = { 0x700, whichip, 0, 0, 0 }; // ufun call to whichip, done
	    Object[] newoinst = new Object[0];
	    String name = "run-startup";
	    whichip = app.sl.internalSetInstructions(name,  newiinst, newoinst);
	    System.out.println("new whichip = " + whichip);
	    StackProcessParent.runObserver(app.sl, whichip, false, null);
	} else {
	    Object[] listtocompile = { Symbol.lookup("compile-run-startup") };
	    Object[] newlist = new Object[listtocompile.length + 1];
	    System.arraycopy(listtocompile, 0, newlist, 0, listtocompile.length);
	    newlist[newlist.length -1] = app.procWin.getSLCommand(ControlCenter.obs);
	    app.cc.enqueue(newlist, app.message, app.err, null);
	    app.procWin.getSLProc(app.procWin.tur).gotoLine(0);
	    app.procWin.getSLProc(app.procWin.obs).gotoLine(0);
	}
    }


    public static Project loadFile(SLURL newname, Application app, boolean htmlp) {
	Project result = app.project;
	try {
	    app.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
	    newname = maybeUseBackupFile(newname, app);
	    // If newname is null, cancel the load.
	    if (newname == null) {
		// Turn on the timer again
		timer = new Timer(result, BACKUP_DELAY);
		return result;
	    }
	    String newtitle = newname.getFileNoExtension();

	    // always stop the old backup timer
	    if (timer != null) timer.abort();
	    timer = null;

	    try {
		String newcontents = newname.load(app.sl.c, "UTF-8");
		result = loadFromString(app, newcontents, newtitle, newname, 
					app.project, "UTF-8", htmlp);
	    } catch (WrongEncodingException wee) {
		try {
		    String newcontents1 = newname.load(app.sl.c, nativeEncoding);
		    result = loadFromString(app, newcontents1, newtitle, newname, 
					    app.project, nativeEncoding, htmlp);
		    
		} catch (UnsupportedCharsetException ucse) {
		    try {
			String newcontents1 = newname.load(app.sl.c);
			result = loadFromString(app, newcontents1, newtitle, newname, 
						app.project, null, htmlp);
		    } catch (WrongEncodingException wee1) {
			assert false;
		    }
		} catch (WrongEncodingException wee1) {
		    assert false;
		}
	    }		    
	} catch(Exception e) {
	    e.printStackTrace();
	    Application.createMessageDialog(app, app.activeFrame, app.textBundle.getString("starlogo_dialog_title"),
					    "Error loading project file: " + e.toString(), false);
	} finally {
	    app.restoreCursor();
	}
	return result;
    }

    public static void openURLProject(final Application app) {

	Runnable r = new Runnable() {
		public void run() {
		    TextDialog t = Application.createTextDialog((Frame)app.activeFrame, 
						    app.textBundle.getString("file_menu_open_project"),
						    app.textBundle.getString("type_url") + ":");
	String urlname = t.getResult();
	t.dispose();
	Project result = app.project;
	if (urlname != null) {
	    try {
		app.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		SLURL u = new SLURL(new URL(urlname));
		String newtitle = u.getFileNoExtension();

		try {
		    String newcontents = u.load(app.sl.c, "UTF-8");
		    result = loadFromString(app, newcontents, newtitle, u, 
					    app.project, "UTF-8", false);
		} catch (WrongEncodingException wee) {
		    try {
			String newcontents1 = u.load(app.sl.c, nativeEncoding);
			result = loadFromString(app, newcontents1, newtitle, u, 
						app.project, nativeEncoding, false);
		    } catch (UnsupportedCharsetException ucse) {
			try {
			    String newcontents1 = u.load(app.sl.c);
			    result = loadFromString(app, newcontents1, newtitle, u, 
						    app.project, null, false);
			    
			} catch (WrongEncodingException wee1) {
			    assert false;
			}
		    } catch (WrongEncodingException wee1) {
			assert false;
		    }
		}		    

		result.filename = u;
	    }
	    catch (MalformedURLException mue) {
		mue.printStackTrace();
		Application.createMessageDialog(app, app.activeFrame, app.textBundle.getString("starlogo_dialog_title"),
						"Error loading project file:  The specified URL does not exist.", false);
	    }
	    catch (Exception e) {
		e.printStackTrace();
		Application.createMessageDialog(app, app.activeFrame, app.textBundle.getString("starlogo_dialog_title"),
						"Error loading project file: " + e.getMessage(), false);
	    }
	    finally {
		app.restoreCursor();
	    }
	}
	app.project = result;
	runStartup(app);

		}
	    };

	// if necessary, ask user to save changes, and return if answer is "cancel"
	if (app.project.saveChanges(app, r) == CANCEL) {
	    return;
	}

    }

    static private Project loadFromString(Application app, String newcontents, String newtitle,
					  SLURL filename, Project result, String encoding, 
					  boolean htmlp)
	throws Exception {
	try {
	    newcontents = newcontents.trim();
	    if (!newcontents.startsWith(HEADER))
		{newcontents = convertFormat(newcontents);}
	    if (newcontents.startsWith(HEADER)) {
		// stop the world once we've committed to loading
		synchronized(result.sl.runningp) {
		    result.sl.stopAllStarLogo();
		}
		
		result = new Project(app);
		app.project = result;
		result.setVersion(newcontents);
		if (!htmlp) result.procWin.deleteObservers();
		result.sl.restoreTables(newcontents, result.sl.c);

		// New files should be loaded using UTF-8. Old files use the
		// native encoding.
		if (encoding != null) {
		    if (result.version < 2.1f) {
			// Old file, don't use UTF-8
			if (encoding.equals("UTF-8")) throw new WrongEncodingException();
		    } else {
			// New file, don't use native encoding
			if (encoding.equals(nativeEncoding)) throw new WrongEncodingException();
		    }
		}
		
		TurtleShapes.loadShapes(newcontents, true, app);
		result.iface.loadFromString(newcontents, htmlp);
		result.sl.restoreSettings(newcontents, htmlp);
		//System.out.println("Finished restore settings");
		result.sl.restoreWorld(newcontents, result.sl.c);
		//System.out.println("finished restore world");
		if (!htmlp) {
		    //System.out.println("procwin clearprocs");
		    result.procWin.clearProcs();
		    //System.out.println("procwin clear text");
		    result.procWin.resetText(newcontents);
		    //System.out.println("info win clear text");
		    result.infoWin.resetText(newcontents);
		    //System.out.println("out win clear text");
		    result.outWin.reset();
		    //System.out.println("template load contents");
		    if (YoYo.swingavailablep && !YoYo.macintoshp) 
			result.theTemplate.load(newcontents);
		    //result.plotWin.clearplot();
		}
		//System.out.println("clearing pools");
		StackProcess.pool.clearPool();
		FastPatchStackProcess.pool.clearPool();
		//result.contents = result.getContents();
		result.filename = filename;
		if (!htmlp) {
		    app.iface.setTitle("StarLogo - " + newtitle);
		    app.procWin.setTitle(app.textBundle.getString("windows_menu_control_center") +
					 " - " + newtitle);
		}
		//System.out.println("repain the iface");
		result.iface.repaint();
		//System.out.println("compile");
		result.compile(app, false, new ProgressBar(app, app.textBundle.getString("compiling")));
		//System.out.println("done compiling");
		result.clearChanged();
	    } else {
		Application.createMessageDialog(app, app.activeFrame, 
						app.textBundle.getString("starlogo_dialog_title"),
						filename.toString() + 
						app.textBundle.getString("not_valid_project") + ".", 
						false);
	    }
	} catch (Exception e) {
	    if (!(e instanceof WrongEncodingException)) {
		e.printStackTrace(app.log);
	    }
	    throw e;
	}
	return(result);

    }

    /** Quits the application.  Returns true if application is closed.  
     * False if close cancelled.
     */
    public static boolean quittingp = false;

    public boolean quit() {
	// if necessary, ask user to save changes, and return if answer is "cancel"
	//System.out.println("quit via dialog");
	// guard against quit being called twice. 
	if (quittingp) return false;
	quittingp = true;
	Runnable r = new Runnable() {
		public void run() {
		    quittingp = false;
		    System.exit(0);
		}
	    };
	if (saveChanges(app, r) == CANCEL) {
	    quittingp = false;
	    return false;
	} 
	return false;
    }


    public void markChanged() {
	//new Exception().printStackTrace();
	contentsChanged = true;
	app.markChanged();
    }

    public void clearChanged() {
	contentsChanged = false;
	app.clearChanged();
    }

    static final int YES = 0;
    static final int NO = 1;
    static final int CANCEL = 2;

    /** returns YES/NO if caller should continue (the user answered yes/no to save changes,
     * and completed save process successfully if answer was yes, OR there is no need to
     * save changes) and CANCEL if user canceled
     */
    int saveChanges(Application app, Runnable andThenRun) {
	//System.out.println("need to save changes? " + contentsChanged);
	if (contentsChanged) {
	    ChoiceDialog cd = Application.createChoiceDialog(((Frame)app.activeFrame), 
							     app.textBundle.getString("starlogo_dialog_title"),
							     app.textBundle.getString("save_changes"), 
							     app.textBundle.getString("save_label"),
							     app.textBundle.getString("cancel_label"), 
							     app.textBundle.getString("dont_save_label"));
	    String choice = cd.getResult();
	    if (choice.equals(app.textBundle.getString("save_label"))) {
		saveProject(app, andThenRun, false);
		//while(savingProject) { Thread.currentThread().yield(); }
		return (projectSaved) ? YES : CANCEL;
	    } else if (choice.equals(app.textBundle.getString("dont_save_label"))) {
		if (backupPendingDeletion != null) {
		    File backupfile = new File(backupPendingDeletion.getFullPath());
		    backupfile.delete();
		    backupPendingDeletion = null;
		}
		andThenRun.run();
		return NO;
	    } else { // choice.equals("cancel")
		return CANCEL;
	    }
	    
	} else {
	    andThenRun.run();
	}
	return YES;
    }

    private class ProjectRunner implements Runnable {
	private Application app;
	private boolean htmlp;
	private Runnable andThenRun;

	public ProjectRunner(Application app, Runnable andThenRun, boolean htmlp) {
	    this.app = app;
	    this.htmlp = htmlp;
	    this.andThenRun = andThenRun;
	}
	public void run() {
	    if (filename==null) {
		projectSaved = saveProjectAs(app, htmlp);
	    } else {
		projectSaved = saveFile(filename, app, htmlp);
	    }
	    savingProject = false;
	    andThenRun.run();
	}
    }

    volatile boolean projectSaved;
    volatile boolean savingProject;

    public void saveProject(Application app, boolean htmlp) {
	saveProject(app, new Runnable() { public void run() {} }, htmlp);
    }

    public void saveProject(Application app, Runnable andThenRun, boolean htmlp) {
	projectSaved = false;
	savingProject = true;
	sl.app.maybeCompileAndThenAlwaysRun(false, new ProjectRunner(app, andThenRun, htmlp));
    }

    public boolean saveVersion(Application app) {
	if (filename==null) {
	    return saveProjectAs(app, false);
	} else {
	    return saveProjectVersion(filename, app, false);
	}
    }

    public void setProjectFilename(Application app, String simplename) {
	if (simplename != null && !simplename.equals("")) {
	    try {
		if (curDir == null) {
		    FileDialog d = new FileDialog(((Frame)app.activeFrame), 
						  app.textBundle.getString("file_menu_save_project") + "...", 
						  FileDialog.SAVE);


		    curDir = new SLURL(SLURL.FILE, d.getDirectory());
		}
		filename = new SLURL(SLURL.FILE, curDir.getPath(), simplename);
		app.iface.setTitle("StarLogo - " + filename.getFileNoExtension());
		if (!app.kioskp && YoYo.applet == null) 
		    app.procWin.setTitle(app.textBundle.getString("windows_menu_control_center") + 
					 " - " + filename.getFileNoExtension());
	    app.iface.frame().requestFocusInWindow() ;
	    }
	    catch (Exception e) {
		Application.createMessageDialog(app, app.activeFrame, 
						app.textBundle.getString("starlogo_dialog_title"),
						app.textBundle.getString("error_setting_project_name") +
						": " + e.getMessage(), false);
	    }
	}
    }
    
    public boolean saveText(Application app, int whichone) {
	SLURL filename = null;
	switch (whichone) {
	case SAVE_OUTPUT: filename = outputFilename; break;
	case SAVE_INFO: filename = infoFilename; break;
	}
	if (filename==null) {
	    return saveTextAs(app, whichone);
	} else {
	    return saveTextFile(filename, app, whichone);
	}
    }

    public void setTextFilename(Application app, String simplename, int whichone) {
	String window = null;
	switch (whichone) {
	case SAVE_OUTPUT: window = app.textBundle.getString("windows_menu_output_window"); break;
	case SAVE_INFO: window = app.textBundle.getString("windows_menu_info_window"); break;
	default: assert false;
	}
	if (simplename != null && !simplename.equals("")) {
	    try {
		if (curDir == null) {
		    FileDialog d = new FileDialog(((Frame)app.activeFrame),
						  app.textBundle.getString("save_label") + " " + 
						  window + "...", FileDialog.SAVE);


		    curDir = new SLURL(SLURL.FILE, d.getDirectory());
		}
		switch (whichone) {
		case SAVE_OUTPUT: 
		    outputFilename = new SLURL(SLURL.FILE, curDir.getPath(), simplename);
			app.iface.frame().requestFocusInWindow() ;
		    //System.out.println("creating file obj: " + outputFilename.getPath() + " " + outputFilename.getFile());
		    break;
		case SAVE_INFO: 
		    infoFilename = new SLURL(SLURL.FILE, curDir.getPath(), simplename);
			app.iface.frame().requestFocusInWindow() ;
		    break;
		default: assert false;
		}
	    }
	    catch (Exception e) {
		window = null;
		switch (whichone) {
		case SAVE_OUTPUT: window = app.textBundle.getString("windows_menu_output_window"); break;
		case SAVE_INFO: window = app.textBundle.getString("windows_menu_info_window"); break;
		default: assert false;
		}
		e.printStackTrace(System.err);
		Application.createMessageDialog(app, app.activeFrame, app.textBundle.getString("starlogo_dialog_title"),
						app.textBundle.getString("error_setting") + " " +
						window + " " + app.textBundle.getString("filename") +
						": " + e.getMessage(), false);
	    }
	}
    }

    // return true if file was saved successfully
    public boolean saveTextAs(Application app, int whichone) {
	String window = null;
	switch (whichone) {
	case SAVE_OUTPUT: window = app.textBundle.getString("windows_menu_output_window"); break;
	case SAVE_INFO: window = app.textBundle.getString("windows_menu_info_window"); break;
	default: assert false;
	}

	FileDialog d = new FileDialog((Frame)app.activeFrame, 
				      app.textBundle.getString("save_label") + " " + 
				      window + "...", FileDialog.SAVE);

	if (curDir!=null && curDir.getProtocol() == SLURL.FILE) {
	    d.setDirectory(curDir.getPath());
	}
	d.setFilenameFilter(app.textFileFilter);
	d.show();
	curDir = new SLURL(SLURL.FILE, d.getDirectory());
	if (d.getFile()!=null) {
	    SLURL filename = new SLURL(SLURL.FILE, curDir.getPath(), d.getFile());
	    saveTextFile(filename, app, whichone);
		app.iface.frame().requestFocusInWindow() ;
	    return true;
	} else {
	    return false;
	}
    }

    public static final int SAVE_OUTPUT = 1;
    public static final int SAVE_INFO = 2;

    // return true iff the project was saved successfully
    private boolean saveTextFile(SLURL fname, Application app, int whichone) {
	//System.out.println("1: creating file obj: " + fname.getPath() + " " + fname.getFile());
	try {
	    app.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
	    if (!(YoYo.macintoshp && YoYo.macosxp) && 
		!fname.getFileExtension().toLowerCase().equals("txt")) {
		fname.setFileExtension("txt");
	    }
	    //System.out.println("creating file obj: " + fname.getPath() + " " + fname.getFile());
	    File file = new File(fname.getFullPath());
	    if (YoYo.macintoshp && !YoYo.macosxp) {
		//app.macutils.setTextFileType(file);
	    }
	    BufferedWriter out = new BufferedWriter(new FileWriter(file));
	    // stop world once we're committed to writing state to file
	    //synchronized(sl.runningp) {
	    //   sl.stopAll();
	    //}
	    String fileContents = "";
	    synchronized(sl.runningp) {
		switch (whichone) {
      		case SAVE_OUTPUT: fileContents = outWin.getContents(); break;
      		case SAVE_INFO: fileContents = infoWin.getContents(); break;
		}
	    }
	    out.write(fileContents, 0, fileContents.length());
	    out.close();
	    switch (whichone) {
	    case SAVE_OUTPUT: outputFilename = fname; break;
	    case SAVE_INFO: infoFilename = fname; break;
	    }
	    return true;
	} catch (Exception ioe) {
	    String window = "unknown";
	    switch (whichone) {
	    case SAVE_OUTPUT: window = "output"; break;
	    case SAVE_INFO: window = "info"; break;
	    }
	    ioe.printStackTrace(System.err);
	    Application.createMessageDialog(app, app.activeFrame, 
					    app.textBundle.getString("starlogo_dialog_title"),
					    app.textBundle.getString("error_saving") + " " + 
					    window + app.textBundle.getString("to_file") + ": " + 
					    ioe.getMessage(), false);
	    return false;
	}
	finally {
	    app.restoreCursor();
	}
    }


    // return true if file was saved successfully
    public boolean saveProjectAs(Application app, boolean htmlp) {
	FileDialog d = new FileDialog((Frame)app.activeFrame, 
				      app.textBundle.getString("file_menu_save_project") + "...", 
				      FileDialog.SAVE);

	if (curDir!=null) {
	    d.setDirectory(curDir.getPath());
	}
	d.setFilenameFilter(app.projectFileFilter);
	d.show();
	curDir = new SLURL(SLURL.FILE, d.getDirectory());
	if (d.getFile()!=null) {
	    //    don't ensure .slogo ending 'cause it might overwrite existing file (need to check)
	    //    if (filename.indexOf(".")==-1)
	    //      filename = filename + ".slogo";
	    filename = new SLURL(SLURL.FILE, curDir.getPath(), d.getFile());
	    saveFile(filename, app, htmlp);
		app.iface.frame().requestFocusInWindow() ;
	    return true;
	} else {
	    return false;
	}
    }



    // return true iff the project was saved successfully
    private boolean saveFile(SLURL fname, Application app, boolean htmlp) {
	try {
	    app.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
	    if (!(YoYo.macintoshp && !YoYo.macosxp)) { // add the extension on mac os x too.
		if (!fname.getFileExtension().equalsIgnoreCase("slogo") &&
		    !fname.getFileExtension().equalsIgnoreCase("slo")) {
		    fname.addFileExtension("slogo");
		}
	    }
	    String fileContents = "";
	    try {
		// always save the bytecodes, force htmlp to be true
		fileContents = getContents(true);
	    } catch (BadWorldException bwe) {
		fileContents = getContentsMinusWorld(true);
		// Change filename to recovered version
		String directory = fname.getPath();
		String shortfilename = fname.getFile();
		// XXX Needs localization
		String s = app.textBundle.getString("error_saving_file") + " " +
		    app.textBundle.getString("recovered_with_name") + " \"(" +
		    app.textBundle.getString("recovered") + ") " + shortfilename + "\". " +
		    app.textBundle.getString("old_version_intact");
		fname.setFile("(" + app.textBundle.getString("recovered") + ") " + 
			      fname.getFile());
		Application.createMessageDialog(app, app.activeFrame, app.textBundle.getString("error_label"), 
						s, true);
	    }
	    //System.out.println("file contents = " + fileContents);

	    fname.save(fileContents, "UTF-8");

	    clearChanged();
	    //filename = fname;
	    app.iface.setTitle("StarLogo - " + filename.getFileNoExtension());
	    if (!htmlp) app.procWin.setTitle(app.textBundle.getString("windows_menu_control_center") +
					     " - " + filename.getFileNoExtension());

	    /* Save HTML */
	    if (htmlp) {
		FileDialog d = new FileDialog((Frame)app.activeFrame, 
					      app.textBundle.getString("save_html_template"),
					      FileDialog.SAVE);
		if (curDir != null) {
		    d.setDirectory(curDir.getPath());
		}
		d.setFilenameFilter(app.htmlFileFilter);
		d.show();
		curDir = new SLURL(SLURL.FILE, d.getDirectory());
		if (d.getFile() != null) {
		    SLURL htmlfilename = new SLURL(SLURL.FILE, curDir.getPath(), d.getFile());
		    saveHTMLTemplate(htmlfilename, fname.getFile(), 
				     app.iface.getTitle(), app);
		}
	    }
	    if (backupPendingDeletion != null) {
		// backup ready to be deleted now that we have saved successfully
		File backupfile = new File(backupPendingDeletion.getFullPath());
		backupfile.delete();
		backupPendingDeletion = null;
	    }
	    if (timer != null) timer.abort();
	    timer = new Timer(this, BACKUP_DELAY);
		app.iface.frame().requestFocusInWindow() ;
	    return true;
	} catch (Exception ioe) {
	    ioe.printStackTrace();
	    Application.createMessageDialog(app, app.activeFrame, 
					    app.textBundle.getString("starlogo_dialog_title"),
					    app.textBundle.getString("error_saving_project") + ": " +
					    ioe.getMessage(), false);
	    return false;
	}
	finally {
	    app.restoreCursor();
	}
    }

    private boolean saveHTMLTemplate(SLURL fname, String pname, String ptitle, Application app) {
	//System.out.println("fname = " + fname + " pname = " + pname + " ptitle = " + ptitle + " app = " + app);
	try {
	    if (!(YoYo.macintoshp && !YoYo.macosxp)) {
		if (!fname.getFileExtension().equalsIgnoreCase("html") &&
		    !fname.getFileExtension().equalsIgnoreCase("htm")) {
		    fname.setFileExtension("html");
		}
	    }
	    int iface_width = app.iface.getSize().width;
	    int iface_height = app.iface.getSize().height;
	    String template = readTemplate(pname, ptitle, YoYo.getLanguageAsString(), iface_width, iface_height);
	    File file = new File(fname.getFullPath());
	    BufferedWriter out = new BufferedWriter(new FileWriter(file));
	    out.write(template);
	    out.close();
	    if (YoYo.macintoshp && !YoYo.macosxp) {
		//app.macutils.setHTMLFileType(file);
	    }
	    return true;
	} catch (Exception ioe) {
	    ioe.printStackTrace();
	    Application.createMessageDialog(app, app.activeFrame, 
					    app.textBundle.getString("starlogo_dialog_title"),
					    app.textBundle.getString("error_saving_project") + ": " +
					    ioe.getMessage(), false);
	    return false;
	}
    }

    private String readTemplate(String pname, String ptitle, String plang, int width, int height) 
	throws IOException, MalformedURLException {
  	Pattern rewidth = Pattern.compile("\\@width\\@");
  	Pattern reheight = Pattern.compile("\\@height\\@");
  	Pattern repname = Pattern.compile("\\@project\\@");
  	Pattern reptitle = Pattern.compile("\\@projecttitle\\@");
	Pattern replang = Pattern.compile("\\@language\\@");

	URL u = Project.class.getResource("project-template.html");

  	InputStreamReader isr = new InputStreamReader(u.openStream());
	String template = PrimFile.readStream(isr, -1, null);
	
	template = rewidth.matcher(template).replaceAll(String.valueOf(width));
	template = reheight.matcher(template).replaceAll(String.valueOf(height));
  	template = repname.matcher(template).replaceAll(pname);
  	template = reptitle.matcher(template).replaceAll(ptitle);
  	template = replang.matcher(template).replaceAll(plang);

	return template;
    }

    public String getNewVersionFilenamePC(String fname) {
	int dotindex = fname.lastIndexOf(".");
	//if the file name has no numbers at the end
	if (!(Character.isDigit(fname.charAt(dotindex - 1))))
	    {fname = fname.substring(0, dotindex) + "01" + ".slogo";}
	//else if the file name has one number at the end (= the user implementing this by hand then switching)
	else if (!(Character.isDigit(fname.charAt(dotindex - 2))) && (Character.isDigit(fname.charAt(dotindex - 1)))) {
	    int tmp = Character.digit (fname.charAt(dotindex - 1), 10);
	    tmp++;
	    if (tmp < 10)
		{fname = fname.substring(0, dotindex - 1) + "0" + Integer.toString(tmp) + ".slogo";}
	    else {fname = fname.substring(0, dotindex - 1) + Integer.toString(tmp) + ".slogo";}
	}
	//else the file has previously been saved using the save next version feature
	else {
	    int tmp = Character.digit(fname.charAt(dotindex - 2), 10);
	    tmp *= 10;
	    tmp += Character.digit(fname.charAt(dotindex - 1), 10);
	    tmp ++;
	    if (tmp < 10)
		{fname = fname.substring(0, dotindex - 1) + Integer.toString(tmp) + ".slogo";}
	    else {fname = fname.substring(0, dotindex - 2) + Integer.toString(tmp) + ".slogo";}
	}
	return fname;
    }


    public static SLURL maybeUseBackupFile(SLURL name, Application app) {

	String promptIsBackup = app.textBundle.getString("prompt_is_backup");
	String promptHasBackup = app.textBundle.getString("prompt_has_backup");
	String promptRemove = app.textBundle.getString("prompt_remove_backup");
	String backupTag = "(" + app.textBundle.getString("backup") + ") ";

	// We only check for backup files, not URLs
	if (name.getProtocol() != SLURL.FILE) return name;
	
	// We don't check for backups in web applets
	if (app.kioskp || YoYo.applet != null) return name;

	if (name.getFileNoExtension().indexOf(backupTag) < 0) {
	    // This filename is not a backup
	    SLURL backup = addBackupTag(app, name);
	    File backupfile = new File(backup.getFullPath());
	    if (backupfile.exists()) {
		// there already is a backup file
		// XXX needs localization
		ChoiceDialog bd = 
		    new JBackupChoiceDialog(((Frame)app.activeFrame),
					    app.textBundle.getString("starlogo_dialog_title"),
					    promptHasBackup + " " + promptRemove,
					    app.textBundle.getString("yes_label"),
					    app.textBundle.getString("cancel_label"),
					    app.textBundle.getString("no_label"));
		String bakresult = bd.getResult();
		if (bakresult.equals(app.textBundle.getString("yes_label"))) {
		    // Delete the original file
		    File restoredfile = new File(name.getFullPath());
		    restoredfile.delete();
		    // Rename the backup to the original name
		    backupfile.renameTo(restoredfile);
		    return name;
		} else if (bakresult.equals(app.textBundle.getString("no_label"))) {
		    backupfile.delete();
		    return name;
		} else { // bakresult.equals("Cancel") || ..
		    return null;
		}
	    } else {
		return name;
	    }
	} else {
	    // looks like a backup file
	    SLURL nonbackup = removeBackupTag(app, name);
	    File nonbackupfile = new File(nonbackup.getFullPath());
	    if (nonbackupfile.exists()) {
		// XXX Needs localization
		ChoiceDialog bd = 
		    new JBackupChoiceDialog(((Frame)app.activeFrame),
					    app.textBundle.getString("starlogo_dialog_title"),
					    promptIsBackup + " " + promptRemove,
					    app.textBundle.getString("yes_label"),
					    app.textBundle.getString("cancel_label"),
					    app.textBundle.getString("no_label"));
		String bakresult = bd.getResult();
		if (bakresult.equals(app.textBundle.getString("yes_label"))) {
		    File replacedfile = new File(nonbackup.getFullPath());
		    replacedfile.delete();
		    File backupfile = new File(name.getFullPath());
		    backupfile.renameTo(replacedfile);
		    return nonbackup;
		} else if (bakresult.equals(app.textBundle.getString("no_label"))) {
		    File backupfile = new File(name.getFullPath());
		    backupfile.delete();
		    return nonbackup;
		} else { // bakresult.equals("Cancel") || ..
		    return null;
		}
	    } else {
		return name;
	    }
	}
    }

    private static SLURL addBackupTag(Application app, SLURL filename) {
	SLURL backup = new SLURL(filename);
	String backupTag = "(" + app.textBundle.getString("backup") + ") ";
	backup.setFile(backupTag + filename.getFileNoExtension());
	backup.setFileExtension(filename.getFileExtension());
	return backup;
    }

    private static SLURL removeBackupTag(Application app, SLURL backup) {
	// Make a copy of the filename
	SLURL filename = new SLURL(backup);
	String backupTag = "(" + app.textBundle.getString("backup") + ") ";
	filename.setFile(filename.getFileNoExtension().substring(backupTag.length()));
	filename.setFileExtension(backup.getFileExtension());
	return filename;
    }

    public void ding() {
	if (filename == null || (app.kioskp || YoYo.applet != null) || !contentsChanged) {
	    // no backup needed, try again later
	    timer = new Timer(this, BACKUP_DELAY);
	} else {
	    // don't delete the backup when we try to save it :)
	    this.backupPendingDeletion = null;
	    // what is the backup?
	    SLURL backup = addBackupTag(app, filename);
	    // check that we can save a backup
	    if ( ! new File((new File(backup.getFullPath())).getParent()).canWrite()) {
		// we will not be able to write the backup; bail out!
		return;
	    }
	    // save a backup
	    saveFile(backup, sl.app, false);
	    // mark the file as changed (it must have been to reach this clause)
	    // because the changed flag was reset by the saveFile method
	    markChanged();
	    // remember to delete this backup when the file is saved normally
	    this.backupPendingDeletion = backup;
	}
    }

    public String getNewVersionFilenameMac(String fname) {
	//if we're on a mac, do the mac version of adding version numbers.
	//this version has to check to see if there's an extension or not
	if (fname.toLowerCase().endsWith(".slogo")) {
	    return getNewVersionFilenamePC(fname);
	} else {
	    int dotindex = fname.length() - 1;
	    if (!(Character.isDigit(fname.charAt(dotindex))))
		{fname = fname + "01";}
	    else if (!(Character.isDigit(fname.charAt(dotindex - 1))) &&
		     (Character.isDigit(fname.charAt(dotindex)))) {
		int tmp = Character.digit(fname.charAt(dotindex), 10);
		tmp++;
		if (tmp < 10)
		    {fname = fname.substring(0, dotindex) + "0" + Integer.toString(tmp);}
		else {fname = fname.substring(0, dotindex - 1) + Integer.toString(tmp);}
	    }
	    else {
		int tmp = Character.digit(fname.charAt(dotindex - 1), 10);
		tmp *= 10;
		tmp += Character.digit(fname.charAt(dotindex), 10);
		tmp ++;
		if (tmp < 10)
		    {fname = fname.substring(0, dotindex) + Integer.toString(tmp);}
		else {fname = fname.substring(0, dotindex - 1) + Integer.toString(tmp);}
	    }
	}
	return fname;

    }

    // return true if the project was saved successfully
    private boolean saveProjectVersion(SLURL fname, Application app, boolean htmlp) {
	app.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
	try {

	    //if we're on a pc, force the file extension
	    if (!(YoYo.macintoshp && !YoYo.macosxp)) {
		if (!fname.getFileExtension().toLowerCase().equals("slogo") &&
		    !fname.getFileExtension().toLowerCase().equals("slo")) {
		    fname.setFileExtension("slogo");
		}
		fname.setFile(getNewVersionFilenamePC(fname.getFile()));
	    } else {
		fname.setFile(getNewVersionFilenameMac(fname.getFile()));
	    }
	    String fileContents = "";
	    try {
		fileContents = getContents(true);
	    } catch (BadWorldException bwe) {
		String directory = fname.getPath();
		String shortfilename = fname.getFile();
		// XXX Needs localization
		String s = app.textBundle.getString("error_saving_file") + " " +
		    app.textBundle.getString("recovered_with_name") + " \"(" +
		    app.textBundle.getString("recovered") + ") " + shortfilename + "\". " +
		    app.textBundle.getString("old_version_intact");
		fname.setFile("(" + app.textBundle.getString("recovered") + ") " + 
			      fname.getFile());
		Application.createMessageDialog(app, app.activeFrame,
						app.textBundle.getString("error_label"), s, true);
	    }
	    //System.out.println("file contents = " + fileContents);

	    fname.save(fileContents, "UTF-8");

	    clearChanged();
	    filename = fname;
	    app.iface.setTitle("StarLogo - " + filename.getFileNoExtension());
	    if (!htmlp) app.procWin.setTitle(app.textBundle.getString("windows_menu_control_center") +
					     " - " + filename.getFileNoExtension());
	    
	    if (backupPendingDeletion != null) {
		// backup ready to be deleted now that we have saved successfully
		File backupfile = new File(backupPendingDeletion.getFullPath());
		backupfile.delete();
		backupPendingDeletion = null;
	    }
	    if (timer != null) timer.abort();
	    timer = new Timer(this, BACKUP_DELAY);

	    return true;
	} catch (Exception ioe) {
	    ioe.printStackTrace();
	    Application.createMessageDialog(app, app.activeFrame, 
					    app.textBundle.getString("starlogo_dialog_title"),
					    app.textBundle.getString("error_saving_project") + ": " +
					    ioe.getMessage(), false);
	    return false;
	}
	finally {
	    app.restoreCursor();
	}
    }


    private String getContents(boolean htmlp) throws BadWorldException, IOException {
	synchronized(sl.runningp) {
	    String world = sl.saveWorld(htmlp);
	    assert world != null;
	    return getContentsMinusWorld(htmlp) + 
		sl.saveWorld(htmlp) + "\r\n"; 
	}
    }
    
    private String getContentsMinusWorld(boolean htmlp)	throws IOException {
	return HEADER + "\r\n" +
	    Float.toString(CURRENT_VERSION) + "\r\n" +
	    YoYo.getLanguageAsString() + "\r\n" +
	    procWin.getSaveString() + "\r\n" +
	    infoWin.getSaveString() + "\r\n" +
	    iface.getSaveString(htmlp) + "\r\n" +
	    TurtleShapes.saveShapes() + "\r\n" +
	    ((YoYo.swingavailablep && !YoYo.macintoshp) 
	     ? theTemplate.save() : "") + "\r\n" +
	    sl.saveSettings() + "\r\n";
    }

    boolean compilingp = false;

    public void compile(Application app, boolean justVariables, ProgressBar pb) {
	//try { throw new Exception(); } catch (Exception e) { e.printStackTrace(); }
	//System.out.println("compiling with pb: " + pb + "...");
	if (!compilingp) {
	    //System.out.println("hadn't compiled before");
	    if (!app.kioskp && YoYo.applet == null) {
		//System.out.println("beginning compile");
		compilingp = true;
		markChanged();
		app.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		//System.out.println("calling yoyo compiler");
		procWin.compileText(pb, justVariables);
		if (!justVariables) iface.compileWidgets(pb);
		enqueueCompileDone(app, justVariables, pb);
	    } else {
		app.compiled = true;
		app.rancompilerp = true;
		app.compiledVariables = true;
		app.rancompilerpForVariables = true;
	    }
	}
    }

    public void enqueueCompileDone(Application app, boolean justVariables, ProgressBar pb) {
	//System.out.println("enqueuing compile done");
	if (justVariables) {
	    Object[] newlist = { Symbol.lookup("project-compile-variables-done"), pb };
	    app.cc.enqueue(newlist, app.message, app.err, null);
	} else {
	    Object[] newlist = { Symbol.lookup("project-compile-done"), pb };
	    app.cc.enqueue(newlist, app.message, app.err, null);
	}
    }

   
    public void compileDone(Application app, ProgressBar pb) {
	//System.out.println("compile done");
	compilingp = false;
	app.compiled = true;
	app.rancompilerp = true;
	pb.done();
	app.restoreCursor();
	Application.updateEntityMonitors();
	app.runMonitorsOnce();
    }

    public void compileVariablesDone(Application app, ProgressBar pb) {
	//System.out.println("compile done");
	compilingp = false;
	app.compiledVariables = true;
	app.rancompilerpForVariables = true;
	pb.done();
	app.restoreCursor();
	Application.updateEntityMonitors();
	//app.runMonitorsOnce();
    }

    public void compileFailed(Application app, ProgressBar pb) {
	//System.out.println("compile failed");
	compilingp = false;
	app.compiled = false;
	app.rancompilerp = true;
	app.compiledVariables = false;
	app.rancompilerpForVariables = true;
	pb.done();
	app.restoreCursor();
    }

    private static String convertFormat(String old) {
	int first = old.indexOf("@#$#@#$#@");
	// for now assume that a file is a valid old starlogo file if and only if it
	// contains the string "@#$#@#$#@"
	if (first==-1) return old;
	String procs = old.substring(0, first);
	int endofiface = old.indexOf("@#$#@#$#@", first + 9);
	String iface = old.substring(first+9, endofiface);
	int endofinfo = old.indexOf("@#$#@#$#@", endofiface + 9);
	String info = (endofinfo == -1) ?
	    old.substring(endofiface+9) : old.substring(endofiface+9, endofinfo);
	String result =
	    HEADER + "\r\n" +
	    Float.toString(MAC_VERSION) + "\r\n" +
	    "`observer`\r\n`turtle`\r\n" + procs + "\r\n" +
	    "`information`\r\n" + info + "\r\n" +
	    "`interface`\r\n";

	Tokenizer tokenizer = new Tokenizer(iface);
	ArrayList items = tokenizer.tokenize();
	int pos = 0;
	String word1, word2 = "";
	boolean quoting = false;
	for (int i = 0; i<items.size()-1; i++) {
	    word1 = items.get(i).toString();
	    word2 = items.get(i+1).toString();
	    pos = iface.indexOf(word1, pos)+word1.length();
	    int npos = iface.indexOf("\n", pos);
	    int rnpos = iface.indexOf("\r\n", pos);
	    int rpos = iface.indexOf("\r", pos);
	    int tpos = iface.indexOf("\t", pos);
	    int word2pos = iface.indexOf(word2, pos);
	    boolean linebreak = (npos > -1 && npos < word2pos) ||
		(rnpos > -1 && rnpos < word2pos) ||
		(rpos > -1 && rpos < word2pos) ||
		(tpos > -1 && tpos < word2pos);
	    if (word1.equals("BUTTON")||
		word1.equals("SLIDER")||
		word1.equals("MONITOR"))
		result = result.concat("\r\n");
	    if (quoting) {
		if (linebreak) {
		    quoting = false;
		    result = result.concat(word1 + "\"\r\n");
		} else {
		    result = result.concat(word1 + " ");
		}
	    } else {
		if (linebreak) {
		    result = result.concat(word1 + "\r\n");
		} else {
		    quoting = true;
		    result = result.concat("\""+word1 + " ");
		}
	    }
	}
	result = result.concat(word2);
	return(result);
    }

    private void setVersion(String str) {
	Tokenizer tokenizer = new Tokenizer(str.substring(HEADER.length()));
	version = Float.parseFloat(tokenizer.getNextToken());
    }

    public void done(Context c) {}

    private static class WrongEncodingException extends Exception {}

}


