/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.awt.*;
import java.awt.event.*;
import java.util.Enumeration;
import java.util.Vector;
import yoyo.*;

public class SColorChooser extends Component implements StarLogoCaller, DisplayMessage {
  
    private boolean selected = false;

    public JInterface iface;
    public StarLogo sl;

    private SColortool colort;
    public int drawColor;
    String cnum;
    String cname;
    Rectangle position;
    public Rectangle clipRect = new Rectangle();
    boolean runningp;
    StringBuffer tempStringBuffer = new StringBuffer();
    Component keyboardFocus;
    
    
    Font WIDGET_FONT = new Font("Dialog", Font.PLAIN, 10);
    public FontMetrics fm = getFontMetrics(WIDGET_FONT);

    public SColorChooser(SColortool colortool, JInterface iface) {
	colort = colortool;
	this.iface = iface;
	this.sl = iface.sl();
	drawColor = colort.curcolor;
	cnum = String.valueOf(drawColor);
	//System.out.println(fm);
	position = new Rectangle(0,0,70,50);
    }
  
    public void setColor(int color) {
	drawColor = colort.curcolor;
	drawColor = color;
	addToClipRect(position);
	Graphics g = getGraphics();
	paint(g);
	g.dispose();
    }
  
    public void paint(Graphics g) {
	//      System.out.println(colort.curcolor);
	drawColor = colort.curcolor;
	g.translate(position.x,position.y);
	Font oldFont = g.getFont();
	Color oldColor = g.getColor();
    
	// draw boxes
	g.setColor(SystemColor.control);
	g.fillRect(0,0,70,20);
	g.draw3DRect(0,1,69,14,false);
	g.draw3DRect(0,17,69,14,false);
    
	// draw top string
	g.setFont(WIDGET_FONT);
	cnum = String.valueOf(drawColor);
	if (keyboardFocus == this) {
	    highlight(g);
	} else {
	    g.setColor(Color.black);
	    int length = fm.stringWidth(cnum);
	    g.drawString(cnum,66-length,11);
	}
  	
  	// draw bottom string
	g.setColor(SystemColor.control);
	g.fill3DRect(0,17,69,14,false);
	g.setFont(WIDGET_FONT);
	g.setColor(Color.white);	
	cname = getColorString();
	int s2pos = fm.stringWidth(cname);
	g.drawString(cname,66-s2pos,27);
   
	// restore old values
	g.setFont(oldFont);
	g.setColor(oldColor);
	g.translate(-position.x,-position.y);
    }

    private String getColorString() {
	drawColor = colort.curcolor;
	//paint(getGraphics());
	if (drawColor == 0 || drawColor == 9) 
	    return Colors.colorNumToName(drawColor);
		
	String col = Colors.colorNumToName((drawColor / 10) * 10 + 5);
	    
	String colsuffix = "";
		
	int maincol = (drawColor % 10);
	if (maincol < 5)
	    colsuffix = " - " + (5-maincol);
	if (maincol == 5)
	    colsuffix = "";
	if (maincol > 5)
	    colsuffix = " + " + (maincol-5);
			
	return (col + colsuffix);
    }
    public String showColor(){
	return (getColorString());
    }

    public void mouseClicked(MouseEvent e) {
	drawColor = colort.curcolor;
	//need to find the equivalent thing
	if (keyboardFocus == this) {
	    enter();
	} 
	else {
	    select();
	}
	addToClipRect(position);
    }
	
    private void highlight(Graphics g) {	
	String s = tempStringBuffer.toString();
	int length = fm.stringWidth(s);
	g.setColor(Color.white);
	g.fillRect(1,2,68,13);
	g.setColor(Color.blue);		
	g.fillRect(65-length,2,length+3,13);
	g.setColor(Color.white);
	g.setFont(WIDGET_FONT);
	g.drawString(s,66-length,11);
    }
	
    private void highlight() {
	Graphics g = iface.getGraphics();
	g.translate(position.x,position.y);
	highlight(g);
    }
	
    public void chop() {
	if (tempStringBuffer.length() == 0) return;
	tempStringBuffer.setLength(tempStringBuffer.length()-1);
	highlight();
    }
	
    public void add(char c) {
	if (tempStringBuffer.length() > 10) return;
	tempStringBuffer.append(c);
	highlight();
    }
	
    public void restoreOld() {
	if (selected) addToClipRect(position);
	deselect();
    }
	
    public void keyTyped(char c) {
  	if (c == KeyEvent.VK_ESCAPE) {
	    restoreOld();
  	} else if ((c == KeyEvent.VK_DELETE) || (c == KeyEvent.VK_BACK_SPACE)) {
	    chop();
  	} else if (c == KeyEvent.VK_ENTER) {
	    enter();
  	}	else if ((c > 31) && (c < 128)) {
	    add(c);
  	} else {
	    return;
  	}
  	addToClipRect(position);
    }
	
    public void select() {
	selected = true;
	keyboardFocus = this;
	tempStringBuffer = new StringBuffer("");
	//tempStringBuffer = new StringBuffer(cnum);
    }
	
    public void deselect() {
	selected = false;
	keyboardFocus = null;
	tempStringBuffer = new StringBuffer();
    }
	
    public void enter() {
	String s = tempStringBuffer.toString();
	if (s.trim().equals("")) {
	    restoreOld();
	    return;
	}
	handlecr(tempStringBuffer.toString());
	deselect();
    }
	
    private void handlecr(String s) {
	runline("compile-colorchooser [%setcc " + s + "]");
    }
	
    private class SColorChooserRunner implements Runnable {
	private String s;
	public SColorChooserRunner(String s) {
	    this.s = s;
	}
	public void run() {
	    runningp = true;
	    if (s != null && s != "") {
		Object[] listtocompile = Reader.read(s);
		Object[] newlist = new Object[listtocompile.length + 1];
		System.arraycopy(listtocompile, 0, newlist, 0, listtocompile.length);
		newlist[newlist.length -1] = SColorChooser.this;
		sl.app.cc.enqueue(newlist, sl.app.message, sl.app.err, null);
	    }
	}
    }

    private void runline(String s) {
	sl.app.maybeCompileAndThenRun(false, new SColorChooserRunner(s));
    }

    public void done(Context c) {};
	
    public void runCompiledCommand(Object[] ilist, Object[] olist) {
	int[] newarray = new int[ilist.length];
	for(int i = 0; i<ilist.length; i++) {
	    newarray[i] = ((Number)ilist[i]).intValue();
	}
	if (sl.locked == true) {
	    displayOneString(sl.app.textBundle.getString("cant_run_color_chooser") + " " +
			     sl.app.textBundle.getString("cant_run_while_obscured"));
	}
	else {
	    synchronized(sl.runningp) {
		int whichip = sl.internalSetInstructions(toString(), newarray, olist);
		StackProcessParent.runObserver(sl, whichip, false, this);
	    }
	}
    }

    private Vector spps = new Vector();
    private int sppcount = 0;

    public void done(StackProcessParent spp) {
	//		System.out.println("sl command done " + spp);
	spps.removeElement(spp);
	sppcount--;
	//		System.out.println("slcommand empty? " + spps.isEmpty() + " runningp = " + runningp);
	if (sppcount == 0 && runningp) {
	    //		    System.out.println("hi");
	    //super.done((Context)null);
	    runningp = false;
	    //linemark = -1;
	}
    }

    public boolean donep() {
	//System.out.println("slcommand done? " + (sppcount == 0));
	return (sppcount == 0);
    }

    public StarLogoCaller caller() {
	return this;
    }

    public void addStackProcessParent(StackProcessParent s) {
	spps.addElement(s);
	sppcount++;
    }

    public void stopAll() {
	//System.out.println("slcommand stop all");
	for(Enumeration e = ((Vector)spps.clone()).elements(); e.hasMoreElements();) {
	    StackProcessParent spp = (StackProcessParent)e.nextElement();
	    //System.out.println("trying to stop " + spp);
	    spp.stopAll();
	}
	//spps.removeAllElements();
    }

    public void callerPrintln(String s) {
	iface.statusbar().setStatusMessage(s);
    }

    public void callerClearText() {
	iface.statusbar().setStatusMessage("");
    }
  
    public void displayOneString(String s) {
	Application.createMessageDialog(sl.app, sl.app.activeFrame, sl.app.textBundle.getString("error_label"), 
					s, true);
    }

    public void displayAString(String s) {
	Application.createMessageDialog(sl.app, sl.app.activeFrame, sl.app.textBundle.getString("error_label"), 
					s, true);
    }
    public void addToClipRect(Rectangle r) {
	//        System.out.println(r);
	//        if (r.y == 10) {       try { throw (new Exception());} catch(Exception e) {e.printStackTrace();} }
  	if (clipRect.isEmpty()) {
	    clipRect.setBounds(r);
	} else clipRect = clipRect.union(r);
    }

  			
}

