/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;
import starlogo.Widget;
import java.lang.Math;
import java.awt.event.*;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;
import java.awt.*;
import java.net.*;
import yoyo.*;

public class SColortool extends Component {

    static final int cellWidth1 = 15;
    static final int cellWidth2 = 21;
    static final int cellHeight = 24;
    public Rectangle clipRect = new Rectangle();
    private JInterface iface;
    public Rectangle position;
    Font WIDGET_FONT = new Font("Dialog", Font.PLAIN, 10);
    public Font font = WIDGET_FONT; 
    public FontMetrics fm = getFontMetrics(font);
    public int colorDown = 0;
    public int shadeDown = 5;
    public int curcolor = 5;
    public static Color passColor = new Color(0);
    MediaTracker tracker;
    static Image emptyFrameImage;
    //Image im;

    HasColorChooser parent;

    public SColortool(Object parent){
	position = new Rectangle(0, 0, cellWidth2 * 10, cellHeight * 2); 
	if (parent instanceof HasColorChooser) 
	    this.parent = (HasColorChooser)parent;
	init();
    }

    public void init(){
	addMouseListener(new MouseAdapter() {
		public void mousePressed(MouseEvent e) {
		    if (!position.contains(new Point(e.getX(), e.getY()))) return;
		    int newcolor = -1, newshade = -1;
		    int row = (int)java.lang.Math.floor((e.getY()-position.y)/cellHeight);
		    if (row == 0) {
			newcolor = (int)java.lang.Math.floor((e.getX()-position.x)/cellWidth1);
			setColor(newcolor, shadeDown);
		    }
		    else {
			newshade = (int)java.lang.Math.floor((e.getX()-position.x)/cellWidth2);
			setColor(colorDown, newshade);
		    }
		}
		public void mouseReleased(MouseEvent e) {
		    mousePressed(e);
		}
	    });
	addMouseMotionListener(new MouseMotionAdapter() {
		public void mouseDragged(MouseEvent e) {
		    if (!position.contains(new Point(e.getX(), e.getY()))) return;
		    int newcolor = -1, newshade = -1;
		    int row = (int)java.lang.Math.floor((e.getY()-position.y)/cellHeight);
		    if (row == 0) {
			newcolor = (int)java.lang.Math.floor((e.getX()-position.x)/cellWidth1);
			setColor(newcolor, shadeDown);
		    }
		    else {
			newshade = (int)java.lang.Math.floor((e.getX()-position.x)/cellWidth2);
			setColor(colorDown, newshade);
		    }
		}
	    });

    }


    public Dimension getPreferredSize() {
	return new Dimension(210, 48);
    }

	
    public void setColor(int wholeColor) {
    	if ((wholeColor < 0) || (wholeColor >= Colors.numcolors)) {
	    return;
    	}

	//do a println in the other setcolor of what the curcolor is
    	setColor((wholeColor / 10), (wholeColor % 10));
	//Graphics g = getGraphics();
	//paint(g);
	//g.dispose();
    } 
    
    public void setColor(int newColor, int newShade) {
	if (newColor == colorDown && newShade == shadeDown) return;
	this.colorDown = newColor;
	this.shadeDown = newShade;
	curcolor = colorDown*10+shadeDown;
	if (parent != null) parent.colorchooser().setColor(curcolor);
	addToClipRect(position);
	repaint(clipRect);
    }
 
    public void paint(Graphics g) {
	g.translate(position.x, position.y);
	Color oldcolor = g.getColor();
	for (int i = 0; i < Colors.numprimarycolors; i++) {
	    int color = i*10+shadeDown;
	    g.setColor(Colors.colorarray[color * 32]);
	    g.fillRect(i*cellWidth1, 0, cellWidth1, cellHeight);
	}
	Color selectedColor = Colors.colorarray[curcolor * 32];
	passColor = selectedColor;
	int blueComponent = 255 - selectedColor.getBlue();
	int greenComponent = 255 - selectedColor.getGreen();
	int redComponent = 255 - selectedColor.getRed();
	Color highlightColor = new Color(redComponent, greenComponent, blueComponent);
	g.setColor(highlightColor);
	g.drawRect(colorDown*cellWidth1, 0, cellWidth1-1, cellHeight-1);
	for (int i = 0; i < 10; i++) {
	    int color = colorDown*10+i;
	    g.setColor(Colors.colorarray[color * 32]);
	    g.fillRect(i*cellWidth2, cellHeight, cellWidth2, cellHeight);
	}
	//the line below this was commented
	g.setColor(highlightColor);
	g.drawRect(shadeDown*cellWidth2, cellHeight, cellWidth2-1, cellHeight-1);
	
	g.translate(-position.x, -position.y);
    } 
 
    public static void loadImages(String gifdir, MediaTracker tracker, Toolkit kit) throws MalformedURLException {
	URL u1 = ToolbarWidget.class.getResource(gifdir + "emptycolorframe.gif");
	if (u1 == null) throw new MalformedURLException(); 
	emptyFrameImage = kit.createImage(u1);
	tracker.addImage(emptyFrameImage, Widget.trackerID++);
    }

    public void addToClipRect(Rectangle r) {
  	if (clipRect.isEmpty()) {
	    clipRect.setBounds(r);
	} else clipRect = clipRect.union(r);
    }

    public void repaint() {
	Graphics g = getGraphics();
	// set the clip for jview 5.00.3165 (and below?) that otherwise
	// assumes clip is from 0,0 -> 32767, 32767 and ends up not drawing
	// anything in paint
	g.setClip(new Rectangle(getSize()));
	paint(g);
	g.dispose();
    }

    public void repaint(Rectangle clip) {
	trim(clip);
	if (!clip.isEmpty()) {
	    Graphics g = getGraphics();
	    g.setClip(clip);
	    paint(g);
	    g.dispose();
	}
    }

    public void trim(Rectangle r) {
	// trims r so left boundary is non-negative (otherwise repaint causes bug in jdk1.2)
	if (r.getLocation().x<0)
	    r.setLocation(0, r.getLocation().y);
	if (r.getLocation().y<0) r.setLocation(r.getLocation().x, 0);
    }



} 


