/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.net.URL;
import java.net.MalformedURLException;
import java.io.File;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.OutputStreamWriter;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.charset.Charset;
import yoyo.LogoError;
import yoyo.YoYo;
import yoyo.Context;
import yoyo.PrimFile;

public class SLURL {

    static final int HTTP = 1;
    static final int FTP = 2;
    static final int FILE = 3;
    static final int HTTPS = 4;

    int protocol;

    String hostname = "";
    String path = "";
    String file = "";
    
    boolean pathp;

    public static final String fileseparator = System.getProperty("file.separator");
    public static final String urlseparator = "/";

    // if it's a URL, name is a hostname
    // if it's a file, name is a path
    // if it's an ftp link, name is a hostname
    public SLURL(int type, String name) {
	protocol = type;
	switch (type) {
	case HTTP: 
	case HTTPS:
	case FTP: hostname = name; break;
	case FILE: 
	    path = name;
	    if (path.endsWith(fileseparator)) 
		path = path.substring(0, path.length() - fileseparator.length());
	    pathp = true;
	    break;
	}
    }

    // if it's a URL, name1 is hostname, name2 is path
    // if it's a file, name1 is path and name2 is the file.
    public SLURL(int type, String name1, String name2) {
	protocol = type;
	switch (type) {
	case HTTP:
	case HTTPS:
	case FTP:
	    hostname = name1;
	    setNetworkPath(name2);
	    break;
	case FILE:
	    if (name1.endsWith(fileseparator)) {
		path = name1.substring(0, name1.length() - fileseparator.length());
	    } else path = name1;
	    file = name2;
	    break;
	}
    }
    
    public SLURL(int type, String hostname, String path, String file) {
	this.hostname = hostname;
	if (path.endsWith(urlseparator)) 
	    path = path.substring(0, path.length() - urlseparator.length());
	this.path = path;
	this.file = file;
	this.pathp = false;
    }

    public SLURL(URL u) {
	//System.out.println("Make SLURL from: " + u.toString());
	if (u.getProtocol().equals("file")) {
//use newish code to work for macs
//but, this newish code doesn't work! for mac loading locally. setFilePath not complete?
	  if (YoYo.macintoshp)
	     {this.protocol = FILE;
	      setFilePath(u.getHost() + u.getFile());}
	  else {
//use older code (just for the if protocol = file if) from 2/5/01 build for pcs
//windows remote and locally now works fine for ie, netscape4.7
		this.protocol = FILE;
		this.path = u.getFile();
		if (u.getFile().indexOf(':') < 0) {
	  	  if (u.getHost() != null && u.getHost().length() == 1) 
		   {this.path = u.getHost() + ":" + u.getFile();}
		}
		if (this.path.startsWith("/"))
	  	  {this.path = this.path.substring(1, this.path.length());}
		this.path = swapSlashes(this.path, urlseparator.charAt(0), fileseparator.charAt(0));
		if (path.endsWith(fileseparator)) {
	  	  path = path.substring(0, path.length() - fileseparator.length());
	  	  this.pathp = true;
		} else if (path.endsWith(urlseparator)) {
	  	  path = path.substring(0, path.length() - urlseparator.length());
	  	  this.pathp = true;
		} else {
	  	  int index = path.lastIndexOf(fileseparator.charAt(0));
	  	  if (index >= 0) {
		    String newpath = path.substring(0, index);
		    this.file = path.substring(index + 1, path.length());
		    this.path = newpath;
		    this.pathp = false;
	  	  } else {
		    index = path.lastIndexOf(urlseparator.charAt(0));
		    if (index >= 0) {
		      String newpath = path.substring(0, index);
		      this.file = path.substring(index + 1, path.length());
		      this.path = newpath;
		      this.pathp = false;}
	          }
	        }
	}

	} else if (u.getProtocol().equals("http")) {
	    this.protocol = HTTP;
	    this.hostname = u.getHost();
	    setNetworkPath(u.getFile());
	} else if (u.getProtocol().equals("https")) {
	    this.protocol = HTTPS;
	    this.hostname = u.getHost();
	    setNetworkPath(u.getFile());
	} else if (u.getProtocol().equals("ftp")) {
	    this.protocol = FTP;
	    this.hostname = u.getHost();
	    setNetworkPath(u.getFile());
	} else {
	    throw new LogoError("unknown protocol type: " + u.toString());
	}

	//System.out.println("slurl becomes: " + toString());
    }

    // interpret file in relation to URL u. file may be absolute.
    public SLURL(URL u, String file) {
	this(u);
	switch (protocol) {
	case FTP:
	case HTTP:
	case HTTPS:
	    if (absolutePathp(file)) {
		setNetworkPath(file);
	    } else setFile(file);
	    break;
	case FILE:
	    if (absolutePathp(file)) {
		setFilePath(file);
	    } else setFile(file);
	    break;
	}
    }

    public SLURL(SLURL u) {
	protocol = u.protocol;
	hostname = u.hostname;
	path = u.path;
	file = u.file;
	pathp = u.pathp;
    }

    public SLURL(SLURL u, String filename) {
	protocol = u.protocol;
	hostname = u.hostname;
	path = u.path;
	setFile(filename);
    }

    public boolean absolutePathp(String path) {
	if (path.startsWith(urlseparator) || path.startsWith(fileseparator)) {
	    return true;
	} 
	if (path.indexOf(':') == 1) return true;
	return false;
    }

    public String getFile() {
	if (pathp) {
	    return "";
	} else {
	    return file;
	}
    }

    public String getFileExtension() {
	if (pathp) {
	    return "";
	} else {
	    int index = file.lastIndexOf(".");
	    if (index < 0) return "";
	    return file.substring(index + 1, file.length());
	}
    }

    public String getFileNoExtension() {
	if (pathp) {
	    return "";
	} else {
	    int index = file.lastIndexOf(".");
	    if (index < 0) return file;
	    return file.substring(0, index);
	}
    }

    public void setFile(String newfile) {
	file = newfile;
	pathp = false;
    }

    public void setFileExtension(String ext) {
	if (!pathp) {
	    setFile(getFileNoExtension() + "." + ext);
	}
    }

    public void dropFile() {
	file = "";
	pathp = true;
    }

    public void addFileExtension(String ext) {
	if (!pathp) {
	    setFile(file + "." + ext);
	}
    }

    public void setPath(String newpath) {
	switch(protocol) {
	case HTTP:
	case HTTPS:
	case FTP:
	    setNetworkPath(newpath);
	    break;
	case FILE:
	    setFilePath(newpath);
	    break;
	}
    }

    protected void setNetworkPath(String newpath) {
	while (newpath.startsWith(urlseparator)) 
	    newpath = newpath.substring(urlseparator.length(), newpath.length());
	if (newpath.endsWith(urlseparator)) {
	    path = newpath.substring(0, newpath.length() - urlseparator.length());
	    pathp = true;
	} else {
	    // get the file at the end
	    int index = newpath.lastIndexOf(urlseparator.charAt(0));
	    if (index >= 0) {
		path = newpath.substring(0, index);
		file = newpath.substring(index + 1, newpath.length());
		pathp = false;
	    } else {
		path = newpath;
		pathp = true;
	    }
	}
    }
    
    protected void setFilePath(String newpath) {
	if (YoYo.pcp) {
	    // if it's missing the drive letter, and it's not a network file
	    if (newpath.indexOf(':') < 0 && !newpath.startsWith("\\")) {
		// get the old path's drive letter and stick it to the front
		if (path.indexOf(':') == 1) {
		    StringBuffer sb = new StringBuffer();
		    sb.append(path.charAt(0));
		    sb.append(':');
		    sb.append(newpath);
		    newpath = sb.toString();
		}
	    } else {
		// check for windows networking (oy)
		if (!newpath.startsWith("\\")) {
		    while (newpath.startsWith(fileseparator)) 
			newpath = newpath.substring(fileseparator.length(), newpath.length());
		}
	    }
	} else {
	    while (newpath.startsWith(fileseparator)) 
		newpath = newpath.substring(fileseparator.length(), newpath.length());
	}
	//   System.out.println("2. path is " + newpath);
	newpath = swapSlashes(newpath, urlseparator.charAt(0), 
			      fileseparator.charAt(0));
	//   System.out.println("3. path is " + newpath);
	if (newpath.endsWith(fileseparator)) {
	    path = newpath.substring(0, newpath.length() - fileseparator.length());
	    pathp = true;
	} else if (newpath.endsWith(urlseparator)) {
	    path = newpath.substring(0, newpath.length() - urlseparator.length());
	    pathp = true;
	} else {
	    int index = newpath.lastIndexOf(fileseparator.charAt(0));
	    if (index < 0) index = newpath.lastIndexOf(urlseparator.charAt(0));
	    if (index >= 0) {
		path = newpath.substring(0, index);
		file = newpath.substring(index + 1, newpath.length());
		pathp = false;
	    } else {
		path = newpath;
		pathp = true;
	    }
	}
    }


    public int getProtocol() {
	return protocol;
    }
    
    public String getPath() {
	return path;
    }
    
    public String getFullPath() {
	if (pathp) {
	    return path;
	} else {
	    switch(protocol) {
	    case HTTP:
	    case HTTPS:
	    case FTP:
		return urlseparator + path + urlseparator + file;
	    case FILE:
		if (!YoYo.pcp) return fileseparator + path + fileseparator + file;
		else return path + fileseparator + file;
	    }
	    return "unknown protocol";
	}
    }

    public String encode(String url) {
	StringBuffer buf = new StringBuffer(url.length());
	StringBuffer src = new StringBuffer(url);
	for(int i = 0; i < src.length(); i++) {
	    char c = src.charAt(i);
	    if ((c >= 0 && c <= 0x20) || (c == 0x7f)) {
		buf.append('%');
		buf.append(Integer.toHexString((int)(c & 0xFF)));
		continue;
	    }
	    switch(c) {
	    case '<': 
	    case '>':
	    case '\"':
	    case '#':
	    case '%':
	    case '{':
	    case '}':
	    case '|':
	    case '\\':
	    case '^':
	    case '~':
	    case '[':
	    case ']':
	    case '`':
	    case '?':
	    case ':':
	    case '@':
	    case '=':
	    case '&':
		buf.append('%');
		buf.append(Integer.toHexString((int)(c & 0xFF)));
		break;
	    default:
		buf.append(c);
	    }
	}
	return buf.toString();
    }

    public URL getURL() {
	try {
	    switch (protocol) {
	    case HTTP:
		return new URL("http", hostname, urlseparator + encode(path) + urlseparator + encode(file));
	    case HTTPS:
		return new URL("https", hostname, urlseparator + encode(path) + urlseparator + encode(file));
	    case FTP:
		return new URL("ftp", hostname, urlseparator + encode(path) + urlseparator + encode(file));
	    case FILE:
		return new URL("file", "", ((YoYo.pcp) ? "" : fileseparator) + path + fileseparator + file);
	    }
	} catch (MalformedURLException mue) { mue.printStackTrace(); }
	return null;
    }

    public URL getWebURL() {
	try {
	    switch (protocol) {
	    case HTTP:
		return new URL("http", hostname, urlseparator + encode(path) + urlseparator + encode(file));
	    case HTTPS:
		return new URL("https", hostname, urlseparator + encode(path) + urlseparator + encode(file));
	    case FTP:
		return new URL("ftp", hostname, urlseparator + encode(path) + urlseparator + encode(file));
	    case FILE:
		return new URL("file", "", urlseparator + 
			       swapSlashes(((YoYo.pcp || YoYo.macintoshp) ? "" : fileseparator) + path + fileseparator + file, 
					   fileseparator.charAt(0),
					   urlseparator.charAt(0)));
	    }
	} catch (MalformedURLException mue) { mue.printStackTrace(); }
	return null;
    }
	
    private String swapSlashes(String path, char fromslash, char toslash) {
	char[] pathchars = path.toCharArray();
	char[] newpathchars = new char[pathchars.length];
	for(int i = 0; i < pathchars.length; i++) {
	    if (pathchars[i] == fromslash) newpathchars[i] = toslash;
	    else newpathchars[i] = pathchars[i];
	}
	return new String(newpathchars);
    } 

    public String load(Context c) {
	switch (protocol) {
	case FTP:
	case HTTP:
	case HTTPS:
	    return (String)PrimFile.prim_urltostring(getWebURL(), c);
	case FILE:
	    if (YoYo.applet == null) {
		return (String)PrimFile.prim_filetostring(toString(), c);
	    } else return (String)PrimFile.prim_urltostring(getWebURL(), c);
	}
	return "";
    }

    public String load(Context c, String encoding) {
	switch (protocol) {
	case FTP:
	case HTTP:
	case HTTPS:
	    return (String)PrimFile.prim_urlwithencodingtostring(getWebURL(), encoding, c);
	case FILE:
	    if (YoYo.applet == null) {
		return (String)PrimFile.prim_filewithencodingtostring(toString(), encoding, c);
	    } else return (String)PrimFile.prim_urlwithencodingtostring(getWebURL(), encoding, c);
	}
	return "";
    }

    public byte[] loadBinary(Context c) {
//System.out.println("top of SLURL.load. getWebURL() = " + getWebURL());
	switch (protocol) {
	case FTP:
	case HTTP:
	case HTTPS:
	    return (byte[])PrimFile.prim_urltobytes(getWebURL(), c);
	case FILE:
	    if (YoYo.applet == null) {
		return (byte[])PrimFile.prim_filetobytes(toString(), c);
	    } else return (byte[])PrimFile.prim_urltobytes(getWebURL(), c);
	}
	return new byte[0];
    }

    public void save(String contents) throws IOException {
	assert protocol == FILE;
	File file = new File(getFullPath());
	BufferedWriter out = new BufferedWriter(new FileWriter(file));
	out.write(contents);
	out.close();
    }


    public void save(String contents, String encoding) throws IOException {
	assert protocol == FILE;
	File file = new File(getFullPath());
	BufferedWriter out = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(file),
								       Charset.forName(encoding)));
	out.write(contents);
	out.close();
    }

    public String toString() {
	switch (protocol) {
	case HTTP:
	    return "http://" + hostname + urlseparator + path + urlseparator + file;
	case HTTPS:
	    return "https://" + hostname + urlseparator + path + urlseparator + file;
	case FTP:
	    return "ftp://" + hostname + urlseparator + path + urlseparator + file;
	case FILE:
	    return ((YoYo.pcp) ? "" : fileseparator) + path + fileseparator + file;
	}
	return "unknown protocol: ";
    }

    public boolean exists() {
	assert protocol == FILE;
	File file = new File(getFullPath());
	return file.exists();
    }

}
