/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.IOException;
import java.net.*;
import java.util.Vector;
import java.util.Observable;
import java.util.Random;
import java.util.Enumeration;
import yoyo.*;
import java.lang.*;
import java.awt.Image;
import java.awt.image.ImageProducer;
import java.awt.image.ImageObserver;
import java.awt.image.ImageConsumer;
//import java.awt.geom.Rectangle2D; 
import java.awt.Graphics;
//import java.awt.dnd.*;
import java.awt.datatransfer.*;

public class ShapeIcon extends WidgetDrag implements Icon, ActionListener, ImageProducer {
    JShapeEditor parent;
    int shapeNum;
    //	int tcolmask = 0xFFFFFFFF;
    //int pcolmask = 0x00000000;
    int tcolmask;
    int pcolmask;
    Image image;
    ImageConsumer ic;  
    int[] pixels;
    private int size;
    int rowbytes;
    int width;
    int height;
    int heading;
    Application app;
    StarLogo sl;
    private boolean selected;
    private boolean rotatable;
    boolean rotating; 
    TurtleShapes tempShape;
    javax.swing.Timer timer;
    boolean frozen = false;
    int delay = 500; //milliseconds
    ActionListener taskPerformer;

    ShapeIcon(int shapeNum, int size, JShapeEditor parent) {
	
	this.tcolmask = parent.jsc.mainPanel.tcolmask;
	this.pcolmask = parent.jsc.mainPanel.pcolmask;
	this.shapeNum = shapeNum;
	this.rotatable = false;
	this.size = size;
	this.width = size;
	this.height = size;
	this.rowbytes = size; 
	this.parent = parent;
	image = parent.createImage(this);
	refresh();
    }

    public void refresh() {
	pixels = drawShape(shapeNum);
	if (ic != null) startProduction(ic);
    }

    public void updateSize(int newSize) {
	this.size = newSize;
	this.width = newSize;
	this.height = newSize;
	this.rowbytes = newSize;
	removeConsumer(null);
	image = parent.createImage(this);
	refresh();
    }

    public TurtleShapes getEditedShape() {
	return parent.editedShape;
    }

    public void setSelected(boolean value)
    {
	selected = value;
    }
	
    public boolean isSelected()
    {
	return selected;
	
    }
    public void setRotatable(boolean value)
    {
	rotatable = value;
	if (rotatable) {
	    timer= new javax.swing.Timer(delay, this);
	    timer.setInitialDelay(0);
	}
	
    }
	
    public boolean isRotatable()
    {
	return rotatable;
    }	

    public int getIconWidth()
    {
	return size;
    }

    public int getIconHeight()
    {
	return size;
    }

    public void actionPerformed(ActionEvent e){
	if(heading < 16){
	    //refresh();
	    rotating = true;
	    heading +=1;
	    refresh();
	    parent.repaint();
	}
	else{
	    timer.stop();
	    heading = 0;	//update the preview labels every time something happens

	}
    }

    public void rotate(){
	rotating = true;
	heading +=4;
	refresh();
    }
    
	
    public int[] drawShape(int shape) {
	if (!isRotatable()) {
	    tempShape = getEditedShape().copy();
	} else {
	    if(!rotating) {
		tempShape = getEditedShape().copy();
	    } else {
		tempShape = getEditedShape().copy();
		double[] rotcolors = new double[tempShape.getColorsForSize(size)[0].length];
		int[] rotmasks     = new int[tempShape.getMasksForSize(size)[0].length];
		TurtleShapes.rotate(heading, tempShape.getColorsForSize(size)[0], tempShape.getMasksForSize(size)[0], 
				    rotcolors, rotmasks);
		tempShape.setColorsForSize(rotcolors, size);
		tempShape.setMasksForSize(rotmasks, size);
		tempShape.reRotateShapes();
	    }
	}

	double[] colors = (tempShape.getColorsForSize(size))[0];
	int[]    masks  = (tempShape.getMasksForSize(size))[0];
	int[] legendImagePixels = new int[size*size];
	for (int z = 0; z < legendImagePixels.length; z++) {
	    int z2 = 2*z;
	    int tmask = masks[z2];
	    int pmask = masks[z2+1];
	    int colindex = (int)(colors[z] * 32.0);
	    int color = Colors.colorarrayints[colindex];
	    legendImagePixels[z] = (tmask & tcolmask) | (pmask & pcolmask) | color;
	    // (masks[h][(x + y*size) * 2] == ON) ? tc : ((masks[h][(x + y*size) * 2 + 1] == ON) ? 0 : c);
	    // 	    if (masks[2*z] == TurtleShapes.ON) {
	    // 		legendImagePixels[z] = tcolmask;
	    // 	    } else if (masks[2*z+1] == TurtleShapes.ON) {
	    // 		legendImagePixels[z] = pcolmask;
	    // 	    } else {
	    // 		legendImagePixels[z] = 
	    // 		    Colors.colorarrayints[(int)(colors[z] * 32.0)];
	    // 	    }
	}
	return legendImagePixels;
    }


    public void startProduction(ImageConsumer ic) {
	if (ic == null) return;
	if (this.ic == null) {
	    this.ic = ic; 
	    ic.setColorModel(Colors.cm);
	    ic.setDimensions(width, height);
	}
	ic.setHints(ImageConsumer.TOPDOWNLEFTRIGHT);
	if (pixels == null) pixels = drawShape(shapeNum);
	ic.setPixels(0, 0, width, height,
		     Colors.cm, pixels, 0, rowbytes);	// As many times as needed
	ic.imageComplete(ImageConsumer.SINGLEFRAMEDONE);
	ic.setHints(ImageConsumer.RANDOMPIXELORDER);
    }

    public void addConsumer(ImageConsumer ic) {
	startProduction(ic);
	//System.out.println("called add Consumer with " + ic);
    }

    public boolean isConsumer(ImageConsumer ic) {
	return (this.ic == ic); 
    }



    public void removeConsumer(ImageConsumer ic) {
	//System.out.println("called remove consumer wiht " + ic);
	this.ic = null; 
    }

    public void requestTopDownLeftRightResend(ImageConsumer ic) {
	ic.setHints(ImageConsumer.TOPDOWNLEFTRIGHT);
	if (pixels == null) pixels = drawShape(shapeNum);
	ic.setPixels(0, 0, width, height,
		     Colors.cm, pixels, 0, rowbytes);	// As many times as needed
	ic.imageComplete(ImageConsumer.SINGLEFRAMEDONE);
	ic.setHints(ImageConsumer.RANDOMPIXELORDER);
    }

    public void paintIcon(Component comp, Graphics g, int x, int y) {
	g.translate(position.x, position.y);
	g.drawImage(image, 0, 0,this);
	/*	if(selected){
	    g.setColor(Color.red);
	    g.drawRoundRect(0,0,getIconWidth(), getIconHeight(), 2, 2); 
	    }*/

	g.translate(-position.x, -position.y);
    }

    public void toggleRotation() {
	if (rotatable) {
	    if (timer.isRunning()) {
		timer.stop();
	    } else {
		timer.start();
	    }
	}
    }
 
}
