/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.util.Enumeration;
import java.util.Vector;
import java.util.ArrayList;
import yoyo.*;

public class SliderWidget extends WidgetDrag {

    private String oldname = null;
    public String var, name;
    // max = 1 to avoid divide by zero on init
    public int min, max = 1;
    Font font;
    public int id;
    public int cur;
    public Integer curInt;
    public boolean showName;

    Rectangle thumbPos;
    int handle;
    boolean sliding = false;
    public static int SLIDER_HEIGHT = 25;
    public static int idCount = 0;

    public int maxPixelVal = 77;
    public int minPixelVal = 4;

    static final int sliderTrackEdgeHeight = 11;
    static final int sliderTrackEdgeWidth = 6;

    static final int sliderBottomEdgeHeight = 13;
    static final int sliderBottomEdgeWidth = 5;

    static Image sliderEdges;
    static Image sliderMiddleTrack;
    static Image sliderMiddleBottom;
    static Image sliThumb;
    static Image sliTick;

    FontMetrics fm = null;

    int textDescent, textHeight, textWidth;
    int maxTextWidth, minTextWidth, maxValueWidth, minValueWidth, minmaxValueWidth;

    public Rectangle topRect;
    public Rectangle botRect;
    StarLogo sl;
    Application app;

    static Font SLIDER_FONT;

    static {
    	try {
	    SLIDER_FONT = WIDGET_FONT;
	    if (YoYo.macintoshp) SLIDER_FONT = Font.getFont("starlogo.slider.font", SLIDER_FONT);
    	} catch (Exception e) { e.printStackTrace(); }
    }

    public SliderWidget(String name, String var, Rectangle position,
			int min, int max, int cur, boolean shownamep, int sid,
			JInterface iface, StarLogo sl, String descript) {
	this.name = name;
	this.var = var;
	this.showName = shownamep;
	this.position = new Rectangle(position);
	this.position.height = SLIDER_HEIGHT;

	this.min = min;
	this.max = max;
	this.cur = cur;
	this.curInt = new Integer(cur);
	this.id = sid;
	this.parent = iface;
	this.sl = sl;
	this.app = sl.app;

	this.font = SLIDER_FONT;
	//addObserver(parent);
	topRect = new Rectangle(0, 0, position.width, sliderTrackEdgeHeight);
	botRect = new Rectangle(3, sliderTrackEdgeHeight + 1,
				position.width - 8, sliderBottomEdgeHeight);
	maxPixelVal = position.width - 13;
	thumbPos = new Rectangle(minPixelVal+(maxPixelVal - minPixelVal)*
				 (cur-min)/(max-min), 1, 9, 9);
	fm = parent.getToolkit().getFontMetrics(font);
	textDescent = fm.getMaxDescent();
	textHeight =  fm.getMaxAscent()+textDescent;

	if (!(ToolTipWidget.empty(descript))) setToolTip(new ToolTipWidget(parent, descript));
    }

    public SliderWidget(JInterface iface, StarLogo sl) {
	// trim iface.outline() to cover slider but also fit on screen
	if (iface.outline().x + MIN_WIDTH > iface.getSize().width) {
	    iface.outline().width = iface.outline().width + iface.outline().x -
		iface.getSize().width + MIN_WIDTH;
	    iface.outline().x=iface.getSize().width-MIN_WIDTH;
	}
	if (iface.outline().width<MIN_WIDTH)
	    iface.outline().width = MIN_WIDTH;
	if (iface.outline().y+SLIDER_HEIGHT>iface.getSize().height) {
	    iface.outline().height=iface.outline().height+iface.outline().y-
		iface.getSize().height+SLIDER_HEIGHT;
	    iface.outline().y=iface.getSize().height-SLIDER_HEIGHT;
	}
	if (iface.outline().height<SLIDER_HEIGHT)
	    iface.outline().height = SLIDER_HEIGHT;

	this.position = new Rectangle(iface.outline());
	this.position.height = SLIDER_HEIGHT;

	this.id = idCount++;
	this.parent = iface;
	this.sl = sl;
	this.app = sl.app;
	this.font = WIDGET_FONT;
	this.showName = false;
	topRect = new Rectangle(0, 0, position.width, sliderTrackEdgeHeight);
	botRect = new Rectangle(3, sliderTrackEdgeHeight + 1,
				position.width - 8, sliderBottomEdgeHeight);
	maxPixelVal = position.width - 13;
	thumbPos = new Rectangle(minPixelVal+(maxPixelVal - minPixelVal)*
				 (cur-min)/(max-min), 1, 9, 9);
	fm = parent.getToolkit().getFontMetrics(font);
	textDescent = fm.getMaxDescent();
	textHeight =  fm.getMaxAscent()+textDescent;

	//addObserver(parent);
    }

    //used for loading from file
    /**
     * this is used for loading mac star logo files
     */
    public SliderWidget(ArrayList args, int i, JInterface parent) {
	this.parent = parent;
	position = new Rectangle();
	position.x = new Integer(args.get(i++).toString()).intValue()+parent.getBoundaries().x;
	position.y = new Integer(args.get(i++).toString()).intValue()+parent.getBoundaries().y;
	position.width = new Integer(args.get(i++).toString()).intValue()+parent.getBoundaries().x-position.x;
	i+=1;
	//  position.height = new Integer(args.get(i++).toString()).intValue()+parent.getBoundaries().y-position.y;
	//  position.width = SLIDER_WIDTH;
	position.height = SLIDER_HEIGHT;
	name = args.get(i++).toString();
	var = args.get(i++).toString();
	min = new Integer(args.get(i++).toString()).intValue();
	max = new Integer(args.get(i++).toString()).intValue();
	cur = new Integer(args.get(i++).toString()).intValue();
	curInt = new Integer(cur);
	id = new Integer(args.get(i++).toString()).intValue();
	if (id >= idCount)
	    idCount = id + 1;
	showName = false;
	// this init is common to initializations from dialog and loading
	font = WIDGET_FONT;
	//addObserver(parent);
	//addObserver(sl.observer);
	topRect = new Rectangle(0, 0, position.width, sliderTrackEdgeHeight);
	botRect = new Rectangle(3, sliderTrackEdgeHeight + 1, position.width - 8, sliderBottomEdgeHeight);
	maxPixelVal = position.width - 13;
	thumbPos = new Rectangle(minPixelVal+(maxPixelVal - minPixelVal)*(cur-min)/(max-min), 1, 9, 9);
	fm = parent.getToolkit().getFontMetrics(font);
	textDescent = fm.getMaxDescent();
	textHeight =  fm.getMaxAscent()+textDescent;
	updateMinSize();

    }

    static public void createNew(JInterface iface) {
	SliderDialog sd = Application.createSliderDialog((Frame)iface.frame(), iface.app(), iface.app().textBundle.getString("new_slider"));
	sd.show();
	if (sd.pressedOK()) {
	    SliderWidget sw = new SliderWidget(iface, iface.sl());
	    sd.getValues(sw);
	    sw.updateMinSize();
	    iface.addWidget(sw);
	    iface.app().rancompilerp = false;
	    iface.app().rancompilerpForVariables = false;
	    //iface.app().project.compile(iface.app());
	}
	sd.dispose();
    }

    public void edit() {
	SliderDialog sd = Application.createSliderDialog((Frame)parent.frame(), app, app.textBundle.getString("edit_slider"), this);
	sd.show();
	if (sd.pressedOK()) {
	    boolean showNameOld = showName;
	    String oldName = name;
	    String oldVar = var;
	    // why add the observer here?
	    //addObserver(parent);
	    parent.addHandlesToClipRect(this);
	    sd.getValues(this);
	    //addObserver(sl.observer);
	    updateThumbPos();
	    if ((showNameOld != showName) ||
		(showName && !(oldName.equals(name))) ||
		(!showName && !(oldVar.equals(var)))) updateMinSize();
	    parent.addToClipRect(position);
	    app.rancompilerp = false;
	    app.rancompilerpForVariables = false;
	    app.project.markChanged();
	    //app.project.compile(app);
	}
	sd.dispose();
	super.edit();
    }

    /**
     * This is for current starlogo project files
     */
    public static SliderWidget load(ArrayList items, JInterface iface, StarLogo sl) {

	int top = 0, left = 0, width = 50, height = 20, sid = 1;
	int min = 0, max = 99, cur = 50;
	boolean shownamep = false;
	String name = "slider1", var = "nothing";
	String descript = new String();
	int i = 0;
	while (i<items.size()) {
	    String item = items.get(i).toString();
	    if (item.equals("top-left")) {
		String value = items.get(i+1).toString();
		String value2 = items.get(i+2).toString();
		top = Integer.parseInt(value);
		left = Integer.parseInt(value2);
		i += 3;
		continue;
	    }
	    if (item.equals("width-height")) {
		String value = items.get(i+1).toString();
		String value2 = items.get(i+2).toString();
		width = Integer.parseInt(value);
		height = Integer.parseInt(value2);
		i += 3;
		continue;
	    }
	    if (item.equals("name")) {
		name = items.get(i+1).toString();
		i += 2;
		continue;
	    }
	    if (item.equals("variable")) {
		var = items.get(i+1).toString();
		i += 2;
		continue;
	    }
	    if (item.equals("show-name?")) {
		String value = items.get(i+1).toString();
		shownamep = value.equals("true");
		i+=2;
		continue;
	    }
	    if (item.equals("slider-number")) {
		String value = items.get(i+1).toString();
		sid = Integer.parseInt(value);
		i += 2;
		continue;
	    }
	    if (item.equals("min-value")) {
		String value = items.get(i+1).toString();
		min = Integer.parseInt(value);
		i += 2;
		continue;
	    }
	    if (item.equals("max-value")) {
		String value = items.get(i+1).toString();
		max = Integer.parseInt(value);
		i += 2;
		continue;
	    }
	    if (item.equals("current-value")) {
		String value = items.get(i+1).toString();
		cur = Integer.parseInt(value);
		i += 2;
		continue;
	    }
	    if (item.equals("toolTip-text")) {
		String value = items.get(i+1).toString();
		descript = value;
		i += 2;
		continue;
	    }
	    i+=2;
	}

	Rectangle bounds = iface.getBoundaries();
	Rectangle position = new Rectangle(left + bounds.x, top + bounds.y, width, height);
	if (sid >= idCount) idCount = sid + 1;

	return new SliderWidget(name, var, position, min, max, cur,
				shownamep, sid, iface, sl, descript);
    }


    public String getSaveString(boolean htmlp) {
	Rectangle bounds = parent.getBoundaries();
	return "SLSlider\r\n" +
	    "top-left " + (position.y - bounds.y) + " " + (position.x- bounds.x) + "\r\n" +
	    "width-height " + position.width + " " + position.height + "\r\n" +
	    "name \"" + name + "\"\r\n" +
	    "variable \"" + var + "\"\r\n" +
	    "min-value " + min + "\r\n" +
	    "max-value " + max + "\r\n" +
	    "current-value " + cur + "\r\n" +
	    "slider-number " + id + "\r\n" +
	    "show-name? " + ((showName) ? "true\r\n" : "false\r\n") +
	    getToolTipSaveString() +
	    "\r\n";
    }

    public String toString() {
	return("Slider: " + var);
    }

    public void updateMinSize() {
	int textWidth = (showName ? fm.stringWidth(name) : fm.stringWidth(var));
	maxValueWidth = fm.stringWidth(String.valueOf(max));
	minValueWidth = fm.stringWidth(String.valueOf(min));
	maxValueWidth = (minValueWidth > maxValueWidth) ? minValueWidth : maxValueWidth;
	minmaxValueWidth = fm.stringWidth(String.valueOf(-999));
	//    if (maxValueWidth < minmaxValueWidth) maxValueWidth = minmaxValueWidth;

	maxTextWidth = textWidth + 2 * maxValueWidth;
	minTextWidth = textWidth + minValueWidth;
	maxTextWidth = (minTextWidth > maxTextWidth) ? minTextWidth : maxTextWidth;
	//    MIN_WIDTH = maxTextWidth + 6 + 2 * sliderBottomEdgeWidth;
	//    MIN_WIDTH = 6 + 2 * sliderBottomEdgeWidth;
	int rec_width = maxTextWidth + 6 + 2 * sliderBottomEdgeWidth;
	if (position.width < rec_width) {
	    position.width = rec_width;
	    //System.out.println("resetting width to min size = " + MIN_WIDTH);
	    resizeMe();
	}
	if (parent.outline() != null) {parent.outline().width = position.width;}
    }

    public boolean draggable(MouseEvent e) {
    	if (app.kioskp || app.locked || YoYo.applet != null) return false;
	boolean draggable = thumbPos.contains(e.getX() - position.x,
					      e.getY() - position.y);
	return !draggable;
    }

    public boolean activep;

    // only called on non-draggable slider widgets
    // the ones in the toolbar
    public void mousePressed(MouseEvent e) {
	//System.out.println("top of mousePressed in SliderWidget: " + e);
	if (app.kioskp || app.locked || YoYo.applet != null) {
	    if (thumbPos.contains(e.getX() - position.x, e.getY() - position.y)) {
		//System.out.println("point in thumb");
		handle = e.getX() - position.x - thumbPos.x;
		sliding = true;
		activep = true;
	    } else {
		//System.out.println("point not in thumb");
		activep = false;
	    }
	    return;
	}
	if (thumbPos.contains(e.getX() - position.x, e.getY() - position.y)) {
	    handle = e.getX()-position.x-thumbPos.x;
	    sliding = true;
	} else {
	    sliding = false;
	}
	activep = true;
	//sliding = false;
	//updateValue();
	//System.out.println("sliding is false");
    }

    public void mouseDragged(MouseEvent e) {
	//System.out.println("top of mouseDragged in SliderWidget: " + e);
	if (!activep) return;
	//System.out.println("about to update value");
	updateValue();
	if (sliding) {
	    thumbPos.x=e.getX()-handle-position.x;
	    updateValue();
	}
    }

    public void mouseClicked(MouseEvent e) {
	//System.out.println("top of mouseClicked in SliderWidget");
	if (sliding) {
	    //System.out.println("sliding in mouseClicked in SliderWidget");
	    sliding = false;
	    updateValueAtEnd();
	} else if (parent.getToolbar().buttonDown == ToolbarWidget.POINTER) {
	    //System.out.println("Else if in mouseClicked in SliderWidget");
	    if ((topRect.contains(e.getX()-position.x, e.getY()-position.y))&&
		(!thumbPos.contains(e.getX()-position.x, e.getY()-position.y))) {
		//System.out.println("position: " + position + " topRect: " + topRect + " thumbPos " + thumbPos);
		jumpThumb((e.getX()-position.x>thumbPos.x));
	    }
	}
    }

    /*
      public void mouseReleased(MouseEvent e) {
      System.out.println("top of mouseReleased in SliderWidget");
      }
    */

    void updateThumbPos() {
	thumbPos.x = minPixelVal + (maxPixelVal - minPixelVal)*(cur-min)/(max-min);
    }

    public void updateValue() {
	if (thumbPos.x < minPixelVal)
	    thumbPos.x = minPixelVal;
	if (thumbPos.x > maxPixelVal)
	    thumbPos.x = maxPixelVal;
	cur = (int)Math.round(min+(max-min)*(thumbPos.x-minPixelVal)/
			      ((double)maxPixelVal - minPixelVal));
	curInt = new Integer(cur);
	// by calling this (again) here, we generate a "snapping" behavior
	// updateThumbPos();
	// interface will redraw upon seeing position as arg
	parent.addToClipRect(position);
    }

    public void updateValueAtEnd() {
	if (thumbPos.x < minPixelVal)
	    thumbPos.x = minPixelVal;
	if (thumbPos.x > maxPixelVal)
	    thumbPos.x = maxPixelVal;
	cur = (int)Math.round(min+(max-min)*(thumbPos.x-minPixelVal)/
			      ((double)maxPixelVal - minPixelVal));
	curInt = new Integer(cur);
	// by calling this (again) here, we generate a "snapping" behavior
	updateThumbPos();
	// interface will redraw upon seeing position as arg
	parent.addToClipRect(position);
    }

    public Integer getValue() {
	return curInt;
    }

    public void setValue(int newval) {
	//System.out.println("Top of setValue in SliderWidget");
	if (newval >= min && newval <= max) {
	    cur = newval;
	    curInt = new Integer(cur);
	    thumbPos = new Rectangle(minPixelVal+(maxPixelVal - minPixelVal)*
				     (cur-min)/(max-min), 1, 9, 9);
	    updateThumbPos();
	    parent.addToClipRect(position);
	}
    }

    public boolean nameChanged() {
	// checks to see if name has changed, and resets oldname to name
	boolean result = (!name.equals(oldname));
	oldname = name;
	return result;
    }

    /*
      public void startDragging() {
      if (sliding)
      cursorNum = parent.getCursor().getType();
      super.startDragging();
      }
    */


    public void jumpThumb(boolean increasing) {
	if (increasing)
	    {if (max < 10) {cur = cur + 1;}
	    else {cur = (int)Math.min(max, cur + Math.floor((max - min) / 10));}}
	else
	    {if (max < 10) {cur = cur - 1;}
	    else {cur = (int)Math.max(min, cur - Math.floor((max - min) / 10));}}
	curInt = new Integer(cur);
	updateThumbPos();
	parent.addToClipRect(position);
    }

    public void delete() {
	packID();
	SliderWidget.idCount--;
	parent.sliders().removeElement(this);
	super.delete();
    }

    public void paint(Graphics g) {
	g.setFont(font);
	if (maxTextWidth < 0) { }

	//System.out.println("0. clip is " + g.getClip());
	g.translate(position.x, position.y);
	//System.out.println("translate to position (" + position.x + ", " + position.y + ")");
	Shape oldclip = g.getClip();
	//System.out.println("1. clip is " + g.getClip());
	/* Slider Track */
	/* Left side of the green track */
	g.clipRect(0, 0, sliderTrackEdgeWidth, sliderTrackEdgeHeight);
	//System.out.println("2. clip is " + g.getClip());
	g.drawImage(sliderEdges, 0, 0, null);
	
	/* Right side of the green track */
	g.setClip(oldclip);
	g.clipRect(position.width - sliderTrackEdgeWidth, 0,
		   sliderTrackEdgeWidth, sliderTrackEdgeHeight);
	// Move picture over to the left, so the right half shows up in the clip region
	g.drawImage(sliderEdges, position.width - (2 * sliderTrackEdgeWidth), 0, null);

	/* Slider Bottom */
	/* Left side of the gray track */
	g.setClip(oldclip);
	g.clipRect(3, sliderTrackEdgeHeight + 1, sliderBottomEdgeWidth, sliderBottomEdgeHeight);
	//System.out.println("3. clip is " + g.getClip());
	g.drawImage(sliderEdges, 3, 1, null);
	/* Right side of the gray track */
	g.setClip(oldclip);
	g.clipRect(position.width - 3 - sliderBottomEdgeWidth, sliderTrackEdgeHeight + 1,
		   sliderBottomEdgeWidth, sliderBottomEdgeHeight);
	//System.out.println("4. clip is " + g.getClip());
	g.drawImage(sliderEdges, position.width - 3 - (2 * sliderBottomEdgeWidth), 1, null);

	//System.out.println("5. clip is " + g.getClip());
	g.setClip(oldclip);
	g.clipRect(0, 0, position.width, position.height);
	g.drawImage(sliderMiddleTrack, sliderTrackEdgeWidth, 0,
		    position.width - (2 * sliderTrackEdgeWidth), sliderTrackEdgeHeight, null);
	g.drawImage(sliderMiddleBottom, 3 + sliderBottomEdgeWidth, sliderTrackEdgeHeight + 1,
		    position.width - (3*2) - (2*sliderBottomEdgeWidth),
		    sliderBottomEdgeHeight, null);
	g.drawImage(sliThumb, thumbPos.x, thumbPos.y, null);

	g.setColor(Color.black);

	/* Variable */
	g.setClip(oldclip);
	g.clipRect(6, sliderTrackEdgeHeight + 2, position.width - 12 - maxValueWidth - 6, 10);
	//System.out.println("6. clip is " + g.getClip());
	int heightAdjustment = (YoYo.macosxp) ? 2 : (YoYo.macintoshp || YoYo.linuxp) ? 1 : 0;
	g.drawString((showName ? name : var), 6, sliderTrackEdgeHeight
+ 7 + heightAdjustment + textHeight / 2 - textDescent);

	/* Current Value */
	g.setClip(oldclip);
	paintValue(g);

	g.setClip(oldclip);
	//System.out.println("8. clip is " + g.getClip());
	g.translate(-position.x, -position.y);
    }

    public void paintValue(Graphics g) {
	String curString = String.valueOf(cur);
	paintValue(g, curString);
    }

    public void paintValue(Graphics g, String curString) {
	textWidth = fm.stringWidth(curString);
	g.clipRect(position.width - 6 - textWidth, 
		  sliderTrackEdgeHeight + 2, 
		  textWidth, 10);
	//System.out.println("7. clip is " + g.getClip());
	int heightAdjustment = (YoYo.macosxp) ? 2 : (YoYo.macintoshp || YoYo.linuxp) ? 1 : 0;
	g.drawString(curString,
		     position.width - 6 - textWidth,
		     sliderTrackEdgeHeight + 7 + heightAdjustment + textHeight / 2 - textDescent);
    }

    private void packID() {
	Object o;
	for (Enumeration e = parent.wh().listDragWidgets(); e.hasMoreElements(); ) {
	    o = e.nextElement();
	    if (o instanceof SliderWidget) {
		SliderWidget sd = (SliderWidget)o;
		if (sd.id>id) {
		    sd.id--;
		    sd.name= app.textBundle.getString("slider_label")+sd.id;
		}
	    }
	}
    }

    public boolean contains(int x, int y) {
	return((topRect.contains(x-position.x, y-position.y)||
		botRect.contains(x-position.x, y-position.y)));
    }

    public boolean intersects(Rectangle rect) {
	Rectangle r = new Rectangle(rect);
	if (isSelected())
	    r.grow(WidgetDrag.CORNER_WIDTH, WidgetDrag.CORNER_WIDTH);
	r.translate(-position.x, -position.y);
	return((topRect.intersects(r)||(botRect.intersects(r))));
    }

    public int getToolbarButton() {
	// what button on the toolbar corresponds to this widgetdrag-- overrridden
	return ToolbarWidget.SLIDER;
    }

    /*
      public int calcCursor(int x, int y) {
      int i = super.calcCursor(x,y);
      if ((i==Cursor.NE_RESIZE_CURSOR)||(i==Cursor.SE_RESIZE_CURSOR))
      return(Cursor.E_RESIZE_CURSOR);
      else if ((i==Cursor.NW_RESIZE_CURSOR)||(i==Cursor.SW_RESIZE_CURSOR))
      return(Cursor.W_RESIZE_CURSOR);
      else
      return(i);
      }
    */

    public int resizeWhichCorner(Point anchor) {
	Rectangle test = new Rectangle();
	test.setBounds(position.x-CORNER_WIDTH, position.y-CORNER_WIDTH,
		       CORNER_WIDTH, CORNER_WIDTH);
	if (test.contains(anchor.x, anchor.y)) return JInterface.RESIZELEFT;

	test.setBounds(position.x+position.width, position.y-CORNER_WIDTH,
		       CORNER_WIDTH, CORNER_WIDTH);
	if (test.contains(anchor.x, anchor.y)) return JInterface.RESIZERIGHT;

	test.setBounds(position.x-CORNER_WIDTH, position.y+position.height,
		       CORNER_WIDTH, CORNER_WIDTH);
	if (test.contains(anchor.x, anchor.y)) return JInterface.RESIZELEFT;

	test.setBounds(position.x+position.width, position.y + position.height,
		       CORNER_WIDTH, CORNER_WIDTH);
	if (test.contains(anchor.x, anchor.y)) return JInterface.RESIZERIGHT;

	return 0;
    }


    public void resize(Point where, int anchorCorner) {
	super.resize(where, anchorCorner);
	resizeMe();
    }

    public void resizeMe() {
	topRect = new Rectangle(0, 0, position.width, sliderTrackEdgeHeight);
	botRect = new Rectangle(3, sliderTrackEdgeHeight + 1, position.width - 8,
				sliderBottomEdgeHeight);
	maxPixelVal = position.width - 13;
	updateThumbPos();
    }

    public static void loadImages(String gifdir, MediaTracker tracker, Toolkit kit)
	throws MalformedURLException{
	// requires: trackerID in Widget indexes start of empty ID #s in <tracker>


	sliderEdges = kit.createImage(Pool.class.getResource(gifdir + "slider-edges.gif"));
	sliderMiddleTrack = kit.createImage(Pool.class.getResource(gifdir + "slider-middle.gif"));
	sliderMiddleBottom = kit.createImage(Pool.class.getResource(gifdir + "slider-bottom-middle.gif"));
	sliTick = kit.createImage(Pool.class.getResource(gifdir + "slider-bottom-tick.gif"));
	sliThumb = kit.createImage(Pool.class.getResource(gifdir + "slider-thumb.gif"));

	tracker.addImage(sliderEdges, trackerID++);
	tracker.addImage(sliderMiddleTrack, trackerID++);
	tracker.addImage(sliderMiddleBottom, trackerID++);
	tracker.addImage(sliTick, trackerID++);
	tracker.addImage(sliThumb, trackerID++);
    }

    public WidgetDrag copy(int edition, Rectangle r) {
	String s = name + " copy" + WidgetDrag.editionString(edition);
	String v = var;
	if (edition > 1)
	    v += edition;
	return new SliderWidget(s, v, r, min, max, cur, showName, ++SliderWidget.idCount, parent, sl, getToolText());
    }
}
