/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import yoyo.YoYo;
import java.awt.*;
import java.awt.event.*;

public class SplashDialog extends Frame {

  static Image slime, about;
  Insets insets;

    public SplashDialog() {
	super("Loading StarLogo...");
	setResizable(false);
	loadImages();
	addNotify();
	insets = getInsets();
	setSize(about.getWidth(this)+ insets.left + insets.right, 
		about.getHeight(this) + insets.top + insets.bottom);
	
	addMouseListener(new MouseAdapter() {
		public void mouseReleased(MouseEvent e) {
			((SplashDialog)e.getComponent()).setVisible(false);
		}
	    });

	addKeyListener(new KeyAdapter() {
		public void keyPressed(KeyEvent e) {
		    e.consume();
		    ((SplashDialog)e.getComponent()).setVisible(false);
		}
	    });

	initLocation();
    }
    
    public void initLocation() {
	Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
	int screenwidth = screen.width;
	int screenheight = screen.height;
	Dimension dialog = getSize();
	int dialogwidth = dialog.width;
	int dialogheight = dialog.height;
	if (!YoYo.solarisp) setLocation((screenwidth - dialogwidth) / 2, 
					(screenheight - dialogheight) / 2);
    }
    
    public void loadImages() {
	String gifdir = "gifs/";
	try {
	    Toolkit kit = Toolkit.getDefaultToolkit();
	    MediaTracker tracker = new MediaTracker((Component)this);
	    about = kit.createImage(SplashDialog.class.getResource(gifdir + "about-box.gif"));
	    slime = kit.createImage(SplashDialog.class.getResource(gifdir + "slime.gif"));
	    tracker.addImage(slime, Widget.trackerID++);
	    tracker.addImage(about, Widget.trackerID++);
	    try {
		tracker.waitForAll();
	    } catch (InterruptedException ie) {}
	} catch (Exception e) {
	    System.err.println("Error loading images.");
	    e.printStackTrace(System.err);
	}
    }

    public void paint(Graphics g) {
	if (about != null)
	    g.drawImage(about, insets.left, insets.top, this);
	if (slime != null)
	    g.drawImage(slime, insets.left + 13, insets.top + 12, this);
    }
  
}
