/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import yoyo.Context;
import yoyo.LogoError;
import yoyo.YoYoCaller;
import yoyo.YoYoObject;
import yoyo.ThrowReturn;
import yoyo.Symbol;
import yoyo.CompoundSymbol;
import yoyo.YoYo;
import yoyo.PrimObject;
import java.util.Vector;

public class StackProcess implements YoYoCaller, DisplayMessage {

    Application app;
  int ip, sp, fp, isp, ifp, here, whichip, currentwhichip;
  int[] istack, iinst;
  Object[] ostack, oinst;
  long timer, waittimer;
  StackProcessParent vmspp;
  VM callerentity, originalentity;

  VM entity, nextentity;
  Turtle tur; Patch pat; Observer obs;

    StackProcess prevInList, nextInList;
    StackProcess prevInVM, nextInVM;

  static int stacksize = 500;
  static int ostacksize = 500;
  StackProcessParent spp;
  boolean runstate;
  boolean outputsp;
  boolean donep;
    boolean inwith;
    boolean deadp = false;

  static Pool pool;
  
  static {
   	try {
   	  pool = new Pool(StackProcess.class);
   	} catch (Exception e) { e.printStackTrace(); }
  }

  public StackProcess(Application app) {
      this.app = app;
    istack = new int[stacksize];
    ostack = new Object[stacksize];
  }

  public int compare(Object o1, Object o2) {
    int one = o1.hashCode();
    int two = o2.hashCode();
    if (one < two) return -1;
    if (one > two) return 1;
    return 0;
  }

    public static StackProcess create(Application app, int whichip, int ip, 
				      StackProcessParent spp, VM vm,
				      Turtle t, Patch p, Observer o,
				      boolean outputsp, Pool pool) {
    StackProcess stackp = pool.getStackProcess(app);
    stackp.iinst = spp.sl.iinstarray[whichip];
    stackp.oinst = spp.sl.oinstarray[whichip];
    stackp.whichip = stackp.currentwhichip = whichip;
    stackp.outputsp = outputsp;
    stackp.inwith = outputsp;
    stackp.sp = stackp.isp = 0;
    stackp.fp = stackp.ifp = 0;
    stackp.ip = ip;

    if ((stackp.istack.length == stacksize) && (t != null)) {
    	stackp.istack = new int[ostacksize];
    	stackp.ostack = new Object[ostacksize];
    }    
    if ((stackp.istack.length == ostacksize) && (t == null)) {
    	stackp.istack = new int[stacksize];
    	stackp.ostack = new Object[stacksize];
    }
    	
    //stackp.istack = new int[stacksize];
    //stackp.ostack = new Object[stacksize];
    
    stackp.originalentity = vm;
    stackp.entity = vm;
    stackp.nextentity = vm;
    stackp.tur = t;
    stackp.pat = p;
    stackp.obs = o;
    stackp.spp = spp;
    stackp.here = stackp.iinst[stackp.ip++];
    stackp.runstate = false;
    stackp.donep = false;
    return stackp;
  }

    public static StackProcess create(Application app, int whichip, int ip, StackProcessParent spp, Observer vm,
		      boolean outputsp) {
	return create(app, whichip, ip, spp, vm, null, null, vm, outputsp, pool);
  }


    public static StackProcess create(Application app, StackProcess process, int whichip, int ip,
		      StackProcessParent spp, Observer vm,
		      boolean outputsp) {
	StackProcess stackp = create(app, whichip, ip, spp, vm, null, null, vm, outputsp, pool);
    stackp.callerentity = process.entity;
    copyStack(process, stackp);
    return stackp;
  }

    public static StackProcess create(Application app, int whichip, int ip, StackProcessParent spp, Turtle vm, boolean outputsp) {
	return create(app, whichip, ip, spp, vm, vm, null, null, outputsp, pool);
  }

    public static StackProcess create(Application app, StackProcess process, int whichip, int ip, StackProcessParent spp,
		      Turtle vm, boolean outputsp) {
	StackProcess stackp = create(app, whichip, ip, spp, vm, vm, null, null, outputsp, pool);
    stackp.callerentity = process.entity;
    copyStack(process, stackp);
    return stackp;
  }

    public static StackProcess create(Application app, int whichip, int ip, StackProcessParent spp, Patch vm, boolean outputsp) {
	return create(app, whichip, ip, spp, vm, null, vm, null,  outputsp, pool);
  }

    public static StackProcess create(Application app, StackProcess process, int whichip, int ip,
		      StackProcessParent spp, Patch vm, boolean outputsp) {
	StackProcess stackp = create(app, whichip, ip, spp, vm, null, vm, null,  outputsp, pool);
    stackp.callerentity = process.entity;
    copyStack(process, stackp);
    return stackp;
  }

    public static StackProcess create(Application app, int whichip, int ip, StackProcessParent spp, Observer vm) {
	return create(app, whichip, ip, spp, vm, null, null, vm, false, pool);
  }

    public static StackProcess create(Application app, int whichip, StackProcessParent spp, Observer vm) {
	return create(app, whichip, 0, spp, vm, null, null, vm, false, pool);
  }

    public static StackProcess create(Application app, int whichip, StackProcessParent spp, Observer vm, int bytecode) {
	StackProcess stackp = create(app, whichip, 0, spp, vm, null, null, vm, false, pool);
    stackp.istack[stackp.isp++] = bytecode;
    return stackp;
  }

    public static StackProcess create(Application app, int whichip, int ip, StackProcessParent spp, Observer vm, int bytecode) {
	StackProcess stackp = create(app, whichip, ip, spp, vm, null, null, vm, false, pool);
    stackp.istack[stackp.isp++] = bytecode;
    return stackp;
  }

    public static StackProcess create(Application app, int whichip, int ip, StackProcessParent spp, Observer vm, int bytecode,
		      boolean outputsp) {
	StackProcess stackp = create(app, whichip, ip, spp, vm, null, null, vm, outputsp, pool);
    stackp.istack[stackp.isp++] = bytecode;
    return stackp;
  }

  public static StackProcess create(Application app, StackProcess process, int whichip, int ip,
		      StackProcessParent spp, Observer vm, int bytecode,
		      boolean outputsp) {
    StackProcess stackp = create(app, whichip, ip, spp, vm, null, null, vm, outputsp, pool);
    stackp.istack[stackp.isp++] = bytecode;
    stackp.callerentity = process.entity;
    copyStack(process, stackp);
    return stackp;
  }

  public static StackProcess create(Application app, int whichip, int ip, StackProcessParent spp, Turtle vm) {
    return create(app, whichip, ip, spp, vm, vm, null, null, false, pool);
  }

  public static StackProcess create(Application app, StackProcess process, int whichip, int ip,
		      StackProcessParent spp, Turtle vm) {
    StackProcess stackp = create(app, whichip, ip, spp, vm, vm, null, null, false, pool);
    stackp.callerentity = process.entity;
    copyStack(process, stackp);
    return stackp;
  }

  public static StackProcess create(Application app, int whichip, StackProcessParent spp, Turtle vm) {
    return create(app, whichip, 0, spp, vm, vm, null, null, false, pool);
  }

  public static StackProcess create(Application app, int whichip, StackProcessParent spp, Turtle vm, int bytecode) {
    StackProcess stackp = create(app, whichip, 0, spp, vm, vm, null, null, false, pool);
    stackp.istack[stackp.isp++] = bytecode;
    return stackp;
  }

  public static StackProcess create(Application app, int whichip, int ip, StackProcessParent spp, Turtle vm, int bytecode) {
    StackProcess stackp = create(app, whichip, ip, spp, vm, vm, null, null, false, pool);
    stackp.istack[stackp.isp++] = bytecode;
    return stackp;
  }

  public static StackProcess create(Application app, StackProcess process, int whichip, int ip,
		      StackProcessParent spp, Turtle vm, int bytecode) {
    StackProcess stackp = create(app, whichip, ip, spp, vm, vm, null, null, false, pool);
    //System.out.println("start process in turtle");
    stackp.istack[stackp.isp++] = bytecode;
    stackp.callerentity = process.entity;
    copyStack(process, stackp);
    return stackp;

  }

  public static StackProcess create(Application app, int whichip, int ip, StackProcessParent spp, Turtle vm, int bytecode,
		      boolean outputsp) {
    StackProcess stackp = create(app, whichip, ip, spp, vm, vm, null, null, outputsp, pool);
    stackp.istack[stackp.isp++] = bytecode;
    return stackp;
  }

  public static StackProcess create(Application app, StackProcess process, int whichip, int ip,
		      StackProcessParent spp, Turtle vm, int bytecode,
		      boolean outputsp) {
    StackProcess stackp = create(app, whichip, ip, spp, vm, vm, null, null, outputsp, pool);
    stackp.istack[stackp.isp++] = bytecode;
    stackp.callerentity = process.entity;
    copyStack(process, stackp);
    return stackp;
  }

  public static StackProcess create(Application app, StackProcess process, int whichip, int ip, StackProcessParent spp, Patch vm) {
    StackProcess stackp = create(app, whichip, ip, spp, vm, null, vm, null, false, pool);
    stackp.callerentity = process.entity;
    copyStack(process, stackp);
    return stackp;
  }

  public static StackProcess create(Application app, int whichip, int ip, StackProcessParent spp, Patch vm) {
    return create(app, whichip, ip, spp, vm, null, vm, null, false, pool);
  }

  public static StackProcess create(Application app, int whichip, StackProcessParent spp, Patch vm) {
    return create(app, whichip, 0, spp, vm, null, vm, null, false, pool);
  }

  public static StackProcess create(Application app, int whichip, StackProcessParent spp, Patch vm, int bytecode) {
    StackProcess stackp = create(app, whichip, 0, spp, vm, null, vm, null, false, pool);
    stackp.istack[stackp.isp++] = bytecode;
    return stackp;
  }

  public static StackProcess create(Application app, int whichip, int ip, StackProcessParent spp, Patch vm, int bytecode) {
    StackProcess stackp = create(app, whichip, ip, spp, vm, null, vm, null, false, pool);
    stackp.istack[stackp.isp++] = bytecode;
    return stackp;
  }

  public static StackProcess create(Application app, StackProcess process, int whichip, int ip, StackProcessParent spp,
		      Patch vm, int bytecode) {
    StackProcess stackp = create(app, whichip, ip, spp, vm, null, vm, null, false, pool);
    stackp.istack[stackp.isp++] = bytecode;
    stackp.callerentity = process.entity;
    copyStack(process, stackp);
    return stackp;
  }

  public static StackProcess create(Application app, int whichip, int ip, StackProcessParent spp, Patch vm, int bytecode,
		      boolean outputsp) {
    StackProcess stackp = create(app, whichip, ip, spp, vm, null, vm, null, outputsp, pool);
    stackp.istack[stackp.isp++] = bytecode;
    return stackp;
  }

  public static StackProcess create(Application app, StackProcess process, int whichip, int ip,
		      StackProcessParent spp, Patch vm, int bytecode,
		      boolean outputsp) {
    StackProcess stackp = create(app, whichip, ip, spp, vm, null, vm, null, outputsp, pool);
    stackp.istack[stackp.isp++] = bytecode;
    stackp.callerentity = process.entity;
    copyStack(process, stackp);
    return stackp;
  }

    public static void copyStack(StackProcess process, StackProcess stackp) {
	if (process.ifp > 0) {
	    int i = process.fp;
	    int nargs = process.istack[process.ifp - 5]; // nargs
	    i -= nargs;
	    while(i < process.sp) {
		stackp.ostack[stackp.sp++] = process.ostack[i++];
	    }
	    stackp.fp = nargs;
	}
    }

    public void unwindStack() {
	int tempsp = sp;
	while (tempsp > 0) {
	    Object val = ostack[--tempsp];
	    if (val == VM.grabbedMarker) {
		Turtle st = ((Turtle)ostack[--tempsp]);
		st.grabber = null;
		if (!st.alive) spp.sl.recycleDeadWho(st);
	    } else if (val instanceof StackProcessParent) {
		StackProcessParent spp = (StackProcessParent)val;
		assert spp.donep();
	    } else if (val == VM.grabbedPatchMarker) {
		((Patch)ostack[--tempsp]).grabbedp = false;
	    }
	}
    }      


    // only called by a dying turtle
    public void unwindStack(VM entity) {
	int tempsp = entity.sp;
	while (tempsp > 0) {
	    Object val = entity.ostack[--tempsp];
	    if (val == VM.grabbedMarker) {
		Turtle st = ((Turtle)entity.ostack[--tempsp]);
		st.grabber = null;
		if (!st.alive) spp.sl.recycleDeadWho(st);
	    } else if (val instanceof StackProcessParent) {
		StackProcessParent spp = (StackProcessParent)val;
		assert spp.donep();
		// Don't do the following. it's reentrant on spenumerator.
		//((StackProcessParent)val).stop(); // stop dependent processes
	    } else if (val == VM.grabbedPatchMarker) {
		((Patch)entity.ostack[--tempsp]).grabbedp = false;
	    }
	}
    }      


  public void switchme(Context c) {
    //System.out.println("switchme: sp = " + this + ", entity = " + entity + ", ip = " + ip + ", here = " + here + " isp = " +isp + ".");
      if (deadp) c.log.println("switchme on dead process! sp = " + this + ", entity = " + entity + ", ip = " + ip + ", here = " + here + " isp = " +isp + ".");
    entity.here = here;
    entity.ip = ip;
    entity.sp = sp;
    entity.isp = isp;
    entity.istack = istack;
    entity.ostack = ostack;
    entity.iinst = iinst;
    entity.process = this;
   try {
      entity.dispatch(c, obs, tur, pat);
      //      System.out.println("exit dispatch: " + this.toString());
    }
    catch (ThrowReturn tr) {
      if (StarLogo.DEBUG) {
	  c.log.println("Throw Return Error in dispatch on entity: " + entity);
	  c.log.println("Error is " + tr.errortype + " and has value " + YoYo.printToString(PrimObject.prim_printobject((YoYoObject)tr.value, null)));
	  tr.printStackTrace(c.log);
      }
      StarLogo sl = entity.sl;
      Application app = sl.app;
      Object[] ilist = { cserror, tr.errortype, tr.value, this };
      app.cc.enqueue(ilist, app.message, app.err, this);
      sl.stopAllStarLogo();
    }
    catch (LogoError e) {
      StarLogo sl = entity.sl;
      Application app = sl.app;
      saveState();
      YoYoObject yob = new YoYoObject();
      yob.defineValue(Symbol.lookup("instruction"), new Integer(here));
      yob.defineValue(Symbol.lookup("message"), e.getMessage());
      entity.setTPO(yob);
      Object[] crawl = stackCrawl();
      yob.defineValue(Symbol.lookup("stack-crawl"), crawl);
      if (StarLogo.DEBUG) {
	c.log.println("Logo Error in dispatch on entity: " + entity);
	c.log.println("Error has value " + YoYo.printToString(PrimObject.prim_printobject(yob, null)));
	c.log.println(stackCrawlToString(crawl));
	c.log.println("Entity with error: " + entity);
	c.log.println("IP: " + ip);
	c.log.println("WhichIP: " + whichip);
	c.log.println("Category: " + ((here & 0xFF00) >> 8));
	c.log.println("Bytecode: " + (here & 0xFF));
	c.log.println("ISP: " + isp);
	c.log.println("IStack: " + YoYo.printToString(istack));
	c.log.println("OSP: " + sp);
	c.log.println("OStack: " + YoYo.printToString(ostack));
	e.printStackTrace(c.log);
      }
      Object[] ilist = { cserror, "vm-logo-error", yob, this };
      app.cc.enqueue(ilist, app.message, app.err, this);
      sl.stopAllStarLogo();
    }
    catch (ArrayIndexOutOfBoundsException e) {
	StarLogo sl = entity.sl;
	Application app = sl.app;
	saveState();
	YoYoObject yob = new YoYoObject();
	yob.defineValue(Symbol.lookup("instruction"), new Integer(here));
	if (sp >= ostack.length ||
	    isp >= istack.length) {
	    yob.defineValue(Symbol.lookup("message"), app.textBundle.getString("stack_overflow"));
	} else {
	    yob.defineValue(Symbol.lookup("message"), app.textBundle.getString("internal_error") +
			    ": " + app.textBundle.getString("array_out_of_bounds"));
	}
	entity.setTPO(yob);
	Object[] crawl = stackCrawl();
	yob.defineValue(Symbol.lookup("stack-crawl"), crawl);
	if (StarLogo.DEBUG) {
	    c.log.println("Array index out of bounds Exception in dispatch on entity: " + entity);
	    c.log.println("Error has value " + YoYo.printToString(PrimObject.prim_printobject(yob, null)));
	    c.log.println(stackCrawlToString(crawl));
	    c.log.println("Entity with error: " + entity);
	    c.log.println("IP: " + ip);
	    c.log.println("WhichIP: " + whichip);
	    c.log.println("Category: " + ((here & 0xFF00) >> 8));
	    c.log.println("Bytecode: " + (here & 0xFF));
	    c.log.println("ISP: " + isp);
	    c.log.println("IStack: " + YoYo.printToString(istack));
	    c.log.println("OSP: " + sp);
	    c.log.println("OStack: " + YoYo.printToString(ostack));
	    e.printStackTrace(c.log);
	}
	Object[] ilist = { cserror, "vm-stack-overflow", yob, this };
	app.cc.enqueue(ilist, app.message, app.err, this);
	sl.stopAllStarLogo();
    }
    catch (RuntimeException e) {
	if (StarLogo.DEBUG) {
	    c.log.println("Caught unhandled error in dispatch: " + e.toString());
	    c.log.println("StackProcess: " + this);
	    if (entity != null) {
	      saveState();
	      c.log.println(stackCrawlToString(stackCrawl()));
	      c.log.println("Entity with error: " + entity);
	      c.log.println("IP: " + ip);
	      c.log.println("WhichIP: " + whichip);
	      c.log.println("Category: " + ((here & 0xFF00) >> 8));
	      c.log.println("Bytecode: " + (here & 0xFF));
	      c.log.println("ISP: " + isp);
	      c.log.println("IStack: " + YoYo.printToString(istack));
	      c.log.println("OSP: " + sp);
	      c.log.println("OStack: " + YoYo.printToString(ostack));
	    }
	    e.printStackTrace(c.log);
	}
	StarLogo sl = entity.sl;
	sl.stopAllStarLogo();
	throw e;
    }
    finally {
      //      if (!donep) {
      if (entity !=  null) {
	here = entity.here;
	ip = entity.ip;
	sp = entity.sp;
	isp = entity.isp;
	istack = entity.istack;
	ostack = entity.ostack;
	iinst = entity.iinst;
	entity = nextentity;
      }
    //System.out.println("stackprocess done");
    }
  }

  public void saveState() {
    if (entity !=  null) {
      here = entity.here;
      ip = entity.ip;
      sp = entity.sp;
      isp = entity.isp;
      istack = entity.istack;
      ostack = entity.ostack;
      iinst = entity.iinst;
      entity = nextentity;
    }
  }

  public String stackCrawlToString(Object[] stackCrawl) {
    StringBuffer out = new StringBuffer();
    for(int i = 0; i < stackCrawl.length; i++) {
      Object[] call = (Object[])stackCrawl[i];
      out.append("  called by ");
      out.append(YoYo.printToString(call[0]));
      out.append(' ');
      Object[] args = (Object[])call[1];
      if (args.length > 0) {
	for(int j = 0; j < args.length-1; j++) {
	  out.append(YoYo.printToString(args[j]));
	  out.append(' ');
	}
	out.append(YoYo.printToString(args[args.length-1]));
      }
      out.append("\n\r");
    }
    return out.toString();
  }

  public Object[] stackCrawl() {
    Vector callStack = new Vector();
    int myfp = fp, myifp = ifp, mywhichip = whichip;
    Object[] first = new Object[2];
    first[0] = new Integer(ip-1);
    Object[] rest = new Object[7];
    first[1] = rest;
    rest[0] = new Integer(((here & 0xFF00) >> 8));
    rest[1] = new Integer(here & 0xFF);
    rest[2] = new Integer(isp);
    rest[3] = intArrayToIntegerArray(istack);
    rest[4] = new Integer(sp);
    rest[5] = ostack;
    callStack.addElement(first);
    while(myifp > 0) {
      String funcname = entity.sl.internalGetName(mywhichip);
      Object[] args = new Object[istack[myifp-5]];
      for(int i = args.length-1; i >= 0; i--) {
	args[i] = ostack[myfp - (args.length - i)];
      }
      Object[] funcall = new Object[2];
      funcall[0] = funcname;
      String[] arglist = new String[args.length];
      for(int i = 0; i < args.length; i++) {
	arglist[i] = YoYo.printToString(args[i]);
      }
      funcall[1] = arglist;
      callStack.addElement(funcall);

      mywhichip = istack[myifp - 4];
      myfp = istack[myifp-2];
      myifp = istack[myifp-1];
    }
    Object[] output = new Object[callStack.size()];
    callStack.copyInto(output);
    return output;
  }

  static Integer[] intArrayToIntegerArray(int[] inst) {
    Integer[] ilist = new Integer[inst.length];
    for (int i = 0; i<inst.length; i++) {
      ilist[i] = new Integer(inst[i]);
    }
    return ilist;
  }


  static Object[] theerror = { Symbol.lookup("compiler"), Symbol.lookup("show-vm-error") };
  static CompoundSymbol cserror = new CompoundSymbol(theerror);

  public void done(Context c) {};

  public void done() {
      assert !donep : "stackprocess " + this + " already done";
      //if (!outputsp) unwindStack();
      //System.out.println("StackProcess: done(): " + this);
    donep = true;
    spp.done(this);
    //System.out.println("StackProcess: " + this + " done() method finished.");
    if (!outputsp) {
	//System.out.println("drop stack process: " + this.toString());
      pool.dropStackProcess(this);
    }

  }

  public Object output() {
    if (outputsp && donep) {
      //System.out.println("output sp = " + (sp-1));
      //System.out.println("value = " + ostack[sp-1]);
      return ostack[sp-1];
    } else return null;

  }

  public void displayOneString(String s) {
      Application.createMessageDialog(entity.sl.app, app.activeFrame, app.textBundle.getString("error_label"), 
				      s, true);
  }

  public void displayAString(String s) {
      Application.createMessageDialog(entity.sl.app, app.activeFrame, app.textBundle.getString("error_label"), 
				      s, true);
  }

}




