/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.util.Hashtable;
import java.util.Enumeration;
import yoyo.*;
import java.util.Vector;
import java.util.Random;

public class StackProcessList {

    StackProcess firstProcess;
    StackProcess lastProcess;

  public boolean randomschedp = false;
  public boolean changeschedp = false;

  int count = 0;
  Application app;
  SPEnumerator spe, spe2;

  public StackProcessList(Application app) {
    this.app = app;
    spe2 = (randomschedp) ? new RandomSPEnumerator(this) :
      new SPEnumerator(this);
  }

  public String getScheduler() {
      return (randomschedp) ? app.textBundle.getString("random_scheduler") : 
	  app.textBundle.getString("fixed_scheduler");
  }

  public void create(VM entity) {
    //System.out.println("SPL create entity: " + entity);
    entity.firstProcess = null;
    entity.lastProcess = null;
    entity.count = 0;
  }

  // in case we ever mutate something in here when adding entities.
  // only run these when the list is empty.
  public void deletePatches() {};
  public void deleteTurtles() {};
  public void deleteObserver() {};


  int size() {
    return count;
  }

  public void add(StackProcess sp) {
    //System.out.println("SPL add process " + sp + " to entity: " + sp.entity);
    addProcess(sp);
    if (count == 1) app.runMonitors();
  }


  public void addMonitor(StackProcess sp) {
    if (StarLogo.DEBUG) { app.log.println("SPL add monitor " + sp + " to entity: " + sp.entity);}
    addProcess(sp);
  }

  public void addLineWidget(StackProcess sp) {
      if (StarLogo.DEBUG){ app.log.println("SPL add line widget " + sp + " to entity: " + sp.entity);}
      addProcess(sp);
  }

  public void addProcess(StackProcess sp) {
      //  System.out.println("Adding process " + sp);
      //System.out.println("There are " + (count+1) + " processes running.");
      //printList();
    boolean emptyp = count == 0;
    if (firstProcess == null) {
	firstProcess = sp;
	lastProcess = sp;
	sp.prevInList = null;
	sp.nextInList = null;
    } else {
	/* Add it to the beginning of the list */
	sp.prevInList = null;
	sp.nextInList = firstProcess;
	firstProcess.prevInList = sp;
	firstProcess = sp;
    }
    count++;
    //System.out.println("sp.originalentity = " + sp.originalentity);
    sp.originalentity.addSP(sp);
    //printList();
    if (emptyp && !app.sl.locked) {
      synchronized (app.sl.runningp) {
	app.sl.runningp.notify();
      }
    }
  }

    public void printList() {
	System.out.println("printing stackprocess list forwards");
	for(StackProcess current = firstProcess; current != null; current = current.nextInList) {
	    System.out.print(current + ", ");
	}
	System.out.println();
	System.out.println("printing stackprocess list backwards");
	for(StackProcess current = lastProcess; current != null; current = current.prevInList) {
	    System.out.print(current + ", ");
	}
	System.out.println();
    }

   boolean SPInList(StackProcess sp) {
	for(StackProcess current = lastProcess; current != null; current = current.prevInList) {
	    if (current == sp) return true;
	}	
	for(StackProcess current = firstProcess; current != null; current = current.nextInList) {
	    if (current == sp) return true;
	}
	return false;
    }


  public void remove(StackProcess sp) {
    /*  if (sp.originalentity instanceof Turtle) {
      	System.out.println("SPL remove process " + sp + " from entity: "
      						+ sp.entity);
      } */
    removeVMProcess(sp); 
    if (count == app.numMonitors()) app.stopMonitors();
  }

  public void removeMonitor(StackProcess sp) {
      if (StarLogo.DEBUG) { app.log.println("SPL remove monitor " + sp + " from entity: "
		       + sp.entity);}
    removeVMProcess(sp);
  }
    public void removeLineWidget(StackProcess sp) {
     if (StarLogo.DEBUG) { app.log.println("SPL remove line widget " + sp + " from entity: "
			 + sp.entity);}
      removeVMProcess(sp);
  }

    public void removeVMProcess(StackProcess sp) {
	//System.out.print("removing sp from original entity: "  + sp.originalentity +
	//	       " entity = " + sp.entity);
      if (!sp.donep) {
        sp.originalentity.removeSP(sp);
        removeProcess(sp);
      }
    }

  public void removeProcess(StackProcess sp) {
      /* If there's someone before me... */
      if (sp.prevInList != null) {
	  /* Make his nextptr equal to my next pointer */
	  sp.prevInList.nextInList = sp.nextInList;
      } else {
	  /* If not, check if there is someone after me... */
	  if (sp.nextInList == null) {
	      /* No, so I'm the last process. Since we remove me ... */
	      firstProcess = null;
	      lastProcess = null;
	  } else {
	      /* Yes, since I was the firstProcess, make it now the next guy */
	      firstProcess = sp.nextInList;
	  }
      }
      /* If there's someone after me ... */
      if (sp.nextInList != null) {
	  /* Make his prevptr equal to my prev ptr */
	  sp.nextInList.prevInList = sp.prevInList;
      } else {
	  /* If not, check if there's someone before me... */
	  if (sp.prevInList == null) {
	      /* No, so i'm the last process. Since we remove me ... */
	      firstProcess = null;
	      lastProcess = null;
	  } else {
	      /* Yes, since I was the lastProcess, make it now the previous guy */
	      lastProcess = sp.prevInList;
	  }
      }

      /* Mark this Stack Process as finished */
      count--;
      //assert !SPInList(sp);
      sp.done();
  }


  public void remove(VM vm) {
      /*
	if (vm instanceof Turtle) {
	System.out.println("turtle " + vm + " has died.");
	printList();
	vm.printList();
	System.out.println("Removing his stackProcesses: ");
	}
      */
      
      // if he's asking anyone else to run something, stop it
      if (vm.vmspp != null) stop(vm.vmspp);

    for(SPEnumeration e = vm.spList(); e.hasMoreElements(); ) {
	StackProcess sp = e.nextElement();
	//if (vm instanceof Turtle) System.out.print(sp + ", ");
	removeProcess(sp);
	try {
	    sp.unwindStack(vm); // let go of grabbed turtles
	} catch (RuntimeException re) {
	    re.printStackTrace();
	} catch (Exception exp) {
	    exp.printStackTrace();
	}
    }

    vm.removeAllSPs();

    /*    
      if (vm instanceof Turtle) {
	System.out.println("\ndone removing, verifying empty list");
	vm.printList();
	printList();
      }
    */
    
    if (count == app.numMonitors()) {
	//System.out.println("SPE remove entity: time to stop monitors");
      app.stopMonitors();
      // remove monitors
    }
  }

    public void removeAllSPs() {
	removeTurtleSPs();
	removePatchSPs();
	removeObserverSPs();
	app.stopMonitors();
	firstProcess = lastProcess = null;
	count = 0;
    }
    public void removeTurtleSPs() {
	for(Turtle t = app.sl.firstturtle; t != null; t = t.next) {
	    remove(t);
	}
    }
    public void removePatchSPs() {
	for(int y = 1; y<app.sl.height+1; y++) {
	    int col = y*app.sl.widthp2;
	    for(int x = 1; x<app.sl.width+1; x++) {
		Patch p = app.sl.patches[col + x];
		remove(p);
	    }
	}
    }
    public void removeObserverSPs() {
	remove(app.sl.observer);
    }

    void stop(StackProcessParent spp) {
	//System.out.println("SPL: stopping SPP: " + spp);
	int num = 0;
	int allnum = 0;
	int nonnum = 0;
	try {
	    for(SPEnumeration e = elements(); e.hasMoreElements(); ) {
		StackProcess sp = e.nextElement();
		if (spp.contains(sp)) {
		    assert sp.spp == spp;
		    num++;
		    //System.out.println("SPL: Stopping stack process " + num + "/" + spp.count + ": " + sp + " from SPP: " + spp);
		    remove(sp);
		    //System.out.println("SPL: unwind stack process " + num + "/" + spp.count + ": " + sp + " from SPP: " + spp);
		    sp.unwindStack();
		    //System.out.println("SPL: Finished stopping stack process " + num + "/" + spp.count + ": " + sp + " from SPP: " + spp);
		} else {
		    assert sp.spp != spp;
		    nonnum++;
		}
		allnum++;
	    }
	} catch (RuntimeException re) {
	    re.printStackTrace();
	} catch (Exception e) {
	    e.printStackTrace(); 
	}
	//System.out.println("SPL: stopped " + num + " stack processes out of " + allnum + " from spp " + spp);
	//System.out.println("SPL: didn't stop " + nonnum + " stack processes out of " + allnum + " from spp " + spp);
    }



  public static int seed = (int)System.currentTimeMillis();
  public static Random rand = new Random(seed);

  public void permute() {

    StackProcess[] array = new StackProcess[count];
	int i = 0;

    for(StackProcess sp = firstProcess; sp != null; sp = sp.nextInList, i++) {
       array[i] = sp;
    }

    int[] permute = new int[count];
    for(i = 0; i < count; i++) {
      permute[i] = i;
    }

    for(i = 0; i < count; i++) {
      int j = (int)Math.floor(rand.nextDouble()*count);
      int temp = permute[i];
      permute[i] = permute[j];
      permute[j] = temp;
    }

    firstProcess = array[permute[0]];
    firstProcess.prevInList = null;
    StackProcess sp = firstProcess;

    for(i = 1; i < count; i++) {
       StackProcess next = array[permute[i]];
       sp.nextInList = next;
       next.prevInList = sp;
       sp = next;
    }
    lastProcess = array[permute[count-1]];
    lastProcess.nextInList = null;
  }


  SPEnumeration elements() {
    if (changeschedp) {
      spe2 = (randomschedp) ? new RandomSPEnumerator(this) : new SPEnumerator(this);
      changeschedp = false;
    } else spe2.reset();
    spe = spe2;
    return spe;
  }


}


