/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import yoyo.YoYoCaller;
import yoyo.Context;
import java.util.Vector;
import java.util.Enumeration;
import yoyo.YoYo;

public class StackProcessParent implements StarLogoCaller {


    static int ids = 0;
    int id = 0;
  boolean dead = false;
  int numdone = 0;
  int count;
  int whichip;
  int startip;
  StarLogo sl;
  StarLogoCaller caller;

  boolean foreverp = false;
  boolean turtlep = false;

  Vector spps = new Vector();
  int sppcount = 0;

  public StackProcessParent(StarLogo sl, int number, int whichip, int ip,
			    boolean foreverp, boolean turtlep, StarLogoCaller caller) {
    count = number;
    this.whichip = whichip;
    this.startip = ip;
    this.sl = sl;
    this.caller = caller;
    this.foreverp = foreverp;
    this.turtlep = turtlep;
    id = ids++;
    if (caller != null) caller.addStackProcessParent(this);
  }

  public StackProcessParent(StarLogo sl, int whichip, int ip,
			    boolean foreverp, boolean turtlep, StarLogoCaller caller) {
    this(sl, 1, whichip, ip, foreverp, turtlep, caller);
  }

  public void setCount(int count) {
    //System.out.println("spp: set count to " + count);
    this.count = count;
  }

  public void incCount() {
    this.count++;
  }

  public StarLogoCaller caller() {
    return caller;
  }

  public void callerPrintln(String s) {
    caller.callerPrintln(s);
  }

  public void callerClearText() {
    caller.callerClearText();
  }

  public void addStackProcessParent(StackProcessParent child) {
    spps.addElement(child);
    sppcount++;
  }

  public boolean donep() {
    return (numdone >= count && sppcount == 0);
  }

  public void done(Context c) {}

  public void done() {
      //System.out.println("spp.done()");
      assert !dead;
      numdone++;
      //System.out.println("SPP: " + this + " numdone " + numdone + " / " + count);
      checkdone();
      //System.out.println("SPP: " + this + " done() method finished.");
  }

  public void done(StackProcess sp) {
    done();
    //System.out.println("stack process sp = " + sp.sp);
  }

  // a child spp telling its parent that he's done.
  public void done(StackProcessParent spp) {
      //System.out.println("Child SPP done: " + spp + " parent is " + this);
      assert spps.contains(spp) && !dead;
      spps.removeElement(spp);
      sppcount--;
      checkdone();
  }

  public void checkdone() {
      //System.out.println("SPP: " + this + " done? " + donep());
      if (donep() && !dead) {
	  dead = true;
	  //System.out.println("Dead spp: "+ this);
	  if (caller != null) {
	      //System.out.println("calling done on " + caller);
	      caller.done(this);
	      return;
	  }
      } else {
	  //System.out.println("SPP: " + this + " not done.");
      }
  }


  public boolean contains(StackProcess sp) {
    if (sp.spp == this) return true;
    if (sppcount == 0) return false;
    for(Enumeration e = spps.elements(); e.hasMoreElements();) {
      StackProcessParent spp = (StackProcessParent)e.nextElement();
      if (spp.contains(sp)) return true;
    }
    return false;
  }

  public void stop() {
    sl.stopProcess(this);
  }

  public void stopAll() {
      //System.out.println("SPP: Stop all " + sppcount + " child SPPs in " + this);
      int total = sppcount;
      int num = 0;
    for(Enumeration e = ((Vector)spps.clone()).elements(); e.hasMoreElements();) {
      StackProcessParent spp = (StackProcessParent)e.nextElement();
      //System.out.println("SPP: Stop child SPP " + (++num) + "/" + total + ": " + spp);
      sl.stopProcess(spp);
    }
    //System.out.println("SPP: Finished stopping " + total + " child SPPs in " + this);
    //spps.removeAllElements();
    //System.out.println("SPP: Stop all my stack processes in " + this);
    sl.stopProcess(this);
  }

  public String toString() {
    return "<StackProcessParent (" + id + "): IP = " + whichip + ", SPP-children = "  + sppcount + " count = " + count +
      ", numdone = " + numdone + ">";
  }

  public int hashCode() {
    return whichip; // << 16 + startip;
  }

  public static int hashCode(int whichip, int ip) {
    return whichip; // << 16 + ip;
  }

  /* Monitors */

  static StackProcess runMonitor(StarLogo sl, int whichip, boolean foreverp,
				 StarLogoCaller caller) {
	//System.out.println("run monitor whichip = " + whichip);
	StackProcessParent spp = new StackProcessParent(sl, 1, whichip,
							foreverp, false, caller);
	StackProcess sp = StackProcess.create(sl.app, whichip, spp, sl.observer);
	sl.list.addMonitor(sp);
	return sp;
    }

      static StackProcess runLineWidget(StarLogo sl, int whichip, boolean foreverp,
					 StarLogoCaller caller) {
		//System.out.println("run monitor whichip = " + whichip);
		StackProcessParent spp = new StackProcessParent(sl, 1, whichip,
								foreverp, false, caller);
		StackProcess sp = StackProcess.create(sl.app, whichip, spp, sl.observer);
		sl.list.addLineWidget(sp);
		return sp;
    }

  /* Observer */

  static StackProcessParent runObserver(StarLogo sl, int whichip, boolean foreverp,
					StarLogoCaller caller) {
	//System.out.println("run observer whichip = " + whichip);
	StackProcessParent spp = new StackProcessParent(sl, 1, whichip, foreverp,
							false, caller);
	sl.list.add(StackProcess.create(sl.app, whichip, spp, sl.observer));
	return spp;
    }


    // Yes
  static StackProcessParent runObserver(StarLogo sl, int whichip, int ip, int bytecode,
					boolean foreverp, StarLogoCaller caller) {
	StackProcessParent spp = new StackProcessParent(sl, 1, whichip, ip, foreverp, false,
							caller);
	if (bytecode >= 0) {
	    sl.list.add(StackProcess.create(sl.app, whichip, ip, spp, sl.observer, bytecode));
	} else {
	    sl.list.add(StackProcess.create(sl.app, whichip, ip, spp, sl.observer));
	}
	return spp;
    }

  static StackProcessParent runObserver(StarLogo sl, StackProcess process, int whichip, int ip, int bytecode,
					boolean foreverp, StarLogoCaller caller) {
	StackProcessParent spp = new StackProcessParent(sl, 1, whichip, ip, foreverp, false,
							caller);
	if (bytecode >= 0) {
	    sl.list.add(StackProcess.create(sl.app, process, whichip, ip, spp, sl.observer, bytecode, false));
	} else {
	    sl.list.add(StackProcess.create(sl.app, process, whichip, ip, spp, sl.observer, false));
	}
	return spp;
    }

  /* Turtle */

  static StackProcessParent runTurtle(StarLogo sl, int whichip, int ip, int bytecode,
				      Turtle t, boolean foreverp, StarLogoCaller caller) {
      if (t == null || !t.alive) return null;
	StackProcessParent spp = new StackProcessParent(sl, 1, whichip, ip, foreverp, true,
							caller);
	sl.list.add(StackProcess.create(sl.app, whichip, ip, spp, t, bytecode));
	return spp;
    }


  static StackProcessParent runTurtle(StarLogo sl, StackProcess process,
				       int whichip, int ip, int bytecode,
				       Turtle t, boolean foreverp, 
				      StarLogoCaller caller) {
      if (t == null || !t.alive) return null;
      //System.out.println("running turtles:  " + YoYo.printToString(ts));
      StackProcessParent spp = new StackProcessParent(sl, 1, whichip, ip,
						      foreverp, true, caller);
      sl.list.add(StackProcess.create(sl.app, process, whichip, ip, spp, t, bytecode));
      return spp;
  }



  static StackProcessParent runAsyncTurtle(StarLogo sl, StackProcess sp, int whichip,
					   int ip, int bytecode, Turtle t,
					   boolean foreverp, StarLogoCaller caller) {
      if (t == null || !t.alive) return null;
    StackProcessParent spp = new StackProcessParent(sl, 1, whichip, ip, foreverp,
						    true, caller.caller());
    sl.list.add(StackProcess.create(sl.app, sp, whichip, ip, spp, t, bytecode));
    return spp;
  }

  static void askTurtleRunForeverButtons(StarLogo sl, Turtle t) {
	ButtonWidget[] bws = sl.app.iface.getButtons();
	for(int i = 0; i<bws.length; i++) {
	    if (bws[i].spp != null) {
		StackProcessParent spp = bws[i].spp;
		if (spp.foreverp && spp.turtlep && !spp.dead) {
		    spp.incCount();
		    sl.list.add(StackProcess.create(sl.app, spp.whichip, 0, spp, t));
		}
	    }
	}
    }

  static void askTurtleRunForeverButtons(StarLogo sl, Turtle[] sts) {
	ButtonWidget[] bws = sl.app.iface.getButtons();
	for(int i = 0; i<bws.length; i++) {
	  if (bws[i].spp != null) {
	    StackProcessParent spp = bws[i].spp;
	    if (spp.foreverp && spp.turtlep && !spp.dead) {
	      for(int j = 0; j < sts.length; j++) {
		spp.incCount();
		sl.list.add(StackProcess.create(sl.app, spp.whichip, 0, spp, sts[j]));
	      }
	    }
	  }
	}
  }


    // Yes
  static StackProcessParent runTurtles(StarLogo sl, StackProcess process,
				       int whichip, int ip, int bytecode,
				       Turtle[] ts, boolean foreverp, StarLogoCaller caller) {
      if (ts.length == 0) return null;
      //System.out.println("running turtles:  " + YoYo.printToString(ts));
      int count = 0;
      StackProcessParent spp = new StackProcessParent(sl, 0, whichip, ip,
						      foreverp, true, caller);
      for (int i = 0; i<ts.length; i++) {
	  //System.out.println("adding stack process for turtle #" + ts[i].who);
	  if (ts[i] == null || !ts[i].alive) continue;
	  sl.list.add(StackProcess.create(sl.app, process, whichip, ip, spp, ts[i], bytecode));
	  count++;
      }
      if (count > 0) {
	  spp.setCount(count);
	  return spp;
      } else spp.done();
      return null;
      
  }

  // Yes
  static StackProcessParent runTurtles(StarLogo sl, int whichip, int ip,
				       int bytecode, boolean foreverp,
				       StarLogoCaller caller) {
    StackProcessParent spp = new StackProcessParent(sl, 0, whichip, ip,
						    foreverp, true, caller);
	int count = 0;
	  Turtle current = sl.firstturtle;
	if (bytecode >= 0) {
	  while(current!=null) {
	      if (!current.alive) continue;
	    count ++;
	    sl.list.add(StackProcess.create(sl.app, whichip, ip, spp, current, bytecode));
	    current = current.next;
	  }
	} else {
	  while(current!=null) {
	      if (!current.alive) continue;
	      count ++;
	      sl.list.add(StackProcess.create(sl.app, whichip, ip, spp, current));
	      current = current.next;
	  }
	}
	
	//System.out.println("count is " + count);
	if (count > 0) {
	    spp.setCount(count);
	    return spp;
	} else spp.done();
	return null;
    }

    // Yes
      static StackProcessParent runTurtles(StarLogo sl, StackProcess process, int whichip,
					   int ip, int bytecode,
					   boolean foreverp, StarLogoCaller caller) {
	StackProcessParent spp = new StackProcessParent(sl, 0, whichip, ip,
							foreverp, true, caller);
	int count = 0;
	Turtle current = sl.firstturtle;
	if (bytecode >= 0) {
	  while(current!=null) {
	      if (!current.alive) continue;
	    count ++;
	    sl.list.add(StackProcess.create(sl.app, process, whichip, ip, spp, current, bytecode));
	    current = current.next;
	  }
	} else {
	  while(current!=null) {
	      if (!current.alive) continue;
	    count ++;
	    sl.list.add(StackProcess.create(sl.app, process, whichip, ip, spp, current));
	    current = current.next;
	  }
	}

	//System.out.println("count is " + count);
	if (count > 0) {
	    spp.setCount(count);
	    return spp;
	} else spp.done();
	return null;
    }

  /*

  static StackProcessParent runTurtlesOutput(int whichip, int ip, int bytecode,
					boolean foreverp, StarLogoCaller caller) {
	StackProcessParentOutput spp =
	    new StackProcessParentOutput(this, 0, whichip,
					 ip, foreverp, true, caller, null);
	int count = 0;
	Turtle current = sl.firstturtle;
	StackProcess sp;
	Vector sps = new Vector();
	if (bytecode >= 0) {
	    while(current != null) {
		count++;
		sp = StackProcess.create(sl.app, whichip, ip, spp, current, bytecode, true);
		list.add(sp);
		sps.addElement(sp);
		current = current.next;
	    }
	} else {
	    while(current!=null) {
		count ++;
		sp = StackProcess.create(sl.app, whichip, ip, spp, current, true);
		list.add(sp);
		sps.addElement(sp);
		current = current.next;
	    }
	}
	if (count > 0) {
	    spp.setCount(count, sps);
	    return spp;
	} else spp.done();
	return null;
    }

      static StackProcessParent runPatchesOutput(int whichip, int ip, int bytecode,
					boolean foreverp, StarLogoCaller caller) {
	StackProcessParentOutput spp =
	    new StackProcessParentOutput(this, 0, whichip,
					 ip, foreverp, true, caller, null);
	StackProcess sp;
	Vector sps = new Vector();
	if (bytecode >= 0) {
	    sp = new FastPatchStackProcess(whichip, ip, spp, bytecode, true);
	    list.add(sp);
	    sps.addElement(sp);
	} else {
	    sp = new FastPatchStackProcess(whichip, ip, spp, true);
	    list.add(sp);
	    sps.addElement(sp);
	}
	spp.setCount(1, sps);
	return spp;
    }

      static StackProcessParent runPatchesOutputSlow(int whichip, int ip, int bytecode,
					    boolean foreverp, StarLogoCaller caller) {
	StackProcessParentOutput spp =
	    new StackProcessParentOutput(this, 0, whichip,
					 ip, foreverp, true, caller, null);
	StackProcess sp;
	Vector sps = new Vector();
	if (bytecode >= 0) {
	    for(int y = 1; y<sl.height+1; y++) {
		int col = y*sl.widthp2;
		for(int x = 1; x<sl.width+1; x++) {
		    sp = StackProcess.create(sl.app, whichip, ip, spp, patches[col + x], bytecode, true);
		    list.add(sp);
		    sps.addElement(sp);
		}
	    }
	} else {
	    for(int y = 1; y<sl.height+1; y++) {
		int col = y*sl.widthp2;
		for(int x = 1; x<sl.width+1; x++) {
		    sp = StackProcess.create(sl.app, whichip, ip, spp, patches[col + x], true);
		    list.add(sp);
		    sps.addElement(sp);
		}
	    }
	}
	spp.setCount(1, sps);
	return spp;
    }

  */

  static StackProcessParent runTurtlesOutput(StarLogo sl, StackProcess process, int whichip,
					     int ip, int bytecode,
					     boolean foreverp, StarLogoCaller caller) {
      return runTurtlesOutput(sl, process, whichip, ip, bytecode, foreverp, caller, null);
  }

  static StackProcessParent runTurtlesOutput(StarLogo sl, StackProcess process, int whichip,
					     int ip, int bytecode,
					     boolean foreverp, StarLogoCaller caller,
					     Turtle notme) {
    //System.out.println("runTurtlesOutput " + process + " " + whichip + " " + ip +
    //	       " " + bytecode + " " + foreverp);
    StackProcessParentOutput spp =
      new StackProcessParentOutput(sl, 0, whichip,
				   ip, foreverp, true, caller, null);
    int count = 0;
    Turtle current = sl.firstturtle;
    StackProcess sp;
    Vector sps = new Vector();
    if (bytecode >= 0) {
      while(current != null) {
	  if (current != notme) {
	      count++;
	      sp = StackProcess.create(sl.app, process, whichip, ip, spp, current, bytecode, true);
	      //System.out.println("make new stack process " + sp);
	      sl.list.add(sp);
	      sps.addElement(sp);
	  }
	  current = current.next;
      }
    } else {
      while(current!=null) {
	if (current != notme) {
	    count ++;
	    sp = StackProcess.create(sl.app, process, whichip, ip, spp, current, true);
	    sl.list.add(sp);
	    sps.addElement(sp);
	}
	current = current.next;
      }
    }
    if (count > 0) {
      spp.setCount(count, sps);
      return spp;
    } else spp.done();
    return null;
  }

  static StackProcessParent runTurtlesOutput(StarLogo sl, StackProcess process, int whichip,
					     int ip, int bytecode,
					     boolean foreverp, Turtle[] turtles, 
					     StarLogoCaller caller) {
      return runTurtlesOutput(sl, process, whichip, ip, bytecode, foreverp, turtles, caller, null);
  }

  static StackProcessParent runTurtlesOutput(StarLogo sl, StackProcess process, int whichip,
					     int ip, int bytecode,
					     boolean foreverp, Turtle[] turtles, 
					     StarLogoCaller caller,
					     Turtle notme) {
    //System.out.println("runTurtlesOutput " + process + " " + whichip + " " + ip +
    //	       " " + bytecode + " " + foreverp);
    StackProcessParentOutput spp =
      new StackProcessParentOutput(sl, 0, whichip,
				   ip, foreverp, true, caller, null);
    int count = 0;
    StackProcess sp;
    Vector sps = new Vector();
    if (bytecode >= 0) {
	for(int i = 0; i < turtles.length; i++) {
	    Turtle current = turtles[i];
	    if (current != notme) {
		count++;
		sp = StackProcess.create(sl.app, process, whichip, ip, spp, current, bytecode, true);
		//System.out.println("make new stack process " + sp);
		sl.list.add(sp);
		sps.addElement(sp);
	    }
	}
    } else {
	for(int i = 0; i < turtles.length; i++) {
	    Turtle current = turtles[i];
	    if (current != notme) {
		count ++;
		sp = StackProcess.create(sl.app, process, whichip, ip, spp, current, true);
		sl.list.add(sp);
		sps.addElement(sp);
	    }
	}
    }
    if (count > 0) {
      spp.setCount(count, sps);
      return spp;
    } else spp.done();
    return null;
  }

  /*
    // Yes
      static StackProcessParent runTurtles(int whichip, int ip, int bytecode,
				  double breed, boolean foreverp, StarLogoCaller caller) {
	StackProcessParent spp = new StackProcessParent(this, 0, whichip, ip, foreverp,
							true, caller);
	int count = 0;
	Turtle current = sl.firstturtle;
	while(current!=null) {
	    if (current.breed == breed) {
		count ++;
		list.add(StackProcess.create(sl.app, whichip, ip, spp, current, bytecode));
	    }
	    current = current.next;
	}
	if (count > 0) {
	    spp.setCount(count);
	    return spp;
	} else spp.done();
	return null;
    }
  */

    // Yes
      static StackProcessParent runTurtles(StarLogo sl, StackProcess process, int whichip,
					   int ip, int bytecode,
					   double breed, boolean foreverp, StarLogoCaller caller) {
	StackProcessParent spp = new StackProcessParent(sl, 0, whichip, ip, foreverp,
							true, caller);
	int count = 0;
	Turtle current = sl.firstturtle;
	while(current!=null) {
	    if (current.breed == breed) {
		if (!current.alive) continue;
		count ++;
		sl.list.add(StackProcess.create(sl.app, process, whichip, ip, spp, current, bytecode));
	    }
	    current = current.next;
	}
	if (count > 0) {
	    spp.setCount(count);
	    return spp;
	} else spp.done();
	return null;
    }

  static StackProcessParent runTurtlesOutput(StarLogo sl, StackProcess process,
					     int whichip, int ip, int bytecode,
					     double breed, boolean foreverp,
					     StarLogoCaller caller) {
      return runTurtlesOutput(sl, process, whichip, ip, bytecode, breed, foreverp, caller, null);
  }

  static StackProcessParent runTurtlesOutput(StarLogo sl, StackProcess process,
					     int whichip, int ip, int bytecode,
					     double breed, boolean foreverp,
					     StarLogoCaller caller, Turtle notme) {
    StackProcessParentOutput spp = new StackProcessParentOutput(sl, 0, whichip, ip,
								foreverp, true,
								caller, null);
	int count = 0;
	Turtle current = sl.firstturtle;
	StackProcess sp;
	Vector sps = new Vector();
	while(current!=null) {
	    if (current != notme && current.breed == breed) {
		count ++;
		sp = StackProcess.create(sl.app, process, whichip, ip, spp, current, bytecode, true);
		sl.list.add(sp);
		sps.addElement(sp);
	    }
	    current = current.next;
	}
	if (count > 0) {
	    spp.setCount(count, sps);
	    return spp;
	} else spp.done();
	return null;
    }


  static StackProcessParent runTurtlesOutput(StarLogo sl, StackProcess process,
					     int whichip, int ip, int bytecode,
					     double breed, boolean foreverp,
					     Turtle[] turtles, StarLogoCaller caller) {
      return runTurtlesOutput(sl, process, whichip, ip, bytecode, breed, foreverp, turtles, caller, null);
  }

  static StackProcessParent runTurtlesOutput(StarLogo sl, StackProcess process,
					     int whichip, int ip, int bytecode,
					     double breed, boolean foreverp,
					     Turtle[] turtles, 
					     StarLogoCaller caller, Turtle notme) {
    StackProcessParentOutput spp = new StackProcessParentOutput(sl, 0, whichip, ip,
								foreverp, true,
								caller, null);
	int count = 0;
	StackProcess sp;
	Vector sps = new Vector();
	for(int i = 0; i < turtles.length; i++) {
	    Turtle current = turtles[i];
	    if (current != notme && current.breed == breed) {
		count ++;
		sp = StackProcess.create(sl.app, process, whichip, ip, spp, current, bytecode, true);
		sl.list.add(sp);
		sps.addElement(sp);
	    }
	}
	if (count > 0) {
	    spp.setCount(count, sps);
	    return spp;
	} else spp.done();
	return null;
    }

  // Yes from GUI
  static StackProcessParent runTurtles(StarLogo sl, int whichip, boolean foreverp,
				       StarLogoCaller caller) {
    return runTurtles(sl, whichip, 0, -1,  foreverp, caller);
  }

  static StackProcessParent runTurtle(StarLogo sl, int whichip, Turtle t,
				      boolean foreverp, StarLogoCaller caller) {
    return runTurtle(sl, whichip, 0, -1, t, foreverp, caller);
  }

  static StackProcessParent runPatch(StarLogo sl, int whichip, int ip, int bytecode,
				     Patch p, boolean foreverp, StarLogoCaller caller) {
    StackProcessParent spp = new StackProcessParent(sl, 1, whichip, ip, foreverp, false,
						    caller);
    sl.list.add(StackProcess.create(sl.app, whichip, ip, spp, p, bytecode));
    return spp;
  }

  static StackProcessParent runPatch(StarLogo sl, int whichip, Patch p,
				     boolean foreverp, StarLogoCaller caller) {
    return runPatch(sl, whichip, 0, -1,  p, foreverp, caller);
  }

  static StackProcessParent runPatches(StarLogo sl, StackProcess process,
				       int whichip, int ip,
				       int bytecode, boolean foreverp,
				       StarLogoCaller caller) {
    StackProcessParent spp = new StackProcessParent(sl, 0, whichip, ip,
							foreverp, false, caller);
    if (bytecode >= 0) {
	sl.list.add(FastPatchStackProcess.create(sl.app, process, whichip, ip, spp, bytecode));
    } else {
	sl.list.add(FastPatchStackProcess.create(sl.app, process, whichip, ip, spp));
    }
    spp.setCount(1);
    return spp;
  }

  /*
    static StackProcessParent runPatchesSlow(StarLogo sp, int whichip, int ip,
    int bytecode, boolean foreverp,
					   StarLogoCaller caller) {
	StackProcessParent spp = new StackProcessParent(this, 0, whichip, ip,
							foreverp, true, caller);
	if (bytecode >= 0) {
	    for(int y = 1; y<height+1; y++) {
		int col = y*widthp2;
		for(int x = 1; x<width+1; x++) {
		    list.add(StackProcess.create(sl.app, whichip, ip, spp, sl.patches[col + x], bytecode));
		}
	    }
	} else {
	    for(int y = 1; y<height+1; y++) {
		int col = y*widthp2;
		for(int x = 1; x<width+1; x++) {
		    list.add(StackProcess.create(sl.app, whichip, ip, spp, sl.patches[col + x]));
		}
	    }
	}
	spp.setCount(width * height);
	return spp;
    }
  */

  static StackProcessParent runPatchSlow(StarLogo sl, StackProcess process,
				       int whichip, int ip,
				       int bytecode, Patch p, boolean foreverp,
				       StarLogoCaller caller) {
    StackProcessParent spp = new StackProcessParent(sl, 1, whichip, ip,
						    foreverp, false, caller);
    if (bytecode >= 0) {
	sl.list.add(StackProcess.create(sl.app, process, whichip, ip, spp, p, bytecode));
    } else {
	sl.list.add(StackProcess.create(sl.app, process, whichip, ip, spp, p));
    }
    return spp;
  }


  static StackProcessParent runPatchesSlow(StarLogo sl, StackProcess process,
					   int whichip, int ip,
					   int bytecode, boolean foreverp,
					   StarLogoCaller caller) {
    StackProcessParent spp = new StackProcessParent(sl, 0, whichip, ip,
						    foreverp, false, caller);
    if (bytecode >= 0) {
      for(int y = 1; y<sl.height+1; y++) {
	int col = y*sl.widthp2;
	for(int x = 1; x<sl.width+1; x++) {
	  sl.list.add(StackProcess.create(sl.app, process, whichip, ip, spp, sl.patches[col + x], bytecode));
	}
      }
    } else {
      for(int y = 1; y<sl.height+1; y++) {
	int col = y*sl.widthp2;
	for(int x = 1; x<sl.width+1; x++) {
	  sl.list.add(StackProcess.create(sl.app, process, whichip, ip, spp, sl.patches[col + x]));
	}
      }
    }
    spp.setCount(sl.width * sl.height);
    return spp;
  }

  static StackProcessParent runPatchesOutput(StarLogo sl, StackProcess process, int whichip,
					     int ip, int bytecode,
					     boolean foreverp, StarLogoCaller caller) {
    StackProcessParentOutput spp =
      new StackProcessParentOutput(sl, 0, whichip,
				   ip, foreverp, false, caller, null);
    StackProcess sp;
    Vector sps = new Vector();
    if (bytecode >= 0) {
	sp = FastPatchStackProcess.create(sl.app, process, whichip, ip, spp, bytecode, true);
      sl.list.add(sp);
      sps.addElement(sp);
    } else {
	sp = FastPatchStackProcess.create(sl.app, process, whichip, ip, spp, true);
      sl.list.add(sp);
      sps.addElement(sp);
    }
    spp.setCount(1, sps);
    return spp;
  }

  static StackProcessParent runPatchesOutputSlow(StarLogo sl, StackProcess process,
						 int whichip, int ip, int bytecode,
						 boolean foreverp, StarLogoCaller caller) {
    StackProcessParentOutput spp =
      new StackProcessParentOutput(sl, 0, whichip,
				   ip, foreverp, false, caller, null);
    StackProcess sp;
    Vector sps = new Vector();
    if (bytecode >= 0) {
      for(int y = 1; y<sl.height+1; y++) {
	int col = y*sl.widthp2;
	for(int x = 1; x<sl.width+1; x++) {
	  sp = StackProcess.create(sl.app, process, whichip, ip, spp, sl.patches[col + x], bytecode, true);
	  sl.list.add(sp);
	  sps.addElement(sp);
	}
      }
    } else {
      for(int y = 1; y<sl.height+1; y++) {
	int col = y*sl.widthp2;
	for(int x = 1; x<sl.width+1; x++) {
	  sp = StackProcess.create(sl.app, process, whichip, ip, spp, sl.patches[col + x], true);
	  sl.list.add(sp);
	  sps.addElement(sp);
	}
      }
    }
    spp.setCount(sl.width * sl.height, sps);
    return spp;
  }


}
