/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.awt.Toolkit;
import java.util.Enumeration;
import java.util.prefs.Preferences;
import java.util.prefs.BackingStoreException;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Frame;
import java.awt.Image;
import yoyo.Context;
import yoyo.LogoError;
import java.util.Hashtable;
import java.util.Enumeration;
import yoyo.YoYoCaller;
import yoyo.YoYo;
import yoyo.Tokenizer;
import yoyo.YoYoObjectInputStream;
import java.io.PrintWriter;
import java.util.Vector;
import java.io.*;
import java.util.zip.*;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.List;
import java.net.*;
import java.awt.Cursor;
import java.lang.reflect.Method;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

public final class StarLogo implements Runnable, StarLogoCaller {

    public static final int MIN_CELLSIZE = 1;
    public static final int MAX_CELLSIZE = 32;

    /* default settings */

    public static final int DEFAULT_CELLSIZE = 8;
    public static final int DEFAULT_HALFHEIGHT = 25;
    public static final int DEFAULT_HALFWIDTH = 25;

    /* things saved in settings */
    int numshapes;
    int cellsize;
    int halfheight, halfwidth;
    /****************************/

    public static final int SQUARES = 0;
    public static final int IMAGEPRODUCER = 1;
    public static final int BUFFERED = 2;
    public static final int NUMPCTYPES = 3;
    public static int altgraphics = getDefaultPatchCanvasType();

    public static Pictures PictInstance = new Pictures();
    public static boolean DEBUG = false;
    public static boolean DEBUG2 = false;
    public static boolean serverside = false;

    public boolean inconsistent = false;
    public PatchCanvasInterface patchcanvas;
    public boolean patchesneeddrawingp = false, patchesneedalittleredrawingp = false;
    Patch[] patches;
    public int height, width;
    int widthp2, heightp2;
    double turhalfheight, turhalfwidth;
    Turtle firstturtle=null;
    public Observer observer;
    int numturtles=0;
    //Graphics theg;
    StackProcessList list;
    Context c;
    public Application app;

    int[][] iinstarray = new int[128][];
    Object[][] oinstarray = new Object[128][];
    String[] ninstarray = new String[128];
    int nextinst = 0;

    int[] whostack = new int[4096];
    int whop = -1;
    ArrayList deadbutgrabbed = new ArrayList();


    boolean offscreen = false;
    boolean patcheson = false;
    public volatile boolean locked = false;
    public Object runningp = new Object();
    public volatile boolean runningnowp = false;
    public volatile boolean pausedp = false;
    public Frame myframe;

    static int getDefaultPatchCanvasType() {
	    return (YoYo.macosxp) ? BUFFERED : (YoYo.linuxp) ? IMAGEPRODUCER : SQUARES;
    }



    StarLogo(Context c, boolean htmlp) {
	this.halfwidth = DEFAULT_HALFWIDTH;
	this.halfheight = DEFAULT_HALFHEIGHT;
	this.cellsize = DEFAULT_CELLSIZE;
	this.numshapes = TurtleShapes.MAXSHAPES;
	this.width = halfwidth * 2 + 1; this.height = halfheight * 2 + 1;
	this.turhalfheight = height / 2.0; this.turhalfwidth = width / 2.0;
	this.widthp2 = width+2; this.heightp2 = height+2;
	this.c = new Context(c);
	patches = new Patch[widthp2*heightp2];
	observer = new Observer(this);
	readCalibrationData(htmlp);
    }

    void setPatchCanvas() {
	setPatchCanvas((Socket)null);
    }

    void setPatchCanvas(Socket client) {
	if (client != null)
	    patchcanvas = new NetworkPatchCanvas(this, client);
	else {
	    switch (altgraphics) {
	    case SQUARES:
		patchcanvas = new PatchCanvas(this);
		break;
	    case IMAGEPRODUCER:
		patchcanvas = new AltPatchCanvas(this);
		break;
	    case BUFFERED:
		patchcanvas = new BufferedPatchCanvas(this);
		break;
	    }
	}
    }

    void setPatchCanvas(PatchCanvasInterface oldpc) {
	oldpc.dispose();
	if (serverside)
	    patchcanvas = new NetworkPatchCanvas(this, oldpc);
	else {
	    switch (altgraphics) {
	    case SQUARES:
		patchcanvas = new PatchCanvas(this, oldpc);
		break;
	    case IMAGEPRODUCER:
		patchcanvas = new AltPatchCanvas(this, oldpc);
		break;
	    case BUFFERED:
		patchcanvas = new BufferedPatchCanvas(this, oldpc);
		break;
	    }
	}
    }

    public void maybeSwitchPatchCanvas(double average) {
 	if (average < fastestPatchCanvasTurtles[0]) return;
	
 	for(int i = 0; i < fastestPatchCanvasTurtles.length - 1; i++) {
 	    int numTurtles = fastestPatchCanvasTurtles[i];
 	    int nextNumTurtles = fastestPatchCanvasTurtles[i+1];
 	    if (average >= numTurtles && average < nextNumTurtles) {
 		int patchCanvasNumber = fastestPatchCanvas[i];
 		if (altgraphics != patchCanvasNumber) {
 		    altgraphics = patchCanvasNumber;
		    //System.out.println("switching patch canvas based on avg: " + average);
 		    switchPatches();
 		} 
 		return;
 	    }
 	}
	
 	int patchCanvasNumber = fastestPatchCanvas[fastestPatchCanvas.length - 1];
 	if (altgraphics != patchCanvasNumber) {
 	    altgraphics = patchCanvasNumber;
	    //System.out.println("switching patch canvas based on avg: " + average);
 	    switchPatches();
 	} 
    }

    /* ----------- Calibration routines ----------- */

    int[] fastestPatchCanvasTurtles = new int[]{ 1, 5, 10, 20, 35, 
						 50, 75, 100, 250, 500 };
    int[] fastestPatchCanvas = new int[10];

    Object calibrationp = new Object();

    /* If true, compute calibration data. Implies that countFrames is false */
    volatile boolean timeFrames = false;

    /* If true, use calibration data in StarLogo.run() to 
       dynamically pick the patchcanvas. */
    volatile boolean countFrames = false;

    void calibrateGraphics() {

	stopAllStarLogo();
	countFrames = false;

	Runnable r = new Runnable() {
		public void run() {
		    //System.out.println("Calibrate graphics");

		    app.disableMonitors = true;
		    
		    int[] fd50 = new int[]{ VM.assembleByteCode(VM.DATA, 0, false, false),
					    0,
					    VM.assembleByteCode(VM.PRIMTURTLESTATE, VM.FD, true, false),
					    VM.assembleByteCode(VM.DONE, 0, false, false) };
		    Object[] fd50objects = new Object[]{ new Double(50) };
		    int whichip = 
			internalSetInstructions("calibrate-turtles", fd50, fd50objects);
		    
		    //System.out.println("whichip = " + whichip);
		    
		    double[][] patchCanvasTimes = new double[NUMPCTYPES][];
		    // For each patchcanvas, run calibration routine
		    // We can't use SQUARES on MacOSX due to dead lock issues.
		    int firstPC = (YoYo.macosxp) ? 1 : 0;
		    
		    altgraphics = firstPC;
		    do {
			//System.out.println("set graphics to " + altgraphics);
			switchPatches();
			Thread.currentThread().yield();
			patchCanvasTimes[altgraphics] = calibratePatchCanvas(whichip);
			altgraphics++;
		    } while(altgraphics < NUMPCTYPES);
		    
		    // For each number of turtles, choose which is the fastest
		    // patch canvas. 
		    for(int i = 0; i < fastestPatchCanvasTurtles.length; i++) {
			double minVal = Integer.MAX_VALUE;
			int minPC = firstPC;
			for(int j = firstPC; j < NUMPCTYPES; j++) {
			    //  System.out.println("patchcanvastimes[" + j +
			    //	       "][" + i + "] = " + patchCanvasTimes[j][i]);
			    if (patchCanvasTimes[j][i] <= minVal) {
				minVal = patchCanvasTimes[j][i];
				minPC = j;
			    }
			}
			fastestPatchCanvas[i] = minPC;
// 			System.out.println("fastest patch canvas for " + 
// 					       fastestPatchCanvasTurtles[i] + 
// 					       " is " + minPC); 
		    }
		    
		    app.disableMonitors = false;

		    Application.createMessageDialog(app, app.activeFrame, app.textBundle.getString("calibration"),
						    app.textBundle.getString("calibration_done"), true);
		    
		    writeCalibrationData();
		}
	    };
	
	if (app.project.saveChanges(app, r) == Project.CANCEL) {
	    return;
	}
	

    }

    double[] calibratePatchCanvas(int whichip) {
	// Collect for various numbers of turtles
	// how fast does each patch canvas run 
	double[] patchCanvasTimes = new double[fastestPatchCanvasTurtles.length];
	for(int i = 0; i < fastestPatchCanvasTurtles.length; i++) {
	    int numTurtles = fastestPatchCanvasTurtles[i];
	    //System.out.println("calibrating #turtles " + numTurtles);
	    patchcanvas.noDisplay();
	    synchronized(runningp) {
		CA();
		crt(numTurtles);
	    }
	    patchcanvas.display();

	    frameTimes.reset();
	    timeFrames = true;
	    //System.out.println("run turtles");
	    try {
		synchronized(calibrationp) {
		    synchronized(runningp) {
			StackProcessParent spp = StackProcessParent.runTurtles(this, whichip, false, this);
			
		    }
		    calibrationp.wait();
		}
	    } catch (InterruptedException ie) {
		ie.printStackTrace();
	    }
	    //System.out.println("done with test");
	    timeFrames = false;

 	    if (frameTimes.ready()) {
 		patchCanvasTimes[i] = frameTimes.getMedian(); //Average();
 	    } else {
 		patchCanvasTimes[i] = 999999; 
 	    }

	    //System.out.println("test time is " + patchCanvasTimes[i]);
	}

	patchcanvas.noDisplay();
	synchronized(runningp) {
	    CA();
	}
	patchcanvas.display();
	
	return patchCanvasTimes;
    }

    long startTimeFrame;
    AverageCounter frameTimes = new AverageCounter(10);
    AverageCounter frameCount = new AverageCounter(10);

    void startTimer() {
 	if (timeFrames) {
 	    startTimeFrame = System.currentTimeMillis();
 	} else if (countFrames) {
 	    patchcanvas.resetNumPatchesDrawn();
 	}
    }
    
    void stopTimer() {
 	if (timeFrames) {
 	    long frameTime = System.currentTimeMillis() - startTimeFrame;
 	    //assert frameTime > 0 : ("Frame time is " + frameTime);
 	    if (frameTime == 0) {
 		//System.out.println("frame time was 0");
 		frameTime = 1;
 	    }
 	    frameTimes.add((int)frameTime);
 	} else if (countFrames) {
	    //System.out.println("drew " + patchcanvas.getNumPatchesDrawn() + " this once-through");
 	    frameCount.add(patchcanvas.getNumPatchesDrawn());
 	    if (frameCount.ready()) {
 		double avg = frameCount.getMedian(); //Average();
		//System.out.println("frame count avg = " + avg);
 		maybeSwitchPatchCanvas(avg);
 	    }
 	} 
    }

    public Preferences getStarLogoPrefs() {
	return Preferences.userNodeForPackage(StarLogo.class);
    }

    public Preferences getCalibrationPrefs() {
	return getStarLogoPrefs().node("calibration");
    }	
	
    public void writeCalibrationData() {
	Preferences calibration = getCalibrationPrefs();
	for(int i = 0; i < fastestPatchCanvasTurtles.length; i++) {
	    int numTurtles = fastestPatchCanvasTurtles[i];
	    int fastestPC = fastestPatchCanvas[i];
	    calibration.putInt(String.valueOf(numTurtles), fastestPC);
	}
	countFrames = true;
	try {
	    calibration.flush();
	} catch (BackingStoreException bse) {
	    bse.printStackTrace();
	    Application.createMessageDialog(app, app.activeFrame, app.textBundle.getString("prefs_error"),
					    app.textBundle.getString("error_cant_save_calibration") +
					    " " + 
					    app.textBundle.getString("error_calibration_single_session"), 
					    true);
	}
    }

    public void readCalibrationData(boolean htmlp) {
	if (htmlp) {
	    int defaultPC = getDefaultPatchCanvasType();
	    for(int i = 0; i < fastestPatchCanvasTurtles.length; i++) {
		fastestPatchCanvas[i] = defaultPC;
	    }
	    countFrames = true;
	} else {
	    Preferences calibration = getCalibrationPrefs();
	    int defaultPC = getDefaultPatchCanvasType();
	    for(int i = 0; i < fastestPatchCanvasTurtles.length; i++) {
		int numTurtles = fastestPatchCanvasTurtles[i];
		
		fastestPatchCanvas[i] = 
		    calibration.getInt(String.valueOf(numTurtles), defaultPC);
	    }
	    countFrames = true;
	} 
    }


    public StackProcessParent spp;
    private Vector spps = new Vector();
    private int sppcount = 0;

    public void done(StackProcessParent spp) {
	spps.removeElement(spp);
	sppcount--;
	//System.out.println("empty? " + spps.isEmpty() + " sppcount " + sppcount);
	if (sppcount == 0) {
	    //System.out.println("button is done. spp = null");
	    this.spp = null;
	    /*
	      while ((System.currentTimeMillis() - timePushed) < 20) { 
	      Thread.currentThread().yield(); 
	      }
	    */
	    synchronized(calibrationp) {
		calibrationp.notify();
	    }
	}
    }

    public void done(Context c) {}

    public boolean donep() {
	return (sppcount == 0);
    }

    public StarLogoCaller caller() {
	return this;
    }

    //int count = 1;

    public void addStackProcessParent(StackProcessParent s) {
	//System.out.println("adding spp " + count++);
	spps.addElement(s);
	sppcount++;
    }

    public void stopAll() {
	int num = 0;
	for(Enumeration e = ((Vector)spps.clone()).elements(); e.hasMoreElements();) {
	    StackProcessParent spp = (StackProcessParent)e.nextElement();
	    //System.out.println("BW: stopping stack process parent " + num + " spp:" + spp);
	    spp.stopAll(); //sl.stopProcess(spp);
	    num++;
	}
	//spps.removeAllElements();
	//assert(spp.numdone == spp.count);
	//assert(spp.numdone == num);
	//System.out.println("stopped " + num + " spps");
    }

    public void callerPrintln(String s) {}
    public void callerClearText() {}

    
    /* ----------- End of calibration routines ----------- */


    void init() {
	init(null);
    }

    void init(Socket client) {
	if (client != null) serverside = true;
	this.list = new StackProcessList(app);
	for (int y = 0; y<heightp2; y++) {
	    int col = y*widthp2;
	    for (int x = 0; x<widthp2; x++) {
		patches[col + x] = new Patch(x, y, this);
	    }
	}
	list.create(observer);
	setPatchCanvas(client);
    }




    Turtle[] crt(int num) {
	return crt(num, 0, 0);
    }

    Turtle[] crt(int num, double breed, int shape) {
	Turtle[] ts = new Turtle[num];
	Turtle t=null; double dh = 360.0 / num; double h = 0;
	for(int i = 0; i<num; i++) {
	    int who = (whop < 0) ? numturtles : whostack[whop--];
	    numturtles++;
	    t = new Turtle(this, 0,0,h,(numturtles%(Colors.numcolors/10))*10 + 5,
			   who,breed,shape);
	    h+=dh;
	    ts[i] = t;
	    if (firstturtle != null) {
		t.next = firstturtle;
		firstturtle.prev = t;
	    }
	    firstturtle = t;
	    wrapandlink(t);
	    list.create(t);
	    Application.setTurtleMonitor(t);
	}
	return ts;
    }

    Turtle cct(double xcor, double ycor, double heading, double color, double breed, int shape) {
	return cct(xcor, ycor, heading, color, breed, shape, false, true, 1, 1);
    }

    Turtle cct(double xcor, double ycor, double heading, double color, double breed, int shape,
	       boolean pendownp, boolean shownp, int plotpen, int plotwidget) {
	Turtle t=null;
	int who = (whop < 0) ? numturtles : whostack[whop--];
	numturtles++;
	t = new Turtle(this, xcor,ycor,heading,color, who, breed, shape, pendownp, 
		       shownp, plotpen, plotwidget);
	if (firstturtle != null) {
	    t.next = firstturtle;
	    firstturtle.prev = t;
	}
	firstturtle = t;
	wrapandlink(t);
	list.create(t);
	Application.setTurtleMonitor(t);
	return t;
    }


    int countBreeds(double breed) {
	if (firstturtle == null) return 0;
	int num = 0;
	for(Turtle pick = firstturtle; pick != null; pick = pick.next) {
	    if (pick.breed == breed) num++;
	}
	return num;
    }

    int countColor(double color) {
	if (firstturtle == null) return 0;
	int num = 0;
	for(Turtle pick = firstturtle; pick != null; pick = pick.next) {
	    if (Math.round(pick.color) == Math.round(color)) num++;
	}
	return num;
    }

    int countColor(double mincolor, double maxcolor) {
	if (firstturtle == null) return 0;
	int num = 0;
	for(Turtle pick = firstturtle; pick != null; pick = pick.next) {
	    if (pick.color >= mincolor && pick.color <= maxcolor) num++;
	}
	return num;
    }

    Object[] listTurtles() {
	return listTurtles(null);
    }

    Object[] listTurtles(Turtle notme) {
	Object[] ret = new Object[numturtles - ((notme != null) ? 1 : 0)];
	int i = 0;
	for(Turtle pick = firstturtle; pick != null; pick = pick.next) {
	    if (pick != notme) ret[i++] = pick.whoInt;
	}
	return ret;
    }

    Object[] listBreeds(double breed) {
	return listBreeds(breed, null);
    }
    Object[] listBreeds(double breed, Turtle notme) {
	Object[] ret = new Object[countBreeds(breed) - ((notme != null &&
							   notme.breed == breed) ? 1 : 0)];
	Turtle pick; int i;
	for(i = 0, pick = firstturtle; pick != null; pick = pick.next) {
	    if (pick.breed == breed && pick != notme) {
		ret[i] = pick.whoInt;
		i++;
	    }
	}
	return ret;
    }

    int randomTurtle() {
	return randomTurtle(null);
    }
    int randomTurtle(Turtle notme) {
	if (firstturtle == null) return -1;
	if (numturtles == 1 && notme != null) return -1;
	int num = (int)Math.floor(VM.rand.nextDouble()*(numturtles - ((notme == null) ? 0 : 1)));
	Turtle pick = firstturtle;
	for(int i = 0; i<num; i++) pick = pick.next;
	if (pick == null) return -1; // shouldn't happen.
	if (pick == notme) pick = pick.next;
	if (pick == null) return -1; // shouldn't happen.
	return pick.who;
    }

    int randomBreed(double breed) {
	return randomBreed(breed, null);
    }

    int randomBreed(double breed, Turtle notme) {
	if (firstturtle == null) return -1;
	int numBreeds = countBreeds(breed);
	if (numBreeds == 1 && notme != null && notme.breed == breed) return -1;
	int num = (int)Math.floor(VM.rand.nextDouble()* (numBreeds -
							 ((notme != null &&
							   notme.breed == breed) ? 1 : 0)));
	for(Turtle pick = firstturtle; pick != null; pick=pick.next) {
	    if (pick.breed == breed) {
		if (num == 0) {
		    if (pick == notme) continue;
		    else return pick.who;
		}
		else num--;
	    }
	}
	return -1;
    }

    Turtle getWho(int who) {
	if (who < 0) return null;
	Turtle current = firstturtle;
	while(current!=null) {
	    if (current.who == who) return current;
	    current = current.next;
	}
	for(Iterator i = deadbutgrabbed.iterator(); i.hasNext(); ) {
	    Turtle t = (Turtle)i.next();
	    if (t.who == who) return t;
	}
	return null;
    }



    int patchat(double x, double y) {
	y %= height; if (y < 0) y += height;
	x %= width; if (x < 0) x += width;
	int poffset = ((int)y + 1) * widthp2 + (int)x + 1;
	return poffset;
    }

    int patchtowards(double angle, double magnitude, double xcor, double ycor) {
	double angletemp = angle - (int)angle;
	angle = (double)((int)angle)%360;
	if (angle<0) angle += 360;
	angle += angletemp;
	double x = magnitude * Math.sin(VM.deg2Rad(angle));
	double y = -magnitude * Math.cos(VM.deg2Rad(angle));
	return patchat(xcor + x, ycor + y);
    }

    void killAll() {
	Turtle current = firstturtle;
	while(current!=null) {
	    unlink(current);
	    die(current);
	    current = current.next;
	}
	whostack = new int[4096];
	whop = -1;
    }

    void wrapandlink(Turtle t) {
	//t.ycor= (t.ycor + heightsquared) % height;
	//t.xcor= (t.xcor + widthsquared) % width;
	//t.ycor = (((t.ycor % height) + height) % height);
	//t.xcor = (((t.xcor % width) + width) % width);
	t.ycor %= height; 
	if (t.ycor < 0.0) t.ycor += height;
	if (t.ycor == (double)height) t.ycor = 0.0;

	t.xcor %= width;
	if (t.xcor < 0.0) t.xcor += width;
	if (t.xcor == (double)width) t.xcor = 0.0;

// 	int ycor = (int)t.ycor;
// 	int ycor2 = ycor % height;
// 	if (ycor2 < 0 || (ycor2 == 0 && t.ycor < 0.0)) ycor2 += height;
// 	t.ycor = (double)ycor2 + (t.ycor - (double)ycor);
// 	if (t.ycor == height) t.ycor = 0.0;
// 	assert(t.ycor != height);

// 	int xcor = (int)t.xcor;
// 	int xcor2 = xcor % width;
// 	if (xcor2 < 0 || (xcor2 == 0 && t.xcor < 0.0)) xcor2 += width;
// 	t.xcor = (double)xcor2 + (t.xcor - (double)xcor);
// 	if (t.xcor == width) t.xcor = 0.0;
// 	assert(t.xcor != width);

	link(t);
    }

    void linkAll(int xoff, int yoff) {
	for(Turtle t= firstturtle; t != null; t = t.next) {
	    t.xcor += xoff;
	    t.ycor += yoff;
	    link(t);
	}
    }


    void link(Turtle t) {
	int poffset = ((int)t.ycor+1) * widthp2 + (int)t.xcor + 1;
	Patch p = patches[poffset];
	if (!p.visible) {
	    System.out.println("Turtle @ (" + t.xcor + ", " + t.ycor + ") is on bad patch. Fixing it.");
	    if (t.xcor >= (double)width) {
		t.xcor = 0.0;
	    } else if (t.xcor < 0.0) {
		t.xcor = (double)width;
	    }
	    if (t.ycor >= (double)height) {
		t.ycor = 0.0;
	    } else if (t.ycor < 0.0) {
		t.ycor = (double)height;
	    }
	    poffset = ((int)t.ycor+1) * widthp2 + (int)t.xcor + 1;
	    p = patches[poffset];
	    assert(p.visible);
	}
	t.patch = poffset;
	if (t.shownp) {
	    Turtle him = p.topturtle;
	    if (him != null) {
		t.underme = him;
		him.overme = t;
	    }
	    p.topturtle = t;
	    patchcanvas.drawTurtle(t);
	} else {
	    Turtle him = p.tophiddenturtle;
	    if (him != null) {
		t.underme = him;
		him.overme = t;
	    }
	    p.tophiddenturtle = t;
	}
    }


    void unlink(Turtle t) {
	int poffset = ((int)t.ycor + 1) * widthp2 + (int)t.xcor + 1;
	Patch p = patches[poffset];
	Turtle under = t.underme;
	Turtle over = t.overme;
	if (t.shownp) {
	    if (under == null) {
		if (over == null) { /* was the only shown one on the patch */
		    p.topturtle = null;
		    patchcanvas.drawPatch(p);
		} else { /* 2 turtles, i'm on bottom */
		    over.underme = null;
		}
	    } else {
		if (over == null) { /* I'm on the top of the patch */
		    under.overme = null;
		    p.topturtle = under;
		    patchcanvas.drawTurtle(under);
		} else { /* I'm in the middle of the stack */
		    over.underme = under;
		    under.overme = over;
		}
	    }
	} else {
	    if (under == null) {
		if (over == null) { /* was the only hidden one on the patch */
		    p.tophiddenturtle = null;
		} else { /* 2 turtles, i'm on bottom */
		    over.underme = null;
		}
	    } else {
		if (over == null) { /* I'm on the top of the patch */
		    under.overme = null;
		    p.tophiddenturtle = under;
		} else { /* I'm in the middle of the stack */
		    over.underme = under;
		    under.overme = over;
		}
	    }
	}
	t.underme = null;
	t.overme = null;
	t.patch = -1;
    }


    void unlinkAll(Patch[] patches, int widthp2) {
	for(Turtle t = firstturtle; t != null; t = t.next) {
	    int poffset = ((int)t.ycor + 1) * widthp2 + (int)t.xcor + 1;
	    patches[poffset].topturtle = null;
	    patches[poffset].tophiddenturtle = null;
	    t.patch = -1;
	    t.underme = null;
	    t.overme = null;
	}
    }

    void unlinkDuringResize(Turtle t, Patch[] patches, int widthp2) {
	int poffset = ((int)t.ycor + 1) * widthp2 + (int)t.xcor + 1;
	Patch p = patches[poffset];
	Turtle under = t.underme;
	Turtle over = t.overme;
	if (t.shownp) {
	    if (under == null) {
		if (over == null) { /* was the only shown one on the patch */
		    p.topturtle = null;
		    patchcanvas.drawPatch(p, patches);
		} else { /* 2 turtles, i'm on bottom */
		    over.underme = null;
		}
	    } else {
		if (over == null) { /* I'm on the top of the patch */
		    under.overme = null;
		    p.topturtle = under;
		    patchcanvas.drawTurtle(under, patches);
		} else { /* I'm in the middle of the stack */
		    over.underme = under;
		    under.overme = over;
		}
	    }
	} else {
	    if (under == null) {
		if (over == null) { /* was the only hidden one on the patch */
		    p.tophiddenturtle = null;
		} else { /* 2 turtles, i'm on bottom */
		    over.underme = null;
		}
	    } else {
		if (over == null) { /* I'm on the top of the patch */
		    under.overme = null;
		    p.tophiddenturtle = under;
		} else { /* I'm in the middle of the stack */
		    over.underme = under;
		    under.overme = over;
		}
	    }
	}
	t.underme = null;
	t.overme = null;
	t.patch = -1;
    }


    void showTurtle(Turtle t) {
	int poffset = ((int)t.ycor+1) * widthp2 + (int)t.xcor + 1;
	Patch p = patches[poffset];
	Turtle under = t.underme;
	Turtle over = t.overme;
	/* first, get it out of the hidden turtle list */
	if (under == null) {
	    if (over == null) { /* was the only hidden one on the patch */
		p.tophiddenturtle = null;
	    } else { /* 2 turtles, i'm on bottom */
		over.underme = null;
	    }
	} else {
	    if (over == null) { /* I'm on the top of the patch */
		under.overme = null;
		p.tophiddenturtle = under;
	    } else { /* I'm in the middle of the stack */
		over.underme = under;
		under.overme = over;
	    }
	}
	/* Now link him in the shown turtles */
	Turtle him = p.topturtle;
	if (him != null) {
	    t.underme = him;
	    him.overme = t;
	} else t.underme = null;
	t.overme = null;
	p.topturtle = t;

	patchcanvas.drawTurtle(t);
    }

    void hideTurtle(Turtle t) {
	int poffset = ((int)t.ycor+1) * widthp2 + (int)t.xcor + 1;
	Patch p = patches[poffset];
	Turtle under = t.underme;
	Turtle over = t.overme;
	/* first, get it out of the shown turtle list */
	if (under == null) {
	    if (over == null) { /* was the only shown one on the patch */
		p.topturtle = null;
		patchcanvas.drawPatch(p); //t.xcor, t.ycor, p.pc);
	    } else { /* 2 turtles, i'm on bottom */
		over.underme = null;
	    }
	} else {
	    if (over == null) { /* I'm on the top of the patch */
		under.overme = null;
		p.topturtle = under;
		patchcanvas.drawTurtle(under);
	    } else { /* I'm in the middle of the stack */
		over.underme = under;
		under.overme = over;
	    }
	}
	/* Now link him in the hidden turtles */
	Turtle him = p.tophiddenturtle;
	if (him != null) {
	    t.underme = him;
	    him.overme = t;
	} else t.underme = null;
	t.overme = null;
	p.tophiddenturtle = t;
    }



    void die(Turtle t) {
	t.alive = false;
	t.die();
	Turtle prev = t.prev;
	Turtle next = t.next;
	if (next == null) {
	    if (prev == null) { firstturtle = null; }  /* only turtle around */
	    else {
		prev.next = null;  // last on the patch
	    }
	}
	else {
	    if (prev == null) {
		next.prev = null;
		firstturtle = next;
	    }
	    else {
		prev.next = next;
		next.prev = prev;
	    }
	}
	numturtles--;
	if (t.grabber == null) recycleWho(t);
	else deadbutgrabbed.add(t);
	done(t);

	//remove point
	// called from t.die()
	//app.iface.erasePointsOnAllWidgets(t.who);
    }

    void recycleDeadWho(Turtle t) {
	recycleWho(t);
	deadbutgrabbed.remove(t);
    }

    void recycleWho(Turtle t) {
	if (whop == whostack.length-1) {
	    int[] oldwhostack = whostack;
	    whostack = new int[oldwhostack.length * 2];
	    System.arraycopy(oldwhostack, 0, whostack, 0, oldwhostack.length);
	}
	whostack[++whop] = t.who;

    }




    public void turtlePrint(String s) {
	if (!app.kioskp && YoYo.applet == null) {
	    app.procWin.turtlePrint(s);
	} else if (YoYo.applet == null) YoYo.applet.showStatus(s);
    }

    public void observerPrint(String s) {
	if (!app.kioskp && YoYo.applet == null) {
	    app.procWin.observerPrint(s);
	} else if (YoYo.applet == null) YoYo.applet.showStatus(s);
    }

    public void turtlePrintln(String s) {
	if (!app.kioskp && YoYo.applet == null) {
	    app.procWin.turtlePrintln(s);
	} else if (YoYo.applet == null) YoYo.applet.showStatus(s);
    }

    public void observerPrintln(String s) {
	if (!app.kioskp && YoYo.applet == null) {
	    app.procWin.observerPrintln(s);
	} else if (YoYo.applet == null) YoYo.applet.showStatus(s);
    }

    public void setSpeed(int speed) {
	this.speed = speed;
	if (speed == 0) {
	    speedtimer.pause();
	    pausedp = true;
	} else {
	    pausedp = false;
	    if (speed == 100) speedtimer.pause();
	    else {
		int adjustedspeed = 4 * speed - 48;
		if (speed < 16) adjustedspeed = 3 * speed - 16;
		if (speed < 8) adjustedspeed = 2 * speed - 4;
		if (speed < 4) adjustedspeed = speed;
		long waitperiod = 2000 / adjustedspeed;
		waitres = Math.min(MAX_WAIT_RESOLUTION, waitperiod / DIV_FACTOR);
		speedtimer.setWaitPeriod(waitperiod);
		//used to do this check instead: if ((!speedtimer.isAlive()) && runningnowp) 
		//but runningnowp was not getting reset after a pause
		if (!speedtimer.isAlive()) 
		{
			synchronized(runningp){
			    runningp.notify();
			    speedtimer.restart();
			}
		    }
		
	    }
	    //if (speed < 100) time2 = System.currentTimeMillis();
	}
    }

    private volatile int speed = 100;
	//private long time1 = 0;
	//private long time2 = 0;
	//private long sleeptime;
	public long waitres = 50;
	public long MAX_WAIT_RESOLUTION = 50;
	public long DIV_FACTOR = 4;
	public volatile boolean go = true;
	private SpeedTimer speedtimer = new SpeedTimer(this);

    public int loops = 0;

    public void run() {
	StackProcessList list = this.list;
	Context c = this.c;
	Thread thread = Thread.currentThread();
	//Runtime.getRuntime().traceMethodCalls(true);
	while (true) {
	    try {
		startTimer();
		synchronized (runningp) {
		    // when nothing is going on, enter this block
		    if (locked || list.size() == 0 || pausedp) {
			runningnowp = false;
			speedtimer.pause();
			runningp.wait();
			if (speed < 100) speedtimer.restart();
			runningnowp = true;
			// why is this here?!
			// abb shapestub();
		    }
		    synchronized(patchcanvas.getGraphicsLock()) {
			if (patchcanvas instanceof PatchCanvas)
			    ((PatchCanvas)patchcanvas).setDrawingSurface();
			try {
			    for(SPEnumeration e = list.elements(); e.hasMoreElements(); ) {
				StackProcess p = e.nextElement();
				p.switchme(c);
			}
			} catch (RuntimeException e) {
			    e.printStackTrace();
			    stopAllStarLogo();
			}
			finally {
			    if (patchesneeddrawingp || patchesneedalittleredrawingp) loops++;
			    if (patchesneeddrawingp) {
				redraw();
				patchesneeddrawingp = false;
				patchesneedalittleredrawingp = false;
			    } else {
				if (patchesneedalittleredrawingp) {
				    refresh();
				    patchesneedalittleredrawingp = false;
				}
			    }
			    if (patchcanvas instanceof PatchCanvas) 
				((PatchCanvas)patchcanvas).deleteDrawingSurface();
			    app.flushTextAreas();
			}
		    }
		}
		Toolkit.getDefaultToolkit().sync();
		stopTimer();
		thread.yield();
		if (speed < 100) {
			// currentTime lies like a bitch!@!!@
			//time1 = System.currentTimeMillis();
			//sleeptime = (Math.max(Math.min(1000, ((10000 / (speed * speed)) - 1) * (time1 - time2)), 0) + sleeptime) / 2;
			//sleeptime = (Math.min(1000, (100 / speed) * (time1 - time2)) + sleeptime) / 2;
			//thread.sleep(sleeptime);
			//System.out.println(sleeptime);
			//time2 = System.currentTimeMillis();
			while (!go) thread.sleep(waitres);
			go = false;
		}
	    }
	    catch (LogoError e) {
		stopAllStarLogo();
	    }
	    catch (InterruptedException e) {
		stopAllStarLogo();
		break;
	    }
	    catch (Exception e) {
		//System.out.println(e.toString());
		e.printStackTrace();
		stopAllStarLogo();
		//break;
	    }
	}
    }


    int setInstructions(String name, Object[] inst, Object[] objs) {
// 	try {
// 	    throw new Exception();
// 	} catch (Exception e) {
// 	    e.printStackTrace();
// 	}
	int[] ilist = new int[inst.length];
	for (int i = 0; i<inst.length; i++) {
	    ilist[i] = ((Number)inst[i]).intValue();
	}
	return internalSetInstructions(name, ilist, objs);
    }

    synchronized int internalSetInstructions(String name, int[] ilist, Object[] olist) {
// 	try {
// 	    throw new Exception();
// 	} catch (Exception e) {
// 	    e.printStackTrace();
// 	}
	if (DEBUG) {
	    app.log.println("Set Instructions: " + name + " #" + nextinst + " to: ");
	    app.log.println("ilist = " + YoYo.printToString(ilist, true));
	    app.log.println("olist = " + YoYo.printToString(olist));
	    System.out.println("Set Instructions: " + name + " #" + nextinst + " to: ");
	    System.out.println("ilist = " + YoYo.printToString(ilist, true));
	    System.out.println("olist = " + YoYo.printToString(olist));
	}

		if (nextinst >= iinstarray.length) {
			int[][] newiinstarray = new int[iinstarray.length * 2][];
			Object[][] newoinstarray = new Object[iinstarray.length * 2][];
			String[] newninstarray = new String[iinstarray.length * 2];
			System.arraycopy(iinstarray, 0, newiinstarray, 0, iinstarray.length);
			System.arraycopy(oinstarray, 0, newoinstarray, 0, iinstarray.length);
			System.arraycopy(ninstarray, 0, newninstarray, 0, iinstarray.length);
			iinstarray = newiinstarray;
			oinstarray = newoinstarray;
			ninstarray = newninstarray;
		}
		iinstarray[nextinst] = ilist;
		oinstarray[nextinst] = olist;
		ninstarray[nextinst] = name;
	return nextinst++;
    }

    synchronized int internalSetInstructions(String name, int[] ilist,
					     Object[] olist, int whichip) {
	if (StarLogo.DEBUG) {
	    app.log.println("Set Instructions: " + name + " #" + whichip + " to: ");
	    app.log.println("ilist = " + YoYo.printToString(ilist, true));
	    app.log.println("olist = " + YoYo.printToString(olist));
	    System.out.println("Set Instructions: " + name + " #" + whichip + " to: ");
	    System.out.println("ilist = " + YoYo.printToString(ilist, true));
	    System.out.println("olist = " + YoYo.printToString(olist));
	}

	if (whichip >= iinstarray.length) {
			int[][] newiinstarray = new int[whichip + 5][];
			Object[][] newoinstarray = new Object[whichip + 5][];
			String[] newninstarray = new String[whichip + 5];
			System.arraycopy(iinstarray, 0, newiinstarray, 0, iinstarray.length);
			System.arraycopy(oinstarray, 0, newoinstarray, 0, iinstarray.length);
			System.arraycopy(ninstarray, 0, newninstarray, 0, iinstarray.length);
			iinstarray = newiinstarray;
			oinstarray = newoinstarray;
			ninstarray = newninstarray;
		}
	iinstarray[whichip] = ilist;
	oinstarray[whichip] = olist;
	ninstarray[whichip] = name;
	return whichip;
    }

    synchronized int[] internalGetIlist(int whichip) {
	return iinstarray[whichip];
    }

    synchronized Object[] internalGetOlist(int whichip) {
	return oinstarray[whichip];
    }

    synchronized String internalGetName(int whichip) {
	return ninstarray[whichip];
    }

    synchronized int internalGetWhichip(String procname) {
	for(int i = 0; i < nextinst; i++) {
	    if (ninstarray[i].equals(procname)) return i;
	}
	return -1;
    }

    void done(StackProcess sp) {
	//System.out.println("entity: " + sp.entity + " is done running spp: " + sp.spp);
	list.remove(sp);
    }


    void done(Turtle t) {
	//System.out.println("turtle: " + t + " is done.");
	list.remove(t);
    }

    public void stopAllStarLogo() {
	//System.out.println("stop all");
	if (list.size() != 0) {
	    list.removeAllSPs();
	}
    }

    void stopTurtles() {
	//System.out.println("stop turtles");
	Turtle current = firstturtle;
	while(current!=null) {
	    //System.out.println("stop turtle: " + current);
	    list.remove(current);
	    current = current.next;
	}
    }

    void stopObserver() {
	list.remove(observer);
    }

    void stopPatches() {
	for(int y = 1; y<height+1; y++) {
	    int col = y*widthp2;
	    for(int x = 1; x<width+1; x++) {
		list.remove(patches[col + x]);
	    }
	}
    }


    public void stopProcess(StackProcessParent spp) {
	//System.out.println("starlogo stop spp " + spp);
	//try { throw new Exception(); } catch (Exception e) { e.printStackTrace(); }
	list.stop(spp);
    }


    void CA() {
	CG();
	CP();
	CO();
	killAll();
	if (!app.kioskp && YoYo.applet == null) {
	    app.outWin.reset();
	    app.iface.clearLineWidgets();
	    //app.plotWin.clearplot();
	}
    }


    void CG() { setPC(0.0); }
    void CP() { setPatchVars(new Integer(0)); }
    void CO() { setObserverVars(new Integer(0)); }

    void setPC(double color) {
	for(int y = 1; y<height+1; y++) {
	    int col = y*widthp2;
	    for(int x = 1; x<width+1; x++) {
		Patch p = patches[col+x];
		if (p.pc != color) {
		    patches[col + x].pc = color;
		}
	    }
	}
    }

    int countPC(double color) {
	int num = 0;
	for(int y = 1; y<height+1; y++) {
	    int col = y*widthp2;
	    for(int x = 1; x<width+1; x++) {
		if (Math.round(patches[col + x].pc) == Math.round(color)) num++;
	    }
	}
	return num;
    }

    int countPC(double mincolor, double maxcolor) {
	int num = 0;
	for(int y = 1; y<height+1; y++) {
	    int col = y*widthp2;
	    for(int x = 1; x<width+1; x++) {
		if (patches[col + x].pc >= mincolor && patches[col+x].pc <= maxcolor) num++;
	    }
	}
	return num;
    }

    void scalePC(int patchvar, double color, double max, double min) {
	color %= Colors.numcolors;  // color
	color = (color<0) ? color+Colors.numcolors : color;
	color = ((int)(color / 10)) * 10.0;
	double range = max - min; // range
	for(int y = 1; y<height+1; y++) {
	    int col = y*widthp2;
	    for(int x = 1; x<width+1; x++) {
		Patch p = patches[col+x];
		double val = p.patchesowndoubles[patchvar]; //observer.aDouble(p.patchesown[patchvar]);
		double mindiff = val - min; // diff from min
		double fract = mindiff / range; // diff / range
		if (fract < 0.0) {
		    p.pc = 0;
		}
		else {
		    if (fract > 1.0) fract = 1;
		    fract *= 9.999999999;
		    p.pc = color + fract;
		}
	    }
	}
    }

    void setTurtleVar(int offset, Object val) {
	Turtle current = firstturtle;
	while(current!=null) {
	    current.turtlesown[offset] = val;
	    current = current.next;
	}
    }


    void setPatchVar(int offset, Object val) {
	if (val instanceof Number) {
	    double dval = ((Number)val).doubleValue();
	    for(int y = 1; y<height+1; y++) {
		int col = y*widthp2;
		for(int x = 1; x<width+1; x++) {
		    patches[col + x].turtlesown[offset] = val;
		    patches[col + x].patchesowndoubles[offset] = dval;
		}
	    }
	}
	else {
	    for(int y = 1; y<height+1; y++) {
		int col = y*widthp2;
		for(int x = 1; x<width+1; x++) {
		    patches[col + x].turtlesown[offset] = val;
		}
	    }
	}
    }

    void setPatchVars(Object val) {
	if (val instanceof Number) {
	    double dval = ((Number)val).doubleValue();
	    for(int y = 1; y<height+1; y++) {
		int col = y*widthp2;
		for(int x = 1; x<width+1; x++) {
		    Object[] p = patches[col + x].turtlesown;
		    double[] pd = patches[col + x].patchesowndoubles;
		    for (int i = 0; i < p.length; i++) {
			p[i] = val;
			pd[i] = dval;
		    }
		}
	    }
	} else {
	    for(int y = 1; y<height+1; y++) {
		int col = y*widthp2;
		for(int x = 1; x<width+1; x++) {
		    Object[] p = patches[col + x].turtlesown;
		    for (int i = 0; i < p.length; i++) {
			p[i] = val;
		    }
		}
	    }
	}
    }

    void setObserverVars(Object val) {
	for(int offset = 0; offset < observer.turtlesown.length; offset++) {
	    observer.turtlesown[offset] = val;
	}
    }

    void redraw() {
	patchcanvas.updateImage();
	patchcanvas.refresh();
    }

    void refresh() {
	patchcanvas.refresh();
    }



    public String saveSettings() {
	String settings = "`settings`\r\n" +
	    "patch-size " + cellsize + "\r\n" +
	    "num-shapes " + numshapes + "\r\n" +
	    "screen-half-width " + halfwidth + "\r\n" +
	    "screen-half-height " + halfheight + "\r\n" +
	    "interface-window-xcor " + app.iface.frame().getLocation().x + "\r\n" +
	    "interface-window-ycor " + app.iface.frame().getLocation().y + "\r\n" +
	    "interface-window-size " + app.iface.getVisibleSize().width + " " +
	    app.iface.getVisibleSize().height + "\r\n" +
	    "output-window-xcor " + app.outWin.getLocation().x + "\r\n" +
	    "output-window-ycor " + app.outWin.getLocation().y + "\r\n" +
	    "output-window-width " + app.outWin.getSize().width + "\r\n" +
	    "output-window-height " + app.outWin.getSize().height + "\r\n" +
	    "info-window-xcor " + app.infoWin.getLocation().x + "\r\n" +
	    "info-window-ycor " + app.infoWin.getLocation().y + "\r\n" +
	    "info-window-width " + app.infoWin.getSize().width + "\r\n" +
	    "info-window-height " + app.infoWin.getSize().height + "\r\n" +
	    "control-center-xcor " + app.procWin.getLocation().x + "\r\n" +
	    "control-center-ycor " + app.procWin.getLocation().y + "\r\n" +
	    "control-center-width " + app.procWin.getSize().width + "\r\n" +
	    "control-center-height " + app.procWin.getSize().height + "\r\n" +
	    "shape-chooser-xcor " + app.shapeChooser.getLocation().x + "\r\n" +
	    "shape-chooser-ycor " + app.shapeChooser.getLocation().y + "\r\n" +
	    "shape-chooser-width " + app.shapeChooser.getSize().width + "\r\n" +
	    "shape-chooser-height " + app.shapeChooser.getSize().height + "\r\n" +
	    "turtle-command-center-height " + app.procWin.getCCHeight(JControlCenter.tur) + "\r\n" +
	    "observer-command-center-height " + app.procWin.getCCHeight(JControlCenter.obs) + "\r\n" +
	    "background-image \"" + Utilities.printable(app.iface.getBackgroundImageName()) + "\"\r\n"+
		"interface-locked? " + app.locked + "\r\n";
	return settings;
    }

    public void restoreSettings(String str, boolean htmlp) {
	String title = "`settings`";
	int start = str.indexOf(title);
	if (start == -1) return;
	int end = str.indexOf("`", start+title.length());
	if (end == -1) end = str.length();
	Tokenizer tokenizer = new Tokenizer(str.substring(start+title.length(), end));
	ArrayList items = tokenizer.tokenize();

	int new_halfheight = halfheight, new_halfwidth = halfwidth, new_cellsize = cellsize;
	int new_iface_xcor = 0;
	int new_iface_ycor = 0;

	int new_out_xcor = 0, new_out_ycor = 0, new_out_width = 0, new_out_height = 0;
	int new_info_xcor = 0, new_info_ycor = 0, new_info_width = 0, new_info_height = 0;
	int new_cc_xcor = 0, new_cc_ycor = 0, new_cc_width = 0, new_cc_height = 0;
	int new_plot_xcor = 0, new_plot_ycor = 0, new_plot_width = 0, new_plot_height = 0;
	int new_tur_cc_height = 0, new_obs_cc_height = 0;
	int new_shape_xcor = 0, new_shape_ycor = 0, new_shape_width = 0, new_shape_height = 0;

	String new_bg_image_name = "";

	if (!htmlp) {
	    new_iface_xcor = app.iface.frame().getLocation().x;
	    new_iface_ycor = app.iface.frame().getLocation().y;

	    new_out_xcor = app.outWin.getLocation().x;
	    new_out_ycor = app.outWin.getLocation().y;
	    new_out_width = app.outWin.getSize().width;
	    new_out_height = app.outWin.getSize().height;

	    new_info_xcor = app.infoWin.getLocation().x;
	    new_info_ycor = app.infoWin.getLocation().y;
	    new_info_width = app.infoWin.getSize().width;
	    new_info_height = app.infoWin.getSize().height;

	    new_cc_xcor = app.procWin.getLocation().x;
	    new_cc_ycor = app.procWin.getLocation().y;
	    new_cc_width = app.procWin.getSize().width;
	    new_cc_height = app.procWin.getSize().height;

	    new_tur_cc_height = app.procWin.getCCHeight(JControlCenter.tur);
	    new_obs_cc_height = app.procWin.getCCHeight(JControlCenter.obs);

	    new_shape_xcor = app.shapeChooser.getLocation().x;
	    new_shape_ycor = app.shapeChooser.getLocation().y;
	    new_shape_width = app.shapeChooser.getSize().width;
	    new_shape_height = app.shapeChooser.getSize().height;


	    //new_plot_xcor = app.plotWin.getLocation().x;
	    //new_plot_ycor = app.plotWin.getLocation().y;
	    //new_plot_width = app.plotWin.getSize().width;
	    //new_plot_height = app.plotWin.getSize().height;
	}

	int i = 0;
	while (i<items.size()) {
	    String item = items.get(i).toString();
	    if (item.equals("patch-size")) {
		String value = items.get(i+1).toString();
		new_cellsize = Integer.parseInt(value);
		i += 2;
		continue;
	    }
	    if (item.equals("num-shapes")) {
		String value = items.get(i+1).toString();
		numshapes = Integer.parseInt(value);
		i += 2;
		continue;
	    }
	    if (item.equals("background-image")) {
		new_bg_image_name = JInterface.removeQuotes(items.get(i+1).toString());
		i += 2;
		continue;
	    }
	    if (item.equals("screen-half-width")) {
		String value = items.get(i+1).toString();
		new_halfwidth = Integer.parseInt(value);
		i += 2;
		continue;
	    }
	    if (item.equals("screen-half-height")) {
		String value = items.get(i+1).toString();
		new_halfheight = Integer.parseInt(value);
		i += 2;
		continue;
	    }
	    if (item.equals("interface-window-xcor")) {
		String value = items.get(i+1).toString();
		//if (Integer.parseInt(value) < 800) {new_iface_xcor = Integer.parseInt(value);}
		if (Integer.parseInt(value) < Toolkit.getDefaultToolkit().getScreenSize().width)
			{new_iface_xcor = Integer.parseInt(value);}
		else {new_iface_xcor = 100;}
		i += 2;
		continue;
	    }
	    if (item.equals("interface-window-ycor")) {
		String value = items.get(i+1).toString();
		//if (Integer.parseInt(value) < 600) {new_iface_ycor = Integer.parseInt(value);}
		if (Integer.parseInt(value) < Toolkit.getDefaultToolkit().getScreenSize().height)					{new_iface_ycor = Integer.parseInt(value);}
		else {new_iface_ycor = 0;}
		i += 2;
		continue;
	    }
	    if (item.equals("interface-window-size")) {
		int reqwidth = Integer.parseInt(items.get(i+1).toString());
		int effectivewidth = reqwidth + 50; // toolbar
		int reqheight = Integer.parseInt(items.get(i+2).toString());
		int effectiveheight = reqheight + 80; // toolbar, statusbar
		java.awt.Dimension size = Toolkit.getDefaultToolkit().getScreenSize();
		int fudge = 10; // fudge factor for window borders, start menu bars, etc
		if (effectivewidth > size.width) {
		    reqwidth -= effectivewidth - (size.width - 2 * fudge);
		    new_iface_xcor = fudge;
		} else if (effectivewidth + new_iface_xcor > size.width)
		    new_iface_xcor = (size.width - effectivewidth) / 2;
		if (effectiveheight > size.height) {
		    reqheight -= effectiveheight - (size.height - 2 * fudge);
		    new_iface_ycor = fudge;
		} else if (effectiveheight + new_iface_ycor > size.height)
		    new_iface_ycor = (size.height - effectiveheight) / 2;
		app.iface.setVisibleSize(reqwidth, reqheight);
		//app.iface.statusbar.updatePosition();
		//System.out.println("requested height= " + reqheight);
		i += 3;
		continue;
	    }
	    if (item.equals("output-window-xcor")) {
		String value = items.get(i+1).toString();
		//if (Integer.parseInt(value) < 800) {new_out_xcor = Integer.parseInt(value);}
		if (Integer.parseInt(value) < Toolkit.getDefaultToolkit().getScreenSize().width)
			{new_out_xcor = Integer.parseInt(value);}
		else {new_out_xcor = 0;}
		i += 2;
		continue;
	    }
	    if (item.equals("output-window-ycor")) {
		String value = items.get(i+1).toString();
		//if (Integer.parseInt(value) < 600) {new_out_ycor = Integer.parseInt(value);}
		if (Integer.parseInt(value) < Toolkit.getDefaultToolkit().getScreenSize().height)					{new_out_ycor = Integer.parseInt(value);}
		else {new_out_ycor = 0;}
		i += 2;
		continue;
	    }
	    if (item.equals("output-window-width")) {
		String value = items.get(i+1).toString();
		new_out_width = Integer.parseInt(value);
		i += 2;
		continue;
	    }
	    if (item.equals("output-window-height")) {
		String value = items.get(i+1).toString();
		new_out_height = Integer.parseInt(value);
		i += 2;
		continue;
	    }
	    if (item.equals("info-window-xcor")) {
		String value = items.get(i+1).toString();
		//if (Integer.parseInt(value) < 800) {new_info_xcor = Integer.parseInt(value);}
		if (Integer.parseInt(value) < Toolkit.getDefaultToolkit().getScreenSize().width)
			{new_info_xcor = Integer.parseInt(value);}
		else {new_info_xcor = 0;}
		i += 2;
		continue;
	    }
	    if (item.equals("info-window-ycor")) {
		String value = items.get(i+1).toString();
		//if (Integer.parseInt(value) < 600) {new_info_ycor = Integer.parseInt(value);}
		if (Integer.parseInt(value) < Toolkit.getDefaultToolkit().getScreenSize().height)					{new_info_ycor = Integer.parseInt(value);}
		else {new_info_ycor = 0;}
		i += 2;
		continue;
	    }
	    if (item.equals("info-window-width")) {
		String value = items.get(i+1).toString();
		new_info_width = Integer.parseInt(value);
		i += 2;
		continue;
	    }
	    if (item.equals("info-window-height")) {
		String value = items.get(i+1).toString();
		new_info_height = Integer.parseInt(value);
		i += 2;
		continue;
	    }
	    if (item.equals("control-center-xcor")) {
		String value = items.get(i+1).toString();
		//if (Integer.parseInt(value) < 800) {new_cc_xcor = Integer.parseInt(value);}
		if (Integer.parseInt(value) < Toolkit.getDefaultToolkit().getScreenSize().width)
			{new_cc_xcor = Integer.parseInt(value);}
		else {new_cc_xcor = 0;}
		i += 2;
		continue;
	    }
	    if (item.equals("control-center-ycor")) {
		String value = items.get(i+1).toString();
		//if (Integer.parseInt(value) < 600) {new_cc_ycor = Integer.parseInt(value);}
		if (Integer.parseInt(value) < Toolkit.getDefaultToolkit().getScreenSize().height)					{new_cc_ycor = Integer.parseInt(value);}
		else {new_cc_ycor = 100;}
		i += 2;
		continue;
	    }
	    if (item.equals("control-center-width")) {
		String value = items.get(i+1).toString();
		new_cc_width = Integer.parseInt(value);
		i += 2;
		continue;
	    }
	    if (item.equals("control-center-height")) {
		String value = items.get(i+1).toString();
		new_cc_height = Integer.parseInt(value);
		i += 2;
		continue;
	    }
	    if (item.equals("shape-chooser-xcor")) {
		String value = items.get(i+1).toString();
		if (Integer.parseInt(value) < Toolkit.getDefaultToolkit().getScreenSize().width)
			{new_shape_xcor = Integer.parseInt(value);}
		else {new_shape_xcor = 0;}
		i += 2;
		continue;
	    }
	    if (item.equals("shape-chooser-ycor")) {
		String value = items.get(i+1).toString();
		if (Integer.parseInt(value) < Toolkit.getDefaultToolkit().getScreenSize().height)					{new_shape_ycor = Integer.parseInt(value);}
		else {new_shape_ycor = 0;}
		i += 2;
		continue;
	    }
	    if (item.equals("shape-chooser-width")) {
		String value = items.get(i+1).toString();
		new_shape_width = Integer.parseInt(value);
		i += 2;
		continue;
	    }
	    if (item.equals("shape-chooser-height")) {
		String value = items.get(i+1).toString();
		new_shape_height = Integer.parseInt(value);
		i += 2;
		continue;
	    }
	    if (item.equals("turtle-command-center-height")) {
		String value = items.get(i+1).toString();
		new_tur_cc_height = Integer.parseInt(value);
		i += 2;
		continue;
	    }
	    if (item.equals("observer-command-center-height")) {
		String value = items.get(i+1).toString();
		new_obs_cc_height = Integer.parseInt(value);
		i += 2;
		continue;
	    }
	    if (item.equals("plot-window-xcor")) {
		String value = items.get(i+1).toString();
		//if (Integer.parseInt(value) < 800) {new_plot_xcor = Integer.parseInt(value);}
		if (Integer.parseInt(value) < Toolkit.getDefaultToolkit().getScreenSize().width)
			{new_plot_xcor = Integer.parseInt(value);}
		else {new_plot_xcor = 0;}
		i += 2;
		continue;
	    }
	    if (item.equals("plot-window-ycor")) {
		String value = items.get(i+1).toString();
		//if (Integer.parseInt(value) < 600) {new_plot_ycor = Integer.parseInt(value);}
		if (Integer.parseInt(value) < Toolkit.getDefaultToolkit().getScreenSize().height)					{new_plot_ycor = Integer.parseInt(value);}
		else {new_plot_ycor = 0;}
		i += 2;
		continue;
	    }
	    if (item.equals("plot-window-width")) {
		String value = items.get(i+1).toString();
		new_plot_width = Integer.parseInt(value);
		i += 2;
		continue;
	    }
	    if (item.equals("plot-window-height")) {
		String value = items.get(i+1).toString();
		new_plot_height = Integer.parseInt(value);
		i += 2;
		continue;
	    }
		 if (item.equals("interface-locked?")) {
		String value = items.get(i+1).toString();
		if (!app.kioskp && YoYo.applet == null)
		{app.iface.frame.getMenu().setLockState(value.equals("true"));
		app.locked = value.equals("true");}
		i += 2;
		continue;
		}
	    i += 2;
	}
	if (!htmlp) {
	    // on Macintosh, make sure ycor is at least 20 to account for menu bar height
	    if (YoYo.macintoshp) {
		if (new_iface_ycor < 20) new_iface_ycor = 20;
		if (new_out_ycor < 20) new_out_ycor = 20;
		if (new_info_ycor < 20) new_info_ycor = 20;
		if (new_cc_ycor < 20) new_cc_ycor = 20;
		if (new_shape_ycor < 20) new_shape_ycor = 20;
	    }
	    if (YoYo.applet == null) {
		//if (app.kioskp) {
		//   app.iface.frame().setLocation(0, 0);
		//} else {
		    app.iface.frame().setLocation(new_iface_xcor, new_iface_ycor);
		    //}
	    }
	    app.outWin.setBounds(new_out_xcor, new_out_ycor, new_out_width, new_out_height);
	    //app.plotWin.setBounds(new_plot_xcor, new_plot_ycor, new_plot_width, new_plot_height);
	    app.infoWin.setBounds(new_info_xcor, new_info_ycor, new_info_width, new_info_height);
	    app.procWin.setBounds(new_cc_xcor, new_cc_ycor, new_cc_width, new_cc_height);
	    app.shapeChooser.setBounds(new_shape_xcor, new_shape_ycor, new_shape_width, new_shape_height);
	    // this call to doResize() is necessary to ensure that the contents
	    // of the control center have been re-arranged following the
	    // setBounds() call above.  The event notice from AWT is late to
	    // arrive, and we need instant resizing so that we can drag the
	    // command center / procedure window boundary to the right spot
	    // in <setCCHeight> below.
	    //app.procWin.doResize();
	    app.procWin.setCCHeight(ControlCenter.tur, new_tur_cc_height);
	    app.procWin.setCCHeight(ControlCenter.obs, new_obs_cc_height);
	}
	if (new_bg_image_name.equals("")) {
	    app.iface.setBackgroundImage(null, new_bg_image_name);
	} else {
	    // try to load the image from the project directory
	    System.out.println("curdir = " + Project.curDir);
	    SLURL imageFileName = new SLURL(Project.curDir, new_bg_image_name);
	    System.out.println("image file = " + imageFileName);
	    Image image = Toolkit.getDefaultToolkit().createImage(imageFileName.loadBinary(c));
	    if (image != null) {
		app.iface.setBackgroundImage(image, new_bg_image_name);
	    } else {
		app.iface.setBackgroundImage(null, "");
	    }
	}
	resizePatches(new_halfwidth, new_halfheight, new_cellsize);
    }

    // if dimensions are different than current dimensions of patchcanvas, then change
    // the dimension settings and resize the patchcanvas
    public boolean resizePatches(int new_halfwidth, int new_halfheight, int new_cellsize) {
	return resizePatches(new_halfwidth, new_halfheight, new_cellsize,
			     patchcanvas.getPosition().x,
			     patchcanvas.getPosition().y,
			     JInterface.RESIZEBOTRIGHT);
    }

    public void switchPatches() {
	//System.out.println("Switch patches");
	PatchCanvasInterface oldpc = patchcanvas;
	setPatchCanvas((PatchCanvasInterface)oldpc);
	//System.out.println("Switched patches to " + patchcanvas);
	if (!app.iface.replacePatchCanvas(oldpc, patchcanvas)) {
	    altgraphics = (altgraphics + 1) % NUMPCTYPES;
	    setPatchCanvas((PatchCanvasInterface)oldpc);
	    boolean ret = app.iface.replacePatchCanvas(oldpc, patchcanvas);
	    assert(ret);
	}

    }

    public boolean resizePatches(int new_halfwidth, int new_halfheight, int new_cellsize,
				 int x, int y, int anchorCorner) {
	//System.out.println("top of more args version of resizePatches");
	if (patchcanvas instanceof PatchCanvas) {
	    if (halfwidth!=new_halfwidth || halfheight!=new_halfheight || cellsize != new_cellsize) {
		synchronized(runningp) {	
		    if (halfwidth != new_halfwidth || halfheight != new_halfheight) {
		    boolean biggerx = new_halfwidth > halfwidth;
		    boolean biggery = new_halfheight > halfheight;
		    halfwidth = new_halfwidth;
		    halfheight = new_halfheight;
		    synchronized(patchcanvas.getGraphicsLock()) {
			((PatchCanvas)patchcanvas).setDrawingSurface();
			createPatches(anchorCorner, biggerx, biggery);
			PatchCanvas oldpc = (PatchCanvas)patchcanvas;
			setPatchCanvas(oldpc);
			if (!app.iface.replacePatchCanvas(oldpc, patchcanvas, x, y)) {
			    altgraphics = (altgraphics + 1) % NUMPCTYPES;
			    setPatchCanvas(oldpc);
			    boolean ret = app.iface.replacePatchCanvas(oldpc, patchcanvas, x, y);
			    assert(ret);
			}
			oldpc.deleteDrawingSurface();
		    }
		}
	    
		if (cellsize != new_cellsize) {
		    if (new_cellsize > 8 && new_cellsize < 16) {
			new_cellsize = 8;
		    } else if (new_cellsize > 16 && new_cellsize < 32) {
			new_cellsize = 16;
		    } else if (new_cellsize > 32) {
			new_cellsize = 32;
		    }
		    cellsize = new_cellsize;
		    PatchCanvas oldpc = (PatchCanvas)patchcanvas;
		    setPatchCanvas(oldpc);
		    if (!app.iface.replacePatchCanvas(oldpc, patchcanvas, x, y)) {
			altgraphics = (altgraphics + 1) % NUMPCTYPES;
			setPatchCanvas(oldpc);
			boolean ret = app.iface.replacePatchCanvas(oldpc, patchcanvas, x, y);
			assert(ret);
		    }
		    app.iface.setCellScroll(cellsize);
		}
	    }
	    //System.out.println("End of more args version of resizePatches - return true");
	    return true;
	}
	//System.out.println("End of more args version of resizePatches - return false");
	    return false;
	} else return false;
    }
    
    public void resetPatches() {
	list.deletePatches();
	resizePatches(StarLogo.DEFAULT_HALFHEIGHT,
		      StarLogo.DEFAULT_HALFWIDTH,
		      StarLogo.DEFAULT_CELLSIZE);
    }


    private void createPatches(int anchorCorner, boolean biggerx, boolean biggery) {
	int x, y;
	int oldwidthp2 = widthp2; int oldheightp2 = heightp2;
	Patch[] oldpatches = patches;
	this.height = halfheight * 2 + 1;
	this.width = halfwidth * 2 + 1;
	this.widthp2 = this.width + 2;
	this.heightp2 = this.height + 2;
	this.turhalfheight = height / 2.0;
	this.turhalfwidth = width / 2.0;

	try {
	    patches = new Patch[widthp2*heightp2];

	    int ydiff = heightp2 - oldheightp2;
	    int xdiff = widthp2 - oldwidthp2;
	    if (ydiff < 0) ydiff = -ydiff;
	    if (xdiff < 0) xdiff = -xdiff;

	    int xstart, ystart, xend, yend, xoff, yoff;
	    switch(anchorCorner) {
	    case JInterface.RESIZEBOTRIGHT:
		if (biggerx) {
		    xend = oldwidthp2-1;
		    makePatchColumn(1, oldwidthp2-1, heightp2-1, widthp2-1, patches, widthp2);
		} else {
		    xend = widthp2-1;
		    killTurtleColumn(1, widthp2-1, oldheightp2-1, oldwidthp2-1, oldpatches, oldwidthp2);
		}
		if (biggery) {
		    yend = oldheightp2-1;
		    makePatchColumn(oldheightp2-1, 1, heightp2-1, xend, patches, widthp2);
		} else {
		    yend = heightp2-1;
		    killTurtleColumn(heightp2-1, 1, oldheightp2-1, oldwidthp2-1, oldpatches, oldwidthp2);
		}
		unlinkAll(oldpatches, oldwidthp2);
		copyPatches(1, 1, yend, xend, oldpatches, patches, 0, 0, oldwidthp2, widthp2);
		linkAll(0, 0);
		break;
	    case JInterface.RESIZEBOTLEFT:
		if (biggerx) {
		    xstart = 1; xoff = xdiff;
		    makePatchColumn(1, 1, heightp2-1, xdiff+1, patches, widthp2);
		} else {
		    xstart = xdiff + 1; xoff = -xdiff;
		    killTurtleColumn(1, 1, oldheightp2-1, xdiff, oldpatches, oldwidthp2);
		}
		if (biggery) {
		    yend = oldheightp2-1;
		    makePatchColumn(oldheightp2-1, (biggerx) ? xdiff+1 : 1, heightp2-1, widthp2-1,
				    patches, widthp2);
		} else {
		    yend = heightp2-1;
		    killTurtleColumn(heightp2-1, 1, oldheightp2-1, oldwidthp2-1, oldpatches, oldwidthp2);
		}
		unlinkAll(oldpatches, oldwidthp2);
		copyPatches(1, xstart, yend, oldwidthp2-1, oldpatches, patches,
			    0, xoff, oldwidthp2, widthp2);
		linkAll(xoff, 0);
		break;
	    case JInterface.RESIZETOPLEFT:
		if (biggerx) {
		    xstart = 1; xoff = xdiff;
		    makePatchColumn(1, 1, heightp2-1, xdiff+1, patches, widthp2);
		} else {
		    xstart = xdiff+1; xoff = -xdiff;
		    killTurtleColumn(1, 1, oldheightp2-1, xdiff, oldpatches, oldwidthp2);
		}
		if (biggery) {
		    ystart = 1; yoff = ydiff;
		    makePatchColumn(1, (biggerx) ? xdiff+1 : 1, ydiff + 1, widthp2-1, patches, widthp2);
		} else {
		    ystart = ydiff+1; yoff = -ydiff;
		    killTurtleColumn(1, 1, ydiff, oldwidthp2-1, oldpatches, oldwidthp2);
		}
		unlinkAll(oldpatches, oldwidthp2);
		copyPatches(ystart, xstart, oldheightp2-1, oldwidthp2-1, oldpatches, patches,
			    yoff, xoff, oldwidthp2, widthp2);
		linkAll(xoff, yoff);
		break;
	    case JInterface.RESIZETOPRIGHT:
		if (biggerx) {
		    xend = oldwidthp2-1;
		    makePatchColumn(1, oldwidthp2-1, heightp2-1, widthp2-1, patches, widthp2);
		}
		else {
		    xend = widthp2-1;
		    killTurtleColumn(1, widthp2-1, oldheightp2-1, oldwidthp2-1, oldpatches, oldwidthp2);
		}
		if (biggery) {
		    ystart = 1; yoff = ydiff;
		    makePatchColumn(1, 1, ydiff+1, (biggerx) ? oldwidthp2-1 : widthp2-1,
				    patches, widthp2);
		}
		else {
		    ystart = ydiff+1; yoff = -ydiff;
		    killTurtleColumn(1, 1, ydiff, oldwidthp2-1, oldpatches, oldwidthp2);
		}

		unlinkAll(oldpatches, oldwidthp2);
		copyPatches(ystart, 1, oldheightp2-1, xend, oldpatches, patches,
			    yoff, 0, oldwidthp2, widthp2);
		linkAll(0, yoff);
		break;
	    default:
		app.err.println("uh oh, unknown resize anchor corner in resizing patch canvas");
	    }

	    for(y = 0; y < heightp2; y++) {
		patches[y * widthp2] = new Patch(0, y, this);
		patches[(y+1) * widthp2 - 1] = new Patch(widthp2 - 1, y, this);
	    }
	    for(x = 1; x < widthp2-1; x++) {
		patches[x] = new Patch(x, 0, this);
		patches[widthp2 * (heightp2-1) + x] = new Patch(x, heightp2 - 1, this);
	    }
	}
	catch (OutOfMemoryError e) {
	    patches = oldpatches;
	    throw new LogoError(app.textBundle.getString("fatal_error") + ": " +
				app.textBundle.getString("error_pc_resize_no_memory") + " " +
				app.textBundle.getString("error_quit_now"));
	}
	//Patch.setVars(pown, this, c);

    }

    void copyPatches(int ystart, int xstart, int yend, int xend,
		     Patch[] oldpatches, Patch[] newpatches,
		     int yoffset, int xoffset,
		     int oldwidthp2, int newwidthp2) {
	for (int y = ystart; y < yend; y++) {
	    for (int x = xstart; x < xend; x++) {
		Patch oldpatch = oldpatches[y * oldwidthp2 + x];
		newpatches[(y + yoffset) * newwidthp2 + (x + xoffset)] = oldpatch;
		oldpatch.xcor = x + xoffset;
		oldpatch.ycor = y + yoffset;
	    }
	}
    }

    void makePatchColumn(int ystart, int xstart, int yend, int xend,
			 Patch[] patches, int widthp2) {
	for (int y = ystart; y < yend; y++) {
	    for(int x = xstart; x < xend; x++) {
		patches[y * widthp2 + x] = new Patch(x, y, this);
	    }
	}
    }

    void killTurtleColumn(int ystart, int xstart, int yend, int xend,
			  Patch[] patches, int widthp2) {
	for(Turtle t = firstturtle; t != null; t=t.next) {
	    if ((int)t.xcor >= (xstart-1) && (int)t.xcor <= (xend-1) &&
		(int)t.ycor >= (ystart-1) && (int)t.ycor <= (yend-1)) {
		unlinkDuringResize(t, patches, widthp2);
		die(t);
	    }
	}
    }

    //  public String saveWorld(boolean htmlp) { return "error";}

    //comment and replace this for testing

    public String saveWorld(boolean htmlp) throws BadWorldException {

	ByteArrayOutputStream obs, tur, pat, byt = null, str, sym, doub, list;
	byte[] obscomp, turcomp, patcomp, bytcomp = null, stringcomp, doublecomp, symbolcomp, listcomp;
	String obschars, patchars, turchars, bytchars = "", stringchars, doublechars, symbolchars, listchars;
	byte[] turbytes, obsbytes, patbytes;
	byte[] bytbytes = null, stringbytes, doublebytes, symbolbytes, listbytes;

	try {
	    DataOutputStream oos = new DataOutputStream(obs = new ByteArrayOutputStream());
	    saveObserver(oos);
	    oos.close();
	    obsbytes = obs.toByteArray();

	    DataOutputStream oos1 = new DataOutputStream(pat = new ByteArrayOutputStream());
	    savePatches(oos1);
	    oos1.close();
	    patbytes = pat.toByteArray();

	    DataOutputStream oos2 = new DataOutputStream(tur = new ByteArrayOutputStream());
	    saveTurtles(oos2);
	    oos2.close();
	    turbytes = tur.toByteArray();
	    //printBytes(turbytes);

	    if (htmlp) {
		DataOutputStream oos7 = new DataOutputStream(byt = new ByteArrayOutputStream());
		saveByteCodes(oos7);
		oos7.close();
		bytbytes = byt.toByteArray();
	    }

	    DataOutputStream oos3 = new DataOutputStream(str = new ByteArrayOutputStream());
	    Utilities.dumpStringTable(oos3);
	    oos3.close();
	    stringbytes = str.toByteArray();

	    DataOutputStream oos4 = new DataOutputStream(sym = new ByteArrayOutputStream());
	    Utilities.dumpSymbolTable(oos4);
	    oos4.close();
	    symbolbytes = sym.toByteArray();

	    DataOutputStream oos5 = new DataOutputStream(doub = new ByteArrayOutputStream());
	    Utilities.dumpDoubleTable(oos5);
	    oos5.close();
	    doublebytes = doub.toByteArray();

	    DataOutputStream oos6 = new DataOutputStream(list = new ByteArrayOutputStream());
	    Utilities.dumpListTable(oos6);
	    oos6.close();
	    listbytes = list.toByteArray();


	    obscomp = Utilities.gzipByteArray(obsbytes);
	    patcomp = Utilities.gzipByteArray(patbytes);
	    turcomp = Utilities.gzipByteArray(turbytes);
	    if (htmlp) bytcomp = Utilities.gzipByteArray(bytbytes);
	    stringcomp = Utilities.gzipByteArray(stringbytes);
	    symbolcomp = Utilities.gzipByteArray(symbolbytes);
	    doublecomp = Utilities.gzipByteArray(doublebytes);
	    listcomp = Utilities.gzipByteArray(listbytes);

	    obschars = yoyo.Base64.encode(obscomp);
	    patchars = yoyo.Base64.encode(patcomp);
	    turchars = yoyo.Base64.encode(turcomp);
	    if (htmlp) bytchars = yoyo.Base64.encode(bytcomp);
	    stringchars = yoyo.Base64.encode(stringcomp);
	    symbolchars = yoyo.Base64.encode(symbolcomp);
	    doublechars = yoyo.Base64.encode(doublecomp);
	    listchars = yoyo.Base64.encode(listcomp);

	    StringBuffer output = new StringBuffer();
	    output.append("`string table`\r\n");
	    output.append(stringchars);
	    output.append("\r\n`symbol table`\r\n");
	    output.append(symbolchars);
	    output.append("\r\n`double table`\r\n");
	    output.append(doublechars);
	    output.append("\r\n`list table`\r\n");
	    output.append(listchars);
	    if (htmlp) {
		output.append("\r\n`bytecodes`\r\n");
		output.append(bytchars);
	    }
	    output.append("\r\n`observer world`\r\n");
	    output.append(obschars);
	    //System.out.println("output is " + output);
	    output.append("\r\n`patch world`\r\n");
	    output.append(patchars);
	    //System.out.println("output is " + output);
	    output.append("\r\n`turtle world`\r\n");
	    output.append(turchars);
	    //System.out.println("output is " + output);

	    return output.toString();
	}
	catch (RuntimeException g) {
	    g.printStackTrace();
	    throw new BadWorldException(); 
	}
	catch (IOException e) { 
	    e.printStackTrace(); 
	}
	catch (Exception f) { 
	    f.printStackTrace(); 
	}
	finally {
	    Utilities.clearTable();
	}
	return null;
    }


    public void restoreTables(String str, Context c) {
	extractPeace(str, "`string table`", c);
	extractPeace(str, "`symbol table`", c);
	extractPeace(str, "`double table`", c);
	extractPeace(str, "`list table`", c);
    }

    public void restoreWorld(String str, Context c) {
	int start, end; String title;

	try {
	    if (!extractPeace(str, "`observer world`", c)) {
		list.deleteObserver();
		observer = new Observer(this);
		list.create(observer);
		Observer.setVars(new Object[0], this, c);
	    }

	    if (!extractPeace(str, "`patch world`", c)) {
		list.deletePatches();
		createPatches(JInterface.RESIZEBOTLEFT, false, false);
		CG();
		Object[] pown = new Object[0];
		Patch.setVars(pown, this, c);
	    }

	    if (!extractPeace(str, "`turtle world`", c)) {
		killAll();
		Turtle.setVars(new Object[0], this, c);
		Turtle.setBreeds(new Object[0], c);
	    }
	    //System.out.println("finished loading worlds. about to load bytecodes");
	    if (app.kioskp || YoYo.applet != null) extractPeace(str, "`bytecodes`", c);
	    //System.out.println("finished loading bytecodes");
	}
	finally {
	    Utilities.clearTable();
	    VM.initStateOffsets();
	    //System.out.println("done loading bytecodes");
	}
    }

    public boolean extractPeace(String str, String title, Context c) {
	int start = str.indexOf(title);
	if (start >= 0) {
	    int end = str.indexOf("`", start+title.length());
	    if (end == -1) end = str.length();
	    String peace = str.substring(start+title.length(), end).trim()+"\r\n";
	    try {
		byte[] bytes = Utilities.gunzipByteArray(yoyo.Base64.decode(peace.getBytes()));
		if (bytes == null) return false;
		try {
		    DataInputStream ois = new DataInputStream(new ByteArrayInputStream(bytes));
		    if (title.equals("`observer world`")) restoreObserver(ois, c);
		    else if (title.equals("`patch world`")) restorePatches(ois, c);
		    else if (title.equals("`turtle world`")) restoreTurtles(ois, c);
		    else if (title.equals("`bytecodes`")) restoreByteCodes(ois, c);
		    else if (title.equals("`symbol table`")) Utilities.loadSymbolTable(ois);
		    else if (title.equals("`string table`")) Utilities.loadStringTable(ois);
		    else if (title.equals("`double table`")) Utilities.loadDoubleTable(ois);
		    else if (title.equals("`list table`")) Utilities.loadListTable(ois);
		}
		catch (StreamCorruptedException sce) { sce.printStackTrace(app.log); }
	    }
	    catch (IOException e) { e.printStackTrace(app.log); }
	    return true;
	} else return false;
    }


    public void saveByteCodes(DataOutputStream oos) throws IOException {
	oos.writeUTF("`bytecodes`");
	oos.writeDouble(2.1);
	oos.writeInt(nextinst); // number of instruction lists
	for(int i = 0; i < nextinst; i++) {
	    //System.out.println("save byte code " + i);
	    int[] iinst = iinstarray[i];
	    Object[] oinst = oinstarray[i];
	    oos.writeUTF(ninstarray[i]);
	    //System.out.println("named " + ninstarray[i]);
	    oos.writeInt(iinst.length);
	    for(int j = 0; j < iinst.length; j++) {
		oos.writeInt(iinst[j]);
	    }
	    oos.writeInt(Utilities.addIntoTable(oinst, this));
	}
    }

    public void restoreByteCodes(DataInputStream ois, Context c) throws IOException {
	//System.out.println("restore byte codes");

	String header = "";
	try {
	    header = ois.readUTF();
	}
	catch (UTFDataFormatException e) {
	    e.printStackTrace();
	    return;
	}
	if (!header.equals("`bytecodes`")) return;
	double version = ois.readDouble();
	if (version > 2.1 || version < 2.0) {
	    StarLogoError.error(app, app.textBundle.getString("error_loading_project") + ": " +
				app.textBundle.getString("error_cant_read_bytecodes") + ". " +
				app.textBundle.getString("error_version_unknown") + ": " +
				version, (app.kioskp || YoYo.applet != null));
	    return;
	}
	try {
	    int numInsts = ois.readInt();
	    //System.out.println("read instructions " + numInsts);
	    for(int i = 0; i < numInsts; i++) {
		String name = (version >= 2.1) ? ois.readUTF() : app.textBundle.getString("unknown_proc");
		//System.out.println("name = " + name);
		int[] iinst = new int[ois.readInt()];
		for(int j = 0; j < iinst.length; j++) {
		    iinst[j] = ois.readInt();
		}
		Object[] oinst = (Object[])Utilities.lookupInTable(ois.readInt());
		internalSetInstructions(name, iinst, oinst, i);
	    }
	    nextinst = numInsts;
	} catch (Exception e) {
	    e.printStackTrace(app.log);
	    StarLogoError.error(app, app.textBundle.getString("error_loading_project") + ": " +
				app.textBundle.getString("error_cant_read_bytecodes") + ".", true);
	    return;
	}

    }


    public void saveTurtles(DataOutputStream oos) throws IOException {
	// write out this information:
	// version of saveTurtles
	// number of turtles
	// number of variables
	// turtle variable names and position in turtle array
	// then call each turtle to save its state

	oos.writeUTF("`turtle state`");
	oos.writeDouble(2.0);

	byte[] state = Turtle.getStateDesc();
	oos.writeInt(state.length);
	oos.write(state);

	oos.writeInt(Turtle.numVars);
	for(int i =0; i< Turtle.numVars; i++) {
	    oos.writeUTF(Turtle.getVarName(i));
	    oos.writeInt(Turtle.getVarPos(i));
	}
	oos.writeInt(Turtle.numBreeds);
	for(int i =0; i< Turtle.numBreeds; i++) {
	    oos.writeUTF(Turtle.getBreedName(i));
	    oos.writeInt(Turtle.getBreedNum(i));
	}
	oos.writeInt(whop);
	for(int i = 0; i<whop+1; i++) {
	    oos.writeInt(whostack[i]);
	}
	oos.writeInt(numturtles);
//System.out.println("after writeint numturtles. numturtles = " + numturtles);
	int turlength = Turtle.getLength(), offset = 0;
//System.out.println("turlength = " + turlength);
	byte[] tur = new byte[numturtles * turlength];
	Turtle current = firstturtle;

	    while(current!=null) {
		current.save(tur, offset);
		current = current.next;
		offset += turlength;
	    }

	oos.write(tur);
    }

    /* Assumes world is stopped and all turtles are no
       longer interesting or viable. Must do this after
       restoring patches
    */
    public void restoreTurtles(DataInputStream ois, Context c) throws IOException  {
	String header = "";
	try {
	    header = ois.readUTF();
	}
	catch (UTFDataFormatException e) {
	    killAll();
	    return;
	}

	if (!header.equals("`turtle state`")) return;
	double version = ois.readDouble();
	if (version > 2.0 || version < 2.0) {
	    killAll();
	    StarLogoError.error(app, app.textBundle.getString("error_loading_project") + ": " +
				app.textBundle.getString("error_cant_read_turtles") + ". " +
				app.textBundle.getString("error_version_unknown") + ": " +
				version, false);
	    return;
	}

	killAll();

	int stateLength = ois.readInt();
	byte[] statebytes = new byte[stateLength];
	ois.read(statebytes);
	Turtle.setStateDesc(statebytes);

	try {
	    int numVars = ois.readInt();
	    Object[] town = new Object[numVars];
	    for(int i = 0; i<numVars; i++) {
		Object[] pair = new Object[3];
		pair[0] = ois.readUTF();
		pair[1] = new Integer(ois.readInt());
		pair[2] = YoYo.symfalse;
		town[i] = pair;
	    }
	    Turtle.setVars(town, this, c);

	    int numBreeds = ois.readInt();
	    Object[] breeds = new Object[numBreeds];
	    for(int i = 0; i< numBreeds; i++) {
		Object[] pair = new Object[2];
		pair[0] = ois.readUTF();
		pair[1] = new Integer(ois.readInt());
		breeds[i] = pair;
	    }
	    Turtle.setBreeds(breeds, c);
	}
	catch (Exception e) {
	    e.printStackTrace(app.log);
	    StarLogoError.error(app, app.textBundle.getString("error_loading_project") + ": " +
				app.textBundle.getString("error_cant_read_turvars") + ". " +
				app.textBundle.getString("continuing") + "...", false);
	    return;
	}

	try {
	    whop = ois.readInt();
	    if (whop > 4096) {
		whostack = new int[whop+1];
	    }
	    for(int i = 0; i< whop+1; i++) {
		whostack[i] = ois.readInt();
	    }
	    numturtles = ois.readInt();
	    int turlength = Turtle.getLength();
	    byte[] turbytes = new byte[numturtles * turlength];
	    ois.read(turbytes);
	    for(int i = 0, offset = 0; i<numturtles; i++, offset += turlength) {
		Turtle t = Turtle.restore(this, turbytes, offset);
		t.runstate = false;
		t.timer = 0;
		t.count = 0;
		if (firstturtle != null) {
		    t.next = firstturtle;
		    firstturtle.prev = t;
		}
		firstturtle = t;
		//boolean oldshownp = t.shownp;
		//t.shownp = false;
		wrapandlink(t);
		//t.shownp = oldshownp;
		list.create(t);
	    }
	}
	catch (Exception e) {
	    e.printStackTrace(app.log);
	    killAll();
	    whop = -1;
	    StarLogoError.error(app, app.textBundle.getString("error_loading_project") + ": " +
				app.textBundle.getString("error_cant_read_turtles") + ". " +
				app.textBundle.getString("continuing") + "...", false);
	    redraw();
	    return;
	};
	redraw();
    }

    public void savePatches(DataOutputStream oos) throws IOException {
	// write out this information:
	// version of savePatches
	// screen-height, screen-width
	// number of patch variables
	// patch variable names and position in patch array
	// then call each patch to save its state
	oos.writeUTF("`patch state`");
	oos.writeDouble(2.0);

	byte[] state = Patch.getStateDesc();
	oos.writeInt(state.length);
	oos.write(state);

	oos.writeInt(halfwidth);
	oos.writeInt(halfheight);
	oos.writeInt(Patch.numVars);
	for(int i =0; i< Patch.numVars; i++) {
	    oos.writeUTF(Patch.getVarName(i));
	    oos.writeInt(Patch.getVarPos(i));
	}

	int x, y, poffy, offset;
	int patchlength = Patch.getLength();
	byte[] pat = new byte[height * width * patchlength];
	for (y = 0, poffy=widthp2, offset = 0; y < height; y++, poffy+=widthp2) {
	    for (x = 0; x<width; x++, offset += patchlength) {
		int poffset = poffy + x+1;
		patches[poffset].save(pat, offset);
	    }
	}
	oos.write(pat);
    }

    public void restorePatches(DataInputStream ois, Context c) throws IOException {
	String header = "";
	try {
	    header = ois.readUTF();
	}
	catch (UTFDataFormatException e) {
	    list.deletePatches();
	    createPatches(JInterface.RESIZEBOTLEFT, false, false);
	    return;
	}

	if (!header.equals("`patch state`")) return;
	double version = ois.readDouble();
	if (version > 2.0 || version < 2.0) {
	    list.deletePatches();
	    createPatches(JInterface.RESIZEBOTLEFT, false, false);
	    StarLogoError.error(app, app.textBundle.getString("error_loading_project") + ": " +
				app.textBundle.getString("error_cant_read_patches") + ". " +
				app.textBundle.getString("error_version_unknown") + ": " +
				version, false);
	    return;
	}

	int stateLength = ois.readInt();
	byte[] statebytes = new byte[stateLength];
	ois.read(statebytes);
	Patch.setStateDesc(statebytes);

	halfwidth = ois.readInt();
	halfheight = ois.readInt();

	this.height = halfheight * 2 + 1;
	this.width = halfwidth * 2 + 1;
	this.widthp2 = this.width + 2;
	this.heightp2 = this.height + 2;
	this.turhalfheight = height / 2.0;
	this.turhalfwidth = width / 2.0;

	Patch[] oldpatches = patches;
	patches = null;

	int numVars = 0;
	Object[] pown = null;
	try {
	    numVars = ois.readInt();
	    //System.out.println("numVars: " + numVars);
	    pown = new Object[numVars];
	    for(int i = 0; i < numVars; i++) {
		Object[] pair = new Object[3];
		pair[0] = ois.readUTF();
		pair[1] = new Integer(ois.readInt());
		pair[2] = YoYo.symfalse;
		pown[i] = pair;
	    }
	    Patch.setVars(pown, this, c);
	    //System.out.println("pown = " + YoYo.printToString(pown));
	}
	catch (Exception e) {
	    e.printStackTrace(app.log);
	    StarLogoError.error(app, app.textBundle.getString("error_loading_project") + ": " +
				app.textBundle.getString("error_cant_read_patchvars") + ". " +
				app.textBundle.getString("continuing") + "...", false);
	    patches = oldpatches;
	    list.deletePatches();
	    createPatches(JInterface.RESIZEBOTLEFT, false, false);
	    return;
	}

	int x, y, poffy, offset;
	Patch[] newpatches = new Patch[widthp2*heightp2];
	int patchlength = Patch.getLength();
	try {
	    byte[] patbytes = new byte[width*height*patchlength];
	    ois.read(patbytes);
	    for (y = 0, poffy=widthp2, offset = 0; y < height; y++, poffy+=widthp2) {
		int poffset = poffy + 1;
		for (x = 0; x<width; x++, offset += patchlength) {
		    //System.out.println("restore patch: ("  + x + ", " + y + ")");
		    Patch p = Patch.restore(this, patbytes, offset);
		    newpatches[poffset++] = p;
		    p.runstate = false;
		    p.timer = 0;
		    p.count = 0;
		    list.create(p);
		}
	    }

	    /* Create border patches */
	    poffy = widthp2 * (heightp2-1);
	    for(x = 0; x<widthp2; x++) {
		newpatches[x] = new Patch(x, 0, this);
		newpatches[x + poffy] = new Patch(x, heightp2 - 1, this);
	    }
	    for(y = widthp2; y<(heightp2-1)*widthp2; y+=widthp2) {
		newpatches[y] = new Patch(0, y, this);
		newpatches[y + width + 1] = new Patch(widthp2 - 1, y, this);
	    }

	    patches = newpatches;
	}
	catch (Exception e) {
	    e.printStackTrace(app.log);
	    patches = oldpatches;
	    list.deletePatches();
	    createPatches(JInterface.RESIZEBOTLEFT, false, false);
	    StarLogoError.error(app, app.textBundle.getString("error_loading_project") + ": " +
				app.textBundle.getString("error_cant_read_patches") + ". " +
				app.textBundle.getString("continuing") + "...", false);
	};

	PatchCanvasInterface oldpc = patchcanvas;

       	if (serverside)
	    setPatchCanvas(((NetworkPatchCanvas)oldpc).socket);
	else {
	    setPatchCanvas((Socket)null);
	}
	//patchcanvas.CORNER_WIDTH = (cellsize > 5) ? 8 : 5;
	if (!app.iface.replacePatchCanvas(oldpc, patchcanvas)) {
	    altgraphics = (altgraphics + 1) % NUMPCTYPES;
	    if (serverside)
		setPatchCanvas(((NetworkPatchCanvas)oldpc).socket);
	    else {
		setPatchCanvas((Socket)null);
	    }
	    boolean ret = app.iface.replacePatchCanvas(oldpc, patchcanvas);
	    assert(ret);
	}
    }


    public void saveObserver(DataOutputStream oos) throws IOException {
	// write out this information:
	// version of saveObserver
	// number of observer variables
	// observer variable names and position in observer array
	// then call observer to save its state

	oos.writeUTF("`observer state`");
	oos.writeDouble(2.0);

	byte[] state = Observer.getStateDesc();
	oos.writeInt(state.length);
	oos.write(state);

	oos.writeInt(Observer.numVars);
	for(int i =0; i< Observer.numVars; i++) {
	    oos.writeUTF(Observer.getVarName(i));
	    oos.writeInt(Observer.getVarPos(i));
	}
	byte[] obs = new byte[Observer.getLength()];
	observer.save(obs, 0);
	oos.write(obs);
    }

    public void restoreObserver(DataInputStream ois, Context c) throws IOException {
	String header = "";
	try {
	    header = ois.readUTF();
	}
	catch (UTFDataFormatException e) { return; }
	if (!header.equals("`observer state`")) return;
	double version = ois.readDouble();
	if (version > 2.0 || version < 2.0) {
	    list.deleteObserver();
	    observer = new Observer(this);
	    list.create(observer);
	    StarLogoError.error(app, app.textBundle.getString("error_loading_project") + ": " +
				app.textBundle.getString("error_cant_read_observer") + ". " +
				app.textBundle.getString("error_version_unknown") + ": " +
				version, false);
	    return;
	}

	int stateLength = ois.readInt();
	byte[] statebytes = new byte[stateLength];
	ois.read(statebytes);
	Observer.setStateDesc(statebytes);

	try {
	    int numVars = ois.readInt();
	    Object[] town = new Object[numVars];
	    for(int i = 0; i< numVars; i++) {
		Object[] pair = new Object[3];
		pair[0] = ois.readUTF();
		pair[1] = new Integer(ois.readInt());
		pair[2] = YoYo.symfalse;
		town[i] = pair;
	    }
	    Observer.setVars(town, this, c);
	}
	catch (Exception e) {
	    e.printStackTrace(app.log);
	    StarLogoError.error(app, app.textBundle.getString("error_loading_project") + ": " +
				app.textBundle.getString("error_cant_read_globalvars") + ". " +
				app.textBundle.getString("continuing") + "...", false);
	    list.deleteObserver();
	    observer = new Observer(this);
	    list.create(observer);
	    return;
	}

	list.remove(observer);
	try {
	    byte[] obs = new byte[Observer.getLength()];
	    ois.read(obs);
	    observer = Observer.restore(this, obs, 0);
	    observer.runstate = false;
	    observer.timer = 0;
	    observer.count = 0;
	    list.create(observer);
	}
	catch (Exception e) {
	    e.printStackTrace(app.log);
	    list.deleteObserver();
	    observer = new Observer(this);
	    list.create(observer);
	    StarLogoError.error(app, app.textBundle.getString("error_loading_project") + ": " +
				app.textBundle.getString("error_cant_read_observer") + ". " +
				app.textBundle.getString("continuing") + "...", false);
	};

    }
}


