/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import yoyo.YoYo;
import java.awt.*;
import javax.swing.UIManager;

public class StatusToolbar extends Widget {
    private JInterface iface; 
    public Font font;
	static final int MESSAGE_WIDTH = 300;

  FontMetrics fm;
    String statusMessage;
  
    StopWidget stopwidget;
    TrashWidget trashwidget;
  
    // these are the coordinates of the pointer
    Integer xcor;
    Integer ycor;
    //Thread t;
 
    private ToolTipWidget coordinateToolTip, statusToolTip;

    public StatusToolbar(JInterface iface) {
	super(iface);
	this.iface = iface;
	//System.out.println("create toolbar at " + position);
	updatePosition();
	this.font = WIDGET_FONT;
	fm = Toolkit.getDefaultToolkit().getFontMetrics(font);
	xcor = null;
	ycor = null;
	statusMessage = "";
	coordinateToolTip = new ToolTipWidget(iface, iface.app().textBundle.getString("coordinate_tooltip"));
	statusToolTip = new ToolTipWidget(iface, iface.app().textBundle.getString("status_tooltip"));
    }

    public void updatePosition() { 
	//System.out.println("update status bar position from " + iface.getSize().height);
	position = new Rectangle(0, iface.getSize().height - iface.statusbarheight(), 
				 iface.getSize().width, iface.statusbarheight() );
	//System.out.println("update status bar position to " + position);
    }

    public void paint(Graphics g) {
  	// correct for a window resize
  	//setLocation(0,iface.getSize().height-iface.statusbarheight());
  	
	// restore these when done painting
	g.translate(position.x, position.y);
	Color oldColor = g.getColor();
	Font oldFont = g.getFont();
    
	// draw the status bar
	g.setColor(SystemColor.control);
	g.fillRect(0, 0,iface.getSize().width,iface.statusbarheight());
    
	// draw the windows for the coordinates
	//g.setColor(Color.white);    
	g.draw3DRect(17,5,28,16,false);
	g.draw3DRect(62,5,28,16,false);
	g.draw3DRect(95,5,MESSAGE_WIDTH,16,false);
    
	// write some text
	g.setColor(Color.black);
	g.setFont(font);
	g.drawString("X:",5,17);
	g.drawString("Y:",50,17);
	g.drawString(statusMessage,100,17);
    
	// write the coordinates of the pointer if they are nonnull
	if ((xcor != null) && (ycor != null))
	    {
		g.drawString(xcor.toString(),22,17);  
		g.drawString(ycor.toString(),67,17);
	    }
    
	// restore old stuff
	g.translate(-position.x, -position.y);
	g.setColor(oldColor);
	g.setFont(oldFont);
    
	//stopwidget.paint(g);
    }
  
  
  
    public void setStatusMessage(String s) {
			if (s == null) s = "";
			while (fm.stringWidth(s) > MESSAGE_WIDTH) s = s.substring(0, s.length()-1);
			statusMessage = s;
			iface.addToClipRect(position);
			//setStatusMessage(s, 2000);
    }
  
    public void setLocation(int x, int y) {
  	super.setLocation(x,y);
    }
  

    public void displayToolTip(Point point) {
		
	if (trashwidget.position.contains(point)) {
	    trashwidget.displayToolTip(point);
	    return;
	}
	if (stopwidget.position.contains(point)) {
	    stopwidget.displayToolTip(point);
	    return;
	}
	point.translate(-position.x, -position.y);
	if (point.x < 95) {
	    coordinateToolTip.fixPosition(position.x + 5, position.y - coordinateToolTip.position.height - 2);
	    setToolTip(coordinateToolTip);
	    super.displayToolTip(point);
	    return;
	}
	if ((point.x > 95) && (point.x < 400)) {
	    statusToolTip.fixPosition(position.x + 100, position.y - statusToolTip.position.height - 2);
	    setToolTip(statusToolTip);
	    super.displayToolTip(point);
	    return;
	}
    }

    public void dropToolTip() {
	trashwidget.dropToolTip();
	stopwidget.dropToolTip();
	super.dropToolTip();
    }
  
}
