/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import yoyo.*;
import java.awt.*;
import java.net.MalformedURLException;
import java.awt.event.MouseEvent;

public class StopWidget extends Widget implements Pushable {
	
    JInterface iface; //is parent in widget class
    StarLogo sl;
	
    static Image stopimage;
    static Image stoppressed;
	
    public StopWidget(JInterface iface) {
	this.iface = iface;
	this.parent = iface; //parent must be initialized for this to have tooltip
	sl = iface.sl();
	updatePosition();
	image = stopimage;
	setToolTip(new ToolTipWidget(iface, iface.app().textBundle.getString("stop_widget_tooltip")));
    }

    public void updatePosition() {
	position = new Rectangle(iface.getSize().width-96, 6, 24, 24);
    }
    
    public static void loadImages(String gifdir, MediaTracker tracker, Toolkit kit) 
	throws MalformedURLException {
	if (YoYo.language.toString().lastIndexOf("en") != -1) {
	    stopimage = kit.createImage(StopWidget.class.getResource(gifdir + "stop.gif"));
	    stoppressed = kit.createImage(StopWidget.class.getResource(gifdir + "stop-pressed.gif"));
	} else {
	    // assume there's a language specific one
	    stopimage = kit.createImage(StopWidget.class.getResource(gifdir + "stop_" + YoYo.language.toString() + ".gif"));
	    stoppressed = kit.createImage(StopWidget.class.getResource(gifdir + "stop-pressed_" + YoYo.language.toString() + ".gif"));
	}

	tracker.addImage(stopimage, Widget.trackerID++);
	tracker.addImage(stoppressed, Widget.trackerID++);
    }
  
    public void mousePressed(MouseEvent e) {
	if (onPressable(e))
	    press();
    }
	
    public void mouseDragged(MouseEvent e) {
	if ( pressed() ) {
	    if ( onPressable(e) )
		pushDown();
	    else
		popUp();
	}
    }
	
    public void mouseReleased(MouseEvent e) {
	if ( pressed() && onPressable(e) ) {
	    execute();
	    reset();
	} else {
	    reset();
	}
    }
  
    boolean pressed = false;
  
    public boolean pressed() {
  	return pressed;
    }
  
    public void pushDown() {
  	if (image != stoppressed) {
	    image = stoppressed;
	    iface.addToClipRect(this.position);
  	}
    }
  		
    public void popUp() {
  	if (image != stopimage) {
	    image = stopimage;
	    iface.addToClipRect(this.position);
  	}
    }
  
    public boolean onPressable(MouseEvent e) {
  	return position.contains(new Point(e.getX(), e.getY()));
    }
  
    public void execute() {
	synchronized(sl.runningp) {
	    sl.stopAllStarLogo();
	}
	iface.setStatus(iface.app().textBundle.getString("stop_button_pressed"));
    }
	
    public void press() {
	pressed = true;
	pushDown();
    }
	
    public void reset() {
	pressed = false;
	popUp();
    }
}
