/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.awt.event.*;
import java.awt.*;
import java.util.Vector;
import java.util.ArrayList;
import java.util.Observable;
import java.util.Random;
import java.util.Enumeration;
import java.util.*;
import java.io.*;

import java.net.*;
import yoyo.*;

public class TextWidget extends WidgetDrag
 implements java.util.Observer {

    boolean drawBorder = false;
  public int id;
  public Font font;
  public FontMetrics fm;
  public static int TEXTWIDGET_HEIGHT = 15;
  public static int idCount = 1;

  StarLogo sl;
  Application app;

  int textWidth, textHeight, textDescent;
  int maxWidth, minWidth;

  public String words;
  public int totallinenums;
  
  static Font TEXT_WIDGET_FONT;
  
  static {
  	try {
	     TEXT_WIDGET_FONT = WIDGET_FONT;
   	     if (YoYo.macintoshp) TEXT_WIDGET_FONT = Font.getFont("starlogo.text-widget.font", TEXT_WIDGET_FONT);
  	} catch (Exception e) { e.printStackTrace(); }
  }

//this is the one that actually gets used when loading from file
  public TextWidget(String twwords, int twid, int twtextwidth,int twtotallinenums, 
			Rectangle position, boolean twborder, JInterface iface, StarLogo sl) {
//System.out.println("in constructor(words, id, position, iface, sl)");
    this.position = new Rectangle(position);
    this.position.height = TEXTWIDGET_HEIGHT + 10;
    this.id = twid;
    this.parent = iface;
    this.sl = sl;
    this.app = sl.app;
    this.font = TEXT_WIDGET_FONT;
    this.drawBorder = twborder;
    this.words = twwords;
    this.totallinenums = twtotallinenums;
   
    fm = iface.getToolkit().getFontMetrics(font);
    textDescent = fm.getMaxDescent();
    textHeight = fm.getMaxAscent()+textDescent;
    textWidth = twtextwidth;

    updateMinSize();

    //addObserver(parent);
    //app.procWin.addObserver(this);
  }

  public TextWidget(JInterface iface, StarLogo sl) {
//System.out.println("in constructor(iface, sl)");
    Dimension size = iface.getSize();
    Rectangle outline = iface.outline();

    if (outline.height < (TEXTWIDGET_HEIGHT + 10)) {outline.height = TEXTWIDGET_HEIGHT + 10;}

    this.position = new Rectangle(outline);
    this.position.height = TEXTWIDGET_HEIGHT + 10;
    this.id = idCount++;
    this.parent = iface;
    this.sl = sl;
    this.app = sl.app;
    this.font = WIDGET_FONT;

    fm = iface.getToolkit().getFontMetrics(font);
    textDescent = fm.getMaxDescent();
    textHeight = fm.getMaxAscent()+textDescent;

    //addObserver(parent);
    //app.procWin.addObserver(this);
  }

  public static TextWidget load(ArrayList items, JInterface iface, StarLogo sl) {
    int i = 0;
    String twwords="";
    boolean twborder= true;
    int twtop = 0, twleft = 0, twwidth = 50, twheight = 20, twid = 1, 
	twtextwidth = 0, twtotallinenums = 1;
    //System.out.println("there should be this many textwidgets: " + items.size()/16);
    while (i < items.size()) {
      String item = items.get(i).toString();
      if (item.equals("top-left")) {
        String value = items.get(i+1).toString();
        String value2 = items.get(i+2).toString();
	twtop = Integer.parseInt(value);
        twleft = Integer.parseInt(value2);
        i += 3;
        continue;
      }
      if (item.equals("width-height")) {
        String value = items.get(i+1).toString();
        String value2 = items.get(i+2).toString();
        twwidth = Integer.parseInt(value);
        twheight = Integer.parseInt(value2);
        i += 3;
        continue;
      }
      if (item.equals("textwidth")) {
        String value = items.get(i+1).toString();
        twtextwidth = Integer.parseInt(value);
        i += 2;
        continue;
      }
//new
      if (item.equals("linenums")) {
        String value = items.get(i+1).toString();
        twtotallinenums = Integer.parseInt(value);
        i += 2;
        continue;
      }
      if (item.equals("words")) {
        twwords = items.get(i+1).toString();
        i += 2;
        continue;
      }
      if (item.equals("drawBorder")) {
	  String value = items.get(i+1).toString();
	  twborder = Boolean.valueOf(value).booleanValue();
	  i += 2;
	  continue;
      }
      if (item.equals("textwidget-number")) {
	String value = items.get(i+1).toString();
	twid = Integer.parseInt(value);
	i += 2;
	continue;
      }
    
       i += 2;
    }
    Rectangle bounds = iface.getBoundaries();
    Rectangle position = new Rectangle(twleft + bounds.x, twtop + bounds.y, twwidth, twheight);
    if (twid >= idCount) {idCount = twid + 1;}
    return new TextWidget(twwords, twid, twtextwidth, twtotallinenums, position, twborder, iface, sl);
  }

  public String getSaveString(boolean htmlp) {
//the htmlp is andy's thing for applet. don't know how i'll end up incorporating it
    Rectangle bounds = parent.getBoundaries();
    String returnme = "SLTextWidget\r\n" +
	"top-left " + (position.y - bounds.y) + " " + (position.x - bounds.x) + "\r\n" + 		"width-height " + position.width + " " + position.height + "\r\n" +
//addding in an extra field to save so that when loading from file, the text is correctly centered
//(not too far right). means projects made with older version of texttool will be too far right still
	"textwidth " + textWidth + "\r\n" + 
	"linenums " + totallinenums + "\r\n" +
	"words \"" + Utilities.printable(this.words) + "\"\r\n" +
	"drawBorder " + this.drawBorder + "\r\n" +
	"textwidget-number " + this.id + "\r\n\r\n";
    //    System.out.println(returnme);
    return returnme;
  }

  public static void createNew(JInterface iface) {
      
      TextWidgetDialog twd = Application.createTextWidgetDialog((Frame)iface.frame(), iface.app(), 
								iface.app().textBundle.getString("new_text_label"));

    twd.show();
    if (twd.pressedOK()) {
      TextWidget tw = new TextWidget(iface, iface.sl());
      twd.getValues(tw);
      tw.updateMinSize();
      iface.addWidget(tw);
      //this is the only line we need from iface.app.project.compile(iface.app)
      //because there isn't any reason to force the whole project to compile
      iface.app().project.markChanged();
    }
    twd.dispose();
  }

  public void edit() {
    TextWidgetDialog twd = Application.createTextWidgetDialog((Frame)parent.frame(), app, 
							      app.textBundle.getString("edit_text_label"), this);
    twd.show();
    if (twd.pressedOK()) {
//these next 3 lines make sure that the old text is forced to redraw to background color
//otherwise, when you dramatically change the length of words during editing, when the widget
//resizes itself, there are redraw errors
      words = "";
	parent.addHandlesToClipRect(this);
      update(position);
      updateMinSize();
      twd.getValues(this);
      updateMinSize();
      update(position);
      app.project.markChanged();
    }
    twd.dispose();
    super.edit();
  }

  public void delete() {
    parent.textwidgets().removeElement(this);
    super.delete();
  }

  public void paintText (Graphics g) {
    Color col = g.getColor();
    g.setColor(Color.darkGray);
    g.setFont(font);
    StringBuffer sb = new StringBuffer(words.length());
//if there is only one line, draw the string
    if (totallinenums == 1)
       {g.drawString(words, (position.width - textWidth) / 2,
	  (position.height / 2) + (textHeight / 2) - textDescent); }
//else draw the string seperately for each line, dividing the height of the widget (position.height)
//into one more pieces than the number of lines, and putting the text on the appropriate line.
    else {
      int linenums = 1;
      sb = new StringBuffer(words.length());
      for(int i = 0; i < words.length(); i ++) {
        char c = words.charAt(i);
        if (c == '\n')
          {linenums = linenums + 1;
	   g.drawString(sb.toString(), (position.width - textWidth) / 2,
	      ((linenums - 1) * (position.height / (totallinenums + 1))) +
	      (textHeight / 2) -  textDescent);
           int tmplength = sb.toString().length();
           sb = new StringBuffer(words.length() - tmplength); }
        else {sb.append(c);} }
       linenums = linenums + 1;
       g.drawString(sb.toString(), (position.width - textWidth) / 2,
	  ((linenums - 1) * (position.height / (totallinenums + 1))) +
	  (textHeight / 2) -  textDescent);}

  }

  public void paint(Graphics g) {
    Shape oldClip = g.getClip();

    g.translate(position.x, position.y);

    position.height = TEXTWIDGET_HEIGHT * totallinenums + 10;
    g.setColor(Color.lightGray);
    if(drawBorder == true){
	g.drawRect(0,0,position.width-1, position.height-1);
    }
    
    g.clipRect(1,1, position.width -1, position.height - 1);
    
    paintText(g);
    
    g.translate(-position.x, -position.y);
    g.setClip(oldClip);

  }

  public void updateMinSize() {

//need to loop through here so that position.height gets set to the correct height.
    StringBuffer sb = new StringBuffer(words.length());
    int linenums = 1;
    int longestlength = 1;
    String longestline = "";
    for(int i = 0; i < words.length(); i++) {
	char c = words.charAt(i);
	if (c == '\n')
	  {linenums = linenums + 1;
	   int tmplength = fm.stringWidth(sb.toString());
           if (tmplength > longestlength)
		{ longestlength = tmplength;
		  longestline = sb.toString();}
	   sb = new StringBuffer(words.length() - sb.toString().length());
          }
	 else {sb.append(c);}
    }
      if (fm.stringWidth(sb.toString()) > longestlength)
        {longestlength = fm.stringWidth(sb.toString());
         longestline = sb.toString();}

    totallinenums = linenums;
    position.height = TEXTWIDGET_HEIGHT * totallinenums + 10;
//need to set iface.outline().height also, so that the entire text of the widget shows up.
    if (totallinenums > 1)
	{if (parent.outline() != null) 
		{if (parent.outline().height < position.height)
			{parent.outline().height = position.height;}}}

    if (totallinenums == 1) {textWidth = fm.stringWidth(getText());}
    else { textWidth = fm.stringWidth(longestline); }

    int minwidth = 4 + textWidth;
    if (minwidth < 40) minwidth = 40;

    position.width = minwidth;

    if (parent.outline() != null) {
	//trim iface.outline() to cover textwidget but also fit on screen
	if (parent.outline().x + minwidth > parent.getSize().width) {
	  parent.outline().width = parent.outline().width + parent.outline().x - parent.getSize().width + minwidth;
          parent.outline().x = parent.getSize().width - minwidth;
	  }
    	if (parent.outline().width < minwidth)
	  parent.outline().width = minwidth;
    }

    MIN_WIDTH = 4 + textWidth;

/*
System.out.print("position.width = " + position.width);
System.out.println(" position.height = " + position.height);
System.out.print("parent.outline().width = " + parent.outline().width);
System.out.println(" parent.outline().height = " + parent.outline().height);
*/
  }

  public String getText() {
//just returns the string
    return words;
  }

// return a string that can be read back in using the yoyo tokenizer
// use this for generating the save string
  public String twSavePrintable(String s) {
    StringBuffer sb = new StringBuffer(s.length());
    for(int i = 0; i < s.length(); i++) {
      char c = s.charAt(i);
      if (c == '"') {sb.append("\\\"");}
//maybe need this next line when doing x-platform?? *testing needed* -- maybe platform check needed
//      else if (c == '\n') {sb.append("\\r\\n");}
      else if (c == '\n') {sb.append("\\n");}
      else {sb.append(c);}
    }
//System.out.println(sb.toString());
    return sb.toString();
  }

  public void handleMouseClick(MouseEvent e) {
//only want to be able to edit textwidgets in starlogo (not in applet)
    if (parent.getToolbar().buttonDown == ToolbarWidget.TEXT)
        if (!app.kioskp && YoYo.applet == null) edit();
  }

  public int getToolbarButton() {
//what button on the toolbar corresponds to this widget
    return ToolbarWidget.TEXT;
  }

  public String toString() {
    return("TextWidget id: " + this.id + " words: " + this.words);
  }

  public void update(Object arg) {
//these two function calls are taken care of by parent class
    setChanged();
    notifyObservers(arg);
  }

  public void update(Observable obs, Object arg) {
    if (arg instanceof SLProc)
      update();
  }

  public void update() {
   //in other versions of this function, there is the line "compileCommand();"
   //but that's not needed here
  }

	public static boolean isBlank(char[] ch) {
		for (int i=0; i<ch.length; i++) {
			if ((ch[i] == ' ') || (ch[i] == '\n') || (ch[i] == '\t'))
				return false;
		}
		return true;
	}

	public static boolean isBlank(String s) {
		return isBlank(s.toCharArray());
	}

  public WidgetDrag copy(int edition, Rectangle r) {
    return new TextWidget(words, ++TextWidget.idCount,
					textWidth, totallinenums, r, drawBorder, parent, sl);
	}	

}//end of class
