/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.awt.*;
import java.util.*;
import java.lang.Math;

import yoyo.*;

public class ToolTipWidget extends Widget {

  private Font font;
  private FontMetrics fm;

  private int textWidth, textHeight, textDescent;
  private int maxWidth, minWidth;

  private String words;
  private String words2;
  private int totallinenums;
  
  private boolean fixed = false;
  private Color backgroundColor;

  private static Font TOOLTIP_WIDGET_FONT;
  private static Color DEFAULT_COLOR;
  private static int TOP_BUFFER = 50;
  private static int BOTTOM_BUFFER = 30;
  
  static {
  	try {
     TOOLTIP_WIDGET_FONT = WIDGET_FONT;
     if (YoYo.macintoshp) TOOLTIP_WIDGET_FONT = Font.getFont("starlogo.text-widget.font", TOOLTIP_WIDGET_FONT);
	 try {DEFAULT_COLOR = new Color(255, 255, 96); } //, 224);}
	 catch(Error er) {DEFAULT_COLOR = new Color(255, 255, 96);}
     catch(Exception ex) {DEFAULT_COLOR = new Color(255, 255, 96);}
    } catch (Exception e) { e.printStackTrace(); }
  }

  public ToolTipWidget(JInterface iface, String text) {
    words2 = text;
    words = text.trim();

    this.position = new Rectangle();
    this.position.height = 4;
    this.parent = iface;
    this.font = WIDGET_FONT;

	backgroundColor = DEFAULT_COLOR;
    fm = iface.getToolkit().getFontMetrics(font);
    textDescent = fm.getMaxDescent();
    textHeight = fm.getMaxAscent()+textDescent;

    updateMinSize();
  }

  public void paint(Graphics g) {

//    System.out.println("text height " + textHeight);
    Color col = g.getColor();
	Font fon = g.getFont();

    g.setColor(backgroundColor);
    g.fillRoundRect(position.x, position.y, position.width - 1, position.height - 1, 12, 12);
    g.setColor(Color.gray);
    g.drawRoundRect(position.x, position.y, position.width - 1, position.height - 1, 12, 12);

    g.setColor(Color.black);
    g.setFont(font);
    g.translate(position.x, position.y);

    StringBuffer sb = new StringBuffer(words.length());
	//if there is only one line, draw the string
    if (totallinenums == 1)
       {g.drawString(words, (position.width - textWidth) / 2,
	  (position.height / 2) + (textHeight / 2) - textDescent); }
	//else draw the string seperately for each line, dividing the height of the widget (position.height)
	//into one more pieces than the number of lines, and putting the text on the appropriate line.
    else {
      int linenums = 1;
      sb = new StringBuffer(words.length());
      for(int i = 0; i < words.length(); i ++) {
        char c = words.charAt(i);
        if (c == '\n')
          {linenums = linenums + 1;
	   g.drawString(sb.toString(), (position.width - textWidth) / 2,
	      ((linenums - 1) * (position.height / (totallinenums + 1))) +
	      (textHeight / 2) -  textDescent);
           int tmplength = sb.toString().length();
           sb = new StringBuffer(words.length() - tmplength); }
        else {sb.append(c);} }
       linenums = linenums + 1;
       g.drawString(sb.toString(), (position.width - textWidth) / 2,
	  ((linenums - 1) * (position.height / (totallinenums + 1))) +
	  (textHeight / 2) -  textDescent);}

    g.translate(-position.x, -position.y);
	g.setColor(col);
	g.setFont(fon);
  }
/*
  public void paint(Graphics g) {
    
	g.translate(position.x, position.y);
    Shape oldClip = g.getClip();
    g.clipRect(4, 4, position.width, position.height);
    g.setClip(oldClip);
    g.translate(-position.x, -position.y);
    
    paintText(g);
  }
*/
  public void updateMinSize() {

	//need to loop through here so that position.height gets set to the correct height.
    StringBuffer sb = new StringBuffer(words.length());
    int linenums = 1;
    int longestlength = 1;
    String longestline = "";
    for(int i = 0; i < words.length(); i++) {
	char c = words.charAt(i);
	if (c == '\n')
	  {linenums = linenums + 1;
	   int tmplength = fm.stringWidth(sb.toString());
           if (tmplength > longestlength)
		{ longestlength = tmplength;
		  longestline = sb.toString();}
	   sb = new StringBuffer(words.length() - sb.toString().length());
          }
	 else {sb.append(c);}
    }
      if (fm.stringWidth(sb.toString()) > longestlength)
        {longestlength = fm.stringWidth(sb.toString());
         longestline = sb.toString();}

    totallinenums = linenums;
    position.height = textHeight * totallinenums + 4;

    if (totallinenums == 1) {textWidth = fm.stringWidth(getText());}
    else { textWidth = fm.stringWidth(longestline); }

    int minwidth = 12 + textWidth;
    if (minwidth < 24) minwidth = 24;

    position.width = minwidth;
  }

  public void fixPosition(Point p) {
    if (p != null) {
       position.setLocation(p);
       fixed = true;
    }
  }

  public void fixPosition(int x, int y) {
	position.setLocation(x, y);
	fixed = true;
  }

  public void releasePosition() {
    fixed = false;
  }

  public boolean isFixed() {
    return fixed;
  }

  public void setPosition(Rectangle r) { 
    if (!(fixed)) {
       Point anchor = r.getLocation();
       int s = Math.max( -2, anchor.x + position.width - parent.getSize().width);
       if ((anchor.y - (position.height + 2)) < TOP_BUFFER) anchor.translate(-s, r.height + 2);
       else anchor.translate(-s, -(position.height + 2));
       if (anchor.x < 0) anchor.setLocation(0, anchor.y);
       if (anchor.y + position.height > parent.getSize().height) anchor.setLocation(anchor.x, TOP_BUFFER);
       position.setLocation(anchor);
    }
  }

  public void setPosition(Rectangle owner, Rectangle avoid) {
    if (!fixed) {
		setPosition(owner);
        if (position.intersects(avoid)) {
			if ((owner.x < avoid.x) && (position.x + position.width - avoid.x < position.x)) { 
				position.x = position.x - (position.x + position.width - avoid.x + 1);
				return;
			}
			else {
				if ((owner.y < avoid.y) && (TOP_BUFFER + position.height < avoid.y)) {
					if (position.width + owner.x + owner.width < parent.getSize().width) {
						position.x = owner.x + owner.width + 1;
						position.y = avoid.y - position.height - 1;
						return;
					}
					if (position.width < owner.x) {
						position.x = owner.x - position.width - 1;
						position.y = avoid.y - position.height - 1;
						return;
					}
				}
				if ((owner.y > avoid.y + avoid.height) && (BOTTOM_BUFFER + position.height < parent.getSize().height - avoid.y - avoid.height)) {
					if (position.width + owner.x + owner.width < parent.getSize().width) {
						position.x = owner.x + owner.width + 1;
						position.y = avoid.y + avoid.height + 1;
						return;
					}
					if (position.width < owner.x) {
						position.x = owner.x - position.width - 1;
						position.y = avoid.y + avoid.height + 1;
						return;
					}
				}
			}
        }
    }
  }

  public String getText() { 
//just returns the string
    return words2;
  }	

  public String toString() {
     return "Tool Tip " + words;
  }

  public static boolean empty(String text) {
        boolean out = true;
        if ((text == null) || (text.trim().length() == 0)) out = true;
        else out = false;
        return out;
  }
 
  public String getSaveString() {
	return "toolTip-text \"" + Utilities.printable(this.words) + "\"\r\n";
  }
		

}//end of class
