/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.lang.Math;
import java.awt.event.*;
import java.awt.*;
import java.net.*;
import javax.swing.UIManager;
import yoyo.*;

public class ToolbarWidget extends Widget {

    public static final int NONE    = 0;
    public static final int POINTER = 1;
    public static final int BUTTON  = 2;
    public static final int SLIDER  = 3;
    public static final int MONITOR = 4;
    public static final int TEXT = 5;
    public static final int LINE = 6;
    public static final int LEGEND = 7;
    public static int NUM = 7;
    
    static int buttonWidth = 31;
    static int buttonHeight = 31;
    
    static Image toolbarHorizImagePush, toolbarHorizImage,
	toolbarVertImagePush, toolbarVertImage;

	static ToolTipWidget pointerToolTip;
	static ToolTipWidget buttonToolTip;
	static ToolTipWidget sliderToolTip;
	static ToolTipWidget monitorToolTip;
	static ToolTipWidget textToolTip;
	static ToolTipWidget plotToolTip;
	static ToolTipWidget legendToolTip;
    
    /* when buttonDown = NONE, rowDown = -1 or columnDown = -1 */
    /* also, buttonDown = NONE => paint tools are active */
    public int buttonDown;
    public int rowDown;
    public int columnDown;
    
    public StarLogo sl;
    
    int oldButtonDown;
    int firstButtonDown;
    boolean horizontalp;

    ToolbarWidget(JInterface parent) {
			this(parent, true);
    }

    ToolbarWidget(JInterface parent, boolean horizontalp) {
			super(parent, (horizontalp) ? toolbarHorizImage : toolbarVertImage);
			sl = parent.sl();
			oldButtonDown = POINTER;
			buttonDown=POINTER;
			rowDown = 0;
			columnDown = 0;
			this.horizontalp = horizontalp;
			pointerToolTip = new ToolTipWidget(parent, sl.app.textBundle.getString("pointer_tooltip"));
			buttonToolTip = new ToolTipWidget(parent, sl.app.textBundle.getString("button_tooltip"));
			sliderToolTip = new ToolTipWidget(parent, sl.app.textBundle.getString("slider_tooltip"));
			monitorToolTip = new ToolTipWidget(parent, sl.app.textBundle.getString("monitor_tooltip"));
			textToolTip = new ToolTipWidget(parent, sl.app.textBundle.getString("textbox_tooltip"));
			plotToolTip = new ToolTipWidget(parent, sl.app.textBundle.getString("plot_tooltip"));
			legendToolTip = new ToolTipWidget(parent, sl.app.textBundle.getString("legend_tooltip"));
    }

    public boolean paintActivep() {
	return buttonDown == NONE;
    }

    public void setButtonDown(int button) {
	drawButtonDown(button);
	if (oldButtonDown == NONE &&
	    buttonDown != NONE) {
	    parent.painttool().drawButtonDown(Painttool.NONE);
	
	}
    }

    public void drawButtonDown(int button) {
	oldButtonDown = buttonDown;
	buttonDown = button;
	buttonToRowColumn(button);
	if (oldButtonDown == buttonDown) return;
	parent.addToClipRect(position);
    }

    public void buttonToRowColumn(int button) {
	if (horizontalp) {
	    rowDown = 0;
	    columnDown = button - 1;
	} else {
	    if (button == 9) button = 8; else
		if (button == 8) button = 7; else button = button - 1;
	    rowDown = button % 4;
	    columnDown = button % 2;
	}	    
    }

    public int pointToButton(int x, int y) {
	int button = 0;
	if (horizontalp) {
	    columnDown = (int)java.lang.Math.ceil(NUM*(x - position.x)/image.getWidth(parent));
	    rowDown = 0;
	    button = columnDown + 1;
	} else {
	    columnDown =  (int)java.lang.Math.ceil(2 * (x - position.x) / image.getWidth(parent));
	    rowDown = (int)java.lang.Math.ceil(4 * (y - position.y) / image.getHeight(parent));
	    button = 1 + (rowDown * 2 + columnDown);
	    if (button == 7) button = 8; else
		if (button == 8) button = 9;
	}
	return button;	    
    }

  public void mouseClicked(MouseEvent e) {
      if (!position.contains(new Point(e.getX(), e.getY()))) return;
      oldButtonDown = buttonDown;
      buttonDown = pointToButton(e.getX(), e.getY());
      if (oldButtonDown == buttonDown) return;

      if (oldButtonDown == NONE) {
	  parent.painttool().drawButtonDown(Painttool.NONE);
      }

    parent.addToClipRect(position);
  }

    public void mouseDragged(MouseEvent e) {
	mouseClicked(e);
    }

    public void mousePressed(MouseEvent e) {
	mouseClicked(e);
    }

  public void paint(Graphics g) {
      //System.out.println("old clip before translation by " + position.x + ", " + position.y + ": " + g.getClip());
      g.translate(position.x,position.y);
      if (YoYo.macosxp) {
	  g.setColor(UIManager.getColor("window"));
      } else {
	  g.setColor(SystemColor.control);
      }
      /* repaint background */
      g.fillRect(0,0,position.width, position.height);
      g.drawImage(image, 0, 0, null);
      if (buttonDown != NONE) {
	  Shape oldclip = g.getClip();
	  //System.out.println("old clip: " + oldclip);
	  g.setClip(buttonWidth * columnDown, buttonHeight * rowDown, buttonWidth, buttonHeight);
	  //System.out.println("set clip to " + (buttonWidth * columnDown) + ", " + 
	  //		     (buttonHeight * rowDown) + ", " + buttonWidth + ", " + buttonHeight);
	  //System.out.println("new clip: " + g.getClip());
	  g.drawImage((horizontalp) ? toolbarHorizImagePush : toolbarVertImagePush, 0, 0, null);
	  g.setClip(oldclip);
      }
      g.translate(-position.x,-position.y);
  }

  public static void loadImages(String gifdir, MediaTracker tracker, Toolkit kit)
    throws MalformedURLException{
      // requires: trackerID in Widget indexes start of empty ID #s in <tracker>

      toolbarHorizImagePush = kit.createImage(ToolbarWidget.class.getResource(gifdir + "toolbar-horiz-down1.gif"));
      toolbarHorizImage = kit.createImage(ToolbarWidget.class.getResource(gifdir + "toolbar-horiz-up1.gif"));
      toolbarVertImagePush = kit.createImage(ToolbarWidget.class.getResource(gifdir + "toolbar-vertical-down.gif"));
      toolbarVertImage = kit.createImage(ToolbarWidget.class.getResource(gifdir + "toolbar-vertical-up.gif"));
      tracker.addImage(toolbarHorizImagePush, trackerID++);
      tracker.addImage(toolbarHorizImage, trackerID++);
      tracker.addImage(toolbarVertImagePush, trackerID++);
      tracker.addImage(toolbarVertImage, trackerID++);

  }

	public void displayToolTip(Point point) {
		int index = 1 + point.x / buttonWidth;
		switch (index) {
			case POINTER:
				setToolTip(pointerToolTip);
				break;
			case BUTTON:
				setToolTip(buttonToolTip);
				break;
			case SLIDER:
				setToolTip(sliderToolTip);
				break;
			case MONITOR:
				setToolTip(monitorToolTip);
				break;
			case TEXT:
				setToolTip(textToolTip);
				break;
			case LINE:
				setToolTip(plotToolTip);
				break;
			case LEGEND:
			default:
				setToolTip(legendToolTip);
				break;
		}
		super.displayToolTip(point);
	}

}
