/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import yoyo.TopApplication;
import yoyo.Reader;
import yoyo.Context;
import yoyo.YoYo;
import yoyo.PrimStartup;
import java.awt.Frame;
import java.awt.EventQueue;
import java.net.*;
import javax.swing.SwingUtilities;
import java.util.Locale;

public class Toplevel extends TopApplication {
	
    public static String filetoload = null;

    public static Toplevel toplevel = new Toplevel();
    public Application app;
    
    private static SplashDialog sd;

    public static void main(String[] args) {
	try {
	    sd = new SplashDialog();
	    sd.show();
	    toplevel = new Toplevel();
	    toplevel.init(args);
	} 
	catch (Exception e) { 
	    e.printStackTrace(); 
	}
	finally {
	    EventQueue.invokeLater(new Runnable() {
		    public void run() {
			if (sd != null) sd.setVisible(false);
		    }
		});
	}

    }
	
    public void initArgs(String[] args) {
	try {
	    if (args.length>0) bootfile = args[0];
	    if (args.length>1){
		YoYo.language = new Locale(args[1],Locale.getDefault().getCountry());
		PrimStartup.reGetYoYoErrorsBundle();
	    }

	    if (args.length>2) listtorun = args[2];
	    // if the list to run doesn't contain a space, assume that it's
	    // because of the parsing of arguments in UNIX, and that we should
	    // treat the rest of the command line as the list to run.  This will
	    // cause problems whenever the default list to run is just one command
	    // AND we are trying to load a project through the command line.  Also,
	    // can't load project through command line in UNIX.
	    if (listtorun.indexOf(" ")==-1) {
		for (int i=3; i<args.length; i++) {
		    listtorun += " " + args[i];
		}
	    } else {
		if (args.length>3) filetoload = args[3];
	    }
	} catch (Exception e) { System.out.println("hello"); e.printStackTrace(); }
    }	
	

    public void loadBootFile() {
  	//System.out.println("starting starlogo");
	Object[] ilist = 
	    Reader.read("load-resource \"starlogo.StarLogo\" \"" + bootfile + ".yoyo\"");
	c.runList(ilist);
	c.waitUntilDone();
    
    }
    
    

    SLURL file;

    public void run() {

	if (listtorun != null && !listtorun.equals("")) {
	    //c.out.println("Running " + listtorun);
	    Object[] ilist = Reader.read(listtorun);
	    c.runList(ilist);
	    c.waitUntilDone();
	    if (filetoload != null) {
		file = new SLURL(c.base, filetoload);
		//System.out.println("file url is " + file.toString());
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
			    Project.loadFile(file, Application.starlogoapplication, false);
			    Project.runStartup(Application.starlogoapplication);
			    //System.out.println("asynchronously finished loading project file.");
			}
		    });
	    }
	}
	// moved out of Application.java so that everything would be initialized
	// before these calls were made -MTB
	toplevel.app.iface.frame().show();
	toplevel.app.procWin.show();
    }
}
