/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.awt.*;
import java.net.MalformedURLException;
import java.awt.event.MouseEvent;
import yoyo.YoYo;

public class TrashWidget extends Widget {

    JInterface iface; // is parent in widget class
    StarLogo sl;
    Application app;

    static Image trashcan;
    static Image trashover;

    public TrashWidget(JInterface iface) {
	this.iface = iface;
	this.parent = iface; // parent must be initialized for tooltips to work
	sl = iface.sl();
	app = sl.app;
	//on pcs, the trash widget was being drawn off the right side of the screen
	//position = new Rectangle(iface.getSize().width - ((YoYo.macintoshp) ? 48 : 24),
	//1, 32, 32);
	updatePosition();

	//if (YoYo.macintoshp) {
	//	image = iface.createImage(24, 24); // to fake it out
	//} else {
	image = trashcan;
	//}
	setToolTip(new ToolTipWidget(iface, iface.app().textBundle.getString("trash_tooltip")));
    }

    public void updatePosition() {
	//position = new Rectangle(iface.getSize().width - 48, 1, (YoYo.macintoshp) ? 32 : 24, (YoYo.macintoshp) ? 32 : 24);
	position = new Rectangle(iface.getSize().width - 48, 
				 (YoYo.macintoshp && !YoYo.macosxp) ? 8 : 1, 
				 32, (YoYo.macintoshp && !YoYo.macosxp) ? 25 : 32);
    }
	
    public static void loadImages(String gifdir, MediaTracker tracker, Toolkit kit)
	throws MalformedURLException {
	
	if (YoYo.macintoshp) {
	    if (YoYo.macosxp) {
		trashcan = kit.createImage(TrashWidget.class.getResource(gifdir + "trash-empty-macosx.png"));
		trashover = kit.createImage(TrashWidget.class.getResource(gifdir + "trash-full-macosx.png"));
	    } else {
		trashcan = kit.createImage(TrashWidget.class.getResource(gifdir + "trash-unselected-macos9.gif"));
		trashover = kit.createImage(TrashWidget.class.getResource(gifdir + "trash-selected-macos9.gif"));
	    }	    
	} else if (YoYo.winxpp) {
	    trashcan = kit.createImage(TrashWidget.class.getResource(gifdir + "trash_unselected_xp.gif"));
	    trashover = kit.createImage(TrashWidget.class.getResource(gifdir + "trash_selected_xp.gif"));
	} else if (YoYo.win2kp) {
	    trashcan = kit.createImage(TrashWidget.class.getResource(gifdir + "trash_unselected_2k.gif"));
	    trashover = kit.createImage(TrashWidget.class.getResource(gifdir + "trash_selected_2k.gif"));
	} else {
	    trashcan = kit.createImage(TrashWidget.class.getResource(gifdir + "recycle.gif"));
	    trashover = kit.createImage(TrashWidget.class.getResource(gifdir + "trashover.gif"));
	}
	tracker.addImage(trashcan, Widget.trackerID++);
	tracker.addImage(trashover, Widget.trackerID++);
	
    }


    public void mouseDragged(MouseEvent e) {
	if ( pressed() && onPressable(e) )
	    pushDown();
	else
	    popUp();
    }

    public void mouseReleased(MouseEvent e) {
	if ( pressed() && onPressable(e) ) {
	    execute();
	    reset();
	} else {
	    reset();
	}
    }


    boolean pressed;

    public boolean pressed() {
	return pressed;
    }

    public boolean onPressable(MouseEvent e) {
  	return position.contains(new Point(e.getX(), e.getY()));
    }

    public void paint(Graphics g) {
		if (YoYo.macintoshp) {
	   g.drawImage(image, position.x, position.y,
			position.width + position.x, position.height + position.y,
			0, 0, image.getWidth(this), image.getHeight(this), this);
	} else {
	    g.drawImage(image, position.x, position.y + 5, this);
	    //super.paint(g);
	}
    }

    public void pushDown() {
	//if (YoYo.macintoshp) {
	//sl.app.macutils.drawTrashIcon((Component)iface, (short)position.x, (short)position.y, true);
	//} else {
	if (image != trashover) {
	    image = trashover;
	    iface.addToClipRect(this.position);
	}
	//}
    }
    
    public void popUp() {
	//if (YoYo.macintoshp) {
	//sl.app.macutils.drawTrashIcon((Component)iface, (short)position.x, (short)position.y, false);
	//} else {
	if (image != trashcan) {
	    image = trashcan;
	    iface.addToClipRect(this.position);
	}
	//}
    }
    
    public void press() {
	pressed = true;
	pushDown();
    }
    
    public void reset() {
	pressed = false;
	popUp();
    }
    
    public void execute() {
	WidgetDrag[] wd = iface.wh().getSelectedWidgets(false);
	if (wd != null) {
	    for (int i=0; i<wd.length; i++)
		iface.removeWidget(wd[i]);
	    sl.app.rancompilerp = false;
	    sl.app.rancompilerpForVariables = false;
	    if (wd.length == 1) {
		iface.setStatus(String.valueOf(wd.length) + " " +
			       app.textBundle.getString("widget_removed"));
	    } else {
		iface.setStatus(String.valueOf(wd.length) + " " +
			       app.textBundle.getString("widgets_removed"));
	    }
	} else if ((iface.widgetTouched() != null) &&
		   (iface.widgetTouched() != iface.getPatchCanvas())) {
	    iface.removeWidget(iface.widgetTouched());
	    sl.app.rancompilerp = false;
	    sl.app.rancompilerpForVariables = false;
	    //sl.app.project.compile(sl.app);
	    iface.setStatus("1 " + app.textBundle.getString("widget_removed"));
	}
	reset();
    }

}
