/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.io.*;
import yoyo.YoYo;
import yoyo.Context;
import yoyo.LogoError;
import yoyo.PrimList;
import yoyo.PrimMath;
import yoyo.Primitives;
import yoyo.Symbol;
import java.io.Serializable;
import java.awt.Image;
import java.util.*;

class Turtle extends VM implements Serializable {

    public int hashCode() {
	return who;
    }

    public boolean equals(Object o) {
	if (o instanceof Turtle) {
	    return ((Turtle)o).who == who;
	} else return false;
    }

    transient Turtle next, prev;
    transient Turtle overme, underme;
    transient int patch;
    int who;
    int shape;
    Integer whoInt;
    double xcor, ycor, heading, breed, color;
    double xdelta, ydelta;
    boolean pendownp=false, shownp=true, alive=true;
    transient VM grabber = null;
    transient boolean hasTMp = false;
    transient TurtleMonitor tm;

    Turtle(StarLogo sl, double xcor, double ycor, double heading,
	   double color, int who, double breed, int shape) {
	this(sl, xcor, ycor, heading, color, who, breed, shape, 1, 1,
	     false, true, new Object[numVars]);
	for(int i = 0; i<numVars; i++) turtlesown[i] = zeroInt;
    }

    Turtle(StarLogo sl, double xcor, double ycor, double heading,
	   double color, int who, double breed, int shape, boolean pendownp,
	   boolean shownp, int plotpen, int plotwidget) {
	this(sl, xcor, ycor, heading, color, who, breed, shape, plotpen, plotwidget,
	     pendownp, shownp, new Object[numVars]);
	for(int i = 0; i<numVars; i++) turtlesown[i] = zeroInt;
    }

    Turtle(StarLogo sl, double xcor, double ycor, double heading,
	   double color, int who, double breed, int shape, int plotpen, 
	   int plotwidget, boolean pendownp, boolean shownp, Object[] town) {
	this.sl = sl;
	this.xcor = xcor-sl.turhalfwidth;
	this.ycor = sl.turhalfheight-ycor;
	this.color = color;
	this.who = who; this.whoInt = new Integer(who);
	this.breed = breed;
	this.shape = shape;
	this.plotpen = plotpen;
	this.plotwidget = plotwidget;
	this.shownp = shownp;
	this.pendownp = pendownp;
	this.patch = -1;
	turtlesown = town;
	setheading(heading);
    }

    public static Turtle restore(StarLogo sl, byte[] out, int offset) {
	// int who, shape, plotpen;
	// double color, xcor, ycor, heading, breed
	// boolean pendownp, shownp
	// save order:
	// who, xcor, ycor, heading, color, breed, shape, plotpen, pendownp, shownp
	int who = 0, shape = 0, plotpen = 1, plotwidget = 1;
	double xcor = sl.turhalfwidth, ycor = sl.turhalfheight, heading = 0, breed = 0,
	    color = 5.0;
	boolean pendownp = false, shownp = true;

	int off = getOffset("who");
	if (off >= 0)     who = Utilities.readIntFromByteArray(out, offset + off);
	off = getOffset("xcor");
	if (off >= 0)     xcor = Utilities.readDoubleFromByteArray(out, offset + off);
	off = getOffset("ycor");
	if (off >= 0)     ycor = Utilities.readDoubleFromByteArray(out, offset + off);
	off = getOffset("heading");
	if (off >= 0)     heading = Utilities.readDoubleFromByteArray(out, offset + off);
	off = getOffset("color");
	if (off >= 0)     color = Utilities.readDoubleFromByteArray(out, offset + off);
	off = getOffset("breed");
	if (off >= 0)     breed = Utilities.readDoubleFromByteArray(out, offset + off);
	off = getOffset("shape");
	if (off >= 0)     shape = Utilities.readIntFromByteArray(out, offset + off);
	off = getOffset("plotpen");
	if (off >= 0)     plotpen = Utilities.readIntFromByteArray(out, offset + off);
	off = getOffset("plotwidget");
	if (off >= 0)     plotwidget = Utilities.readIntFromByteArray(out, offset + off);
	off = getOffset("pendownp");
	if (off >= 0)     pendownp = Utilities.readBooleanFromByteArray(out, offset + off);
	off = getOffset("shownp");
	if (off >= 0)     shownp = Utilities.readBooleanFromByteArray(out, offset + off);
	//System.out.println("make turtle: xcor " + xcor + " ycor " + ycor + " heading " + heading);
	//System.out.println("color " + color + " breed " + breed + " shape " + shape + " plotpen " + plotpen);
	//System.out.println("pendownp " + pendownp + " shownp " + shownp);
	Object[] turtlesown = restoreVariables(out, offset + getStateLength());
	return new Turtle(sl, xcor, ycor, heading, color, who, breed, shape, plotpen,
			  plotwidget, pendownp, shownp, turtlesown);
    }

    public void save(byte[] out, int offset) {
	// int who, shape, plotpen;
	// double color, xcor, ycor, heading, breed
	// boolean pendownp, shownp
	// save order:
	// who, xcor, ycor, heading, color, breed, shape, plotpen, pendownp, shownp
	int off = getOffset("who");
	if (off >= 0)     Utilities.writeIntoByteArray(who, out, offset + off);
	off = getOffset("xcor");
	if (off >= 0)     Utilities.writeIntoByteArray(xcor - sl.turhalfwidth, out, offset + off);
	off = getOffset("ycor");
	if (off >= 0)     Utilities.writeIntoByteArray(sl.turhalfheight - ycor, out, offset + off);
	off = getOffset("heading");
	if (off >= 0)     Utilities.writeIntoByteArray(heading, out, offset + off);
	off = getOffset("color");
	if (off >= 0)     Utilities.writeIntoByteArray(color, out, offset + off);
	off = getOffset("breed");
	if (off >= 0)     Utilities.writeIntoByteArray(breed, out, offset + off);
	off = getOffset("shape");
	if (off >= 0)     Utilities.writeIntoByteArray(shape, out, offset + off);
	off = getOffset("plotpen");
	if (off >= 0)     Utilities.writeIntoByteArray(plotpen, out, offset + off);
	off = getOffset("plotwidget");
	if (off >= 0)     Utilities.writeIntoByteArray(plotwidget, out, offset + off);
	off = getOffset("pendownp");
	if (off >= 0)     Utilities.writeIntoByteArray(pendownp, out, offset + off);
	off = getOffset("shownp");
	if (off >= 0)     Utilities.writeIntoByteArray(shownp, out, offset + off);
	saveVariables(out, offset + getStateLength());
    }

    public void saveVariables(byte[] out, int offset) {
	int off = 0;
	for(int i = 0; i < numVars; i++) {
	    int pos = getVarPos(i);
	    Object val = turtlesown[pos];
	    int index = Utilities.addIntoTable(val, sl);
	    //System.out.println("index = " + index);
	    Utilities.writeIntoByteArray(index, out, offset + off);
	    off += 4;
	}
    }

    public static Object[] restoreVariables(byte[] out, int offset) {
	Object[] town = new Object[numVars];
	int off = 0;
	for(int i = 0; i < town.length; i++) {
	    int intval = Utilities.readIntFromByteArray(out, offset + off);
	    off += 4;
	    Object val = Utilities.lookupInTable(intval);
	    town[getVarPos(i)] = val;
	}
	return town;
    }

    public static Hashtable stateOffsetTable = initStateOffsets();

    public static Hashtable initStateOffsets() {
	Hashtable table = new Hashtable();
	table.put("who", new Integer(0));
	table.put("xcor", new Integer(4));
	table.put("ycor", new Integer(12));
	table.put("heading", new Integer(20));
	table.put("color", new Integer(28));
	table.put("breed", new Integer(36));
	table.put("shape", new Integer(42));
	table.put("plotpen", new Integer(46));
	table.put("pendownp", new Integer(50));
	table.put("shownp", new Integer(51));
	table.put("plotwidget", new Integer(52));
	table.put("statelength", new Integer(56));
	table.put("length", new Integer(56));
	return table;
    }

    public static int getLength() {
	return getOffset("length");
    }

    public static void setLength(Integer val) {
	stateOffsetTable.put("length", val);
    }

    public static int getStateLength() {
	return getOffset("statelength");
    }

    public static int getOffset(String whichone) {
	if (stateOffsetTable.containsKey(whichone)) {
	    return ((Integer)stateOffsetTable.get(whichone)).intValue();
	}
	return -1000;
    }

    public static byte[] getStateDesc() throws IOException {
	ByteArrayOutputStream baos;
	DataOutputStream dos = new DataOutputStream(baos = new ByteArrayOutputStream());
	for(Enumeration e = stateOffsetTable.keys(); e.hasMoreElements();) {
	    String key = (String)e.nextElement();
	    int val = ((Integer)stateOffsetTable.get(key)).intValue();
	    dos.writeUTF(key);
	    dos.writeInt(val);
	}
	dos.flush();
	dos.close();
	return baos.toByteArray();
    }

    public static void setStateDesc(byte[] bytes) throws IOException {
	stateOffsetTable.clear();
	DataInputStream dis =
	    new DataInputStream(new ByteArrayInputStream(bytes));
	while(dis.available() > 0) {
	    String key = dis.readUTF();
	    int val = dis.readInt();
	    stateOffsetTable.put(key, new Integer(val));
	}
	dis.close();
    }


    public final boolean grab(Object val) {
	if (grabber != null) return false;
	Turtle st; int trueip, tempint; Object[] list;
	if (val instanceof Number) {
	    st = sl.getWho(((Number)val).intValue());
	    if (st != null && st != this && (st.grabber == null) && st.partner == null) {
		ostack[sp++] = st;
		ostack[sp++] = VM.grabbedMarker;
		partner = st;
		st.grabber = this;
		trueip = istack[--isp];
		istack[isp++] = 1;
		istack[isp++] = ip;
		ip = trueip;
		istack[isp++] = (VM.GRAB2 | (PRIMGRAB << 8) | (OTHER << 16));
		return true;
	    }
	    else return false;
	}
	else if (val instanceof Object[]) {
	    list = (Object[])val;
	    int count = 0;
	    for(tempint = list.length-1; tempint >= 0; tempint--) {
		st = sl.getWho(aInteger(list[tempint]));
		if (st != null && st != this && (st.grabber==null) && st.partner == null) {
		    ostack[sp++] = st;
		    ostack[sp++] = VM.grabbedMarker;
		    partner = st;
		    st.grabber = this;
		    count++;
		}
	    }
	    if (count > 0) {
		trueip = istack[--isp];
		istack[isp++] = count;
		istack[isp++] = ip;
		ip = trueip;
		istack[isp++] = (VM.GRAB2 | (PRIMGRAB << 8) | (OTHER << 16));
		return true;
	    }
	    else return false;
	}
	else return false;
    }

    public final boolean grabPatch(Patch p) {
	if (grabber != null) return false;
	int trueip, tempint; Object[] list;
	if (!p.grabbedp && p.patchpartner == null) {
	    ostack[sp++] = p;
	    ostack[sp++] = VM.grabbedPatchMarker;
	    patchpartner = p;
	    p.grabbedp = true;
	    trueip = istack[--isp];
	    istack[isp++] = 1;
	    istack[isp++] = ip;
	    ip = trueip;
	    istack[isp++] = (VM.GRABPATCH2 | (PRIMGRAB << 8) | (OTHER << 16));
	    return true;
	}
	else return false;
    }


    final public void setheading(double h) {
	double htemp = h - (int)h;
	heading = (double)((int)h)%360;
	if (heading<0) heading += 360;
	heading+=htemp;
	double r = heading * Math.PI / 180.0;
	xdelta = Math.sin(r);
	ydelta = -1 * Math.cos(r);
    }

    final public void setState(int which, Object val) {
	switch(which & 0xFF) {
	case VM.TXCOR:
	    sl.unlink(this);
	    xcor = sl.turhalfwidth + aDouble(val);
	    sl.wrapandlink(this);
	    break;
	case VM.TYCOR:
	    sl.unlink(this);
	    ycor = sl.turhalfheight - aDouble(val);
	    sl.wrapandlink(this);
	    break;
	case VM.TCOLOR:
	    color = aDouble(val)% Colors.numcolors;
	    color = (color<0) ? color+ Colors.numcolors : color;
	    if (this == sl.patches[patch].topturtle)
		sl.patchcanvas.drawTurtle(this);
	    break;
	case VM.THEADING:
	    setheading(aDouble(val));
	    break;
	case VM.TBREED:
	    breed = aDouble(val);
	    break;
	case VM.TSHAPE:
	    shape = aInteger(val)%sl.numshapes;
	    shape = (shape<0) ? shape+sl.numshapes : shape;
	    if (this == sl.patches[patch].topturtle) sl.patchcanvas.drawTurtle(this);
	    break;
	case VM.PENDOWNP:
	    if (val != YoYo.symfalse) pendownp = true; else pendownp = false;
	    break;
	case VM.SHOWNP:
	    if (val != YoYo.symfalse) {
		if (!shownp) {
		    shownp = true;
		    sl.showTurtle(this);
		}
	    }
	    else if (shownp) {
		shownp = false;
		sl.hideTurtle(this);
	    }
	    break;
	default:
	    throw new LogoError("Oops, compiler error in Turtle. setState of " + which + " in instruction: " + iinst[ip-1]);
	}
    }

    final public Object getState(int which) {
	switch(which & 0xFF) {
	case VM.TXCOR: return new Double(xcor - sl.turhalfwidth);
	case VM.TYCOR: return new Double(sl.turhalfheight - ycor);
	case VM.TCOLOR: return new Double(color);
	case VM.THEADING: return new Double(heading);
	case VM.TBREED: return new Double(breed);
	case VM.TSHAPE: return new Integer(shape);
	case VM.PENDOWNP: return (pendownp) ? YoYo.symtrue : YoYo.symfalse;
	case VM.SHOWNP: return (shownp) ? YoYo.symtrue : YoYo.symfalse;
	}
	throw new LogoError("Oops, compiler error in Turtle. getState of " + which + " in instruction: " + iinst[ip-1]);
    }

    public void die() {
	if (hasTMp)
	    tm.die();
	//remove point
	sl.app.iface.erasePointsOnAllWidgets(who);
    }

    static class VariableDesc {
	String name;
	int position;
	boolean enump;
	
	VariableDesc(String name, int position, boolean enump) {
	    this.name = name;
	    this.position = position;
	    this.enump = enump;
	}
    }

    static int numVars = 0;
    static VariableDesc[] vars = null;
    
    static public void setVars(Object[] town, StarLogo sl, Context c) {

	//System.out.println("turtle setVars: " + YoYo.printToString(town));

	int newNumVars = town.length;
	setLength(new Integer(getStateLength() + (4 * newNumVars)));
	
	VariableDesc[] newvars = new VariableDesc[newNumVars];

	for (int i = 0; i < newNumVars; i++) {
	    Object[] pair = YoYo.aList(c, town[i]);
	    newvars[i] = new VariableDesc(YoYo.aString(c, pair[0]), 
					  YoYo.aInteger(c, pair[1]),
					  YoYo.aBoolean(c, pair[2]));
	}

	boolean[] donep = new boolean[newNumVars]; // they're all initialized false
	HashMap varMap = new HashMap(newNumVars);
	if (vars != null) {
	    // loop over old vars and map old pos to new pos
	    for(int i = 0; i < numVars; i++) {
		Integer oldPos = new Integer(vars[i].position);
		for(int j = 0; j < newNumVars; j++) {
		    if (newvars[j].name.equals(vars[i].name)) {
			Integer newPos = new Integer(newvars[j].position);
			varMap.put(oldPos, newPos);
			donep[newvars[j].position] = true;
			break;
		    }
		}
	    }
	} 
	
	for(Turtle pick = sl.firstturtle; pick != null; pick=pick.next) {
	    Object[] newtown = new Object[newNumVars];
	    for(Iterator i = varMap.entrySet().iterator(); i.hasNext(); ) {
		Map.Entry entry = (Map.Entry)i.next();
		int oldPos = ((Integer)entry.getKey()).intValue();
		int newPos = ((Integer)entry.getValue()).intValue();
		newtown[newPos] = pick.turtlesown[oldPos];
	    }
	    for(int i = 0; i < newNumVars; i++) {
		if (!donep[i]) newtown[i] = zeroInt;
	    }
	    pick.turtlesown = newtown;
	}

	vars = newvars;
	numVars = newNumVars;
    }

    public static int getNumVars() {
	return numVars;
    }
    public static String getVarName(int num) {
	return vars[num].name;
    }
    public static int getVarPos(int num) {
	return vars[num].position;
    }
    public static boolean getVarEnump(int num) {
	return vars[num].enump;
    }



    static int numBreeds = 0;
    static String[] breednames = new String[0];
    static int[] breednums = new int[0];

    static public void setBreeds(Object[] town, Context c) {
	numBreeds = town.length;
	breednames = new String[numBreeds];
	breednums = new int[numBreeds];

	for (int i = 0; i < numBreeds; i++) {
	    Object[] pair = YoYo.aList(c, town[i]);
	    breednames[i] = YoYo.aString(c, pair[0]);
	    breednums[i] = YoYo.aInteger(c, pair[1]);
	}

    }

    public static int getNumBreeds() {
	return numBreeds;
    }

    public static String getBreedName(int num) {
	return breednames[num];
    }

    public static int getBreedNum(int num) {
	return breednums[num];
    }


    public String toString() {
    	return Integer.toString(who);
    }

    public String niceToString() {
	return "Turtle #" + toString();
    }

    public String dumpToString() {
	return "Turtle #" + who + "\n" +
	    "patch = " + patch + "\n" +
	    "shape = " + shape + "\n" +
	    "xcor = " + xcor + "\n" +
	    "ycor = " + ycor + "\n" +
	    "heading = " + heading + "\n" +
	    "breed = " + breed + "\n" +
	    "color = " + color + "\n" +
	    "pendownp = " + pendownp + "\n" +
	    "shownp = " + shownp + "\n" +
	    "alivep = " + alive + "\n" +
	    "grabber = " + grabber.niceToString() + "\n";
    }

}
