/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.io.*;
import java.util.zip.*;
import java.util.*;
import yoyo.Symbol;
import yoyo.YoYo;
import yoyo.Ilist;

public class Utilities {

    public static String[] stringTable = new String[1];
    public static Hashtable stringHash = new Hashtable(1);
    public static int stringptr = 0;

    public static Symbol[] symbolTable = new Symbol[1];
    public static Hashtable symbolHash = new Hashtable(1);
    public static int symbolptr = 0;

    public static Double[] doubleTable = new Double[1];
    public static Hashtable doubleHash = new Hashtable(1);
    public static int doubleptr = 0;

    public static Object[][] listTable = new Object[1][];
    public static Hashtable listHash = new Hashtable(1);
    public static int[][] savedListTable = new int[1][];
    public static int listptr = 0;

    public static final int INTTYPE = 1;
    public static final int DOUBLETYPE = 2;
    public static final int SYMBOLTYPE = 3;
    public static final int STRINGTYPE = 4;
    public static final int LISTTYPE = 5;
  
    public static void clearTable() {
	stringTable = new String[1];
	stringHash.clear();
	stringptr = 0;
	symbolTable = new Symbol[1];
	symbolHash.clear();
	symbolptr = 0;
	doubleTable = new Double[1];
	doubleHash.clear();
	doubleptr = 0;
	listTable = new Object[1][];
	listHash.clear();
	savedListTable = new int[1][];
	listptr = 0;
    }

    public static Object[] sortTable(Hashtable hash) {
  	Object[] out = new Object[hash.size()];
  	for(Enumeration e = hash.keys(); e.hasMoreElements(); ) {
	    Object o = e.nextElement();
	    out[((Integer)hash.get(o)).intValue()] = o;
  	}
  	return out;
    }

    public static void dumpStringTable(DataOutputStream out) throws IOException {
	out.writeInt(stringptr);
	Object[] sortedStrings = sortTable(stringHash);
	for (int i = 0; i < stringptr; i++) {
	    String s = (String)sortedStrings[i];
	    out.writeUTF(s);
	}
    }

    public static void dumpSymbolTable(DataOutputStream out) throws IOException {
	out.writeInt(symbolptr);
	Object[] sortedSymbols = sortTable(symbolHash);
	for (int i = 0; i < symbolptr; i++) {
	    Symbol s = (Symbol)sortedSymbols[i];
	    out.writeUTF(s.name);
	}
    }

    public static void dumpDoubleTable(DataOutputStream out) throws IOException {
	out.writeInt(doubleptr);
	Object[] sortedDoubles = sortTable(doubleHash);
	for (int i = 0; i < doubleptr; i++) {
	    Double d = (Double)sortedDoubles[i];
	    out.writeDouble(d.doubleValue());
	}
    }

    public static void dumpListTable(DataOutputStream out) throws IOException {
  	//System.out.println("dumping list table: " + listptr + " entries");
	out.writeInt(listptr);
	for (int i = 0; i < listptr; i++) {
	    //System.out.println("trying to write list# " + i);
	    int[] list = savedListTable[i];
	    out.writeInt(list.length);
	    for(int j = 0; j < list.length; j++) {
		out.writeInt(list[j]);
	    }
	}
    }

    public static void loadStringTable(DataInputStream in) throws IOException {
	int length = in.readInt();
	stringTable = new String[length];
	for(int i = 0; i < length; i++) {
	    stringTable[i] = in.readUTF();
	}
	stringptr = length;
    }

    public static void loadSymbolTable(DataInputStream in) throws IOException {
	int length = in.readInt();
	symbolTable = new Symbol[length];
	for(int i = 0; i < length; i++) {
	    symbolTable[i] = Symbol.lookup(in.readUTF());
	    //System.out.println("Symbol table: " + i + " = " + symbolTable[i]);
	}
	symbolptr = length;
    }

    public static void loadDoubleTable(DataInputStream in) throws IOException {
	int length = in.readInt();
	doubleTable = new Double[length];
	for(int i = 0; i < length; i++) {
	    double d = in.readDouble();
	    doubleTable[i] = new Double(d);
	}
	doubleptr = length;
    }

    public static void loadListTable(DataInputStream in) throws IOException {
	int length = in.readInt();
	listTable = new Object[length][];
	savedListTable = new int[length][];
	for(int i = 0; i < length; i++) {
	    int listlen = in.readInt();
	    Object[] list = new Object[listlen];
	    listTable[i] = list;
	    int[] savedList = new int[listlen];
	    savedListTable[i] = savedList;
	    for(int j = 0; j < listlen; j++) {
		savedList[j] = in.readInt();
	    }
	}
	for(int i = 0; i < length; i++) {
	    Object[] list = listTable[i];
	    int[] savedList = savedListTable[i];
	    int listlen = list.length;
	    for(int j = 0; j < listlen; j++) {
		list[j] = lookupInTable(savedList[j]);
	    }
	}
	listptr = length;
    }

    public static int addIntoTable(String s) {
  	Object val = null;
  	if ((val = stringHash.get(s)) != null) {
	    return (STRINGTYPE << 24) | ((Integer)val).intValue();
  	}
  	stringHash.put(s, new Integer(stringptr));
  	return (STRINGTYPE << 24) | stringptr++;
    }

    public static int addIntoTable(Symbol s) {
  	Object val = null;
  	if ((val = symbolHash.get(s)) != null) {
	    return (SYMBOLTYPE << 24) | ((Integer)val).intValue();
  	}
  	symbolHash.put(s, new Integer(symbolptr));
  	return (SYMBOLTYPE << 24) | symbolptr++;
    }

    public static int addIntoTable(Double d) {
	//System.out.println("doubleptr = " + doubleptr);
	//System.out.println("adding double " + d);
  	Object val = null;
  	if ((val = doubleHash.get(d)) != null) {
	    return (DOUBLETYPE << 24) | ((Integer)val).intValue();
  	}
  	doubleHash.put(d, new Integer(doubleptr));
  	return (DOUBLETYPE << 24) | doubleptr++;
    }

    public static int addIntoTable(int in) {
  	if (((in >>> 24) & 0xFF) != 0) return addIntoTable(new Double(in));
  	return (INTTYPE << 24) | in;
    }

    public static int addIntoTable(Integer in) {
	int i = in.intValue();
	if (((i >>> 24) & 0xFF) != 0) return addIntoTable(new Double(i));
	return (INTTYPE << 24) | i;
    }

    public static int addIntoTable(SliderWidget sw) {
	return addIntoTable(sw.id);
    }
    
    public static int addIntoTable(MonitorWidget mw) {
	return addIntoTable(mw.id);
    }
    

    public static int addIntoTable(Object[] list, StarLogo sl) {
	//System.out.println("adding list# " + listptr + " into table. list = " + YoYo.printToString(list));
  	Object val = null;
  	if ((val = listHash.get(list)) != null) {
	    //System.out.println("it was already in the table");
	    return (LISTTYPE << 24) | ((Integer)val).intValue();
  	}

	if (listptr >= savedListTable.length) {
	    //System.out.println("doubling size of savedListTable to " + savedListTable.length * 2);
	    int[][] newstab = new int[savedListTable.length * 2][];
	    System.arraycopy(savedListTable, 0, newstab, 0, savedListTable.length);
	    savedListTable = newstab;
	}
  	listHash.put(list, new Integer(listptr));
  	
	int[] savedList = new int[list.length];
	int thislistptr = listptr++;
	savedListTable[thislistptr] = savedList;
	for(int i = 0; i < list.length; i++) {
	    savedList[i] = addIntoTable(list[i], sl);
	}
  	return (LISTTYPE << 24) | thislistptr;
    }

    public static int addIntoTable(Object val, StarLogo sl) {
	if (val instanceof String) return addIntoTable((String)val);
	else if (val instanceof Symbol) return addIntoTable((Symbol)val);
	else if (val instanceof Double) return addIntoTable((Double)val);
	else if (val instanceof Integer) return addIntoTable((Integer)val);
	else if (val instanceof Object[]) return addIntoTable((Object[])val, sl);
	else if (val instanceof SliderWidget) return addIntoTable((SliderWidget)val); 
	else if (val instanceof MonitorWidget) return addIntoTable((MonitorWidget)val);
	else if (val instanceof Ilist) return addIntoTable(((Ilist)val).list, sl);
	else StarLogoError.error(sl.app, "Can't save object of type " + val.getClass(), true);
	return 0;
    }

    public static Object lookupInTable(int i) {
	int typ = (i >>> 24) & 0xFF;
	//System.out.println("lookup in table: type; " + typ + " index " + (i & 0xFFFFFF));
	switch (typ) {
	case STRINGTYPE: return stringTable[(i & 0xFFFFFF)];
	case SYMBOLTYPE: return symbolTable[(i & 0xFFFFFF)];
	case DOUBLETYPE: return doubleTable[(i & 0xFFFFFF)];
	case LISTTYPE: return listTable[(i & 0xFFFFFF)];
	case INTTYPE: return new Integer(i & 0xFFFFFF);
	}
	return null;
    }


    public static int writeIntoByteArray(int val, byte[] out, int offset) {
	// big endian
	out[offset] = (byte)((val >>> 24) & 0xFF);
	out[offset+1] = (byte)((val >>> 16) & 0xFF);
	out[offset+2] = (byte)((val >>> 8) & 0xFF);
	out[offset+3] = (byte)(val & 0xFF);
	return 4;
    }

    public static int writeIntoByteArray(double val, byte[] out, int offset) {
	// big endian
	long lval = Double.doubleToRawLongBits(val);
	writeIntoByteArray((int)(lval >>> 32), out, offset);
	//System.out.println("write int 0x" + Integer.toString(((int)(lval >>> 32)), 16));
	writeIntoByteArray((int)(lval & 0xFFFFFFFFL), out, offset + 4);
	//System.out.println("write int 0x" + Integer.toString(((int)(lval & 0xFFFFFFFFL)), 16));
	//System.out.println("wrote double " + val + " long 0x" + Long.toString(lval, 16));
	double newval = readDoubleFromByteArray(out, offset);
	//if (newval != val) {
	//System.out.println(val + " != " + newval);
	//}
	return 8;
    }

    public static int writeIntoByteArray(boolean val, byte[] out, int offset) {
	out[offset] = (val) ? (byte)1 : (byte)0;
	return 1;
    }

    public static int readIntFromByteArray(byte[] out, int offset) {
	// big endian
	int val = ((((int)out[offset]) & 0xFF) << 24) |
	    ((((int)out[offset+1]) & 0xFF) << 16) | 
	    ((((int)out[offset+2]) & 0xFF) << 8) |
	    (((int)out[offset+3]) & 0xFF);
	//System.out.println("read int 0x" + Integer.toString(val, 16));
	return val;
    }

    public static double readDoubleFromByteArray(byte[] out, int offset) {
	// big endian
	long val = ((long)readIntFromByteArray(out, offset) << 32) |
	    (readIntFromByteArray(out, offset + 4) & 0xFFFFFFFFL);
	//System.out.println("read double " + Double.longBitsToDouble(val) + " long 0x" + 
	//		       Long.toString(val, 16));
	return Double.longBitsToDouble(val);
    }

    public static boolean readBooleanFromByteArray(byte[] out, int offset) {
	byte val = out[offset];
	if (val == 0) return false;
	else return true;
    }

    public static int getEntrySize(byte[] extra) {
	int end = extra.length - 4;
	return readIntFromByteArray(extra, end);
    }

    public static void setEntrySize(ByteArrayOutputStream extra, int size) {
	extra.write((byte)((size >> 24) & 0xff));
	extra.write((byte)((size >> 16) & 0xff));
	extra.write((byte)((size >> 8) & 0xff));
	extra.write((byte) (size & 0xff));
    }


    public static byte[] gzipByteArray(byte[] bytes) throws IOException {
	ByteArrayOutputStream baos;
	GZIPOutputStream gzout = new GZIPOutputStream(baos = new ByteArrayOutputStream());
	gzout.write(bytes);
	gzout.finish();
	gzout.flush();
	setEntrySize(baos, bytes.length);
	gzout.close();
	return baos.toByteArray();
    }

    public static byte[] gunzipByteArray(byte[] compbytes) throws IOException {
	int length = getEntrySize(compbytes);
	try {
	    GZIPInputStream zip = new GZIPInputStream(new ByteArrayInputStream(compbytes));
      
	    byte[] bytes = new byte[length];
      
	    int count = 0;
	    do {
		int numread = zip.read(bytes, count, length - count);
		if (numread < 0) break;
		count += numread;
	    } while(count < length);

	    return bytes;
	}
	catch (IOException e) { e.printStackTrace(); }
	return null;
    }

    // return a string that can be read back in using the yoyo tokenizer
    public static String printable(String s) {
	assert s != null;
	StringBuffer sb = new StringBuffer(s.length());
	for(int i = 0; i < s.length(); i++) {
	    char c = s.charAt(i);
	    if (c == '"') sb.append("\\\"");
	    else if (c == '\n') sb.append("\\n");
	    else sb.append(c);
	}
	return sb.toString();
    }

    public static String saveIlist(int[] input) throws IOException {
	byte[] out = new byte[input.length * 4];
	int index = 0;
	for (int i = 0; i < input.length; i++) {
	    writeIntoByteArray(input[i], out, index);
	    index += 4;
	}
	byte[] compbytes = gzipByteArray(out);
	return yoyo.Base64.encode(compbytes);
    }


    public static int[] restoreIlist(String input, int numbytecodes) throws IOException {	 
	byte[] in = gunzipByteArray(yoyo.Base64.decode(input.getBytes()));
	int[] out = new int[numbytecodes];
	 
	int index = 0;
	for (int i = 0; i < out.length; i++) {
	    out[i] = readIntFromByteArray(in, index);
	    index += 4;
	}
	return out;
    }


    public static String saveOlist(Object[] input, StarLogo sl) throws IOException {
	byte[] out = new byte[input.length * 4];
	int off = 0;
	for (int i = 0; i < input.length; i++) {
	    int index = addIntoTable(input[i], sl);	 
	    writeIntoByteArray(index, out, off);
	    off += 4;
	}
	byte[] compbytes = gzipByteArray(out);
	return yoyo.Base64.encode(compbytes);
    }

    public static Object[] restoreOlist(String input, int numbytecodes) throws IOException {	 
	byte[] in = gunzipByteArray(yoyo.Base64.decode(input.getBytes()));
	Object[] out = new Object[numbytecodes];
	 
	int index = 0;
	for (int i = 0; i < out.length; i++) {
	    int intval = readIntFromByteArray(in, index);
	    index += 4;
	    out[i] = lookupInTable(intval);
	}
	return out;
    }


}
