/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.util.Observable;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.net.*; 
import yoyo.YoYo;
import java.io.IOException;

public abstract class Widget extends WidgetLite {
	
  public static int trackerID = 0;
  public JInterface parent;
  protected boolean mouseDown = false;

	
  public Widget(JInterface parent, Image im) {
      super(parent, im);
    this.parent = parent;
    position = new Rectangle(0, 0, (im == null) ? 0 : im.getWidth(this), 
			     (im == null) ? 0 : im.getHeight(this));
    //System.out.println("widget's position is " + position);
  }
	
  public Widget(JInterface parent) {
      super(parent);
    this.parent = parent;
  }
	
  public Widget() {
    position = new Rectangle(0, 0, 10, 10);
  }
	
	
	public void execute() { execute(null); }
	
	public void execute(MouseEvent e) { if (e != null) mouseClicked(e); }
	
    void moveToFront() {
	parent.moveToFront(this);
	parent.repaint(position);
    }

    void mouseDoubleClicked(MouseEvent e) {
	if (isSelected()) {
	    edit();
	    parent.deselect(this);
	}
    }

    public boolean isSelected() { 
	return false;
    }

    public void delete() {
       removeToolTip();
    }

    public void edit() {
	parent.deselect(this);
	parent.addHandlesToClipRect(this);
	parent.getToolbar().setButtonDown(ToolbarWidget.POINTER);
    }

    public boolean resizable() {
	return false;
    }

    public boolean draggable(MouseEvent e) {
	return false;
    }

    public int resizeWhichCorner(Point anchor) {
	return 0;
    }

    public void resize(Point where, int resizeCorner) {
    }

    public void endResize(MouseEvent e, int resizeCorner) {}

  //  public void mouseReleased(MouseEvent e) {}

      public static boolean editableWidget(Widget w) {
  	    if ((w instanceof ButtonWidget) || (w instanceof TextWidget) ||
  	     (w instanceof SliderWidget) || (w instanceof MonitorWidget))
  		return true;
  	return false;
  }

  public void displayToolTip(Point dwellPoint) {
     synchronized(toolTipLock) {
        if ((toolTip != null) && !(showingToolTip) && (parent != null)) {
	    if (parent.sl() != null && parent.sl().runningnowp && 
		parent.getPatchCanvas() instanceof WidgetDrag) {
		toolTip.setPosition(position, ((WidgetDrag)parent.getPatchCanvas()).position);
            } else {
		toolTip.setPosition(position);
	    }
	    parent.setTooltipWidget(toolTip);
	    showingToolTip = true;
            //parent.addWidget(toolTip);
            parent.repaint(toolTip.position);
        }
     }
     // System.out.println("display tool tip " + this.toString());
  }

  public void dropToolTip()  {
       if ((toolTip != null) && showingToolTip && (parent != null)) { 
	   //parent.removeWidget(toolTip);
          showingToolTip = false;
	  parent.setTooltipWidget(null);
          parent.repaint(toolTip.position);
       }
     // System.out.println("stop displaying tool tip " + this.toString());
  }	

    public Rectangle getPosition() {
	return position;
    }

}
