/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import yoyo.YoYo;
import java.lang.Math;
import java.awt.event.*;
import java.awt.*;
import java.awt.image.*;
import java.net.*;

public class WidgetDrag extends Widget {

  public int MIN_WIDTH = 30;
  public int MIN_HEIGHT = 30;
  public static final int CORNER_WIDTH = 8;

  //  public static boolean beingDragged = false;
  public static int xHandle = 0, yHandle = 0;
   // public static WidgetDrag[] widgetDragging = null;
   // public static Rectangle widgetDraggingPosition = null;
    //public static WidgetDrag widgetDragging = null;
  //public int cursorNum = Cursor.MOVE_CURSOR;
  //public Cursor previousCursor = Cursor.getDefaultCursor();
  protected boolean selected = false;
  //public boolean resizable = true;

   // static Rectangle boundaries = new Rectangle();
  Rectangle oldPosition = new Rectangle();
  //static Rectangle clipRect = new Rectangle();
    //  Timer timer;

  public WidgetDrag(JInterface parent, Image image) {
    super(parent, image);
  }

  public WidgetDrag(JInterface parent) {
    super(parent);
  }

  public WidgetDrag() {}

  protected WidgetDrag(WidgetDrag old) {
    //beingDragged = old.beingDragged;
    //    resizing = false;
      if (old != null) {
	  xHandle = old.xHandle; yHandle = old.yHandle;
	  //cursorNum = old.cursorNum;
	  //previousCursor = old.previousCursor;
	  selected = old.selected;
	  //boundaries = old.boundaries;
	  oldPosition = old.oldPosition;
	  //clipRect = old.clipRect;
	  mouseDown = old.mouseDown;
      } else {
	  xHandle = 0; yHandle = 0;
	  selected = false;
	  oldPosition = new Rectangle();
	  mouseDown = false;
      }
  }

  public boolean isSelected() {
    return selected;
  }

  public void setSelected (boolean b) {
    selected = b;
    //parent.select(this, b);
  }


    protected void drawCorners(Graphics g) {
    g.translate(position.x,position.y);
    // draw black square
    g.setColor(Color.darkGray);
    g.fillRect(-CORNER_WIDTH, -CORNER_WIDTH, CORNER_WIDTH, CORNER_WIDTH);
    g.fillRect(position.width, -CORNER_WIDTH, CORNER_WIDTH, CORNER_WIDTH);
    g.fillRect(position.width, position.height, CORNER_WIDTH, CORNER_WIDTH);
    g.fillRect(-CORNER_WIDTH, position.height, CORNER_WIDTH, CORNER_WIDTH);
    g.translate(-position.x,-position.y);
  }

  public void checkBounds(Rectangle boundaries) {
    if (position.x < boundaries.x)
      position.x = boundaries.x;
    if (position.y < boundaries.y)
      position.y = boundaries.y;
    if (position.x + position.width > boundaries.x + boundaries.width)
      position.width = boundaries.x + boundaries.width - position.x;
    if (position.y + position.height > boundaries.y + boundaries.height)
      position.height = boundaries.y + boundaries.height - position.y;
  }

  public void resize(Point where, int resizeCorner) {
	parent.addHandlesToClipRect(this);
      int x = where.x; int y = where.y;
      switch(resizeCorner) {
      case JInterface.RESIZETOPLEFT:
	  if (position.width-x+position.x<MIN_WIDTH)
	      x=-MIN_WIDTH+position.x+position.width;
	  if (position.height-y+position.y<MIN_HEIGHT)
	      y=-MIN_HEIGHT+position.y+position.height;
	  position.width-=x-position.x;
	  position.x=x;
	  position.height-=y-position.y;
	  position.y=y;
	  break;
      case JInterface.RESIZEBOTLEFT:
	  if (position.width-x+position.x<MIN_WIDTH)
	      x=-MIN_WIDTH+position.x+position.width;
	  if (y-position.y<MIN_HEIGHT)
	      y=MIN_HEIGHT+position.y;
	  position.width-=x-position.x;
	  position.x=x;
	  position.height+=y-position.y-position.height;
	  break;
      case JInterface.RESIZEBOTRIGHT:
	  if (x-position.x<MIN_WIDTH)
	      x=MIN_WIDTH+position.x;
	  if (y-position.y<MIN_HEIGHT)
	      y=MIN_HEIGHT+position.y;
	  position.width+=x-position.x-position.width;
	  position.height+=y-position.y-position.height;
	  break;
      case JInterface.RESIZETOPRIGHT:
	  if (x-position.x<MIN_WIDTH)
	      x=MIN_WIDTH+position.x;
	  if (position.height-y+position.y<MIN_HEIGHT)
	      y=-MIN_HEIGHT+position.y+position.height;
	  position.width+=x-position.x-position.width;
	  position.height-=y-position.y;
	  position.y=y;
	  break;
      case JInterface.RESIZERIGHT:
	  if (x-position.x<MIN_WIDTH)
	      x=MIN_WIDTH+position.x;
	  position.width+=x-position.x-position.width;
	  break;
      case JInterface.RESIZELEFT:
	  if (position.width-x+position.x<MIN_WIDTH)
	      x=-MIN_WIDTH+position.x+position.width;
	  position.width-=x-position.x;
	  position.x=x;
	  break;
      case JInterface.RESIZETOP:
	  if (position.height-y+position.y<MIN_HEIGHT)
	      y=-MIN_HEIGHT+position.y+position.height;
	  position.height-=y-position.y;
	  position.y=y;
	  break;
      case JInterface.RESIZEBOT:
	  if (y-position.y<MIN_HEIGHT)
	      y=MIN_HEIGHT+position.y;
	  position.height+=y-position.y-position.height;
	  break;
      }
      parent.addHandlesToClipRect(this);
  }

  void moveBy(Point displacement) {
      position.translate(displacement.x, displacement.y);
  }

    public boolean resizable() {
	return true;
    }

    public boolean draggable(MouseEvent e) {
    	if (parent.app.kioskp || parent.app.locked || YoYo.applet != null)
    		return false;
    	else
	    return true;
    }

    public int resizeWhichCorner(Point anchor) {
	Rectangle test = new Rectangle();
	test.setBounds(position.x-CORNER_WIDTH, position.y-CORNER_WIDTH,
		       CORNER_WIDTH, CORNER_WIDTH);
	if (test.contains(anchor.x, anchor.y)) return JInterface.RESIZETOPLEFT;

	test.setBounds(position.x+position.width, position.y-CORNER_WIDTH,
		       CORNER_WIDTH, CORNER_WIDTH);
	if (test.contains(anchor.x, anchor.y)) return JInterface.RESIZETOPRIGHT;

	test.setBounds(position.x-CORNER_WIDTH, position.y+position.height,
		       CORNER_WIDTH, CORNER_WIDTH);
	if (test.contains(anchor.x, anchor.y)) return JInterface.RESIZEBOTLEFT;

	test.setBounds(position.x+position.width, position.y + position.height,
		       CORNER_WIDTH, CORNER_WIDTH);
	if (test.contains(anchor.x, anchor.y)) return JInterface.RESIZEBOTRIGHT;

	return 0;
    }

  public boolean intersects(Rectangle rect) {
    if (selected) {
      Rectangle clipRect = new Rectangle(position);
      clipRect.grow(CORNER_WIDTH, CORNER_WIDTH);
      return(clipRect.intersects(rect));
    } else
      return(super.intersects(rect));
  }

    public boolean cornerContains(Point p) {
	return (resizeWhichCorner(p) != 0);
    }

  public boolean contains(int x, int y) {
    return(super.contains(x, y));
  }

  public int getToolbarButton() {
    // what button on the toolbar corresponds to this widgetdrag-- overrridden
    return ToolbarWidget.NONE;
  }
  
  public WidgetDrag copy(int edition, Rectangle r) { return null; }
  
  public static String editionString(int edition) {
		if (edition == 1)
			return "";
		else
			return (" " + edition);
  }
  
  public void snapToGrid() {
      parent.addToClipRect(position);
      int xcor = position.x;
      int ycor = position.y;
      int x = (xcor % BlueGrid.gridunit);
      int y = ((ycor-parent.toolbarheight()) % BlueGrid.gridunit);
      if (x != 0)
	  		position.x -= x;
      if (y != 0)
	  		position.y -= y; 
      parent.addToClipRect(position);
  }
    
  public void mouseReleased(MouseEvent e) {
      if (parent.trashwidget().pressed())
	  parent.trashwidget().execute();
      if (parent.snapToGrid())
	  snapToGrid();
      parent.maybeReleasePatchCanvas();
  }

}

