/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.awt.Rectangle;

import java.util.Vector;
import java.util.Enumeration;
import yoyo.YoYo;

public class WidgetHolder {
	
	Application app;
	JInterface iface;
	
	private PatchCanvas patchcanvas;
    Widget[] toolbarwidgets = new Widget[10];
    BlueGrid bluegrid;
	private Vector widgets;
	private int selections;
	
	public boolean locked;
	public Object synchObject = new Object();
	
	public WidgetHolder(Application app, JInterface iface) {
		this(app, iface, null);
	}
	
	public WidgetHolder(Application app, JInterface iface, PatchCanvas pc) {
		this.app = app;
		this.iface = iface;
		widgets = new Vector();
		this.patchcanvas = pc;
		selections = 0;
	}
	
	public void setPatchCanvas(PatchCanvas pc) {
		patchcanvas = pc;
	}
	
	public PatchCanvas getPatchCanvas() {
		return patchcanvas;
	}

    public void removePatchCanvas() {
	patchcanvas = null;
    }

	public void addToolbarWidget(Widget tw) {
	    for (int i = 0; i < toolbarwidgets.length; i++) {
		if (toolbarwidgets[i] == null) {
		    toolbarwidgets[i] = tw;
		    return;
		}
	    }
	    app.err.println("Internal Error: Not enough toolbar widget slots!");
	    new Exception().printStackTrace(app.err);
	}

    public int numToolbarWidgets() {
	int count = 0;
	for(int i = 0; i < toolbarwidgets.length; i++) {
	    if (toolbarwidgets[i] != null) count++;
	} 
	return count;
    }

    public void removeToolbarWidget(Widget tw) {
	for (int i = 0; i < toolbarwidgets.length; i++) {
	    if (toolbarwidgets[i] == tw) {
		toolbarwidgets[i] = null;
	    } 
	}
    }

	public void addGrid(BlueGrid bg) {
		bluegrid = bg;
	}
	
	public void setGrid(boolean b) {
	    if (app.kioskp || YoYo.applet != null) return;
		if (b != bluegrid.on) {
		   bluegrid.on = b;
		   iface.addToClipRect(bluegrid.position); //repaint();
		}
	}
	
	public boolean contains(Widget wd) {
		if  ( (widgets.indexOf(wd) != -1) || (wd == getPatchCanvas()) )
			return true;
		return false;
	}

	public void addWidget(Widget wd) {
		//System.out.println("New widget " + wd.toString() + " has been added.");
		if (wd.isSelected()) {			
			incSelections();
			//System.out.println("You have added a selected widget");
			}
		widgets.addElement(wd);
		//app.project.compile(app);
		//System.out.println("There are now " + widgets.size() + " widgets in total.");
	}

	public void removeWidget(Widget wd) {
		int i;
		if ( (-1 == (i = widgets.indexOf(wd))) && (!(wd.equals( getPatchCanvas() ))) )
		    return; // the widget is not contained in this holder
		if ( wd.isSelected() )
		    decSelections();
		if (i > -1)
		    widgets.removeElementAt(i);
		else
		    setPatchCanvas(null);
		//app.project.compile(app);
		//System.out.println("Widget " + wd.toString() + " has been removed");
	}
	
	public void moveToFront(Widget wd) {
		if (wd instanceof PatchCanvas) return;
		widgets.removeElement(wd);
		widgets.addElement(wd);
	}

	public void removeAllWidgets() {
		widgets.removeAllElements();
		if (patchcanvas != null && patchcanvas.isSelected() )
			selections = 1;
		else
			selections = 0;
	}
	
	public Vector getDragWidgets() {
                Vector out = new Vector();
                for (int i = 0; i < widgets.size(); i++) {
			Widget o = (Widget)widgets.elementAt(i);
                        if (o instanceof WidgetDrag) out.addElement(o);
		}
		return out;
	}

	public Vector getWidgets() {
		return widgets;
	}
	
	private void incSelections() {
		selections++;
		//System.out.println(Integer.toString(selections));
	}
	
	private void decSelections() {
		selections--;
		//System.out.println(Integer.toString(selections));
	}

	public void setSelected(WidgetDrag wd, boolean b) {
	    if (wd == null) return;
	    if (b == wd.isSelected()) return;
	    if (wd.isSelected() && (!b)) {
  		decSelections();
  		//System.out.println(wd.toString() + " was deselected.");
	    }
	    if ((!wd.isSelected()) && b) {
  		incSelections();
  		//System.out.println(wd.toString() + " was selected.");
	    }
	    wd.setSelected(b);
  	/*
  	if (wd != patchcanvas)
    	wd.setSelected(b);
    else {
    	//iface.repaint();
    	patchcanvas.setSelected(b);
    }
    */
    //System.out.println("selections is at " + Integer.toString(selections));
  }

	public boolean hasSelections() {
		return (selections > 0);
	}

	public WidgetDrag[] getSelectedWidgets(boolean b) {
		// the boolean indicates if the patchcanvas should be included
		if ((selections == 0) || ( (!b) && (selections == 1) && (patchcanvas != null && patchcanvas.isSelected()) ))
			return null;
		WidgetDrag[] wda = null;
		if ( patchcanvas != null && patchcanvas.isSelected() && (!b) )
			wda = new WidgetDrag[selections-1];
		else
			wda = new WidgetDrag[selections];
		int j=0;
		for (int i=0; i<widgets.size(); i++) {
			Widget wd = (Widget)widgets.elementAt(i);
			if ( wd.isSelected() )
				wda[j++] = (WidgetDrag)wd;
		}
		if (j < wda.length && patchcanvas != null)
			wda[j++] = patchcanvas;
		return wda;
	}
	
	public Rectangle getSelectRectWithHandles() {
		Rectangle r = getSelectRect();
		r.grow(WidgetDrag.CORNER_WIDTH, WidgetDrag.CORNER_WIDTH);
		return r;
	}
	
	public Rectangle getSelectRect() {
		Rectangle result = new Rectangle();
		WidgetDrag[] wda = getSelectedWidgets(true);
		//System.out.println("getting select rect of selected widgets: " + YoYo.printToString(wda));
		for (int i = 0 ; i<wda.length; i++) {
		    //System.out.println("widget #" + i + " position = " + wda[i].position);
			result = result.union(wda[i].position);
		}
		return result;
	}
	
	public void clearSelections() {
		//for (Enumeration e = listDragWidgets(); e.hasMoreElements(); )
		for (Enumeration e = widgets.elements(); e.hasMoreElements(); )
			setSelected((WidgetDrag)e.nextElement(),false);
		setSelected(patchcanvas,false);
		if (selections != 0)
			//System.out.println("clearing selections should have brought the counter to zero, but it didn't!");
		selections = 0;
	}

	public Enumeration listDragWidgets() {
		return new WHEnumeration(this,"drag");
	}

	public Enumeration listAllWidgets() {
		return new WHEnumeration(this,"all");
	}

	public Enumeration listAllWidgetsTopDown() {
		return new WHEnumeration(this,"all",true);
	}

    public Enumeration listToolbars() {
	return new WHEnumeration(this, "toolbars");
    }
}


class WHEnumeration implements Enumeration {
    
    WidgetHolder wh;
	Enumeration e;
	boolean locked;
	
	public WHEnumeration(WidgetHolder wh, String str) {
	    this(wh, str, false);
	}

	public WHEnumeration(WidgetHolder wh, String str, boolean reversep) {
		wh.locked = true;
		this.wh = wh;
		Vector v = new Vector();
		if (str.equals("drag") || str.equals("all")) {
		    if ((wh.iface.draggingp() || wh.iface.resizingp()) && wh.iface.snapToGrid() && wh.bluegrid.on)
					v.addElement( wh.bluegrid );
		    if (wh.getPatchCanvas() != null) 
			v.addElement( wh.getPatchCanvas() );
		}
		if (!wh.app.kioskp && YoYo.applet == null && 
		    (str.equals("all") || str.equals("toolbars"))) {
		    	//System.out.println(String.valueOf(numToolbarWidgets()));
		    for (int i = 0; i < wh.toolbarwidgets.length; i++) {
			if (wh.toolbarwidgets[i] != null) 
			    v.addElement( wh.toolbarwidgets[i] );
		    }
		}
		if (str.equals("drag")) {
		    Vector widgets = wh.getDragWidgets();
		    for (int i=0; i<widgets.size(); i++)
			v.addElement( widgets.elementAt(i) );
		}
                if (str.equals("all")) {
                    Vector widgets = wh.getWidgets();
                    for (int i=0; i < widgets.size(); i++)
                        v.addElement(widgets.elementAt(i));
                }
		setEnumeration(v, reversep);
	}

	private void setEnumeration(Vector v, boolean reversep) {
	    if (reversep) {
		Vector obj = new Vector(v.size());
		for(int i = v.size()-1; i >= 0; i--) {
		    obj.addElement(v.elementAt(i));
		}
		e = obj.elements();
	    } else e = v.elements();
	}

	
	public boolean hasMoreElements() {
		boolean b = e.hasMoreElements();
		if (!b) {
			wh.locked = false;
		}
		return b;
	}
	
	public Object nextElement() {
		return e.nextElement();
	}
}
    







