/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import java.util.Observable;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.net.*; 
import yoyo.YoYo;
import java.io.IOException;

public abstract class WidgetLite extends Observable implements java.awt.image.ImageObserver {
	
  public Rectangle position;
  public Image image;
  public WidgetOwner owner;

  public static Font WIDGET_FONT;
	
  static {
    // jview makes Dialog 10 pt as large as Dialog 12 pt, which is too big for sliders & monitors
    try {
    	if (System.getProperty("java.vendor").toLowerCase().indexOf("microsoft")!=-1) {
      		WIDGET_FONT = new Font("SansSerif", Font.PLAIN, 10);
    	} else {
      		WIDGET_FONT = new Font("Dialog", Font.PLAIN, 10);
    	}
    	if (YoYo.macintoshp) WIDGET_FONT = Font.getFont("starlogo.widget.font", WIDGET_FONT);
  	} catch (Exception e) { e.printStackTrace(); }
  }
	
  public WidgetLite() {
    position = new Rectangle(0,0,0,0);
  }

  public WidgetLite(WidgetOwner owner) {
	this.owner = owner;
	position = new Rectangle(0,0,0,0);
  }

  public WidgetLite(WidgetOwner owner, Image image) {
    this.owner = owner;
    this.image = image;
    position = new Rectangle(0,0,0,0);
  }
	
  public boolean imageUpdate(Image img, int infoflags, int x, int y, int width, int height) {
    return true;
  }
	
  public void update(Graphics g) {
    paint(g);
  }
	
  public void paint(Graphics g) {
      //System.out.println("paint " + this);
      g.drawImage(image, position.x, position.y, this);
  }
	
  void mouseClicked(MouseEvent e) {}
  void mouseDoubleClicked(MouseEvent e) {}
  void mousePressed(MouseEvent e) {}
  void mouseDragged(MouseEvent e) {}
  void keyTyped(char c) { }
  void mouseReleased(MouseEvent e) {}
	
  public void setLocation(int x, int y) {
    position.x = x;
    position.y = y;
  }
	
  public void setLocation(Point p) {
    position.setLocation(p);
  }
	
  public Point getLocation() {	
    return position.getLocation();
  }
	
  public void setSize(int width, int height) {
    position.width = width;
    position.height = height;
  }
	
  public boolean contains(int x, int y) {
    return(position.contains(x, y));
  }
	
  public boolean intersects(Rectangle rect) {
    return(position.intersects(rect));
  }
	
  public void update() {}
	
  public String getSaveString(boolean htmlp) throws IOException { return ""; }
  
  public boolean isSelected() { 
	return false;
  }

  public void delete() {
     removeToolTip();
  }

  protected ToolTipWidget toolTip;  
  protected boolean showingToolTip = false;
  public Object toolTipLock = new Object();

  public ToolTipWidget getToolTip() {
	return toolTip;
  }

  public String getToolText() {
	String out = null;
        if (toolTip != null) out = toolTip.getText();
        return out;
  }

  public String getToolTipSaveString() {
        String out = new String();
	if (toolTip != null) out = toolTip.getSaveString();
        return out;
  }

  public void setToolTip(ToolTipWidget tp) {
     synchronized(toolTipLock) {
        dropToolTip();
        toolTip = tp;
     }
  }

  public void removeToolTip() {
     synchronized(toolTipLock) {
        dropToolTip();
        toolTip = null;
     }
  }
       
  public void displayToolTip(Point dwellPoint) {
     synchronized(toolTipLock) {
        if ((toolTip != null) && !(showingToolTip) && (owner != null)) {
	    toolTip.setPosition(position);
	    owner.setTooltipWidget(toolTip);
	    showingToolTip = true;
	    //owner.addWidget(toolTip);
	    owner.repaint(toolTip.position);
        }
     }
     // System.out.println("display tool tip " + this.toString());
  }

  public void dropToolTip()  {
       if ((toolTip != null) && showingToolTip && (owner != null)) { 
	   //owner.removeWidget(toolTip);
          showingToolTip = false;
	  owner.setTooltipWidget(null);
          owner.repaint(toolTip.position);
       }
     // System.out.println("stop displaying tool tip " + this.toString());
  }	
 
}
