/**********************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *********************************************************************/



/* BaseFunc is the base class for all of the function classes.  Takes
 * care of some features common to all functions, like displaying the
 * name, replacing strings, and updating breed arguments' list of
 * breeds.
 * 
 * All function classes replace certain elements (words starting with
 * ":") in "final" strings (the raw function) with the values from
 * their arguments and output them through the getBody() function.
 * 
 * Almost all functions use the template.argument classes to offer
 * choices to the user.  There is the SliderArg, BreedArg, ChoiceArg
 * and ColorArg.  
 */

package template;

import template.arguments.*;
import javax.swing.*;
import java.awt.*;
import java.util.*;  //for Java2 collections

public abstract class BaseFunc extends JPanel implements Comparable {
  protected JLabel funcName;
  protected SliderArg[] sliderArgs;
  protected BreedArg[] breedArgs;
  protected ColorArg[] colorArgs;
  protected ChoiceArg[] choiceArgs;
  protected boolean obsFuncP = false,
    inWanderP = true,
    predefinedP = false;
  protected Collection breeds;
  public final static int TURTLE_VARS = 0,
    GLOBAL_VARS = 1,
    PATCH_VARS = 2;
  public static String functionWord = "Function";
  
  protected BaseFunc(){
    funcName = new JLabel();
    Font f = funcName.getFont();
    funcName.setFont(new Font(f.getName(), Font.BOLD, 16));
  }
  public static String replace(String in, String toReplace, String with){
    return Util.replace(in, toReplace, with);
  }
  public void updateBreeds(){
    if(breedArgs == null)
      return;
    for(int i = 0; i < breedArgs.length; i++){
      breedArgs[i].updateBreeds(breeds);
    }
  }
  abstract public String getBody();
  abstract public String describe();
  public String getName(){
    return funcName.getText();
  }
  public String toString(){
    return getName();
  }
  public String getCommand(){
    return toCommand(funcName.getText());
  }
  static public String toCommand(String input){
    return replace(input.toLowerCase(), " ", "-");
  }
  static public String quote(String input){
    return Util.quote(input);
  }
  public boolean finished(){
    return true;
  }
  public SliderInfo[] getSliderInfo(){
    if(sliderArgs == null)
      return null;
    Collection newSliders = new ArrayList();
    for(int i = 0; i < sliderArgs.length; i++){
      if(sliderArgs[i].isNewSlider())
	newSliders.add(sliderArgs[i].getSliderInfo());
    }
    SliderInfo[] dummy = new SliderInfo[0];
    return (SliderInfo[])newSliders.toArray(dummy);
  }
  public String writeSliders(){
    SliderInfo[] sliders = getSliderInfo();
    String body = "";
    if(sliders == null)
      return body;
    for(int i = 0; i < sliders.length; i++){
      body += sliders[i].writeSlider();
      SliderInfo.y += 30;
      ButtonInfo.y += 30;
      SliderInfo.sliders++;
    }
    return body;
  }
  public ButtonInfo[] getButtonInfo(){
    return null;
  }
  public String writeButtons(){
    ButtonInfo[] buttons = getButtonInfo();
    String body = "";
    if(buttons == null)
      return body;
    for(int i = 0; i < buttons.length; i++){
      body += buttons[i].writeButton();
      ButtonInfo.y += 40;
      SliderInfo.y += 40;
      ButtonInfo.buttons++;
    }
    return body;
  }
  public boolean isObsFunc(){
    return obsFuncP;
  }
  public boolean isInWander(){
    return inWanderP;
  }
  public boolean isPredefined(){
    return predefinedP;
  }
  public String[] getVariables(int variableType){
    return null;
  }
  public String save(){
    int i;
    String output = functionWord + " " + getName() + "\n";
    if(sliderArgs != null){
      for(i = 0; i < sliderArgs.length; i++)
	output += sliderArgs[i].save();
    }
    if(breedArgs != null){
      for(i = 0; i < breedArgs.length; i++)
	output += breedArgs[i].save();
    }
    if(colorArgs != null){
      for(i = 0; i < colorArgs.length; i++)
	output += colorArgs[i].save();
    }
    if(choiceArgs != null){
      for(i = 0; i < choiceArgs.length; i++)
	output += choiceArgs[i].save();
    }
    output += "\n";
    return output;
  }
  public boolean load(String input){
    int sliderCount=0, colorCount=0, breedCount=0, choiceCount=0;
    String[] strings = Util.split(input, "\n");
    for(int i = 0; i < strings.length; i++){
      if(strings[i].startsWith(SliderArg.argType())){
	if(sliderCount < sliderArgs.length)
	  sliderArgs[sliderCount++].load(Util.parse(strings[i]));
	else 
	  return false;
      }
      else if(strings[i].startsWith(BreedArg.argType())){
	if(breedCount < breedArgs.length)
	  breedArgs[breedCount++].load(Util.parse(strings[i]));
	else
	  return false;
      }
      else if(strings[i].startsWith(ColorArg.argType())){
	if(colorCount < colorArgs.length)
	  colorArgs[colorCount++].load(Util.parse(strings[i]));
	else
	  return false;
      }
      else if(strings[i].startsWith(ChoiceArg.argType())){
	if(choiceCount < choiceArgs.length)
	  choiceArgs[choiceCount++].load(Util.parse(strings[i]));
	else
	  return false;
      }
    }

    return true;
  }

  public int compareTo(Object o){
    if(o instanceof BaseFunc){
      return funcName.getText().compareTo(((BaseFunc)o).funcName.getText());
    }
    return -1;
  }
}

