/**********************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *********************************************************************/



/* BreedFuncs holds all of the functions a single breed can possibly define.
 * Also holds the breed's properties, which record the functions the
 * breed has defined.  Processes the defined functions to create the
 * final string representation of them for output to the starlogo file.
 */

package template;

import starlogo.*;
   
import template.turtleFuncs.*;
import template.observerFuncs.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;  //for Java2 collections

public class BreedFuncs extends JPanel {
  String breed;
  Set pages = new TreeSet();
  protected BreedProps properties;
  CardLayout layout = new CardLayout();
  String lastFunction;
  JPanel lowerPanel = new JPanel(),
    mainPanel = new JPanel();
  Preview preview = new Preview();
  protected Template t;
  BaseFunc currentFunction;
  JScrollPane scrollPane = new JScrollPane();
  public static String breedFuncsWord = "Breed";
  
  public BreedFuncs(String s, Template temp){
    breed = s;
    t = temp;
  }
  public void setup(){
    mainPanel.setLayout(new javax.swing.BoxLayout(mainPanel, javax.swing.BoxLayout.Y_AXIS));
    mainPanel.add(lowerPanel);
    mainPanel.add(javax.swing.Box.createVerticalGlue());
    mainPanel.setBorder(BorderFactory.createEmptyBorder(10,10,0,10));
    lowerPanel.setLayout(new javax.swing.BoxLayout(lowerPanel, javax.swing.BoxLayout.X_AXIS));
    lowerPanel.add(scrollPane);
    scrollPane.setBorder(null);

    setLayout(new javax.swing.BoxLayout(this, javax.swing.BoxLayout.Y_AXIS));
    add(mainPanel);
    add(preview);
    
    properties = new BreedProps(breed, this);
    addFunctions();    
    properties.setPossibleFuncs(getAllFuncs());
    addPredefinedFuncs();
    properties.enable();
  }
  public void addFunctions(){
//      add(new ChangeTarget(t.breeds));
//      add(new Death(t.breeds));
//      add(new Draw(t.breeds));
//      add(new EatGrass(t.breeds));
//      add(new EatPatch(t.breeds));
//      add(new EatTurtles(t.breeds));
//      add(new Reproduce(t.breeds));
//      add(new template.turtleFuncs.Setup(t.breeds));
//      add(new Track(t.breeds));
//      add(new Walk(t.breeds));
//      add(new template.turtleFuncs.Custom(t.breeds));
//      add(new Infect(t.breeds));
//      add(new Mutate(t.breeds));
  }
  public void addPredefinedFuncs(){
    BaseFunc[] array = getAllFuncs();
    for(int i = 0; i < array.length; i++)
      if((!array[i].isObsFunc()) && array[i].isPredefined())
	properties.addFunc(array[i]);
  }
  public void add(BaseFunc f){
    pages.add(f);
  }
  public void remove(BaseFunc f){
    pages.remove(f);
    properties.remove(f);
  }
  public void removeAll(){
    pages.clear();
    properties.removeAll();
  }
  public void restart(){
    removeAll();
  }
  public void show(BaseFunc f){
    f.add(javax.swing.Box.createVerticalGlue());
    scrollPane.setViewportView(f);
    currentFunction = f;
    updatePreview();
    lowerPanel.validate();
    lowerPanel.revalidate();
    lowerPanel.repaint();
  }
  public void updatePreview(){
    updatePreview(null);
  }
  public void updatePreview(Point position){
    if(position == null 
       || !preview.getBounds().contains(position))
      preview.preview(currentFunction, this);
  }
  public String getBreed(){
    return breed;
  }
  public void setBreed(String s){
    breed = s;
  }
  public String toString(){
    return getBreed();
  }
  public BaseFunc getFunction(String fName){
    Iterator i = pages.iterator();
    BaseFunc f;
    while(i.hasNext()){
      f = (BaseFunc)i.next();
      if(f.getName().equals(fName))
	return f;
    }
    return null;
  }
  public void updateFuncBreeds(){
    Iterator i = pages.iterator();
    while(i.hasNext())
      ((BaseFunc)i.next()).updateBreeds();
  }
  public BreedProps getProps(){
    return properties;
  }
  public String writeFunction(BaseFunc function){
    String body = "";
    String connector = "-";
    if(function == null)
      return null;
    if(breed.startsWith("<")) //i.e. <default>
      body = "to " + function.getCommand() + "\n";
    else
      body = "to " + function.getCommand() + connector + breed + "\n";
    return body + function.getBody() + "\n";
  }
  public String writeFunctions(boolean obsFuncP){
    return writeFunctions(null, obsFuncP);
  }
  public String writeFunctions(String oldFunctions, boolean obsFuncP){
    //eventually look at oldFunctions and replace/change whatever is needed
    BaseFunc[] functions = getDefinedFuncs();
    String body = "";
    
    if(functions == null)
      return body;
    
    for(int i = 0; i < functions.length; i++){
      if(obsFuncP == functions[i].isObsFunc())
	body += writeFunction(functions[i]) + "\n";
    }
    return body;
  }
  public String writeSliders(){
    BaseFunc[] functions = getDefinedFuncs();
    String body = "";
    if(functions == null)
      return body;
    for(int i = 0; i < functions.length; i++)
      body += functions[i].writeSliders();

    return body;
  }
  public void makeSliders(){
    // This function is not used
    BaseFunc[] funcNames = getDefinedFuncs();
    if(funcNames == null)
      return;

    SliderInfo[] sliders;
    JInterface iface = t.app.iface;

    for(int i = 0; i < funcNames.length; i++){
      sliders = funcNames[i].getSliderInfo();
      if(sliders == null)
	continue;
      for(int j = 0; j < sliders.length; j++){
	System.out.println("writing slider " + j);
	sliders[j].makeSlider(iface);
	SliderInfo.sliders++;
	SliderInfo.y += 30;
	ButtonInfo.y += 30;
      }
    }
  }
  public void makeButtons(){
    // This function is not used
    BaseFunc[] funcNames = getDefinedFuncs();
    if(funcNames == null)
      return;

    ButtonInfo[] buttons;
    JInterface iface = t.app.iface;

    for(int i = 0; i < funcNames.length; i++){
      buttons = funcNames[i].getButtonInfo();
      if(buttons == null)
	continue;
      for(int j = 0; j < buttons.length; j++){
	System.out.println("writing button " + j);
	buttons[j].makeButton(iface);
	ButtonInfo.buttons++;
	SliderInfo.y += 40;
	ButtonInfo.y += 40;
      }
    }
  }
  public String writeButtons(){
    BaseFunc[] funcNames = getDefinedFuncs();
    String body = "";
    if(funcNames == null)
      return body;

    ButtonInfo[] buttons;
    JInterface iface = t.app.iface;

    for(int i = 0; i < funcNames.length; i++){
      body += funcNames[i].writeButtons();
    }
    return body;
  }
  public String writeVariables(String currentVars, int varType){
    BaseFunc[] funcNames = getDefinedFuncs();
    String[] variables;
    String output = new String(currentVars);
    if(funcNames == null)
      return output;
    for(int i = 0; i < funcNames.length; i++){
      variables = funcNames[i].getVariables(varType);
      if(variables == null)
	continue;
      for(int j = 0; j < variables.length; j++){
	if(output.indexOf(variables[j]) == -1)
	  output += variables[j] + " ";
      }
    }
    return output;
  }  
  public String[] getAllFuncNames(){
    BaseFunc[] bfa = getAllFuncs();
    String[] strings = new String[bfa.length];
    
    for(int i = 0; i < bfa.length; i++)
      strings[i] = bfa[i].getName();

    return strings;
  }
  public BaseFunc[] getDefinedFuncs(){
    return properties.getDefinedFuncs();
  }
  public BaseFunc[] getAllFuncs(){
    return (BaseFunc[])pages.toArray(new BaseFunc[0]);
  }
  public String save(){
    String output = breedFuncsWord + " " + getBreed() + "\n";
    BaseFunc[] functions = getDefinedFuncs();
    if(functions == null)
      return output;

    for(int i = 0; i < functions.length; i++)
      output += functions[i].save();
    
    return output;
  }
  public boolean load(String input){
    return load(input, false);
  }
  public boolean load(String input, boolean clearOld){
    if(clearOld){
      removeAll();
      addFunctions();
    }
    BaseFunc function;
    String[] strings = Util.split(input, BaseFunc.functionWord + " ");
    String funcName;
    int pos;
    boolean returnVal = true;
    for(int i = 0; i < strings.length; i++){
      strings[i] = strings[i].trim();
      if(strings[i].equals(""))
	continue;
      pos = strings[i].indexOf("\n");
      funcName = strings[i].substring(0, pos);
      function = getFunction(funcName);
      if(function == null)
	continue;
      returnVal = returnVal && function.load(strings[i]);
      if(returnVal)
	properties.addFunc(function);
    }
    return returnVal;
  }
}

