/**********************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *********************************************************************/



/* BreedProps keeps track of which functions a breed has defined.
 * Also contains the buttons to create new functions by displaying a
 * BreedFuncs and a number of functions to interface with other
 * template classes.  
 */

package template;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;  //for Java2 collections

public class BreedProps extends JPanel implements ActionListener, ListSelectionListener{
  Collection commands = new ArrayList(),
    functions = new ArrayList();
  DefaultListModel fdModel = new DefaultListModel();
  JList functionsDefined = new JList(fdModel);
  String name;
  JLabel title;
  boolean firstTime = true;
  static String noFunctions = "<no functions>";
  BaseFunc[] possibleFunctions;
  JButton editFunction = new JButton("Edit Function"),
    newFunction = new JButton("New"),
    removeFunction = new JButton("Remove"),
    describeFunction = new JButton("Describe");
  final String editCommand = "edit function",
    newCommand = "new function",
    removeCommand = "remove function",
    listCommand = "list command",
    describeCommand = "describe command";
  BreedFuncs funcs;
  JPanel subPanel = new JPanel();
  boolean enabled = false;
  
  public BreedProps(String n, BreedFuncs funcs){
    name = n;
    this.funcs = funcs;
    if(name.startsWith("<")){
      String s = name.trim();
      s = s.substring(0, s.length() - 1);  //to remove ">"
      title = new JLabel(s + " definitions>");
    }
    else
      title = new JLabel(name + " define");
    Font f = title.getFont();
    title.setFont(new Font(f.getName(), Font.BOLD, f.getSize()));
    
    functionsDefined.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    functionsDefined.setVisibleRowCount(6);
    title.setAlignmentX(CENTER_ALIGNMENT);
    newFunction.setAlignmentX(CENTER_ALIGNMENT);
    subPanel.setLayout(new BoxLayout(subPanel, BoxLayout.X_AXIS));
    subPanel.add(newFunction);
    subPanel.add(Box.createHorizontalStrut(5));
    subPanel.add(removeFunction);
    subPanel.add(Box.createHorizontalStrut(5));
    subPanel.add(describeFunction);

    setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
    setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder("Breed functions"), 
						 BorderFactory.createEmptyBorder(5,15,5,15)));
    add(title);
    add(Box.createVerticalStrut(5));
    add(new JScrollPane(functionsDefined));
    add(Box.createVerticalStrut(5));
    add(subPanel);
    
    fdModel.addElement(noFunctions);
    editFunction.setEnabled(false);
    removeFunction.setEnabled(false);
    
    editFunction.setActionCommand(editCommand);
    editFunction.addActionListener(this);
    newFunction.setActionCommand(newCommand);
    newFunction.addActionListener(this);
    removeFunction.setActionCommand(removeCommand);
    removeFunction.addActionListener(this);
    describeFunction.setActionCommand(describeCommand);
    describeFunction.addActionListener(this);
    functionsDefined.addListSelectionListener(this);
  }
  public boolean addFunc(BaseFunc function){
    if(firstTime){
      fdModel.removeElement(noFunctions);
      firstTime = false;
      editFunction.setEnabled(true);
    }
    if(!functions.contains(function)){
      functions.add(function);
      fdModel.addElement(function);
      functionsDefined.setSelectedIndex(fdModel.indexOf(function));
    }
    return true;
  }
  public void setPossibleFuncs(BaseFunc[] functions){
    possibleFunctions = functions;
  }
  public void removeFunc(BaseFunc function){
    if(fdModel.contains(function)){
      int pos = functionsDefined.getSelectedIndex();
      if(pos < 1)
	pos = 1;
      fdModel.removeElement(function);
      functionsDefined.setSelectedIndex(pos - 1);
    }
    if(functions.contains(function))
      functions.remove(function);
  }
  public void removeAll(){
    fdModel.clear();
    functions.clear();
  }
  public boolean contains(BaseFunc function){
    return functions.contains(function);
  }
  public BaseFunc[] getDefinedFuncs(){
    if(functions.isEmpty())
      return null;
    return (BaseFunc[]) functions.toArray(new BaseFunc[0]);
  }
  public String[] getFuncNamesDefined(){
    BaseFunc[] array = getDefinedFuncs();
    if(array == null)
      return null;
    String[] strings = new String[array.length];
    for(int i = 0; i < array.length; i++)
      strings[i] = array[i].getName();
    return strings;
  }
  public BaseFunc getSelectedFunction(){
    if(functionsDefined.getSelectedIndex() < 0)
      return null;
    Object o = fdModel.elementAt(functionsDefined.getSelectedIndex());
    if(!(o instanceof BaseFunc))
      return null;
    return (BaseFunc)o;
  }
  public void enable(){
    enabled = true;
  }
  public void updateCustoms(){
    boolean areCustoms = false;
    boolean areNewCustoms = false;
    BaseFunc[] functions = getDefinedFuncs();
    for(int i = 0; i < functions.length; i++){
      if(functions[i] instanceof template.turtleFuncs.Custom){
	areCustoms = true;
	if(!((template.turtleFuncs.Custom)functions[i]).isChanged()){
	  if(areNewCustoms){
	    removeFunc(functions[i]);
	    funcs.remove(functions[i]);
	  }
	  else
	    areNewCustoms = true;
	}
	else{
	  BaseFunc f = new template.turtleFuncs.Custom(funcs.t.breeds);
	  Util.replaceFirst(possibleFunctions, functions[i], f);
	  funcs.add(f);
	}
      }
    }
  }
  
  //listening functions
  public void actionPerformed(ActionEvent e){
    if(!enabled)
      return;

    String command = e.getActionCommand();
    
    if(command == editCommand){
      editListen();
    } else if(command == newCommand){
      newListen();
    } else if(command == removeCommand){
      removeListen();
    } else if(command == describeCommand){
      describeListen();
    }
  }
  protected void editListen(){
    //this "if" should never be executed because the button that 
    //would make this happen is disabled when firstTime == true
    if(firstTime){
      JOptionPane.showMessageDialog(this, 
				    "You must create a new function first", 
				    "Edit Function",
				    JOptionPane.ERROR_MESSAGE);
      return;
    }
    if(fdModel.getSize() > 0){
      funcs.show(getSelectedFunction());
    }
    updateCustoms();
  }
  protected void newListen(){
    if(possibleFunctions == null || possibleFunctions.length < 1){
      JOptionPane.showMessageDialog(this, 
				    "This Template has no functions defined for " 
				    + funcs.getBreed(), 
				    "Nothing to do",
				    JOptionPane.ERROR_MESSAGE);
      return;
    }
    BaseFunc functionToMake = 
      (BaseFunc) JOptionPane.showInputDialog(this, 
					     "Choose a new Function to make", "New Function",
					     JOptionPane.INFORMATION_MESSAGE, null,
					     possibleFunctions, possibleFunctions[0]);
    if(functionToMake == null)
      return;
    if(funcs == null)
      System.out.println("funcs == null");

    addFunc(functionToMake);
    funcs.t.app.project.markChanged();
  }
  protected void removeListen(){
    BaseFunc f = getSelectedFunction();
    int answer = JOptionPane.showConfirmDialog(this,
					  "Are you sure you want to remove the function \"" 
					       + f.getName() 
					       + "\"",
					  "Remove Function",
					  JOptionPane.YES_NO_OPTION);
    if(answer == JOptionPane.YES_OPTION)
      removeFunc(f);
  }
  protected void describeListen(){
    BaseFunc function = getSelectedFunction();
    if(function == null)
      return;
    JOptionPane.showMessageDialog(this, function.describe(), 
                                  function.getName() + " description", 
                                  JOptionPane.INFORMATION_MESSAGE);
  }

  public void valueChanged(ListSelectionEvent e){
    if(functionsDefined == null)
      return;

    BaseFunc function = getSelectedFunction();
    if(function == null)
      return;

    if(function.isPredefined())
      removeFunction.setEnabled(false);
    else
      removeFunction.setEnabled(true);

    functionsDefined.ensureIndexIsVisible(functionsDefined.getSelectedIndex());
    editListen();
  } 
}
