/**********************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *********************************************************************/



package template;

import starlogo.*;
import java.awt.*;

public class ButtonInfo {
  public static int x, y, buttons;

  public int topLeftX, topLeftY;
  public boolean isTurtleFunction;
  public int width, height;
  public String name, lineToRun;
  public boolean foreverP;
  public int buttonNumber;
  public boolean showNameP;

  String wholeThing;

  static {
    try {
    	resetStatics();
    } catch (Exception e)  { e.printStackTrace(); }
  }

  public ButtonInfo(String s){
    wholeThing = s;
  }

  public ButtonInfo(int tlx, int tly, boolean itf, int w, int h, 
		    String n, String ltr, boolean forever, 
		    int num, boolean showName){
    topLeftX = tlx;
    topLeftY = tly;
    isTurtleFunction = itf;
    width = w;
    height = h;
    name = BaseFunc.toCommand(n);
    lineToRun = ltr;  //just store the line to run, don't add quotes or anything
    foreverP = forever;
    buttonNumber = num;
    showNameP = showName;
  }
  public ButtonInfo(boolean turtleP, String ltr, boolean foreverP){
    this(-1, -1, turtleP, 100, 30, "button" + buttons, ltr, foreverP, -1, false);
  }
  public ButtonInfo(boolean turtleP, String name, String lineToRun, boolean foreverP, boolean showNameP){
    this(-1, -1, turtleP, 100, 30, name, lineToRun, foreverP, -1, showNameP);
  }
  public String writeButton(){
    if(wholeThing != null)
      return wholeThing;

    String body = "\nSLButton\n";
    body += "turtle-or-observer? " + (isTurtleFunction ? "turtle" : "observer") + "\n";
    body += "top-left " + y + " " + x + "\n";
    body += "width-height " + width + " " + height + "\n";
    if(name.equals(lineToRun))
      body += "name button" + buttons + "\n";
    else
      body += "name " + name +"\n"; 
    body += "line-to-run " + Util.quote(lineToRun) + "\n";
    body += "forever? " + foreverP + "\n";
    body += "button-number " + buttons + "\n";
    body += "show-name? " + showNameP + "\n";

    return body;
  }

  public void getValues(WidgetDrag wd) {
    ButtonWidget bw = (ButtonWidget)wd;
    bw.command = lineToRun;
    bw.name = BaseFunc.toCommand(lineToRun);
    bw.forever = foreverP;
    bw.showName = showNameP;
    bw.font = Widget.WIDGET_FONT;
    bw.turtlep = isTurtleFunction;
  }

  public void makeButton(JInterface iface){
    System.out.println("ButtonInfo.makeButton(), line-to-run = " + lineToRun);
    System.out.println("position = (" + x + ", " + y + ")");
    iface.setOutline(x, y, width, height);
    ButtonWidget bw = new ButtonWidget(iface, iface.sl(), lineToRun, name, 
				       foreverP, showNameP, isTurtleFunction, "","");
    bw.updateMinSize();
    iface.resetOutline();
    iface.addWidget(bw);
  }

  public static void resetStatics(){
    x = 22;
    y = 22;
    buttons = 0;
  }
}


