/**********************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *********************************************************************/



package template;

import java.util.StringTokenizer;
import java.util.*;

public class Fraction {
  boolean numIsString;
  String numString;
  int num, denom;
  public static int[] primes = {
      2,     3,     5,     7,    11,    13,    17,    19,    23,    29,
     31,    37,    41,    43,    47,    53,    59,    61,    67,    71,
     73,    79,    83,    89,    97,   101,   103,   107,   109,   113,
    127,   131,   137,   139,   149,   151,   157,   163,   167,   173,
    179,   181,   191,   193,   197,   199,   211,   223,   227,   229,
    233,   239,   241,   251,   257,   263,   269,   271,   277,   281,
    283,   293,   307,   311,   313,   317,   331,   337,   347,   349,
    353,   359,   367,   373,   379,   383,   389,   397,   401,   409,
    419,   421,   431,   433,   439,   443,   449,   457,   461,   463,
    467,   479,   487,   491,   499,   503,   509,   521,   523,   541,
    547,   557,   563,   569,   571,   577,   587,   593,   599,   601,
    607,   613,   617,   619,   631,   641,   643,   647,   653,   659,
    661,   673,   677,   683,   691,   701,   709,   719,   727,   733,
    739,   743,   751,   757,   761,   769,   773,   787,   797,   809,
    811,   821,   823,   827,   829,   839,   853,   857,   859,   863,
    877,   881,   883,   887,   907,   911,   919,   929,   937,   941,
    947,   953,   967,   971,   977,   983,   991,   997
  };  

  public Fraction(int numerator, int denominator){
    int GCD = gcd(numerator, denominator);
    num = numerator / GCD;
    denom = denominator / GCD;
    numIsString = false;
  }
  public Fraction(String numerator, int denominator){
    numString = numerator;
    denom = denominator;
    numIsString = true;
  }
  public String toString(){
    if(numIsString && denom == 1)
      return numString;
    if(numIsString)
      return numString + " / " + denom;
    
    if(denom == 1)
      return "" + num;
    else
      return "" + num + " / " + denom;
  }
  public static int gcd(int n1, int n2){
    if(n2 == 0)
      return n1;
    else
      return gcd(n2, n1%n2);
  }
  public static String reduce(String input){
    ArrayList list = new ArrayList();
    StringTokenizer st = new StringTokenizer(input, " \t\n\r\f)]([", true);
    while(st.hasMoreTokens())
      list.add(st.nextToken());
    String[] strings = (String[])list.toArray(new String[0]);
    String output = "", temp;

    for(int i = 0; i < strings.length; i++){
      if((i < strings.length - 4) && strings[i+2].equals("/")){
	output += singleReduce(strings[i], strings[i+4]);
	i += 4;
      }
      else
	output += strings[i];
    }
    return output;
  }

  public static String singleReduce(String numerator, String denominator){
    int n=0, d;
    boolean numIsString = false;
    try {
      d = Integer.parseInt(denominator);
    } catch (NumberFormatException e) {
      return numerator + " / " + denominator;
    }

    try {
      n = Integer.parseInt(numerator);
    } catch (NumberFormatException e){
      numIsString = true;
    }
    
    if(numIsString)      
      return new Fraction(numerator, d).toString();
    else
      return new Fraction(n, d).toString();
  }
}


