/**********************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *********************************************************************/



package template;

import starlogo.*;
import java.awt.*;

public class SliderInfo {
  public static int x, y, sliders;

  public int topLeftX, topLeftY;
  public int width, height;
  public String name;
  public String variable;
  public int minimum, maximum, current;
  public int sliderNumber;
  public boolean showNameP;
  
  String wholeThing;

  static{
  	try {
	    resetStatics();
  	} catch (Exception e) { e.printStackTrace(); }
  }
  
  public SliderInfo(String s){
    wholeThing = s;
  }

  public SliderInfo(String v, int min, int max, int cur){
    this(-1, -1, 160, 25, v, v, min, max, cur, -1, false);
  }

  public SliderInfo(int tlx, int tly, int w, int h, 
		    String n, String v, int min, int max, 
		    int cur, int number, boolean showName){
    topLeftX = tlx;
    topLeftY = tly;
    width = w;
    height = h;
    name = BaseFunc.toCommand(n);
    variable = BaseFunc.toCommand(v);
    minimum = min;
    maximum = max;
    current = cur;
    sliderNumber = number;
    showNameP = showName;
  }

  public String writeSlider(){
    if(wholeThing != null)
      return wholeThing;
    
    String body = "SLSlider\n";
    body += "top-left " + y + " " + x + "\n";
    body += "width-height " + width + " " + height + "\n";
    body += "name " + Util.quote(name) + "\n";
    body += "variable " + Util.quote(variable) + "\n";
    body += "min-value " + minimum + "\n";
    body += "max-value " + maximum + "\n";
    body += "current-value " + current + "\n";
    body += "slider-number " + sliders + "\n";
    body += "show-name? " + showNameP + "\n\n";

    return body;
  }

  public String getBody(){
    String body = "SLSlider\n";
    body += "top-left " + topLeftY + " " + topLeftX + "\n";
    body += "width-height " + width + " " + height + "\n";
    body += "name " + Util.quote(name) + "\n";
    body += "variable " + Util.quote(variable) + "\n";
    body += "min-value " + minimum + "\n";
    body += "max-value " + maximum + "\n";
    body += "current-value " + current + "\n";
    body += "slider-number " + sliderNumber + "\n";
    body += "show-name? " + showNameP + "\n\n";
    
    return body;
  }

  public void getValues(SliderWidget sw) {
    sw.name = name;
    sw.var = variable;
    sw.min = minimum;
    sw.max = maximum;
    sw.cur = current;
    sw.showName = showNameP;
    sw.update();
  }

  public void makeSlider(JInterface iface){
    System.out.println("SliderInfo.makeSlider() name = " + name);
    System.out.println("position = (" + x + ", " + y + ")");
    iface.setOutline(x, y, width, height);
    SliderWidget sw = new SliderWidget(iface, iface.sl());
    getValues(sw);
    sw.updateMinSize();
    iface.resetOutline();
    iface.addWidget(sw);
  }

  public static void resetStatics(){
    x = 22;
    y = 22;
    sliders = 0;
  }
}
