/**********************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *********************************************************************/


/* Template is the main class in the template architecture.  It
 * organizes all of the individual template functions into a useable
 * interface.  There isn't much to Template except its constructor,
 * which does all of the swing layout.  
 */

package template;

import template.turtleFuncs.*;
import template.observerFuncs.*;
import starlogo.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;  //for Java2 collections

public class Template extends JFrame 
  implements ActionListener, ListSelectionListener, CaretListener//  , AWTEventListener
{
  public Map allBreedFuncs = new HashMap(); //holds all of the BreedFuncs classes
  public Collection  breeds = new ArrayList();  //holds Strings of all of the breeds' names
  public BreedFuncs funcs; //BreedFuncs of the currently selected breed
  public BreedProps props; //BreedProps of the currently selected breed
  protected DefaultListModel blModel = new DefaultListModel();
  protected JList breedList = new JList(blModel);
  protected JTextField breedBox = new JTextField(10);
  JButton makeBreed = new JButton("Add breed"),
    removeBreedButton = new JButton("Remove"),
    renameBreedButton = new JButton("Rename"),
    done = new JButton("Done"),
    clearButton = new JButton("Clear");
  JPanel menu = new JPanel(),  //the main menu panel
    breedPanel = new JPanel(),  //the subpanel containing the breed-making components
    outerBreedPanel = new JPanel(),
    buttonPanel = new JPanel(),
    leftPanel = new JPanel(),
    topButtonPanel = new JPanel();
  Container CP;
  final String removeBreedCommand = "remove",
    makeBreedCommand = "make breed",
    doneCommand = "done",
    clearCommand = "clear",
    renameCommand = "rename";
  public final String DEFAULT = "<default>",
    OBSERVER = "<observer>";
  public Application app;
  public static String templateWord = "`template`";
  
  static {
    try{
      UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
    } catch(Exception ignored){}
  }

  protected Template() {}
  protected Template(Application app){
    this.app = app;
  }
  protected void setup(){
    CP = getContentPane();
    CP.setLayout(new javax.swing.BoxLayout(CP, javax.swing.BoxLayout.X_AXIS));
    setTitle("StarLogo Template");
    
    breedList.addListSelectionListener(this);
    breedList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    breedList.setVisibleRowCount(6);
    removeBreedButton.setActionCommand(removeBreedCommand);
    removeBreedButton.addActionListener(this);
    removeBreedButton.setEnabled(false);
    makeBreed.setActionCommand(makeBreedCommand);
    makeBreed.addActionListener(this);
    makeBreed.setEnabled(false);
    breedBox.setActionCommand(makeBreedCommand);
    breedBox.addActionListener(this);
    breedBox.addCaretListener(this);
    clearButton.setActionCommand(clearCommand);
    clearButton.addActionListener(this);
    renameBreedButton.setActionCommand(renameCommand);
    renameBreedButton.addActionListener(this);

    addFirstBreeds();
    breedList.setSelectedIndex(0);
    props = funcs.getProps();

    int h = breedBox.getFont().getSize() + 10;
    breedBox.setMaximumSize(new Dimension(breedBox.getMaximumSize().width, h));
    breedPanel.add(breedBox);
    breedPanel.add(makeBreed);
    done.setActionCommand(doneCommand);
    done.addActionListener(this);
    
    breedPanel.setAlignmentX(CENTER_ALIGNMENT);
    done.setAlignmentX(CENTER_ALIGNMENT);
    removeBreedButton.setAlignmentX(CENTER_ALIGNMENT);

    buttonPanel.setLayout(new javax.swing.BoxLayout(buttonPanel, javax.swing.BoxLayout.X_AXIS));
    buttonPanel.setBorder(BorderFactory.createEmptyBorder(5,0,5,0));
    buttonPanel.add(clearButton);
    buttonPanel.add(done);

    topButtonPanel.setLayout(new javax.swing.BoxLayout(topButtonPanel, javax.swing.BoxLayout.X_AXIS));
    topButtonPanel.add(removeBreedButton);
    topButtonPanel.add(renameBreedButton);

    outerBreedPanel.setLayout(new javax.swing.BoxLayout(outerBreedPanel, javax.swing.BoxLayout.Y_AXIS));
    outerBreedPanel.add(breedPanel);
    outerBreedPanel.add(new JScrollPane(breedList));
    outerBreedPanel.add(topButtonPanel);
    outerBreedPanel.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder("Breeds"), 
							   BorderFactory.createEmptyBorder(0,5,5,5)));
    leftPanel.setLayout(new javax.swing.BoxLayout(leftPanel, javax.swing.BoxLayout.Y_AXIS));
    leftPanel.add(outerBreedPanel);
    leftPanel.add(props);
    leftPanel.add(buttonPanel);

    CP.add(leftPanel);
    CP.add(funcs);
    pack();

    setSize(getSize().width + 30, getSize().height);
  }
  protected void addFirstBreeds(){
    addBreed(DEFAULT, false);
    addBreed(new ObsFuncs(this), false);
  }
  public void addBreed(String funcName){
    addBreed(funcName, true);
  }
  public void addBreed(String funcName, boolean addToBreeds){
    addBreed(new BreedFuncs(funcName, this), addToBreeds);
  }
  public void addBreed(BreedFuncs bf, boolean addToBreeds){
    allBreedFuncs.put(bf.getBreed(), bf);
    funcs = bf;
    blModel.addElement(bf.getBreed());
    if(addToBreeds)
      breeds.add(bf.getBreed());
  }
  public BreedFuncs selectFuncs(String breedName){
    return (BreedFuncs)allBreedFuncs.get(breedName);
  }
  public BreedProps selectProps(String breedName){
    BreedFuncs f = selectFuncs(breedName);
    if(f != null)
      return f.getProps();
    return null;
  }
  public BreedFuncs[] getAllFuncs(){
    return (BreedFuncs[])allBreedFuncs.values().toArray(new BreedFuncs[0]);
  }
  public String getSelectedBreed(){
    if(breedList.getSelectedIndex() < 0)
      return null;
    return (String)blModel.elementAt(breedList.getSelectedIndex());
  }
  public void updateBreeds(){
    Iterator i = allBreedFuncs.values().iterator();
    while(i.hasNext())
      ((BreedFuncs)i.next()).updateFuncBreeds();
  }
  public void removeAllFuncs(){
    allBreedFuncs.clear();
    blModel.clear();
    breeds.clear();
  }
  public static void restart(Application app){
    app.theTemplate.removeAllFuncs();
    app.theTemplate.addFirstBreeds();
  }
  public String writeWander(){
    return writeWander(null);
  }
  public String writeWander(String overWriting){
    //this will be much simpler if/when overloading is implemented
    String output = "",
      breedString = "",
      breedZeroCalls = "",
      allCalls = "";
    BaseFunc[] defaultDefs = selectFuncs(DEFAULT).getDefinedFuncs();
    Collection defaultDefList = new ArrayList();
    String connector = "-";
    String[] breedNames = (String[]) breeds.toArray(new String[0]);
    BaseFunc[] functions;
    BreedFuncs f;

    for(int x = 0; x < breedNames.length; x++)
      breedString += breedNames[x] + " ";

    if(!breedString.equals(""))
      output += "breeds [ " + breedString + "]\n\n";
    output += "to wander";

    String s;
    if(defaultDefs != null)
      for(int x = 0; x < defaultDefs.length; x++){
	if(defaultDefs[x].isInWander())
	  breedZeroCalls += defaultDefs[x].getCommand() + "\n    ";
	defaultDefList.add(defaultDefs[x]);
      }
    
    if(!breedZeroCalls.equals(""))
      allCalls += "\n  if breed = 0\n    [" + breedZeroCalls + "]";

    for(int x = 0; x < breedNames.length; x++){
      if(breedNames[x].startsWith("<")) //don't do this for <default> or <observer>
	continue;
      f = selectFuncs(breedNames[x]);
      if((functions = f.getDefinedFuncs()) == null) 
	continue;     /*don't write anything if this breed 
			hasn't defined any functions*/
      if(functions.length == 1 && !functions[0].isInWander())
	  continue;     /*don't write anything if this breed has only 
			  defined one function that isn't written (e.g. setup)*/

      allCalls += "\n  if (breed = " + breedNames[x] + ")\n    ["; 
      for(int y = 0; y < functions.length; y++){
	if(functions[y].isInWander())
	  allCalls += functions[y].getCommand() + connector + breedNames[x] + "\n    ";
      }
      allCalls += "]";
    }
    if(allCalls.equals(""))
      output += "\n  ;this function is empty\n  ;the wander button will not do anything";
    else
      output += allCalls;

    output += "\nend\n";
    return output;
  }
  public String writeWanderButton(){
    ButtonInfo bi = new ButtonInfo(true, "wander", true);
    String output = bi.writeButton();
    ButtonInfo.y += 40;
    SliderInfo.y += 40;
    ButtonInfo.buttons++;
    return output;
  }
  public String templateType(){
    return "hard-coded ecology";
  }
  public String save(){
    String output = templateWord + "\n" + templateType() + "\n";
    BreedFuncs[] allFuncs = getAllFuncs();
    if(allFuncs == null)
      return output;

    for(int i = 0; i < allFuncs.length; i++)
      output += allFuncs[i].save() + "\n";

    return output;
  }
  public boolean load(String input){
    try{
      int stringStart = input.indexOf(templateWord) + templateWord.length() + 1;
      int stringStop = input.indexOf("`", stringStart);
      if(stringStop == -1)
	stringStop = input.length();
      input = input.substring(stringStart, stringStop);
      if(input.indexOf(templateType()) != 0)
	return false;
      else 
	input = input.substring(input.indexOf(templateType()) + templateType().length());
    } catch(Exception e){
      return false;
    }
    String[] strings = Util.split(input, "\n" + BreedFuncs.breedFuncsWord + " ");
    BreedFuncs bf;
    String[] temp;
    String breedName;
    int pos;
    boolean returnVal = true;
    restart(app);
    for(int i = 0; i < strings.length; i++){
      pos = strings[i].indexOf("\n");
      breedName = strings[i].substring(0, pos);
      strings[i] = strings[i].substring(pos);
      bf = selectFuncs(breedName);
      if(bf == null){
	addBreed(breedName);
	bf = funcs;
	updateBreeds();
      }
      boolean b = bf.load(strings[i]);
      breedList.setSelectedIndex(blModel.indexOf(breedName));
      returnVal = returnVal && b;
    }
    return returnVal;
  }
  
  //listening functions for Template
  public void actionPerformed(ActionEvent e){
    String command = e.getActionCommand();
    
    if(command == removeBreedCommand){
      removeBreedListen();
    } else if(command == makeBreedCommand){
      makeBreedListen();
    } else if(command == doneCommand){
      doneListen();
    } else if(command == clearCommand){
      clearListen();
    } else if(command == renameCommand){
      renameListen();
    }
  }
  public void valueChanged(ListSelectionEvent e){
    //display the breedProps corresponding to the currently selected breed
    CP.removeAll();

    funcs = selectFuncs(getSelectedBreed());
    if(funcs == null)
      return;
    if(props != null){
      leftPanel.remove(props);
      props = funcs.getProps();
      leftPanel.add(props);
      leftPanel.add(buttonPanel);
    } else{
      props = funcs.getProps();
    }
    leftPanel.validate();
    leftPanel.repaint();
    if(funcs.getBreed().startsWith("<")){ //i.e. "<default>"
      removeBreedButton.setEnabled(false);
      renameBreedButton.setEnabled(false);
    }
    else{
      removeBreedButton.setEnabled(true);
      renameBreedButton.setEnabled(true);
    }

    breedList.ensureIndexIsVisible(breedList.getSelectedIndex());
    CP.add(leftPanel);
    CP.add(funcs);
    CP.validate();
    CP.repaint();
  }
  public void caretUpdate(CaretEvent e){
    if(breedBox.getText().trim().equals(""))
      makeBreed.setEnabled(false);
    else
      makeBreed.setEnabled(true);
  }

  protected void removeBreedListen(){
    String breed = getSelectedBreed();
    //this "if" should never be executed, but just in case
    if(breed.startsWith("<")){ //i.e. "<default>"
      JOptionPane.showMessageDialog(Template.this, 
				    "You can't remove the breed " + breed, 
				    "Remove Breed",
				    JOptionPane.ERROR_MESSAGE);
      return;
    }
    int i = JOptionPane.showConfirmDialog(Template.this,
					  "Are you sure you want to delete the breed \"" + breed + "\"",
					  "Remove Breed",
					  JOptionPane.YES_NO_OPTION);
    if(i == JOptionPane.YES_OPTION){
      int index = breedList.getSelectedIndex();
      if(index < 1)
	index = 1;
      
      breeds.remove(breed);

      updateBreeds();
      
      allBreedFuncs.remove(breed);
      blModel.removeElement(breed);
      breedList.setSelectedIndex(index - 1);
    }
  }
  protected void makeBreedListen(){
    //called either when user presses return in breedBox, or presses the "make breed" button
    String s = breedBox.getText();
    if((!allBreedFuncs.containsKey(s)) && !s.trim().equals("") && (s != null))
      addBreed(s);

    breedList.setSelectedIndex(blModel.indexOf(s));
    breedBox.setText("");
    app.project.markChanged();
  }
  protected void renameListen(){
    String newName = JOptionPane.showInputDialog(this, 
						 "Change breed \"" 
						 + funcs.getBreed() 
						 + "\" to: ");
    if(newName == null || newName.equals(""))
      return;
    String oldName = funcs.getBreed();
    int index = breedList.getSelectedIndex();
    funcs.setBreed(newName);
    blModel.removeElement(oldName);
    blModel.addElement(newName);
    breeds.remove(oldName);
    breeds.add(newName);
    allBreedFuncs.remove(oldName);
    allBreedFuncs.put(newName, funcs);
    updateBreeds();
    breedList.setSelectedIndex(index);
  }
  protected void doneListen(){
    Iterator i = allBreedFuncs.values().iterator();
    BreedFuncs f;
    String disclaimer = "; These functions are generated automatically\n; Any changes will be erased the next time \n; the template is run\n\n",
      turtleFunctions = "", 
      turtleHeader = "Java StarLogo\n1.1\n`turtle`\n",
      wander = "",
      iface = "`interface`\n",
      obsHeader = "`observer`\n",
      observerFunctions = "",
      tVars = "",
      gVars = "",
      pVars = "";

    SliderInfo.resetStatics();
    ButtonInfo.resetStatics();
    iface += writeWanderButton();

    while(i.hasNext()){
      f = (BreedFuncs)i.next();
      observerFunctions += f.writeFunctions(true);
      turtleFunctions += f.writeFunctions(false);
      iface += f.writeSliders();
      iface += f.writeButtons();
      gVars = f.writeVariables(gVars, BaseFunc.GLOBAL_VARS);
      tVars = f.writeVariables(tVars, BaseFunc.TURTLE_VARS);
      pVars = f.writeVariables(pVars, BaseFunc.PATCH_VARS);
    }
    if(!tVars.trim().equals(""))
      tVars = "turtles-own [ " + tVars + "]\n";
    if(!gVars.trim().equals(""))
      gVars = "globals [ " + gVars + "]\n";
    if(!pVars.trim().equals(""))
      pVars = "patches-own [ " + pVars + "]\n";
    wander += writeWander();
    turtleFunctions = disclaimer + wander.trim() + "\n\n" + turtleFunctions.trim() + "\n\n";
    observerFunctions = disclaimer + tVars + gVars + pVars + "\n" + observerFunctions.trim();

    app.procWin.getSLProc(ControlCenter.tur).setText(turtleFunctions);
    app.procWin.getSLProc(ControlCenter.obs).setText(observerFunctions);
    
    //load from the string, maybe later use internal SL commands
    app.iface.loadFromString(iface.trim(), false);
    app.iface.repaint();
    app.maybeCompileAndThenRun(false, new Runnable() {public void run() {} });
//      hide();
  }
  protected void saveListen(){
    FileDialog fd = new FileDialog(this, "Save Template as...", FileDialog.SAVE);
    fd.show();
    String directory = fd.getDirectory();
    String fileName = fd.getFile();
    if(fileName == null)
      return;
    if(fileName.indexOf(".") == -1)
      fileName += ".tem";
    fd.dispose();

    Writer writer = null;
    try {
      writer = new FileWriter(new File(directory, fileName));
      String s = save();
      writer.write(s);
      writer.flush();
      writer.close();
    } catch(IOException e){
      System.err.println("uh oh, bad file name");
    }
  }
  protected void loadListen(){
    FileDialog fd = new FileDialog(this, "Load Template...", FileDialog.SAVE);
    fd.show();
    String directory = fd.getDirectory();
    String fileName = fd.getFile();
    if(fileName == null)
      return;
    if(fileName.indexOf(".") == -1)
      fileName += ".tem";
    fd.dispose();

    Reader reader = null;
    try {
      reader = new FileReader(new File(directory, fileName));
      String s = Util.readToString(reader);
      load(s);
    } catch(IOException e){
      System.err.println("uh oh, bad file name");
    }
  }
  
  protected void clearListen(){
    int answer = JOptionPane.showConfirmDialog(this,
					       "Are you sure you want to start over?",
					       "Clear Template",
					       JOptionPane.YES_NO_OPTION);
    if(answer == JOptionPane.YES_OPTION)
      restart(app);
  }
}

