/**********************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *********************************************************************/



/* class to hold a bunch of useful static methods */

package template;

import yoyo.*;
import java.util.*;  //for Java2 collections

public class Util{
  public static String replace(String in, String toReplace, String with){
    int end;
    String remainder = in, output = "";
    while((end = remainder.indexOf(toReplace)) != -1){
      try{
	output += remainder.substring(0, end) + with;
	remainder = remainder.substring(end + toReplace.length(), remainder.length());
      } catch(Exception ignored){}
    }
    return output + remainder;
  }
  public static void replaceFirst(Object[] array, Object toReplace, Object with){
    for(int i = 0; i < array.length; i++){
      if(array[i].equals(toReplace)){
	array[i] = with;
	break;
      }
    }
  }
  public static void replaceAll(Object[] array, Object toReplace, Object with){
    for(int i = 0; i < array.length; i++){
      if(array[i].equals(toReplace))
	array[i] = with;
    }
  }
  public static String replaceWord(String in, String toReplace, String with){
    String[] strings = (String[]) parse(in).toArray(new String[0]);
    String output = "";
    for(int i = 0; i < strings.length; i++){
      if(strings[i].equals(toReplace))
	output += with;
      else
	output += strings[i];
    }
    return output;
  }
  public static int countChar(String in, int c){
    int count = 0;
    int pos = 0;
    while((pos = in.indexOf(c, pos+1)) >= 0)
      count++;
    return count;
  }
  public static String quote(String input){
    return "\"" + replace(input, "\"", "\\\"") + "\"";
  }
  public static String unquote(String input){
    return unescapeQuotes(removeQuotes(input));
  }
  public static ArrayList parse(String input){
    Tokenizer t = new Tokenizer(input);
    java.util.ArrayList al = t.tokenize();
    al = removeQuotes(al);
    return al;
  }
  public static String removeQuotes(String input){
    input = input.trim();
    int start = 0, end = input.length();
    if(input.startsWith("\""))
      start++;
    if(input.endsWith("\""))
      end--;
    return input.substring(start, end);
  }
  public static String unescapeQuotes(String input){
    return replace(input, "\\\"", "\"");
  }
  public static ArrayList removeQuotes(ArrayList input){
    for(int i = 0; i < input.size(); i++)
      input.set(i, removeQuotes((String)input.get(i)));
    return input;
  }
  public static String[] split(String input, String splitOn){
    int nextSpot = 0;
    ArrayList list = new ArrayList();
    while((nextSpot = input.indexOf(splitOn)) != -1){
      if(nextSpot > 1)
	list.add(input.substring(0, nextSpot));
      if(nextSpot < input.length())
	input = input.substring(nextSpot + splitOn.length());
    }
    if(input != null)
      list.add(input);
    return (String[])list.toArray(new String[0]);
  }
  public static String isolateChunk(String input, String startToken, String endToken){
    int startPos = input.indexOf(startToken);
    int endPos = input.indexOf(endToken, startPos);
    if(startPos < 0)
      return null;
    if(endPos < 0)
      endPos = input.length();
    startPos += startToken.length();
    return input.substring(startPos, endPos).trim();
  }
  public static String readToString(java.io.Reader reader) throws java.io.IOException {
    final int startBufSize = 2048;
    final int increment = 1024;

    StringBuffer buffer = new StringBuffer(startBufSize);
    int next;
    int pos = 0;
    while((next = reader.read()) != -1){
      if(buffer.capacity() <= pos)
	buffer.ensureCapacity(buffer.capacity() + increment);
      buffer.append((char)next);
      pos++;
    }
    return buffer.toString();
  }
  public static String indent(String s, String offset){
    return replace(offset + s, "\n", "\n" + offset);
  }
}

