/**********************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *********************************************************************/



package template.arguments;

import template.*;
import javax.swing.*;
import java.awt.*;
import java.util.*;  //for Java2 collections
//import com.sun.java.util.collections.*;  //for Java2 collections in Java 1.1.x

public class BreedArg extends BaseArg{
  /*represents an argument to a function that is a breed.  Bases its
    menu on a Collection passed to its constructor, but only updates
    the menu when updateBreeds() is called, when it goes through all
    of the breeds and adds any new ones to the combo box*/
  JComboBox cb;
  JLabel name;
  
  public BreedArg(String myName){
    this(myName, null);
  }
  public BreedArg(String myName, String variable){
    this.variable = variable;
    cb = new JComboBox();
    cb.addItem("<none>");
    name = new JLabel(myName + "  ");

    name.setFont(argFont);
    cb.setFont(argFont);
    java.awt.Dimension d = cb.getMaximumSize();
    d.height = cb.getPreferredSize().height;
    cb.setMaximumSize(d);

    setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
    add(name);
    add(cb);
  }
  
  public String toString(){
    if(((String)cb.getSelectedItem()).equals("<none>"))
      return "0";
    else
      return (String)(cb.getSelectedItem());
  }
  
  public String save(){
    String output = argType() + "   ";
    output += Util.quote(getLabel()) + " ";
    output += Util.quote((String)cb.getSelectedItem());
    return output;
  }

  public boolean load(ArrayList input){
    try{
//        if(((String)input.get(0)).equals(argType()))
//  	input.remove(0);
      Iterator i = input.iterator();
      String s = (String)i.next();
      if(s.equals(argType()))
	s = (String)i.next();
      setLabel(s);
      cb.setSelectedItem(i.next());
      return true;
    } catch (Exception e){
      System.err.println("Error loading breedArg: " + e);
      return false;
    }
  }
    
  public void updateBreeds(Collection breeds){
    Iterator i = breeds.iterator();
    Object current = cb.getSelectedItem();
    cb.removeAllItems();
    cb.addItem("<none>");
    while(i.hasNext())
      cb.addItem((String)i.next());
    cb.setSelectedItem(current);
  }
  
  public String getLabel(){
    return name.getText();
  }
  
  public String setLabel(String newLabel){
    String s = getLabel();
    name.setText(newLabel + "  ");
    return s;
  }
  public static String argType(){
    return "BreedArg";
  }
}
