/**********************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *********************************************************************/



package template.arguments;

import javax.swing.*;
import java.awt.event.*;
import java.util.*;  //for Java2 collections
//import com.sun.java.util.collections.*;  //for Java2 collections in Java 1.1.x

public class ChoiceArg extends BaseArg implements ActionListener{
  ButtonGroup bg = new ButtonGroup();
  JLabel name;
  JRadioButton defaultRB, otherRB;
  JPanel defaultPanel, otherPanel;
  final String defaultCommand = "default command",
    otherCommand = "other command";
  

  public ChoiceArg(String label, String defaultChoice, String otherChoice){
    this(label, defaultChoice, otherChoice, null, null);
  }
  public ChoiceArg(String label, String defaultChoice, String otherChoice, 
		   JPanel defaultPanel, JPanel otherPanel){
    this(label, null, defaultChoice, otherChoice, defaultPanel, otherPanel);
  }
  public ChoiceArg(String label, String variable, String defaultChoice, String otherChoice,
		   JPanel defaultPanel, JPanel otherPanel){
    name = new JLabel(label + "  ");
    defaultRB = new JRadioButton(defaultChoice);
    otherRB = new JRadioButton(otherChoice);
    setPanels(defaultPanel, otherPanel);
    this.variable = variable;
    
    defaultRB.setActionCommand(defaultCommand);
    otherRB.setActionCommand(otherCommand);
    defaultRB.addActionListener(this);
    otherRB.addActionListener(this);
    
    bg.add(defaultRB);
    bg.add(otherRB);
    defaultRB.setSelected(true);

    setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
    add(name);
    add(defaultRB);
    add(otherRB);

    name.setFont(argFont);
    defaultRB.setFont(argFont);
    otherRB.setFont(argFont);
  }
  public String setLabel(String newLabel){
    String s = getLabel();
    name.setText(newLabel + "  ");
    return s;
  }
  public String getLabel(){
    return name.getText().trim();
  }
  public void setDefaultRBLabel(String s){
    defaultRB.setText(s);
  }
  public String getDefaultRBLabel(){
    return defaultRB.getText();
  }
  public void setOtherRBLabel(String s){
    otherRB.setText(s);
  }
  public String getOtherRBLabel(){
    return otherRB.getText();
  }
  public static String argType(){
    return "ChoiceArg";
  }
  public String save(){
    return argType() + "  " + defaultRB.isSelected();
  }
  public boolean load(ArrayList input){
    try{
      Iterator i = input.iterator();
      String s = (String)i.next();
      if(s.equals(argType()))
	s = (String)i.next();
      
      if(new Boolean(s).booleanValue())
	defaultRB.doClick();
      else
	otherRB.doClick();
      return true;
    } catch(Exception e){
      System.err.println("Error loading choiceArg: " + e);
      return false;
    }
  }
  public boolean isDefaultSelected(){
    return defaultRB.isSelected();
  }
  public String toString(){
    if(defaultRB.isSelected())
      return defaultRB.getText();
    else
      return otherRB.getText();
  }
  public void setPanels(JPanel defaultPanel, JPanel otherPanel){
    this.defaultPanel = defaultPanel;
    this.otherPanel = otherPanel;

    if(otherPanel != null)
      otherPanel.setVisible(false);
  }
  public JPanel getDefaultPanel(){
    return defaultPanel;
  }
  public JPanel getOtherPanel(){
    return otherPanel;
  }

  //listeners
  public void actionPerformed(ActionEvent e){
    String command = e.getActionCommand();
   
    if((defaultPanel == null) && (otherPanel == null))
      return;
 
    if(otherPanel != null){
      if(command == defaultCommand){
	otherPanel.setVisible(false);
      } else if(command == otherCommand){
	otherPanel.setVisible(true);
      }
    }
    if (defaultPanel != null){
      if(command == defaultCommand){
	defaultPanel.setVisible(true);
      } else if(command == otherCommand){
	defaultPanel.setVisible(false);
      }
    }
  }
}
