/**********************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *********************************************************************/



package template.arguments;

import template.*;
import javax.swing.*;
import java.util.*;  //for Java2 collections
//import com.sun.java.util.collections.*;  //for Java2 collections in Java 1.1.x

public class ColorArg extends BaseArg{
  /*represents a color that can be an argument to a function.  Uses
    all of the starlogo colors in order in the combo box*/
  final String[] colors = {"black", "grey", "white", "red", "orange", 
			   "brown", "yellow", "green", "lime", "turquoise",
			   "cyan", "sky", "blue", "violet", "magenta", "pink"};
  JComboBox colorList = new JComboBox();
  JLabel name;

  public ColorArg(String myName){
    this(myName, "black");
  }
  public ColorArg(String myName, String startingColor){
    this(myName, null, startingColor);
  }
  public ColorArg(String myName, String variable, String startingColor){
    this.variable = variable;
    name = new JLabel(myName + "  ");
    for(int i = 0; i < colors.length; i++)
      colorList.addItem(colors[i]);
    if(startingColor.equals(""))
      colorList.setSelectedIndex((int)(Math.random() * colors.length));
    else
      colorList.setSelectedItem(startingColor.toLowerCase());
    setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
    add(name);
    add(colorList);

    name.setFont(argFont);
    colorList.setFont(argFont);
    java.awt.Dimension d = colorList.getMaximumSize();
    d.height = colorList.getPreferredSize().height;
    colorList.setMaximumSize(d);
  }
  
  public String toString(){
    return (String)(colorList.getSelectedItem());
  }
  
  public String getLabel(){
    return name.getText().trim();
  }
  
  public String setLabel(String newLabel){
    String s = getLabel();
    name.setText(newLabel + "  ");
    return s;
  }

  public String save(){
    String output = argType() + "   ";
    output += Util.quote(getLabel()) + " ";
    output += Util.quote(toString());
    return output;
  }

  public boolean load(ArrayList input){
    try{
      if(((String)input.get(0)).equals(argType()))
	input.remove(0);
      Iterator i = input.iterator();
      setLabel((String) i.next());
      colorList.setSelectedItem(i.next());
      return true;
    } catch (Exception e){
      System.err.println("Error loading colorArg: " + e);
      return false;
    }
  }
  public static String argType(){
    return "ColorArg";
  }
}
