/**********************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *********************************************************************/



package template.arguments;

import template.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;  //for Java2 collections


public class SliderArg extends BaseArg implements ActionListener, ChangeListener{
  /*class representing an argument to a starlogo function that can
    be either a constant value or a new slider added to the model.
    Displays a name and either a textField to input a new slider name or a
    slider representing the fixed value*/
  JRadioButton newSlider = new JRadioButton("Create StarLogo slider"),
    noNewSlider = new JRadioButton("Use a constant value");
  ButtonGroup bg = new ButtonGroup();
  JSlider value;
  JTextField sliderName = new JTextField(10);
  String prompt;
  JLabel info = new JLabel();
  boolean newSliderP = false;
  JPanel left = new JPanel(),
    right = new JPanel();
  final String newSliderCommand = "new slider",
    noNewSliderCommand = "no new slider",
    valueCommand = "value";

  public SliderArg(){
    this(null, 0, 0);
  }
  public SliderArg(String name, int max){
    this(name, max, max/2);
  }
  public SliderArg(String prompt, int max, int initialValue) {
    this(prompt, null, 0, initialValue, max);
  }
  public SliderArg(String prompt, String var, int min, int initialValue, int max){
    value = new JSlider(JSlider.HORIZONTAL, min, max, initialValue);
//      value.setPreferredSize(new Dimension(sliderName.getPreferredSize().width, 40));
//      value.setMajorTickSpacing(max / 2);
    value.setPaintLabels(true);
    setSliderLabels(min, max);
    sliderName.setVisible(false);
    this.prompt = prompt;
    variable = var;

    int h = sliderName.getFont().getSize() + 10;
    sliderName.setMaximumSize(new Dimension(value.getPreferredSize().width + 45, h));
    
    bg.add(newSlider);
    bg.add(noNewSlider);
    noNewSlider.setSelected(true);
    
    left.setLayout(new BoxLayout(left, BoxLayout.Y_AXIS));
    left.add(noNewSlider);
    left.add(newSlider);
    
    right.setLayout(new BoxLayout(right, BoxLayout.Y_AXIS));
    info.setAlignmentX((float)0.5);
    right.add(info);
    right.add(value);
    info.setText("" + value.getValue());
    
    setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
    setBorder(BorderFactory.createTitledBorder(null, prompt, TitledBorder.LEFT, 
					       TitledBorder.TOP, argFont));
    add(left);
    add(right);
    add(sliderName);
    
    newSlider.setActionCommand(newSliderCommand);
    noNewSlider.setActionCommand(noNewSliderCommand);
    newSlider.addActionListener(this);
    noNewSlider.addActionListener(this);
    value.addChangeListener(this);

    newSlider.setFont(argFont);
    noNewSlider.setFont(argFont);
    value.setFont(argFont);
    sliderName.setFont(argFont);
    info.setFont(argFont);
  }
  public void setSliderLabels(int min, int max){
    Hashtable labelTable = new Hashtable();
    JLabel l1 = new JLabel("" + min),
      l2 = new JLabel("" + ((min + max) / 2)),
      l3 = new JLabel("" + max);

    l1.setFont(argFont);
    l2.setFont(argFont);
    l3.setFont(argFont);

    l1.setAlignmentX(JLabel.RIGHT_ALIGNMENT);
    l2.setAlignmentX(JLabel.RIGHT_ALIGNMENT);
    l3.setAlignmentX(JLabel.RIGHT_ALIGNMENT);

    labelTable.put( new Integer(min), l1 );
    labelTable.put( new Integer((min+max) / 2), l2);
    labelTable.put( new Integer(max), l3);
    value.setLabelTable(labelTable);
  }
  public String toString(){   
    if((newSlider.isSelected()) && (!sliderName.getText().trim().equals("")))
      return replace(sliderName.getText().trim(), " ", "-");
    else
      return "" + value.getValue();
  }
  public String setLabel(String newLabel){
    String s = getLabel();
    setBorder(BorderFactory.createTitledBorder(null, newLabel, TitledBorder.LEFT, 
					       TitledBorder.TOP, argFont));
    return s;
  }
  public String getLabel(){
    return ((TitledBorder)getBorder()).getTitle();
  }
  public boolean isNewSlider(){
    return newSlider.isSelected() && isVisible();
  }
  public SliderInfo getSliderInfo(){
    if(newSlider.isSelected() && isVisible())
      return new SliderInfo(sliderName.getText(), value.getMinimum(), 
			    value.getMaximum(), value.getValue());
    else
      return null;
  }
  public String save(){
    String output = argType() + "   ";
    output += Util.quote(getLabel()) + " ";
    output += newSlider.isSelected() + " ";
    output += isVisible() + " ";
    output += Util.quote(sliderName.getText()) + " ";
    output += value.getMinimum() + " ";
    output += value.getMaximum() + " ";
    output += value.getValue() + "\n";

    return output;
  }
  public boolean load(ArrayList input){
    try{
      Iterator i = input.iterator();
      String s = (String)i.next();
      if(!s.equals(argType()))
	setLabel(s);
      else
	setLabel((String)i.next());
      if(new Boolean((String) i.next()).booleanValue())
	newSlider.doClick();
      else 
	noNewSlider.doClick();
      setVisible(new Boolean((String)i.next()).booleanValue());
      sliderName.setText((String) i.next());
      value.setMinimum(new Integer((String) i.next()).intValue());
      value.setMaximum(new Integer((String) i.next()).intValue());
      value.setValue(new Integer((String) i.next()).intValue());
      //setup the slider's labels correctly
//        value.setLabelTable(null);
//        value.setMajorTickSpacing(value.getMaximum() / 2);
      setSliderLabels(value.getMinimum(), value.getMaximum());
      value.setPaintLabels(true);
      return true;
    } catch (Exception e){
      System.err.println("Error loading sliderArg: " + e);
      return false;
    }
  } 
  
  //listener functions
  public void actionPerformed(ActionEvent e){
    String command = e.getActionCommand();
    
    if(newSlider.isSelected()){
      right.setVisible(false);
      sliderName.setVisible(true);
    }
    else{
      sliderName.setVisible(false);
      right.setVisible(true);
    }
  }
  public void stateChanged(ChangeEvent e){
    info.setText("" + value.getValue());
  }
  public static String argType(){
    return "SliderArg";
  }
}
