/**********************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *********************************************************************/



package template.general;

import template.*;
import java.lang.reflect.*;
import java.util.*;

public class Breed extends template.BreedFuncs{
  String[] functionStrings;
  static String functionWord = "\n<function>",
    javaString = "java";

  public Breed(String breedName, Template t, String loadFrom){
    super(breedName, t);
    functionStrings = Util.split(loadFrom, functionWord);
    setup();
  }
  public boolean loadFuncs(){
    return loadFuncs(false);
  }
  public boolean loadFuncs(boolean observerP){
    BaseFunc f;
    for(int i = 0; i < functionStrings.length; i++){
//        System.out.println("functionStrings[" + i + "] = " + functionStrings[i]);
      if(functionStrings[i].trim().equals(""))
	continue;
      if(functionStrings[i].trim().startsWith(javaString))
	f = loadJavaFunction(functionStrings[i].trim());
      else
	f = new Function(functionStrings[i], t.breeds);
      if(f == null)
	continue;
      if(observerP == f.isObsFunc())
	add(f);
    }
    return true;
  }
  public void addFunctions(){
    loadFuncs();
  }
  public BaseFunc loadJavaFunction(String funcName){
    try{
      int pos = funcName.indexOf(" ");
      if(pos < 0)
	pos = 0;
      funcName = funcName.substring(pos+1);
      if(funcName.indexOf(".") < 0)          //  no package specified, assume template.*
	funcName = "template." + funcName;

      Object[] args = new Object[] {t.breeds};
      Class[] prototype = new Class[] {Collection.class};
      Class funcDefinition = Class.forName(funcName);
      Constructor constructor = funcDefinition.getConstructor(prototype);

      return (BaseFunc) constructor.newInstance(args);

    } catch (ClassNotFoundException e){
      System.err.println("The class \"" + e.getMessage() + "\" was not found");
      return null;

    } catch (InvocationTargetException e){
      System.err.println("InvocationTargetException: but the real exception was: " 
			 + e.getTargetException());
      e.getTargetException().printStackTrace();
      return null;

    } catch (Exception e){
      System.err.println("Exception: " + e);
      System.err.println("  class \"" + funcName + "\" not found!");
      return null;
    }
  }
}






