/**********************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *********************************************************************/



package template.general;

import template.*;
import template.arguments.*;
import java.util.*;
import javax.swing.*;
import java.io.*;
import java.awt.*;

public class Function extends BaseFunc{
  static String sliderWord = "<slider>",
    breedWord = "<breed>",
    colorWord = "<color>",
    choiceWord = "<choice>",
    buttonWord = "<button>",
    bodyStart = "<body>",
    bodyEnd = "</body>",
    descStart = "<description>",
    descEnd = "</description>",
    switches = "<switches>",
    obsFuncSW = "observer",
    noWander = "notinwander",
    predefined = "predefined",
    tVars = "<turtlevars>",
    gVars = "<globalvars>",
    pVars = "<patchvars>",
    ifString = "if", 
    elseString = "else",
    endifString = "endif";
    
  
  String[] body,
    turtleVars,
    globalVars,
    patchVars;
  String desc = "This function has no description";
  Collection sliderList = new ArrayList(), 
    breedList = new ArrayList(), 
    colorList = new ArrayList(), 
    choiceList = new ArrayList(),
    buttonList = new ArrayList();
  ButtonInfo[] buttons;

  public Function(String loadString, Collection breeds){
    this.breeds = breeds;
    loadName(loadString);
    body = tokenize(Util.isolateChunk(loadString, bodyStart, bodyEnd));
//      for(int i = 0; i < body.length; i++)
//        System.out.println("'" + body[i] + "'");
    desc = Util.unquote(Util.isolateChunk(loadString, descStart, descEnd));
    setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
    add(funcName);
    loadArgs(loadString);
    loadSwitches(loadString);
    loadVariables(loadString);
    loadButtons(loadString);
    finalizeArrays();
  }

  //functions overloaded/needed from BaseFunc
  public String getBody(){
    String temp;
    String output = "";
    int choice;
    boolean outputP = true;
    BoolStack stack = new BoolStack(8);  //8 nested ifs/choiceArgs should be plenty
    stack.push(true);  //so after all the ifs end, it defaults to printing

    for(int i = 0; i < body.length; i++){
      if(body[i].startsWith(ifString)){
	try{
	  i++;
	  choice = new Integer(body[i]).intValue();
	} catch (NumberFormatException e){
	  i++;
	  choice = new Integer(body[i]).intValue();
	}

	if(!choiceArgs[choice - 1].isDefaultSelected()){
	  i = matchElse(i);
	  stack.push(false);
	}
	else
	  stack.push(true);
      } else if(body[i].startsWith(elseString)){
	if(stack.isEmpty()){
	  System.err.println("No " + ifString + " for this " + elseString + ": " + body[i]);
	  return null;
	} 
	if(stack.peek())
	  i = matchEndIf(i);
      } else if(body[i].startsWith(endifString)){
	if(stack.isEmpty()){
	  System.err.println("No " + ifString + " for this " + endifString + ": " + body[i]);
	  return null;
	}
	stack.pop();
      } else{
	if(!output.trim().equals(""))
	  output += "\n";
	output += body[i];
      }
    }
    return replaceVariables(output);
  }
  public String describe(){
    return desc;
  }
  public void add(BaseArg arg){
    if(arg instanceof SliderArg)
      sliderList.add(arg);
    else if(arg instanceof BreedArg)
      breedList.add(arg);
    else if(arg instanceof ColorArg)
      colorList.add(arg);
    else if(arg instanceof ChoiceArg)
      choiceList.add(arg);
    super.add(arg);
  }
  public String[] getVariables(int varType){
    if(varType == TURTLE_VARS)
      return turtleVars;
    if(varType == GLOBAL_VARS)
      return globalVars;
    if(varType == PATCH_VARS)
      return patchVars;
    return null;
  }
  public ButtonInfo[] getButtonInfo(){
    return buttons;
  }


  //functions for loading general function from string
  public boolean loadArgs(String input){
    //choice args are not recursive yet
    String[] strings = Util.split(input, "\n");
    BaseArg arg;
    ChoiceArg ca = null;
    int addToChoice = 0;
    Component tempStrut;
    for(int i = 0; i < strings.length; i++){
      arg = loadArg(strings[i]);
      tempStrut = Box.createVerticalStrut(0);
      if(arg != null){
	if((getComponentCount() > 0) && (addToChoice == 0))
	  add(tempStrut);
	add(arg);
      }
      if((addToChoice == 2) && (ca != null)){
	ca.setPanels(arg, null);
	addToChoice--;
      } else if((addToChoice == 1) && (ca != null)){
	ca.setPanels(ca.getDefaultPanel(), arg);
	addToChoice--;
      } else if(arg instanceof ChoiceArg){
	ca = (ChoiceArg) arg; 
	addToChoice = 2;
      }
    }
    return true;
  }
  public void loadSwitches(String input){
    input = Util.isolateChunk(input, switches, "\n");
    if(input == null)
      return;
    if(input.indexOf(obsFuncSW) != -1)
      obsFuncP = true;
    if(input.indexOf(noWander) != -1)
      inWanderP = false;
    if(input.indexOf(predefined) != -1)
      predefinedP = true;
  }
  public void loadVariables(String input){
    String tInput = Util.isolateChunk(input, tVars, "\n");
    String gInput = Util.isolateChunk(input, gVars, "\n");
    String pInput = Util.isolateChunk(input, pVars, "\n");
    Iterator i;
    Collection list = new ArrayList();
    if(tInput != null){
      list.clear();
      i = Util.parse(tInput).iterator();
      while(i.hasNext())
	list.add(i.next());
      turtleVars = (String[]) list.toArray(new String[0]);
    }
    if(gInput != null){
      list.clear();
      i = Util.parse(gInput).iterator();
      while(i.hasNext())
	list.add(i.next());
      globalVars = (String[]) list.toArray(new String[0]);
    }
    if(pInput != null){
      list.clear();
      i = Util.parse(pInput).iterator();
      while(i.hasNext())
	list.add(i.next());
      patchVars = (String[]) list.toArray(new String[0]);
    }
  }
  public void loadButtons(String input){
    String[] strings = Util.split(input, "\n");
    for(int i = 0; i < strings.length; i++)
      if(strings[i].startsWith(buttonWord))
	buttonList.add(loadButton(strings[i]));
  }
  public void loadName(String input){
    String name = input.substring(0, input.indexOf("\n"));
    name = Util.unquote(name);
    funcName.setText(name);
  }
  public BaseArg loadArg(String input){
    try{
      String temp = input.toLowerCase();
      if(temp.startsWith(sliderWord))
	return loadSlider(input);
      else if(temp.startsWith(breedWord))
	return loadBreed(input);
      else if(temp.startsWith(colorWord))
	return loadColor(input);
      else if(temp.startsWith(choiceWord))
	return loadChoice(input);
      return null;
    } catch (Exception e){
      System.err.println("Bad argument definition: " + input);
      return null;
    }
  }
  public BaseArg loadChoice(String input) throws Exception{
    Iterator i = Util.parse(input).iterator();
    String s = (String)i.next();
    if(s.equals(choiceWord))
      s = (String)i.next();
    String prompt = s;
    String defaultLabel = (String)i.next();
    String otherLabel = (String)i.next();
    return new ChoiceArg(prompt, defaultLabel, otherLabel);
  }
  public BaseArg loadSlider(String input) throws Exception{
    //add min and variable to SliderArg
    Iterator i = Util.parse(input).iterator();
    String prompt = (String)i.next();
    if(prompt.equals(sliderWord))
      prompt = (String)i.next();
    String variable = (String)i.next();
    int min = new Integer((String) i.next()).intValue();
    int cur = new Integer((String) i.next()).intValue();
    int max = new Integer((String) i.next()).intValue();
    return new SliderArg(prompt, variable, min, cur, max);
  }
  public BaseArg loadBreed(String input) throws Exception{
    Iterator i = Util.parse(input).iterator();
    String prompt = (String)i.next();
    if(prompt.equals(breedWord))
      prompt = (String)i.next();
    String variable = (String)i.next();
    return new BreedArg(prompt, variable);
  }
  public BaseArg loadColor(String input) throws Exception{
    Iterator i = Util.parse(input).iterator();
    String prompt = (String)i.next();
    if(prompt.equals(colorWord))
      prompt = (String)i.next();
    String variable = (String)i.next();
    String defColor = "";
    if(i.hasNext())
      defColor = (String)i.next();
    return new ColorArg(prompt, variable, defColor);
  }
  public ButtonInfo loadButton(String input){
    Iterator i = Util.parse(input).iterator();
    String lineToRun = (String)i.next();
    if(lineToRun.equals(buttonWord))
      lineToRun = (String)i.next();
    boolean turtleP = new Boolean((String) i.next()).booleanValue();
    boolean foreverP = new Boolean((String) i.next()).booleanValue();
    if(i.hasNext()) 
      return new ButtonInfo(turtleP, (String)i.next(), 
			    lineToRun, foreverP, true);
    else
      return new ButtonInfo(turtleP, lineToRun, foreverP);
  }
  public void finalizeArrays(){
    sliderArgs = (SliderArg[])  sliderList.toArray(new SliderArg[0]);
    colorArgs  = (ColorArg[])   colorList.toArray(new ColorArg[0]);
    breedArgs  = (BreedArg[])   breedList.toArray(new BreedArg[0]);
    choiceArgs = (ChoiceArg[])  choiceList.toArray(new ChoiceArg[0]);
    buttons    = (ButtonInfo[]) buttonList.toArray(new ButtonInfo[0]);
  }


  //helper functions
  public static String[] tokenize(String input){
    return (String[])Util.parse(input).toArray(new String[0]);
  }
  public String replaceVariables(String input){
    if(sliderArgs != null)
      for(int i = 0; i < sliderArgs.length; i++)
	if(sliderArgs[i].isVisible() && sliderArgs[i].isEnabled())
	  input = Util.replace(input, sliderArgs[i].getVariable(), sliderArgs[i].toString());
    if(breedArgs != null)
      for(int i = 0; i < breedArgs.length; i++)
	if(breedArgs[i].isVisible() && breedArgs[i].isEnabled())
	  input = Util.replace(input, breedArgs[i].getVariable(), breedArgs[i].toString());
    if(colorArgs != null)
      for(int i = 0; i < colorArgs.length; i++)
	if(colorArgs[i].isVisible() && colorArgs[i].isEnabled())
	  input = Util.replace(input, colorArgs[i].getVariable(), colorArgs[i].toString());
    return input;
  } 
  public int matchElse(int ifIndex){
    int total = 0, i;
    
    for(i = ifIndex+1; i < body.length; i++){
      if(body[i].startsWith(ifString)){
	total++;
      } else if(body[i].startsWith(elseString)){
	if(total == 0)
	  break;
      } else if(body[i].startsWith(endifString)){
	total--;
      }
    }

    return i;
  }
  public int matchEndIf(int elseIndex){
    int total = 0, i;
    
    for(i = elseIndex+1; i < body.length; i++){
      if(body[i].startsWith(ifString)){
	total++;
      } else if(body[i].startsWith(elseString)){
      } else if(body[i].startsWith(endifString)){
	if(total <= 0)
	  break;
	total--;
      }
    }

    return i;
  }

  //helper class used in evaluating the body of a Function
  class BoolStack {
    boolean[] array;
    int pos = -1;

    public BoolStack(int size){
      array = new boolean[size];
    }

    public boolean push(boolean toPush){
      if(pos >= array.length)
	return false;
      array[++pos] = toPush;
      return true;
    }

    public boolean pop(){
      if(pos == -1)
	return false;
      boolean output = array[pos];
      pos--;
      return output;
    }

    public boolean peek(){
      if(!isEmpty())
	return array[pos];
      return false;
    }

    public boolean isEmpty(){
      if (pos == -1)
	return true;
      return false;
    }
  }
}


