/**********************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *********************************************************************/



package template.general;

import starlogo.*;
import java.io.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.lang.reflect.*;
import java.util.*;

public class Template extends template.Template{
  String loadString = null;
  String type;
  static String templateWord = "<template>",
    templateExtension = ".slogt",
    commentString = "//";
  static File loadFile;

  public static Template makeTemplate(Application app){
      return new Template_java1_2(app);
  }

  protected Template(Application app){
    super(app);
  }  

  public void setup(String input){
    input = deComment(input);
    input = input.substring(input.indexOf(templateWord));
    String temp = template.Util.isolateChunk(input, templateWord, "\n");
    loadString = input.substring(input.indexOf(temp) + temp.length());
    type = template.Util.unquote(temp);
    super.setup();
    setTitle(type);
  }
  public void show(){
    if(loadString == null){
      String s = load();
      if(s == null)
	return;
      setup(s);
    }
    super.show();
  }
  //function for loading a Template from a file
  public static String load(){
      String directory = System.getProperty("user.dir") + "\\Templates\\";
      String fileName = "";
      if(loadFile == null){
	  FileDialog fd = new FileDialog(new JFrame(), "Load Template...", FileDialog.LOAD);
	  fd.setDirectory(directory);
	  fd.setFilenameFilter( new java.io.FilenameFilter() {
		  public boolean accept(File file, String name){
		      if(name.indexOf(templateExtension) < 0)
			  return false;
		      else
			  return true;
		  }
	      });
	  fd.show();
	  directory = fd.getDirectory();
	  fileName = fd.getFile();
	  if(fileName == null)
	      return null;
	  fd.dispose();
      }

    Reader reader = null;
    try {
      if(loadFile == null)
	loadFile = new File(directory, fileName);
      reader = new FileReader(loadFile);  // new FileReader(loadFile);
      String s = template.Util.readToString(reader);
      if (s.indexOf(templateWord) < 0) {
          // bad file
          loadFile = null;
          JOptionPane.showMessageDialog(null,
                                        "Invalid template file."
                                        + " Check that the file has a "
                                        + templateExtension + " extension.\n"
                                        + "(see StarLogo documentation for more information"
                                        + " on templates)",
                                        "Invalid Template",
                                        JOptionPane.ERROR_MESSAGE);
          return null;
      }
      return s;
    } catch(IOException e){
      System.err.println("uh oh, bad file name");
      return null;
    }
  }
  public void addFirstBreeds(){
    addBreed(new Breed(DEFAULT, this, loadString), false);
    addBreed(new Observer(this, loadString), false);
  }
  public void addBreed(String breedName){
    addBreed(new Breed(breedName, this, loadString), true);
  }
  public String templateType(){
    return type;
  }
  public static String deComment(String input){
    int commentPos, endl;
    while((commentPos = input.indexOf(commentString)) != -1){
      endl = input.indexOf("\n", commentPos + commentString.length());
      input = input.substring(0, commentPos) + input.substring(endl);
    }
    return input;
  }
  protected void makeBreedListen(){
    //called either when user presses return in breedBox, or presses the "make breed" button
    String s = breedBox.getText();
    if((!allBreedFuncs.containsKey(s)) && !s.trim().equals("") && (s != null)){
      Breed b = new Breed(s, this, loadString);
      breeds.add(s);
      allBreedFuncs.put(b.getBreed(), b);
      blModel.addElement(s);

      updateBreeds();
    }
    breedList.setSelectedIndex(blModel.indexOf(s));
    breedBox.setText("");
    app.project.markChanged();
  }
  public static void restart(Application app){
    try{
      app.theTemplate.hide();
      //app.template.finalize();
      app.theTemplate = new Template(app);
      app.theTemplate.show();
    } catch (Throwable e){
      System.err.println("Exception: " + e);
    }
  }

  protected void clearListen(){
    Object[] choices = {"Same", "Different", "Cancel"};

    int answer = JOptionPane.showOptionDialog(this,
					      "Do you want to start over with the\n" 
					      + "same template or a different one?",
					      "Clear Template",
					      JOptionPane.YES_NO_CANCEL_OPTION,
					      JOptionPane.QUESTION_MESSAGE,
					      null,
					      choices,
					      choices[2]);
    if (answer == JOptionPane.YES_OPTION) {
      restart(app);
    } else if (answer == JOptionPane.NO_OPTION) {
      loadFile = null;
      restart(app);
    } 
  }
}
