/**********************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *********************************************************************/


/* This function should eventually ask how many of each breed to 
 * make and should also incorporate setting up of the plot area, 
 * either by calling another function, calling another function 
 * the user designs in the template, or have the code right in the 
 * body of this function
 */

package template.observerFuncs;

import template.*;
import template.arguments.*;
import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import java.util.*;  //for Java2 collections

public class Setup extends BaseFunc implements ActionListener{
  final String ca = "  ca\n";
  final String bodyCentered = "  ask-patches\n    [set grass-color :grass-color\n    if (random 100) < :percent-grass [ setpc grass-color ]]\nend";
  final String bodyRandom = "  ask-turtles\n    [setxy (random screen-width) (random screen-height)]\n  ask-patches\n    [set grass-color :grass-color\n    if (random 100) < :percent-grass [ setpc grass-color ]]\nend";
  final String name = "Setup";

  ButtonGroup bg = new ButtonGroup();
  JRadioButton centered = new JRadioButton("centered at the origin");
  JRadioButton randomized = new JRadioButton("distributed randomly");
  JPanel top = new JPanel(),
    nested = new JPanel(),
    panel = new JPanel();
  JLabel turtlesStart = new JLabel("Turtles start  ");
  final String breedCodeInit = "  create-turtles-and-do :turtles [setup]\n";
  String breedCode = breedCodeInit;
  final String initialNumberOf = "Initial number of ",
    grass = "grass",
    grassColor = "grass-color";
  BreedName breedlessTurtles = new BreedName("breedless turtles");
  Map breedSliders = new TreeMap();
  Dimension preferredSize;
  SliderArg grassSlider = new SliderArg("Initial amount of " + grass + " (% of canvas)",100, 25);

  final String centeredCommand = "centered";
  final String randomizedCommand = "randomized";

  public Setup(Collection b){
    breeds = b;

    funcName.setText(name);
    obsFuncP = true;
    predefinedP = true;
    bg.add(centered);
    bg.add(randomized);
    centered.setActionCommand(centeredCommand);
    centered.addActionListener(this);
    randomized.setActionCommand(randomizedCommand);
    randomized.addActionListener(this);
    randomized.setSelected(true);
    nested.setLayout(new BoxLayout(nested, BoxLayout.X_AXIS));
    nested.add(randomized);
    nested.add(centered);
    top.setLayout(new BoxLayout(top, BoxLayout.Y_AXIS));
    top.add(funcName);
    top.add(Box.createVerticalStrut(10));
    top.add(turtlesStart);
    turtlesStart.setAlignmentX(CENTER_ALIGNMENT);
    top.add(nested);
    colorArgs = new ColorArg[1];
    colorArgs[0] = new ColorArg("Color of grass", "green");

    setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

    breedSliders.put(breedlessTurtles, 
		     new SliderArg(initialNumberOf + breedlessTurtles.getName(), 500, 100));
    updateBreeds();
  }
  public String getBody(){
    //assumes breedNames and sliderArgs list the breeds in the same order
    String body = ca + breedCode + (randomized.isSelected() ? bodyRandom : bodyCentered);
    body = replace(body, ":turtles", breedSliders.get(breedlessTurtles).toString());
    body = replace(body, ":percent-grass", grassSlider.toString());
    body = replace(body, ":grass-color", colorArgs[0].toString());
    
    String[] breedNames = (String[])breeds.toArray(new String[0]);
    if(breedNames.length < 1)
      return body;

    for(int i = 0; i < breedNames.length; i++) 
      body = replace(body, 
		     ":" + breedNames[i], 
		     breedSliders.get(new BreedName(breedNames[i])).toString());

    return body;
  }
  public void updateBreeds(){
    Collection oldBreedNames = breedSliders.keySet();
    Iterator i;
    ArrayList breedNames = new ArrayList();
    int j;

    // convert strings in breeds to BreedNames in breedNames
    i = breeds.iterator();
    while(i.hasNext())
      breedNames.add(new BreedName((String)i.next()));
    breedNames.add(breedlessTurtles);

    if((oldBreedNames.containsAll(breedNames) 
	&& breedNames.containsAll(oldBreedNames))){ //breeds hasn't changed
      addComponents();
      return;
    }

    if(!oldBreedNames.containsAll(breedNames)){  //at least one breed has been added
      for(j = 0; j < breedNames.size(); ++j){
	if(breedSliders.get(breedNames.get(j)) == null)
	  breedSliders.put(breedNames.get(j), 
			   new SliderArg(initialNumberOf 
					 + ((BreedName)breedNames.get(j)).getName(), 
					 500, 100));
      }
    }

    if(!breedNames.containsAll(oldBreedNames)){  //at least one breed has been removed
      ArrayList al = new ArrayList(oldBreedNames);

      for(j = 0; j < al.size(); ++j){
	BreedName bn = (BreedName)al.get(j);
	if(!breedNames.contains(bn)) 
	  breedSliders.remove(bn);
      }
    }

    i = breeds.iterator();
    breedCode = breedCodeInit;
    while(i.hasNext()){
      String s = (String)i.next();
      breedCode += replace(replace("  create-:breed-and-do :number [setup-:breed]\n", ":breed", s),
			   ":number",
			   ":" + s);
    }

    addComponents();
  }
  private void addComponents(){
    removeAll();
    add(top);
    Iterator i = breedSliders.values().iterator();
    sliderArgs = new SliderArg[breedSliders.values().size()];
    int j = 0;

    add(Box.createVerticalStrut(10));
    add(colorArgs[0]);
    add(Box.createVerticalStrut(10));
    add(grassSlider);

    while(i.hasNext()){
      sliderArgs[j] = (SliderArg)i.next();
      add(Box.createVerticalStrut(10));
      add(sliderArgs[j++]);
    }
  }
  public ButtonInfo[] getButtonInfo(){
    ButtonInfo[] array = new ButtonInfo[1];
    array[0] = new ButtonInfo(false, "setup", false);
    return array;
  }
  public String[] getVariables(int varType){
    if(varType == GLOBAL_VARS)
      return new String[] {"grass-color"};
    return null;
  }
  public String describe(){
    return "This function sets up the turtles' variables, calls each breed's Setup\nprocedure, and sets the turtles' position to either be randomly\ndistributed around the patch canvas, or grouped at the origin.";
  }

  //listening commands
  public void actionPerformed(ActionEvent e){
    String command = e.getActionCommand();
  }
}

//  running in circles to make this Java 1.1 compatible.  String
//  implements java.lang.Comparable, not
//  java.util.collections.Comparable, thus this is really a
//  wrapper class to make String compatible with
//  com.sun.java.util.collections.TreeMap, etc. etc.
class BreedName implements Comparable {
  String name;

  public BreedName(String s){
    name = s.intern();
  }
  
  public int compareTo(Object o){
    if(o instanceof BreedName)
      return name.compareTo(((BreedName)o).name);
    else
      return -1;
  }

  public boolean equals(Object o){
    if(o instanceof BreedName)
      return name.equals(((BreedName)o).name);
    else
      return false;
  }

  public String getName(){
    return name;
  }

  public int hashCode(){
    return name.hashCode();
  }
}
