/**********************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *********************************************************************/


package template.turtleFuncs;

import template.*;
import starlogo.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;  //for Java2 collections


public class Custom extends BaseFunc implements ActionListener{
  JTextField title = new JTextField(13);
  JLabel nameLabel = new JLabel("Function name:  "),
    bodyLabel = new JLabel("Function Body:  ");
  JTextArea body = new JTextArea(15,25);
  JPanel top = new JPanel(),
    middle = new JPanel();
  JButton addSlider = new JButton("Add slider"),
    addButton = new JButton("Add button");
  final String offset = "  ";  //amount to indent in indent()
  final String addSliderCommand = "add slider",
    addButtonCommand = "add button",
    toBodyCommand = "to body";
  Collection sliders = new ArrayList(),
    buttons = new ArrayList();
  public static String customWord = "<custom function>";
  public static String buttonWord = "Button";
  public static String sliderWord = "Slider";
  public static String nameWord = "Name";
  public static String bodyWord = "Body";
  String oldName = "";
  
  public Custom(Collection breeds){
    this.breeds = breeds;
    body.setLineWrap(true);
    top.setLayout(new javax.swing.BoxLayout(top, javax.swing.BoxLayout.X_AXIS));
    title.setMaximumSize(new Dimension(title.getMaximumSize().width, 
				       title.getFont().getSize() + 10));
    title.setNextFocusableComponent(body);
    top.add(nameLabel);
    top.add(title);
    addSlider.setActionCommand(addSliderCommand);
    addSlider.addActionListener(this);
    addButton.setActionCommand(addButtonCommand);
    addButton.addActionListener(this);
    title.setActionCommand(toBodyCommand);
    title.addActionListener(this);
    middle.setLayout(new javax.swing.BoxLayout(middle, javax.swing.BoxLayout.X_AXIS));
    middle.add(addSlider);
    middle.add(addButton);
    setLayout(new javax.swing.BoxLayout(this, javax.swing.BoxLayout.Y_AXIS));
    add(top);
    add(javax.swing.Box.createVerticalStrut(10));
    add(middle);
    add(javax.swing.Box.createVerticalStrut(10));
    add(bodyLabel); 
    add(new JScrollPane(body));
    add(javax.swing.Box.createVerticalGlue());
  }
  public String getName(){
    String name;
    if(title.getText().trim().equals(""))
      name = customWord;
    else
      name = "u-" + title.getText();
    if(!oldName.equals(name))
      oldName = name;
    return name;
  }
  public String getOldName(){
    return oldName;
  }
  public String getCommand(){
    if(title.getText().trim().equals(""))
      return "custom";
    else
      return "u-" + replace(title.getText(), " ", "-");
  }
  public String getBody(){
    String s = body.getText().trim();
    if(s.indexOf("\nend") != -1){
      s = s.substring(0, s.indexOf("\nend"));
    }
    if(s.startsWith("to ")){
      String temp = s;
      try{
	s = s.substring(body.getLineEndOffset(0), s.length()) + "\nend";
      }catch(Exception ignored){
	System.out.println("uh oh");
	s = temp;
      }
    }
    else
      s += "\nend";
    return replace(Util.indent(s, offset), "\n" + offset + "end", "\nend"); 
    // to keep from indenting the "end"
  }
  public boolean finished(){
    if(title.getText().trim().equals("")){
      JOptionPane.showMessageDialog(this,
				    "Please name your custom function",
				    "Naming error",
				    JOptionPane.ERROR_MESSAGE);
      return false;
    }
    else
      return true;
  }
  public String describe(){
    return "Write your own function here.  You don't need to include the\n\"to [function name]\", just write the body of the function.  You\ndon't need the \"end\" either.  A \"u-\" will be appended to the\nbeginning of the name you give this function.  Keep in mind that you\nwill have to append \"-<breed>\" to any other function in this model\nyou would like to call from your custom procedure, i.e. \"walk-frogs\",\nthis even applies to other custom functions, which already have the\n\"u-\" appended to the beginning of their names, i.e. \"u-jump-frogs\".";
  }
  public SliderInfo[] getSliderInfo(){
    SliderInfo[] dummy = new SliderInfo[0];
    return (SliderInfo[])sliders.toArray(dummy);
  }
  public ButtonInfo[] getButtonInfo(){
    ButtonInfo[] dummy = new ButtonInfo[0];
    return (ButtonInfo[])buttons.toArray(dummy);
  }
  public void addButton(String lineToRun){
    buttons.add(new ButtonInfo(!obsFuncP, lineToRun, false));
  }
  public void addSlider(String variable){
    sliders.add(new SliderInfo(variable, 0, 99, 50));
  }
  public void setObsFunc(boolean b){
    obsFuncP = b;
  }
  public String save(){
    String output = functionWord + " " + customWord + "\n";
    output += nameWord + "  " + Util.quote(title.getText()) + "\n";
    output += saveButtons();
    output += saveSliders();
    output += bodyWord + "  " + Util.quote(Util.replace(body.getText().trim(), "\n", "\\n"));
    return output;
  }
  public String saveButtons(){
    String output = "";
    ButtonInfo[] bi = getButtonInfo();
    for(int i = 0; i < bi.length; i++)
      output += buttonWord + "  " + Util.quote(bi[i].lineToRun) + "\n";
    return output;
  }
  public String saveSliders(){
    String output = "";
    SliderInfo[] si = getSliderInfo();
    for(int i = 0; i < si.length; i++)
      output += sliderWord + "  " + Util.quote(si[i].variable) + "\n";
    return output;
  }
  public boolean load(String input){
    String[] strings = Util.split(input, "\n");
    boolean returnVal = true;
    for(int i = 0; i < strings.length; i++){
      if(strings[i].startsWith(nameWord)){
	title.setText(Util.unquote(strings[i].substring(nameWord.length())).trim());
      } else if(strings[i].startsWith(bodyWord)){
	String temp = Util.unquote(strings[i].substring(bodyWord.length())).trim();
	body.setText(Util.replace(temp, "\\n", "\n"));
      } else if(strings[i].startsWith(buttonWord)){
	returnVal &= loadButton(strings[i]);
      } else if(strings[i].startsWith(sliderWord)){
	returnVal &= loadSlider(strings[i]);
      }
    }
    return returnVal;
  }
  public boolean loadButton(String input){
    input = input.substring(buttonWord.length());
    addButton(Util.quote(input.trim()));
    return true;
  }
  public boolean loadSlider(String input){
    input = input.substring(sliderWord.length());
    addSlider(Util.unquote(input.trim()));
    return true;
  }
  public boolean isChanged(){
    boolean temp = (getName() == customWord) && (body.getText().trim().equals(""));
    return !temp;
  }

  //listener function
  public void actionPerformed(ActionEvent e){
    String command = e.getActionCommand();
    if(command == addSliderCommand){
      String sliderName = JOptionPane.showInputDialog(this, 
						      "What would you like to name the\nnew slider variable?", 
  						      "New Slider", 
  						      JOptionPane.PLAIN_MESSAGE);
      if(sliderName == null)
  	return;
      addSlider(sliderName);

//        SliderDialog sd = SliderDialog.create(new Frame(), "Create Slider");
//        sd.show();
//        SliderWidget.createNew(app.iface);
    }
    else if(command == addButtonCommand){
      String buttonName = JOptionPane.showInputDialog(this, 
						      "What command would you like \nthe new button to execute?", 
						      "New Button", 
						      JOptionPane.PLAIN_MESSAGE);
      if(buttonName == null)
	return;
      
      addButton(buttonName);
//        ButtonWidget.createNew(app.iface);
    }
    else if(command == toBodyCommand){
      body.requestFocus();
    }
  }
}

