/********************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *******************************************************************/

package yoyo;
 
public class Base64
{
    private static final int LINEBREAK_LENGTH = 48;

    protected final static char[] enc_table =
    {
    //   0   1   2   3   4   5   6   7
        'A','B','C','D','E','F','G','H', // 0 
        'I','J','K','L','M','N','O','P', // 1 
        'Q','R','S','T','U','V','W','X', // 2 
        'Y','Z','a','b','c','d','e','f', // 3 
        'g','h','i','j','k','l','m','n', // 4 
        'o','p','q','r','s','t','u','v', // 5 
        'w','x','y','z','0','1','2','3', // 6 
        '4','5','6','7','8','9','+','/'  // 7 
    };

    protected final static byte[] dec_table = 
    { 
        -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 62, -1, -1, -1, 63,
        52, 53, 54, 55, 56, 57, 58, 59, 60, 61, -1, -1, -1, -1, -1, -1,
        -1,  0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14,
        15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, -1, -1, -1, -1, -1,
        -1, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40,
        41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, -1, -1, -1, -1, -1,
        -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
        -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    };
 
    public Base64() {}

    public static String encode(byte[] data)
    {
        return new String(encodeAsByteArray(data));
    }

    public static byte[] encodeAsByteArray(byte[] data)
    {
        int i = 0, j = 0;
        int len = data.length;
        int delta = len % 3;
        int outlen = ((len+LINEBREAK_LENGTH-1)/LINEBREAK_LENGTH)*2 +
            ((len+2)/3)*4 + (len == 0 ? 2 : 0);
        byte[] output = new byte[outlen];

        byte a, b, c;
        for (int count = len / 3; count > 0; count--)
        {
            a = data[i++];
            b = data[i++];
            c = data[i++];
            output[j++] = (byte) (enc_table[(a >>> 2) & 0x3F]); 
            output[j++] = (byte) (enc_table[((a << 4) & 0x30) + ((b >>> 4) & 0x0F)]); 
            output[j++] = (byte) (enc_table[((b << 2) & 0x3C) + ((c >>> 6) & 0x03)]); 
            output[j++] = (byte) (enc_table[c & 0x3F]); 

            if (i % LINEBREAK_LENGTH == 0) {
                output[j++] = (byte) '\r';
                output[j++] = (byte) '\n';
            }
        }

        if (delta == 1)
        {
            a = data[i++];
            output[j++] = (byte) (enc_table[(a >>> 2) & 0x3F]); 
            output[j++] = (byte) (enc_table[((a << 4) & 0x30)]); 
            output[j++] = (byte) '='; 
            output[j++] = (byte) '='; 
        }
        else if (delta == 2)
        {
            a = data[i++];
            b = data[i++];
            output[j++] = (byte) (enc_table[(a >>> 2) & 0x3F]); 
            output[j++] = (byte) (enc_table[((a << 4) & 0x30) + ((b >>> 4) & 0x0F)]); 
            output[j++] = (byte) (enc_table[((b << 2) & 0x3C)]); 
            output[j++] = (byte) '='; 
        }
        if (i == 0 || i % LINEBREAK_LENGTH != 0) {
            output[j++] = (byte) '\r';
            output[j++] = (byte) '\n';
        }

        if (j != outlen)
            throw new InternalError("Bug in Base64.java: incorrect length calculated for base64 output");

        return output;
    }

    public static byte[] decode(byte[] data) 
    {
        int padCount = 0;
        int i, len = data.length;
          int real_len = 0;

        for (i=len-1; i >= 0; --i)
        {
            if (data[i] > ' ')
                real_len++;

            if (data[i] == 0x3D) 
                padCount++;
        }

        if (real_len % 4 != 0)
            throw new IllegalArgumentException("Length not a multiple of 4 in Base64.decode()");

        int ret_len = (real_len/4)*3 - padCount;
        byte[] ret = new byte[ret_len];

        i = 0;
        byte[] t = new byte[4];
        int output_index = 0;
        int j = 0;
        t[0] = t[1] = t[2] = t[3] = 0x3D; 
        while (i < len)
        {
            byte c = data[i++];
            if (c > ' ')
                t[j++] = c;

            if (j == 4)
            {
                output_index += decode(ret, output_index, t[0], t[1], t[2], t[3]);
                j = 0;
                t[0] = t[1] = t[2] = t[3] = 0x3D; 
            }
        }
        if (j > 0)
            decode(ret, output_index, t[0], t[1], t[2], t[3]);

        return ret;
    }

    public static byte[] decode(String msg) throws IllegalArgumentException
    {
        return decode(msg.getBytes());
    }

    private static int decode(byte[] ret, int ret_off, byte a, byte b, byte c, byte d)
    {
        byte da = dec_table[a];
        byte db = dec_table[b];
        byte dc = dec_table[c];
        byte dd = dec_table[d];

        if (da == -1 || db == -1 || (dc == -1 && c != 0x3D) || (dd == -1 && d != 0x3D))
            throw new IllegalArgumentException("Invalid character [" +
                (a & 0xFF) + ", " + (b & 0xFF) + ", " + (c & 0xFF) + ", " + (d & 0xFF) + "] in Base64.decodeByteArray()");

        ret[ret_off++] = (byte) (da << 2 | db >>> 4);
        if (c == 0x3D) 
            return 1;
        ret[ret_off++] = (byte) (db << 4 | dc >>> 2);
        if (d == 0x3D) 
            return 2;
        ret[ret_off++] = (byte) (dc << 6 | dd);
        return 3;
    }
}
