/********************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *******************************************************************/
 package yoyo;

import java.io.PrintWriter;
import java.net.URL;
import java.util.Vector;
import java.util.Enumeration;

public class Context implements Runnable, YoYoMethods, ExprList {

    UberContext ub;
    public PrintWriter out;
    public PrintWriter err;
    public PrintWriter log;
    public URL base;

    public Object[] ilist;
    public int ilistposptr = 0;
    Sym currentfun;
    Symbol ufun;
    Ufun ufunobj;
    Locals locals;
    YoYoObject ufunenv, self;
    public YoYoObject objenv;
    Vector savedynamics, traced = new Vector();
    int nargs=0, precedence = 0, indent = 0, indentlevel = 0;
    long time = 0;
    protected volatile boolean gonow = false, dienow = false;
    public volatile boolean stopnow = false;
    public YoYoCaller caller;
    int contextnum;

    public LogoError errorobj = null;
    public Object ufunreturn = null;
    public static Object novalue = new Object();

    public Object runningp = new Object();
    public volatile boolean runningnowp = false;

    public Context() {}

    public Context(Context c, String s) {
	this(Reader.read(s), c.base, c.out, c.err, c.log, c.caller, c.ub);
    }

    public Context(Context c, Object[] i) {
	this(i, c.base, c.out, c.err, c.log, c.caller, c.ub);
    }

    public Context(Context c) {
	this((Object[])null, c.base, c.out, c.err, c.log, c.caller, c.ub);
    }

    public Context(Context c, PrintWriter out, PrintWriter err) {
	this((Object[])null, c.base, out, err, err, c.caller, c.ub);
    }

    public Context(Context c, PrintWriter out, PrintWriter err, PrintWriter log) {
	this((Object[])null, c.base, out, err, log, c.caller, c.ub);
    }

    public Context(Context c, PrintWriter out, YoYoCaller who) {
	this((Object[])null, c.base, out, out, out, who, c.ub);
    }

    public Context(Object[] ilist, URL base, PrintWriter out,
		   YoYoCaller who, UberContext ub) {
	this(ilist, base, out, out, out, who, ub);
    }

    public Context(Object[] ilist, URL base, PrintWriter out,
		   PrintWriter err, PrintWriter log, YoYoCaller who, UberContext ub) {
	this.ilist = ilist;
	this.base = base;
	this.ub = ub;
	locals = new Locals();
	objenv = new YoYoObject();
	ufunenv = objenv;
	this.out = out;
	this.err = err;
	this.log = log;
	caller = who;
	ub.addContext(this);
	self = toObject();
	new Thread(this).start();
    }

    public YoYoObject getGlobalObj() {
	return ub.globalobj;
    }

    public void run() {
	Thread cur = Thread.currentThread();
	try {
	    while(true) {
		try {
		    if (dienow) break;
		    synchronized (runningp) {
			if (!gonow) {
			    runningnowp = false;
			    runningp.wait();
			}
			try {
			    YoYo.evalExternal(this);
			}
			finally {
			    gonow = false;
			    runningnowp = false;
			    if (caller != null) caller.done(this);
			}
		    }
		}
		catch (InterruptedException ie) {
		    // gracefully die
		}
		catch(Exception e) {
		    err.println("uncaught exception in YoYo.Context:");
		    e.printStackTrace(err);
		    if (err != log) {
			log.println("uncaught exception in YoYo.Context:");
			e.printStackTrace(log);
		    }	
		    System.out.println("uncaught exception in YoYo.Context:");
		    e.printStackTrace(System.out);
		}
	    }
	}
	finally {
	    ub.removeContext(this);
	}
    }

    public void runList(Object[] i) {
	if (runningnowp) waitUntilDone();
	synchronized (runningp) {
	    ilist = i;
	    ilistposptr = 0;
	    indent = 0;
	    stopnow = false;
	    gonow = true;
	    runningnowp = true;
	    runningp.notify();
	    //System.out.println("go is true");
	}
    }

    public void stop() {
	stopnow = true;
	waitUntilDone();
    }

    public void waitUntilDone() {
	while (runningnowp) {
	    try {
		//System.out.println("waiting until death");
		Thread.currentThread().sleep(250);
	    }
	    catch (InterruptedException e) {}
	}
    }

    public void die() {
	stop();
	dienow = true;
    }

    public String toString() {
	return "<YoYo Context #" + contextnum + ">";
    }

    public YoYoObject toObject() {
	YoYoObject obj = new YoYoObject();
	PrimStartup.dload("yoyo.Context", obj);
	obj.defineValue(YoYoObject.kindsym, "thread");
	obj.defineValue(Symbol.lookup("self"), this);
	return obj;
    }

    public String[] methodList() {
	String[] output = {
	    "stop-thread", "prim_stop",
	    "ask-thread", "prim_runlist"
	};
	return output;
    }

    public String[] infixList() {
	return null;
    }

    public void prePrimCall(Primitive p, Object[] arglist, Context c) {}
    public void postPrimCall(Primitive p, Context c) {}

    public static void prim_runlist(Object v0, Object v1, Context c) {
	((Context)((YoYoObject)v0).lookupValue((Symbol.lookup("self")))).runList(YoYo.aList(c, v1));
    }

    public static void prim_stop(Object v0, Context c) {
	((Context)((YoYoObject)v0).lookupValue((Symbol.lookup("self")))).stop();
    }

    public boolean isRunning() {
	return (runningnowp);
    }


    final public Object ilistNext() {
	if (ilistposptr < ilist.length) {
	    return ilist[ilistposptr++];
	}
	else return null;
    }

    final public Object ilistPeek() {
	if (ilistposptr < ilist.length) {
	    return ilist[ilistposptr];
	}
	else return null;
    }

    final public void ilistSkip() {
	ilistposptr++;
    }

    final public void ilistSkipToEnd() {
	ilistposptr = ilist.length;
    }

    final public void setPrev(Object o) {
	// Make this thread-safe
	synchronized(ilist) {
	    if (ilist[ilistposptr-1] instanceof Ilist) {
		ilist[ilist.length - 1] = o;
	    }
	    else ilist[ilistposptr-1]= o;
	}
    }

    /*
      public void traceEnter(String name, Object[] arglist) {
      double currenttime = (System.currentTimeMillis() - time) / 1000.0;
      System.out.println("Entering " + name + " at " + currenttime);
      }

      public void traceExit(String name, Object value) {
      double currenttime = (System.currentTimeMillis() - time) / 1000.0;
      System.out.println("Exiting " + name + " at " + currenttime);
      }

      public void traceThrow(String name, Throwable e) {
      double currenttime = (System.currentTimeMillis() - time) / 1000.0;
      System.out.println("Throwing " + name + " at " + currenttime);
      }
    */

    public void traceEnter(String name, Object[] arglist) {
	StringBuffer toprint = new StringBuffer();
	for(int i = 0; i<indent; i++) toprint.append(' ');
	toprint.append("Calling ");
	toprint.append(name);
	StringBuffer args = new StringBuffer();
	int anyargs = 0;
	for(int i = 0; i< arglist.length; i++) {
	    if (arglist[i] != null && arglist[i] != Pool.undefined) {
		anyargs++;
		args.append(YoYo.printToString(arglist[i]));
		args.append(' ');
	    }
	}
	if (anyargs > 0) {
	    if (anyargs > 1) toprint.append(" with args: ");
	    else toprint.append(" with arg: ");
	    toprint.append(args.toString());
	}
	err.println(toprint.toString());
	indent += 2;
	if (indent > 12) { indentlevel++; indent = 2; }
    }

    public void traceExit(String name, Object value) {
	indent -= 2;
	if (indentlevel > 0 && indent <= 0) { indentlevel--; indent = 12; }
	StringBuffer toprint = new StringBuffer();
	for(int i = 0; i<indent; i++) toprint.append(' ');

	toprint.append("Returning from ");
	toprint.append(name);
	if (value != null && value != Pool.undefined && value != novalue) {
	    toprint.append(" with result: ");
	    toprint.append(YoYo.printToString(value));
	}
	err.println(toprint.toString());
    }

    public void traceThrow(String name, Throwable e) {
	indent -= 2;
	if (indentlevel > 0 && indent <= 0) { indentlevel--; indent = 12; }
	StringBuffer toprint = new StringBuffer();
	for(int i = 0; i<indent; i++) toprint.append(' ');
	toprint.append("Abnormal exit from ");
	toprint.append(name);
	toprint.append(" because: ");
	if (e != null) {
	    if (e instanceof ThrowReturn) {
		ThrowReturn tr = (ThrowReturn)e;
		toprint.append("Error: ");
		toprint.append(YoYo.printToString(tr.errortype));
		toprint.append(' ');
		toprint.append(YoYo.printToString(tr.value));
	    } else {
		toprint.append(YoYo.printToString(e));
	    }
	}
	err.println(toprint.toString());
    }


    public Vector trace(Function f) {
	traced.insertElementAt(f, 0);
	return traced;
    }

    public Vector untrace(Function f) {
	traced.removeElement(f);
	return traced;
    }

    public void untraceAll() {
	for(Enumeration e = traced.elements(); e.hasMoreElements();) {
	    ((Function)e.nextElement()).untrace();
	}
	traced.removeAllElements();
    }
}

