/********************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *******************************************************************/
 package yoyo;

import javax.swing.*;
import javax.swing.text.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;

public class JCommandCenter extends JTextPane implements YoYoCaller {

    public Context c;
    public Ufun formatInput;

    String bullet = "\u00b7";
    Position bulletPosition;
    PrintWriter out, err;

    protected static Font LISTENER_FONT = new Font("Monospaced", Font.PLAIN, 12);
    Style boldon;
    SimpleAttributeSet boldoff;
    protected Keymap km;

    public JCommandCenter() {
	super(new DefaultStyledDocument());

	setCaretPosition(0);
	setMargin(new Insets(5,5,5,5));
	LISTENER_FONT = Font.getFont("yoyo.listener.font", LISTENER_FONT);
	setFont(LISTENER_FONT);

	boldoff = new SimpleAttributeSet();
	StyleConstants.setBold(boldoff, false);

	boldon = addStyle("normal bold", StyleContext.getDefaultStyleContext().getStyle(StyleContext.DEFAULT_STYLE));
	StyleConstants.setBold(boldon, true);
	//setLogicalStyle(boldon);

	km = addKeymap("Command Center", getKeymap());
	KeyStroke ks = KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0);
	Action act = new TextAction("Enter") {
		public void actionPerformed(ActionEvent e) {
		    handlecr();
		}
	    };
	km.addActionForKeyStroke(ks, act);
	setKeymap(km);

    }

    public void replaceSelection(String content) {
        if (! isEditable()) {
            getToolkit().beep();
            return;
        }
        Document doc = getStyledDocument();
        if (doc != null) {
            try {
                Caret caret = getCaret();
                int p0 = Math.min(caret.getDot(), caret.getMark());
                int p1 = Math.max(caret.getDot(), caret.getMark());
                if (p0 != p1) {
                    doc.remove(p0, p1 - p0);
                }
                if (content != null && content.length() > 0) {
		    setCharacterAttributes(boldon, false);
                    doc.insertString(p0, content, getInputAttributes());
                }
            } catch (BadLocationException e) {
                getToolkit().beep();
            }
        }
    }

    public void handlecr() {
	Element para = getStyledDocument().getParagraphElement(getCaretPosition());
	int start = para.getStartOffset();
	int end = para.getEndOffset() - 1;
	try {
	    String torun = getStyledDocument().getText(start, end-start);
	    Caret caret = getCaret();
	    int dot = caret.getDot();
	    int mark = caret.getMark();
	    if (dot != mark) {
		// there is a selection
		replaceSelection("\n");
		return;
	    }
	    // there's no selection
	    if (!torun.trim().equals("")) {
		bulletPosition = getStyledDocument().createPosition(end - 1);
		getStyledDocument().insertString(end, bullet, boldoff);
		if (end == getStyledDocument().getLength() - 1) {
		    replaceSelection("\n");
		} else setCaretPosition(end + 2);
		runLine(torun);
	    }
	} catch (BadLocationException e) { e.printStackTrace(); }
    }

    public void runLine(String torun) {
	if (formatInput == null) {
	    c.runList(Reader.read(torun));
	}
	else {
	    Object[] ilist = Reader.read("");
	    Object lineToRun = null;
	    Object[] formatArgs={ torun };
	    int oldnargs = c.nargs;
	    Object[] oldilist = c.ilist;
	    int oldilistposptr = c.ilistposptr;
	    try {
		c.ilist = ilist;
		c.ilistposptr = 0;
		c.nargs = 1;
		lineToRun = YoYo.evalFunction(formatInput.name,
					      formatInput,
					      c.getGlobalObj(), formatArgs, c);
	    }
	    finally {
		c.nargs = oldnargs;
		c.ilist = oldilist;
		c.ilistposptr = oldilistposptr;
	    }
	    if (lineToRun instanceof String)
		c.runList(Reader.read((String)lineToRun));
	    else if (lineToRun instanceof Object[])
		c.runList((Object[])lineToRun);
	    else LogoError.error("The output from listener-function: " +
				 YoYo.printToString(lineToRun) + "," +
				 LogoError.NOT_STRING);
	}
    }

    public void done(Context c) {
	if (bulletPosition != null) {
	    try {
		Position cursor = getStyledDocument().createPosition(getCaretPosition());
		int bulletOffset = bulletPosition.getOffset() + 1;


		//trying to get rid of exception thrown here during "cc" in the command center

		//getStyledDocument().remove(bulletOffset, 1);

		try {
			getStyledDocument().remove(bulletOffset, 1);
		} catch (BadLocationException e) { getStyledDocument().remove(bulletOffset, 0); }


		Caret caret = getCaret();
		int dot = caret.getDot();
		int mark = caret.getMark();
		if (dot == mark) {
		    setCaretPosition(cursor.getOffset());
		} else {
		    caret.setDot(mark - 1);
		    caret.moveDot(dot - 1);
		}
		bulletPosition = null;
	    }
	    catch (BadLocationException e) { e.printStackTrace(); }
	}
    }

    public void setFormatInput(Ufun x) {
	formatInput = x;
    }

    public PrintWriter getOut() {
	if (out == null) {
	    out = new PrintWriter(new CommandCenterWriter(), true);
	}
	return out;
    }

    public PrintWriter getErr() {
	if (err == null) {
	    err = new PrintWriter(new CommandCenterWriter(), true);
	}
	return err;
    }

    public void flush() {
	getOut().flush();
    }

    private class CommandCenterWriter extends Writer {

	StringBuffer buf = new StringBuffer(1024);
	StringBuffer flushingBuf;

	public void close() throws IOException {
	    flush();
	}

	public void flush() throws IOException {
	    if (buf.length() == 0) return;
	    flushingBuf = buf;
	    buf = new StringBuffer(buf.capacity());
	    SwingUtilities.invokeLater(new Runnable() {
		    public void run() {
			try {
			    int location = getStyledDocument().getLength();
			    //System.out.println("inserting text at location: " + location);
			    if (bulletPosition != null) {
				location = bulletPosition.getOffset() + 3;
				//System.out.println("moving position to bullet location: " + location);
			    }
			    getStyledDocument().insertString(location, flushingBuf.toString(), boldoff);
			    setCaretPosition(location + 1);
			    moveCaretPosition(location + flushingBuf.length());
			    
			} catch (BadLocationException e) { e.printStackTrace(); }
		    }
		});
	}

	public void write(char[] cbuf, int off, int len) throws IOException {
	    synchronized(lock) {
		buf.append(cbuf, off, len);
	    }
	}

    }
}


