/********************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *******************************************************************/
 
package yoyo;

import javax.swing.*;
import javax.swing.text.BadLocationException;
import java.awt.Font;
import java.awt.Color;
import java.awt.SystemColor;

public class JListener extends JTextArea implements YoYoCaller {

  public Context c;
  public int linemark;
  public Ufun formatInput;
  public String bullet = (YoYo.solarisp) ? "\u00b7" : "\u25cf"; // "\u2022";
  protected static Font LISTENER_FONT = new Font("Monospaced", Font.PLAIN, 12);
  

  public JListener(String s, Context c) {
  	super(s, 15, 60);
  	this.c=c;
	setBackground(Color.white);
	setForeground(SystemColor.textText);
	LISTENER_FONT = Font.getFont("yoyo.listener.font", LISTENER_FONT);
	setFont(LISTENER_FONT);
	setLineWrap(false);
	setWrapStyleWord(true);
	setEditable(true);
  }

  public JListener(String s) {
    this(s, null);
  }

  public JListener(Context c) {
    this("", c);
  }
  
  public JListener() {
    this("", null);
  }

  public void handlecr(){
    //System.out.println("handle cr");
    String s=getText();
    //System.out.println("got text");
    try {
      int line = getLineOfOffset(getCaretPosition());
      //System.out.println("line is: " + line);
      int sol = getLineStartOffset(line);
      //System.out.println("start is " + sol);
      int eol = getLineEndOffset(line) - 1 ;
      //System.out.println("run line: " + s.substring(sol, eol));
      linemark = eol;
      String torun = s.substring(sol,eol);

    if (getSelectionStart() == sol && getSelectionEnd() == eol) {
      replaceRange("\n", sol, eol);
      setCaretPosition(sol+1);
      return;
    }

    if (!torun.equals("")) {
      getDocument().insertString(eol, bullet, null);
      if (eol == s.length()) { 
	getDocument().insertString(eol+1, "\n", null);
      }
      else setCaretPosition(eol+2);
      String toRun = s.substring(sol, eol);
      if (formatInput == null)
	runLine(toRun);
      else {
	Object[] ilist = Reader.read("");
	Object lineToRun = null;
	Object[] formatArgs={ toRun };
	int oldnargs = c.nargs;
	  Object[] oldilist = c.ilist;
	  int oldilistposptr = c.ilistposptr;
	  try {
	    c.ilist = ilist;
	    c.nargs = 1;
	    lineToRun = YoYo.evalFunction(formatInput.name, 
					  formatInput, 
					  c.getGlobalObj(), formatArgs, c);
	  }
	  finally { 
	    c.nargs = oldnargs;
	    c.ilist = oldilist;
	    c.ilistposptr = oldilistposptr;
	  }
	  if (lineToRun instanceof String)
	    runLine((String)lineToRun);
	  else
	    runLine(toRun);
	}
      }
    }
    catch (BadLocationException e) {
	c.err.println("bad location: " + e.toString());
    }
    catch (Exception f) {
	c.err.println("uncaught: " + f.toString());
    }
    //System.out.println("end handle cr");
  }
  
  
  public void runLine(String s) {
    c.runList(Reader.read(s));
  }  
  
  public void done(Context c) {
    int i = getCaretPosition();
    if (linemark > 0) {
      replaceRange("", linemark, linemark+1);
    }
    if (i > 0) {
      if (i > getText().length()) {
	setCaretPosition(getText().length());
      } else setCaretPosition(i-1);
    }
  }

  public void setFormatInput(Ufun x) {
    formatInput = x;
  }
  
}
