/********************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *******************************************************************/
 package yoyo;

import java.io.OutputStream;
import javax.swing.JTextArea;
import javax.swing.text.BadLocationException;
import java.awt.Component;

public class JTEStream extends OutputStream implements TEStream {
  
  JTextArea te;
  char linesep;
  StringBuffer buffer = new StringBuffer();

  public JTEStream() {
    this.linesep = System.getProperty("line.separator").charAt(0);
  }    

  public JTEStream(JTextArea te) {
    this.te = te;
    this.linesep = System.getProperty("line.separator").charAt(0);
  }

    public void setTextArea(Component c) {
	this.te = (JTextArea)c;
    }

  public void write(int n) {
    if (n=='\r') buffer.append('\n');
    else if (n=='\n') {
      if (linesep == '\n') buffer.append('\n'); else return;
    }
    else buffer.append((char)n);
  }

  public void write(char[] buf, int start, int length) {
    buffer.ensureCapacity(buf.length + buffer.length());
    for(int i = 0; i < length; i++) {
      char c = buf[i + start];
      if (c=='\r') 
	buffer.append('\n');
      else if (c=='\n') {
	if (linesep == '\n') 
	  buffer.append('\n'); 
      }
      else buffer.append(c);
    }
  }

  public void flush(){
    int pos = te.getCaretPosition();
    try {
      te.getDocument().insertString(pos, buffer.toString(), null);
    }
    catch (BadLocationException e) {};
    te.setCaretPosition(pos+buffer.length());
    buffer.setLength(0);
  }
}
