/********************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *******************************************************************/
 

package yoyo;

import java.awt.datatransfer.*;
import java.io.PrintWriter;
import java.io.IOException;
import java.net.URL;
import java.net.MalformedURLException;
import java.awt.Container;
import java.awt.Toolkit;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowEvent;
import java.awt.event.ItemEvent;
import java.awt.event.KeyEvent;
import java.awt.event.ItemListener;
import java.awt.event.WindowAdapter;
import java.awt.event.KeyAdapter;
import java.awt.GraphicsEnvironment;
import javax.swing.JFrame;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.UIManager;
import javax.swing.JScrollPane;
import javax.swing.JMenuBar;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.text.*;
import javax.swing.*;

public class JToplevel extends JFrame implements ClipboardOwner, ActionListener, ItemListener {

  static String[] sizes = { "6", "8", "9", "10", "11", "12", "14", "16", "18", 
			    "24", "36", "48", "72" };

    public static String bootpath;
    public static String bootfile;
    static Clipboard clip = Toolkit.getDefaultToolkit().getSystemClipboard();
    public static UberContext ub;
    public static URL base = null;

    protected Context c;
  JCommandCenter cc;
    JScrollPane jsp;
  JCheckBoxMenuItem currentfont;
  JCheckBoxMenuItem currentsize;
    protected boolean firstone;

    public int commandMask = Toolkit.getDefaultToolkit().getMenuShortcutKeyMask();

    public JToplevel(String s, boolean firstone){
    super(s);
    //YoYo.yoyoframe = this;
    this.firstone = firstone;
    if (firstone) appInit();
    windowInit();
    String title = c.toString();
    if (!firstone) setTitle(title.substring(1, title.length()-1));
    loadBootFile();
  }

    /* invoke a JToplevel from another application, not JToplevel */
    public JToplevel(String s, UberContext ub) {
	super(s);
	//YoYo.yoyoframe = this;
	this.firstone = false;
	this.ub = ub;
	this.base = ub.base;
	windowInit();
	String title = c.toString();
	setTitle(title.substring(1, title.length()-1));
    }

    public static String quote(String path) {
	StringBuffer sb = new StringBuffer(path.length());
	for(int i =0 ; i < path.length(); i++) {
	    char c = path.charAt(i);
	    if (c == '\\')  sb.append("\\\\");
	    else sb.append(c);
	}
	return sb.toString();
    }

  public static void main(String[] args){
      readArgs(args);
      JToplevel t1 = new JToplevel("YoYo", true);
  }

    public static void readArgs(String[] args) {
	if (args.length>1) {
	    bootpath = args[0].trim();
	    if (bootpath.equals("")) bootpath = null;
	    //System.out.println("bootpath = " + bootpath);
	    bootfile = args[1].trim();
	    if (bootfile.equals("")) bootfile = null;
	    //System.out.println("bootfile = " + bootfile);
	} else if (args.length > 0) {
	    bootfile = args[0].trim();
	    if (bootfile.equals("")) bootfile = null;
	    //System.out.println("bootfile = " + bootfile);
	}
    }


  void appInit() {
    try { 
      base = new URL("file", "", System.getProperty((YoYo.pcp) ? "user.home" : "user.dir")+"/");
    }
    catch (MalformedURLException e) {System.out.println(e);}
    //System.out.println("about to make Context");
    ub = new UberContext(base);
  }

    public void windowInit() {
	setupMenus();
	createComponents();
	addComponentsToLayout();
	addListeners();
	cc.requestFocus();
	sendWelcome();
    }
    
    public void createComponents() {
	cc=new JCommandCenter(); 
	c = new Context(null, base, cc.getOut(), cc, ub);
	cc.c = c;
	jsp = new JScrollPane(cc, 
			      JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
			      JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
    }

    public void addComponentsToLayout() {
	getContentPane().add(jsp);
	setSize(400, 250);
	show();
    }

    public void addListeners() {
    addWindowListener(new WindowAdapter() {
	public void windowClosing(WindowEvent e) {
	  c.die();
	  e.getWindow().dispose();
	}
      });

    cc.addKeyListener(new KeyAdapter() {
	public void keyPressed(KeyEvent e) {
	  JCommandCenter cc = (JCommandCenter) e.getComponent();
	  char key = e.getKeyChar();
		
	  if (cc.c.runningnowp) {
	    cc.c.stop(); 
	    e.consume(); 
	    return; 
	  }
		
	  if(key== '\n' || key =='\r') {
	    if (!(e.isShiftDown())) {
	      cc.handlecr(); 
	      e.consume(); 
	    }
	  }
	  if(key == '\t') {
	    cc.c.out.print("   ");
	    cc.c.out.flush();
	    e.consume();
	  }
	}
	    
	public void keyTyped(KeyEvent e) {
	  char c = e.getKeyChar();
	  if (c=='\n' || c == '\r') {
	    if (!e.isShiftDown()) e.consume();
	  } else if (c == '\t') e.consume();
	}
      });
    }

    public void sendWelcome() {
	c.out.println("Welcome to YoYo!");
    }

    public void loadBootFile() {
	if (firstone && bootfile != null) {
	    cc.runLine("load \"file:" + quote(bootfile) + "\"");
	    c.waitUntilDone();
	    if (bootpath != null) {
		cc.runLine("cd \"file:" + quote(bootpath) + "\"");
		c.waitUntilDone();
	    }
	    runStartup();
	}
    }

    public void runStartup() {
	Object val= c.getGlobalObj().lookupValue(Symbol.lookup("startup"));
	if (val != null) {
	    if (val instanceof Function) {
		cc.runLine("startup");
	    }
	}
    }




  void setupMenus() {
    JMenuBar mb = new JMenuBar();	
    // set up menu bar
    JMenu filemenu = setupFileMenu();
    JMenu editmenu = setupEditMenu();
    JMenu fontmenu = setupFontMenu();
    JMenu sizemenu = setupSizeMenu();

    mb.add(filemenu);
    mb.add(editmenu);
    mb.add(fontmenu);
    mb.add(sizemenu);

    setJMenuBar(mb);
  }

    JMenu setupFileMenu() {
	// file menu
	JMenu filemenu = new JMenu("File");
	JMenuItem mi = new JMenuItem("New", KeyEvent.VK_N);
	mi.addActionListener((ActionListener)this);
	filemenu.add(mi);
	mi = new JMenuItem("Open...", KeyEvent.VK_O);
	mi.addActionListener((ActionListener)this);
	filemenu.add(mi);
	mi = new JMenuItem("Open URL...");
	mi.addActionListener((ActionListener)this);
	filemenu.add(mi);
	filemenu.addSeparator();
	mi = new JMenuItem("New Listener");
	mi.addActionListener((ActionListener)this);
	filemenu.add(mi);
	mi = new JMenuItem("Close Listener", KeyEvent.VK_W);
	mi.addActionListener((ActionListener)this);
	filemenu.add(mi);
	if (!YoYo.macosxp) {
	    filemenu.addSeparator();
	    mi = new JMenuItem("Quit", KeyEvent.VK_Q);
	    mi.addActionListener((ActionListener)this);
	    filemenu.add(mi);
	}
	return filemenu;
    }
    
    JMenu setupEditMenu() {
	// edit menu
	JMenu editmenu = new JMenu("Edit");
	JMenuItem mi = new JMenuItem("Undo", KeyEvent.VK_Z);
	mi.addActionListener((ActionListener)this);
	editmenu.add(mi);
	mi = new JMenuItem(DefaultEditorKit.cutAction);
	mi.setText("Cut");
	mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X, commandMask));
	editmenu.add(mi);
	mi = new JMenuItem(DefaultEditorKit.copyAction);
	mi.setText("Copy");
	mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C, commandMask));
	editmenu.add(mi);
	mi = new JMenuItem(DefaultEditorKit.pasteAction);
	mi.setText("Paste");
	mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_V, commandMask));
	editmenu.add(mi);
	editmenu.addSeparator();
	mi = new JMenuItem(DefaultEditorKit.selectAllAction);
	mi.setText("Select All");
	mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_A, commandMask));
	editmenu.add(mi);
	return editmenu;
    }

    JMenu setupFontMenu() {
	// fontmenu
	JMenu fontmenu = new JMenu("Font");
	JCheckBoxMenuItem cmi;
	String[] fonts = GraphicsEnvironment.getLocalGraphicsEnvironment().getAvailableFontFamilyNames();
	for (int i = 0; i< fonts.length; i++) {
	    if (fonts[i].equals("Monospaced")) {
		cmi = new JCheckBoxMenuItem(fonts[i], true);
		currentfont = cmi;
	    } else {
		cmi = new JCheckBoxMenuItem(fonts[i], false);
	    }
	    cmi.addItemListener((ItemListener)this);
	    //cmi.setFont(new Font(fonts[i], Font.PLAIN, 11));
	    fontmenu.add(cmi);
	}
	return fontmenu;
    }
    
    JMenu setupSizeMenu() {
	JMenu sizemenu = new JMenu("Size");
	for (int i = 0; i< sizes.length; i++) {
	    JCheckBoxMenuItem cmi;
	    if (sizes[i].equals("12")) {
		cmi = new JCheckBoxMenuItem(sizes[i], true);
		currentsize = cmi;
	    } else {
		cmi = new JCheckBoxMenuItem(sizes[i], false);
	    }
	    cmi.addItemListener((ItemListener)this);
	    sizemenu.add(cmi);
	}
	return sizemenu;
    }
  
  public void actionPerformed(ActionEvent evt) {
    JMenuItem mi = (JMenuItem)evt.getSource();
    String arg = mi.getText();
    if (arg.equals("New Listener")) {
      new JToplevel("YoYo", false);
      return;
    }
    if (arg.equals("Close Listener")) {
      c.die();
      dispose();
    }
    if (arg.equals("Quit")) {
      System.exit(0);
      return;
    }
//     if (arg.equals("Cut")) {
//       String foo = cc.getSelectedText();
//       if(foo.equals("")) return;
//       StringSelection bar = new StringSelection(foo);
//       clip.setContents(bar, this);
//       int start = cc.getSelectionStart();
//       cc.replaceRange("", start, cc.getSelectionEnd());      
//       cc.select(start, start);
//       return;
//     }
//     if (arg.equals("Copy")) {
//       String foo = cc.getSelectedText();
//       if(foo.equals("")) return;
//       StringSelection bar = new StringSelection(foo);
//       clip.setContents(bar, this);      
//       return;
//     }
//     if (arg.equals("Paste")) {
//       Transferable trans = clip.getContents(this);
//       if (trans != null) {
// 	try {
// 	  Object thing = trans.getTransferData(DataFlavor.stringFlavor);
// 	  if (thing instanceof String) {
// 	    String s = (String)thing;
// 	    int len = s.length();
// 	    int cur = cc.getCaretPosition();
// 	    if (len == 0) {
// 	      cc.insert(s, cur);
// 	      cc.select(cur + len, cur + len);
// 	    }
// 	    else { 
// 	      int start = cc.getSelectionStart();
// 	      cc.replaceRange(s, start, cc.getSelectionEnd());      
// 	      cc.select(start + len, start + len);
// 	    }
// 	  }
// 	}
// 	catch (IOException e) {}
// 	catch (UnsupportedFlavorException e) {}
//       }
//      return;
//    }
//    if (arg.equals("Select All")) {
//      cc.selectAll();
//         return;
//    }
    
  }
  
  public void itemStateChanged(ItemEvent evt) {
    JCheckBoxMenuItem cmi = (JCheckBoxMenuItem)evt.getSource();
    String arg = cmi.getText();
    if (!cmi.isSelected()) return;
    try {
      int value = Integer.parseInt(arg);
      if (currentsize == cmi) return;
      Font current = cc.getFont();
      cc.setFont(new Font(current.getName(), current.getStyle(), value));
      currentsize.setSelected(false);
      //cmi.setState(true);
      currentsize = cmi;
      return;
    }
    catch (Exception e) {
      if (currentfont == cmi) return;
      Font current = cc.getFont();
      cc.setFont(new Font(arg, current.getStyle(), current.getSize()));
      currentfont.setSelected(false);
      //cmi.setState(true);
      currentfont = cmi;
      return;
    }
  }

  public void lostOwnership(Clipboard clip, Transferable t) {}

}
