/********************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *******************************************************************/
 package yoyo;

import java.awt.TextArea;
import java.awt.Color;
import java.awt.SystemColor;
import java.awt.Font;
import java.awt.Toolkit;

public class Listener extends TextArea implements YoYoCaller {

  public Context c;
  public int linemark;
  Ufun formatInput;
  //String bullet = "\u00b7";
  String bullet = (YoYo.solarisp) ? "\u00b7" : "\u2022";
  private static Font LISTENER_FONT = new Font("Monospaced", Font.PLAIN, 12);
    
  public Listener(String s, Context c, int scroll) {
    super(s, 15, 60, scroll);
    this.c=c;
    setBackground(Color.white);
    setForeground(SystemColor.textText);
    //System.out.println(YoYo.printToString(Toolkit.getDefaultToolkit().getFontList()));
    LISTENER_FONT = Font.getFont("yoyo.listener.font", LISTENER_FONT);
    setFont(LISTENER_FONT);
  }
  
  public Listener(String s) {
    this(s, null, SCROLLBARS_BOTH);
  }

  public Listener(Context c) {
    this("", c, SCROLLBARS_BOTH);
  }
  
  public Listener(String s, int scroll) {
    this(s, null, scroll);
  }

  public Listener() {
    this("", null, SCROLLBARS_BOTH); 
  }

  
  public void handlecr() {
    String s=getText();
    int sol=findStartOfLine(s, getCaretPosition());
    int eol=findEndOfLine(s, sol);
    //System.out.println("run line: " + s.substring(sol, eol));
    linemark = eol;
    String torun = s.substring(sol,eol);

    if (getSelectionStart() == sol && getSelectionEnd() == eol) {
      replaceRange("\n", sol, eol);
      setCaretPosition(sol+1);
      return;
    }

    if (!torun.equals("")) {
      insert(bullet, eol);
      if (eol == s.length()) { 
	    insert("\n", eol+1);
      } 
      setCaretPosition(eol+2);
      String toRun = s.substring(sol, eol);
      if (formatInput == null)
	     runLine(toRun);
      else {
		Object[] ilist = Reader.read("");
		Object lineToRun = null;
		Object[] formatArgs={ toRun };
		int oldnargs = c.nargs;
		Object[] oldilist = c.ilist;
		int oldilistposptr = c.ilistposptr;
		try {
		  c.ilist = ilist;
		  c.nargs = 1;
		  lineToRun = YoYo.evalFunction(formatInput.name, 
						formatInput, 
						c.getGlobalObj(), formatArgs, c);
		}
		finally { 
		  c.nargs = oldnargs;
		  c.ilist = oldilist;
		  c.ilistposptr = oldilistposptr;
		}
		if (lineToRun instanceof String)
	  runLine((String)lineToRun);
	else
	  runLine(toRun);
      }
    }
  }
    
  int findStartOfLine(String s, int i){
    int val = s.lastIndexOf(10,i-1);
    if (val<0) return 0;
    return val+1;
  }
    
  int findEndOfLine(String s, int i){
    int val = s.indexOf("\n",i);
    if (val<0) return s.length();
    return val;
  }
    
  public void runLine(String s) {
      //System.out.println("running: " + s);
    c.runList(Reader.read(s));
  }  
    
  public void done(Context c) {
    int i = getCaretPosition();
    // the following is a space instead of empty string as workaround for mrj2.1
    if (linemark > 0) {
      //if (YoYo.applet == null) { // it's an application, we're legal
      //if (System.getProperty("os.name").equals("Mac OS")) 
      //  replaceRange(" ", linemark, linemark + 1);
      //else replaceRange("", linemark, linemark + 1);
      //} else 
      replaceRange("", linemark, linemark + 1);
    }
    if (i > 0) {
	 	if (i == getText().length()) {
	 	   	setCaretPosition(i);
	 	} else setCaretPosition(i-1);
    }
  }
    
  public void setFormatInput(Ufun x) {
    formatInput = x;
  }
}
