/********************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *******************************************************************/
 package yoyo;

import java.util.Hashtable;

public class Locals {

    Locals parent;
    Symbol[] names;
    Object[] values;
	
    static Symbol[] nonames = new Symbol[0];
    static Object[] empty = new Object[0];
    
    public Locals(Locals l) {
	super();
	parent = l;
	names = nonames;
	values = empty;
    }
	
    public Locals() {
	super();
	parent = null;
	names = nonames;
	values = empty;
    }
	
    public Locals(Symbol[] argnames, Object[] values) {
	super();
	parent = null;
	this.names = argnames;
	this.values = values;
    }

  public void defineVars(Symbol[] s) {
    Symbol[] newnames = new Symbol[names.length+s.length];
    Object[] newvalues = new Object[values.length+s.length];
    System.arraycopy(names, 0, newnames, 0, names.length);
    System.arraycopy(values, 0, newvalues, 0, values.length);
    System.arraycopy(s, 0, newnames, names.length, s.length);
    for(int i = 0; i<s.length; i++) {
      values[i+names.length] = Pool.undefined;
    }
    names = newnames;
    values = newvalues;
  }

  public void defineLocals(Symbol[] s) {
    names = s;
    values = new Object[s.length];
  }

    public int defineVar(Symbol s) {
	Symbol[] newnames = new Symbol[names.length+1];
	Object[] newvalues = new Object[values.length+1];
	System.arraycopy(names, 0, newnames, 0, names.length);
	System.arraycopy(values, 0, newvalues, 0, values.length);
	newnames[names.length] = s;
	newvalues[values.length] = Pool.undefined;
	names = newnames;
	values = newvalues;
	return names.length - 1; // index of new var.
    }
	
    public void defineValue(Symbol s, Object o) {
	Symbol[] newnames = new Symbol[names.length+1];
	Object[] newvalues = new Object[values.length+1];
	System.arraycopy(names, 0, newnames, 0, names.length);
	System.arraycopy(values, 0, newvalues, 0, values.length);
	newnames[names.length] = s;
	newvalues[values.length] = o;
	names = newnames;
	values = newvalues;
    }

    public void setValue(Symbol s, Object o) {
      for(int i = 0; i<names.length; i++) {
	if (names[i] == s) {
	  values[i] = o;
	  return;
	}
      }
      defineValue(s, o);
    }

    public void setValue(LocalRef l, Object o) {
	values[l.num] = o;
    }

    public Object lookupValue(Symbol s) {
	for(int i = 0; i<names.length; i++) {
	  if (names[i] == s) {
	    if (values[i] == Pool.undefined) return null;
	    else return values[i];
	  }
	}
	return null;
    }

    public Object lookupValue(LocalRef l) {
	if (values[l.num] == Pool.undefined) return null;
	else return values[l.num];
    }

    public void undefineVar(LocalRef l) {
	values[l.num] = Pool.undefined;
    }

    public void undefineLocals() {
      names = new Symbol[0];
      values = new Object[0];
    }

    public boolean boundp(Symbol s) {
      for(int i = 0; i<names.length; i++) {
	if (names[i] == s) {
	  if (values[i] == Pool.undefined) return false;
	  else return true;
	}
      }
      return false;
    }

    public boolean boundp(LocalRef l) {
	if (values[l.num] != Pool.undefined) return true;
	return false;
    }

    public int index(Symbol s) {
      for(int i = 0; i<names.length; i++) {
	if (names[i] == s) {
	  return i;
	}
      }
      return -1;
    }


}
