/********************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *******************************************************************/
 package yoyo;

import java.util.ResourceBundle;

public class LogoError extends RuntimeException {

    public static String TOO_FEW_REPORTERS, TOO_MANY_REPORTERS;
    public static String I_DONT_KNOW_HOW, I_DONT_KNOW_WHAT;
    
    public static String NOT_STRING, NOT_OBJECT, NOT_LIST, NOT_SYMBOL,
	NOT_NUMBER, NOT_INTEGER, NOT_ILIST, NOT_PROCEDURE, NOT_WEAK,
	NOT_LOCALE, NOT_RESOURCE_BUNDLE, NOT_LIST_OR_STRING;

    public static String STOPPED, IN, ANDOR;
    public static String CANTTAKE,BF,BL,OFEMPTYSTRING,FIRST,LAST,
	CANTPICK,FROMEMPTYLIST,FROMEMPTYSTRING, INDEX, INDEXTOOLOW, INDEXTOOHIGH,
	STARTINGINDEX,ENDINGINDEX,ISGREATERTHAN,CANTCOMPUTE,OFEMPTYLIST,
	OFLIST,HASNONUMBERS,MEDIAN,AVERAGE,VARIANCE,SD,MODE,SUM,ELEMENT,CHARACTER;

    static {
	initLogoErrorStrings();
    }

    public static void initLogoErrorStrings() {
	ResourceBundle rb = PrimStartup.getYoYoErrorsBundle();
	TOO_FEW_REPORTERS = rb.getString("too_few_reporters");
	TOO_MANY_REPORTERS = rb.getString("too_many_reporters");
	I_DONT_KNOW_HOW = rb.getString("i_dont_know_how");
	I_DONT_KNOW_WHAT = rb.getString("i_dont_know_what");
	NOT_STRING = rb.getString("not_string");
	NOT_OBJECT = rb.getString("not_object");
	NOT_LIST = rb.getString("not_list");
	NOT_SYMBOL = rb.getString("not_symbol");
	NOT_NUMBER = rb.getString("not_number");
	NOT_INTEGER = rb.getString("not_integer");
	NOT_ILIST = rb.getString("not_ilist");
	NOT_PROCEDURE = rb.getString("not_procedure");
	NOT_WEAK = rb.getString("not_weak");
	NOT_LOCALE = rb.getString("not_locale");
	NOT_RESOURCE_BUNDLE = rb.getString("not_resource_bundle");
	NOT_LIST_OR_STRING = rb.getString("not_list_or_string");
	STOPPED = rb.getString("stopped");
	IN = rb.getString("in");
	ANDOR = rb.getString("and_or");
	CANTTAKE = rb.getString("cant_take");
	BF = rb.getString("bf");
	BL = rb.getString("bl");
	OFEMPTYSTRING = rb.getString("of_empty_string");
	FIRST = rb.getString("first");
	LAST = rb.getString("last");
	CANTPICK = rb.getString("cant_pick");
	FROMEMPTYLIST = rb.getString("from_empty_list");
	FROMEMPTYSTRING = rb.getString("from_empty_string");
	INDEX = rb.getString("index");
	INDEXTOOLOW = rb.getString("is_too_low");
	INDEXTOOHIGH = rb.getString("is_too_high");
	STARTINGINDEX = rb.getString("starting_index");
	ENDINGINDEX = rb.getString("ending_index");
	ISGREATERTHAN = rb.getString("is_greater_than");
	CANTCOMPUTE = rb.getString("cant_compute");
	OFEMPTYLIST = rb.getString("of_empty_list");
	OFLIST = rb.getString("of_list");
	HASNONUMBERS = rb.getString("has_no_numbers");
	MEDIAN = rb.getString("median");
	AVERAGE = rb.getString("average");
	VARIANCE = rb.getString("variance");
	SD = rb.getString("sd");
	MODE = rb.getString("mode");	
	SUM = rb.getString("sum");	
	ELEMENT = rb.getString("element");
	CHARACTER = rb.getString("character");
    }

    public LogoError() {
	super();
    }
	
    public LogoError(String s) {
	super(s);
    }

    public static String errorToString(Context c) {
	StringBuffer output = new StringBuffer();
	//output.append(getLocalizedMessage());
	if (c.currentfun != null) {
	    output.append(' '); output.append(IN); output.append(' ');
	    output.append(c.currentfun.toString());
	}
	if (c.ufun != null) {
	    output.append(' '); output.append(IN); output.append(' ');
	    output.append(c.ufun.toString());
	}	 
	return output.toString();
    }			  

    public static void error(String s, Context c) {
	if (c == null) throw new LogoError(s);
	throw new LogoError(s + errorToString(c));
    }

    public static void error(String s) {
	throw new LogoError(s);
    }
	
    public String toString() {
	String foo = getMessage();
	if (foo == null) return "LogoError";
	else return foo;
    }
}
