/********************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *******************************************************************/
 package yoyo;

import java.util.Stack;

public class Pool implements Runnable {

  static Object undefined = new Object();

  static Object[][][] pool;
  static int[] ptrs;

  static {
      pool = new Object[8][10][];
    ptrs = new int[8];
    for(int i = 0; i<8; i++) {
      ptrs[i] = 0;
    }
    new Thread(new Pool()).start();
  }

  public static synchronized Object[] getArray(int size, Context c) {
    //System.out.println("get Array size: " + size);
    if (size >= 8) return new Object[size];
    //System.out.println("thepool: " + YoYo.printToString(pool[size]));
    //System.out.println("ptrs[size]: " + ptrs[size]);
    Object[][] thepool = pool[size];
    if (ptrs[size] == 0) {
      Object[] output = new Object[size];
      for(int i = 0; i < size; i++) {
	output[i] = undefined;
      }
      return output;
    }
    return thepool[--ptrs[size]];
  }

  public static synchronized void dropArray(Object[] array, Context c) {
    int size = array.length;
    if (size >= 8) return;
    for(int i = 0; i<size; i++) {
      array[i] = undefined;
    }
   
    Object[][] thepool = pool[size];
    
    if (ptrs[size] >= thepool.length) {
      Object[][] newarray = new Object[thepool.length * 2][];
      System.arraycopy(thepool, 0, newarray, 0, thepool.length);
      pool[size] = newarray;
      thepool = newarray;
    }
    thepool[ptrs[size]++] = array;
  }

  // Cleanup thread. Runs every 5 seconds or so to clean out the pools.
  public void run() {
    Thread thread = Thread.currentThread();
    while (true) {
      try {
	thread.sleep(5000);
      }
      catch (InterruptedException e) {}
      cleanPool();
    }
  }

  public static synchronized void cleanPool() {
    for(int i = 0; i < 8; i++) {
      Object[][] thepool = pool[i];
      int numentries = ptrs[i];
      if (numentries > 10) {
	ptrs[i] = 10;
	for(int j = 10; j < numentries; j++) {
	  thepool[j] = null;
	}
      }
    }
  }
}
