/********************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *******************************************************************/
 package yoyo;

import java.util.ResourceBundle;

public class PrimControl implements YoYoMethods {

  public String[] methodList() {
      ResourceBundle rb = PrimStartup.getYoYoCommandsBundle();
    String[] output = {
	rb.getString("primcontrol_repeat"), "prim_repeat",
        rb.getString("primcontrol_stop"), "prim_stop",
        rb.getString("primcontrol_output"), "prim_output",
        rb.getString("primcontrol_if"), "prim_if",
        rb.getString("primcontrol_ifelse"), "prim_ifelse",
        rb.getString("primcontrol_while"), "prim_while",
        rb.getString("primcontrol_loop"), "prim_loop",
        rb.getString("primcontrol_carefully"), "prim_carefully",
        rb.getString("primcontrol_always"), "prim_always",
        rb.getString("primcontrol_run"), "prim_run",
        rb.getString("primcontrol_ask"), "prim_ask",
        rb.getString("primcontrol_dotimes"), "prim_dotimes",
        rb.getString("primcontrol_launch"), "prim_launch",
        rb.getString("primcontrol_thread_self"), "prim_thread_self",
        rb.getString("primcontrol_throw"), "prim_throw",
        rb.getString("primcontrol_error"), "prim_error",
        rb.getString("primcontrol_error_type"), "prim_errortype",
        rb.getString("primcontrol_system_yield"), "prim_yield",
        rb.getString("primcontrol_wait"), "prim_wait",
        rb.getString("primcontrol_case"), "prim_case",
      };
    return output;
  }

  public String[] infixList() {
    return null;
  }

    public void prePrimCall(Primitive p, Object[] arglist, Context c) {}
    public void postPrimCall(Primitive p, Context c) {}

  public static void prim_stop(Context c)  {
    //System.out.println("stop");
    c.ufunreturn = Context.novalue;
    c.ilistSkipToEnd();
    //throw new UfunReturn();
  }

  public static void prim_output(Object v, Context c)  {
    //System.out.println("output");
    c.ufunreturn = v;
    c.ilistSkipToEnd();
    //throw new UfunReturn(v);
  }

  public static Object prim_error(Context c)  {
    if (c.errorobj != null) {
      if (c.errorobj instanceof ThrowReturn) {
        return ((ThrowReturn)c.errorobj).value;
      }
      else return c.errorobj.toString();
    }
    else return YoYo.symfalse;
  }

  public static Object prim_errortype(Context c)  {
    if (c.errorobj != null) {
      if (c.errorobj instanceof ThrowReturn) {
        return ((ThrowReturn)c.errorobj).errortype;
      }
      else return symerror;
    }
    else return YoYo.symfalse;
  }

  public static Symbol symerror = Symbol.lookup(PrimStartup.getYoYoCommandsBundle().getString("primcontrol_system_error"));

  public static void prim_throw(Object v1, Object v2, Context c)  {
    Symbol s = YoYo.aSymbol(c, v1);
    if (s == symerror) {
      LogoError.error(YoYo.aString(c, v2), c);
    }
    else throw new ThrowReturn(s, v2);
  }

  public static void prim_repeat(Object v0, Object v1, Context c) {

    if (v1 instanceof Object[]) {
      int n = ((Number)v0).intValue();
      if (n > 0) {
        Object[] listtorun = YoYo.aList(c, v1);
        for(int i = 0; i<n; i++) {
          YoYo.evalList(listtorun, c);
          if (c.ufunreturn != null) return;
        }
      }
    }
  }

  public static Object prim_if(Object v0, Object v1, Context c)  {
    Object[] list = YoYo.aList(c, v1);
    if (YoYo.aBoolean(c, v0)) {
      Object value = YoYo.evalList(list, c);
      return value;
    }
    return null;
  }

  public static Object prim_ifelse(Object v0, Object v1, Object v2, Context c)  {
    int oldnargs = c.nargs;
    Object[] cons = YoYo.aList(c, v1);
    Object[] alt = YoYo.aList(c, v2);
    try {
      c.nargs = 1;
      if (YoYo.aBoolean(c, v0)) return YoYo.evalList(cons,c);
      else return YoYo.evalList(alt, c);
    }
    finally {
      c.nargs = oldnargs;
    }
  }

  public static Object prim_while(Object v0, Object v1, Context c)  {

    Object[] test = YoYo.aList(c, v0);
    Object[] torun = YoYo.aList(c, v1);
    int oldnargs = c.nargs;

    try {
      while(true) {
        c.nargs = 1;
        Object pred = YoYo.evalList(test, c);
        if (c.ufunreturn != null) return null;
        c.nargs = 0;
        if (!YoYo.aBoolean(c, pred)) return null;
        YoYo.evalList(torun, c);
        if (c.ufunreturn != null) return null;
      }
    }
    finally { c.nargs = oldnargs; }
  }

  public static Object prim_loop(Object v0, Context c)  {

    Object[] list = YoYo.aList(c, v0);
    while(true) {
      YoYo.evalList(list, c);
      if (c.ufunreturn != null) return null;
    }
  }

  public static Object prim_carefully(Object v0, Object v1, Context c)  {

    Object[] run = YoYo.aList(c, v0);
    Object[] exc = YoYo.aList(c, v1);
    LogoError olderrorobj = c.errorobj;
    try {
      return YoYo.evalList(run, c);
    }
    catch (LogoError e) {
      c.errorobj = e;
      Object val = c.ufunreturn;
      c.ufunreturn = null;
      try {
        return YoYo.evalList(exc, c);
      }
      finally {
        if (c.ufunreturn != null) return null;
        c.ufunreturn = val;
        if (val != null) return null;
      }
    }
    finally {
      c.errorobj = olderrorobj;
    }
  }

  public static Object prim_always(Object v0, Object v1, Context c)  {
    Object[] run = YoYo.aList(c, v0);
    Object[] exc = YoYo.aList(c, v1);
    try {
      return YoYo.evalList(run, c);
    }
    finally {
      Object val = c.ufunreturn;
      c.ufunreturn = null;
      try {
        YoYo.evalList(exc, c);
      }
      finally {
        if (c.ufunreturn != null) return null;
        c.ufunreturn = val;
        if (val != null) return null;
      }
    }
  }

  public static Object prim_run(Object v0, Context c)  {
    Object[] l = YoYo.aList(c, v0);
    int oldnargs = c.nargs; c.nargs = 1; Object value = null;
    try { value = YoYo.evalList(l, c); }
    finally { c.nargs = oldnargs; }
    return value;
  }

  public static Object prim_ask(Object v0, Object v1, Context c)  {
    Object[] l = YoYo.aList(c, v1);
    YoYoObject oldYoYobj = c.objenv;
    c.objenv = YoYo.aYoYoObject(c, v0);
    int oldnargs = c.nargs; c.nargs = 1; Object value = null;
    try { value = YoYo.evalList(l, c); }
    finally { c.nargs = oldnargs; c.objenv = oldYoYobj;}
    return value;
  }

  public static Object prim_launch(Object v0, Context c)  {
    Object[] i = YoYo.aList(c, v0);
    Context newc = new Context(c);
    new Thread(newc).start();
    newc.runList(i);
    return newc.self;
  }

  public static Object prim_thread_self(Context c) {
    return c.self;
  }

  public static Object prim_dotimes(Object v0, Object v1, Context c)  {

    Object[] deflist = YoYo.aList(c, v0);
    Object[] torun = YoYo.aList(c, v1);

    Object oldvalue = null;
    LocalRef l;
    int value;
    Symbol name;

    Object[] oldilist = c.ilist;
    int oldilistposptr = c.ilistposptr;

    try {
      c.ilist = deflist;
      c.ilistposptr = 0;

      Object var = c.ilistNext();
      if (var instanceof LocalRef) {
        l = (LocalRef)var;
        oldvalue = c.locals.lookupValue(l);
      }
      else {
        name = YoYo.aSymbol(c, var);
        if (c.locals.boundp(name)) {
          int index = c.locals.index(name);
          l = new LocalRef(index, name);
          c.setPrev(l);
          oldvalue = c.locals.lookupValue(l);
        }
        else {
          int index = c.locals.defineVar(name);
          l = new LocalRef(index, name);
          c.setPrev(l);
          oldvalue = null;
        }
      }
      if (c.ufunobj != null) {
        if (c.locals.names.length > c.ufunobj.arglist.length)
          c.ufunobj.arglist = c.locals.names;
      }
      value = YoYo.aInteger(c, YoYo.evalOneFromHere(c));
    }
    finally {
      c.ilist = oldilist;
      c.ilistposptr = oldilistposptr;
    }

    try {
      int current = 1;
      while (true) {
        if (current > value) break;
        c.locals.setValue(l, new Integer(current++));
        YoYo.evalList(torun, c);
        if (c.ufunreturn != null) return null;
      }
      return null;
    }
    catch (Stop e) {
      //System.out.println("time to stop: dotimes");
      throw e;
    }
    finally {
      if (oldvalue!=null) c.locals.setValue(l, oldvalue);
      else c.locals.undefineVar(l);
    }
  }

  public static void prim_yield(Context c)  {
    Thread.currentThread().yield();
  }


  public static void prim_wait(Object v0, Context c)  {
    int time = (int)(1000*YoYo.aDouble(c, v0));
    try {
      while(time >= 0) {
        YoYo.pollStop(c);
        Thread.currentThread().sleep(10);
        time -= 10;
      }
    }
    catch (InterruptedException e) {}
  }

  public static Object prim_case(Object v0, Object v1, Context c) {

    Object[] list = YoYo.aList(c, v1);
    Object[] oldilist = c.ilist;
    int oldilistposptr = c.ilistposptr;

    try {
      c.ilist = list;
      c.ilistposptr = 0;

      while(c.ilistPeek() != null) {
        Object test = YoYo.evalOneFromHere(c);
        if (c.ufunreturn != null) return null;
        if (c.ilistPeek() == null)
          LogoError.error(LogoError.TOO_FEW_REPORTERS + PrimStartup.getYoYoCommandsBundle().getString("primitives_case"), c);
        Object thing = c.ilistNext();
        if (YoYo.equal(v0, test) == YoYo.symtrue || test == YoYo.symtrue) {
          return YoYo.evalList(YoYo.aList(c, thing), c);
        }
      }
    }
    finally {
      c.ilist = oldilist;
      c.ilistposptr = oldilistposptr;
      if (c.ufunreturn != null) c.ilistSkipToEnd();
    }
    return null;
  }
}
