/********************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *******************************************************************/
 package yoyo;

import java.io.*;
import java.net.*;
import java.text.SimpleDateFormat;
import java.util.*;
import java.nio.charset.Charset;

public class PrimFile implements YoYoMethods {
	
  static SimpleDateFormat sdf = new SimpleDateFormat("yyyy/MM/dd HH:mm");
	
  public String[] methodList() {
      ResourceBundle rb = PrimStartup.getYoYoCommandsBundle();
    String[] output = {
	rb.getString("primfile_load_yoyo"), "prim_load",
	rb.getString("primfile_load_yoyo_verbose"), "prim_load_verbose",
	rb.getString("primfile_load_yoyo_from_string"), "prim_load_from_string",
	rb.getString("primfile_load_yoyo_from_string_verbose"), "prim_load_from_string_verbose",
	rb.getString("primfile_launch_process"), "prim_launch_process",
	rb.getString("primfile_read_file_to_string"), "prim_filetostring",
	rb.getString("primfile_read_file_to_string_with_encoding"), "prim_filewithencodingtostring",
	rb.getString("primfile_write_string_to_file"), "prim_stringtofile",
	rb.getString("primfile_read_url_to_string"), "prim_urltostring",
	rb.getString("primfile_read_url_to_string_with_encoding"), "prim_urlwithencodingtostring",
	rb.getString("primfile_home_directory"), "prim_home_dir",
	rb.getString("primfile_list_files"), "prim_ls",
	rb.getString("primfile_filep"), "prim_filep",
	rb.getString("primfile_directoryp"), "prim_dirp",
	rb.getString("primfile_file_date"), "prim_filedate",
	rb.getString("primfile_file_readablep"), "prim_filereadp",
	rb.getString("primfile_file_writeablep"), "prim_filewritep",
	rb.getString("primfile_file_rename"), "prim_rename",
	rb.getString("primfile_make_directory"), "prim_mkdir",
	rb.getString("primfile_urlp"), "prim_urlp",
	rb.getString("primfile_file_canonical_path"), "prim_canonpath",
	rb.getString("primfile_make_resource_url"), "prim_makeresourceurl",
	rb.getString("primfile_make_locale"), "prim_make_locale",
	rb.getString("primfile_get_resource_bundle"), "prim_get_resource_bundle",
	rb.getString("primfile_get_bundle_property"), "prim_get_bundle_property",
    };
    return output;
  }
	
  public String[] infixList() {
    return null;
  }

    public void prePrimCall(Primitive p, Object[] arglist, Context c) {}
    public void postPrimCall(Primitive p, Context c) {}
	
  public static Object prim_canonpath(Object v0, Context c) {
    try {
      URL u = new URL(YoYo.aString(c, v0));
      if (u.getProtocol().equals("file")) {
	String filename = u.getFile();
	File f = new File(filename);
	if (!f.exists()) LogoError.error(YoYo.printToString(v0) + " does not exist", c);
	try {
	  return new URL("file", "/" + f.getCanonicalPath(), "");
	}
	catch (IOException e) {
	  LogoError.error("Could not determine canonical path for " + 
			  YoYo.printToString(v0), c);
	}
      }
      else {
	LogoError.error(YoYo.printToString(v0) + " must be a file URL", c);
      }
    }
    catch (MalformedURLException e) { 
      LogoError.error(YoYo.printToString(v0) + " is a bad URL", c);
    }
    return null;
  }
	
  public static Object prim_urlp(Object v0, Context c) {
    try {
      URL u = new URL(YoYo.aString(c, v0));
      return YoYo.symtrue;
    }
    catch (MalformedURLException e) { return YoYo.symfalse; }
  }
	
  public static Object prim_rename(Object v0, Object v1, Context c) {
    try {
      URL u1 = new URL(YoYo.aString(c, v0));
      URL u2 = new URL(YoYo.aString(c, v1));
      if (u1.getProtocol().equals("file") &&
	  u2.getProtocol().equals("file")) {
	String filename1 = u1.getFile();
	String filename2 = u1.getFile();
	File f1 = new File(filename1);
	File f2 = new File(filename2);
	if (!f1.exists()) LogoError.error(YoYo.printToString(v0) + " does not exist", c);
	boolean successp = f1.renameTo(f2);
	if (!successp) LogoError.error("could not rename file " + 
				       YoYo.printToString(v0), c);
	return null;
      }
      else {
	LogoError.error(YoYo.printToString(v0) + " and " + YoYo.printToString(v1) + 
			" must both be file URLs", c);
      }
    }
    catch (MalformedURLException e) { 
      LogoError.error("Either " + YoYo.printToString(v0) + " or " + 
		      YoYo.printToString(v1) + " is a bad URL", c);
    }
    return null;
  }
	
  public static Object prim_mkdir(Object v0, Context c) {
    try {
      URL u = new URL(YoYo.aString(c, v0));
      if (u.getProtocol().equals("file")) {
	String filename = u.getFile();
	File f = new File(filename);
	if (f.exists()) LogoError.error(YoYo.printToString(v0) + " already exists", c);
	boolean successp = f.mkdirs();
	if (!successp) LogoError.error("could not create directory " + 
				       YoYo.printToString(v0), c);
	return null;
      }
      else {
	LogoError.error(YoYo.printToString(v0) + " must be a file URL", c);
      }
    }
    catch (MalformedURLException e) { 
      LogoError.error(YoYo.printToString(v0) + " is a bad URL", c);
    }
    return null;
  }
	
  public static Object prim_file_writep(Object v0, Context c) {
    try {
      URL u = new URL(YoYo.aString(c, v0));
      if (u.getProtocol().equals("file")) {
	String filename = u.getFile();
	File f = new File(filename);
	return (f.canWrite()) ? YoYo.symtrue : YoYo.symfalse;
      }
      else {
	LogoError.error(YoYo.printToString(v0) + " must be a file URL", c);
      }
    }
    catch (MalformedURLException e) { 
      LogoError.error(YoYo.printToString(v0) + " is a bad URL", c);
    }
    return null;
  }
	
  public static Object prim_file_readp(Object v0, Context c) {
    try {
      URL u = new URL(YoYo.aString(c, v0));
      if (u.getProtocol().equals("file")) {
	String filename = u.getFile();
	File f = new File(filename);
	return (f.canRead()) ? YoYo.symtrue : YoYo.symfalse;
      }
      else {
	LogoError.error(YoYo.printToString(v0) + " must be a file URL", c);
      }
    }
    catch (MalformedURLException e) { 
      LogoError.error(YoYo.printToString(v0) + " is a bad URL", c);
    }
    return null;
  }

    public static void prim_launch_process(Object v0, Context c) {
	if (YoYo.applet != null) return;
	Object[] list = YoYo.aList(c, v0);
	String[] cmds = new String[list.length];
	for(int i = 0; i < cmds.length; i++) {
	    cmds[i] = YoYo.printToString(list[i]);
	}
	try {
	    Runtime.getRuntime().exec(cmds);
	} catch (IOException e) { e.printStackTrace(); 
	LogoError.error("Couldn't launch the process: " + YoYo.printToString(v0), c);
	}
    }
	
  public static Object prim_filedate(Object v0, Context c) {
    try {
      URL u = new URL(YoYo.aString(c, v0));
      if (u.getProtocol().equals("file")) {
	String filename = u.getFile();
	File f = new File(filename);
	return (sdf.format(new Date(f.lastModified())));
      }
      else {
	LogoError.error(YoYo.printToString(v0) + " must be a file URL", c);
      }
    }
    catch (MalformedURLException e) { 
      LogoError.error(YoYo.printToString(v0) + " is a bad URL", c);
    }
    return null;
  }
	
	
  public static Object prim_dirp(Object v0, Context c) {
    try {
      URL u = new URL(YoYo.aString(c, v0));
      if (u.getProtocol().equals("file")) {
	String filename = u.getFile();
	File f = new File(filename);
	return (f.isDirectory()) ? YoYo.symtrue : YoYo.symfalse;
      }
      else {
	LogoError.error(YoYo.printToString(v0) + " must be a file URL", c);
      }
    }
    catch (MalformedURLException e) { 
      e.printStackTrace();
      LogoError.error(YoYo.printToString(v0) + " is a bad URL", c);
    }
    return null;
  }
	
  public static Object prim_filep(Object v0, Context c) {
    try {
      URL u = new URL(YoYo.aString(c, v0));
      if (u.getProtocol().equals("file")) {
	String filename = u.getFile();
	File f = new File(filename);
	return (f.isFile()) ? YoYo.symtrue : YoYo.symfalse;
      }
      else {
	LogoError.error(YoYo.printToString(v0) + " must be a file URL", c);
      }
    }
    catch (MalformedURLException e) { 
      LogoError.error(YoYo.printToString(v0) + " is a bad URL", c);
    }
    return null;
  }
	
	
  public static Object prim_ls(Object v0, Context c) {
    try {
	URL u = new URL(YoYo.aString(c, v0));
	if (u.getProtocol().equals("file")) {
	    String filename = u.getFile();
	    File f = new File(filename);
	    if (f.isDirectory()) {
		String[] files = f.list();
		//Object[] out = new Object[files.length];
		//for(int i = 0; i < files.length; i++) {
		//  out[i] = files[i];  //new URL("file", "/" + f.getPath(), "/" + files[i]);
		//}
		return files;
	    }
	    else {
		LogoError.error(YoYo.printToString(v0) + " must be a directory", c);
	    }
	}
	else {
	    LogoError.error(YoYo.printToString(v0) + " must be a file URL", c);
	}
    }
    catch (MalformedURLException e) { 
	e.printStackTrace();
	LogoError.error(YoYo.printToString(v0) + " is a bad URL", c);
    }
    return null;
  }
	
  public static Object prim_home_dir(Context c) {
    return c.base.toString();
  }
	
  public static Object prim_stringtofile(Object v0, Object v1, Context c) {
    try {
      if (YoYo.applet != null) {
	LogoError.error("Sorry, can't write to a file from an applet", c);
      }
      String filecontents = YoYo.aString(c, v0);
      String filename = YoYo.aString(c, v1);
      File f = new File(filename);
      if (!f.canWrite()) {
	LogoError.error("Sorry, you can't write to this file", c);
      }
      PrintWriter pw = new PrintWriter(new BufferedWriter(new FileWriter(f)));
      pw.print(filecontents);
      pw.flush();
      pw.close();
    }
    catch (IOException e) {
      LogoError.error("Problem writing file", c);
    }
    return null;
		
  }
	
  public static Object prim_filetostring(Object v0, Context c) {
    if (YoYo.applet == null) {
      String filename = YoYo.aString(c, v0);
			
      byte[] buffer=null; String content=null;
      try {
	//System.out.println("prim file made new file from " + filename);
	File f = new File(filename);
	//System.out.println("new file is " + f.toString());
	//try { 
	    //System.out.println("canonical name is " + f.getCanonicalPath());
	    //f = f.getCanonicalFile();
	    //System.out.println("absolute name is " + f.getAbsolutePath());
	//}	
	//catch (IOException e) { e.printStackTrace(); }
	if (!f.canRead()) LogoError.error(YoYo.printToString(v0) + " is not readable");
	FileReader fir = new FileReader(f);
	return readStream(fir, (int)f.length(), c);
      }
      catch (FileNotFoundException e) {
	LogoError.error(e.getMessage(), c);
      }
      return null;
    }
    else {
      String filename = "./" + YoYo.aString(c, v0);
      try {
	return prim_urltostring(new URL(c.base, filename), c);
      }
      catch (MalformedURLException e) {LogoError.error("url does not exist", c);}
      return null;
    }
  }

  public static Object prim_filewithencodingtostring(Object v0, Object v1, Context c) {
    if (YoYo.applet == null) {
      String filename = YoYo.aString(c, v0);
      String encoding = YoYo.aString(c, v1);
			
      byte[] buffer=null; String content=null;
      try {
	  //System.out.println("prim file made new file from " + filename);
	  File f = new File(filename);
	  //System.out.println("new file is " + f.toString());
	  //try { 
	  //System.out.println("canonical name is " + f.getCanonicalPath());
	  //f = f.getCanonicalFile();
	  //System.out.println("absolute name is " + f.getAbsolutePath());
	  //}	
	  //catch (IOException e) { e.printStackTrace(); }
	  if (!f.canRead()) LogoError.error(YoYo.printToString(v0) + " is not readable");
	  //System.out.println("read from file " + f + " in encoding " + encoding);
	  //Charset cs = Charset.forName(encoding);
	  //System.out.println("encoding is " + cs + " display " + cs.displayName() + 
	  //		     " name " + cs.name());
	  java.io.Reader fir = new InputStreamReader(new FileInputStream(f), 
						     Charset.forName(encoding));
	  return readStream(fir, (int)f.length(), c);
      }
      catch (FileNotFoundException e) {
	  LogoError.error(e.getMessage(), c);
      }
      return null;
    }
    else {
      String filename = "./" + YoYo.aString(c, v0);
      try {
	return prim_urltostring(new URL(c.base, filename), c);
      }
      catch (MalformedURLException e) {LogoError.error("url does not exist", c);}
      return null;
    }
  }

  public static Object prim_filetobytes(Object v0, Context c) {
    if (YoYo.applet == null) {
      String filename = YoYo.aString(c, v0);
			
      byte[] buffer=null; String content=null;
      try {
	//System.out.println("prim file made new file from " + filename);
	File f = new File(filename);
	//System.out.println("new file is " + f.toString());
	//try { 
	    //System.out.println("canonical name is " + f.getCanonicalPath());
	    //f = f.getCanonicalFile();
	    //System.out.println("absolute name is " + f.getAbsolutePath());
	//}	
	//catch (IOException e) { e.printStackTrace(); }
	if (!f.canRead()) LogoError.error(YoYo.printToString(v0) + " is not readable");
	FileInputStream fir = new FileInputStream(f);
	return readStream(fir, (int)f.length(), c);
      }
      catch (FileNotFoundException e) {
	LogoError.error(e.getMessage(), c);
      }
      return null;
    }
    else {
      String filename = "./" + YoYo.aString(c, v0);
      try {
	return prim_urltobytes(new URL(c.base, filename), c);
      }
      catch (MalformedURLException e) {LogoError.error("url does not exist", c);}
      return null;
    }
  }
	
  public static Object prim_makeresourceurl(Object v0, Object v1, Context c) {
    try {
      URL u = Class.forName(YoYo.aString(c, v0)).getResource(YoYo.aString(c, v1));
      if (u == null) {
	LogoError.error("File " + v0 + "." + v1 + " could not be found.");
      }
      return u;
    } 
    catch (ClassNotFoundException e) {
      LogoError.error("Class " + v0 + " cannot be found. Check your classpath."); 
    }
    return null;
  }
	
  public static Object prim_urltostring(Object v0, Context c) {
//System.out.println("top of prim_urltostring. v0 = " + v0);
    try {
	//System.out.println("url to string: " + YoYo.printToString(v0));
      URL u = new URL(YoYo.aString(c, v0));
      byte[] buffer=null; String content=null;
			
      try { 
	InputStream is = null;
	try {
//System.out.println("before openstream. u = " + u);
	  is = u.openStream();
//System.out.println("after openstream");
	  return readStream(new InputStreamReader(is), -1, c);
	}
	catch (Exception e) {
	    e.printStackTrace();
	  LogoError.error(e.getMessage(), c);
	}
	finally { if (is != null) is.close(); }
      }
      catch (IOException f) {LogoError.error("error reading url: " + u.toString(), c);}
    }
    catch (MalformedURLException e) {LogoError.error("url does not exist: " + YoYo.printToString(v0), c);}
    return null;
  }

  public static Object prim_urlwithencodingtostring(Object v0, Object v1, Context c) {
//System.out.println("top of prim_urltostring. v0 = " + v0);
    try {
	//System.out.println("url to string: " + YoYo.printToString(v0));
      URL u = new URL(YoYo.aString(c, v0));
      String encoding = YoYo.aString(c, v1);
      byte[] buffer=null; String content=null;
			
      try { 
	InputStream is = null;
	try {
//System.out.println("before openstream. u = " + u);
	  is = u.openStream();
//System.out.println("after openstream");
	  return readStream(new InputStreamReader(is, Charset.forName(encoding)), 
			    -1, c);
	}
	catch (Exception e) {
	    e.printStackTrace();
	  LogoError.error(e.getMessage(), c);
	}
	finally { if (is != null) is.close(); }
      }
      catch (IOException f) {LogoError.error("error reading url: " + u.toString(), c);}
    }
    catch (MalformedURLException e) {LogoError.error("url does not exist: " + YoYo.printToString(v0), c);}
    return null;
  }


  public static Object prim_urltobytes(Object v0, Context c) {
//System.out.println("top of prim_urltobytes. v0 = " + v0);
    try {
	//System.out.println("url to bytes: " + YoYo.printToString(v0));
      URL u = new URL(YoYo.aString(c, v0));
      byte[] buffer=null; String content=null;
			
      try { 
	InputStream is = null;
	try {
//System.out.println("before openstream. u = " + u);
	  is = u.openStream();
//System.out.println("after openstream");
	  return readStream(is, -1, c);
	}
	catch (Exception e) {
	    e.printStackTrace();
	  LogoError.error(e.getMessage(), c);
	}
	finally { if (is != null) is.close(); }
      }
      catch (IOException f) {LogoError.error("error reading url: " + u.toString(), c);}
    }
    catch (MalformedURLException e) {LogoError.error("url does not exist: " + YoYo.printToString(v0), c);}
    return null;
  }
	
  public static String readStream(java.io.Reader ir, int length, Context c) {
    BufferedReader in = null;
    if (length < 0) length = 4096;
    try {
      in = new BufferedReader(ir, length);
      StringBuffer buffer = new StringBuffer(length);
      char[] chars = new char[length];
      do {
	int result = 0;
	do {
	  result = in.read(chars, 0, length);
	} while (result == 0);
	if (result < 0) break;
	buffer.append(chars, 0, result);
      } while(true);
      return buffer.toString();
    }
    catch (IOException e) { 
      LogoError.error(e.getMessage(), c);
    }
    finally { if (in != null) try { in.close(); } catch (IOException f) {} }
    return null;
  }

  public static byte[] readStream(InputStream ir, int length, Context c) {
    BufferedInputStream in = null;
    if (length < 0) length = 4096;
    try {
      in = new BufferedInputStream(ir, length);
      byte[] file = new byte[length];
      int filepos = 0;
      byte[] buffer = new byte[length];
      do {
	int result = 0;
	do {
	  result = in.read(buffer, 0, length);
	} while (result == 0);
	if (result < 0) break;
	System.arraycopy(buffer, 0, file, filepos, result);
	filepos += result;
	if (filepos > file.length) {
	    byte[] temp = new byte[file.length * 2];
	    System.arraycopy(file, 0, temp, 0, file.length);
	    file = temp;
	}
      } while(true);
      byte[] toreturn = new byte[filepos];
      System.arraycopy(file, 0, toreturn, 0, filepos);
      return file;
    }
    catch (IOException e) { 
      LogoError.error(e.getMessage(), c);
    }
    finally { if (in != null) try { in.close(); } catch (IOException f) {} }
    return null;
  }
	
    public static Object prim_make_locale(Object v1, Object v2, Context c) {
	return new Locale(YoYo.aString(c, v1), YoYo.aString(c, v2));
    }

    public static Object prim_get_resource_bundle(Object v1, Object v2, Context c) {
	try {
	    ResourceBundle rb = ResourceBundle.getBundle(YoYo.aString(c, v1), YoYo.aLocale(c, v2));
	    return rb;
	} catch (MissingResourceException e) { return YoYo.symfalse; }
    }

    public static Object prim_get_bundle_property(Object v1, Object v2, Context c) {
	try {
	    return YoYo.aResourceBundle(c, v1).getObject(YoYo.aString(c, v2));
	} catch (MissingResourceException e) { return YoYo.symfalse; }
    }

  public static void prim_load_verbose(Object v, Context c)  {
    load(v, true, c);
  }
	
  public static void prim_load(Object v, Context c) {
    load(v, false, c);
  }
	
  static void load(Object v, boolean verbose, Context c) {
    String s = YoYo.aString(c, v);
    //System.out.println("loading in load" + s);
    //try { throw new Exception(); } catch (Exception e) { e.printStackTrace(); }
    if (!s.endsWith(".yoyo")) s += ".yoyo";
    URL u = null;
    try { 
      u = new URL(s);
      InputStream is = null;
      try {
	is = u.openStream();
	InputStreamReader in = new InputStreamReader(is);
	readprocs(in, verbose, c);
      }
      catch (Exception e) {
	  c.err.println(e);
	  e.printStackTrace(c.err);
	  if (c.log != c.err) {
	      c.log.println(e);
	      e.printStackTrace(c.log);
	  }
	  LogoError.error(e.getMessage(), c);
      }
      finally { if (is != null) is.close(); }
    }
    catch (MalformedURLException e) {LogoError.error("file does not exist", c);}
    catch (FileNotFoundException e) {LogoError.error("file does not exist", c);}
    catch (IOException f) {LogoError.error("error reading file", c);}
  }
	
  public static void prim_load__from_string_verbose(Object v, Context c)  {
    loadFromString(YoYo.aString(c, v), true, c);
  }
	
  public static void prim_load_from_string(Object v, Context c) {
    loadFromString(YoYo.aString(c, v), false, c);
  }
	
    public static void loadFromString(String s, boolean verbose, Context c) {
    StringReader in = new StringReader(s);
    try {
      readprocs(in, verbose, c);
    }
    catch (Exception e) {
	c.err.println(e);
	e.printStackTrace(c.err);
	if (c.log != c.err) {
	    c.log.println(e);
	    e.printStackTrace(c.log);
	}
	LogoError.error(e.getMessage(), c);
    }
    finally { in.close(); }
  }
	
	
  static void readprocs(java.io.Reader is, boolean verbose, Context c) {
    String name;
    Symbol[] args;
		
    BufferedReader in = new BufferedReader(is);
		
    YoYoObject oldufunenv = c.ufunenv;
    Object[] oldilist = c.ilist;
    int oldilistposptr = c.ilistposptr;
		
    try {
      c.ufunenv = c.getGlobalObj();
      while(true) {  
				//System.out.println("reading title");
	String title=readtitle(c, in);
				//System.out.println("title = " + title);
	if (title==null) return;
	title=title.substring(3).trim();  // to blah
	int space=title.indexOf(" ");
	if (space==-1) { // no inputs
	  name=title; 
	  args = new Symbol[0];
	}
	else {
	  name=title.substring(0,space);
	  Object[] argsarray = Reader.read(title.substring(space+1));
	  args = new Symbol[argsarray.length];
	  System.arraycopy(argsarray, 0, args, 0, argsarray.length);
	}
				//System.out.println("procname = " + name);
				//System.out.println("args = " + YoYo.printToString(args));
	String bodyString = readbody(in);
	Object[] body = Reader.read(bodyString);
	Symbol[] locals = readlocals(args, new Symbol[0], body, 0);
				//System.out.println("body = " + YoYo.printToString(body));
				//System.out.println("making ufun");
	Symbol symname = Symbol.lookup(name);
	c.ufunenv.defineValue(symname, 
			      new Ufun(symname, args, locals, 
				       body, bodyString, c.ufunenv));
				//System.out.println("made ufun");
	if (verbose) c.out.println(name + " defined as " + YoYo.printToString(args) + " body: " + YoYo.printToString(body));
      }
    }
    catch (IOException e) { } //System.out.println("ioexception on load");}
    finally {
      c.ufunenv = oldufunenv;
      c.ilist = oldilist;
      c.ilistposptr = oldilistposptr;
    }
    //System.out.println("load finished");
  }
	
  static String readtitle(Context c, BufferedReader in) throws IOException {
    String line;
    do {
      line=readaline(in);
      //System.out.println("readtitle: " + line);
      if (line==null) return null;
      if (line.startsWith("to ",0)) return line;
      if (line.startsWith("create-object ",0)) {
	createObject(line.substring(14).trim(), c);
      } else
      if (line.startsWith("in-object ",0)) {
	switchObject(line.substring(10).trim(), c);
      } else
      if (line.startsWith("instance-var ", 0)) {
	createInstanceVar(line.substring(13).trim(), c);
      }
    } while(true);
  }
	
	
  static void createObject(String line, Context c) {
    //System.out.println("create-object " + line);
    Object[] input = Reader.read(line);
    //System.out.println(input);
    c.ilist = input;
    c.ilistposptr = 0;
    Object name = c.ilistNext();
    if (name instanceof Symbol) {
      Object parent = YoYo.evalOneFromHere(c);
      if (parent instanceof YoYoObject) {
	YoYoObject newobj = new YoYoObject((YoYoObject)parent);
	c.getGlobalObj().defineValue((Symbol)name, newobj);
	c.ufunenv = newobj;
	return;
      }
      else LogoError.error("parent of " + name + " is not an object", c);
    }
    else LogoError.error("create-object needs an object name", c);
  }
	
  static void switchObject(String line, Context c) {
    //System.out.println("in-object " + line);
    Object[] input = Reader.read(line);
    //System.out.println(input);
    Object obj = YoYo.evalOne(input, c);
    //System.out.println(obj);
    if (obj instanceof YoYoObject) {
      c.ufunenv = (YoYoObject)obj;
    }
    else LogoError.error("in-object needs an object", c);
  }
	
  static void createInstanceVar(String line, Context c) {
    //System.out.println("instance-var " + line);
    Object[] input = Reader.read(line);
    //System.out.println(input);
    c.ilist = input;
    c.ilistposptr = 0;
    Object name = c.ilistNext();
    if (name instanceof Symbol) {
      Object val = YoYo.evalOneFromHere(c);
      c.ufunenv.defineValue((Symbol)name, val);
      return;
    }
    else LogoError.error("instance-var needs a variable name");
  }
	
	
  static String readbody (BufferedReader in) throws IOException {
      StringBuffer body= new StringBuffer();
      String line;
      do {
	  line=readaline(in);
	  //System.out.println("body: " + line);
	  if (line==null || line.startsWith("end",0)) {
	      //System.out.println("readbody returns: " + body);
	      return body.toString();
	  }
	  body.append(line);
	  body.append("\r\n");
      } while(true);
  }
	
  static String readaline(BufferedReader in) throws IOException {
    //System.out.println("about to read a line");
    String s = in.readLine();
    if (s == null) {
      //System.out.println("end of file");
      throw new IOException();
    }
    //System.out.println("readaline: " + s);
    return s;
  }  
	
  static Symbol letsym = Symbol.lookup("let"), dolist = Symbol.lookup("dolist"),
    dotimes = Symbol.lookup("dotimes"), set = Symbol.lookup("set"),
    lambda = Symbol.lookup("lambda"), repeat = Symbol.lookup("repeat");
	
  static Symbol[] readlocals(Symbol[] arglist, Symbol[] output, Object[] body, int start) {
    try  {
      //System.out.println("calling readlocals on " + YoYo.printToString(body));
      for(int i = start; i < body.length; i++) {
	//System.out.println("body[i] = " + YoYo.printToString(body[i]));
	if (body[i] instanceof Symbol) {
	  Symbol token = (Symbol)body[i];
	  if (readlocals_Symbol(token, arglist, output, body, i)) {
	    //System.out.println("found a local " + token);
	    continue;
	  }
	  else if (token == set) {
	    if (i+1 >= body.length) continue;
	    if (body[i+1] instanceof Symbol) {
	      Symbol var = (Symbol)body[++i];
	      readlocals_Symbol(var, arglist, output, body, i);
	    }
	    else if (body[i+1] instanceof CompoundSymbol) {
	      CompoundSymbol cs = (CompoundSymbol)body[++i];
	      output = readlocals_CompoundSymbol(cs, arglist, output);
	    }
	  }
	  else if (token == letsym) {
	    if (i+1 >= body.length) continue;
	    if (body[i+1] instanceof Object[]) {
	      Object[] letlist = (Object[])body[++i];
	      output = readlocals_Let(letlist, arglist, output);
	    }
	  }
	  else if (token == dolist || token == dotimes) {
	    if (i+1 >= body.length) continue;
	    if (body[i+1] instanceof Object[]) {
	      Object[] letlist = (Object[])body[++i];
	      output = readlocals_InnerLet(letlist, arglist, output);
	    }
	  }
	  else if (token == lambda) {
	    if (i+2 >= body.length) continue;
	    if (body[i+1] instanceof Object[] &&
		body[i+2] instanceof Object[]) {
	      i ++;
	    }
	  }
	}
	else if (body[i] instanceof CompoundSymbol) {
	  output = readlocals_CompoundSymbol((CompoundSymbol)body[i], arglist, output);
	}
	else if (body[i] instanceof Object[]) {
	  output = readlocals(arglist, output, (Object[])body[i], 0);
	}
	else if (body[i] instanceof Ilist) {
	  output = readlocals(arglist, output, ((Ilist)body[i]).list, 0);
	}
      }
    } catch (NullPointerException e) { }
    return output;
  }
	
  static Symbol[] readlocals_Let(Object[] letlist, Symbol[] arglist, Symbol[] output) {
    if (letlist.length > 0) {
      for(int j = 0; j < letlist.length; j++) {
	if (letlist[j] instanceof Object[]) {
	  Object[] innerletlist = (Object[])letlist[j];
	  output = readlocals_InnerLet(innerletlist, arglist, output);
	}
      }
    }
    return output;
  }
	
  static Symbol[] readlocals_InnerLet(Object[] innerletlist, Symbol[] arglist, Symbol[] output) {
    if (innerletlist.length > 1) { 
      if (innerletlist[0] instanceof Symbol) {
	Symbol letvar = (Symbol)innerletlist[0];
	if (!readlocals_Symbol(letvar, arglist, output, innerletlist, 0)) {
	  innerletlist[0] = new LocalRef(arglist.length + output.length, letvar);
	  Symbol[] newoutput = new Symbol[output.length+1];
	  System.arraycopy(output, 0, newoutput, 0, output.length);
	  newoutput[output.length] = letvar;
	  output = newoutput;
	}
	output = readlocals(arglist, output, innerletlist, 1);
      }
    }
    return output;
  }
	
	
  static Symbol[] readlocals_CompoundSymbol(CompoundSymbol cs, Symbol[] arglist, Symbol[] output) {
    if (cs.parts.length > 0) {
      Object first = cs.parts[0];
      if (first instanceof Symbol) {
	Symbol token = (Symbol)first;
	readlocals_Symbol(token, arglist, output, cs.parts, 0);
      }
      for(int j = 0; j < cs.parts.length; j++) {
	Object part = cs.parts[j];
	if (part instanceof Ilist) {
	  output = readlocals(arglist, output, ((Ilist)part).list, 0);
	}
      }
    }
    return output;
  }
	
  static boolean readlocals_Symbol(Symbol var, Symbol[] arglist, Symbol[] output, 
				   Object[] body, int i) {
    int pos = position(var, arglist);
    if (pos >= 0) {
      body[i] = new LocalRef(pos, var);
      return true;
    }
    else {
      pos = position(var, output);
      if (pos >= 0) {
	body[i] = new LocalRef(pos + arglist.length, var);
	return true;
      }
    }
    return false;
  }
	
	
  static int position(Symbol s, Symbol[] list) {
    for(int i = 0; i < list.length; i++) {
      if (list[i] == s) return i;
    }
    return -1;
  }

    
	
}
