/********************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *******************************************************************/
 package yoyo;

import java.util.Random;
import java.util.ResourceBundle;

public class PrimMath implements YoYoMethods {
	
    static Random random = new Random();
	
    public String[] methodList() {
	ResourceBundle rb = PrimStartup.getYoYoCommandsBundle();
	String[] output = {
	    rb.getString("primmath_sum"), "prim_sum",
	    rb.getString("primmath_rem"), "prim_rem",
	    rb.getString("primmath_difference"), "prim_sub",
	    rb.getString("primmath_product"), "prim_mul",
	    rb.getString("primmath_quotient"), "prim_div",
	    rb.getString("primmath_power"), "prim_power",
	    rb.getString("primmath_greaterp"), "prim_greater",
	    rb.getString("primmath_lessp"), "prim_less",
	    rb.getString("primmath_minus"), "prim_sub",
	    rb.getString("primmath_rsh"), "prim_rsh",
	    rb.getString("primmath_lsh"), "prim_lsh",
	    rb.getString("primmath_bitneg"), "prim_bitneg",
	    rb.getString("primmath_min"), "prim_min",
	    rb.getString("primmath_max"), "prim_max",
	    rb.getString("primmath_random"), "prim_random",
	    rb.getString("primmath_sin"), "prim_sin",
	    rb.getString("primmath_cos"), "prim_cos",
	    rb.getString("primmath_tan"), "prim_tan",
	    rb.getString("primmath_atan"), "prim_atan",
	    rb.getString("primmath_acos"), "prim_acos",
	    rb.getString("primmath_asin"), "prim_asin",
	    rb.getString("primmath_exp"), "prim_exp",
	    rb.getString("primmath_ln"), "prim_ln",
	    rb.getString("primmath_int"), "prim_int",
	    rb.getString("primmath_round"), "prim_round",
	    rb.getString("primmath_abs"), "prim_abs",
	    rb.getString("primmath_numberp"), "prim_numberp",
	    rb.getString("primmath_sqrt"), "prim_sqrt",
	    rb.getString("primmath_string_to_number"), "prim_ston",
	    rb.getString("primmath_not"), "prim_not",
	    rb.getString("primmath_string_is_numberp"), "prim_stnum",
	};
	return output;
    }
	
    public String[] infixList() {
	ResourceBundle rb = PrimStartup.getYoYoCommandsBundle();
	String[] output = {
	    "+", "prim_sum", "-3",
	    "*", "prim_mul", "-4",
	    "-", "prim_sub", "-3",
	    "/", "prim_div", "-4",
	    "<=", "prim_lessequal", "-2",
	    ">=", "prim_greaterequal", "-2",
	    ">", "prim_greater","-2",
	    "<", "prim_less", "-2",
	    "^", "prim_power", "-5",
	    rb.getString("primmath_mod"), "prim_mod", "-4",
	    rb.getString("primmath_bitand"), "prim_bitand", "-6",
	    rb.getString("primmath_bitor"), "prim_bitor", "-6",
	    rb.getString("primmath_bitxor"), "prim_bitxor", "-6",
	    "=", "prim_equal", "-2",
	    "!=", "prim_notequal", "-2",
	    rb.getString("primmath_and"), "prim_and", "-1",
	    rb.getString("primmath_or"), "prim_or", "-1",
	};
	return output;
    }

    public void prePrimCall(Primitive p, Object[] arglist, Context c) {}
    public void postPrimCall(Primitive p, Context c) {}
	
    public static Object prim_sum(Object v0, Object v1, Context c)  {
	return new Double(YoYo.aDouble(c,v0)+YoYo.aDouble(c,v1));
    }
	
    public static Object prim_sub(Object v0, Object v1, Context c)  {
	return new Double(YoYo.aDouble(c,v0)-YoYo.aDouble(c,v1));
    }
	
    public static Object prim_mul(Object v0, Object v1, Context c)  {
	return new Double(YoYo.aDouble(c,v0)*YoYo.aDouble(c,v1));
    }
	
    public static Object prim_div(Object v0, Object v1, Context c)  {
	return new Double(YoYo.aDouble(c,v0)/YoYo.aDouble(c,v1));
    }
	
    public static Object prim_mod(Object v0, Object v1, Context c)  {
	double divisor = YoYo.aDouble(c, v1);
	double answer = YoYo.aDouble(c,v0) % divisor;
	if (answer < 0) return new Double(answer + divisor);
	return new Double(answer);
    }
	
    public static Object prim_power(Object v0, Object v1, Context c) {
	return new Double(Math.pow(YoYo.aDouble(c, v0), YoYo.aDouble(c, v1)));
    }
	
    public static Object prim_less(Object v0, Object v1, Context c)  {
	if (YoYo.aDouble(c,v0)<YoYo.aDouble(c,v1))
	    { return YoYo.symtrue; }
	else {return YoYo.symfalse; }
    }
	
    public static Object prim_greater(Object v0, Object v1, Context c)  {
	if (YoYo.aDouble(c,v0)>YoYo.aDouble(c,v1))
	    { return YoYo.symtrue; }
	else {return YoYo.symfalse; }
    }
	
    public static Object prim_lessequal(Object v0, Object v1, Context c)  {
	if (YoYo.aDouble(c,v0)<=YoYo.aDouble(c,v1))
	    { return YoYo.symtrue; }
	else {return YoYo.symfalse; }
    }
	
    public static Object prim_greaterequal(Object v0, Object v1, Context c)  {
	if (YoYo.aDouble(c,v0)>=YoYo.aDouble(c,v1))
	    { return YoYo.symtrue; }
	else {return YoYo.symfalse; }
    }
	
    public static Object prim_rsh(Object v0, Object v1, Context c)  {
	return new Integer(YoYo.aInteger(c,v0)>>YoYo.aInteger(c,v1));
    }
	
    public static Object prim_lsh(Object v0, Object v1, Context c)  {
	return new Integer(YoYo.aInteger(c,v0)<<YoYo.aInteger(c,v1));
    }
	
    public static Object prim_bitand(Object v0, Object v1, Context c)  {
	return new Integer(YoYo.aInteger(c,v0)&YoYo.aInteger(c,v1));
    }
	
    public static Object prim_bitor(Object v0, Object v1, Context c)  {
	return new Integer(YoYo.aInteger(c,v0)|YoYo.aInteger(c,v1));
    }
	
    public static Object prim_bitxor(Object v0, Object v1, Context c)  {
	return new Integer(YoYo.aInteger(c,v0)^YoYo.aInteger(c,v1));
    }
	
    public static Object prim_bitneg(Object v0, Context c)  {
	return new Integer(~YoYo.aInteger(c,v0));
    }
	
    public static Object prim_min(Object v0, Object v1, Context c)  {
	if (YoYo.aDouble(c, v0) < YoYo.aDouble(c, v1))
	    return v0;
	else return v1;
    }
	
    public static Object prim_max(Object v0, Object v1, Context c)  {
	if (YoYo.aDouble(c, v0) > YoYo.aDouble(c, v1))
	    return v0;
	else return v1;
    }
	
    public static Object prim_random(Object v0, Context c)  {
	return new Integer((int)Math.floor(random.nextDouble() * 
					   YoYo.aInteger(c, v0))); 
    }
	
    public static double deg2Rad(double deg) {
	return ((deg % 360) / 180.0) * Math.PI;
    }
	
    public static double rad2Deg(double rad) {
	return (rad * 180.0) / Math.PI;
    }
	
    public static Object prim_sin(Object v0, Context c)  {
	return new Double(Math.sin(deg2Rad(YoYo.aDouble(c,v0))));
    }
	
    public static Object prim_cos(Object v0, Context c)  {
	return new Double(Math.cos(deg2Rad(YoYo.aDouble(c,v0))));
    }
	
    public static Object prim_tan(Object v0, Context c)  {
	return new Double(Math.tan(deg2Rad(YoYo.aDouble(c,v0))));
    }
	
    public static Object prim_atan(Object v0, Object v1, Context c)  {
	return new Double(rad2Deg(Math.atan(
					    YoYo.aDouble(c,v0)/YoYo.aDouble(c,v1))));
    }
	
    public static Object prim_asin(Object v0, Context c)  {
	return new Double(rad2Deg(Math.asin(YoYo.aDouble(c,v0))));
    }
	
    public static Object prim_acos(Object v0, Context c)  {
	return new Double(rad2Deg(Math.acos(YoYo.aDouble(c,v0))));
    }
	
    public static Object prim_exp(Object v0, Context c)  {
	return new Double(Math.exp(YoYo.aDouble(c,v0)));
    }
	
    public static Object prim_ln(Object v0, Context c)  {
	return new Double(Math.log(YoYo.aDouble(c,v0)));
    }
	
    public static Object prim_int(Object v0, Context c)  {
	return new Double(YoYo.aInteger(c, v0));
    }
	
    public static Object prim_round(Object v0, Context c)  {
	return new Double(Math.round(YoYo.aDouble(c,v0)));
    }
	
    public static Object prim_abs(Object v0, Context c)  {	
	double foo = YoYo.aDouble(c, v0); 
	if (foo < 0) return new Double(-foo); else return v0; 
    }
	
    public static Object prim_numberp(Object v0, Context c)  {
	if (v0 instanceof Number) 
	    return YoYo.symtrue;
	else return YoYo.symfalse;
    }
	
    public static Object prim_equal (Object v0, Object v1, Context c)  {
	return YoYo.equal(v0, v1);
    }
	
    public static Object prim_notequal (Object v0, Object v1, Context c)  {
	return (YoYo.equal(v0, v1) == YoYo.symtrue) 
	    ? YoYo.symfalse : YoYo.symtrue;
    }
	
    public static Symbol truefalse(boolean tf) {
	return (tf) ? YoYo.symtrue : YoYo.symfalse;
    }
	
    public static Object prim_and(Object v0, Object v1, Context c)  {
	//System.out.println("and: " + YoYo.printToString(v0) + " " 
	//		   + YoYo.printToString(v1));
	if (YoYo.aBoolean(c, v0) && YoYo.aBoolean(c, v1))
	    return YoYo.symtrue;
	else return YoYo.symfalse;
    }
	
    public static Object prim_or(Object v0, Object v1, Context c)  {
	if (YoYo.aBoolean(c, v0) || YoYo.aBoolean(c, v1))
	    return YoYo.symtrue;
	else return YoYo.symfalse;
    }
	
    public static Object prim_xor(Object v0, Object v1, Context c)  {
	if (YoYo.aBoolean(c, v0) ^ YoYo.aBoolean(c, v1))
	    return YoYo.symtrue;
	else return YoYo.symfalse;
    }
	
	
    public static Object prim_sqrt(Object v0, Context c)  {
	return new Double(Math.sqrt(YoYo.aDouble(c, v0)));
    }
	
    public static Object prim_ston(Object v0, Context c)  {
	String foo = YoYo.aString(c, v0);
	Object[] bar = Reader.read(foo);
	if (bar.length > 0 && bar[0] instanceof Number) return bar[0];
	LogoError.error(foo + LogoError.NOT_NUMBER, c);
	return null;
    }
	
    public static Object prim_stnum(Object v0, Context c)  {
	String foo = YoYo.aString(c, v0);
	Object[] bar = Reader.read(foo);
	if (bar.length > 0 && bar[0] instanceof Number) return YoYo.symtrue;
	return YoYo.symfalse;
    }
	
}
