/********************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *******************************************************************/
 package yoyo;

import java.util.ResourceBundle;

public class PrimObject implements YoYoMethods {
	
	public String[] methodList() {
	    ResourceBundle rb = PrimStartup.getYoYoCommandsBundle();
	    String[] output = {
		rb.getString("primobject_new_object"), "prim_new_object",
		rb.getString("primobject_copy_object"), "prim_copyobject",
		rb.getString("primobject_get_fields"), "prim_fields",
		rb.getString("primobject_set_field"), "prim_setfield",
		rb.getString("primobject_get_field"), "prim_getfield",
		rb.getString("primobject_get_fields_and_values"), "prim_printobject",
		rb.getString("primobject_self"), "prim_thisobject",
		rb.getString("primobject_objectp"), "prim_objectp",
		rb.getString("primobject_global_environment"), "prim_global",
		rb.getString("primobject_to_object"), "prim_to_object",
	    };
	    return output;
	}
	
	public String[] infixList() {
		return null;
	}
	
    public void prePrimCall(Primitive p, Object[] arglist, Context c) {}
    public void postPrimCall(Primitive p, Context c) {}

	public static Object prim_new_object(Context c)  {
		return new YoYoObject(c);
	}
	
	public static Object prim_objectp(Object v, Context c)  {
		return (v instanceof YoYoObject)
			? YoYo.symtrue : YoYo.symfalse;
	}
	
	public static Object prim_copyobject(Object v0, Context c)  {
		return new YoYoObject(YoYo.aYoYoObject(c, v0));	
	}
	
	public static Object prim_fields(Object v0, Context c)  {
		YoYoObject r = YoYo.aYoYoObject(c, v0);
		return r.fields();
	}
	
	public static Object prim_printobject(Object v0, Context c)  {
		YoYoObject e = YoYo.aYoYoObject(c,v0);
		return e.fieldsAndValues();
	} 
	
	public static Object prim_getfield(Object v0, Object v1, Context c)  {
		Object foo = YoYo.aYoYoObject(c, v0).lookupValue(Symbol.lookup(YoYo.aString(c,v1)));
		if (foo != null) return foo;
		return YoYo.symfalse;			
	}
	
	public static Object prim_setfield(Object v0, Object v1, Object v2, Context c)  {
		YoYo.aYoYoObject(c, v0).setValue(Symbol.lookup(YoYo.aString(c,v1)), v2);
		return null;
	}
	
	public static Object prim_thisobject(Context c)  {
		return c.objenv;
	}
	
	public static Object prim_global(Context c) {
		return c.getGlobalObj();
	}
	
	public static Object prim_to_object(Object v0, Context c) {
		Object[] list = YoYo.aList(c, v0);
		Object[] oldilist = c.ilist;
		int oldilistposptr = c.ilistposptr;
		YoYoObject obj = new YoYoObject();
		try {
			c.ilist = list;
			c.ilistposptr = 0;
			while(c.ilistPeek() != null) {
				Symbol name = YoYo.aSymbol(c, c.ilistNext());
				if (c.ilistPeek() == null) 
				    LogoError.error(LogoError.TOO_FEW_REPORTERS + PrimStartup.getYoYoCommandsBundle().getString("primobject_to_object"), c);
				obj.defineValue(name, YoYo.evalOneFromHere(c));
			}
		}
		finally {
			c.ilist = oldilist;
			c.ilistposptr = oldilistposptr;
		}
		return obj;
	}
	
}
