/********************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *******************************************************************/
 package yoyo;

import java.lang.reflect.Method;
import java.lang.reflect.Field;
import java.net.URL;
import java.net.MalformedURLException;
import java.io.DataInputStream;
import java.io.InputStream;
import java.io.IOException;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.ResourceBundle;
import java.util.Locale;

public class PrimStartup implements YoYoMethods {

    public static ResourceBundle yoyoCommandsBundle, yoyoErrorsBundle;

    public static ResourceBundle getYoYoCommandsBundle() {
	if (yoyoCommandsBundle == null) {
	    yoyoCommandsBundle = ResourceBundle.getBundle("yoyo.YoYoCommands", YoYo.language);
	}
	return yoyoCommandsBundle;
    }

    public static ResourceBundle reGetYoYoCommandsBundle() {
	yoyoCommandsBundle = ResourceBundle.getBundle("yoyo.YoYoCommands", YoYo.language);
	return yoyoCommandsBundle;
    }

    public static ResourceBundle getYoYoErrorsBundle() {
	if (yoyoErrorsBundle == null) {
	    yoyoErrorsBundle = ResourceBundle.getBundle("yoyo.YoYoErrors", YoYo.language);
	}
	return yoyoErrorsBundle;
    }
    
    public static ResourceBundle reGetYoYoErrorsBundle() {
	yoyoErrorsBundle = ResourceBundle.getBundle("yoyo.YoYoErrors", YoYo.language);
	return yoyoErrorsBundle;
    }

    public static void initClasses(YoYoObject global) {
	try {
	    dload("yoyo.PrimStartup", global);
	    dload("yoyo.Primitives", global);
	    dload("yoyo.PrimMath", global);
	    dload("yoyo.PrimList", global);
	    dload("yoyo.PrimControl", global);
	    dload("yoyo.PrimVar", global);
	    dload("yoyo.PrimObject", global);
	    dload("yoyo.PrimFile", global);
	    dload("yoyo.PrimTCP", global);
	    dload("yoyo.StringParser", global);
	}
	catch (LogoError e) {
	    System.out.println(e.getLocalizedMessage());
	}
    }

    public String[] methodList() {
	ResourceBundle rb = getYoYoCommandsBundle();
	String[] output = {
	    rb.getString("primstartup_dload"), "prim_dload",
	    rb.getString("primstartup_set_listener_function"), "prim_setlistenerfunc",
	};
	return output;
    }

    public String[] infixList() {
	return null;
    }

    public void prePrimCall(Primitive p, Object[] arglist, Context c) {}
    public void postPrimCall(Primitive p, Context c) {}

    public static Object prim_dload(Object v, Context c) {
	dload(YoYo.aString(c, v), c.getGlobalObj());
	return null;
    }

    static void dload(String s, YoYoObject global) {

	Class cl=null;
	String[] primlist = null;
	String[] infixlist = null;
	YoYoMethods instance = null;

	try{cl=Class.forName(s);}
	catch (Exception e) { e.printStackTrace(); }
	if (cl==null) {
	    LogoError.error("Missing or Invalid Yoyo Class File: " + s);
	}

	//System.out.println("loaded class: " + cl.toString());

	try {
	    instance = (YoYoMethods)cl.newInstance();
	    primlist = instance.methodList();
	    infixlist = instance.infixList();
	}
	catch (Exception e) {
	    e.printStackTrace();
	    LogoError.error("Class " + s + " has no YoYo methods inside.");
	}

	//System.out.println(YoYo.printToString(primlist));
	//if (infixlist != null) System.out.println(infixlist);

	Method[] methods = null;
	try {
	    methods = cl.getDeclaredMethods();
	}
	catch(SecurityException e) {
	    e.printStackTrace();
	    System.out.println("Security Managers suck. You can't run YoYo here.");
	}
	if (primlist!=null) {
	    for (int i = 0; i<primlist.length; i+=2) {
		Method m = null;
		for (int j = 0; j<methods.length; j++) {
		    if (methods[j].getName().equalsIgnoreCase(primlist[i+1])) {
			//System.out.println("got method " + methods[j].getName());
			global.defineValue(Symbol.lookup(primlist[i]),
					   new Primitive(primlist[i], methods[j], instance));
		    }
		}
	    }
	}

	int precedence=2;
	if (infixlist!=null) {
	    for (int i = 0; i<infixlist.length; i+=3) {
		Method m = null;
		for (int j = 0; j<methods.length; j++) {
		    if (methods[j].getName().equalsIgnoreCase(infixlist[i+1])) {
			try {
			    precedence = Integer.parseInt(infixlist[i+2]);
			    global.defineValue(Symbol.lookup(infixlist[i]),
					       new Primitive(infixlist[i], methods[j], precedence, instance));
			}
			catch (NumberFormatException fff) {};
		    }
		}
	    }
	}

	return;
    }

    public static void prim_setlistenerfunc(Object v0, Context c) {
	if (c.caller instanceof Listener) {
	    ((Listener)c.caller).setFormatInput(aUfun(c, v0));
	}
    }
    
    public static Ufun aUfun(Context c, Object o) {
	if (o instanceof Ufun) return ((Ufun)o);
	LogoError.error(YoYo.printToString(o) + LogoError.NOT_PROCEDURE, c);
	return null;
    }


}
