/********************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *******************************************************************/
 package yoyo;

import java.io.*;
import java.net.*;
import java.util.Date;
import java.text.SimpleDateFormat;
import java.util.ResourceBundle;

public class PrimTCP implements YoYoMethods {

  static SimpleDateFormat sdf = new SimpleDateFormat("yyyy/MM/dd HH:mm");

    public String[] methodList() {
	ResourceBundle rb = PrimStartup.getYoYoCommandsBundle();
	String[] output = {
	    rb.getString("primtcp_tcp_open_receive"), "prim_openserver",
	    rb.getString("primtcp_tcp_accept"), "prim_accept",
	    rb.getString("primtcp_tcp_send_packet"), "prim_sendpacket",
	    rb.getString("primtcp_tcp_read_packet"), "prim_readpacket",
	    rb.getString("primtcp_tcp_close"), "prim_closesocket",
	};
	return output;
    }

  public String[] infixList() {
    return null;
  }

    public void prePrimCall(Primitive p, Object[] arglist, Context c) {}
    public void postPrimCall(Primitive p, Context c) {}

  public static Object prim_openserver(Object v0, Context c) {
    try {
      ServerSocket out = new ServerSocket(YoYo.aInteger(c, v0));
      out.setSoTimeout(30);
      return out;
    } catch (IOException e) {
      e.printStackTrace();
      LogoError.error("could not open socket on port: " + YoYo.printToString(v0));
    }
    return YoYo.symfalse;
  }

  public static Object prim_accept(Object v0, Context c) {
     while (true) {
       try {
         Socket sock = ((ServerSocket)v0).accept();
	 sock.setSoTimeout(20);
	 return sock;
       } catch (InterruptedIOException e) {
       } catch (IOException e) {
         e.printStackTrace();
         LogoError.error("could not accept socket");
       }
       YoYo.pollStop(c);
     }
  }


  public static void prim_sendpacket(Object v0, Object v1, Context c) {
    try {
      Socket sock = (Socket)v0;
      Object[] tosend = YoYo.aList(c, v1);
      OutputStream out = sock.getOutputStream();
      byte[] bytes = listToByteArray(tosend);
      writeInt(bytes.length, out);
      out.write(bytes);
      out.flush();
    } catch (IOException e) {
      e.printStackTrace();
      LogoError.error("could not send byte array " + YoYo.printToString(v1) + " to socket " + YoYo.printToString(v0));
    }
  }

  public static byte[] listToByteArray(Object[] foo) {
    byte[] out = new byte[foo.length];
    for(int i = 0; i < foo.length; i++) {
      out[i] = (byte)(((Number)foo[i]).intValue());
    }
    return out;
  }

  public static Object[] byteArrayToList(byte[] foo) {
    Object[] out = new Object[foo.length];
    for(int i = 0; i < foo.length; i++) {
      out[i] = new Integer(((int)foo[i]) & 0xFF);
    }
    return out;
  }

  public static void writeInt(int i, OutputStream out) throws IOException {
    out.write((byte)((i >> 24) & 0xFF));
    out.write((byte)((i >> 16) & 0xff));
    out.write((byte)((i >> 8) & 0xff));
    out.write((byte)(i & 0xff));
  }

    public static int readByte(InputStream in, Context c) throws IOException {
      while (true) {
	  try {
	      return in.read();
	  } catch (InterruptedIOException iie) { 
	      YoYo.pollStop(c);
	  }
      }
    }

  public static int readInt(InputStream in, Context c) throws IOException {
      return (((int)readByte(in, c) & 0xFF) << 24) |
	  (((int)readByte(in, c) & 0xFF) << 16) |
	  (((int)readByte(in, c) & 0xFF) << 8) |
	  ((int)readByte(in, c) & 0xFF);
  }


  public static Object prim_getstring(Object v0, Context c) {
    try {
       Socket sock = (Socket)v0;
       InputStream in = sock.getInputStream();
       while (in.available() < 4) YoYo.pollStop(c);
       int length = readInt(in, c);
       byte[] result = new byte[length];
       while (in.available() < length) YoYo.pollStop(c);
       in.read(result);
       return new String(result);
    } catch (IOException e) {
      e.printStackTrace();
      LogoError.error("can't get string from socket " + YoYo.printToString(v0));
    }
    return YoYo.symfalse; // stupid compiler
  }

  public static Object prim_readpacket(Object v0, Context c) {
    try {
       Socket sock = (Socket)v0;
       InputStream in = sock.getInputStream();
              /*
	 while (in.available() < 4) { 
	 YoYo.pollStop(c);
	 Thread.currentThread().sleep(20);
	 }
       */
       int length = readInt(in, c);
       //System.out.println("receiving packet length: " + length);
       byte[] result = new byte[length];
       while (in.available() < length) YoYo.pollStop(c);
       in.read(result);
       return byteArrayToList(result);
    } catch (IOException e) {
      e.printStackTrace();
      LogoError.error("can't get string from socket " + YoYo.printToString(v0));
    } //catch (InterruptedException f) {}

    return YoYo.symfalse; // stupid compiler

  }
  public static void prim_closesocket(Object v0, Context c) {
    try {
      if (v0 instanceof Socket) {
        ((Socket)v0).close();
      } else if (v0 instanceof ServerSocket) {
        ((ServerSocket)v0).close();
      } else LogoError.error("can't tcp-close something that isn't a socket: " + YoYo.printToString(v0));
    } catch (IOException e) {
      e.printStackTrace();
      LogoError.error("can't close the socket. Java sucks.");
    }
  }


}
