/********************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *******************************************************************/
 package yoyo;

import java.util.Enumeration;
import java.util.Vector;
import java.util.Properties;
import java.util.ResourceBundle;
import java.lang.reflect.*;

public class PrimVar implements YoYoMethods {
	
    public String[] methodList() {
	ResourceBundle rb = PrimStartup.getYoYoCommandsBundle();
	String[] output = {
	    rb.getString("primvar_set"), "prim_set",
	    rb.getString("primvar_unset"), "prim_unset",
	    rb.getString("primvar_type_of"), "prim_typeof",
	    rb.getString("primvar_value_of"), "prim_valueof",
	    rb.getString("primvar_boundp"), "prim_boundp",
	    rb.getString("primvar_let"), "prim_let",
	    rb.getString("primvar_dlet"), "prim_dlet",
	    rb.getString("primvar_get_properties"), "prim_getprops",
	    rb.getString("primvar_get_property"), "prim_getprop",
	    rb.getString("primvar_get_parameter"), "prim_getparam",
	    rb.getString("primvar_weak_reference"), "prim_weakref",
	    rb.getString("primvar_dereference"), "prim_deref",
	    rb.getString("primvar_weak_referencep"), "prim_weakrefp"
	};
	return output;
    }
	
    public String[] infixList() {
	return null;
    }
	
    public void prePrimCall(Primitive p, Object[] arglist, Context c) {}
    public void postPrimCall(Primitive p, Context c) {}
	
    public static void prim_set(Context c)  {
	Object o = c.ilistNext();
		
	if (o instanceof LocalRef) {
	    c.locals.setValue((LocalRef)o, YoYo.evalOneFromHere(c));
	    return;
	}
		
	Sym sym = YoYo.aSym(c, o);
		
	if (c.ilistPeek() == null) 
	    LogoError.error(LogoError.TOO_FEW_REPORTERS + " " + PrimStartup.getYoYoCommandsBundle().getString("primvar_set"), c);
		
	if (sym instanceof CompoundSymbol) {
	    prim_set_record((CompoundSymbol)sym, c);
	    return;
	}
		
	Symbol s = (Symbol)sym;
		
	if (c.locals.boundp(s)) {
	    int index = c.locals.index(s);
	    LocalRef l = new LocalRef(index, s);
	    c.setPrev(l);
	    c.locals.setValue(l, YoYo.evalOneFromHere(c));
	    return;
	}
		
	Object v1 = YoYo.evalOneFromHere(c);
		
	if (c.objenv.boundp(s)) {
	    c.objenv.setValue(s, v1);
	    return;
	}
		
	if (c.ufunenv != c.objenv) {
	    if (c.ufunenv.boundp(s)) {
		c.ufunenv.setValue(s, v1);
		return;
	    }
	}
		
	c.getGlobalObj().setValue(s, v1);
    }
	
    public static void prim_set_record(CompoundSymbol s, Context c) {
		
	Object value = YoYo.evalOneFromHere(c);
	Closure cs = YoYo.evalCompound(s, c);
	if (cs.object instanceof YoYoObject) {
	    YoYoObject obj = (YoYoObject)cs.object;
	    Object p = cs.last;
	    if (p instanceof String) p = Symbol.lookup((String)p);
	    if (p instanceof Symbol) obj.setValue((Symbol)p, value);
	    if (p instanceof Ilist) {
		obj.setValue(YoYo.aSymbol(c, YoYo.evalOne((Ilist)p, c)), value);
	    }
	    return;
	} else {
	    Object obj = cs.object;
	    Object p = cs.last;
	    if (p instanceof String) p = Symbol.lookup((String)p);
	    Class clazz = obj.getClass();
	    Field f = null;
	    try {
		if (p instanceof Symbol) {
		    f = clazz.getField(((Symbol)p).name);
		} else if (p instanceof Ilist) {
		    f = clazz.getField(YoYo.aSymbol(c, YoYo.evalOne((Ilist)p, c)).name);
		} 
	    }
	    catch (NoSuchFieldException nsfe) {}
	    if (f == null) {
		LogoError.error("no such field of object type " + clazz.getName(), c);
	    }
	    try {
		f.set(obj, value);
	    } catch (IllegalAccessException iae) {
		LogoError.error("you forgot to make " + f.toString() +
				"public", c); 
	    }
	    return;
	}
    }
	
    public static void prim_unset(Context c)  {
	Object o = c.ilistNext();
		
	if (o instanceof LocalRef) {
	    c.locals.undefineVar((LocalRef)o);
	    return;
	}
		
	Sym sym = YoYo.aSym(c, o);
		
	if (sym instanceof CompoundSymbol) {
	    prim_unset_record((CompoundSymbol)sym, c);
	    return;
	}
	Symbol s = (Symbol)sym;
		
	if (c.locals.boundp(s)) {
	    int index = c.locals.index(s);
	    LocalRef l = new LocalRef(index, s);
	    c.setPrev(l);
	    c.locals.undefineVar(l);
	    return;
	}
		
	if (c.objenv.boundp(s)) {
	    c.objenv.undefineValue(s);
	    return;
	}
		
	if (c.ufunenv != c.objenv) {
	    if (c.ufunenv.boundp(s)) {
		c.ufunenv.undefineValue(s);
		return;
	    }
	}
		
	c.getGlobalObj().undefineValue(s);
    }
	
    public static void prim_unset_record(CompoundSymbol s, Context c) {
		
	Closure cs = YoYo.evalCompound(s, c);
	if (cs.object instanceof YoYoObject) {
	    YoYoObject obj = (YoYoObject)cs.object;
	    Object p = cs.last;
	    if (p instanceof String) p = Symbol.lookup((String)p);
	    if (p instanceof Symbol) obj.undefineValue((Symbol)p);
	    if (p instanceof Ilist) {
		obj.undefineValue(YoYo.aSymbol(c, YoYo.evalOne((Ilist)p, c)));
	    }
	    return;
	} else LogoError.error("can't undefine a java field", c);
    }
	
    public static Object prim_typeof(Object v0, Context c)  {
	if (v0 instanceof Number) { return "number"; }
	if (v0 instanceof Sym || v0 instanceof LocalRef) { 
	    if (v0 == YoYo.symtrue || v0 == YoYo.symfalse) {
		return "boolean";
	    }
	    else return "symbol"; 
	}
	if (v0 instanceof String) { return "string"; }
	if (v0 instanceof YoYoObject) {
	    return ((YoYoObject)v0).lookupValue(Symbol.lookup("type"));
	}
	if (v0 instanceof Object[]) { return "list"; }
	if (v0 instanceof Ilist) { return "paren-expression"; }
	return v0.getClass().getName();
    }
	
    public static Object prim_valueof(Object v0, Context c)  {
		
	if (v0 instanceof String) {
	    Sym name = YoYo.aSym(c, Reader.read(YoYo.aString(c, v0))[0]);
	    Object o = YoYo.lookupSymbol(name, c);
	    if (o == null) 
		LogoError.error(LogoError.I_DONT_KNOW_HOW + YoYo.printToString(v0), c);
	    return o;
	}
	else return v0;
    }
	
    public static Object prim_boundp(Object v0, Context c)  {
		
	Sym name = YoYo.aSym(c, Reader.read(YoYo.aString(c, v0))[0]);
	try { 
	    Object o = YoYo.lookupSymbol(name, c);
	    if (o==null) return YoYo.symfalse;
	    return YoYo.symtrue;
	}
	catch (LogoError e) { return YoYo.symfalse; }
		
    }
	
    public static Object prim_getprops(Context c)  {
	Properties temp = System.getProperties();
	Object[] proplist = new Object[temp.size()];
	int index = 0;
	for (Enumeration e = temp.propertyNames(); e.hasMoreElements(); index++) {
	    String prop = (String)e.nextElement();
	    Object[] list = { prop, temp.getProperty(prop) };
	    proplist[index] = list;
	}
	return proplist;
    }	 	
	
    public static Object prim_getprop(Object v0, Context c)  {
	Properties temp = System.getProperties();
	Object value = temp.getProperty(YoYo.aString(c, v0));
	if (value!=null) return value;
	return YoYo.symfalse;
    }
	
    public static Object prim_weakref(Object v0, Context c) {
	return newWeakReference(v0);
    }

    public static Object prim_deref(Object v0, Context c) {
	return aWeakReference(v0,c);
    }
    
    public static Object prim_weakrefp(Object v0, Context c) {
	return YoYo.truefalse(weakReferencep(v0));
    }

    public static Object prim_let(Object v0, Context c) {
	Object[] list = YoYo.aList(c, v0);
	Object[] oldilist = c.ilist;
	int oldilistposptr = c.ilistposptr;
		
	try {
	    for(int i = 0; i<list.length; i++) {
		Object[] varlist = YoYo.aList(c, list[i]);
		if (varlist.length < 2) {
		    LogoError.error(LogoError.TOO_FEW_REPORTERS + PrimStartup.getYoYoCommandsBundle().getString("primvar_let"), c);
		}
		c.ilist = varlist;
		c.ilistposptr = 0;
				
		Object var = c.ilistNext();
		if (var instanceof LocalRef) {
		    c.locals.setValue((LocalRef)var, YoYo.evalOneFromHere(c));
		}
		else {
		    Symbol name = YoYo.aSymbol(c, var);
		    int index = (c.locals.boundp(name)) ? c.locals.index(name)
			: c.locals.defineVar(name);
		    LocalRef l = new LocalRef(index, name);
		    c.setPrev(l);
		    c.locals.setValue(l, YoYo.evalOneFromHere(c));
		    //if (c.ufunobj != null) {
		    //  if (c.locals.names.length > c.ufunobj.arglist.length) 
		    //    c.ufunobj.arglist = c.locals.names;
		    //}
		}
	    }
	}
	finally {
	    c.ilist = oldilist;
	    c.ilistposptr = oldilistposptr;
	}
	return null;
    }
	
    public static Object prim_dlet(Object v0, Context c) {
	Object[] list = YoYo.aList(c, v0);
	if (c.savedynamics==null) c.savedynamics = new Vector();
	Object[] oldilist = c.ilist;
	int oldilistposptr = c.ilistposptr;
	try {
	    c.ilist = list;
	    c.ilistposptr = 0;
			
	    while(c.ilistPeek() != null) {
		Symbol name = YoYo.aSymbol(c, c.ilistNext());
		if (c.ilistPeek() == null) 
		    LogoError.error(LogoError.TOO_FEW_REPORTERS + PrimStartup.getYoYoCommandsBundle().getString("primvar_dlet"), c);
		Object value = YoYo.evalOneFromHere(c);
		c.savedynamics.addElement(name);
		c.savedynamics.addElement(c.objenv.lookupValue(name));
		c.objenv.defineValue(name, value);
	    }
	}
	finally {
	    c.ilist = oldilist;
	    c.ilistposptr = oldilistposptr;
	}
		
	return null;
    }					
	
    static Object newWeakReference(Object referent) {
	try {
	    Class weakref = Class.forName("java.lang.ref.WeakReference");
	    Constructor cons = weakref.getConstructor(new Class[]{Object.class});
	    return cons.newInstance(new Object[]{referent});	
	} catch (ClassNotFoundException cnfe) {
	    return referent;
	} catch (NoSuchMethodException nsme) {
	    return referent;
	} catch (IllegalAccessException iae) {
	    return referent;
	} catch (IllegalArgumentException iarge) {
	    return referent;
	} catch (InstantiationException ie) {
	    return referent;
	} catch (InvocationTargetException ite) {
	    return referent;
	}
    }	
	
    static boolean weakReferencep(Object v0) {
	try {
	    if (v0 != null) {
		Class weakref = Class.forName("java.lang.ref.WeakReference");
		return (weakref.isAssignableFrom(v0.getClass()));
	    }
	} catch (ClassNotFoundException cnfe) {
	}
	return false;
    }
						
    public static Object aWeakReference(Object v0, Context c) {
	try {
	    if (v0 != null) {
		Class weakref = Class.forName("java.lang.ref.WeakReference");
		if (weakref.isAssignableFrom(v0.getClass())) {
		    Method weakrefget = weakref.getMethod("get", new Class[]{});
		    Object ret = weakrefget.invoke(v0, new Object[]{});
		    if (ret == null) return YoYo.symfalse;
		    return ret;	
		}
	    }
	} catch (ClassNotFoundException cnfe) {
	} catch (NoSuchMethodException nsme) {
	} catch (IllegalAccessException iae) {
	} catch (IllegalArgumentException iarge) {
	} catch (InvocationTargetException ite) {
	} catch (NullPointerException npe) {
	}
	LogoError.error(YoYo.printToString(v0) + LogoError.NOT_WEAK, c);
	return null;
    }
}
