/********************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *******************************************************************/
 package yoyo;

import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;
import java.util.Vector;
import java.util.Enumeration;
import java.io.Serializable;

public class Primitive implements Function, Serializable {
	
	String name = "";
	Symbol symname;
	Method method;
	int nargs;
	Object[] emptyarglist;
	boolean trace = false;
	static Vector tracedFuncs = new Vector();
    boolean yoyoprimp = true;
    Object nativeobj = null;
    YoYoMethods definingClass;

	public Primitive(String name, Method obj, YoYoMethods definingClass) {
		this.name = name;
		symname = Symbol.lookup(name);
		symname.primitive = this;
		method = obj;
		this.definingClass = definingClass;
		this.nargs = obj.getParameterTypes().length-1;
		if (this.nargs == 0) emptyarglist = new Object[1];
	}

	public Primitive(String name, Method obj, Object nativeobj) {
	    this.name = name;
	    if (nativeobj != null) {
		this.yoyoprimp = false;
		this.nativeobj = nativeobj;
	    }
	    symname = Symbol.lookup(name);
	    method = obj;
	    this.nargs = obj.getParameterTypes().length - ((yoyoprimp) ? 1 : 0);
	    if (this.nargs == 0) emptyarglist = new Object[(yoyoprimp) ? 1 : 0];
	}
	
	public Primitive(String name, Method obj, int prec, YoYoMethods definingClass) {
		this.name = name;
		symname = Symbol.lookup(name);
		symname.primitive = this;
		method = obj;
		this.definingClass = definingClass;
		this.nargs = prec;
	}
	
	public Symbol getName() {
		return symname;
	}
	
	public int getNargs() {
		return nargs;
	}
	
	public Object[] getArgArray(Context c) {
	    if (nargs>0) return Pool.getArray(nargs + ((yoyoprimp) ? 1 : 0), c);
	    else if (nargs == 0) {
		if (c.ub.numContexts() == 1) return emptyarglist;
		else return Pool.getArray((yoyoprimp) ? 1 : 0, c);
	    }
	    // negative args can't be !yoyoprimp
	    else return Pool.getArray(3, c);
	}
	
	public Object run(Object[] arglist, Context c) {
		if (trace) c.traceEnter(name, arglist);
		YoYoObject oldenv = c.objenv;
		
		if (yoyoprimp) arglist[arglist.length-1] = c;
		Object value = null;
		try { 
			//if (env != null) c.objenv = env;
		    if (!yoyoprimp) {
			for(int i = 0; i < arglist.length; i++) {
			    if (arglist[i] == YoYo.symtrue) arglist[i] = Boolean.TRUE;
			    else if (arglist[i] == YoYo.symfalse) arglist[i] = Boolean.FALSE;
			}
			//System.out.println("args are " + YoYo.printToString(arglist));
		    }
		    if (definingClass != null) {
			definingClass.prePrimCall(this, arglist, c);
		    }
		    //System.out.println("args are " + YoYo.printToString(arglist));
		    value = method.invoke((yoyoprimp) ? null : nativeobj, arglist); 
		}
		catch (IllegalAccessException e) {
			LogoError.error("you forgot to make " + this.toString() +
				"public", c); 
		}
		catch (InvocationTargetException f) {
			Throwable g = f.getTargetException();
			if (g instanceof LogoError) {
				if (trace) c.traceThrow(name, g);
				throw (LogoError)g;
			}
			if (g instanceof Stop) throw (Stop)g;
			if (trace) c.traceThrow(name, g);
			g.printStackTrace(c.err);
			LogoError.error(g.toString(), c);
		}
		finally {
			// clear out the context
			if (arglist == emptyarglist) arglist[0] = null;
			else Pool.dropArray(arglist, c);
			if (trace) c.traceExit(name, value);
			if (definingClass != null) {
			    definingClass.postPrimCall(this, c);
			}
		}
		//System.out.println("returned from primitive call: " + name);		
		return value;
	}
	
	public void trace() {
		trace = true;
	}
	
	public void untrace() {
		trace = false;
	}
	
	public String toString() {
		return "<Primitive: " + ((!yoyoprimp) ? nativeobj.getClass().getName() + "." : "") + name + ">";    
	}
}
