/********************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *******************************************************************/
 package yoyo;

import java.awt.Container;
import java.awt.Frame;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.io.StreamTokenizer;
import java.io.InputStream;
import java.io.DataInputStream;
import java.net.URL;
import java.io.FileInputStream;
import java.util.Enumeration;
import java.util.Vector;
import java.util.Properties;
import java.net.MalformedURLException;
import java.net.URLConnection;
import java.applet.Applet;
import java.util.Hashtable;
import java.io.BufferedInputStream;
import java.lang.reflect.Method;
import java.lang.reflect.Field;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.File;
import java.io.FileReader;
import java.io.Reader;
import java.io.FileNotFoundException;
//import sun.audio.*;
import java.util.ResourceBundle;

public class Primitives implements YoYoMethods {
	
  public String[] methodList() {
      ResourceBundle rb = PrimStartup.getYoYoCommandsBundle();
    String[] output = {
	rb.getString("primitives_print"), "prim_print",
	rb.getString("primitives_show"), "prim_print",
	rb.getString("primitives_type"), "prim_type",
	rb.getString("primitives_resett"), "prim_resett",
	rb.getString("primitives_timer"), "prim_timer",
	rb.getString("primitives_freememory"), "prim_freememory",
	rb.getString("primitives_not"), "prim_not",
	rb.getString("primitives_ignore"), "prim_ignore",
	rb.getString("primitives_system_quit"), "prim_system_quit",
	rb.getString("primitives_symbolp"), "prim_symbolp",
	rb.getString("primitives_booleanp"), "prim_booleanp",
	rb.getString("primitives_colon"), "prim_delay",
	rb.getString("primitives_lambda"), "prim_lambda",
	rb.getString("primitives_to_string"), "prim_to_string",
	rb.getString("primitives_unquote"), "prim_unquote",
	rb.getString("primitives_ilistp"), "prim_ilistp",
	rb.getString("primitives_ilist_to_list"), "prim_ilisttolist",
	rb.getString("primitives_list_to_ilist"), "prim_listtoilist",
	rb.getString("primitives_ascii_to_integer"), "prim_a2i",
	rb.getString("primitives_integer_to_ascii"), "prim_i2a",
	rb.getString("primitives_trace"), "prim_trace",
	rb.getString("primitives_untrace"), "prim_untrace",
	rb.getString("primitives_get_string"), "prim_get_string",
	rb.getString("primitives_untrace_all"), "prim_untrace_all",
	rb.getString("primitives_load_sound"), "prim_load_sound",
	rb.getString("primitives_setup_sound"), "prim_setup_sound",
	rb.getString("primitives_play_sound"), "prim_play_sound",
	rb.getString("primitives_stop_sound"), "prim_stop_sound",
	rb.getString("primitives_system_os"), "prim_system_os",
	rb.getString("primitives_system_appletp"), "prim_system_appletp",
	rb.getString("primitives_system_language"), "prim_system_language",
	rb.getString("primitives_system_locale"), "prim_system_locale",
	rb.getString("primitives_system_print"), "prim_system_print",
	rb.getString("primitives_system_type"), "prim_system_type",
    };
    return output;
  }
	
  public String[] infixList() {
    return null;
  }

    public void prePrimCall(Primitive p, Object[] arglist, Context c) {}
    public void postPrimCall(Primitive p, Context c) {}
	
  public static void prim_print(Object v, Context c)  {
    c.out.println(YoYo.printToString(v)); 
  }

  public static void prim_type(Object v, Context c)   {
    c.out.print(YoYo.printToString(v)); 
  }
	
  public static void prim_system_print(Object v, Context c)  {
    System.out.println(YoYo.printToString(v)); 
  }
	
  public static void prim_system_type(Object v, Context c)   {
    System.out.print(YoYo.printToString(v)); 
  }

  public static Object prim_symbolp(Object v, Context c)  {
    return (v instanceof Sym || v instanceof LocalRef) 
      ? YoYo.symtrue : YoYo.symfalse;
  }
	
  public static Object prim_booleanp(Object v, Context c)  {
    return (v == YoYo.symtrue || v == YoYo.symfalse)
      ? YoYo.symtrue : YoYo.symfalse;
		
  }
	
  public static Object prim_delay(Context c) {
    Object o = c.ilistNext();
    if (o instanceof Function) return o;
    if (o instanceof Symbol) return YoYo.getSymbolValue(c, (Symbol)o, c);
    if (o instanceof LocalRef) return c.locals.lookupValue((LocalRef)o);
    if (o instanceof CompoundSymbol) return YoYo.getCompoundValue((CompoundSymbol)o, c);
    return o;
  }
	
  public static Object prim_lambda(Object v0, Object v1, Context c) {
    if (v0 instanceof Object[]) {
      Symbol[] arglist = new Symbol[((Object[])v0).length];
      System.arraycopy((Object[])v0, 0, arglist, 0, arglist.length);
      Object[] body = YoYo.aList(c, v1);
      Symbol[] locals = PrimFile.readlocals(arglist, new Symbol[0], body, 0);
      return new Ufun(arglist, locals, body, YoYo.printToString(body), c.ufunenv);
    }	
    LogoError.error(v0 + LogoError.NOT_LIST, c);			
    return null;		
  }
	
  public static Object prim_unquote(Object v0, Context c) {
    if (v0 instanceof String) 
      return yoyo.Reader.read((String)v0)[0];
    return v0;
  }
	
	
  public static void prim_resett(Context c) {
    c.time=System.currentTimeMillis();
  }
	
  public static Object prim_timer(Context c) {
    return new Double((System.currentTimeMillis() - c.time) / 1000.0);
  }
	
  public static void prim_ignore(Object v0, Context c) {
  }	
	
  public static Object prim_freememory(Context c)  {
    return new Integer((int)Runtime.getRuntime().freeMemory());
  }
	
	
	
	
  public static Object prim_not(Object v0, Context c)  {
    if (YoYo.aBoolean(c, v0))
      return YoYo.symfalse; 
    else return YoYo.symtrue; 
  }
	
  public static void prim_system_quit(Context c)  {
    Runtime.getRuntime().exit(0);
  }
	
	
  public static Object prim_to_string(Object v0, Context c)  {
		
    Object[] list = YoYo.aList(c, v0);
    Object[] oldilist = c.ilist;
    int oldilistposptr = c.ilistposptr;
    StringBuffer out = new StringBuffer();
		
    try {
      c.ilist = list;
      c.ilistposptr = 0;
      while (true) {
	if (c.ilistPeek() == null) break;
	out.append(YoYo.aString(c, YoYo.evalOneFromHere(c)));
      }
      return out.toString();
    }
    finally {
      c.ilist = oldilist;
      c.ilistposptr = oldilistposptr;
    }
  }
	
  public static Object prim_ilistp(Object v0, Context c) {
    if (v0 instanceof Ilist) return YoYo.symtrue; 
    return YoYo.symfalse;
  }
	
  public static Object prim_ilisttolist(Object v0, Context c) {
    if (v0 instanceof Ilist) return ((Ilist)v0).list;
    LogoError.error(v0 + LogoError.NOT_ILIST);
    return null;
  }
	
  public static Object prim_listtoilist(Object v0, Context c) {
    return YoYo.aIlist(c, v0);
  }
	
  public static Object prim_a2i(Object v0, Context c) {
    String s = YoYo.aString(c, v0);
    return new Integer((int)s.charAt(0));
  }
	
  public static Object prim_i2a(Object v0, Context c) {
    int i = YoYo.aInteger(c, v0);
    char[] foo = { (char)i };
    return new String(foo);
  }
	
  public static void prim_trace(Object v0, Context c) {
    if (v0 instanceof Object[]) {
      Object[] list = (Object[])v0;
      for (int i = 0; i < list.length; i++) {
	traceObj(list[i], c);
      }
    }
    else traceObj(v0, c);
  }
	
  static void traceObj(Object v0, Context c) {
    if (v0 instanceof Function) {
      ((Function)v0).trace();
      Vector val = c.trace((Function)v0);
      c.out.println("Tracing " + YoYo.vectorToString(val, false));
      return;
    }
    else if (v0 instanceof String) {
      Sym name = YoYo.aSym(c, yoyo.Reader.read(YoYo.aString(c, v0))[0]);
      Object o = YoYo.lookupSymbol(name, c);
      if (o == null) 
	LogoError.error(LogoError.I_DONT_KNOW_HOW + YoYo.printToString(v0), c);
      if (o instanceof Function) {
	((Function)o).trace();
	Vector val = c.trace((Function)o);
	c.out.println("Tracing " + YoYo.vectorToString(val, false));
	return;
      }
    }
    LogoError.error(YoYo.printToString(v0) + LogoError.NOT_PROCEDURE, c);
  }
	
  public static void prim_untrace(Object v0, Context c) {
    if (v0 instanceof Object[]) {
      Object[] list = (Object[])v0;
      for (int i = 0; i < list.length; i++) {
	untraceObj(list[i], c);
      }
    }
    else untraceObj(v0, c);
  }
	
  public static void untraceObj(Object v0, Context c) {
    if (v0 instanceof Function) {
      ((Function)v0).untrace();
      Vector val = c.untrace((Function)v0);
      c.out.println("Tracing " + YoYo.vectorToString(val, false));
      return;
    }
    else if (v0 instanceof String) {
      Sym name = YoYo.aSym(c, yoyo.Reader.read(YoYo.aString(c, v0))[0]);
      Object o = YoYo.lookupSymbol(name, c);
      if (o == null) 
	LogoError.error(LogoError.I_DONT_KNOW_HOW + YoYo.printToString(v0), c);
      if (o instanceof Function) {
	((Function)o).untrace();
	Vector val = c.untrace((Function)o);
	c.out.println("Tracing " + YoYo.vectorToString(val, false));
	return;
      }
    }
    LogoError.error(YoYo.printToString(v0) + LogoError.NOT_PROCEDURE, c);
  }
	
  public static void prim_untrace_all(Context c) {
    c.untraceAll();
  }
	
  public static Object prim_get_string(Object v0, Context c) {
    if (YoYo.yoyoframe != null) {
      String s = YoYo.aString(c, v0);
      TextDialog td = new TextDialog(YoYo.yoyoframe, "YoYo Input", s);
      String val = td.getResult();
      if (val == null) return YoYo.symfalse;
      return val;
    }
    else {
      Frame f = frameParent(YoYo.applet);
      String s = YoYo.aString(c, v0);
      TextDialog td = new TextDialog(f, "YoYo Input", s);
      String val = td.getResult();
      if (val == null) return YoYo.symfalse;
      return val;
    }
  }
	
  public static Frame frameParent(Applet applet) {
    Container parent = applet.getParent();
    while (parent != null) {
      if (parent instanceof Frame) {
	return (Frame)parent;
      } 
      else parent = parent.getParent();
    }
    return(null);
  }
	
  public static void prim_play_sound(Object v0, Context c) {
    //AudioDataStream ads = (AudioDataStream)v0;
    //AudioPlayer.player.start(ads);
  }
	
  public static void prim_stop_sound(Object v0, Context c) {
    //AudioDataStream ads = (AudioDataStream)v0;
    //AudioPlayer.player.stop(ads);
  }
	
  public static Object prim_setup_sound(Object v0, Context c) {
    //return new AudioDataStream((AudioData)v0);
    return null;
  }
	
  public static Object prim_load_sound(Object v0, Context c) {
    URL u = null;
    try {
      u = new URL(YoYo.aString(c, v0));
      //System.out.println("trying to load url: " + u);
      if (u.getProtocol().equals("file")) {
	String filename = u.getFile();
	File f = new File(filename);
	try {
	  if (!f.canRead()) 
	    LogoError.error(YoYo.printToString(v0) + " is not readable");
	  FileInputStream fir = new FileInputStream(f);
	  //return new AudioData(readStream(fir, (int)f.length(), c));
	  return null;
	}
	catch (FileNotFoundException e) {
	  LogoError.error(e.getMessage(), c);
	}
      }
      else {
	try {
	  InputStream is = null;
	  try {
	    URLConnection uc = u.openConnection();
	    is = uc.getInputStream();
	    //return new AudioData(readStream(is, uc.getContentLength(), c));
	    return null;
	  }
	  catch (Exception e) {
	    System.out.println(e);
	    System.out.println(e.getMessage());
	    LogoError.error(e.getMessage(), c);
	  }
	  finally { is.close(); }
	}
	catch (IOException f) {LogoError.error("error reading url " + u, c);}
      }
    }
    catch (MalformedURLException e) {LogoError.error("url " + YoYo.aString(c, v0) + 
						     " does not exist", c);}
    catch (Exception e) { c.err.println("load sound: uh oh " + e); }
    return null;
  }
	
  static byte[] readStream(InputStream ir, int length, Context c) {
    BufferedInputStream in = null;
    if (length < 0) length = 4096;
    byte[] output = new byte[length];
    int bytesread = 0;
    try {
      in = new BufferedInputStream(ir, length);
      byte[] bytes = new byte[length];
      do {
	int result = in.read(bytes, 0, length);
	if (result < 0) break;
	if (bytesread + result > output.length) {
	  byte[] newresult = new byte[output.length * 2];
	  System.arraycopy(output, 0, newresult, 0, output.length);
	  output = newresult;
	}
	System.arraycopy(bytes, 0, output, bytesread, result);
	bytesread += result;
      } while(in.available() > 0);
			
      if (bytesread <= length) return bytes;
			
      byte[] newoutput = new byte[bytesread];
      System.arraycopy(output, 0, newoutput, 0, bytesread);
      return output;
    }
    catch (IOException e) { 
      LogoError.error(e.getMessage(), c);
    }
    finally { if (in != null) try { in.close(); } catch (IOException f) {} }
    return null;
  }
	
  public static Object prim_system_os(Context c) {
    return System.getProperty("os.name");
  }

    public static Object prim_system_language(Context c) {
	return YoYo.language.getDisplayLanguage();
    }

    public static Object prim_system_locale(Context c) {
	return YoYo.language;
    }

  public static Object prim_system_appletp(Context c) {
    return YoYo.truefalse(YoYo.applet != null);
  }
	
}
