/********************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *******************************************************************/
 package yoyo;

import java.io.StringReader;
import java.util.ArrayList;


public class Reader {
	
  public static Object[] read(String s) {
    Tokenizer tokenizer = new Tokenizer(s);
    ArrayList v = tokenizer.tokenize();
    return readUntil(v, "");
  } 

  public static Object[] readUntil(ArrayList v, String end) {
    ArrayList result = new ArrayList();
    while(!v.isEmpty()) {
      if (peek(v).equals(end)) {
	skip(v);
	break;
      }
      result.add(readTokenOrCompound(v));
    }
    return result.toArray();
  }
	
  public static Object readTokenOrCompound(ArrayList v) {
    Object result = readToken(v);
    if (peek(v).equals(".")) return gatherParts(v, result);
    return result;
  }
		
  public static Object readToken(ArrayList v) {
    if (peek(v).equals("[")) {
      skip(v);
      return readUntil(v, "]");
    }
    if (peek(v).equals("(")) {
      skip(v);
      return new Ilist(readUntil(v, ")"));
    }
    return readString(next(v));
  }

  public static CompoundSymbol gatherParts(ArrayList v, Object first) {
    ArrayList result = new ArrayList();
    result.add(first);
    while(true) {
      if (!peek(v).equals(".")) return new CompoundSymbol(result);
      skip(v);
      result.add(readToken(v));
    }
  }
	
  public static Object readString(String s) {
    if (s.charAt(0) == '\"') { 
      return s.substring(1, s.length()-1);
    }
    if (s.charAt(0) == '-' && s.length() == 1) {
      return Symbol.lookup("-");
    }
    try {
      if (s.charAt(0) == '$' && s.length() > 1) {
	return new Integer(Integer.parseInt(s.substring(1, s.length()), 
					    16));
      }
    }
    catch (Exception e) {}
    if (possibleNumberp(s)) {
      try { return new Integer(s); } catch (Exception e) {}
      try { return new Double(s); } catch (Exception e) {}
    }
    return Symbol.lookup(s); 
  }

  public static boolean possibleNumberp(String s) {
    for (int i = 0; i < s.length(); i++) {
      if ("eE.+-0123456789".indexOf(s.charAt(i)) == -1) return false;
    }
    return true;
  }

  public static String next(ArrayList v) {
    if (v.isEmpty()) return "";
    Object o = v.get(0);
    v.remove(0);
    return (String)o;
  }		

  public static String peek(ArrayList v) {
    if (v.isEmpty()) return "";
    Object o = v.get(0);
    return (String)o;
  }
	
  public static void skip(ArrayList v) {
    v.remove(0);
  }

	
}
	
