/*
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
package yoyo;

import java.awt.*;
import java.awt.event.*;

public class TextDialog extends Dialog implements ActionListener {
	
	TextField text;
	boolean ok;
	
	public TextDialog(Frame parent, String title, String caption) {
		super(parent, title, true);
		setBackground(SystemColor.control);
		Label l = new Label(caption);
		formatLabel(l);
		text = new TextField(50);
		formatText(text);
		text.addKeyListener(new KeyAdapter() {
			public void keyPressed(KeyEvent e) {
				char ch = e.getKeyChar();
				if ((ch == '\n') || (ch == '\r')) {
					ok = true;
					e.consume();
					setVisible(false);
				}
				else if (ch == '\u001B') {
					ok = false;
					e.consume();
					setVisible(false);
				}
				
			}
		});
		
		Panel p = new Panel(new GridLayout(1,2,5,0));
		p.setBackground(SystemColor.control);
		formatLabel(p);
		Button b = new Button("Cancel");
		formatLabel(b);
		b.addActionListener((ActionListener)this);
		p.add(b);
		
		b = new Button("OK");
		formatLabel(b);
		b.addActionListener((ActionListener)this);
		p.add(b);
		
		GridBagLayout gbl = new GridBagLayout();
		setLayout(gbl);
		
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.anchor = GridBagConstraints.WEST;   
		gbc.weightx = 100;
		gbc.weighty = 100;
		gbc.insets = new Insets(0, 13, 0, 13);
		add(l, gbc, 0, 0, 2, 1);
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.insets = new Insets(0, 13, 0, 13);
		add(text, gbc, 0, 1, 2, 1);
		gbc.weightx = 50;
		gbc.fill = GridBagConstraints.NONE;
		gbc.anchor = GridBagConstraints.EAST;
		gbc.insets = new Insets(0, 0, 13, 13);
		add(p, gbc, 1, 2, 1, 1);
		
		setSize(500,140);
		setResizable(false);
		setLocation(100,100);
		
		addWindowListener(new WindowAdapter() { 
			public void windowClosing(WindowEvent evt) {
				ok = false;
				setVisible(false);
			}
		});
	}
	
	public void actionPerformed(ActionEvent evt)
	{
		Button b = (Button)evt.getSource();
		String arg = b.getLabel();
		ok = arg.equals("OK");
		setVisible(false);    
	}
	
	public String getResult() {
		show();
		if (ok)
			return(text.getText());
		else
			return(null);
	}
	
	public void add(Component c, GridBagConstraints gbc, int x, int y, int w, int h) {
		gbc.gridx = x;
		gbc.gridy = y;
		gbc.gridwidth = w;
		gbc.gridheight = h;
		add(c, gbc);
	}
	
	void formatText(TextComponent tc) {
		tc.setFont(new Font("Monospaced", Font.PLAIN, 12));
		tc.setBackground(SystemColor.white);
		tc.setForeground(SystemColor.textText);
	}
	
	void formatLabel(Component c) {
		c.setFont(new Font("Dialog", Font.PLAIN, 12));
		c.setBackground(SystemColor.control);
		c.setForeground(SystemColor.controlText);
	}
}
