/*
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
package yoyo;

import java.io.IOException;
import java.io.PushbackReader;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.Enumeration;

public class Tokenizer {

  StringStream in;
  
  public Tokenizer(String input) {
    in = new StringStream(input);
  }
  
  public ArrayList tokenize() {
    ArrayList v = new ArrayList();
    while(true) {
      if (in.empty()) return v;
      String o = getNextToken();
      //System.out.println("finished getNextToken");
      if (o != null) v.add(o);
    }
  }

  public boolean balancedp(String open, String close) {
    int level = 0;
    while(!in.empty()) {
      String o = getNextToken();
      if (o == null) break;
      //System.out.println("finished getNextToken");
      if (o.equals(open)) level++;
      if (o.equals(close)) level--;
    }
    return (level == 0);
  }

  public char eatWhiteSpace() {
    //System.out.println("eat white space");
    char t = in.read();
    while(true) {
      if (!(whitespace(t) || comment(t))) break;
      while(whitespace(t)) {
	if (in.empty()) return (char)-1;
	t = in.read();
      }
      //System.out.println("look for comment");		
      if (comment(t)) {
	while(true) {
	  //System.out.println("parsing comment");
	  if (in.empty()) return (char)-1;
	  t = in.read();
	  if (t == '\n' || t == '\r') {
	    if (in.empty()) return (char)-1;
	    t = in.peek();
	    if (t == '\n' || t == '\r') continue; //dumbass /n/r and \r\n!
	    t = in.read();
	    break;
	  }
	}
      }
    }
    //System.out.println("done eating whitespace, not comment nor whitespace");
    return t;
  }

  public String getNextToken() {
    //System.out.println("get next token");
    StringBuffer sb = new StringBuffer();
    
    char t = eatWhiteSpace();
    //System.out.println("getNextToken after whitespace: " + (int)t);
    if (t == 65535 || t < 0) return null;
    //System.out.println("didn't return, t !< 0");
    sb.append(t);
    if (isNumber(t)) return getNextNumber(t);
    if (delimiter(t)) return sb.toString();
    if (string(t)) return getNextString();
    
    while(true) {
      if (in.empty()) return sb.toString();
      char p = in.peek();
      if (delimiter(p) || string(p) || 
	  whitespace(p) || comment(p)) return sb.toString();
      char c = in.read();
      if (escapeChar(c)) c = readEscape();
      if (c == 65535 || c < 0) return null;
      sb.append(c);
    }
  }
  
  public String getNextNumber(char item) {
    StringBuffer sb = new StringBuffer();
    sb.append(item);
    while(true) {
      if (in.empty()) break;
      char p = in.peek();
      if (delimiternotdot(p) || string(p) || 
	  whitespace(p) || comment(p)) break;
      char c = in.read();
      sb.append(c);
    }
    return sb.toString();
  }
  
  
  public String getNextString() {
      boolean lispstring = false;
    StringBuffer sb = new StringBuffer();
    sb.append('"');

    if (in.empty()) { sb.append('"'); return sb.toString(); }
    char c = in.read();
    if (c == '"') { sb.append('"'); return sb.toString(); }
    if (c == '|') {
	lispstring = true;
    } else {
	if (escapeChar(c)) c = readEscape();
	if (c == 65535 || c < 0) { sb.append('"'); return sb.toString(); }
	sb.append(c);
    }
    while(true) {
      if (in.empty()) { sb.append('"'); return sb.toString(); }
      c = in.read();
      if (c == '"' || (lispstring && c == '|')) 
	  { sb.append('"'); return sb.toString(); }
      if (escapeChar(c)) c = readEscape();
      if (c == 65535 || c < 0) { sb.append('"'); return sb.toString(); }
      sb.append(c);
    }
  }
  
  public boolean number(char item) {
    return (((item >= '0') && (item <= '9')));
  }
  
  public boolean isNumber(char item) {
    boolean num = number(item);
    if (num) return true;
    char p = in.peek();
    boolean next = (number(p) && (item == '.' || item == '-')); 
    if (next) return true;
    char pp = in.peekpeek();
    return ((item == '-') && (p == '.') && number(pp));
  }
  
  public boolean string(char item) {
    return (item == '"');
  }
  
  public char readEscape() {
    if (in.empty()) return 65535;
    char p = in.read();
    switch(p) {
    case 'n': return '\n';
    case 't': return '\t';
    case 'b': return '\b';
    case 'r': return '\r';
    case 'f': return '\f';
    case '\\': return '\\';
    case '\'': return '\'';
    case '\"': return '\"';
    }
    return 65535;
  }
      

  public boolean escapeChar(char item) {
    return (item == '\\');
  }

  public boolean delimiter(char item) {
    return ((item == '[') || (item == ']') ||
	    (item == '(') || (item == ')') ||
	    (item == ':') || (item == '.'));
  }
  
  public boolean delimiternotdot(char item) {
    return ((item == '[') || (item == ']') ||
	    (item == '(') || (item == ')') ||
	    (item == ':'));
  }
  
  public boolean whitespace(char item) {
    return ((item == ' ') || (item == '\n') || 
	    (item == '\r') || (item == '\t'));
  }
  
  public boolean comment(char item) {
    return (item == ';');
  }
}
