/*
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
package yoyo;

import java.net.URL;
import java.net.MalformedURLException;
import java.io.PrintWriter;


public class TopApplication {
    
  public Context c;
  public static String bootfile = null;
  public static String listtorun = null;
  public static UberContext ub = null;
  public static URL base = null;

  public static void main(String[] args){
    new TopApplication().init(args);
  }

  public void init(String[] args) {
    initArgs(args);
    try { 
      base = new URL("file", "", 
		     System.getProperty((YoYo.pcp) ? "user.home" : "user.dir")
		     +"/");
    }
    catch (MalformedURLException e) {System.out.println(e);}

    ub = new UberContext(base);
    c = new Context(null, base, new PrintWriter(System.out, true), null, ub);

    if (bootfile != null && !bootfile.equals("")) loadBootFile();	
    run();
  }

  public void initArgs(String[] args) {
    if (args.length>0) bootfile = args[0];
    StringBuffer listtorunbuf = new StringBuffer();
    if (args.length>1) listtorunbuf.append(args[1]);
    for (int i=2; i<args.length; i++) {
	listtorunbuf.append(' ');
	listtorunbuf.append(args[i]);
    }
    listtorun = listtorunbuf.toString();
  }

  public void loadBootFile() {
    //c.out.println("Loading Boot File: " + bootfile);
    Object[] ilist = Reader.read("load \"" + bootfile + "\"");
    c.runList(ilist);
    c.waitUntilDone();
  }
    
  public void run() {
    if (listtorun != null && !listtorun.equals("")) {
      //c.out.println("Running " + listtorun);
      Object[] ilist = Reader.read(listtorun);
      c.runList(ilist);
    }
  }

}



 
