/*
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
package yoyo;

import java.awt.event.KeyEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowAdapter;
import java.awt.Font;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Insets;
import java.awt.Panel;
import java.awt.TextArea;
import java.awt.GridLayout;
import java.awt.GraphicsEnvironment;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.util.Hashtable;
import java.util.Vector;
import java.net.URL;
import java.net.MalformedURLException;
import java.awt.event.*;
import java.awt.Toolkit;
import java.awt.MenuBar;
import java.awt.Menu;
import java.awt.MenuItem;
import java.awt.MenuShortcut;
import java.awt.CheckboxMenuItem;
import java.awt.Toolkit;
import java.awt.datatransfer.*;

public class Toplevel extends Frame implements ActionListener, ItemListener, ClipboardOwner {

  static String[] sizes = { "6", "8", "9", "10", "11", "12", "14", "16", "18", 
			    "24", "36", "48", "72" };

    public static String bootfile;
    public static String bootpath;
    static Clipboard clip = Toolkit.getDefaultToolkit().getSystemClipboard();
    public static UberContext ub;
    public static URL base = null;

  public Context c;
  public Listener cc;
  CheckboxMenuItem currentfont;
  CheckboxMenuItem currentsize;
  public boolean firstone;

    public Toplevel(String s, boolean firstone) {
	super(s);
	YoYo.yoyoframe = this;
	this.firstone = firstone;
	if (firstone) appInit();
	windowInit();
	String title = c.toString();
	if (!firstone) setTitle(title.substring(1, title.length()-1));
	//System.out.println("bootfile: " + bootfile);
	loadBootFile();
    }
    
    public void loadBootFile() {
	if (firstone && bootfile != null) {
	    cc.runLine("load \"file:" + quote(bootfile) + "\"");
	    c.waitUntilDone();
	    if (bootpath != null) {
		cc.runLine("cd \"file:" + quote(bootpath) + "\"");
		c.waitUntilDone();
	    }
	    runStartup();
	}
    }

    public void runStartup() {
	Object val= null;
	if ((val = c.getGlobalObj().lookupValue(Symbol.lookup("startup"))) != null) {
	    if (val instanceof Function) {
		cc.runLine("startup");
	    }
	}
    }

    public static String quote(String path) {
	StringBuffer sb = new StringBuffer(path.length());
	for(int i =0 ; i < path.length(); i++) {
	    char c = path.charAt(i);
	    if (c == '\\')  sb.append("\\\\");
	    else sb.append(c);
	}
	return sb.toString();
    }
   
    public static void main(String[] args){
	readArgs(args);
	Toplevel tl = new Toplevel("YoYo", true);
  }

    public static void readArgs(String[] args) {
	if (args.length>1) {
	    bootpath = args[0].trim();
	    if (bootpath.equals("")) bootpath = null;
	    //System.out.println("bootpath = " + bootpath);
	    bootfile = args[1].trim();
	    if (bootfile.equals("")) bootfile = null;
	    //System.out.println("bootfile = " + bootfile);
	} else if (args.length > 0) {
	    bootfile = args[0].trim();
	    if (bootfile.equals("")) bootfile = null;
	    //System.out.println("bootfile = " + bootfile);
	}
    }

  void appInit() {
    try { 
      base = new URL("file", "", 
		     System.getProperty((YoYo.pcp) ? "user.home" : "user.dir")
		     +"/");
    }
    catch (MalformedURLException e) {System.out.println(e);}
    ub = new UberContext(base);
  }
  
    public void windowInit() {
      createComponents();
      addComponentsToLayout();
      addListeners();
      setupMenus();
      cc.requestFocus();
      sendWelcome();
  }


    public void createComponents() {
	//System.out.println("about to make Context");
	//setLayout(new GridLayout(1,2));
	cc=new Listener();
	c = new Context(null, base, new PrintWriter(new NTEStream(cc), true), cc,
			ub);
	cc.c = c;
    }

    public void addComponentsToLayout() {
	add(cc);
	pack();
	show();
    }

    public void addListeners() {
	addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent e) {
		    c.die();
		    e.getWindow().dispose();
                    System.exit(0);
		}
	    });
	
	cc.addKeyListener(new KeyAdapter() {
		public void keyPressed(KeyEvent e) {
		    Listener cc = (Listener) e.getComponent();
		    char key = e.getKeyChar();
		    
		    if (cc.c.runningnowp) {
			cc.c.stop();
			e.consume(); 
			return;
		    }
		    
		    if(key== '\n' || key =='\r') {
			if (!(e.isShiftDown())) {
			    cc.handlecr(); 
			    e.consume(); 
			}
		    }
		    if(key == '\t') {
			cc.c.out.print("   ");
			cc.c.out.flush();
			e.consume();
		    }
		}
		
		public void keyTyped(KeyEvent e) {
		    char c = e.getKeyChar();
		    if (c=='\n' || c == '\r') {
			if (!e.isShiftDown()) e.consume();
		    } else if (c=='\t') e.consume();
		}
	    });
    }

    public void sendWelcome() {
	c.out.println("Welcome to YoYo!");
    }

   void setupMenus() {
    MenuBar mb = new MenuBar();	
    // set up menu bar
	
    Menu filemenu = new Menu("File");
    Menu editmenu = new Menu("Edit");
    Menu fontmenu = new Menu("Font");
    Menu sizemenu = new Menu("Size");

    setupFileMenu(filemenu);
    mb.add(filemenu);
    setupEditMenu(editmenu);
    mb.add(editmenu);
    setupFontMenu(fontmenu);
    mb.add(fontmenu);
    setupSizeMenu(sizemenu);
    mb.add(sizemenu);
    setMenuBar(mb);
   }

    void setupFileMenu(Menu filemenu) {
	// file menu
	MenuItem mi = new MenuItem("New", new MenuShortcut('N'));
	mi.addActionListener((ActionListener)this);
	filemenu.add(mi);
	mi = new MenuItem("Open...", new MenuShortcut('O'));
	mi.addActionListener((ActionListener)this);
	filemenu.add(mi);
	mi = new MenuItem("Open URL...");
	mi.addActionListener((ActionListener)this);
	filemenu.add(mi);
	filemenu.addSeparator();
	mi = new MenuItem("New Listener");
	mi.addActionListener((ActionListener)this);
	filemenu.add(mi);
	mi = new MenuItem("Close Listener", new MenuShortcut('W'));
	mi.addActionListener((ActionListener)this);
	filemenu.add(mi);
	if (!YoYo.macosxp) {
	    filemenu.addSeparator();
	    mi = new MenuItem("Quit", new MenuShortcut('Q'));
	    mi.addActionListener((ActionListener)this);
	    filemenu.add(mi);
	}
    }

    void setupEditMenu(Menu editmenu) {
	// edit menu
	MenuItem mi = new MenuItem("Undo", new MenuShortcut('Z'));
	mi.addActionListener((ActionListener)this);
	editmenu.add(mi);
	mi = new MenuItem("Cut", new MenuShortcut('X'));
	mi.addActionListener((ActionListener)this);
	editmenu.add(mi);
	mi = new MenuItem("Copy", new MenuShortcut('C'));
	mi.addActionListener((ActionListener)this);
	editmenu.add(mi);
	mi = new MenuItem("Paste", new MenuShortcut('V'));
	mi.addActionListener((ActionListener)this);
	editmenu.add(mi);
	editmenu.addSeparator();
	mi = new MenuItem("Select All", new MenuShortcut('A'));
	mi.addActionListener((ActionListener)this);
	editmenu.add(mi);
    }

    void setupFontMenu(Menu fontmenu) {
	// fontmenu
	CheckboxMenuItem cmi;
	String[] fonts = GraphicsEnvironment.getLocalGraphicsEnvironment().getAvailableFontFamilyNames();
	for (int i = 0; i< fonts.length; i++) {
	    cmi = new CheckboxMenuItem(fonts[i]);
	    cmi.addItemListener((ItemListener)this);
	    cmi.setFont(new Font(fonts[i], Font.PLAIN, 11));
	    if (fonts[i].equals("Monospaced")) {
		cmi.setState(true);
		currentfont = cmi;
	    }
	    fontmenu.add(cmi);
	}
    }

    void setupSizeMenu(Menu sizemenu) {
	CheckboxMenuItem cmi;
	for (int i = 0; i< sizes.length; i++) {
	    cmi = new CheckboxMenuItem(sizes[i]);
	    cmi.addItemListener((ItemListener)this);
	    if (sizes[i].equals("12")) {
		cmi.setState(true);
		currentsize = cmi;
	    }
	    sizemenu.add(cmi);
	}
    }
  
  public void actionPerformed(ActionEvent evt) {
    MenuItem mi = (MenuItem)evt.getSource();
    String arg = mi.getLabel();
    if (arg.equals("New Listener")) {
      new Toplevel("YoYo", false);
      return;
    }
    if (arg.equals("Close Listener")) {
      c.die();
      dispose();
    }
    if (arg.equals("Quit")) {
      System.exit(0);
      return;
    }
    if (arg.equals("Cut")) {
      String foo = cc.getSelectedText();
      if(foo.equals("")) return;
      StringSelection bar = new StringSelection(foo);
      clip.setContents(bar, this);
      int start = cc.getSelectionStart();
      cc.replaceRange("", start, cc.getSelectionEnd());      
      cc.select(start, start);
      return;
    }
    if (arg.equals("Copy")) {
      String foo = cc.getSelectedText();
      if(foo.equals("")) return;
      StringSelection bar = new StringSelection(foo);
      clip.setContents(bar, this);      
      return;
    }
    if (arg.equals("Paste")) {
      Transferable trans = clip.getContents(this);
      if (trans != null) {
	try {
	  Object thing = trans.getTransferData(DataFlavor.stringFlavor);
	  if (thing instanceof String) {
	    String s = (String)thing;
	    int len = s.length();
	    int cur = cc.getCaretPosition();
	    if (len == 0) {
	      cc.insert(s, cur);
	      cc.select(cur + len, cur + len);
	    }
	    else { 
	      int start = cc.getSelectionStart();
	      cc.replaceRange(s, start, cc.getSelectionEnd());      
	      cc.select(start + len, start + len);
	    }
	  }
	}
	catch (IOException e) {}
	catch (UnsupportedFlavorException e) {}
      }
      return;
    }
    if (arg.equals("Select All")) {
      cc.selectAll();
      return;
    }
    
  }
  
  public void itemStateChanged(ItemEvent evt) {
    CheckboxMenuItem cmi = (CheckboxMenuItem)evt.getSource();
    String arg = cmi.getLabel();
    
    try {
      int value = Integer.parseInt(arg);
      Font current = cc.getFont();
      cc.setFont(new Font(current.getName(), current.getStyle(), value));
      currentsize.setState(false);
      cmi.setState(true);
      currentsize = cmi;
      return;
    }
    catch (Exception e) {
      Font current = cc.getFont();
      cc.setFont(new Font(arg, current.getStyle(), current.getSize()));
      currentfont.setState(false);
      cmi.setState(true);
      currentfont = cmi;
      return;
    }
  }

  public void lostOwnership(Clipboard clip, Transferable t) {}

}


