/*
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
package yoyo;

import java.util.Vector;
import java.util.Enumeration;
import java.net.URL;
import java.net.MalformedURLException;
import java.io.PrintWriter;

final public class UberContext {

	// this class keeps track of all users connected to YoYo.
	// basically, all things that would be static are instance variables here.
	// there is one ubercontext per user. static ubercontext variables are 
	// shared among all users.
	
    static Vector userVector = new Vector();
    static YoYoObject communicationObject = new YoYoObject();
    
  Vector contexts = new Vector();
  int numcontexts = 0;
  int contextnumber = 0;
  Context mainc;
    public URL base;

    YoYoObject globalobj;
    int id;
    static int ids = 0;
	
    public UberContext(URL base) {
	this.id = ids++;
	this.base = base;
	userVector.addElement(this);
	globalobj = new YoYoObject();
	YoYo.initConstants(globalobj);
	PrimStartup.initClasses(globalobj);
	try {
	  URL u = YoYo.class.getResource("startup.yoyo");
	  if (u == null) {
	  	System.out.println("System Error: startup.yoyo not found");
	  }
	  PrimFile.load(u.toString(), false, 
			new Context(null, base, new PrintWriter(System.out, true), 
				    null, this));
	}
	//catch (MalformedURLException f) {
	//  System.out.println("System Error: base URL incorrect: " + base);
	//}
	//catch (ClassNotFoundException g) {
	//	System.out.println("System Error: startup.yoyo not found");
	//}
	catch (Exception e) { 
	  System.out.println("startup.yoyo not found");
	  e.printStackTrace();
	}
	globalobj.defineValue(Symbol.lookup("shared-environment"),
			      communicationObject);
	
    }
    
    public void addContext(Context c) {
	if (mainc == null) mainc = c;
	contexts.addElement(c);
	c.contextnum = contextnumber++;
	numcontexts++;
    }
    
    public void removeContext(Context c) {
	contexts.removeElement(c);
	numcontexts--;
	if (numcontexts == 0) mainc = null;
    }
    
    public int numContexts() {
	return numcontexts;
    }

    public static void logoutUser(int id) {
	for(Enumeration e = userVector.elements(); e.hasMoreElements(); ) {
	    UberContext ub = (UberContext)e.nextElement();
	    if (ub.id == id) {
		ub.logoutContexts();
		userVector.removeElement(ub);
		return;
	    }
	}
    }
    
    public static UberContext getUberContext(int id) {
	for(Enumeration e = userVector.elements(); e.hasMoreElements(); ) {
	    UberContext ub = (UberContext)e.nextElement();
	    if (ub.id == id) {
		return ub;
	    }
	}
	return null;
    }
    
    public void logoutUser() {
	logoutContexts();
	userVector.removeElement(this);
    }
    
    private void logoutContexts() {
	for(Enumeration e = contexts.elements(); e.hasMoreElements(); ) {
	    Context c = (Context)e.nextElement();
	    c.stop();
	}
	globalobj = null;
    }		
		
	
}
