/*
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
package yoyo;

import java.util.Vector;
import java.util.Enumeration;

public class Ufun implements Function {

  Symbol name;
    public Symbol[] arglist;
  int nargs;
    public String bodyString;
  Object[] body;
  YoYoObject ufunenv;
  Vector linenos;
  static Object[] emptyarglist = new Object[0];
  

  boolean trace = false;
  static Vector tracedFuncs = new Vector();

  static Symbol anon = Symbol.lookup("anonymous");
  
  public Ufun(Symbol[] arglist, Symbol[] locals, Object[] body, String bodyString, YoYoObject ufun) {
    this(null, arglist, locals, body, bodyString, ufun);
  }
  
  public Ufun(Symbol name, Symbol[] arglist, Symbol[] locals, 
	      Object[] body, String bodyString, YoYoObject ufun) {
    this.body=body; this.name = name;
    this.bodyString = bodyString;
    this.nargs = arglist.length;
    this.arglist = new Symbol[arglist.length + locals.length];
    System.arraycopy(arglist, 0, this.arglist, 0, arglist.length);
    System.arraycopy(locals, 0, this.arglist, arglist.length, locals.length);
    ufunenv = ufun;
  }

  public Symbol getName() {
    return name;
  }

  public int getNargs() {
    return nargs;
  }
    
  public Object[] getArgArray(Context c) {
    if (arglist.length == 0) return emptyarglist;
    else return Pool.getArray(arglist.length, c);
  }

  public Object run(Object[] inputargs, Context c) {
    Object[] oldilist = c.ilist;
    int oldip = c.ilistposptr;
    Locals oldlocals = c.locals;
    YoYoObject oldufunenv = c.ufunenv;
    Symbol oldufun = c.ufun;
    Ufun oldufunobj = c.ufunobj;
    Vector oldsavedynamics = c.savedynamics;
    
    try {
      c.locals = new Locals(arglist, inputargs);
      c.ufunenv = ufunenv;
      c.ufun = (name == null) ? anon : name;
      if (trace) c.traceEnter(c.ufun.name, inputargs);
      c.ufunobj = this;
      c.savedynamics = null;
      YoYo.evalList(body, c);
      //System.out.println("return from ufun call " + c.ufunreturn);
      Object value = c.ufunreturn;
      if (trace) c.traceExit((name == null) ? anon.name : name.name, value);
      if (value == Context.novalue) return null;
      return value;
    }
    catch (LogoError u) {
      if (trace) c.traceThrow(c.ufun.name, u);
      throw u;
    }
    finally {
      //if (c.locals.names.length > arglist.length) 
      //  arglist = c.locals.names; // update names of vars from lets.
      if (inputargs != emptyarglist) Pool.dropArray(inputargs, c);
      c.locals = oldlocals;
      c.ufunenv = oldufunenv;
      c.ufunobj = oldufunobj;
      c.ufun = oldufun;
      if (c.savedynamics!=null) 
	restoreDynamics(c.savedynamics, c.objenv);
      c.savedynamics = oldsavedynamics;
      c.ilist = oldilist;
      c.ilistposptr = oldip;
      c.ufunreturn = null;
    }
  }
    
    void restoreDynamics(Vector bindings, YoYoObject obj){
	Enumeration e = bindings.elements();
	while(e.hasMoreElements()) {
	    Symbol name = (Symbol)e.nextElement();
	    Object value = e.nextElement(); 
	    if (value != null) {
		obj.setValue(name, value);
	    }
	    else obj.undefineValue(name);
	}
    }
	
  public String toShortString() {
      return "<" + ((name == null) ? "lambda " : (name + ": lambda ")) +
	  YoYo.printToString(arglist) + ">";
  }

    public String toString() {
    return "<" + ((name == null) ? "lambda " : (name + ": lambda ")) +
      YoYo.printToString(arglist) + " " + 
      YoYo.printToString(body) + ">";
    }

  public void trace() {
    trace = true;
  }

  public void untrace() {
    trace = false;
  }
    
}
