/*
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
package yoyo;

//import quicktime.*;
import java.lang.reflect.Method;
import java.lang.reflect.Field;
import java.util.Vector;
import java.util.Enumeration;
import java.net.URL;
import java.net.MalformedURLException;
import java.io.PrintWriter;
import java.applet.Applet;
import java.awt.Frame;
import java.util.ResourceBundle;
import java.util.Locale;

public class  YoYo {

    public static Symbol symtrue = Symbol.lookup("true");
    public static Symbol symfalse = Symbol.lookup("false");

    public static Applet applet = null;
    public static Frame yoyoframe = null;

    public static boolean macintoshp = false;
    public static boolean macosxp = false;
    public static boolean microsoftvmp = false;
    public static boolean solarisp = false;
    public static boolean linuxp = false;
    public static boolean unixp = false;
    public static boolean netscapep = false;
    public static boolean pcp = false;
    public static boolean winxpp = false;
    public static boolean winmep = false;
    public static boolean win98p = false;
    public static boolean win95p = false;
    public static boolean win2kp = false;
    public static boolean winntp = false;

    public static boolean swingavailablep = false;

    public static Locale language = new Locale("en", Locale.getDefault().getCountry());

    static {
	String jv = System.getProperty("java.vendor");
	String os = System.getProperty("os.name");
	String ver = System.getProperty("os.version");
	if (os != null) {
	    if (os.startsWith("Windows")) {
		pcp = true;
		if (os.startsWith("Windows 95")) {
		    win95p = true;
		} else if (os.startsWith("Windows 98")) {
		    win98p = true;
		} else if (os.startsWith("Windows Me")) {
		    winmep = true;
		} else if (os.startsWith("Windows NT")) {
		    winntp = true;
		} else if (os.startsWith("Windows 2000")) {
		    if (ver != null && ver.equals("5.0")) {
			win2kp = true;
		    } else {
			winxpp = true;
		    }
		} else if (os.startsWith("Windows XP")) {
		    winxpp = true;
		} 
		if (jv != null) {
		    if (jv.startsWith("Microsoft")) {
			microsoftvmp = true;
		    } else if (jv.startsWith("Netscape")) {
			netscapep = true;
		    }
		}
	    } else if (os.startsWith("Linux")) {
		linuxp = true;
		unixp = true;
	    } else if (os.startsWith("Solaris") || os.startsWith("SunOS")) {
		solarisp = true;
		unixp = true;
	    } else if (os.startsWith("Mac")) {
		macintoshp = true;
		if (os.startsWith("Mac OS X")) {
		    macosxp = true;
		}
	    } else {
		unixp = true;
	    }
	} else {
	    unixp = true;
	}
	try {
	    Class.forName("javax.swing.JPanel").newInstance();
	    if (!YoYo.macosxp) javax.swing.UIManager.setLookAndFeel(javax.swing.UIManager.getSystemLookAndFeelClassName());
	    swingavailablep = true;
	} catch (Exception e) { }
    }

    public static void initConstants(YoYoObject globals) {
	globals.defineValue(Symbol.lookup("pi"), new Double(Math.PI));
	globals.defineValue(Symbol.lookup("min-int"), new Integer(Integer.MIN_VALUE));
	globals.defineValue(Symbol.lookup("max-int"), new Integer(Integer.MAX_VALUE));
	globals.defineValue(Symbol.lookup("min-num"), new Double(Double.MIN_VALUE));
	globals.defineValue(Symbol.lookup("max-num"), new Double(Double.MAX_VALUE));
	globals.defineValue(YoYo.symtrue, YoYo.symtrue);
	globals.defineValue(YoYo.symfalse, YoYo.symfalse);
	globals.defineValue(Symbol.lookup("system-version"), new Double(3.0));
    }

    public static void evalExternal(Context c) {
	Symbol[] newlocals = PrimFile.readlocals(new Symbol[0], new Symbol[0], c.ilist, 0);
	try {
	    c.locals.defineLocals(newlocals);
	    while(true) {
		Object obj = (c.ilistposptr < c.ilist.length
			      ? c.ilist[c.ilistposptr++] : null);
		if (obj == null) return;
		eval(obj, c);
	    }
	}
	catch (LogoError le) {
	    c.err.println(le.toString());
	}
	catch (Stop se) {
	    c.stopnow = false;
	}
	finally {
	    c.locals.undefineLocals();
	}
    }

    public static Object evalList(Ilist i, Context c) {
	int oldilistposptr = c.ilistposptr;
	Object[] oldilist = c.ilist;
	int oldprec = c.precedence;
	try {
	    // reset precedence to 0
	    c.precedence = 0;
	    c.ilistposptr = 0;
	    c.ilist = i.list;
	    return evalFromHere(c);
	}
	finally {
	    c.ilistposptr = oldilistposptr;
	    c.ilist = oldilist;
	    c.precedence = oldprec;
	    if (c.ufunreturn != null) c.ilistSkipToEnd();
	}
    }

    public static Object evalList(Object[] o, Context c) {
	//System.out.println("evalList: " + YoYo.printToString(o));
	int oldilistposptr = c.ilistposptr;
	Object[] oldilist = c.ilist;
	int oldprec = c.precedence;
	try {
	    c.ilistposptr = 0;
	    c.ilist = o;
	    c.precedence = 0;
	    return evalFromHere(c);
	}
	finally {
	    c.ilistposptr = oldilistposptr;
	    c.ilist = oldilist;
	    c.precedence = oldprec;
	    if (c.ufunreturn != null) c.ilistSkipToEnd();
	}
    }

    public static Object evalFromHere(Context c) {
	Object output = null;
	try {
	    while(true) {
		Object obj = (c.ilistposptr < c.ilist.length
			      ? c.ilist[c.ilistposptr++] : null);
		if (obj == null) return output;
		//System.out.println("evalFromHere: " + YoYo.printToString(obj));
		output = eval(obj, c);
	    }
	}
	finally {
	    if (c.ufunreturn != null) c.ilistSkipToEnd();
	}
    }

    public static Object evalOne(Ilist i, Context c) {
	//System.out.println("evalOne " + i.toString());
	Object[] oldilist = c.ilist;
	int oldilistposptr = c.ilistposptr;
	try {
	    c.ilist = i.list;
	    c.ilistposptr = 0;
	    return evalOneFromHere(c);
	}
	finally {
	    c.ilist = oldilist;
	    c.ilistposptr = oldilistposptr;
	}
    }

    public static Object evalOne(Object[] o, Context c) {
	//System.out.println("evalOne " + i.toString());
	Object[] oldilist = c.ilist;
	int oldilistposptr = c.ilistposptr;
	try {
	    c.ilist = o;
	    c.ilistposptr = 0;
	    return evalOneFromHere(c);
	}
	finally {
	    c.ilist = oldilist;
	    c.ilistposptr = oldilistposptr;
	}
    }

    public static Object evalOneFromHere(Context c) {
	Object output = null;
	int oldnargs = c.nargs;
	try {
	    c.nargs = 1;
	    Object obj = (c.ilistposptr < c.ilist.length
			  ? c.ilist[c.ilistposptr++] : null);
	    // if the list is empty, we're done
	    if (obj == null) return null;
	    return eval(obj, c);
	}
	finally { c.nargs = oldnargs; }
    }

    public static void pollStop(Context c) {
	if (c.stopnow) { throw new Stop(); }
    }

    public static Object eval(Object o, Context c) {
	//System.out.println("eval " + o.toString());
	if (c.stopnow) { throw new Stop(); }
	Object output = o;
	if (o instanceof Primitive) {
	    Primitive p = (Primitive)o;
	    output = evalFunction(p.symname, p, c.objenv, c);
	    if (output != null && c.nargs == 0) {
		LogoError.error(LogoError.TOO_MANY_REPORTERS +
				printToString(output), c);
	    }
	} else if (o instanceof LocalRef) {
	    output = c.locals.lookupValue((LocalRef)o);
	    if (output instanceof Function) {
		//System.out.println(((LocalRef)o).name.toString() + " is a function!");
		output = evalFunction(((LocalRef)o).name, (Function)output, c.objenv, c);
		//System.out.println(((LocalRef)o).name.toString() + " returned " + output);
		if (output != null && c.nargs == 0) {
		    LogoError.error(LogoError.TOO_MANY_REPORTERS +
				    printToString(output), c);
		}
	    } else if (output == null) {
		LogoError.error(LogoError.I_DONT_KNOW_HOW + o, c);
	    } 
	} else if (o instanceof Sym) {
	    output = evalSymbol((Sym)o, c);
	}
	else if (o instanceof Ilist) {
	    output = evalList((Ilist)o, c);
	}
	//System.out.println("output from eval is " + YoYo.printToString(output));
	if (output != null) return rtv(output, c);
	return null;
    }


    static Object rtv(Object o, Context c) {
	//System.out.println("rtv");
	if (o==null) return null;
	//System.out.println("rtv: " + YoYo.printToString(o));
	if (c.nargs==0)
	    LogoError.error(LogoError.TOO_MANY_REPORTERS +
			    printToString(o), c);

	Object moo = (c.ilistposptr < c.ilist.length ? c.ilist[c.ilistposptr]
		      : null);
	if (moo == null) return o;
	Primitive foo = null;
	if (moo instanceof Symbol) {
	    foo = ((Symbol)moo).primitive;
	    if (foo == null) return o;
	} else if (moo instanceof Primitive) {
	    foo = (Primitive)moo;
	} else return o;
	//System.out.println("rtv foo = " + foo.getName());
	//c.getGlobalObj().lookupValue((Symbol)moo);
	int args = foo.getNargs();
	//System.out.println("args = " + args + " c.prec = " + c.precedence);	
	if (args < 0 && args < c.precedence) {
	    //System.out.println("args < 0 and args < c.precedence");
	    c.ilistposptr++;
	    c.setPrev(foo);
	    //System.out.println("call infix call.");
	    return infixCall(args, o, foo, c);
	}
	return o;
    }

    public static Object lookupSymbol(Sym s, Context c) {
	//System.out.println("lookup Sym: " + s.toString());
	Object o = null;

	if (s instanceof Symbol) {
	    o = getSymbolValue(c, (Symbol)s, c);
	}
	else if (s instanceof CompoundSymbol) {
	    Closure cs = evalCompound((CompoundSymbol)s, c);
	    if (cs.object instanceof YoYoObject) {
		o = evalLast((YoYoObject)cs.object, cs.last, c);
	    } else {
		o = evalLast(cs.object, cs.last, c);
	    }
	}
	return o;
    }

    public static Object evalSymbol(Sym s, Context c) {
	//System.out.println("eval Sym: " + s.toString());
	Object o = null;
	YoYoObject env = c.objenv;

	if (s instanceof Symbol) {
	    o = getSymbolValue(c, (Symbol)s, c);
	}
	else if (s instanceof CompoundSymbol) {
	    Closure cs = evalCompound((CompoundSymbol)s, c);
	    if (cs.object instanceof YoYoObject) {
		o = evalLast((YoYoObject)cs.object, cs.last, c);
		env = (YoYoObject)cs.object;
	    } else {
		o = evalLast(cs.object, cs.last, c);
	    }		
	}

	if (o == null) {
	    LogoError.error(LogoError.I_DONT_KNOW_HOW + s, c);
	}

	if (o instanceof Function) {
	    //System.out.println(s.toString() + " is a function!");
	    Object output = evalFunction((Sym)s, (Function)o, env, c);
	    //System.out.println(s.toString() + " returned " + output);
	    if (output != null) {
		if (c.nargs == 0) {
		    LogoError.error(LogoError.TOO_MANY_REPORTERS +
				    printToString(output), c);
		}
		return output;
	    }
	    return null;
	}
	return o;
    }

    public static Object getSymbolValue(ExprList el, Symbol s, Context c) {
	//System.out.println("get symbol value: " + s.toString());
	// Look for primitive?
	//if (s.primitive != null) return s.primitive;

	// Look in locals
	/*
	  Object output = c.locals.lookupValue(s);
	  if (output != null) {
	  el.setPrev(new LocalRef(c.locals.index(s), s));
	  return output;
	  }
	*/
	//System.out.println("not in locals.");

	// Maybe it's a primitive
	Object output = s.primitive;
	if (output != null) {
	    el.setPrev(output);
	    return output;
	}
	// Look in current object env
	output = c.objenv.lookupValue(s);
	if (output != null) return output;

	//System.out.println("not in object env.");
	// Look in current ufun env
	if (c.objenv != c.ufunenv) {
	    output = c.ufunenv.lookupValue(s);
	    if (output != null) return output;
	}
	//System.out.println("not in ufun env.");

	// Look in global environment
	//System.out.println("global env is "  + c.getGlobalObj());
	output = c.getGlobalObj().lookupValue(s);
	//System.out.println("eval Symbol global lookup = " + output);
	return output;
    }

    public static Object getJavaValue(Object o, Symbol s, Context c) {
	//System.out.println("get java value: " + YoYo.printToString(o) + " sym: " + s);
	Class clazz = o.getClass();
	String name = s.name;
	// is it a field?
	try {
	    Field f = clazz.getField(name);
	    try {
		return f.get(o);
	    }
	    catch (IllegalAccessException iae) { 
		LogoError.error("you forgot to make " + f.toString() +
				"public", c); 
	    }
	} catch (NoSuchFieldException nsfe) {}
	Method[] methods = clazz.getMethods();
	for (int i = 0; i < methods.length; i++) {
	    if (methods[i].getName().equalsIgnoreCase(name)) {
		return methods[i];
	    }
	}
	return null;
    }

    public static Object evalPart(CompoundSymbol s, YoYoObject y, Object p, Context c) {
	//System.out.println("eval Part yoyoobj: " + y + " sym: " + YoYo.printToString(p));
	if (p instanceof LocalRef) {
	    Object value = c.locals.lookupValue((LocalRef)p);
	    if (value instanceof Function) {
		if (((Function)value).getNargs() == 0) {
		    return evalFunction(((LocalRef)p).name, (Function)value, c.objenv, c);
		}
		else LogoError.error(LogoError.TOO_FEW_REPORTERS + ((LocalRef)p).name, c);
	    }
	    return value;
	}
	if (p instanceof Ilist) p = evalOne((Ilist)p, c);
	if (p instanceof YoYoObject) return p;
	if (p instanceof String) p = Symbol.lookup((String)p);
	if (p instanceof Symbol) {
	    Object value = (y==null) ? getSymbolValue(s, (Symbol)p, c)
		: y.lookupValue((Symbol)p);
	    if (value instanceof Function) {
		if (((Function)value).getNargs() == 0) {
		    return evalFunction((Sym)p, (Function)value, c.objenv, c);
		}
		else LogoError.error(LogoError.TOO_FEW_REPORTERS + p, c);
	    }
	    return value;
	}
	LogoError.error(LogoError.I_DONT_KNOW_WHAT + printToString(p), c);
	return null;
    }
    
    // more general than the above version. 
    // Used for arbitrary java objects.
    public static Object evalPart(CompoundSymbol s, Object y, Object p, Context c) {
	//System.out.println("eval Part obj: " + y + " sym: " + YoYo.printToString(p));
	if (p instanceof LocalRef) {
	    Object value = c.locals.lookupValue((LocalRef)p);
	    if (value instanceof Function) {
		if (((Function)value).getNargs() == 0) {
		    return evalFunction(((LocalRef)p).name, (Function)value, c.objenv, c);
		}
		else LogoError.error(LogoError.TOO_FEW_REPORTERS + ((LocalRef)p).name, c);
	    }
	    return value;
	}
	if (p instanceof Ilist) p = evalOne((Ilist)p, c);
	if (p instanceof YoYoObject) return p;
	if (p instanceof String) p = Symbol.lookup((String)p);
	if (p instanceof Symbol) {
	    Object value = (y==null) ? getSymbolValue(s, (Symbol)p, c)
		: getJavaValue(y, (Symbol)p, c);
	    if (value instanceof Method) 
		value = new Primitive(((Symbol)p).name, (Method)value, y);
	    if (value instanceof Function) {
		if (((Function)value).getNargs() == 0) {
		    return evalFunction((Sym)p, (Function)value, c.objenv, c);
		}
		else LogoError.error(LogoError.TOO_FEW_REPORTERS + p, c);
	    }
	    return value;
	}
	LogoError.error(LogoError.I_DONT_KNOW_WHAT + printToString(p), c);
	return null;
    }

    public static Object evalLast(YoYoObject y, Object p, Context c) {
	Object output = null;
	if (p instanceof String) p = Symbol.lookup((String)p);
	if (p instanceof Symbol) output = y.lookupValue((Symbol)p);
	else if (p instanceof Ilist) {
	    output = y.lookupValue(YoYo.aSymbol(c, evalOne((Ilist)p, c)));
	}
	if (output == null) return YoYo.symfalse;
	return output;
    }

    public static Object evalLast(Object y, Object p, Context c) {
	Object output = null;
	if (p instanceof String) p = Symbol.lookup((String)p);
	if (p instanceof Symbol) {
	    output = getJavaValue(y, (Symbol)p, c);
	    if (output instanceof Method) {
		output = new Primitive(((Symbol)p).name, (Method)output, y);
	    }
	}
	else if (p instanceof Ilist) {
	    output = getJavaValue(y, YoYo.aSymbol(c, evalOne((Ilist)p, c)), c);
	    if (output instanceof Method) {
		output = new Primitive(((Symbol)p).name, (Method)output, y);
	    }
	}
	if (output == null) return YoYo.symfalse;
	return output;
    }

    public static Object getCompoundValue(CompoundSymbol s, Context c) {
	Closure cs = evalCompound(s, c);
	//System.out.println("cs.object is " + cs.object);
	if (cs.object instanceof YoYoObject) {
	    return evalLast((YoYoObject)cs.object, cs.last, c);
	} else {
	    return evalLast(cs.object, cs.last, c);
	}
    }

    public static Closure evalCompound(CompoundSymbol s, Context c) {
	Object[] parts = s.parts;
	Object first = null;

	for(int i = 0; i < parts.length-1; i++) {
	    Object next = null;
	    //System.out.println("first is " + first);
	    if (first == null || first instanceof YoYoObject) {
		next = evalPart(s, (YoYoObject)first, parts[i], c);
	    } else {
		next = evalPart(s, first, parts[i], c);
	    }
	    if (next == null)
		LogoError.error(LogoError.I_DONT_KNOW_HOW +
				parts[i].toString(), c);
	    if (!(next instanceof YoYoObject)) {
		// this is ok, as long as next is a java object
		//LogoError.error("Compound Part " + parts[i].toString() + " of Object " +
		//		s.toString() + LogoError.NOT_OBJECT, c);
	    }
	    first = next;
	}
	return new Closure(first, parts[parts.length-1]);
    }

    public static Object evalFunction(Sym name, Function command,
				      YoYoObject env, Context c) {
	Sym oldcurrentfun = c.currentfun;
	int oldnargs = c.nargs;
	int oldprec = c.precedence;
	YoYoObject oldenv = c.objenv;
	//System.out.println("Eval function: " +name);
	Object r = null; Object[] v; int i;
	assert(command.getNargs() >= 0);
	try {
	    //System.out.println("set prec to 0");
	    for(c.nargs = command.getNargs(), v = command.getArgArray(c),
		    i = 0, c.precedence = 0;
		c.nargs > 0; c.nargs--, i++) {
		// if (c.ilistPeek() == null)
		if (c.ilistposptr >= c.ilist.length)
		    LogoError.error(LogoError.TOO_FEW_REPORTERS + name, c);
		r = null;
		//System.out.println("eval arg: " + c.nargs);
		Object foo = eval((c.ilistposptr < c.ilist.length
				   ? c.ilist[c.ilistposptr++] : null), c);
		if (foo == null)
		    LogoError.error(LogoError.TOO_FEW_REPORTERS + name, c);
		v[i] = foo;
	    }
	    c.currentfun = name;
	    c.objenv = env;
	    r = command.run(v, c);
	}
	finally {
	    c.currentfun = oldcurrentfun;
	    c.nargs = oldnargs;
	    c.precedence = oldprec;
	    //System.out.println("evalFunc: " + name + " restore prec to " + c.precedence);
	    c.objenv = oldenv;
	}
	return r;
    }

    public static Object evalFunction(Sym name, Function command,
				      YoYoObject env, Object[] argArray, Context c) {
	Sym oldcurrentfun = c.currentfun;
	int oldnargs = c.nargs;
	int oldprec = c.precedence;
	YoYoObject oldenv = c.objenv;

	//System.out.println("Eval function with argarray: " +name);
	//System.out.println("set prec to 0");
	Object r = null;
	try {
	    c.currentfun = name;
	    c.nargs = 0;
	    c.precedence = 0;

	    if (command.getNargs() < argArray.length)
		LogoError.error(LogoError.TOO_FEW_REPORTERS + name, c);
	    else if (command.getNargs() > argArray.length)
		LogoError.error(LogoError.TOO_MANY_REPORTERS + name, c);

	    c.objenv = env;
	    r = command.run(argArray, c);
	}
	finally {
	    c.currentfun = oldcurrentfun;
	    c.nargs = oldnargs;
	    c.precedence = oldprec;
	    //System.out.println("evalFunc: " + name + " restore prec to " + c.precedence);
	    c.objenv = oldenv;
	}
	return r;
    }

    static Object infixCall(int nargs, Object firstarg,
			    Function f, Context c) {
	//	System.out.println("infix call: " + printToString(f));
	Sym oldcurrentfun = c.currentfun;
	int oldnargs = c.nargs;
	int oldprec = c.precedence;
	YoYoObject oldenv = c.objenv;

	Object r = null;
	c.currentfun = f.getName();
	c.nargs=1;
	c.precedence=nargs;
	//System.out.println("set prec to " + nargs);
	Object[] v = f.getArgArray(c);
	try {
	    v[0]=firstarg;
// 	    if (f.getName() == Symbol.lookup("and") ||
// 		f.getName() == Symbol.lookup("or") ||
// 		f.getName() == Symbol.lookup("=")) {
// 		System.out.println("v0 = " + YoYo.printToString(v[0]));
// 	    }
	    //if (c.ilistPeek() == null)
	    if (c.ilistposptr >= c.ilist.length)
		LogoError.error(LogoError.TOO_FEW_REPORTERS +
				c.currentfun, c);
	    r=null;
	    v[1]=eval((c.ilistposptr < c.ilist.length
		       ? c.ilist[c.ilistposptr++] : null), c);
// 	    if (f.getName() == Symbol.lookup("and") ||
// 		f.getName() == Symbol.lookup("or") ||
// 		f.getName() == Symbol.lookup("=")) {
// 		System.out.println("v1 = " + YoYo.printToString(v[1]));
// 	    }
	    if (v[1] == null)
		LogoError.error(LogoError.TOO_FEW_REPORTERS +
				c.currentfun, c);
	    r=f.run(v, c);
	}
	finally {
	    c.currentfun = oldcurrentfun;
	    c.nargs = oldnargs;
	    c.precedence = oldprec;
	    //System.out.println("infixCall: " + printToString(f) + " restore prec to " + c.precedence);
	    c.objenv = oldenv;
	}
	if (r!=null) return rtv(r, c);
	return null;
    }


    public static String aString(Context c, Object o) {
	if (o instanceof String) return (String)o;
	if (o instanceof Sym) return ((Sym)o).toString();
	if (o instanceof Character) {
	    char[] foo = { ((Character)o).charValue() };
	    return new String(foo);
	}
	return printToString(o);
    }

    public static YoYoObject aYoYoObject(Context c, Object o) {
	try { return (YoYoObject)o; }
	catch (Exception e) {
	    LogoError.error(printToString(o) + LogoError.NOT_OBJECT, c);
	}
	return null;
    }

    public static boolean aBoolean(Context c, Object o) {
	if (o instanceof Symbol) {
	    return ((Symbol)o != symfalse);
	}
	else return true;
    }

    public static Ilist aIlist(Context c, Object o) {
	if (o instanceof Ilist) return (Ilist)o; //((Ilist)o).reset();
	if (o instanceof Object[]) return new Ilist((Object[])o);
	LogoError.error(printToString(o) + LogoError.NOT_LIST, c);
	return null;
    }

    public static Object[] aList(Context c, Object o) {
	if (o instanceof Object[]) return (Object[])o;
	LogoError.error(printToString(o) + LogoError.NOT_LIST, c);
	return null;
    }

    public static Sym aSym(Context c, Object o) {
	if (o instanceof Sym) return (Sym)o;
	if (o instanceof String) return Symbol.lookup((String)o);
	LogoError.error(printToString(o) + LogoError.NOT_SYMBOL, c);
	return null;
    }

    public static Symbol aSymbol(Context c, Object o) {
	if (o instanceof Symbol) return (Symbol)o;
	if (o instanceof String) return Symbol.lookup((String)o);
	LogoError.error(printToString(o) + LogoError.NOT_SYMBOL, c);
	return null;
    }

    public static Number aNumber(Context c, Object o) {
	if (o instanceof Number) return (Number)o;
	LogoError.error(printToString(o) + LogoError.NOT_NUMBER, c);
	return null;
    }

    public static double aDouble(Context c, Object o) {
	if (o instanceof Number) return ((Number)o).doubleValue();
	LogoError.error(printToString(o) + LogoError.NOT_NUMBER, c);
	return 0.0;
    }

    public static double aFloat(Context c, Object o) {
	if (o instanceof Number) return ((Number)o).floatValue();
	LogoError.error(printToString(o) + LogoError.NOT_NUMBER, c);
	return 0.0f;
    }

    public static double aDoubleNoWarn(Context c, Object o) {
	if (o instanceof Number) return ((Number)o).doubleValue();
	return 0.0;
    }

    public static int aInteger(Context c, Object o) {
	if (o instanceof Number) return ((Number)o).intValue();
	LogoError.error(printToString(o) + LogoError.NOT_NUMBER, c);
	return 0;
    }

    public static long aLong(Context c, Object o) {
	if (o instanceof Number) return ((Number)o).longValue();
	LogoError.error(printToString(o) + LogoError.NOT_NUMBER, c);
	return 0;
    }

    public static Locale aLocale(Context c, Object o) {
	try { return (Locale)o; }
	catch (Exception e) {
	    LogoError.error(printToString(o) + LogoError.NOT_LOCALE, c);
	}
	return null;
    }

    public static ResourceBundle aResourceBundle(Context c, Object o) {
	try { return (ResourceBundle)o; }
	catch (Exception e) {
	    LogoError.error(printToString(o) + LogoError.NOT_RESOURCE_BUNDLE, c);
	}
	return null;
    }

    public static Symbol equal (Object v0, Object v1)  {
	if (v0 instanceof Object[] && v1 instanceof Object[]) {
	    Object[] w0=(Object[])v0, w1=(Object[])v1;
	    if (w0.length != w1.length) return YoYo.symfalse;
	    for (int i=0; i<w0.length; i++) {
  		if ( !(YoYo.symtrue == (equal(w0[i],w1[i]))) )
		    return YoYo.symfalse;
	    }
	    return YoYo.symtrue;	
	}
	if (v0 instanceof Number && v1 instanceof Number)
	    return truefalse(((Number)v0).doubleValue() ==
			     ((Number)v1).doubleValue());
	if (v0.getClass().equals(v1.getClass())) return truefalse(v0.equals(v1));
	if (v0 instanceof Symbol) return truefalse(((Symbol)v0).equals(v1));
	if (v1 instanceof Symbol) return truefalse(((Symbol)v1).equals(v0));
	return truefalse(v0.toString().equals(v1.toString()));
    }
    
    public static Symbol truefalse(boolean tf) {
	return (tf) ? YoYo.symtrue : YoYo.symfalse;
    }

    public static String printToString(Object o) {
	return printToString(o, false);
    }

    public static String printToString(Object o, boolean hexp) {
	if (o == null) return "<Java null>";
	if (PrimVar.weakReferencep(o)) {
	    Object ret = PrimVar.aWeakReference(o, null);
	    if (ret == null) return "false";
	    return "<Weak reference to " + printToString(ret, hexp) + ">";
	}
	if (o instanceof Object[])
	    return objectArrayToString((Object[])o, '[', ']', hexp);
	if (o instanceof Vector)
	    return vectorToString((Vector)o, hexp);
	if (o instanceof int[])
	    return intArrayToString((int[])o, '[', ']', hexp);
	if ((o instanceof Number) &&
	    ((Number)o).doubleValue()==((Number)o).intValue())
	    return Integer.toString(((Number)o).intValue());
	//    if (o instanceof String)
	//return "\"" + (String)o + "\"";
	return o.toString();
    }

    public static String vectorToString(Vector v, boolean hexp) {
	StringBuffer sb = new StringBuffer();
	sb.append('<');
	for(Enumeration e = v.elements(); e.hasMoreElements();) {
	    sb.append(printToString(e.nextElement(), hexp));
	    sb.append(' ');
	}
	sb.append('>');
	return sb.toString();
    }

    public static String objectArrayToString(Object[] o, char start,
					     char end, boolean hexp) {
	StringBuffer sb = new StringBuffer();
	sb.append(start);
	if (o.length > 0) {
	    for(int i = 0; i< o.length-1; i++) {
		sb.append(printToString(o[i], hexp));
		sb.append(' ');
	    }
	    sb.append(printToString(o[o.length-1], hexp));
	}
	sb.append(end);
	return sb.toString();
    }

    public static String intArrayToString(int[] o, char start,
					  char end, boolean hexp) {
	StringBuffer sb = new StringBuffer();
	sb.append(start);
	if (o.length > 0) {
	    for(int i = 0; i< o.length-1; i++) {
		if (hexp) {
		    sb.append("0x" + Integer.toString(o[i], 16) + " ");
		    sb.append(' ');
		} else {
		    sb.append(Integer.toString(o[i]));
		    sb.append(' ');
		}
	    }
	    sb.append(Integer.toString(o[o.length-1], (hexp) ? 16 :  10));
	}
	sb.append(end);
	return sb.toString();
    }

    public static String getLanguageAsString() {
	return language.getDisplayLanguage();
    }


}
