/*
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
package yoyo;

import java.util.Hashtable;
import java.util.Enumeration;
import java.io.IOException;
import java.io.Serializable;
//import java.lang.ref.WeakReference;

public class YoYoObject extends Hashtable {


    static int numobjs = 1;
    int num;

    static Symbol kindsym = Symbol.lookup("kind");
    static Symbol runsym = Symbol.lookup("run");
    static Symbol parentsym = Symbol.lookup("parent");
    
    public YoYoObject(YoYoObject l) {
	super();
	put(parentsym, PrimVar.newWeakReference(l));
	int i; Enumeration e;
	for(e = l.keys(), i = 0; e.hasMoreElements(); i++) {
	    Symbol s = (Symbol)e.nextElement();
	    put(s, l.get(s));
	}
	num = numobjs++;
    }

    public YoYoObject() {
	super();
	num = numobjs++;
    }

    public YoYoObject(Context c) {
	super();
	num = numobjs++;
	put(runsym, c.getGlobalObj().lookupValue(runsym));
    }

	
    public void defineValue(Symbol s, Object o) {
	put(s, o);
    }
	
    public void setValue(Symbol s, Object o) {
	put(s, o);
    }

    public Object lookupValue(Symbol s) {
	return get(s);
    }
	
    public void undefineValue(Symbol s) {
	remove(s);
    } 	
	
    public boolean boundp(Symbol s) {
	return (get(s) != null);
    }
	
    public synchronized String getKind() {
	Object val = get(kindsym);
	if (val == null) return null;
	return val.toString();
    }

    public synchronized Object getParent(Context c) {
		return PrimVar.aWeakReference(get(parentsym), c);
    }
	
    public synchronized Object[] fields() {
	Object[] output = new Object[size()];
	int i; Enumeration e;
	for(e = keys(), i = 0; e.hasMoreElements(); i++) {
	    output[i] = e.nextElement();
	}
	return output;
    }
	
    public synchronized Object[] fieldsAndValues() {
	Object[] output = new Object[size()];
	int i; Enumeration e;
	for(e = keys(), i = 0; e.hasMoreElements(); i++) {
	    Object[] part = new Object[2];
	    part[0] = e.nextElement();
	    part[1] = get(part[0]);
	    output[i] = part;
	}
	return output;	
    }
	
    public String toString() {	
	String kind = getKind();
	if (kind == null) {
	    return "<Object #" + num + ">";
	}
	else return "<Object #" + num + ": " + kind + ">";
    }
        
    public int getNum() {
	return num;
    }

}
