/******************************************************************
OpenStarLogo

Originally developed by Mitchel Resnick, Andrew Begel, Eric Klopfer, Michael
Bolin, Corey McCaffrey, Molly Jones, Matthew Notowidigdo, Sebastian Ortiz, David
Greenspan, Michael Mandel, Tim Garnett, Max Goldman, Julie Kane, Russell
Zahniser, Weifang Sun, and Robert Tau at the MIT Teacher Education Program and
Media Laboratory, MIT, Cambridge, Massachusetts, with support from the National
Science Foundation and the LEGO Group. With previous work by Vanessa Colella,
Bill Thies, Brian Silverman, Monica Linden, Alice Yang, Ankur Mehta, Adam Eames,
Max Planck, and Sumita Kumar.

Copyright (c) 2006 MIT Teacher Education Program

Created by the MIT Teacher Education Program (http://education.mit.edu), and MIT
Media Lab (http://www.media.mit.edu) at the Massachusetts Institute of
Technology, Cambridge, MA. Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including without
limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom
the Software is furnished to do so, subject to the following conditions:

1. The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software. 2. For rights to publish,
distribute, sublicense or sell the Software or its derivatives for commercial
purposes, please see the Technology Licensing Office, MIT, Five Cambridge
Center, Kendall Square Room NE25-230, Cambridge, MA 02142-1493, (617) 253-6966

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *******************************************************************/

package starlogo;

import yoyo.Context;
import yoyo.YoYo;
import yoyo.LogoError;
import yoyo.Primitives;
import yoyo.TopApplication;
import yoyo.Toplevel;
import yoyo.PrimList;
import java.util.Random;
import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;
import java.util.*;
import java.net.MalformedURLException;
import java.net.URL;
import java.io.Serializable;
import yoyo.YoYoObject;
import yoyo.ThrowReturn;
import yoyo.Symbol;
import yoyo.Sym;
import java.io.*;
import javax.swing.SwingUtilities;

import ptolemy.*;

public abstract class VM implements Serializable {

  public final static int PRIMCONTROL=15,PRIMMATH=16,PRIMGLOBALVAR=17,
    PRIMTURTLEVAR=18,PRIMPATCHVAR=19,PRIMPATCHSTATE=20,PRIMTURTLESTATE=21,
    PRIMTURTLEOTHERSTATE=22, PRIMTOWARDS=23,
    PRIMLIFE=24,PRIMGLOBALCOUNT=25,PRIMATTOWARDS=26,PRIMHERE=27,PRIMSCREEN=28,
    PRIMLIST=29,PRIMREFLECTION=30,PRIMGRAB=31,PRIMPRINT=32,PRIMNSUM=33,
    PRIMCLEAR=34,PRIMASK=35,PRIMMISC=36,PRIMPLOT=37,PRIMSAVE=38;
    
  public final static int COMMAND=0,REPORTER=1,COMMANDREPORTER=2,OTHER=3;

  // Bytecode format: bottom 8 bits, bytecode
  //                  next 8 bits, category
  //                  next 1 bit, command or reporter.

  // PRIMLOGO
  public final static int DONE=0,DATA=1,LIST=2,EOL=3,EOLR=4,LTHING=5,LSET=6,UFUN=7,
    UFUNTAIL=8,DLIST=9;

  // PRIMCONTROL
  public final static int STOP=0,OUTPUT=1,IF=2,IFELSE=3,CASE=4,IGNORE=5,
    LOOP=6,EVERY=7,WAITUNTIL=8,REPEAT=9,WAIT=10,RESETT=11,TIMER=12,
      WITHIFT=13,WITHIFP=14,ASKIFT=15,ASKIFP=16,DOTIMES=17,DOLIST=18;

  public final static int IF2=38,CASE2=37,CASE3=36,CASE4=35,EVERY2=34,
    EVERY3=33,WAITUNTIL2=32,REPEAT2=31,WAIT2=30,DOTIMES2=29,DOLIST2=28;

  // PRIMMATH
  public final static int ADD=0,MINUS=1,TIMES=2,DIV=3,INTDIV=4,MOD=5,
    RANDOM=6,RANDOMG=7,SETRAND=8,GETRAND=9,POWER=10,EXP=11,LN=12,SIN=13,
    COS=14,TAN=15,ATAN=16,SQRT=17,ABS=18,MAX=19,MIN=20,GT=21,LESST=22,EQ=23,
    NEQ=24,LE=25,GE=26,ROUND=27,INT=28,NOT=29,AND=30,OR=31,XOR=32,BITNOT=33,
    BITAND=34,BITOR=35,BITXOR=36,MINNUM=37,MAXNUM=38,NEQ2=39,ASIN=40,ACOS=41,
    LOG=42;

  // PRIMGLOBALVAR
  public final static int OGETG=0,TPGETG=1,OSETG=2,TPSETG=3;

  // PRIMTURTLEVAR
  public final static int OSETT=0,TSETT=1,PSETT=2,SETTOF=3,OSETTAT=4,PSETTAT=5,
    TSETTAT=6,OSETTTOWARDS=7,TSETTTOWARDS=8,PSETTTOWARDS=9,TGETT=10,
    PGETT=11,GETTOF=12,OGETTAT=13,PGETTAT=14,TGETTAT=15,OGETTTOWARDS=16,
    TGETTTOWARDS=17,PGETTTOWARDS=18;

  // PRIMPATCHVAR
  public final static int TGETP=0,PGETP=1,OSETP=2,TSETP=3,PSETP=4,
    OGETPAT=5,PGETPAT=6,TGETPAT=7,OSETPAT=8,PSETPAT=9,TSETPAT=10,
    OGETPTOWARDS=11,TGETPTOWARDS=12,PGETPTOWARDS=13,OSETPTOWARDS=14,
    TSETPTOWARDS=15,PSETPTOWARDS=16;

  // PRIMPATCHSTATE
  public final static int TPC=0,PPC=1,OSETPC=2,TSETPC=3,PSETPC=4,
    TSCALEPC=5,PSCALEPC=6,OPCAT=7,PPCAT=8,TPCAT=9,OSTAMPAT=10,
    PSTAMPAT=11,TSTAMPAT=12,OPCTOWARDS=13,TPCTOWARDS=14,PPCTOWARDS=15,
    OSTAMPTOWARDS=16,TSTAMPTOWARDS=17,PSTAMPTOWARDS=18;

  // PRIMTURTLESTATE
  public final static int TXCOR=0,PXCOR=1,TYCOR=2,PYCOR=3,SETX=4,
    SETY=5,SETXY=6,HOME=7,DX=8,DY=9,JUMP=10,STEP=11,FD=12,BK=13,
    TCOLOR=14,PCOLOR=15,TSETC=16,PSETC=17,TSCALECOLOR=18,PSCALECOLOR=19,
    WHO=20,PD=21,PU=22,PENDOWNP=23,SETH=24,RT=25,LT=26,THEADING=27,
    PHEADING=28,HT=29,ST=30,SHOWNP=31,TBREED=32,PBREED=33,TSETBREED=34,
    PSETBREED=35,TSHAPE=36,PSHAPE=37,TSETSHAPE=38,PSETSHAPE=39,
    LEAP=40,ALIVE=41,ALIVEOF=42;

  // PRIMTURTLEOTHERSTATE
  public final static int GETSTATEOF=0,SETSTATEOF=1,OGETSTATEAT=2,
    TGETSTATEAT=3,PGETSTATEAT=4,OSETSTATEAT=5,TSETSTATEAT=6,PSETSTATEAT=7,
    OGETSTATETOWARDS=8,TGETSTATETOWARDS=9,PGETSTATETOWARDS=10,
    OSETSTATETOWARDS=11,TSETSTATETOWARDS=12,PSETSTATETOWARDS=13,
    SETXYOF=14,OSETXYAT=15,TSETXYAT=16,PSETXYAT=17,OSETXYTOWARDS=18,
    TSETXYTOWARDS=19,PSETXYTOWARDS=20,SETXOF=21,SETYOF=22,
    OSETXAT=23,TSETXAT=24,PSETXAT=25,
    OSETYAT=26,TSETYAT=27,PSETYAT=28,OSETXTOWARDS=29,TSETXTOWARDS=30,
    PSETXTOWARDS=31,OSETYTOWARDS=32,TSETYTOWARDS=33,PSETYTOWARDS=34;

  public final static int FD2=70,BK2=71;

  // PRIMTOWARDS
  public final static int OTOWARDS=0,TTOWARDS=1,PTOWARDS=2,
    OTOWARDSNOWRAP=3,TTOWARDSNOWRAP=4,PTOWARDSNOWRAP=5,ODISTANCE=6,
    TDISTANCE=7,PDISTANCE=8,ODISTANCENOWRAP=9,TDISTANCENOWRAP=10,
    PDISTANCENOWRAP=11;

  // PRIMLIFE
  public final static int DIE=0,KILL=1,HATCH=2,SPROUT=3,CRT=4,CCT=5,
    CRTANDDO=6,CCTANDDO=7;

  public final static int HATCH2=15;


  // PRIMGLOBALCOUNT
  public final static int TONET=0,OPONET=1,TONEB=2,OPONEB=3,TLISTT=4,OPLISTT=5,
      TLISTB=6,OPLISTB=7,COUNTTURTLES=8,COUNTBREEDS=9,COUNTCOLOR=10,
      COUNTPC=11,COUNTCOLORRANGE=12,COUNTPCRANGE=13,COUNTPATCHES=14;

  // PRIMATTOWARDS
  public final static int OCTA=0,PCTA=1,TCTA=2,OCTT=3,TCTT=4,PCTT=5,OOTA=6,POTA=7,
    TOTA=8,OOTT=9,TOTT=10,POTT=11,OLTA=12,PLTA=13,TLTA=14,OLTT=15,TLTT=16,PLTT=17,
    OCBA=18,PCBA=19,TCBA=20,OCBT=21,TCBT=22,PCBT=23,OOBA=24,POBA=25,TOBA=26,OOBT=27,
    TOBT=28,POBT=29,OLBA=30,PLBA=31,TLBA=32,OLBT=33,TLBT=34,PLBT=35;

  // PRIMHERE
  public final static int TCTH=0,PCTH=1,TOTH=2,POTH=3,TLTH=4,PLTH=5,
    TCBH=6,PCBH=7,TOBH=8,POBH=9,TLBH=10,PLBH=11;

  // PRIMSCREEN
  public final static int SCREENEDGEX=0,SCREENEDGEY=1,SCREENWIDTH=2,
    SCREENHEIGHT=3;

  // PRIMLIST
  public final static int MAKEDOUBLELIST=0,SE=1,FPUT=2,LPUT=3,BF=4,BL=5,
    FIRST=6,LAST=7,EMPTYP=8,LENGTH=9,MEMBERP=10,POSITION=11,
    RANDOMITEM=12,ITEM=13,SETITEM=14,TOLIST=15,TOSTRING=16,TODELSTRING=17,
    COPYLIST=18,MAKELIST=19,SORTNUMLIST=20,SUBLIST=21,INSERT=22,REMOVE=23,
    REMOVEELE=24,LISTMIN=25,LISTMAX=26,LISTMEDIAN=27,LISTMEAN=28,LISTVARIANCE=29,
      LISTSDEV=30,REVERSE=31,LISTMODE=32,LISTSUM=33;

  // PRIMREFLECTION
  public final static int NUMBERP=0,WORDP=1,LISTP=2,STACKCRAWL=3;

  // PRIMGRAB
  public final static int GRAB=0,PARTNER=1,PARTNERS=2,OGRABPATCH=3,
    TGRABPATCH=4,PGRABPATCH=5,PATCHPARTNER=6,PATCHPARTNERS=7,GRABBEDP=8;
  public final static int GRAB2=255,GRABPATCH2=254;

  // PRIMPRINT
  public final static int OPRINT=0,TPRINT=1,PPRINT=2,OTYPE=3,TTYPE=4,
    PTYPE=5,OSHOW=6,TSHOW=7,PSHOW=8,CC=9,SPRINT=10;

  // PRIMNSUM
  public final static int NSUM=0,NSUM4=1,NMIN=2,NMIN4=3,NMAX=4,NMAX4=5,
    DIFFUSE=6,DIFFUSE4=7,DIFFUSEPC=8,DIFFUSE4PC=9,NSUMPC=10,NSUM4PC=11;

  // PRIMCLEAR
  public final static int CA=0,CG=1,CP=2,CT=3,CLEAROUTPUT=4,CLEARINFO=5;

  // PRIMASK
  public final static int OASKPATCH=0,TASKPATCH=1,PASKPATCH=2,
    ASKPATCHES=3,ASKPATCHESSLOW=4,ASKTURTLE=5,ASKTURTLES=6,
    ASKBREEDS=7,ASKOBS=8,COUNTTURTLESWITH=9,COUNTBREEDSWITH=10,
    COUNTPATCHESWITH=11,COUNTPATCHESWITHSLOW=12,
      TSUMTURTLES=13,OPSUMTURTLES=14,TSUMBREEDS=15,OPSUMBREEDS=16,
      SUMPATCHES=17,SUMPATCHESSLOW=18,
      TLISTTURTLESWITH=19,OPLISTTURTLESWITH=20,TLISTBREEDSWITH=21,OPLISTBREEDSWITH=22,
      MYSELF=23,TMINTURTLES=24,OPMINTURTLES=25,TMAXTURTLES=26,OPMAXTURTLES=27,
      TMINBREEDS=28,OPMINBREEDS=29,TMAXBREEDS=30,OPMAXBREEDS=31,
      MINPATCHESSLOW=32,MAXPATCHESSLOW=33,
      ASKTLIST=34,TONETURTLEWITH=35,OPONETURTLEWITH=36,
      TONEBREEDWITH=37,OPONEBREEDWITH=38,
      TAVGTURTLES=39,OPAVGTURTLES=40,TAVGBREEDS=41,OPAVGBREEDS=42,
      AVGPATCHES=43,AVGPATCHESSLOW=44,
      TVARTURTLES=45,OPVARTURTLES=46,TVARBREEDS=47,OPVARBREEDS=48,
      VARPATCHES=49,VARPATCHESSLOW=50,
      TMEDTURTLES=51,OPMEDTURTLES=52,TMEDBREEDS=53,OPMEDBREEDS=54,
      MEDPATCHES=55,MEDPATCHESSLOW=56,
      TMODETURTLES=57,OPMODETURTLES=58,TMODEBREEDS=59,OPMODEBREEDS=60,
      MODEPATCHES=61,MODEPATCHESSLOW=62,
      TSDEVTURTLES=63,OPSDEVTURTLES=64,TSDEVBREEDS=65,OPSDEVBREEDS=66,
      SDEVPATCHES=67,SDEVPATCHESSLOW=68,
      TWHOMINTURTLES=69,OPWHOMINTURTLES=70,
      TWHOMAXTURTLES=71,OPWHOMAXTURTLES=72,TWHOMINBREEDS=73,OPWHOMINBREEDS=74,
      TWHOMAXBREEDS=75,OPWHOMAXBREEDS=76,
      TONETURTLEHEREWITH=77,PONETURTLEHEREWITH=78,
      TONEBREEDHEREWITH=79,PONEBREEDHEREWITH=80,
      TONETURTLEATWITH=81,OONETURTLEATWITH=82,PONETURTLEATWITH=83,
      TONEBREEDATWITH=84,OONEBREEDATWITH=85,PONEBREEDATWITH=86,
      TONETURTLETOWITH=87,OONETURTLETOWITH=88,PONETURTLETOWITH=89,
      TONEBREEDTOWITH=90,OONEBREEDTOWITH=91,PONEBREEDTOWITH=92;

  public final static int ASK2=255,CTW2=254,ST2=253,LTW2=252,MINT2=251,
      MAXT2=250,OTW2=249,AVGT2=248,VART2=247,SDEVT2=246,MODET2=245,
      MEDT2=244,WMINT2=243,WMAXT2=242;

  // PRIMMISC
  public final static int SVAL=0,STOPALL=1,PLAYSOUND=2,PRINTMONITOR=3,
    TOGGLEBUTTONON=4,TOGGLEBUTTONOFF=5,YOYO=6,INSPECTT=7,MOUSEXCOR=8,
    MOUSEYCOR=9,MOUSEDOWNP=10,SWITCHPATCHES=11,SWITCHSCHED=12,
    SETSCHEDRANDSEED=13,GETSCHEDRANDSEED=14,GETSCHED=15,
    DISPLAY=16,NODISPLAY=17,DEBUG=18,NODEBUG=19,DEBUG2=20,NODEBUG2=21,
    INSPECTP=22,SETSVAL=23,SETCC=24;

  // PRIMPLOT
  public final static int OPLOT=0,TPLOT=1,PPLOT=2,SETPLOTPPC=3,CLEARPLOT=4,PLOTPPC=5,
      PPXCOR=6,PPYCOR=7,PLOTXMIN=8,PLOTXMAX=9,PLOTYMIN=10,PLOTYMAX=11,
      SETPLOTXMIN=12,SETPLOTXMAX=13,SETPLOTYMIN=14,SETPLOTYMAX=15,PPRESET=16,
      PLOTPEN=17,SETPLOTXRANGE=18,SETPLOTYRANGE=19,PPD=20,PPU=21,SETPLOTTITLE=22,
      PLOTTITLE=23,SETNUMPLOTPENS=24,COUNTPLOTPENS=25,PLOTXLABEL=26,PLOTYLABEL=27,
      SETPLOTXLABEL=28,SETPLOTYLABEL=29,VIEWPLOT=30,CLEARPLOTS=31,
      OPLOTXY=32,TPLOTXY=33,PPLOTXY=34,SETPLOT=35,SETBARWIDTH=36,PTOLEMYPLOT=37,
      PTOLEMYPLOTXY=38,PLOTGRID=39,PLOTGRIDON=40,PLOTGRIDOFF=41;

  // PRIMSAVE
  public final static int SETOUTPUTFILENAME=0,SETPROJECTFILENAME=1,SAVEOUTPUT=2,
    SAVEOUTPUTAS=3,SAVEPROJECT=4,SAVEPROJECTAS=5,SETINFOFILENAME=6,SAVEINFO=7,
    SAVEINFOAS=8,OUTPUTFILENAME=9,PROJECTFILENAME=10,INFOFILENAME=11,WRITEMAP=12,
      READPICTURE=13,READPICTURENAME=14,SAVEVERSION=15,SAVEPATCHESPICTURE=16,
      SAVEPATCHESPICTURENAME=17,SAVETURTLES_AND_PATCHESPICTURE=18,
      SAVETURTLES_AND_PATCHESPICTURENAME=19,SAVEINTERFACEPICTURE=20,
      SAVEINTERFACEPICTURENAME=21;

    static int assembleByteCode(int category, int bytecode, boolean commandp, boolean reporterp) {
	int cr = 0;
	if (commandp) {
	    if (reporterp) { 
		cr = 2;
	    } else {
		cr = 0;
	    }
	} else {
	    if (reporterp) {
		cr = 1;
	    } else {
		assert (bytecode == 0) && (category < PRIMCONTROL);
	    }
	}
	return (category << 8) | bytecode | (cr << 16);
    }

  static Integer zeroInt = new Integer(0);
  static Integer minusoneInt = new Integer(-1);
  static Object grabbedMarker = new Object();
  static Object grabbedPatchMarker = new Object();
  static Object askMarker = new Object();

  public static Hashtable sounds = new Hashtable();

  public static int seed = (int)System.currentTimeMillis();
  public static Random rand = new Random(seed);
  public static int mouseXcor = 0;
  public static int mouseYcor = 0;
  public static boolean mouseDownp = false;

  static StarLogo sl;

  public transient boolean runstate = false;
  public transient int ip, sp, fp, isp, ifp, whichip, here;
  public transient Object[] ostack, oinst;
  public transient int[] istack, iinst;
  Object[] turtlesown;
  transient StackProcess process;
  transient long timer = 0;
  transient StackProcessParent vmspp;
  transient Turtle partner;
  transient Patch patchpartner;
    transient int plotwidget;
    transient int plotpen;

  public static void setMouseDown(boolean downp) {
    mouseDownp = downp;
  }

  public static void setMousePos(int x, int y) {
    mouseXcor = (int)((double)(x-sl.patchcanvas.getPosition().x)/(double)sl.cellsize);
    mouseYcor = (int)((double)(y-sl.patchcanvas.getPosition().y)/(double)sl.cellsize);
    if (mouseXcor < 0) mouseXcor = 0;
    if (mouseYcor < 0) mouseYcor = 0;
    if (mouseXcor > sl.width) mouseXcor = sl.width;
    if (mouseYcor > sl.height) mouseYcor = sl.height;
  }

    StackProcess firstProcess, lastProcess;
    int count = 0;

  public void addSP(StackProcess sp) {
      if (firstProcess == null) {
	  firstProcess = sp;
	  lastProcess = sp;
	  sp.prevInVM = null;
	  sp.nextInVM = null;
      } else {
	  /* Add it to the beginning of the list */
	  sp.prevInVM = null;
	  sp.nextInVM = firstProcess;
	  firstProcess.prevInVM = sp;
	  firstProcess = sp;
      }
      count++;
  }

    public void removeSP(StackProcess sp) {
    	//if (this instanceof Turtle) {
    	//	System.out.println("before: "); printList();
    	//}
      /* If there's someone before me... */
      if (sp.prevInVM != null) {
	  /* Make his nextptr equal to my next pointer */
	  sp.prevInVM.nextInVM = sp.nextInVM;
      } else {
	  /* If not, check if there is someone after me... */
	  if (sp.nextInVM == null) {
	      /* No, so I'm the last process. Since we remove me ... */
	      firstProcess = null;
	      lastProcess = null;
	  } else {
	      /* Yes, since I was the firstProcess, make it now the next guy */
	      firstProcess = sp.nextInVM;
	  }
      }
      /* If there's someone after me ... */
      if (sp.nextInVM != null) {
	  /* Make his prevptr equal to my prev ptr */
	  sp.nextInVM.prevInVM = sp.prevInVM;
      } else {
	  /* If not, check if there's someone before me... */
	  if (sp.prevInVM == null) {
	      /* No, so i'm the last process. Since we remove me ... */
	      firstProcess = null;
	      lastProcess = null;
	  } else {
	      /* Yes, since I was the lastProcess, make it now the previous guy */
	      lastProcess = sp.prevInVM;
	  }
      }
      //if (this instanceof Turtle) {
      //	System.out.println("after: "); printList();
      //}
      count--;
    }

    public void removeAllSPs() {
	count = 0;
	firstProcess = null;
	lastProcess = null;
    }

    public int numSPs() {
	return count;
    }

    public void printList() {
	System.out.println("printing VM stackprocess list forwards");
	for(StackProcess current = firstProcess; current != null; current = current.nextInVM) {
	    System.out.print(current + ", ");
	}
	System.out.println();
    }
    public SPEnumeration spList() {
	/*
	  if (this instanceof Turtle) {
	    System.out.println("getting list of sps for: " + this);
	    printList();
	}
	*/
	class VMSPEnumerator implements SPEnumeration {
	    StackProcess first = firstProcess;
	    public boolean hasMoreElements() {
		return first != null;
	    }
	    public StackProcess nextElement() {
		StackProcess elt = first;
		first = first.nextInVM;
		return elt;
	    }
	}
	return new VMSPEnumerator();
    }


  public final void setTPO(YoYoObject yob) {
      if (this instanceof Turtle) {
	yob.defineValue(Symbol.lookup("turtle?"), YoYo.symtrue);
	yob.defineValue(Symbol.lookup("observer?"), YoYo.symfalse);
	yob.defineValue(Symbol.lookup("patch?"), YoYo.symfalse);
	yob.defineValue(Symbol.lookup("who"), ((Turtle)this).whoInt);
      } else if (this instanceof Patch) {
	yob.defineValue(Symbol.lookup("turtle?"), YoYo.symfalse);
	yob.defineValue(Symbol.lookup("observer?"), YoYo.symfalse);
	yob.defineValue(Symbol.lookup("patch?"), YoYo.symtrue);
	yob.defineValue(Symbol.lookup("xcor"),
			new Double(((Patch)this).xcor - 1 - sl.halfwidth));
	yob.defineValue(Symbol.lookup("ycor"),
			new Double(sl.halfheight - (((Patch)this).ycor - 1)));
      } else {
	yob.defineValue(Symbol.lookup("turtle?"), YoYo.symfalse);
	yob.defineValue(Symbol.lookup("observer?"), YoYo.symtrue);
	yob.defineValue(Symbol.lookup("patch?"), YoYo.symfalse);
      }
  }

  public final int aInteger(Object o) {
    try { return( ((Number)o).intValue()); }
    catch (Exception e) {
	//e.printStackTrace();
	YoYoObject yob = new YoYoObject();
	yob.defineValue(Symbol.lookup("instruction"), new Integer(here));
	yob.defineValue(Symbol.lookup("value"), o);
	setTPO(yob);
	process.saveState();
	Object[] crawl = process.stackCrawl();
	yob.defineValue(Symbol.lookup("stack-crawl"), crawl);
	throw new ThrowReturn("vm-not-integer", yob);
    }
  }


  public final double aDouble(Object o) {
    try { return( ((Number)o).doubleValue()); }
    catch (Exception e) {
	//e.printStackTrace();
      YoYoObject yob = new YoYoObject();
      yob.defineValue(Symbol.lookup("instruction"), new Integer(here));
      yob.defineValue(Symbol.lookup("value"), o);
      setTPO(yob);
      process.saveState();
      Object[] crawl = process.stackCrawl();
      yob.defineValue(Symbol.lookup("stack-crawl"), crawl);
      throw new ThrowReturn("vm-not-number", yob);
    }
  }

  public final Object[] aList(Object o) {
    try { return (Object[])o; }
    catch (Exception e) {
	//e.printStackTrace();
      YoYoObject yob = new YoYoObject();
      yob.defineValue(Symbol.lookup("instruction"), new Integer(here));
      yob.defineValue(Symbol.lookup("value"), o);
      setTPO(yob);
      process.saveState();
      Object[] crawl = process.stackCrawl();
      yob.defineValue(Symbol.lookup("stack-crawl"), crawl);
      throw new ThrowReturn("vm-not-list", yob);
    }
  }

  public final Turtle[] aTurtleList(Object[] ids) {
    Turtle[] out = new Turtle[ids.length];
    int count = 0, i= 0;
    for(i = 0; i < ids.length; i++) {
      int who = aInteger(ids[i]);
      //System.out.println("aturtlelist: who = " + who);
      Turtle t = sl.getWho(who);
      //System.out.println("aturtlelist: turtle = " + t);
      if (t != null) out[count++] = t;
      //System.out.println("count = " + count);
    }
    if (count < ids.length) {
	//System.out.println("some turtles didn't exist: " + count + "  " + ids.length);
      Turtle[] realout = new Turtle[count];
      System.arraycopy(out, 0, realout, 0, count);
      out = realout;
    }
    return out;
  }

  public final String aString(Object o) {
    if (o instanceof String) return (String)o;
    if (o instanceof Sym) return ((Sym)o).toString();
    if (o instanceof Character) {
      char[] foo = { ((Character)o).charValue() };
      return new String(foo);
    }
    return YoYo.printToString(o);
  }

  public final boolean aBoolean(Object o) {
    if (o == YoYo.symfalse) return false;
    return true;
  }

  public final boolean inBounds(double num, double low, double high) {
    if (num < low || num > high) {
      YoYoObject yob = new YoYoObject();
      yob.defineValue(Symbol.lookup("instruction"), new Integer(here));
      yob.defineValue(Symbol.lookup("value"), new Double(num));
      yob.defineValue(Symbol.lookup("low"), new Double(low));
      yob.defineValue(Symbol.lookup("high"), new Double(high));
      setTPO(yob);
      process.saveState();
      Object[] crawl = process.stackCrawl();
      yob.defineValue(Symbol.lookup("stack-crawl"), crawl);
      throw new ThrowReturn("vm-out-of-bounds", yob);
    }
    else return true;
  }


  public static double deg2Rad(double deg) {
    return ((deg % 360) / 180.0) * Math.PI;
  }

  public static double rad2Deg(double rad) {
    double deg = (rad * 180.0) / Math.PI;
    if (deg < 0) deg += 360;
    return deg;
  }

  public static Hashtable initStateOffsets() {
    Turtle.stateOffsetTable = Turtle.initStateOffsets();
    Patch.stateOffsetTable = Patch.initStateOffsets();
    Observer.stateOffsetTable = Observer.initStateOffsets();
    return null;
  }


  public void dispatch(Context c, Observer obs, Turtle tur, Patch pat) {
    int offset, newaddr, nargs, tempint, tempint2, oldip, trueip, falseip, counter;
    double x, y, dl;
    boolean testp;
    Object test, val, thing1 = null, thing2, thing3;
    Turtle partner = null, st;
    Turtle [] sts;
    Vector partners;
    Object[] list;
    if (tur != null && tur.alive == false) {
	System.out.println("Trying to dispatch on a dead turtle!\n" + tur.dumpToString() + "\nin stack process " + process);
	throw new StarLogoError(here, process, this);
    }
    loop:
    while(true) {
      if (StarLogo.DEBUG2) {
	  c.log.println("dispatch: stackprocess = 0x" + Integer.toString(process.hashCode(), 16) +
			   " entity = " + process.entity + ", ip = "
			   + ip + ", here = " + here +
			   ", category = " + ((here & 0xFF00) >> 8) +
			   ", bytecode = " + (here & 0xFF) +
			   ", c-or-r = " + ((here & 0x0003000) >> 16) +
			   ", sp = " + sp + ".");
      }
      switch((here & 0xFF00) >> 8) {
      case DONE:
	//System.out.println("process is done");
	process.sp = sp;
	sl.done(process); return;
      case DATA:
      case DLIST:
	ostack[sp++] = process.oinst[iinst[ip++]];
	break;
      case LTHING:
	offset = iinst[ip++];
	ostack[sp++] = ostack[process.fp-1-offset];
	break;
      case LSET:
	offset = iinst[ip++];
	val = ostack[--sp];
	ostack[process.fp-1-offset] = val;
	here = iinst[ip++]; return;
      case LIST:
	offset = iinst[ip++];
	istack[isp++] = ip; ip+=offset;
	break;
      case EOL:
      case EOLR:
	here = istack[--isp];
	//System.out.println("here = " + here);
	continue loop;
      case UFUN:
      //System.out.println("ufun");
	newaddr = iinst[ip++]; // where to jump to
	istack[isp++] = (process.entity.partner == null) ? 0 : 1;
	istack[isp++] = iinst[ip++]; // nargs
	istack[isp++] = process.whichip; // return iinst
	istack[isp++] = ip + 1; // return address
	istack[isp++] = process.fp; // old fp
	istack[isp++] = process.ifp; // old ifp;
	process.fp = sp;  process.ifp = isp;
	//System.out.println("ufun foo");
	sp += iinst[ip++]; // make room for #local vars - #arguments
	iinst = sl.iinstarray[newaddr];
	process.oinst = sl.oinstarray[newaddr];
	ip = 2; process.whichip = newaddr;
	//process.entity.partner = null;
	//System.out.println("ufun end");
	break;
      case UFUNTAIL:
	ip = 0;
	nargs = iinst[ip++]; tempint = process.fp;
	while (nargs-- > 0) ostack[--tempint] = ostack[--sp];
	sp = process.fp; isp = process.ifp;
	sp += iinst[ip++]; // make room for #local vars - #arguments
	//process.entity.partner = null;
	break;

      case PRIMCONTROL:
	/* Control Flow Primitives */
	switch (here & 0xFF) {
	case STOP:
	  while(sp > process.fp) {
	    val = ostack[--sp];
	    if (val == grabbedMarker) {
	      st = ((Turtle)ostack[--sp]);
	      st.grabber = null;
	      if (!st.alive) sl.recycleDeadWho(st);
	    } else if (val == askMarker) {
	      process.entity = ((VM)ostack[--sp]);
	    } else if (val == grabbedPatchMarker) {
	      ((Patch)ostack[--sp]).grabbedp = false;
	    }
	  }
	  isp = process.ifp;
	  if (isp > 0) {
	    process.ifp = istack[--isp]; process.fp = istack[--isp];
	    oldip = istack[--isp];
	    newaddr = istack[--isp];
	    iinst = sl.iinstarray[newaddr];
	    process.oinst = sl.oinstarray[newaddr];
	    ip = oldip; process.whichip = newaddr;
	    //System.out.println("stop: return to ip = " + ip + " process.whichip = " + process.whichip);
	    nargs = istack[--isp]; sp-=nargs;
	    if (istack[--isp] > 0) { // are there more stack frames above me?
	      boolean gotturtle = false, gotpatch = false;
	      tempint = sp;
	      while (tempint > (process.fp+1) && !gotturtle && !gotpatch) {
		val = ostack[--tempint];
		if (val == grabbedMarker && !gotturtle) {
		  process.entity.partner = (Turtle)ostack[--tempint];
		  gotturtle = true;
		} else if (val == grabbedPatchMarker && !gotpatch) {
		  process.entity.patchpartner = (Patch)ostack[--tempint];
		  gotpatch = true;
		}
	      }
	      if (!gotturtle) process.entity.partner = null;
	      if (!gotpatch) process.entity.patchpartner = null;
	    } else process.entity.partner = null;
	  }
	  here = iinst[ip++]; return; // context-switch on stop, but not output
	case OUTPUT:
	  val = ostack[--sp];
	  while(sp > process.fp) {
	    thing1 = ostack[--sp];
	    if (thing1 == grabbedMarker) {
	      st = ((Turtle)ostack[--sp]);
	      st.grabber = null;
	      if (!st.alive) sl.recycleDeadWho(st);
	    } else if (thing1 == askMarker) {
	      process.entity = ((VM)ostack[--sp]);
	    } else if (thing1 == grabbedPatchMarker) {
	      ((Patch)ostack[--sp]).grabbedp = false;
	    }
	  }
	  isp = process.ifp;
	  if (isp > 0) {
	    process.ifp = istack[--isp]; process.fp = istack[--isp];
	    oldip = istack[--isp];
	    //System.out.println("old ip address " + oldip);
	    newaddr = istack[--isp];
	    //System.out.println("old which ip " + newaddr);
	    iinst = sl.iinstarray[newaddr];
	    process.oinst = sl.oinstarray[newaddr];
	    ip = oldip; process.whichip = newaddr;
	    nargs = istack[--isp]; sp-=nargs;
	    if (istack[--isp] > 0) {
	      boolean gotturtle = false, gotpatch = false;
	      tempint = sp;
	      while (tempint > (process.fp+1) && !gotturtle && !gotpatch) {
		thing1 = ostack[--tempint];
		if (thing1 == grabbedMarker && !gotturtle) {
		  process.entity.partner = (Turtle)ostack[--tempint];
		  gotturtle = true;
		} else if (thing1 == grabbedPatchMarker && !gotpatch) {
		  process.entity.patchpartner = (Patch)ostack[--tempint];
		  gotpatch = true;
		}
	      }
	      if (!gotturtle) process.entity.partner = null;
	      if (!gotpatch) process.entity.patchpartner = null;
	    } else process.entity.partner = null;
	  }
	  ostack[sp++] = val;
	  break;

      case IF: trueip = istack[--isp];
	test = ostack[--sp];
	if (aBoolean(test)) {
	  istack[isp++] = ip;
	  ip = trueip;
	  istack[isp++] = (IF2 | (PRIMCONTROL << 8) | (OTHER << 16));
	  break;
	}
	else here = iinst[ip++]; return;
      case IFELSE: falseip = istack[--isp];
	trueip = istack[--isp];
	test = ostack[--sp];
	istack[isp++] = ip;
	ip = (aBoolean(test)) ? trueip : falseip;
	istack[isp++] = (IF2 | (PRIMCONTROL << 8) | (OTHER << 16));
	break;
      case IF2: ip = istack[--isp]; here = iinst[ip++]; return;

	case WITHIFT:
	case WITHIFP:
	case ASKIFT:
	case ASKIFP:
	    trueip = istack[--isp];
	  test = ostack[--sp];
	  if (aBoolean(test)) {
	      istack[isp++] = ip;
	      ip = trueip;
	      istack[isp++] = (IF2 | (PRIMCONTROL << 8) | (OTHER << 16));
	      break;
	  } else {
	      process.inwith = false;
	      process.sp = sp;
	      sl.done(process);
	      return;
	  }
      case CASE:
	  // test value is on the ostack
	  // case statements are on istack
	//istack[isp++] = iinst[ip]; // num cases;
        //istack[isp++] = iinst[ip++]; // num cases again.
	  istack[isp++] = ip; // save where we are.
	  ip = istack[isp-2];  // go to the beginnig of the cases
      case CASE2:
	  // ip is at the LIST at beginning of case statement #n
	  istack[isp++] = ip;  // remember where the beginning of this test is
	  ip += 2; // skip past LIST <length> of first test
	  istack[isp++] = (CASE3 | (PRIMCONTROL << 8) | (OTHER << 16)); // where to come back to.
	  break;
      case CASE3:
	test = ostack[--sp];
	if (YoYo.equal(test, ostack[sp-1]) == YoYo.symtrue ||
	    test == YoYo.symtrue) {
	    ip = istack[--isp]; // put ip back at beginning of test
	  ip += iinst[ip + 1] + 2; // skip past the test
	  ip += 2; // skip past list-to-run LIST <length> header
	  istack[isp++] = (CASE4 | (PRIMCONTROL << 8) | (OTHER << 16)); // where to come back to.
	  break; // run the list to run
	}
	ip = istack[--isp]; // put ip back at beginnig of test
	ip += iinst[ip+1] + 2; // go to beginnig of case statement
	ip += iinst[ip+1] + 2; // skip past this statement too
	here = (iinst[ip] == LIST << 8) ? (CASE2 | (PRIMCONTROL << 8) | (OTHER << 16)) :
	    (CASE4 | (PRIMCONTROL << 8) | (OTHER << 16)); // still more cases?
	continue loop;
      case CASE4:
	ip = istack[--isp]; // pop where to go back to
	isp--; // pop the case ilist
	sp--; // pop the test value
	here = iinst[ip++];
	return;
      case IGNORE: --sp; here = iinst[ip++]; return;
      case LOOP: istack[isp++] = (LOOP | (PRIMCONTROL << 8) | (OTHER << 16)); // come here after
	ip = istack[isp - 2]; // list to run
	break;
      case EVERY:
	x = aDouble(ostack[--sp]) * 1000.0;
	ostack[sp++] = new Double(x); // time delay
	ip = istack[isp-1]; // list to run
	istack[isp++] = (EVERY2 | (PRIMCONTROL << 8) | (OTHER << 16)); // where to go at EOL
	break;
      case EVERY2:
	x = aDouble(ostack[sp-1]);
	ostack[sp++] = new Double((double)System.currentTimeMillis() + x);
      case EVERY3:
	if (System.currentTimeMillis() < aDouble(ostack[sp-1])) {
	  here = (EVERY3 | (PRIMCONTROL << 8) | (OTHER << 16)); return;
	}
	else {
	  --sp; // pop the start time
	  ip = istack[isp-1]; // list to run
	  istack[isp++] = (EVERY2 | (PRIMCONTROL << 8) | (OTHER << 16)); // where to go at EOL
	  break;
	}
      case WAITUNTIL:
	istack[isp++] = ip;
	ip = istack[isp-2]; // list to run
	istack[isp++] = (WAITUNTIL2 | (PRIMCONTROL << 8) | (OTHER << 16)); // where to go at EOL
	break;
      case WAITUNTIL2:
	testp = aBoolean(ostack[--sp]);
	if (testp) {
	  ip = istack[--isp];
	  --isp; // pop the list to run
	  here = iinst[ip++]; return;
	} else {
	  ip = istack[isp-2];
	  istack[isp++] = (WAITUNTIL2 | (PRIMCONTROL << 8) | (OTHER << 16));
	  here = iinst[ip++]; return;
	}
      case REPEAT: istack[isp++] = ip; // where to go after
	istack[isp++] = aInteger(ostack[--sp]); // repcount
      case REPEAT2:
	tempint = istack[isp-1]; // repcount
	if (tempint > 0) {
	  istack[isp-1] = tempint-1; // repcount
	  ip = istack[isp-3]; // list to run
	  istack[isp++] = (REPEAT2 | (PRIMCONTROL << 8) | (OTHER << 16)); // where to go at EOL
	  break;
	}
	else {
	  --isp; ip = istack[--isp]; --isp;
	  here = iinst[ip++]; return;
	}
	case DOTIMES:
	    istack[isp++] = iinst[ip++]; // local variable offset
	    istack[isp++] = aInteger(ostack[--sp]); // number of times to execute
	    istack[isp++] = ip; // where to go after
	    ostack[sp++] = ostack[process.fp-1-istack[isp-3]]; // remember old local value
	    ostack[process.fp-1-istack[isp-3]] = zeroInt; // initialize the value to zero
	case DOTIMES2:
	    tempint = aInteger(ostack[process.fp-1-istack[isp-3]]) + 1;
	    if (tempint <= istack[isp-2]) { // still more to run
		ostack[process.fp-1-istack[isp-3]] = new Integer(tempint);
		ip = istack[isp-4]; // list to run
		istack[isp++] = (DOTIMES2 | (PRIMCONTROL << 8) | (OTHER << 16)); //where to go at EOL
		break;
	    } else {
		ip = istack[--isp]; --isp;
		ostack[process.fp-1-istack[--isp]] = ostack[--sp];
		--isp; // pop list to run
		here = iinst[ip++]; return;
	    }
	case DOLIST:
	    istack[isp++] = iinst[ip++]; // local variable offset
	    istack[isp++] = sp-1; // place on the stack where the list is.
	    istack[isp++] = 0; // elt of list we're at.
	    istack[isp++] = ip; // where to go after
	    ostack[sp++] = ostack[process.fp-1-istack[isp-4]]; // remember old local value
	    list = aList(ostack[sp-2]);
	    if (list.length == 0) {
		isp-=5; sp -= 2;
		here = iinst[ip++]; return;
	    }
	case DOLIST2:
	    list = (Object[])ostack[istack[isp-3]];
	    if (istack[isp-2] < list.length) { // still more to do
		ostack[process.fp-1-istack[isp-4]] = list[istack[isp-2]++];
		ip = istack[isp-5]; // list to run
		istack[isp++] = (DOLIST2 | (PRIMCONTROL << 8) | (OTHER << 16)); //where to go at EOL
		break;
	    } else {
		ip = istack[--isp]; isp-=2;
		ostack[process.fp-1-istack[--isp]] = ostack[--sp];
		--isp; --sp;
		here = iinst[ip++]; return;
	    }
      case WAIT:
	x = aDouble(ostack[--sp]) * 1000.0;
	process.waittimer = System.currentTimeMillis() + (long)x;
      case WAIT2:
	if (System.currentTimeMillis() < process.waittimer) {
	  here = (WAIT2 | (PRIMCONTROL << 8) | (OTHER << 16));
	}
	else {
	  here = iinst[ip++];
	}
	return;
      case RESETT:
	process.timer = System.currentTimeMillis();
	here = iinst[ip++]; return;
      case TIMER:
	ostack[sp++] = new Double(((double)System.currentTimeMillis() - process.timer) / 1000.0);
	break;
	}
	break;
      case PRIMMATH:
	/* Math and Logic Operations */
	switch (here & 0xFF) {
	case ADD: y = aDouble(ostack[--sp]);
	  x = aDouble(ostack[--sp]);
	  ostack[sp++] = new Double(x+y); break;
	case MINUS: y = aDouble(ostack[--sp]);
	  x = aDouble(ostack[--sp]);
	  ostack[sp++] = new Double(x-y); break;
	case TIMES: y = aDouble(ostack[--sp]);
	  x = aDouble(ostack[--sp]);
	  ostack[sp++] = new Double(x*y); break;
	case DIV: y = aDouble(ostack[--sp]);
	  x = aDouble(ostack[--sp]);
	  ostack[sp++] = (y==0) ? new Double(0) : new Double(x/y);
	  break;
	case INTDIV: y = aDouble(ostack[--sp]);
	  x = aDouble(ostack[--sp]);
	  ostack[sp++] = (y==0) ? zeroInt : new Integer((int)(x/y));
	  break;
	case MOD: y = aDouble(ostack[--sp]);
	  x = aDouble(ostack[--sp]);
	  dl = (y==0) ? 0 : x%y;
	  if (dl<0) dl+=y; ostack[sp++] = new Double(dl);
	  break;
	case RANDOM:
	  x = aInteger(ostack[--sp]);
	  ostack[sp++] = new Integer((int)
				     Math.floor(rand.nextDouble()*x));
	  break;
	case RANDOMG: x = aInteger(ostack[--sp]);
	ostack[sp++] = new Integer((int)
				   Math.round(rand.nextGaussian()*x));
	break;
      case SETRAND:
	tempint = aInteger(ostack[--sp]);
	seed = tempint;
	rand.setSeed((long)tempint);
	here = iinst[ip++]; return;
      case GETRAND:
	ostack[sp++] = new Integer(seed);
	break;
      case POWER: y = aDouble(ostack[--sp]);
	x = aDouble(ostack[--sp]);
	ostack[sp++] = new Double(Math.pow(x,y)); break;
      case EXP: x = aDouble(ostack[--sp]);
	ostack[sp++] = new Double(Math.exp(x)); break;
      case LN: x = aDouble(ostack[--sp]);
	ostack[sp++] = new Double(Math.log(x)); break;
			case LOG: y = aDouble(ostack[--sp]);
	x = aDouble(ostack[--sp]);
	ostack[sp++] = new Double(Math.log(x) / Math.log(y)); break;
      case SIN: x = aDouble(ostack[--sp]);
	ostack[sp++] = new Double(Math.sin(deg2Rad(x))); break;
      case COS: x = aDouble(ostack[--sp]);
	ostack[sp++] = new Double(Math.cos(deg2Rad(x))); break;
			case TAN: x = aDouble(ostack[--sp]);
	ostack[sp++] = new Double(Math.tan(deg2Rad(x))); break;
			case ASIN: x = aDouble(ostack[--sp]);
	ostack[sp++] = new Double(rad2Deg(Math.asin(x))); break;
      case ACOS: x = aDouble(ostack[--sp]);
	ostack[sp++] = new Double(rad2Deg(Math.acos(x))); break;
      case ATAN: y = aDouble(ostack[--sp]);
	x = aDouble(ostack[--sp]);
	ostack[sp++] = new Double(rad2Deg(Math.atan2(x,y))); break;
      case SQRT: x = aDouble(ostack[--sp]);
	ostack[sp++] = new Double(Math.sqrt(x)); break;
      case ABS: x = aDouble(ostack[--sp]);
	ostack[sp++] = new Double((x<0) ? -x : x); break;
      case MAX: y = aDouble(ostack[--sp]);
	x = aDouble(ostack[--sp]);
	ostack[sp++] = new Double((x<y) ? y : x); break;
      case MIN: y = aDouble(ostack[--sp]);
	x = aDouble(ostack[--sp]);
	ostack[sp++] = new Double((x<y) ? x : y); break;
      case GT: y = aDouble(ostack[--sp]);
	x = aDouble(ostack[--sp]);
	ostack[sp++] = (x>y) ? YoYo.symtrue
	  : YoYo.symfalse;
	break;
      case LESST: y = aDouble(ostack[--sp]);
	x = aDouble(ostack[--sp]);
	ostack[sp++] = (x<y) ? YoYo.symtrue
	  : YoYo.symfalse;
	break;
      case EQ: thing1 = ostack[--sp];
	thing2 = ostack[--sp];
	ostack[sp++] = YoYo.equal(thing1, thing2);
	break;
	case NEQ:
	case NEQ2:
	  thing1 = ostack[--sp];
	thing2 = ostack[--sp];
	ostack[sp++] = (YoYo.equal(thing1, thing2) == YoYo.symtrue)
	  ? YoYo.symfalse : YoYo.symtrue;
	break;
      case LE: y = aDouble(ostack[--sp]);
	x = aDouble(ostack[--sp]);
	ostack[sp++] = (x<=y) ? YoYo.symtrue
	  : YoYo.symfalse;
	break;
      case GE: y = aDouble(ostack[--sp]);
	x = aDouble(ostack[--sp]);
	ostack[sp++] = (x>=y) ? YoYo.symtrue
	  : YoYo.symfalse;
	break;
      case ROUND: x = Math.round(aDouble(ostack[sp-1]));
	ostack[sp-1] = new Integer((int)x); break;
      case INT: x = aDouble(ostack[sp-1]);
	ostack[sp-1] = new Integer((int)x);
	break;
      case NOT: thing1 = ostack[--sp];
	ostack[sp++] = (aBoolean(thing1))
	  ? YoYo.symfalse : YoYo.symtrue; break;
      case AND: thing1 = ostack[--sp];
	thing2 = ostack[--sp];
	ostack[sp++] = (aBoolean(thing1) &&
			aBoolean(thing2))
	  ? YoYo.symtrue : YoYo.symfalse; break;
      case OR: thing1 = ostack[--sp];
	thing2 = ostack[--sp];
	ostack[sp++] = (aBoolean(thing1) ||
			aBoolean(thing2))
	  ? YoYo.symtrue : YoYo.symfalse; break;
      case XOR: thing1 = ostack[--sp];
	thing2 = ostack[--sp];
	ostack[sp++] = (aBoolean(thing1) ^
			aBoolean(thing2))
	  ? YoYo.symtrue : YoYo.symfalse; break;
      case BITNOT: tempint = aInteger(ostack[--sp]);
	ostack[sp++] = new Integer(~tempint); break;
      case BITAND: tempint = aInteger(ostack[--sp]);
	tempint2 = aInteger(ostack[--sp]);
	ostack[sp++] = new Integer(tempint & tempint2); break;
      case BITOR: tempint = aInteger(ostack[--sp]);
	tempint2 = aInteger(ostack[--sp]);
	ostack[sp++] = new Integer(tempint | tempint2); break;
      case BITXOR: tempint = aInteger(ostack[--sp]);
	tempint2 = aInteger(ostack[--sp]);
	ostack[sp++] = new Integer(tempint ^ tempint2); break;
	case MINNUM: ostack[sp++] = new Double(-Double.MAX_VALUE); break;
	case MAXNUM: ostack[sp++] = new Double(Double.MAX_VALUE); break;

	}
	break;
      case PRIMGLOBALVAR:
	/* Observer Variables */
	if (((here & 0x00010000) >> 16) == 0) {
	  primGlobalVar(c, obs, tur, pat);
	  return;
	} else {
	  switch (here & 0xFF) {
	  case OGETG:
	      offset = aInteger(ostack[--sp]);
	      ostack[sp++] = obs.turtlesown[offset];
	      break;
	  case TPGETG:
	      offset = aInteger(ostack[--sp]);
	      ostack[sp++] = sl.observer.turtlesown[offset];
	      break;
	  }
	}
	break;
      case PRIMTURTLEVAR:
	/* Turtle Variables */
	if (((here & 0x00010000) >> 16) == 0) {
	  primTurtleVar(c, obs, tur, pat);
	  return;
	} else {
	  switch (here & 0xFF) {
	case TGETT:
	  offset = aInteger(ostack[--sp]);
	  ostack[sp++] = tur.turtlesown[offset];
	  break;
	case PGETT:
	  offset = aInteger(ostack[--sp]);
	  if (pat.topturtle == null) {
	    if (pat.tophiddenturtle == null) {
	    	ostack[sp++] = zeroInt;
	  	} else {
		    ostack[sp++] = pat.tophiddenturtle.turtlesown[offset];
	  	}
	  }	else {
	    ostack[sp++] = pat.topturtle.turtlesown[offset];
	  }
	  break;
	case GETTOF:
	  st = sl.getWho(aInteger(ostack[--sp]));
	  offset = aInteger(ostack[--sp]);
	  if (st != null) { ostack[sp++] = st.turtlesown[offset]; }
	  else ostack[sp++] = new Integer(0);
	  break;
	case OGETTAT:
	  y = sl.halfheight - aDouble(ostack[--sp]);
	  x = sl.halfwidth + aDouble(ostack[--sp]);
	  tempint = sl.patchat(x, y);
	  st = sl.patches[tempint].topturtle;
	  offset = aInteger(ostack[--sp]);
	  if (st == null) st = sl.patches[tempint].tophiddenturtle;
	  if (st != null) ostack[sp++] = st.turtlesown[offset];
	  else ostack[sp++] = zeroInt;
	  break;
	case PGETTAT:
	  y = pat.ycor - 1 - aDouble(ostack[--sp]);
	  x = pat.xcor - 1 + aDouble(ostack[--sp]);
	  tempint = sl.patchat(x, y);
	  st = sl.patches[tempint].topturtle;
	  offset = aInteger(ostack[--sp]);
	  if (st == null) st = sl.patches[tempint].tophiddenturtle;
	  if (st != null) ostack[sp++] = st.turtlesown[offset];
	  else ostack[sp++] = zeroInt;
	  break;
	case TGETTAT:
	  y = tur.ycor - aDouble(ostack[--sp]);
	  x = tur.xcor + aDouble(ostack[--sp]);
	  offset = aInteger(ostack[--sp]);
	  tempint = sl.patchat(x, y);
	  st = sl.patches[tempint].topturtle;
	  if (st == null) st = sl.patches[tempint].tophiddenturtle;
	  if (st != null) ostack[sp++] = st.turtlesown[offset];
	  else ostack[sp++] = zeroInt;
	  break;
	case OGETTTOWARDS:
	  y = aDouble(ostack[--sp]); // distance
	  x = aDouble(ostack[--sp]); // angle
	  tempint = sl.patchtowards(x, y, sl.halfwidth, sl.halfheight);
	  st = sl.patches[tempint].topturtle;
	  offset = aInteger(ostack[--sp]);
	  if (st == null) st = sl.patches[tempint].tophiddenturtle;
	  if (st != null) ostack[sp++] = st.turtlesown[offset];
	  else ostack[sp++] = zeroInt;
	  break;
	case TGETTTOWARDS:
	  y = aDouble(ostack[--sp]); // distance
	  x = aDouble(ostack[--sp]); // angle
	  tempint = sl.patchtowards(x + tur.heading, y, tur.xcor, tur.ycor);
	  st = sl.patches[tempint].topturtle;
	  offset = aInteger(ostack[--sp]);
	  if (st == null) st = sl.patches[tempint].tophiddenturtle;
	  if (st != null) ostack[sp++] = st.turtlesown[offset];
	  else ostack[sp++] = zeroInt;
	  break;
	case PGETTTOWARDS:
	  y = aDouble(ostack[--sp]); // distance
	  x = aDouble(ostack[--sp]); // angle
	  tempint = sl.patchtowards(x, y, pat.xcor-1, pat.ycor-1);
	  st = sl.patches[tempint].topturtle;
	  offset = aInteger(ostack[--sp]);
	  if (st == null) st = sl.patches[tempint].tophiddenturtle;
	  if (st != null) ostack[sp++] = st.turtlesown[offset];
	  else ostack[sp++] = zeroInt;
	  break;
	  }
	}
	break;
      case PRIMPATCHVAR:
	if (((here & 0x00010000) >> 16) == 0) {
	  primPatchVar(c, obs, tur, pat);
	  return;
	} else {
	  switch (here & 0xFF) {
      case TGETP:
	offset = aInteger(ostack[--sp]);
	ostack[sp++] = sl.patches[tur.patch].turtlesown[offset];
	break;
      case PGETP:
	offset = aInteger(ostack[--sp]);
	ostack[sp++] = pat.turtlesown[offset];
	break;
      case OGETPAT:
	y = sl.halfheight - aDouble(ostack[--sp]);
	x = sl.halfwidth + aDouble(ostack[--sp]);
	offset = aInteger(ostack[--sp]);
	tempint = sl.patchat(x, y);
	ostack[sp++] = sl.patches[tempint].turtlesown[offset];
	break;
      case PGETPAT:
	y = pat.ycor - 1 - aDouble(ostack[--sp]);
	x = pat.xcor - 1 + aDouble(ostack[--sp]);
	offset = aInteger(ostack[--sp]);
	tempint = sl.patchat(x, y);
	ostack[sp++] = sl.patches[tempint].turtlesown[offset];
	break;
      case TGETPAT:
	y = tur.ycor - aDouble(ostack[--sp]);
	x = tur.xcor + aDouble(ostack[--sp]);
	offset = aInteger(ostack[--sp]);
	tempint = sl.patchat(x, y);
	ostack[sp++] = sl.patches[tempint].turtlesown[offset];
	break;
      case OGETPTOWARDS:
	y = aDouble(ostack[--sp]); // distance
	x = aDouble(ostack[--sp]); // angle
	tempint = sl.patchtowards(x, y, sl.halfwidth, sl.halfheight);
	offset = aInteger(ostack[--sp]);
	ostack[sp++] = sl.patches[tempint].turtlesown[offset];
	break;
      case TGETPTOWARDS:
	y = aDouble(ostack[--sp]); // distance
	x = aDouble(ostack[--sp]); // angle
	tempint = sl.patchtowards(x + tur.heading, y, tur.xcor, tur.ycor);
	offset = aInteger(ostack[--sp]);
	ostack[sp++] = sl.patches[tempint].turtlesown[offset];
	break;
      case PGETPTOWARDS:
	y = aDouble(ostack[--sp]); // distance
	x = aDouble(ostack[--sp]); // angle
	tempint = sl.patchtowards(x, y, pat.xcor-1, pat.ycor-1);
	offset = aInteger(ostack[--sp]);
	ostack[sp++] = sl.patches[tempint].turtlesown[offset];
	break;
	  }
	}
	break;
      case PRIMPATCHSTATE:
	if (((here & 0x00010000) >> 16) == 0) {
	  primPatchState(c, obs, tur, pat);
	  return;
	} else {
	  switch (here & 0xFF) {
	    /* Patch State Variables */
      case TPC:
	ostack[sp++] = new Double(sl.patches[tur.patch].pc);
	break;
      case PPC:
	ostack[sp++] = new Double(pat.pc);
	break;
      case OPCAT:
	y = sl.halfheight - aDouble(ostack[--sp]);
	x = sl.halfwidth + aDouble(ostack[--sp]);
	tempint = sl.patchat(x, y);
	ostack[sp++] = new Double(sl.patches[tempint].pc);
	break;
      case PPCAT:
	y = pat.ycor - 1 - aDouble(ostack[--sp]);
	x = pat.xcor - 1 + aDouble(ostack[--sp]);
	tempint = sl.patchat(x, y);
	ostack[sp++] = new Double(sl.patches[tempint].pc);
	break;
      case TPCAT:
	y = tur.ycor - aDouble(ostack[--sp]);
	x = tur.xcor + aDouble(ostack[--sp]);
	tempint = sl.patchat(x, y);
	ostack[sp++] = new Double(sl.patches[tempint].pc);
	break;

      case OPCTOWARDS:
	y = aDouble(ostack[--sp]); // distance
	x = aDouble(ostack[--sp]); // angle
	tempint = sl.patchtowards(x, y, sl.halfwidth, sl.halfheight);
	ostack[sp++] = new Double(sl.patches[tempint].pc);
	break;
      case TPCTOWARDS:
	y = aDouble(ostack[--sp]); // distance
	x = aDouble(ostack[--sp]); // angle
	tempint = sl.patchtowards(x + tur.heading, y, tur.xcor, tur.ycor);
	ostack[sp++] = new Double(sl.patches[tempint].pc);
	break;
      case PPCTOWARDS:
	y = aDouble(ostack[--sp]); // distance
	x = aDouble(ostack[--sp]); // angle
	tempint = sl.patchtowards(x, y, pat.xcor-1, pat.ycor-1);
	ostack[sp++] = new Double(sl.patches[tempint].pc);
	break;
	  }
	}
	break;
      case PRIMTURTLESTATE:
	switch (here & 0xFF) {
	/* Turtle State Variables */
      case TXCOR:
	ostack[sp++] = new Double(tur.xcor-sl.turhalfwidth);
	break;
      case PXCOR:
	ostack[sp++] = new Double(pat.xcor-1-sl.halfwidth);
	break;
      case TYCOR:
	ostack[sp++] = new Double(sl.turhalfheight-tur.ycor);
	break;
      case PYCOR:
	ostack[sp++] = new Double(sl.halfheight-(pat.ycor-1));
	break;
      case SETX:
	//System.out.println("setx");
	sl.unlink(tur);
	tur.xcor = sl.turhalfwidth+aDouble(ostack[--sp]);
	sl.wrapandlink(tur);
	here = iinst[ip++]; return;
      case SETY:
	sl.unlink(tur);
	tur.ycor= sl.turhalfheight-aDouble(ostack[--sp]);
	sl.wrapandlink(tur);
	here = iinst[ip++]; return;
      case SETXY:
	sl.unlink(tur);
	tur.ycor = sl.turhalfheight - aDouble(ostack[--sp]);
	tur.xcor = sl.turhalfwidth + aDouble(ostack[--sp]);
	sl.wrapandlink(tur);
	here = iinst[ip++]; return;
      case HOME:
	sl.unlink(tur);
	tur.ycor = sl.turhalfheight;
	tur.xcor = sl.turhalfwidth;
	sl.wrapandlink(tur);
	here = iinst[ip++]; return;
      case DX:
	ostack[sp++] = new Double(tur.xdelta);
	break;
      case DY:
	ostack[sp++] = new Double(-tur.ydelta);
	break;
	case LEAP: {
	  dl = aDouble(ostack[--sp]);
	  Patch p = sl.patches[sl.patchat(tur.xcor + (tur.xdelta * dl),
					  tur.ycor + (tur.ydelta * dl))];
	  if ((p.topturtle == null || p.topturtle == tur) && 
	      (p.tophiddenturtle == null || p.tophiddenturtle == tur)) {
	    sl.unlink(tur);
	    tur.xcor+=tur.xdelta * dl;
	    tur.ycor+=tur.ydelta * dl;
	    sl.wrapandlink(tur);
	  }
	}
	here = iinst[ip++]; return;
			case ALIVE:
	ostack[sp++] = (tur.alive) ? YoYo.symtrue : YoYo.symfalse;
	break;
			case ALIVEOF:
	st = sl.getWho(aInteger(ostack[--sp]));
	if ((st == null) || (st.alive == false))
		ostack[sp++] = YoYo.symfalse;
	else
		ostack[sp++] = YoYo.symtrue;
	break;
      case JUMP:
	dl = aDouble(ostack[--sp]);
	sl.unlink(tur);
	tur.xcor+=tur.xdelta * dl;
	tur.ycor+=tur.ydelta * dl;
	sl.wrapandlink(tur);
	here = iinst[ip++]; return;
      case STEP:
	if (tur.pendownp) {
	  sl.patches[tur.patch].pc = tur.color;
	}
	sl.unlink(tur);
	tur.xcor += tur.xdelta;
	tur.ycor += tur.ydelta;
	sl.wrapandlink(tur);
	if (tur.pendownp) {
	  sl.patches[tur.patch].pc = tur.color;
	}
	here = iinst[ip++]; return;
      case FD:
	//System.out.println("fd");

	dl = aDouble(ostack[sp-1]);
	if (dl < 0) {
	  ostack[sp-1] = new Double(-dl);
	  here = (BK | (PRIMTURTLESTATE << 8) | (COMMAND << 16)); continue loop;
	}
	if (dl == 0.0) {
	  sp--;
	  here = iinst[ip++];
	  return;
	}
	tempint = (int)dl;
	istack[isp++] = tempint; // steps
	ostack[sp-1] = new Double(dl - (double)tempint); // fraction
      case FD2:
	if (istack[isp-1]-- == 0) {
	  dl = aDouble(ostack[--sp]);
	  if (tur.pendownp) {
	    sl.patches[tur.patch].pc = tur.color;
	  }
	  sl.unlink(tur);
	  tur.xcor += tur.xdelta*dl; tur.ycor += tur.ydelta*dl;
	  sl.wrapandlink(tur);
	  if (tur.pendownp) {
	    sl.patches[tur.patch].pc = tur.color;
	  }
	  isp--;
	  here = iinst[ip++];
	  return;
	}
	if (tur.pendownp) {
	  sl.patches[tur.patch].pc = tur.color;
	}
	sl.unlink(tur);
	tur.xcor += tur.xdelta; tur.ycor += tur.ydelta;
	sl.wrapandlink(tur);
	if (tur.pendownp) {
	  sl.patches[tur.patch].pc = tur.color;
	}
	here = (FD2 | (PRIMTURTLESTATE << 8) | (OTHER << 16)); return;
      case BK:
	dl = aDouble(ostack[sp-1]);
	if (dl < 0) {
	  ostack[sp-1] = new Double(-dl);
	  here = (FD | (PRIMTURTLESTATE << 8) | (COMMAND << 16)); continue loop;
	}
	if (dl == 0.0) {
	  sp--;
	  here = iinst[ip++];
	  return;
	}
	tempint = (int)dl;
	istack[isp++] = tempint; // steps
	ostack[sp-1] = new Double(dl - (double)tempint); // fraction
      case BK2:
	if (istack[isp-1]-- == 0) {
	  dl = aDouble(ostack[--sp]);
	  if (tur.pendownp) {
	    sl.patches[tur.patch].pc = tur.color;
	  }
	  sl.unlink(tur);
	  tur.xcor -= tur.xdelta*dl; tur.ycor -= tur.ydelta*dl;
	  sl.wrapandlink(tur);
	  if (tur.pendownp) {
	    sl.patches[tur.patch].pc = tur.color;
	  }
	  isp--;
	  here = iinst[ip++];
	  return;
	}
	if (tur.pendownp) {
	  sl.patches[tur.patch].pc = tur.color;
	}
	sl.unlink(tur);
	tur.xcor -= tur.xdelta; tur.ycor -= tur.ydelta;
	sl.wrapandlink(tur);
	if (tur.pendownp) {
	  sl.patches[tur.patch].pc = tur.color;
	}
	here = (BK2 | (PRIMTURTLESTATE << 8) | (OTHER << 16)); return;
      case TCOLOR:
	ostack[sp++] = new Double(tur.color);
	break;
      case PCOLOR:
	if (pat.topturtle == null) {
	   if (pat.tophiddenturtle == null) {
	     ostack[sp++] = zeroInt;
	   } else {
	     ostack[sp++] = new Double(pat.tophiddenturtle.color);
	   }
	} else ostack[sp++] = new Double(pat.topturtle.color);
	break;
      case TSETC:
	tur.color = aDouble(ostack[--sp])%Colors.numcolors;
	tur.color = (tur.color<0) ? tur.color+Colors.numcolors : tur.color;
	if (tur == sl.patches[tur.patch].topturtle) {
	  //System.out.println("draw turtle " + this);
	  sl.patchcanvas.drawTurtle(tur);
	}
	here = iinst[ip++]; return;
      case PSETC:
	st = pat.topturtle;
	if (st == null) st = pat.tophiddenturtle;
	  if (st == null) {
	  	sp--;
	  } else {
		  x = aDouble(ostack[--sp])%Colors.numcolors;
		  st.color = (x<0)
		    ? x+Colors.numcolors : x;
		  if (st.shownp) sl.patchesneeddrawingp = true;
	  }
	here = iinst[ip++]; return;
      case TSCALECOLOR:
	x = aDouble(ostack[--sp]); // max
	y = aDouble(ostack[--sp]); // min
	dl = aDouble(ostack[--sp]); // value
	x = x - y; // range
	y = dl - y; // diff from min
	dl = y / x; // diif / range
	if (dl < 0.0) {
	  tur.color = 0;
	} else {
	  if (dl > 1) dl = 1;
	  dl = dl * 9.999999999;
	  x = aDouble(ostack[--sp])%Colors.numcolors;  // color
	  x = (x<0) ? x+Colors.numcolors : x;
	  tur.color = (((int)(x / 10)) * 10.0) + dl;
	}
	if (tur == sl.patches[tur.patch].topturtle)
	  sl.patchcanvas.drawTurtle(tur);
	here = iinst[ip++]; return;
      case PSCALECOLOR:
         st = pat.topturtle;
         if (st == null) st = pat.tophiddenturtle;
	     if (st == null) {
	        sp-=4;
	     } else {
	       x = aDouble(ostack[--sp]); // max
	       y = aDouble(ostack[--sp]); // min
	       dl = aDouble(ostack[--sp]); // value
	       x = x - y; // range
	       y = dl - y; // diff from min
	       dl = y / x; // diif / range
	       if (dl < 0.0) {
		 st.color = 0;
	       } else {
		 if (dl > 1) dl = 1;
		 dl = dl * 9.999999999;
		 x = aDouble(ostack[--sp])%Colors.numcolors;  // color
		 x = (x<0) ? x+Colors.numcolors : x;
		 st.color = (((int)(x / 10)) * 10.0) + dl;
	       }
	       if (st.shownp) sl.patchesneeddrawingp = true;
	     }
	     here = iinst[ip++]; return;
      case WHO:
	ostack[sp++] = tur.whoInt;
	break;
      case PD:
	tur.pendownp = true;
	here = iinst[ip++]; return;
      case PU:
	//System.out.println("pu");
	tur.pendownp = false;
	here = iinst[ip++]; return;
      case PENDOWNP:
	ostack[sp++] = (tur.pendownp) ? YoYo.symtrue : YoYo.symfalse;
	break;
      case SETH:
	tur.setheading(aDouble(ostack[--sp]));
	//not sure this will work
	if (tur == sl.patches[tur.patch].topturtle)
	  sl.patchcanvas.drawTurtle(tur);
	here = iinst[ip++]; return;
      case RT:
	tur.setheading(tur.heading+aDouble(ostack[--sp]));
	//not sure this will work
	if (tur == sl.patches[tur.patch].topturtle)
	  sl.patchcanvas.drawTurtle(tur);
	here = iinst[ip++]; return;
      case LT:
	tur.setheading(tur.heading-aDouble(ostack[--sp]));
	//not sure this will work
	if (tur == sl.patches[tur.patch].topturtle)
	  sl.patchcanvas.drawTurtle(tur);
	here = iinst[ip++]; return;
      case THEADING:
	ostack[sp++] = new Double(tur.heading);
	break;
      case PHEADING:
         st = pat.topturtle;
         if (st == null) st = pat.tophiddenturtle;
		 if (st == null) {
	        ostack[sp++] = zeroInt;
	     } else ostack[sp++] = new Double(st.heading);
	break;
      case HT:
	if (tur.shownp) {
	  tur.shownp = false;
	  sl.hideTurtle(tur);
	}
	here = iinst[ip++]; return;
      case ST:
	if (!tur.shownp) {
	  tur.shownp = true;
	  sl.showTurtle(tur);
	}
	here = iinst[ip++]; return;
      case SHOWNP:
	ostack[sp++] = (tur.shownp) ? YoYo.symtrue : YoYo.symfalse;
	break;
      case TBREED:
	ostack[sp++] = new Double(tur.breed);
	break;
      case TSETBREED:
	tur.breed = aDouble(ostack[--sp]);
	here = iinst[ip++]; return;
      case PBREED:
      	st = pat.topturtle;
      	if (st == null) st = pat.tophiddenturtle;
      	if (st == null) {
	  		ostack[sp++] = zeroInt;
		} else ostack[sp++] = new Double(st.breed);
	break;
      case PSETBREED:
		st = pat.topturtle;
		if (st == null) st = pat.tophiddenturtle;
		if (st == null) {
	  		sp--;
		} else st.breed = aDouble(ostack[--sp]);
	here = iinst[ip++]; return;
      case TSETSHAPE:
	tur.shape = 
	    (aInteger(ostack[--sp]))%sl.numshapes;
	tur.shape = (tur.shape<0) ? tur.shape+sl.numshapes : tur.shape;
	if (tur == sl.patches[tur.patch].topturtle)
	  sl.patchcanvas.drawTurtle(tur);
	here = iinst[ip++]; return;
      case TSHAPE:
	  ostack[sp++] = new Integer(tur.shape);
	break;
      case PSETSHAPE:
        st = pat.topturtle;
        if (st == null) st = pat.tophiddenturtle;
		if (st == null) {
	  		--sp;
		} else {
		  st.shape = (aInteger(ostack[--sp]))%sl.numshapes;
		  st.shape = (st.shape<0) ? st.shape + sl.numshapes : st.shape;
	  	  if (st.shownp) sl.patchcanvas.drawTurtle(st);
		}
		here = iinst[ip++]; return;
      case PSHAPE:
		st = pat.topturtle;
		if (st == null) st = pat.tophiddenturtle;
		if (st == null) {
	  		ostack[sp++] = zeroInt;
		} 
		else ostack[sp++] = new Integer(st.shape);
	break;
	}
	break;
      case PRIMTURTLEOTHERSTATE:
	if (((here & 0x00010000) >> 16) == 0) {
	  primTurtleOtherState(c, obs, tur, pat);
	  return;
	} else {
	  primTurtleOtherStateReporters(c, obs, tur, pat);
	  break;
	}
      case PRIMTOWARDS:
	/* Turtle Directions */
	switch (here & 0xFF) {
	case OTOWARDS:
	  y = aDouble(ostack[--sp]);
	  x = aDouble(ostack[--sp]);
	  if (x == 0 && y == 0) {
	    ostack[sp++] = zeroInt;
	    break;
	  }
	  if (y < 0) { trueip = -1; y = -y; } else { trueip = 1; }
	  if (x < 0) { falseip = -1; x = -x; } else { falseip = 1; }
	  if (y >= sl.halfheight) y = y - sl.height;
	  if (x >= sl.halfwidth) x = x - sl.width;
	  y = y * trueip; x = x * falseip;
	  ostack[sp++] = new Double(rad2Deg(Math.atan2(x,y)));
	  break;
	case TTOWARDS:
	  y = tur.ycor + aDouble(ostack[--sp]) - sl.turhalfheight;
	  x = sl.turhalfwidth + aDouble(ostack[--sp]) - tur.xcor;
	  if (x == 0 && y == 0) {
	    ostack[sp++] = new Double(tur.heading);
	    break;
	  }
	  if (y < 0) { trueip = -1; y = -y; } else { trueip = 1; }
	  if (x < 0) { falseip = -1; x = -x; } else { falseip = 1; }
	  if (y >= sl.turhalfheight) y = y - sl.height;
	  if (x >= sl.turhalfwidth) x = x - sl.width;
	  y = y * trueip; x = x * falseip;
	  ostack[sp++] = new Double(rad2Deg(Math.atan2(x,y)));
	  break;
      case PTOWARDS:
	y = pat.ycor -1+ aDouble(ostack[--sp]) - sl.halfheight;
	x = sl.halfwidth + aDouble(ostack[--sp]) - (pat.xcor -1);
	if (x == 0 && y == 0) {
	  ostack[sp++] = zeroInt;
	  break;
	}
	if (y < 0) { trueip = -1; y = -y; } else { trueip = 1; }
	if (x < 0) { falseip = -1; x = -x; } else { falseip = 1; }
	if (y >= sl.halfheight) y = y - sl.height;
	if (x >= sl.halfwidth) x = x - sl.width;
	y = y * trueip; x = x * falseip;
	ostack[sp++] = new Double(rad2Deg(Math.atan2(x,y)));
	break;
	    case OTOWARDSNOWRAP:
		y = aDouble(ostack[--sp]);
		x = aDouble(ostack[--sp]);
		if (x == 0 && y == 0) {
		    ostack[sp++] = zeroInt;
		    break;
		}
		if (y < 0) { trueip = -1; y = -y; } else { trueip = 1; }
		if (x < 0) { falseip = -1; x = -x; } else { falseip = 1; }
		y = y * trueip; x = x * falseip;
		ostack[sp++] = new Double(rad2Deg(Math.atan2(x,y)));
		break;
      case TTOWARDSNOWRAP:
	y = tur.ycor + aDouble(ostack[--sp]) - sl.turhalfheight;
	x = sl.turhalfwidth + aDouble(ostack[--sp]) - tur.xcor;
	if (x == 0 && y == 0) {
	  ostack[sp++] = new Double(tur.heading);
	  break;
	}
	if (y < 0) { trueip = -1; y = -y; } else { trueip = 1; }
	if (x < 0) { falseip = -1; x = -x; } else { falseip = 1; }
	y = y * trueip; x = x * falseip;
	ostack[sp++] = new Double(rad2Deg(Math.atan2(x,y)));
	break;
      case PTOWARDSNOWRAP:
	y = pat.ycor-1 + aDouble(ostack[--sp]) - sl.halfheight;
	x = sl.halfwidth + aDouble(ostack[--sp]) - (pat.xcor-1);
	if (x == 0 && y == 0) {
	  ostack[sp++] = zeroInt;
	  break;
	}
	if (y < 0) { trueip = -1; y = -y; } else { trueip = 1; }
	if (x < 0) { falseip = -1; x = -x; } else { falseip = 1; }
	y = y * trueip; x = x * falseip;
	ostack[sp++] = new Double(rad2Deg(Math.atan2(x,y)));
	break;
	    case ODISTANCE:
		y = aDouble(ostack[--sp]);
		if (y < 0) y = -y;
		if (y >= sl.halfheight) y = sl.height - y;
		x = aDouble(ostack[--sp]);
		if (x < 0) x = -x;
		if (x >= sl.halfwidth) x = sl.width - x;
		ostack[sp++] = new Double(Math.sqrt(x*x + y*y));
		break;
      case TDISTANCE:
	y = sl.turhalfheight - aDouble(ostack[--sp]);
	y = y - tur.ycor; if (y < 0) y = -y;
	if (y >= sl.turhalfheight) y = sl.height - y;
	x = sl.turhalfwidth + aDouble(ostack[--sp]);
	x = x - tur.xcor; if (x < 0) x = -x;
	if (x >= sl.turhalfwidth) x = sl.width - x;
	ostack[sp++] = new Double(Math.sqrt(x*x + y*y));
	break;
      case PDISTANCE:
	y = sl.halfheight - aDouble(ostack[--sp]);
	y = y - (pat.ycor-1); if (y < 0) y = -y;
	if (y >= sl.halfheight) y = sl.height - y;
	x = sl.halfwidth + aDouble(ostack[--sp]);
	x = x - (pat.xcor-1); if (x < 0) x = -x;
	if (x >= sl.halfwidth) x = sl.width - x;
	ostack[sp++] = new Double(Math.sqrt(x*x + y*y));
	break;
	    case ODISTANCENOWRAP:
		y = aDouble(ostack[--sp]);
		if (y < 0) y = -y;
		x = aDouble(ostack[--sp]);
		if (x < 0) x = -x;
		ostack[sp++] = new Double(Math.sqrt(x*x + y*y));
		break;
      case TDISTANCENOWRAP:
	y = sl.turhalfheight - aDouble(ostack[--sp]);
	y = y - tur.ycor; if (y < 0) y = -y;
	x = sl.turhalfwidth + aDouble(ostack[--sp]);
	x = x - tur.xcor; if (x < 0) x = -x;
	ostack[sp++] = new Double(Math.sqrt(x*x + y*y));
	break;
      case PDISTANCENOWRAP:
	y = sl.halfheight - aDouble(ostack[--sp]);
	y = y - (pat.ycor-1); if (y < 0) y = -y;
	x = sl.halfwidth + aDouble(ostack[--sp]);
	x = x - (pat.xcor-1); if (x < 0) x = -x;
	ostack[sp++] = new Double(Math.sqrt(x*x + y*y));
	break;
	}
	break;
      case PRIMLIFE:
	if (((here & 0x00010000) >> 16) == 0) {
	  primLife(c, obs, tur, pat);
	  return;
	} else {
	  switch (here & 0xFF) {
	  case HATCH2:
	    StackProcessParent.askTurtleRunForeverButtons(sl, tur);
	    sl.done(process);
	    return;
	  }
	}
	break;
      case PRIMGLOBALCOUNT:
	/* Global Turtle Reporters */
	switch (here & 0xFF) {
	case TONET:
	  ostack[sp++] = new Integer(sl.randomTurtle(tur));
	  break;
	case OPONET:
	  ostack[sp++] = new Integer(sl.randomTurtle());
	  break;
      case TONEB:
	x = aDouble(ostack[--sp]);
	ostack[sp++] = new Integer(sl.randomBreed(x, tur));
	break;
      case OPONEB:
	x = aDouble(ostack[--sp]);
	ostack[sp++] = new Integer(sl.randomBreed(x));
	break;
      case TLISTT:
	ostack[sp++] = sl.listTurtles(tur);
	break;
      case OPLISTT:
	ostack[sp++] = sl.listTurtles();
	break;
      case TLISTB:
	x = aDouble(ostack[--sp]);
	ostack[sp++] = sl.listBreeds(x, tur);
	break;
      case OPLISTB:
	x = aDouble(ostack[--sp]);
	ostack[sp++] = sl.listBreeds(x);
	break;
      case COUNTTURTLES:
	ostack[sp++] = new Integer(sl.numturtles);
	break;
	    case COUNTPATCHES:
	ostack[sp++] = new Integer(sl.height*sl.width);
	break;
      case COUNTBREEDS:
	x = aDouble(ostack[--sp]);
	ostack[sp++] = new Integer(sl.countBreeds(x));
	break;
      case COUNTCOLOR:
	x = aDouble(ostack[--sp]);
	ostack[sp++] = new Integer(sl.countColor(x));
	break;
      case COUNTPC:
	x = aDouble(ostack[--sp]);
	ostack[sp++] = new Integer(sl.countPC(x));
	break;
      case COUNTCOLORRANGE:
	y = aDouble(ostack[--sp]);
	x = aDouble(ostack[--sp]);
	ostack[sp++] = new Integer(sl.countColor(x, y));
	break;
      case COUNTPCRANGE:
	y = aDouble(ostack[--sp]);
	x = aDouble(ostack[--sp]);
	ostack[sp++] = new Integer(sl.countPC(x, y));
	break;	}
	break;
      case PRIMATTOWARDS:
	    primAtTowardsReporters(c, obs, tur, pat);
	    break;
      case PRIMHERE:
		primHereReporters(c, obs, tur, pat);
  		break;
      case PRIMSCREEN:
	switch (here & 0xFF) {
	/* Global Parameters */
	    case SCREENEDGEX:
		ostack[sp++] = new Integer(sl.halfwidth);
		break;
	    case SCREENEDGEY:
		ostack[sp++] = new Integer(sl.halfheight);
		break;
	    case SCREENWIDTH: ostack[sp++] = new Integer(sl.width); break;
	    case SCREENHEIGHT: ostack[sp++] = new Integer(sl.height); break;
	}
	break;
      case PRIMLIST:
	if (((here & 0x00010000) >> 16) == 0) {
		switch (here & 0xFF) {
		  case SETITEM: thing3 = ostack[--sp];
			thing2 = ostack[--sp];
			thing1 = ostack[--sp];
			PrimList.prim_setitem(thing1, thing2, thing3, (Context)null);
			here = iinst[ip++];
			break;
		}
	  return;
	} else {
	  primListReporters(c, obs, tur, pat);
	  break;
	}
	  case PRIMREFLECTION:
	switch (here & 0xFF) {
		/* Reflection Predicates */
	    case NUMBERP: thing1 = ostack[--sp];
		ostack[sp++] = (thing1 instanceof Number) ?
		    YoYo.symtrue : YoYo.symfalse; break;
	    case WORDP: thing1 = ostack[--sp];
		ostack[sp++] = PrimList.prim_wordp(thing1, (Context)null);
		break;
 	    case LISTP: thing1 = ostack[--sp];
		ostack[sp++] = PrimList.prim_listp(thing1, (Context)null);
		break;
	case STACKCRAWL:
	  process.saveState();
	  ostack[sp++] = process.stackCrawl();
	  break;

	}
	break;
      case PRIMGRAB:
	switch (here & 0xFF) {
		/* Grab */
	case GRAB:
	  if (grab(ostack[--sp])) {
	    //System.out.println(process.entity + " has a successful grab of " + process.entity.partner);
	    break;
	  }
	  else {
	    //System.out.println(process.entity + " fails to grab anyone.");
	    --isp; // pop list-to-run
	    here = iinst[ip++]; return;
	  }
      case GRAB2:
	ip = istack[--isp]; // restore old ip
	tempint = istack[--isp]; // number of partners to pop
	while(tempint-- > 0) {
	  --sp; // pop the grabbed marker
	  st = (Turtle)ostack[--sp]; // pop the partner
	  if (!st.alive) sl.recycleDeadWho(st);
	  st.grabber = null;
	}
	tempint = sp;
	while (tempint > (process.fp+1)) {
	  val = ostack[--tempint];
	  if (val == grabbedMarker) {
	    process.entity.partner = (Turtle)ostack[--tempint];
	    here = iinst[ip++];
	    return;
	  }
	}
	process.entity.partner = null;
	break;

      case OGRABPATCH:
	y = sl.halfheight - aDouble(ostack[--sp]);
	x = sl.halfwidth + aDouble(ostack[--sp]);
	tempint = sl.patchat(x, y);
	if (grabPatch(sl.patches[tempint])) {
	  break;
	} else {
	  --isp; // pop list-to-run
	  here = iinst[ip++]; return;
	}
      case PGRABPATCH:
	y = pat.ycor - 1 - aDouble(ostack[--sp]);
	x = pat.xcor - 1 + aDouble(ostack[--sp]);
	tempint = sl.patchat(x, y);
	if (grabPatch(sl.patches[tempint])) {
	  break;
	} else {
	  --isp; // pop list-to-run
	  here = iinst[ip++]; return;
	}
      case TGRABPATCH:
	y = tur.ycor - aDouble(ostack[--sp]);
	x = tur.xcor + aDouble(ostack[--sp]);
	tempint = sl.patchat(x, y);
	if (grabPatch(sl.patches[tempint])) {
	  break;
	} else {
	  --isp; // pop list-to-run
	  here = iinst[ip++]; return;
	}
      case GRABPATCH2:
	ip = istack[--isp]; // restore old ip
	tempint = istack[--isp]; // number of partners to pop
	while(tempint-- > 0) {
	  --sp; // pop the grabbed marker
	  Patch p = (Patch)ostack[--sp]; // pop the partner
	  p.grabbedp = false;
	}
	tempint = sp;
	while (tempint > (process.fp+1)) {
	  val = ostack[--tempint];
	  if (val == grabbedPatchMarker) {
	    process.entity.patchpartner = (Patch)ostack[--tempint];
	    here = iinst[ip++];
	    return;
	  }
	}
	process.entity.patchpartner = null;
	break;
	case PARTNER:
	if (process.entity.partner != null) {
	  ostack[sp++] = process.entity.partner.whoInt;
	} else ostack[sp++] = minusoneInt;
	break;
      case PARTNERS:
	tempint = sp; partners = new Vector();
	while (tempint > 0) {
	  val = ostack[--tempint];
	  if (val == grabbedMarker) {
	    partners.addElement(((Turtle)ostack[--tempint]).whoInt);
	  }
	}
	list = new Object[partners.size()];
	partners.copyInto(list);
	ostack[sp++] = list;
	break;
	case PATCHPARTNER:
	if (process.entity.patchpartner != null) {
	  Double[] coor = new Double[2];
	  coor[0] = new Double(process.entity.patchpartner.xcor - 1 - sl.halfwidth);
	  coor[1] = new Double(sl.halfheight - (process.entity.patchpartner.ycor - 1));
	  ostack[sp++] = coor;
	} else ostack[sp++] = YoYo.symfalse;
	break;
      case PATCHPARTNERS:
	tempint = sp; partners = new Vector();
	while (tempint > 0) {
	  val = ostack[--tempint];
	  if (val == grabbedPatchMarker) {
	    Patch p = (Patch)ostack[--tempint];
	    Double[] coor = new Double[2];
	    coor[0] = new Double(p.xcor - 1 - sl.halfwidth);
	    coor[1] = new Double(sl.halfheight - (p.ycor-1));
	    partners.addElement(coor);
	  }
	}
	list = new Object[partners.size()];
	partners.copyInto(list);
	ostack[sp++] = list;
	break;
	case GRABBEDP:
	  ostack[sp++] = (tur.grabber == null) ? YoYo.symfalse : YoYo.symtrue;
	  break;
	}
	break;
      case PRIMPRINT:
	if (((here & 0x00010000) >> 16) == 0) {
	  primPrint(c, obs, tur, pat);
	  return;
	} else {
	  //switch (here & 0xFF) {
	  //}
	}
	break;
      case PRIMNSUM:
	if (((here & 0x00010000) >> 16) == 0) {
	  primNsum(c, obs, tur, pat);
	  return;
	} else {
	  //switch (here & 0xFF) {
	  //}
	}
	break;
      case PRIMCLEAR:
	if (((here & 0x00010000) >> 16) == 0) {
	  primClear(c, obs, tur, pat);
	  return;
	} else {
	  //switch (here & 0xFF) {
	  //}
	}
	break;
      case PRIMASK:
	//System.out.println("primAsk");
	if (((here & 0x00010000) >> 16) == 0) {
	  //System.out.println("command");
	  primAsk(c, obs, tur, pat);
	  return;
	} else {
	  //System.out.println("not command");
	  switch (here & 0xFF) {
	  case ASK2:
	      if (process.vmspp != null && !process.vmspp.donep()) {
		  here = (ASK2 | (PRIMASK << 8) | (OTHER << 16)); return;
	      }
	      --sp; // pop the vmspp off the stack
	      process.vmspp = null;
	      here = iinst[ip++]; return;
	  case MYSELF:
	    //System.out.println("caller entity: " + process.callerentity);
	    ostack[sp++] = (process.callerentity != null &&
			    process.callerentity instanceof Turtle) ?
	      ((Turtle)process.callerentity).whoInt : minusoneInt;
	    break;
	  case CTW2:
	    //System.out.println("ctw2 vmspp = " + process.vmspp);
	    if (process.vmspp != null && !process.vmspp.donep()) {
	      //System.out.println("ctw2 not done yet");
	      here = (CTW2 | (PRIMASK << 8) | (OTHER << 16)); return;
	    }
	    else {
	      counter = 0;
	      for(Enumeration e = ((StackProcessParentOutput)process.vmspp).children();
		  e.hasMoreElements();) {
		StackProcess sp = (StackProcess)e.nextElement();
		//System.out.println("counting stack process: " + sp);
		if (aBoolean(sp.output())) counter++;
		//sp.unwindStack();
		sp.pool.dropStackProcess(sp);
	      }
	      //System.out.println("done counting: " + counter);
	      --sp; // pop the vmspp off the stack
	      process.vmspp = null;
	      ostack[sp++] = new Integer(counter);
	      //System.out.println("ip = " + ip);
	      here = iinst[ip++]; return;
	    }
	case MEDT2:
	    if (process.vmspp != null && !process.vmspp.donep()) {
	      here = (MEDT2 | (PRIMASK << 8) | (OTHER << 16)); return;
	    }
	    else {
	      double[] z1 = new double[((StackProcessParentOutput)process.vmspp).count];
	      double[] z2 = new double[((StackProcessParentOutput)process.vmspp).count];
	      int z = 0;
	      for(Enumeration e = ((StackProcessParentOutput)process.vmspp).children();
		  e.hasMoreElements();) {
		StackProcess sp = (StackProcess)e.nextElement();
		z1[z]=aDouble(sp.output());
		z++;
		//sp.unwindStack();
		sp.pool.dropStackProcess(sp);
	      }
	      z2 = PrimList.mergesort(z1, 0, z - 1);
	      --sp; // pop the vmspp off the stack
	      process.vmspp = null;
	      if ( (z & 1) == 0)
	     {ostack[sp++] = new Double((z2 [ z / 2] + z2 [(z /2) - 1]) / 2);}
	      else
	      {ostack[sp++] = new Double( z2 [ z / 2]);}
	      here = iinst[ip++]; return;
	    }
	  case MODET2:  
	      if (process.vmspp != null && !process.vmspp.donep()) {
	      here = (MEDT2 | (PRIMASK << 8) | (OTHER << 16)); return;
	      }
	    else {
		double[] numlist = new double[((StackProcessParentOutput)process.vmspp).count];
		Vector mode = new Vector(0,1);
		int z = 0;
                double currentelem = 0;
		int modesize = 0;
		for(Enumeration e = ((StackProcessParentOutput)process.vmspp).children();
		  e.hasMoreElements();) {
		    StackProcess sp = (StackProcess)e.nextElement();
		    numlist[z]=aDouble(sp.output());
		    z++;
		    //sp.unwindStack();
		    sp.pool.dropStackProcess(sp);
		}
		for (int j=0; j<numlist.length; j++) {
		    currentelem = numlist[j];
		    for (int i=0, currentsize=0; i<numlist.length; i++) {
			Double potential = new Double(numlist[i]);
			if (currentelem == numlist[i]) {
			    currentsize++;
			    if (currentsize == modesize) {
				if  (! (mode.contains(potential))) {mode.addElement(potential);}
			    }
			}
			if (currentsize>modesize) {
			    modesize=currentsize; 
			    mode=new Vector(); 
			    mode.addElement(new Double(numlist[i]));
			}
		    }
		}
	    
		--sp; // pop the vmspp off the stack
		process.vmspp = null;
		if (mode.size() == 1)
		    {ostack[sp++] = mode.firstElement();}
		else {
		    Object[] result = new Object[mode.size()];
		    mode.copyInto(result);
		    ostack[sp++] = result;}
		here = iinst[ip++]; return;
	    }
	  case SDEVT2:
	    if (process.vmspp != null && !process.vmspp.donep()) {
	      here = (SDEVT2 | (PRIMASK << 8) | (OTHER << 16)); return;
	    }
	    else {
	      double z = 0, tempz;
	      x = 0;
	      y = 0;
	      for(Enumeration e = ((StackProcessParentOutput)process.vmspp).children();
		  e.hasMoreElements();) {
		StackProcess sp = (StackProcess)e.nextElement();
		tempz = aDouble(sp.output());
		x += tempz;
		y += tempz * tempz;
		z++;
		//sp.unwindStack();
		sp.pool.dropStackProcess(sp);
	      }
	      --sp; // pop the vmspp off the stack
	      process.vmspp = null;
	      if (z > 1)
	      {ostack[sp++] = new Double(Math.sqrt((y - ((x * x) / z)) / (z - 1)));}
	      else
	      {ostack[sp++] = new Double(0);}
	      here = iinst[ip++]; return;
	    }
	  case VART2:
	    if (process.vmspp != null && !process.vmspp.donep()) {
	      here = (VART2 | (PRIMASK << 8) | (OTHER << 16)); return;
	    }
	    else {
	      double z = 0, tempz;
	      x = 0;
	      y = 0;
	      for(Enumeration e = ((StackProcessParentOutput)process.vmspp).children();
		  e.hasMoreElements();) {
		StackProcess sp = (StackProcess)e.nextElement();
		tempz = aDouble(sp.output());
		x += tempz;
		y += tempz * tempz;
		z++;
		//sp.unwindStack();
		sp.pool.dropStackProcess(sp);
	      }
	      --sp; // pop the vmspp off the stack
	      process.vmspp = null;
	      if (z > 1)
	      {ostack[sp++] = new Double((y - ((x * x) / z)) / (z - 1));}
	      else
	      {ostack[sp++] = new Double(0);}
	      here = iinst[ip++]; return;
	    }
	  case AVGT2:
	    if (process.vmspp != null && !process.vmspp.donep()) {
	      here = (AVGT2 | (PRIMASK << 8) | (OTHER << 16)); return;
	    }
	    else {
	      x = 0;
	      y = 0;
	      for(Enumeration e = ((StackProcessParentOutput)process.vmspp).children();
		  e.hasMoreElements();) {
		StackProcess sp = (StackProcess)e.nextElement();
		x += aDouble(sp.output());
		y++;
		//sp.unwindStack();
		sp.pool.dropStackProcess(sp);
	      }
	      --sp; // pop the vmspp off the stack
	      process.vmspp = null;
	      if (y > 0)
	      {ostack[sp++] = new Double(x / y);}
	      else
	      {ostack[sp++] = new Double(0);}
	      here = iinst[ip++]; return;
	    }
	  case OTW2:
	      //System.out.println("otw2 vmspp = " + process.vmspp);
	    if (process.vmspp != null && !process.vmspp.donep()) {
	      //System.out.println("ctw2 not done yet");
	      here = (OTW2 | (PRIMASK << 8) | (OTHER << 16)); return;
	    }
	    else {
	      //try {
	      counter = 0;
	      for(Enumeration e = ((StackProcessParentOutput)process.vmspp).children();
		  e.hasMoreElements();) {
			StackProcess sp = (StackProcess)e.nextElement();
			//System.out.println("counting stack process: " + sp);
			if (aBoolean(sp.output())) counter++;
			//sp.pool.dropStackProcess(sp);
	      }
	      if (counter > 0) {
	        //System.out.println("counter is " + counter);
			int num = (int)Math.floor(rand.nextDouble()*counter) + 1;
			//System.out.println("num is " + num);
			Turtle t = null;
			for(Enumeration e = ((StackProcessParentOutput)process.vmspp).children();
			    e.hasMoreElements();) {
			  StackProcess sp = (StackProcess)e.nextElement();
			  //System.out.println("counting stack process: " + sp);
			  if (num > 0 && aBoolean(sp.output())) {
			    num--;
			  	//System.out.println("yes, num = " + num);
			  	//System.out.println("turtle is " + sp.tur);
			    if (num == 0) {
			      t = sp.tur;
			    }
			  }
			  // sp.unwindStack();
			  sp.pool.dropStackProcess(sp);
			}
			//System.out.println("chosen turtle is " + t);
			--sp; // pop the vmspp off the stack
			process.vmspp = null;
			ostack[sp++] = t.whoInt;
		  } else {
		      --sp; // pop the vmspp off the stack
		      process.vmspp = null;
		      ostack[sp++] = minusoneInt;
		  }
	      //} catch (Exception e) { e.printStackTrace(); here = iinst[ip++]; return; }
	      here = iinst[ip++]; return;
	  }
	  case ST2:
	    if (process.vmspp != null && !process.vmspp.donep()) {
	      here = (ST2 | (PRIMASK << 8) | (OTHER << 16)); return;
	    }
	    else {
	      x = 0;
	      for(Enumeration e = ((StackProcessParentOutput)process.vmspp).children();
		  e.hasMoreElements();) {
		StackProcess sp = (StackProcess)e.nextElement();
		x += aDouble(sp.output());
		//sp.unwindStack();
		sp.pool.dropStackProcess(sp);
	      }
	      --sp; // pop the vmspp off the stack
	      ostack[sp++] = new Double(x);
	    process.vmspp = null;
	      here = iinst[ip++]; return;
	    }
	  case MINT2:
	    if (process.vmspp != null && !process.vmspp.donep()) {
	      here = (MINT2 | (PRIMASK << 8) | (OTHER << 16)); return;
	    }
	    else {
	      x = Double.MAX_VALUE;
	      for(Enumeration e = ((StackProcessParentOutput)process.vmspp).children();
		  e.hasMoreElements();) {
		StackProcess sp = (StackProcess)e.nextElement();
		y = aDouble(sp.output());
		if (y < x) x = y;
		//sp.unwindStack();
		sp.pool.dropStackProcess(sp);
	      }
	      --sp; // pop the vmspp off the stack
	      ostack[sp++] = new Double(x);
	    process.vmspp = null;
	      here = iinst[ip++]; return;
	    }
	  case MAXT2:
	    if (process.vmspp != null && !process.vmspp.donep()) {
	      here = (MAXT2 | (PRIMASK << 8) | (OTHER << 16)); return;
	    }
	    else {
	      x = -Double.MAX_VALUE;
	      for(Enumeration e = ((StackProcessParentOutput)process.vmspp).children();
		  e.hasMoreElements();) {
		StackProcess sp = (StackProcess)e.nextElement();
		y = aDouble(sp.output());
		if (y > x) x = y;
		//sp.unwindStack();
		sp.pool.dropStackProcess(sp);
	      }
	      --sp; // pop the vmspp off the stack
	      ostack[sp++] = new Double(x);
	    process.vmspp = null;
	      here = iinst[ip++]; return;
	    }
	  case WMINT2:
	      if (process.vmspp != null && !process.vmspp.donep()) {
		  here = (WMINT2 | (PRIMASK << 8) | (OTHER << 16)); return;
	      }
	      else {
		  Object w = minusoneInt;
		  x = Double.MAX_VALUE;
		  for(Enumeration e = ((StackProcessParentOutput)process.vmspp).children();
		      e.hasMoreElements();) {
		      StackProcess sp = (StackProcess)e.nextElement();
		      //System.out.println("output from turtle #" + sp.tur.who + " is " + sp.output());
		      list = aList(sp.output());
		      y = aDouble(list[1]);
		      if (y < x) {
			  x = y;
			  w = list[0];
		      }
		//sp.unwindStack();
		sp.pool.dropStackProcess(sp);
	      }
		  --sp; // pop the vmspp off the stack
	      ostack[sp++] = new Object[]{ w, new Double(x)};
	    process.vmspp = null;
	      here = iinst[ip++]; return;
	    }
	  case WMAXT2:
	    if (process.vmspp != null && !process.vmspp.donep()) {
	      here = (WMAXT2 | (PRIMASK << 8) | (OTHER << 16)); return;
	    }
	    else {
		Object w = minusoneInt;
	      x = -Double.MAX_VALUE;
	      for(Enumeration e = ((StackProcessParentOutput)process.vmspp).children();
		  e.hasMoreElements();) {
		StackProcess sp = (StackProcess)e.nextElement();
		list = aList(sp.output());
		y = aDouble(list[1]);
		if (y > x) {
		    x = y;
		    w = list[0];
		}
		//sp.unwindStack();
		sp.pool.dropStackProcess(sp);
	      }
	      --sp; // pop the vmspp off the stack
	      ostack[sp++] = new Object[]{ w, new Double(x)};
	    process.vmspp = null;
	      here = iinst[ip++]; return;
	    }
	  case LTW2:
	    if (process.vmspp != null && !process.vmspp.donep()) {
	      here = (LTW2 | (PRIMASK << 8) | (OTHER << 16)); return;
	    }
	    else {
	      Vector turs = new Vector();
	      for(Enumeration e = ((StackProcessParentOutput)process.vmspp).children();
		  e.hasMoreElements();) {
		StackProcess sp = (StackProcess)e.nextElement();
		if (aBoolean(sp.output())) turs.addElement(sp.tur.whoInt);
		//sp.unwindStack();
		sp.pool.dropStackProcess(sp);
	      }
	      Object[] out = new Object[turs.size()];
	      turs.copyInto(out);
	      --sp; // pop the vmspp off the stack
	      ostack[sp++] = out;
	    process.vmspp = null;
	      here = iinst[ip++]; return;
	    }
	  }
	}
	break;
      case PRIMMISC:
	if (((here & 0x00010000) >> 16) == 0) {
	  primMisc(c, obs, tur, pat);
	  return;
	} else {
	  switch (here & 0xFF) {
	  case SVAL:
	      val = ostack[--sp];
	      SliderWidget sld = null;
	      //System.out.println("val is " + val);
	      if (val instanceof SliderWidget) sld = (SliderWidget)val;
	      else  {
		  //System.out.println("caching slider: " + val);
		  sld = (SliderWidget)sl.app.iface.getSlider(aInteger(val));
		  // Cache the slider.
		  //  System.out.println("slider is " + sld);
		  process.oinst[iinst[ip-2]] = sld;
	      }
	      ostack[sp++] = sld.getValue();
	      //System.out.println("slider val is " + sld.getValue());
	    break;
	  case MOUSEXCOR:
	    ostack[sp++] = new Double(mouseXcor-sl.halfwidth);
	    break;
	  case MOUSEYCOR:
	    ostack[sp++] = new Double(sl.halfheight-mouseYcor);
	    break;
	  case MOUSEDOWNP:
	    ostack[sp++] = (mouseDownp) ? YoYo.symtrue : YoYo.symfalse;
	    break;
	  case GETSCHED:
	    ostack[sp++] = sl.list.getScheduler();
	    break;
	  case GETSCHEDRANDSEED:
	      ostack[sp++] = new Integer(StackProcessList.seed);
	    break;
	  }
	}
	break;
      case PRIMPLOT:
	  //if (StarLogo.DEBUG) {System.out.println("in primplot...");}
	if (((here & 0x00010000) >> 16) == 0) {
	  primPlot(c, obs, tur, pat);
	  return;
	} else {
	    primPlotReporters(c, obs, tur, pat);
	    break;
	}
      case PRIMSAVE:
	if (((here & 0x00010000) >> 16) == 0) {
	  primSave(c, obs, tur, pat);
	  return;
	} else {
	  switch (here & 0xFF) {
	  case OUTPUTFILENAME:
	    if (sl.app.project.outputFilename != null)
	      ostack[sp++] = sl.app.project.outputFilename;
	    else ostack[sp++] = YoYo.symfalse;
	    break;
	  case PROJECTFILENAME:
	    if (sl.app.project.filename != null)
	      ostack[sp++] = sl.app.project.filename;
	    else ostack[sp++] = YoYo.symfalse;
	    break;
	  case INFOFILENAME:
	    if (sl.app.project.infoFilename != null)
	      ostack[sp++] = sl.app.project.infoFilename;
	    else ostack[sp++] = YoYo.symfalse;
	    break;
	  }
	}
	break;
      default:
	throw new StarLogoError(here, process, this);
      }
      here = iinst[ip++];
    }
  }

  public void primPatchState(Context c, Observer obs, Turtle tur, Patch pat) {
    int tempint, tempint2;
    double x, y, dl,col;
    Turtle st;
    switch (here & 0xFF) {
      /* Patch State */
      case OSETPC:
	x = aDouble(ostack[--sp]) % Colors.numcolors;
	if (x < 0) x = x + Colors.numcolors;
	sl.setPC(x);
	sl.redraw();
	break;
      case TSETPC:
	x = aDouble(ostack[--sp])%Colors.numcolors;
	sl.patches[tur.patch].pc = (x<0)
	  ? x+Colors.numcolors : x;
	if (sl.patches[tur.patch].topturtle == null) {
	  sl.patchcanvas.drawPatch(sl.patches[tur.patch]);
	}
	break;
      case PSETPC:
	pat.pc = aDouble(ostack[--sp])%Colors.numcolors;
	pat.pc = (pat.pc<0) ? pat.pc+Colors.numcolors : pat.pc;
	sl.patchesneeddrawingp = true;
	//if (pat.topturtle == null || !pat.topturtle.shownp) {
	//  sl.patchcanvas.drawPatch(pat);
	//}
	break;
      case OSTAMPAT:
	dl = aDouble(ostack[--sp])%Colors.numcolors;
	y = sl.halfheight - aDouble(ostack[--sp]);
	x = sl.halfwidth + aDouble(ostack[--sp]);
	tempint2 = sl.patchat(x, y);
	sl.patches[tempint2].pc = (dl<0)
	  ? dl+Colors.numcolors : dl;
	if (sl.patches[tempint2].topturtle == null)
	  sl.patchcanvas.drawPatch(sl.patches[tempint2]);
	break;
      case PSTAMPAT:
	dl = aDouble(ostack[--sp])%Colors.numcolors;
	y = pat.ycor - 1 - aDouble(ostack[--sp]);
	x = pat.xcor - 1 + aDouble(ostack[--sp]);
	tempint2 = sl.patchat(x, y);
	sl.patches[tempint2].pc = (dl<0)
	  ? dl+Colors.numcolors : dl;
	//if (sl.patches[tempint2].topturtle == null ||
	//   !sl.patches[tempint2].topturtle.shownp)
	//  sl.patchcanvas.drawPatch(sl.patches[tempint2]);
	sl.patchesneeddrawingp = true;
	break;
      case TSTAMPAT:
	dl = aDouble(ostack[--sp])%Colors.numcolors;
	y = tur.ycor - aDouble(ostack[--sp]);
	x = tur.xcor + aDouble(ostack[--sp]);
	tempint2 = sl.patchat(x, y);
	sl.patches[tempint2].pc = (dl<0)
	  ? dl+Colors.numcolors : dl;
	if (sl.patches[tempint2].topturtle == null)
	  sl.patchcanvas.drawPatch(sl.patches[tempint2]);
	break;

      case OSTAMPTOWARDS:
	dl = aDouble(ostack[--sp])%Colors.numcolors;
	y = aDouble(ostack[--sp]); // distance
	x = aDouble(ostack[--sp]); // angle
	tempint2 = sl.patchtowards(x, y, sl.halfwidth, sl.halfheight);
	sl.patches[tempint2].pc = (dl<0)
	  ? dl+Colors.numcolors : dl;
	if (sl.patches[tempint2].topturtle == null)
	  sl.patchcanvas.drawPatch(sl.patches[tempint2]);
	break;
      case TSTAMPTOWARDS:
	dl = aDouble(ostack[--sp])%Colors.numcolors;
	y = aDouble(ostack[--sp]); // distance
	x = aDouble(ostack[--sp]); // angle
	tempint2 = sl.patchtowards(x + tur.heading, y, tur.xcor, tur.ycor);
	sl.patches[tempint2].pc = (dl<0)
	  ? dl+Colors.numcolors : dl;
	if (sl.patches[tempint2].topturtle == null)
	  sl.patchcanvas.drawPatch(sl.patches[tempint2]);
	break;
      case PSTAMPTOWARDS:
	dl = aDouble(ostack[--sp])%Colors.numcolors;
	y = aDouble(ostack[--sp]); // distance
	x = aDouble(ostack[--sp]); // angle
	tempint2 = sl.patchtowards(x, y, pat.xcor-1, pat.ycor-1);
	st = sl.patches[tempint2].topturtle;
	while (st != null) {
	  st.color = (dl<0) ? dl+Colors.numcolors : dl;
	  st = st.underme;
	}
	st = sl.patches[tempint2].tophiddenturtle;
	while (st != null) {
	  st.color = (dl<0) ? dl+Colors.numcolors : dl;
	  st = st.underme;
	}
	sl.patchesneeddrawingp = true;
	//if (sl.patches[tempint2].topturtle != null)
	//  sl.patchcanvas.drawTurtle(sl.patches[tempint2].topturtle);
	break;

      case TSCALEPC:
	x = aDouble(ostack[--sp]); // max
	y = aDouble(ostack[--sp]); // min
	dl = aDouble(ostack[--sp]); // value
	col = aDouble(ostack[--sp]); // color
	x = x - y; // range
	y = dl - y; // diff from min
	dl = y / x; // diif / range
	if (dl < 0.0) {
	  sl.patches[tur.patch].pc = 0;
	} else {
	  if (dl > 1) dl = 1;
	  dl = dl * 9.999999999;
	  col = col%Colors.numcolors;  // color
	  col = (col<0) ? col+Colors.numcolors : col;
	  sl.patches[tur.patch].pc = (((int)(col / 10)) * 10.0) + dl;
	}
	break;
      case PSCALEPC:
	x = aDouble(ostack[--sp]); // max
	y = aDouble(ostack[--sp]); // min
	dl = aDouble(ostack[--sp]); // value
	col = aDouble(ostack[--sp]); // color
	x = x - y; // range
	y = dl - y; // diff from min
	dl = y / x; // diif / range
	if (dl < 0.0) {
	  pat.pc = 0;
	} else {
	  if (dl > 1) dl = 1;
	  dl = dl * 9.999999999;
	  col = col%Colors.numcolors;  // color
	  col = (col<0) ? col+Colors.numcolors : col;
	  pat.pc = (((int)(col / 10)) * 10.0) + dl;
	}
	//if (pat.topturtle == null || !pat.topturtle.shownp) {
	//  sl.patchcanvas.drawPatch(pat);
	//}
	sl.patchesneeddrawingp = true;
	break;
    }
    here = iinst[ip++];
  }

  public void primPatchVar(Context c, Observer obs, Turtle tur, Patch pat) {
    int offset, tempint;
    Object val;
    double x, y;
    switch (here & 0xFF) {
      /* Patch Variables */
    case OSETP:
      offset = aInteger(ostack[--sp]);
      val = ostack[--sp];
      sl.setPatchVar(offset, val);
      break;
    case TSETP:
      offset = aInteger(ostack[--sp]);
      val = ostack[--sp];
      sl.patches[tur.patch].turtlesown[offset]=val;
      if (val instanceof Number)
	sl.patches[tur.patch].patchesowndoubles[offset] = aDouble(val);
      break;
    case PSETP:
      offset = aInteger(ostack[--sp]);
      val = ostack[--sp];
      pat.turtlesown[offset]=val;
      if (val instanceof Number) pat.patchesowndoubles[offset] = aDouble(val);
      break;

    case OSETPAT:
      offset = aInteger(ostack[--sp]);
      val = ostack[--sp];
      y = sl.halfheight - aDouble(ostack[--sp]);
      x = sl.halfwidth + aDouble(ostack[--sp]);
      tempint = sl.patchat(x, y);
      sl.patches[tempint].turtlesown[offset]=val;
      if (val instanceof Number)
	sl.patches[tempint].patchesowndoubles[offset] = aDouble(val);
      break;
    case PSETPAT:
      offset = aInteger(ostack[--sp]);
      val = ostack[--sp];
      y = pat.ycor - 1 - aDouble(ostack[--sp]);
      x = pat.xcor - 1 + aDouble(ostack[--sp]);
      tempint = sl.patchat(x, y);
      sl.patches[tempint].turtlesown[offset]=val;
      if (val instanceof Number)
	sl.patches[tempint].patchesowndoubles[offset] = aDouble(val);
      break;
    case TSETPAT:
      offset = aInteger(ostack[--sp]);
      val = ostack[--sp];
      y = tur.ycor - aDouble(ostack[--sp]);
      x = tur.xcor + aDouble(ostack[--sp]);
      tempint = sl.patchat(x, y);
      sl.patches[tempint].turtlesown[offset]=val;
      if (val instanceof Number)
	sl.patches[tempint].patchesowndoubles[offset] = aDouble(val);
      break;

    case OSETPTOWARDS:
      offset = aInteger(ostack[--sp]);
      val = ostack[--sp];
      y = aDouble(ostack[--sp]); // distance
      x = aDouble(ostack[--sp]); // angle
      tempint = sl.patchtowards(x, y, sl.halfwidth, sl.halfheight);
      sl.patches[tempint].turtlesown[offset]=val;
      if (val instanceof Number)
	sl.patches[tempint].patchesowndoubles[offset] = aDouble(val);
      break;
    case TSETPTOWARDS:
      offset = aInteger(ostack[--sp]);
      val = ostack[--sp];
      y = aDouble(ostack[--sp]); // distance
      x = aDouble(ostack[--sp]); // angle
      tempint = sl.patchtowards(x + tur.heading, y, tur.xcor, tur.ycor);
      sl.patches[tempint].turtlesown[offset]=val;
      if (val instanceof Number)
	sl.patches[tempint].patchesowndoubles[offset] = aDouble(val);
      break;
    case PSETPTOWARDS:
      offset = aInteger(ostack[--sp]);
      val = ostack[--sp];
      y = aDouble(ostack[--sp]); // distance
      x = aDouble(ostack[--sp]); // angle
      tempint = sl.patchtowards(x, y, pat.xcor-1, pat.ycor-1);
      sl.patches[tempint].turtlesown[offset]=val;
      if (val instanceof Number)
	sl.patches[tempint].patchesowndoubles[offset] = aDouble(val);
      break;
    }
    here = iinst[ip++];
  }


  public void primGlobalVar(Context c, Observer obs, Turtle tur, Patch pat) {
    int offset, tempint;
    Object val;
    switch (here & 0xFF) {
    case OSETG:
	offset = aInteger(ostack[--sp]);
	val = ostack[--sp];
	obs.turtlesown[offset] = val;
	here = iinst[ip++]; return;
    case TPSETG:
	offset = aInteger(ostack[--sp]);
	val = ostack[--sp];
	sl.observer.turtlesown[offset] = val;
	here = iinst[ip++]; return;
    }
    here = iinst[ip++];
  }

  public void primTurtleOtherState(Context c, Observer obs, Turtle tur, Patch pat) {
    double x, y, a, b;
    Object val;
    Turtle st; Vector turs;
    int offset, tempint, counter;
    switch (here & 0xFF) {
      /* Relative Turtle State */
    case SETSTATEOF:
      val = ostack[--sp];
      st = sl.getWho(aInteger(ostack[--sp]));
      offset = aInteger(ostack[--sp]);
      if (st != null && st.alive) st.setState(offset, val);
      break;
    case SETXOF:
      x = sl.turhalfwidth + aDouble(ostack[--sp]);
      st = sl.getWho(aInteger(ostack[--sp]));
      if (st != null && st.alive) {
	  sl.unlink(st);
	  st.xcor = x;
	  sl.wrapandlink(st);
      }
      break;
    case SETYOF:
      y = sl.turhalfheight - aDouble(ostack[--sp]);
      st = sl.getWho(aInteger(ostack[--sp]));
      if (st != null && st.alive) {
	  sl.unlink(st);
	  st.ycor = y;
	  sl.wrapandlink(st);
      }
      break;
    case SETXYOF:
      y = sl.turhalfheight - aDouble(ostack[--sp]);
      x = sl.turhalfwidth + aDouble(ostack[--sp]);
      st = sl.getWho(aInteger(ostack[--sp]));
      if (st != null && st.alive) {
	  sl.unlink(st);
	  st.ycor = y;
	  st.xcor = x;
	  sl.wrapandlink(st);
      }
      break;
    case OSETSTATEAT:
      val = ostack[--sp];
      y = sl.halfheight - aDouble(ostack[--sp]);
      x = sl.halfwidth + aDouble(ostack[--sp]);
      offset = aInteger(ostack[--sp]);
      tempint = sl.patchat(x, y);
      for (st = sl.patches[tempint].topturtle; st != null; st = st.underme) {
	if (st.alive) st.setState(offset, val);
      }
      for (st = sl.patches[tempint].tophiddenturtle; st != null; st = st.underme) {
	if (st.alive) st.setState(offset, val);
      }
      break;
    case TSETSTATEAT:
      val = ostack[--sp];
      y = tur.ycor - aDouble(ostack[--sp]);
      x = tur.xcor + aDouble(ostack[--sp]);
      offset = aInteger(ostack[--sp]);
      tempint = sl.patchat(x, y);
      for (st = sl.patches[tempint].topturtle; st != null; st = st.underme) {
	  if (st.alive) st.setState(offset, val);
      }
      for (st = sl.patches[tempint].tophiddenturtle; st != null; st = st.underme) {
	  if (st.alive) st.setState(offset, val);
      }
      break;
    case PSETSTATEAT:
      val = ostack[--sp];
      y = pat.ycor - 1 - aDouble(ostack[--sp]);
      x = pat.xcor - 1 + aDouble(ostack[--sp]);
      offset = aInteger(ostack[--sp]);
      tempint = sl.patchat(x, y);
      for (st = sl.patches[tempint].topturtle; st != null; st = st.underme) {
	  if (st.alive) st.setState(offset, val);
      }
      for (st = sl.patches[tempint].tophiddenturtle; st != null; st = st.underme) {
	  if (st.alive) st.setState(offset, val);
      }
      break;
    case OSETXAT:
      a = sl.turhalfwidth + aDouble(ostack[--sp]);
      y = sl.halfheight - aDouble(ostack[--sp]);
      x = sl.halfwidth + aDouble(ostack[--sp]);
      tempint = sl.patchat(x, y);
      turs = new Vector(10);
      st = sl.patches[tempint].topturtle;
      while(st!=null) { turs.addElement(st); st = st.underme; }
      st = sl.patches[tempint].tophiddenturtle;
      while(st!=null) { turs.addElement(st); st = st.underme; }
      for(Enumeration e = turs.elements(); e.hasMoreElements();) {
	st = (Turtle)e.nextElement();
	if (st.alive) {
	    sl.unlink(st);
	    st.xcor = a;
	    sl.wrapandlink(st);
	}
      }
      break;
    case TSETXAT:
      a = sl.turhalfwidth + aDouble(ostack[--sp]);
      y = tur.ycor - aDouble(ostack[--sp]);
      x = tur.xcor + aDouble(ostack[--sp]);
      tempint = sl.patchat(x, y);
      turs = new Vector(10);
      st = sl.patches[tempint].topturtle;
      while(st!=null) { turs.addElement(st); st = st.underme; }
      st = sl.patches[tempint].tophiddenturtle;
      while(st!=null) { turs.addElement(st); st = st.underme; }
      for(Enumeration e = turs.elements(); e.hasMoreElements();) {
	st = (Turtle)e.nextElement();
	if (st.alive) {
	    sl.unlink(st);
	    st.xcor = a;
	    sl.wrapandlink(st);
	}
      }
      break;
    case PSETXAT:
      a = sl.turhalfwidth + aDouble(ostack[--sp]);
      y = pat.ycor - 1 - aDouble(ostack[--sp]);
      x = pat.xcor - 1 + aDouble(ostack[--sp]);
      tempint = sl.patchat(x, y);
      turs = new Vector(10);
      st = sl.patches[tempint].topturtle;
      while(st!=null) { turs.addElement(st); st = st.underme; }
      st = sl.patches[tempint].tophiddenturtle;
      while(st!=null) { turs.addElement(st); st = st.underme; }
      for(Enumeration e = turs.elements(); e.hasMoreElements();) {
	st = (Turtle)e.nextElement();
	if (st.alive) {
	    sl.unlink(st);
	    st.xcor = a;
	    sl.wrapandlink(st);
	}
      }
      break;
    case OSETYAT:
      b = sl.turhalfheight - aDouble(ostack[--sp]);
      y = sl.halfheight - aDouble(ostack[--sp]);
      x = sl.halfwidth + aDouble(ostack[--sp]);
      tempint = sl.patchat(x, y);
      turs = new Vector(10);
      st = sl.patches[tempint].topturtle;
      while(st!=null) { turs.addElement(st); st = st.underme; }
      st = sl.patches[tempint].tophiddenturtle;
      while(st!=null) { turs.addElement(st); st = st.underme; }
      for(Enumeration e = turs.elements(); e.hasMoreElements();) {
	st = (Turtle)e.nextElement();
	if (st.alive) {
	    sl.unlink(st);
	    st.ycor = b;
	    sl.wrapandlink(st);
	}
      }
      break;
    case TSETYAT:
      b = sl.turhalfheight - aDouble(ostack[--sp]);
      y = tur.ycor - aDouble(ostack[--sp]);
      x = tur.xcor + aDouble(ostack[--sp]);
      tempint = sl.patchat(x, y);
      turs = new Vector(10);
      st = sl.patches[tempint].topturtle;
      while(st!=null) { turs.addElement(st); st = st.underme; }
      st = sl.patches[tempint].tophiddenturtle;
      while(st!=null) { turs.addElement(st); st = st.underme; }
      for(Enumeration e = turs.elements(); e.hasMoreElements();) {
	st = (Turtle)e.nextElement();
	if (st.alive) {
	    sl.unlink(st);
	    st.ycor = b;
	    sl.wrapandlink(st);
	}
      }
      break;
    case PSETYAT:
      b = sl.turhalfheight - aDouble(ostack[--sp]);
      y = pat.ycor - 1 - aDouble(ostack[--sp]);
      x = pat.xcor - 1 + aDouble(ostack[--sp]);
      tempint = sl.patchat(x, y);
      turs = new Vector(10);
      st = sl.patches[tempint].topturtle;
      while(st!=null) { turs.addElement(st); st = st.underme; }
      st = sl.patches[tempint].tophiddenturtle;
      while(st!=null) { turs.addElement(st); st = st.underme; }
      for(Enumeration e = turs.elements(); e.hasMoreElements();) {
	st = (Turtle)e.nextElement();
	if (st.alive) {
	    sl.unlink(st);
	    st.ycor = b;
	    sl.wrapandlink(st);
	}
      }
      break;
    case OSETXYAT:
      b = sl.turhalfheight - aDouble(ostack[--sp]);
      a = sl.turhalfwidth + aDouble(ostack[--sp]);
      y = sl.halfheight - aDouble(ostack[--sp]);
      x = sl.halfwidth + aDouble(ostack[--sp]);
      tempint = sl.patchat(x, y);
      turs = new Vector(10);
      st = sl.patches[tempint].topturtle;
      while(st!=null) { turs.addElement(st); st = st.underme; }
      st = sl.patches[tempint].tophiddenturtle;
      while(st!=null) { turs.addElement(st); st = st.underme; }
      for(Enumeration e = turs.elements(); e.hasMoreElements();) {
	st = (Turtle)e.nextElement();
	if (st.alive) {
	    sl.unlink(st);
	    st.ycor = b;
	    st.xcor = a;
	    sl.wrapandlink(st);
	}
      }
      break;
    case TSETXYAT:
      b = sl.turhalfheight - aDouble(ostack[--sp]);
      a = sl.turhalfwidth + aDouble(ostack[--sp]);
      y = tur.ycor - aDouble(ostack[--sp]);
      x = tur.xcor + aDouble(ostack[--sp]);
      tempint = sl.patchat(x, y);
      turs = new Vector(10);
      st = sl.patches[tempint].topturtle;
      while(st!=null) { turs.addElement(st); st = st.underme; }
      st = sl.patches[tempint].tophiddenturtle;
      while(st!=null) { turs.addElement(st); st = st.underme; }
      for(Enumeration e = turs.elements(); e.hasMoreElements();) {
	st = (Turtle)e.nextElement();
	if (st.alive) {
	    sl.unlink(st);
	    st.ycor = b;
	    st.xcor = a;
	    sl.wrapandlink(st);
	}
      }
      break;
    case PSETXYAT:
      b = sl.turhalfheight - aDouble(ostack[--sp]);
      a = sl.turhalfwidth + aDouble(ostack[--sp]);
      y = pat.ycor - 1 - aDouble(ostack[--sp]);
      x = pat.xcor - 1 + aDouble(ostack[--sp]);
      tempint = sl.patchat(x, y);
      turs = new Vector(10);
      st = sl.patches[tempint].topturtle;
      while(st!=null) { turs.addElement(st); st = st.underme; }
      st = sl.patches[tempint].tophiddenturtle;
      while(st!=null) { turs.addElement(st); st = st.underme; }
      for(Enumeration e = turs.elements(); e.hasMoreElements();) {
	st = (Turtle)e.nextElement();
	if (st.alive) {
	    sl.unlink(st);
	    st.ycor = b;
	    st.xcor = a;
	    sl.wrapandlink(st);
	}
      }
      break;
    case OSETSTATETOWARDS:
      val = ostack[--sp];
      y = aDouble(ostack[--sp]); // distance
      x = aDouble(ostack[--sp]); // angle
      offset = aInteger(ostack[--sp]);
      tempint = sl.patchtowards(x, y, sl.halfwidth, sl.halfheight);
      for (st = sl.patches[tempint].topturtle; st != null; st = st.underme) {
	if (st.alive) st.setState(offset, val);
      }
      for (st = sl.patches[tempint].tophiddenturtle; st != null; st = st.underme) {
	if (st.alive) st.setState(offset, val);
      }
      break;
    case TSETSTATETOWARDS:
      val = ostack[--sp];
      y = aDouble(ostack[--sp]); // distance
      x = aDouble(ostack[--sp]); // angle
      offset = aInteger(ostack[--sp]);
      tempint = sl.patchtowards(x + tur.heading, y, tur.xcor, tur.ycor);
      for (st = sl.patches[tempint].topturtle; st != null; st = st.underme) {
	if (st.alive) st.setState(offset, val);
      }
      for (st = sl.patches[tempint].tophiddenturtle; st != null; st = st.underme) {
	if (st.alive) st.setState(offset, val);
      }
      break;
    case PSETSTATETOWARDS:
      val = ostack[--sp];
      y = aDouble(ostack[--sp]); // distance
      x = aDouble(ostack[--sp]); // angle
      offset = aInteger(ostack[--sp]);
      tempint = sl.patchtowards(x, y, pat.xcor-1, pat.ycor-1);
      for (st = sl.patches[tempint].topturtle; st != null; st = st.underme) {
	if (st.alive) st.setState(offset, val);
      }
      for (st = sl.patches[tempint].tophiddenturtle; st != null; st = st.underme) {
	if (st.alive) st.setState(offset, val);
      }
      break;
    case OSETXYTOWARDS:
      b = sl.turhalfheight - aDouble(ostack[--sp]);
      a = sl.turhalfwidth + aDouble(ostack[--sp]);
      y = aDouble(ostack[--sp]); // distance
      x = aDouble(ostack[--sp]); // angle
      tempint = sl.patchtowards(x, y, sl.halfwidth, sl.halfheight);
      turs = new Vector(10);
      st = sl.patches[tempint].topturtle;
      while(st!=null) { turs.addElement(st); st = st.underme; }
      st = sl.patches[tempint].tophiddenturtle;
      while(st!=null) { turs.addElement(st); st = st.underme; }
      for(Enumeration e = turs.elements(); e.hasMoreElements();) {
	st = (Turtle)e.nextElement();
	if (st.alive) {
	    sl.unlink(st);
	    st.ycor = b;
	    st.xcor = a;
	    sl.wrapandlink(st);
	}
      }
      break;
    case TSETXYTOWARDS:
      b = sl.turhalfheight - aDouble(ostack[--sp]);
      a = sl.turhalfwidth + aDouble(ostack[--sp]);
      y = aDouble(ostack[--sp]); // distance
      x = aDouble(ostack[--sp]); // angle
      tempint = sl.patchtowards(x + tur.heading, y, tur.xcor, tur.ycor);
      turs = new Vector(10);
      st = sl.patches[tempint].topturtle;
      while(st!=null) { turs.addElement(st); st = st.underme; }
      st = sl.patches[tempint].tophiddenturtle;
      while(st!=null) { turs.addElement(st); st = st.underme; }
      for(Enumeration e = turs.elements(); e.hasMoreElements();) {
	st = (Turtle)e.nextElement();
	if (st.alive) {
	    sl.unlink(st);
	    st.ycor = b;
	    st.xcor = a;
	    sl.wrapandlink(st);
	}
      }
      break;
    case PSETXYTOWARDS:
      b = sl.turhalfheight - aDouble(ostack[--sp]);
      a = sl.turhalfwidth + aDouble(ostack[--sp]);
      y = aDouble(ostack[--sp]); // distance
      x = aDouble(ostack[--sp]); // angle
      tempint = sl.patchtowards(x, y, pat.xcor-1, pat.ycor-1);
      turs = new Vector(10);
      st = sl.patches[tempint].topturtle;
      while(st!=null) { turs.addElement(st); st = st.underme; }
      st = sl.patches[tempint].tophiddenturtle;
      while(st!=null) { turs.addElement(st); st = st.underme; }
      for(Enumeration e = turs.elements(); e.hasMoreElements();) {
	st = (Turtle)e.nextElement();
	if (st.alive) {
	    sl.unlink(st);
	    st.ycor = b;
	    st.xcor = a;
	    sl.wrapandlink(st);
	}
      }
      break;
    }
    here = iinst[ip++];

  }

  public void primTurtleOtherStateReporters(Context c, Observer obs, Turtle tur, Patch pat) {
    int offset, tempint;
    Turtle st;
    double x,y;
    switch (here & 0xFF) {
      /* Relative Turtle State */
    case GETSTATEOF:
      st = sl.getWho(aInteger(ostack[--sp]));
      offset = aInteger(ostack[--sp]);
      if (st != null)
	ostack[sp++] = st.getState(offset);
      else ostack[sp++] = zeroInt;
      break;
            case OGETSTATEAT:
	y = sl.halfheight - aDouble(ostack[--sp]);
	x = sl.halfwidth + aDouble(ostack[--sp]);
	tempint = sl.patchat(x, y);
	st = sl.patches[tempint].topturtle;
	offset = aInteger(ostack[--sp]);
	if (st == null) st = sl.patches[tempint].tophiddenturtle;
	if (st != null) ostack[sp++] = st.getState(offset);
	else ostack[sp++] = zeroInt;
	break;
      case PGETSTATEAT:
	y = pat.ycor - 1 - aDouble(ostack[--sp]);
	x = pat.xcor - 1 + aDouble(ostack[--sp]);
	tempint = sl.patchat(x, y);
	st = sl.patches[tempint].topturtle;
	offset = aInteger(ostack[--sp]);
	if (st == null) st = sl.patches[tempint].tophiddenturtle;
	if (st != null) ostack[sp++] = st.getState(offset);
	else ostack[sp++] = zeroInt;
	break;
      case TGETSTATEAT:
	y = tur.ycor - aDouble(ostack[--sp]);
	x = tur.xcor + aDouble(ostack[--sp]);
	tempint = sl.patchat(x, y);
	offset = aInteger(ostack[--sp]);
	st = sl.patches[tempint].topturtle;
	if (st == null) st = sl.patches[tempint].tophiddenturtle;
	if (st != null) ostack[sp++] = st.getState(offset);
	else ostack[sp++] = zeroInt;
	break;
      case OGETSTATETOWARDS:
	y = aDouble(ostack[--sp]); // distance
	x = aDouble(ostack[--sp]); // angle
	tempint = sl.patchtowards(x, y, sl.halfwidth, sl.halfheight);
	offset = aInteger(ostack[--sp]);
	st = sl.patches[tempint].topturtle;
	if (st == null) st = sl.patches[tempint].tophiddenturtle;
	if (st != null) ostack[sp++] = st.getState(offset);
	else ostack[sp++] = zeroInt;
	break;
      case TGETSTATETOWARDS:
	y = aDouble(ostack[--sp]); // distance
	x = aDouble(ostack[--sp]); // angle
	tempint = sl.patchtowards(x + tur.heading, y, tur.xcor, tur.ycor);
	offset = aInteger(ostack[--sp]);
	st = sl.patches[tempint].topturtle;
	if (st == null) st = sl.patches[tempint].tophiddenturtle;
	if (st != null) ostack[sp++] = st.getState(offset);
	else ostack[sp++] = zeroInt;
	break;
      case PGETSTATETOWARDS:
	y = aDouble(ostack[--sp]); // distance
	x = aDouble(ostack[--sp]); // angle
	tempint = sl.patchtowards(x, y, pat.xcor-1, pat.ycor-1);
	offset = aInteger(ostack[--sp]);
	st = sl.patches[tempint].topturtle;
	if (st == null) st = sl.patches[tempint].tophiddenturtle;
	if (st != null) ostack[sp++] = st.getState(offset);
	else ostack[sp++] = zeroInt;
	break;
    }
  }

  public void primTurtleVar(Context c, Observer obs, Turtle tur, Patch pat) {
    int offset, tempint;
    Turtle st;
    Object val;
    double x, y;
    switch (here & 0xFF) {
    case OSETT:
      offset = aInteger(ostack[--sp]);
      val = ostack[--sp];
      sl.setTurtleVar(offset, val);
      break;
    case TSETT:
      offset = aInteger(ostack[--sp]);
      val = ostack[--sp];
      tur.turtlesown[offset] = val;
      break;
    case PSETT:
       st = pat.topturtle;
       if (st == null) st = pat.tophiddenturtle;
       if (st == null) {
			sp-=2;
      } else {
		offset = aInteger(ostack[--sp]);
		val = ostack[--sp];
		st.turtlesown[offset] = val;
      }
      break;
    case SETTOF:
      offset = aInteger(ostack[--sp]);
      val = ostack[--sp];
      st = sl.getWho(aInteger(ostack[--sp]));
      if (st != null && st.alive) st.turtlesown[offset] = val;
      break;
    case OSETTAT:
      offset = aInteger(ostack[--sp]);
      val = ostack[--sp];
      y = sl.halfheight - aDouble(ostack[--sp]);
      x = sl.halfwidth + aDouble(ostack[--sp]);
      tempint = sl.patchat(x, y);
      for (st = sl.patches[tempint].topturtle; st != null; st = st.underme) {
		if (st.alive) st.turtlesown[offset] = val;
      }
      for (st = sl.patches[tempint].tophiddenturtle; st != null; st = st.underme) {
		if (st.alive) st.turtlesown[offset] = val;
      }
      break;
    case PSETTAT:
      offset = aInteger(ostack[--sp]);
      val = ostack[--sp];
      y = pat.ycor - 1 - aDouble(ostack[--sp]);
      x = pat.xcor - 1 + aDouble(ostack[--sp]);
      tempint = sl.patchat(x, y);
      for (st = sl.patches[tempint].topturtle; st != null; st = st.underme) {
		if (st.alive) st.turtlesown[offset] = val;
      }
      for (st = sl.patches[tempint].tophiddenturtle; st != null; st = st.underme) {
		if (st.alive) st.turtlesown[offset] = val;
      }
      break;
    case TSETTAT:
      offset = aInteger(ostack[--sp]);
      val = ostack[--sp];
      y = tur.ycor - aDouble(ostack[--sp]);
      x = tur.xcor + aDouble(ostack[--sp]);
      tempint = sl.patchat(x, y);
      for (st = sl.patches[tempint].topturtle; st != null; st = st.underme) {
		if (st.alive) st.turtlesown[offset] = val;
      }
      for (st = sl.patches[tempint].tophiddenturtle; st != null; st = st.underme) {
		if (st.alive) st.turtlesown[offset] = val;
      }
      break;
    case OSETTTOWARDS:
      offset = aInteger(ostack[--sp]);
      val = ostack[--sp];
      y = aDouble(ostack[--sp]); // distance
      x = aDouble(ostack[--sp]); // angle
      tempint = sl.patchtowards(x, y, sl.halfwidth, sl.halfheight);
      for (st = sl.patches[tempint].topturtle; st != null; st = st.underme) {
	  if (st.alive) st.turtlesown[offset] = val;
      }
      for (st = sl.patches[tempint].tophiddenturtle; st != null; st = st.underme) {
	  if (st.alive) st.turtlesown[offset] = val;
      }
      break;
    case TSETTTOWARDS:
      val = ostack[--sp];
      y = aDouble(ostack[--sp]); // distance
      x = aDouble(ostack[--sp]); // angle
      offset = aInteger(ostack[--sp]);
      tempint = sl.patchtowards(x + tur.heading, y, tur.xcor, tur.ycor);
      for (st = sl.patches[tempint].topturtle; st != null; st = st.underme) {
	  if (st.alive) st.turtlesown[offset] = val;
      }
      for (st = sl.patches[tempint].tophiddenturtle; st != null; st = st.underme) {
	  if (st.alive) st.turtlesown[offset] = val;
      }
      break;
    case PSETTTOWARDS:
      val = ostack[--sp];
      y = aDouble(ostack[--sp]); // distance
      x = aDouble(ostack[--sp]); // angle
      offset = aInteger(ostack[--sp]);
      tempint = sl.patchtowards(x, y, pat.xcor-1, pat.ycor-1);
      for (st = sl.patches[tempint].topturtle; st != null; st = st.underme) {
	  if (st.alive) st.turtlesown[offset] = val;
      }
      for (st = sl.patches[tempint].tophiddenturtle; st != null; st = st.underme) {
	  if (st.alive) st.turtlesown[offset] = val;
      }
      break;
    }
    here = iinst[ip++];
  }

  public void primHereReporters(Context c, Observer obs, Turtle tur, Patch pat) {
      double x, y;
      Turtle st;
      Object[] list;
      Vector partners;
      int tempint, tempint2, counter;
	switch (here & 0xFF) {
	/* Turtles Here */
      case TCTH:
	counter = 0;
	st = sl.patches[tur.patch].topturtle;
	while(st!=null) { counter++; st = st.underme; }
	st = sl.patches[tur.patch].tophiddenturtle;
	while(st!=null) { counter++; st = st.underme; }
	ostack[sp++] = new Integer(counter);
	break;
      case PCTH:
	counter = 0;
	st = pat.topturtle;
	while(st!=null) { counter++; st = st.underme; }
	st = pat.tophiddenturtle;
	while(st!=null) { counter++; st = st.underme; }
	ostack[sp++] = new Integer(counter);
	break;
      case TOTH:
	  st = pickRandomTurtleHere(tur.patch, tur);
	  if (st!=null) ostack[sp++]=st.whoInt;
	  else ostack[sp++] = minusoneInt;
	  break;
      case POTH:
	st = pat.topturtle;
	if (st == null) st = pat.tophiddenturtle;
	if (st != null) st = pickRandomTurtleHere(st.patch);
	if (st!=null) ostack[sp++]=st.whoInt;
	else ostack[sp++] = minusoneInt;
	break;
      case TLTH:
	partners = new Vector();
	st = sl.patches[tur.patch].topturtle;
	while (st != null) {
	  if (st != tur) partners.addElement(st.whoInt);
	  st = st.underme;
	}
	st = sl.patches[tur.patch].tophiddenturtle;
	while (st != null) {
	  if (st != tur) partners.addElement(st.whoInt);
	  st = st.underme;
	}
	list = new Object[partners.size()];
	partners.copyInto(list);
	ostack[sp++] = list;
	break;
      case PLTH:
	partners = new Vector();
	st = pat.topturtle;
	while (st != null) {
	  partners.addElement(st.whoInt);
	  st = st.underme;
	}
	st = pat.tophiddenturtle;
	while (st != null) {
	  partners.addElement(st.whoInt);
	  st = st.underme;
	}
	list = new Object[partners.size()];
	partners.copyInto(list);
	ostack[sp++] = list;
	break;
      case TCBH:
	tempint2 = aInteger(ostack[--sp]);
	st = sl.patches[tur.patch].topturtle; counter = 0;
	while(st!=null) { if (st.breed == tempint2) {counter++;} st = st.underme; }
	st = sl.patches[tur.patch].tophiddenturtle;
	while(st!=null) { if (st.breed == tempint2) {counter++;} st = st.underme; }
	ostack[sp++] = new Integer(counter);
	break;
      case PCBH:
	tempint2 = aInteger(ostack[--sp]);
	st = pat.topturtle; counter = 0;
	while(st!=null) { if (st.breed == tempint2) {counter++;} st = st.underme; }
	st = sl.patches[tur.patch].tophiddenturtle;
	while(st!=null) { if (st.breed == tempint2) {counter++;} st = st.underme; }
	ostack[sp++] = new Integer(counter);
	break;
      case TOBH:
	  
	  tempint2 = aInteger(ostack[--sp]);
	  st = pickRandomBreedHere(tur.patch, tur, tempint2);
	  if (st!=null) ostack[sp++]=st.whoInt;
	  else ostack[sp++] = minusoneInt;
	break;
      case POBH:
	  st = pat.topturtle;
	  if (st == null) st = pat.tophiddenturtle;
	  tempint2 = aInteger(ostack[--sp]);
	  if (st != null) st = pickRandomBreedHere(st.patch, tempint2);
	  if (st!=null) ostack[sp++]=st.whoInt;
	  else ostack[sp++] = minusoneInt;
	break;
      case TLBH:
	tempint2 = aInteger(ostack[--sp]);
	partners = new Vector();
	st = sl.patches[tur.patch].topturtle;
	while (st != null) {
	  if (st != tur && st.breed == tempint2) partners.addElement(st.whoInt);
	  st = st.underme;
	}
	st = sl.patches[tur.patch].tophiddenturtle;
	while (st != null) {
	  if (st != tur && st.breed == tempint2) partners.addElement(st.whoInt);
	  st = st.underme;
	}
	list = new Object[partners.size()];
	partners.copyInto(list);
	ostack[sp++] = list;
	break;
      case PLBH:
	tempint2 = aInteger(ostack[--sp]);
	partners = new Vector();
	st = pat.topturtle;
	while (st != null) {
	  if (st.breed == tempint2) partners.addElement(st.whoInt);
	  st = st.underme;
	}
	st = pat.tophiddenturtle;
	while (st != null) {
	  if (st.breed == tempint2) partners.addElement(st.whoInt);
	  st = st.underme;
	}
	list = new Object[partners.size()];
	partners.copyInto(list);
	ostack[sp++] = list;
	break;
	}
  }

  public void primAtTowardsReporters(Context c, Observer obs, Turtle tur, Patch pat) {
      double x, y;
      Turtle st;
      Object[] list;
      Vector partners;
      int tempint, tempint2, counter;
	switch (here & 0xFF) {

	/* Relative Turtle Reporters */
      case OCTA:
	y = sl.halfheight - aDouble(ostack[--sp]);
	x = sl.halfwidth + aDouble(ostack[--sp]);
	tempint = sl.patchat(x, y);
	st = sl.patches[tempint].topturtle;
	counter = 0;
	while(st!=null) {
	  counter++; st = st.underme; }
	st = sl.patches[tempint].tophiddenturtle;
	while(st!=null) {
	  counter++; st = st.underme; }
	ostack[sp++] = new Integer(counter);
	break;
      case PCTA:
	y = pat.ycor - 1 - aDouble(ostack[--sp]);
	x = pat.xcor - 1 + aDouble(ostack[--sp]);
	tempint = sl.patchat(x, y);
	st = sl.patches[tempint].topturtle;
	counter = 0;
	while(st!=null) {
	  counter++; st = st.underme; }
	st = sl.patches[tempint].tophiddenturtle;
	while(st!=null) {
	  counter++; st = st.underme; }
	ostack[sp++] = new Integer(counter);
	break;
      case TCTA:
	y = tur.ycor - aDouble(ostack[--sp]);
	x = tur.xcor + aDouble(ostack[--sp]);
	tempint = sl.patchat(x, y);
	st = sl.patches[tempint].topturtle;
	counter = 0;
	while(st!=null) { counter++; st = st.underme; }
	st = sl.patches[tempint].tophiddenturtle;
	while(st!=null) { counter++; st = st.underme; }
	ostack[sp++] = new Integer(counter);
	break;
      case OCTT:
	y = aDouble(ostack[--sp]); // distance
	x = aDouble(ostack[--sp]); // angle
	tempint = sl.patchtowards(x, y, sl.halfwidth, sl.halfheight);
	st = sl.patches[tempint].topturtle;
	counter = 0;
	while(st!=null) { counter++; st = st.underme; }
	st = sl.patches[tempint].tophiddenturtle;
	while(st!=null) {
	  counter++; st = st.underme; }
	ostack[sp++] = new Integer(counter);
	break;
      case TCTT:
	y = aDouble(ostack[--sp]); // distance
	x = aDouble(ostack[--sp]); // angle
	tempint = sl.patchtowards(x + tur.heading, y, tur.xcor, tur.ycor);
	st = sl.patches[tempint].topturtle;
	counter = 0;
	while(st!=null) { counter++; st = st.underme; }
	st = sl.patches[tempint].tophiddenturtle;
	while(st!=null) {
	  counter++; st = st.underme; }
	ostack[sp++] = new Integer(counter);
	break;
      case PCTT:
	y = aDouble(ostack[--sp]); // distance
	x = aDouble(ostack[--sp]); // angle
	tempint = sl.patchtowards(x, y, pat.xcor-1, pat.ycor-1);
	st = sl.patches[tempint].topturtle;
	counter = 0;
	while(st!=null) { counter++; st = st.underme; }
	st = sl.patches[tempint].tophiddenturtle;
	while(st!=null) {
	  counter++; st = st.underme; }
	ostack[sp++] = new Integer(counter);
	break;
      case OOTA:
	y = sl.halfheight - aDouble(ostack[--sp]);
	x = sl.halfwidth + aDouble(ostack[--sp]);
	tempint = sl.patchat(x, y);
	st = pickRandomTurtleHere(tempint);
	if (st != null) ostack[sp++]=st.whoInt;
	else ostack[sp++] = minusoneInt;
	break;
      case POTA:
	y = pat.ycor - 1 - aDouble(ostack[--sp]);
	x = pat.xcor - 1 + aDouble(ostack[--sp]);
	tempint = sl.patchat(x, y);
	st = pickRandomTurtleHere(tempint);
	if (st != null) ostack[sp++]=st.whoInt;
	else ostack[sp++] = minusoneInt;
	break;
      case TOTA:
	y = tur.ycor - aDouble(ostack[--sp]);
	x = tur.xcor + aDouble(ostack[--sp]);
	tempint = sl.patchat(x, y);
	if (tempint == tur.patch) {
	    st = pickRandomTurtleHere(tempint, tur);
	} else {
	    st = pickRandomTurtleHere(tempint);
	}
	if (st != null) ostack[sp++]=st.whoInt;
	else ostack[sp++] = minusoneInt;
	break;
      case OOTT:
	y = aDouble(ostack[--sp]); // distance
	x = aDouble(ostack[--sp]); // angle
	tempint = sl.patchtowards(x, y, sl.halfwidth, sl.halfheight);
	st = pickRandomTurtleHere(tempint);
	if (st != null) ostack[sp++]=st.whoInt;
	else ostack[sp++] = minusoneInt;
	break;
      case TOTT:
	y = aDouble(ostack[--sp]); // distance
	x = aDouble(ostack[--sp]); // angle
	tempint = sl.patchtowards(x + tur.heading, y, tur.xcor, tur.ycor);
	if (tempint == tur.patch) {
	    st = pickRandomTurtleHere(tempint, tur);
	} else {
	    st = pickRandomTurtleHere(tempint);
	}
	if (st != null) ostack[sp++]=st.whoInt;
	else ostack[sp++] = minusoneInt;
	break;
      case POTT:
	y = aDouble(ostack[--sp]); // distance
	x = aDouble(ostack[--sp]); // angle
	tempint = sl.patchtowards(x, y, pat.xcor-1, pat.ycor-1);
	st = pickRandomTurtleHere(tempint);
	if (st != null) ostack[sp++]=st.whoInt;
	else ostack[sp++] = minusoneInt;
	break;
      case OLTA:
	y = sl.halfheight - aDouble(ostack[--sp]);
	x = sl.halfwidth + aDouble(ostack[--sp]);
	tempint = sl.patchat(x, y);
	partners = new Vector();
	st = sl.patches[tempint].topturtle;
	while (st != null) {
	  partners.addElement(st.whoInt);
	  st = st.underme;
	}
	st = sl.patches[tempint].tophiddenturtle;
	while (st != null) {
	  partners.addElement(st.whoInt);
	  st = st.underme;
	}
	list = new Object[partners.size()];
	partners.copyInto(list);
	ostack[sp++] = list;
	break;
      case PLTA:
	y = pat.ycor - 1 - aDouble(ostack[--sp]);
	x = pat.xcor - 1 + aDouble(ostack[--sp]);
	tempint = sl.patchat(x, y);
	partners = new Vector();
	st = sl.patches[tempint].topturtle;
	while (st != null) {
	  partners.addElement(st.whoInt);
	  st = st.underme;
	}
	st = sl.patches[tempint].tophiddenturtle;
	while (st != null) {
	  partners.addElement(st.whoInt);
	  st = st.underme;
	}
	list = new Object[partners.size()];
	partners.copyInto(list);
	ostack[sp++] = list;
	break;
      case TLTA:
	y = tur.ycor - aDouble(ostack[--sp]);
	x = tur.xcor + aDouble(ostack[--sp]);
	tempint = sl.patchat(x, y);
	partners = new Vector();
	st = sl.patches[tempint].topturtle;
	while (st != null) {
	  if (st != tur) partners.addElement(st.whoInt);
	  st = st.underme;
	}
	st = sl.patches[tempint].tophiddenturtle;
	while (st != null) {
	  if (st != tur) partners.addElement(st.whoInt);
	  st = st.underme;
	}
	list = new Object[partners.size()];
	partners.copyInto(list);
	ostack[sp++] = list;
	break;
      case OLTT:
	y = aDouble(ostack[--sp]); // distance
	x = aDouble(ostack[--sp]); // angle
	tempint = sl.patchtowards(x, y, sl.halfwidth, sl.halfheight);
	partners = new Vector();
	st = sl.patches[tempint].topturtle;
	while (st != null) {
	  partners.addElement(st.whoInt);
	  st = st.underme;
	}
	st = sl.patches[tempint].tophiddenturtle;
	while (st != null) {
	  partners.addElement(st.whoInt);
	  st = st.underme;
	}
	list = new Object[partners.size()];
	partners.copyInto(list);
	ostack[sp++] = list;
	break;
      case TLTT:
	y = aDouble(ostack[--sp]); // distance
	x = aDouble(ostack[--sp]); // angle
	tempint = sl.patchtowards(x + tur.heading, y, tur.xcor, tur.ycor);
	partners = new Vector();
	st = sl.patches[tempint].topturtle;
	while (st != null) {
	  if (st != tur) partners.addElement(st.whoInt);
	  st = st.underme;
	}
	st = sl.patches[tempint].tophiddenturtle;
	while (st != null) {
	  if (st != tur) partners.addElement(st.whoInt);
	  st = st.underme;
	}
	list = new Object[partners.size()];
	partners.copyInto(list);
	ostack[sp++] = list;
	break;
      case PLTT:
	y = aDouble(ostack[--sp]); // distance
	x = aDouble(ostack[--sp]); // angle
	tempint = sl.patchtowards(x, y, pat.xcor-1, pat.ycor-1);
	partners = new Vector();
	st = sl.patches[tempint].topturtle;
	while (st != null) {
	  partners.addElement(st.whoInt);
	  st = st.underme;
	}
	st = sl.patches[tempint].tophiddenturtle;
	while (st != null) {
	  partners.addElement(st.whoInt);
	  st = st.underme;
	}
	list = new Object[partners.size()];
	partners.copyInto(list);
	ostack[sp++] = list;
	break;

	/* Relative Breed Reporters */
      case OCBA:
	y = sl.halfheight - aDouble(ostack[--sp]);
	x = sl.halfwidth + aDouble(ostack[--sp]);
	tempint = sl.patchat(x, y);
	st = sl.patches[tempint].topturtle;
	tempint2 = aInteger(ostack[--sp]);
	counter = 0;
	while(st!=null) { if (st.breed == tempint2) {counter++;} st = st.underme; }
	st = sl.patches[tempint].tophiddenturtle;
	while(st!=null) { if (st.breed == tempint2) {counter++;} st = st.underme; }
	ostack[sp++] = new Integer(counter);
	break;
      case PCBA:
	y = pat.ycor - 1 - aDouble(ostack[--sp]);
	x = pat.xcor - 1 + aDouble(ostack[--sp]);
	tempint = sl.patchat(x, y);
	st = sl.patches[tempint].topturtle;
	tempint2 = aInteger(ostack[--sp]);
	counter = 0;
	while(st!=null) { if (st.breed == tempint2) {counter++;} st = st.underme; }
	st = sl.patches[tempint].tophiddenturtle;
	while(st!=null) { if (st.breed == tempint2) {counter++;} st = st.underme; }
	ostack[sp++] = new Integer(counter);
	break;
      case TCBA:
	y = tur.ycor - aDouble(ostack[--sp]);
	x = tur.xcor + aDouble(ostack[--sp]);
	tempint = sl.patchat(x, y);
	st = sl.patches[tempint].topturtle;
	tempint2 = aInteger(ostack[--sp]);
	counter = 0;
	while(st!=null) { if (st.breed == tempint2) {counter++;} st = st.underme; }
	st = sl.patches[tempint].tophiddenturtle;
	while(st!=null) { if (st.breed == tempint2) {counter++;} st = st.underme; }
	ostack[sp++] = new Integer(counter);
	break;
      case OCBT:
	y = aDouble(ostack[--sp]); // distance
	x = aDouble(ostack[--sp]); // angle
	tempint = sl.patchtowards(x, y, sl.halfwidth, sl.halfheight);
	st = sl.patches[tempint].topturtle;
	tempint2 = aInteger(ostack[--sp]);
	counter = 0;
	while(st!=null) { if (tempint2 == st.breed) {counter++;} st = st.underme; }
	st = sl.patches[tempint].tophiddenturtle;
	while(st!=null) { if (st.breed == tempint2) {counter++;} st = st.underme; }
	ostack[sp++] = new Integer(counter);
	break;
      case TCBT:
	y = aDouble(ostack[--sp]); // distance
	x = aDouble(ostack[--sp]); // angle
	tempint = sl.patchtowards(x + tur.heading, y, tur.xcor, tur.ycor);
	st = sl.patches[tempint].topturtle;
	tempint2 = aInteger(ostack[--sp]);
	counter = 0;
	while(st!=null) { if (tempint2 == st.breed) {counter++;} st = st.underme; }
	st = sl.patches[tempint].tophiddenturtle;
	while(st!=null) { if (st.breed == tempint2) {counter++;} st = st.underme; }
	ostack[sp++] = new Integer(counter);
	break;
      case PCBT:
	y = aDouble(ostack[--sp]); // distance
	x = aDouble(ostack[--sp]); // angle
	tempint = sl.patchtowards(x, y, pat.xcor-1, pat.ycor-1);
	st = sl.patches[tempint].topturtle;
	tempint2 = aInteger(ostack[--sp]);
	counter = 0;
	while(st!=null) { if (tempint2 == st.breed) {counter++;} st = st.underme; }
	st = sl.patches[tempint].tophiddenturtle;
	while(st!=null) { if (st.breed == tempint2) {counter++;} st = st.underme; }
	ostack[sp++] = new Integer(counter);
	break;

      case OOBA:
	y = sl.halfheight - aDouble(ostack[--sp]);
	x = sl.halfwidth + aDouble(ostack[--sp]);
	tempint = sl.patchat(x, y);
	tempint2 = aInteger(ostack[--sp]);
	st = pickRandomBreedHere(tempint, tempint2);
	if (st != null) ostack[sp++]=st.whoInt;
	else ostack[sp++] = minusoneInt;
	break;
      case POBA:
	y = pat.ycor - 1 - aDouble(ostack[--sp]);
	x = pat.xcor - 1 + aDouble(ostack[--sp]);
	tempint = sl.patchat(x, y);
	tempint2 = aInteger(ostack[--sp]);
	st = pickRandomBreedHere(tempint, tempint2);
	if (st != null) ostack[sp++]=st.whoInt;
	else ostack[sp++] = minusoneInt;
	break;
      case TOBA:
	y = tur.ycor - aDouble(ostack[--sp]);
	x = tur.xcor + aDouble(ostack[--sp]);
	tempint = sl.patchat(x, y);
	tempint2 = aInteger(ostack[--sp]);
	if (tempint == tur.patch) {
	    st = pickRandomBreedHere(tempint, tur, tempint2);
	} else {
	    st = pickRandomBreedHere(tempint, tempint2);
	}
	if (st != null) ostack[sp++]=st.whoInt;
	else ostack[sp++] = minusoneInt;
	break;
      case OOBT:
	y = aDouble(ostack[--sp]); // distance
	x = aDouble(ostack[--sp]); // angle
	tempint = sl.patchtowards(x, y, sl.halfwidth, sl.halfheight);
	tempint2 = aInteger(ostack[--sp]);
	st = pickRandomBreedHere(tempint, tempint2);
	if (st != null) ostack[sp++]=st.whoInt;
	else ostack[sp++] = minusoneInt;
	break;
      case TOBT:
	y = aDouble(ostack[--sp]); // distance
	x = aDouble(ostack[--sp]); // angle
	tempint = sl.patchtowards(x + tur.heading, y, tur.xcor, tur.ycor);
	tempint2 = aInteger(ostack[--sp]);
	if (tempint == tur.patch) {
	    st = pickRandomBreedHere(tempint, tur, tempint2);
	} else {
	    st = pickRandomBreedHere(tempint, tempint2);
	}
	if (st != null) ostack[sp++]=st.whoInt;
	else ostack[sp++] = minusoneInt;
	break;
      case POBT:
	y = aDouble(ostack[--sp]); // distance
	x = aDouble(ostack[--sp]); // angle
	tempint = sl.patchtowards(x, y, pat.xcor-1, pat.ycor-1);
	tempint2 = aInteger(ostack[--sp]);
	st = pickRandomBreedHere(tempint, tempint2);
	if (st != null) ostack[sp++]=st.whoInt;
	else ostack[sp++] = minusoneInt;
	break;
      case OLBA:
	y = sl.halfheight - aDouble(ostack[--sp]);
	x = sl.halfwidth + aDouble(ostack[--sp]);
	tempint = sl.patchat(x, y);
	tempint2 = aInteger(ostack[--sp]);
	partners = new Vector();
	st = sl.patches[tempint].topturtle;
	while (st != null) {
	  if (st.breed == tempint2) partners.addElement(st.whoInt);
	  st = st.underme;
	}
	st = sl.patches[tempint].tophiddenturtle;
	while (st != null) {
	  if (st.breed == tempint2) partners.addElement(st.whoInt);
	  st = st.underme;
	}
	list = new Object[partners.size()];
	partners.copyInto(list);
	ostack[sp++] = list;
	break;
      case PLBA:
	y = pat.ycor - 1 - aDouble(ostack[--sp]);
	x = pat.xcor - 1 + aDouble(ostack[--sp]);
	tempint = sl.patchat(x, y);
	st = sl.patches[tempint].topturtle;
	tempint2 = aInteger(ostack[--sp]);
	partners = new Vector();
	while (st != null) {
	  if (st.breed == tempint2) partners.addElement(st.whoInt);
	  st = st.underme;
	}
	st = sl.patches[tempint].tophiddenturtle;
	while (st != null) {
	  if (st.breed == tempint2) partners.addElement(st.whoInt);
	  st = st.underme;
	}
	list = new Object[partners.size()];
	partners.copyInto(list);
	ostack[sp++] = list;
	break;
      case TLBA:
	y = tur.ycor - aDouble(ostack[--sp]);
	x = tur.xcor + aDouble(ostack[--sp]);
	tempint = sl.patchat(x, y);
	st = sl.patches[tempint].topturtle;
	tempint2 = aInteger(ostack[--sp]);
	partners = new Vector();
	while (st != null) {
	  if (st != tur && st.breed == tempint2) partners.addElement(st.whoInt);
	  st = st.underme;
	}
	st = sl.patches[tempint].tophiddenturtle;
	while (st != null) {
	  if (st != tur && st.breed == tempint2) partners.addElement(st.whoInt);
	  st = st.underme;
	}
	list = new Object[partners.size()];
	partners.copyInto(list);
	ostack[sp++] = list;
	break;
      case OLBT:
	y = aDouble(ostack[--sp]); // distance
	x = aDouble(ostack[--sp]); // angle
	tempint = sl.patchtowards(x, y, sl.halfwidth, sl.halfheight);
	st = sl.patches[tempint].topturtle;
	tempint2 = aInteger(ostack[--sp]);
	partners = new Vector();
	while (st != null) {
	  if (st.breed == tempint2) partners.addElement(st.whoInt);
	  st = st.underme;
	}
	st = sl.patches[tempint].tophiddenturtle;
	while (st != null) {
	  if (st.breed == tempint2) partners.addElement(st.whoInt);
	  st = st.underme;
	}
	list = new Object[partners.size()];
	partners.copyInto(list);
	ostack[sp++] = list;
	break;
      case TLBT:
	y = aDouble(ostack[--sp]); // distance
	x = aDouble(ostack[--sp]); // angle
	tempint = sl.patchtowards(x + tur.heading, y, tur.xcor, tur.ycor);
	st = sl.patches[tempint].topturtle;
	tempint2 = aInteger(ostack[--sp]);
	partners = new Vector();
	while (st != null) {
	  if (st != tur && st.breed == tempint2) partners.addElement(st.whoInt);
	  st = st.underme;
	}
	st = sl.patches[tempint].tophiddenturtle;
	while (st != null) {
	  if (st != tur && st.breed == tempint2) partners.addElement(st.whoInt);
	  st = st.underme;
	}
	list = new Object[partners.size()];
	partners.copyInto(list);
	ostack[sp++] = list;
	break;
      case PLBT:
	y = aDouble(ostack[--sp]); // distance
	x = aDouble(ostack[--sp]); // angle
	tempint = sl.patchtowards(x, y, pat.xcor-1, pat.ycor-1);
	st = sl.patches[tempint].topturtle;
	tempint2 = aInteger(ostack[--sp]);
	partners = new Vector();
	while (st != null) {
	  if (st.breed == tempint2) partners.addElement(st.whoInt);
	  st = st.underme;
	}
	st = sl.patches[tempint].tophiddenturtle;
	while (st != null) {
	  if (st.breed == tempint2) partners.addElement(st.whoInt);
	  st = st.underme;
	}
	list = new Object[partners.size()];
	partners.copyInto(list);
	ostack[sp++] = list;
	break;
	}
  }


  public void primAsk(Context c, Observer obs, Turtle tur, Patch pat) {
      int tempint2;
    Turtle[] turtles;
    Vector partners;
    int trueip, tempint;
    double x, y;
    Turtle st;
    Object[] list;
    //System.out.println("primAsk command: " + here);
    switch (here & 0xFF) {
      /* Inter-VM Communication */
    case OASKPATCH:
      y = sl.halfheight - aDouble(ostack[--sp]);
      x = sl.halfwidth + aDouble(ostack[--sp]);
      tempint = sl.patchat(x, y);
      Patch p = sl.patches[tempint];
      trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runPatchSlow(sl, process, process.whichip, trueip, DONE, p, false, process.spp);
      if (process.vmspp == null) break;
      ostack[sp++] = process.vmspp;
      here = (ASK2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case TASKPATCH:
      y = tur.ycor - aDouble(ostack[--sp]);
      x = tur.xcor + aDouble(ostack[--sp]);
      tempint = sl.patchat(x, y);
      p = sl.patches[tempint];
      trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runPatchSlow(sl, process, process.whichip, trueip, DONE, p, false, process.spp);
      if (process.vmspp == null) break;
      ostack[sp++] = process.vmspp;
      here = (ASK2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case PASKPATCH:
      y = pat.ycor - 1 - aDouble(ostack[--sp]);
      x = pat.xcor - 1 + aDouble(ostack[--sp]);
      tempint = sl.patchat(x, y);
      p = sl.patches[tempint];
      trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runPatchSlow(sl, process, process.whichip, trueip, DONE, p, false, process.spp);
      if (process.vmspp == null) break;
      ostack[sp++] = process.vmspp;
      here = (ASK2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case ASKTURTLE:
      st = sl.getWho(aInteger(ostack[--sp]));
      trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtle(sl, process, process.whichip, trueip, DONE, st, false, process.spp);
      if (process.vmspp == null) break;
      ostack[sp++] = process.vmspp;
      here = (ASK2 | (PRIMASK << 8) | (OTHER << 16)); 
      return;
    case ASKOBS:
      trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runObserver(sl, process, process.whichip, trueip, DONE, false, process.spp);
      if (process.vmspp == null) break;
      ostack[sp++] = process.vmspp;
      here = (ASK2 | (PRIMASK << 8) | (OTHER << 16)); 
      return;
    case ASKPATCHES:
      trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runPatches(sl, process, process.whichip, trueip, DONE, false, process.spp);
      if (process.vmspp == null) break;
      ostack[sp++] = process.vmspp;
      here = (ASK2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case ASKPATCHESSLOW:
      trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runPatchesSlow(sl, process, process.whichip, trueip, DONE, false, process.spp);
      if (process.vmspp == null) break;
      ostack[sp++] = process.vmspp;
      here = (ASK2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case ASKTLIST:
      trueip = istack[--isp];
      list = aList(ostack[--sp]);
      //System.out.println("ask tlist: " + YoYo.printToString(list));
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtles(sl, process, process.whichip, trueip, DONE, aTurtleList(list), false, process.spp);
      if (process.vmspp == null) break;
      ostack[sp++] = process.vmspp;
      here = (ASK2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case ASKTURTLES:
      trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtles(sl, process, process.whichip, trueip, DONE, false, process.spp);
      if (process.vmspp == null) break;
      ostack[sp++] = process.vmspp;
      here = (ASK2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case ASKBREEDS:
      trueip = istack[--isp];
      x = aDouble(ostack[--sp]);
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtles(sl, process, process.whichip, trueip, DONE, x, false, process.spp);
      if (process.vmspp == null) break;
      ostack[sp++] = process.vmspp;
      here = (ASK2 | (PRIMASK << 8) | (OTHER << 16)); return;
      /* Picking a random turtle with a Predicate */
    case TONETURTLEWITH:
      trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip,
							  trueip, DONE, false, process.spp, tur);
      if (process.vmspp == null) {
	//System.out.println("otw = null");
	ostack[sp++] = minusoneInt;
	break;
      }
      //System.out.println("process.vmspp = " + process.vmspp);
      ostack[sp++] = process.vmspp;
      here = (OTW2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case TONEBREEDWITH:
      trueip = istack[--isp];
      x = aDouble(ostack[--sp]);
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip, trueip, DONE, x, false, process.spp, tur);
      if (process.vmspp == null) {
	ostack[sp++] = minusoneInt;
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (OTW2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case OPONETURTLEWITH:
      trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip,
							  trueip, DONE, false, process.spp);
      if (process.vmspp == null) {
	//System.out.println("otw = null");
	ostack[sp++] = minusoneInt;
	break;
      }
      //System.out.println("process.vmspp = " + process.vmspp);
      ostack[sp++] = process.vmspp;
      here = (OTW2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case OPONEBREEDWITH:
      trueip = istack[--isp];
      x = aDouble(ostack[--sp]);
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip, trueip, DONE, x, false, process.spp);
      if (process.vmspp == null) {
	ostack[sp++] = minusoneInt;
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (OTW2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case TAVGTURTLES:
      trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip, trueip, DONE, false, process.spp, tur);
      if (process.vmspp == null) {
	ostack[sp++] = zeroInt;
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (AVGT2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case OPAVGTURTLES:
      trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip, trueip, DONE, false, process.spp);
      if (process.vmspp == null) {
	ostack[sp++] = zeroInt;
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (AVGT2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case TAVGBREEDS:
      trueip = istack[--isp];
      x = aDouble(ostack[--sp]);
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip, trueip, DONE, x, false, process.spp, tur);
      if (process.vmspp == null) {
	ostack[sp++] = zeroInt;
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (AVGT2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case OPAVGBREEDS:
      trueip = istack[--isp];
      x = aDouble(ostack[--sp]);
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip, trueip, DONE, x, false, process.spp);
      if (process.vmspp == null) {
	ostack[sp++] = zeroInt;
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (AVGT2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case AVGPATCHES:
      trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runPatchesOutput(sl, process, process.whichip, trueip, DONE, false, process.spp);
      if (process.vmspp == null) {
	ostack[sp++] = zeroInt;
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (AVGT2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case AVGPATCHESSLOW:
      trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runPatchesOutputSlow(sl, process, process.whichip, trueip, DONE, false, process.spp);
      if (process.vmspp == null) {
	ostack[sp++] = zeroInt;
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (AVGT2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case TVARTURTLES:
      trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip, trueip, DONE, false, process.spp, tur);
      if (process.vmspp == null) {
	ostack[sp++] = zeroInt;
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (VART2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case OPVARTURTLES:
      trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip, trueip, DONE, false, process.spp);
      if (process.vmspp == null) {
	ostack[sp++] = zeroInt;
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (VART2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case TVARBREEDS:
      trueip = istack[--isp];
      x = aDouble(ostack[--sp]);
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip, trueip, DONE, x, false, process.spp, tur);
      if (process.vmspp == null) {
	ostack[sp++] = zeroInt;
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (VART2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case OPVARBREEDS:
      trueip = istack[--isp];
      x = aDouble(ostack[--sp]);
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip, trueip, DONE, x, false, process.spp);
      if (process.vmspp == null) {
	ostack[sp++] = zeroInt;
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (VART2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case VARPATCHES:
      trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runPatchesOutput(sl, process, process.whichip, trueip, DONE, false, process.spp);
      if (process.vmspp == null) {
	ostack[sp++] = zeroInt;
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (VART2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case VARPATCHESSLOW:
      trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runPatchesOutputSlow(sl, process, process.whichip, trueip, DONE, false, process.spp);
      if (process.vmspp == null) {
	ostack[sp++] = zeroInt;
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (VART2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case TMEDTURTLES:
      trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip, trueip, DONE, false, process.spp, tur);
      if (process.vmspp == null) {
	ostack[sp++] = zeroInt;
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (MEDT2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case OPMEDTURTLES:
      trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip, trueip, DONE, false, process.spp);
      if (process.vmspp == null) {
	ostack[sp++] = zeroInt;
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (MEDT2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case TMEDBREEDS:
      trueip = istack[--isp];
      x = aDouble(ostack[--sp]);
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip, trueip, DONE, x, false, process.spp, tur);
      if (process.vmspp == null) {
	ostack[sp++] = zeroInt;
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (MEDT2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case OPMEDBREEDS:
      trueip = istack[--isp];
      x = aDouble(ostack[--sp]);
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip, trueip, DONE, x, false, process.spp);
      if (process.vmspp == null) {
	ostack[sp++] = zeroInt;
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (MEDT2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case MEDPATCHES:
      trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runPatchesOutput(sl, process, process.whichip, trueip, DONE, false, process.spp);
      if (process.vmspp == null) {
	ostack[sp++] = zeroInt;
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (MEDT2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case MEDPATCHESSLOW:
      trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runPatchesOutputSlow(sl, process, process.whichip, trueip, DONE, false, process.spp);
      if (process.vmspp == null) {
	ostack[sp++] = zeroInt;
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (MEDT2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case TMODETURTLES:
	trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip, trueip, DONE, false, process.spp, tur);
      if (process.vmspp == null) {
	ostack[sp++] = zeroInt;
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (MODET2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case OPMODETURTLES:
	trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip, trueip, DONE, false, process.spp);
      if (process.vmspp == null) {
	ostack[sp++] = zeroInt;
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (MODET2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case TMODEBREEDS:
	trueip = istack[--isp];
      x = aDouble(ostack[--sp]);
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip, trueip, DONE, x, false, process.spp, tur);
      if (process.vmspp == null) {
	ostack[sp++] = zeroInt;
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (MODET2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case OPMODEBREEDS:
	trueip = istack[--isp];
      x = aDouble(ostack[--sp]);
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip, trueip, DONE, x, false, process.spp);
      if (process.vmspp == null) {
	ostack[sp++] = zeroInt;
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (MODET2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case MODEPATCHES:
	 trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runPatchesOutput(sl, process, process.whichip, trueip, DONE, false, process.spp);
      if (process.vmspp == null) {
	ostack[sp++] = zeroInt;
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (MODET2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case MODEPATCHESSLOW:
	 trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runPatchesOutputSlow(sl, process, process.whichip, trueip, DONE, false, process.spp);
      if (process.vmspp == null) {
	ostack[sp++] = zeroInt;
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (MODET2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case TSDEVTURTLES:
	trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip, trueip, DONE, false, process.spp, tur);
      if (process.vmspp == null) {
	ostack[sp++] = zeroInt;
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (SDEVT2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case OPSDEVTURTLES:
	trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip, trueip, DONE, false, process.spp);
      if (process.vmspp == null) {
	ostack[sp++] = zeroInt;
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (SDEVT2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case TSDEVBREEDS:
	trueip = istack[--isp];
      x = aDouble(ostack[--sp]);
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip, trueip, DONE, x, false, process.spp, tur);
      if (process.vmspp == null) {
	ostack[sp++] = zeroInt;
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (SDEVT2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case OPSDEVBREEDS:
	trueip = istack[--isp];
      x = aDouble(ostack[--sp]);
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip, trueip, DONE, x, false, process.spp);
      if (process.vmspp == null) {
	ostack[sp++] = zeroInt;
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (SDEVT2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case SDEVPATCHES:
	 trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runPatchesOutput(sl, process, process.whichip, trueip, DONE, false, process.spp);
      if (process.vmspp == null) {
	ostack[sp++] = zeroInt;
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (SDEVT2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case SDEVPATCHESSLOW:
	 trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runPatchesOutputSlow(sl, process, process.whichip, trueip, DONE, false, process.spp);
      if (process.vmspp == null) {
	ostack[sp++] = zeroInt;
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (SDEVT2 | (PRIMASK << 8) | (OTHER << 16)); return;
	
    case COUNTTURTLESWITH:
      trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip,
							  trueip, DONE, false, process.spp);
      if (process.vmspp == null) {
	//System.out.println("ctw = null");
	ostack[sp++] = zeroInt;
	break;
      }
      //System.out.println("process.vmspp = " + process.vmspp);
      ostack[sp++] = process.vmspp;
      here = (CTW2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case COUNTBREEDSWITH:
      trueip = istack[--isp];
      x = aDouble(ostack[--sp]);
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip, trueip, DONE, x, false, process.spp);
      if (process.vmspp == null) {
	ostack[sp++] = zeroInt;
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (CTW2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case COUNTPATCHESWITH:
      trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runPatchesOutput(sl, process, process.whichip, trueip, DONE, false, process.spp);
      if (process.vmspp == null) {
	ostack[sp++] = zeroInt;
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (CTW2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case COUNTPATCHESWITHSLOW:
      trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runPatchesOutputSlow(sl, process, process.whichip, trueip, DONE, false, process.spp);
      if (process.vmspp == null) {
	ostack[sp++] = zeroInt;
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (CTW2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case TSUMTURTLES:
      trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip, trueip, DONE, false, process.spp, tur);
      if (process.vmspp == null) {
	ostack[sp++] = zeroInt;
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (ST2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case OPSUMTURTLES:
      trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip, trueip, DONE, false, process.spp);
      if (process.vmspp == null) {
	ostack[sp++] = zeroInt;
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (ST2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case TSUMBREEDS:
      trueip = istack[--isp];
      x = aDouble(ostack[--sp]);
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip, trueip, DONE, x, false, process.spp, tur);
      if (process.vmspp == null) {
	ostack[sp++] = zeroInt;
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (ST2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case OPSUMBREEDS:
      trueip = istack[--isp];
      x = aDouble(ostack[--sp]);
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip, trueip, DONE, x, false, process.spp);
      if (process.vmspp == null) {
	ostack[sp++] = zeroInt;
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (ST2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case SUMPATCHES:
      trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runPatchesOutput(sl, process, process.whichip, trueip, DONE, false, process.spp);
      if (process.vmspp == null) {
	ostack[sp++] = zeroInt;
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (ST2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case SUMPATCHESSLOW:
      trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runPatchesOutputSlow(sl, process, process.whichip, trueip, DONE, false, process.spp);
      if (process.vmspp == null) {
	ostack[sp++] = zeroInt;
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (ST2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case TLISTTURTLESWITH:
      trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip, trueip, DONE, false, process.spp, tur);
      if (process.vmspp == null) {
	ostack[sp++] = new Object[0];
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (LTW2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case OPLISTTURTLESWITH:
      trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip, trueip, DONE, false, process.spp);
      if (process.vmspp == null) {
	ostack[sp++] = new Object[0];
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (LTW2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case TLISTBREEDSWITH:
      trueip = istack[--isp];
      x = aDouble(ostack[--sp]);
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip, trueip, DONE, x, false, process.spp, tur);
      if (process.vmspp == null) {
	ostack[sp++] = new Object[0];
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (LTW2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case OPLISTBREEDSWITH:
      trueip = istack[--isp];
      x = aDouble(ostack[--sp]);
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip, trueip, DONE, x, false, process.spp);
      if (process.vmspp == null) {
	ostack[sp++] = new Object[0];
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (LTW2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case MINPATCHESSLOW:
      trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runPatchesOutputSlow(sl, process, process.whichip, trueip, DONE, false, process.spp);
      if (process.vmspp == null) {
	ostack[sp++] = new Double(Double.MAX_VALUE);
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (MINT2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case MAXPATCHESSLOW:
      trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runPatchesOutputSlow(sl, process, process.whichip, trueip, DONE, false, process.spp);
      if (process.vmspp == null) {
	ostack[sp++] = new Double(-Double.MAX_VALUE);
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (MAXT2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case TMINTURTLES:
      trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip, trueip, DONE, false, process.spp, tur);
      if (process.vmspp == null) {
	ostack[sp++] = new Double(Double.MAX_VALUE);
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (MINT2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case OPMINTURTLES:
      trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip, trueip, DONE, false, process.spp);
      if (process.vmspp == null) {
	ostack[sp++] = new Double(Double.MAX_VALUE);
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (MINT2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case TMINBREEDS:
      trueip = istack[--isp];
      x = aDouble(ostack[--sp]);
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip, trueip, DONE, x, false, process.spp, tur);
      if (process.vmspp == null) {
	ostack[sp++] = new Double(Double.MAX_VALUE);
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (MINT2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case OPMINBREEDS:
      trueip = istack[--isp];
      x = aDouble(ostack[--sp]);
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip, trueip, DONE, x, false, process.spp);
      if (process.vmspp == null) {
	ostack[sp++] = new Double(Double.MAX_VALUE);
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (MINT2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case TMAXTURTLES:
      trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip, trueip, DONE, false, process.spp, tur);
      if (process.vmspp == null) {
	ostack[sp++] = new Double(-Double.MAX_VALUE);
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (MAXT2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case OPMAXTURTLES:
      trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip, trueip, DONE, false, process.spp);
      if (process.vmspp == null) {
	ostack[sp++] = new Double(-Double.MAX_VALUE);
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (MAXT2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case TMAXBREEDS:
      trueip = istack[--isp];
      x = aDouble(ostack[--sp]);
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip, trueip, DONE, x, false, process.spp, tur);
      if (process.vmspp == null) {
	ostack[sp++] = new Double(-Double.MAX_VALUE);
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (MAXT2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case OPMAXBREEDS:
      trueip = istack[--isp];
      x = aDouble(ostack[--sp]);
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip, trueip, DONE, x, false, process.spp);
      if (process.vmspp == null) {
	ostack[sp++] = new Double(-Double.MAX_VALUE);
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (MAXT2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case TWHOMINTURTLES:
      trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip, trueip, DONE, false, process.spp, tur);
      if (process.vmspp == null) {
	ostack[sp++] = new Object[]{ minusoneInt, new Double(Double.MAX_VALUE)};
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (WMINT2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case OPWHOMINTURTLES:
      trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip, trueip, DONE, false, process.spp);
      if (process.vmspp == null) {
	ostack[sp++] = new Object[] { minusoneInt, new Double(-Double.MAX_VALUE)};
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (WMINT2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case TWHOMAXTURTLES:
      trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip, trueip, DONE, false, process.spp, tur);
      if (process.vmspp == null) {
	ostack[sp++] = new Object[]{ minusoneInt, new Double(-Double.MAX_VALUE) };
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (WMAXT2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case OPWHOMAXTURTLES:
      trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip, trueip, DONE, false, process.spp);
      if (process.vmspp == null) {
	ostack[sp++] = new Object[]{ minusoneInt, new Double(-Double.MAX_VALUE)};
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (WMAXT2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case TWHOMINBREEDS:
      trueip = istack[--isp];
      x = aDouble(ostack[--sp]);
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip, trueip, DONE, x, false, process.spp, tur);
      if (process.vmspp == null) {
	ostack[sp++] = new Object[]{ minusoneInt, new Double(Double.MAX_VALUE)};
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (WMINT2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case OPWHOMINBREEDS:
      trueip = istack[--isp];
      x = aDouble(ostack[--sp]);
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip, trueip, DONE, x, false, process.spp);
      if (process.vmspp == null) {
	ostack[sp++] = new Object[] { minusoneInt, new Double(-Double.MAX_VALUE)};
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (WMINT2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case TWHOMAXBREEDS:
      trueip = istack[--isp];
      x = aDouble(ostack[--sp]);
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip, trueip, DONE, x, false, process.spp, tur);
      if (process.vmspp == null) {
	ostack[sp++] = new Object[]{ minusoneInt, new Double(-Double.MAX_VALUE) };
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (WMAXT2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case OPWHOMAXBREEDS:
      trueip = istack[--isp];
      x = aDouble(ostack[--sp]);
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip, trueip, DONE, x, false, process.spp);
      if (process.vmspp == null) {
	ostack[sp++] = new Object[]{ minusoneInt, new Double(-Double.MAX_VALUE)};
	break;
      }
      ostack[sp++] = process.vmspp;
      here = (WMAXT2 | (PRIMASK << 8) | (OTHER << 16)); return;

  case TONETURTLEHEREWITH:     

      partners = new Vector();
      st = sl.patches[tur.patch].topturtle;
      while (st != null) {
	  if (st != tur) partners.addElement(st);
	  st = st.underme;
      }
      st = sl.patches[tur.patch].tophiddenturtle;
      while (st != null) {
	  if (st != tur) partners.addElement(st);
	  st = st.underme;
      }
      turtles = new Turtle[partners.size()];
      partners.copyInto(turtles);
	
      
      trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip,
							  trueip, DONE, false, turtles, process.spp, tur);
      if (process.vmspp == null) {
	//System.out.println("otw = null");
	ostack[sp++] = minusoneInt;
	break;
      }
      //System.out.println("process.vmspp = " + process.vmspp);
      ostack[sp++] = process.vmspp;
      here = (OTW2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case PONETURTLEHEREWITH:

	partners = new Vector();
	st = pat.topturtle;
	while (st != null) {
	  partners.addElement(st);
	  st = st.underme;
	}
	st = pat.tophiddenturtle;
	while (st != null) {
	  partners.addElement(st);
	  st = st.underme;
	}
	turtles = new Turtle[partners.size()];
	partners.copyInto(turtles);
	trueip = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip,
							  trueip, DONE, false, turtles, process.spp, tur);
      if (process.vmspp == null) {
	//System.out.println("otw = null");
	ostack[sp++] = minusoneInt;
	break;
      }
      //System.out.println("process.vmspp = " + process.vmspp);
      ostack[sp++] = process.vmspp;
      here = (OTW2 | (PRIMASK << 8) | (OTHER << 16)); return;
     
    case TONEBREEDHEREWITH:
	
	tempint2 = aInteger(ostack[--sp]);
	partners = new Vector();
	st = sl.patches[tur.patch].topturtle;
	while (st != null) {
	  if (st != tur && st.breed == tempint2) partners.addElement(st);
	  st = st.underme;
	}
	st = sl.patches[tur.patch].tophiddenturtle;
	while (st != null) {
	  if (st != tur && st.breed == tempint2) partners.addElement(st);
	  st = st.underme;
	}
	turtles = new Turtle[partners.size()];
	partners.copyInto(turtles);
	trueip = istack[--isp];
	process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip,
							  trueip, DONE, false, turtles, process.spp, tur);
      if (process.vmspp == null) {
	//System.out.println("otw = null");
	ostack[sp++] = minusoneInt;
	break;
      }
      //System.out.println("process.vmspp = " + process.vmspp);
      ostack[sp++] = process.vmspp;
      here = (OTW2 | (PRIMASK << 8) | (OTHER << 16)); return;

    case PONEBREEDHEREWITH:
	tempint2 = aInteger(ostack[--sp]);
	partners = new Vector();
	st = pat.topturtle;
	while (st != null) {
	  if (st.breed == tempint2) partners.addElement(st);
	  st = st.underme;
	}
	st = pat.tophiddenturtle;
	while (st != null) {
	  if (st.breed == tempint2) partners.addElement(st);
	  st = st.underme;
	}
	turtles = new Turtle[partners.size()];
	partners.copyInto(turtles);
	trueip = istack[--isp];
	process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip,
							  trueip, DONE, false, turtles, process.spp, tur);
      if (process.vmspp == null) {
	//System.out.println("otw = null");
	ostack[sp++] = minusoneInt;
	break;
      }
      //System.out.println("process.vmspp = " + process.vmspp);
      ostack[sp++] = process.vmspp;
      here = (OTW2 | (PRIMASK << 8) | (OTHER << 16)); return;


    case TONETURTLEATWITH:
	y = tur.ycor - aDouble(ostack[--sp]);
	x = tur.xcor + aDouble(ostack[--sp]);
	//int a = istack[--sp];
	//int b = istack[--sp];
	trueip = istack[--isp];
	tempint = sl.patchat(x, y);
	partners = new Vector();
	st = sl.patches[tempint].topturtle;
	while (st != null) {
	  if (st != tur) partners.addElement(st);
	  st = st.underme;
	}
	st = sl.patches[tempint].tophiddenturtle;
	while (st != null) {
	  if (st != tur) partners.addElement(st);
	  st = st.underme;
	}
	turtles = new Turtle[partners.size()];
	partners.copyInto(turtles);
        	
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip,
							  trueip, DONE, false, turtles, process.spp, tur);
      if (process.vmspp == null) {
	//System.out.println("otw = null");
	ostack[sp++] = minusoneInt;
	break;
      }
      //System.out.println("process.vmspp = " + process.vmspp);
      ostack[sp++] = process.vmspp;
      here = (OTW2 | (PRIMASK << 8) | (OTHER << 16)); return;

    case OONETURTLEATWITH:
	trueip = istack[--isp];
	y = sl.halfheight - aDouble(ostack[--sp]);
	x = sl.halfwidth + aDouble(ostack[--sp]);
	tempint = sl.patchat(x, y);
	partners = new Vector();
	st = sl.patches[tempint].topturtle;
	while (st != null) {
	  partners.addElement(st);
	  st = st.underme;
	}
	st = sl.patches[tempint].tophiddenturtle;
	while (st != null) {
	  partners.addElement(st);
	  st = st.underme;
	}
	turtles = new Turtle[partners.size()];
	partners.copyInto(turtles);

	
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip,
							  trueip, DONE, false, turtles, process.spp, tur);
      if (process.vmspp == null) {
	//System.out.println("otw = null");
	ostack[sp++] = minusoneInt;
	break;
      }
      //System.out.println("process.vmspp = " + process.vmspp);
      ostack[sp++] = process.vmspp;
      here = (OTW2 | (PRIMASK << 8) | (OTHER << 16)); return;

    case PONETURTLEATWITH:
	trueip = istack[--isp];
	y = pat.ycor - 1 - aDouble(ostack[--sp]);
	x = pat.xcor - 1 + aDouble(ostack[--sp]);
	tempint = sl.patchat(x, y);
	partners = new Vector();
	st = sl.patches[tempint].topturtle;
	while (st != null) {
	  partners.addElement(st);
	  st = st.underme;
	}
	st = sl.patches[tempint].tophiddenturtle;
	while (st != null) {
	  partners.addElement(st);
	  st = st.underme;
	}
	turtles = new Turtle[partners.size()];
	partners.copyInto(turtles);

	process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip,
							  trueip, DONE, false, turtles, process.spp, tur);
      if (process.vmspp == null) {
	//System.out.println("otw = null");
	ostack[sp++] = minusoneInt;
	break;
      }
      //System.out.println("process.vmspp = " + process.vmspp);
      ostack[sp++] = process.vmspp;
      here = (OTW2 | (PRIMASK << 8) | (OTHER << 16)); return;

    case TONEBREEDATWITH:
	y = tur.ycor - aDouble(ostack[--sp]);
	x = tur.xcor + aDouble(ostack[--sp]);
	tempint = sl.patchat(x, y);
	st = sl.patches[tempint].topturtle;
	tempint2 = aInteger(ostack[--sp]);
	partners = new Vector();
	while (st != null) {
	  if (st != tur && st.breed == tempint2) partners.addElement(st);
	  st = st.underme;
	}
	st = sl.patches[tempint].tophiddenturtle;
	while (st != null) {
	  if (st != tur && st.breed == tempint2) partners.addElement(st);
	  st = st.underme;
	}
	turtles = new Turtle[partners.size()];
	partners.copyInto(turtles);
	trueip = istack[--isp];
	process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip,
							  trueip, DONE, false, turtles, process.spp, tur);
      if (process.vmspp == null) {
	//System.out.println("otw = null");
	ostack[sp++] = minusoneInt;
	break;
      }
      //System.out.println("process.vmspp = " + process.vmspp);
      ostack[sp++] = process.vmspp;
      here = (OTW2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case OONEBREEDATWITH:
	y = sl.halfheight - aDouble(ostack[--sp]);
	x = sl.halfwidth + aDouble(ostack[--sp]);
	tempint = sl.patchat(x, y);
	tempint2 = aInteger(ostack[--sp]);
	partners = new Vector();
	st = sl.patches[tempint].topturtle;
	while (st != null) {
	  if (st.breed == tempint2) partners.addElement(st);
	  st = st.underme;
	}
	st = sl.patches[tempint].tophiddenturtle;
	while (st != null) {
	  if (st.breed == tempint2) partners.addElement(st);
	  st = st.underme;
	}
	turtles = new Turtle[partners.size()];
	partners.copyInto(turtles);
	trueip = istack[--isp];
	process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip,
							  trueip, DONE, false, turtles, process.spp, tur);
      if (process.vmspp == null) {
	//System.out.println("otw = null");
	ostack[sp++] = minusoneInt;
	break;
      }
      //System.out.println("process.vmspp = " + process.vmspp);
      ostack[sp++] = process.vmspp;
      here = (OTW2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case PONEBREEDATWITH:
	y = pat.ycor - 1 - aDouble(ostack[--sp]);
	x = pat.xcor - 1 + aDouble(ostack[--sp]);
	tempint = sl.patchat(x, y);
	st = sl.patches[tempint].topturtle;
	tempint2 = aInteger(ostack[--sp]);
	partners = new Vector();
	while (st != null) {
	  if (st.breed == tempint2) partners.addElement(st);
	  st = st.underme;
	}
	st = sl.patches[tempint].tophiddenturtle;
	while (st != null) {
	  if (st.breed == tempint2) partners.addElement(st);
	  st = st.underme;
	}
	turtles = new Turtle[partners.size()];
	partners.copyInto(turtles);
	trueip = istack[--isp];
	process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip,
							  trueip, DONE, false, turtles, process.spp, tur);
      if (process.vmspp == null) {
	//System.out.println("otw = null");
	ostack[sp++] = minusoneInt;
	break;
      }
      //System.out.println("process.vmspp = " + process.vmspp);
      ostack[sp++] = process.vmspp;
      here = (OTW2 | (PRIMASK << 8) | (OTHER << 16)); return;

    case TONETURTLETOWITH:
	y = aDouble(ostack[--sp]); // distance
	x = aDouble(ostack[--sp]); // angle
	tempint = sl.patchtowards(x + tur.heading, y, tur.xcor, tur.ycor);
	partners = new Vector();
	st = sl.patches[tempint].topturtle;
	while (st != null) {
	  if (st != tur) partners.addElement(st);
	  st = st.underme;
	}
	st = sl.patches[tempint].tophiddenturtle;
	while (st != null) {
	  if (st != tur) partners.addElement(st);
	  st = st.underme;
	}
	turtles = new Turtle[partners.size()];
	partners.copyInto(turtles);
	trueip = istack[--isp];
	process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip,
							  trueip, DONE, false, turtles, process.spp, tur);
      if (process.vmspp == null) {
	//System.out.println("otw = null");
	ostack[sp++] = minusoneInt;
	break;
      }
      //System.out.println("process.vmspp = " + process.vmspp);
      ostack[sp++] = process.vmspp;
      here = (OTW2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case OONETURTLETOWITH:
	y = aDouble(ostack[--sp]); // distance
	x = aDouble(ostack[--sp]); // angle
	tempint = sl.patchtowards(x, y, sl.halfwidth, sl.halfheight);
	partners = new Vector();
	st = sl.patches[tempint].topturtle;
	while (st != null) {
	  partners.addElement(st);
	  st = st.underme;
	}
	st = sl.patches[tempint].tophiddenturtle;
	while (st != null) {
	  partners.addElement(st);
	  st = st.underme;
	}
	turtles = new Turtle[partners.size()];
	partners.copyInto(turtles);
	trueip = istack[--isp];
	process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip,
							  trueip, DONE, false, turtles, process.spp, tur);
      if (process.vmspp == null) {
	//System.out.println("otw = null");
	ostack[sp++] = minusoneInt;
	break;
      }
      //System.out.println("process.vmspp = " + process.vmspp);
      ostack[sp++] = process.vmspp;
      here = (OTW2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case PONETURTLETOWITH:
	y = aDouble(ostack[--sp]); // distance
	x = aDouble(ostack[--sp]); // angle
	tempint = sl.patchtowards(x, y, pat.xcor-1, pat.ycor-1);
	partners = new Vector();
	st = sl.patches[tempint].topturtle;
	while (st != null) {
	  partners.addElement(st);
	  st = st.underme;
	}
	st = sl.patches[tempint].tophiddenturtle;
	while (st != null) {
	  partners.addElement(st);
	  st = st.underme;
	}
	turtles = new Turtle[partners.size()];
	partners.copyInto(turtles);
	trueip = istack[--isp];
	process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip,
							  trueip, DONE, false, turtles, process.spp, tur);
      if (process.vmspp == null) {
	//System.out.println("otw = null");
	ostack[sp++] = minusoneInt;
	break;
      }
      //System.out.println("process.vmspp = " + process.vmspp);
      ostack[sp++] = process.vmspp;
      here = (OTW2 | (PRIMASK << 8) | (OTHER << 16)); return;

    case TONEBREEDTOWITH:
	y = aDouble(ostack[--sp]); // distance
	x = aDouble(ostack[--sp]); // angle
	tempint = sl.patchtowards(x + tur.heading, y, tur.xcor, tur.ycor);
	st = sl.patches[tempint].topturtle;
	tempint2 = aInteger(ostack[--sp]);
	partners = new Vector();
	while (st != null) {
	  if (st != tur && st.breed == tempint2) partners.addElement(st);
	  st = st.underme;
	}
	st = sl.patches[tempint].tophiddenturtle;
	while (st != null) {
	  if (st != tur && st.breed == tempint2) partners.addElement(st);
	  st = st.underme;
	}
	turtles = new Turtle[partners.size()];
	partners.copyInto(turtles);
	trueip = istack[--isp];
	process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip,
							  trueip, DONE, false, turtles, process.spp, tur);
      if (process.vmspp == null) {
	//System.out.println("otw = null");
	ostack[sp++] = minusoneInt;
	break;
      }
      //System.out.println("process.vmspp = " + process.vmspp);
      ostack[sp++] = process.vmspp;
      here = (OTW2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case OONEBREEDTOWITH:
	y = aDouble(ostack[--sp]); // distance
	x = aDouble(ostack[--sp]); // angle
	tempint = sl.patchtowards(x, y, sl.halfwidth, sl.halfheight);
	st = sl.patches[tempint].topturtle;
	tempint2 = aInteger(ostack[--sp]);
	partners = new Vector();
	while (st != null) {
	  if (st.breed == tempint2) partners.addElement(st);
	  st = st.underme;
	}
	st = sl.patches[tempint].tophiddenturtle;
	while (st != null) {
	  if (st.breed == tempint2) partners.addElement(st);
	  st = st.underme;
	}
	turtles = new Turtle[partners.size()];
	partners.copyInto(turtles);
	trueip = istack[--isp];
	process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip,
							  trueip, DONE, false, turtles, process.spp, tur);
      if (process.vmspp == null) {
	//System.out.println("otw = null");
	ostack[sp++] = minusoneInt;
	break;
      }
      //System.out.println("process.vmspp = " + process.vmspp);
      ostack[sp++] = process.vmspp;
      here = (OTW2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case PONEBREEDTOWITH:
	y = aDouble(ostack[--sp]); // distance
	x = aDouble(ostack[--sp]); // angle
	tempint = sl.patchtowards(x, y, pat.xcor-1, pat.ycor-1);
	st = sl.patches[tempint].topturtle;
	tempint2 = aInteger(ostack[--sp]);
	partners = new Vector();
	while (st != null) {
	  if (st.breed == tempint2) partners.addElement(st);
	  st = st.underme;
	}
	st = sl.patches[tempint].tophiddenturtle;
	while (st != null) {
	  if (st.breed == tempint2) partners.addElement(st);
	  st = st.underme;
	}
	turtles = new Turtle[partners.size()];
	partners.copyInto(turtles);
	trueip = istack[--isp];
	process.sp = sp;
      process.vmspp = StackProcessParent.runTurtlesOutput(sl, process, process.whichip,
							  trueip, DONE, false, turtles, process.spp, tur);
      if (process.vmspp == null) {
	//System.out.println("otw = null");
	ostack[sp++] = minusoneInt;
	break;
      }
      //System.out.println("process.vmspp = " + process.vmspp);
      ostack[sp++] = process.vmspp;
      here = (OTW2 | (PRIMASK << 8) | (OTHER << 16)); return;

    }
    here = iinst[ip++];

  }


  public void primLife(Context c, Observer obs, Turtle tur, Patch pat) {
    Turtle[] sts; Turtle st;
    double x;
    Object val;
    int counter, newaddr;
    switch (here & 0xFF) {
      /* Turtle Life and Death */
    case CRT:
      sts = sl.crt(aInteger(ostack[--sp]));
      StackProcessParent.askTurtleRunForeverButtons(sl, sts);
      break;
    case CCT:
      counter = aInteger(ostack[--sp]); // num
      x = aDouble(ostack[--sp]); // breed
      sts = sl.crt(counter, x, 0);
      StackProcessParent.askTurtleRunForeverButtons(sl, sts);
      break;
    case CRTANDDO:
      sts = sl.crt(aInteger(ostack[--sp]));
      newaddr = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtles(sl, process, process.whichip, newaddr,
						    (HATCH2 | (PRIMLIFE << 8) | (OTHER << 16)),
						    sts, false, process.spp);
      if (process.vmspp == null) break;
      ostack[sp++] = process.vmspp;
      here = (ASK2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case CCTANDDO:
      counter = aInteger(ostack[--sp]); // num
      x = aDouble(ostack[--sp]); // breed
      sts = sl.crt(counter, x, 0);
      newaddr = istack[--isp];
      process.sp = sp;
      process.vmspp = StackProcessParent.runTurtles(sl, process, process.whichip,
						    newaddr, (HATCH2 | (PRIMLIFE << 8) | (OTHER << 16)), sts, false, process.spp);
      if (process.vmspp == null) break;
      ostack[sp++] = process.vmspp;
      here = (ASK2 | (PRIMASK << 8) | (OTHER << 16)); return;
    case DIE:
	  sl.unlink(tur);
	  sl.die(tur);
	  here = (EOL << 8);   // a dying turtle's last utterance
      return;
    case KILL:
      st = sl.getWho(aInteger(ostack[--sp]));
      if (st != null) {sl.unlink(st); sl.die(st); if (st == this) return; }
      break;
    case HATCH:
      st = sl.cct(tur.xcor - sl.turhalfwidth, sl.turhalfheight - tur.ycor,
		  tur.heading, tur.color, tur.breed, tur.shape, tur.pendownp, tur.shownp,
		  tur.plotpen, tur.plotwidget);
      System.arraycopy(tur.turtlesown, 0, st.turtlesown, 0, Turtle.numVars);
      process.sp = sp;
      StackProcessParent.runAsyncTurtle(sl,process, process.whichip, istack[--isp],
					(HATCH2 | (PRIMLIFE << 8) | (OTHER << 16)),
					st, false, process.spp);
      break;
    case SPROUT:
      st = sl.cct(pat.xcor - 1 - sl.halfwidth, sl.halfheight - (pat.ycor - 1), 0,
		  pat.pc, 0, 0);
      process.sp = sp;
      StackProcessParent.runAsyncTurtle(sl, process, process.whichip,
					istack[--isp],
					(HATCH2 | (PRIMLIFE << 8) | (OTHER << 16)), st,
					false, process.spp);
      break;
    }
    here = iinst[ip++];
  }

  public void primPrint(Context c, Observer obs, Turtle tur, Patch pat) {
    switch (here & 0xFF) {
      /* Viewing output */
    case OPRINT:
      if (!sl.app.kioskp && YoYo.applet == null) {
	sl.app.outWin.println(YoYo.printToString(ostack[--sp]));
      } else --sp;
      break;
    case TPRINT:
      if (!sl.app.kioskp && YoYo.applet == null) {
	sl.app.outWin.println("Turtle #" + tur.who + ": " + YoYo.printToString(ostack[--sp]));
      } else --sp;
      break;
    case PPRINT:
      if (!sl.app.kioskp && YoYo.applet == null) {
	sl.app.outWin.println("Patch (" + ((int)(pat.xcor-sl.halfwidth-1)) + ", " +
			      ((int)(sl.halfheight - (pat.ycor-1))) + "): " +
			    YoYo.printToString(ostack[--sp]));
      } else --sp;
      break;
    case OTYPE:
      if (!sl.app.kioskp && YoYo.applet == null) {
	sl.app.outWin.print(YoYo.printToString(ostack[--sp]));
      } else --sp;
      break;
    case TTYPE:
      if (!sl.app.kioskp && YoYo.applet == null) {
	sl.app.outWin.print("Turtle #" + tur.who + ": " + YoYo.printToString(ostack[--sp]));
      } else --sp;
      break;
    case PTYPE:
      if (!sl.app.kioskp && YoYo.applet == null) {
	sl.app.outWin.print("Patch (" + ((int)(pat.xcor-sl.halfwidth-1)) + ", " +
			    ((int)(sl.halfheight-(pat.ycor-1))) + "): " +
			  YoYo.printToString(ostack[--sp]));
      } else --sp;
      break;
    case OSHOW:
      process.spp.callerPrintln(YoYo.printToString(ostack[--sp]));
      break;
    case TSHOW:
      process.spp.callerPrintln("Turtle #" + tur.who + ": " +
				YoYo.printToString(ostack[--sp])); break;
    case PSHOW:
      process.spp.callerPrintln("Patch (" + ((int)(pat.xcor-sl.halfwidth-1)) + ", " +
				((int)(sl.halfheight - (pat.ycor-1))) + "): " +
				YoYo.printToString(ostack[--sp]));
      break;
			case CC:
      process.spp.callerClearText();
      break;
			case SPRINT:
				sl.app.iface.setStatus(YoYo.printToString(ostack[--sp]));
				break;
    }
    here = iinst[ip++];
  }

  public void primListReporters(Context c, Observer obs, Turtle tur, Patch pat) {
      Object thing1, thing2, thing3;
      Object[] list;
      int tempint, counter;

      switch (here & 0xFF) {
	      /* List Operations */
	    case MAKEDOUBLELIST: thing2 = ostack[--sp];
		thing1 = ostack[--sp];
		ostack[sp++] = PrimList.prim_list(thing1, thing2, (Context)null);
		break;
	    case SE: thing2 = ostack[--sp];
		thing1 = ostack[--sp];
		ostack[sp++] = PrimList.prim_sentence(thing1, thing2, (Context)null);
		break;
	    case FPUT: thing2 = ostack[--sp];
		thing1 = ostack[--sp];
		ostack[sp++] = PrimList.prim_fput(thing1, thing2, (Context)null);
		break;
	    case LPUT: thing2 = ostack[--sp];
		thing1 = ostack[--sp];
		ostack[sp++] = PrimList.prim_lput(thing1, thing2, (Context)null);
		break;
	    case BF: thing1 = ostack[--sp];
		ostack[sp++] = PrimList.prim_butfirst(thing1, (Context)null);
		break;
	    case BL: thing1 = ostack[--sp];
		ostack[sp++] = PrimList.prim_butlast(thing1, (Context)null);
		break;
	    case FIRST: thing1 = ostack[--sp];
		ostack[sp++] = PrimList.prim_first(thing1, (Context)null);
		break;
	    case LAST: thing1 = ostack[--sp];
		ostack[sp++] = PrimList.prim_last(thing1, (Context)null);
		break;
	    case EMPTYP: thing1 = ostack[--sp];
		ostack[sp++] = PrimList.prim_emptyp(thing1, (Context)null);
		break;
	    case LENGTH: thing1 = ostack[--sp];
		ostack[sp++] = PrimList.prim_length(thing1, (Context)null);
		break;
	    case MEMBERP: thing2 = ostack[--sp];
		thing1 = ostack[--sp];
		ostack[sp++] = PrimList.prim_memberp(thing1, thing2, (Context)null);
		break;
	    case POSITION: thing2 = ostack[--sp];
		thing1 = ostack[--sp];
		ostack[sp++] = PrimList.prim_position(thing1, thing2, (Context)null);
		break;
	    case RANDOMITEM: thing1 = ostack[--sp];
		ostack[sp++] = PrimList.prim_random_item(thing1, (Context)null);
		break;
	    case ITEM: thing2 = ostack[--sp];
		thing1 = ostack[--sp];
		ostack[sp++] = PrimList.prim_item(thing1, thing2, (Context)null);
		break;
        case TOLIST:
	    // list values are on the ostack
	    tempint = iinst[ip++]; // num elements of list;
	    list = new Object[tempint];
        for(counter = tempint-1; counter >= 0; counter--) {
        	list[counter] = ostack[--sp];
        }
        ostack[sp++] = list;
        break;
        case COPYLIST:
          thing1 = ostack[--sp];
          ostack[sp++] = PrimList.prim_copylist(thing1, (Context)null);
          break;
        case MAKELIST:
          thing2 = ostack[--sp];
          thing1 = ostack[--sp];
          ostack[sp++] = PrimList.prim_makelist(thing1, thing2, (Context)null);
          break;
        case REVERSE:
        	thing1 = ostack[--sp];
        	ostack[sp++] = PrimList.prim_reverse(thing1, (Context)null);
        	break;
        case SORTNUMLIST:
          thing1 = ostack[--sp]; // list to be sorted
          ostack[sp++] = PrimList.prim_sortnumlist(thing1, (Context)null);
          break;
        case LISTMIN:
        	thing1 = ostack[--sp]; // find min of this list
        	ostack[sp++] = PrimList.prim_listmin(thing1, (Context)null);
        	break;
        case LISTMAX:
        	thing1 = ostack[--sp]; // find max of this list
        	ostack[sp++] = PrimList.prim_listmax(thing1, (Context)null);
        	break;
        case LISTMEDIAN:
        	thing1 = ostack[--sp]; // find median of this list
        	ostack[sp++] = PrimList.prim_listmedian(thing1, (Context)null);
        	break;
        case LISTMEAN:
	        thing1 = ostack[--sp];
        	ostack[sp++] = PrimList.prim_listmean(thing1, (Context)null);
        	break;
        case LISTVARIANCE:
        	thing1 = ostack[--sp];
        	ostack[sp++] = PrimList.prim_listvariance(thing1, (Context)null);
        	break;
        case LISTSDEV:
	        thing1 = ostack[--sp];
        	ostack[sp++] = PrimList.prim_listsdev(thing1, (Context)null);
        	break;
        case LISTMODE:
	        thing1 = ostack[--sp];
	        ostack[sp++] = PrimList.prim_listmode(thing1, (Context)null);
	        break;
        case LISTSUM:
	        thing1 = ostack[--sp];
	        ostack[sp++] = PrimList.prim_listsum(thing1, (Context)null);
	        break;
        case SUBLIST:
          thing3 = ostack[--sp]; // end index
          thing2 = ostack[--sp]; // begin index
        	thing1 = ostack[--sp]; // list
        	ostack[sp++] = PrimList.prim_sublist(thing1, thing2, thing3, (Context)null);
        	break;
        case INSERT:
          thing3 = ostack[--sp]; // object to be inserted
          thing2 = ostack[--sp]; // list
          thing1 = ostack[--sp]; // index
          ostack[sp++] = PrimList.prim_insert(thing1, thing2, thing3, (Context)null);
          break;
        case REMOVE:
        	thing2 = ostack[--sp]; // list
        	thing1 = ostack[--sp]; // index
        	ostack[sp++] = PrimList.prim_remove(thing1, thing2, (Context)null);
        	break;
        case REMOVEELE:
        	thing2 = ostack[--sp]; // list
        	thing1 = ostack[--sp]; // element
        	ostack[sp++] = PrimList.prim_removeele(thing1, thing2, (Context)null);
        	break;
        case TOSTRING:;
	    // string values are on the ostack
	     tempint = iinst[ip++]; // num elements of string;
	    StringBuffer str = new StringBuffer();
	    for(counter = tempint-1; counter >= 0; counter--) {
	      str.insert(0, aString(ostack[--sp]));
	    }
	    ostack[sp++] = str.toString();
	    break;
        case TODELSTRING:
	    // string values are on the ostack
	    tempint = iinst[ip++]; // num elements of string;
	    str = new StringBuffer();
	    for(counter = tempint-1; counter >= 1; counter--) {
	      str.insert(0, aString(ostack[--sp]));
	      str.insert(0, ',');
	    }
	    str.insert(0, aString(ostack[--sp]));
	    ostack[sp++] = str.toString();
	    break;
	}
      }

  public void primNsum(Context c, Observer obs, Turtle tur, Patch pat) {
    int tempint, tempint2;
    double x;
    switch (here & 0xFF) {
      /* Global Patch Operations */
    case NSUM:
      tempint2 = aInteger(ostack[--sp]);
      tempint = aInteger(ostack[--sp]);
      obs.nsum(tempint, tempint2);
      break;
    case NSUM4:
      tempint2 = aInteger(ostack[--sp]);
      tempint = aInteger(ostack[--sp]);
      obs.nsum4(tempint, tempint2);
      break;
    case NSUMPC:
      tempint = aInteger(ostack[--sp]);
      obs.nsumPC(tempint);
      sl.redraw();
      break;
    case NSUM4PC:
      tempint = aInteger(ostack[--sp]);
      obs.nsum4PC(tempint);
      sl.redraw();
      break;
    case NMIN:
      tempint2 = aInteger(ostack[--sp]);
      tempint = aInteger(ostack[--sp]);
      //nmin(tempint, tempint2);
      break;
    case NMIN4:
      tempint2 = aInteger(ostack[--sp]);
      tempint = aInteger(ostack[--sp]);
      //nmin4(tempint, tempint2);
      break;
    case NMAX:
      tempint2 = aInteger(ostack[--sp]);
      tempint = aInteger(ostack[--sp]);
      //nmax(tempint, tempint2);
      break;
    case NMAX4:
      tempint2 = aInteger(ostack[--sp]);
      tempint = aInteger(ostack[--sp]);
      //nmax4(tempint, tempint2);
      break;
    case DIFFUSE:
      x = aDouble(ostack[--sp]);
      tempint = aInteger(ostack[--sp]);
      obs.diffuse(tempint, x);
      break;
    case DIFFUSE4:
      x = aDouble(ostack[--sp]);
      tempint = aInteger(ostack[--sp]);
      obs.diffuse4(tempint, x);
      break;
    case DIFFUSEPC:
      x = aDouble(ostack[--sp]);
      obs.diffusePC(x);
      sl.redraw();
      break;
    case DIFFUSE4PC:
      x = aDouble(ostack[--sp]);
      obs.diffuse4PC(x);
      sl.redraw();
      break;
    }
    here = iinst[ip++];
  }

  public void primClear(Context c, Observer obs, Turtle tur, Patch pat) {
    switch (here & 0xFF) {
      /* Clearing the World */
    case CA:
      sl.CA();
      sl.redraw();
      break;
    case CG:
      sl.CG();
      sl.redraw();
      break;
    case CP:
      sl.CG();
      sl.CP();
      sl.redraw();
      break;
    case CT:
      sl.killAll();
      sl.redraw();
      break;
    case CLEAROUTPUT:
      if (!sl.app.kioskp && YoYo.applet == null) sl.app.outWin.reset();
      break;
    case CLEARINFO:
      if (!sl.app.kioskp && YoYo.applet == null) sl.app.infoWin.reset();
      break;
    }
    here = iinst[ip++];
  }

  public void primMisc(Context c, Observer obs, Turtle tur, Patch pat) {
    Object val;
    Turtle st;
    int tempint;
    double x, y, pt;
    int q, m;
    switch (here & 0xFF) {
      /* Interacting with the User */
    case STOPALL:
      sl.stopAllStarLogo();
      break;
    case PLAYSOUND:
      break;
    case PRINTMONITOR:
	//System.out.println("print monitor");
	val = ostack[--sp];
	q = aInteger(ostack[--sp]);
	//System.out.println("monitor " + q + 
	//		   " attempting to print value: " + val);
	MonitorWidget mw = (MonitorWidget)sl.app.iface.getMonitor(q);
	mw.setValue(val);
      break;
    case TOGGLEBUTTONON:
      ButtonWidget bw = (ButtonWidget)sl.app.iface.getButton(aInteger(ostack[--sp]));
      bw.turnButtonOn();
      break;
    case TOGGLEBUTTONOFF:
      bw = (ButtonWidget)sl.app.iface.getButton(aInteger(ostack[--sp]));
      bw.turnButtonOff();
      break;
    case YOYO:
      Toplevel.ub = TopApplication.ub;
      Toplevel.base = TopApplication.base;
      new Toplevel("YoYo", false);
      break;
    case SWITCHPATCHES:
      sl.altgraphics = (sl.altgraphics + 1) % StarLogo.NUMPCTYPES;
      	sl.switchPatches();
      break;
    case SWITCHSCHED:
      sl.list.randomschedp = !sl.list.randomschedp;
      sl.list.changeschedp = true;
      break;
    case SETSCHEDRANDSEED:
	tempint = aInteger(ostack[--sp]);
	StackProcessList.seed = tempint;
	StackProcessList.rand.setSeed((long)tempint);
	break;
    case INSPECTT:
      st = sl.getWho(aInteger(ostack[--sp]));
      if (st != null) {
         	if (!st.hasTMp) {
		  st.tm = Application.newTurtleMonitor(st, 0.5, sl.app);
		  st.tm.initFrame();
		  st.hasTMp = true;
		}
		st.tm.show();
      }
      break;
    case INSPECTP:
      y = sl.halfheight - aDouble(ostack[--sp]);
      x = sl.halfwidth + aDouble(ostack[--sp]);
      tempint = sl.patchat(x, y);
      Patch p = sl.patches[tempint];
      if (p != null) {
	if (!p.hasPMp) {
	  p.pm = Application.newPatchMonitor(p, 0.5, sl.app);
	  p.pm.initFrame();
	  p.hasPMp = true;
	}
	p.pm.show();
      }
       break;
    case DISPLAY:
      sl.patchcanvas.display();
      break;
    case NODISPLAY:
      sl.patchcanvas.noDisplay();
      break;
    case DEBUG:
      sl.DEBUG = true;
      break;
    case NODEBUG:
      sl.DEBUG = false;
      break;
    case DEBUG2:
      sl.DEBUG2 = true;
      break;
    case NODEBUG2:
      sl.DEBUG2 = false;
      break;
    case SETSVAL:
	SliderWidget sld = (SliderWidget)sl.app.iface.getSlider(aInteger(ostack[--sp]));
	tempint = aInteger(ostack[--sp]);
	sld.setValue(tempint);
	break;
    case SETCC:
	tempint = aInteger(ostack[--sp])%Colors.numcolors;
	tempint = (tempint < 0) ? tempint+Colors.numcolors : tempint;
	sl.app.iface.colortool().setColor(tempint);
	break;
    }
    here = iinst[ip++];
  }


    public void incorrectLineWidgetError(String name) {
	YoYoObject yob = new YoYoObject();
	yob.defineValue(Symbol.lookup("instruction"), new Integer(here));
	yob.defineValue(Symbol.lookup("value"), name);
	setTPO(yob);
	process.saveState();
	Object[] crawl = process.stackCrawl();
	yob.defineValue(Symbol.lookup("stack-crawl"), crawl);
	throw new ThrowReturn("vm-not-plotid", yob);
    }

    public void incorrectLineWidgetError(int plotid) {
	YoYoObject yob = new YoYoObject();
	yob.defineValue(Symbol.lookup("instruction"), new Integer(here));
	yob.defineValue(Symbol.lookup("value"), new Integer(plotid));
	setTPO(yob);
	process.saveState();
	Object[] crawl = process.stackCrawl();
	yob.defineValue(Symbol.lookup("stack-crawl"), crawl);
	throw new ThrowReturn("vm-not-plotid", yob);
    }

  public void primPlotReporters(Context c, Observer obs, Turtle tur, Patch pat) {
      Object val = sl.app.iface.getLineWidget(plotwidget);
      LineWidget lw = null;
      if (val != YoYo.symfalse) lw = (LineWidget)val;
      if (lw == null) { 
	  // no plot widgets on screen
	  // XXX Perhaps return an error
	  ostack[sp++] = YoYo.symfalse;
      } else {
	  if (lw.wizardOn()) {
	      // there's a plot, but it's not an old-style plot
	      switch (here & 0xFF) {
	      case PLOTPPC:
	      case PPXCOR:
	      case PPYCOR:
	      case PLOTXMIN:
	      case PLOTXMAX:
	      case PLOTYMIN:
	      case PLOTYMAX:
	      case COUNTPLOTPENS:
		  ostack[sp++] = zeroInt;
		  break;
	      case PLOTTITLE:
	      case PLOTXLABEL:
	      case PLOTYLABEL:
		  ostack[sp++] = "";
		  break;
	      case PLOTGRID:
		  ostack[sp++] = YoYo.symfalse;
		  break;
	      }
	  } else {
	      switch (here & 0xFF) {
		  /* Plot Commands: These only work for old-style plots */
	    case PLOTPPC:
		ostack[sp++] = lw.plotppc(plotpen);
		break;
	    case PPXCOR:
		ostack[sp++] = lw.ppxcor(plotpen);
		break;
	    case PPYCOR:
		ostack[sp++] = lw.ppycor(plotpen);
		break;
	    case PLOTXMIN:
		ostack[sp++] = lw.plotxmin();
		break;
	    case PLOTXMAX:
		ostack[sp++] = lw.plotxmax();
		break;
	    case PLOTYMIN:
		ostack[sp++] = lw.plotymin();
		break;
	    case PLOTYMAX:
		ostack[sp++] = lw.plotymax();
		break;
	      case PLOTGRID:
		  ostack[sp++] = (lw.plotgrid()) ? YoYo.symtrue : YoYo.symfalse;
		  break;
	      case PLOTTITLE:
		ostack[sp++] = lw.plottitle();
		break;
	  case PLOTXLABEL:
		ostack[sp++] = lw.plotxlabel();
	    break;
	  case PLOTYLABEL:
		ostack[sp++] = lw.plotylabel();
	    break;
	  case COUNTPLOTPENS:
		  ostack[sp++] = lw.countplotpens();
	    break;
	      }
	  }

      }
  }

    public void primPlot(Context c, Observer obs, Turtle tur, Patch pat) {
    double x, y;
    int id, ppid;
    Object val, val2;
    Application app = sl.app;
    int bytecode = here & 0xFF;
    if (bytecode == PTOLEMYPLOTXY  ||
	bytecode == PTOLEMYPLOT ||
	bytecode == SETPLOT ||
	bytecode == CLEARPLOTS) {
	switch (bytecode) {
	case PTOLEMYPLOTXY:
	    y = aDouble(ostack[--sp]);
	    x = aDouble(ostack[--sp]);
	    ppid = aInteger(ostack[--sp]);
	    id = aInteger(ostack[--sp]);
	    val = app.iface.getLineWidget(id);
	    if (val == null || val == YoYo.symfalse) {
		incorrectLineWidgetError(id);
	    }
	    ((LineWidget)val).ptolemyplotxy(ppid, x, y, tur, pat, sl.width);
	    break;
	case PTOLEMYPLOT:
	    x = aDouble(ostack[--sp]);
	    ppid = aInteger(ostack[--sp]);
	    id = aInteger(ostack[--sp]);
	    val = app.iface.getLineWidget(id);
	    if (val == null || val == YoYo.symfalse) {
		incorrectLineWidgetError(id);
	    }
	    ((LineWidget)val).ptolemyplot(ppid, x, tur, pat, sl.width);
	    break;
	case SETPLOT:
	    val = ostack[--sp];
	    if (val instanceof Number) {
		id = ((Number)val).intValue();
		val2 = app.iface.getLineWidget(id);
		if (val2 == null || val2 == YoYo.symfalse) {
		    incorrectLineWidgetError(id);
		}
		plotwidget = id;
	    } else {
		String title = aString(val);
		val2 = app.iface.getLineWidgetFromTitle(title);
		if (val2 == null || val2 == YoYo.symfalse) {
		    incorrectLineWidgetError(title);
		}
		plotwidget = ((LineWidget)val2).id;
	    }
	    break;
	case CLEARPLOTS:
	    app.iface.clearLineWidgets();
	    //app.iface.repaint();
	    break;
	}
	here = iinst[ip++]; 
	return;
    }
    val = app.iface.getLineWidget(plotwidget);
    LineWidget lw = null;
    if (val != YoYo.symfalse) lw = (LineWidget)val;
    if (lw == null || lw.wizardOn()) {
	// XXX report an error?
	switch (bytecode) {
	case OPLOT:  
	case TPLOT:  
	case PPLOT:  
	case SETPLOTPPC: 
	case SETPLOTXMIN:
	case SETPLOTXMAX:
	case SETPLOTYMIN:
	case SETPLOTYMAX:
	case PLOTPEN:
	case SETPLOTTITLE:
	case SETPLOTXLABEL:
	case SETPLOTYLABEL:
	case SETNUMPLOTPENS:
	case SETBARWIDTH:
	    --sp; break;
	case SETPLOTXRANGE:
	case SETPLOTYRANGE:
	case OPLOTXY:
	case TPLOTXY:
	case PPLOTXY:
	    sp-=2; break;
	case CLEARPLOT: 
	case PPRESET:
	case PPD:
	case PPU:
	case VIEWPLOT:
	case PLOTGRIDON:
	case PLOTGRIDOFF:
	    break;
	} 
    } else {
	int tempint, tempint2;
	switch (here & 0xFF) {
	    /* Plot Commands */
	case OPLOT:
	      x = aDouble(ostack[--sp]);
	      if (lw.type == LineWidget.LINE || lw.type == LineWidget.BAR) {
		  lw.plot(plotpen, x);
	      } else incorrectLineWidgetError(lw.id);
	      break;
	case TPLOT:
	      x = aDouble(ostack[--sp]);
	      if (lw.type == LineWidget.LINE || lw.type == LineWidget.BAR) {
		  lw.plot(plotpen, x);
	      } else if (lw.type == LineWidget.HISTOGRAM) {
		  lw.plotdistribution(plotpen, tur.who, x);
	      } else incorrectLineWidgetError(lw.id);
	      break;
	case PPLOT:
	      x = aDouble(ostack[--sp]);
	      if (lw.type == LineWidget.LINE || lw.type == LineWidget.BAR) {
		  lw.plot(plotpen, x);
	      } else if (lw.type == LineWidget.HISTOGRAM) {
		  lw.plotdistribution(plotpen, (int)(pat.xcor * sl.width + pat.ycor), x);
	      } else incorrectLineWidgetError(lw.id);
	      break;
	case OPLOTXY:
	y = aDouble(ostack[--sp]);
	x = aDouble(ostack[--sp]);
	if (lw.type == LineWidget.PHASE) {
	    lw.plotxy(plotpen, x, y);
	} else incorrectLineWidgetError(lw.id);
	break;
	case TPLOTXY:
	y = aDouble(ostack[--sp]);
	x = aDouble(ostack[--sp]);
	if (lw.type == LineWidget.PHASE) {
	    lw.plotxy(plotpen, x, y);
	} else if (lw.type == LineWidget.XYPLOT) {
	    lw.plotscatter(plotpen, tur.who, x, y);
	} else incorrectLineWidgetError(lw.id);
	break;
	case PPLOTXY:
	y = aDouble(ostack[--sp]);
	x = aDouble(ostack[--sp]);
	if (lw.type == LineWidget.PHASE) {
	    lw.plotxy(plotpen, x, y);
	} else if (lw.type == LineWidget.XYPLOT) {
	    lw.plotscatter(plotpen, (int)(pat.xcor * sl.width + pat.ycor), x, y);
	} else incorrectLineWidgetError(lw.id);
	break;
	case SETPLOTPPC:
	    tempint = aInteger(ostack[--sp]) % Colors.numcolors;
	    if (tempint<0) tempint += Colors.numcolors;
	    lw.setplotppc(plotpen, tempint);
	    break;
	case CLEARPLOT:
	  lw.reset();
	  break;
	case PLOTGRIDON:
	    lw.setPlotGrid(true);
	    break;
	case PLOTGRIDOFF:
	    lw.setPlotGrid(false);
	    break;
	case SETPLOTXMIN:
	    lw.setplotxmin(aDouble(ostack[--sp]));
	    break;
    case SETPLOTXMAX:
	    lw.setplotxmax(aDouble(ostack[--sp]));
      break;
    case SETPLOTXRANGE:
	  y = aDouble(ostack[--sp]); //max
	  x = aDouble(ostack[--sp]); // min
	  lw.setplotxrange(x, y);
      break;
    case SETPLOTYMIN:
	  lw.setplotymin(aDouble(ostack[--sp]));
	break;
    case SETPLOTYMAX:
	    lw.setplotymax(aDouble(ostack[--sp]));
	break;
    case SETPLOTYRANGE:
	  y = aDouble(ostack[--sp]); //max
	  x = aDouble(ostack[--sp]); // min
	  lw.setplotyrange(x, y);
      break;
    case PPRESET:
	    lw.ppreset(plotpen);
      break;
    case PLOTPEN:
	    plotpen = aInteger(ostack[--sp]);
	    // XXX Check for validity
	    break;
    case PPD:
	lw.ppd(plotpen);
	break;
	case PPU: 
	    lw.ppu(plotpen);
      break;
    case SETPLOTTITLE:
	  lw.setplottitle(aString(ostack[--sp]));
      break;
    case SETPLOTXLABEL:
	  lw.setplotxlabel(aString(ostack[--sp]));
      break;
    case SETPLOTYLABEL:
	  lw.setplotylabel(aString(ostack[--sp]));
      break;
    case SETNUMPLOTPENS:
	  lw.setnumplotpens(aInteger(ostack[--sp]));
      break;
    case VIEWPLOT:
	lw.show();
	break;
    case SETBARWIDTH:
	lw.setbarwidth(plotpen, aDouble(ostack[--sp]));
      break;
	}
    }
    here = iinst[ip++];
}

    String infofilename, outputfilename, projectfilename;

    public void primSave(Context c, Observer obs, Turtle tur, Patch pat) {
	Application app = sl.app;
	String filename;
      switch (here & 0xFF) {
      case SETOUTPUTFILENAME:
	  outputfilename = aString(ostack[--sp]);
	  if (!sl.app.kioskp && YoYo.applet == null) { 
	      if (!outputfilename.equals("") && sl.app.project.curDir != null) {
		  sl.app.project.setTextFilename(sl.app, outputfilename, Project.SAVE_OUTPUT);
	      } else {
		  SwingUtilities.invokeLater(new Runnable() {
			  public void run() {
			      sl.app.project.setTextFilename(sl.app, outputfilename, Project.SAVE_OUTPUT);
			  }
		      });
	      }
	  }
	break;
    case SETINFOFILENAME:
	 infofilename = aString(ostack[--sp]);
	  if (!sl.app.kioskp && YoYo.applet == null) { 
	      if (!infofilename.equals("") && sl.app.project.curDir != null) {
		  sl.app.project.setTextFilename(sl.app, infofilename, Project.SAVE_INFO);
	      } else {
		  SwingUtilities.invokeLater(new Runnable() {
			  public void run() {
			      sl.app.project.setTextFilename(sl.app, infofilename, Project.SAVE_INFO);
			  }
		      });
	      }
	  }
      break;
    case SETPROJECTFILENAME:
	projectfilename = aString(ostack[--sp]);
	  if (!sl.app.kioskp && YoYo.applet == null) { 
	      if (!projectfilename.equals("") && sl.app.project.curDir != null) {
		  sl.app.project.setProjectFilename(sl.app, projectfilename);
	      } else {
		  SwingUtilities.invokeLater(new Runnable() {
			  public void run() {
			      sl.app.project.setProjectFilename(sl.app, projectfilename);
			  }
		      });
	      }
	  }
      break;
    case SAVEOUTPUT:
	  if (sl.app.kioskp || YoYo.applet != null) { break; }
	SwingUtilities.invokeLater(new Runnable() {
	      public void run() {
	            sl.app.project.saveText(sl.app, Project.SAVE_OUTPUT); 
              }
        });
      break;
    case SAVEOUTPUTAS:
	  if (sl.app.kioskp || YoYo.applet != null) { break; }
	SwingUtilities.invokeLater(new Runnable() {
	      public void run() {
	            sl.app.project.saveTextAs(sl.app, Project.SAVE_OUTPUT);
              }
        });
      break;
    case SAVEINFO:
	  if (sl.app.kioskp || YoYo.applet != null) { break; }
	SwingUtilities.invokeLater(new Runnable() {
	      public void run() {
	            sl.app.project.saveText(sl.app, Project.SAVE_INFO);
	      }
	});
      break;
    case SAVEINFOAS:
	  if (sl.app.kioskp || YoYo.applet != null) { break; }
	SwingUtilities.invokeLater(new Runnable() {
	      public void run() {
	         sl.app.project.saveTextAs(sl.app, Project.SAVE_INFO);
              }					  
        });
      break;
    case SAVEPROJECT:
	  if (sl.app.kioskp || YoYo.applet != null) { break; }
	SwingUtilities.invokeLater(new Runnable() {
		public void run() {
		    sl.app.project.saveProject(sl.app, false);
		}
	    });
      break;
    case SAVEPROJECTAS:
	  if (sl.app.kioskp || YoYo.applet != null) { break; }
	SwingUtilities.invokeLater(new Runnable() {
		public void run() {
		    sl.app.project.saveProjectAs(sl.app, false);
		}
	    }
				   );
      break;
    case SAVEVERSION:
	  if (sl.app.kioskp || YoYo.applet != null) { break; }
	SwingUtilities.invokeLater(new Runnable() {
		public void run() {
		      sl.app.project.saveVersion(sl.app);
                }
        });
      break;
    case WRITEMAP:
      Colors.writeMap(c);
      break;
    case READPICTURE:
	StarLogo.PictInstance.VMloadPicture(sl);
	sl.redraw();
	break;
    case READPICTURENAME:
	StarLogo.PictInstance.VMloadPicture(sl, aString(ostack[--sp]));
	sl.redraw();
	break;
      case SAVEPATCHESPICTURE:
	  StarLogo.PictInstance.VMsavePicture(sl, Pictures.PATCHES);
	sl.redraw();
	break;
      case SAVEPATCHESPICTURENAME:
	  StarLogo.PictInstance.VMsavePicture(sl, aString(ostack[--sp]), Pictures.PATCHES);
	sl.redraw();
	break;
      case SAVETURTLES_AND_PATCHESPICTURE:
	  StarLogo.PictInstance.VMsavePicture(sl, Pictures.TURTLES_AND_PATCHES);
	sl.redraw();
	break;
      case SAVETURTLES_AND_PATCHESPICTURENAME:
	  StarLogo.PictInstance.VMsavePicture(sl, aString(ostack[--sp]), Pictures.TURTLES_AND_PATCHES);
	sl.redraw();
	break;
      case SAVEINTERFACEPICTURE:
	  StarLogo.PictInstance.VMsavePicture(sl, Pictures.INTERFACE);
	sl.redraw();
	break;
      case SAVEINTERFACEPICTURENAME:
	  StarLogo.PictInstance.VMsavePicture(sl, aString(ostack[--sp]), Pictures.INTERFACE);
	sl.redraw();
	break;
      }
      here = iinst[ip++];
    }

    Turtle pickRandomTurtleHere(int patch) {
	return pickRandomTurtleHere(patch, null, -Integer.MAX_VALUE);
    }

    Turtle pickRandomTurtleHere(int patch, Turtle notme) {
	return pickRandomTurtleHere(patch, notme, -Integer.MAX_VALUE);
    }

    Turtle pickRandomBreedHere(int patch, int breed) {
	return pickRandomTurtleHere(patch, null, breed);
    }

    Turtle pickRandomBreedHere(int patch, Turtle notme, int breed) {
	
	return pickRandomTurtleHere(patch, notme, breed);
    }

    Turtle pickRandomTurtleHere(int patch, Turtle notme, int breed) {
	//System.out.println("pick random turtle here. patch: " + patch + " notme " + notme + " breed: " + breed);
	//first count all turtles of the right breed
	int numturtlesShown = 0;
	int numturtlesHidden = 0;
	Turtle st = sl.patches[patch].topturtle;
	if (breed != -Integer.MAX_VALUE) 
	    while(st!=null) { if(st.breed == breed) 
		{numturtlesShown++; st = st.underme; }
	    else st=st.underme;}
	else 
	    while(st!=null) { numturtlesShown++; st = st.underme; };
		    
	st = sl.patches[patch].tophiddenturtle;
	if (breed != -Integer.MAX_VALUE) 
	    while(st!=null) { if(st.breed == breed) 
		{numturtlesHidden++; st = st.underme; }
	    else st=st.underme;}
	else
	    while(st!=null) { numturtlesHidden++; st = st.underme; };	    

	//System.out.println("numturtlesshown: " + numturtlesShown); 
	//System.out.println("numturtleshidden: " + numturtlesHidden); 

	// if there's no one there, return null
	if (numturtlesShown + numturtlesHidden == 0) return null;

	// if there's only one guy there and he's me, return null
	if (numturtlesShown + numturtlesHidden == 1) {
	    if (breed != -Integer.MAX_VALUE) {
		if (notme != null && notme.breed == breed) return null;
	    } else {
		if (notme != null) return null;
	    }
	}
	
	// otherwise pick one
	st = notme;
	//System.out.println("pick one");
	do {
	    int pick = (int)Math.floor(rand.nextDouble()*(numturtlesShown + numturtlesHidden));
	    //     System.out.println("pick = " + pick + " numshown: " + numturtlesShown + " hidden: " + numturtlesHidden);
	    
	    if (pick < numturtlesShown) {
		// System.out.println("shown turtles");
		st = sl.patches[patch].topturtle;
		if (breed != -Integer.MAX_VALUE) {
		    while(st != null) {
			if (st.breed == breed) {
			    if (pick-- == 0) break;
			}
			st = st.underme;
		    }
		} else {
		    while (pick-- > 0) {
			st = st.underme;
		    }
		}
	    } else {
		//System.out.println("pick a hidden turtle");
		pick -= numturtlesShown;
		st = sl.patches[patch].tophiddenturtle;
		
		if (breed != -Integer.MAX_VALUE) {
		    while(st != null) {
			if (st.breed == breed) {
			    if (pick-- == 0) break;
			}
			st = st.underme;
		    }
		} else {
		    while (pick-- > 0) {
			st = st.underme;
		    }
		}
	    } 
	} while (notme != null && notme == st);
	return st;
    }
    

  abstract public boolean grab(Object val);
  abstract public boolean grabPatch(Patch val);

    static String[] enumnames;

    static public void setEnumNames(Object[] enums, StarLogo sl, Context c) {
	enumnames = new String[enums.length];
	for(int i = 0; i < enums.length; i++) {
	    Object[] pair = YoYo.aList(c, enums[i]);
	    int index = YoYo.aInteger(c, pair[1]);
	    enumnames[index-1] = YoYo.aString(c, pair[0]);
	}
    }


  public abstract String niceToString();

}
